/****************************************************************************
** Meta object code from reading C++ file 'smtextstyles.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/smtextstyles.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smtextstyles.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SMParagraphStyle_t {
    QByteArrayData data[81];
    char stringdata0[1148];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SMParagraphStyle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SMParagraphStyle_t qt_meta_stringdata_SMParagraphStyle = {
    {
QT_MOC_LITERAL(0, 0, 16), // "SMParagraphStyle"
QT_MOC_LITERAL(1, 17, 14), // "selectionDirty"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 19), // "slotLineSpacingMode"
QT_MOC_LITERAL(4, 53, 4), // "mode"
QT_MOC_LITERAL(5, 58, 15), // "slotLineSpacing"
QT_MOC_LITERAL(6, 74, 21), // "slotParentParaEffects"
QT_MOC_LITERAL(7, 96, 11), // "slotDropCap"
QT_MOC_LITERAL(8, 108, 4), // "isOn"
QT_MOC_LITERAL(9, 113, 16), // "slotDropCapLines"
QT_MOC_LITERAL(10, 130, 5), // "lines"
QT_MOC_LITERAL(11, 136, 19), // "slotParEffectOffset"
QT_MOC_LITERAL(12, 156, 19), // "slotParEffectIndent"
QT_MOC_LITERAL(13, 176, 22), // "slotParEffectCharStyle"
QT_MOC_LITERAL(14, 199, 10), // "slotBullet"
QT_MOC_LITERAL(15, 210, 13), // "slotBulletStr"
QT_MOC_LITERAL(16, 224, 3), // "str"
QT_MOC_LITERAL(17, 228, 14), // "slotNumeration"
QT_MOC_LITERAL(18, 243, 11), // "slotNumName"
QT_MOC_LITERAL(19, 255, 10), // "slotNumNew"
QT_MOC_LITERAL(20, 266, 18), // "slotSelectionDirty"
QT_MOC_LITERAL(21, 285, 13), // "slotNumFormat"
QT_MOC_LITERAL(22, 299, 12), // "slotNumLevel"
QT_MOC_LITERAL(23, 312, 5), // "level"
QT_MOC_LITERAL(24, 318, 13), // "slotNumPrefix"
QT_MOC_LITERAL(25, 332, 13), // "slotNumSuffix"
QT_MOC_LITERAL(26, 346, 12), // "slotNumStart"
QT_MOC_LITERAL(27, 359, 5), // "start"
QT_MOC_LITERAL(28, 365, 14), // "slotNumRestart"
QT_MOC_LITERAL(29, 380, 7), // "restart"
QT_MOC_LITERAL(30, 388, 12), // "slotNumOther"
QT_MOC_LITERAL(31, 401, 13), // "slotNumHigher"
QT_MOC_LITERAL(32, 415, 14), // "slotSpaceAbove"
QT_MOC_LITERAL(33, 430, 14), // "slotSpaceBelow"
QT_MOC_LITERAL(34, 445, 13), // "slotAlignment"
QT_MOC_LITERAL(35, 459, 13), // "slotDirection"
QT_MOC_LITERAL(36, 473, 17), // "slotOpticalMargin"
QT_MOC_LITERAL(37, 491, 1), // "i"
QT_MOC_LITERAL(38, 493, 25), // "slotOpticalMarginSelector"
QT_MOC_LITERAL(39, 519, 23), // "slotParentOpticalMargin"
QT_MOC_LITERAL(40, 543, 12), // "slotMinSpace"
QT_MOC_LITERAL(41, 556, 15), // "slotMinGlyphExt"
QT_MOC_LITERAL(42, 572, 15), // "slotMaxGlyphExt"
QT_MOC_LITERAL(43, 588, 20), // "slotConsecutiveLines"
QT_MOC_LITERAL(44, 609, 20), // "handleKeepLinesStart"
QT_MOC_LITERAL(45, 630, 18), // "handleKeepLinesEnd"
QT_MOC_LITERAL(46, 649, 18), // "handleKeepTogether"
QT_MOC_LITERAL(47, 668, 18), // "handleKeepWithNext"
QT_MOC_LITERAL(48, 687, 12), // "slotTabRuler"
QT_MOC_LITERAL(49, 700, 14), // "slotLeftIndent"
QT_MOC_LITERAL(50, 715, 15), // "slotRightIndent"
QT_MOC_LITERAL(51, 731, 13), // "slotFirstLine"
QT_MOC_LITERAL(52, 745, 14), // "slotBackPColor"
QT_MOC_LITERAL(53, 760, 14), // "slotBackPShade"
QT_MOC_LITERAL(54, 775, 12), // "slotFontSize"
QT_MOC_LITERAL(55, 788, 11), // "slotEffects"
QT_MOC_LITERAL(56, 800, 1), // "e"
QT_MOC_LITERAL(57, 802, 20), // "slotEffectProperties"
QT_MOC_LITERAL(58, 823, 13), // "slotFillColor"
QT_MOC_LITERAL(59, 837, 13), // "slotFillShade"
QT_MOC_LITERAL(60, 851, 15), // "slotStrokeColor"
QT_MOC_LITERAL(61, 867, 15), // "slotStrokeShade"
QT_MOC_LITERAL(62, 883, 13), // "slotBackColor"
QT_MOC_LITERAL(63, 897, 13), // "slotBackShade"
QT_MOC_LITERAL(64, 911, 12), // "slotLanguage"
QT_MOC_LITERAL(65, 924, 11), // "slotWordMin"
QT_MOC_LITERAL(66, 936, 14), // "slotHyphenChar"
QT_MOC_LITERAL(67, 951, 10), // "slotScaleH"
QT_MOC_LITERAL(68, 962, 10), // "slotScaleV"
QT_MOC_LITERAL(69, 973, 12), // "slotTracking"
QT_MOC_LITERAL(70, 986, 16), // "slotWordTracking"
QT_MOC_LITERAL(71, 1003, 18), // "slotBaselineOffset"
QT_MOC_LITERAL(72, 1022, 8), // "slotFont"
QT_MOC_LITERAL(73, 1031, 1), // "s"
QT_MOC_LITERAL(74, 1033, 17), // "slotParentChanged"
QT_MOC_LITERAL(75, 1051, 6), // "parent"
QT_MOC_LITERAL(76, 1058, 21), // "slotCharParentChanged"
QT_MOC_LITERAL(77, 1080, 16), // "slotFontFeatures"
QT_MOC_LITERAL(78, 1097, 21), // "slotCharStylesDeleted"
QT_MOC_LITERAL(79, 1119, 17), // "QList<RemoveItem>"
QT_MOC_LITERAL(80, 1137, 10) // "removeList"

    },
    "SMParagraphStyle\0selectionDirty\0\0"
    "slotLineSpacingMode\0mode\0slotLineSpacing\0"
    "slotParentParaEffects\0slotDropCap\0"
    "isOn\0slotDropCapLines\0lines\0"
    "slotParEffectOffset\0slotParEffectIndent\0"
    "slotParEffectCharStyle\0slotBullet\0"
    "slotBulletStr\0str\0slotNumeration\0"
    "slotNumName\0slotNumNew\0slotSelectionDirty\0"
    "slotNumFormat\0slotNumLevel\0level\0"
    "slotNumPrefix\0slotNumSuffix\0slotNumStart\0"
    "start\0slotNumRestart\0restart\0slotNumOther\0"
    "slotNumHigher\0slotSpaceAbove\0"
    "slotSpaceBelow\0slotAlignment\0slotDirection\0"
    "slotOpticalMargin\0i\0slotOpticalMarginSelector\0"
    "slotParentOpticalMargin\0slotMinSpace\0"
    "slotMinGlyphExt\0slotMaxGlyphExt\0"
    "slotConsecutiveLines\0handleKeepLinesStart\0"
    "handleKeepLinesEnd\0handleKeepTogether\0"
    "handleKeepWithNext\0slotTabRuler\0"
    "slotLeftIndent\0slotRightIndent\0"
    "slotFirstLine\0slotBackPColor\0"
    "slotBackPShade\0slotFontSize\0slotEffects\0"
    "e\0slotEffectProperties\0slotFillColor\0"
    "slotFillShade\0slotStrokeColor\0"
    "slotStrokeShade\0slotBackColor\0"
    "slotBackShade\0slotLanguage\0slotWordMin\0"
    "slotHyphenChar\0slotScaleH\0slotScaleV\0"
    "slotTracking\0slotWordTracking\0"
    "slotBaselineOffset\0slotFont\0s\0"
    "slotParentChanged\0parent\0slotCharParentChanged\0"
    "slotFontFeatures\0slotCharStylesDeleted\0"
    "QList<RemoveItem>\0removeList"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SMParagraphStyle[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      66,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  344,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,  345,    2, 0x08 /* Private */,
       5,    0,  348,    2, 0x08 /* Private */,
       6,    0,  349,    2, 0x08 /* Private */,
       7,    1,  350,    2, 0x08 /* Private */,
       9,    1,  353,    2, 0x08 /* Private */,
      11,    0,  356,    2, 0x08 /* Private */,
      12,    1,  357,    2, 0x08 /* Private */,
      13,    1,  360,    2, 0x08 /* Private */,
      14,    1,  363,    2, 0x08 /* Private */,
      15,    1,  366,    2, 0x08 /* Private */,
      17,    1,  369,    2, 0x08 /* Private */,
      18,    1,  372,    2, 0x08 /* Private */,
      19,    0,  375,    2, 0x08 /* Private */,
      20,    0,  376,    2, 0x08 /* Private */,
      21,    1,  377,    2, 0x08 /* Private */,
      22,    1,  380,    2, 0x08 /* Private */,
      24,    1,  383,    2, 0x08 /* Private */,
      25,    1,  386,    2, 0x08 /* Private */,
      26,    1,  389,    2, 0x08 /* Private */,
      28,    1,  392,    2, 0x08 /* Private */,
      30,    1,  395,    2, 0x08 /* Private */,
      31,    1,  398,    2, 0x08 /* Private */,
      32,    0,  401,    2, 0x08 /* Private */,
      33,    0,  402,    2, 0x08 /* Private */,
      34,    0,  403,    2, 0x08 /* Private */,
      35,    0,  404,    2, 0x08 /* Private */,
      36,    1,  405,    2, 0x08 /* Private */,
      38,    0,  408,    2, 0x08 /* Private */,
      39,    0,  409,    2, 0x08 /* Private */,
      40,    0,  410,    2, 0x08 /* Private */,
      41,    0,  411,    2, 0x08 /* Private */,
      42,    0,  412,    2, 0x08 /* Private */,
      43,    0,  413,    2, 0x08 /* Private */,
      44,    0,  414,    2, 0x08 /* Private */,
      45,    0,  415,    2, 0x08 /* Private */,
      46,    0,  416,    2, 0x08 /* Private */,
      47,    0,  417,    2, 0x08 /* Private */,
      48,    0,  418,    2, 0x08 /* Private */,
      49,    0,  419,    2, 0x08 /* Private */,
      50,    0,  420,    2, 0x08 /* Private */,
      51,    0,  421,    2, 0x08 /* Private */,
      52,    0,  422,    2, 0x08 /* Private */,
      53,    0,  423,    2, 0x08 /* Private */,
      54,    0,  424,    2, 0x08 /* Private */,
      55,    1,  425,    2, 0x08 /* Private */,
      57,    0,  428,    2, 0x08 /* Private */,
      58,    0,  429,    2, 0x08 /* Private */,
      59,    0,  430,    2, 0x08 /* Private */,
      60,    0,  431,    2, 0x08 /* Private */,
      61,    0,  432,    2, 0x08 /* Private */,
      62,    0,  433,    2, 0x08 /* Private */,
      63,    0,  434,    2, 0x08 /* Private */,
      64,    0,  435,    2, 0x08 /* Private */,
      65,    0,  436,    2, 0x08 /* Private */,
      66,    0,  437,    2, 0x08 /* Private */,
      67,    0,  438,    2, 0x08 /* Private */,
      68,    0,  439,    2, 0x08 /* Private */,
      69,    0,  440,    2, 0x08 /* Private */,
      70,    0,  441,    2, 0x08 /* Private */,
      71,    0,  442,    2, 0x08 /* Private */,
      72,    1,  443,    2, 0x08 /* Private */,
      74,    1,  446,    2, 0x08 /* Private */,
      76,    1,  449,    2, 0x08 /* Private */,
      77,    0,  452,    2, 0x08 /* Private */,
      78,    1,  453,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::Int,   10,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,   23,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void, QMetaType::QString,   16,
    QMetaType::Void, QMetaType::Int,   27,
    QMetaType::Void, QMetaType::Int,   29,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   37,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   56,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   73,
    QMetaType::Void, QMetaType::QString,   75,
    QMetaType::Void, QMetaType::QString,   75,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 79,   80,

       0        // eod
};

void SMParagraphStyle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SMParagraphStyle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->selectionDirty(); break;
        case 1: _t->slotLineSpacingMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->slotLineSpacing(); break;
        case 3: _t->slotParentParaEffects(); break;
        case 4: _t->slotDropCap((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->slotDropCapLines((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->slotParEffectOffset(); break;
        case 7: _t->slotParEffectIndent((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->slotParEffectCharStyle((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->slotBullet((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->slotBulletStr((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 11: _t->slotNumeration((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->slotNumName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->slotNumNew(); break;
        case 14: _t->slotSelectionDirty(); break;
        case 15: _t->slotNumFormat((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->slotNumLevel((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->slotNumPrefix((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 18: _t->slotNumSuffix((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 19: _t->slotNumStart((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->slotNumRestart((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 21: _t->slotNumOther((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->slotNumHigher((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 23: _t->slotSpaceAbove(); break;
        case 24: _t->slotSpaceBelow(); break;
        case 25: _t->slotAlignment(); break;
        case 26: _t->slotDirection(); break;
        case 27: _t->slotOpticalMargin((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 28: _t->slotOpticalMarginSelector(); break;
        case 29: _t->slotParentOpticalMargin(); break;
        case 30: _t->slotMinSpace(); break;
        case 31: _t->slotMinGlyphExt(); break;
        case 32: _t->slotMaxGlyphExt(); break;
        case 33: _t->slotConsecutiveLines(); break;
        case 34: _t->handleKeepLinesStart(); break;
        case 35: _t->handleKeepLinesEnd(); break;
        case 36: _t->handleKeepTogether(); break;
        case 37: _t->handleKeepWithNext(); break;
        case 38: _t->slotTabRuler(); break;
        case 39: _t->slotLeftIndent(); break;
        case 40: _t->slotRightIndent(); break;
        case 41: _t->slotFirstLine(); break;
        case 42: _t->slotBackPColor(); break;
        case 43: _t->slotBackPShade(); break;
        case 44: _t->slotFontSize(); break;
        case 45: _t->slotEffects((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 46: _t->slotEffectProperties(); break;
        case 47: _t->slotFillColor(); break;
        case 48: _t->slotFillShade(); break;
        case 49: _t->slotStrokeColor(); break;
        case 50: _t->slotStrokeShade(); break;
        case 51: _t->slotBackColor(); break;
        case 52: _t->slotBackShade(); break;
        case 53: _t->slotLanguage(); break;
        case 54: _t->slotWordMin(); break;
        case 55: _t->slotHyphenChar(); break;
        case 56: _t->slotScaleH(); break;
        case 57: _t->slotScaleV(); break;
        case 58: _t->slotTracking(); break;
        case 59: _t->slotWordTracking(); break;
        case 60: _t->slotBaselineOffset(); break;
        case 61: _t->slotFont((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 62: _t->slotParentChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 63: _t->slotCharParentChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 64: _t->slotFontFeatures(); break;
        case 65: _t->slotCharStylesDeleted((*reinterpret_cast< const QList<RemoveItem>(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SMParagraphStyle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SMParagraphStyle::selectionDirty)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SMParagraphStyle::staticMetaObject = { {
    QMetaObject::SuperData::link<StyleItem::staticMetaObject>(),
    qt_meta_stringdata_SMParagraphStyle.data,
    qt_meta_data_SMParagraphStyle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SMParagraphStyle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SMParagraphStyle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SMParagraphStyle.stringdata0))
        return static_cast<void*>(this);
    return StyleItem::qt_metacast(_clname);
}

int SMParagraphStyle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StyleItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 66)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 66;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 66)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 66;
    }
    return _id;
}

// SIGNAL 0
void SMParagraphStyle::selectionDirty()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_SMCharacterStyle_t {
    QByteArrayData data[30];
    char stringdata0[404];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SMCharacterStyle_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SMCharacterStyle_t qt_meta_stringdata_SMCharacterStyle = {
    {
QT_MOC_LITERAL(0, 0, 16), // "SMCharacterStyle"
QT_MOC_LITERAL(1, 17, 14), // "selectionDirty"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 17), // "charStylesDeleted"
QT_MOC_LITERAL(4, 51, 17), // "QList<RemoveItem>"
QT_MOC_LITERAL(5, 69, 10), // "removeList"
QT_MOC_LITERAL(6, 80, 12), // "slotFontSize"
QT_MOC_LITERAL(7, 93, 11), // "slotEffects"
QT_MOC_LITERAL(8, 105, 1), // "e"
QT_MOC_LITERAL(9, 107, 20), // "slotEffectProperties"
QT_MOC_LITERAL(10, 128, 13), // "slotFillColor"
QT_MOC_LITERAL(11, 142, 13), // "slotFillShade"
QT_MOC_LITERAL(12, 156, 13), // "slotBackColor"
QT_MOC_LITERAL(13, 170, 13), // "slotBackShade"
QT_MOC_LITERAL(14, 184, 15), // "slotStrokeColor"
QT_MOC_LITERAL(15, 200, 15), // "slotStrokeShade"
QT_MOC_LITERAL(16, 216, 12), // "slotLanguage"
QT_MOC_LITERAL(17, 229, 10), // "slotScaleH"
QT_MOC_LITERAL(18, 240, 10), // "slotScaleV"
QT_MOC_LITERAL(19, 251, 12), // "slotTracking"
QT_MOC_LITERAL(20, 264, 16), // "slotWordTracking"
QT_MOC_LITERAL(21, 281, 18), // "slotBaselineOffset"
QT_MOC_LITERAL(22, 300, 14), // "slotHyphenChar"
QT_MOC_LITERAL(23, 315, 16), // "slotSmallestWord"
QT_MOC_LITERAL(24, 332, 8), // "slotFont"
QT_MOC_LITERAL(25, 341, 1), // "s"
QT_MOC_LITERAL(26, 343, 17), // "slotParentChanged"
QT_MOC_LITERAL(27, 361, 6), // "parent"
QT_MOC_LITERAL(28, 368, 16), // "slotFontFeatures"
QT_MOC_LITERAL(29, 385, 18) // "slotSelectionDirty"

    },
    "SMCharacterStyle\0selectionDirty\0\0"
    "charStylesDeleted\0QList<RemoveItem>\0"
    "removeList\0slotFontSize\0slotEffects\0"
    "e\0slotEffectProperties\0slotFillColor\0"
    "slotFillShade\0slotBackColor\0slotBackShade\0"
    "slotStrokeColor\0slotStrokeShade\0"
    "slotLanguage\0slotScaleH\0slotScaleV\0"
    "slotTracking\0slotWordTracking\0"
    "slotBaselineOffset\0slotHyphenChar\0"
    "slotSmallestWord\0slotFont\0s\0"
    "slotParentChanged\0parent\0slotFontFeatures\0"
    "slotSelectionDirty"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SMCharacterStyle[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      23,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  129,    2, 0x06 /* Public */,
       3,    1,  130,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       6,    0,  133,    2, 0x08 /* Private */,
       7,    1,  134,    2, 0x08 /* Private */,
       9,    0,  137,    2, 0x08 /* Private */,
      10,    0,  138,    2, 0x08 /* Private */,
      11,    0,  139,    2, 0x08 /* Private */,
      12,    0,  140,    2, 0x08 /* Private */,
      13,    0,  141,    2, 0x08 /* Private */,
      14,    0,  142,    2, 0x08 /* Private */,
      15,    0,  143,    2, 0x08 /* Private */,
      16,    0,  144,    2, 0x08 /* Private */,
      17,    0,  145,    2, 0x08 /* Private */,
      18,    0,  146,    2, 0x08 /* Private */,
      19,    0,  147,    2, 0x08 /* Private */,
      20,    0,  148,    2, 0x08 /* Private */,
      21,    0,  149,    2, 0x08 /* Private */,
      22,    0,  150,    2, 0x08 /* Private */,
      23,    0,  151,    2, 0x08 /* Private */,
      24,    1,  152,    2, 0x08 /* Private */,
      26,    1,  155,    2, 0x08 /* Private */,
      28,    0,  158,    2, 0x08 /* Private */,
      29,    0,  159,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 4,    5,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   25,
    QMetaType::Void, QMetaType::QString,   27,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SMCharacterStyle::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SMCharacterStyle *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->selectionDirty(); break;
        case 1: _t->charStylesDeleted((*reinterpret_cast< const QList<RemoveItem>(*)>(_a[1]))); break;
        case 2: _t->slotFontSize(); break;
        case 3: _t->slotEffects((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->slotEffectProperties(); break;
        case 5: _t->slotFillColor(); break;
        case 6: _t->slotFillShade(); break;
        case 7: _t->slotBackColor(); break;
        case 8: _t->slotBackShade(); break;
        case 9: _t->slotStrokeColor(); break;
        case 10: _t->slotStrokeShade(); break;
        case 11: _t->slotLanguage(); break;
        case 12: _t->slotScaleH(); break;
        case 13: _t->slotScaleV(); break;
        case 14: _t->slotTracking(); break;
        case 15: _t->slotWordTracking(); break;
        case 16: _t->slotBaselineOffset(); break;
        case 17: _t->slotHyphenChar(); break;
        case 18: _t->slotSmallestWord(); break;
        case 19: _t->slotFont((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 20: _t->slotParentChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 21: _t->slotFontFeatures(); break;
        case 22: _t->slotSelectionDirty(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SMCharacterStyle::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SMCharacterStyle::selectionDirty)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SMCharacterStyle::*)(const QList<RemoveItem> & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SMCharacterStyle::charStylesDeleted)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SMCharacterStyle::staticMetaObject = { {
    QMetaObject::SuperData::link<StyleItem::staticMetaObject>(),
    qt_meta_stringdata_SMCharacterStyle.data,
    qt_meta_data_SMCharacterStyle,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SMCharacterStyle::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SMCharacterStyle::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SMCharacterStyle.stringdata0))
        return static_cast<void*>(this);
    return StyleItem::qt_metacast(_clname);
}

int SMCharacterStyle::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = StyleItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void SMCharacterStyle::selectionDirty()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void SMCharacterStyle::charStylesDeleted(const QList<RemoveItem> & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
