/********************************************************************************
** Form generated from reading UI file 'prefs_tableofcontentsbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_TABLEOFCONTENTSBASE_H
#define UI_PREFS_TABLEOFCONTENTSBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Prefs_TableOfContents
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QFrame *line;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_2;
    QListWidget *tocListBox;
    QLineEdit *tocNameLineEdit;
    QHBoxLayout *_2;
    QPushButton *tocAddButton;
    QPushButton *tocDeleteButton;
    QSpacerItem *verticalSpacer_2;
    QLabel *label_2;
    QFrame *line_2;
    QFormLayout *formLayout;
    QLabel *tocItemAttrLabel;
    QComboBox *itemAttrComboBox;
    QLabel *tocDestFrameLabel;
    QComboBox *itemDestFrameComboBox;
    QLabel *tocNumberPlacementLabel;
    QComboBox *itemNumberPlacementComboBox;
    QLabel *tocParagraphStyleLabel;
    QComboBox *itemParagraphStyleComboBox;
    QCheckBox *itemListNonPrintingCheckBox;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *Prefs_TableOfContents)
    {
        if (Prefs_TableOfContents->objectName().isEmpty())
            Prefs_TableOfContents->setObjectName(QString::fromUtf8("Prefs_TableOfContents"));
        Prefs_TableOfContents->resize(523, 607);
        verticalLayout = new QVBoxLayout(Prefs_TableOfContents);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(Prefs_TableOfContents);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        verticalLayout->addWidget(label);

        line = new QFrame(Prefs_TableOfContents);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        scrollArea = new QScrollArea(Prefs_TableOfContents);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 495, 536));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, -1);
        tocListBox = new QListWidget(scrollAreaWidgetContents);
        tocListBox->setObjectName(QString::fromUtf8("tocListBox"));

        verticalLayout_2->addWidget(tocListBox);

        tocNameLineEdit = new QLineEdit(scrollAreaWidgetContents);
        tocNameLineEdit->setObjectName(QString::fromUtf8("tocNameLineEdit"));

        verticalLayout_2->addWidget(tocNameLineEdit);

        _2 = new QHBoxLayout();
        _2->setObjectName(QString::fromUtf8("_2"));
        tocAddButton = new QPushButton(scrollAreaWidgetContents);
        tocAddButton->setObjectName(QString::fromUtf8("tocAddButton"));

        _2->addWidget(tocAddButton);

        tocDeleteButton = new QPushButton(scrollAreaWidgetContents);
        tocDeleteButton->setObjectName(QString::fromUtf8("tocDeleteButton"));

        _2->addWidget(tocDeleteButton);


        verticalLayout_2->addLayout(_2);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_2->addItem(verticalSpacer_2);

        label_2 = new QLabel(scrollAreaWidgetContents);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
        label_2->setFont(font1);

        verticalLayout_2->addWidget(label_2);

        line_2 = new QFrame(scrollAreaWidgetContents);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line_2);

        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        tocItemAttrLabel = new QLabel(scrollAreaWidgetContents);
        tocItemAttrLabel->setObjectName(QString::fromUtf8("tocItemAttrLabel"));
        tocItemAttrLabel->setWordWrap(false);

        formLayout->setWidget(0, QFormLayout::LabelRole, tocItemAttrLabel);

        itemAttrComboBox = new QComboBox(scrollAreaWidgetContents);
        itemAttrComboBox->setObjectName(QString::fromUtf8("itemAttrComboBox"));

        formLayout->setWidget(0, QFormLayout::FieldRole, itemAttrComboBox);

        tocDestFrameLabel = new QLabel(scrollAreaWidgetContents);
        tocDestFrameLabel->setObjectName(QString::fromUtf8("tocDestFrameLabel"));
        tocDestFrameLabel->setWordWrap(false);

        formLayout->setWidget(1, QFormLayout::LabelRole, tocDestFrameLabel);

        itemDestFrameComboBox = new QComboBox(scrollAreaWidgetContents);
        itemDestFrameComboBox->setObjectName(QString::fromUtf8("itemDestFrameComboBox"));

        formLayout->setWidget(1, QFormLayout::FieldRole, itemDestFrameComboBox);

        tocNumberPlacementLabel = new QLabel(scrollAreaWidgetContents);
        tocNumberPlacementLabel->setObjectName(QString::fromUtf8("tocNumberPlacementLabel"));
        tocNumberPlacementLabel->setWordWrap(false);

        formLayout->setWidget(2, QFormLayout::LabelRole, tocNumberPlacementLabel);

        itemNumberPlacementComboBox = new QComboBox(scrollAreaWidgetContents);
        itemNumberPlacementComboBox->setObjectName(QString::fromUtf8("itemNumberPlacementComboBox"));

        formLayout->setWidget(2, QFormLayout::FieldRole, itemNumberPlacementComboBox);

        tocParagraphStyleLabel = new QLabel(scrollAreaWidgetContents);
        tocParagraphStyleLabel->setObjectName(QString::fromUtf8("tocParagraphStyleLabel"));
        tocParagraphStyleLabel->setWordWrap(false);

        formLayout->setWidget(3, QFormLayout::LabelRole, tocParagraphStyleLabel);

        itemParagraphStyleComboBox = new QComboBox(scrollAreaWidgetContents);
        itemParagraphStyleComboBox->setObjectName(QString::fromUtf8("itemParagraphStyleComboBox"));

        formLayout->setWidget(3, QFormLayout::FieldRole, itemParagraphStyleComboBox);

        itemListNonPrintingCheckBox = new QCheckBox(scrollAreaWidgetContents);
        itemListNonPrintingCheckBox->setObjectName(QString::fromUtf8("itemListNonPrintingCheckBox"));

        formLayout->setWidget(4, QFormLayout::SpanningRole, itemListNonPrintingCheckBox);


        verticalLayout_2->addLayout(formLayout);

        verticalSpacer = new QSpacerItem(20, 93, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout->addWidget(scrollArea);


        retranslateUi(Prefs_TableOfContents);

        QMetaObject::connectSlotsByName(Prefs_TableOfContents);
    } // setupUi

    void retranslateUi(QWidget *Prefs_TableOfContents)
    {
        label->setText(QCoreApplication::translate("Prefs_TableOfContents", "Table of Contents", nullptr));
        tocAddButton->setText(QCoreApplication::translate("Prefs_TableOfContents", "&Add", nullptr));
#if QT_CONFIG(shortcut)
        tocAddButton->setShortcut(QCoreApplication::translate("Prefs_TableOfContents", "Alt+A", nullptr));
#endif // QT_CONFIG(shortcut)
        tocDeleteButton->setText(QCoreApplication::translate("Prefs_TableOfContents", "&Delete", nullptr));
#if QT_CONFIG(shortcut)
        tocDeleteButton->setShortcut(QCoreApplication::translate("Prefs_TableOfContents", "Alt+D", nullptr));
#endif // QT_CONFIG(shortcut)
        label_2->setText(QCoreApplication::translate("Prefs_TableOfContents", "Properties", nullptr));
        tocItemAttrLabel->setText(QCoreApplication::translate("Prefs_TableOfContents", "Item Attribute Name:", nullptr));
#if QT_CONFIG(tooltip)
        itemAttrComboBox->setToolTip(QCoreApplication::translate("Prefs_TableOfContents", "The Item Attribute that will be set on frames used as a basis for creation of the entries", nullptr));
#endif // QT_CONFIG(tooltip)
        tocDestFrameLabel->setText(QCoreApplication::translate("Prefs_TableOfContents", "Destination Frame:", nullptr));
#if QT_CONFIG(tooltip)
        itemDestFrameComboBox->setToolTip(QCoreApplication::translate("Prefs_TableOfContents", "The frame the table of contents will be placed into", nullptr));
#endif // QT_CONFIG(tooltip)
        tocNumberPlacementLabel->setText(QCoreApplication::translate("Prefs_TableOfContents", "Page Number Placement:", nullptr));
#if QT_CONFIG(tooltip)
        itemNumberPlacementComboBox->setToolTip(QCoreApplication::translate("Prefs_TableOfContents", "Place page numbers of the entries at the beginning or the end of the line, or not at all", nullptr));
#endif // QT_CONFIG(tooltip)
        tocParagraphStyleLabel->setText(QCoreApplication::translate("Prefs_TableOfContents", "Paragraph Style:", nullptr));
#if QT_CONFIG(tooltip)
        itemParagraphStyleComboBox->setToolTip(QCoreApplication::translate("Prefs_TableOfContents", "The paragraph style used for the entry lines", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        itemListNonPrintingCheckBox->setToolTip(QCoreApplication::translate("Prefs_TableOfContents", "Include frames that are set to not print as well", nullptr));
#endif // QT_CONFIG(tooltip)
        itemListNonPrintingCheckBox->setText(QCoreApplication::translate("Prefs_TableOfContents", "List Non-Printing Entries", nullptr));
        (void)Prefs_TableOfContents;
    } // retranslateUi

};

namespace Ui {
    class Prefs_TableOfContents: public Ui_Prefs_TableOfContents {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_TABLEOFCONTENTSBASE_H
