/********************************************************************************
** Form generated from reading UI file 'tabpdfoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TABPDFOPTIONS_H
#define UI_TABPDFOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/fontembeddingcombo.h"
#include "ui/pdfversioncombo.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_TabPDFOptions
{
public:
    QWidget *tabGeneral;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QGroupBox *RangeGroup;
    QVBoxLayout *rangeGroupLayout;
    QRadioButton *AllPages;
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout_2;
    QLineEdit *PageNr;
    QPushButton *pageNrButton;
    QRadioButton *OnlySome;
    QHBoxLayout *horizontalLayout_3;
    QLabel *rotationLabel;
    QComboBox *RotateDeg;
    QHBoxLayout *horizontalLayout_4;
    QToolButton *MirrorH;
    QToolButton *MirrorV;
    QCheckBox *ClipMarg;
    QGroupBox *groupBox_2;
    QGridLayout *fileOptionsLayout;
    QLabel *label;
    PdfVersionCombo *PDFVersionCombo;
    QLabel *label_2;
    QComboBox *ComboBind;
    QCheckBox *CheckBox1;
    QSpinBox *Resolution;
    QCheckBox *CheckBM;
    QCheckBox *Article;
    QCheckBox *useLayers;
    QLabel *label_3;
    QCheckBox *EmbedPDF;
    QCheckBox *Compression;
    QGroupBox *groupBox;
    QGridLayout *imageQualityLayout;
    QLabel *label_4;
    QComboBox *CMethod;
    QLabel *label_5;
    QComboBox *CQuality;
    QSpinBox *ValC;
    QCheckBox *DSColor;
    QSpacerItem *verticalSpacer_6;
    QWidget *tabFonts;
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupFont;
    QVBoxLayout *groupFontLayout;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_6;
    FontEmbeddingCombo *fontEmbeddingCombo;
    QHBoxLayout *horizontalLayout_9;
    QVBoxLayout *verticalLayout_6;
    QLabel *label_8;
    QListWidget *EmbedList;
    QPushButton *EmbedFonts;
    QVBoxLayout *verticalLayout_8;
    QSpacerItem *verticalSpacer;
    QPushButton *ToSubset;
    QPushButton *FromSubset;
    QSpacerItem *verticalSpacer_2;
    QVBoxLayout *verticalLayout_5;
    QLabel *label_9;
    QListWidget *SubsetList;
    QPushButton *SubsetFonts;
    QWidget *tabPresentation;
    QGridLayout *gridLayout_4;
    QCheckBox *CheckBox10;
    QListWidget *Pages;
    QCheckBox *PagePrev;
    QGroupBox *Effects;
    QGridLayout *effectsLayout;
    QComboBox *EDirection_2_2;
    QLabel *label_15;
    QLabel *label_13;
    QComboBox *EDirection;
    QComboBox *EffectType;
    QLabel *label_14;
    QComboBox *EDirection_2;
    QLabel *label_11;
    QLabel *label_12;
    QLabel *label_10;
    QSpinBox *EffectTime;
    QSpinBox *PageTime;
    QPushButton *EonAllPg;
    QWidget *tabSpecial;
    QVBoxLayout *verticalLayout_7;
    QGroupBox *groupDisplay;
    QVBoxLayout *groupDisplayLayout;
    QHBoxLayout *horizontalLayout_7;
    QGroupBox *pageLayout;
    QVBoxLayout *pageLayoutLayout;
    QRadioButton *singlePage;
    QRadioButton *continuousPages;
    QRadioButton *facingPagesLeft;
    QRadioButton *facingPagesRight;
    QGroupBox *groupNavigation;
    QVBoxLayout *groupNavigationLayout;
    QRadioButton *useViewDefault;
    QRadioButton *useFullScreen;
    QRadioButton *useBookmarks;
    QRadioButton *useThumbnails;
    QRadioButton *useLayers2;
    QCheckBox *hideToolBar;
    QCheckBox *hideMenuBar;
    QCheckBox *fitWindow;
    QGroupBox *groupJava;
    QHBoxLayout *groupJavaLayout;
    QLabel *actionLabel;
    QComboBox *actionCombo;
    QWidget *tabSecurity;
    QVBoxLayout *verticalLayout_9;
    QCheckBox *Encry;
    QGroupBox *groupPass;
    QGridLayout *groupPassLayout;
    QLabel *TextSec2;
    QLabel *TextSec1;
    QLineEdit *PassOwner;
    QLineEdit *PassUser;
    QGroupBox *groupSecSet;
    QVBoxLayout *groupSecSetLayout;
    QCheckBox *PrintSec;
    QCheckBox *ModifySec;
    QCheckBox *CopySec;
    QCheckBox *AddSec;
    QSpacerItem *verticalSpacer_5;
    QWidget *tabColor;
    QVBoxLayout *tabColorLayout;
    QGroupBox *ColorGroup;
    QHBoxLayout *colorGroupLayout;
    QLabel *label_16;
    QComboBox *OutCombo;
    QCheckBox *useSpot;
    QCheckBox *UseLPI;
    QGroupBox *LPIgroup;
    QGridLayout *lpiGroupLayout;
    QComboBox *LPIcolor;
    QHBoxLayout *horizontalLayout_10;
    QLabel *label_17;
    QSpinBox *LPIfreq;
    QHBoxLayout *horizontalLayout_11;
    QLabel *label_18;
    QSpinBox *LPIangle;
    QHBoxLayout *horizontalLayout_12;
    QLabel *label_19;
    QComboBox *LPIfunc;
    QGroupBox *solidsProfileGroup;
    QGridLayout *solidColorsGroupLayout;
    QLabel *solidsProfileLabel;
    QCheckBox *EmbedProfs;
    QLabel *solidsIntentLabel;
    QComboBox *SolidPr;
    QComboBox *IntendS;
    QGroupBox *imageProfileGroup;
    QGridLayout *imageProfileGroupLayout;
    QCheckBox *EmbedProfs2;
    QLabel *imageProfileLabel;
    QLabel *imageIntentLabel;
    QCheckBox *NoEmbedded;
    QComboBox *ImageP;
    QComboBox *IntendI;
    QSpacerItem *verticalSpacer_3;
    QWidget *tabPDFX;
    QVBoxLayout *tabPDFXLayout;
    QGroupBox *MarkGroup;
    QGridLayout *markGroupLayout;
    QCheckBox *colorMarks;
    QCheckBox *registrationMarks;
    QCheckBox *bleedMarks;
    QCheckBox *cropMarks;
    QHBoxLayout *horizontalLayout_6;
    QLabel *label_25;
    ScrSpinBox *markOffset;
    QHBoxLayout *horizontalLayout_8;
    QLabel *label_24;
    ScrSpinBox *markLength;
    QCheckBox *docInfoMarks;
    QGroupBox *BleedGroup;
    QGridLayout *bleedGroupLayout;
    QLabel *bleedTopLabel;
    ScrSpinBox *BleedTop;
    QLabel *bleedBottomLabel;
    ScrSpinBox *BleedBottom;
    ScrSpinBox *BleedRight;
    QLabel *bleedLeftLabel;
    ScrSpinBox *BleedLeft;
    QLabel *bleedRightLabel;
    QCheckBox *docBleeds;
    QGroupBox *X3Group;
    QGridLayout *X3GroupLayout;
    QComboBox *PrintProfC;
    QLabel *label_30;
    QLabel *label_31;
    QLineEdit *InfoString;
    QSpacerItem *verticalSpacer_4;

    void setupUi(QTabWidget *TabPDFOptions)
    {
        if (TabPDFOptions->objectName().isEmpty())
            TabPDFOptions->setObjectName(QString::fromUtf8("TabPDFOptions"));
        TabPDFOptions->resize(657, 549);
        tabGeneral = new QWidget();
        tabGeneral->setObjectName(QString::fromUtf8("tabGeneral"));
        verticalLayout = new QVBoxLayout(tabGeneral);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        RangeGroup = new QGroupBox(tabGeneral);
        RangeGroup->setObjectName(QString::fromUtf8("RangeGroup"));
        rangeGroupLayout = new QVBoxLayout(RangeGroup);
        rangeGroupLayout->setObjectName(QString::fromUtf8("rangeGroupLayout"));
        AllPages = new QRadioButton(RangeGroup);
        AllPages->setObjectName(QString::fromUtf8("AllPages"));

        rangeGroupLayout->addWidget(AllPages);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        PageNr = new QLineEdit(RangeGroup);
        PageNr->setObjectName(QString::fromUtf8("PageNr"));

        horizontalLayout_2->addWidget(PageNr);

        pageNrButton = new QPushButton(RangeGroup);
        pageNrButton->setObjectName(QString::fromUtf8("pageNrButton"));

        horizontalLayout_2->addWidget(pageNrButton);


        gridLayout->addLayout(horizontalLayout_2, 1, 0, 1, 1);

        OnlySome = new QRadioButton(RangeGroup);
        OnlySome->setObjectName(QString::fromUtf8("OnlySome"));

        gridLayout->addWidget(OnlySome, 0, 0, 1, 1);


        rangeGroupLayout->addLayout(gridLayout);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        rotationLabel = new QLabel(RangeGroup);
        rotationLabel->setObjectName(QString::fromUtf8("rotationLabel"));

        horizontalLayout_3->addWidget(rotationLabel);

        RotateDeg = new QComboBox(RangeGroup);
        RotateDeg->addItem(QString());
        RotateDeg->addItem(QString());
        RotateDeg->addItem(QString());
        RotateDeg->addItem(QString());
        RotateDeg->setObjectName(QString::fromUtf8("RotateDeg"));
        RotateDeg->setEditable(false);

        horizontalLayout_3->addWidget(RotateDeg);


        rangeGroupLayout->addLayout(horizontalLayout_3);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        MirrorH = new QToolButton(RangeGroup);
        MirrorH->setObjectName(QString::fromUtf8("MirrorH"));
        MirrorH->setCheckable(true);

        horizontalLayout_4->addWidget(MirrorH);

        MirrorV = new QToolButton(RangeGroup);
        MirrorV->setObjectName(QString::fromUtf8("MirrorV"));
        MirrorV->setCheckable(true);

        horizontalLayout_4->addWidget(MirrorV);


        rangeGroupLayout->addLayout(horizontalLayout_4);

        ClipMarg = new QCheckBox(RangeGroup);
        ClipMarg->setObjectName(QString::fromUtf8("ClipMarg"));

        rangeGroupLayout->addWidget(ClipMarg);


        horizontalLayout->addWidget(RangeGroup);

        groupBox_2 = new QGroupBox(tabGeneral);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        groupBox_2->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        fileOptionsLayout = new QGridLayout(groupBox_2);
        fileOptionsLayout->setObjectName(QString::fromUtf8("fileOptionsLayout"));
        label = new QLabel(groupBox_2);
        label->setObjectName(QString::fromUtf8("label"));
        label->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        fileOptionsLayout->addWidget(label, 0, 0, 1, 1);

        PDFVersionCombo = new PdfVersionCombo(groupBox_2);
        PDFVersionCombo->setObjectName(QString::fromUtf8("PDFVersionCombo"));
        PDFVersionCombo->setEditable(false);

        fileOptionsLayout->addWidget(PDFVersionCombo, 0, 1, 1, 2);

        label_2 = new QLabel(groupBox_2);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        fileOptionsLayout->addWidget(label_2, 1, 0, 1, 1);

        ComboBind = new QComboBox(groupBox_2);
        ComboBind->addItem(QString());
        ComboBind->addItem(QString());
        ComboBind->setObjectName(QString::fromUtf8("ComboBind"));
        ComboBind->setEditable(false);

        fileOptionsLayout->addWidget(ComboBind, 1, 1, 1, 2);

        CheckBox1 = new QCheckBox(groupBox_2);
        CheckBox1->setObjectName(QString::fromUtf8("CheckBox1"));

        fileOptionsLayout->addWidget(CheckBox1, 2, 0, 1, 3);

        Resolution = new QSpinBox(groupBox_2);
        Resolution->setObjectName(QString::fromUtf8("Resolution"));
        Resolution->setMinimum(35);
        Resolution->setMaximum(4000);

        fileOptionsLayout->addWidget(Resolution, 6, 1, 1, 1, Qt::AlignLeft);

        CheckBM = new QCheckBox(groupBox_2);
        CheckBM->setObjectName(QString::fromUtf8("CheckBM"));

        fileOptionsLayout->addWidget(CheckBM, 4, 0, 1, 3);

        Article = new QCheckBox(groupBox_2);
        Article->setObjectName(QString::fromUtf8("Article"));

        fileOptionsLayout->addWidget(Article, 3, 0, 1, 3);

        useLayers = new QCheckBox(groupBox_2);
        useLayers->setObjectName(QString::fromUtf8("useLayers"));

        fileOptionsLayout->addWidget(useLayers, 5, 0, 1, 3);

        label_3 = new QLabel(groupBox_2);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        label_3->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        fileOptionsLayout->addWidget(label_3, 6, 0, 1, 1);

        EmbedPDF = new QCheckBox(groupBox_2);
        EmbedPDF->setObjectName(QString::fromUtf8("EmbedPDF"));

        fileOptionsLayout->addWidget(EmbedPDF, 7, 0, 1, 3);


        horizontalLayout->addWidget(groupBox_2);


        verticalLayout->addLayout(horizontalLayout);

        Compression = new QCheckBox(tabGeneral);
        Compression->setObjectName(QString::fromUtf8("Compression"));

        verticalLayout->addWidget(Compression);

        groupBox = new QGroupBox(tabGeneral);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        imageQualityLayout = new QGridLayout(groupBox);
        imageQualityLayout->setObjectName(QString::fromUtf8("imageQualityLayout"));
        label_4 = new QLabel(groupBox);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        imageQualityLayout->addWidget(label_4, 0, 0, 1, 1);

        CMethod = new QComboBox(groupBox);
        CMethod->addItem(QString());
        CMethod->addItem(QString());
        CMethod->addItem(QString());
        CMethod->addItem(QString());
        CMethod->setObjectName(QString::fromUtf8("CMethod"));
        CMethod->setEditable(false);

        imageQualityLayout->addWidget(CMethod, 0, 1, 1, 1);

        label_5 = new QLabel(groupBox);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        imageQualityLayout->addWidget(label_5, 1, 0, 1, 1);

        CQuality = new QComboBox(groupBox);
        CQuality->addItem(QString());
        CQuality->addItem(QString());
        CQuality->addItem(QString());
        CQuality->addItem(QString());
        CQuality->addItem(QString());
        CQuality->setObjectName(QString::fromUtf8("CQuality"));
        CQuality->setEditable(false);

        imageQualityLayout->addWidget(CQuality, 1, 1, 1, 1);

        ValC = new QSpinBox(groupBox);
        ValC->setObjectName(QString::fromUtf8("ValC"));
        ValC->setMinimum(35);
        ValC->setMaximum(4000);
        ValC->setValue(72);

        imageQualityLayout->addWidget(ValC, 2, 1, 1, 1, Qt::AlignLeft);

        DSColor = new QCheckBox(groupBox);
        DSColor->setObjectName(QString::fromUtf8("DSColor"));

        imageQualityLayout->addWidget(DSColor, 2, 0, 1, 1);


        verticalLayout->addWidget(groupBox);

        verticalSpacer_6 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer_6);

        TabPDFOptions->addTab(tabGeneral, QString());
        tabFonts = new QWidget();
        tabFonts->setObjectName(QString::fromUtf8("tabFonts"));
        verticalLayout_2 = new QVBoxLayout(tabFonts);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        groupFont = new QGroupBox(tabFonts);
        groupFont->setObjectName(QString::fromUtf8("groupFont"));
        groupFontLayout = new QVBoxLayout(groupFont);
        groupFontLayout->setObjectName(QString::fromUtf8("groupFontLayout"));
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        label_6 = new QLabel(groupFont);
        label_6->setObjectName(QString::fromUtf8("label_6"));
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy);

        horizontalLayout_5->addWidget(label_6);

        fontEmbeddingCombo = new FontEmbeddingCombo(groupFont);
        fontEmbeddingCombo->setObjectName(QString::fromUtf8("fontEmbeddingCombo"));

        horizontalLayout_5->addWidget(fontEmbeddingCombo);


        groupFontLayout->addLayout(horizontalLayout_5);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName(QString::fromUtf8("horizontalLayout_9"));
        verticalLayout_6 = new QVBoxLayout();
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        label_8 = new QLabel(groupFont);
        label_8->setObjectName(QString::fromUtf8("label_8"));

        verticalLayout_6->addWidget(label_8);

        EmbedList = new QListWidget(groupFont);
        EmbedList->setObjectName(QString::fromUtf8("EmbedList"));

        verticalLayout_6->addWidget(EmbedList);

        EmbedFonts = new QPushButton(groupFont);
        EmbedFonts->setObjectName(QString::fromUtf8("EmbedFonts"));

        verticalLayout_6->addWidget(EmbedFonts);


        horizontalLayout_9->addLayout(verticalLayout_6);

        verticalLayout_8 = new QVBoxLayout();
        verticalLayout_8->setObjectName(QString::fromUtf8("verticalLayout_8"));
        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_8->addItem(verticalSpacer);

        ToSubset = new QPushButton(groupFont);
        ToSubset->setObjectName(QString::fromUtf8("ToSubset"));

        verticalLayout_8->addWidget(ToSubset);

        FromSubset = new QPushButton(groupFont);
        FromSubset->setObjectName(QString::fromUtf8("FromSubset"));

        verticalLayout_8->addWidget(FromSubset);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_8->addItem(verticalSpacer_2);


        horizontalLayout_9->addLayout(verticalLayout_8);

        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        label_9 = new QLabel(groupFont);
        label_9->setObjectName(QString::fromUtf8("label_9"));

        verticalLayout_5->addWidget(label_9);

        SubsetList = new QListWidget(groupFont);
        SubsetList->setObjectName(QString::fromUtf8("SubsetList"));

        verticalLayout_5->addWidget(SubsetList);

        SubsetFonts = new QPushButton(groupFont);
        SubsetFonts->setObjectName(QString::fromUtf8("SubsetFonts"));

        verticalLayout_5->addWidget(SubsetFonts);


        horizontalLayout_9->addLayout(verticalLayout_5);


        groupFontLayout->addLayout(horizontalLayout_9);


        verticalLayout_2->addWidget(groupFont);

        TabPDFOptions->addTab(tabFonts, QString());
        tabPresentation = new QWidget();
        tabPresentation->setObjectName(QString::fromUtf8("tabPresentation"));
        gridLayout_4 = new QGridLayout(tabPresentation);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        CheckBox10 = new QCheckBox(tabPresentation);
        CheckBox10->setObjectName(QString::fromUtf8("CheckBox10"));

        gridLayout_4->addWidget(CheckBox10, 0, 0, 1, 1);

        Pages = new QListWidget(tabPresentation);
        Pages->setObjectName(QString::fromUtf8("Pages"));

        gridLayout_4->addWidget(Pages, 1, 0, 1, 1);

        PagePrev = new QCheckBox(tabPresentation);
        PagePrev->setObjectName(QString::fromUtf8("PagePrev"));

        gridLayout_4->addWidget(PagePrev, 2, 0, 1, 1);

        Effects = new QGroupBox(tabPresentation);
        Effects->setObjectName(QString::fromUtf8("Effects"));
        effectsLayout = new QGridLayout(Effects);
        effectsLayout->setObjectName(QString::fromUtf8("effectsLayout"));
        EDirection_2_2 = new QComboBox(Effects);
        EDirection_2_2->addItem(QString());
        EDirection_2_2->addItem(QString());
        EDirection_2_2->addItem(QString());
        EDirection_2_2->addItem(QString());
        EDirection_2_2->addItem(QString());
        EDirection_2_2->setObjectName(QString::fromUtf8("EDirection_2_2"));
        EDirection_2_2->setEditable(false);

        effectsLayout->addWidget(EDirection_2_2, 5, 1, 1, 1);

        label_15 = new QLabel(Effects);
        label_15->setObjectName(QString::fromUtf8("label_15"));

        effectsLayout->addWidget(label_15, 5, 0, 1, 1);

        label_13 = new QLabel(Effects);
        label_13->setObjectName(QString::fromUtf8("label_13"));

        effectsLayout->addWidget(label_13, 3, 0, 1, 1);

        EDirection = new QComboBox(Effects);
        EDirection->addItem(QString());
        EDirection->addItem(QString());
        EDirection->setObjectName(QString::fromUtf8("EDirection"));
        EDirection->setEditable(false);

        effectsLayout->addWidget(EDirection, 3, 1, 1, 1);

        EffectType = new QComboBox(Effects);
        EffectType->addItem(QString());
        EffectType->addItem(QString());
        EffectType->addItem(QString());
        EffectType->addItem(QString());
        EffectType->addItem(QString());
        EffectType->addItem(QString());
        EffectType->addItem(QString());
        EffectType->setObjectName(QString::fromUtf8("EffectType"));
        EffectType->setEditable(false);

        effectsLayout->addWidget(EffectType, 2, 1, 1, 1);

        label_14 = new QLabel(Effects);
        label_14->setObjectName(QString::fromUtf8("label_14"));

        effectsLayout->addWidget(label_14, 4, 0, 1, 1);

        EDirection_2 = new QComboBox(Effects);
        EDirection_2->addItem(QString());
        EDirection_2->addItem(QString());
        EDirection_2->setObjectName(QString::fromUtf8("EDirection_2"));
        EDirection_2->setEditable(false);

        effectsLayout->addWidget(EDirection_2, 4, 1, 1, 1);

        label_11 = new QLabel(Effects);
        label_11->setObjectName(QString::fromUtf8("label_11"));

        effectsLayout->addWidget(label_11, 1, 0, 1, 1);

        label_12 = new QLabel(Effects);
        label_12->setObjectName(QString::fromUtf8("label_12"));

        effectsLayout->addWidget(label_12, 2, 0, 1, 1);

        label_10 = new QLabel(Effects);
        label_10->setObjectName(QString::fromUtf8("label_10"));

        effectsLayout->addWidget(label_10, 0, 0, 1, 1);

        EffectTime = new QSpinBox(Effects);
        EffectTime->setObjectName(QString::fromUtf8("EffectTime"));
        EffectTime->setMinimum(1);
        EffectTime->setMaximum(3600);

        effectsLayout->addWidget(EffectTime, 1, 1, 1, 1);

        PageTime = new QSpinBox(Effects);
        PageTime->setObjectName(QString::fromUtf8("PageTime"));
        PageTime->setMinimum(0);
        PageTime->setMaximum(3600);

        effectsLayout->addWidget(PageTime, 0, 1, 1, 1);

        EonAllPg = new QPushButton(Effects);
        EonAllPg->setObjectName(QString::fromUtf8("EonAllPg"));

        effectsLayout->addWidget(EonAllPg, 6, 0, 1, 2);


        gridLayout_4->addWidget(Effects, 1, 1, 1, 1);

        TabPDFOptions->addTab(tabPresentation, QString());
        tabSpecial = new QWidget();
        tabSpecial->setObjectName(QString::fromUtf8("tabSpecial"));
        verticalLayout_7 = new QVBoxLayout(tabSpecial);
        verticalLayout_7->setObjectName(QString::fromUtf8("verticalLayout_7"));
        groupDisplay = new QGroupBox(tabSpecial);
        groupDisplay->setObjectName(QString::fromUtf8("groupDisplay"));
        groupDisplayLayout = new QVBoxLayout(groupDisplay);
        groupDisplayLayout->setObjectName(QString::fromUtf8("groupDisplayLayout"));
        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        horizontalLayout_7->setContentsMargins(0, 0, 0, 0);
        pageLayout = new QGroupBox(groupDisplay);
        pageLayout->setObjectName(QString::fromUtf8("pageLayout"));
        pageLayoutLayout = new QVBoxLayout(pageLayout);
        pageLayoutLayout->setObjectName(QString::fromUtf8("pageLayoutLayout"));
        singlePage = new QRadioButton(pageLayout);
        singlePage->setObjectName(QString::fromUtf8("singlePage"));

        pageLayoutLayout->addWidget(singlePage);

        continuousPages = new QRadioButton(pageLayout);
        continuousPages->setObjectName(QString::fromUtf8("continuousPages"));

        pageLayoutLayout->addWidget(continuousPages);

        facingPagesLeft = new QRadioButton(pageLayout);
        facingPagesLeft->setObjectName(QString::fromUtf8("facingPagesLeft"));

        pageLayoutLayout->addWidget(facingPagesLeft);

        facingPagesRight = new QRadioButton(pageLayout);
        facingPagesRight->setObjectName(QString::fromUtf8("facingPagesRight"));

        pageLayoutLayout->addWidget(facingPagesRight);


        horizontalLayout_7->addWidget(pageLayout);

        groupNavigation = new QGroupBox(groupDisplay);
        groupNavigation->setObjectName(QString::fromUtf8("groupNavigation"));
        groupNavigationLayout = new QVBoxLayout(groupNavigation);
        groupNavigationLayout->setObjectName(QString::fromUtf8("groupNavigationLayout"));
        useViewDefault = new QRadioButton(groupNavigation);
        useViewDefault->setObjectName(QString::fromUtf8("useViewDefault"));

        groupNavigationLayout->addWidget(useViewDefault);

        useFullScreen = new QRadioButton(groupNavigation);
        useFullScreen->setObjectName(QString::fromUtf8("useFullScreen"));

        groupNavigationLayout->addWidget(useFullScreen);

        useBookmarks = new QRadioButton(groupNavigation);
        useBookmarks->setObjectName(QString::fromUtf8("useBookmarks"));

        groupNavigationLayout->addWidget(useBookmarks);

        useThumbnails = new QRadioButton(groupNavigation);
        useThumbnails->setObjectName(QString::fromUtf8("useThumbnails"));

        groupNavigationLayout->addWidget(useThumbnails);

        useLayers2 = new QRadioButton(groupNavigation);
        useLayers2->setObjectName(QString::fromUtf8("useLayers2"));

        groupNavigationLayout->addWidget(useLayers2);

        hideToolBar = new QCheckBox(groupNavigation);
        hideToolBar->setObjectName(QString::fromUtf8("hideToolBar"));

        groupNavigationLayout->addWidget(hideToolBar);

        hideMenuBar = new QCheckBox(groupNavigation);
        hideMenuBar->setObjectName(QString::fromUtf8("hideMenuBar"));

        groupNavigationLayout->addWidget(hideMenuBar);

        fitWindow = new QCheckBox(groupNavigation);
        fitWindow->setObjectName(QString::fromUtf8("fitWindow"));

        groupNavigationLayout->addWidget(fitWindow);


        horizontalLayout_7->addWidget(groupNavigation);


        groupDisplayLayout->addLayout(horizontalLayout_7);

        groupJava = new QGroupBox(groupDisplay);
        groupJava->setObjectName(QString::fromUtf8("groupJava"));
        groupJavaLayout = new QHBoxLayout(groupJava);
        groupJavaLayout->setObjectName(QString::fromUtf8("groupJavaLayout"));
        actionLabel = new QLabel(groupJava);
        actionLabel->setObjectName(QString::fromUtf8("actionLabel"));

        groupJavaLayout->addWidget(actionLabel);

        actionCombo = new QComboBox(groupJava);
        actionCombo->addItem(QString());
        actionCombo->setObjectName(QString::fromUtf8("actionCombo"));
        actionCombo->setEditable(false);

        groupJavaLayout->addWidget(actionCombo);


        groupDisplayLayout->addWidget(groupJava);


        verticalLayout_7->addWidget(groupDisplay);

        TabPDFOptions->addTab(tabSpecial, QString());
        tabSecurity = new QWidget();
        tabSecurity->setObjectName(QString::fromUtf8("tabSecurity"));
        verticalLayout_9 = new QVBoxLayout(tabSecurity);
        verticalLayout_9->setObjectName(QString::fromUtf8("verticalLayout_9"));
        Encry = new QCheckBox(tabSecurity);
        Encry->setObjectName(QString::fromUtf8("Encry"));

        verticalLayout_9->addWidget(Encry);

        groupPass = new QGroupBox(tabSecurity);
        groupPass->setObjectName(QString::fromUtf8("groupPass"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(groupPass->sizePolicy().hasHeightForWidth());
        groupPass->setSizePolicy(sizePolicy1);
        groupPassLayout = new QGridLayout(groupPass);
        groupPassLayout->setObjectName(QString::fromUtf8("groupPassLayout"));
        TextSec2 = new QLabel(groupPass);
        TextSec2->setObjectName(QString::fromUtf8("TextSec2"));

        groupPassLayout->addWidget(TextSec2, 1, 0, 1, 1);

        TextSec1 = new QLabel(groupPass);
        TextSec1->setObjectName(QString::fromUtf8("TextSec1"));

        groupPassLayout->addWidget(TextSec1, 0, 0, 1, 1);

        PassOwner = new QLineEdit(groupPass);
        PassOwner->setObjectName(QString::fromUtf8("PassOwner"));
        PassOwner->setEchoMode(QLineEdit::Password);

        groupPassLayout->addWidget(PassOwner, 0, 1, 1, 1);

        PassUser = new QLineEdit(groupPass);
        PassUser->setObjectName(QString::fromUtf8("PassUser"));
        PassUser->setEchoMode(QLineEdit::Password);

        groupPassLayout->addWidget(PassUser, 1, 1, 1, 1);


        verticalLayout_9->addWidget(groupPass);

        groupSecSet = new QGroupBox(tabSecurity);
        groupSecSet->setObjectName(QString::fromUtf8("groupSecSet"));
        sizePolicy1.setHeightForWidth(groupSecSet->sizePolicy().hasHeightForWidth());
        groupSecSet->setSizePolicy(sizePolicy1);
        groupSecSetLayout = new QVBoxLayout(groupSecSet);
        groupSecSetLayout->setObjectName(QString::fromUtf8("groupSecSetLayout"));
        PrintSec = new QCheckBox(groupSecSet);
        PrintSec->setObjectName(QString::fromUtf8("PrintSec"));

        groupSecSetLayout->addWidget(PrintSec);

        ModifySec = new QCheckBox(groupSecSet);
        ModifySec->setObjectName(QString::fromUtf8("ModifySec"));

        groupSecSetLayout->addWidget(ModifySec);

        CopySec = new QCheckBox(groupSecSet);
        CopySec->setObjectName(QString::fromUtf8("CopySec"));

        groupSecSetLayout->addWidget(CopySec);

        AddSec = new QCheckBox(groupSecSet);
        AddSec->setObjectName(QString::fromUtf8("AddSec"));

        groupSecSetLayout->addWidget(AddSec);


        verticalLayout_9->addWidget(groupSecSet);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_9->addItem(verticalSpacer_5);

        TabPDFOptions->addTab(tabSecurity, QString());
        tabColor = new QWidget();
        tabColor->setObjectName(QString::fromUtf8("tabColor"));
        tabColorLayout = new QVBoxLayout(tabColor);
        tabColorLayout->setObjectName(QString::fromUtf8("tabColorLayout"));
        ColorGroup = new QGroupBox(tabColor);
        ColorGroup->setObjectName(QString::fromUtf8("ColorGroup"));
        colorGroupLayout = new QHBoxLayout(ColorGroup);
        colorGroupLayout->setObjectName(QString::fromUtf8("colorGroupLayout"));
        label_16 = new QLabel(ColorGroup);
        label_16->setObjectName(QString::fromUtf8("label_16"));

        colorGroupLayout->addWidget(label_16);

        OutCombo = new QComboBox(ColorGroup);
        OutCombo->addItem(QString());
        OutCombo->addItem(QString());
        OutCombo->addItem(QString());
        OutCombo->setObjectName(QString::fromUtf8("OutCombo"));

        colorGroupLayout->addWidget(OutCombo);


        tabColorLayout->addWidget(ColorGroup);

        useSpot = new QCheckBox(tabColor);
        useSpot->setObjectName(QString::fromUtf8("useSpot"));

        tabColorLayout->addWidget(useSpot);

        UseLPI = new QCheckBox(tabColor);
        UseLPI->setObjectName(QString::fromUtf8("UseLPI"));

        tabColorLayout->addWidget(UseLPI);

        LPIgroup = new QGroupBox(tabColor);
        LPIgroup->setObjectName(QString::fromUtf8("LPIgroup"));
        lpiGroupLayout = new QGridLayout(LPIgroup);
        lpiGroupLayout->setObjectName(QString::fromUtf8("lpiGroupLayout"));
        LPIcolor = new QComboBox(LPIgroup);
        LPIcolor->setObjectName(QString::fromUtf8("LPIcolor"));

        lpiGroupLayout->addWidget(LPIcolor, 0, 0, 1, 1);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName(QString::fromUtf8("horizontalLayout_10"));
        label_17 = new QLabel(LPIgroup);
        label_17->setObjectName(QString::fromUtf8("label_17"));

        horizontalLayout_10->addWidget(label_17);

        LPIfreq = new QSpinBox(LPIgroup);
        LPIfreq->setObjectName(QString::fromUtf8("LPIfreq"));
        LPIfreq->setMinimum(10);
        LPIfreq->setMaximum(1000);

        horizontalLayout_10->addWidget(LPIfreq);


        lpiGroupLayout->addLayout(horizontalLayout_10, 0, 1, 1, 1);

        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName(QString::fromUtf8("horizontalLayout_11"));
        label_18 = new QLabel(LPIgroup);
        label_18->setObjectName(QString::fromUtf8("label_18"));

        horizontalLayout_11->addWidget(label_18);

        LPIangle = new QSpinBox(LPIgroup);
        LPIangle->setObjectName(QString::fromUtf8("LPIangle"));
        LPIangle->setMinimum(-180);
        LPIangle->setMaximum(180);

        horizontalLayout_11->addWidget(LPIangle);


        lpiGroupLayout->addLayout(horizontalLayout_11, 1, 1, 1, 1);

        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName(QString::fromUtf8("horizontalLayout_12"));
        label_19 = new QLabel(LPIgroup);
        label_19->setObjectName(QString::fromUtf8("label_19"));

        horizontalLayout_12->addWidget(label_19);

        LPIfunc = new QComboBox(LPIgroup);
        LPIfunc->addItem(QString());
        LPIfunc->addItem(QString());
        LPIfunc->addItem(QString());
        LPIfunc->addItem(QString());
        LPIfunc->setObjectName(QString::fromUtf8("LPIfunc"));
        LPIfunc->setEditable(false);

        horizontalLayout_12->addWidget(LPIfunc);


        lpiGroupLayout->addLayout(horizontalLayout_12, 2, 1, 1, 1);


        tabColorLayout->addWidget(LPIgroup);

        solidsProfileGroup = new QGroupBox(tabColor);
        solidsProfileGroup->setObjectName(QString::fromUtf8("solidsProfileGroup"));
        solidColorsGroupLayout = new QGridLayout(solidsProfileGroup);
        solidColorsGroupLayout->setObjectName(QString::fromUtf8("solidColorsGroupLayout"));
        solidsProfileLabel = new QLabel(solidsProfileGroup);
        solidsProfileLabel->setObjectName(QString::fromUtf8("solidsProfileLabel"));

        solidColorsGroupLayout->addWidget(solidsProfileLabel, 1, 0, 1, 1);

        EmbedProfs = new QCheckBox(solidsProfileGroup);
        EmbedProfs->setObjectName(QString::fromUtf8("EmbedProfs"));

        solidColorsGroupLayout->addWidget(EmbedProfs, 0, 0, 1, 1);

        solidsIntentLabel = new QLabel(solidsProfileGroup);
        solidsIntentLabel->setObjectName(QString::fromUtf8("solidsIntentLabel"));

        solidColorsGroupLayout->addWidget(solidsIntentLabel, 1, 1, 1, 1);

        SolidPr = new QComboBox(solidsProfileGroup);
        SolidPr->setObjectName(QString::fromUtf8("SolidPr"));

        solidColorsGroupLayout->addWidget(SolidPr, 2, 0, 1, 1);

        IntendS = new QComboBox(solidsProfileGroup);
        IntendS->addItem(QString());
        IntendS->addItem(QString());
        IntendS->addItem(QString());
        IntendS->addItem(QString());
        IntendS->setObjectName(QString::fromUtf8("IntendS"));
        IntendS->setEditable(false);

        solidColorsGroupLayout->addWidget(IntendS, 2, 1, 1, 1);


        tabColorLayout->addWidget(solidsProfileGroup);

        imageProfileGroup = new QGroupBox(tabColor);
        imageProfileGroup->setObjectName(QString::fromUtf8("imageProfileGroup"));
        imageProfileGroupLayout = new QGridLayout(imageProfileGroup);
        imageProfileGroupLayout->setObjectName(QString::fromUtf8("imageProfileGroupLayout"));
        EmbedProfs2 = new QCheckBox(imageProfileGroup);
        EmbedProfs2->setObjectName(QString::fromUtf8("EmbedProfs2"));

        imageProfileGroupLayout->addWidget(EmbedProfs2, 0, 0, 1, 1);

        imageProfileLabel = new QLabel(imageProfileGroup);
        imageProfileLabel->setObjectName(QString::fromUtf8("imageProfileLabel"));

        imageProfileGroupLayout->addWidget(imageProfileLabel, 2, 0, 1, 1);

        imageIntentLabel = new QLabel(imageProfileGroup);
        imageIntentLabel->setObjectName(QString::fromUtf8("imageIntentLabel"));

        imageProfileGroupLayout->addWidget(imageIntentLabel, 2, 1, 1, 1);

        NoEmbedded = new QCheckBox(imageProfileGroup);
        NoEmbedded->setObjectName(QString::fromUtf8("NoEmbedded"));

        imageProfileGroupLayout->addWidget(NoEmbedded, 1, 0, 1, 1);

        ImageP = new QComboBox(imageProfileGroup);
        ImageP->setObjectName(QString::fromUtf8("ImageP"));

        imageProfileGroupLayout->addWidget(ImageP, 3, 0, 1, 1);

        IntendI = new QComboBox(imageProfileGroup);
        IntendI->addItem(QString());
        IntendI->addItem(QString());
        IntendI->addItem(QString());
        IntendI->addItem(QString());
        IntendI->setObjectName(QString::fromUtf8("IntendI"));
        IntendI->setEditable(false);

        imageProfileGroupLayout->addWidget(IntendI, 3, 1, 1, 1);


        tabColorLayout->addWidget(imageProfileGroup);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        tabColorLayout->addItem(verticalSpacer_3);

        TabPDFOptions->addTab(tabColor, QString());
        tabPDFX = new QWidget();
        tabPDFX->setObjectName(QString::fromUtf8("tabPDFX"));
        tabPDFXLayout = new QVBoxLayout(tabPDFX);
        tabPDFXLayout->setObjectName(QString::fromUtf8("tabPDFXLayout"));
        MarkGroup = new QGroupBox(tabPDFX);
        MarkGroup->setObjectName(QString::fromUtf8("MarkGroup"));
        markGroupLayout = new QGridLayout(MarkGroup);
        markGroupLayout->setObjectName(QString::fromUtf8("markGroupLayout"));
        colorMarks = new QCheckBox(MarkGroup);
        colorMarks->setObjectName(QString::fromUtf8("colorMarks"));

        markGroupLayout->addWidget(colorMarks, 3, 0, 1, 1);

        registrationMarks = new QCheckBox(MarkGroup);
        registrationMarks->setObjectName(QString::fromUtf8("registrationMarks"));

        markGroupLayout->addWidget(registrationMarks, 2, 0, 1, 1);

        bleedMarks = new QCheckBox(MarkGroup);
        bleedMarks->setObjectName(QString::fromUtf8("bleedMarks"));

        markGroupLayout->addWidget(bleedMarks, 1, 0, 1, 1);

        cropMarks = new QCheckBox(MarkGroup);
        cropMarks->setObjectName(QString::fromUtf8("cropMarks"));

        markGroupLayout->addWidget(cropMarks, 0, 0, 1, 1);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        label_25 = new QLabel(MarkGroup);
        label_25->setObjectName(QString::fromUtf8("label_25"));

        horizontalLayout_6->addWidget(label_25);

        markOffset = new ScrSpinBox(MarkGroup);
        markOffset->setObjectName(QString::fromUtf8("markOffset"));

        horizontalLayout_6->addWidget(markOffset);


        markGroupLayout->addLayout(horizontalLayout_6, 3, 1, 1, 1);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName(QString::fromUtf8("horizontalLayout_8"));
        label_24 = new QLabel(MarkGroup);
        label_24->setObjectName(QString::fromUtf8("label_24"));

        horizontalLayout_8->addWidget(label_24);

        markLength = new ScrSpinBox(MarkGroup);
        markLength->setObjectName(QString::fromUtf8("markLength"));

        horizontalLayout_8->addWidget(markLength);


        markGroupLayout->addLayout(horizontalLayout_8, 2, 1, 1, 1);

        docInfoMarks = new QCheckBox(MarkGroup);
        docInfoMarks->setObjectName(QString::fromUtf8("docInfoMarks"));

        markGroupLayout->addWidget(docInfoMarks, 0, 1, 1, 1);


        tabPDFXLayout->addWidget(MarkGroup);

        BleedGroup = new QGroupBox(tabPDFX);
        BleedGroup->setObjectName(QString::fromUtf8("BleedGroup"));
        bleedGroupLayout = new QGridLayout(BleedGroup);
        bleedGroupLayout->setObjectName(QString::fromUtf8("bleedGroupLayout"));
        bleedTopLabel = new QLabel(BleedGroup);
        bleedTopLabel->setObjectName(QString::fromUtf8("bleedTopLabel"));

        bleedGroupLayout->addWidget(bleedTopLabel, 1, 0, 1, 1);

        BleedTop = new ScrSpinBox(BleedGroup);
        BleedTop->setObjectName(QString::fromUtf8("BleedTop"));

        bleedGroupLayout->addWidget(BleedTop, 1, 1, 1, 1);

        bleedBottomLabel = new QLabel(BleedGroup);
        bleedBottomLabel->setObjectName(QString::fromUtf8("bleedBottomLabel"));

        bleedGroupLayout->addWidget(bleedBottomLabel, 2, 0, 1, 1);

        BleedBottom = new ScrSpinBox(BleedGroup);
        BleedBottom->setObjectName(QString::fromUtf8("BleedBottom"));

        bleedGroupLayout->addWidget(BleedBottom, 2, 1, 1, 1);

        BleedRight = new ScrSpinBox(BleedGroup);
        BleedRight->setObjectName(QString::fromUtf8("BleedRight"));

        bleedGroupLayout->addWidget(BleedRight, 2, 3, 1, 1);

        bleedLeftLabel = new QLabel(BleedGroup);
        bleedLeftLabel->setObjectName(QString::fromUtf8("bleedLeftLabel"));

        bleedGroupLayout->addWidget(bleedLeftLabel, 1, 2, 1, 1);

        BleedLeft = new ScrSpinBox(BleedGroup);
        BleedLeft->setObjectName(QString::fromUtf8("BleedLeft"));

        bleedGroupLayout->addWidget(BleedLeft, 1, 3, 1, 1);

        bleedRightLabel = new QLabel(BleedGroup);
        bleedRightLabel->setObjectName(QString::fromUtf8("bleedRightLabel"));

        bleedGroupLayout->addWidget(bleedRightLabel, 2, 2, 1, 1);

        docBleeds = new QCheckBox(BleedGroup);
        docBleeds->setObjectName(QString::fromUtf8("docBleeds"));

        bleedGroupLayout->addWidget(docBleeds, 0, 0, 1, 1);


        tabPDFXLayout->addWidget(BleedGroup);

        X3Group = new QGroupBox(tabPDFX);
        X3Group->setObjectName(QString::fromUtf8("X3Group"));
        X3GroupLayout = new QGridLayout(X3Group);
        X3GroupLayout->setObjectName(QString::fromUtf8("X3GroupLayout"));
        PrintProfC = new QComboBox(X3Group);
        PrintProfC->setObjectName(QString::fromUtf8("PrintProfC"));
        PrintProfC->setEditable(false);

        X3GroupLayout->addWidget(PrintProfC, 0, 1, 1, 1);

        label_30 = new QLabel(X3Group);
        label_30->setObjectName(QString::fromUtf8("label_30"));

        X3GroupLayout->addWidget(label_30, 0, 0, 1, 1);

        label_31 = new QLabel(X3Group);
        label_31->setObjectName(QString::fromUtf8("label_31"));

        X3GroupLayout->addWidget(label_31, 1, 0, 1, 1);

        InfoString = new QLineEdit(X3Group);
        InfoString->setObjectName(QString::fromUtf8("InfoString"));

        X3GroupLayout->addWidget(InfoString, 1, 1, 1, 1);


        tabPDFXLayout->addWidget(X3Group);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        tabPDFXLayout->addItem(verticalSpacer_4);

        TabPDFOptions->addTab(tabPDFX, QString());
#if QT_CONFIG(shortcut)
        rotationLabel->setBuddy(RotateDeg);
        label->setBuddy(PDFVersionCombo);
        label_2->setBuddy(ComboBind);
        label_3->setBuddy(Resolution);
        label_4->setBuddy(CMethod);
        label_5->setBuddy(CQuality);
        label_15->setBuddy(EDirection_2_2);
        label_13->setBuddy(EDirection);
        label_14->setBuddy(EDirection_2);
        label_11->setBuddy(EffectTime);
        label_12->setBuddy(EffectType);
        label_10->setBuddy(PageTime);
        TextSec2->setBuddy(PassUser);
        TextSec1->setBuddy(PassOwner);
        label_16->setBuddy(OutCombo);
        label_17->setBuddy(LPIfreq);
        label_18->setBuddy(LPIangle);
        label_19->setBuddy(LPIfunc);
        label_30->setBuddy(PrintProfC);
        label_31->setBuddy(InfoString);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(AllPages, OnlySome);
        QWidget::setTabOrder(OnlySome, PageNr);
        QWidget::setTabOrder(PageNr, pageNrButton);
        QWidget::setTabOrder(pageNrButton, RotateDeg);
        QWidget::setTabOrder(RotateDeg, MirrorH);
        QWidget::setTabOrder(MirrorH, MirrorV);
        QWidget::setTabOrder(MirrorV, ClipMarg);
        QWidget::setTabOrder(ClipMarg, PDFVersionCombo);
        QWidget::setTabOrder(PDFVersionCombo, ComboBind);
        QWidget::setTabOrder(ComboBind, CheckBox1);
        QWidget::setTabOrder(CheckBox1, Article);
        QWidget::setTabOrder(Article, CheckBM);
        QWidget::setTabOrder(CheckBM, useLayers);
        QWidget::setTabOrder(useLayers, Resolution);
        QWidget::setTabOrder(Resolution, EmbedPDF);
        QWidget::setTabOrder(EmbedPDF, Compression);
        QWidget::setTabOrder(Compression, CMethod);
        QWidget::setTabOrder(CMethod, CQuality);
        QWidget::setTabOrder(CQuality, DSColor);
        QWidget::setTabOrder(DSColor, ValC);
        QWidget::setTabOrder(ValC, fontEmbeddingCombo);
        QWidget::setTabOrder(fontEmbeddingCombo, EmbedList);
        QWidget::setTabOrder(EmbedList, SubsetList);
        QWidget::setTabOrder(SubsetList, ToSubset);
        QWidget::setTabOrder(ToSubset, FromSubset);
        QWidget::setTabOrder(FromSubset, EmbedFonts);
        QWidget::setTabOrder(EmbedFonts, SubsetFonts);
        QWidget::setTabOrder(SubsetFonts, CheckBox10);
        QWidget::setTabOrder(CheckBox10, Pages);
        QWidget::setTabOrder(Pages, PageTime);
        QWidget::setTabOrder(PageTime, EffectTime);
        QWidget::setTabOrder(EffectTime, EffectType);
        QWidget::setTabOrder(EffectType, EDirection);
        QWidget::setTabOrder(EDirection, EDirection_2);
        QWidget::setTabOrder(EDirection_2, EDirection_2_2);
        QWidget::setTabOrder(EDirection_2_2, EonAllPg);
        QWidget::setTabOrder(EonAllPg, PagePrev);
        QWidget::setTabOrder(PagePrev, singlePage);
        QWidget::setTabOrder(singlePage, continuousPages);
        QWidget::setTabOrder(continuousPages, facingPagesLeft);
        QWidget::setTabOrder(facingPagesLeft, facingPagesRight);
        QWidget::setTabOrder(facingPagesRight, useViewDefault);
        QWidget::setTabOrder(useViewDefault, useFullScreen);
        QWidget::setTabOrder(useFullScreen, useBookmarks);
        QWidget::setTabOrder(useBookmarks, useThumbnails);
        QWidget::setTabOrder(useThumbnails, useLayers2);
        QWidget::setTabOrder(useLayers2, hideToolBar);
        QWidget::setTabOrder(hideToolBar, hideMenuBar);
        QWidget::setTabOrder(hideMenuBar, fitWindow);
        QWidget::setTabOrder(fitWindow, actionCombo);
        QWidget::setTabOrder(actionCombo, Encry);
        QWidget::setTabOrder(Encry, PassOwner);
        QWidget::setTabOrder(PassOwner, PassUser);
        QWidget::setTabOrder(PassUser, PrintSec);
        QWidget::setTabOrder(PrintSec, ModifySec);
        QWidget::setTabOrder(ModifySec, CopySec);
        QWidget::setTabOrder(CopySec, AddSec);
        QWidget::setTabOrder(AddSec, OutCombo);
        QWidget::setTabOrder(OutCombo, useSpot);
        QWidget::setTabOrder(useSpot, UseLPI);
        QWidget::setTabOrder(UseLPI, LPIcolor);
        QWidget::setTabOrder(LPIcolor, LPIfreq);
        QWidget::setTabOrder(LPIfreq, LPIangle);
        QWidget::setTabOrder(LPIangle, LPIfunc);
        QWidget::setTabOrder(LPIfunc, EmbedProfs);
        QWidget::setTabOrder(EmbedProfs, SolidPr);
        QWidget::setTabOrder(SolidPr, IntendS);
        QWidget::setTabOrder(IntendS, EmbedProfs2);
        QWidget::setTabOrder(EmbedProfs2, NoEmbedded);
        QWidget::setTabOrder(NoEmbedded, ImageP);
        QWidget::setTabOrder(ImageP, IntendI);
        QWidget::setTabOrder(IntendI, cropMarks);
        QWidget::setTabOrder(cropMarks, bleedMarks);
        QWidget::setTabOrder(bleedMarks, registrationMarks);
        QWidget::setTabOrder(registrationMarks, colorMarks);
        QWidget::setTabOrder(colorMarks, docInfoMarks);
        QWidget::setTabOrder(docInfoMarks, markLength);
        QWidget::setTabOrder(markLength, markOffset);
        QWidget::setTabOrder(markOffset, docBleeds);
        QWidget::setTabOrder(docBleeds, BleedTop);
        QWidget::setTabOrder(BleedTop, BleedBottom);
        QWidget::setTabOrder(BleedBottom, BleedLeft);
        QWidget::setTabOrder(BleedLeft, BleedRight);
        QWidget::setTabOrder(BleedRight, PrintProfC);
        QWidget::setTabOrder(PrintProfC, InfoString);

        retranslateUi(TabPDFOptions);

        TabPDFOptions->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(TabPDFOptions);
    } // setupUi

    void retranslateUi(QTabWidget *TabPDFOptions)
    {
        RangeGroup->setTitle(QCoreApplication::translate("TabPDFOptions", "Export Range", nullptr));
        AllPages->setText(QCoreApplication::translate("TabPDFOptions", "&All Pages", nullptr));
        OnlySome->setText(QCoreApplication::translate("TabPDFOptions", "C&hoose Pages", nullptr));
        rotationLabel->setText(QCoreApplication::translate("TabPDFOptions", "&Rotation:", nullptr));
        RotateDeg->setItemText(0, QCoreApplication::translate("TabPDFOptions", "0 \302\260", nullptr));
        RotateDeg->setItemText(1, QCoreApplication::translate("TabPDFOptions", "90 \302\260", nullptr));
        RotateDeg->setItemText(2, QCoreApplication::translate("TabPDFOptions", "180 \302\260", nullptr));
        RotateDeg->setItemText(3, QCoreApplication::translate("TabPDFOptions", "270 \302\260", nullptr));

        MirrorH->setText(QString());
        MirrorV->setText(QString());
        ClipMarg->setText(QCoreApplication::translate("TabPDFOptions", "Clip to Printer Margins", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("TabPDFOptions", "File Options", nullptr));
        label->setText(QCoreApplication::translate("TabPDFOptions", "Compatibilit&y:", nullptr));
        label_2->setText(QCoreApplication::translate("TabPDFOptions", "&Binding:", nullptr));
        ComboBind->setItemText(0, QCoreApplication::translate("TabPDFOptions", "Left Margin", nullptr));
        ComboBind->setItemText(1, QCoreApplication::translate("TabPDFOptions", "Right Margin", nullptr));

        CheckBox1->setText(QCoreApplication::translate("TabPDFOptions", "Generate &Thumbnails", nullptr));
        Resolution->setSuffix(QCoreApplication::translate("TabPDFOptions", " dpi", nullptr));
        CheckBM->setText(QCoreApplication::translate("TabPDFOptions", "&Include Bookmarks", nullptr));
        Article->setText(QCoreApplication::translate("TabPDFOptions", "Save &Linked Text Frames as PDF Articles", nullptr));
        useLayers->setText(QCoreApplication::translate("TabPDFOptions", "Include La&yers", nullptr));
        label_3->setText(QCoreApplication::translate("TabPDFOptions", "&Resolution for EPS Graphics:", nullptr));
        EmbedPDF->setText(QCoreApplication::translate("TabPDFOptions", "Embed PDF && EPS files (EXPERIMENTAL)", nullptr));
        Compression->setText(QCoreApplication::translate("TabPDFOptions", "Com&press Text and Vector Graphics", nullptr));
        groupBox->setTitle(QCoreApplication::translate("TabPDFOptions", "Image Quality Settings", nullptr));
        label_4->setText(QCoreApplication::translate("TabPDFOptions", "Compression Metho&d:", nullptr));
        CMethod->setItemText(0, QCoreApplication::translate("TabPDFOptions", "Automatic", nullptr));
        CMethod->setItemText(1, QCoreApplication::translate("TabPDFOptions", "Lossy - JPEG", nullptr));
        CMethod->setItemText(2, QCoreApplication::translate("TabPDFOptions", "Lossless - Zip", nullptr));
        CMethod->setItemText(3, QCoreApplication::translate("TabPDFOptions", "None", nullptr));

        label_5->setText(QCoreApplication::translate("TabPDFOptions", "Compression &Quality:", nullptr));
        CQuality->setItemText(0, QCoreApplication::translate("TabPDFOptions", "Maximum", nullptr));
        CQuality->setItemText(1, QCoreApplication::translate("TabPDFOptions", "High", nullptr));
        CQuality->setItemText(2, QCoreApplication::translate("TabPDFOptions", "Medium", nullptr));
        CQuality->setItemText(3, QCoreApplication::translate("TabPDFOptions", "Low", nullptr));
        CQuality->setItemText(4, QCoreApplication::translate("TabPDFOptions", "Minimum", nullptr));

        ValC->setSuffix(QCoreApplication::translate("TabPDFOptions", " dpi", nullptr));
        DSColor->setText(QCoreApplication::translate("TabPDFOptions", "Maximum Image Resolution:", nullptr));
        TabPDFOptions->setTabText(TabPDFOptions->indexOf(tabGeneral), QCoreApplication::translate("TabPDFOptions", "&General", nullptr));
        groupFont->setTitle(QCoreApplication::translate("TabPDFOptions", "Embedding", nullptr));
        label_6->setText(QCoreApplication::translate("TabPDFOptions", "Embedding Mode:", nullptr));
        label_8->setText(QCoreApplication::translate("TabPDFOptions", "Fonts to embed:", nullptr));
        EmbedFonts->setText(QCoreApplication::translate("TabPDFOptions", "&Embed all", nullptr));
        ToSubset->setText(QString());
        FromSubset->setText(QString());
        label_9->setText(QCoreApplication::translate("TabPDFOptions", "Fonts to subset:", nullptr));
        SubsetFonts->setText(QCoreApplication::translate("TabPDFOptions", "&Subset all", nullptr));
        TabPDFOptions->setTabText(TabPDFOptions->indexOf(tabFonts), QCoreApplication::translate("TabPDFOptions", "&Fonts", nullptr));
        CheckBox10->setText(QCoreApplication::translate("TabPDFOptions", "Enable &Presentation Effects", nullptr));
        PagePrev->setText(QCoreApplication::translate("TabPDFOptions", "Show Page Pre&views", nullptr));
        Effects->setTitle(QCoreApplication::translate("TabPDFOptions", "Effects", nullptr));
        EDirection_2_2->setItemText(0, QCoreApplication::translate("TabPDFOptions", "Left to Right", nullptr));
        EDirection_2_2->setItemText(1, QCoreApplication::translate("TabPDFOptions", "Top to Bottom", nullptr));
        EDirection_2_2->setItemText(2, QCoreApplication::translate("TabPDFOptions", "Bottom to Top", nullptr));
        EDirection_2_2->setItemText(3, QCoreApplication::translate("TabPDFOptions", "Right to Left", nullptr));
        EDirection_2_2->setItemText(4, QCoreApplication::translate("TabPDFOptions", "Top-left to Bottom-Right", nullptr));

        label_15->setText(QCoreApplication::translate("TabPDFOptions", "D&irection:", nullptr));
        label_13->setText(QCoreApplication::translate("TabPDFOptions", "&Moving Lines:", nullptr));
        EDirection->setItemText(0, QCoreApplication::translate("TabPDFOptions", "Horizontal", nullptr));
        EDirection->setItemText(1, QCoreApplication::translate("TabPDFOptions", "Vertical", nullptr));

        EffectType->setItemText(0, QCoreApplication::translate("TabPDFOptions", "No Effect", nullptr));
        EffectType->setItemText(1, QCoreApplication::translate("TabPDFOptions", "Blinds", nullptr));
        EffectType->setItemText(2, QCoreApplication::translate("TabPDFOptions", "Box", nullptr));
        EffectType->setItemText(3, QCoreApplication::translate("TabPDFOptions", "Dissolve", nullptr));
        EffectType->setItemText(4, QCoreApplication::translate("TabPDFOptions", "Glitter", nullptr));
        EffectType->setItemText(5, QCoreApplication::translate("TabPDFOptions", "Split", nullptr));
        EffectType->setItemText(6, QCoreApplication::translate("TabPDFOptions", "Wipe", nullptr));

        label_14->setText(QCoreApplication::translate("TabPDFOptions", "F&rom the:", nullptr));
        EDirection_2->setItemText(0, QCoreApplication::translate("TabPDFOptions", "Inside", nullptr));
        EDirection_2->setItemText(1, QCoreApplication::translate("TabPDFOptions", "Outside", nullptr));

        label_11->setText(QCoreApplication::translate("TabPDFOptions", "Effec&t Duration:", nullptr));
        label_12->setText(QCoreApplication::translate("TabPDFOptions", "Effect T&ype:", nullptr));
        label_10->setText(QCoreApplication::translate("TabPDFOptions", "&Display Duration:", nullptr));
        EffectTime->setSuffix(QCoreApplication::translate("TabPDFOptions", " sec", nullptr));
        PageTime->setSuffix(QCoreApplication::translate("TabPDFOptions", " sec", nullptr));
        EonAllPg->setText(QCoreApplication::translate("TabPDFOptions", "&Apply Effect to all Pages", nullptr));
        TabPDFOptions->setTabText(TabPDFOptions->indexOf(tabPresentation), QCoreApplication::translate("TabPDFOptions", "E&xtras", nullptr));
        groupDisplay->setTitle(QCoreApplication::translate("TabPDFOptions", "Display Settings", nullptr));
        pageLayout->setTitle(QCoreApplication::translate("TabPDFOptions", "Document Layout", nullptr));
        singlePage->setText(QCoreApplication::translate("TabPDFOptions", "Single Page", nullptr));
        continuousPages->setText(QCoreApplication::translate("TabPDFOptions", "Continuous", nullptr));
        facingPagesLeft->setText(QCoreApplication::translate("TabPDFOptions", "Facing Page Left", nullptr));
        facingPagesRight->setText(QCoreApplication::translate("TabPDFOptions", "Facing Page Right", nullptr));
        groupNavigation->setTitle(QCoreApplication::translate("TabPDFOptions", "Visual Appearance", nullptr));
        useViewDefault->setText(QCoreApplication::translate("TabPDFOptions", "Use Viewers Defaults", nullptr));
        useFullScreen->setText(QCoreApplication::translate("TabPDFOptions", "Use Full Screen Mode", nullptr));
        useBookmarks->setText(QCoreApplication::translate("TabPDFOptions", "Display Bookmarks Tab", nullptr));
        useThumbnails->setText(QCoreApplication::translate("TabPDFOptions", "Display Thumbnails", nullptr));
        useLayers2->setText(QCoreApplication::translate("TabPDFOptions", "Display Layers Tab", nullptr));
        hideToolBar->setText(QCoreApplication::translate("TabPDFOptions", "Hide Viewers Toolbar", nullptr));
        hideMenuBar->setText(QCoreApplication::translate("TabPDFOptions", "Hide Viewers Menubar", nullptr));
        fitWindow->setText(QCoreApplication::translate("TabPDFOptions", "Zoom Pages to fit Viewer Window", nullptr));
        groupJava->setTitle(QCoreApplication::translate("TabPDFOptions", "Special Actions", nullptr));
        actionLabel->setText(QCoreApplication::translate("TabPDFOptions", "Javascript to be executed\n"
"when PDF document is opened:", nullptr));
        actionCombo->setItemText(0, QCoreApplication::translate("TabPDFOptions", "No Script", nullptr));

        TabPDFOptions->setTabText(TabPDFOptions->indexOf(tabSpecial), QCoreApplication::translate("TabPDFOptions", "Viewer", nullptr));
        Encry->setText(QCoreApplication::translate("TabPDFOptions", "&Use Encryption", nullptr));
        groupPass->setTitle(QCoreApplication::translate("TabPDFOptions", "Passwords", nullptr));
        TextSec2->setText(QCoreApplication::translate("TabPDFOptions", "&User:", nullptr));
        TextSec1->setText(QCoreApplication::translate("TabPDFOptions", "&Owner:", nullptr));
        groupSecSet->setTitle(QCoreApplication::translate("TabPDFOptions", "Settings", nullptr));
        PrintSec->setText(QCoreApplication::translate("TabPDFOptions", "Allow &Printing the Document", nullptr));
        ModifySec->setText(QCoreApplication::translate("TabPDFOptions", "Allow &Changing the Document", nullptr));
        CopySec->setText(QCoreApplication::translate("TabPDFOptions", "Allow Cop&ying Text and Graphics", nullptr));
        AddSec->setText(QCoreApplication::translate("TabPDFOptions", "Allow Adding &Annotations and Fields", nullptr));
        TabPDFOptions->setTabText(TabPDFOptions->indexOf(tabSecurity), QCoreApplication::translate("TabPDFOptions", "S&ecurity", nullptr));
        ColorGroup->setTitle(QCoreApplication::translate("TabPDFOptions", "General", nullptr));
        label_16->setText(QCoreApplication::translate("TabPDFOptions", "Output &Intended For:", nullptr));
        OutCombo->setItemText(0, QCoreApplication::translate("TabPDFOptions", "Screen / Web", nullptr));
        OutCombo->setItemText(1, QCoreApplication::translate("TabPDFOptions", "Printer", nullptr));
        OutCombo->setItemText(2, QCoreApplication::translate("TabPDFOptions", "Grayscale", nullptr));

        useSpot->setText(QCoreApplication::translate("TabPDFOptions", "Convert Spot Colors to Process Colors", nullptr));
        UseLPI->setText(QCoreApplication::translate("TabPDFOptions", "&Use Custom Rendering Settings", nullptr));
        LPIgroup->setTitle(QCoreApplication::translate("TabPDFOptions", "Rendering Settings", nullptr));
        label_17->setText(QCoreApplication::translate("TabPDFOptions", "Fre&quency:", nullptr));
        label_18->setText(QCoreApplication::translate("TabPDFOptions", "&Angle:", nullptr));
        LPIangle->setSuffix(QCoreApplication::translate("TabPDFOptions", " \302\260", nullptr));
        label_19->setText(QCoreApplication::translate("TabPDFOptions", "S&pot Function:", nullptr));
        LPIfunc->setItemText(0, QCoreApplication::translate("TabPDFOptions", "Simple Dot", nullptr));
        LPIfunc->setItemText(1, QCoreApplication::translate("TabPDFOptions", "Line", nullptr));
        LPIfunc->setItemText(2, QCoreApplication::translate("TabPDFOptions", "Round", nullptr));
        LPIfunc->setItemText(3, QCoreApplication::translate("TabPDFOptions", "Ellipse", nullptr));

        solidsProfileGroup->setTitle(QCoreApplication::translate("TabPDFOptions", "Solid Colors:", nullptr));
        solidsProfileLabel->setText(QCoreApplication::translate("TabPDFOptions", "Profile:", nullptr));
        EmbedProfs->setText(QCoreApplication::translate("TabPDFOptions", "Use Color Profile", nullptr));
        solidsIntentLabel->setText(QCoreApplication::translate("TabPDFOptions", "Rendering Intent:", nullptr));
        IntendS->setItemText(0, QCoreApplication::translate("TabPDFOptions", "Perceptual", nullptr));
        IntendS->setItemText(1, QCoreApplication::translate("TabPDFOptions", "Relative Colorimetric", nullptr));
        IntendS->setItemText(2, QCoreApplication::translate("TabPDFOptions", "Saturation", nullptr));
        IntendS->setItemText(3, QCoreApplication::translate("TabPDFOptions", "Absolute Colorimetric", nullptr));

        imageProfileGroup->setTitle(QCoreApplication::translate("TabPDFOptions", "Images:", nullptr));
        EmbedProfs2->setText(QCoreApplication::translate("TabPDFOptions", "Use Color Profile", nullptr));
        imageProfileLabel->setText(QCoreApplication::translate("TabPDFOptions", "Profile:", nullptr));
        imageIntentLabel->setText(QCoreApplication::translate("TabPDFOptions", "Rendering Intent:", nullptr));
        NoEmbedded->setText(QCoreApplication::translate("TabPDFOptions", "Do not use embedded color profiles", nullptr));
        IntendI->setItemText(0, QCoreApplication::translate("TabPDFOptions", "Perceptual", nullptr));
        IntendI->setItemText(1, QCoreApplication::translate("TabPDFOptions", "Relative Colorimetric", nullptr));
        IntendI->setItemText(2, QCoreApplication::translate("TabPDFOptions", "Saturation", nullptr));
        IntendI->setItemText(3, QCoreApplication::translate("TabPDFOptions", "Absolute Colorimetric", nullptr));

        TabPDFOptions->setTabText(TabPDFOptions->indexOf(tabColor), QCoreApplication::translate("TabPDFOptions", "C&olor", nullptr));
        MarkGroup->setTitle(QCoreApplication::translate("TabPDFOptions", "Printer Marks", nullptr));
        colorMarks->setText(QCoreApplication::translate("TabPDFOptions", "Color Bars", nullptr));
        registrationMarks->setText(QCoreApplication::translate("TabPDFOptions", "Registration Marks", nullptr));
        bleedMarks->setText(QCoreApplication::translate("TabPDFOptions", "Bleed Marks", nullptr));
        cropMarks->setText(QCoreApplication::translate("TabPDFOptions", "Crop Marks", nullptr));
        label_25->setText(QCoreApplication::translate("TabPDFOptions", "Offset:", nullptr));
        label_24->setText(QCoreApplication::translate("TabPDFOptions", "Length:", nullptr));
        docInfoMarks->setText(QCoreApplication::translate("TabPDFOptions", "Page Information", nullptr));
        BleedGroup->setTitle(QCoreApplication::translate("TabPDFOptions", "Bleed Settings", nullptr));
        bleedTopLabel->setText(QCoreApplication::translate("TabPDFOptions", "Top:", nullptr));
        bleedBottomLabel->setText(QCoreApplication::translate("TabPDFOptions", "Bottom:", nullptr));
        bleedLeftLabel->setText(QCoreApplication::translate("TabPDFOptions", "Left:", nullptr));
        bleedRightLabel->setText(QCoreApplication::translate("TabPDFOptions", "Right:", nullptr));
        docBleeds->setText(QCoreApplication::translate("TabPDFOptions", "Use Document Bleeds", nullptr));
        X3Group->setTitle(QCoreApplication::translate("TabPDFOptions", "PDF/X Output Intent", nullptr));
        label_30->setText(QCoreApplication::translate("TabPDFOptions", "Output &Profile:", nullptr));
        label_31->setText(QCoreApplication::translate("TabPDFOptions", "&Info String:", nullptr));
        TabPDFOptions->setTabText(TabPDFOptions->indexOf(tabPDFX), QCoreApplication::translate("TabPDFOptions", "Pre-Press", nullptr));
        (void)TabPDFOptions;
    } // retranslateUi

};

namespace Ui {
    class TabPDFOptions: public Ui_TabPDFOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TABPDFOPTIONS_H
