/********************************************************************************
** Form generated from reading UI file 'prefs_scrapbookbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_SCRAPBOOKBASE_H
#define UI_PREFS_SCRAPBOOKBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Prefs_Scrapbook
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *titleLabel;
    QFrame *line;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *autoSendCopiedToScrapCheckBox;
    QCheckBox *keepCopiedAcrossSessionsCheckBox;
    QCheckBox *writePreviewsCheckBox;
    QHBoxLayout *horizontalLayout;
    QLabel *itemCountKeptInScrapbookLabel;
    QSpinBox *itemCountKeptInScrapbookSpinBox;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *Prefs_Scrapbook)
    {
        if (Prefs_Scrapbook->objectName().isEmpty())
            Prefs_Scrapbook->setObjectName(QString::fromUtf8("Prefs_Scrapbook"));
        Prefs_Scrapbook->resize(705, 386);
        verticalLayout = new QVBoxLayout(Prefs_Scrapbook);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleLabel = new QLabel(Prefs_Scrapbook);
        titleLabel->setObjectName(QString::fromUtf8("titleLabel"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        titleLabel->setFont(font);

        verticalLayout->addWidget(titleLabel);

        line = new QFrame(Prefs_Scrapbook);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        scrollArea = new QScrollArea(Prefs_Scrapbook);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 677, 316));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        autoSendCopiedToScrapCheckBox = new QCheckBox(scrollAreaWidgetContents);
        autoSendCopiedToScrapCheckBox->setObjectName(QString::fromUtf8("autoSendCopiedToScrapCheckBox"));

        verticalLayout_2->addWidget(autoSendCopiedToScrapCheckBox);

        keepCopiedAcrossSessionsCheckBox = new QCheckBox(scrollAreaWidgetContents);
        keepCopiedAcrossSessionsCheckBox->setObjectName(QString::fromUtf8("keepCopiedAcrossSessionsCheckBox"));

        verticalLayout_2->addWidget(keepCopiedAcrossSessionsCheckBox);

        writePreviewsCheckBox = new QCheckBox(scrollAreaWidgetContents);
        writePreviewsCheckBox->setObjectName(QString::fromUtf8("writePreviewsCheckBox"));

        verticalLayout_2->addWidget(writePreviewsCheckBox);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setSizeConstraint(QLayout::SetDefaultConstraint);
        itemCountKeptInScrapbookLabel = new QLabel(scrollAreaWidgetContents);
        itemCountKeptInScrapbookLabel->setObjectName(QString::fromUtf8("itemCountKeptInScrapbookLabel"));

        horizontalLayout->addWidget(itemCountKeptInScrapbookLabel);

        itemCountKeptInScrapbookSpinBox = new QSpinBox(scrollAreaWidgetContents);
        itemCountKeptInScrapbookSpinBox->setObjectName(QString::fromUtf8("itemCountKeptInScrapbookSpinBox"));

        horizontalLayout->addWidget(itemCountKeptInScrapbookSpinBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_2->addLayout(horizontalLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout->addWidget(scrollArea);

#if QT_CONFIG(shortcut)
        itemCountKeptInScrapbookLabel->setBuddy(itemCountKeptInScrapbookSpinBox);
#endif // QT_CONFIG(shortcut)

        retranslateUi(Prefs_Scrapbook);

        QMetaObject::connectSlotsByName(Prefs_Scrapbook);
    } // setupUi

    void retranslateUi(QWidget *Prefs_Scrapbook)
    {
        titleLabel->setText(QCoreApplication::translate("Prefs_Scrapbook", "Scrapbook", nullptr));
        autoSendCopiedToScrapCheckBox->setText(QCoreApplication::translate("Prefs_Scrapbook", "Send Copied Items Automatically to the Scrapbook", nullptr));
        keepCopiedAcrossSessionsCheckBox->setText(QCoreApplication::translate("Prefs_Scrapbook", "Keep Copied Items Permanently Across Sessions", nullptr));
        writePreviewsCheckBox->setText(QCoreApplication::translate("Prefs_Scrapbook", "Write Item Previews to the Scrapbook Directory", nullptr));
        itemCountKeptInScrapbookLabel->setText(QCoreApplication::translate("Prefs_Scrapbook", "Number of Copied Items to Keep in Scrapbook", nullptr));
        (void)Prefs_Scrapbook;
    } // retranslateUi

};

namespace Ui {
    class Prefs_Scrapbook: public Ui_Prefs_Scrapbook {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_SCRAPBOOKBASE_H
