/********************************************************************************
** Form generated from reading UI file 'helpbrowsernavigation.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HELPBROWSERNAVIGATION_H
#define UI_HELPBROWSERNAVIGATION_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_HelpBrowserNavigation
{
public:
    QGridLayout *gridLayout;
    QSplitter *splitter;
    QTabWidget *tabWidget;
    QWidget *tabContents;
    QVBoxLayout *vboxLayout;
    QTreeView *listView;
    QWidget *tabSearching;
    QVBoxLayout *_2;
    QHBoxLayout *_3;
    QLineEdit *searchingEdit;
    QPushButton *searchingButton;
    QTreeWidget *searchingView;
    QWidget *tabBookmarks;
    QVBoxLayout *_4;
    QTreeWidget *bookmarksView;
    QHBoxLayout *_5;
    QPushButton *bookmarkButton;
    QPushButton *deleteBookmarkButton;
    QPushButton *deleteAllBookmarkButton;

    void setupUi(QWidget *HelpBrowserNavigation)
    {
        if (HelpBrowserNavigation->objectName().isEmpty())
            HelpBrowserNavigation->setObjectName(QString::fromUtf8("HelpBrowserNavigation"));
        HelpBrowserNavigation->resize(438, 682);
        gridLayout = new QGridLayout(HelpBrowserNavigation);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        splitter = new QSplitter(HelpBrowserNavigation);
        splitter->setObjectName(QString::fromUtf8("splitter"));
        splitter->setOrientation(Qt::Horizontal);
        splitter->setChildrenCollapsible(false);
        tabWidget = new QTabWidget(splitter);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tabContents = new QWidget();
        tabContents->setObjectName(QString::fromUtf8("tabContents"));
        vboxLayout = new QVBoxLayout(tabContents);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        listView = new QTreeView(tabContents);
        listView->setObjectName(QString::fromUtf8("listView"));
        listView->setVerticalScrollBarPolicy(Qt::ScrollBarAsNeeded);

        vboxLayout->addWidget(listView);

        tabWidget->addTab(tabContents, QString());
        tabSearching = new QWidget();
        tabSearching->setObjectName(QString::fromUtf8("tabSearching"));
        _2 = new QVBoxLayout(tabSearching);
        _2->setObjectName(QString::fromUtf8("_2"));
        _3 = new QHBoxLayout();
        _3->setObjectName(QString::fromUtf8("_3"));
        searchingEdit = new QLineEdit(tabSearching);
        searchingEdit->setObjectName(QString::fromUtf8("searchingEdit"));

        _3->addWidget(searchingEdit);

        searchingButton = new QPushButton(tabSearching);
        searchingButton->setObjectName(QString::fromUtf8("searchingButton"));

        _3->addWidget(searchingButton);


        _2->addLayout(_3);

        searchingView = new QTreeWidget(tabSearching);
        searchingView->setObjectName(QString::fromUtf8("searchingView"));

        _2->addWidget(searchingView);

        tabWidget->addTab(tabSearching, QString());
        tabBookmarks = new QWidget();
        tabBookmarks->setObjectName(QString::fromUtf8("tabBookmarks"));
        _4 = new QVBoxLayout(tabBookmarks);
        _4->setObjectName(QString::fromUtf8("_4"));
        bookmarksView = new QTreeWidget(tabBookmarks);
        bookmarksView->setObjectName(QString::fromUtf8("bookmarksView"));

        _4->addWidget(bookmarksView);

        _5 = new QHBoxLayout();
        _5->setObjectName(QString::fromUtf8("_5"));
        bookmarkButton = new QPushButton(tabBookmarks);
        bookmarkButton->setObjectName(QString::fromUtf8("bookmarkButton"));

        _5->addWidget(bookmarkButton);

        deleteBookmarkButton = new QPushButton(tabBookmarks);
        deleteBookmarkButton->setObjectName(QString::fromUtf8("deleteBookmarkButton"));

        _5->addWidget(deleteBookmarkButton);

        deleteAllBookmarkButton = new QPushButton(tabBookmarks);
        deleteAllBookmarkButton->setObjectName(QString::fromUtf8("deleteAllBookmarkButton"));

        _5->addWidget(deleteAllBookmarkButton);


        _4->addLayout(_5);

        tabWidget->addTab(tabBookmarks, QString());
        splitter->addWidget(tabWidget);

        gridLayout->addWidget(splitter, 0, 0, 1, 1);


        retranslateUi(HelpBrowserNavigation);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(HelpBrowserNavigation);
    } // setupUi

    void retranslateUi(QWidget *HelpBrowserNavigation)
    {
        tabWidget->setTabText(tabWidget->indexOf(tabContents), QCoreApplication::translate("HelpBrowserNavigation", "&Contents", nullptr));
#if QT_CONFIG(tooltip)
        searchingEdit->setToolTip(QCoreApplication::translate("HelpBrowserNavigation", "Searching is case insensitive", nullptr));
#endif // QT_CONFIG(tooltip)
        searchingButton->setText(QCoreApplication::translate("HelpBrowserNavigation", "&Search", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = searchingView->headerItem();
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("HelpBrowserNavigation", "1", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabSearching), QCoreApplication::translate("HelpBrowserNavigation", "Se&arch", nullptr));
        QTreeWidgetItem *___qtreewidgetitem1 = bookmarksView->headerItem();
        ___qtreewidgetitem1->setText(0, QCoreApplication::translate("HelpBrowserNavigation", "1", nullptr));
        bookmarkButton->setText(QCoreApplication::translate("HelpBrowserNavigation", "&New", nullptr));
        deleteBookmarkButton->setText(QCoreApplication::translate("HelpBrowserNavigation", "&Delete", nullptr));
        deleteAllBookmarkButton->setText(QCoreApplication::translate("HelpBrowserNavigation", "D&elete All", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabBookmarks), QCoreApplication::translate("HelpBrowserNavigation", "Book&marks", nullptr));
        (void)HelpBrowserNavigation;
    } // retranslateUi

};

namespace Ui {
    class HelpBrowserNavigation: public Ui_HelpBrowserNavigation {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_HELPBROWSERNAVIGATION_H
