/********************************************************************************
** Form generated from reading UI file 'markvariabletext.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MARKVARIABLETEXT_H
#define UI_MARKVARIABLETEXT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_MarkVariableTextDlg
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *labelLayout;
    QLabel *label;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_2;
    QLineEdit *textEdit;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *MarkVariableTextDlg)
    {
        if (MarkVariableTextDlg->objectName().isEmpty())
            MarkVariableTextDlg->setObjectName(QString::fromUtf8("MarkVariableTextDlg"));
        MarkVariableTextDlg->resize(394, 117);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(MarkVariableTextDlg->sizePolicy().hasHeightForWidth());
        MarkVariableTextDlg->setSizePolicy(sizePolicy);
        MarkVariableTextDlg->setMinimumSize(QSize(0, 117));
        MarkVariableTextDlg->setMaximumSize(QSize(16777215, 117));
        verticalLayout = new QVBoxLayout(MarkVariableTextDlg);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        labelLayout = new QHBoxLayout();
        labelLayout->setObjectName(QString::fromUtf8("labelLayout"));
        label = new QLabel(MarkVariableTextDlg);
        label->setObjectName(QString::fromUtf8("label"));

        labelLayout->addWidget(label);


        verticalLayout->addLayout(labelLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        label_2 = new QLabel(MarkVariableTextDlg);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        horizontalLayout_2->addWidget(label_2);

        textEdit = new QLineEdit(MarkVariableTextDlg);
        textEdit->setObjectName(QString::fromUtf8("textEdit"));

        horizontalLayout_2->addWidget(textEdit);


        verticalLayout->addLayout(horizontalLayout_2);

        buttonBox = new QDialogButtonBox(MarkVariableTextDlg);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(buttonBox->sizePolicy().hasHeightForWidth());
        buttonBox->setSizePolicy(sizePolicy1);
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(MarkVariableTextDlg);
        QObject::connect(buttonBox, SIGNAL(accepted()), MarkVariableTextDlg, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), MarkVariableTextDlg, SLOT(reject()));

        QMetaObject::connectSlotsByName(MarkVariableTextDlg);
    } // setupUi

    void retranslateUi(QDialog *MarkVariableTextDlg)
    {
        MarkVariableTextDlg->setWindowTitle(QCoreApplication::translate("MarkVariableTextDlg", "Add new Mark for Variable Text", nullptr));
        label->setText(QCoreApplication::translate("MarkVariableTextDlg", "Label:", nullptr));
        label_2->setText(QCoreApplication::translate("MarkVariableTextDlg", "Text to insert:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MarkVariableTextDlg: public Ui_MarkVariableTextDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MARKVARIABLETEXT_H
