/****************************************************************************
** Meta object code from reading C++ file 'scribusview.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/scribusview.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scribusview.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ScribusView_t {
    QByteArrayData data[72];
    char stringdata0[749];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ScribusView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ScribusView_t qt_meta_stringdata_ScribusView = {
    {
QT_MOC_LITERAL(0, 0, 11), // "ScribusView"
QT_MOC_LITERAL(1, 12, 11), // "unitChanged"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 12), // "layerChanged"
QT_MOC_LITERAL(4, 38, 7), // "HaveSel"
QT_MOC_LITERAL(5, 46, 10), // "DocChanged"
QT_MOC_LITERAL(6, 57, 8), // "ItemGeom"
QT_MOC_LITERAL(7, 66, 8), // "PolyOpen"
QT_MOC_LITERAL(8, 75, 13), // "ItemCharStyle"
QT_MOC_LITERAL(9, 89, 9), // "CharStyle"
QT_MOC_LITERAL(10, 99, 13), // "ItemTextAlign"
QT_MOC_LITERAL(11, 113, 15), // "ItemTextEffects"
QT_MOC_LITERAL(12, 129, 5), // "AddBM"
QT_MOC_LITERAL(13, 135, 9), // "PageItem*"
QT_MOC_LITERAL(14, 145, 5), // "DelBM"
QT_MOC_LITERAL(15, 151, 8), // "ChBMText"
QT_MOC_LITERAL(16, 160, 8), // "LoadElem"
QT_MOC_LITERAL(17, 169, 11), // "ScribusDoc*"
QT_MOC_LITERAL(18, 181, 12), // "ScribusView*"
QT_MOC_LITERAL(19, 194, 9), // "HavePoint"
QT_MOC_LITERAL(20, 204, 6), // "ClipPo"
QT_MOC_LITERAL(21, 211, 10), // "PolyStatus"
QT_MOC_LITERAL(22, 222, 8), // "MousePos"
QT_MOC_LITERAL(23, 231, 13), // "iconSetChange"
QT_MOC_LITERAL(24, 245, 14), // "languageChange"
QT_MOC_LITERAL(25, 260, 9), // "toggleCMS"
QT_MOC_LITERAL(26, 270, 5), // "cmsOn"
QT_MOC_LITERAL(27, 276, 19), // "switchPreviewVisual"
QT_MOC_LITERAL(28, 296, 3), // "vis"
QT_MOC_LITERAL(29, 300, 17), // "togglePreviewEdit"
QT_MOC_LITERAL(30, 318, 4), // "edit"
QT_MOC_LITERAL(31, 323, 13), // "togglePreview"
QT_MOC_LITERAL(32, 337, 9), // "inPreview"
QT_MOC_LITERAL(33, 347, 10), // "unitChange"
QT_MOC_LITERAL(34, 358, 14), // "setRulersShown"
QT_MOC_LITERAL(35, 373, 7), // "isShown"
QT_MOC_LITERAL(36, 381, 11), // "slotZoom100"
QT_MOC_LITERAL(37, 393, 10), // "slotZoomIn"
QT_MOC_LITERAL(38, 404, 2), // "mx"
QT_MOC_LITERAL(39, 407, 2), // "my"
QT_MOC_LITERAL(40, 410, 13), // "preservePoint"
QT_MOC_LITERAL(41, 424, 11), // "slotZoomOut"
QT_MOC_LITERAL(42, 436, 7), // "DrawNew"
QT_MOC_LITERAL(43, 444, 6), // "GotoPa"
QT_MOC_LITERAL(44, 451, 10), // "pageNumber"
QT_MOC_LITERAL(45, 462, 9), // "GotoLayer"
QT_MOC_LITERAL(46, 472, 1), // "l"
QT_MOC_LITERAL(47, 474, 8), // "GotoPage"
QT_MOC_LITERAL(48, 483, 9), // "pageIndex"
QT_MOC_LITERAL(49, 493, 7), // "ChgUnit"
QT_MOC_LITERAL(50, 501, 3), // "art"
QT_MOC_LITERAL(51, 505, 27), // "editExtendedImageProperties"
QT_MOC_LITERAL(52, 533, 15), // "RefreshGradient"
QT_MOC_LITERAL(53, 549, 8), // "currItem"
QT_MOC_LITERAL(54, 558, 2), // "dx"
QT_MOC_LITERAL(55, 561, 2), // "dy"
QT_MOC_LITERAL(56, 564, 10), // "ToPicFrame"
QT_MOC_LITERAL(57, 575, 11), // "ToPolyFrame"
QT_MOC_LITERAL(58, 587, 11), // "ToTextFrame"
QT_MOC_LITERAL(59, 599, 13), // "ToBezierFrame"
QT_MOC_LITERAL(60, 613, 10), // "ToPathText"
QT_MOC_LITERAL(61, 624, 12), // "FromPathText"
QT_MOC_LITERAL(62, 637, 11), // "Bezier2Poly"
QT_MOC_LITERAL(63, 649, 11), // "PasteToPage"
QT_MOC_LITERAL(64, 661, 10), // "TextToPath"
QT_MOC_LITERAL(65, 672, 7), // "setZoom"
QT_MOC_LITERAL(66, 680, 11), // "setRulerPos"
QT_MOC_LITERAL(67, 692, 1), // "x"
QT_MOC_LITERAL(68, 694, 1), // "y"
QT_MOC_LITERAL(69, 696, 17), // "setObjectUndoMode"
QT_MOC_LITERAL(70, 714, 17), // "setGlobalUndoMode"
QT_MOC_LITERAL(71, 732, 16) // "dragTimerTimeOut"

    },
    "ScribusView\0unitChanged\0\0layerChanged\0"
    "HaveSel\0DocChanged\0ItemGeom\0PolyOpen\0"
    "ItemCharStyle\0CharStyle\0ItemTextAlign\0"
    "ItemTextEffects\0AddBM\0PageItem*\0DelBM\0"
    "ChBMText\0LoadElem\0ScribusDoc*\0"
    "ScribusView*\0HavePoint\0ClipPo\0PolyStatus\0"
    "MousePos\0iconSetChange\0languageChange\0"
    "toggleCMS\0cmsOn\0switchPreviewVisual\0"
    "vis\0togglePreviewEdit\0edit\0togglePreview\0"
    "inPreview\0unitChange\0setRulersShown\0"
    "isShown\0slotZoom100\0slotZoomIn\0mx\0my\0"
    "preservePoint\0slotZoomOut\0DrawNew\0"
    "GotoPa\0pageNumber\0GotoLayer\0l\0GotoPage\0"
    "pageIndex\0ChgUnit\0art\0editExtendedImageProperties\0"
    "RefreshGradient\0currItem\0dx\0dy\0"
    "ToPicFrame\0ToPolyFrame\0ToTextFrame\0"
    "ToBezierFrame\0ToPathText\0FromPathText\0"
    "Bezier2Poly\0PasteToPage\0TextToPath\0"
    "setZoom\0setRulerPos\0x\0y\0setObjectUndoMode\0"
    "setGlobalUndoMode\0dragTimerTimeOut"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ScribusView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      57,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      17,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  299,    2, 0x06 /* Public */,
       3,    1,  302,    2, 0x06 /* Public */,
       4,    0,  305,    2, 0x06 /* Public */,
       5,    0,  306,    2, 0x06 /* Public */,
       6,    0,  307,    2, 0x06 /* Public */,
       7,    0,  308,    2, 0x06 /* Public */,
       8,    1,  309,    2, 0x06 /* Public */,
      10,    1,  312,    2, 0x06 /* Public */,
      11,    1,  315,    2, 0x06 /* Public */,
      12,    1,  318,    2, 0x06 /* Public */,
      14,    1,  321,    2, 0x06 /* Public */,
      15,    1,  324,    2, 0x06 /* Public */,
      16,    7,  327,    2, 0x06 /* Public */,
      19,    2,  342,    2, 0x06 /* Public */,
      20,    2,  347,    2, 0x06 /* Public */,
      21,    2,  352,    2, 0x06 /* Public */,
      22,    2,  357,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      23,    0,  362,    2, 0x0a /* Public */,
      24,    0,  363,    2, 0x0a /* Public */,
      25,    1,  364,    2, 0x0a /* Public */,
      27,    1,  367,    2, 0x0a /* Public */,
      29,    1,  370,    2, 0x0a /* Public */,
      31,    1,  373,    2, 0x0a /* Public */,
      33,    0,  376,    2, 0x0a /* Public */,
      34,    1,  377,    2, 0x0a /* Public */,
      36,    0,  380,    2, 0x0a /* Public */,
      37,    3,  381,    2, 0x0a /* Public */,
      37,    2,  388,    2, 0x2a /* Public | MethodCloned */,
      37,    1,  393,    2, 0x2a /* Public | MethodCloned */,
      37,    0,  396,    2, 0x2a /* Public | MethodCloned */,
      41,    3,  397,    2, 0x0a /* Public */,
      41,    2,  404,    2, 0x2a /* Public | MethodCloned */,
      41,    1,  409,    2, 0x2a /* Public | MethodCloned */,
      41,    0,  412,    2, 0x2a /* Public | MethodCloned */,
      42,    0,  413,    2, 0x0a /* Public */,
      43,    1,  414,    2, 0x0a /* Public */,
      45,    1,  417,    2, 0x0a /* Public */,
      47,    1,  420,    2, 0x0a /* Public */,
      49,    1,  423,    2, 0x0a /* Public */,
      51,    0,  426,    2, 0x0a /* Public */,
      52,    3,  427,    2, 0x0a /* Public */,
      52,    2,  434,    2, 0x2a /* Public | MethodCloned */,
      52,    1,  439,    2, 0x2a /* Public | MethodCloned */,
      56,    0,  442,    2, 0x0a /* Public */,
      57,    0,  443,    2, 0x0a /* Public */,
      58,    0,  444,    2, 0x0a /* Public */,
      59,    0,  445,    2, 0x0a /* Public */,
      60,    0,  446,    2, 0x0a /* Public */,
      61,    0,  447,    2, 0x0a /* Public */,
      62,    0,  448,    2, 0x0a /* Public */,
      63,    0,  449,    2, 0x0a /* Public */,
      64,    0,  450,    2, 0x0a /* Public */,
      65,    0,  451,    2, 0x08 /* Private */,
      66,    2,  452,    2, 0x08 /* Private */,
      69,    0,  457,    2, 0x08 /* Private */,
      70,    0,  458,    2, 0x08 /* Private */,
      71,    0,  459,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, 0x80000000 | 13,    2,
    QMetaType::Void, 0x80000000 | 13,    2,
    QMetaType::Void, 0x80000000 | 13,    2,
    QMetaType::Void, QMetaType::QString, QMetaType::Double, QMetaType::Double, QMetaType::Bool, QMetaType::Bool, 0x80000000 | 17, 0x80000000 | 18,    2,    2,    2,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,    2,    2,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,    2,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::UInt,    2,    2,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,    2,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   26,
    QMetaType::Void, QMetaType::Int,   28,
    QMetaType::Void, QMetaType::Bool,   30,
    QMetaType::Void, QMetaType::Bool,   32,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   35,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Bool,   38,   39,   40,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   38,   39,
    QMetaType::Void, QMetaType::Int,   38,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Bool,   38,   39,   40,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   38,   39,
    QMetaType::Void, QMetaType::Int,   38,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   44,
    QMetaType::Void, QMetaType::Int,   46,
    QMetaType::Void, QMetaType::Int,   48,
    QMetaType::Void, QMetaType::Int,   50,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13, QMetaType::Double, QMetaType::Double,   53,   54,   55,
    QMetaType::Void, 0x80000000 | 13, QMetaType::Double,   53,   54,
    QMetaType::Void, 0x80000000 | 13,   53,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   67,   68,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ScribusView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ScribusView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->unitChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->layerChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->HaveSel(); break;
        case 3: _t->DocChanged(); break;
        case 4: _t->ItemGeom(); break;
        case 5: _t->PolyOpen(); break;
        case 6: _t->ItemCharStyle((*reinterpret_cast< const CharStyle(*)>(_a[1]))); break;
        case 7: _t->ItemTextAlign((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->ItemTextEffects((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->AddBM((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 10: _t->DelBM((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 11: _t->ChBMText((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 12: _t->LoadElem((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4])),(*reinterpret_cast< bool(*)>(_a[5])),(*reinterpret_cast< ScribusDoc*(*)>(_a[6])),(*reinterpret_cast< ScribusView*(*)>(_a[7]))); break;
        case 13: _t->HavePoint((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 14: _t->ClipPo((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 15: _t->PolyStatus((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2]))); break;
        case 16: _t->MousePos((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 17: _t->iconSetChange(); break;
        case 18: _t->languageChange(); break;
        case 19: _t->toggleCMS((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->switchPreviewVisual((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 21: _t->togglePreviewEdit((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->togglePreview((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 23: _t->unitChange(); break;
        case 24: _t->setRulersShown((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 25: _t->slotZoom100(); break;
        case 26: _t->slotZoomIn((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 27: _t->slotZoomIn((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 28: _t->slotZoomIn((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 29: _t->slotZoomIn(); break;
        case 30: _t->slotZoomOut((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 31: _t->slotZoomOut((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 32: _t->slotZoomOut((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 33: _t->slotZoomOut(); break;
        case 34: _t->DrawNew(); break;
        case 35: _t->GotoPa((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 36: _t->GotoLayer((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 37: _t->GotoPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 38: _t->ChgUnit((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 39: _t->editExtendedImageProperties(); break;
        case 40: _t->RefreshGradient((*reinterpret_cast< PageItem*(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 41: _t->RefreshGradient((*reinterpret_cast< PageItem*(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 42: _t->RefreshGradient((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 43: _t->ToPicFrame(); break;
        case 44: _t->ToPolyFrame(); break;
        case 45: _t->ToTextFrame(); break;
        case 46: _t->ToBezierFrame(); break;
        case 47: _t->ToPathText(); break;
        case 48: _t->FromPathText(); break;
        case 49: _t->Bezier2Poly(); break;
        case 50: _t->PasteToPage(); break;
        case 51: _t->TextToPath(); break;
        case 52: _t->setZoom(); break;
        case 53: _t->setRulerPos((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 54: _t->setObjectUndoMode(); break;
        case 55: _t->setGlobalUndoMode(); break;
        case 56: _t->dragTimerTimeOut(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 9:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 10:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 11:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 12:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 5:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ScribusDoc* >(); break;
            case 6:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ScribusView* >(); break;
            }
            break;
        case 40:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 41:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 42:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ScribusView::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::unitChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::layerChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::HaveSel)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::DocChanged)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::ItemGeom)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::PolyOpen)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)(const CharStyle & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::ItemCharStyle)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::ItemTextAlign)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::ItemTextEffects)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)(PageItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::AddBM)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)(PageItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::DelBM)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)(PageItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::ChBMText)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)(QString , double , double , bool , bool , ScribusDoc * , ScribusView * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::LoadElem)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)(bool , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::HavePoint)) {
                *result = 13;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)(double , double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::ClipPo)) {
                *result = 14;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)(int , uint );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::PolyStatus)) {
                *result = 15;
                return;
            }
        }
        {
            using _t = void (ScribusView::*)(double , double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusView::MousePos)) {
                *result = 16;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ScribusView::staticMetaObject = { {
    QMetaObject::SuperData::link<QScrollArea::staticMetaObject>(),
    qt_meta_stringdata_ScribusView.data,
    qt_meta_data_ScribusView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ScribusView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ScribusView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ScribusView.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Observer<QRectF>"))
        return static_cast< Observer<QRectF>*>(this);
    return QScrollArea::qt_metacast(_clname);
}

int ScribusView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QScrollArea::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 57)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 57;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 57)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 57;
    }
    return _id;
}

// SIGNAL 0
void ScribusView::unitChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ScribusView::layerChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ScribusView::HaveSel()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ScribusView::DocChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void ScribusView::ItemGeom()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void ScribusView::PolyOpen()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void ScribusView::ItemCharStyle(const CharStyle & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void ScribusView::ItemTextAlign(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void ScribusView::ItemTextEffects(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void ScribusView::AddBM(PageItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void ScribusView::DelBM(PageItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void ScribusView::ChBMText(PageItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void ScribusView::LoadElem(QString _t1, double _t2, double _t3, bool _t4, bool _t5, ScribusDoc * _t6, ScribusView * _t7)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t6))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t7))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void ScribusView::HavePoint(bool _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}

// SIGNAL 14
void ScribusView::ClipPo(double _t1, double _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 14, _a);
}

// SIGNAL 15
void ScribusView::PolyStatus(int _t1, uint _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 15, _a);
}

// SIGNAL 16
void ScribusView::MousePos(double _t1, double _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 16, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
