/********************************************************************************
** Form generated from reading UI file 'prefs_pdfexportbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_PDFEXPORTBASE_H
#define UI_PREFS_PDFEXPORTBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/newmarginwidget.h>
#include "ui/fontembeddingcombo.h"
#include "ui/pdfversioncombo.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_Prefs_PDFExport
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QFrame *line;
    QTabWidget *tabWidget;
    QWidget *tabPages;
    QVBoxLayout *verticalLayout_3;
    QLabel *label_2;
    QFrame *line_2;
    QRadioButton *exportAllPagesRadioButton;
    QRadioButton *exportChosenPagesRadioButton;
    QHBoxLayout *horizontalLayout_3;
    QLineEdit *exportPageListLineEdit;
    QPushButton *exportRangeMorePushButton;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer_11;
    QLabel *label_8;
    QFrame *line_3;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_3;
    QComboBox *rotationComboBox;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout;
    QLabel *label_4;
    QToolButton *pageMirrorHorizontalToolButton;
    QToolButton *pageMirrorVerticalToolButton;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer_2;
    QWidget *tabFileOptions;
    QVBoxLayout *verticalLayout_7;
    QScrollArea *scrollArea_3;
    QWidget *scrollAreaWidgetContents_3;
    QVBoxLayout *verticalLayout_19;
    QFormLayout *formLayout_2;
    QLabel *label_9;
    PdfVersionCombo *pdfVersionComboBox;
    QLabel *label_10;
    QComboBox *pageBindingComboBox;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *clipToPrinterMarginsCheckBox;
    QCheckBox *generateThumbnailsCheckBox;
    QCheckBox *saveLinkedTextFramesAsArticlesCheckBox;
    QCheckBox *includeLayersCheckBox;
    QCheckBox *includeBookmarksCheckBox;
    QCheckBox *embedPDFAndEPSFilesCheckBox;
    QSpacerItem *verticalSpacer_3;
    QLabel *label_5;
    QFrame *line_4;
    QCheckBox *compressTextAndVectorGraphicsCheckBox;
    QFormLayout *formLayout;
    QLabel *label_6;
    QComboBox *imageCompressionMethodComboBox;
    QLabel *label_7;
    QComboBox *imageCompressionQualityComboBox;
    QHBoxLayout *horizontalLayout_6;
    QCheckBox *maxResolutionLimitCheckBox;
    QSpinBox *maxExportResolutionSpinBox;
    QSpacerItem *horizontalSpacer_6;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_19;
    QSpinBox *epsExportResolutionSpinBox;
    QSpacerItem *horizontalSpacer_5;
    QSpacerItem *verticalSpacer;
    QWidget *tabSecurity;
    QVBoxLayout *verticalLayout_4;
    QCheckBox *useEncryptionCheckBox;
    QSpacerItem *verticalSpacer_5;
    QLabel *label_11;
    QFrame *line_5;
    QFormLayout *formLayout_3;
    QLabel *label_13;
    QLineEdit *passwordOwnerLineEdit;
    QLabel *label_14;
    QLineEdit *passwordUserLineEdit;
    QSpacerItem *verticalSpacer_6;
    QLabel *label_12;
    QFrame *line_6;
    QCheckBox *allowPrintingCheckBox;
    QCheckBox *allowChangingCheckBox;
    QCheckBox *allowCopyingCheckBox;
    QCheckBox *allowAnnotatingCheckBox;
    QSpacerItem *verticalSpacer_4;
    QWidget *tabColor;
    QVBoxLayout *verticalLayout_5;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_8;
    QFormLayout *formLayout_4;
    QLabel *label_15;
    QComboBox *outputIntentionComboBox;
    QCheckBox *convertSpotsToProcessCheckBox;
    QSpacerItem *verticalSpacer_9;
    QLabel *solidColorsLabel;
    QFrame *solidColorsLine;
    QCheckBox *useSolidColorProfileCheckBox;
    QFormLayout *formLayout_8;
    QLabel *label_28;
    QComboBox *solidColorProfileComboBox;
    QLabel *label_27;
    QComboBox *solidColorRenderingIntentComboBox;
    QSpacerItem *verticalSpacer_12;
    QLabel *imagesLabel;
    QFrame *imagesLine;
    QCheckBox *useImageProfileCheckBox;
    QCheckBox *doNotUseEmbeddedImageProfileCheckBox;
    QFormLayout *formLayout_6;
    QLabel *label_22;
    QComboBox *imageProfileComboBox;
    QLabel *label_26;
    QComboBox *imageRenderingIntentComboBox;
    QSpacerItem *verticalSpacer_13;
    QLabel *label_29;
    QFrame *customRenderingLine;
    QCheckBox *useCustomRenderingCheckBox;
    QFormLayout *formLayout_9;
    QLabel *label_30;
    QComboBox *customRenderingColorComboBox;
    QLabel *label_31;
    QSpinBox *customRenderingFrequencySpinBox;
    QLabel *label_32;
    QSpinBox *customRenderingAngleSpinBox;
    QLabel *label_33;
    QComboBox *customRenderingSpotFunctionComboBox;
    QSpacerItem *verticalSpacer_7;
    QWidget *tabPrePress;
    QVBoxLayout *verticalLayout_6;
    QScrollArea *scrollArea_2;
    QWidget *scrollAreaWidgetContents_2;
    QVBoxLayout *verticalLayout_18;
    QLabel *label_16;
    QFrame *line_7;
    QGridLayout *gridLayout;
    QHBoxLayout *horizontalLayout_11;
    QLabel *label_43;
    ScrSpinBox *markLengthSpinBox;
    QCheckBox *printBleedMarksCheckBox;
    QCheckBox *printColorBarsCheckBox;
    QCheckBox *printRegistrationMarksCheckBox;
    QCheckBox *printPageInfoCheckBox;
    QHBoxLayout *horizontalLayout_13;
    QCheckBox *printCropMarksCheckBox;
    QHBoxLayout *horizontalLayout_12;
    QLabel *label_17;
    ScrSpinBox *markOffsetSpinBox;
    QSpacerItem *verticalSpacer_10;
    QLabel *label_18;
    QFrame *line_8;
    QCheckBox *useDocumentBleedsCheckBox;
    QHBoxLayout *horizontalLayout_4;
    NewMarginWidget *bleedsWidget;
    QSpacerItem *horizontalSpacer_4;
    QSpacerItem *bleedSettingsVerticalSpacer;
    QLabel *label_23;
    QFrame *line_9;
    QFormLayout *formLayout_5;
    QLabel *label_24;
    QComboBox *pdfx3OutputProfileComboBox;
    QLabel *label_25;
    QLineEdit *pdfx3InfoStringLineEdit;
    QSpacerItem *verticalSpacer_8;
    QWidget *tabFonts;
    QVBoxLayout *verticalLayout_12;
    QLabel *label_20;
    QFrame *line_10;
    QHBoxLayout *horizontalLayout_7;
    QLabel *label_44;
    FontEmbeddingCombo *fontEmbeddingCombo;
    QHBoxLayout *horizontalLayout_8;
    QVBoxLayout *verticalLayout_11;
    QLabel *embedListLabel;
    QListWidget *embeddedFontsListWidget;
    QPushButton *embedAllButton;
    QVBoxLayout *verticalLayout_10;
    QSpacerItem *verticalSpacer_20;
    QPushButton *toSubsetButton;
    QPushButton *fromSubsetButton;
    QSpacerItem *verticalSpacer_14;
    QVBoxLayout *verticalLayout_9;
    QLabel *subsetListLabel;
    QListWidget *subsettedFontsListWidget;
    QPushButton *subsetAllButton;
    QWidget *tabExtras;
    QVBoxLayout *verticalLayout_17;
    QCheckBox *enabledEffectsCheckBox;
    QHBoxLayout *horizontalLayout_10;
    QListWidget *effectsPageListWidget;
    QVBoxLayout *verticalLayout_16;
    QFormLayout *formLayout_11;
    QLabel *label_37;
    QLabel *label_38;
    QLabel *label_39;
    QComboBox *effectTypeComboBox;
    QLabel *label_40;
    QComboBox *effectMovingDirectionComboBox;
    QLabel *label_41;
    QComboBox *effectInOutComboBox;
    QLabel *label_42;
    QComboBox *effectDirectionComboBox;
    QSpinBox *displayDurationSpinBox;
    QSpinBox *effectDurationSpinBox;
    QPushButton *applyEffectToAllPagesPushButton;
    QSpacerItem *verticalSpacer_19;
    QCheckBox *showPagePreviewsCheckBox;
    QWidget *tabViewer;
    QVBoxLayout *verticalLayout_15;
    QLabel *label_21;
    QFrame *line_11;
    QRadioButton *singlePageRadioButton;
    QRadioButton *continuousPagesRadioButton;
    QRadioButton *facingPagesLeftRadioButton;
    QRadioButton *facingPagesRightRadioButton;
    QSpacerItem *verticalSpacer_15;
    QLabel *label_34;
    QFrame *line_12;
    QHBoxLayout *horizontalLayout_9;
    QVBoxLayout *verticalLayout_14;
    QRadioButton *useViewerDefaultsRadioButton;
    QRadioButton *useFullScreenRadioButton;
    QRadioButton *useBookmarksRadioButton;
    QRadioButton *useThumbnailsRadioButton;
    QRadioButton *useLayersRadioButton;
    QVBoxLayout *verticalLayout_13;
    QCheckBox *hideViewerMenuBarCheckBox;
    QCheckBox *hideViewerToolBarCheckBox;
    QCheckBox *fitViewerWindowCheckBox;
    QSpacerItem *verticalSpacer_18;
    QSpacerItem *verticalSpacer_16;
    QLabel *label_35;
    QFrame *line_13;
    QFormLayout *formLayout_10;
    QLabel *label_36;
    QComboBox *startupJavascriptComboBox;
    QSpacerItem *verticalSpacer_17;
    QButtonGroup *rangeButtonGroup;

    void setupUi(QWidget *Prefs_PDFExport)
    {
        if (Prefs_PDFExport->objectName().isEmpty())
            Prefs_PDFExport->setObjectName(QString::fromUtf8("Prefs_PDFExport"));
        Prefs_PDFExport->resize(648, 550);
        verticalLayout = new QVBoxLayout(Prefs_PDFExport);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(Prefs_PDFExport);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        verticalLayout->addWidget(label);

        line = new QFrame(Prefs_PDFExport);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        tabWidget = new QTabWidget(Prefs_PDFExport);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tabPages = new QWidget();
        tabPages->setObjectName(QString::fromUtf8("tabPages"));
        verticalLayout_3 = new QVBoxLayout(tabPages);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        label_2 = new QLabel(tabPages);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
        label_2->setFont(font1);

        verticalLayout_3->addWidget(label_2);

        line_2 = new QFrame(tabPages);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        verticalLayout_3->addWidget(line_2);

        exportAllPagesRadioButton = new QRadioButton(tabPages);
        rangeButtonGroup = new QButtonGroup(Prefs_PDFExport);
        rangeButtonGroup->setObjectName(QString::fromUtf8("rangeButtonGroup"));
        rangeButtonGroup->addButton(exportAllPagesRadioButton);
        exportAllPagesRadioButton->setObjectName(QString::fromUtf8("exportAllPagesRadioButton"));

        verticalLayout_3->addWidget(exportAllPagesRadioButton);

        exportChosenPagesRadioButton = new QRadioButton(tabPages);
        rangeButtonGroup->addButton(exportChosenPagesRadioButton);
        exportChosenPagesRadioButton->setObjectName(QString::fromUtf8("exportChosenPagesRadioButton"));

        verticalLayout_3->addWidget(exportChosenPagesRadioButton);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        exportPageListLineEdit = new QLineEdit(tabPages);
        exportPageListLineEdit->setObjectName(QString::fromUtf8("exportPageListLineEdit"));

        horizontalLayout_3->addWidget(exportPageListLineEdit);

        exportRangeMorePushButton = new QPushButton(tabPages);
        exportRangeMorePushButton->setObjectName(QString::fromUtf8("exportRangeMorePushButton"));

        horizontalLayout_3->addWidget(exportRangeMorePushButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);


        verticalLayout_3->addLayout(horizontalLayout_3);

        verticalSpacer_11 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_3->addItem(verticalSpacer_11);

        label_8 = new QLabel(tabPages);
        label_8->setObjectName(QString::fromUtf8("label_8"));
        label_8->setFont(font1);

        verticalLayout_3->addWidget(label_8);

        line_3 = new QFrame(tabPages);
        line_3->setObjectName(QString::fromUtf8("line_3"));
        line_3->setFrameShape(QFrame::HLine);
        line_3->setFrameShadow(QFrame::Sunken);

        verticalLayout_3->addWidget(line_3);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        label_3 = new QLabel(tabPages);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        horizontalLayout_2->addWidget(label_3);

        rotationComboBox = new QComboBox(tabPages);
        rotationComboBox->setObjectName(QString::fromUtf8("rotationComboBox"));

        horizontalLayout_2->addWidget(rotationComboBox);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);


        verticalLayout_3->addLayout(horizontalLayout_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label_4 = new QLabel(tabPages);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        horizontalLayout->addWidget(label_4);

        pageMirrorHorizontalToolButton = new QToolButton(tabPages);
        pageMirrorHorizontalToolButton->setObjectName(QString::fromUtf8("pageMirrorHorizontalToolButton"));
        pageMirrorHorizontalToolButton->setCheckable(true);

        horizontalLayout->addWidget(pageMirrorHorizontalToolButton);

        pageMirrorVerticalToolButton = new QToolButton(tabPages);
        pageMirrorVerticalToolButton->setObjectName(QString::fromUtf8("pageMirrorVerticalToolButton"));
        pageMirrorVerticalToolButton->setCheckable(true);

        horizontalLayout->addWidget(pageMirrorVerticalToolButton);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);


        verticalLayout_3->addLayout(horizontalLayout);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_2);

        tabWidget->addTab(tabPages, QString());
        tabFileOptions = new QWidget();
        tabFileOptions->setObjectName(QString::fromUtf8("tabFileOptions"));
        verticalLayout_7 = new QVBoxLayout(tabFileOptions);
        verticalLayout_7->setObjectName(QString::fromUtf8("verticalLayout_7"));
        verticalLayout_7->setContentsMargins(0, 0, 0, 0);
        scrollArea_3 = new QScrollArea(tabFileOptions);
        scrollArea_3->setObjectName(QString::fromUtf8("scrollArea_3"));
        scrollArea_3->setFrameShape(QFrame::NoFrame);
        scrollArea_3->setFrameShadow(QFrame::Plain);
        scrollArea_3->setLineWidth(0);
        scrollArea_3->setWidgetResizable(true);
        scrollAreaWidgetContents_3 = new QWidget();
        scrollAreaWidgetContents_3->setObjectName(QString::fromUtf8("scrollAreaWidgetContents_3"));
        scrollAreaWidgetContents_3->setGeometry(QRect(0, 0, 577, 455));
        verticalLayout_19 = new QVBoxLayout(scrollAreaWidgetContents_3);
        verticalLayout_19->setObjectName(QString::fromUtf8("verticalLayout_19"));
        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName(QString::fromUtf8("formLayout_2"));
        formLayout_2->setFormAlignment(Qt::AlignHCenter|Qt::AlignTop);
        label_9 = new QLabel(scrollAreaWidgetContents_3);
        label_9->setObjectName(QString::fromUtf8("label_9"));

        formLayout_2->setWidget(0, QFormLayout::LabelRole, label_9);

        pdfVersionComboBox = new PdfVersionCombo(scrollAreaWidgetContents_3);
        pdfVersionComboBox->setObjectName(QString::fromUtf8("pdfVersionComboBox"));

        formLayout_2->setWidget(0, QFormLayout::FieldRole, pdfVersionComboBox);

        label_10 = new QLabel(scrollAreaWidgetContents_3);
        label_10->setObjectName(QString::fromUtf8("label_10"));

        formLayout_2->setWidget(1, QFormLayout::LabelRole, label_10);

        pageBindingComboBox = new QComboBox(scrollAreaWidgetContents_3);
        pageBindingComboBox->setObjectName(QString::fromUtf8("pageBindingComboBox"));

        formLayout_2->setWidget(1, QFormLayout::FieldRole, pageBindingComboBox);


        verticalLayout_19->addLayout(formLayout_2);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        clipToPrinterMarginsCheckBox = new QCheckBox(scrollAreaWidgetContents_3);
        clipToPrinterMarginsCheckBox->setObjectName(QString::fromUtf8("clipToPrinterMarginsCheckBox"));

        verticalLayout_2->addWidget(clipToPrinterMarginsCheckBox);

        generateThumbnailsCheckBox = new QCheckBox(scrollAreaWidgetContents_3);
        generateThumbnailsCheckBox->setObjectName(QString::fromUtf8("generateThumbnailsCheckBox"));

        verticalLayout_2->addWidget(generateThumbnailsCheckBox);

        saveLinkedTextFramesAsArticlesCheckBox = new QCheckBox(scrollAreaWidgetContents_3);
        saveLinkedTextFramesAsArticlesCheckBox->setObjectName(QString::fromUtf8("saveLinkedTextFramesAsArticlesCheckBox"));

        verticalLayout_2->addWidget(saveLinkedTextFramesAsArticlesCheckBox);

        includeLayersCheckBox = new QCheckBox(scrollAreaWidgetContents_3);
        includeLayersCheckBox->setObjectName(QString::fromUtf8("includeLayersCheckBox"));

        verticalLayout_2->addWidget(includeLayersCheckBox);

        includeBookmarksCheckBox = new QCheckBox(scrollAreaWidgetContents_3);
        includeBookmarksCheckBox->setObjectName(QString::fromUtf8("includeBookmarksCheckBox"));

        verticalLayout_2->addWidget(includeBookmarksCheckBox);

        embedPDFAndEPSFilesCheckBox = new QCheckBox(scrollAreaWidgetContents_3);
        embedPDFAndEPSFilesCheckBox->setObjectName(QString::fromUtf8("embedPDFAndEPSFilesCheckBox"));

        verticalLayout_2->addWidget(embedPDFAndEPSFilesCheckBox);


        verticalLayout_19->addLayout(verticalLayout_2);

        verticalSpacer_3 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_19->addItem(verticalSpacer_3);

        label_5 = new QLabel(scrollAreaWidgetContents_3);
        label_5->setObjectName(QString::fromUtf8("label_5"));
        label_5->setFont(font1);

        verticalLayout_19->addWidget(label_5);

        line_4 = new QFrame(scrollAreaWidgetContents_3);
        line_4->setObjectName(QString::fromUtf8("line_4"));
        line_4->setFrameShape(QFrame::HLine);
        line_4->setFrameShadow(QFrame::Sunken);

        verticalLayout_19->addWidget(line_4);

        compressTextAndVectorGraphicsCheckBox = new QCheckBox(scrollAreaWidgetContents_3);
        compressTextAndVectorGraphicsCheckBox->setObjectName(QString::fromUtf8("compressTextAndVectorGraphicsCheckBox"));

        verticalLayout_19->addWidget(compressTextAndVectorGraphicsCheckBox);

        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setFormAlignment(Qt::AlignHCenter|Qt::AlignTop);
        label_6 = new QLabel(scrollAreaWidgetContents_3);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        formLayout->setWidget(0, QFormLayout::LabelRole, label_6);

        imageCompressionMethodComboBox = new QComboBox(scrollAreaWidgetContents_3);
        imageCompressionMethodComboBox->setObjectName(QString::fromUtf8("imageCompressionMethodComboBox"));

        formLayout->setWidget(0, QFormLayout::FieldRole, imageCompressionMethodComboBox);

        label_7 = new QLabel(scrollAreaWidgetContents_3);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        formLayout->setWidget(1, QFormLayout::LabelRole, label_7);

        imageCompressionQualityComboBox = new QComboBox(scrollAreaWidgetContents_3);
        imageCompressionQualityComboBox->setObjectName(QString::fromUtf8("imageCompressionQualityComboBox"));

        formLayout->setWidget(1, QFormLayout::FieldRole, imageCompressionQualityComboBox);


        verticalLayout_19->addLayout(formLayout);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        maxResolutionLimitCheckBox = new QCheckBox(scrollAreaWidgetContents_3);
        maxResolutionLimitCheckBox->setObjectName(QString::fromUtf8("maxResolutionLimitCheckBox"));

        horizontalLayout_6->addWidget(maxResolutionLimitCheckBox);

        maxExportResolutionSpinBox = new QSpinBox(scrollAreaWidgetContents_3);
        maxExportResolutionSpinBox->setObjectName(QString::fromUtf8("maxExportResolutionSpinBox"));
        maxExportResolutionSpinBox->setMinimum(35);
        maxExportResolutionSpinBox->setMaximum(4000);

        horizontalLayout_6->addWidget(maxExportResolutionSpinBox);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_6);


        verticalLayout_19->addLayout(horizontalLayout_6);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        label_19 = new QLabel(scrollAreaWidgetContents_3);
        label_19->setObjectName(QString::fromUtf8("label_19"));

        horizontalLayout_5->addWidget(label_19);

        epsExportResolutionSpinBox = new QSpinBox(scrollAreaWidgetContents_3);
        epsExportResolutionSpinBox->setObjectName(QString::fromUtf8("epsExportResolutionSpinBox"));
        epsExportResolutionSpinBox->setMinimum(35);
        epsExportResolutionSpinBox->setMaximum(4000);

        horizontalLayout_5->addWidget(epsExportResolutionSpinBox);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_5);


        verticalLayout_19->addLayout(horizontalLayout_5);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_19->addItem(verticalSpacer);

        scrollArea_3->setWidget(scrollAreaWidgetContents_3);

        verticalLayout_7->addWidget(scrollArea_3);

        tabWidget->addTab(tabFileOptions, QString());
        tabSecurity = new QWidget();
        tabSecurity->setObjectName(QString::fromUtf8("tabSecurity"));
        verticalLayout_4 = new QVBoxLayout(tabSecurity);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        useEncryptionCheckBox = new QCheckBox(tabSecurity);
        useEncryptionCheckBox->setObjectName(QString::fromUtf8("useEncryptionCheckBox"));

        verticalLayout_4->addWidget(useEncryptionCheckBox);

        verticalSpacer_5 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_4->addItem(verticalSpacer_5);

        label_11 = new QLabel(tabSecurity);
        label_11->setObjectName(QString::fromUtf8("label_11"));
        label_11->setFont(font1);

        verticalLayout_4->addWidget(label_11);

        line_5 = new QFrame(tabSecurity);
        line_5->setObjectName(QString::fromUtf8("line_5"));
        line_5->setFrameShape(QFrame::HLine);
        line_5->setFrameShadow(QFrame::Sunken);

        verticalLayout_4->addWidget(line_5);

        formLayout_3 = new QFormLayout();
        formLayout_3->setObjectName(QString::fromUtf8("formLayout_3"));
        formLayout_3->setFormAlignment(Qt::AlignHCenter|Qt::AlignTop);
        label_13 = new QLabel(tabSecurity);
        label_13->setObjectName(QString::fromUtf8("label_13"));

        formLayout_3->setWidget(0, QFormLayout::LabelRole, label_13);

        passwordOwnerLineEdit = new QLineEdit(tabSecurity);
        passwordOwnerLineEdit->setObjectName(QString::fromUtf8("passwordOwnerLineEdit"));
        passwordOwnerLineEdit->setEchoMode(QLineEdit::Password);

        formLayout_3->setWidget(0, QFormLayout::FieldRole, passwordOwnerLineEdit);

        label_14 = new QLabel(tabSecurity);
        label_14->setObjectName(QString::fromUtf8("label_14"));

        formLayout_3->setWidget(1, QFormLayout::LabelRole, label_14);

        passwordUserLineEdit = new QLineEdit(tabSecurity);
        passwordUserLineEdit->setObjectName(QString::fromUtf8("passwordUserLineEdit"));
        passwordUserLineEdit->setEchoMode(QLineEdit::Password);

        formLayout_3->setWidget(1, QFormLayout::FieldRole, passwordUserLineEdit);


        verticalLayout_4->addLayout(formLayout_3);

        verticalSpacer_6 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_4->addItem(verticalSpacer_6);

        label_12 = new QLabel(tabSecurity);
        label_12->setObjectName(QString::fromUtf8("label_12"));
        label_12->setFont(font1);

        verticalLayout_4->addWidget(label_12);

        line_6 = new QFrame(tabSecurity);
        line_6->setObjectName(QString::fromUtf8("line_6"));
        line_6->setFrameShape(QFrame::HLine);
        line_6->setFrameShadow(QFrame::Sunken);

        verticalLayout_4->addWidget(line_6);

        allowPrintingCheckBox = new QCheckBox(tabSecurity);
        allowPrintingCheckBox->setObjectName(QString::fromUtf8("allowPrintingCheckBox"));

        verticalLayout_4->addWidget(allowPrintingCheckBox);

        allowChangingCheckBox = new QCheckBox(tabSecurity);
        allowChangingCheckBox->setObjectName(QString::fromUtf8("allowChangingCheckBox"));

        verticalLayout_4->addWidget(allowChangingCheckBox);

        allowCopyingCheckBox = new QCheckBox(tabSecurity);
        allowCopyingCheckBox->setObjectName(QString::fromUtf8("allowCopyingCheckBox"));

        verticalLayout_4->addWidget(allowCopyingCheckBox);

        allowAnnotatingCheckBox = new QCheckBox(tabSecurity);
        allowAnnotatingCheckBox->setObjectName(QString::fromUtf8("allowAnnotatingCheckBox"));

        verticalLayout_4->addWidget(allowAnnotatingCheckBox);

        verticalSpacer_4 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_4);

        tabWidget->addTab(tabSecurity, QString());
        tabColor = new QWidget();
        tabColor->setObjectName(QString::fromUtf8("tabColor"));
        verticalLayout_5 = new QVBoxLayout(tabColor);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        scrollArea = new QScrollArea(tabColor);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setLineWidth(0);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 266, 606));
        verticalLayout_8 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_8->setObjectName(QString::fromUtf8("verticalLayout_8"));
        formLayout_4 = new QFormLayout();
        formLayout_4->setObjectName(QString::fromUtf8("formLayout_4"));
        formLayout_4->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_15 = new QLabel(scrollAreaWidgetContents);
        label_15->setObjectName(QString::fromUtf8("label_15"));

        formLayout_4->setWidget(0, QFormLayout::LabelRole, label_15);

        outputIntentionComboBox = new QComboBox(scrollAreaWidgetContents);
        outputIntentionComboBox->setObjectName(QString::fromUtf8("outputIntentionComboBox"));

        formLayout_4->setWidget(0, QFormLayout::FieldRole, outputIntentionComboBox);


        verticalLayout_8->addLayout(formLayout_4);

        convertSpotsToProcessCheckBox = new QCheckBox(scrollAreaWidgetContents);
        convertSpotsToProcessCheckBox->setObjectName(QString::fromUtf8("convertSpotsToProcessCheckBox"));

        verticalLayout_8->addWidget(convertSpotsToProcessCheckBox);

        verticalSpacer_9 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_8->addItem(verticalSpacer_9);

        solidColorsLabel = new QLabel(scrollAreaWidgetContents);
        solidColorsLabel->setObjectName(QString::fromUtf8("solidColorsLabel"));
        solidColorsLabel->setFont(font1);

        verticalLayout_8->addWidget(solidColorsLabel);

        solidColorsLine = new QFrame(scrollAreaWidgetContents);
        solidColorsLine->setObjectName(QString::fromUtf8("solidColorsLine"));
        solidColorsLine->setFont(font);
        solidColorsLine->setFrameShape(QFrame::HLine);
        solidColorsLine->setFrameShadow(QFrame::Sunken);

        verticalLayout_8->addWidget(solidColorsLine);

        useSolidColorProfileCheckBox = new QCheckBox(scrollAreaWidgetContents);
        useSolidColorProfileCheckBox->setObjectName(QString::fromUtf8("useSolidColorProfileCheckBox"));

        verticalLayout_8->addWidget(useSolidColorProfileCheckBox);

        formLayout_8 = new QFormLayout();
        formLayout_8->setObjectName(QString::fromUtf8("formLayout_8"));
        label_28 = new QLabel(scrollAreaWidgetContents);
        label_28->setObjectName(QString::fromUtf8("label_28"));

        formLayout_8->setWidget(0, QFormLayout::LabelRole, label_28);

        solidColorProfileComboBox = new QComboBox(scrollAreaWidgetContents);
        solidColorProfileComboBox->setObjectName(QString::fromUtf8("solidColorProfileComboBox"));

        formLayout_8->setWidget(0, QFormLayout::FieldRole, solidColorProfileComboBox);

        label_27 = new QLabel(scrollAreaWidgetContents);
        label_27->setObjectName(QString::fromUtf8("label_27"));

        formLayout_8->setWidget(1, QFormLayout::LabelRole, label_27);

        solidColorRenderingIntentComboBox = new QComboBox(scrollAreaWidgetContents);
        solidColorRenderingIntentComboBox->setObjectName(QString::fromUtf8("solidColorRenderingIntentComboBox"));

        formLayout_8->setWidget(1, QFormLayout::FieldRole, solidColorRenderingIntentComboBox);


        verticalLayout_8->addLayout(formLayout_8);

        verticalSpacer_12 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_8->addItem(verticalSpacer_12);

        imagesLabel = new QLabel(scrollAreaWidgetContents);
        imagesLabel->setObjectName(QString::fromUtf8("imagesLabel"));
        imagesLabel->setFont(font1);

        verticalLayout_8->addWidget(imagesLabel);

        imagesLine = new QFrame(scrollAreaWidgetContents);
        imagesLine->setObjectName(QString::fromUtf8("imagesLine"));
        imagesLine->setFrameShape(QFrame::HLine);
        imagesLine->setFrameShadow(QFrame::Sunken);

        verticalLayout_8->addWidget(imagesLine);

        useImageProfileCheckBox = new QCheckBox(scrollAreaWidgetContents);
        useImageProfileCheckBox->setObjectName(QString::fromUtf8("useImageProfileCheckBox"));

        verticalLayout_8->addWidget(useImageProfileCheckBox);

        doNotUseEmbeddedImageProfileCheckBox = new QCheckBox(scrollAreaWidgetContents);
        doNotUseEmbeddedImageProfileCheckBox->setObjectName(QString::fromUtf8("doNotUseEmbeddedImageProfileCheckBox"));

        verticalLayout_8->addWidget(doNotUseEmbeddedImageProfileCheckBox);

        formLayout_6 = new QFormLayout();
        formLayout_6->setObjectName(QString::fromUtf8("formLayout_6"));
        label_22 = new QLabel(scrollAreaWidgetContents);
        label_22->setObjectName(QString::fromUtf8("label_22"));

        formLayout_6->setWidget(0, QFormLayout::LabelRole, label_22);

        imageProfileComboBox = new QComboBox(scrollAreaWidgetContents);
        imageProfileComboBox->setObjectName(QString::fromUtf8("imageProfileComboBox"));

        formLayout_6->setWidget(0, QFormLayout::FieldRole, imageProfileComboBox);

        label_26 = new QLabel(scrollAreaWidgetContents);
        label_26->setObjectName(QString::fromUtf8("label_26"));

        formLayout_6->setWidget(1, QFormLayout::LabelRole, label_26);

        imageRenderingIntentComboBox = new QComboBox(scrollAreaWidgetContents);
        imageRenderingIntentComboBox->setObjectName(QString::fromUtf8("imageRenderingIntentComboBox"));

        formLayout_6->setWidget(1, QFormLayout::FieldRole, imageRenderingIntentComboBox);


        verticalLayout_8->addLayout(formLayout_6);

        verticalSpacer_13 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Minimum);

        verticalLayout_8->addItem(verticalSpacer_13);

        label_29 = new QLabel(scrollAreaWidgetContents);
        label_29->setObjectName(QString::fromUtf8("label_29"));
        label_29->setFont(font1);

        verticalLayout_8->addWidget(label_29);

        customRenderingLine = new QFrame(scrollAreaWidgetContents);
        customRenderingLine->setObjectName(QString::fromUtf8("customRenderingLine"));
        customRenderingLine->setFrameShape(QFrame::HLine);
        customRenderingLine->setFrameShadow(QFrame::Sunken);

        verticalLayout_8->addWidget(customRenderingLine);

        useCustomRenderingCheckBox = new QCheckBox(scrollAreaWidgetContents);
        useCustomRenderingCheckBox->setObjectName(QString::fromUtf8("useCustomRenderingCheckBox"));

        verticalLayout_8->addWidget(useCustomRenderingCheckBox);

        formLayout_9 = new QFormLayout();
        formLayout_9->setObjectName(QString::fromUtf8("formLayout_9"));
        formLayout_9->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        label_30 = new QLabel(scrollAreaWidgetContents);
        label_30->setObjectName(QString::fromUtf8("label_30"));

        formLayout_9->setWidget(0, QFormLayout::LabelRole, label_30);

        customRenderingColorComboBox = new QComboBox(scrollAreaWidgetContents);
        customRenderingColorComboBox->setObjectName(QString::fromUtf8("customRenderingColorComboBox"));

        formLayout_9->setWidget(0, QFormLayout::FieldRole, customRenderingColorComboBox);

        label_31 = new QLabel(scrollAreaWidgetContents);
        label_31->setObjectName(QString::fromUtf8("label_31"));

        formLayout_9->setWidget(1, QFormLayout::LabelRole, label_31);

        customRenderingFrequencySpinBox = new QSpinBox(scrollAreaWidgetContents);
        customRenderingFrequencySpinBox->setObjectName(QString::fromUtf8("customRenderingFrequencySpinBox"));
        customRenderingFrequencySpinBox->setMinimum(10);
        customRenderingFrequencySpinBox->setMaximum(1000);

        formLayout_9->setWidget(1, QFormLayout::FieldRole, customRenderingFrequencySpinBox);

        label_32 = new QLabel(scrollAreaWidgetContents);
        label_32->setObjectName(QString::fromUtf8("label_32"));

        formLayout_9->setWidget(2, QFormLayout::LabelRole, label_32);

        customRenderingAngleSpinBox = new QSpinBox(scrollAreaWidgetContents);
        customRenderingAngleSpinBox->setObjectName(QString::fromUtf8("customRenderingAngleSpinBox"));
        customRenderingAngleSpinBox->setMinimum(-180);
        customRenderingAngleSpinBox->setMaximum(180);

        formLayout_9->setWidget(2, QFormLayout::FieldRole, customRenderingAngleSpinBox);

        label_33 = new QLabel(scrollAreaWidgetContents);
        label_33->setObjectName(QString::fromUtf8("label_33"));

        formLayout_9->setWidget(3, QFormLayout::LabelRole, label_33);

        customRenderingSpotFunctionComboBox = new QComboBox(scrollAreaWidgetContents);
        customRenderingSpotFunctionComboBox->setObjectName(QString::fromUtf8("customRenderingSpotFunctionComboBox"));

        formLayout_9->setWidget(3, QFormLayout::FieldRole, customRenderingSpotFunctionComboBox);


        verticalLayout_8->addLayout(formLayout_9);

        verticalSpacer_7 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_8->addItem(verticalSpacer_7);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_5->addWidget(scrollArea);

        tabWidget->addTab(tabColor, QString());
        tabPrePress = new QWidget();
        tabPrePress->setObjectName(QString::fromUtf8("tabPrePress"));
        verticalLayout_6 = new QVBoxLayout(tabPrePress);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        verticalLayout_6->setContentsMargins(0, 0, 0, 0);
        scrollArea_2 = new QScrollArea(tabPrePress);
        scrollArea_2->setObjectName(QString::fromUtf8("scrollArea_2"));
        scrollArea_2->setFrameShape(QFrame::NoFrame);
        scrollArea_2->setFrameShadow(QFrame::Plain);
        scrollArea_2->setLineWidth(0);
        scrollArea_2->setWidgetResizable(true);
        scrollAreaWidgetContents_2 = new QWidget();
        scrollAreaWidgetContents_2->setObjectName(QString::fromUtf8("scrollAreaWidgetContents_2"));
        scrollAreaWidgetContents_2->setGeometry(QRect(0, 0, 313, 436));
        verticalLayout_18 = new QVBoxLayout(scrollAreaWidgetContents_2);
        verticalLayout_18->setObjectName(QString::fromUtf8("verticalLayout_18"));
        label_16 = new QLabel(scrollAreaWidgetContents_2);
        label_16->setObjectName(QString::fromUtf8("label_16"));
        label_16->setFont(font1);

        verticalLayout_18->addWidget(label_16);

        line_7 = new QFrame(scrollAreaWidgetContents_2);
        line_7->setObjectName(QString::fromUtf8("line_7"));
        line_7->setFrameShape(QFrame::HLine);
        line_7->setFrameShadow(QFrame::Sunken);

        verticalLayout_18->addWidget(line_7);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName(QString::fromUtf8("horizontalLayout_11"));
        label_43 = new QLabel(scrollAreaWidgetContents_2);
        label_43->setObjectName(QString::fromUtf8("label_43"));

        horizontalLayout_11->addWidget(label_43);

        markLengthSpinBox = new ScrSpinBox(scrollAreaWidgetContents_2);
        markLengthSpinBox->setObjectName(QString::fromUtf8("markLengthSpinBox"));
        markLengthSpinBox->setMinimum(1.000000000000000);
        markLengthSpinBox->setMaximum(100.000000000000000);
        markLengthSpinBox->setValue(20.000000000000000);

        horizontalLayout_11->addWidget(markLengthSpinBox);


        gridLayout->addLayout(horizontalLayout_11, 0, 1, 1, 1);

        printBleedMarksCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        printBleedMarksCheckBox->setObjectName(QString::fromUtf8("printBleedMarksCheckBox"));

        gridLayout->addWidget(printBleedMarksCheckBox, 1, 0, 1, 1);

        printColorBarsCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        printColorBarsCheckBox->setObjectName(QString::fromUtf8("printColorBarsCheckBox"));

        gridLayout->addWidget(printColorBarsCheckBox, 2, 0, 1, 1);

        printRegistrationMarksCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        printRegistrationMarksCheckBox->setObjectName(QString::fromUtf8("printRegistrationMarksCheckBox"));

        gridLayout->addWidget(printRegistrationMarksCheckBox, 3, 0, 1, 1);

        printPageInfoCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        printPageInfoCheckBox->setObjectName(QString::fromUtf8("printPageInfoCheckBox"));

        gridLayout->addWidget(printPageInfoCheckBox, 4, 0, 1, 1);

        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setObjectName(QString::fromUtf8("horizontalLayout_13"));
        horizontalLayout_13->setContentsMargins(-1, -1, 0, 0);
        printCropMarksCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        printCropMarksCheckBox->setObjectName(QString::fromUtf8("printCropMarksCheckBox"));

        horizontalLayout_13->addWidget(printCropMarksCheckBox);


        gridLayout->addLayout(horizontalLayout_13, 0, 0, 1, 1);

        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName(QString::fromUtf8("horizontalLayout_12"));
        label_17 = new QLabel(scrollAreaWidgetContents_2);
        label_17->setObjectName(QString::fromUtf8("label_17"));

        horizontalLayout_12->addWidget(label_17);

        markOffsetSpinBox = new ScrSpinBox(scrollAreaWidgetContents_2);
        markOffsetSpinBox->setObjectName(QString::fromUtf8("markOffsetSpinBox"));

        horizontalLayout_12->addWidget(markOffsetSpinBox);


        gridLayout->addLayout(horizontalLayout_12, 1, 1, 1, 1);


        verticalLayout_18->addLayout(gridLayout);

        verticalSpacer_10 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_18->addItem(verticalSpacer_10);

        label_18 = new QLabel(scrollAreaWidgetContents_2);
        label_18->setObjectName(QString::fromUtf8("label_18"));
        label_18->setFont(font1);

        verticalLayout_18->addWidget(label_18);

        line_8 = new QFrame(scrollAreaWidgetContents_2);
        line_8->setObjectName(QString::fromUtf8("line_8"));
        line_8->setFrameShape(QFrame::HLine);
        line_8->setFrameShadow(QFrame::Sunken);

        verticalLayout_18->addWidget(line_8);

        useDocumentBleedsCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        useDocumentBleedsCheckBox->setObjectName(QString::fromUtf8("useDocumentBleedsCheckBox"));

        verticalLayout_18->addWidget(useDocumentBleedsCheckBox);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        bleedsWidget = new NewMarginWidget(scrollAreaWidgetContents_2);
        bleedsWidget->setObjectName(QString::fromUtf8("bleedsWidget"));

        horizontalLayout_4->addWidget(bleedsWidget);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_4);


        verticalLayout_18->addLayout(horizontalLayout_4);

        bleedSettingsVerticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_18->addItem(bleedSettingsVerticalSpacer);

        label_23 = new QLabel(scrollAreaWidgetContents_2);
        label_23->setObjectName(QString::fromUtf8("label_23"));
        label_23->setFont(font1);

        verticalLayout_18->addWidget(label_23);

        line_9 = new QFrame(scrollAreaWidgetContents_2);
        line_9->setObjectName(QString::fromUtf8("line_9"));
        line_9->setFrameShape(QFrame::HLine);
        line_9->setFrameShadow(QFrame::Sunken);

        verticalLayout_18->addWidget(line_9);

        formLayout_5 = new QFormLayout();
        formLayout_5->setObjectName(QString::fromUtf8("formLayout_5"));
        formLayout_5->setFormAlignment(Qt::AlignHCenter|Qt::AlignTop);
        label_24 = new QLabel(scrollAreaWidgetContents_2);
        label_24->setObjectName(QString::fromUtf8("label_24"));

        formLayout_5->setWidget(0, QFormLayout::LabelRole, label_24);

        pdfx3OutputProfileComboBox = new QComboBox(scrollAreaWidgetContents_2);
        pdfx3OutputProfileComboBox->setObjectName(QString::fromUtf8("pdfx3OutputProfileComboBox"));

        formLayout_5->setWidget(0, QFormLayout::FieldRole, pdfx3OutputProfileComboBox);

        label_25 = new QLabel(scrollAreaWidgetContents_2);
        label_25->setObjectName(QString::fromUtf8("label_25"));

        formLayout_5->setWidget(1, QFormLayout::LabelRole, label_25);

        pdfx3InfoStringLineEdit = new QLineEdit(scrollAreaWidgetContents_2);
        pdfx3InfoStringLineEdit->setObjectName(QString::fromUtf8("pdfx3InfoStringLineEdit"));

        formLayout_5->setWidget(1, QFormLayout::FieldRole, pdfx3InfoStringLineEdit);


        verticalLayout_18->addLayout(formLayout_5);

        verticalSpacer_8 = new QSpacerItem(20, 18, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_18->addItem(verticalSpacer_8);

        scrollArea_2->setWidget(scrollAreaWidgetContents_2);

        verticalLayout_6->addWidget(scrollArea_2);

        tabWidget->addTab(tabPrePress, QString());
        tabFonts = new QWidget();
        tabFonts->setObjectName(QString::fromUtf8("tabFonts"));
        verticalLayout_12 = new QVBoxLayout(tabFonts);
        verticalLayout_12->setObjectName(QString::fromUtf8("verticalLayout_12"));
        label_20 = new QLabel(tabFonts);
        label_20->setObjectName(QString::fromUtf8("label_20"));
        label_20->setFont(font1);

        verticalLayout_12->addWidget(label_20);

        line_10 = new QFrame(tabFonts);
        line_10->setObjectName(QString::fromUtf8("line_10"));
        line_10->setFrameShape(QFrame::HLine);
        line_10->setFrameShadow(QFrame::Sunken);

        verticalLayout_12->addWidget(line_10);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        label_44 = new QLabel(tabFonts);
        label_44->setObjectName(QString::fromUtf8("label_44"));
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label_44->sizePolicy().hasHeightForWidth());
        label_44->setSizePolicy(sizePolicy);

        horizontalLayout_7->addWidget(label_44);

        fontEmbeddingCombo = new FontEmbeddingCombo(tabFonts);
        fontEmbeddingCombo->setObjectName(QString::fromUtf8("fontEmbeddingCombo"));

        horizontalLayout_7->addWidget(fontEmbeddingCombo);


        verticalLayout_12->addLayout(horizontalLayout_7);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName(QString::fromUtf8("horizontalLayout_8"));
        verticalLayout_11 = new QVBoxLayout();
        verticalLayout_11->setObjectName(QString::fromUtf8("verticalLayout_11"));
        embedListLabel = new QLabel(tabFonts);
        embedListLabel->setObjectName(QString::fromUtf8("embedListLabel"));

        verticalLayout_11->addWidget(embedListLabel);

        embeddedFontsListWidget = new QListWidget(tabFonts);
        embeddedFontsListWidget->setObjectName(QString::fromUtf8("embeddedFontsListWidget"));

        verticalLayout_11->addWidget(embeddedFontsListWidget);

        embedAllButton = new QPushButton(tabFonts);
        embedAllButton->setObjectName(QString::fromUtf8("embedAllButton"));

        verticalLayout_11->addWidget(embedAllButton);


        horizontalLayout_8->addLayout(verticalLayout_11);

        verticalLayout_10 = new QVBoxLayout();
        verticalLayout_10->setObjectName(QString::fromUtf8("verticalLayout_10"));
        verticalSpacer_20 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_10->addItem(verticalSpacer_20);

        toSubsetButton = new QPushButton(tabFonts);
        toSubsetButton->setObjectName(QString::fromUtf8("toSubsetButton"));

        verticalLayout_10->addWidget(toSubsetButton);

        fromSubsetButton = new QPushButton(tabFonts);
        fromSubsetButton->setObjectName(QString::fromUtf8("fromSubsetButton"));
        QFont font2;
        font2.setStyleStrategy(QFont::PreferDefault);
        fromSubsetButton->setFont(font2);

        verticalLayout_10->addWidget(fromSubsetButton);

        verticalSpacer_14 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_10->addItem(verticalSpacer_14);


        horizontalLayout_8->addLayout(verticalLayout_10);

        verticalLayout_9 = new QVBoxLayout();
        verticalLayout_9->setObjectName(QString::fromUtf8("verticalLayout_9"));
        subsetListLabel = new QLabel(tabFonts);
        subsetListLabel->setObjectName(QString::fromUtf8("subsetListLabel"));

        verticalLayout_9->addWidget(subsetListLabel);

        subsettedFontsListWidget = new QListWidget(tabFonts);
        subsettedFontsListWidget->setObjectName(QString::fromUtf8("subsettedFontsListWidget"));

        verticalLayout_9->addWidget(subsettedFontsListWidget);

        subsetAllButton = new QPushButton(tabFonts);
        subsetAllButton->setObjectName(QString::fromUtf8("subsetAllButton"));

        verticalLayout_9->addWidget(subsetAllButton);


        horizontalLayout_8->addLayout(verticalLayout_9);


        verticalLayout_12->addLayout(horizontalLayout_8);

        tabWidget->addTab(tabFonts, QString());
        tabExtras = new QWidget();
        tabExtras->setObjectName(QString::fromUtf8("tabExtras"));
        verticalLayout_17 = new QVBoxLayout(tabExtras);
        verticalLayout_17->setObjectName(QString::fromUtf8("verticalLayout_17"));
        enabledEffectsCheckBox = new QCheckBox(tabExtras);
        enabledEffectsCheckBox->setObjectName(QString::fromUtf8("enabledEffectsCheckBox"));

        verticalLayout_17->addWidget(enabledEffectsCheckBox);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName(QString::fromUtf8("horizontalLayout_10"));
        effectsPageListWidget = new QListWidget(tabExtras);
        effectsPageListWidget->setObjectName(QString::fromUtf8("effectsPageListWidget"));

        horizontalLayout_10->addWidget(effectsPageListWidget);

        verticalLayout_16 = new QVBoxLayout();
        verticalLayout_16->setObjectName(QString::fromUtf8("verticalLayout_16"));
        formLayout_11 = new QFormLayout();
        formLayout_11->setObjectName(QString::fromUtf8("formLayout_11"));
        formLayout_11->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        label_37 = new QLabel(tabExtras);
        label_37->setObjectName(QString::fromUtf8("label_37"));

        formLayout_11->setWidget(0, QFormLayout::LabelRole, label_37);

        label_38 = new QLabel(tabExtras);
        label_38->setObjectName(QString::fromUtf8("label_38"));

        formLayout_11->setWidget(1, QFormLayout::LabelRole, label_38);

        label_39 = new QLabel(tabExtras);
        label_39->setObjectName(QString::fromUtf8("label_39"));

        formLayout_11->setWidget(2, QFormLayout::LabelRole, label_39);

        effectTypeComboBox = new QComboBox(tabExtras);
        effectTypeComboBox->setObjectName(QString::fromUtf8("effectTypeComboBox"));

        formLayout_11->setWidget(2, QFormLayout::FieldRole, effectTypeComboBox);

        label_40 = new QLabel(tabExtras);
        label_40->setObjectName(QString::fromUtf8("label_40"));

        formLayout_11->setWidget(3, QFormLayout::LabelRole, label_40);

        effectMovingDirectionComboBox = new QComboBox(tabExtras);
        effectMovingDirectionComboBox->setObjectName(QString::fromUtf8("effectMovingDirectionComboBox"));

        formLayout_11->setWidget(3, QFormLayout::FieldRole, effectMovingDirectionComboBox);

        label_41 = new QLabel(tabExtras);
        label_41->setObjectName(QString::fromUtf8("label_41"));

        formLayout_11->setWidget(4, QFormLayout::LabelRole, label_41);

        effectInOutComboBox = new QComboBox(tabExtras);
        effectInOutComboBox->setObjectName(QString::fromUtf8("effectInOutComboBox"));

        formLayout_11->setWidget(4, QFormLayout::FieldRole, effectInOutComboBox);

        label_42 = new QLabel(tabExtras);
        label_42->setObjectName(QString::fromUtf8("label_42"));

        formLayout_11->setWidget(5, QFormLayout::LabelRole, label_42);

        effectDirectionComboBox = new QComboBox(tabExtras);
        effectDirectionComboBox->setObjectName(QString::fromUtf8("effectDirectionComboBox"));

        formLayout_11->setWidget(5, QFormLayout::FieldRole, effectDirectionComboBox);

        displayDurationSpinBox = new QSpinBox(tabExtras);
        displayDurationSpinBox->setObjectName(QString::fromUtf8("displayDurationSpinBox"));
        displayDurationSpinBox->setMaximum(3600);

        formLayout_11->setWidget(0, QFormLayout::FieldRole, displayDurationSpinBox);

        effectDurationSpinBox = new QSpinBox(tabExtras);
        effectDurationSpinBox->setObjectName(QString::fromUtf8("effectDurationSpinBox"));
        effectDurationSpinBox->setMinimum(1);
        effectDurationSpinBox->setMaximum(3600);

        formLayout_11->setWidget(1, QFormLayout::FieldRole, effectDurationSpinBox);


        verticalLayout_16->addLayout(formLayout_11);

        applyEffectToAllPagesPushButton = new QPushButton(tabExtras);
        applyEffectToAllPagesPushButton->setObjectName(QString::fromUtf8("applyEffectToAllPagesPushButton"));

        verticalLayout_16->addWidget(applyEffectToAllPagesPushButton);

        verticalSpacer_19 = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_16->addItem(verticalSpacer_19);


        horizontalLayout_10->addLayout(verticalLayout_16);


        verticalLayout_17->addLayout(horizontalLayout_10);

        showPagePreviewsCheckBox = new QCheckBox(tabExtras);
        showPagePreviewsCheckBox->setObjectName(QString::fromUtf8("showPagePreviewsCheckBox"));

        verticalLayout_17->addWidget(showPagePreviewsCheckBox);

        tabWidget->addTab(tabExtras, QString());
        tabViewer = new QWidget();
        tabViewer->setObjectName(QString::fromUtf8("tabViewer"));
        verticalLayout_15 = new QVBoxLayout(tabViewer);
        verticalLayout_15->setObjectName(QString::fromUtf8("verticalLayout_15"));
        label_21 = new QLabel(tabViewer);
        label_21->setObjectName(QString::fromUtf8("label_21"));
        label_21->setFont(font1);

        verticalLayout_15->addWidget(label_21);

        line_11 = new QFrame(tabViewer);
        line_11->setObjectName(QString::fromUtf8("line_11"));
        line_11->setFrameShape(QFrame::HLine);
        line_11->setFrameShadow(QFrame::Sunken);

        verticalLayout_15->addWidget(line_11);

        singlePageRadioButton = new QRadioButton(tabViewer);
        singlePageRadioButton->setObjectName(QString::fromUtf8("singlePageRadioButton"));

        verticalLayout_15->addWidget(singlePageRadioButton);

        continuousPagesRadioButton = new QRadioButton(tabViewer);
        continuousPagesRadioButton->setObjectName(QString::fromUtf8("continuousPagesRadioButton"));

        verticalLayout_15->addWidget(continuousPagesRadioButton);

        facingPagesLeftRadioButton = new QRadioButton(tabViewer);
        facingPagesLeftRadioButton->setObjectName(QString::fromUtf8("facingPagesLeftRadioButton"));

        verticalLayout_15->addWidget(facingPagesLeftRadioButton);

        facingPagesRightRadioButton = new QRadioButton(tabViewer);
        facingPagesRightRadioButton->setObjectName(QString::fromUtf8("facingPagesRightRadioButton"));

        verticalLayout_15->addWidget(facingPagesRightRadioButton);

        verticalSpacer_15 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_15->addItem(verticalSpacer_15);

        label_34 = new QLabel(tabViewer);
        label_34->setObjectName(QString::fromUtf8("label_34"));
        label_34->setFont(font1);

        verticalLayout_15->addWidget(label_34);

        line_12 = new QFrame(tabViewer);
        line_12->setObjectName(QString::fromUtf8("line_12"));
        line_12->setFrameShape(QFrame::HLine);
        line_12->setFrameShadow(QFrame::Sunken);

        verticalLayout_15->addWidget(line_12);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName(QString::fromUtf8("horizontalLayout_9"));
        verticalLayout_14 = new QVBoxLayout();
        verticalLayout_14->setObjectName(QString::fromUtf8("verticalLayout_14"));
        useViewerDefaultsRadioButton = new QRadioButton(tabViewer);
        useViewerDefaultsRadioButton->setObjectName(QString::fromUtf8("useViewerDefaultsRadioButton"));

        verticalLayout_14->addWidget(useViewerDefaultsRadioButton);

        useFullScreenRadioButton = new QRadioButton(tabViewer);
        useFullScreenRadioButton->setObjectName(QString::fromUtf8("useFullScreenRadioButton"));

        verticalLayout_14->addWidget(useFullScreenRadioButton);

        useBookmarksRadioButton = new QRadioButton(tabViewer);
        useBookmarksRadioButton->setObjectName(QString::fromUtf8("useBookmarksRadioButton"));

        verticalLayout_14->addWidget(useBookmarksRadioButton);

        useThumbnailsRadioButton = new QRadioButton(tabViewer);
        useThumbnailsRadioButton->setObjectName(QString::fromUtf8("useThumbnailsRadioButton"));

        verticalLayout_14->addWidget(useThumbnailsRadioButton);

        useLayersRadioButton = new QRadioButton(tabViewer);
        useLayersRadioButton->setObjectName(QString::fromUtf8("useLayersRadioButton"));

        verticalLayout_14->addWidget(useLayersRadioButton);


        horizontalLayout_9->addLayout(verticalLayout_14);

        verticalLayout_13 = new QVBoxLayout();
        verticalLayout_13->setObjectName(QString::fromUtf8("verticalLayout_13"));
        hideViewerMenuBarCheckBox = new QCheckBox(tabViewer);
        hideViewerMenuBarCheckBox->setObjectName(QString::fromUtf8("hideViewerMenuBarCheckBox"));

        verticalLayout_13->addWidget(hideViewerMenuBarCheckBox);

        hideViewerToolBarCheckBox = new QCheckBox(tabViewer);
        hideViewerToolBarCheckBox->setObjectName(QString::fromUtf8("hideViewerToolBarCheckBox"));

        verticalLayout_13->addWidget(hideViewerToolBarCheckBox);

        fitViewerWindowCheckBox = new QCheckBox(tabViewer);
        fitViewerWindowCheckBox->setObjectName(QString::fromUtf8("fitViewerWindowCheckBox"));

        verticalLayout_13->addWidget(fitViewerWindowCheckBox);

        verticalSpacer_18 = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_13->addItem(verticalSpacer_18);


        horizontalLayout_9->addLayout(verticalLayout_13);


        verticalLayout_15->addLayout(horizontalLayout_9);

        verticalSpacer_16 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_15->addItem(verticalSpacer_16);

        label_35 = new QLabel(tabViewer);
        label_35->setObjectName(QString::fromUtf8("label_35"));
        label_35->setFont(font1);

        verticalLayout_15->addWidget(label_35);

        line_13 = new QFrame(tabViewer);
        line_13->setObjectName(QString::fromUtf8("line_13"));
        line_13->setFrameShape(QFrame::HLine);
        line_13->setFrameShadow(QFrame::Sunken);

        verticalLayout_15->addWidget(line_13);

        formLayout_10 = new QFormLayout();
        formLayout_10->setObjectName(QString::fromUtf8("formLayout_10"));
        label_36 = new QLabel(tabViewer);
        label_36->setObjectName(QString::fromUtf8("label_36"));

        formLayout_10->setWidget(0, QFormLayout::LabelRole, label_36);

        startupJavascriptComboBox = new QComboBox(tabViewer);
        startupJavascriptComboBox->setObjectName(QString::fromUtf8("startupJavascriptComboBox"));

        formLayout_10->setWidget(0, QFormLayout::FieldRole, startupJavascriptComboBox);


        verticalLayout_15->addLayout(formLayout_10);

        verticalSpacer_17 = new QSpacerItem(20, 60, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_15->addItem(verticalSpacer_17);

        tabWidget->addTab(tabViewer, QString());

        verticalLayout->addWidget(tabWidget);

        QWidget::setTabOrder(tabWidget, exportAllPagesRadioButton);
        QWidget::setTabOrder(exportAllPagesRadioButton, exportChosenPagesRadioButton);
        QWidget::setTabOrder(exportChosenPagesRadioButton, exportPageListLineEdit);
        QWidget::setTabOrder(exportPageListLineEdit, exportRangeMorePushButton);
        QWidget::setTabOrder(exportRangeMorePushButton, rotationComboBox);
        QWidget::setTabOrder(rotationComboBox, pageMirrorHorizontalToolButton);
        QWidget::setTabOrder(pageMirrorHorizontalToolButton, pageMirrorVerticalToolButton);
        QWidget::setTabOrder(pageMirrorVerticalToolButton, scrollArea_3);
        QWidget::setTabOrder(scrollArea_3, pdfVersionComboBox);
        QWidget::setTabOrder(pdfVersionComboBox, pageBindingComboBox);
        QWidget::setTabOrder(pageBindingComboBox, clipToPrinterMarginsCheckBox);
        QWidget::setTabOrder(clipToPrinterMarginsCheckBox, generateThumbnailsCheckBox);
        QWidget::setTabOrder(generateThumbnailsCheckBox, saveLinkedTextFramesAsArticlesCheckBox);
        QWidget::setTabOrder(saveLinkedTextFramesAsArticlesCheckBox, includeLayersCheckBox);
        QWidget::setTabOrder(includeLayersCheckBox, includeBookmarksCheckBox);
        QWidget::setTabOrder(includeBookmarksCheckBox, embedPDFAndEPSFilesCheckBox);
        QWidget::setTabOrder(embedPDFAndEPSFilesCheckBox, compressTextAndVectorGraphicsCheckBox);
        QWidget::setTabOrder(compressTextAndVectorGraphicsCheckBox, imageCompressionMethodComboBox);
        QWidget::setTabOrder(imageCompressionMethodComboBox, imageCompressionQualityComboBox);
        QWidget::setTabOrder(imageCompressionQualityComboBox, maxResolutionLimitCheckBox);
        QWidget::setTabOrder(maxResolutionLimitCheckBox, maxExportResolutionSpinBox);
        QWidget::setTabOrder(maxExportResolutionSpinBox, epsExportResolutionSpinBox);
        QWidget::setTabOrder(epsExportResolutionSpinBox, useEncryptionCheckBox);
        QWidget::setTabOrder(useEncryptionCheckBox, passwordOwnerLineEdit);
        QWidget::setTabOrder(passwordOwnerLineEdit, passwordUserLineEdit);
        QWidget::setTabOrder(passwordUserLineEdit, allowPrintingCheckBox);
        QWidget::setTabOrder(allowPrintingCheckBox, allowChangingCheckBox);
        QWidget::setTabOrder(allowChangingCheckBox, allowCopyingCheckBox);
        QWidget::setTabOrder(allowCopyingCheckBox, allowAnnotatingCheckBox);
        QWidget::setTabOrder(allowAnnotatingCheckBox, scrollArea);
        QWidget::setTabOrder(scrollArea, outputIntentionComboBox);
        QWidget::setTabOrder(outputIntentionComboBox, convertSpotsToProcessCheckBox);
        QWidget::setTabOrder(convertSpotsToProcessCheckBox, useSolidColorProfileCheckBox);
        QWidget::setTabOrder(useSolidColorProfileCheckBox, solidColorProfileComboBox);
        QWidget::setTabOrder(solidColorProfileComboBox, solidColorRenderingIntentComboBox);
        QWidget::setTabOrder(solidColorRenderingIntentComboBox, useImageProfileCheckBox);
        QWidget::setTabOrder(useImageProfileCheckBox, doNotUseEmbeddedImageProfileCheckBox);
        QWidget::setTabOrder(doNotUseEmbeddedImageProfileCheckBox, imageProfileComboBox);
        QWidget::setTabOrder(imageProfileComboBox, imageRenderingIntentComboBox);
        QWidget::setTabOrder(imageRenderingIntentComboBox, useCustomRenderingCheckBox);
        QWidget::setTabOrder(useCustomRenderingCheckBox, customRenderingColorComboBox);
        QWidget::setTabOrder(customRenderingColorComboBox, customRenderingFrequencySpinBox);
        QWidget::setTabOrder(customRenderingFrequencySpinBox, customRenderingAngleSpinBox);
        QWidget::setTabOrder(customRenderingAngleSpinBox, customRenderingSpotFunctionComboBox);
        QWidget::setTabOrder(customRenderingSpotFunctionComboBox, scrollArea_2);
        QWidget::setTabOrder(scrollArea_2, printCropMarksCheckBox);
        QWidget::setTabOrder(printCropMarksCheckBox, printBleedMarksCheckBox);
        QWidget::setTabOrder(printBleedMarksCheckBox, printColorBarsCheckBox);
        QWidget::setTabOrder(printColorBarsCheckBox, printRegistrationMarksCheckBox);
        QWidget::setTabOrder(printRegistrationMarksCheckBox, printPageInfoCheckBox);
        QWidget::setTabOrder(printPageInfoCheckBox, markLengthSpinBox);
        QWidget::setTabOrder(markLengthSpinBox, markOffsetSpinBox);
        QWidget::setTabOrder(markOffsetSpinBox, pdfx3OutputProfileComboBox);
        QWidget::setTabOrder(pdfx3OutputProfileComboBox, pdfx3InfoStringLineEdit);
        QWidget::setTabOrder(pdfx3InfoStringLineEdit, subsettedFontsListWidget);
        QWidget::setTabOrder(subsettedFontsListWidget, enabledEffectsCheckBox);
        QWidget::setTabOrder(enabledEffectsCheckBox, effectsPageListWidget);
        QWidget::setTabOrder(effectsPageListWidget, effectTypeComboBox);
        QWidget::setTabOrder(effectTypeComboBox, effectMovingDirectionComboBox);
        QWidget::setTabOrder(effectMovingDirectionComboBox, effectInOutComboBox);
        QWidget::setTabOrder(effectInOutComboBox, effectDirectionComboBox);
        QWidget::setTabOrder(effectDirectionComboBox, displayDurationSpinBox);
        QWidget::setTabOrder(displayDurationSpinBox, effectDurationSpinBox);
        QWidget::setTabOrder(effectDurationSpinBox, applyEffectToAllPagesPushButton);
        QWidget::setTabOrder(applyEffectToAllPagesPushButton, showPagePreviewsCheckBox);
        QWidget::setTabOrder(showPagePreviewsCheckBox, singlePageRadioButton);
        QWidget::setTabOrder(singlePageRadioButton, continuousPagesRadioButton);
        QWidget::setTabOrder(continuousPagesRadioButton, facingPagesLeftRadioButton);
        QWidget::setTabOrder(facingPagesLeftRadioButton, facingPagesRightRadioButton);
        QWidget::setTabOrder(facingPagesRightRadioButton, useViewerDefaultsRadioButton);
        QWidget::setTabOrder(useViewerDefaultsRadioButton, useFullScreenRadioButton);
        QWidget::setTabOrder(useFullScreenRadioButton, useBookmarksRadioButton);
        QWidget::setTabOrder(useBookmarksRadioButton, useThumbnailsRadioButton);
        QWidget::setTabOrder(useThumbnailsRadioButton, useLayersRadioButton);
        QWidget::setTabOrder(useLayersRadioButton, hideViewerMenuBarCheckBox);
        QWidget::setTabOrder(hideViewerMenuBarCheckBox, hideViewerToolBarCheckBox);
        QWidget::setTabOrder(hideViewerToolBarCheckBox, fitViewerWindowCheckBox);
        QWidget::setTabOrder(fitViewerWindowCheckBox, startupJavascriptComboBox);

        retranslateUi(Prefs_PDFExport);
        QObject::connect(printCropMarksCheckBox, SIGNAL(clicked(bool)), markLengthSpinBox, SLOT(setEnabled(bool)));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Prefs_PDFExport);
    } // setupUi

    void retranslateUi(QWidget *Prefs_PDFExport)
    {
        label->setText(QCoreApplication::translate("Prefs_PDFExport", "PDF Export", nullptr));
        label_2->setText(QCoreApplication::translate("Prefs_PDFExport", "Export Range", nullptr));
        exportAllPagesRadioButton->setText(QCoreApplication::translate("Prefs_PDFExport", "All Pages", nullptr));
        exportChosenPagesRadioButton->setText(QCoreApplication::translate("Prefs_PDFExport", "Choose Pages", nullptr));
        label_8->setText(QCoreApplication::translate("Prefs_PDFExport", "Rotation and Mirroring", nullptr));
        label_3->setText(QCoreApplication::translate("Prefs_PDFExport", "Rotation:", nullptr));
        label_4->setText(QCoreApplication::translate("Prefs_PDFExport", "Page Mirroring:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabPages), QCoreApplication::translate("Prefs_PDFExport", "Pages", nullptr));
        label_9->setText(QCoreApplication::translate("Prefs_PDFExport", "Compatibility:", nullptr));
        label_10->setText(QCoreApplication::translate("Prefs_PDFExport", "Page Binding:", nullptr));
        clipToPrinterMarginsCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Clip to Printer Margins", nullptr));
        generateThumbnailsCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Generate Thumbnails", nullptr));
        saveLinkedTextFramesAsArticlesCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Save Linked Text Frames as PDF Articles", nullptr));
        includeLayersCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Include Layers", nullptr));
        includeBookmarksCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Include Bookmarks", nullptr));
        embedPDFAndEPSFilesCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Embed PDF && EPS Files (Experimental)", nullptr));
        label_5->setText(QCoreApplication::translate("Prefs_PDFExport", "Compression", nullptr));
        compressTextAndVectorGraphicsCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Compress Text and Vector Graphics", nullptr));
        label_6->setText(QCoreApplication::translate("Prefs_PDFExport", "Image Compression Method:", nullptr));
        label_7->setText(QCoreApplication::translate("Prefs_PDFExport", "Image Compression Quality:", nullptr));
        maxResolutionLimitCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Limit Maximum Image Resolution to:", nullptr));
        maxExportResolutionSpinBox->setSuffix(QCoreApplication::translate("Prefs_PDFExport", " dpi", nullptr));
        label_19->setText(QCoreApplication::translate("Prefs_PDFExport", "Resolution for EPS Graphics:", nullptr));
        epsExportResolutionSpinBox->setSuffix(QCoreApplication::translate("Prefs_PDFExport", " dpi", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabFileOptions), QCoreApplication::translate("Prefs_PDFExport", "File Options", nullptr));
        useEncryptionCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Use Encryption", nullptr));
        label_11->setText(QCoreApplication::translate("Prefs_PDFExport", "Passwords", nullptr));
        label_13->setText(QCoreApplication::translate("Prefs_PDFExport", "Owner:", nullptr));
        label_14->setText(QCoreApplication::translate("Prefs_PDFExport", "User:", nullptr));
        label_12->setText(QCoreApplication::translate("Prefs_PDFExport", "Settings", nullptr));
        allowPrintingCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Allow Printing the Document", nullptr));
        allowChangingCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Allow Changing the Document", nullptr));
        allowCopyingCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Allow Copying Text and Graphics", nullptr));
        allowAnnotatingCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Allow Adding Annotations and Fields", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabSecurity), QCoreApplication::translate("Prefs_PDFExport", "Security", nullptr));
        label_15->setText(QCoreApplication::translate("Prefs_PDFExport", "Output Intended For:", nullptr));
        convertSpotsToProcessCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Convert Spot Colors to Process Colors", nullptr));
        solidColorsLabel->setText(QCoreApplication::translate("Prefs_PDFExport", "Solid Colors", nullptr));
        useSolidColorProfileCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Use Color Profile", nullptr));
        label_28->setText(QCoreApplication::translate("Prefs_PDFExport", "Profile:", nullptr));
        label_27->setText(QCoreApplication::translate("Prefs_PDFExport", "Rendering Intent:", nullptr));
        imagesLabel->setText(QCoreApplication::translate("Prefs_PDFExport", "Images", nullptr));
        useImageProfileCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Use Color Profile", nullptr));
        doNotUseEmbeddedImageProfileCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Do not use Embedded Color Profiles", nullptr));
        label_22->setText(QCoreApplication::translate("Prefs_PDFExport", "Profile:", nullptr));
        label_26->setText(QCoreApplication::translate("Prefs_PDFExport", "Rendering Intent:", nullptr));
        label_29->setText(QCoreApplication::translate("Prefs_PDFExport", "Custom Rendering", nullptr));
        useCustomRenderingCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Use Custom Rendering", nullptr));
        label_30->setText(QCoreApplication::translate("Prefs_PDFExport", "Color:", nullptr));
        label_31->setText(QCoreApplication::translate("Prefs_PDFExport", "Frequency:", nullptr));
        label_32->setText(QCoreApplication::translate("Prefs_PDFExport", "Angle:", nullptr));
        customRenderingAngleSpinBox->setSuffix(QCoreApplication::translate("Prefs_PDFExport", " \302\260", nullptr));
        label_33->setText(QCoreApplication::translate("Prefs_PDFExport", "Spot Function:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabColor), QCoreApplication::translate("Prefs_PDFExport", "Color", nullptr));
        label_16->setText(QCoreApplication::translate("Prefs_PDFExport", "Printer Marks", nullptr));
        label_43->setText(QCoreApplication::translate("Prefs_PDFExport", "Mark Length:", nullptr));
        printBleedMarksCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Bleed Marks", nullptr));
        printColorBarsCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Color Bars", nullptr));
        printRegistrationMarksCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Registration Marks", nullptr));
        printPageInfoCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Page Information", nullptr));
        printCropMarksCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Crop Marks", nullptr));
        label_17->setText(QCoreApplication::translate("Prefs_PDFExport", "Mark Offset:", nullptr));
        label_18->setText(QCoreApplication::translate("Prefs_PDFExport", "Bleed Settings", nullptr));
        useDocumentBleedsCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Use Document Bleeds", nullptr));
        label_23->setText(QCoreApplication::translate("Prefs_PDFExport", "PDF/X Output Intent", nullptr));
        label_24->setText(QCoreApplication::translate("Prefs_PDFExport", "Output Profile:", nullptr));
        label_25->setText(QCoreApplication::translate("Prefs_PDFExport", "Information String:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabPrePress), QCoreApplication::translate("Prefs_PDFExport", "Pre-Press", nullptr));
        label_20->setText(QCoreApplication::translate("Prefs_PDFExport", "Embedding", nullptr));
        label_44->setText(QCoreApplication::translate("Prefs_PDFExport", "Embedding mode:", nullptr));
        embedListLabel->setText(QCoreApplication::translate("Prefs_PDFExport", "Fonts to Embed", nullptr));
        embedAllButton->setText(QCoreApplication::translate("Prefs_PDFExport", "Embed All", nullptr));
        toSubsetButton->setText(QString());
        fromSubsetButton->setText(QString());
        subsetListLabel->setText(QCoreApplication::translate("Prefs_PDFExport", "Fonts to Subset", nullptr));
        subsetAllButton->setText(QCoreApplication::translate("Prefs_PDFExport", "Subset All", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabFonts), QCoreApplication::translate("Prefs_PDFExport", "Fonts", nullptr));
        enabledEffectsCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Enable Presentation Effects", nullptr));
        label_37->setText(QCoreApplication::translate("Prefs_PDFExport", "Display Duration:", nullptr));
        label_38->setText(QCoreApplication::translate("Prefs_PDFExport", "Effect Duration:", nullptr));
        label_39->setText(QCoreApplication::translate("Prefs_PDFExport", "Effect Type:", nullptr));
        label_40->setText(QCoreApplication::translate("Prefs_PDFExport", "Moving Lines:", nullptr));
        label_41->setText(QCoreApplication::translate("Prefs_PDFExport", "From the:", nullptr));
        label_42->setText(QCoreApplication::translate("Prefs_PDFExport", "Direction:", nullptr));
        displayDurationSpinBox->setSuffix(QCoreApplication::translate("Prefs_PDFExport", " s", nullptr));
        effectDurationSpinBox->setSuffix(QCoreApplication::translate("Prefs_PDFExport", " s", nullptr));
        applyEffectToAllPagesPushButton->setText(QCoreApplication::translate("Prefs_PDFExport", "Apply Effect to All Pages", nullptr));
        showPagePreviewsCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Show Page Previews", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabExtras), QCoreApplication::translate("Prefs_PDFExport", "Extras", nullptr));
        label_21->setText(QCoreApplication::translate("Prefs_PDFExport", "Document Layout", nullptr));
        singlePageRadioButton->setText(QCoreApplication::translate("Prefs_PDFExport", "Single Page", nullptr));
        continuousPagesRadioButton->setText(QCoreApplication::translate("Prefs_PDFExport", "Continuous Pages", nullptr));
        facingPagesLeftRadioButton->setText(QCoreApplication::translate("Prefs_PDFExport", "Facing Pages Left", nullptr));
        facingPagesRightRadioButton->setText(QCoreApplication::translate("Prefs_PDFExport", "Facing Pages Right", nullptr));
        label_34->setText(QCoreApplication::translate("Prefs_PDFExport", "Visual Appearance", nullptr));
        useViewerDefaultsRadioButton->setText(QCoreApplication::translate("Prefs_PDFExport", "Use Viewer's Default", nullptr));
        useFullScreenRadioButton->setText(QCoreApplication::translate("Prefs_PDFExport", "Use Full Screen Mode", nullptr));
        useBookmarksRadioButton->setText(QCoreApplication::translate("Prefs_PDFExport", "Display Bookmarks Tab", nullptr));
        useThumbnailsRadioButton->setText(QCoreApplication::translate("Prefs_PDFExport", "Display Thumbnails", nullptr));
        useLayersRadioButton->setText(QCoreApplication::translate("Prefs_PDFExport", "Display Layers Tab", nullptr));
        hideViewerMenuBarCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Hide Viewer's Menu Bar", nullptr));
        hideViewerToolBarCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Hide Viewer's Tool Bar", nullptr));
        fitViewerWindowCheckBox->setText(QCoreApplication::translate("Prefs_PDFExport", "Zoom Pages to fit Viewer Window", nullptr));
        label_35->setText(QCoreApplication::translate("Prefs_PDFExport", "Special Actions", nullptr));
        label_36->setText(QCoreApplication::translate("Prefs_PDFExport", "Javascript to be executed when PDF document is opened:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabViewer), QCoreApplication::translate("Prefs_PDFExport", "Viewer", nullptr));
        (void)Prefs_PDFExport;
    } // retranslateUi

};

namespace Ui {
    class Prefs_PDFExport: public Ui_Prefs_PDFExport {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_PDFEXPORTBASE_H
