/********************************************************************************
** Form generated from reading UI file 'pconsole.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PCONSOLE_H
#define UI_PCONSOLE_H

#include <QtCore/QVariant>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenu>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_PythonConsole
{
public:
    QAction *action_Open;
    QAction *action_Save;
    QAction *actionSave_As;
    QAction *action_Exit;
    QAction *action_Run;
    QAction *actionRun_As_Console;
    QAction *action_Save_Output;
    QWidget *centralwidget;
    QGridLayout *gridLayout;
    QSplitter *splitter;
    QTextEdit *commandEdit;
    QTextEdit *outputEdit;
    QMenuBar *menubar;
    QMenu *menu_File;
    QMenu *menu_Script;
    QStatusBar *statusbar;

    void setupUi(QMainWindow *PythonConsole)
    {
        if (PythonConsole->objectName().isEmpty())
            PythonConsole->setObjectName(QString::fromUtf8("PythonConsole"));
        PythonConsole->resize(535, 471);
        action_Open = new QAction(PythonConsole);
        action_Open->setObjectName(QString::fromUtf8("action_Open"));
        action_Save = new QAction(PythonConsole);
        action_Save->setObjectName(QString::fromUtf8("action_Save"));
        actionSave_As = new QAction(PythonConsole);
        actionSave_As->setObjectName(QString::fromUtf8("actionSave_As"));
        action_Exit = new QAction(PythonConsole);
        action_Exit->setObjectName(QString::fromUtf8("action_Exit"));
        action_Run = new QAction(PythonConsole);
        action_Run->setObjectName(QString::fromUtf8("action_Run"));
        actionRun_As_Console = new QAction(PythonConsole);
        actionRun_As_Console->setObjectName(QString::fromUtf8("actionRun_As_Console"));
        action_Save_Output = new QAction(PythonConsole);
        action_Save_Output->setObjectName(QString::fromUtf8("action_Save_Output"));
        centralwidget = new QWidget(PythonConsole);
        centralwidget->setObjectName(QString::fromUtf8("centralwidget"));
        gridLayout = new QGridLayout(centralwidget);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setHorizontalSpacing(6);
        gridLayout->setVerticalSpacing(6);
        gridLayout->setContentsMargins(9, 9, 9, 9);
        splitter = new QSplitter(centralwidget);
        splitter->setObjectName(QString::fromUtf8("splitter"));
        splitter->setOrientation(Qt::Vertical);
        commandEdit = new QTextEdit(splitter);
        commandEdit->setObjectName(QString::fromUtf8("commandEdit"));
        commandEdit->setLineWrapMode(QTextEdit::NoWrap);
        splitter->addWidget(commandEdit);
        outputEdit = new QTextEdit(splitter);
        outputEdit->setObjectName(QString::fromUtf8("outputEdit"));
        outputEdit->setLineWrapMode(QTextEdit::NoWrap);
        outputEdit->setReadOnly(true);
        outputEdit->setAcceptRichText(false);
        splitter->addWidget(outputEdit);

        gridLayout->addWidget(splitter, 0, 0, 1, 1);

        PythonConsole->setCentralWidget(centralwidget);
        menubar = new QMenuBar(PythonConsole);
        menubar->setObjectName(QString::fromUtf8("menubar"));
        menubar->setGeometry(QRect(0, 0, 535, 29));
        menu_File = new QMenu(menubar);
        menu_File->setObjectName(QString::fromUtf8("menu_File"));
        menu_Script = new QMenu(menubar);
        menu_Script->setObjectName(QString::fromUtf8("menu_Script"));
        PythonConsole->setMenuBar(menubar);
        statusbar = new QStatusBar(PythonConsole);
        statusbar->setObjectName(QString::fromUtf8("statusbar"));
        PythonConsole->setStatusBar(statusbar);

        menubar->addAction(menu_File->menuAction());
        menubar->addAction(menu_Script->menuAction());
        menu_File->addAction(action_Open);
        menu_File->addAction(action_Save);
        menu_File->addAction(actionSave_As);
        menu_File->addSeparator();
        menu_File->addAction(action_Exit);
        menu_Script->addAction(action_Run);
        menu_Script->addAction(actionRun_As_Console);
        menu_Script->addAction(action_Save_Output);

        retranslateUi(PythonConsole);

        QMetaObject::connectSlotsByName(PythonConsole);
    } // setupUi

    void retranslateUi(QMainWindow *PythonConsole)
    {
        PythonConsole->setWindowTitle(QCoreApplication::translate("PythonConsole", "Script Console", nullptr));
        action_Open->setText(QCoreApplication::translate("PythonConsole", "&Open...", nullptr));
        action_Save->setText(QCoreApplication::translate("PythonConsole", "&Save", nullptr));
        actionSave_As->setText(QCoreApplication::translate("PythonConsole", "Save &As...", nullptr));
        action_Exit->setText(QCoreApplication::translate("PythonConsole", "&Quit", nullptr));
        action_Run->setText(QCoreApplication::translate("PythonConsole", "&Run", nullptr));
        actionRun_As_Console->setText(QCoreApplication::translate("PythonConsole", "Run As &Console", nullptr));
        action_Save_Output->setText(QCoreApplication::translate("PythonConsole", "&Save Output...", nullptr));
        menu_File->setTitle(QCoreApplication::translate("PythonConsole", "&File", nullptr));
        menu_Script->setTitle(QCoreApplication::translate("PythonConsole", "&Script", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PythonConsole: public Ui_PythonConsole {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PCONSOLE_H
