/****************************************************************************
** Meta object code from reading C++ file 'latexeditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/latexeditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'latexeditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_LatexEditor_t {
    QByteArrayData data[28];
    char stringdata0[415];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LatexEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LatexEditor_t qt_meta_stringdata_LatexEditor = {
    {
QT_MOC_LITERAL(0, 0, 11), // "LatexEditor"
QT_MOC_LITERAL(1, 12, 17), // "extEditorFinished"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 8), // "exitCode"
QT_MOC_LITERAL(4, 40, 20), // "QProcess::ExitStatus"
QT_MOC_LITERAL(5, 61, 10), // "exitStatus"
QT_MOC_LITERAL(6, 72, 20), // "extEditorFileChanged"
QT_MOC_LITERAL(7, 93, 8), // "filename"
QT_MOC_LITERAL(8, 102, 14), // "extEditorError"
QT_MOC_LITERAL(9, 117, 22), // "QProcess::ProcessError"
QT_MOC_LITERAL(10, 140, 5), // "error"
QT_MOC_LITERAL(11, 146, 9), // "okClicked"
QT_MOC_LITERAL(12, 156, 13), // "cancelClicked"
QT_MOC_LITERAL(13, 170, 13), // "updateClicked"
QT_MOC_LITERAL(14, 184, 13), // "revertClicked"
QT_MOC_LITERAL(15, 198, 10), // "dpiChanged"
QT_MOC_LITERAL(16, 209, 14), // "formulaChanged"
QT_MOC_LITERAL(17, 224, 18), // "applicationChanged"
QT_MOC_LITERAL(18, 243, 13), // "latexFinished"
QT_MOC_LITERAL(19, 257, 12), // "stateChanged"
QT_MOC_LITERAL(20, 270, 22), // "QProcess::ProcessState"
QT_MOC_LITERAL(21, 293, 16), // "tagButtonClicked"
QT_MOC_LITERAL(22, 310, 19), // "insertButtonClicked"
QT_MOC_LITERAL(23, 330, 15), // "newItemSelected"
QT_MOC_LITERAL(24, 346, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(25, 363, 17), // "itemDoubleClicked"
QT_MOC_LITERAL(26, 381, 16), // "updateConfigFile"
QT_MOC_LITERAL(27, 398, 16) // "extEditorClicked"

    },
    "LatexEditor\0extEditorFinished\0\0exitCode\0"
    "QProcess::ExitStatus\0exitStatus\0"
    "extEditorFileChanged\0filename\0"
    "extEditorError\0QProcess::ProcessError\0"
    "error\0okClicked\0cancelClicked\0"
    "updateClicked\0revertClicked\0dpiChanged\0"
    "formulaChanged\0applicationChanged\0"
    "latexFinished\0stateChanged\0"
    "QProcess::ProcessState\0tagButtonClicked\0"
    "insertButtonClicked\0newItemSelected\0"
    "QListWidgetItem*\0itemDoubleClicked\0"
    "updateConfigFile\0extEditorClicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LatexEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,  104,    2, 0x09 /* Protected */,
       6,    1,  109,    2, 0x09 /* Protected */,
       8,    1,  112,    2, 0x09 /* Protected */,
      11,    0,  115,    2, 0x0a /* Public */,
      12,    0,  116,    2, 0x0a /* Public */,
      13,    1,  117,    2, 0x0a /* Public */,
      14,    1,  120,    2, 0x0a /* Public */,
      15,    0,  123,    2, 0x0a /* Public */,
      16,    2,  124,    2, 0x0a /* Public */,
      17,    0,  129,    2, 0x0a /* Public */,
      18,    0,  130,    2, 0x0a /* Public */,
      19,    1,  131,    2, 0x0a /* Public */,
      21,    1,  134,    2, 0x0a /* Public */,
      22,    1,  137,    2, 0x0a /* Public */,
      23,    2,  140,    2, 0x0a /* Public */,
      25,    1,  145,    2, 0x0a /* Public */,
      26,    0,  148,    2, 0x0a /* Public */,
      27,    0,  149,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, 0x80000000 | 4,    3,    5,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 20,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QObjectStar,    2,
    QMetaType::Void, 0x80000000 | 24, 0x80000000 | 24,    2,    2,
    QMetaType::Void, 0x80000000 | 24,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void LatexEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LatexEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->extEditorFinished((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QProcess::ExitStatus(*)>(_a[2]))); break;
        case 1: _t->extEditorFileChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->extEditorError((*reinterpret_cast< QProcess::ProcessError(*)>(_a[1]))); break;
        case 3: _t->okClicked(); break;
        case 4: _t->cancelClicked(); break;
        case 5: _t->updateClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->revertClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->dpiChanged(); break;
        case 8: _t->formulaChanged((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 9: _t->applicationChanged(); break;
        case 10: _t->latexFinished(); break;
        case 11: _t->stateChanged((*reinterpret_cast< QProcess::ProcessState(*)>(_a[1]))); break;
        case 12: _t->tagButtonClicked((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 13: _t->insertButtonClicked((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 14: _t->newItemSelected((*reinterpret_cast< QListWidgetItem*(*)>(_a[1])),(*reinterpret_cast< QListWidgetItem*(*)>(_a[2]))); break;
        case 15: _t->itemDoubleClicked((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 16: _t->updateConfigFile(); break;
        case 17: _t->extEditorClicked(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LatexEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_LatexEditor.data,
    qt_meta_data_LatexEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LatexEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LatexEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LatexEditor.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::LatexEditor"))
        return static_cast< Ui::LatexEditor*>(this);
    return QDialog::qt_metacast(_clname);
}

int LatexEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
    return _id;
}
struct qt_meta_stringdata_DataPushButton_t {
    QByteArrayData data[6];
    char stringdata0[59];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_DataPushButton_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_DataPushButton_t qt_meta_stringdata_DataPushButton = {
    {
QT_MOC_LITERAL(0, 0, 14), // "DataPushButton"
QT_MOC_LITERAL(1, 15, 15), // "clickedWithData"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 4), // "data"
QT_MOC_LITERAL(4, 37, 13), // "buttonClicked"
QT_MOC_LITERAL(5, 51, 7) // "checked"

    },
    "DataPushButton\0clickedWithData\0\0data\0"
    "buttonClicked\0checked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_DataPushButton[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   27,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QObjectStar,    3,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    5,

       0        // eod
};

void DataPushButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<DataPushButton *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->clickedWithData((*reinterpret_cast< QObject*(*)>(_a[1]))); break;
        case 1: _t->buttonClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (DataPushButton::*)(QObject * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&DataPushButton::clickedWithData)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject DataPushButton::staticMetaObject = { {
    QMetaObject::SuperData::link<QPushButton::staticMetaObject>(),
    qt_meta_stringdata_DataPushButton.data,
    qt_meta_data_DataPushButton,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *DataPushButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *DataPushButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_DataPushButton.stringdata0))
        return static_cast<void*>(this);
    return QPushButton::qt_metacast(_clname);
}

int DataPushButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPushButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void DataPushButton::clickedWithData(QObject * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
struct qt_meta_stringdata_StringPushButton_t {
    QByteArrayData data[6];
    char stringdata0[61];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StringPushButton_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StringPushButton_t qt_meta_stringdata_StringPushButton = {
    {
QT_MOC_LITERAL(0, 0, 16), // "StringPushButton"
QT_MOC_LITERAL(1, 17, 15), // "clickedWithData"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 4), // "data"
QT_MOC_LITERAL(4, 39, 13), // "buttonClicked"
QT_MOC_LITERAL(5, 53, 7) // "checked"

    },
    "StringPushButton\0clickedWithData\0\0"
    "data\0buttonClicked\0checked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StringPushButton[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       2,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   24,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,   27,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    5,

       0        // eod
};

void StringPushButton::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StringPushButton *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->clickedWithData((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->buttonClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StringPushButton::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StringPushButton::clickedWithData)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StringPushButton::staticMetaObject = { {
    QMetaObject::SuperData::link<QPushButton::staticMetaObject>(),
    qt_meta_stringdata_StringPushButton.data,
    qt_meta_data_StringPushButton,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StringPushButton::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StringPushButton::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StringPushButton.stringdata0))
        return static_cast<void*>(this);
    return QPushButton::qt_metacast(_clname);
}

int StringPushButton::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QPushButton::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 2)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 2;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 2)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 2;
    }
    return _id;
}

// SIGNAL 0
void StringPushButton::clickedWithData(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
