/********************************************************************************
** Form generated from reading UI file 'lensdialogbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_LENSDIALOGBASE_H
#define UI_LENSDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QGraphicsView>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_LensDialogBase
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *vboxLayout1;
    QGraphicsView *previewWidget;
    QHBoxLayout *hboxLayout1;
    QHBoxLayout *hboxLayout2;
    QPushButton *buttonAdd;
    QPushButton *buttonRemove;
    QHBoxLayout *hboxLayout3;
    QSpacerItem *spacerItem;
    QToolButton *buttonZoomI;
    QToolButton *buttonZoomOut;
    QVBoxLayout *vboxLayout2;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *label;
    QDoubleSpinBox *spinXPos;
    QLabel *label_2;
    QDoubleSpinBox *spinYPos;
    QRadioButton *buttonMagnify;
    QRadioButton *buttonFishEye;
    QLabel *label_3;
    QDoubleSpinBox *spinRadius;
    QLabel *label_4;
    QDoubleSpinBox *spinStrength;
    QSpacerItem *spacerItem1;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *LensDialogBase)
    {
        if (LensDialogBase->objectName().isEmpty())
            LensDialogBase->setObjectName(QString::fromUtf8("LensDialogBase"));
        LensDialogBase->resize(526, 372);
        vboxLayout = new QVBoxLayout(LensDialogBase);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        previewWidget = new QGraphicsView(LensDialogBase);
        previewWidget->setObjectName(QString::fromUtf8("previewWidget"));
        previewWidget->setMinimumSize(QSize(280, 280));

        vboxLayout1->addWidget(previewWidget);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        buttonAdd = new QPushButton(LensDialogBase);
        buttonAdd->setObjectName(QString::fromUtf8("buttonAdd"));

        hboxLayout2->addWidget(buttonAdd);

        buttonRemove = new QPushButton(LensDialogBase);
        buttonRemove->setObjectName(QString::fromUtf8("buttonRemove"));

        hboxLayout2->addWidget(buttonRemove);


        hboxLayout1->addLayout(hboxLayout2);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setObjectName(QString::fromUtf8("hboxLayout3"));
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout3->addItem(spacerItem);

        buttonZoomI = new QToolButton(LensDialogBase);
        buttonZoomI->setObjectName(QString::fromUtf8("buttonZoomI"));

        hboxLayout3->addWidget(buttonZoomI);

        buttonZoomOut = new QToolButton(LensDialogBase);
        buttonZoomOut->setObjectName(QString::fromUtf8("buttonZoomOut"));

        hboxLayout3->addWidget(buttonZoomOut);


        hboxLayout1->addLayout(hboxLayout3);


        vboxLayout1->addLayout(hboxLayout1);


        hboxLayout->addLayout(vboxLayout1);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        groupBox = new QGroupBox(LensDialogBase);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label = new QLabel(groupBox);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 0, 1, 1);

        spinXPos = new QDoubleSpinBox(groupBox);
        spinXPos->setObjectName(QString::fromUtf8("spinXPos"));
        spinXPos->setMinimum(-30000.000000000000000);
        spinXPos->setMaximum(30000.000000000000000);

        gridLayout->addWidget(spinXPos, 0, 1, 1, 1);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        spinYPos = new QDoubleSpinBox(groupBox);
        spinYPos->setObjectName(QString::fromUtf8("spinYPos"));
        spinYPos->setMinimum(-30000.000000000000000);
        spinYPos->setMaximum(30000.000000000000000);

        gridLayout->addWidget(spinYPos, 1, 1, 1, 1);

        buttonMagnify = new QRadioButton(groupBox);
        buttonMagnify->setObjectName(QString::fromUtf8("buttonMagnify"));
        buttonMagnify->setChecked(true);

        gridLayout->addWidget(buttonMagnify, 2, 0, 1, 2);

        buttonFishEye = new QRadioButton(groupBox);
        buttonFishEye->setObjectName(QString::fromUtf8("buttonFishEye"));

        gridLayout->addWidget(buttonFishEye, 3, 0, 1, 2);

        label_3 = new QLabel(groupBox);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 4, 0, 1, 1);

        spinRadius = new QDoubleSpinBox(groupBox);
        spinRadius->setObjectName(QString::fromUtf8("spinRadius"));
        spinRadius->setMinimum(-30000.000000000000000);
        spinRadius->setMaximum(30000.000000000000000);

        gridLayout->addWidget(spinRadius, 4, 1, 1, 1);

        label_4 = new QLabel(groupBox);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout->addWidget(label_4, 5, 0, 1, 1);

        spinStrength = new QDoubleSpinBox(groupBox);
        spinStrength->setObjectName(QString::fromUtf8("spinStrength"));
        spinStrength->setMinimum(0.000000000000000);
        spinStrength->setMaximum(100.000000000000000);

        gridLayout->addWidget(spinStrength, 5, 1, 1, 1);


        vboxLayout2->addWidget(groupBox);

        spacerItem1 = new QSpacerItem(165, 91, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout2->addItem(spacerItem1);


        hboxLayout->addLayout(vboxLayout2);


        vboxLayout->addLayout(hboxLayout);

        buttonBox = new QDialogButtonBox(LensDialogBase);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label->setBuddy(spinXPos);
        label_2->setBuddy(spinYPos);
        label_3->setBuddy(spinRadius);
        label_4->setBuddy(spinStrength);
#endif // QT_CONFIG(shortcut)

        retranslateUi(LensDialogBase);

        QMetaObject::connectSlotsByName(LensDialogBase);
    } // setupUi

    void retranslateUi(QDialog *LensDialogBase)
    {
        LensDialogBase->setWindowTitle(QCoreApplication::translate("LensDialogBase", "Optical Lens", nullptr));
#if QT_CONFIG(tooltip)
        buttonAdd->setToolTip(QCoreApplication::translate("LensDialogBase", "Add a new lens", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonAdd->setText(QCoreApplication::translate("LensDialogBase", "&Add Lens", nullptr));
#if QT_CONFIG(tooltip)
        buttonRemove->setToolTip(QCoreApplication::translate("LensDialogBase", "Remove selected lens", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRemove->setText(QCoreApplication::translate("LensDialogBase", "&Remove Lens", nullptr));
#if QT_CONFIG(tooltip)
        buttonZoomI->setToolTip(QCoreApplication::translate("LensDialogBase", "Zoom In", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonZoomI->setText(QCoreApplication::translate("LensDialogBase", "+", nullptr));
#if QT_CONFIG(tooltip)
        buttonZoomOut->setToolTip(QCoreApplication::translate("LensDialogBase", "Zoom Out", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonZoomOut->setText(QCoreApplication::translate("LensDialogBase", "-", nullptr));
        groupBox->setTitle(QCoreApplication::translate("LensDialogBase", "Lens Parameters", nullptr));
        label->setText(QCoreApplication::translate("LensDialogBase", "&X Pos:", nullptr));
#if QT_CONFIG(tooltip)
        spinXPos->setToolTip(QCoreApplication::translate("LensDialogBase", "Horizontal position of the lens", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(QCoreApplication::translate("LensDialogBase", "&Y Pos:", nullptr));
#if QT_CONFIG(tooltip)
        spinYPos->setToolTip(QCoreApplication::translate("LensDialogBase", "Vertical position of the lens", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonMagnify->setToolTip(QCoreApplication::translate("LensDialogBase", "The selected lens acts like a magnification lens", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonMagnify->setText(QCoreApplication::translate("LensDialogBase", "&Magnification Lens", nullptr));
#if QT_CONFIG(tooltip)
        buttonFishEye->setToolTip(QCoreApplication::translate("LensDialogBase", "The selected lens acts like a fish eye lens", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonFishEye->setText(QCoreApplication::translate("LensDialogBase", "&Fish Eye Lens", nullptr));
        label_3->setText(QCoreApplication::translate("LensDialogBase", "Ra&dius:", nullptr));
#if QT_CONFIG(tooltip)
        spinRadius->setToolTip(QCoreApplication::translate("LensDialogBase", "Radius of the lens", nullptr));
#endif // QT_CONFIG(tooltip)
        label_4->setText(QCoreApplication::translate("LensDialogBase", "&Strength:", nullptr));
#if QT_CONFIG(tooltip)
        spinStrength->setToolTip(QCoreApplication::translate("LensDialogBase", "Strength of the lens", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class LensDialogBase: public Ui_LensDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_LENSDIALOGBASE_H
