/****************************************************************************
** Meta object code from reading C++ file 'contentpalette_default.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/contentpalette_default.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'contentpalette_default.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ContentPalette_Default_t {
    QByteArrayData data[17];
    char stringdata0[199];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ContentPalette_Default_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ContentPalette_Default_t qt_meta_stringdata_ContentPalette_Default = {
    {
QT_MOC_LITERAL(0, 0, 22), // "ContentPalette_Default"
QT_MOC_LITERAL(1, 23, 13), // "setMainWindow"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 18), // "ScribusMainWindow*"
QT_MOC_LITERAL(4, 57, 2), // "mw"
QT_MOC_LITERAL(5, 60, 6), // "setDoc"
QT_MOC_LITERAL(6, 67, 11), // "ScribusDoc*"
QT_MOC_LITERAL(7, 79, 1), // "d"
QT_MOC_LITERAL(8, 81, 14), // "setCurrentItem"
QT_MOC_LITERAL(9, 96, 9), // "PageItem*"
QT_MOC_LITERAL(10, 106, 4), // "item"
QT_MOC_LITERAL(11, 111, 8), // "unsetDoc"
QT_MOC_LITERAL(12, 120, 9), // "unsetItem"
QT_MOC_LITERAL(13, 130, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(14, 153, 19), // "handleUpdateRequest"
QT_MOC_LITERAL(15, 173, 14), // "languageChange"
QT_MOC_LITERAL(16, 188, 10) // "unitChange"

    },
    "ContentPalette_Default\0setMainWindow\0"
    "\0ScribusMainWindow*\0mw\0setDoc\0ScribusDoc*\0"
    "d\0setCurrentItem\0PageItem*\0item\0"
    "unsetDoc\0unsetItem\0handleSelectionChanged\0"
    "handleUpdateRequest\0languageChange\0"
    "unitChange"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ContentPalette_Default[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x0a /* Public */,
       5,    1,   62,    2, 0x0a /* Public */,
       8,    1,   65,    2, 0x0a /* Public */,
      11,    0,   68,    2, 0x0a /* Public */,
      12,    0,   69,    2, 0x0a /* Public */,
      13,    0,   70,    2, 0x0a /* Public */,
      14,    1,   71,    2, 0x0a /* Public */,
      15,    0,   74,    2, 0x0a /* Public */,
      16,    0,   75,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ContentPalette_Default::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ContentPalette_Default *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setMainWindow((*reinterpret_cast< ScribusMainWindow*(*)>(_a[1]))); break;
        case 1: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 2: _t->setCurrentItem((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 3: _t->unsetDoc(); break;
        case 4: _t->unsetItem(); break;
        case 5: _t->handleSelectionChanged(); break;
        case 6: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->languageChange(); break;
        case 8: _t->unitChange(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 1:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ScribusDoc* >(); break;
            }
            break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ContentPalette_Default::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ContentPalette_Default.data,
    qt_meta_data_ContentPalette_Default,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ContentPalette_Default::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ContentPalette_Default::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ContentPalette_Default.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::ContentPalette_DefaultBase"))
        return static_cast< Ui::ContentPalette_DefaultBase*>(this);
    return QWidget::qt_metacast(_clname);
}

int ContentPalette_Default::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
