/****************************************************************************
** Meta object code from reading C++ file 'charselect.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/charselect.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'charselect.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CharSelect_t {
    QByteArrayData data[19];
    char stringdata0[312];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CharSelect_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CharSelect_t qt_meta_stringdata_CharSelect = {
    {
QT_MOC_LITERAL(0, 0, 10), // "CharSelect"
QT_MOC_LITERAL(1, 11, 17), // "insertSpecialChar"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 21), // "insertUserSpecialChar"
QT_MOC_LITERAL(4, 52, 13), // "iconSetChange"
QT_MOC_LITERAL(5, 66, 28), // "enhancedDialogButton_toggled"
QT_MOC_LITERAL(6, 95, 22), // "slot_insertSpecialChar"
QT_MOC_LITERAL(7, 118, 23), // "slot_insertSpecialChars"
QT_MOC_LITERAL(8, 142, 13), // "QVector<uint>"
QT_MOC_LITERAL(9, 156, 5), // "chars"
QT_MOC_LITERAL(10, 162, 26), // "slot_insertUserSpecialChar"
QT_MOC_LITERAL(11, 189, 4), // "font"
QT_MOC_LITERAL(12, 194, 21), // "slot_selectionChanged"
QT_MOC_LITERAL(13, 216, 14), // "QItemSelection"
QT_MOC_LITERAL(14, 231, 22), // "uniClearButton_clicked"
QT_MOC_LITERAL(15, 254, 21), // "uniLoadButton_clicked"
QT_MOC_LITERAL(16, 276, 21), // "uniSaveButton_clicked"
QT_MOC_LITERAL(17, 298, 11), // "userNewChar"
QT_MOC_LITERAL(18, 310, 1) // "i"

    },
    "CharSelect\0insertSpecialChar\0\0"
    "insertUserSpecialChar\0iconSetChange\0"
    "enhancedDialogButton_toggled\0"
    "slot_insertSpecialChar\0slot_insertSpecialChars\0"
    "QVector<uint>\0chars\0slot_insertUserSpecialChar\0"
    "font\0slot_selectionChanged\0QItemSelection\0"
    "uniClearButton_clicked\0uniLoadButton_clicked\0"
    "uniSaveButton_clicked\0userNewChar\0i"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CharSelect[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x06 /* Public */,
       3,    2,   75,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   80,    2, 0x08 /* Private */,
       5,    1,   81,    2, 0x08 /* Private */,
       6,    0,   84,    2, 0x08 /* Private */,
       7,    1,   85,    2, 0x08 /* Private */,
      10,    2,   88,    2, 0x08 /* Private */,
      12,    2,   93,    2, 0x08 /* Private */,
      14,    0,   98,    2, 0x08 /* Private */,
      15,    0,   99,    2, 0x08 /* Private */,
      16,    0,  100,    2, 0x08 /* Private */,
      17,    2,  101,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    2,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    2,   11,
    QMetaType::Void, 0x80000000 | 13, 0x80000000 | 13,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::UInt, QMetaType::QString,   18,   11,

       0        // eod
};

void CharSelect::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CharSelect *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->insertSpecialChar(); break;
        case 1: _t->insertUserSpecialChar((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 2: _t->iconSetChange(); break;
        case 3: _t->enhancedDialogButton_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 4: _t->slot_insertSpecialChar(); break;
        case 5: _t->slot_insertSpecialChars((*reinterpret_cast< const QVector<uint>(*)>(_a[1]))); break;
        case 6: _t->slot_insertUserSpecialChar((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 7: _t->slot_selectionChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 8: _t->uniClearButton_clicked(); break;
        case 9: _t->uniLoadButton_clicked(); break;
        case 10: _t->uniSaveButton_clicked(); break;
        case 11: _t->userNewChar((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<uint> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (CharSelect::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CharSelect::insertSpecialChar)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (CharSelect::*)(QString , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&CharSelect::insertUserSpecialChar)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CharSelect::staticMetaObject = { {
    QMetaObject::SuperData::link<ScrPaletteBase::staticMetaObject>(),
    qt_meta_stringdata_CharSelect.data,
    qt_meta_data_CharSelect,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CharSelect::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CharSelect::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CharSelect.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::CharSelect"))
        return static_cast< Ui::CharSelect*>(this);
    return ScrPaletteBase::qt_metacast(_clname);
}

int CharSelect::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScrPaletteBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void CharSelect::insertSpecialChar()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void CharSelect::insertUserSpecialChar(QString _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
