/********************************************************************************
** Form generated from reading UI file 'contentpalette_pagebase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONTENTPALETTE_PAGEBASE_H
#define UI_CONTENTPALETTE_PAGEBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ContentPalette_PageBase
{
public:
    QVBoxLayout *verticalLayout_3;
    QLabel *label;

    void setupUi(QWidget *ContentPalette_PageBase)
    {
        if (ContentPalette_PageBase->objectName().isEmpty())
            ContentPalette_PageBase->setObjectName(QString::fromUtf8("ContentPalette_PageBase"));
        ContentPalette_PageBase->resize(407, 230);
        verticalLayout_3 = new QVBoxLayout(ContentPalette_PageBase);
        verticalLayout_3->setSpacing(3);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(3, 3, 3, 3);
        label = new QLabel(ContentPalette_PageBase);
        label->setObjectName(QString::fromUtf8("label"));
        label->setAlignment(Qt::AlignCenter);

        verticalLayout_3->addWidget(label);


        retranslateUi(ContentPalette_PageBase);

        QMetaObject::connectSlotsByName(ContentPalette_PageBase);
    } // setupUi

    void retranslateUi(QWidget *ContentPalette_PageBase)
    {
        label->setText(QCoreApplication::translate("ContentPalette_PageBase", "Select an item to see its properties.", nullptr));
        (void)ContentPalette_PageBase;
    } // retranslateUi

};

namespace Ui {
    class ContentPalette_PageBase: public Ui_ContentPalette_PageBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONTENTPALETTE_PAGEBASE_H
