/****************************************************************************
** Meta object code from reading C++ file 'outlinepalette.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/outlinepalette.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'outlinepalette.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OutlineWidget_t {
    QByteArrayData data[1];
    char stringdata0[14];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OutlineWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OutlineWidget_t qt_meta_stringdata_OutlineWidget = {
    {
QT_MOC_LITERAL(0, 0, 13) // "OutlineWidget"

    },
    "OutlineWidget"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OutlineWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void OutlineWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject OutlineWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QTreeWidget::staticMetaObject>(),
    qt_meta_stringdata_OutlineWidget.data,
    qt_meta_data_OutlineWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OutlineWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OutlineWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OutlineWidget.stringdata0))
        return static_cast<void*>(this);
    return QTreeWidget::qt_metacast(_clname);
}

int OutlineWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTreeWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_OutlinePalette_t {
    QByteArrayData data[32];
    char stringdata0[396];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OutlinePalette_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OutlinePalette_t qt_meta_stringdata_OutlinePalette = {
    {
QT_MOC_LITERAL(0, 0, 14), // "OutlinePalette"
QT_MOC_LITERAL(1, 15, 19), // "selectElementByItem"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 9), // "PageItem*"
QT_MOC_LITERAL(4, 46, 17), // "editElementByItem"
QT_MOC_LITERAL(5, 64, 10), // "selectPage"
QT_MOC_LITERAL(6, 75, 16), // "selectMasterPage"
QT_MOC_LITERAL(7, 92, 9), // "BuildTree"
QT_MOC_LITERAL(8, 102, 9), // "storeVals"
QT_MOC_LITERAL(9, 112, 10), // "filterTree"
QT_MOC_LITERAL(10, 123, 7), // "keyword"
QT_MOC_LITERAL(11, 131, 13), // "iconSetChange"
QT_MOC_LITERAL(12, 145, 14), // "languageChange"
QT_MOC_LITERAL(13, 160, 14), // "slotShowSelect"
QT_MOC_LITERAL(14, 175, 6), // "pageNr"
QT_MOC_LITERAL(15, 182, 8), // "pageItem"
QT_MOC_LITERAL(16, 191, 15), // "setPaletteShown"
QT_MOC_LITERAL(17, 207, 14), // "slotRightClick"
QT_MOC_LITERAL(18, 222, 5), // "point"
QT_MOC_LITERAL(19, 228, 14), // "setActiveLayer"
QT_MOC_LITERAL(20, 243, 7), // "layerID"
QT_MOC_LITERAL(21, 251, 15), // "setLayerVisible"
QT_MOC_LITERAL(22, 267, 14), // "setLayerLocked"
QT_MOC_LITERAL(23, 282, 17), // "setLayerPrintable"
QT_MOC_LITERAL(24, 300, 14), // "slotRenameItem"
QT_MOC_LITERAL(25, 315, 12), // "slotDoRename"
QT_MOC_LITERAL(26, 328, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(27, 345, 3), // "ite"
QT_MOC_LITERAL(28, 349, 3), // "col"
QT_MOC_LITERAL(29, 353, 15), // "slotMultiSelect"
QT_MOC_LITERAL(30, 369, 10), // "slotSelect"
QT_MOC_LITERAL(31, 380, 15) // "slotDoubleClick"

    },
    "OutlinePalette\0selectElementByItem\0\0"
    "PageItem*\0editElementByItem\0selectPage\0"
    "selectMasterPage\0BuildTree\0storeVals\0"
    "filterTree\0keyword\0iconSetChange\0"
    "languageChange\0slotShowSelect\0pageNr\0"
    "pageItem\0setPaletteShown\0slotRightClick\0"
    "point\0setActiveLayer\0layerID\0"
    "setLayerVisible\0setLayerLocked\0"
    "setLayerPrintable\0slotRenameItem\0"
    "slotDoRename\0QTreeWidgetItem*\0ite\0col\0"
    "slotMultiSelect\0slotSelect\0slotDoubleClick"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OutlinePalette[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    2,  119,    2, 0x06 /* Public */,
       4,    1,  124,    2, 0x06 /* Public */,
       5,    1,  127,    2, 0x06 /* Public */,
       6,    1,  130,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    1,  133,    2, 0x0a /* Public */,
       7,    0,  136,    2, 0x2a /* Public | MethodCloned */,
       9,    1,  137,    2, 0x0a /* Public */,
      11,    0,  140,    2, 0x0a /* Public */,
      12,    0,  141,    2, 0x0a /* Public */,
      13,    2,  142,    2, 0x0a /* Public */,
      16,    1,  147,    2, 0x0a /* Public */,
      17,    1,  150,    2, 0x0a /* Public */,
      19,    1,  153,    2, 0x0a /* Public */,
      21,    1,  156,    2, 0x0a /* Public */,
      22,    1,  159,    2, 0x0a /* Public */,
      23,    1,  162,    2, 0x0a /* Public */,
      24,    0,  165,    2, 0x09 /* Protected */,
      25,    2,  166,    2, 0x09 /* Protected */,
      29,    0,  171,    2, 0x09 /* Protected */,
      30,    2,  172,    2, 0x09 /* Protected */,
      31,    2,  177,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3, QMetaType::Bool,    2,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Bool,    8,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 3,   14,   15,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QPoint,   18,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 26, QMetaType::Int,   27,   28,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 26, QMetaType::Int,   27,   28,
    QMetaType::Void, 0x80000000 | 26, QMetaType::Int,   27,   28,

       0        // eod
};

void OutlinePalette::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OutlinePalette *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->selectElementByItem((*reinterpret_cast< PageItem*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 1: _t->editElementByItem((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 2: _t->selectPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->selectMasterPage((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 4: _t->BuildTree((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->BuildTree(); break;
        case 6: _t->filterTree((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->iconSetChange(); break;
        case 8: _t->languageChange(); break;
        case 9: _t->slotShowSelect((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< PageItem*(*)>(_a[2]))); break;
        case 10: _t->setPaletteShown((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->slotRightClick((*reinterpret_cast< QPoint(*)>(_a[1]))); break;
        case 12: _t->setActiveLayer((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->setLayerVisible((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->setLayerLocked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->setLayerPrintable((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->slotRenameItem(); break;
        case 17: _t->slotDoRename((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 18: _t->slotMultiSelect(); break;
        case 19: _t->slotSelect((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 20: _t->slotDoubleClick((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (OutlinePalette::*)(PageItem * , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OutlinePalette::selectElementByItem)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (OutlinePalette::*)(PageItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OutlinePalette::editElementByItem)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (OutlinePalette::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OutlinePalette::selectPage)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (OutlinePalette::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&OutlinePalette::selectMasterPage)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject OutlinePalette::staticMetaObject = { {
    QMetaObject::SuperData::link<ScDockPalette::staticMetaObject>(),
    qt_meta_stringdata_OutlinePalette.data,
    qt_meta_data_OutlinePalette,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OutlinePalette::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OutlinePalette::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OutlinePalette.stringdata0))
        return static_cast<void*>(this);
    return ScDockPalette::qt_metacast(_clname);
}

int OutlinePalette::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScDockPalette::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 21;
    }
    return _id;
}

// SIGNAL 0
void OutlinePalette::selectElementByItem(PageItem * _t1, bool _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void OutlinePalette::editElementByItem(PageItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void OutlinePalette::selectPage(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void OutlinePalette::selectMasterPage(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
