/****************************************************************************
** Meta object code from reading C++ file 'pageitem_table.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/pageitem_table.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pageitem_table.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PageItem_Table_t {
    QByteArrayData data[17];
    char stringdata0[192];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PageItem_Table_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PageItem_Table_t qt_meta_stringdata_PageItem_Table = {
    {
QT_MOC_LITERAL(0, 0, 14), // "PageItem_Table"
QT_MOC_LITERAL(1, 15, 7), // "changed"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 16), // "selectionChanged"
QT_MOC_LITERAL(4, 41, 18), // "handleStyleChanged"
QT_MOC_LITERAL(5, 60, 4), // "rows"
QT_MOC_LITERAL(6, 65, 7), // "columns"
QT_MOC_LITERAL(7, 73, 9), // "fillColor"
QT_MOC_LITERAL(8, 83, 10), // "leftBorder"
QT_MOC_LITERAL(9, 94, 11), // "TableBorder"
QT_MOC_LITERAL(10, 106, 11), // "rightBorder"
QT_MOC_LITERAL(11, 118, 9), // "topBorder"
QT_MOC_LITERAL(12, 128, 12), // "bottomBorder"
QT_MOC_LITERAL(13, 141, 5), // "style"
QT_MOC_LITERAL(14, 147, 14), // "ResizeStrategy"
QT_MOC_LITERAL(15, 162, 13), // "MoveFollowing"
QT_MOC_LITERAL(16, 176, 15) // "ResizeFollowing"

    },
    "PageItem_Table\0changed\0\0selectionChanged\0"
    "handleStyleChanged\0rows\0columns\0"
    "fillColor\0leftBorder\0TableBorder\0"
    "rightBorder\0topBorder\0bottomBorder\0"
    "style\0ResizeStrategy\0MoveFollowing\0"
    "ResizeFollowing"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PageItem_Table[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       8,   32, // properties
       1,   64, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x06 /* Public */,
       3,    0,   30,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   31,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,

 // properties: name, type, flags
       5, QMetaType::Int, 0x00495001,
       6, QMetaType::Int, 0x00495001,
       7, QMetaType::QString, 0x00495107,
       8, 0x80000000 | 9, 0x0049510f,
      10, 0x80000000 | 9, 0x0049510f,
      11, 0x80000000 | 9, 0x0049510f,
      12, 0x80000000 | 9, 0x0049510f,
      13, QMetaType::QString, 0x00495107,

 // properties: notify_signal_id
       0,
       0,
       0,
       0,
       0,
       0,
       0,
       0,

 // enums: name, alias, flags, count, data
      14,   14, 0x0,    2,   69,

 // enum data: key, value
      15, uint(PageItem_Table::MoveFollowing),
      16, uint(PageItem_Table::ResizeFollowing),

       0        // eod
};

void PageItem_Table::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PageItem_Table *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->changed(); break;
        case 1: _t->selectionChanged(); break;
        case 2: _t->handleStyleChanged(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PageItem_Table::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageItem_Table::changed)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PageItem_Table::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageItem_Table::selectionChanged)) {
                *result = 1;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<PageItem_Table *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< int*>(_v) = _t->rows(); break;
        case 1: *reinterpret_cast< int*>(_v) = _t->columns(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->fillColor(); break;
        case 3: *reinterpret_cast< TableBorder*>(_v) = _t->leftBorder(); break;
        case 4: *reinterpret_cast< TableBorder*>(_v) = _t->rightBorder(); break;
        case 5: *reinterpret_cast< TableBorder*>(_v) = _t->topBorder(); break;
        case 6: *reinterpret_cast< TableBorder*>(_v) = _t->bottomBorder(); break;
        case 7: *reinterpret_cast< QString*>(_v) = _t->styleName(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<PageItem_Table *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 2: _t->setFillColor(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setLeftBorder(*reinterpret_cast< TableBorder*>(_v)); break;
        case 4: _t->setRightBorder(*reinterpret_cast< TableBorder*>(_v)); break;
        case 5: _t->setTopBorder(*reinterpret_cast< TableBorder*>(_v)); break;
        case 6: _t->setBottomBorder(*reinterpret_cast< TableBorder*>(_v)); break;
        case 7: _t->setStyle(*reinterpret_cast< QString*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
        PageItem_Table *_t = static_cast<PageItem_Table *>(_o);
        (void)_t;
        switch (_id) {
        case 2: _t->unsetFillColor(); break;
        case 3: _t->unsetLeftBorder(); break;
        case 4: _t->unsetRightBorder(); break;
        case 5: _t->unsetTopBorder(); break;
        case 6: _t->unsetBottomBorder(); break;
        case 7: _t->unsetStyle(); break;
        default: break;
        }
    }
#endif // QT_NO_PROPERTIES
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject PageItem_Table::staticMetaObject = { {
    QMetaObject::SuperData::link<PageItem::staticMetaObject>(),
    qt_meta_stringdata_PageItem_Table.data,
    qt_meta_data_PageItem_Table,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PageItem_Table::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageItem_Table::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PageItem_Table.stringdata0))
        return static_cast<void*>(this);
    return PageItem::qt_metacast(_clname);
}

int PageItem_Table::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = PageItem::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 8;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 8;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void PageItem_Table::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void PageItem_Table::selectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
