/********************************************************************************
** Form generated from reading UI file 'printdialogbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PRINTDIALOGBASE_H
#define UI_PRINTDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/printlanguagecombo.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_PrintDialogBase
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *PrintDialogGroup;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout;
    QComboBox *PrintDest;
    QPushButton *optionsButton;
    QSpacerItem *spacerItem;
    QHBoxLayout *hboxLayout1;
    QLabel *DateiT;
    QLineEdit *fileNameEdit;
    QToolButton *selectFileButton;
    QCheckBox *altComCheckBox;
    QHBoxLayout *hboxLayout2;
    QLabel *OthText;
    QLineEdit *altCommand;
    QGroupBox *rangeGroup;
    QGridLayout *gridLayout;
    QRadioButton *printAllRadio;
    QSpacerItem *spacerItem1;
    QLabel *TextLabel3;
    QSpinBox *Copies;
    QRadioButton *printCurrentRadio;
    QRadioButton *printRangeRadio;
    QLineEdit *pageNr;
    QPushButton *pageNrButton;
    QSpacerItem *spacerItem2;
    QTabWidget *printOptions;
    QWidget *tab;
    QGridLayout *gridLayoutOptions;
    QComboBox *printSepCombo;
    QComboBox *colorType;
    QComboBox *separationsCombo;
    PrintLanguageCombo *printLanguages;
    QCheckBox *usePDFMarks;
    QWidget *tab_2;
    QHBoxLayout *hboxLayout3;
    QGroupBox *pageOpts;
    QVBoxLayout *vboxLayout2;
    QCheckBox *mirrorHor;
    QCheckBox *mirrorVert;
    QCheckBox *setMediaSize;
    QCheckBox *clipMargins;
    QGroupBox *colorOpts;
    QVBoxLayout *vboxLayout3;
    QCheckBox *applyGCR;
    QCheckBox *convertSpots;
    QSpacerItem *spacerItem3;
    QWidget *tab_3;
    QGridLayout *gridLayout1;
    QCheckBox *bleedMarks;
    QCheckBox *cropMarks;
    QCheckBox *colorMarks;
    QCheckBox *registrationMarks;
    QHBoxLayout *horizontalLayout;
    QLabel *MarkTxt2;
    ScrSpinBox *markLength;
    QHBoxLayout *hboxLayout4;
    QLabel *MarkTxt1;
    ScrSpinBox *markOffset;
    QSpacerItem *verticalSpacer;
    QWidget *tab_4;
    QGridLayout *gridLayout2;
    QLabel *bleedTopText;
    ScrSpinBox *bleedTop;
    QLabel *bleedLeftText;
    ScrSpinBox *bleedRight;
    QLabel *bleedBottomText;
    ScrSpinBox *bleedBottom;
    QLabel *bleedRightText;
    ScrSpinBox *bleedLeft;
    QCheckBox *docBleeds;
    QHBoxLayout *hboxLayout5;
    QSpacerItem *spacerItem4;
    QPushButton *previewButton;
    QPushButton *okButton;
    QPushButton *cancelButton;

    void setupUi(QDialog *PrintDialogBase)
    {
        if (PrintDialogBase->objectName().isEmpty())
            PrintDialogBase->setObjectName(QString::fromUtf8("PrintDialogBase"));
        PrintDialogBase->resize(572, 529);
        PrintDialogBase->setSizeGripEnabled(false);
        vboxLayout = new QVBoxLayout(PrintDialogBase);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        PrintDialogGroup = new QGroupBox(PrintDialogBase);
        PrintDialogGroup->setObjectName(QString::fromUtf8("PrintDialogGroup"));
        vboxLayout1 = new QVBoxLayout(PrintDialogGroup);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        PrintDest = new QComboBox(PrintDialogGroup);
        PrintDest->setObjectName(QString::fromUtf8("PrintDest"));
        PrintDest->setMinimumSize(QSize(250, 0));

        hboxLayout->addWidget(PrintDest);

        optionsButton = new QPushButton(PrintDialogGroup);
        optionsButton->setObjectName(QString::fromUtf8("optionsButton"));

        hboxLayout->addWidget(optionsButton);

        spacerItem = new QSpacerItem(101, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);


        vboxLayout1->addLayout(hboxLayout);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        DateiT = new QLabel(PrintDialogGroup);
        DateiT->setObjectName(QString::fromUtf8("DateiT"));
        DateiT->setEnabled(false);
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(DateiT->sizePolicy().hasHeightForWidth());
        DateiT->setSizePolicy(sizePolicy);

        hboxLayout1->addWidget(DateiT);

        fileNameEdit = new QLineEdit(PrintDialogGroup);
        fileNameEdit->setObjectName(QString::fromUtf8("fileNameEdit"));

        hboxLayout1->addWidget(fileNameEdit);

        selectFileButton = new QToolButton(PrintDialogGroup);
        selectFileButton->setObjectName(QString::fromUtf8("selectFileButton"));
        selectFileButton->setEnabled(false);

        hboxLayout1->addWidget(selectFileButton);


        vboxLayout1->addLayout(hboxLayout1);

        altComCheckBox = new QCheckBox(PrintDialogGroup);
        altComCheckBox->setObjectName(QString::fromUtf8("altComCheckBox"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(altComCheckBox->sizePolicy().hasHeightForWidth());
        altComCheckBox->setSizePolicy(sizePolicy1);

        vboxLayout1->addWidget(altComCheckBox);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        OthText = new QLabel(PrintDialogGroup);
        OthText->setObjectName(QString::fromUtf8("OthText"));
        OthText->setEnabled(false);
        sizePolicy.setHeightForWidth(OthText->sizePolicy().hasHeightForWidth());
        OthText->setSizePolicy(sizePolicy);

        hboxLayout2->addWidget(OthText);

        altCommand = new QLineEdit(PrintDialogGroup);
        altCommand->setObjectName(QString::fromUtf8("altCommand"));
        altCommand->setEnabled(false);

        hboxLayout2->addWidget(altCommand);


        vboxLayout1->addLayout(hboxLayout2);


        vboxLayout->addWidget(PrintDialogGroup);

        rangeGroup = new QGroupBox(PrintDialogBase);
        rangeGroup->setObjectName(QString::fromUtf8("rangeGroup"));
        gridLayout = new QGridLayout(rangeGroup);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        printAllRadio = new QRadioButton(rangeGroup);
        printAllRadio->setObjectName(QString::fromUtf8("printAllRadio"));

        gridLayout->addWidget(printAllRadio, 0, 0, 1, 1);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(spacerItem1, 0, 2, 1, 1);

        TextLabel3 = new QLabel(rangeGroup);
        TextLabel3->setObjectName(QString::fromUtf8("TextLabel3"));

        gridLayout->addWidget(TextLabel3, 0, 3, 1, 2);

        Copies = new QSpinBox(rangeGroup);
        Copies->setObjectName(QString::fromUtf8("Copies"));
        Copies->setMinimum(1);
        Copies->setMaximum(1000);

        gridLayout->addWidget(Copies, 0, 5, 1, 1);

        printCurrentRadio = new QRadioButton(rangeGroup);
        printCurrentRadio->setObjectName(QString::fromUtf8("printCurrentRadio"));

        gridLayout->addWidget(printCurrentRadio, 1, 0, 1, 2);

        printRangeRadio = new QRadioButton(rangeGroup);
        printRangeRadio->setObjectName(QString::fromUtf8("printRangeRadio"));

        gridLayout->addWidget(printRangeRadio, 2, 0, 1, 1);

        pageNr = new QLineEdit(rangeGroup);
        pageNr->setObjectName(QString::fromUtf8("pageNr"));
        pageNr->setEnabled(false);

        gridLayout->addWidget(pageNr, 2, 1, 1, 2);

        pageNrButton = new QPushButton(rangeGroup);
        pageNrButton->setObjectName(QString::fromUtf8("pageNrButton"));
        pageNrButton->setEnabled(false);
        pageNrButton->setMinimumSize(QSize(16, 16));

        gridLayout->addWidget(pageNrButton, 2, 3, 1, 1);

        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(spacerItem2, 2, 4, 1, 1);


        vboxLayout->addWidget(rangeGroup);

        printOptions = new QTabWidget(PrintDialogBase);
        printOptions->setObjectName(QString::fromUtf8("printOptions"));
        tab = new QWidget();
        tab->setObjectName(QString::fromUtf8("tab"));
        gridLayoutOptions = new QGridLayout(tab);
        gridLayoutOptions->setObjectName(QString::fromUtf8("gridLayoutOptions"));
        printSepCombo = new QComboBox(tab);
        printSepCombo->addItem(QString());
        printSepCombo->addItem(QString());
        printSepCombo->setObjectName(QString::fromUtf8("printSepCombo"));

        gridLayoutOptions->addWidget(printSepCombo, 0, 0, 1, 1);

        colorType = new QComboBox(tab);
        colorType->addItem(QString());
        colorType->addItem(QString());
        colorType->setObjectName(QString::fromUtf8("colorType"));

        gridLayoutOptions->addWidget(colorType, 0, 1, 1, 1);

        separationsCombo = new QComboBox(tab);
        separationsCombo->setObjectName(QString::fromUtf8("separationsCombo"));
        separationsCombo->setEnabled(false);

        gridLayoutOptions->addWidget(separationsCombo, 1, 0, 1, 1);

        printLanguages = new PrintLanguageCombo(tab);
        printLanguages->setObjectName(QString::fromUtf8("printLanguages"));

        gridLayoutOptions->addWidget(printLanguages, 1, 1, 1, 1);

        usePDFMarks = new QCheckBox(tab);
        usePDFMarks->setObjectName(QString::fromUtf8("usePDFMarks"));

        gridLayoutOptions->addWidget(usePDFMarks, 2, 0, 1, 2);

        printOptions->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName(QString::fromUtf8("tab_2"));
        hboxLayout3 = new QHBoxLayout(tab_2);
        hboxLayout3->setObjectName(QString::fromUtf8("hboxLayout3"));
        pageOpts = new QGroupBox(tab_2);
        pageOpts->setObjectName(QString::fromUtf8("pageOpts"));
        vboxLayout2 = new QVBoxLayout(pageOpts);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        mirrorHor = new QCheckBox(pageOpts);
        mirrorHor->setObjectName(QString::fromUtf8("mirrorHor"));

        vboxLayout2->addWidget(mirrorHor);

        mirrorVert = new QCheckBox(pageOpts);
        mirrorVert->setObjectName(QString::fromUtf8("mirrorVert"));

        vboxLayout2->addWidget(mirrorVert);

        setMediaSize = new QCheckBox(pageOpts);
        setMediaSize->setObjectName(QString::fromUtf8("setMediaSize"));

        vboxLayout2->addWidget(setMediaSize);

        clipMargins = new QCheckBox(pageOpts);
        clipMargins->setObjectName(QString::fromUtf8("clipMargins"));

        vboxLayout2->addWidget(clipMargins);


        hboxLayout3->addWidget(pageOpts);

        colorOpts = new QGroupBox(tab_2);
        colorOpts->setObjectName(QString::fromUtf8("colorOpts"));
        vboxLayout3 = new QVBoxLayout(colorOpts);
        vboxLayout3->setObjectName(QString::fromUtf8("vboxLayout3"));
        applyGCR = new QCheckBox(colorOpts);
        applyGCR->setObjectName(QString::fromUtf8("applyGCR"));

        vboxLayout3->addWidget(applyGCR);

        convertSpots = new QCheckBox(colorOpts);
        convertSpots->setObjectName(QString::fromUtf8("convertSpots"));

        vboxLayout3->addWidget(convertSpots);

        spacerItem3 = new QSpacerItem(20, 19, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

        vboxLayout3->addItem(spacerItem3);


        hboxLayout3->addWidget(colorOpts);

        printOptions->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName(QString::fromUtf8("tab_3"));
        gridLayout1 = new QGridLayout(tab_3);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        bleedMarks = new QCheckBox(tab_3);
        bleedMarks->setObjectName(QString::fromUtf8("bleedMarks"));

        gridLayout1->addWidget(bleedMarks, 5, 0, 1, 1);

        cropMarks = new QCheckBox(tab_3);
        cropMarks->setObjectName(QString::fromUtf8("cropMarks"));

        gridLayout1->addWidget(cropMarks, 2, 0, 1, 1);

        colorMarks = new QCheckBox(tab_3);
        colorMarks->setObjectName(QString::fromUtf8("colorMarks"));

        gridLayout1->addWidget(colorMarks, 6, 0, 1, 1);

        registrationMarks = new QCheckBox(tab_3);
        registrationMarks->setObjectName(QString::fromUtf8("registrationMarks"));

        gridLayout1->addWidget(registrationMarks, 7, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        MarkTxt2 = new QLabel(tab_3);
        MarkTxt2->setObjectName(QString::fromUtf8("MarkTxt2"));

        horizontalLayout->addWidget(MarkTxt2);

        markLength = new ScrSpinBox(tab_3);
        markLength->setObjectName(QString::fromUtf8("markLength"));
        markLength->setMinimum(1.000000000000000);
        markLength->setMaximum(100.000000000000000);
        markLength->setValue(20.000000000000000);

        horizontalLayout->addWidget(markLength);


        gridLayout1->addLayout(horizontalLayout, 2, 1, 1, 1);

        hboxLayout4 = new QHBoxLayout();
        hboxLayout4->setObjectName(QString::fromUtf8("hboxLayout4"));
        MarkTxt1 = new QLabel(tab_3);
        MarkTxt1->setObjectName(QString::fromUtf8("MarkTxt1"));

        hboxLayout4->addWidget(MarkTxt1);

        markOffset = new ScrSpinBox(tab_3);
        markOffset->setObjectName(QString::fromUtf8("markOffset"));

        hboxLayout4->addWidget(markOffset);


        gridLayout1->addLayout(hboxLayout4, 5, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout1->addItem(verticalSpacer, 8, 0, 1, 1);

        printOptions->addTab(tab_3, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName(QString::fromUtf8("tab_4"));
        gridLayout2 = new QGridLayout(tab_4);
        gridLayout2->setObjectName(QString::fromUtf8("gridLayout2"));
        bleedTopText = new QLabel(tab_4);
        bleedTopText->setObjectName(QString::fromUtf8("bleedTopText"));

        gridLayout2->addWidget(bleedTopText, 1, 0, 1, 1);

        bleedTop = new ScrSpinBox(tab_4);
        bleedTop->setObjectName(QString::fromUtf8("bleedTop"));

        gridLayout2->addWidget(bleedTop, 1, 1, 1, 1);

        bleedLeftText = new QLabel(tab_4);
        bleedLeftText->setObjectName(QString::fromUtf8("bleedLeftText"));

        gridLayout2->addWidget(bleedLeftText, 1, 2, 1, 1);

        bleedRight = new ScrSpinBox(tab_4);
        bleedRight->setObjectName(QString::fromUtf8("bleedRight"));

        gridLayout2->addWidget(bleedRight, 1, 3, 1, 1);

        bleedBottomText = new QLabel(tab_4);
        bleedBottomText->setObjectName(QString::fromUtf8("bleedBottomText"));

        gridLayout2->addWidget(bleedBottomText, 2, 0, 1, 1);

        bleedBottom = new ScrSpinBox(tab_4);
        bleedBottom->setObjectName(QString::fromUtf8("bleedBottom"));

        gridLayout2->addWidget(bleedBottom, 2, 1, 1, 1);

        bleedRightText = new QLabel(tab_4);
        bleedRightText->setObjectName(QString::fromUtf8("bleedRightText"));

        gridLayout2->addWidget(bleedRightText, 2, 2, 1, 1);

        bleedLeft = new ScrSpinBox(tab_4);
        bleedLeft->setObjectName(QString::fromUtf8("bleedLeft"));

        gridLayout2->addWidget(bleedLeft, 2, 3, 1, 1);

        docBleeds = new QCheckBox(tab_4);
        docBleeds->setObjectName(QString::fromUtf8("docBleeds"));

        gridLayout2->addWidget(docBleeds, 0, 0, 1, 4);

        printOptions->addTab(tab_4, QString());

        vboxLayout->addWidget(printOptions);

        hboxLayout5 = new QHBoxLayout();
        hboxLayout5->setObjectName(QString::fromUtf8("hboxLayout5"));
        spacerItem4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout5->addItem(spacerItem4);

        previewButton = new QPushButton(PrintDialogBase);
        previewButton->setObjectName(QString::fromUtf8("previewButton"));
        previewButton->setAutoDefault(false);

        hboxLayout5->addWidget(previewButton);

        okButton = new QPushButton(PrintDialogBase);
        okButton->setObjectName(QString::fromUtf8("okButton"));

        hboxLayout5->addWidget(okButton);

        cancelButton = new QPushButton(PrintDialogBase);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));
        cancelButton->setAutoDefault(false);

        hboxLayout5->addWidget(cancelButton);


        vboxLayout->addLayout(hboxLayout5);

#if QT_CONFIG(shortcut)
        DateiT->setBuddy(fileNameEdit);
        OthText->setBuddy(altCommand);
        TextLabel3->setBuddy(Copies);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(PrintDest, optionsButton);
        QWidget::setTabOrder(optionsButton, fileNameEdit);
        QWidget::setTabOrder(fileNameEdit, selectFileButton);
        QWidget::setTabOrder(selectFileButton, altComCheckBox);
        QWidget::setTabOrder(altComCheckBox, altCommand);
        QWidget::setTabOrder(altCommand, printAllRadio);
        QWidget::setTabOrder(printAllRadio, printCurrentRadio);
        QWidget::setTabOrder(printCurrentRadio, printRangeRadio);
        QWidget::setTabOrder(printRangeRadio, pageNr);
        QWidget::setTabOrder(pageNr, pageNrButton);
        QWidget::setTabOrder(pageNrButton, Copies);
        QWidget::setTabOrder(Copies, printOptions);
        QWidget::setTabOrder(printOptions, printSepCombo);
        QWidget::setTabOrder(printSepCombo, colorType);
        QWidget::setTabOrder(colorType, separationsCombo);
        QWidget::setTabOrder(separationsCombo, printLanguages);
        QWidget::setTabOrder(printLanguages, usePDFMarks);
        QWidget::setTabOrder(usePDFMarks, mirrorHor);
        QWidget::setTabOrder(mirrorHor, mirrorVert);
        QWidget::setTabOrder(mirrorVert, setMediaSize);
        QWidget::setTabOrder(setMediaSize, clipMargins);
        QWidget::setTabOrder(clipMargins, applyGCR);
        QWidget::setTabOrder(applyGCR, convertSpots);
        QWidget::setTabOrder(convertSpots, cropMarks);
        QWidget::setTabOrder(cropMarks, bleedMarks);
        QWidget::setTabOrder(bleedMarks, colorMarks);
        QWidget::setTabOrder(colorMarks, registrationMarks);
        QWidget::setTabOrder(registrationMarks, markLength);
        QWidget::setTabOrder(markLength, markOffset);
        QWidget::setTabOrder(markOffset, docBleeds);
        QWidget::setTabOrder(docBleeds, bleedTop);
        QWidget::setTabOrder(bleedTop, bleedRight);
        QWidget::setTabOrder(bleedRight, bleedBottom);
        QWidget::setTabOrder(bleedBottom, bleedLeft);
        QWidget::setTabOrder(bleedLeft, previewButton);
        QWidget::setTabOrder(previewButton, okButton);
        QWidget::setTabOrder(okButton, cancelButton);

        retranslateUi(PrintDialogBase);
        QObject::connect(altComCheckBox, SIGNAL(clicked(bool)), OthText, SLOT(setEnabled(bool)));
        QObject::connect(altComCheckBox, SIGNAL(clicked(bool)), altCommand, SLOT(setEnabled(bool)));
        QObject::connect(printRangeRadio, SIGNAL(toggled(bool)), pageNr, SLOT(setEnabled(bool)));
        QObject::connect(printRangeRadio, SIGNAL(toggled(bool)), pageNrButton, SLOT(setEnabled(bool)));
        QObject::connect(cropMarks, SIGNAL(clicked(bool)), markLength, SLOT(setEnabled(bool)));

        printOptions->setCurrentIndex(0);
        okButton->setDefault(true);


        QMetaObject::connectSlotsByName(PrintDialogBase);
    } // setupUi

    void retranslateUi(QDialog *PrintDialogBase)
    {
        PrintDialogBase->setWindowTitle(QCoreApplication::translate("PrintDialogBase", "Setup Printer", nullptr));
        PrintDialogGroup->setTitle(QCoreApplication::translate("PrintDialogBase", "Print Destination", nullptr));
        optionsButton->setText(QCoreApplication::translate("PrintDialogBase", "&Options...", nullptr));
        DateiT->setText(QCoreApplication::translate("PrintDialogBase", "&File:", nullptr));
        selectFileButton->setText(QCoreApplication::translate("PrintDialogBase", "C&hange...", nullptr));
#if QT_CONFIG(tooltip)
        altComCheckBox->setToolTip(QCoreApplication::translate("PrintDialogBase", "Use an alternative print manager, such as kprinter or gtklp, to utilize additional printing options", nullptr));
#endif // QT_CONFIG(tooltip)
        altComCheckBox->setText(QCoreApplication::translate("PrintDialogBase", "A&lternative Printer Command", nullptr));
        OthText->setText(QCoreApplication::translate("PrintDialogBase", "Co&mmand:", nullptr));
        rangeGroup->setTitle(QCoreApplication::translate("PrintDialogBase", "Range", nullptr));
        printAllRadio->setText(QCoreApplication::translate("PrintDialogBase", "Print &All", nullptr));
        TextLabel3->setText(QCoreApplication::translate("PrintDialogBase", "N&umber of Copies:", nullptr));
        printCurrentRadio->setText(QCoreApplication::translate("PrintDialogBase", "Print Current Pa&ge", nullptr));
        printRangeRadio->setText(QCoreApplication::translate("PrintDialogBase", "Print &Range", nullptr));
#if QT_CONFIG(tooltip)
        pageNr->setToolTip(QCoreApplication::translate("PrintDialogBase", "Insert a comma separated list of tokens where\n"
"a token can be * for all the pages, 1-5 for\n"
"a range of pages or a single page number", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        pageNrButton->setToolTip(QCoreApplication::translate("PrintDialogBase", "Create a range of pages", nullptr));
#endif // QT_CONFIG(tooltip)
        pageNrButton->setText(QString());
        printSepCombo->setItemText(0, QCoreApplication::translate("PrintDialogBase", "Print Normal", nullptr));
        printSepCombo->setItemText(1, QCoreApplication::translate("PrintDialogBase", "Print Separations", nullptr));

        colorType->setItemText(0, QCoreApplication::translate("PrintDialogBase", "Print in Color if Available", nullptr));
        colorType->setItemText(1, QCoreApplication::translate("PrintDialogBase", "Print in Grayscale", nullptr));

#if QT_CONFIG(tooltip)
        printLanguages->setToolTip(QCoreApplication::translate("PrintDialogBase", "Sets the PostScript Level. Setting to Level 1 or 2 can create huge files.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        usePDFMarks->setToolTip(QCoreApplication::translate("PrintDialogBase", "Include PDF Annotations and Links into the output. Note: PDF Forms will not be exported.", nullptr));
#endif // QT_CONFIG(tooltip)
        usePDFMarks->setText(QCoreApplication::translate("PrintDialogBase", "Include PDF Annotations and Links", nullptr));
        printOptions->setTabText(printOptions->indexOf(tab), QCoreApplication::translate("PrintDialogBase", "Options", nullptr));
        pageOpts->setTitle(QCoreApplication::translate("PrintDialogBase", "Page", nullptr));
        mirrorHor->setText(QCoreApplication::translate("PrintDialogBase", "Mirror Page(s) Horizontal", nullptr));
        mirrorVert->setText(QCoreApplication::translate("PrintDialogBase", "Mirror Page(s) Vertical", nullptr));
#if QT_CONFIG(tooltip)
        setMediaSize->setToolTip(QCoreApplication::translate("PrintDialogBase", "This enables you to explicitly set the media size of the PostScript file. Not recommended unless requested by your printer.", nullptr));
#endif // QT_CONFIG(tooltip)
        setMediaSize->setText(QCoreApplication::translate("PrintDialogBase", "Set Media Size", nullptr));
        clipMargins->setText(QCoreApplication::translate("PrintDialogBase", "Clip to Printer Margins", nullptr));
        colorOpts->setTitle(QCoreApplication::translate("PrintDialogBase", "Color", nullptr));
#if QT_CONFIG(tooltip)
        applyGCR->setToolTip(QCoreApplication::translate("PrintDialogBase", "<qt>A way of switching off some of the gray shades which are composed of cyan, yellow and magenta and using black instead. Under Color Removal mostly affects parts of images which are neutral and/or dark tones which are close to the gray. Use of this may improve printing some images and some experimentation and testing is need on a case by case basis. Under Color Removal reduces the possibility of over saturation with CMY inks.</qt>", nullptr));
#endif // QT_CONFIG(tooltip)
        applyGCR->setText(QCoreApplication::translate("PrintDialogBase", "Apply Under Color Removal", nullptr));
#if QT_CONFIG(tooltip)
        convertSpots->setToolTip(QCoreApplication::translate("PrintDialogBase", "<qt>Enables Spot Colors to be converted to composite colors. Unless you are planning to print spot colors at a commercial printer, this is probably best left enabled.</qt>", nullptr));
#endif // QT_CONFIG(tooltip)
        convertSpots->setText(QCoreApplication::translate("PrintDialogBase", "Convert Spot Colors to Process Colors", nullptr));
        printOptions->setTabText(printOptions->indexOf(tab_2), QCoreApplication::translate("PrintDialogBase", "Advanced Options", nullptr));
#if QT_CONFIG(tooltip)
        bleedMarks->setToolTip(QCoreApplication::translate("PrintDialogBase", "This creates bleed marks which are indicated by  _ . _ and show the bleed limit", nullptr));
#endif // QT_CONFIG(tooltip)
        bleedMarks->setText(QCoreApplication::translate("PrintDialogBase", "Bleed Marks", nullptr));
#if QT_CONFIG(tooltip)
        cropMarks->setToolTip(QCoreApplication::translate("PrintDialogBase", "This creates crop marks in the PDF indicating where the paper should be cut or trimmed after printing", nullptr));
#endif // QT_CONFIG(tooltip)
        cropMarks->setText(QCoreApplication::translate("PrintDialogBase", "Crop Marks", nullptr));
#if QT_CONFIG(tooltip)
        colorMarks->setToolTip(QCoreApplication::translate("PrintDialogBase", "Add color calibration bars", nullptr));
#endif // QT_CONFIG(tooltip)
        colorMarks->setText(QCoreApplication::translate("PrintDialogBase", "Color Bars", nullptr));
#if QT_CONFIG(tooltip)
        registrationMarks->setToolTip(QCoreApplication::translate("PrintDialogBase", "Add registration marks which are added to each separation", nullptr));
#endif // QT_CONFIG(tooltip)
        registrationMarks->setText(QCoreApplication::translate("PrintDialogBase", "Registration Marks", nullptr));
        MarkTxt2->setText(QCoreApplication::translate("PrintDialogBase", "Length:", nullptr));
        MarkTxt1->setText(QCoreApplication::translate("PrintDialogBase", "Offset:", nullptr));
#if QT_CONFIG(tooltip)
        markOffset->setToolTip(QCoreApplication::translate("PrintDialogBase", "Indicate the distance offset for the registration marks", nullptr));
#endif // QT_CONFIG(tooltip)
        printOptions->setTabText(printOptions->indexOf(tab_3), QCoreApplication::translate("PrintDialogBase", "Marks", nullptr));
        bleedTopText->setText(QCoreApplication::translate("PrintDialogBase", "Top:", nullptr));
#if QT_CONFIG(tooltip)
        bleedTop->setToolTip(QCoreApplication::translate("PrintDialogBase", "Distance for bleed from the top of the physical page", nullptr));
#endif // QT_CONFIG(tooltip)
        bleedLeftText->setText(QCoreApplication::translate("PrintDialogBase", "Left:", nullptr));
#if QT_CONFIG(tooltip)
        bleedRight->setToolTip(QCoreApplication::translate("PrintDialogBase", "Distance for bleed from the right of the physical page", nullptr));
#endif // QT_CONFIG(tooltip)
        bleedBottomText->setText(QCoreApplication::translate("PrintDialogBase", "Bottom:", nullptr));
#if QT_CONFIG(tooltip)
        bleedBottom->setToolTip(QCoreApplication::translate("PrintDialogBase", "Distance for bleed from the bottom of the physical page", nullptr));
#endif // QT_CONFIG(tooltip)
        bleedRightText->setText(QCoreApplication::translate("PrintDialogBase", "Right:", nullptr));
#if QT_CONFIG(tooltip)
        bleedLeft->setToolTip(QCoreApplication::translate("PrintDialogBase", "Distance for bleed from the left of the physical page", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        docBleeds->setToolTip(QCoreApplication::translate("PrintDialogBase", "Use the existing bleed settings from the document preferences", nullptr));
#endif // QT_CONFIG(tooltip)
        docBleeds->setText(QCoreApplication::translate("PrintDialogBase", "Use Document Bleeds", nullptr));
        printOptions->setTabText(printOptions->indexOf(tab_4), QCoreApplication::translate("PrintDialogBase", "Bleeds", nullptr));
        previewButton->setText(QCoreApplication::translate("PrintDialogBase", "Preview...", nullptr));
        okButton->setText(QCoreApplication::translate("PrintDialogBase", "&Print", nullptr));
        cancelButton->setText(QCoreApplication::translate("PrintDialogBase", "Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PrintDialogBase: public Ui_PrintDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PRINTDIALOGBASE_H
