/********************************************************************************
** Form generated from reading UI file 'propertywidget_optmarginsbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYWIDGET_OPTMARGINSBASE_H
#define UI_PROPERTYWIDGET_OPTMARGINSBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_PropertyWidget_OptMarginsBase
{
public:
    QVBoxLayout *verticalLayout;
    QRadioButton *optMarginRadioNone;
    QRadioButton *optMarginRadioBoth;
    QRadioButton *optMarginRadioLeft;
    QRadioButton *optMarginRadioRight;
    QPushButton *optMarginResetButton;

    void setupUi(QFrame *PropertyWidget_OptMarginsBase)
    {
        if (PropertyWidget_OptMarginsBase->objectName().isEmpty())
            PropertyWidget_OptMarginsBase->setObjectName(QString::fromUtf8("PropertyWidget_OptMarginsBase"));
        PropertyWidget_OptMarginsBase->resize(210, 137);
        verticalLayout = new QVBoxLayout(PropertyWidget_OptMarginsBase);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        optMarginRadioNone = new QRadioButton(PropertyWidget_OptMarginsBase);
        optMarginRadioNone->setObjectName(QString::fromUtf8("optMarginRadioNone"));

        verticalLayout->addWidget(optMarginRadioNone);

        optMarginRadioBoth = new QRadioButton(PropertyWidget_OptMarginsBase);
        optMarginRadioBoth->setObjectName(QString::fromUtf8("optMarginRadioBoth"));

        verticalLayout->addWidget(optMarginRadioBoth);

        optMarginRadioLeft = new QRadioButton(PropertyWidget_OptMarginsBase);
        optMarginRadioLeft->setObjectName(QString::fromUtf8("optMarginRadioLeft"));

        verticalLayout->addWidget(optMarginRadioLeft);

        optMarginRadioRight = new QRadioButton(PropertyWidget_OptMarginsBase);
        optMarginRadioRight->setObjectName(QString::fromUtf8("optMarginRadioRight"));

        verticalLayout->addWidget(optMarginRadioRight);

        optMarginResetButton = new QPushButton(PropertyWidget_OptMarginsBase);
        optMarginResetButton->setObjectName(QString::fromUtf8("optMarginResetButton"));

        verticalLayout->addWidget(optMarginResetButton);


        retranslateUi(PropertyWidget_OptMarginsBase);

        QMetaObject::connectSlotsByName(PropertyWidget_OptMarginsBase);
    } // setupUi

    void retranslateUi(QFrame *PropertyWidget_OptMarginsBase)
    {
        optMarginRadioNone->setText(QCoreApplication::translate("PropertyWidget_OptMarginsBase", "None", nullptr));
        optMarginRadioBoth->setText(QCoreApplication::translate("PropertyWidget_OptMarginsBase", "Both Sides", nullptr));
        optMarginRadioLeft->setText(QCoreApplication::translate("PropertyWidget_OptMarginsBase", "Left Only", nullptr));
        optMarginRadioRight->setText(QCoreApplication::translate("PropertyWidget_OptMarginsBase", "Right Only", nullptr));
        optMarginResetButton->setText(QCoreApplication::translate("PropertyWidget_OptMarginsBase", "Reset", nullptr));
        (void)PropertyWidget_OptMarginsBase;
    } // retranslateUi

};

namespace Ui {
    class PropertyWidget_OptMarginsBase: public Ui_PropertyWidget_OptMarginsBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTYWIDGET_OPTMARGINSBASE_H
