/********************************************************************************
** Form generated from reading UI file 'gradienteditor.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GRADIENTEDITOR_H
#define UI_GRADIENTEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <ui/scrspinbox.h>
#include "ui/colorcombo.h"
#include "ui/gradientpreview.h"

QT_BEGIN_NAMESPACE

class Ui_GradientEditorBase
{
public:
    QGridLayout *gridLayout;
    GradientPreview *Preview;
    QLabel *stopPosLabel;
    ScrSpinBox *stopPos;
    QLabel *stopColorLabel;
    ColorCombo *stopColor;
    QLabel *stopOpacityLabel;
    ScrSpinBox *stopOpacity;
    QLabel *stopShadeLabel;
    ScrSpinBox *stopShade;

    void setupUi(QFrame *GradientEditorBase)
    {
        if (GradientEditorBase->objectName().isEmpty())
            GradientEditorBase->setObjectName(QString::fromUtf8("GradientEditorBase"));
        GradientEditorBase->resize(210, 178);
        GradientEditorBase->setFrameShape(QFrame::NoFrame);
        GradientEditorBase->setFrameShadow(QFrame::Plain);
        gridLayout = new QGridLayout(GradientEditorBase);
        gridLayout->setSpacing(3);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(0, 0, 0, 0);
        Preview = new GradientPreview(GradientEditorBase);
        Preview->setObjectName(QString::fromUtf8("Preview"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Preview->sizePolicy().hasHeightForWidth());
        Preview->setSizePolicy(sizePolicy);
        Preview->setMinimumSize(QSize(200, 70));
        Preview->setMaximumSize(QSize(3000, 70));
        Preview->setFrameShape(QFrame::Panel);
        Preview->setFrameShadow(QFrame::Sunken);
        Preview->setLineWidth(2);

        gridLayout->addWidget(Preview, 0, 0, 1, 2);

        stopPosLabel = new QLabel(GradientEditorBase);
        stopPosLabel->setObjectName(QString::fromUtf8("stopPosLabel"));

        gridLayout->addWidget(stopPosLabel, 1, 0, 1, 1);

        stopPos = new ScrSpinBox(GradientEditorBase);
        stopPos->setObjectName(QString::fromUtf8("stopPos"));
        stopPos->setMaximum(100);

        gridLayout->addWidget(stopPos, 1, 1, 1, 1);

        stopColorLabel = new QLabel(GradientEditorBase);
        stopColorLabel->setObjectName(QString::fromUtf8("stopColorLabel"));

        gridLayout->addWidget(stopColorLabel, 2, 0, 1, 1);

        stopColor = new ColorCombo(GradientEditorBase);
        stopColor->setObjectName(QString::fromUtf8("stopColor"));

        gridLayout->addWidget(stopColor, 2, 1, 1, 1);

        stopOpacityLabel = new QLabel(GradientEditorBase);
        stopOpacityLabel->setObjectName(QString::fromUtf8("stopOpacityLabel"));

        gridLayout->addWidget(stopOpacityLabel, 3, 0, 1, 1);

        stopOpacity = new ScrSpinBox(GradientEditorBase);
        stopOpacity->setObjectName(QString::fromUtf8("stopOpacity"));
        stopOpacity->setMaximum(100);
        stopOpacity->setSingleStep(10);
        stopOpacity->setValue(100);

        gridLayout->addWidget(stopOpacity, 3, 1, 1, 1);

        stopShadeLabel = new QLabel(GradientEditorBase);
        stopShadeLabel->setObjectName(QString::fromUtf8("stopShadeLabel"));

        gridLayout->addWidget(stopShadeLabel, 4, 0, 1, 1);

        stopShade = new ScrSpinBox(GradientEditorBase);
        stopShade->setObjectName(QString::fromUtf8("stopShade"));
        stopShade->setMaximum(100);
        stopShade->setSingleStep(10);
        stopShade->setValue(100);

        gridLayout->addWidget(stopShade, 4, 1, 1, 1);


        retranslateUi(GradientEditorBase);

        QMetaObject::connectSlotsByName(GradientEditorBase);
    } // setupUi

    void retranslateUi(QFrame *GradientEditorBase)
    {
        stopPosLabel->setText(QCoreApplication::translate("GradientEditorBase", "Position:", nullptr));
        stopPos->setSuffix(QCoreApplication::translate("GradientEditorBase", " %", nullptr));
        stopColorLabel->setText(QCoreApplication::translate("GradientEditorBase", "Color:", nullptr));
        stopOpacityLabel->setText(QCoreApplication::translate("GradientEditorBase", "Opacity:", nullptr));
        stopOpacity->setSuffix(QCoreApplication::translate("GradientEditorBase", " %", nullptr));
        stopShadeLabel->setText(QCoreApplication::translate("GradientEditorBase", "Shade:", nullptr));
        stopShade->setSuffix(QCoreApplication::translate("GradientEditorBase", " %", nullptr));
        (void)GradientEditorBase;
    } // retranslateUi

};

namespace Ui {
    class GradientEditorBase: public Ui_GradientEditorBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_GRADIENTEDITOR_H
