/****************************************************************************
** Meta object code from reading C++ file 'marksmanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/marksmanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'marksmanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MarksManager_t {
    QByteArrayData data[16];
    char stringdata0[245];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MarksManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MarksManager_t qt_meta_stringdata_MarksManager = {
    {
QT_MOC_LITERAL(0, 0, 12), // "MarksManager"
QT_MOC_LITERAL(1, 13, 6), // "setDoc"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 11), // "ScribusDoc*"
QT_MOC_LITERAL(4, 33, 3), // "doc"
QT_MOC_LITERAL(5, 37, 14), // "languageChange"
QT_MOC_LITERAL(6, 52, 13), // "paletteChange"
QT_MOC_LITERAL(7, 66, 19), // "handleUpdateRequest"
QT_MOC_LITERAL(8, 86, 11), // "updateFlags"
QT_MOC_LITERAL(9, 98, 23), // "on_UpdateButton_clicked"
QT_MOC_LITERAL(10, 122, 21), // "on_EditButton_clicked"
QT_MOC_LITERAL(11, 144, 23), // "on_DeleteButton_clicked"
QT_MOC_LITERAL(12, 168, 25), // "on_listView_doubleClicked"
QT_MOC_LITERAL(13, 194, 11), // "QModelIndex"
QT_MOC_LITERAL(14, 206, 5), // "index"
QT_MOC_LITERAL(15, 212, 32) // "on_listView_itemSelectionChanged"

    },
    "MarksManager\0setDoc\0\0ScribusDoc*\0doc\0"
    "languageChange\0paletteChange\0"
    "handleUpdateRequest\0updateFlags\0"
    "on_UpdateButton_clicked\0on_EditButton_clicked\0"
    "on_DeleteButton_clicked\0"
    "on_listView_doubleClicked\0QModelIndex\0"
    "index\0on_listView_itemSelectionChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MarksManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   59,    2, 0x0a /* Public */,
       5,    0,   62,    2, 0x0a /* Public */,
       6,    0,   63,    2, 0x0a /* Public */,
       7,    1,   64,    2, 0x0a /* Public */,
       9,    0,   67,    2, 0x08 /* Private */,
      10,    0,   68,    2, 0x08 /* Private */,
      11,    0,   69,    2, 0x08 /* Private */,
      12,    1,   70,    2, 0x08 /* Private */,
      15,    0,   73,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void,

       0        // eod
};

void MarksManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MarksManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 1: _t->languageChange(); break;
        case 2: _t->paletteChange(); break;
        case 3: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->on_UpdateButton_clicked(); break;
        case 5: _t->on_EditButton_clicked(); break;
        case 6: _t->on_DeleteButton_clicked(); break;
        case 7: _t->on_listView_doubleClicked((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 8: _t->on_listView_itemSelectionChanged(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MarksManager::staticMetaObject = { {
    QMetaObject::SuperData::link<ScrPaletteBase::staticMetaObject>(),
    qt_meta_stringdata_MarksManager.data,
    qt_meta_data_MarksManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MarksManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MarksManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MarksManager.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::MarksManager"))
        return static_cast< Ui::MarksManager*>(this);
    return ScrPaletteBase::qt_metacast(_clname);
}

int MarksManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScrPaletteBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
