/********************************************************************************
** Form generated from reading UI file 'polygonwidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_POLYGONWIDGETBASE_H
#define UI_POLYGONWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/scrspinbox.h>

QT_BEGIN_NAMESPACE

class Ui_PolygonWidgetBase
{
public:
    QGridLayout *gridLayout_3;
    QGridLayout *gridLayout;
    QLabel *cornersSpinBoxLabel;
    ScrSpinBox *cornersSpinBox;
    QLabel *rotationSpinBoxLabel;
    ScrSpinBox *rotationSpinBox;
    QSlider *rotationSlider;
    QVBoxLayout *verticalLayout_2;
    QLabel *Preview;
    QSpacerItem *verticalSpacer;
    QGroupBox *applyConvexGroupBox;
    QGridLayout *gridLayout_2;
    QLabel *factorSpinBoxLabel;
    ScrSpinBox *factorSpinBox;
    QSlider *factorSlider;
    QLabel *innerRotationSpinBoxLabel;
    ScrSpinBox *innerRotationSpinBox;
    QSlider *innerRotationSlider;
    QLabel *curvatureSpinBoxLabel;
    ScrSpinBox *curvatureSpinBox;
    QSlider *curvatureSlider;
    QLabel *outerCurvatureSpinBoxLabel;
    ScrSpinBox *outerCurvatureSpinBox;
    QSlider *outerCurvatureSlider;

    void setupUi(QWidget *PolygonWidgetBase)
    {
        if (PolygonWidgetBase->objectName().isEmpty())
            PolygonWidgetBase->setObjectName(QString::fromUtf8("PolygonWidgetBase"));
        PolygonWidgetBase->resize(495, 359);
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(PolygonWidgetBase->sizePolicy().hasHeightForWidth());
        PolygonWidgetBase->setSizePolicy(sizePolicy);
        PolygonWidgetBase->setWindowTitle(QString::fromUtf8(""));
        gridLayout_3 = new QGridLayout(PolygonWidgetBase);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        gridLayout_3->setContentsMargins(0, 0, 0, 0);
        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        cornersSpinBoxLabel = new QLabel(PolygonWidgetBase);
        cornersSpinBoxLabel->setObjectName(QString::fromUtf8("cornersSpinBoxLabel"));

        gridLayout->addWidget(cornersSpinBoxLabel, 0, 0, 1, 1);

        cornersSpinBox = new ScrSpinBox(PolygonWidgetBase);
        cornersSpinBox->setObjectName(QString::fromUtf8("cornersSpinBox"));
        cornersSpinBox->setSuffix(QString::fromUtf8(""));
        cornersSpinBox->setMinimum(3);
        cornersSpinBox->setMaximum(999);

        gridLayout->addWidget(cornersSpinBox, 0, 1, 1, 1);

        rotationSpinBoxLabel = new QLabel(PolygonWidgetBase);
        rotationSpinBoxLabel->setObjectName(QString::fromUtf8("rotationSpinBoxLabel"));

        gridLayout->addWidget(rotationSpinBoxLabel, 1, 0, 1, 1);

        rotationSpinBox = new ScrSpinBox(PolygonWidgetBase);
        rotationSpinBox->setObjectName(QString::fromUtf8("rotationSpinBox"));
        rotationSpinBox->setMinimum(-180);
        rotationSpinBox->setMaximum(180);

        gridLayout->addWidget(rotationSpinBox, 1, 1, 1, 1);

        rotationSlider = new QSlider(PolygonWidgetBase);
        rotationSlider->setObjectName(QString::fromUtf8("rotationSlider"));
        rotationSlider->setMinimum(-180);
        rotationSlider->setMaximum(180);
        rotationSlider->setOrientation(Qt::Horizontal);
        rotationSlider->setTickPosition(QSlider::TicksBelow);

        gridLayout->addWidget(rotationSlider, 2, 0, 1, 2);


        gridLayout_3->addLayout(gridLayout, 0, 0, 1, 1);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        Preview = new QLabel(PolygonWidgetBase);
        Preview->setObjectName(QString::fromUtf8("Preview"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(Preview->sizePolicy().hasHeightForWidth());
        Preview->setSizePolicy(sizePolicy1);
        Preview->setMinimumSize(QSize(280, 280));
        Preview->setMaximumSize(QSize(280, 280));
        Preview->setSizeIncrement(QSize(0, 0));
        Preview->setFrameShape(QFrame::Panel);
        Preview->setFrameShadow(QFrame::Sunken);
        Preview->setLineWidth(2);
        Preview->setAlignment(Qt::AlignCenter);

        verticalLayout_2->addWidget(Preview);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        gridLayout_3->addLayout(verticalLayout_2, 0, 1, 2, 1);

        applyConvexGroupBox = new QGroupBox(PolygonWidgetBase);
        applyConvexGroupBox->setObjectName(QString::fromUtf8("applyConvexGroupBox"));
        applyConvexGroupBox->setCheckable(true);
        gridLayout_2 = new QGridLayout(applyConvexGroupBox);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        factorSpinBoxLabel = new QLabel(applyConvexGroupBox);
        factorSpinBoxLabel->setObjectName(QString::fromUtf8("factorSpinBoxLabel"));

        gridLayout_2->addWidget(factorSpinBoxLabel, 0, 0, 1, 1);

        factorSpinBox = new ScrSpinBox(applyConvexGroupBox);
        factorSpinBox->setObjectName(QString::fromUtf8("factorSpinBox"));
        factorSpinBox->setMinimum(-100);
        factorSpinBox->setMaximum(100);

        gridLayout_2->addWidget(factorSpinBox, 0, 1, 1, 1);

        factorSlider = new QSlider(applyConvexGroupBox);
        factorSlider->setObjectName(QString::fromUtf8("factorSlider"));
        factorSlider->setMinimum(-100);
        factorSlider->setMaximum(100);
        factorSlider->setOrientation(Qt::Horizontal);
        factorSlider->setTickPosition(QSlider::TicksBelow);

        gridLayout_2->addWidget(factorSlider, 1, 0, 1, 2);

        innerRotationSpinBoxLabel = new QLabel(applyConvexGroupBox);
        innerRotationSpinBoxLabel->setObjectName(QString::fromUtf8("innerRotationSpinBoxLabel"));

        gridLayout_2->addWidget(innerRotationSpinBoxLabel, 2, 0, 1, 1);

        innerRotationSpinBox = new ScrSpinBox(applyConvexGroupBox);
        innerRotationSpinBox->setObjectName(QString::fromUtf8("innerRotationSpinBox"));
        innerRotationSpinBox->setMinimum(-180);
        innerRotationSpinBox->setMaximum(180);

        gridLayout_2->addWidget(innerRotationSpinBox, 2, 1, 1, 1);

        innerRotationSlider = new QSlider(applyConvexGroupBox);
        innerRotationSlider->setObjectName(QString::fromUtf8("innerRotationSlider"));
        innerRotationSlider->setMinimum(-180);
        innerRotationSlider->setMaximum(180);
        innerRotationSlider->setOrientation(Qt::Horizontal);
        innerRotationSlider->setTickPosition(QSlider::TicksBelow);

        gridLayout_2->addWidget(innerRotationSlider, 3, 0, 1, 2);

        curvatureSpinBoxLabel = new QLabel(applyConvexGroupBox);
        curvatureSpinBoxLabel->setObjectName(QString::fromUtf8("curvatureSpinBoxLabel"));

        gridLayout_2->addWidget(curvatureSpinBoxLabel, 4, 0, 1, 1);

        curvatureSpinBox = new ScrSpinBox(applyConvexGroupBox);
        curvatureSpinBox->setObjectName(QString::fromUtf8("curvatureSpinBox"));
        curvatureSpinBox->setMaximum(100);

        gridLayout_2->addWidget(curvatureSpinBox, 4, 1, 1, 1);

        curvatureSlider = new QSlider(applyConvexGroupBox);
        curvatureSlider->setObjectName(QString::fromUtf8("curvatureSlider"));
        curvatureSlider->setMaximum(100);
        curvatureSlider->setOrientation(Qt::Horizontal);
        curvatureSlider->setTickPosition(QSlider::TicksBelow);

        gridLayout_2->addWidget(curvatureSlider, 5, 0, 1, 2);

        outerCurvatureSpinBoxLabel = new QLabel(applyConvexGroupBox);
        outerCurvatureSpinBoxLabel->setObjectName(QString::fromUtf8("outerCurvatureSpinBoxLabel"));

        gridLayout_2->addWidget(outerCurvatureSpinBoxLabel, 6, 0, 1, 1);

        outerCurvatureSpinBox = new ScrSpinBox(applyConvexGroupBox);
        outerCurvatureSpinBox->setObjectName(QString::fromUtf8("outerCurvatureSpinBox"));
        outerCurvatureSpinBox->setMaximum(100);

        gridLayout_2->addWidget(outerCurvatureSpinBox, 6, 1, 1, 1);

        outerCurvatureSlider = new QSlider(applyConvexGroupBox);
        outerCurvatureSlider->setObjectName(QString::fromUtf8("outerCurvatureSlider"));
        outerCurvatureSlider->setMaximum(100);
        outerCurvatureSlider->setOrientation(Qt::Horizontal);
        outerCurvatureSlider->setTickPosition(QSlider::TicksBelow);

        gridLayout_2->addWidget(outerCurvatureSlider, 7, 0, 1, 2);

        factorSpinBoxLabel->raise();
        factorSpinBox->raise();
        factorSlider->raise();
        curvatureSpinBoxLabel->raise();
        curvatureSpinBox->raise();
        curvatureSlider->raise();
        outerCurvatureSpinBoxLabel->raise();
        outerCurvatureSpinBox->raise();
        outerCurvatureSlider->raise();
        innerRotationSlider->raise();
        innerRotationSpinBox->raise();
        innerRotationSpinBoxLabel->raise();

        gridLayout_3->addWidget(applyConvexGroupBox, 1, 0, 1, 1);

#if QT_CONFIG(shortcut)
        cornersSpinBoxLabel->setBuddy(cornersSpinBox);
        rotationSpinBoxLabel->setBuddy(rotationSpinBox);
        factorSpinBoxLabel->setBuddy(factorSpinBox);
        curvatureSpinBoxLabel->setBuddy(curvatureSpinBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(cornersSpinBox, rotationSpinBox);
        QWidget::setTabOrder(rotationSpinBox, rotationSlider);
        QWidget::setTabOrder(rotationSlider, applyConvexGroupBox);
        QWidget::setTabOrder(applyConvexGroupBox, factorSpinBox);
        QWidget::setTabOrder(factorSpinBox, factorSlider);
        QWidget::setTabOrder(factorSlider, innerRotationSpinBox);
        QWidget::setTabOrder(innerRotationSpinBox, innerRotationSlider);
        QWidget::setTabOrder(innerRotationSlider, curvatureSpinBox);
        QWidget::setTabOrder(curvatureSpinBox, curvatureSlider);
        QWidget::setTabOrder(curvatureSlider, outerCurvatureSpinBox);
        QWidget::setTabOrder(outerCurvatureSpinBox, outerCurvatureSlider);

        retranslateUi(PolygonWidgetBase);

        QMetaObject::connectSlotsByName(PolygonWidgetBase);
    } // setupUi

    void retranslateUi(QWidget *PolygonWidgetBase)
    {
        cornersSpinBoxLabel->setText(QCoreApplication::translate("PolygonWidgetBase", "Corn&ers:", nullptr));
#if QT_CONFIG(tooltip)
        cornersSpinBox->setToolTip(QCoreApplication::translate("PolygonWidgetBase", "Number of corners for polygons", nullptr));
#endif // QT_CONFIG(tooltip)
        rotationSpinBoxLabel->setText(QCoreApplication::translate("PolygonWidgetBase", "&Rotation:", nullptr));
#if QT_CONFIG(tooltip)
        rotationSpinBox->setToolTip(QCoreApplication::translate("PolygonWidgetBase", "Degrees of rotation for polygons", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        rotationSlider->setToolTip(QCoreApplication::translate("PolygonWidgetBase", "Degrees of rotation for polygons", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        Preview->setToolTip(QCoreApplication::translate("PolygonWidgetBase", "Sample Polygon", nullptr));
#endif // QT_CONFIG(tooltip)
        Preview->setText(QString());
#if QT_CONFIG(tooltip)
        applyConvexGroupBox->setToolTip(QCoreApplication::translate("PolygonWidgetBase", "Apply Convex/Concave Factor to change shape of Polygons", nullptr));
#endif // QT_CONFIG(tooltip)
        applyConvexGroupBox->setTitle(QCoreApplication::translate("PolygonWidgetBase", "Apply &Factor", nullptr));
        factorSpinBoxLabel->setText(QCoreApplication::translate("PolygonWidgetBase", "&Factor:", nullptr));
#if QT_CONFIG(tooltip)
        factorSpinBox->setToolTip(QCoreApplication::translate("PolygonWidgetBase", "A negative value will make the polygon concave (or star shaped), a positive value will make it convex", nullptr));
#endif // QT_CONFIG(tooltip)
        factorSpinBox->setSuffix(QCoreApplication::translate("PolygonWidgetBase", " %", nullptr));
#if QT_CONFIG(tooltip)
        factorSlider->setToolTip(QCoreApplication::translate("PolygonWidgetBase", "A negative value will make the polygon concave (or star shaped), a positive value will make it convex", nullptr));
#endif // QT_CONFIG(tooltip)
        innerRotationSpinBoxLabel->setText(QCoreApplication::translate("PolygonWidgetBase", "Inner Rotation:", nullptr));
        curvatureSpinBoxLabel->setText(QCoreApplication::translate("PolygonWidgetBase", "Inner C&urvature:", nullptr));
        curvatureSpinBox->setSuffix(QCoreApplication::translate("PolygonWidgetBase", " %", nullptr));
        outerCurvatureSpinBoxLabel->setText(QCoreApplication::translate("PolygonWidgetBase", "Outer Curvature:", nullptr));
        outerCurvatureSpinBox->setSuffix(QCoreApplication::translate("PolygonWidgetBase", " %", nullptr));
        (void)PolygonWidgetBase;
    } // retranslateUi

};

namespace Ui {
    class PolygonWidgetBase: public Ui_PolygonWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_POLYGONWIDGETBASE_H
