/********************************************************************************
** Form generated from reading UI file 'transformdialogbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TRANSFORMDIALOGBASE_H
#define UI_TRANSFORMDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/scrspinbox.h>
#include "ui/basepointwidget.h"
#include "ui/linkbutton.h"

QT_BEGIN_NAMESPACE

class Ui_TransformDialogBase
{
public:
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_2;
    QListWidget *transformSelector;
    QVBoxLayout *verticalLayout_3;
    QStackedWidget *transformStack;
    QWidget *blankPage;
    QWidget *pageScale;
    QGridLayout *gridLayout;
    QLabel *verticalScaleLabel;
    ScrSpinBox *horizontalScale;
    ScrSpinBox *verticalScale;
    QLabel *horizontalScaleLabel;
    LinkButton *scaleLink;
    QLabel *ScalingStackLabel;
    QSpacerItem *verticalSpacer_5;
    QWidget *pageTranslate;
    QGridLayout *gridLayout1;
    QLabel *translateHorizontalLabel;
    ScrSpinBox *translateHorizontal;
    QLabel *translateVerticalLabel;
    QLabel *TranslationStackLabel;
    ScrSpinBox *translateVertical;
    QSpacerItem *verticalSpacer_6;
    QWidget *pageRotate;
    QGridLayout *gridLayout2;
    QLabel *RotateStackLabel;
    QSpacerItem *verticalSpacer_3;
    QLabel *rotationValueLabel;
    ScrSpinBox *rotationValue;
    QWidget *Seite;
    QGridLayout *gridLayout3;
    LinkButton *linkSkew;
    ScrSpinBox *horizontalSkew;
    QLabel *horizontalSkewLabel;
    ScrSpinBox *verticalSkew;
    QLabel *verticalSkewLabel;
    QLabel *SkewStackLabel;
    QSpacerItem *verticalSpacer_4;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QVBoxLayout *verticalLayout;
    QLabel *label_10;
    BasePointWidget *basePoint;
    QSpacerItem *verticalSpacer_2;
    QSpacerItem *horizontalSpacer_3;
    QVBoxLayout *verticalLayout_2;
    QLabel *label_9;
    QSpinBox *numberOfCopies;
    QSpacerItem *verticalSpacer;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *spacerItem;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *buttonAdd;
    QPushButton *buttonRemove;
    QPushButton *buttonUp;
    QPushButton *buttonDown;
    QSpacerItem *spacerItem1;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *TransformDialogBase)
    {
        if (TransformDialogBase->objectName().isEmpty())
            TransformDialogBase->setObjectName(QString::fromUtf8("TransformDialogBase"));
        TransformDialogBase->resize(500, 260);
        TransformDialogBase->setMinimumSize(QSize(500, 260));
        verticalLayout_4 = new QVBoxLayout(TransformDialogBase);
        verticalLayout_4->setSpacing(5);
        verticalLayout_4->setContentsMargins(10, 10, 10, 10);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(5);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        transformSelector = new QListWidget(TransformDialogBase);
        transformSelector->setObjectName(QString::fromUtf8("transformSelector"));

        horizontalLayout_2->addWidget(transformSelector);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setSpacing(5);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        transformStack = new QStackedWidget(TransformDialogBase);
        transformStack->setObjectName(QString::fromUtf8("transformStack"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(transformStack->sizePolicy().hasHeightForWidth());
        transformStack->setSizePolicy(sizePolicy);
        transformStack->setMinimumSize(QSize(160, 16));
        transformStack->setFrameShape(QFrame::Panel);
        blankPage = new QWidget();
        blankPage->setObjectName(QString::fromUtf8("blankPage"));
        transformStack->addWidget(blankPage);
        pageScale = new QWidget();
        pageScale->setObjectName(QString::fromUtf8("pageScale"));
        gridLayout = new QGridLayout(pageScale);
        gridLayout->setSpacing(5);
        gridLayout->setContentsMargins(10, 10, 10, 10);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        verticalScaleLabel = new QLabel(pageScale);
        verticalScaleLabel->setObjectName(QString::fromUtf8("verticalScaleLabel"));

        gridLayout->addWidget(verticalScaleLabel, 2, 0, 1, 1);

        horizontalScale = new ScrSpinBox(pageScale);
        horizontalScale->setObjectName(QString::fromUtf8("horizontalScale"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(horizontalScale->sizePolicy().hasHeightForWidth());
        horizontalScale->setSizePolicy(sizePolicy1);
        horizontalScale->setMinimumSize(QSize(60, 0));
        horizontalScale->setDecimals(1);
        horizontalScale->setMinimum(0.000000000000000);
        horizontalScale->setMaximum(1000.000000000000000);
        horizontalScale->setValue(100.000000000000000);

        gridLayout->addWidget(horizontalScale, 1, 1, 1, 1);

        verticalScale = new ScrSpinBox(pageScale);
        verticalScale->setObjectName(QString::fromUtf8("verticalScale"));
        QSizePolicy sizePolicy2(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(verticalScale->sizePolicy().hasHeightForWidth());
        verticalScale->setSizePolicy(sizePolicy2);
        verticalScale->setMinimumSize(QSize(60, 0));
        verticalScale->setDecimals(1);
        verticalScale->setMinimum(0.000000000000000);
        verticalScale->setMaximum(1000.000000000000000);
        verticalScale->setValue(100.000000000000000);

        gridLayout->addWidget(verticalScale, 2, 1, 1, 1);

        horizontalScaleLabel = new QLabel(pageScale);
        horizontalScaleLabel->setObjectName(QString::fromUtf8("horizontalScaleLabel"));

        gridLayout->addWidget(horizontalScaleLabel, 1, 0, 1, 1);

        scaleLink = new LinkButton(pageScale);
        scaleLink->setObjectName(QString::fromUtf8("scaleLink"));
        QSizePolicy sizePolicy3(QSizePolicy::Fixed, QSizePolicy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(scaleLink->sizePolicy().hasHeightForWidth());
        scaleLink->setSizePolicy(sizePolicy3);
        scaleLink->setMinimumSize(QSize(15, 0));
        scaleLink->setMaximumSize(QSize(15, 16777215));
        scaleLink->setCheckable(true);
        scaleLink->setAutoRaise(true);

        gridLayout->addWidget(scaleLink, 1, 2, 2, 1);

        ScalingStackLabel = new QLabel(pageScale);
        ScalingStackLabel->setObjectName(QString::fromUtf8("ScalingStackLabel"));
        QFont font;
        font.setBold(true);
        font.setWeight(75);
        ScalingStackLabel->setFont(font);
        ScalingStackLabel->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(ScalingStackLabel, 0, 0, 1, 3);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(verticalSpacer_5, 3, 1, 1, 1);

        transformStack->addWidget(pageScale);
        pageTranslate = new QWidget();
        pageTranslate->setObjectName(QString::fromUtf8("pageTranslate"));
        gridLayout1 = new QGridLayout(pageTranslate);
        gridLayout1->setSpacing(5);
        gridLayout1->setContentsMargins(10, 10, 10, 10);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        translateHorizontalLabel = new QLabel(pageTranslate);
        translateHorizontalLabel->setObjectName(QString::fromUtf8("translateHorizontalLabel"));

        gridLayout1->addWidget(translateHorizontalLabel, 1, 0, 1, 1);

        translateHorizontal = new ScrSpinBox(pageTranslate);
        translateHorizontal->setObjectName(QString::fromUtf8("translateHorizontal"));
        translateHorizontal->setMinimum(-30000.000000000000000);
        translateHorizontal->setMaximum(30000.000000000000000);

        gridLayout1->addWidget(translateHorizontal, 1, 1, 1, 1);

        translateVerticalLabel = new QLabel(pageTranslate);
        translateVerticalLabel->setObjectName(QString::fromUtf8("translateVerticalLabel"));

        gridLayout1->addWidget(translateVerticalLabel, 2, 0, 1, 1);

        TranslationStackLabel = new QLabel(pageTranslate);
        TranslationStackLabel->setObjectName(QString::fromUtf8("TranslationStackLabel"));
        TranslationStackLabel->setFont(font);
        TranslationStackLabel->setAlignment(Qt::AlignCenter);

        gridLayout1->addWidget(TranslationStackLabel, 0, 0, 1, 2);

        translateVertical = new ScrSpinBox(pageTranslate);
        translateVertical->setObjectName(QString::fromUtf8("translateVertical"));
        translateVertical->setMinimum(-30000.000000000000000);
        translateVertical->setMaximum(30000.000000000000000);

        gridLayout1->addWidget(translateVertical, 2, 1, 1, 1);

        verticalSpacer_6 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout1->addItem(verticalSpacer_6, 3, 0, 1, 1);

        transformStack->addWidget(pageTranslate);
        pageRotate = new QWidget();
        pageRotate->setObjectName(QString::fromUtf8("pageRotate"));
        gridLayout2 = new QGridLayout(pageRotate);
        gridLayout2->setSpacing(5);
        gridLayout2->setContentsMargins(10, 10, 10, 10);
        gridLayout2->setObjectName(QString::fromUtf8("gridLayout2"));
        RotateStackLabel = new QLabel(pageRotate);
        RotateStackLabel->setObjectName(QString::fromUtf8("RotateStackLabel"));
        RotateStackLabel->setFont(font);
        RotateStackLabel->setAlignment(Qt::AlignCenter);

        gridLayout2->addWidget(RotateStackLabel, 0, 0, 1, 2);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout2->addItem(verticalSpacer_3, 2, 1, 1, 1);

        rotationValueLabel = new QLabel(pageRotate);
        rotationValueLabel->setObjectName(QString::fromUtf8("rotationValueLabel"));

        gridLayout2->addWidget(rotationValueLabel, 1, 0, 1, 1);

        rotationValue = new ScrSpinBox(pageRotate);
        rotationValue->setObjectName(QString::fromUtf8("rotationValue"));
        rotationValue->setDecimals(1);
        rotationValue->setMinimum(-360.000000000000000);
        rotationValue->setMaximum(360.000000000000000);

        gridLayout2->addWidget(rotationValue, 1, 1, 1, 1);

        transformStack->addWidget(pageRotate);
        Seite = new QWidget();
        Seite->setObjectName(QString::fromUtf8("Seite"));
        gridLayout3 = new QGridLayout(Seite);
        gridLayout3->setSpacing(5);
        gridLayout3->setContentsMargins(10, 10, 10, 10);
        gridLayout3->setObjectName(QString::fromUtf8("gridLayout3"));
        linkSkew = new LinkButton(Seite);
        linkSkew->setObjectName(QString::fromUtf8("linkSkew"));
        sizePolicy3.setHeightForWidth(linkSkew->sizePolicy().hasHeightForWidth());
        linkSkew->setSizePolicy(sizePolicy3);
        linkSkew->setMinimumSize(QSize(15, 0));
        linkSkew->setMaximumSize(QSize(15, 16777215));
        linkSkew->setCheckable(true);
        linkSkew->setAutoRaise(true);

        gridLayout3->addWidget(linkSkew, 1, 2, 2, 1);

        horizontalSkew = new ScrSpinBox(Seite);
        horizontalSkew->setObjectName(QString::fromUtf8("horizontalSkew"));
        horizontalSkew->setDecimals(1);
        horizontalSkew->setMinimum(-89.000000000000000);
        horizontalSkew->setMaximum(89.000000000000000);

        gridLayout3->addWidget(horizontalSkew, 1, 1, 1, 1);

        horizontalSkewLabel = new QLabel(Seite);
        horizontalSkewLabel->setObjectName(QString::fromUtf8("horizontalSkewLabel"));

        gridLayout3->addWidget(horizontalSkewLabel, 1, 0, 1, 1);

        verticalSkew = new ScrSpinBox(Seite);
        verticalSkew->setObjectName(QString::fromUtf8("verticalSkew"));
        verticalSkew->setDecimals(1);
        verticalSkew->setMinimum(-89.000000000000000);
        verticalSkew->setMaximum(89.000000000000000);

        gridLayout3->addWidget(verticalSkew, 2, 1, 1, 1);

        verticalSkewLabel = new QLabel(Seite);
        verticalSkewLabel->setObjectName(QString::fromUtf8("verticalSkewLabel"));

        gridLayout3->addWidget(verticalSkewLabel, 2, 0, 1, 1);

        SkewStackLabel = new QLabel(Seite);
        SkewStackLabel->setObjectName(QString::fromUtf8("SkewStackLabel"));
        SkewStackLabel->setFont(font);
        SkewStackLabel->setAlignment(Qt::AlignCenter);

        gridLayout3->addWidget(SkewStackLabel, 0, 0, 1, 3);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout3->addItem(verticalSpacer_4, 3, 1, 1, 1);

        transformStack->addWidget(Seite);

        verticalLayout_3->addWidget(transformStack);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(5);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalSpacer = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(5);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        label_10 = new QLabel(TransformDialogBase);
        label_10->setObjectName(QString::fromUtf8("label_10"));

        verticalLayout->addWidget(label_10);

        basePoint = new BasePointWidget(TransformDialogBase);
        basePoint->setObjectName(QString::fromUtf8("basePoint"));
        basePoint->setMinimumSize(QSize(56, 56));

        verticalLayout->addWidget(basePoint);

        verticalSpacer_2 = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);


        horizontalLayout->addLayout(verticalLayout);

        horizontalSpacer_3 = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_3);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setSpacing(5);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        label_9 = new QLabel(TransformDialogBase);
        label_9->setObjectName(QString::fromUtf8("label_9"));

        verticalLayout_2->addWidget(label_9);

        numberOfCopies = new QSpinBox(TransformDialogBase);
        numberOfCopies->setObjectName(QString::fromUtf8("numberOfCopies"));
        numberOfCopies->setMinimum(0);
        numberOfCopies->setMaximum(200);

        verticalLayout_2->addWidget(numberOfCopies);

        verticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout_2);

        horizontalSpacer_2 = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        verticalLayout_3->addLayout(horizontalLayout);

        spacerItem = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

        verticalLayout_3->addItem(spacerItem);


        horizontalLayout_2->addLayout(verticalLayout_3);


        verticalLayout_4->addLayout(horizontalLayout_2);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(5);
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        buttonAdd = new QPushButton(TransformDialogBase);
        buttonAdd->setObjectName(QString::fromUtf8("buttonAdd"));
        QSizePolicy sizePolicy4(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(buttonAdd->sizePolicy().hasHeightForWidth());
        buttonAdd->setSizePolicy(sizePolicy4);

        horizontalLayout_3->addWidget(buttonAdd);

        buttonRemove = new QPushButton(TransformDialogBase);
        buttonRemove->setObjectName(QString::fromUtf8("buttonRemove"));
        buttonRemove->setEnabled(false);

        horizontalLayout_3->addWidget(buttonRemove);

        buttonUp = new QPushButton(TransformDialogBase);
        buttonUp->setObjectName(QString::fromUtf8("buttonUp"));
        buttonUp->setEnabled(false);

        horizontalLayout_3->addWidget(buttonUp);

        buttonDown = new QPushButton(TransformDialogBase);
        buttonDown->setObjectName(QString::fromUtf8("buttonDown"));
        buttonDown->setEnabled(false);
        QSizePolicy sizePolicy5(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(buttonDown->sizePolicy().hasHeightForWidth());
        buttonDown->setSizePolicy(sizePolicy5);

        horizontalLayout_3->addWidget(buttonDown);

        spacerItem1 = new QSpacerItem(21, 26, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(spacerItem1);

        buttonBox = new QDialogButtonBox(TransformDialogBase);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        horizontalLayout_3->addWidget(buttonBox);


        verticalLayout_4->addLayout(horizontalLayout_3);


        retranslateUi(TransformDialogBase);
        QObject::connect(buttonBox, SIGNAL(accepted()), TransformDialogBase, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), TransformDialogBase, SLOT(reject()));

        transformStack->setCurrentIndex(2);


        QMetaObject::connectSlotsByName(TransformDialogBase);
    } // setupUi

    void retranslateUi(QDialog *TransformDialogBase)
    {
        TransformDialogBase->setWindowTitle(QCoreApplication::translate("TransformDialogBase", "Transform", nullptr));
        verticalScaleLabel->setText(QCoreApplication::translate("TransformDialogBase", "Vertical:", nullptr));
        horizontalScale->setSuffix(QCoreApplication::translate("TransformDialogBase", " %", nullptr));
        verticalScale->setSuffix(QCoreApplication::translate("TransformDialogBase", " %", nullptr));
        horizontalScaleLabel->setText(QCoreApplication::translate("TransformDialogBase", "Horizontal:", nullptr));
        scaleLink->setText(QString());
        ScalingStackLabel->setText(QCoreApplication::translate("TransformDialogBase", "Scaling", nullptr));
        translateHorizontalLabel->setText(QCoreApplication::translate("TransformDialogBase", "Horizontal:", nullptr));
        translateVerticalLabel->setText(QCoreApplication::translate("TransformDialogBase", "Vertical:", nullptr));
        TranslationStackLabel->setText(QCoreApplication::translate("TransformDialogBase", "Translation", nullptr));
        RotateStackLabel->setText(QCoreApplication::translate("TransformDialogBase", "Rotate", nullptr));
        rotationValueLabel->setText(QCoreApplication::translate("TransformDialogBase", "Angle:", nullptr));
        linkSkew->setText(QString());
        horizontalSkewLabel->setText(QCoreApplication::translate("TransformDialogBase", "Horizontal:", nullptr));
        verticalSkewLabel->setText(QCoreApplication::translate("TransformDialogBase", "Vertical:", nullptr));
        SkewStackLabel->setText(QCoreApplication::translate("TransformDialogBase", "Skew", nullptr));
        label_10->setText(QCoreApplication::translate("TransformDialogBase", "Basepoint", nullptr));
        label_9->setText(QCoreApplication::translate("TransformDialogBase", "Copies", nullptr));
        buttonAdd->setText(QCoreApplication::translate("TransformDialogBase", "Add", nullptr));
        buttonRemove->setText(QCoreApplication::translate("TransformDialogBase", "Remove", nullptr));
        buttonUp->setText(QString());
        buttonDown->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class TransformDialogBase: public Ui_TransformDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TRANSFORMDIALOGBASE_H
