/********************************************************************************
** Form generated from reading UI file 'copypagetomasterpage.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_COPYPAGETOMASTERPAGE_H
#define UI_COPYPAGETOMASTERPAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_CopyPageToMasterPageBase
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *nameLabel;
    QLineEdit *nameLineEdit;
    QComboBox *pagePositionComboBox;
    QCheckBox *copyMasterItemsCheckBox;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *CopyPageToMasterPageBase)
    {
        if (CopyPageToMasterPageBase->objectName().isEmpty())
            CopyPageToMasterPageBase->setObjectName(QString::fromUtf8("CopyPageToMasterPageBase"));
        CopyPageToMasterPageBase->setWindowModality(Qt::WindowModal);
        CopyPageToMasterPageBase->resize(252, 139);
        vboxLayout = new QVBoxLayout(CopyPageToMasterPageBase);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        nameLabel = new QLabel(CopyPageToMasterPageBase);
        nameLabel->setObjectName(QString::fromUtf8("nameLabel"));

        hboxLayout->addWidget(nameLabel);

        nameLineEdit = new QLineEdit(CopyPageToMasterPageBase);
        nameLineEdit->setObjectName(QString::fromUtf8("nameLineEdit"));

        hboxLayout->addWidget(nameLineEdit);


        vboxLayout->addLayout(hboxLayout);

        pagePositionComboBox = new QComboBox(CopyPageToMasterPageBase);
        pagePositionComboBox->setObjectName(QString::fromUtf8("pagePositionComboBox"));

        vboxLayout->addWidget(pagePositionComboBox);

        copyMasterItemsCheckBox = new QCheckBox(CopyPageToMasterPageBase);
        copyMasterItemsCheckBox->setObjectName(QString::fromUtf8("copyMasterItemsCheckBox"));

        vboxLayout->addWidget(copyMasterItemsCheckBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout->addItem(verticalSpacer);

        buttonBox = new QDialogButtonBox(CopyPageToMasterPageBase);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::NoButton|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(CopyPageToMasterPageBase);
        QObject::connect(buttonBox, SIGNAL(accepted()), CopyPageToMasterPageBase, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), CopyPageToMasterPageBase, SLOT(reject()));

        QMetaObject::connectSlotsByName(CopyPageToMasterPageBase);
    } // setupUi

    void retranslateUi(QDialog *CopyPageToMasterPageBase)
    {
        CopyPageToMasterPageBase->setWindowTitle(QCoreApplication::translate("CopyPageToMasterPageBase", "Convert Page to Master Page", nullptr));
        nameLabel->setText(QCoreApplication::translate("CopyPageToMasterPageBase", "Name:", nullptr));
        copyMasterItemsCheckBox->setText(QCoreApplication::translate("CopyPageToMasterPageBase", "Copy Applied Master Page Items", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CopyPageToMasterPageBase: public Ui_CopyPageToMasterPageBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_COPYPAGETOMASTERPAGE_H
