/********************************************************************************
** Form generated from reading UI file 'prefs_fontsbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_FONTSBASE_H
#define UI_PREFS_FONTSBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/fontlistview.h"

QT_BEGIN_NAMESPACE

class Ui_Prefs_Fonts
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QFrame *line;
    QTabWidget *tabWidget;
    QWidget *availableFontsTab;
    QVBoxLayout *verticalLayout_2;
    FontListView *fontListTableView;
    QWidget *fontSubstitutionsTab;
    QVBoxLayout *verticalLayout_3;
    QTableWidget *fontSubstitutionsTableWidget;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *deleteSubstitutionButton;
    QWidget *additionalPathsTab;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_2;
    QListWidget *pathListWidget;
    QVBoxLayout *verticalLayout_4;
    QPushButton *changeButton;
    QPushButton *addButton;
    QPushButton *removeButton;
    QSpacerItem *additionalPathsButtonsVerticalSpacer;
    QLabel *whyBlankLabel;
    QSpacerItem *additionalPathsLabelVerticalSpacer;
    QWidget *rejectedFontsTab;
    QGridLayout *gridLayout;
    QTableWidget *fontsRejectedTableWidget;

    void setupUi(QWidget *Prefs_Fonts)
    {
        if (Prefs_Fonts->objectName().isEmpty())
            Prefs_Fonts->setObjectName(QString::fromUtf8("Prefs_Fonts"));
        Prefs_Fonts->resize(509, 426);
        verticalLayout = new QVBoxLayout(Prefs_Fonts);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(Prefs_Fonts);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        verticalLayout->addWidget(label);

        line = new QFrame(Prefs_Fonts);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        tabWidget = new QTabWidget(Prefs_Fonts);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        availableFontsTab = new QWidget();
        availableFontsTab->setObjectName(QString::fromUtf8("availableFontsTab"));
        verticalLayout_2 = new QVBoxLayout(availableFontsTab);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        fontListTableView = new FontListView(availableFontsTab);
        fontListTableView->setObjectName(QString::fromUtf8("fontListTableView"));

        verticalLayout_2->addWidget(fontListTableView);

        tabWidget->addTab(availableFontsTab, QString());
        fontSubstitutionsTab = new QWidget();
        fontSubstitutionsTab->setObjectName(QString::fromUtf8("fontSubstitutionsTab"));
        verticalLayout_3 = new QVBoxLayout(fontSubstitutionsTab);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        fontSubstitutionsTableWidget = new QTableWidget(fontSubstitutionsTab);
        fontSubstitutionsTableWidget->setObjectName(QString::fromUtf8("fontSubstitutionsTableWidget"));

        verticalLayout_3->addWidget(fontSubstitutionsTableWidget);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        deleteSubstitutionButton = new QPushButton(fontSubstitutionsTab);
        deleteSubstitutionButton->setObjectName(QString::fromUtf8("deleteSubstitutionButton"));

        horizontalLayout->addWidget(deleteSubstitutionButton);


        verticalLayout_3->addLayout(horizontalLayout);

        tabWidget->addTab(fontSubstitutionsTab, QString());
        additionalPathsTab = new QWidget();
        additionalPathsTab->setObjectName(QString::fromUtf8("additionalPathsTab"));
        verticalLayout_5 = new QVBoxLayout(additionalPathsTab);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        pathListWidget = new QListWidget(additionalPathsTab);
        pathListWidget->setObjectName(QString::fromUtf8("pathListWidget"));

        horizontalLayout_2->addWidget(pathListWidget);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        changeButton = new QPushButton(additionalPathsTab);
        changeButton->setObjectName(QString::fromUtf8("changeButton"));

        verticalLayout_4->addWidget(changeButton);

        addButton = new QPushButton(additionalPathsTab);
        addButton->setObjectName(QString::fromUtf8("addButton"));

        verticalLayout_4->addWidget(addButton);

        removeButton = new QPushButton(additionalPathsTab);
        removeButton->setObjectName(QString::fromUtf8("removeButton"));

        verticalLayout_4->addWidget(removeButton);

        additionalPathsButtonsVerticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_4->addItem(additionalPathsButtonsVerticalSpacer);


        horizontalLayout_2->addLayout(verticalLayout_4);


        verticalLayout_5->addLayout(horizontalLayout_2);

        whyBlankLabel = new QLabel(additionalPathsTab);
        whyBlankLabel->setObjectName(QString::fromUtf8("whyBlankLabel"));
        whyBlankLabel->setWordWrap(true);

        verticalLayout_5->addWidget(whyBlankLabel);

        additionalPathsLabelVerticalSpacer = new QSpacerItem(0, 0, QSizePolicy::Minimum, QSizePolicy::Minimum);

        verticalLayout_5->addItem(additionalPathsLabelVerticalSpacer);

        tabWidget->addTab(additionalPathsTab, QString());
        rejectedFontsTab = new QWidget();
        rejectedFontsTab->setObjectName(QString::fromUtf8("rejectedFontsTab"));
        gridLayout = new QGridLayout(rejectedFontsTab);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        fontsRejectedTableWidget = new QTableWidget(rejectedFontsTab);
        if (fontsRejectedTableWidget->columnCount() < 3)
            fontsRejectedTableWidget->setColumnCount(3);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        fontsRejectedTableWidget->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        fontsRejectedTableWidget->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        fontsRejectedTableWidget->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        fontsRejectedTableWidget->setObjectName(QString::fromUtf8("fontsRejectedTableWidget"));
        fontsRejectedTableWidget->setColumnCount(3);

        gridLayout->addWidget(fontsRejectedTableWidget, 0, 0, 1, 1);

        tabWidget->addTab(rejectedFontsTab, QString());

        verticalLayout->addWidget(tabWidget);


        retranslateUi(Prefs_Fonts);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Prefs_Fonts);
    } // setupUi

    void retranslateUi(QWidget *Prefs_Fonts)
    {
        label->setText(QCoreApplication::translate("Prefs_Fonts", "Fonts", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(availableFontsTab), QCoreApplication::translate("Prefs_Fonts", "Available Fonts", nullptr));
        deleteSubstitutionButton->setText(QCoreApplication::translate("Prefs_Fonts", "Delete", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(fontSubstitutionsTab), QCoreApplication::translate("Prefs_Fonts", "Font Substitutions", nullptr));
        changeButton->setText(QCoreApplication::translate("Prefs_Fonts", "C&hange...", nullptr));
        addButton->setText(QCoreApplication::translate("Prefs_Fonts", "A&dd...", nullptr));
        removeButton->setText(QCoreApplication::translate("Prefs_Fonts", "&Remove", nullptr));
        whyBlankLabel->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(additionalPathsTab), QCoreApplication::translate("Prefs_Fonts", "Additional Paths", nullptr));
        QTableWidgetItem *___qtablewidgetitem = fontsRejectedTableWidget->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("Prefs_Fonts", "Font name", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = fontsRejectedTableWidget->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("Prefs_Fonts", "Error message", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = fontsRejectedTableWidget->horizontalHeaderItem(2);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("Prefs_Fonts", "Font file", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(rejectedFontsTab), QCoreApplication::translate("Prefs_Fonts", "Rejected Fonts", nullptr));
        (void)Prefs_Fonts;
    } // retranslateUi

};

namespace Ui {
    class Prefs_Fonts: public Ui_Prefs_Fonts {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_FONTSBASE_H
