/****************************************************************************
** Meta object code from reading C++ file 'transparencypalette.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/transparencypalette.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'transparencypalette.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TransparencyPalette_t {
    QByteArrayData data[39];
    char stringdata0[488];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TransparencyPalette_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TransparencyPalette_t qt_meta_stringdata_TransparencyPalette = {
    {
QT_MOC_LITERAL(0, 0, 19), // "TransparencyPalette"
QT_MOC_LITERAL(1, 20, 8), // "NewTrans"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 9), // "NewTransS"
QT_MOC_LITERAL(4, 40, 8), // "NewBlend"
QT_MOC_LITERAL(5, 49, 9), // "NewBlendS"
QT_MOC_LITERAL(6, 59, 15), // "gradientChanged"
QT_MOC_LITERAL(7, 75, 12), // "editGradient"
QT_MOC_LITERAL(8, 88, 11), // "NewGradient"
QT_MOC_LITERAL(9, 100, 10), // "NewSpecial"
QT_MOC_LITERAL(10, 111, 10), // "NewPattern"
QT_MOC_LITERAL(11, 122, 15), // "NewPatternProps"
QT_MOC_LITERAL(12, 138, 19), // "handleUpdateRequest"
QT_MOC_LITERAL(13, 158, 22), // "editLineSelectorButton"
QT_MOC_LITERAL(14, 181, 22), // "editFillSelectorButton"
QT_MOC_LITERAL(15, 204, 8), // "slotGrad"
QT_MOC_LITERAL(16, 213, 6), // "number"
QT_MOC_LITERAL(17, 220, 12), // "slotGradType"
QT_MOC_LITERAL(18, 233, 4), // "type"
QT_MOC_LITERAL(19, 238, 16), // "setNamedGradient"
QT_MOC_LITERAL(20, 255, 4), // "name"
QT_MOC_LITERAL(21, 260, 18), // "switchGradientMode"
QT_MOC_LITERAL(22, 279, 17), // "switchPatternMode"
QT_MOC_LITERAL(23, 297, 18), // "editGradientVector"
QT_MOC_LITERAL(24, 316, 16), // "setActiveGradDia"
QT_MOC_LITERAL(25, 333, 6), // "active"
QT_MOC_LITERAL(26, 340, 18), // "hideEditedPatterns"
QT_MOC_LITERAL(27, 359, 5), // "names"
QT_MOC_LITERAL(28, 365, 13), // "selectPattern"
QT_MOC_LITERAL(29, 379, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(30, 396, 1), // "c"
QT_MOC_LITERAL(31, 398, 18), // "changePatternProps"
QT_MOC_LITERAL(32, 417, 11), // "setActTrans"
QT_MOC_LITERAL(33, 429, 11), // "setActBlend"
QT_MOC_LITERAL(34, 441, 10), // "slotTransS"
QT_MOC_LITERAL(35, 452, 3), // "val"
QT_MOC_LITERAL(36, 456, 10), // "slotTransF"
QT_MOC_LITERAL(37, 467, 10), // "unitChange"
QT_MOC_LITERAL(38, 478, 9) // "unitIndex"

    },
    "TransparencyPalette\0NewTrans\0\0NewTransS\0"
    "NewBlend\0NewBlendS\0gradientChanged\0"
    "editGradient\0NewGradient\0NewSpecial\0"
    "NewPattern\0NewPatternProps\0"
    "handleUpdateRequest\0editLineSelectorButton\0"
    "editFillSelectorButton\0slotGrad\0number\0"
    "slotGradType\0type\0setNamedGradient\0"
    "name\0switchGradientMode\0switchPatternMode\0"
    "editGradientVector\0setActiveGradDia\0"
    "active\0hideEditedPatterns\0names\0"
    "selectPattern\0QListWidgetItem*\0c\0"
    "changePatternProps\0setActTrans\0"
    "setActBlend\0slotTransS\0val\0slotTransF\0"
    "unitChange\0unitIndex"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TransparencyPalette[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      28,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  154,    2, 0x06 /* Public */,
       3,    1,  157,    2, 0x06 /* Public */,
       4,    1,  160,    2, 0x06 /* Public */,
       5,    1,  163,    2, 0x06 /* Public */,
       6,    0,  166,    2, 0x06 /* Public */,
       7,    0,  167,    2, 0x06 /* Public */,
       8,    1,  168,    2, 0x06 /* Public */,
       9,   10,  171,    2, 0x06 /* Public */,
      10,    1,  192,    2, 0x06 /* Public */,
      11,    9,  195,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      12,    1,  214,    2, 0x0a /* Public */,
      13,    0,  217,    2, 0x0a /* Public */,
      14,    0,  218,    2, 0x0a /* Public */,
      15,    1,  219,    2, 0x0a /* Public */,
      17,    1,  222,    2, 0x0a /* Public */,
      19,    1,  225,    2, 0x0a /* Public */,
      21,    0,  228,    2, 0x0a /* Public */,
      22,    0,  229,    2, 0x0a /* Public */,
      23,    0,  230,    2, 0x0a /* Public */,
      24,    1,  231,    2, 0x0a /* Public */,
      26,    1,  234,    2, 0x0a /* Public */,
      28,    1,  237,    2, 0x0a /* Public */,
      31,    0,  240,    2, 0x0a /* Public */,
      32,    2,  241,    2, 0x0a /* Public */,
      33,    2,  246,    2, 0x0a /* Public */,
      34,    1,  251,    2, 0x0a /* Public */,
      36,    1,  254,    2, 0x0a /* Public */,
      37,    3,  257,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Bool, QMetaType::Bool,    2,    2,    2,    2,    2,    2,    2,    2,    2,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void, QMetaType::Int,   18,
    QMetaType::Void, QMetaType::QString,   20,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   25,
    QMetaType::Void, QMetaType::QStringList,   27,
    QMetaType::Void, 0x80000000 | 29,   30,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,    2,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    2,    2,
    QMetaType::Void, QMetaType::Double,   35,
    QMetaType::Void, QMetaType::Double,   35,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Int,    2,    2,   38,

       0        // eod
};

void TransparencyPalette::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TransparencyPalette *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->NewTrans((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 1: _t->NewTransS((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 2: _t->NewBlend((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->NewBlendS((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->gradientChanged(); break;
        case 5: _t->editGradient(); break;
        case 6: _t->NewGradient((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->NewSpecial((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6])),(*reinterpret_cast< double(*)>(_a[7])),(*reinterpret_cast< double(*)>(_a[8])),(*reinterpret_cast< double(*)>(_a[9])),(*reinterpret_cast< double(*)>(_a[10]))); break;
        case 8: _t->NewPattern((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 9: _t->NewPatternProps((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6])),(*reinterpret_cast< double(*)>(_a[7])),(*reinterpret_cast< bool(*)>(_a[8])),(*reinterpret_cast< bool(*)>(_a[9]))); break;
        case 10: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->editLineSelectorButton(); break;
        case 12: _t->editFillSelectorButton(); break;
        case 13: _t->slotGrad((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->slotGradType((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->setNamedGradient((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 16: _t->switchGradientMode(); break;
        case 17: _t->switchPatternMode(); break;
        case 18: _t->editGradientVector(); break;
        case 19: _t->setActiveGradDia((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->hideEditedPatterns((*reinterpret_cast< QStringList(*)>(_a[1]))); break;
        case 21: _t->selectPattern((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 22: _t->changePatternProps(); break;
        case 23: _t->setActTrans((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 24: _t->setActBlend((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 25: _t->slotTransS((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 26: _t->slotTransF((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 27: _t->unitChange((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TransparencyPalette::*)(double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransparencyPalette::NewTrans)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TransparencyPalette::*)(double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransparencyPalette::NewTransS)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (TransparencyPalette::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransparencyPalette::NewBlend)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (TransparencyPalette::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransparencyPalette::NewBlendS)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (TransparencyPalette::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransparencyPalette::gradientChanged)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (TransparencyPalette::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransparencyPalette::editGradient)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (TransparencyPalette::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransparencyPalette::NewGradient)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (TransparencyPalette::*)(double , double , double , double , double , double , double , double , double , double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransparencyPalette::NewSpecial)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (TransparencyPalette::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransparencyPalette::NewPattern)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (TransparencyPalette::*)(double , double , double , double , double , double , double , bool , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TransparencyPalette::NewPatternProps)) {
                *result = 9;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TransparencyPalette::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_TransparencyPalette.data,
    qt_meta_data_TransparencyPalette,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TransparencyPalette::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TransparencyPalette::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TransparencyPalette.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::transparencyPalette"))
        return static_cast< Ui::transparencyPalette*>(this);
    return QWidget::qt_metacast(_clname);
}

int TransparencyPalette::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 28)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 28;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 28)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 28;
    }
    return _id;
}

// SIGNAL 0
void TransparencyPalette::NewTrans(double _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void TransparencyPalette::NewTransS(double _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void TransparencyPalette::NewBlend(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void TransparencyPalette::NewBlendS(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void TransparencyPalette::gradientChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void TransparencyPalette::editGradient()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void TransparencyPalette::NewGradient(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void TransparencyPalette::NewSpecial(double _t1, double _t2, double _t3, double _t4, double _t5, double _t6, double _t7, double _t8, double _t9, double _t10)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t6))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t7))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t8))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t9))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t10))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void TransparencyPalette::NewPattern(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void TransparencyPalette::NewPatternProps(double _t1, double _t2, double _t3, double _t4, double _t5, double _t6, double _t7, bool _t8, bool _t9)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t6))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t7))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t8))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t9))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
