/********************************************************************************
** Form generated from reading UI file 'colorpalette.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_COLORPALETTE_H
#define UI_COLORPALETTE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/colorcombo.h"
#include "ui/colorlistbox.h"
#include "ui/gradienteditor.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_colorPalette
{
public:
    QVBoxLayout *verticalLayout;
    QTabWidget *tabFillStroke;
    QWidget *tabFill;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QLabel *fillModeLabel;
    QComboBox *fillModeCombo;
    QFrame *line;
    QStackedWidget *fillModeStack;
    QWidget *solidFillPage;
    QVBoxLayout *verticalLayout_3;
    ColorListBox *colorListFill;
    QHBoxLayout *horizontalLayout_3;
    QLabel *fillShadeLabel;
    ScrSpinBox *fillShade;
    QWidget *gradientFillPage;
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout_4;
    QComboBox *gradientType;
    QToolButton *gradEditButton;
    QStackedWidget *stackedWidget_2;
    QWidget *page_3;
    QVBoxLayout *verticalLayout_4;
    QComboBox *namedGradient;
    GradientEditor *gradEdit;
    QHBoxLayout *horizontalLayout_10;
    QLabel *gradientExtendLabel;
    QComboBox *gradientExtend;
    QWidget *page_4;
    QVBoxLayout *verticalLayout_8;
    QGridLayout *gridLayout_4;
    QLabel *colorPointLabel;
    QLabel *colorShadeLabel;
    QLabel *colorAlphaLabel;
    ColorCombo *colorPoint1;
    ScrSpinBox *color1Shade;
    ScrSpinBox *color1Alpha;
    ColorCombo *colorPoint2;
    ScrSpinBox *color2Shade;
    ScrSpinBox *color2Alpha;
    ColorCombo *colorPoint3;
    ScrSpinBox *color3Shade;
    ScrSpinBox *color3Alpha;
    ColorCombo *colorPoint4;
    ScrSpinBox *color4Shade;
    ScrSpinBox *color4Alpha;
    QSpacerItem *verticalSpacer;
    QWidget *page_5;
    QVBoxLayout *verticalLayout_9;
    QGridLayout *gridLayout_5;
    QLabel *colorMeshPointLabel;
    ColorCombo *colorMeshPoint;
    QLabel *shadeMeshPointLabel;
    ScrSpinBox *shadeMeshPoint;
    QLabel *transparencyMeshPointLabel;
    ScrSpinBox *transparencyMeshPoint;
    QHBoxLayout *horizontalLayout_8;
    QToolButton *editMeshColors;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer_2;
    QWidget *page;
    QVBoxLayout *verticalLayout_12;
    QGridLayout *gridLayout;
    QLabel *hatchDistLabel;
    ScrSpinBox *hatchDist;
    QLabel *hatchAngleLabel;
    ScrSpinBox *hatchAngle;
    QLabel *hatchTypeLabel;
    QComboBox *hatchType;
    QLabel *hatchLineColorLabel;
    ColorCombo *hatchLineColor;
    QLabel *hatchBackgroundLabel;
    ColorCombo *hatchBackground;
    QSpacerItem *verticalSpacer_3;
    QWidget *patternFillPage;
    QVBoxLayout *verticalLayout_6;
    QListWidget *patternBox;
    QHBoxLayout *horizontalLayout_5;
    QToolButton *editPatternProps;
    QSpacerItem *horizontalSpacer_3;
    QWidget *tabStroke;
    QVBoxLayout *verticalLayout_11;
    QHBoxLayout *horizontalLayout_6;
    QLabel *strokeModeLabel;
    QComboBox *strokeModeCombo;
    QFrame *line_2;
    QStackedWidget *strokeModeStack;
    QWidget *solidStrokePage;
    QVBoxLayout *verticalLayout_7;
    ColorListBox *colorListStroke;
    QHBoxLayout *horizontalLayout_7;
    QLabel *strokeShadeLabel;
    ScrSpinBox *strokeShade;
    QWidget *gradientStrokePage;
    QVBoxLayout *verticalLayout_10;
    QHBoxLayout *horizontalLayout_9;
    QComboBox *gradientTypeStroke;
    QToolButton *gradEditButtonStroke;
    QComboBox *namedGradientStroke;
    GradientEditor *gradEditStroke;
    QHBoxLayout *horizontalLayout_12;
    QLabel *GradientExtendSLabel;
    QComboBox *GradientExtendS;
    QWidget *patternStrokePage;
    QVBoxLayout *verticalLayout_14;
    QListWidget *patternBoxStroke;
    QHBoxLayout *horizontalLayout_11;
    QToolButton *editPatternPropsStroke;
    QSpacerItem *horizontalSpacer_5;
    QCheckBox *followsPath;
    QHBoxLayout *horizontalLayout;
    QLabel *overPrintLabel;
    QComboBox *overPrintCombo;
    QCheckBox *displayAllColors;

    void setupUi(QWidget *colorPalette)
    {
        if (colorPalette->objectName().isEmpty())
            colorPalette->setObjectName(QString::fromUtf8("colorPalette"));
        colorPalette->resize(320, 360);
        verticalLayout = new QVBoxLayout(colorPalette);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        tabFillStroke = new QTabWidget(colorPalette);
        tabFillStroke->setObjectName(QString::fromUtf8("tabFillStroke"));
        tabFill = new QWidget();
        tabFill->setObjectName(QString::fromUtf8("tabFill"));
        verticalLayout_2 = new QVBoxLayout(tabFill);
        verticalLayout_2->setSpacing(3);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(3, 3, 3, 3);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(3);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        fillModeLabel = new QLabel(tabFill);
        fillModeLabel->setObjectName(QString::fromUtf8("fillModeLabel"));

        horizontalLayout_2->addWidget(fillModeLabel);

        fillModeCombo = new QComboBox(tabFill);
        fillModeCombo->setObjectName(QString::fromUtf8("fillModeCombo"));

        horizontalLayout_2->addWidget(fillModeCombo);


        verticalLayout_2->addLayout(horizontalLayout_2);

        line = new QFrame(tabFill);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line);

        fillModeStack = new QStackedWidget(tabFill);
        fillModeStack->setObjectName(QString::fromUtf8("fillModeStack"));
        solidFillPage = new QWidget();
        solidFillPage->setObjectName(QString::fromUtf8("solidFillPage"));
        verticalLayout_3 = new QVBoxLayout(solidFillPage);
        verticalLayout_3->setSpacing(3);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        colorListFill = new ColorListBox(solidFillPage);
        colorListFill->setObjectName(QString::fromUtf8("colorListFill"));

        verticalLayout_3->addWidget(colorListFill);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(3);
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        fillShadeLabel = new QLabel(solidFillPage);
        fillShadeLabel->setObjectName(QString::fromUtf8("fillShadeLabel"));

        horizontalLayout_3->addWidget(fillShadeLabel);

        fillShade = new ScrSpinBox(solidFillPage);
        fillShade->setObjectName(QString::fromUtf8("fillShade"));
        fillShade->setMaximum(100.000000000000000);
        fillShade->setValue(100.000000000000000);

        horizontalLayout_3->addWidget(fillShade);


        verticalLayout_3->addLayout(horizontalLayout_3);

        fillModeStack->addWidget(solidFillPage);
        gradientFillPage = new QWidget();
        gradientFillPage->setObjectName(QString::fromUtf8("gradientFillPage"));
        verticalLayout_5 = new QVBoxLayout(gradientFillPage);
        verticalLayout_5->setSpacing(3);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(3);
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        gradientType = new QComboBox(gradientFillPage);
        gradientType->addItem(QString());
        gradientType->addItem(QString());
        gradientType->addItem(QString());
        gradientType->addItem(QString());
        gradientType->addItem(QString());
        gradientType->addItem(QString());
        gradientType->addItem(QString());
        gradientType->setObjectName(QString::fromUtf8("gradientType"));

        horizontalLayout_4->addWidget(gradientType);

        gradEditButton = new QToolButton(gradientFillPage);
        gradEditButton->setObjectName(QString::fromUtf8("gradEditButton"));
        gradEditButton->setCheckable(true);

        horizontalLayout_4->addWidget(gradEditButton);


        verticalLayout_5->addLayout(horizontalLayout_4);

        stackedWidget_2 = new QStackedWidget(gradientFillPage);
        stackedWidget_2->setObjectName(QString::fromUtf8("stackedWidget_2"));
        page_3 = new QWidget();
        page_3->setObjectName(QString::fromUtf8("page_3"));
        verticalLayout_4 = new QVBoxLayout(page_3);
        verticalLayout_4->setSpacing(3);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        namedGradient = new QComboBox(page_3);
        namedGradient->setObjectName(QString::fromUtf8("namedGradient"));

        verticalLayout_4->addWidget(namedGradient);

        gradEdit = new GradientEditor(page_3);
        gradEdit->setObjectName(QString::fromUtf8("gradEdit"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(gradEdit->sizePolicy().hasHeightForWidth());
        gradEdit->setSizePolicy(sizePolicy);
        gradEdit->setFrameShape(QFrame::NoFrame);
        gradEdit->setFrameShadow(QFrame::Raised);

        verticalLayout_4->addWidget(gradEdit);

        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setSpacing(3);
        horizontalLayout_10->setObjectName(QString::fromUtf8("horizontalLayout_10"));
        gradientExtendLabel = new QLabel(page_3);
        gradientExtendLabel->setObjectName(QString::fromUtf8("gradientExtendLabel"));

        horizontalLayout_10->addWidget(gradientExtendLabel);

        gradientExtend = new QComboBox(page_3);
        gradientExtend->addItem(QString());
        gradientExtend->addItem(QString());
        gradientExtend->setObjectName(QString::fromUtf8("gradientExtend"));

        horizontalLayout_10->addWidget(gradientExtend);


        verticalLayout_4->addLayout(horizontalLayout_10);

        stackedWidget_2->addWidget(page_3);
        page_4 = new QWidget();
        page_4->setObjectName(QString::fromUtf8("page_4"));
        verticalLayout_8 = new QVBoxLayout(page_4);
        verticalLayout_8->setSpacing(3);
        verticalLayout_8->setObjectName(QString::fromUtf8("verticalLayout_8"));
        verticalLayout_8->setContentsMargins(0, 0, 0, 0);
        gridLayout_4 = new QGridLayout();
        gridLayout_4->setSpacing(3);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        colorPointLabel = new QLabel(page_4);
        colorPointLabel->setObjectName(QString::fromUtf8("colorPointLabel"));

        gridLayout_4->addWidget(colorPointLabel, 0, 0, 1, 1);

        colorShadeLabel = new QLabel(page_4);
        colorShadeLabel->setObjectName(QString::fromUtf8("colorShadeLabel"));

        gridLayout_4->addWidget(colorShadeLabel, 0, 1, 1, 1);

        colorAlphaLabel = new QLabel(page_4);
        colorAlphaLabel->setObjectName(QString::fromUtf8("colorAlphaLabel"));

        gridLayout_4->addWidget(colorAlphaLabel, 0, 2, 1, 1);

        colorPoint1 = new ColorCombo(page_4);
        colorPoint1->setObjectName(QString::fromUtf8("colorPoint1"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(colorPoint1->sizePolicy().hasHeightForWidth());
        colorPoint1->setSizePolicy(sizePolicy1);

        gridLayout_4->addWidget(colorPoint1, 1, 0, 1, 1);

        color1Shade = new ScrSpinBox(page_4);
        color1Shade->setObjectName(QString::fromUtf8("color1Shade"));
        color1Shade->setMinimum(0.000000000000000);
        color1Shade->setMaximum(100.000000000000000);
        color1Shade->setSingleStep(10.000000000000000);
        color1Shade->setValue(100.000000000000000);

        gridLayout_4->addWidget(color1Shade, 1, 1, 1, 1);

        color1Alpha = new ScrSpinBox(page_4);
        color1Alpha->setObjectName(QString::fromUtf8("color1Alpha"));
        color1Alpha->setMaximum(100.000000000000000);
        color1Alpha->setSingleStep(10.000000000000000);
        color1Alpha->setValue(100.000000000000000);

        gridLayout_4->addWidget(color1Alpha, 1, 2, 1, 1);

        colorPoint2 = new ColorCombo(page_4);
        colorPoint2->setObjectName(QString::fromUtf8("colorPoint2"));
        sizePolicy1.setHeightForWidth(colorPoint2->sizePolicy().hasHeightForWidth());
        colorPoint2->setSizePolicy(sizePolicy1);

        gridLayout_4->addWidget(colorPoint2, 2, 0, 1, 1);

        color2Shade = new ScrSpinBox(page_4);
        color2Shade->setObjectName(QString::fromUtf8("color2Shade"));
        color2Shade->setMinimum(0.000000000000000);
        color2Shade->setMaximum(100.000000000000000);
        color2Shade->setSingleStep(10.000000000000000);
        color2Shade->setValue(100.000000000000000);

        gridLayout_4->addWidget(color2Shade, 2, 1, 1, 1);

        color2Alpha = new ScrSpinBox(page_4);
        color2Alpha->setObjectName(QString::fromUtf8("color2Alpha"));
        color2Alpha->setMaximum(100.000000000000000);
        color2Alpha->setSingleStep(10.000000000000000);
        color2Alpha->setValue(100.000000000000000);

        gridLayout_4->addWidget(color2Alpha, 2, 2, 1, 1);

        colorPoint3 = new ColorCombo(page_4);
        colorPoint3->setObjectName(QString::fromUtf8("colorPoint3"));
        sizePolicy1.setHeightForWidth(colorPoint3->sizePolicy().hasHeightForWidth());
        colorPoint3->setSizePolicy(sizePolicy1);

        gridLayout_4->addWidget(colorPoint3, 3, 0, 1, 1);

        color3Shade = new ScrSpinBox(page_4);
        color3Shade->setObjectName(QString::fromUtf8("color3Shade"));
        color3Shade->setMinimum(0.000000000000000);
        color3Shade->setMaximum(100.000000000000000);
        color3Shade->setSingleStep(10.000000000000000);
        color3Shade->setValue(100.000000000000000);

        gridLayout_4->addWidget(color3Shade, 3, 1, 1, 1);

        color3Alpha = new ScrSpinBox(page_4);
        color3Alpha->setObjectName(QString::fromUtf8("color3Alpha"));
        color3Alpha->setMaximum(100.000000000000000);
        color3Alpha->setSingleStep(10.000000000000000);
        color3Alpha->setValue(100.000000000000000);

        gridLayout_4->addWidget(color3Alpha, 3, 2, 1, 1);

        colorPoint4 = new ColorCombo(page_4);
        colorPoint4->setObjectName(QString::fromUtf8("colorPoint4"));
        sizePolicy1.setHeightForWidth(colorPoint4->sizePolicy().hasHeightForWidth());
        colorPoint4->setSizePolicy(sizePolicy1);

        gridLayout_4->addWidget(colorPoint4, 4, 0, 1, 1);

        color4Shade = new ScrSpinBox(page_4);
        color4Shade->setObjectName(QString::fromUtf8("color4Shade"));
        color4Shade->setMinimum(0.000000000000000);
        color4Shade->setMaximum(100.000000000000000);
        color4Shade->setSingleStep(10.000000000000000);
        color4Shade->setValue(100.000000000000000);

        gridLayout_4->addWidget(color4Shade, 4, 1, 1, 1);

        color4Alpha = new ScrSpinBox(page_4);
        color4Alpha->setObjectName(QString::fromUtf8("color4Alpha"));
        color4Alpha->setMaximum(100.000000000000000);
        color4Alpha->setSingleStep(10.000000000000000);
        color4Alpha->setValue(100.000000000000000);

        gridLayout_4->addWidget(color4Alpha, 4, 2, 1, 1);


        verticalLayout_8->addLayout(gridLayout_4);

        verticalSpacer = new QSpacerItem(20, 70, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_8->addItem(verticalSpacer);

        stackedWidget_2->addWidget(page_4);
        page_5 = new QWidget();
        page_5->setObjectName(QString::fromUtf8("page_5"));
        verticalLayout_9 = new QVBoxLayout(page_5);
        verticalLayout_9->setSpacing(3);
        verticalLayout_9->setObjectName(QString::fromUtf8("verticalLayout_9"));
        verticalLayout_9->setContentsMargins(0, 0, 0, 0);
        gridLayout_5 = new QGridLayout();
        gridLayout_5->setSpacing(3);
        gridLayout_5->setObjectName(QString::fromUtf8("gridLayout_5"));
        colorMeshPointLabel = new QLabel(page_5);
        colorMeshPointLabel->setObjectName(QString::fromUtf8("colorMeshPointLabel"));

        gridLayout_5->addWidget(colorMeshPointLabel, 0, 0, 1, 1);

        colorMeshPoint = new ColorCombo(page_5);
        colorMeshPoint->setObjectName(QString::fromUtf8("colorMeshPoint"));
        colorMeshPoint->setEnabled(false);
        sizePolicy1.setHeightForWidth(colorMeshPoint->sizePolicy().hasHeightForWidth());
        colorMeshPoint->setSizePolicy(sizePolicy1);

        gridLayout_5->addWidget(colorMeshPoint, 0, 1, 1, 1);

        shadeMeshPointLabel = new QLabel(page_5);
        shadeMeshPointLabel->setObjectName(QString::fromUtf8("shadeMeshPointLabel"));

        gridLayout_5->addWidget(shadeMeshPointLabel, 1, 0, 1, 1);

        shadeMeshPoint = new ScrSpinBox(page_5);
        shadeMeshPoint->setObjectName(QString::fromUtf8("shadeMeshPoint"));
        shadeMeshPoint->setEnabled(false);
        shadeMeshPoint->setMinimum(0.000000000000000);
        shadeMeshPoint->setMaximum(100.000000000000000);
        shadeMeshPoint->setSingleStep(10.000000000000000);
        shadeMeshPoint->setValue(100.000000000000000);

        gridLayout_5->addWidget(shadeMeshPoint, 1, 1, 1, 1);

        transparencyMeshPointLabel = new QLabel(page_5);
        transparencyMeshPointLabel->setObjectName(QString::fromUtf8("transparencyMeshPointLabel"));

        gridLayout_5->addWidget(transparencyMeshPointLabel, 2, 0, 1, 1);

        transparencyMeshPoint = new ScrSpinBox(page_5);
        transparencyMeshPoint->setObjectName(QString::fromUtf8("transparencyMeshPoint"));
        transparencyMeshPoint->setEnabled(false);
        transparencyMeshPoint->setMaximum(100.000000000000000);
        transparencyMeshPoint->setSingleStep(10.000000000000000);
        transparencyMeshPoint->setValue(100.000000000000000);

        gridLayout_5->addWidget(transparencyMeshPoint, 2, 1, 1, 1);


        verticalLayout_9->addLayout(gridLayout_5);

        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setSpacing(3);
        horizontalLayout_8->setObjectName(QString::fromUtf8("horizontalLayout_8"));
        editMeshColors = new QToolButton(page_5);
        editMeshColors->setObjectName(QString::fromUtf8("editMeshColors"));
        editMeshColors->setCheckable(true);

        horizontalLayout_8->addWidget(editMeshColors);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_2);


        verticalLayout_9->addLayout(horizontalLayout_8);

        verticalSpacer_2 = new QSpacerItem(20, 101, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_9->addItem(verticalSpacer_2);

        stackedWidget_2->addWidget(page_5);

        verticalLayout_5->addWidget(stackedWidget_2);

        fillModeStack->addWidget(gradientFillPage);
        page = new QWidget();
        page->setObjectName(QString::fromUtf8("page"));
        verticalLayout_12 = new QVBoxLayout(page);
        verticalLayout_12->setSpacing(3);
        verticalLayout_12->setObjectName(QString::fromUtf8("verticalLayout_12"));
        verticalLayout_12->setContentsMargins(0, 0, 0, 0);
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(3);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        hatchDistLabel = new QLabel(page);
        hatchDistLabel->setObjectName(QString::fromUtf8("hatchDistLabel"));

        gridLayout->addWidget(hatchDistLabel, 0, 0, 1, 1);

        hatchDist = new ScrSpinBox(page);
        hatchDist->setObjectName(QString::fromUtf8("hatchDist"));

        gridLayout->addWidget(hatchDist, 0, 1, 1, 1);

        hatchAngleLabel = new QLabel(page);
        hatchAngleLabel->setObjectName(QString::fromUtf8("hatchAngleLabel"));

        gridLayout->addWidget(hatchAngleLabel, 1, 0, 1, 1);

        hatchAngle = new ScrSpinBox(page);
        hatchAngle->setObjectName(QString::fromUtf8("hatchAngle"));
        hatchAngle->setMinimum(-360.000000000000000);
        hatchAngle->setMaximum(360.000000000000000);

        gridLayout->addWidget(hatchAngle, 1, 1, 1, 1);

        hatchTypeLabel = new QLabel(page);
        hatchTypeLabel->setObjectName(QString::fromUtf8("hatchTypeLabel"));

        gridLayout->addWidget(hatchTypeLabel, 2, 0, 1, 1);

        hatchType = new QComboBox(page);
        hatchType->addItem(QString());
        hatchType->addItem(QString());
        hatchType->addItem(QString());
        hatchType->setObjectName(QString::fromUtf8("hatchType"));

        gridLayout->addWidget(hatchType, 2, 1, 1, 1);

        hatchLineColorLabel = new QLabel(page);
        hatchLineColorLabel->setObjectName(QString::fromUtf8("hatchLineColorLabel"));

        gridLayout->addWidget(hatchLineColorLabel, 3, 0, 1, 1);

        hatchLineColor = new ColorCombo(page);
        hatchLineColor->setObjectName(QString::fromUtf8("hatchLineColor"));

        gridLayout->addWidget(hatchLineColor, 3, 1, 1, 1);

        hatchBackgroundLabel = new QLabel(page);
        hatchBackgroundLabel->setObjectName(QString::fromUtf8("hatchBackgroundLabel"));

        gridLayout->addWidget(hatchBackgroundLabel, 4, 0, 1, 1);

        hatchBackground = new ColorCombo(page);
        hatchBackground->setObjectName(QString::fromUtf8("hatchBackground"));

        gridLayout->addWidget(hatchBackground, 4, 1, 1, 1);


        verticalLayout_12->addLayout(gridLayout);

        verticalSpacer_3 = new QSpacerItem(20, 75, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_12->addItem(verticalSpacer_3);

        fillModeStack->addWidget(page);
        patternFillPage = new QWidget();
        patternFillPage->setObjectName(QString::fromUtf8("patternFillPage"));
        verticalLayout_6 = new QVBoxLayout(patternFillPage);
        verticalLayout_6->setSpacing(3);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        verticalLayout_6->setContentsMargins(0, 0, 0, 0);
        patternBox = new QListWidget(patternFillPage);
        patternBox->setObjectName(QString::fromUtf8("patternBox"));
        patternBox->setMovement(QListView::Snap);
        patternBox->setFlow(QListView::LeftToRight);
        patternBox->setProperty("isWrapping", QVariant(true));
        patternBox->setResizeMode(QListView::Adjust);
        patternBox->setViewMode(QListView::IconMode);
        patternBox->setWordWrap(true);

        verticalLayout_6->addWidget(patternBox);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setSpacing(3);
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        editPatternProps = new QToolButton(patternFillPage);
        editPatternProps->setObjectName(QString::fromUtf8("editPatternProps"));

        horizontalLayout_5->addWidget(editPatternProps);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_3);


        verticalLayout_6->addLayout(horizontalLayout_5);

        fillModeStack->addWidget(patternFillPage);

        verticalLayout_2->addWidget(fillModeStack);

        tabFillStroke->addTab(tabFill, QString());
        tabStroke = new QWidget();
        tabStroke->setObjectName(QString::fromUtf8("tabStroke"));
        verticalLayout_11 = new QVBoxLayout(tabStroke);
        verticalLayout_11->setSpacing(3);
        verticalLayout_11->setObjectName(QString::fromUtf8("verticalLayout_11"));
        verticalLayout_11->setContentsMargins(3, 3, 3, 3);
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setSpacing(3);
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        strokeModeLabel = new QLabel(tabStroke);
        strokeModeLabel->setObjectName(QString::fromUtf8("strokeModeLabel"));

        horizontalLayout_6->addWidget(strokeModeLabel);

        strokeModeCombo = new QComboBox(tabStroke);
        strokeModeCombo->setObjectName(QString::fromUtf8("strokeModeCombo"));

        horizontalLayout_6->addWidget(strokeModeCombo);


        verticalLayout_11->addLayout(horizontalLayout_6);

        line_2 = new QFrame(tabStroke);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        verticalLayout_11->addWidget(line_2);

        strokeModeStack = new QStackedWidget(tabStroke);
        strokeModeStack->setObjectName(QString::fromUtf8("strokeModeStack"));
        solidStrokePage = new QWidget();
        solidStrokePage->setObjectName(QString::fromUtf8("solidStrokePage"));
        verticalLayout_7 = new QVBoxLayout(solidStrokePage);
        verticalLayout_7->setSpacing(3);
        verticalLayout_7->setObjectName(QString::fromUtf8("verticalLayout_7"));
        verticalLayout_7->setContentsMargins(0, 0, 0, 0);
        colorListStroke = new ColorListBox(solidStrokePage);
        colorListStroke->setObjectName(QString::fromUtf8("colorListStroke"));

        verticalLayout_7->addWidget(colorListStroke);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setSpacing(3);
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        strokeShadeLabel = new QLabel(solidStrokePage);
        strokeShadeLabel->setObjectName(QString::fromUtf8("strokeShadeLabel"));

        horizontalLayout_7->addWidget(strokeShadeLabel);

        strokeShade = new ScrSpinBox(solidStrokePage);
        strokeShade->setObjectName(QString::fromUtf8("strokeShade"));
        strokeShade->setMaximum(100.000000000000000);
        strokeShade->setValue(100.000000000000000);

        horizontalLayout_7->addWidget(strokeShade);


        verticalLayout_7->addLayout(horizontalLayout_7);

        strokeModeStack->addWidget(solidStrokePage);
        gradientStrokePage = new QWidget();
        gradientStrokePage->setObjectName(QString::fromUtf8("gradientStrokePage"));
        verticalLayout_10 = new QVBoxLayout(gradientStrokePage);
        verticalLayout_10->setSpacing(3);
        verticalLayout_10->setObjectName(QString::fromUtf8("verticalLayout_10"));
        verticalLayout_10->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setSpacing(3);
        horizontalLayout_9->setObjectName(QString::fromUtf8("horizontalLayout_9"));
        gradientTypeStroke = new QComboBox(gradientStrokePage);
        gradientTypeStroke->addItem(QString());
        gradientTypeStroke->addItem(QString());
        gradientTypeStroke->setObjectName(QString::fromUtf8("gradientTypeStroke"));

        horizontalLayout_9->addWidget(gradientTypeStroke);

        gradEditButtonStroke = new QToolButton(gradientStrokePage);
        gradEditButtonStroke->setObjectName(QString::fromUtf8("gradEditButtonStroke"));
        gradEditButtonStroke->setCheckable(true);

        horizontalLayout_9->addWidget(gradEditButtonStroke);


        verticalLayout_10->addLayout(horizontalLayout_9);

        namedGradientStroke = new QComboBox(gradientStrokePage);
        namedGradientStroke->setObjectName(QString::fromUtf8("namedGradientStroke"));

        verticalLayout_10->addWidget(namedGradientStroke);

        gradEditStroke = new GradientEditor(gradientStrokePage);
        gradEditStroke->setObjectName(QString::fromUtf8("gradEditStroke"));
        sizePolicy.setHeightForWidth(gradEditStroke->sizePolicy().hasHeightForWidth());
        gradEditStroke->setSizePolicy(sizePolicy);
        gradEditStroke->setFrameShape(QFrame::NoFrame);
        gradEditStroke->setFrameShadow(QFrame::Raised);

        verticalLayout_10->addWidget(gradEditStroke);

        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setSpacing(3);
        horizontalLayout_12->setObjectName(QString::fromUtf8("horizontalLayout_12"));
        GradientExtendSLabel = new QLabel(gradientStrokePage);
        GradientExtendSLabel->setObjectName(QString::fromUtf8("GradientExtendSLabel"));

        horizontalLayout_12->addWidget(GradientExtendSLabel);

        GradientExtendS = new QComboBox(gradientStrokePage);
        GradientExtendS->addItem(QString());
        GradientExtendS->addItem(QString());
        GradientExtendS->setObjectName(QString::fromUtf8("GradientExtendS"));

        horizontalLayout_12->addWidget(GradientExtendS);


        verticalLayout_10->addLayout(horizontalLayout_12);

        strokeModeStack->addWidget(gradientStrokePage);
        patternStrokePage = new QWidget();
        patternStrokePage->setObjectName(QString::fromUtf8("patternStrokePage"));
        verticalLayout_14 = new QVBoxLayout(patternStrokePage);
        verticalLayout_14->setSpacing(3);
        verticalLayout_14->setObjectName(QString::fromUtf8("verticalLayout_14"));
        verticalLayout_14->setContentsMargins(0, 0, 0, 0);
        patternBoxStroke = new QListWidget(patternStrokePage);
        patternBoxStroke->setObjectName(QString::fromUtf8("patternBoxStroke"));
        patternBoxStroke->setMovement(QListView::Snap);
        patternBoxStroke->setFlow(QListView::LeftToRight);
        patternBoxStroke->setProperty("isWrapping", QVariant(true));
        patternBoxStroke->setResizeMode(QListView::Adjust);
        patternBoxStroke->setViewMode(QListView::IconMode);
        patternBoxStroke->setWordWrap(true);

        verticalLayout_14->addWidget(patternBoxStroke);

        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setSpacing(3);
        horizontalLayout_11->setObjectName(QString::fromUtf8("horizontalLayout_11"));
        editPatternPropsStroke = new QToolButton(patternStrokePage);
        editPatternPropsStroke->setObjectName(QString::fromUtf8("editPatternPropsStroke"));

        horizontalLayout_11->addWidget(editPatternPropsStroke);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_5);

        followsPath = new QCheckBox(patternStrokePage);
        followsPath->setObjectName(QString::fromUtf8("followsPath"));
        followsPath->setLayoutDirection(Qt::RightToLeft);

        horizontalLayout_11->addWidget(followsPath);


        verticalLayout_14->addLayout(horizontalLayout_11);

        strokeModeStack->addWidget(patternStrokePage);

        verticalLayout_11->addWidget(strokeModeStack);

        tabFillStroke->addTab(tabStroke, QString());

        verticalLayout->addWidget(tabFillStroke);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(3);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        overPrintLabel = new QLabel(colorPalette);
        overPrintLabel->setObjectName(QString::fromUtf8("overPrintLabel"));

        horizontalLayout->addWidget(overPrintLabel);

        overPrintCombo = new QComboBox(colorPalette);
        overPrintCombo->addItem(QString());
        overPrintCombo->addItem(QString());
        overPrintCombo->setObjectName(QString::fromUtf8("overPrintCombo"));

        horizontalLayout->addWidget(overPrintCombo);


        verticalLayout->addLayout(horizontalLayout);

        displayAllColors = new QCheckBox(colorPalette);
        displayAllColors->setObjectName(QString::fromUtf8("displayAllColors"));
        displayAllColors->setLayoutDirection(Qt::RightToLeft);

        verticalLayout->addWidget(displayAllColors);


        retranslateUi(colorPalette);

        tabFillStroke->setCurrentIndex(0);
        fillModeStack->setCurrentIndex(0);
        stackedWidget_2->setCurrentIndex(0);
        strokeModeStack->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(colorPalette);
    } // setupUi

    void retranslateUi(QWidget *colorPalette)
    {
        fillModeLabel->setText(QCoreApplication::translate("colorPalette", "Fill Mode", nullptr));
        fillShadeLabel->setText(QCoreApplication::translate("colorPalette", "Shade:", nullptr));
        fillShade->setSuffix(QCoreApplication::translate("colorPalette", "%", nullptr));
        gradientType->setItemText(0, QCoreApplication::translate("colorPalette", "Linear", nullptr));
        gradientType->setItemText(1, QCoreApplication::translate("colorPalette", "Radial", nullptr));
        gradientType->setItemText(2, QCoreApplication::translate("colorPalette", "Conical", nullptr));
        gradientType->setItemText(3, QCoreApplication::translate("colorPalette", "4 Colors", nullptr));
        gradientType->setItemText(4, QCoreApplication::translate("colorPalette", "Diamond", nullptr));
        gradientType->setItemText(5, QCoreApplication::translate("colorPalette", "Mesh", nullptr));
        gradientType->setItemText(6, QCoreApplication::translate("colorPalette", "Patch Mesh", nullptr));

        gradEditButton->setText(QCoreApplication::translate("colorPalette", "Move Vector", nullptr));
        gradientExtendLabel->setText(QCoreApplication::translate("colorPalette", "Extend:", nullptr));
        gradientExtend->setItemText(0, QCoreApplication::translate("colorPalette", "None", nullptr));
        gradientExtend->setItemText(1, QCoreApplication::translate("colorPalette", "Pad", nullptr));

        colorPointLabel->setText(QCoreApplication::translate("colorPalette", "Color", nullptr));
        colorShadeLabel->setText(QCoreApplication::translate("colorPalette", "Shade", nullptr));
        colorAlphaLabel->setText(QCoreApplication::translate("colorPalette", "Transparency", nullptr));
        color1Shade->setSuffix(QCoreApplication::translate("colorPalette", " %", nullptr));
        color1Alpha->setSuffix(QCoreApplication::translate("colorPalette", " %", nullptr));
        color2Shade->setSuffix(QCoreApplication::translate("colorPalette", " %", nullptr));
        color2Alpha->setSuffix(QCoreApplication::translate("colorPalette", " %", nullptr));
        color3Shade->setSuffix(QCoreApplication::translate("colorPalette", " %", nullptr));
        color3Alpha->setSuffix(QCoreApplication::translate("colorPalette", " %", nullptr));
        color4Shade->setSuffix(QCoreApplication::translate("colorPalette", " %", nullptr));
        color4Alpha->setSuffix(QCoreApplication::translate("colorPalette", " %", nullptr));
        colorMeshPointLabel->setText(QCoreApplication::translate("colorPalette", "Color:", nullptr));
        shadeMeshPointLabel->setText(QCoreApplication::translate("colorPalette", "Shade:", nullptr));
        shadeMeshPoint->setSuffix(QCoreApplication::translate("colorPalette", " %", nullptr));
        transparencyMeshPointLabel->setText(QCoreApplication::translate("colorPalette", "Opacity:", nullptr));
        transparencyMeshPoint->setSuffix(QCoreApplication::translate("colorPalette", " %", nullptr));
        editMeshColors->setText(QCoreApplication::translate("colorPalette", "Edit Colors", nullptr));
        hatchDistLabel->setText(QCoreApplication::translate("colorPalette", "Distance:", nullptr));
        hatchAngleLabel->setText(QCoreApplication::translate("colorPalette", "Angle:", nullptr));
        hatchTypeLabel->setText(QCoreApplication::translate("colorPalette", "Type:", nullptr));
        hatchType->setItemText(0, QCoreApplication::translate("colorPalette", "Single", nullptr));
        hatchType->setItemText(1, QCoreApplication::translate("colorPalette", "Double", nullptr));
        hatchType->setItemText(2, QCoreApplication::translate("colorPalette", "Triple", nullptr));

        hatchLineColorLabel->setText(QCoreApplication::translate("colorPalette", "Line Color:", nullptr));
        hatchBackgroundLabel->setText(QCoreApplication::translate("colorPalette", "Background:", nullptr));
        editPatternProps->setText(QCoreApplication::translate("colorPalette", "Properties...", nullptr));
        tabFillStroke->setTabText(tabFillStroke->indexOf(tabFill), QCoreApplication::translate("colorPalette", "Fill", nullptr));
        strokeModeLabel->setText(QCoreApplication::translate("colorPalette", "Stroke Mode", nullptr));
        strokeShadeLabel->setText(QCoreApplication::translate("colorPalette", "Shade:", nullptr));
        strokeShade->setSuffix(QCoreApplication::translate("colorPalette", "%", nullptr));
        gradientTypeStroke->setItemText(0, QCoreApplication::translate("colorPalette", "Linear", nullptr));
        gradientTypeStroke->setItemText(1, QCoreApplication::translate("colorPalette", "Radial", nullptr));

        gradEditButtonStroke->setText(QCoreApplication::translate("colorPalette", "Move Vector", nullptr));
        GradientExtendSLabel->setText(QCoreApplication::translate("colorPalette", "Extend:", nullptr));
        GradientExtendS->setItemText(0, QCoreApplication::translate("colorPalette", "None", nullptr));
        GradientExtendS->setItemText(1, QCoreApplication::translate("colorPalette", "Pad", nullptr));

        editPatternPropsStroke->setText(QCoreApplication::translate("colorPalette", "Properties...", nullptr));
        followsPath->setText(QCoreApplication::translate("colorPalette", "Pattern follows Path", nullptr));
        tabFillStroke->setTabText(tabFillStroke->indexOf(tabStroke), QCoreApplication::translate("colorPalette", "Stroke", nullptr));
        overPrintLabel->setText(QCoreApplication::translate("colorPalette", "Overprinting", nullptr));
        overPrintCombo->setItemText(0, QCoreApplication::translate("colorPalette", "Knockout", nullptr));
        overPrintCombo->setItemText(1, QCoreApplication::translate("colorPalette", "Overprint", nullptr));

        displayAllColors->setText(QCoreApplication::translate("colorPalette", "Display only used colors", nullptr));
        (void)colorPalette;
    } // retranslateUi

};

namespace Ui {
    class colorPalette: public Ui_colorPalette {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_COLORPALETTE_H
