/********************************************************************************
** Form generated from reading UI file 'propertywidget_hyphenationbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYWIDGET_HYPHENATIONBASE_H
#define UI_PROPERTYWIDGET_HYPHENATIONBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_PropertyWidget_HyphenationBase
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QSpinBox *smallestWordSpinBox;
    QLabel *label_2;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_3;
    QSpinBox *maxConsecutiveCountSpinBox;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_4;
    QLineEdit *hyphenCharLineEdit;

    void setupUi(QFrame *PropertyWidget_HyphenationBase)
    {
        if (PropertyWidget_HyphenationBase->objectName().isEmpty())
            PropertyWidget_HyphenationBase->setObjectName(QString::fromUtf8("PropertyWidget_HyphenationBase"));
        PropertyWidget_HyphenationBase->resize(305, 95);
        verticalLayout = new QVBoxLayout(PropertyWidget_HyphenationBase);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(3);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label = new QLabel(PropertyWidget_HyphenationBase);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout->addWidget(label);

        smallestWordSpinBox = new QSpinBox(PropertyWidget_HyphenationBase);
        smallestWordSpinBox->setObjectName(QString::fromUtf8("smallestWordSpinBox"));
        smallestWordSpinBox->setMinimum(3);

        horizontalLayout->addWidget(smallestWordSpinBox);

        label_2 = new QLabel(PropertyWidget_HyphenationBase);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        horizontalLayout->addWidget(label_2);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(3);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        label_3 = new QLabel(PropertyWidget_HyphenationBase);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        horizontalLayout_2->addWidget(label_3);

        maxConsecutiveCountSpinBox = new QSpinBox(PropertyWidget_HyphenationBase);
        maxConsecutiveCountSpinBox->setObjectName(QString::fromUtf8("maxConsecutiveCountSpinBox"));

        horizontalLayout_2->addWidget(maxConsecutiveCountSpinBox);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(3);
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        label_4 = new QLabel(PropertyWidget_HyphenationBase);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        horizontalLayout_3->addWidget(label_4);

        hyphenCharLineEdit = new QLineEdit(PropertyWidget_HyphenationBase);
        hyphenCharLineEdit->setObjectName(QString::fromUtf8("hyphenCharLineEdit"));

        horizontalLayout_3->addWidget(hyphenCharLineEdit);


        verticalLayout->addLayout(horizontalLayout_3);


        retranslateUi(PropertyWidget_HyphenationBase);

        QMetaObject::connectSlotsByName(PropertyWidget_HyphenationBase);
    } // setupUi

    void retranslateUi(QFrame *PropertyWidget_HyphenationBase)
    {
        label->setText(QCoreApplication::translate("PropertyWidget_HyphenationBase", "Smallest Word:", nullptr));
#if QT_CONFIG(tooltip)
        smallestWordSpinBox->setToolTip(QCoreApplication::translate("PropertyWidget_HyphenationBase", "Length of the smallest word to be hyphenated", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(QCoreApplication::translate("PropertyWidget_HyphenationBase", "Characters", nullptr));
        label_3->setText(QCoreApplication::translate("PropertyWidget_HyphenationBase", "Consecutive Hyphenations Allowed:", nullptr));
#if QT_CONFIG(tooltip)
        maxConsecutiveCountSpinBox->setToolTip(QCoreApplication::translate("PropertyWidget_HyphenationBase", "Maximum number of Hyphenations following each other.\n"
"A value of 0 means unlimited hyphenations.", nullptr));
#endif // QT_CONFIG(tooltip)
        label_4->setText(QCoreApplication::translate("PropertyWidget_HyphenationBase", "Hyphenation Character:", nullptr));
#if QT_CONFIG(tooltip)
        hyphenCharLineEdit->setToolTip(QCoreApplication::translate("PropertyWidget_HyphenationBase", "The character used for automatic hyphen. Can be empty.", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)PropertyWidget_HyphenationBase;
    } // retranslateUi

};

namespace Ui {
    class PropertyWidget_HyphenationBase: public Ui_PropertyWidget_HyphenationBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTYWIDGET_HYPHENATIONBASE_H
