/********************************************************************************
** Form generated from reading UI file 'annot.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ANNOT_H
#define UI_ANNOT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/colorcombo.h"
#include "ui/propertiespalette_utils.h"

QT_BEGIN_NAMESPACE

class Ui_ScAnnot
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QComboBox *ComboBox1;
    QStackedWidget *Fram;
    QWidget *page;
    QVBoxLayout *verticalLayout_2;
    QGroupBox *PropertiesGroup;
    QGridLayout *gridLayout;
    QLabel *label_2;
    QLineEdit *Tip;
    QTabWidget *TabWidget2;
    QWidget *tabAppearance;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *TextGroup;
    QGridLayout *gridLayout_2;
    QComboBox *Schrift;
    QLabel *label_4;
    QHBoxLayout *horizontalLayout_3;
    QGroupBox *BorderGroup;
    QGridLayout *gridLayout_3;
    QLabel *label_5;
    ColorCombo *BorderC;
    QComboBox *BorderW;
    QLabel *label_6;
    QLabel *label_7;
    QComboBox *BorderS;
    QGroupBox *OtherGroup;
    QGridLayout *gridLayout_4;
    QLabel *label_8;
    QComboBox *Visib;
    QCheckBox *NoExport;
    QCheckBox *Required;
    QCheckBox *ReadOnly;
    QSpacerItem *verticalSpacer_14;
    QWidget *tabOptions;
    QVBoxLayout *verticalLayout_4;
    QStackedWidget *FramOp;
    QWidget *page_3;
    QVBoxLayout *verticalLayout_5;
    QGroupBox *TextOptionGroup;
    QGridLayout *gridLayout_5;
    QLabel *label_9;
    QLineEdit *DownT;
    QLabel *label_10;
    QLineEdit *TextO;
    QHBoxLayout *horizontalLayout_4;
    QGroupBox *IconOptionGroup;
    QVBoxLayout *verticalLayout_6;
    QCheckBox *UseIcons;
    QHBoxLayout *horizontalLayout_5;
    QGridLayout *gridLayout_6;
    QPushButton *IconNR;
    QLabel *NiconPrev;
    QPushButton *IconN;
    QGridLayout *gridLayout_9;
    QLabel *PiconPrev;
    QPushButton *IconP;
    QPushButton *IconPR;
    QGridLayout *gridLayout_11;
    QPushButton *IconR;
    QLabel *RiconPrev;
    QPushButton *IconRR;
    QHBoxLayout *horizontalLayout_6;
    QPushButton *PlaceIcon;
    QGroupBox *HighlightOptionGroup;
    QVBoxLayout *verticalLayout_7;
    QComboBox *ComboBox7_2;
    QWidget *page_4;
    QVBoxLayout *verticalLayout_8;
    QCheckBox *MultiL;
    QCheckBox *Passwd;
    QHBoxLayout *horizontalLayout_7;
    QCheckBox *Limit;
    QSpinBox *MaxChars;
    QLabel *label_11;
    QSpacerItem *horizontalSpacer_2;
    QCheckBox *NoScroll;
    QCheckBox *NoSpell;
    QCheckBox *AutoTextSize;
    QSpacerItem *verticalSpacer_5;
    QWidget *page_5;
    QVBoxLayout *verticalLayout_9;
    QHBoxLayout *horizontalLayout_8;
    QLabel *CText1;
    QComboBox *ChkStil;
    QSpacerItem *horizontalSpacer_3;
    QCheckBox *isChkd;
    QSpacerItem *verticalSpacer_4;
    QWidget *page_6;
    QVBoxLayout *verticalLayout_10;
    QCheckBox *CanEdit;
    QSpacerItem *verticalSpacer_6;
    QSpacerItem *verticalSpacer_13;
    QWidget *tabAction;
    QVBoxLayout *verticalLayout_11;
    QHBoxLayout *horizontalLayout_9;
    QLabel *label_12;
    QComboBox *ActionCombo;
    QStackedWidget *Fram2;
    QWidget *page_7;
    QVBoxLayout *verticalLayout_12;
    QFrame *frame;
    QWidget *page_8;
    QVBoxLayout *verticalLayout_13;
    QFrame *frame_2;
    QVBoxLayout *verticalLayout_14;
    QHBoxLayout *horizontalLayout_10;
    QLabel *label_13;
    QComboBox *SelAction;
    QLabel *label_14;
    QHBoxLayout *horizontalLayout_11;
    QTextEdit *EditJava;
    QPushButton *EditJ;
    QWidget *page_9;
    QVBoxLayout *verticalLayout_15;
    QGroupBox *GroupBoxDestination;
    QGridLayout *GroupBoxDestinationLayout;
    QPushButton *ChFile;
    QLabel *label_15;
    QLineEdit *Destfile;
    QSpinBox *SpinBox11;
    QLabel *label_17;
    QCheckBox *useAbsolute;
    QSpinBox *SpinBox31;
    QCheckBox *LExtern;
    QLabel *label_16;
    QSpinBox *SpinBox21;
    QSpacerItem *verticalSpacer_12;
    QWidget *page_10;
    QVBoxLayout *verticalLayout_16;
    QFrame *frame_3;
    QVBoxLayout *verticalLayout_17;
    QLabel *label_18;
    QLineEdit *SubURL;
    QLabel *label_19;
    QComboBox *SelAsHtml;
    QSpacerItem *verticalSpacer;
    QWidget *page_11;
    QVBoxLayout *verticalLayout_18;
    QFrame *frame_4;
    QVBoxLayout *verticalLayout_19;
    QLabel *label_20;
    QLineEdit *SubURLa;
    QSpacerItem *verticalSpacer_2;
    QWidget *page_12;
    QVBoxLayout *verticalLayout_20;
    QFrame *frame_5;
    QVBoxLayout *verticalLayout_21;
    QLabel *label_21;
    QComboBox *nameActionCombo;
    QSpacerItem *verticalSpacer_3;
    QWidget *tabFormat;
    QVBoxLayout *verticalLayout_22;
    QHBoxLayout *horizontalLayout_12;
    QLabel *label_22;
    QComboBox *TxFormat;
    QStackedWidget *FoFram;
    QWidget *page_13;
    QVBoxLayout *verticalLayout_23;
    QFrame *frame_6;
    QWidget *page_14;
    QVBoxLayout *verticalLayout_24;
    QGroupBox *NumbGroup;
    QVBoxLayout *verticalLayout_25;
    QHBoxLayout *horizontalLayout_13;
    QLabel *label_23;
    QSpinBox *Decim;
    QSpacerItem *horizontalSpacer_4;
    QHBoxLayout *horizontalLayout_14;
    QCheckBox *UseCurr;
    QLineEdit *CurSym;
    QCheckBox *PreCurr;
    QGroupBox *NumGroup2;
    QGridLayout *gridLayout_7;
    QRadioButton *Format0;
    QRadioButton *Format2;
    QRadioButton *Format1;
    QRadioButton *Format3;
    QSpacerItem *verticalSpacer_10;
    QWidget *page_15;
    QVBoxLayout *verticalLayout_26;
    QGroupBox *PercGroup;
    QVBoxLayout *verticalLayout_27;
    QHBoxLayout *horizontalLayout_15;
    QLabel *label_24;
    QSpinBox *Decim2;
    QSpacerItem *horizontalSpacer_5;
    QGroupBox *NumGroup2a;
    QGridLayout *gridLayout_8;
    QRadioButton *Format0a;
    QRadioButton *Format2a;
    QRadioButton *Format1a;
    QRadioButton *Format3a;
    QSpacerItem *verticalSpacer_7;
    QWidget *page_16;
    QVBoxLayout *verticalLayout_28;
    QGroupBox *DateGroup;
    QVBoxLayout *verticalLayout_29;
    QHBoxLayout *horizontalLayout_16;
    QComboBox *Format0c;
    QSpacerItem *horizontalSpacer_6;
    QLabel *TextDa1;
    QSpacerItem *verticalSpacer_8;
    QWidget *page_17;
    QVBoxLayout *verticalLayout_30;
    QGroupBox *TimeGroup;
    QVBoxLayout *verticalLayout_31;
    QRadioButton *Format0b;
    QRadioButton *Format1b;
    QRadioButton *Format2b;
    QRadioButton *Format3b;
    QSpacerItem *verticalSpacer_9;
    QWidget *page_18;
    QVBoxLayout *verticalLayout_32;
    QGroupBox *GroupCust;
    QVBoxLayout *verticalLayout_33;
    QHBoxLayout *horizontalLayout_17;
    QLabel *label_25;
    QPushButton *EditFormat;
    QTextEdit *FormatScript;
    QHBoxLayout *horizontalLayout_18;
    QLabel *label_26;
    QPushButton *EditKeystr;
    QTextEdit *KeyScript;
    QSpacerItem *verticalSpacer_11;
    QWidget *tabValidate;
    QVBoxLayout *verticalLayout_34;
    QRadioButton *NoValid;
    QGridLayout *gridLayout_10;
    QRadioButton *SimpleValid;
    QLabel *label_27;
    QLineEdit *MinValid;
    QLineEdit *MaxValid;
    QRadioButton *CustomValid;
    QHBoxLayout *horizontalLayout_19;
    QTextEdit *ValidScript;
    QPushButton *EditValScript;
    QWidget *tabCalculate;
    QVBoxLayout *verticalLayout_36;
    QRadioButton *NoCalc;
    QHBoxLayout *horizontalLayout_20;
    QRadioButton *SimpleCalc;
    QComboBox *CalcArt;
    QLabel *label_28;
    QHBoxLayout *horizontalLayout_21;
    QLineEdit *CalcFields;
    QPushButton *SeField;
    QRadioButton *CustomCalc;
    QHBoxLayout *horizontalLayout_22;
    QTextEdit *CalcScript;
    QPushButton *EditCalc;
    NameWidget *Name;
    QLabel *label_3;
    QWidget *page_2;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer;
    QPushButton *PushButton1;
    QPushButton *PushButton2;

    void setupUi(QDialog *ScAnnot)
    {
        if (ScAnnot->objectName().isEmpty())
            ScAnnot->setObjectName(QString::fromUtf8("ScAnnot"));
        ScAnnot->resize(537, 562);
        verticalLayout = new QVBoxLayout(ScAnnot);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label = new QLabel(ScAnnot);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout->addWidget(label);

        ComboBox1 = new QComboBox(ScAnnot);
        ComboBox1->addItem(QString());
        ComboBox1->addItem(QString());
        ComboBox1->addItem(QString());
        ComboBox1->addItem(QString());
        ComboBox1->addItem(QString());
        ComboBox1->addItem(QString());
        ComboBox1->setObjectName(QString::fromUtf8("ComboBox1"));

        horizontalLayout->addWidget(ComboBox1);


        verticalLayout->addLayout(horizontalLayout);

        Fram = new QStackedWidget(ScAnnot);
        Fram->setObjectName(QString::fromUtf8("Fram"));
        Fram->setMinimumSize(QSize(519, 0));
        page = new QWidget();
        page->setObjectName(QString::fromUtf8("page"));
        verticalLayout_2 = new QVBoxLayout(page);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        PropertiesGroup = new QGroupBox(page);
        PropertiesGroup->setObjectName(QString::fromUtf8("PropertiesGroup"));
        gridLayout = new QGridLayout(PropertiesGroup);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label_2 = new QLabel(PropertiesGroup);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 0, 0, 1, 1);

        Tip = new QLineEdit(PropertiesGroup);
        Tip->setObjectName(QString::fromUtf8("Tip"));

        gridLayout->addWidget(Tip, 1, 1, 1, 1);

        TabWidget2 = new QTabWidget(PropertiesGroup);
        TabWidget2->setObjectName(QString::fromUtf8("TabWidget2"));
        tabAppearance = new QWidget();
        tabAppearance->setObjectName(QString::fromUtf8("tabAppearance"));
        verticalLayout_3 = new QVBoxLayout(tabAppearance);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        TextGroup = new QGroupBox(tabAppearance);
        TextGroup->setObjectName(QString::fromUtf8("TextGroup"));
        gridLayout_2 = new QGridLayout(TextGroup);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        Schrift = new QComboBox(TextGroup);
        Schrift->addItem(QString());
        Schrift->addItem(QString());
        Schrift->addItem(QString());
        Schrift->addItem(QString());
        Schrift->addItem(QString());
        Schrift->addItem(QString());
        Schrift->addItem(QString());
        Schrift->addItem(QString());
        Schrift->addItem(QString());
        Schrift->addItem(QString());
        Schrift->addItem(QString());
        Schrift->addItem(QString());
        Schrift->addItem(QString());
        Schrift->addItem(QString());
        Schrift->setObjectName(QString::fromUtf8("Schrift"));

        gridLayout_2->addWidget(Schrift, 0, 1, 1, 1);

        label_4 = new QLabel(TextGroup);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout_2->addWidget(label_4, 0, 0, 1, 1);


        verticalLayout_3->addWidget(TextGroup);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        BorderGroup = new QGroupBox(tabAppearance);
        BorderGroup->setObjectName(QString::fromUtf8("BorderGroup"));
        gridLayout_3 = new QGridLayout(BorderGroup);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        label_5 = new QLabel(BorderGroup);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        gridLayout_3->addWidget(label_5, 0, 0, 1, 1);

        BorderC = new ColorCombo(BorderGroup);
        BorderC->setObjectName(QString::fromUtf8("BorderC"));

        gridLayout_3->addWidget(BorderC, 0, 1, 1, 1);

        BorderW = new QComboBox(BorderGroup);
        BorderW->addItem(QString());
        BorderW->addItem(QString());
        BorderW->addItem(QString());
        BorderW->addItem(QString());
        BorderW->setObjectName(QString::fromUtf8("BorderW"));

        gridLayout_3->addWidget(BorderW, 1, 1, 1, 1);

        label_6 = new QLabel(BorderGroup);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        gridLayout_3->addWidget(label_6, 1, 0, 1, 1);

        label_7 = new QLabel(BorderGroup);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        gridLayout_3->addWidget(label_7, 2, 0, 1, 1);

        BorderS = new QComboBox(BorderGroup);
        BorderS->addItem(QString());
        BorderS->addItem(QString());
        BorderS->addItem(QString());
        BorderS->addItem(QString());
        BorderS->addItem(QString());
        BorderS->setObjectName(QString::fromUtf8("BorderS"));

        gridLayout_3->addWidget(BorderS, 2, 1, 1, 1);


        horizontalLayout_3->addWidget(BorderGroup);

        OtherGroup = new QGroupBox(tabAppearance);
        OtherGroup->setObjectName(QString::fromUtf8("OtherGroup"));
        gridLayout_4 = new QGridLayout(OtherGroup);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        label_8 = new QLabel(OtherGroup);
        label_8->setObjectName(QString::fromUtf8("label_8"));

        gridLayout_4->addWidget(label_8, 3, 0, 1, 1);

        Visib = new QComboBox(OtherGroup);
        Visib->addItem(QString());
        Visib->addItem(QString());
        Visib->addItem(QString());
        Visib->addItem(QString());
        Visib->setObjectName(QString::fromUtf8("Visib"));

        gridLayout_4->addWidget(Visib, 3, 1, 1, 1);

        NoExport = new QCheckBox(OtherGroup);
        NoExport->setObjectName(QString::fromUtf8("NoExport"));

        gridLayout_4->addWidget(NoExport, 2, 0, 1, 2);

        Required = new QCheckBox(OtherGroup);
        Required->setObjectName(QString::fromUtf8("Required"));

        gridLayout_4->addWidget(Required, 1, 0, 1, 2);

        ReadOnly = new QCheckBox(OtherGroup);
        ReadOnly->setObjectName(QString::fromUtf8("ReadOnly"));

        gridLayout_4->addWidget(ReadOnly, 0, 0, 1, 2);


        horizontalLayout_3->addWidget(OtherGroup);


        verticalLayout_3->addLayout(horizontalLayout_3);

        verticalSpacer_14 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_14);

        TabWidget2->addTab(tabAppearance, QString());
        tabOptions = new QWidget();
        tabOptions->setObjectName(QString::fromUtf8("tabOptions"));
        verticalLayout_4 = new QVBoxLayout(tabOptions);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        FramOp = new QStackedWidget(tabOptions);
        FramOp->setObjectName(QString::fromUtf8("FramOp"));
        page_3 = new QWidget();
        page_3->setObjectName(QString::fromUtf8("page_3"));
        verticalLayout_5 = new QVBoxLayout(page_3);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        TextOptionGroup = new QGroupBox(page_3);
        TextOptionGroup->setObjectName(QString::fromUtf8("TextOptionGroup"));
        gridLayout_5 = new QGridLayout(TextOptionGroup);
        gridLayout_5->setObjectName(QString::fromUtf8("gridLayout_5"));
        label_9 = new QLabel(TextOptionGroup);
        label_9->setObjectName(QString::fromUtf8("label_9"));

        gridLayout_5->addWidget(label_9, 0, 0, 1, 1);

        DownT = new QLineEdit(TextOptionGroup);
        DownT->setObjectName(QString::fromUtf8("DownT"));

        gridLayout_5->addWidget(DownT, 0, 1, 1, 1);

        label_10 = new QLabel(TextOptionGroup);
        label_10->setObjectName(QString::fromUtf8("label_10"));

        gridLayout_5->addWidget(label_10, 1, 0, 1, 1);

        TextO = new QLineEdit(TextOptionGroup);
        TextO->setObjectName(QString::fromUtf8("TextO"));

        gridLayout_5->addWidget(TextO, 1, 1, 1, 1);


        verticalLayout_5->addWidget(TextOptionGroup);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        IconOptionGroup = new QGroupBox(page_3);
        IconOptionGroup->setObjectName(QString::fromUtf8("IconOptionGroup"));
        verticalLayout_6 = new QVBoxLayout(IconOptionGroup);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        UseIcons = new QCheckBox(IconOptionGroup);
        UseIcons->setObjectName(QString::fromUtf8("UseIcons"));

        verticalLayout_6->addWidget(UseIcons);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        gridLayout_6 = new QGridLayout();
        gridLayout_6->setObjectName(QString::fromUtf8("gridLayout_6"));
        IconNR = new QPushButton(IconOptionGroup);
        IconNR->setObjectName(QString::fromUtf8("IconNR"));

        gridLayout_6->addWidget(IconNR, 2, 0, 1, 1);

        NiconPrev = new QLabel(IconOptionGroup);
        NiconPrev->setObjectName(QString::fromUtf8("NiconPrev"));
        NiconPrev->setMinimumSize(QSize(25, 25));
        NiconPrev->setMaximumSize(QSize(25, 25));
        NiconPrev->setFrameShape(QFrame::Panel);
        NiconPrev->setFrameShadow(QFrame::Raised);
        NiconPrev->setScaledContents(true);

        gridLayout_6->addWidget(NiconPrev, 1, 0, 1, 1, Qt::AlignHCenter);

        IconN = new QPushButton(IconOptionGroup);
        IconN->setObjectName(QString::fromUtf8("IconN"));

        gridLayout_6->addWidget(IconN, 0, 0, 1, 1);


        horizontalLayout_5->addLayout(gridLayout_6);

        gridLayout_9 = new QGridLayout();
        gridLayout_9->setObjectName(QString::fromUtf8("gridLayout_9"));
        PiconPrev = new QLabel(IconOptionGroup);
        PiconPrev->setObjectName(QString::fromUtf8("PiconPrev"));
        PiconPrev->setMinimumSize(QSize(25, 25));
        PiconPrev->setMaximumSize(QSize(25, 25));
        PiconPrev->setFrameShape(QFrame::Panel);
        PiconPrev->setFrameShadow(QFrame::Raised);
        PiconPrev->setScaledContents(true);

        gridLayout_9->addWidget(PiconPrev, 1, 0, 1, 1, Qt::AlignHCenter);

        IconP = new QPushButton(IconOptionGroup);
        IconP->setObjectName(QString::fromUtf8("IconP"));

        gridLayout_9->addWidget(IconP, 0, 0, 1, 1);

        IconPR = new QPushButton(IconOptionGroup);
        IconPR->setObjectName(QString::fromUtf8("IconPR"));

        gridLayout_9->addWidget(IconPR, 2, 0, 1, 1);


        horizontalLayout_5->addLayout(gridLayout_9);

        gridLayout_11 = new QGridLayout();
        gridLayout_11->setObjectName(QString::fromUtf8("gridLayout_11"));
        IconR = new QPushButton(IconOptionGroup);
        IconR->setObjectName(QString::fromUtf8("IconR"));

        gridLayout_11->addWidget(IconR, 0, 0, 1, 1);

        RiconPrev = new QLabel(IconOptionGroup);
        RiconPrev->setObjectName(QString::fromUtf8("RiconPrev"));
        RiconPrev->setMinimumSize(QSize(25, 25));
        RiconPrev->setMaximumSize(QSize(25, 25));
        RiconPrev->setFrameShape(QFrame::Panel);
        RiconPrev->setFrameShadow(QFrame::Raised);
        RiconPrev->setScaledContents(true);

        gridLayout_11->addWidget(RiconPrev, 1, 0, 1, 1, Qt::AlignHCenter);

        IconRR = new QPushButton(IconOptionGroup);
        IconRR->setObjectName(QString::fromUtf8("IconRR"));

        gridLayout_11->addWidget(IconRR, 2, 0, 1, 1);


        horizontalLayout_5->addLayout(gridLayout_11);


        verticalLayout_6->addLayout(horizontalLayout_5);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        PlaceIcon = new QPushButton(IconOptionGroup);
        PlaceIcon->setObjectName(QString::fromUtf8("PlaceIcon"));

        horizontalLayout_6->addWidget(PlaceIcon);


        verticalLayout_6->addLayout(horizontalLayout_6);


        horizontalLayout_4->addWidget(IconOptionGroup);

        HighlightOptionGroup = new QGroupBox(page_3);
        HighlightOptionGroup->setObjectName(QString::fromUtf8("HighlightOptionGroup"));
        verticalLayout_7 = new QVBoxLayout(HighlightOptionGroup);
        verticalLayout_7->setObjectName(QString::fromUtf8("verticalLayout_7"));
        ComboBox7_2 = new QComboBox(HighlightOptionGroup);
        ComboBox7_2->addItem(QString());
        ComboBox7_2->addItem(QString());
        ComboBox7_2->addItem(QString());
        ComboBox7_2->addItem(QString());
        ComboBox7_2->addItem(QString());
        ComboBox7_2->setObjectName(QString::fromUtf8("ComboBox7_2"));

        verticalLayout_7->addWidget(ComboBox7_2, 0, Qt::AlignTop);


        horizontalLayout_4->addWidget(HighlightOptionGroup);


        verticalLayout_5->addLayout(horizontalLayout_4);

        FramOp->addWidget(page_3);
        page_4 = new QWidget();
        page_4->setObjectName(QString::fromUtf8("page_4"));
        verticalLayout_8 = new QVBoxLayout(page_4);
        verticalLayout_8->setObjectName(QString::fromUtf8("verticalLayout_8"));
        verticalLayout_8->setContentsMargins(0, 0, 0, 0);
        MultiL = new QCheckBox(page_4);
        MultiL->setObjectName(QString::fromUtf8("MultiL"));

        verticalLayout_8->addWidget(MultiL);

        Passwd = new QCheckBox(page_4);
        Passwd->setObjectName(QString::fromUtf8("Passwd"));

        verticalLayout_8->addWidget(Passwd);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        Limit = new QCheckBox(page_4);
        Limit->setObjectName(QString::fromUtf8("Limit"));

        horizontalLayout_7->addWidget(Limit);

        MaxChars = new QSpinBox(page_4);
        MaxChars->setObjectName(QString::fromUtf8("MaxChars"));
        MaxChars->setMinimum(0);
        MaxChars->setMaximum(32768);

        horizontalLayout_7->addWidget(MaxChars);

        label_11 = new QLabel(page_4);
        label_11->setObjectName(QString::fromUtf8("label_11"));

        horizontalLayout_7->addWidget(label_11);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_2);


        verticalLayout_8->addLayout(horizontalLayout_7);

        NoScroll = new QCheckBox(page_4);
        NoScroll->setObjectName(QString::fromUtf8("NoScroll"));

        verticalLayout_8->addWidget(NoScroll);

        NoSpell = new QCheckBox(page_4);
        NoSpell->setObjectName(QString::fromUtf8("NoSpell"));

        verticalLayout_8->addWidget(NoSpell);

        AutoTextSize = new QCheckBox(page_4);
        AutoTextSize->setObjectName(QString::fromUtf8("AutoTextSize"));

        verticalLayout_8->addWidget(AutoTextSize);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_8->addItem(verticalSpacer_5);

        FramOp->addWidget(page_4);
        page_5 = new QWidget();
        page_5->setObjectName(QString::fromUtf8("page_5"));
        verticalLayout_9 = new QVBoxLayout(page_5);
        verticalLayout_9->setObjectName(QString::fromUtf8("verticalLayout_9"));
        verticalLayout_9->setContentsMargins(0, 0, 0, 0);
        horizontalLayout_8 = new QHBoxLayout();
        horizontalLayout_8->setObjectName(QString::fromUtf8("horizontalLayout_8"));
        CText1 = new QLabel(page_5);
        CText1->setObjectName(QString::fromUtf8("CText1"));

        horizontalLayout_8->addWidget(CText1);

        ChkStil = new QComboBox(page_5);
        ChkStil->addItem(QString());
        ChkStil->addItem(QString());
        ChkStil->addItem(QString());
        ChkStil->addItem(QString());
        ChkStil->addItem(QString());
        ChkStil->addItem(QString());
        ChkStil->setObjectName(QString::fromUtf8("ChkStil"));

        horizontalLayout_8->addWidget(ChkStil);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_8->addItem(horizontalSpacer_3);


        verticalLayout_9->addLayout(horizontalLayout_8);

        isChkd = new QCheckBox(page_5);
        isChkd->setObjectName(QString::fromUtf8("isChkd"));

        verticalLayout_9->addWidget(isChkd);

        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_9->addItem(verticalSpacer_4);

        FramOp->addWidget(page_5);
        page_6 = new QWidget();
        page_6->setObjectName(QString::fromUtf8("page_6"));
        verticalLayout_10 = new QVBoxLayout(page_6);
        verticalLayout_10->setObjectName(QString::fromUtf8("verticalLayout_10"));
        verticalLayout_10->setContentsMargins(0, 0, 0, 0);
        CanEdit = new QCheckBox(page_6);
        CanEdit->setObjectName(QString::fromUtf8("CanEdit"));

        verticalLayout_10->addWidget(CanEdit);

        verticalSpacer_6 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_10->addItem(verticalSpacer_6);

        FramOp->addWidget(page_6);

        verticalLayout_4->addWidget(FramOp);

        verticalSpacer_13 = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_13);

        TabWidget2->addTab(tabOptions, QString());
        tabAction = new QWidget();
        tabAction->setObjectName(QString::fromUtf8("tabAction"));
        verticalLayout_11 = new QVBoxLayout(tabAction);
        verticalLayout_11->setObjectName(QString::fromUtf8("verticalLayout_11"));
        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName(QString::fromUtf8("horizontalLayout_9"));
        label_12 = new QLabel(tabAction);
        label_12->setObjectName(QString::fromUtf8("label_12"));

        horizontalLayout_9->addWidget(label_12);

        ActionCombo = new QComboBox(tabAction);
        ActionCombo->addItem(QString());
        ActionCombo->addItem(QString());
        ActionCombo->addItem(QString());
        ActionCombo->addItem(QString());
        ActionCombo->addItem(QString());
        ActionCombo->addItem(QString());
        ActionCombo->addItem(QString());
        ActionCombo->setObjectName(QString::fromUtf8("ActionCombo"));

        horizontalLayout_9->addWidget(ActionCombo);


        verticalLayout_11->addLayout(horizontalLayout_9);

        Fram2 = new QStackedWidget(tabAction);
        Fram2->setObjectName(QString::fromUtf8("Fram2"));
        page_7 = new QWidget();
        page_7->setObjectName(QString::fromUtf8("page_7"));
        verticalLayout_12 = new QVBoxLayout(page_7);
        verticalLayout_12->setObjectName(QString::fromUtf8("verticalLayout_12"));
        verticalLayout_12->setContentsMargins(0, 0, 0, 0);
        frame = new QFrame(page_7);
        frame->setObjectName(QString::fromUtf8("frame"));
        frame->setFrameShape(QFrame::NoFrame);
        frame->setFrameShadow(QFrame::Raised);

        verticalLayout_12->addWidget(frame);

        Fram2->addWidget(page_7);
        page_8 = new QWidget();
        page_8->setObjectName(QString::fromUtf8("page_8"));
        verticalLayout_13 = new QVBoxLayout(page_8);
        verticalLayout_13->setObjectName(QString::fromUtf8("verticalLayout_13"));
        verticalLayout_13->setContentsMargins(0, 0, 0, 0);
        frame_2 = new QFrame(page_8);
        frame_2->setObjectName(QString::fromUtf8("frame_2"));
        frame_2->setFrameShape(QFrame::Box);
        frame_2->setFrameShadow(QFrame::Sunken);
        verticalLayout_14 = new QVBoxLayout(frame_2);
        verticalLayout_14->setObjectName(QString::fromUtf8("verticalLayout_14"));
        horizontalLayout_10 = new QHBoxLayout();
        horizontalLayout_10->setObjectName(QString::fromUtf8("horizontalLayout_10"));
        label_13 = new QLabel(frame_2);
        label_13->setObjectName(QString::fromUtf8("label_13"));

        horizontalLayout_10->addWidget(label_13);

        SelAction = new QComboBox(frame_2);
        SelAction->addItem(QString());
        SelAction->addItem(QString());
        SelAction->addItem(QString());
        SelAction->addItem(QString());
        SelAction->addItem(QString());
        SelAction->addItem(QString());
        SelAction->setObjectName(QString::fromUtf8("SelAction"));

        horizontalLayout_10->addWidget(SelAction);


        verticalLayout_14->addLayout(horizontalLayout_10);

        label_14 = new QLabel(frame_2);
        label_14->setObjectName(QString::fromUtf8("label_14"));

        verticalLayout_14->addWidget(label_14);

        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName(QString::fromUtf8("horizontalLayout_11"));
        EditJava = new QTextEdit(frame_2);
        EditJava->setObjectName(QString::fromUtf8("EditJava"));
        EditJava->setMinimumSize(QSize(0, 50));
        EditJava->setAutoFillBackground(true);
        EditJava->setReadOnly(true);

        horizontalLayout_11->addWidget(EditJava);

        EditJ = new QPushButton(frame_2);
        EditJ->setObjectName(QString::fromUtf8("EditJ"));

        horizontalLayout_11->addWidget(EditJ, 0, Qt::AlignTop);


        verticalLayout_14->addLayout(horizontalLayout_11);


        verticalLayout_13->addWidget(frame_2);

        Fram2->addWidget(page_8);
        page_9 = new QWidget();
        page_9->setObjectName(QString::fromUtf8("page_9"));
        verticalLayout_15 = new QVBoxLayout(page_9);
        verticalLayout_15->setObjectName(QString::fromUtf8("verticalLayout_15"));
        verticalLayout_15->setContentsMargins(0, 0, 0, 0);
        GroupBoxDestination = new QGroupBox(page_9);
        GroupBoxDestination->setObjectName(QString::fromUtf8("GroupBoxDestination"));
        GroupBoxDestinationLayout = new QGridLayout(GroupBoxDestination);
        GroupBoxDestinationLayout->setObjectName(QString::fromUtf8("GroupBoxDestinationLayout"));
        ChFile = new QPushButton(GroupBoxDestination);
        ChFile->setObjectName(QString::fromUtf8("ChFile"));

        GroupBoxDestinationLayout->addWidget(ChFile, 0, 2, 1, 1);

        label_15 = new QLabel(GroupBoxDestination);
        label_15->setObjectName(QString::fromUtf8("label_15"));

        GroupBoxDestinationLayout->addWidget(label_15, 2, 0, 1, 1);

        Destfile = new QLineEdit(GroupBoxDestination);
        Destfile->setObjectName(QString::fromUtf8("Destfile"));
        Destfile->setReadOnly(true);

        GroupBoxDestinationLayout->addWidget(Destfile, 0, 1, 1, 1);

        SpinBox11 = new QSpinBox(GroupBoxDestination);
        SpinBox11->setObjectName(QString::fromUtf8("SpinBox11"));
        SpinBox11->setMinimum(1);
        SpinBox11->setMaximum(1000);

        GroupBoxDestinationLayout->addWidget(SpinBox11, 2, 1, 1, 1);

        label_17 = new QLabel(GroupBoxDestination);
        label_17->setObjectName(QString::fromUtf8("label_17"));

        GroupBoxDestinationLayout->addWidget(label_17, 4, 0, 1, 1);

        useAbsolute = new QCheckBox(GroupBoxDestination);
        useAbsolute->setObjectName(QString::fromUtf8("useAbsolute"));

        GroupBoxDestinationLayout->addWidget(useAbsolute, 1, 0, 1, 3);

        SpinBox31 = new QSpinBox(GroupBoxDestination);
        SpinBox31->setObjectName(QString::fromUtf8("SpinBox31"));

        GroupBoxDestinationLayout->addWidget(SpinBox31, 4, 1, 1, 1);

        LExtern = new QCheckBox(GroupBoxDestination);
        LExtern->setObjectName(QString::fromUtf8("LExtern"));

        GroupBoxDestinationLayout->addWidget(LExtern, 0, 0, 1, 1);

        label_16 = new QLabel(GroupBoxDestination);
        label_16->setObjectName(QString::fromUtf8("label_16"));

        GroupBoxDestinationLayout->addWidget(label_16, 3, 0, 1, 1);

        SpinBox21 = new QSpinBox(GroupBoxDestination);
        SpinBox21->setObjectName(QString::fromUtf8("SpinBox21"));

        GroupBoxDestinationLayout->addWidget(SpinBox21, 3, 1, 1, 1);

        verticalSpacer_12 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        GroupBoxDestinationLayout->addItem(verticalSpacer_12, 5, 1, 1, 1);


        verticalLayout_15->addWidget(GroupBoxDestination);

        Fram2->addWidget(page_9);
        page_10 = new QWidget();
        page_10->setObjectName(QString::fromUtf8("page_10"));
        verticalLayout_16 = new QVBoxLayout(page_10);
        verticalLayout_16->setObjectName(QString::fromUtf8("verticalLayout_16"));
        verticalLayout_16->setContentsMargins(0, 0, 0, 0);
        frame_3 = new QFrame(page_10);
        frame_3->setObjectName(QString::fromUtf8("frame_3"));
        frame_3->setFrameShape(QFrame::Box);
        frame_3->setFrameShadow(QFrame::Sunken);
        verticalLayout_17 = new QVBoxLayout(frame_3);
        verticalLayout_17->setObjectName(QString::fromUtf8("verticalLayout_17"));
        label_18 = new QLabel(frame_3);
        label_18->setObjectName(QString::fromUtf8("label_18"));

        verticalLayout_17->addWidget(label_18);

        SubURL = new QLineEdit(frame_3);
        SubURL->setObjectName(QString::fromUtf8("SubURL"));

        verticalLayout_17->addWidget(SubURL);

        label_19 = new QLabel(frame_3);
        label_19->setObjectName(QString::fromUtf8("label_19"));

        verticalLayout_17->addWidget(label_19);

        SelAsHtml = new QComboBox(frame_3);
        SelAsHtml->addItem(QString());
        SelAsHtml->addItem(QString());
        SelAsHtml->addItem(QString());
        SelAsHtml->addItem(QString());
        SelAsHtml->setObjectName(QString::fromUtf8("SelAsHtml"));

        verticalLayout_17->addWidget(SelAsHtml);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_17->addItem(verticalSpacer);


        verticalLayout_16->addWidget(frame_3);

        Fram2->addWidget(page_10);
        page_11 = new QWidget();
        page_11->setObjectName(QString::fromUtf8("page_11"));
        verticalLayout_18 = new QVBoxLayout(page_11);
        verticalLayout_18->setObjectName(QString::fromUtf8("verticalLayout_18"));
        verticalLayout_18->setContentsMargins(0, 0, 0, 0);
        frame_4 = new QFrame(page_11);
        frame_4->setObjectName(QString::fromUtf8("frame_4"));
        frame_4->setFrameShape(QFrame::Box);
        frame_4->setFrameShadow(QFrame::Sunken);
        verticalLayout_19 = new QVBoxLayout(frame_4);
        verticalLayout_19->setObjectName(QString::fromUtf8("verticalLayout_19"));
        label_20 = new QLabel(frame_4);
        label_20->setObjectName(QString::fromUtf8("label_20"));

        verticalLayout_19->addWidget(label_20);

        SubURLa = new QLineEdit(frame_4);
        SubURLa->setObjectName(QString::fromUtf8("SubURLa"));

        verticalLayout_19->addWidget(SubURLa);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_19->addItem(verticalSpacer_2);


        verticalLayout_18->addWidget(frame_4);

        Fram2->addWidget(page_11);
        page_12 = new QWidget();
        page_12->setObjectName(QString::fromUtf8("page_12"));
        verticalLayout_20 = new QVBoxLayout(page_12);
        verticalLayout_20->setObjectName(QString::fromUtf8("verticalLayout_20"));
        verticalLayout_20->setContentsMargins(0, 0, 0, 0);
        frame_5 = new QFrame(page_12);
        frame_5->setObjectName(QString::fromUtf8("frame_5"));
        frame_5->setFrameShape(QFrame::Box);
        frame_5->setFrameShadow(QFrame::Sunken);
        verticalLayout_21 = new QVBoxLayout(frame_5);
        verticalLayout_21->setObjectName(QString::fromUtf8("verticalLayout_21"));
        label_21 = new QLabel(frame_5);
        label_21->setObjectName(QString::fromUtf8("label_21"));

        verticalLayout_21->addWidget(label_21);

        nameActionCombo = new QComboBox(frame_5);
        nameActionCombo->setObjectName(QString::fromUtf8("nameActionCombo"));

        verticalLayout_21->addWidget(nameActionCombo);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_21->addItem(verticalSpacer_3);


        verticalLayout_20->addWidget(frame_5);

        Fram2->addWidget(page_12);

        verticalLayout_11->addWidget(Fram2);

        TabWidget2->addTab(tabAction, QString());
        tabFormat = new QWidget();
        tabFormat->setObjectName(QString::fromUtf8("tabFormat"));
        verticalLayout_22 = new QVBoxLayout(tabFormat);
        verticalLayout_22->setObjectName(QString::fromUtf8("verticalLayout_22"));
        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName(QString::fromUtf8("horizontalLayout_12"));
        label_22 = new QLabel(tabFormat);
        label_22->setObjectName(QString::fromUtf8("label_22"));

        horizontalLayout_12->addWidget(label_22);

        TxFormat = new QComboBox(tabFormat);
        TxFormat->addItem(QString());
        TxFormat->addItem(QString());
        TxFormat->addItem(QString());
        TxFormat->addItem(QString());
        TxFormat->addItem(QString());
        TxFormat->addItem(QString());
        TxFormat->setObjectName(QString::fromUtf8("TxFormat"));

        horizontalLayout_12->addWidget(TxFormat);


        verticalLayout_22->addLayout(horizontalLayout_12);

        FoFram = new QStackedWidget(tabFormat);
        FoFram->setObjectName(QString::fromUtf8("FoFram"));
        page_13 = new QWidget();
        page_13->setObjectName(QString::fromUtf8("page_13"));
        verticalLayout_23 = new QVBoxLayout(page_13);
        verticalLayout_23->setObjectName(QString::fromUtf8("verticalLayout_23"));
        verticalLayout_23->setContentsMargins(0, 0, 0, 0);
        frame_6 = new QFrame(page_13);
        frame_6->setObjectName(QString::fromUtf8("frame_6"));
        frame_6->setFrameShape(QFrame::NoFrame);
        frame_6->setFrameShadow(QFrame::Plain);

        verticalLayout_23->addWidget(frame_6);

        FoFram->addWidget(page_13);
        page_14 = new QWidget();
        page_14->setObjectName(QString::fromUtf8("page_14"));
        verticalLayout_24 = new QVBoxLayout(page_14);
        verticalLayout_24->setObjectName(QString::fromUtf8("verticalLayout_24"));
        verticalLayout_24->setContentsMargins(0, 0, 0, 0);
        NumbGroup = new QGroupBox(page_14);
        NumbGroup->setObjectName(QString::fromUtf8("NumbGroup"));
        verticalLayout_25 = new QVBoxLayout(NumbGroup);
        verticalLayout_25->setObjectName(QString::fromUtf8("verticalLayout_25"));
        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setObjectName(QString::fromUtf8("horizontalLayout_13"));
        label_23 = new QLabel(NumbGroup);
        label_23->setObjectName(QString::fromUtf8("label_23"));

        horizontalLayout_13->addWidget(label_23);

        Decim = new QSpinBox(NumbGroup);
        Decim->setObjectName(QString::fromUtf8("Decim"));
        Decim->setMinimum(0);
        Decim->setMaximum(12);

        horizontalLayout_13->addWidget(Decim);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_4);


        verticalLayout_25->addLayout(horizontalLayout_13);

        horizontalLayout_14 = new QHBoxLayout();
        horizontalLayout_14->setObjectName(QString::fromUtf8("horizontalLayout_14"));
        UseCurr = new QCheckBox(NumbGroup);
        UseCurr->setObjectName(QString::fromUtf8("UseCurr"));

        horizontalLayout_14->addWidget(UseCurr);

        CurSym = new QLineEdit(NumbGroup);
        CurSym->setObjectName(QString::fromUtf8("CurSym"));

        horizontalLayout_14->addWidget(CurSym);


        verticalLayout_25->addLayout(horizontalLayout_14);

        PreCurr = new QCheckBox(NumbGroup);
        PreCurr->setObjectName(QString::fromUtf8("PreCurr"));

        verticalLayout_25->addWidget(PreCurr);

        NumGroup2 = new QGroupBox(NumbGroup);
        NumGroup2->setObjectName(QString::fromUtf8("NumGroup2"));
        gridLayout_7 = new QGridLayout(NumGroup2);
        gridLayout_7->setObjectName(QString::fromUtf8("gridLayout_7"));
        Format0 = new QRadioButton(NumGroup2);
        Format0->setObjectName(QString::fromUtf8("Format0"));
        Format0->setText(QString::fromUtf8("9,999.99"));
        Format0->setChecked(true);

        gridLayout_7->addWidget(Format0, 0, 0, 1, 1);

        Format2 = new QRadioButton(NumGroup2);
        Format2->setObjectName(QString::fromUtf8("Format2"));
        Format2->setText(QString::fromUtf8("9.999,99"));

        gridLayout_7->addWidget(Format2, 0, 1, 1, 1);

        Format1 = new QRadioButton(NumGroup2);
        Format1->setObjectName(QString::fromUtf8("Format1"));
        Format1->setText(QString::fromUtf8("9999.99"));

        gridLayout_7->addWidget(Format1, 1, 0, 1, 1);

        Format3 = new QRadioButton(NumGroup2);
        Format3->setObjectName(QString::fromUtf8("Format3"));
        Format3->setText(QString::fromUtf8("9999,99"));

        gridLayout_7->addWidget(Format3, 1, 1, 1, 1);


        verticalLayout_25->addWidget(NumGroup2);

        verticalSpacer_10 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_25->addItem(verticalSpacer_10);


        verticalLayout_24->addWidget(NumbGroup);

        FoFram->addWidget(page_14);
        page_15 = new QWidget();
        page_15->setObjectName(QString::fromUtf8("page_15"));
        verticalLayout_26 = new QVBoxLayout(page_15);
        verticalLayout_26->setObjectName(QString::fromUtf8("verticalLayout_26"));
        verticalLayout_26->setContentsMargins(0, 0, 0, 0);
        PercGroup = new QGroupBox(page_15);
        PercGroup->setObjectName(QString::fromUtf8("PercGroup"));
        verticalLayout_27 = new QVBoxLayout(PercGroup);
        verticalLayout_27->setObjectName(QString::fromUtf8("verticalLayout_27"));
        horizontalLayout_15 = new QHBoxLayout();
        horizontalLayout_15->setObjectName(QString::fromUtf8("horizontalLayout_15"));
        label_24 = new QLabel(PercGroup);
        label_24->setObjectName(QString::fromUtf8("label_24"));

        horizontalLayout_15->addWidget(label_24);

        Decim2 = new QSpinBox(PercGroup);
        Decim2->setObjectName(QString::fromUtf8("Decim2"));
        Decim2->setMinimum(0);
        Decim2->setMaximum(12);

        horizontalLayout_15->addWidget(Decim2);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_15->addItem(horizontalSpacer_5);


        verticalLayout_27->addLayout(horizontalLayout_15);

        NumGroup2a = new QGroupBox(PercGroup);
        NumGroup2a->setObjectName(QString::fromUtf8("NumGroup2a"));
        gridLayout_8 = new QGridLayout(NumGroup2a);
        gridLayout_8->setObjectName(QString::fromUtf8("gridLayout_8"));
        Format0a = new QRadioButton(NumGroup2a);
        Format0a->setObjectName(QString::fromUtf8("Format0a"));
        Format0a->setText(QString::fromUtf8("9,999.99"));
        Format0a->setChecked(true);

        gridLayout_8->addWidget(Format0a, 0, 0, 1, 1);

        Format2a = new QRadioButton(NumGroup2a);
        Format2a->setObjectName(QString::fromUtf8("Format2a"));
        Format2a->setText(QString::fromUtf8("9.999,99"));

        gridLayout_8->addWidget(Format2a, 0, 1, 1, 1);

        Format1a = new QRadioButton(NumGroup2a);
        Format1a->setObjectName(QString::fromUtf8("Format1a"));
        Format1a->setText(QString::fromUtf8("9999.99"));

        gridLayout_8->addWidget(Format1a, 1, 0, 1, 1);

        Format3a = new QRadioButton(NumGroup2a);
        Format3a->setObjectName(QString::fromUtf8("Format3a"));
        Format3a->setText(QString::fromUtf8("9999,99"));

        gridLayout_8->addWidget(Format3a, 1, 1, 1, 1);


        verticalLayout_27->addWidget(NumGroup2a);

        verticalSpacer_7 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_27->addItem(verticalSpacer_7);


        verticalLayout_26->addWidget(PercGroup);

        FoFram->addWidget(page_15);
        page_16 = new QWidget();
        page_16->setObjectName(QString::fromUtf8("page_16"));
        verticalLayout_28 = new QVBoxLayout(page_16);
        verticalLayout_28->setObjectName(QString::fromUtf8("verticalLayout_28"));
        verticalLayout_28->setContentsMargins(0, 0, 0, 0);
        DateGroup = new QGroupBox(page_16);
        DateGroup->setObjectName(QString::fromUtf8("DateGroup"));
        verticalLayout_29 = new QVBoxLayout(DateGroup);
        verticalLayout_29->setObjectName(QString::fromUtf8("verticalLayout_29"));
        horizontalLayout_16 = new QHBoxLayout();
        horizontalLayout_16->setObjectName(QString::fromUtf8("horizontalLayout_16"));
        Format0c = new QComboBox(DateGroup);
        Format0c->addItem(QString::fromUtf8("m/d"));
        Format0c->addItem(QString::fromUtf8("m/d/yy"));
        Format0c->addItem(QString::fromUtf8("mm/dd/yy"));
        Format0c->addItem(QString::fromUtf8("mm/yy"));
        Format0c->addItem(QString::fromUtf8("d-mmm"));
        Format0c->addItem(QString::fromUtf8("d-mmm-yy"));
        Format0c->addItem(QString::fromUtf8("dd-mmm-yy"));
        Format0c->addItem(QString::fromUtf8("dd/mm/yy"));
        Format0c->addItem(QString::fromUtf8("dd/mm/yyyy"));
        Format0c->addItem(QString::fromUtf8("dd.mm.yyyy"));
        Format0c->addItem(QString::fromUtf8("yy-mm-dd"));
        Format0c->addItem(QString::fromUtf8("mmm-yy"));
        Format0c->addItem(QString::fromUtf8("mmmm-yy"));
        Format0c->addItem(QString::fromUtf8("mmm d, yyyy"));
        Format0c->addItem(QString::fromUtf8("mmmm d, yyyy"));
        Format0c->addItem(QString::fromUtf8("m/d/yy h:MM tt"));
        Format0c->addItem(QString::fromUtf8("m/d/yy HH:MM"));
        Format0c->addItem(QString::fromUtf8("yyyy-mm-dd"));
        Format0c->setObjectName(QString::fromUtf8("Format0c"));
        Format0c->setCurrentText(QString::fromUtf8("m/d"));

        horizontalLayout_16->addWidget(Format0c);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_16->addItem(horizontalSpacer_6);


        verticalLayout_29->addLayout(horizontalLayout_16);

        TextDa1 = new QLabel(DateGroup);
        TextDa1->setObjectName(QString::fromUtf8("TextDa1"));

        verticalLayout_29->addWidget(TextDa1);

        verticalSpacer_8 = new QSpacerItem(20, 188, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_29->addItem(verticalSpacer_8);


        verticalLayout_28->addWidget(DateGroup);

        FoFram->addWidget(page_16);
        page_17 = new QWidget();
        page_17->setObjectName(QString::fromUtf8("page_17"));
        verticalLayout_30 = new QVBoxLayout(page_17);
        verticalLayout_30->setObjectName(QString::fromUtf8("verticalLayout_30"));
        verticalLayout_30->setContentsMargins(0, 0, 0, 0);
        TimeGroup = new QGroupBox(page_17);
        TimeGroup->setObjectName(QString::fromUtf8("TimeGroup"));
        verticalLayout_31 = new QVBoxLayout(TimeGroup);
        verticalLayout_31->setObjectName(QString::fromUtf8("verticalLayout_31"));
        Format0b = new QRadioButton(TimeGroup);
        Format0b->setObjectName(QString::fromUtf8("Format0b"));
        Format0b->setText(QString::fromUtf8("HH:MM"));
        Format0b->setChecked(true);

        verticalLayout_31->addWidget(Format0b);

        Format1b = new QRadioButton(TimeGroup);
        Format1b->setObjectName(QString::fromUtf8("Format1b"));
        Format1b->setText(QString::fromUtf8("h:MM tt"));

        verticalLayout_31->addWidget(Format1b);

        Format2b = new QRadioButton(TimeGroup);
        Format2b->setObjectName(QString::fromUtf8("Format2b"));
        Format2b->setText(QString::fromUtf8("HH:MM:ss"));

        verticalLayout_31->addWidget(Format2b);

        Format3b = new QRadioButton(TimeGroup);
        Format3b->setObjectName(QString::fromUtf8("Format3b"));
        Format3b->setText(QString::fromUtf8("h:MM:ss tt"));

        verticalLayout_31->addWidget(Format3b);

        verticalSpacer_9 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_31->addItem(verticalSpacer_9);


        verticalLayout_30->addWidget(TimeGroup);

        FoFram->addWidget(page_17);
        page_18 = new QWidget();
        page_18->setObjectName(QString::fromUtf8("page_18"));
        verticalLayout_32 = new QVBoxLayout(page_18);
        verticalLayout_32->setObjectName(QString::fromUtf8("verticalLayout_32"));
        verticalLayout_32->setContentsMargins(0, 0, 0, 0);
        GroupCust = new QGroupBox(page_18);
        GroupCust->setObjectName(QString::fromUtf8("GroupCust"));
        verticalLayout_33 = new QVBoxLayout(GroupCust);
        verticalLayout_33->setObjectName(QString::fromUtf8("verticalLayout_33"));
        horizontalLayout_17 = new QHBoxLayout();
        horizontalLayout_17->setObjectName(QString::fromUtf8("horizontalLayout_17"));
        label_25 = new QLabel(GroupCust);
        label_25->setObjectName(QString::fromUtf8("label_25"));

        horizontalLayout_17->addWidget(label_25);

        EditFormat = new QPushButton(GroupCust);
        EditFormat->setObjectName(QString::fromUtf8("EditFormat"));

        horizontalLayout_17->addWidget(EditFormat);


        verticalLayout_33->addLayout(horizontalLayout_17);

        FormatScript = new QTextEdit(GroupCust);
        FormatScript->setObjectName(QString::fromUtf8("FormatScript"));
        FormatScript->setMaximumSize(QSize(32000, 50));
        FormatScript->setAutoFillBackground(true);
        FormatScript->setReadOnly(true);

        verticalLayout_33->addWidget(FormatScript);

        horizontalLayout_18 = new QHBoxLayout();
        horizontalLayout_18->setObjectName(QString::fromUtf8("horizontalLayout_18"));
        label_26 = new QLabel(GroupCust);
        label_26->setObjectName(QString::fromUtf8("label_26"));

        horizontalLayout_18->addWidget(label_26);

        EditKeystr = new QPushButton(GroupCust);
        EditKeystr->setObjectName(QString::fromUtf8("EditKeystr"));

        horizontalLayout_18->addWidget(EditKeystr);


        verticalLayout_33->addLayout(horizontalLayout_18);

        KeyScript = new QTextEdit(GroupCust);
        KeyScript->setObjectName(QString::fromUtf8("KeyScript"));
        KeyScript->setMaximumSize(QSize(32000, 50));
        KeyScript->setAutoFillBackground(true);
        KeyScript->setReadOnly(true);

        verticalLayout_33->addWidget(KeyScript);

        verticalSpacer_11 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_33->addItem(verticalSpacer_11);


        verticalLayout_32->addWidget(GroupCust);

        FoFram->addWidget(page_18);

        verticalLayout_22->addWidget(FoFram);

        TabWidget2->addTab(tabFormat, QString());
        tabValidate = new QWidget();
        tabValidate->setObjectName(QString::fromUtf8("tabValidate"));
        verticalLayout_34 = new QVBoxLayout(tabValidate);
        verticalLayout_34->setObjectName(QString::fromUtf8("verticalLayout_34"));
        NoValid = new QRadioButton(tabValidate);
        NoValid->setObjectName(QString::fromUtf8("NoValid"));
        NoValid->setChecked(true);

        verticalLayout_34->addWidget(NoValid);

        gridLayout_10 = new QGridLayout();
        gridLayout_10->setObjectName(QString::fromUtf8("gridLayout_10"));
        SimpleValid = new QRadioButton(tabValidate);
        SimpleValid->setObjectName(QString::fromUtf8("SimpleValid"));

        gridLayout_10->addWidget(SimpleValid, 0, 0, 1, 1);

        label_27 = new QLabel(tabValidate);
        label_27->setObjectName(QString::fromUtf8("label_27"));
        label_27->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout_10->addWidget(label_27, 1, 0, 1, 1);

        MinValid = new QLineEdit(tabValidate);
        MinValid->setObjectName(QString::fromUtf8("MinValid"));

        gridLayout_10->addWidget(MinValid, 0, 1, 1, 1);

        MaxValid = new QLineEdit(tabValidate);
        MaxValid->setObjectName(QString::fromUtf8("MaxValid"));

        gridLayout_10->addWidget(MaxValid, 1, 1, 1, 1);


        verticalLayout_34->addLayout(gridLayout_10);

        CustomValid = new QRadioButton(tabValidate);
        CustomValid->setObjectName(QString::fromUtf8("CustomValid"));

        verticalLayout_34->addWidget(CustomValid);

        horizontalLayout_19 = new QHBoxLayout();
        horizontalLayout_19->setObjectName(QString::fromUtf8("horizontalLayout_19"));
        ValidScript = new QTextEdit(tabValidate);
        ValidScript->setObjectName(QString::fromUtf8("ValidScript"));
        ValidScript->setMinimumSize(QSize(0, 50));

        horizontalLayout_19->addWidget(ValidScript);

        EditValScript = new QPushButton(tabValidate);
        EditValScript->setObjectName(QString::fromUtf8("EditValScript"));
        EditValScript->setAutoDefault(false);

        horizontalLayout_19->addWidget(EditValScript);


        verticalLayout_34->addLayout(horizontalLayout_19);

        TabWidget2->addTab(tabValidate, QString());
        tabCalculate = new QWidget();
        tabCalculate->setObjectName(QString::fromUtf8("tabCalculate"));
        verticalLayout_36 = new QVBoxLayout(tabCalculate);
        verticalLayout_36->setObjectName(QString::fromUtf8("verticalLayout_36"));
        NoCalc = new QRadioButton(tabCalculate);
        NoCalc->setObjectName(QString::fromUtf8("NoCalc"));
        NoCalc->setChecked(true);

        verticalLayout_36->addWidget(NoCalc);

        horizontalLayout_20 = new QHBoxLayout();
        horizontalLayout_20->setObjectName(QString::fromUtf8("horizontalLayout_20"));
        SimpleCalc = new QRadioButton(tabCalculate);
        SimpleCalc->setObjectName(QString::fromUtf8("SimpleCalc"));

        horizontalLayout_20->addWidget(SimpleCalc);

        CalcArt = new QComboBox(tabCalculate);
        CalcArt->addItem(QString());
        CalcArt->addItem(QString());
        CalcArt->addItem(QString());
        CalcArt->addItem(QString());
        CalcArt->addItem(QString());
        CalcArt->setObjectName(QString::fromUtf8("CalcArt"));

        horizontalLayout_20->addWidget(CalcArt);

        label_28 = new QLabel(tabCalculate);
        label_28->setObjectName(QString::fromUtf8("label_28"));

        horizontalLayout_20->addWidget(label_28);


        verticalLayout_36->addLayout(horizontalLayout_20);

        horizontalLayout_21 = new QHBoxLayout();
        horizontalLayout_21->setObjectName(QString::fromUtf8("horizontalLayout_21"));
        CalcFields = new QLineEdit(tabCalculate);
        CalcFields->setObjectName(QString::fromUtf8("CalcFields"));

        horizontalLayout_21->addWidget(CalcFields);

        SeField = new QPushButton(tabCalculate);
        SeField->setObjectName(QString::fromUtf8("SeField"));

        horizontalLayout_21->addWidget(SeField);


        verticalLayout_36->addLayout(horizontalLayout_21);

        CustomCalc = new QRadioButton(tabCalculate);
        CustomCalc->setObjectName(QString::fromUtf8("CustomCalc"));

        verticalLayout_36->addWidget(CustomCalc);

        horizontalLayout_22 = new QHBoxLayout();
        horizontalLayout_22->setObjectName(QString::fromUtf8("horizontalLayout_22"));
        CalcScript = new QTextEdit(tabCalculate);
        CalcScript->setObjectName(QString::fromUtf8("CalcScript"));
        CalcScript->setMinimumSize(QSize(0, 50));
        CalcScript->setFocusPolicy(Qt::NoFocus);
        CalcScript->setAutoFillBackground(true);
        CalcScript->setReadOnly(true);

        horizontalLayout_22->addWidget(CalcScript);

        EditCalc = new QPushButton(tabCalculate);
        EditCalc->setObjectName(QString::fromUtf8("EditCalc"));

        horizontalLayout_22->addWidget(EditCalc);


        verticalLayout_36->addLayout(horizontalLayout_22);

        TabWidget2->addTab(tabCalculate, QString());

        gridLayout->addWidget(TabWidget2, 2, 0, 1, 2);

        Name = new NameWidget(PropertiesGroup);
        Name->setObjectName(QString::fromUtf8("Name"));

        gridLayout->addWidget(Name, 0, 1, 1, 1);

        label_3 = new QLabel(PropertiesGroup);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 1, 0, 1, 1);


        verticalLayout_2->addWidget(PropertiesGroup);

        Fram->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName(QString::fromUtf8("page_2"));
        Fram->addWidget(page_2);

        verticalLayout->addWidget(Fram);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        PushButton1 = new QPushButton(ScAnnot);
        PushButton1->setObjectName(QString::fromUtf8("PushButton1"));

        horizontalLayout_2->addWidget(PushButton1);

        PushButton2 = new QPushButton(ScAnnot);
        PushButton2->setObjectName(QString::fromUtf8("PushButton2"));

        horizontalLayout_2->addWidget(PushButton2);


        verticalLayout->addLayout(horizontalLayout_2);


        retranslateUi(ScAnnot);

        Fram->setCurrentIndex(0);
        TabWidget2->setCurrentIndex(0);
        FramOp->setCurrentIndex(0);
        Fram2->setCurrentIndex(0);
        FoFram->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(ScAnnot);
    } // setupUi

    void retranslateUi(QDialog *ScAnnot)
    {
        ScAnnot->setWindowTitle(QCoreApplication::translate("ScAnnot", "Field Properties", nullptr));
        label->setText(QCoreApplication::translate("ScAnnot", "Type:", nullptr));
        ComboBox1->setItemText(0, QCoreApplication::translate("ScAnnot", "Button", nullptr));
        ComboBox1->setItemText(1, QCoreApplication::translate("ScAnnot", "Text Field", nullptr));
        ComboBox1->setItemText(2, QCoreApplication::translate("ScAnnot", "Check Box", nullptr));
        ComboBox1->setItemText(3, QCoreApplication::translate("ScAnnot", "Combo Box", nullptr));
        ComboBox1->setItemText(4, QCoreApplication::translate("ScAnnot", "List Box", nullptr));
        ComboBox1->setItemText(5, QCoreApplication::translate("ScAnnot", "Radio Button", nullptr));

        PropertiesGroup->setTitle(QCoreApplication::translate("ScAnnot", "Properties", nullptr));
        label_2->setText(QCoreApplication::translate("ScAnnot", "Name:", nullptr));
        TextGroup->setTitle(QCoreApplication::translate("ScAnnot", "Text", nullptr));
        Schrift->setItemText(0, QCoreApplication::translate("ScAnnot", "Courier", nullptr));
        Schrift->setItemText(1, QCoreApplication::translate("ScAnnot", "Courier Bold", nullptr));
        Schrift->setItemText(2, QCoreApplication::translate("ScAnnot", "Courier Italic", nullptr));
        Schrift->setItemText(3, QCoreApplication::translate("ScAnnot", "Courier Bold Italic", nullptr));
        Schrift->setItemText(4, QCoreApplication::translate("ScAnnot", "Helvetica", nullptr));
        Schrift->setItemText(5, QCoreApplication::translate("ScAnnot", "Helvetica Bold", nullptr));
        Schrift->setItemText(6, QCoreApplication::translate("ScAnnot", "Helvetica Italic", nullptr));
        Schrift->setItemText(7, QCoreApplication::translate("ScAnnot", "Helvetica Bold Italic", nullptr));
        Schrift->setItemText(8, QCoreApplication::translate("ScAnnot", "Times", nullptr));
        Schrift->setItemText(9, QCoreApplication::translate("ScAnnot", "Times Bold", nullptr));
        Schrift->setItemText(10, QCoreApplication::translate("ScAnnot", "Times Italic", nullptr));
        Schrift->setItemText(11, QCoreApplication::translate("ScAnnot", "Times Bold Italic", nullptr));
        Schrift->setItemText(12, QCoreApplication::translate("ScAnnot", "Zapf Dingbats", nullptr));
        Schrift->setItemText(13, QCoreApplication::translate("ScAnnot", "Symbols", nullptr));

        label_4->setText(QCoreApplication::translate("ScAnnot", "Font for use with PDF 1.3:", nullptr));
        BorderGroup->setTitle(QCoreApplication::translate("ScAnnot", "Border", nullptr));
        label_5->setText(QCoreApplication::translate("ScAnnot", "Color:", nullptr));
        BorderW->setItemText(0, QCoreApplication::translate("ScAnnot", "None", nullptr));
        BorderW->setItemText(1, QCoreApplication::translate("ScAnnot", "Thin", nullptr));
        BorderW->setItemText(2, QCoreApplication::translate("ScAnnot", "Normal", nullptr));
        BorderW->setItemText(3, QCoreApplication::translate("ScAnnot", "Wide", nullptr));

        label_6->setText(QCoreApplication::translate("ScAnnot", "Width:", nullptr));
        label_7->setText(QCoreApplication::translate("ScAnnot", "Style:", nullptr));
        BorderS->setItemText(0, QCoreApplication::translate("ScAnnot", "Solid", nullptr));
        BorderS->setItemText(1, QCoreApplication::translate("ScAnnot", "Dashed", nullptr));
        BorderS->setItemText(2, QCoreApplication::translate("ScAnnot", "Underline", nullptr));
        BorderS->setItemText(3, QCoreApplication::translate("ScAnnot", "Beveled", nullptr));
        BorderS->setItemText(4, QCoreApplication::translate("ScAnnot", "Inset", nullptr));

        OtherGroup->setTitle(QCoreApplication::translate("ScAnnot", "Other", nullptr));
        label_8->setText(QCoreApplication::translate("ScAnnot", "Visibility:", nullptr));
        Visib->setItemText(0, QCoreApplication::translate("ScAnnot", "Visible", nullptr));
        Visib->setItemText(1, QCoreApplication::translate("ScAnnot", "Hidden", nullptr));
        Visib->setItemText(2, QCoreApplication::translate("ScAnnot", "No Print", nullptr));
        Visib->setItemText(3, QCoreApplication::translate("ScAnnot", "No View", nullptr));

        NoExport->setText(QCoreApplication::translate("ScAnnot", "Do Not Export Value", nullptr));
        Required->setText(QCoreApplication::translate("ScAnnot", "Required", nullptr));
        ReadOnly->setText(QCoreApplication::translate("ScAnnot", "Read Only", nullptr));
        TabWidget2->setTabText(TabWidget2->indexOf(tabAppearance), QCoreApplication::translate("ScAnnot", "Appearance", nullptr));
        TextOptionGroup->setTitle(QCoreApplication::translate("ScAnnot", "Text", nullptr));
        label_9->setText(QCoreApplication::translate("ScAnnot", "Text for Button Down", nullptr));
        label_10->setText(QCoreApplication::translate("ScAnnot", "Text for Roll Over", nullptr));
        IconOptionGroup->setTitle(QCoreApplication::translate("ScAnnot", "Icons", nullptr));
        UseIcons->setText(QCoreApplication::translate("ScAnnot", "Use Icons", nullptr));
        IconNR->setText(QCoreApplication::translate("ScAnnot", "Remove", nullptr));
        NiconPrev->setText(QString());
        IconN->setText(QCoreApplication::translate("ScAnnot", "Normal", nullptr));
        PiconPrev->setText(QString());
        IconP->setText(QCoreApplication::translate("ScAnnot", "Pressed", nullptr));
        IconPR->setText(QCoreApplication::translate("ScAnnot", "Remove", nullptr));
        IconR->setText(QCoreApplication::translate("ScAnnot", "Roll Over", nullptr));
        RiconPrev->setText(QString());
        IconRR->setText(QCoreApplication::translate("ScAnnot", "Remove", nullptr));
        PlaceIcon->setText(QCoreApplication::translate("ScAnnot", "Icon Placement...", nullptr));
        HighlightOptionGroup->setTitle(QCoreApplication::translate("ScAnnot", "Highlight", nullptr));
        ComboBox7_2->setItemText(0, QCoreApplication::translate("ScAnnot", "None", nullptr));
        ComboBox7_2->setItemText(1, QCoreApplication::translate("ScAnnot", "Highlight", nullptr));
        ComboBox7_2->setItemText(2, QCoreApplication::translate("ScAnnot", "Invert", nullptr));
        ComboBox7_2->setItemText(3, QCoreApplication::translate("ScAnnot", "Outlined", nullptr));
        ComboBox7_2->setItemText(4, QCoreApplication::translate("ScAnnot", "Push", nullptr));

        MultiL->setText(QCoreApplication::translate("ScAnnot", "Multi-Line", nullptr));
        Passwd->setText(QCoreApplication::translate("ScAnnot", "Password", nullptr));
        Limit->setText(QCoreApplication::translate("ScAnnot", "Limit of", nullptr));
        label_11->setText(QCoreApplication::translate("ScAnnot", "Characters", nullptr));
        NoScroll->setText(QCoreApplication::translate("ScAnnot", "Do Not Scroll", nullptr));
        NoSpell->setText(QCoreApplication::translate("ScAnnot", "Do Not Spell Check", nullptr));
        AutoTextSize->setText(QCoreApplication::translate("ScAnnot", "Auto Text Size", nullptr));
        CText1->setText(QCoreApplication::translate("ScAnnot", "Check Style:", nullptr));
        ChkStil->setItemText(0, QCoreApplication::translate("ScAnnot", "Check", nullptr));
        ChkStil->setItemText(1, QCoreApplication::translate("ScAnnot", "Cross", nullptr));
        ChkStil->setItemText(2, QCoreApplication::translate("ScAnnot", "Diamond", nullptr));
        ChkStil->setItemText(3, QCoreApplication::translate("ScAnnot", "Circle", nullptr));
        ChkStil->setItemText(4, QCoreApplication::translate("ScAnnot", "Star", nullptr));
        ChkStil->setItemText(5, QCoreApplication::translate("ScAnnot", "Square", nullptr));

        isChkd->setText(QCoreApplication::translate("ScAnnot", "Default is Checked", nullptr));
        CanEdit->setText(QCoreApplication::translate("ScAnnot", "Editable", nullptr));
        TabWidget2->setTabText(TabWidget2->indexOf(tabOptions), QCoreApplication::translate("ScAnnot", "Options", nullptr));
        label_12->setText(QCoreApplication::translate("ScAnnot", "Type:", nullptr));
        ActionCombo->setItemText(0, QCoreApplication::translate("ScAnnot", "None", "action"));
        ActionCombo->setItemText(1, QCoreApplication::translate("ScAnnot", "JavaScript", nullptr));
        ActionCombo->setItemText(2, QCoreApplication::translate("ScAnnot", "Go To", nullptr));
        ActionCombo->setItemText(3, QCoreApplication::translate("ScAnnot", "Submit Form", nullptr));
        ActionCombo->setItemText(4, QCoreApplication::translate("ScAnnot", "Reset Form", nullptr));
        ActionCombo->setItemText(5, QCoreApplication::translate("ScAnnot", "Import Data", nullptr));
        ActionCombo->setItemText(6, QCoreApplication::translate("ScAnnot", "Named", nullptr));

        label_13->setText(QCoreApplication::translate("ScAnnot", "Event:", nullptr));
        SelAction->setItemText(0, QCoreApplication::translate("ScAnnot", "Mouse Up", nullptr));
        SelAction->setItemText(1, QCoreApplication::translate("ScAnnot", "Mouse Down", nullptr));
        SelAction->setItemText(2, QCoreApplication::translate("ScAnnot", "Mouse Enter", nullptr));
        SelAction->setItemText(3, QCoreApplication::translate("ScAnnot", "Mouse Exit", nullptr));
        SelAction->setItemText(4, QCoreApplication::translate("ScAnnot", "On Focus", nullptr));
        SelAction->setItemText(5, QCoreApplication::translate("ScAnnot", "On Blur", nullptr));

        label_14->setText(QCoreApplication::translate("ScAnnot", "Script:", nullptr));
        EditJ->setText(QCoreApplication::translate("ScAnnot", "Edit...", nullptr));
        GroupBoxDestination->setTitle(QCoreApplication::translate("ScAnnot", "Destination", nullptr));
        ChFile->setText(QCoreApplication::translate("ScAnnot", "Change...", nullptr));
        label_15->setText(QCoreApplication::translate("ScAnnot", "Page:", nullptr));
        label_17->setText(QCoreApplication::translate("ScAnnot", "Y-Pos:", nullptr));
        useAbsolute->setText(QCoreApplication::translate("ScAnnot", "Export absolute Filename", nullptr));
        SpinBox31->setSuffix(QCoreApplication::translate("ScAnnot", " pt", nullptr));
        LExtern->setText(QCoreApplication::translate("ScAnnot", "To File:", nullptr));
        label_16->setText(QCoreApplication::translate("ScAnnot", "X-Pos:", nullptr));
        SpinBox21->setSuffix(QCoreApplication::translate("ScAnnot", " pt", nullptr));
        label_18->setText(QCoreApplication::translate("ScAnnot", "Submit to URL:", nullptr));
        label_19->setText(QCoreApplication::translate("ScAnnot", "Submit format:", nullptr));
        SelAsHtml->setItemText(0, QCoreApplication::translate("ScAnnot", "FDF", nullptr));
        SelAsHtml->setItemText(1, QCoreApplication::translate("ScAnnot", "HTML", nullptr));
        SelAsHtml->setItemText(2, QCoreApplication::translate("ScAnnot", "XFDF", nullptr));
        SelAsHtml->setItemText(3, QCoreApplication::translate("ScAnnot", "PDF", nullptr));

        label_20->setText(QCoreApplication::translate("ScAnnot", "Import Data from:", nullptr));
        label_21->setText(QCoreApplication::translate("ScAnnot", "Action Name", nullptr));
        TabWidget2->setTabText(TabWidget2->indexOf(tabAction), QCoreApplication::translate("ScAnnot", "Action", nullptr));
        label_22->setText(QCoreApplication::translate("ScAnnot", "Field is formatted as:", nullptr));
        TxFormat->setItemText(0, QCoreApplication::translate("ScAnnot", "Plain Text", nullptr));
        TxFormat->setItemText(1, QCoreApplication::translate("ScAnnot", "Number", nullptr));
        TxFormat->setItemText(2, QCoreApplication::translate("ScAnnot", "Percentage", nullptr));
        TxFormat->setItemText(3, QCoreApplication::translate("ScAnnot", "Date", nullptr));
        TxFormat->setItemText(4, QCoreApplication::translate("ScAnnot", "Time", nullptr));
        TxFormat->setItemText(5, QCoreApplication::translate("ScAnnot", "Custom", nullptr));

        NumbGroup->setTitle(QCoreApplication::translate("ScAnnot", "Number Format", nullptr));
        label_23->setText(QCoreApplication::translate("ScAnnot", "Decimals:", nullptr));
        UseCurr->setText(QCoreApplication::translate("ScAnnot", "Use Currency Symbol", nullptr));
        PreCurr->setText(QCoreApplication::translate("ScAnnot", "Prepend Currency Symbol", nullptr));
        NumGroup2->setTitle(QCoreApplication::translate("ScAnnot", "Formatting", nullptr));
        PercGroup->setTitle(QCoreApplication::translate("ScAnnot", "Percent Format", nullptr));
        label_24->setText(QCoreApplication::translate("ScAnnot", "Decimals:", nullptr));
        NumGroup2a->setTitle(QCoreApplication::translate("ScAnnot", "Formatting", nullptr));
        DateGroup->setTitle(QCoreApplication::translate("ScAnnot", "Date Format", nullptr));

        TextDa1->setText(QString());
        TimeGroup->setTitle(QCoreApplication::translate("ScAnnot", "Time Format", nullptr));
        GroupCust->setTitle(QCoreApplication::translate("ScAnnot", "Custom Scripts", nullptr));
        label_25->setText(QCoreApplication::translate("ScAnnot", "Format:", nullptr));
        EditFormat->setText(QCoreApplication::translate("ScAnnot", "Edit...", nullptr));
        label_26->setText(QCoreApplication::translate("ScAnnot", "Keystroke:", nullptr));
        EditKeystr->setText(QCoreApplication::translate("ScAnnot", "Edit...", nullptr));
        TabWidget2->setTabText(TabWidget2->indexOf(tabFormat), QCoreApplication::translate("ScAnnot", "Format", nullptr));
        NoValid->setText(QCoreApplication::translate("ScAnnot", "Value is not validated", nullptr));
        SimpleValid->setText(QCoreApplication::translate("ScAnnot", "Value must be greater than or equal to:", nullptr));
        label_27->setText(QCoreApplication::translate("ScAnnot", "and less or equal to:", nullptr));
        CustomValid->setText(QCoreApplication::translate("ScAnnot", "Custom validate script:", nullptr));
        EditValScript->setText(QCoreApplication::translate("ScAnnot", "Edit...", nullptr));
        TabWidget2->setTabText(TabWidget2->indexOf(tabValidate), QCoreApplication::translate("ScAnnot", "Validate", nullptr));
        NoCalc->setText(QCoreApplication::translate("ScAnnot", "Value is not calculated", nullptr));
        SimpleCalc->setText(QCoreApplication::translate("ScAnnot", "Value is the", nullptr));
        CalcArt->setItemText(0, QCoreApplication::translate("ScAnnot", "sum", nullptr));
        CalcArt->setItemText(1, QCoreApplication::translate("ScAnnot", "product", nullptr));
        CalcArt->setItemText(2, QCoreApplication::translate("ScAnnot", "average", nullptr));
        CalcArt->setItemText(3, QCoreApplication::translate("ScAnnot", "minimum", nullptr));
        CalcArt->setItemText(4, QCoreApplication::translate("ScAnnot", "maximum", nullptr));

        label_28->setText(QCoreApplication::translate("ScAnnot", "of the following fields:", nullptr));
        SeField->setText(QCoreApplication::translate("ScAnnot", "Pick...", nullptr));
        CustomCalc->setText(QCoreApplication::translate("ScAnnot", "Custom calculation script:", nullptr));
        EditCalc->setText(QCoreApplication::translate("ScAnnot", "Edit...", nullptr));
        TabWidget2->setTabText(TabWidget2->indexOf(tabCalculate), QCoreApplication::translate("ScAnnot", "Calculate", nullptr));
        label_3->setText(QCoreApplication::translate("ScAnnot", "Tooltip:", nullptr));
        PushButton1->setText(QCoreApplication::translate("ScAnnot", "OK", nullptr));
        PushButton2->setText(QCoreApplication::translate("ScAnnot", "Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ScAnnot: public Ui_ScAnnot {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ANNOT_H
