/****************************************************************************
** Meta object code from reading C++ file 'guidemanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/guidemanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'guidemanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_GuideManager_t {
    QByteArrayData data[28];
    char stringdata0[757];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_GuideManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_GuideManager_t qt_meta_stringdata_GuideManager = {
    {
QT_MOC_LITERAL(0, 0, 12), // "GuideManager"
QT_MOC_LITERAL(1, 13, 15), // "forceDrawGuides"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 14), // "QItemSelection"
QT_MOC_LITERAL(4, 45, 26), // "verticalModel_valueChanged"
QT_MOC_LITERAL(5, 72, 28), // "horizontalModel_valueChanged"
QT_MOC_LITERAL(6, 101, 20), // "addHorButton_clicked"
QT_MOC_LITERAL(7, 122, 20), // "delHorButton_clicked"
QT_MOC_LITERAL(8, 143, 20), // "addVerButton_clicked"
QT_MOC_LITERAL(9, 164, 20), // "delVerButton_clicked"
QT_MOC_LITERAL(10, 185, 27), // "applyToAllStdButton_clicked"
QT_MOC_LITERAL(11, 213, 22), // "lockCheck_stateChanged"
QT_MOC_LITERAL(12, 236, 35), // "horizontalAutoGapCheck_stateC..."
QT_MOC_LITERAL(13, 272, 33), // "verticalAutoGapCheck_stateCha..."
QT_MOC_LITERAL(14, 306, 28), // "applyToAllAutoButton_clicked"
QT_MOC_LITERAL(15, 335, 36), // "horizontalAutoCountSpin_value..."
QT_MOC_LITERAL(16, 372, 34), // "verticalAutoCountSpin_valueCh..."
QT_MOC_LITERAL(17, 407, 34), // "horizontalAutoGapSpin_valueCh..."
QT_MOC_LITERAL(18, 442, 32), // "verticalAutoGapSpin_valueChanged"
QT_MOC_LITERAL(19, 475, 32), // "horizontalPageAutoButton_toggled"
QT_MOC_LITERAL(20, 508, 35), // "horizontalMarginsAutoButton_t..."
QT_MOC_LITERAL(21, 544, 37), // "horizontalSelectionAutoButton..."
QT_MOC_LITERAL(22, 582, 30), // "verticalPageAutoButton_toggled"
QT_MOC_LITERAL(23, 613, 33), // "verticalMarginsAutoButton_tog..."
QT_MOC_LITERAL(24, 647, 35), // "verticalSelectionAutoButton_t..."
QT_MOC_LITERAL(25, 683, 24), // "tabWidget_currentChanged"
QT_MOC_LITERAL(26, 708, 24), // "deletePageButton_clicked"
QT_MOC_LITERAL(27, 733, 23) // "deleteAllGuides_clicked"

    },
    "GuideManager\0forceDrawGuides\0\0"
    "QItemSelection\0verticalModel_valueChanged\0"
    "horizontalModel_valueChanged\0"
    "addHorButton_clicked\0delHorButton_clicked\0"
    "addVerButton_clicked\0delVerButton_clicked\0"
    "applyToAllStdButton_clicked\0"
    "lockCheck_stateChanged\0"
    "horizontalAutoGapCheck_stateChanged\0"
    "verticalAutoGapCheck_stateChanged\0"
    "applyToAllAutoButton_clicked\0"
    "horizontalAutoCountSpin_valueChanged\0"
    "verticalAutoCountSpin_valueChanged\0"
    "horizontalAutoGapSpin_valueChanged\0"
    "verticalAutoGapSpin_valueChanged\0"
    "horizontalPageAutoButton_toggled\0"
    "horizontalMarginsAutoButton_toggled\0"
    "horizontalSelectionAutoButton_toggled\0"
    "verticalPageAutoButton_toggled\0"
    "verticalMarginsAutoButton_toggled\0"
    "verticalSelectionAutoButton_toggled\0"
    "tabWidget_currentChanged\0"
    "deletePageButton_clicked\0"
    "deleteAllGuides_clicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_GuideManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      25,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,  139,    2, 0x08 /* Private */,
       4,    0,  144,    2, 0x08 /* Private */,
       5,    0,  145,    2, 0x08 /* Private */,
       6,    0,  146,    2, 0x08 /* Private */,
       7,    0,  147,    2, 0x08 /* Private */,
       8,    0,  148,    2, 0x08 /* Private */,
       9,    0,  149,    2, 0x08 /* Private */,
      10,    0,  150,    2, 0x08 /* Private */,
      11,    1,  151,    2, 0x08 /* Private */,
      12,    1,  154,    2, 0x08 /* Private */,
      13,    1,  157,    2, 0x08 /* Private */,
      14,    0,  160,    2, 0x08 /* Private */,
      15,    1,  161,    2, 0x08 /* Private */,
      16,    1,  164,    2, 0x08 /* Private */,
      17,    1,  167,    2, 0x08 /* Private */,
      18,    1,  170,    2, 0x08 /* Private */,
      19,    1,  173,    2, 0x08 /* Private */,
      20,    1,  176,    2, 0x08 /* Private */,
      21,    1,  179,    2, 0x08 /* Private */,
      22,    1,  182,    2, 0x08 /* Private */,
      23,    1,  185,    2, 0x08 /* Private */,
      24,    1,  188,    2, 0x08 /* Private */,
      25,    1,  191,    2, 0x08 /* Private */,
      26,    0,  194,    2, 0x08 /* Private */,
      27,    0,  195,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 3,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void GuideManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<GuideManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->forceDrawGuides((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 1: _t->verticalModel_valueChanged(); break;
        case 2: _t->horizontalModel_valueChanged(); break;
        case 3: _t->addHorButton_clicked(); break;
        case 4: _t->delHorButton_clicked(); break;
        case 5: _t->addVerButton_clicked(); break;
        case 6: _t->delVerButton_clicked(); break;
        case 7: _t->applyToAllStdButton_clicked(); break;
        case 8: _t->lockCheck_stateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->horizontalAutoGapCheck_stateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->verticalAutoGapCheck_stateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->applyToAllAutoButton_clicked(); break;
        case 12: _t->horizontalAutoCountSpin_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 13: _t->verticalAutoCountSpin_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 14: _t->horizontalAutoGapSpin_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 15: _t->verticalAutoGapSpin_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 16: _t->horizontalPageAutoButton_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 17: _t->horizontalMarginsAutoButton_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: _t->horizontalSelectionAutoButton_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->verticalPageAutoButton_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->verticalMarginsAutoButton_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->verticalSelectionAutoButton_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->tabWidget_currentChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 23: _t->deletePageButton_clicked(); break;
        case 24: _t->deleteAllGuides_clicked(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject GuideManager::staticMetaObject = { {
    QMetaObject::SuperData::link<ScrPaletteBase::staticMetaObject>(),
    qt_meta_stringdata_GuideManager.data,
    qt_meta_data_GuideManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *GuideManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GuideManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_GuideManager.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::GuideManager"))
        return static_cast< Ui::GuideManager*>(this);
    return ScrPaletteBase::qt_metacast(_clname);
}

int GuideManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScrPaletteBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 25;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
