/****************************************************************************
** Meta object code from reading C++ file 'pagepalette_pages.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/pagepalette_pages.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pagepalette_pages.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PagePalette_Pages_t {
    QByteArrayData data[31];
    char stringdata0[350];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PagePalette_Pages_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PagePalette_Pages_t qt_meta_stringdata_PagePalette_Pages = {
    {
QT_MOC_LITERAL(0, 0, 17), // "PagePalette_Pages"
QT_MOC_LITERAL(1, 18, 14), // "gotoMasterPage"
QT_MOC_LITERAL(2, 33, 0), // ""
QT_MOC_LITERAL(3, 34, 8), // "gotoPage"
QT_MOC_LITERAL(4, 43, 7), // "setView"
QT_MOC_LITERAL(5, 51, 12), // "ScribusView*"
QT_MOC_LITERAL(6, 64, 4), // "view"
QT_MOC_LITERAL(7, 69, 16), // "deleteMasterPage"
QT_MOC_LITERAL(8, 86, 3), // "tmp"
QT_MOC_LITERAL(9, 90, 13), // "enablePalette"
QT_MOC_LITERAL(10, 104, 16), // "handlePageLayout"
QT_MOC_LITERAL(11, 121, 6), // "layout"
QT_MOC_LITERAL(12, 128, 15), // "handleFirstPage"
QT_MOC_LITERAL(13, 144, 2), // "fp"
QT_MOC_LITERAL(14, 147, 14), // "rebuildMasters"
QT_MOC_LITERAL(15, 162, 12), // "rebuildPages"
QT_MOC_LITERAL(16, 175, 7), // "rebuild"
QT_MOC_LITERAL(17, 183, 8), // "markPage"
QT_MOC_LITERAL(18, 192, 2), // "nr"
QT_MOC_LITERAL(19, 195, 13), // "selMasterPage"
QT_MOC_LITERAL(20, 209, 14), // "languageChange"
QT_MOC_LITERAL(21, 224, 13), // "iconSetChange"
QT_MOC_LITERAL(22, 238, 24), // "pageView_applyMasterPage"
QT_MOC_LITERAL(23, 263, 14), // "masterpageName"
QT_MOC_LITERAL(24, 278, 9), // "pageIndex"
QT_MOC_LITERAL(25, 288, 17), // "pageView_movePage"
QT_MOC_LITERAL(26, 306, 1), // "r"
QT_MOC_LITERAL(27, 308, 1), // "c"
QT_MOC_LITERAL(28, 310, 17), // "pageView_gotoPage"
QT_MOC_LITERAL(29, 328, 1), // "b"
QT_MOC_LITERAL(30, 330, 19) // "pageView_deletePage"

    },
    "PagePalette_Pages\0gotoMasterPage\0\0"
    "gotoPage\0setView\0ScribusView*\0view\0"
    "deleteMasterPage\0tmp\0enablePalette\0"
    "handlePageLayout\0layout\0handleFirstPage\0"
    "fp\0rebuildMasters\0rebuildPages\0rebuild\0"
    "markPage\0nr\0selMasterPage\0languageChange\0"
    "iconSetChange\0pageView_applyMasterPage\0"
    "masterpageName\0pageIndex\0pageView_movePage\0"
    "r\0c\0pageView_gotoPage\0b\0pageView_deletePage"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PagePalette_Pages[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  104,    2, 0x06 /* Public */,
       3,    1,  107,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,  110,    2, 0x0a /* Public */,
       7,    1,  113,    2, 0x0a /* Public */,
       9,    1,  116,    2, 0x0a /* Public */,
      10,    1,  119,    2, 0x0a /* Public */,
      12,    1,  122,    2, 0x0a /* Public */,
      14,    0,  125,    2, 0x0a /* Public */,
      15,    0,  126,    2, 0x0a /* Public */,
      16,    0,  127,    2, 0x0a /* Public */,
      17,    1,  128,    2, 0x0a /* Public */,
      19,    0,  131,    2, 0x0a /* Public */,
      20,    0,  132,    2, 0x0a /* Public */,
      21,    0,  133,    2, 0x08 /* Private */,
      22,    2,  134,    2, 0x08 /* Private */,
      25,    2,  139,    2, 0x08 /* Private */,
      28,    3,  144,    2, 0x08 /* Private */,
      30,    1,  151,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::UInt,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,   23,   24,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   26,   27,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Int,   26,   27,   29,
    QMetaType::Void, QMetaType::Int,   24,

       0        // eod
};

void PagePalette_Pages::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PagePalette_Pages *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->gotoMasterPage((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->gotoPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->setView((*reinterpret_cast< ScribusView*(*)>(_a[1]))); break;
        case 3: _t->deleteMasterPage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->enablePalette((*reinterpret_cast< const bool(*)>(_a[1]))); break;
        case 5: _t->handlePageLayout((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->handleFirstPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->rebuildMasters(); break;
        case 8: _t->rebuildPages(); break;
        case 9: _t->rebuild(); break;
        case 10: _t->markPage((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 11: _t->selMasterPage(); break;
        case 12: _t->languageChange(); break;
        case 13: _t->iconSetChange(); break;
        case 14: _t->pageView_applyMasterPage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 15: _t->pageView_movePage((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 16: _t->pageView_gotoPage((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 17: _t->pageView_deletePage((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PagePalette_Pages::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PagePalette_Pages::gotoMasterPage)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PagePalette_Pages::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PagePalette_Pages::gotoPage)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PagePalette_Pages::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_PagePalette_Pages.data,
    qt_meta_data_PagePalette_Pages,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PagePalette_Pages::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PagePalette_Pages::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PagePalette_Pages.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PagePalette_PagesBase"))
        return static_cast< Ui::PagePalette_PagesBase*>(this);
    return QWidget::qt_metacast(_clname);
}

int PagePalette_Pages::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void PagePalette_Pages::gotoMasterPage(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PagePalette_Pages::gotoPage(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
