/****************************************************************************
** Meta object code from reading C++ file 'colorsandfills.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/colorsandfills.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'colorsandfills.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ColorsAndFillsDialog_t {
    QByteArrayData data[18];
    char stringdata0[235];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ColorsAndFillsDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ColorsAndFillsDialog_t qt_meta_stringdata_ColorsAndFillsDialog = {
    {
QT_MOC_LITERAL(0, 0, 20), // "ColorsAndFillsDialog"
QT_MOC_LITERAL(1, 21, 14), // "slotRightClick"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 1), // "p"
QT_MOC_LITERAL(4, 39, 12), // "selEditColor"
QT_MOC_LITERAL(5, 52, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(6, 69, 2), // "it"
QT_MOC_LITERAL(7, 72, 20), // "itemSelectionChanged"
QT_MOC_LITERAL(8, 93, 12), // "itemSelected"
QT_MOC_LITERAL(9, 106, 9), // "createNew"
QT_MOC_LITERAL(10, 116, 13), // "editColorItem"
QT_MOC_LITERAL(11, 130, 18), // "duplicateColorItem"
QT_MOC_LITERAL(12, 149, 15), // "removeColorItem"
QT_MOC_LITERAL(13, 165, 21), // "removeUnusedColorItem"
QT_MOC_LITERAL(14, 187, 16), // "importColorItems"
QT_MOC_LITERAL(15, 204, 12), // "loadDefaults"
QT_MOC_LITERAL(16, 217, 4), // "item"
QT_MOC_LITERAL(17, 222, 12) // "saveDefaults"

    },
    "ColorsAndFillsDialog\0slotRightClick\0"
    "\0p\0selEditColor\0QTreeWidgetItem*\0it\0"
    "itemSelectionChanged\0itemSelected\0"
    "createNew\0editColorItem\0duplicateColorItem\0"
    "removeColorItem\0removeUnusedColorItem\0"
    "importColorItems\0loadDefaults\0item\0"
    "saveDefaults"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ColorsAndFillsDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x08 /* Private */,
       4,    1,   77,    2, 0x08 /* Private */,
       7,    0,   80,    2, 0x08 /* Private */,
       8,    1,   81,    2, 0x08 /* Private */,
       9,    0,   84,    2, 0x08 /* Private */,
      10,    0,   85,    2, 0x08 /* Private */,
      11,    0,   86,    2, 0x08 /* Private */,
      12,    0,   87,    2, 0x08 /* Private */,
      13,    0,   88,    2, 0x08 /* Private */,
      14,    0,   89,    2, 0x08 /* Private */,
      15,    1,   90,    2, 0x08 /* Private */,
      17,    0,   93,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::QPoint,    3,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 5,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 5,   16,
    QMetaType::Void,

       0        // eod
};

void ColorsAndFillsDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ColorsAndFillsDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotRightClick((*reinterpret_cast< QPoint(*)>(_a[1]))); break;
        case 1: _t->selEditColor((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1]))); break;
        case 2: _t->itemSelectionChanged(); break;
        case 3: _t->itemSelected((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1]))); break;
        case 4: _t->createNew(); break;
        case 5: _t->editColorItem(); break;
        case 6: _t->duplicateColorItem(); break;
        case 7: _t->removeColorItem(); break;
        case 8: _t->removeUnusedColorItem(); break;
        case 9: _t->importColorItems(); break;
        case 10: _t->loadDefaults((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1]))); break;
        case 11: _t->saveDefaults(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ColorsAndFillsDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<ScDialog::staticMetaObject>(),
    qt_meta_stringdata_ColorsAndFillsDialog.data,
    qt_meta_data_ColorsAndFillsDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ColorsAndFillsDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ColorsAndFillsDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ColorsAndFillsDialog.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::ColorsAndFillsBase"))
        return static_cast< Ui::ColorsAndFillsBase*>(this);
    return ScDialog::qt_metacast(_clname);
}

int ColorsAndFillsDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
