/********************************************************************************
** Form generated from reading UI file 'prefs_experimentalbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_EXPERIMENTALBASE_H
#define UI_PREFS_EXPERIMENTALBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Prefs_Experimental
{
public:
    QVBoxLayout *verticalLayout_3;
    QLabel *titleLabel;
    QFrame *line;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout;
    QCheckBox *enableNotesCheckBox;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *Prefs_Experimental)
    {
        if (Prefs_Experimental->objectName().isEmpty())
            Prefs_Experimental->setObjectName(QString::fromUtf8("Prefs_Experimental"));
        Prefs_Experimental->resize(691, 420);
        verticalLayout_3 = new QVBoxLayout(Prefs_Experimental);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        titleLabel = new QLabel(Prefs_Experimental);
        titleLabel->setObjectName(QString::fromUtf8("titleLabel"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        titleLabel->setFont(font);

        verticalLayout_3->addWidget(titleLabel);

        line = new QFrame(Prefs_Experimental);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout_3->addWidget(line);

        scrollArea = new QScrollArea(Prefs_Experimental);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 691, 378));
        verticalLayout = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, -1);
        enableNotesCheckBox = new QCheckBox(scrollAreaWidgetContents);
        enableNotesCheckBox->setObjectName(QString::fromUtf8("enableNotesCheckBox"));

        verticalLayout->addWidget(enableNotesCheckBox);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

        verticalLayout->addItem(verticalSpacer);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_3->addWidget(scrollArea);


        retranslateUi(Prefs_Experimental);

        QMetaObject::connectSlotsByName(Prefs_Experimental);
    } // setupUi

    void retranslateUi(QWidget *Prefs_Experimental)
    {
        titleLabel->setText(QCoreApplication::translate("Prefs_Experimental", "Experimental Features", nullptr));
        enableNotesCheckBox->setText(QCoreApplication::translate("Prefs_Experimental", "Enable Notes, Marks and other related features", nullptr));
        (void)Prefs_Experimental;
    } // retranslateUi

};

namespace Ui {
    class Prefs_Experimental: public Ui_Prefs_Experimental {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_EXPERIMENTALBASE_H
