/****************************************************************************
** Meta object code from reading C++ file 'tabpdfoptions.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/tabpdfoptions.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tabpdfoptions.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_TabPDFOptions_t {
    QByteArrayData data[44];
    char stringdata0[438];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_TabPDFOptions_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_TabPDFOptions_t qt_meta_stringdata_TabPDFOptions = {
    {
QT_MOC_LITERAL(0, 0, 13), // "TabPDFOptions"
QT_MOC_LITERAL(1, 14, 6), // "noInfo"
QT_MOC_LITERAL(2, 21, 0), // ""
QT_MOC_LITERAL(3, 22, 7), // "hasInfo"
QT_MOC_LITERAL(4, 30, 11), // "doDocBleeds"
QT_MOC_LITERAL(5, 42, 10), // "ToggleEncr"
QT_MOC_LITERAL(6, 53, 10), // "EnablePDFX"
QT_MOC_LITERAL(7, 64, 1), // "a"
QT_MOC_LITERAL(8, 66, 12), // "DoDownsample"
QT_MOC_LITERAL(9, 79, 19), // "EmbeddingModeChange"
QT_MOC_LITERAL(10, 99, 12), // "RemoveSubset"
QT_MOC_LITERAL(11, 112, 11), // "PutToSubset"
QT_MOC_LITERAL(12, 124, 8), // "SelEFont"
QT_MOC_LITERAL(13, 133, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(14, 150, 8), // "SelSFont"
QT_MOC_LITERAL(15, 159, 8), // "EmbedAll"
QT_MOC_LITERAL(16, 168, 9), // "SubsetAll"
QT_MOC_LITERAL(17, 178, 6), // "PagePr"
QT_MOC_LITERAL(18, 185, 8), // "SetPgEff"
QT_MOC_LITERAL(19, 194, 7), // "current"
QT_MOC_LITERAL(20, 202, 8), // "previous"
QT_MOC_LITERAL(21, 211, 10), // "SetEffOpts"
QT_MOC_LITERAL(22, 222, 2), // "nr"
QT_MOC_LITERAL(23, 225, 7), // "ValidDI"
QT_MOC_LITERAL(24, 233, 9), // "PDFMirror"
QT_MOC_LITERAL(25, 243, 8), // "Rotation"
QT_MOC_LITERAL(26, 252, 5), // "value"
QT_MOC_LITERAL(27, 258, 9), // "DoEffects"
QT_MOC_LITERAL(28, 268, 11), // "EffectOnAll"
QT_MOC_LITERAL(29, 280, 8), // "SelRange"
QT_MOC_LITERAL(30, 289, 1), // "e"
QT_MOC_LITERAL(31, 291, 8), // "EnablePr"
QT_MOC_LITERAL(32, 300, 8), // "EnablePG"
QT_MOC_LITERAL(33, 309, 9), // "EnablePGI"
QT_MOC_LITERAL(34, 319, 10), // "EnablePGI2"
QT_MOC_LITERAL(35, 330, 9), // "EnableLPI"
QT_MOC_LITERAL(36, 340, 10), // "EnableLPI2"
QT_MOC_LITERAL(37, 351, 9), // "SelLPIcol"
QT_MOC_LITERAL(38, 361, 9), // "enableCMS"
QT_MOC_LITERAL(39, 371, 6), // "enable"
QT_MOC_LITERAL(40, 378, 9), // "checkInfo"
QT_MOC_LITERAL(41, 388, 21), // "createPageNumberRange"
QT_MOC_LITERAL(42, 410, 23), // "handleCompressionMethod"
QT_MOC_LITERAL(43, 434, 3) // "ind"

    },
    "TabPDFOptions\0noInfo\0\0hasInfo\0doDocBleeds\0"
    "ToggleEncr\0EnablePDFX\0a\0DoDownsample\0"
    "EmbeddingModeChange\0RemoveSubset\0"
    "PutToSubset\0SelEFont\0QListWidgetItem*\0"
    "SelSFont\0EmbedAll\0SubsetAll\0PagePr\0"
    "SetPgEff\0current\0previous\0SetEffOpts\0"
    "nr\0ValidDI\0PDFMirror\0Rotation\0value\0"
    "DoEffects\0EffectOnAll\0SelRange\0e\0"
    "EnablePr\0EnablePG\0EnablePGI\0EnablePGI2\0"
    "EnableLPI\0EnableLPI2\0SelLPIcol\0enableCMS\0"
    "enable\0checkInfo\0createPageNumberRange\0"
    "handleCompressionMethod\0ind"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_TabPDFOptions[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      33,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  179,    2, 0x06 /* Public */,
       3,    0,  180,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,  181,    2, 0x0a /* Public */,
       5,    0,  182,    2, 0x0a /* Public */,
       6,    1,  183,    2, 0x0a /* Public */,
       8,    0,  186,    2, 0x0a /* Public */,
       9,    0,  187,    2, 0x0a /* Public */,
      10,    0,  188,    2, 0x0a /* Public */,
      11,    0,  189,    2, 0x0a /* Public */,
      12,    1,  190,    2, 0x0a /* Public */,
      14,    1,  193,    2, 0x0a /* Public */,
      15,    0,  196,    2, 0x0a /* Public */,
      16,    0,  197,    2, 0x0a /* Public */,
      17,    0,  198,    2, 0x0a /* Public */,
      18,    2,  199,    2, 0x0a /* Public */,
      21,    1,  204,    2, 0x0a /* Public */,
      23,    1,  207,    2, 0x0a /* Public */,
      24,    0,  210,    2, 0x0a /* Public */,
      25,    1,  211,    2, 0x0a /* Public */,
      27,    0,  214,    2, 0x0a /* Public */,
      28,    0,  215,    2, 0x0a /* Public */,
      29,    1,  216,    2, 0x0a /* Public */,
      31,    1,  219,    2, 0x0a /* Public */,
      32,    0,  222,    2, 0x0a /* Public */,
      33,    0,  223,    2, 0x0a /* Public */,
      34,    0,  224,    2, 0x0a /* Public */,
      35,    1,  225,    2, 0x0a /* Public */,
      36,    0,  228,    2, 0x0a /* Public */,
      37,    1,  229,    2, 0x0a /* Public */,
      38,    1,  232,    2, 0x0a /* Public */,
      40,    0,  235,    2, 0x0a /* Public */,
      41,    0,  236,    2, 0x09 /* Protected */,
      42,    1,  237,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13,    2,
    QMetaType::Void, 0x80000000 | 13,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13, 0x80000000 | 13,   19,   20,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   26,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   30,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,   39,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   43,

       0        // eod
};

void TabPDFOptions::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<TabPDFOptions *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->noInfo(); break;
        case 1: _t->hasInfo(); break;
        case 2: _t->doDocBleeds(); break;
        case 3: _t->ToggleEncr(); break;
        case 4: _t->EnablePDFX((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->DoDownsample(); break;
        case 6: _t->EmbeddingModeChange(); break;
        case 7: _t->RemoveSubset(); break;
        case 8: _t->PutToSubset(); break;
        case 9: _t->SelEFont((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 10: _t->SelSFont((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 11: _t->EmbedAll(); break;
        case 12: _t->SubsetAll(); break;
        case 13: _t->PagePr(); break;
        case 14: _t->SetPgEff((*reinterpret_cast< QListWidgetItem*(*)>(_a[1])),(*reinterpret_cast< QListWidgetItem*(*)>(_a[2]))); break;
        case 15: _t->SetEffOpts((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->ValidDI((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->PDFMirror(); break;
        case 18: _t->Rotation((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->DoEffects(); break;
        case 20: _t->EffectOnAll(); break;
        case 21: _t->SelRange((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->EnablePr((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 23: _t->EnablePG(); break;
        case 24: _t->EnablePGI(); break;
        case 25: _t->EnablePGI2(); break;
        case 26: _t->EnableLPI((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 27: _t->EnableLPI2(); break;
        case 28: _t->SelLPIcol((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 29: _t->enableCMS((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 30: _t->checkInfo(); break;
        case 31: _t->createPageNumberRange(); break;
        case 32: _t->handleCompressionMethod((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (TabPDFOptions::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TabPDFOptions::noInfo)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (TabPDFOptions::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&TabPDFOptions::hasInfo)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject TabPDFOptions::staticMetaObject = { {
    QMetaObject::SuperData::link<QTabWidget::staticMetaObject>(),
    qt_meta_stringdata_TabPDFOptions.data,
    qt_meta_data_TabPDFOptions,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *TabPDFOptions::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *TabPDFOptions::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_TabPDFOptions.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::TabPDFOptions"))
        return static_cast< Ui::TabPDFOptions*>(this);
    return QTabWidget::qt_metacast(_clname);
}

int TabPDFOptions::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QTabWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 33)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 33)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 33;
    }
    return _id;
}

// SIGNAL 0
void TabPDFOptions::noInfo()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void TabPDFOptions::hasInfo()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
