/********************************************************************************
** Form generated from reading UI file 'measurementsbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MEASUREMENTSBASE_H
#define UI_MEASUREMENTSBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>

QT_BEGIN_NAMESPACE

class Ui_MeasurementsBase
{
public:
    QGridLayout *gridLayout;
    QLabel *x1Label;
    QLabel *x1Data;
    QLabel *y1Label;
    QLabel *y1Data;
    QLabel *x2Label;
    QLabel *x2Data;
    QLabel *y2Label;
    QLabel *y2Data;
    QLabel *lengthLabel;
    QLabel *lengthData;
    QLabel *dxLabel;
    QLabel *dXData;
    QLabel *dyLabel;
    QLabel *dYData;
    QLabel *angleLabel;
    QLabel *angleData;
    QLabel *label;
    QComboBox *unitSwitch;

    void setupUi(QDialog *MeasurementsBase)
    {
        if (MeasurementsBase->objectName().isEmpty())
            MeasurementsBase->setObjectName(QString::fromUtf8("MeasurementsBase"));
        MeasurementsBase->resize(154, 236);
        gridLayout = new QGridLayout(MeasurementsBase);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        x1Label = new QLabel(MeasurementsBase);
        x1Label->setObjectName(QString::fromUtf8("x1Label"));

        gridLayout->addWidget(x1Label, 0, 0, 1, 1);

        x1Data = new QLabel(MeasurementsBase);
        x1Data->setObjectName(QString::fromUtf8("x1Data"));
        x1Data->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(x1Data, 0, 1, 1, 1);

        y1Label = new QLabel(MeasurementsBase);
        y1Label->setObjectName(QString::fromUtf8("y1Label"));

        gridLayout->addWidget(y1Label, 1, 0, 1, 1);

        y1Data = new QLabel(MeasurementsBase);
        y1Data->setObjectName(QString::fromUtf8("y1Data"));
        y1Data->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(y1Data, 1, 1, 1, 1);

        x2Label = new QLabel(MeasurementsBase);
        x2Label->setObjectName(QString::fromUtf8("x2Label"));

        gridLayout->addWidget(x2Label, 2, 0, 1, 1);

        x2Data = new QLabel(MeasurementsBase);
        x2Data->setObjectName(QString::fromUtf8("x2Data"));
        x2Data->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(x2Data, 2, 1, 1, 1);

        y2Label = new QLabel(MeasurementsBase);
        y2Label->setObjectName(QString::fromUtf8("y2Label"));

        gridLayout->addWidget(y2Label, 3, 0, 1, 1);

        y2Data = new QLabel(MeasurementsBase);
        y2Data->setObjectName(QString::fromUtf8("y2Data"));
        y2Data->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(y2Data, 3, 1, 1, 1);

        lengthLabel = new QLabel(MeasurementsBase);
        lengthLabel->setObjectName(QString::fromUtf8("lengthLabel"));

        gridLayout->addWidget(lengthLabel, 4, 0, 1, 1);

        lengthData = new QLabel(MeasurementsBase);
        lengthData->setObjectName(QString::fromUtf8("lengthData"));
        lengthData->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(lengthData, 4, 1, 1, 1);

        dxLabel = new QLabel(MeasurementsBase);
        dxLabel->setObjectName(QString::fromUtf8("dxLabel"));

        gridLayout->addWidget(dxLabel, 5, 0, 1, 1);

        dXData = new QLabel(MeasurementsBase);
        dXData->setObjectName(QString::fromUtf8("dXData"));
        dXData->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(dXData, 5, 1, 1, 1);

        dyLabel = new QLabel(MeasurementsBase);
        dyLabel->setObjectName(QString::fromUtf8("dyLabel"));

        gridLayout->addWidget(dyLabel, 6, 0, 1, 1);

        dYData = new QLabel(MeasurementsBase);
        dYData->setObjectName(QString::fromUtf8("dYData"));
        dYData->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(dYData, 6, 1, 1, 1);

        angleLabel = new QLabel(MeasurementsBase);
        angleLabel->setObjectName(QString::fromUtf8("angleLabel"));

        gridLayout->addWidget(angleLabel, 7, 0, 1, 1);

        angleData = new QLabel(MeasurementsBase);
        angleData->setObjectName(QString::fromUtf8("angleData"));
        angleData->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(angleData, 7, 1, 1, 1);

        label = new QLabel(MeasurementsBase);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 8, 0, 1, 1);

        unitSwitch = new QComboBox(MeasurementsBase);
        unitSwitch->setObjectName(QString::fromUtf8("unitSwitch"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(unitSwitch->sizePolicy().hasHeightForWidth());
        unitSwitch->setSizePolicy(sizePolicy);
        unitSwitch->setMinimumContentsLength(3);
        unitSwitch->setFrame(true);

        gridLayout->addWidget(unitSwitch, 8, 1, 1, 1);


        retranslateUi(MeasurementsBase);

        QMetaObject::connectSlotsByName(MeasurementsBase);
    } // setupUi

    void retranslateUi(QDialog *MeasurementsBase)
    {
        MeasurementsBase->setWindowTitle(QCoreApplication::translate("MeasurementsBase", "Measurements", nullptr));
        x1Label->setText(QCoreApplication::translate("MeasurementsBase", "X1:", nullptr));
        x1Data->setText(QString());
        y1Label->setText(QCoreApplication::translate("MeasurementsBase", "Y1:", nullptr));
        y1Data->setText(QString());
        x2Label->setText(QCoreApplication::translate("MeasurementsBase", "X2:", nullptr));
        x2Data->setText(QString());
        y2Label->setText(QCoreApplication::translate("MeasurementsBase", "Y2:", nullptr));
        y2Data->setText(QString());
        lengthLabel->setText(QCoreApplication::translate("MeasurementsBase", "Length:", nullptr));
        lengthData->setText(QString());
        dxLabel->setText(QCoreApplication::translate("MeasurementsBase", "DX:", nullptr));
        dXData->setText(QString());
        dyLabel->setText(QCoreApplication::translate("MeasurementsBase", "DY:", nullptr));
        dYData->setText(QString());
        angleLabel->setText(QCoreApplication::translate("MeasurementsBase", "Angle:", nullptr));
        angleData->setText(QString());
        label->setText(QCoreApplication::translate("MeasurementsBase", "Unit:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class MeasurementsBase: public Ui_MeasurementsBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MEASUREMENTSBASE_H
