/****************************************************************************
** Meta object code from reading C++ file 'propertiespalette_line.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/propertiespalette_line.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertiespalette_line.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PropertiesPalette_Line_t {
    QByteArrayData data[43];
    char stringdata0[519];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PropertiesPalette_Line_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PropertiesPalette_Line_t qt_meta_stringdata_PropertiesPalette_Line = {
    {
QT_MOC_LITERAL(0, 0, 22), // "PropertiesPalette_Line"
QT_MOC_LITERAL(1, 23, 15), // "lineModeChanged"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 13), // "setMainWindow"
QT_MOC_LITERAL(4, 54, 18), // "ScribusMainWindow*"
QT_MOC_LITERAL(5, 73, 2), // "mw"
QT_MOC_LITERAL(6, 76, 6), // "setDoc"
QT_MOC_LITERAL(7, 83, 11), // "ScribusDoc*"
QT_MOC_LITERAL(8, 95, 1), // "d"
QT_MOC_LITERAL(9, 97, 14), // "setCurrentItem"
QT_MOC_LITERAL(10, 112, 9), // "PageItem*"
QT_MOC_LITERAL(11, 122, 4), // "item"
QT_MOC_LITERAL(12, 127, 8), // "unsetDoc"
QT_MOC_LITERAL(13, 136, 9), // "unsetItem"
QT_MOC_LITERAL(14, 146, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(15, 169, 19), // "handleUpdateRequest"
QT_MOC_LITERAL(16, 189, 13), // "showLineWidth"
QT_MOC_LITERAL(17, 203, 1), // "s"
QT_MOC_LITERAL(18, 205, 14), // "showLineValues"
QT_MOC_LITERAL(19, 220, 12), // "Qt::PenStyle"
QT_MOC_LITERAL(20, 233, 1), // "p"
QT_MOC_LITERAL(21, 235, 15), // "Qt::PenCapStyle"
QT_MOC_LITERAL(22, 251, 2), // "pc"
QT_MOC_LITERAL(23, 254, 16), // "Qt::PenJoinStyle"
QT_MOC_LITERAL(24, 271, 2), // "pj"
QT_MOC_LITERAL(25, 274, 13), // "iconSetChange"
QT_MOC_LITERAL(26, 288, 14), // "languageChange"
QT_MOC_LITERAL(27, 303, 10), // "unitChange"
QT_MOC_LITERAL(28, 314, 12), // "localeChange"
QT_MOC_LITERAL(29, 327, 15), // "handleLineWidth"
QT_MOC_LITERAL(30, 343, 15), // "handleLineStyle"
QT_MOC_LITERAL(31, 359, 14), // "handleLineJoin"
QT_MOC_LITERAL(32, 374, 13), // "handleLineEnd"
QT_MOC_LITERAL(33, 388, 14), // "handleLineMode"
QT_MOC_LITERAL(34, 403, 16), // "handleDashChange"
QT_MOC_LITERAL(35, 420, 16), // "handleStartArrow"
QT_MOC_LITERAL(36, 437, 2), // "id"
QT_MOC_LITERAL(37, 440, 14), // "handleEndArrow"
QT_MOC_LITERAL(38, 455, 21), // "handleStartArrowScale"
QT_MOC_LITERAL(39, 477, 2), // "sc"
QT_MOC_LITERAL(40, 480, 19), // "handleEndArrowScale"
QT_MOC_LITERAL(41, 500, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(42, 517, 1) // "c"

    },
    "PropertiesPalette_Line\0lineModeChanged\0"
    "\0setMainWindow\0ScribusMainWindow*\0mw\0"
    "setDoc\0ScribusDoc*\0d\0setCurrentItem\0"
    "PageItem*\0item\0unsetDoc\0unsetItem\0"
    "handleSelectionChanged\0handleUpdateRequest\0"
    "showLineWidth\0s\0showLineValues\0"
    "Qt::PenStyle\0p\0Qt::PenCapStyle\0pc\0"
    "Qt::PenJoinStyle\0pj\0iconSetChange\0"
    "languageChange\0unitChange\0localeChange\0"
    "handleLineWidth\0handleLineStyle\0"
    "handleLineJoin\0handleLineEnd\0"
    "handleLineMode\0handleDashChange\0"
    "handleStartArrow\0id\0handleEndArrow\0"
    "handleStartArrowScale\0sc\0handleEndArrowScale\0"
    "QListWidgetItem*\0c"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PropertiesPalette_Line[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      25,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  139,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,  142,    2, 0x0a /* Public */,
       6,    1,  145,    2, 0x0a /* Public */,
       9,    1,  148,    2, 0x0a /* Public */,
      12,    0,  151,    2, 0x0a /* Public */,
      13,    0,  152,    2, 0x0a /* Public */,
      14,    0,  153,    2, 0x0a /* Public */,
      15,    1,  154,    2, 0x0a /* Public */,
      16,    1,  157,    2, 0x0a /* Public */,
      18,    3,  160,    2, 0x0a /* Public */,
      25,    0,  167,    2, 0x0a /* Public */,
      26,    0,  168,    2, 0x0a /* Public */,
      27,    0,  169,    2, 0x0a /* Public */,
      28,    0,  170,    2, 0x0a /* Public */,
      29,    0,  171,    2, 0x08 /* Private */,
      30,    0,  172,    2, 0x08 /* Private */,
      31,    0,  173,    2, 0x08 /* Private */,
      32,    0,  174,    2, 0x08 /* Private */,
      33,    0,  175,    2, 0x08 /* Private */,
      34,    0,  176,    2, 0x08 /* Private */,
      35,    1,  177,    2, 0x08 /* Private */,
      37,    1,  180,    2, 0x08 /* Private */,
      38,    1,  183,    2, 0x08 /* Private */,
      40,    1,  186,    2, 0x08 /* Private */,
      30,    1,  189,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double,   17,
    QMetaType::Void, 0x80000000 | 19, 0x80000000 | 21, 0x80000000 | 23,   20,   22,   24,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   36,
    QMetaType::Void, QMetaType::Int,   36,
    QMetaType::Void, QMetaType::Double,   39,
    QMetaType::Void, QMetaType::Double,   39,
    QMetaType::Void, 0x80000000 | 41,   42,

       0        // eod
};

void PropertiesPalette_Line::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertiesPalette_Line *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->lineModeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->setMainWindow((*reinterpret_cast< ScribusMainWindow*(*)>(_a[1]))); break;
        case 2: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 3: _t->setCurrentItem((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 4: _t->unsetDoc(); break;
        case 5: _t->unsetItem(); break;
        case 6: _t->handleSelectionChanged(); break;
        case 7: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->showLineWidth((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 9: _t->showLineValues((*reinterpret_cast< Qt::PenStyle(*)>(_a[1])),(*reinterpret_cast< Qt::PenCapStyle(*)>(_a[2])),(*reinterpret_cast< Qt::PenJoinStyle(*)>(_a[3]))); break;
        case 10: _t->iconSetChange(); break;
        case 11: _t->languageChange(); break;
        case 12: _t->unitChange(); break;
        case 13: _t->localeChange(); break;
        case 14: _t->handleLineWidth(); break;
        case 15: _t->handleLineStyle(); break;
        case 16: _t->handleLineJoin(); break;
        case 17: _t->handleLineEnd(); break;
        case 18: _t->handleLineMode(); break;
        case 19: _t->handleDashChange(); break;
        case 20: _t->handleStartArrow((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 21: _t->handleEndArrow((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 22: _t->handleStartArrowScale((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 23: _t->handleEndArrowScale((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 24: _t->handleLineStyle((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PropertiesPalette_Line::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PropertiesPalette_Line::lineModeChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PropertiesPalette_Line::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_PropertiesPalette_Line.data,
    qt_meta_data_PropertiesPalette_Line,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PropertiesPalette_Line::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PropertiesPalette_Line::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PropertiesPalette_Line.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PropertiesPalette_LineBase"))
        return static_cast< Ui::PropertiesPalette_LineBase*>(this);
    return QWidget::qt_metacast(_clname);
}

int PropertiesPalette_Line::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void PropertiesPalette_Line::lineModeChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
