/****************************************************************************
** Meta object code from reading C++ file 'cwdialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../scribus-1.6.6/scribus/plugins/colorwheel/cwdialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'cwdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_CWDialog_t {
    QByteArrayData data[24];
    char stringdata0[446];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_CWDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_CWDialog_t qt_meta_stringdata_CWDialog = {
    {
QT_MOC_LITERAL(0, 0, 8), // "CWDialog"
QT_MOC_LITERAL(1, 9, 28), // "colorspaceTab_currentChanged"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 22), // "angleSpin_valueChanged"
QT_MOC_LITERAL(4, 62, 18), // "colorWheel_clicked"
QT_MOC_LITERAL(5, 81, 19), // "typeCombo_activated"
QT_MOC_LITERAL(6, 101, 32), // "documentColorList_currentChanged"
QT_MOC_LITERAL(7, 134, 8), // "itemText"
QT_MOC_LITERAL(8, 143, 21), // "defectCombo_activated"
QT_MOC_LITERAL(9, 165, 17), // "addButton_clicked"
QT_MOC_LITERAL(10, 183, 21), // "replaceButton_clicked"
QT_MOC_LITERAL(11, 205, 20), // "cancelButton_clicked"
QT_MOC_LITERAL(12, 226, 24), // "colorList_currentChanged"
QT_MOC_LITERAL(13, 251, 4), // "text"
QT_MOC_LITERAL(14, 256, 18), // "cSpin_valueChanged"
QT_MOC_LITERAL(15, 275, 18), // "mSpin_valueChanged"
QT_MOC_LITERAL(16, 294, 18), // "ySpin_valueChanged"
QT_MOC_LITERAL(17, 313, 18), // "kSpin_valueChanged"
QT_MOC_LITERAL(18, 332, 18), // "rSpin_valueChanged"
QT_MOC_LITERAL(19, 351, 18), // "gSpin_valueChanged"
QT_MOC_LITERAL(20, 370, 18), // "bSpin_valueChanged"
QT_MOC_LITERAL(21, 389, 18), // "hSpin_valueChanged"
QT_MOC_LITERAL(22, 408, 18), // "sSpin_valueChanged"
QT_MOC_LITERAL(23, 427, 18) // "vSpin_valueChanged"

    },
    "CWDialog\0colorspaceTab_currentChanged\0"
    "\0angleSpin_valueChanged\0colorWheel_clicked\0"
    "typeCombo_activated\0"
    "documentColorList_currentChanged\0"
    "itemText\0defectCombo_activated\0"
    "addButton_clicked\0replaceButton_clicked\0"
    "cancelButton_clicked\0colorList_currentChanged\0"
    "text\0cSpin_valueChanged\0mSpin_valueChanged\0"
    "ySpin_valueChanged\0kSpin_valueChanged\0"
    "rSpin_valueChanged\0gSpin_valueChanged\0"
    "bSpin_valueChanged\0hSpin_valueChanged\0"
    "sSpin_valueChanged\0vSpin_valueChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_CWDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      20,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  114,    2, 0x08 /* Private */,
       3,    1,  117,    2, 0x08 /* Private */,
       4,    2,  120,    2, 0x08 /* Private */,
       5,    1,  125,    2, 0x08 /* Private */,
       6,    1,  128,    2, 0x08 /* Private */,
       8,    1,  131,    2, 0x08 /* Private */,
       9,    0,  134,    2, 0x08 /* Private */,
      10,    0,  135,    2, 0x08 /* Private */,
      11,    0,  136,    2, 0x08 /* Private */,
      12,    1,  137,    2, 0x08 /* Private */,
      14,    1,  140,    2, 0x08 /* Private */,
      15,    1,  143,    2, 0x08 /* Private */,
      16,    1,  146,    2, 0x08 /* Private */,
      17,    1,  149,    2, 0x08 /* Private */,
      18,    1,  152,    2, 0x08 /* Private */,
      19,    1,  155,    2, 0x08 /* Private */,
      20,    1,  158,    2, 0x08 /* Private */,
      21,    1,  161,    2, 0x08 /* Private */,
      22,    1,  164,    2, 0x08 /* Private */,
      23,    1,  167,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::QPoint,    2,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    7,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   13,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void CWDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<CWDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->colorspaceTab_currentChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->angleSpin_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->colorWheel_clicked((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QPoint(*)>(_a[2]))); break;
        case 3: _t->typeCombo_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->documentColorList_currentChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->defectCombo_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->addButton_clicked(); break;
        case 7: _t->replaceButton_clicked(); break;
        case 8: _t->cancelButton_clicked(); break;
        case 9: _t->colorList_currentChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->cSpin_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->mSpin_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->ySpin_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->kSpin_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->rSpin_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->gSpin_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->bSpin_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->hSpin_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->sSpin_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->vSpin_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject CWDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_CWDialog.data,
    qt_meta_data_CWDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *CWDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *CWDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_CWDialog.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::CWDialog"))
        return static_cast< Ui::CWDialog*>(this);
    return QDialog::qt_metacast(_clname);
}

int CWDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 20;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
