/****************************************************************************
** Meta object code from reading C++ file 'colorlistbox.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/colorlistbox.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'colorlistbox.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ColorListBox_t {
    QByteArrayData data[20];
    char stringdata0[282];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ColorListBox_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ColorListBox_t qt_meta_stringdata_ColorListBox = {
    {
QT_MOC_LITERAL(0, 0, 12), // "ColorListBox"
QT_MOC_LITERAL(1, 13, 17), // "currentRowChanged"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 10), // "currentRow"
QT_MOC_LITERAL(4, 43, 18), // "currentTextChanged"
QT_MOC_LITERAL(5, 62, 11), // "currentText"
QT_MOC_LITERAL(6, 74, 20), // "contextMenuRequested"
QT_MOC_LITERAL(7, 95, 11), // "itemClicked"
QT_MOC_LITERAL(8, 107, 17), // "itemDoubleClicked"
QT_MOC_LITERAL(9, 125, 20), // "itemSelectionChanged"
QT_MOC_LITERAL(10, 146, 5), // "clear"
QT_MOC_LITERAL(11, 152, 14), // "slotRightClick"
QT_MOC_LITERAL(12, 167, 18), // "emitCurrentChanged"
QT_MOC_LITERAL(13, 186, 11), // "QModelIndex"
QT_MOC_LITERAL(14, 198, 7), // "current"
QT_MOC_LITERAL(15, 206, 8), // "previous"
QT_MOC_LITERAL(16, 215, 15), // "emitItemClicked"
QT_MOC_LITERAL(17, 231, 21), // "emitItemDoubleClicked"
QT_MOC_LITERAL(18, 253, 13), // "iconSetChange"
QT_MOC_LITERAL(19, 267, 14) // "languageChange"

    },
    "ColorListBox\0currentRowChanged\0\0"
    "currentRow\0currentTextChanged\0currentText\0"
    "contextMenuRequested\0itemClicked\0"
    "itemDoubleClicked\0itemSelectionChanged\0"
    "clear\0slotRightClick\0emitCurrentChanged\0"
    "QModelIndex\0current\0previous\0"
    "emitItemClicked\0emitItemDoubleClicked\0"
    "iconSetChange\0languageChange"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ColorListBox[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   79,    2, 0x06 /* Public */,
       4,    1,   82,    2, 0x06 /* Public */,
       6,    0,   85,    2, 0x06 /* Public */,
       7,    1,   86,    2, 0x06 /* Public */,
       8,    1,   89,    2, 0x06 /* Public */,
       9,    0,   92,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    0,   93,    2, 0x0a /* Public */,
      11,    0,   94,    2, 0x08 /* Private */,
      12,    2,   95,    2, 0x09 /* Protected */,
      16,    1,  100,    2, 0x09 /* Protected */,
      17,    1,  103,    2, 0x09 /* Protected */,
      18,    0,  106,    2, 0x09 /* Protected */,
      19,    0,  107,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13, 0x80000000 | 13,   14,   15,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ColorListBox::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ColorListBox *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->currentRowChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->currentTextChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->contextMenuRequested(); break;
        case 3: _t->itemClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->itemDoubleClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->itemSelectionChanged(); break;
        case 6: _t->clear(); break;
        case 7: _t->slotRightClick(); break;
        case 8: _t->emitCurrentChanged((*reinterpret_cast< const QModelIndex(*)>(_a[1])),(*reinterpret_cast< const QModelIndex(*)>(_a[2]))); break;
        case 9: _t->emitItemClicked((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 10: _t->emitItemDoubleClicked((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 11: _t->iconSetChange(); break;
        case 12: _t->languageChange(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ColorListBox::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorListBox::currentRowChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ColorListBox::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorListBox::currentTextChanged)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ColorListBox::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorListBox::contextMenuRequested)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ColorListBox::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorListBox::itemClicked)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ColorListBox::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorListBox::itemDoubleClicked)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (ColorListBox::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorListBox::itemSelectionChanged)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ColorListBox::staticMetaObject = { {
    QMetaObject::SuperData::link<QListView::staticMetaObject>(),
    qt_meta_stringdata_ColorListBox.data,
    qt_meta_data_ColorListBox,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ColorListBox::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ColorListBox::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ColorListBox.stringdata0))
        return static_cast<void*>(this);
    return QListView::qt_metacast(_clname);
}

int ColorListBox::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void ColorListBox::currentRowChanged(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ColorListBox::currentTextChanged(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ColorListBox::contextMenuRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void ColorListBox::itemClicked(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ColorListBox::itemDoubleClicked(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void ColorListBox::itemSelectionChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
