/********************************************************************************
** Form generated from reading UI file 'imagedialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_IMAGEDIALOG_H
#define UI_IMAGEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include "iview.h"

QT_BEGIN_NAMESPACE

class Ui_imagedialog
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *hboxLayout;
    QRadioButton *fitToWindowRadiobutton;
    QSpacerItem *spacerItem;
    QRadioButton *zoomRadiobutton;
    QSpinBox *zoomSpinbox;
    QSpacerItem *spacerItem1;
    QPushButton *showOriginalSizeButton;
    IView *pView;
    QSpacerItem *horizontalSpacer;
    QPushButton *pushButton;

    void setupUi(QDialog *imagedialog)
    {
        if (imagedialog->objectName().isEmpty())
            imagedialog->setObjectName(QString::fromUtf8("imagedialog"));
        imagedialog->resize(470, 316);
        gridLayout = new QGridLayout(imagedialog);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        fitToWindowRadiobutton = new QRadioButton(imagedialog);
        fitToWindowRadiobutton->setObjectName(QString::fromUtf8("fitToWindowRadiobutton"));
        fitToWindowRadiobutton->setChecked(true);

        hboxLayout->addWidget(fitToWindowRadiobutton);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        zoomRadiobutton = new QRadioButton(imagedialog);
        zoomRadiobutton->setObjectName(QString::fromUtf8("zoomRadiobutton"));

        hboxLayout->addWidget(zoomRadiobutton);

        zoomSpinbox = new QSpinBox(imagedialog);
        zoomSpinbox->setObjectName(QString::fromUtf8("zoomSpinbox"));
        zoomSpinbox->setEnabled(false);
        zoomSpinbox->setMinimum(1);
        zoomSpinbox->setMaximum(60000);
        zoomSpinbox->setValue(100);

        hboxLayout->addWidget(zoomSpinbox);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem1);

        showOriginalSizeButton = new QPushButton(imagedialog);
        showOriginalSizeButton->setObjectName(QString::fromUtf8("showOriginalSizeButton"));

        hboxLayout->addWidget(showOriginalSizeButton);


        gridLayout->addLayout(hboxLayout, 0, 0, 1, 2);

        pView = new IView(imagedialog);
        pView->setObjectName(QString::fromUtf8("pView"));

        gridLayout->addWidget(pView, 1, 0, 1, 2);

        horizontalSpacer = new QSpacerItem(387, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer, 2, 0, 1, 1);

        pushButton = new QPushButton(imagedialog);
        pushButton->setObjectName(QString::fromUtf8("pushButton"));

        gridLayout->addWidget(pushButton, 2, 1, 1, 1);


        retranslateUi(imagedialog);
        QObject::connect(pushButton, SIGNAL(clicked()), imagedialog, SLOT(close()));

        QMetaObject::connectSlotsByName(imagedialog);
    } // setupUi

    void retranslateUi(QDialog *imagedialog)
    {
        imagedialog->setWindowTitle(QCoreApplication::translate("imagedialog", "Dialog", nullptr));
        fitToWindowRadiobutton->setText(QCoreApplication::translate("imagedialog", "Fit to Window", nullptr));
        zoomRadiobutton->setText(QCoreApplication::translate("imagedialog", "Custom Zoom", nullptr));
#if QT_CONFIG(tooltip)
        zoomSpinbox->setToolTip(QCoreApplication::translate("imagedialog", "Current zoom level", nullptr));
#endif // QT_CONFIG(tooltip)
        zoomSpinbox->setSuffix(QCoreApplication::translate("imagedialog", "%", nullptr));
#if QT_CONFIG(tooltip)
        showOriginalSizeButton->setToolTip(QCoreApplication::translate("imagedialog", "Set zoom to 100%", nullptr));
#endif // QT_CONFIG(tooltip)
        showOriginalSizeButton->setText(QCoreApplication::translate("imagedialog", "Original Size", nullptr));
        pushButton->setText(QCoreApplication::translate("imagedialog", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class imagedialog: public Ui_imagedialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_IMAGEDIALOG_H
