/****************************************************************************
** Meta object code from reading C++ file 'propertiespalette_xyz.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/propertiespalette_xyz.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertiespalette_xyz.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PropertiesPalette_XYZ_t {
    QByteArrayData data[54];
    char stringdata0[646];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PropertiesPalette_XYZ_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PropertiesPalette_XYZ_t qt_meta_stringdata_PropertiesPalette_XYZ = {
    {
QT_MOC_LITERAL(0, 0, 21), // "PropertiesPalette_XYZ"
QT_MOC_LITERAL(1, 22, 13), // "setMainWindow"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 18), // "ScribusMainWindow*"
QT_MOC_LITERAL(4, 56, 2), // "mw"
QT_MOC_LITERAL(5, 59, 6), // "setDoc"
QT_MOC_LITERAL(6, 66, 11), // "ScribusDoc*"
QT_MOC_LITERAL(7, 78, 1), // "d"
QT_MOC_LITERAL(8, 80, 14), // "setCurrentItem"
QT_MOC_LITERAL(9, 95, 9), // "PageItem*"
QT_MOC_LITERAL(10, 105, 4), // "item"
QT_MOC_LITERAL(11, 110, 8), // "unsetDoc"
QT_MOC_LITERAL(12, 119, 9), // "unsetItem"
QT_MOC_LITERAL(13, 129, 13), // "iconSetChange"
QT_MOC_LITERAL(14, 143, 14), // "languageChange"
QT_MOC_LITERAL(15, 158, 10), // "unitChange"
QT_MOC_LITERAL(16, 169, 12), // "localeChange"
QT_MOC_LITERAL(17, 182, 6), // "showXY"
QT_MOC_LITERAL(18, 189, 1), // "x"
QT_MOC_LITERAL(19, 191, 1), // "y"
QT_MOC_LITERAL(20, 193, 6), // "showWH"
QT_MOC_LITERAL(21, 200, 12), // "showRotation"
QT_MOC_LITERAL(22, 213, 1), // "r"
QT_MOC_LITERAL(23, 215, 10), // "showLocked"
QT_MOC_LITERAL(24, 226, 14), // "showSizeLocked"
QT_MOC_LITERAL(25, 241, 19), // "showPrintingEnabled"
QT_MOC_LITERAL(26, 261, 12), // "showFlippedH"
QT_MOC_LITERAL(27, 274, 12), // "showFlippedV"
QT_MOC_LITERAL(28, 287, 20), // "handleAppModeChanged"
QT_MOC_LITERAL(29, 308, 7), // "oldMode"
QT_MOC_LITERAL(30, 316, 4), // "mode"
QT_MOC_LITERAL(31, 321, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(32, 344, 10), // "handleNewX"
QT_MOC_LITERAL(33, 355, 10), // "handleNewY"
QT_MOC_LITERAL(34, 366, 10), // "handleNewW"
QT_MOC_LITERAL(35, 377, 10), // "handleNewH"
QT_MOC_LITERAL(36, 388, 14), // "handleRotation"
QT_MOC_LITERAL(37, 403, 11), // "handleFlipH"
QT_MOC_LITERAL(38, 415, 11), // "handleFlipV"
QT_MOC_LITERAL(39, 427, 11), // "handleLower"
QT_MOC_LITERAL(40, 439, 11), // "handleRaise"
QT_MOC_LITERAL(41, 451, 11), // "handleFront"
QT_MOC_LITERAL(42, 463, 10), // "handleBack"
QT_MOC_LITERAL(43, 474, 15), // "handleBasePoint"
QT_MOC_LITERAL(44, 490, 1), // "m"
QT_MOC_LITERAL(45, 492, 10), // "handleLock"
QT_MOC_LITERAL(46, 503, 14), // "handleLockSize"
QT_MOC_LITERAL(47, 518, 11), // "handlePrint"
QT_MOC_LITERAL(48, 530, 13), // "handleNewName"
QT_MOC_LITERAL(49, 544, 14), // "handleGrouping"
QT_MOC_LITERAL(50, 559, 16), // "handleUngrouping"
QT_MOC_LITERAL(51, 576, 22), // "spinboxStartUserAction"
QT_MOC_LITERAL(52, 599, 23), // "spinboxFinishUserAction"
QT_MOC_LITERAL(53, 623, 22) // "updateSpinBoxConstants"

    },
    "PropertiesPalette_XYZ\0setMainWindow\0"
    "\0ScribusMainWindow*\0mw\0setDoc\0ScribusDoc*\0"
    "d\0setCurrentItem\0PageItem*\0item\0"
    "unsetDoc\0unsetItem\0iconSetChange\0"
    "languageChange\0unitChange\0localeChange\0"
    "showXY\0x\0y\0showWH\0showRotation\0r\0"
    "showLocked\0showSizeLocked\0showPrintingEnabled\0"
    "showFlippedH\0showFlippedV\0"
    "handleAppModeChanged\0oldMode\0mode\0"
    "handleSelectionChanged\0handleNewX\0"
    "handleNewY\0handleNewW\0handleNewH\0"
    "handleRotation\0handleFlipH\0handleFlipV\0"
    "handleLower\0handleRaise\0handleFront\0"
    "handleBack\0handleBasePoint\0m\0handleLock\0"
    "handleLockSize\0handlePrint\0handleNewName\0"
    "handleGrouping\0handleUngrouping\0"
    "spinboxStartUserAction\0spinboxFinishUserAction\0"
    "updateSpinBoxConstants"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PropertiesPalette_XYZ[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      40,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  214,    2, 0x0a /* Public */,
       5,    1,  217,    2, 0x0a /* Public */,
       8,    1,  220,    2, 0x0a /* Public */,
      11,    0,  223,    2, 0x0a /* Public */,
      12,    0,  224,    2, 0x0a /* Public */,
      13,    0,  225,    2, 0x0a /* Public */,
      14,    0,  226,    2, 0x0a /* Public */,
      15,    0,  227,    2, 0x0a /* Public */,
      16,    0,  228,    2, 0x0a /* Public */,
      17,    2,  229,    2, 0x0a /* Public */,
      20,    2,  234,    2, 0x0a /* Public */,
      21,    1,  239,    2, 0x0a /* Public */,
      23,    1,  242,    2, 0x0a /* Public */,
      24,    1,  245,    2, 0x0a /* Public */,
      25,    1,  248,    2, 0x0a /* Public */,
      26,    1,  251,    2, 0x0a /* Public */,
      27,    1,  254,    2, 0x0a /* Public */,
      28,    2,  257,    2, 0x0a /* Public */,
      31,    0,  262,    2, 0x0a /* Public */,
      32,    0,  263,    2, 0x08 /* Private */,
      33,    0,  264,    2, 0x08 /* Private */,
      34,    0,  265,    2, 0x08 /* Private */,
      35,    0,  266,    2, 0x08 /* Private */,
      36,    0,  267,    2, 0x08 /* Private */,
      37,    0,  268,    2, 0x08 /* Private */,
      38,    0,  269,    2, 0x08 /* Private */,
      39,    0,  270,    2, 0x08 /* Private */,
      40,    0,  271,    2, 0x08 /* Private */,
      41,    0,  272,    2, 0x08 /* Private */,
      42,    0,  273,    2, 0x08 /* Private */,
      43,    1,  274,    2, 0x08 /* Private */,
      45,    0,  277,    2, 0x08 /* Private */,
      46,    0,  278,    2, 0x08 /* Private */,
      47,    0,  279,    2, 0x08 /* Private */,
      48,    0,  280,    2, 0x08 /* Private */,
      49,    0,  281,    2, 0x08 /* Private */,
      50,    0,  282,    2, 0x08 /* Private */,
      51,    0,  283,    2, 0x09 /* Protected */,
      52,    0,  284,    2, 0x09 /* Protected */,
      53,    0,  285,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,   18,   19,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,   18,   19,
    QMetaType::Void, QMetaType::Double,   22,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   29,   30,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   44,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PropertiesPalette_XYZ::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertiesPalette_XYZ *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setMainWindow((*reinterpret_cast< ScribusMainWindow*(*)>(_a[1]))); break;
        case 1: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 2: _t->setCurrentItem((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 3: _t->unsetDoc(); break;
        case 4: _t->unsetItem(); break;
        case 5: _t->iconSetChange(); break;
        case 6: _t->languageChange(); break;
        case 7: _t->unitChange(); break;
        case 8: _t->localeChange(); break;
        case 9: _t->showXY((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 10: _t->showWH((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 11: _t->showRotation((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 12: _t->showLocked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 13: _t->showSizeLocked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->showPrintingEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 15: _t->showFlippedH((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->showFlippedV((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 17: _t->handleAppModeChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 18: _t->handleSelectionChanged(); break;
        case 19: _t->handleNewX(); break;
        case 20: _t->handleNewY(); break;
        case 21: _t->handleNewW(); break;
        case 22: _t->handleNewH(); break;
        case 23: _t->handleRotation(); break;
        case 24: _t->handleFlipH(); break;
        case 25: _t->handleFlipV(); break;
        case 26: _t->handleLower(); break;
        case 27: _t->handleRaise(); break;
        case 28: _t->handleFront(); break;
        case 29: _t->handleBack(); break;
        case 30: _t->handleBasePoint((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 31: _t->handleLock(); break;
        case 32: _t->handleLockSize(); break;
        case 33: _t->handlePrint(); break;
        case 34: _t->handleNewName(); break;
        case 35: _t->handleGrouping(); break;
        case 36: _t->handleUngrouping(); break;
        case 37: _t->spinboxStartUserAction(); break;
        case 38: _t->spinboxFinishUserAction(); break;
        case 39: _t->updateSpinBoxConstants(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PropertiesPalette_XYZ::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_PropertiesPalette_XYZ.data,
    qt_meta_data_PropertiesPalette_XYZ,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PropertiesPalette_XYZ::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PropertiesPalette_XYZ::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PropertiesPalette_XYZ.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PropertiesPalette_XYZBase"))
        return static_cast< Ui::PropertiesPalette_XYZBase*>(this);
    return QWidget::qt_metacast(_clname);
}

int PropertiesPalette_XYZ::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 40)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 40;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 40)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 40;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
