/********************************************************************************
** Form generated from reading UI file 'prefs_keyboardshortcutsbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_KEYBOARDSHORTCUTSBASE_H
#define UI_PREFS_KEYBOARDSHORTCUTSBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Prefs_KeyboardShortcuts
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QFrame *line;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *hboxLayout;
    QPushButton *clearSearchButton;
    QLabel *textLabel2;
    QLineEdit *searchTextLineEdit;
    QTreeWidget *keyTable;
    QSpacerItem *verticalSpacer_2;
    QLabel *label_3;
    QFrame *line_3;
    QHBoxLayout *horizontalLayout;
    QRadioButton *noKey;
    QRadioButton *userDef;
    QSpacerItem *spacerItem;
    QPushButton *setKeyButton;
    QLabel *keyDisplay;
    QSpacerItem *verticalSpacer;
    QLabel *label_4;
    QFrame *line_4;
    QHBoxLayout *horizontalLayout_2;
    QComboBox *loadableSets;
    QPushButton *loadSetButton;
    QSpacerItem *spacerItem1;
    QPushButton *importSetButton;
    QPushButton *exportSetButton;
    QSpacerItem *spacerItem2;
    QPushButton *resetSetButton;

    void setupUi(QWidget *Prefs_KeyboardShortcuts)
    {
        if (Prefs_KeyboardShortcuts->objectName().isEmpty())
            Prefs_KeyboardShortcuts->setObjectName(QString::fromUtf8("Prefs_KeyboardShortcuts"));
        Prefs_KeyboardShortcuts->resize(815, 572);
        verticalLayout = new QVBoxLayout(Prefs_KeyboardShortcuts);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(Prefs_KeyboardShortcuts);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        verticalLayout->addWidget(label);

        line = new QFrame(Prefs_KeyboardShortcuts);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        scrollArea = new QScrollArea(Prefs_KeyboardShortcuts);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 789, 504));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        clearSearchButton = new QPushButton(scrollAreaWidgetContents);
        clearSearchButton->setObjectName(QString::fromUtf8("clearSearchButton"));

        hboxLayout->addWidget(clearSearchButton);

        textLabel2 = new QLabel(scrollAreaWidgetContents);
        textLabel2->setObjectName(QString::fromUtf8("textLabel2"));
        textLabel2->setWordWrap(false);

        hboxLayout->addWidget(textLabel2);

        searchTextLineEdit = new QLineEdit(scrollAreaWidgetContents);
        searchTextLineEdit->setObjectName(QString::fromUtf8("searchTextLineEdit"));

        hboxLayout->addWidget(searchTextLineEdit);


        verticalLayout_2->addLayout(hboxLayout);

        keyTable = new QTreeWidget(scrollAreaWidgetContents);
        keyTable->setObjectName(QString::fromUtf8("keyTable"));

        verticalLayout_2->addWidget(keyTable);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_2->addItem(verticalSpacer_2);

        label_3 = new QLabel(scrollAreaWidgetContents);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
        label_3->setFont(font1);

        verticalLayout_2->addWidget(label_3);

        line_3 = new QFrame(scrollAreaWidgetContents);
        line_3->setObjectName(QString::fromUtf8("line_3"));
        line_3->setFrameShape(QFrame::HLine);
        line_3->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line_3);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        noKey = new QRadioButton(scrollAreaWidgetContents);
        noKey->setObjectName(QString::fromUtf8("noKey"));

        horizontalLayout->addWidget(noKey);

        userDef = new QRadioButton(scrollAreaWidgetContents);
        userDef->setObjectName(QString::fromUtf8("userDef"));

        horizontalLayout->addWidget(userDef);

        spacerItem = new QSpacerItem(158, 18, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(spacerItem);

        setKeyButton = new QPushButton(scrollAreaWidgetContents);
        setKeyButton->setObjectName(QString::fromUtf8("setKeyButton"));
        setKeyButton->setCheckable(true);

        horizontalLayout->addWidget(setKeyButton);

        keyDisplay = new QLabel(scrollAreaWidgetContents);
        keyDisplay->setObjectName(QString::fromUtf8("keyDisplay"));
        keyDisplay->setFrameShape(QFrame::Panel);
        keyDisplay->setFrameShadow(QFrame::Sunken);
        keyDisplay->setTextFormat(Qt::RichText);
        keyDisplay->setAlignment(Qt::AlignCenter);
        keyDisplay->setWordWrap(false);

        horizontalLayout->addWidget(keyDisplay);


        verticalLayout_2->addLayout(horizontalLayout);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_2->addItem(verticalSpacer);

        label_4 = new QLabel(scrollAreaWidgetContents);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        label_4->setFont(font1);

        verticalLayout_2->addWidget(label_4);

        line_4 = new QFrame(scrollAreaWidgetContents);
        line_4->setObjectName(QString::fromUtf8("line_4"));
        line_4->setFrameShape(QFrame::HLine);
        line_4->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line_4);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        loadableSets = new QComboBox(scrollAreaWidgetContents);
        loadableSets->setObjectName(QString::fromUtf8("loadableSets"));

        horizontalLayout_2->addWidget(loadableSets);

        loadSetButton = new QPushButton(scrollAreaWidgetContents);
        loadSetButton->setObjectName(QString::fromUtf8("loadSetButton"));

        horizontalLayout_2->addWidget(loadSetButton);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(spacerItem1);

        importSetButton = new QPushButton(scrollAreaWidgetContents);
        importSetButton->setObjectName(QString::fromUtf8("importSetButton"));

        horizontalLayout_2->addWidget(importSetButton);

        exportSetButton = new QPushButton(scrollAreaWidgetContents);
        exportSetButton->setObjectName(QString::fromUtf8("exportSetButton"));

        horizontalLayout_2->addWidget(exportSetButton);

        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(spacerItem2);

        resetSetButton = new QPushButton(scrollAreaWidgetContents);
        resetSetButton->setObjectName(QString::fromUtf8("resetSetButton"));

        horizontalLayout_2->addWidget(resetSetButton);


        verticalLayout_2->addLayout(horizontalLayout_2);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout->addWidget(scrollArea);

        QWidget::setTabOrder(clearSearchButton, searchTextLineEdit);
        QWidget::setTabOrder(searchTextLineEdit, keyTable);

        retranslateUi(Prefs_KeyboardShortcuts);

        QMetaObject::connectSlotsByName(Prefs_KeyboardShortcuts);
    } // setupUi

    void retranslateUi(QWidget *Prefs_KeyboardShortcuts)
    {
        Prefs_KeyboardShortcuts->setWindowTitle(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Keyboard Shortcuts", nullptr));
        label->setText(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Keyboard Shortcuts", nullptr));
        clearSearchButton->setText(QString());
        textLabel2->setText(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Search:", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = keyTable->headerItem();
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("Prefs_KeyboardShortcuts", "Shortcut", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("Prefs_KeyboardShortcuts", "Action", nullptr));
        label_3->setText(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Shortcut for Selected Action", nullptr));
        noKey->setText(QCoreApplication::translate("Prefs_KeyboardShortcuts", "&No Key", nullptr));
#if QT_CONFIG(shortcut)
        noKey->setShortcut(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Alt+N", nullptr));
#endif // QT_CONFIG(shortcut)
        userDef->setText(QCoreApplication::translate("Prefs_KeyboardShortcuts", "&User Defined Key", nullptr));
#if QT_CONFIG(shortcut)
        userDef->setShortcut(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Alt+U", nullptr));
#endif // QT_CONFIG(shortcut)
        setKeyButton->setText(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Set &Key", nullptr));
#if QT_CONFIG(shortcut)
        setKeyButton->setShortcut(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Alt+K", nullptr));
#endif // QT_CONFIG(shortcut)
        keyDisplay->setText(QCoreApplication::translate("Prefs_KeyboardShortcuts", "CTRL+ALT+SHIFT+W", nullptr));
        label_4->setText(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Loadable Shortcut Sets", nullptr));
#if QT_CONFIG(tooltip)
        loadableSets->setToolTip(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Keyboard shortcut sets available to load", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        loadSetButton->setToolTip(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Load the selected shortcut set", nullptr));
#endif // QT_CONFIG(tooltip)
        loadSetButton->setText(QCoreApplication::translate("Prefs_KeyboardShortcuts", "&Load", nullptr));
#if QT_CONFIG(shortcut)
        loadSetButton->setShortcut(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Alt+L", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        importSetButton->setToolTip(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Import a shortcut set into the current configuration", nullptr));
#endif // QT_CONFIG(tooltip)
        importSetButton->setText(QCoreApplication::translate("Prefs_KeyboardShortcuts", "&Import...", nullptr));
#if QT_CONFIG(shortcut)
        importSetButton->setShortcut(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Alt+I", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        exportSetButton->setToolTip(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Export the current shortcuts into an importable file", nullptr));
#endif // QT_CONFIG(tooltip)
        exportSetButton->setText(QCoreApplication::translate("Prefs_KeyboardShortcuts", "&Export...", nullptr));
#if QT_CONFIG(shortcut)
        exportSetButton->setShortcut(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Alt+E", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        resetSetButton->setToolTip(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Reload the default Scribus shortcuts", nullptr));
#endif // QT_CONFIG(tooltip)
        resetSetButton->setText(QCoreApplication::translate("Prefs_KeyboardShortcuts", "&Reset", nullptr));
#if QT_CONFIG(shortcut)
        resetSetButton->setShortcut(QCoreApplication::translate("Prefs_KeyboardShortcuts", "Alt+R", nullptr));
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class Prefs_KeyboardShortcuts: public Ui_Prefs_KeyboardShortcuts {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_KEYBOARDSHORTCUTSBASE_H
