/********************************************************************************
** Form generated from reading UI file 'contentpalette_defaultbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CONTENTPALETTE_DEFAULTBASE_H
#define UI_CONTENTPALETTE_DEFAULTBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_ContentPalette_DefaultBase
{
public:
    QVBoxLayout *verticalLayout_3;
    QLabel *label;

    void setupUi(QWidget *ContentPalette_DefaultBase)
    {
        if (ContentPalette_DefaultBase->objectName().isEmpty())
            ContentPalette_DefaultBase->setObjectName(QString::fromUtf8("ContentPalette_DefaultBase"));
        ContentPalette_DefaultBase->resize(407, 230);
        verticalLayout_3 = new QVBoxLayout(ContentPalette_DefaultBase);
        verticalLayout_3->setSpacing(3);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(3, 3, 3, 3);
        label = new QLabel(ContentPalette_DefaultBase);
        label->setObjectName(QString::fromUtf8("label"));
        label->setAlignment(Qt::AlignCenter);

        verticalLayout_3->addWidget(label);


        retranslateUi(ContentPalette_DefaultBase);

        QMetaObject::connectSlotsByName(ContentPalette_DefaultBase);
    } // setupUi

    void retranslateUi(QWidget *ContentPalette_DefaultBase)
    {
        label->setText(QCoreApplication::translate("ContentPalette_DefaultBase", "<html><head/><body><p align=\"center\">Open a document</p><p align=\"center\">and select an item to see its properties</p></body></html>", nullptr));
        (void)ContentPalette_DefaultBase;
    } // retranslateUi

};

namespace Ui {
    class ContentPalette_DefaultBase: public Ui_ContentPalette_DefaultBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CONTENTPALETTE_DEFAULTBASE_H
