/****************************************************************************
** Meta object code from reading C++ file 'resourcemanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/resourcemanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'resourcemanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ResourceManager_t {
    QByteArrayData data[18];
    char stringdata0[321];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ResourceManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ResourceManager_t qt_meta_stringdata_ResourceManager = {
    {
QT_MOC_LITERAL(0, 0, 15), // "ResourceManager"
QT_MOC_LITERAL(1, 16, 14), // "languageChange"
QT_MOC_LITERAL(2, 31, 0), // ""
QT_MOC_LITERAL(3, 32, 15), // "categoryChanged"
QT_MOC_LITERAL(4, 48, 19), // "updateDownloadLists"
QT_MOC_LITERAL(5, 68, 20), // "downloadListFinished"
QT_MOC_LITERAL(6, 89, 21), // "downloadFilesFinished"
QT_MOC_LITERAL(7, 111, 23), // "downloadLicenseFinished"
QT_MOC_LITERAL(8, 135, 27), // "downloadLicenseFileFinished"
QT_MOC_LITERAL(9, 163, 25), // "downloadLicenseFileFailed"
QT_MOC_LITERAL(10, 189, 25), // "installedSelectionChanged"
QT_MOC_LITERAL(11, 215, 17), // "updateProgressBar"
QT_MOC_LITERAL(12, 233, 18), // "updateProgressData"
QT_MOC_LITERAL(13, 252, 13), // "bytesReceived"
QT_MOC_LITERAL(14, 266, 10), // "bytesTotal"
QT_MOC_LITERAL(15, 277, 13), // "startDownload"
QT_MOC_LITERAL(16, 291, 17), // "showInFileBrowser"
QT_MOC_LITERAL(17, 309, 11) // "showLicense"

    },
    "ResourceManager\0languageChange\0\0"
    "categoryChanged\0updateDownloadLists\0"
    "downloadListFinished\0downloadFilesFinished\0"
    "downloadLicenseFinished\0"
    "downloadLicenseFileFinished\0"
    "downloadLicenseFileFailed\0"
    "installedSelectionChanged\0updateProgressBar\0"
    "updateProgressData\0bytesReceived\0"
    "bytesTotal\0startDownload\0showInFileBrowser\0"
    "showLicense"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ResourceManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   84,    2, 0x0a /* Public */,
       3,    0,   85,    2, 0x09 /* Protected */,
       4,    0,   86,    2, 0x09 /* Protected */,
       5,    0,   87,    2, 0x09 /* Protected */,
       6,    0,   88,    2, 0x09 /* Protected */,
       7,    0,   89,    2, 0x09 /* Protected */,
       8,    1,   90,    2, 0x09 /* Protected */,
       9,    1,   93,    2, 0x09 /* Protected */,
      10,    0,   96,    2, 0x09 /* Protected */,
      11,    0,   97,    2, 0x09 /* Protected */,
      12,    2,   98,    2, 0x09 /* Protected */,
      15,    0,  103,    2, 0x09 /* Protected */,
      16,    0,  104,    2, 0x09 /* Protected */,
      17,    0,  105,    2, 0x09 /* Protected */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::LongLong, QMetaType::LongLong,   13,   14,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ResourceManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ResourceManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->languageChange(); break;
        case 1: _t->categoryChanged(); break;
        case 2: _t->updateDownloadLists(); break;
        case 3: _t->downloadListFinished(); break;
        case 4: _t->downloadFilesFinished(); break;
        case 5: _t->downloadLicenseFinished(); break;
        case 6: _t->downloadLicenseFileFinished((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->downloadLicenseFileFailed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 8: _t->installedSelectionChanged(); break;
        case 9: _t->updateProgressBar(); break;
        case 10: _t->updateProgressData((*reinterpret_cast< qint64(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2]))); break;
        case 11: _t->startDownload(); break;
        case 12: _t->showInFileBrowser(); break;
        case 13: _t->showLicense(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ResourceManager::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_ResourceManager.data,
    qt_meta_data_ResourceManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ResourceManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ResourceManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ResourceManager.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::ResourceManagerBase"))
        return static_cast< Ui::ResourceManagerBase*>(this);
    return QDialog::qt_metacast(_clname);
}

int ResourceManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
