/****************************************************************************
** Meta object code from reading C++ file 'picturebrowser.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../scribus-1.6.6/scribus/plugins/picbrowser/picturebrowser.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'picturebrowser.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PictureBrowser_t {
    QByteArrayData data[67];
    char stringdata0[1566];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PictureBrowser_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PictureBrowser_t qt_meta_stringdata_PictureBrowser = {
    {
QT_MOC_LITERAL(0, 0, 14), // "PictureBrowser"
QT_MOC_LITERAL(1, 15, 12), // "loadImageJob"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 10), // "selectPage"
QT_MOC_LITERAL(4, 40, 16), // "selectMasterPage"
QT_MOC_LITERAL(5, 57, 10), // "unitChange"
QT_MOC_LITERAL(6, 68, 13), // "iconSetChange"
QT_MOC_LITERAL(7, 82, 8), // "navigate"
QT_MOC_LITERAL(8, 91, 5), // "index"
QT_MOC_LITERAL(9, 97, 18), // "previewIconClicked"
QT_MOC_LITERAL(10, 116, 11), // "QModelIndex"
QT_MOC_LITERAL(11, 128, 24), // "previewIconDoubleClicked"
QT_MOC_LITERAL(12, 153, 11), // "sortChanged"
QT_MOC_LITERAL(13, 165, 18), // "previewModeChanged"
QT_MOC_LITERAL(14, 184, 17), // "moreButtonClicked"
QT_MOC_LITERAL(15, 202, 26), // "resetSettingsButtonClicked"
QT_MOC_LITERAL(16, 229, 22), // "actionsGoButtonClicked"
QT_MOC_LITERAL(17, 252, 22), // "sortOrderButtonClicked"
QT_MOC_LITERAL(18, 275, 21), // "zoomPlusButtonClicked"
QT_MOC_LITERAL(19, 297, 22), // "zoomMinusButtonClicked"
QT_MOC_LITERAL(20, 320, 23), // "tabWidgetCurrentChanged"
QT_MOC_LITERAL(21, 344, 21), // "gotoPageButtonClicked"
QT_MOC_LITERAL(22, 366, 31), // "alwaysOnTopCheckboxStateChanged"
QT_MOC_LITERAL(23, 398, 32), // "saveSettingsCheckboxStateChanged"
QT_MOC_LITERAL(24, 431, 27), // "subdirsCheckboxStateChanged"
QT_MOC_LITERAL(25, 459, 9), // "dirChosen"
QT_MOC_LITERAL(26, 469, 14), // "documentChosen"
QT_MOC_LITERAL(27, 484, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(28, 501, 4), // "item"
QT_MOC_LITERAL(29, 506, 6), // "column"
QT_MOC_LITERAL(30, 513, 16), // "collectionChosen"
QT_MOC_LITERAL(31, 530, 27), // "collectionsWidgetItemEdited"
QT_MOC_LITERAL(32, 558, 24), // "findImagesThreadFinished"
QT_MOC_LITERAL(33, 583, 30), // "collectionReaderThreadFinished"
QT_MOC_LITERAL(34, 614, 34), // "collectionListReaderThreadFin..."
QT_MOC_LITERAL(35, 649, 38), // "collectionReaderThreadListFin..."
QT_MOC_LITERAL(36, 688, 33), // "collectionsDbWriterThreadFini..."
QT_MOC_LITERAL(37, 722, 34), // "collectionWriterThreadListFin..."
QT_MOC_LITERAL(38, 757, 36), // "insertPagesComboboxCheckstate..."
QT_MOC_LITERAL(39, 794, 3), // "row"
QT_MOC_LITERAL(40, 798, 24), // "insertImageButtonClicked"
QT_MOC_LITERAL(41, 823, 29), // "insertPositionComboboxChanged"
QT_MOC_LITERAL(42, 853, 25), // "insertSizeComboboxChanged"
QT_MOC_LITERAL(43, 879, 29), // "filterCriteriaComboboxChanged"
QT_MOC_LITERAL(44, 909, 27), // "filterTargetComboboxChanged"
QT_MOC_LITERAL(45, 937, 25), // "filterFilterButtonClicked"
QT_MOC_LITERAL(46, 963, 24), // "filterClearButtonClicked"
QT_MOC_LITERAL(47, 988, 31), // "filterAddCriterionButtonClicked"
QT_MOC_LITERAL(48, 1020, 28), // "filterSearchDirButtonClicked"
QT_MOC_LITERAL(49, 1049, 35), // "collectionsNewCategoryButtonC..."
QT_MOC_LITERAL(50, 1085, 27), // "collectionsNewButtonClicked"
QT_MOC_LITERAL(51, 1113, 30), // "collectionsImportButtonClicked"
QT_MOC_LITERAL(52, 1144, 30), // "collectionsExportButtonClicked"
QT_MOC_LITERAL(53, 1175, 30), // "collectionsDeleteButtonClicked"
QT_MOC_LITERAL(54, 1206, 30), // "collectionsRenameButtonClicked"
QT_MOC_LITERAL(55, 1237, 33), // "collectionsAddImagesButtonCli..."
QT_MOC_LITERAL(56, 1271, 36), // "collectionsRemoveImagesButton..."
QT_MOC_LITERAL(57, 1308, 33), // "collectionsTagImagesButtonCli..."
QT_MOC_LITERAL(58, 1342, 35), // "collectionsAddImagesOkButtonC..."
QT_MOC_LITERAL(59, 1378, 39), // "collectionsAddImagesCancelBut..."
QT_MOC_LITERAL(60, 1418, 28), // "previewImageSelectionChanged"
QT_MOC_LITERAL(61, 1447, 14), // "QItemSelection"
QT_MOC_LITERAL(62, 1462, 8), // "selected"
QT_MOC_LITERAL(63, 1471, 10), // "deselected"
QT_MOC_LITERAL(64, 1482, 31), // "collectionsSetTagsButtonClicked"
QT_MOC_LITERAL(65, 1514, 33), // "collectionsAddNewTagButtonCli..."
QT_MOC_LITERAL(66, 1548, 17) // "jumpToImageFolder"

    },
    "PictureBrowser\0loadImageJob\0\0selectPage\0"
    "selectMasterPage\0unitChange\0iconSetChange\0"
    "navigate\0index\0previewIconClicked\0"
    "QModelIndex\0previewIconDoubleClicked\0"
    "sortChanged\0previewModeChanged\0"
    "moreButtonClicked\0resetSettingsButtonClicked\0"
    "actionsGoButtonClicked\0sortOrderButtonClicked\0"
    "zoomPlusButtonClicked\0zoomMinusButtonClicked\0"
    "tabWidgetCurrentChanged\0gotoPageButtonClicked\0"
    "alwaysOnTopCheckboxStateChanged\0"
    "saveSettingsCheckboxStateChanged\0"
    "subdirsCheckboxStateChanged\0dirChosen\0"
    "documentChosen\0QTreeWidgetItem*\0item\0"
    "column\0collectionChosen\0"
    "collectionsWidgetItemEdited\0"
    "findImagesThreadFinished\0"
    "collectionReaderThreadFinished\0"
    "collectionListReaderThreadFinished\0"
    "collectionReaderThreadListFinishedSave\0"
    "collectionsDbWriterThreadFinished\0"
    "collectionWriterThreadListFinished\0"
    "insertPagesComboboxCheckstateChanged\0"
    "row\0insertImageButtonClicked\0"
    "insertPositionComboboxChanged\0"
    "insertSizeComboboxChanged\0"
    "filterCriteriaComboboxChanged\0"
    "filterTargetComboboxChanged\0"
    "filterFilterButtonClicked\0"
    "filterClearButtonClicked\0"
    "filterAddCriterionButtonClicked\0"
    "filterSearchDirButtonClicked\0"
    "collectionsNewCategoryButtonClicked\0"
    "collectionsNewButtonClicked\0"
    "collectionsImportButtonClicked\0"
    "collectionsExportButtonClicked\0"
    "collectionsDeleteButtonClicked\0"
    "collectionsRenameButtonClicked\0"
    "collectionsAddImagesButtonClicked\0"
    "collectionsRemoveImagesButtonClicked\0"
    "collectionsTagImagesButtonClicked\0"
    "collectionsAddImagesOkButtonClicked\0"
    "collectionsAddImagesCancelButtonClicked\0"
    "previewImageSelectionChanged\0"
    "QItemSelection\0selected\0deselected\0"
    "collectionsSetTagsButtonClicked\0"
    "collectionsAddNewTagButtonClicked\0"
    "jumpToImageFolder"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PictureBrowser[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      56,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    4,  294,    2, 0x06 /* Public */,
       3,    1,  303,    2, 0x06 /* Public */,
       4,    1,  306,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,  309,    2, 0x0a /* Public */,
       6,    0,  310,    2, 0x08 /* Private */,
       7,    1,  311,    2, 0x08 /* Private */,
       9,    1,  314,    2, 0x08 /* Private */,
      11,    1,  317,    2, 0x08 /* Private */,
      12,    1,  320,    2, 0x08 /* Private */,
      13,    1,  323,    2, 0x08 /* Private */,
      14,    0,  326,    2, 0x08 /* Private */,
      15,    0,  327,    2, 0x08 /* Private */,
      16,    0,  328,    2, 0x08 /* Private */,
      17,    0,  329,    2, 0x08 /* Private */,
      18,    0,  330,    2, 0x08 /* Private */,
      19,    0,  331,    2, 0x08 /* Private */,
      20,    1,  332,    2, 0x08 /* Private */,
      21,    0,  335,    2, 0x08 /* Private */,
      22,    0,  336,    2, 0x08 /* Private */,
      23,    0,  337,    2, 0x08 /* Private */,
      24,    0,  338,    2, 0x08 /* Private */,
      25,    1,  339,    2, 0x08 /* Private */,
      26,    2,  342,    2, 0x08 /* Private */,
      30,    2,  347,    2, 0x08 /* Private */,
      31,    2,  352,    2, 0x08 /* Private */,
      32,    0,  357,    2, 0x08 /* Private */,
      33,    0,  358,    2, 0x08 /* Private */,
      34,    0,  359,    2, 0x08 /* Private */,
      35,    0,  360,    2, 0x08 /* Private */,
      36,    0,  361,    2, 0x08 /* Private */,
      37,    0,  362,    2, 0x08 /* Private */,
      38,    1,  363,    2, 0x08 /* Private */,
      40,    0,  366,    2, 0x08 /* Private */,
      41,    1,  367,    2, 0x08 /* Private */,
      42,    1,  370,    2, 0x08 /* Private */,
      43,    1,  373,    2, 0x08 /* Private */,
      44,    1,  376,    2, 0x08 /* Private */,
      45,    0,  379,    2, 0x08 /* Private */,
      46,    0,  380,    2, 0x08 /* Private */,
      47,    0,  381,    2, 0x08 /* Private */,
      48,    0,  382,    2, 0x08 /* Private */,
      49,    0,  383,    2, 0x08 /* Private */,
      50,    0,  384,    2, 0x08 /* Private */,
      51,    0,  385,    2, 0x08 /* Private */,
      52,    0,  386,    2, 0x08 /* Private */,
      53,    0,  387,    2, 0x08 /* Private */,
      54,    0,  388,    2, 0x08 /* Private */,
      55,    0,  389,    2, 0x08 /* Private */,
      56,    0,  390,    2, 0x08 /* Private */,
      57,    0,  391,    2, 0x08 /* Private */,
      58,    0,  392,    2, 0x08 /* Private */,
      59,    0,  393,    2, 0x08 /* Private */,
      60,    2,  394,    2, 0x08 /* Private */,
      64,    0,  399,    2, 0x08 /* Private */,
      65,    0,  400,    2, 0x08 /* Private */,
      66,    0,  401,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::QString, QMetaType::Int, QMetaType::Int,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, 0x80000000 | 10,    8,
    QMetaType::Void, 0x80000000 | 10,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,    8,
    QMetaType::Void, 0x80000000 | 27, QMetaType::Int,   28,   29,
    QMetaType::Void, 0x80000000 | 27, QMetaType::Int,   28,   29,
    QMetaType::Void, 0x80000000 | 27, QMetaType::Int,   28,   29,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   39,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void, QMetaType::Int,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 61, 0x80000000 | 61,   62,   63,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PictureBrowser::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PictureBrowser *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->loadImageJob((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 1: _t->selectPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->selectMasterPage((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 3: _t->unitChange(); break;
        case 4: _t->iconSetChange(); break;
        case 5: _t->navigate((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->previewIconClicked((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 7: _t->previewIconDoubleClicked((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 8: _t->sortChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->previewModeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->moreButtonClicked(); break;
        case 11: _t->resetSettingsButtonClicked(); break;
        case 12: _t->actionsGoButtonClicked(); break;
        case 13: _t->sortOrderButtonClicked(); break;
        case 14: _t->zoomPlusButtonClicked(); break;
        case 15: _t->zoomMinusButtonClicked(); break;
        case 16: _t->tabWidgetCurrentChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->gotoPageButtonClicked(); break;
        case 18: _t->alwaysOnTopCheckboxStateChanged(); break;
        case 19: _t->saveSettingsCheckboxStateChanged(); break;
        case 20: _t->subdirsCheckboxStateChanged(); break;
        case 21: _t->dirChosen((*reinterpret_cast< const QModelIndex(*)>(_a[1]))); break;
        case 22: _t->documentChosen((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 23: _t->collectionChosen((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 24: _t->collectionsWidgetItemEdited((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 25: _t->findImagesThreadFinished(); break;
        case 26: _t->collectionReaderThreadFinished(); break;
        case 27: _t->collectionListReaderThreadFinished(); break;
        case 28: _t->collectionReaderThreadListFinishedSave(); break;
        case 29: _t->collectionsDbWriterThreadFinished(); break;
        case 30: _t->collectionWriterThreadListFinished(); break;
        case 31: _t->insertPagesComboboxCheckstateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 32: _t->insertImageButtonClicked(); break;
        case 33: _t->insertPositionComboboxChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 34: _t->insertSizeComboboxChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 35: _t->filterCriteriaComboboxChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 36: _t->filterTargetComboboxChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 37: _t->filterFilterButtonClicked(); break;
        case 38: _t->filterClearButtonClicked(); break;
        case 39: _t->filterAddCriterionButtonClicked(); break;
        case 40: _t->filterSearchDirButtonClicked(); break;
        case 41: _t->collectionsNewCategoryButtonClicked(); break;
        case 42: _t->collectionsNewButtonClicked(); break;
        case 43: _t->collectionsImportButtonClicked(); break;
        case 44: _t->collectionsExportButtonClicked(); break;
        case 45: _t->collectionsDeleteButtonClicked(); break;
        case 46: _t->collectionsRenameButtonClicked(); break;
        case 47: _t->collectionsAddImagesButtonClicked(); break;
        case 48: _t->collectionsRemoveImagesButtonClicked(); break;
        case 49: _t->collectionsTagImagesButtonClicked(); break;
        case 50: _t->collectionsAddImagesOkButtonClicked(); break;
        case 51: _t->collectionsAddImagesCancelButtonClicked(); break;
        case 52: _t->previewImageSelectionChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 53: _t->collectionsSetTagsButtonClicked(); break;
        case 54: _t->collectionsAddNewTagButtonClicked(); break;
        case 55: _t->jumpToImageFolder(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PictureBrowser::*)(int , QString , int , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PictureBrowser::loadImageJob)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PictureBrowser::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PictureBrowser::selectPage)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PictureBrowser::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PictureBrowser::selectMasterPage)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PictureBrowser::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_PictureBrowser.data,
    qt_meta_data_PictureBrowser,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PictureBrowser::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PictureBrowser::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PictureBrowser.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PictureBrowser"))
        return static_cast< Ui::PictureBrowser*>(this);
    return QDialog::qt_metacast(_clname);
}

int PictureBrowser::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 56)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 56;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 56)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 56;
    }
    return _id;
}

// SIGNAL 0
void PictureBrowser::loadImageJob(int _t1, QString _t2, int _t3, int _t4)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PictureBrowser::selectPage(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void PictureBrowser::selectMasterPage(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
