/********************************************************************************
** Form generated from reading UI file 'propertywidget_pathtextbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYWIDGET_PATHTEXTBASE_H
#define UI_PROPERTYWIDGET_PATHTEXTBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_PropertyWidget_PathTextBase
{
public:
    QGridLayout *gridLayout;
    QLabel *pathTextTypeLabel;
    QComboBox *pathTextType;
    QLabel *startOffsetLabel;
    ScrSpinBox *startOffset;
    QLabel *distFromCurveLabel;
    ScrSpinBox *distFromCurve;
    QCheckBox *flippedPathText;
    QCheckBox *showCurveCheckBox;

    void setupUi(QFrame *PropertyWidget_PathTextBase)
    {
        if (PropertyWidget_PathTextBase->objectName().isEmpty())
            PropertyWidget_PathTextBase->setObjectName(QString::fromUtf8("PropertyWidget_PathTextBase"));
        PropertyWidget_PathTextBase->resize(243, 143);
        gridLayout = new QGridLayout(PropertyWidget_PathTextBase);
        gridLayout->setSpacing(3);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(3, 3, 3, 3);
        pathTextTypeLabel = new QLabel(PropertyWidget_PathTextBase);
        pathTextTypeLabel->setObjectName(QString::fromUtf8("pathTextTypeLabel"));

        gridLayout->addWidget(pathTextTypeLabel, 0, 0, 1, 1);

        pathTextType = new QComboBox(PropertyWidget_PathTextBase);
        pathTextType->setObjectName(QString::fromUtf8("pathTextType"));

        gridLayout->addWidget(pathTextType, 0, 1, 1, 1);

        startOffsetLabel = new QLabel(PropertyWidget_PathTextBase);
        startOffsetLabel->setObjectName(QString::fromUtf8("startOffsetLabel"));

        gridLayout->addWidget(startOffsetLabel, 1, 0, 1, 1);

        startOffset = new ScrSpinBox(PropertyWidget_PathTextBase);
        startOffset->setObjectName(QString::fromUtf8("startOffset"));

        gridLayout->addWidget(startOffset, 1, 1, 1, 1);

        distFromCurveLabel = new QLabel(PropertyWidget_PathTextBase);
        distFromCurveLabel->setObjectName(QString::fromUtf8("distFromCurveLabel"));

        gridLayout->addWidget(distFromCurveLabel, 2, 0, 1, 1);

        distFromCurve = new ScrSpinBox(PropertyWidget_PathTextBase);
        distFromCurve->setObjectName(QString::fromUtf8("distFromCurve"));

        gridLayout->addWidget(distFromCurve, 2, 1, 1, 1);

        flippedPathText = new QCheckBox(PropertyWidget_PathTextBase);
        flippedPathText->setObjectName(QString::fromUtf8("flippedPathText"));

        gridLayout->addWidget(flippedPathText, 3, 0, 1, 1);

        showCurveCheckBox = new QCheckBox(PropertyWidget_PathTextBase);
        showCurveCheckBox->setObjectName(QString::fromUtf8("showCurveCheckBox"));

        gridLayout->addWidget(showCurveCheckBox, 4, 0, 1, 1);


        retranslateUi(PropertyWidget_PathTextBase);

        QMetaObject::connectSlotsByName(PropertyWidget_PathTextBase);
    } // setupUi

    void retranslateUi(QFrame *PropertyWidget_PathTextBase)
    {
        pathTextTypeLabel->setText(QCoreApplication::translate("PropertyWidget_PathTextBase", "Type:", nullptr));
        startOffsetLabel->setText(QCoreApplication::translate("PropertyWidget_PathTextBase", "Start Offset:", nullptr));
        distFromCurveLabel->setText(QCoreApplication::translate("PropertyWidget_PathTextBase", "Distance from Curve:", nullptr));
        flippedPathText->setText(QCoreApplication::translate("PropertyWidget_PathTextBase", "Flip Text", nullptr));
        showCurveCheckBox->setText(QCoreApplication::translate("PropertyWidget_PathTextBase", "Show Curve", nullptr));
        (void)PropertyWidget_PathTextBase;
    } // retranslateUi

};

namespace Ui {
    class PropertyWidget_PathTextBase: public Ui_PropertyWidget_PathTextBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTYWIDGET_PATHTEXTBASE_H
