/********************************************************************************
** Form generated from reading UI file 'xpsimportoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_XPSIMPORTOPTIONS_H
#define UI_XPSIMPORTOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_XpsImportOptions
{
public:
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label;
    QLabel *fileLabel;
    QVBoxLayout *verticalLayout_3;
    QGroupBox *groupBox;
    QVBoxLayout *verticalLayout;
    QRadioButton *allPages;
    QHBoxLayout *horizontalLayout_2;
    QRadioButton *singlePage;
    QSpinBox *spinBox;
    QRadioButton *selectedPages;
    QHBoxLayout *horizontalLayout;
    QLineEdit *pageRangeString;
    QPushButton *pageSelectButton;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *XpsImportOptions)
    {
        if (XpsImportOptions->objectName().isEmpty())
            XpsImportOptions->setObjectName(QString::fromUtf8("XpsImportOptions"));
        XpsImportOptions->resize(213, 272);
        XpsImportOptions->setSizeGripEnabled(true);
        verticalLayout_4 = new QVBoxLayout(XpsImportOptions);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        verticalLayout_4->setContentsMargins(9, 9, 9, 9);
        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        label = new QLabel(XpsImportOptions);
        label->setObjectName(QString::fromUtf8("label"));
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy);

        horizontalLayout_5->addWidget(label);

        fileLabel = new QLabel(XpsImportOptions);
        fileLabel->setObjectName(QString::fromUtf8("fileLabel"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(fileLabel->sizePolicy().hasHeightForWidth());
        fileLabel->setSizePolicy(sizePolicy1);
        fileLabel->setMinimumSize(QSize(0, 0));

        horizontalLayout_5->addWidget(fileLabel);


        verticalLayout_4->addLayout(horizontalLayout_5);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        groupBox = new QGroupBox(XpsImportOptions);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Maximum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy2);
        verticalLayout = new QVBoxLayout(groupBox);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        allPages = new QRadioButton(groupBox);
        allPages->setObjectName(QString::fromUtf8("allPages"));
        allPages->setChecked(true);

        verticalLayout->addWidget(allPages);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        singlePage = new QRadioButton(groupBox);
        singlePage->setObjectName(QString::fromUtf8("singlePage"));

        horizontalLayout_2->addWidget(singlePage);

        spinBox = new QSpinBox(groupBox);
        spinBox->setObjectName(QString::fromUtf8("spinBox"));
        spinBox->setEnabled(false);
        QSizePolicy sizePolicy3(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(spinBox->sizePolicy().hasHeightForWidth());
        spinBox->setSizePolicy(sizePolicy3);

        horizontalLayout_2->addWidget(spinBox);


        verticalLayout->addLayout(horizontalLayout_2);

        selectedPages = new QRadioButton(groupBox);
        selectedPages->setObjectName(QString::fromUtf8("selectedPages"));

        verticalLayout->addWidget(selectedPages);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(15, -1, -1, -1);
        pageRangeString = new QLineEdit(groupBox);
        pageRangeString->setObjectName(QString::fromUtf8("pageRangeString"));
        pageRangeString->setEnabled(false);

        horizontalLayout->addWidget(pageRangeString);

        pageSelectButton = new QPushButton(groupBox);
        pageSelectButton->setObjectName(QString::fromUtf8("pageSelectButton"));
        pageSelectButton->setEnabled(false);

        horizontalLayout->addWidget(pageSelectButton);


        verticalLayout->addLayout(horizontalLayout);


        verticalLayout_3->addWidget(groupBox);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);


        verticalLayout_4->addLayout(verticalLayout_3);

        buttonBox = new QDialogButtonBox(XpsImportOptions);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_4->addWidget(buttonBox);


        retranslateUi(XpsImportOptions);
        QObject::connect(buttonBox, SIGNAL(accepted()), XpsImportOptions, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), XpsImportOptions, SLOT(reject()));
        QObject::connect(singlePage, SIGNAL(toggled(bool)), spinBox, SLOT(setEnabled(bool)));
        QObject::connect(selectedPages, SIGNAL(toggled(bool)), pageRangeString, SLOT(setEnabled(bool)));
        QObject::connect(selectedPages, SIGNAL(toggled(bool)), pageSelectButton, SLOT(setEnabled(bool)));

        QMetaObject::connectSlotsByName(XpsImportOptions);
    } // setupUi

    void retranslateUi(QDialog *XpsImportOptions)
    {
        XpsImportOptions->setWindowTitle(QCoreApplication::translate("XpsImportOptions", "XPS Import", nullptr));
        label->setText(QCoreApplication::translate("XpsImportOptions", "Importing File:", nullptr));
        fileLabel->setText(QString());
        groupBox->setTitle(QCoreApplication::translate("XpsImportOptions", "Page Range", nullptr));
        allPages->setText(QCoreApplication::translate("XpsImportOptions", "All Pages", nullptr));
        singlePage->setText(QCoreApplication::translate("XpsImportOptions", "Single Page", nullptr));
        selectedPages->setText(QCoreApplication::translate("XpsImportOptions", "Choose Pages", nullptr));
        pageSelectButton->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class XpsImportOptions: public Ui_XpsImportOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_XPSIMPORTOPTIONS_H
