/****************************************************************************
** Meta object code from reading C++ file 'stylemanager.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/stylemanager.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'stylemanager.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_StyleManager_t {
    QByteArrayData data[37];
    char stringdata0[406];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StyleManager_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StyleManager_t qt_meta_stringdata_StyleManager = {
    {
QT_MOC_LITERAL(0, 0, 12), // "StyleManager"
QT_MOC_LITERAL(1, 13, 6), // "closed"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 6), // "setDoc"
QT_MOC_LITERAL(4, 28, 11), // "ScribusDoc*"
QT_MOC_LITERAL(5, 40, 3), // "doc"
QT_MOC_LITERAL(6, 44, 14), // "languageChange"
QT_MOC_LITERAL(7, 59, 10), // "unitChange"
QT_MOC_LITERAL(8, 70, 6), // "slotOk"
QT_MOC_LITERAL(9, 77, 9), // "slotApply"
QT_MOC_LITERAL(10, 87, 10), // "slotDelete"
QT_MOC_LITERAL(11, 98, 10), // "slotImport"
QT_MOC_LITERAL(12, 109, 8), // "slotEdit"
QT_MOC_LITERAL(13, 118, 9), // "slotClone"
QT_MOC_LITERAL(14, 128, 7), // "slotNew"
QT_MOC_LITERAL(15, 136, 12), // "slotNewPopup"
QT_MOC_LITERAL(16, 149, 8), // "QAction*"
QT_MOC_LITERAL(17, 158, 9), // "slotScrap"
QT_MOC_LITERAL(18, 168, 14), // "slotRightClick"
QT_MOC_LITERAL(19, 183, 5), // "point"
QT_MOC_LITERAL(20, 189, 15), // "slotDoubleClick"
QT_MOC_LITERAL(21, 205, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(22, 222, 4), // "item"
QT_MOC_LITERAL(23, 227, 6), // "column"
QT_MOC_LITERAL(24, 234, 15), // "slotNameChanged"
QT_MOC_LITERAL(25, 250, 4), // "name"
QT_MOC_LITERAL(26, 255, 19), // "slotShortcutChanged"
QT_MOC_LITERAL(27, 275, 8), // "shortcut"
QT_MOC_LITERAL(28, 284, 15), // "slotSetupWidget"
QT_MOC_LITERAL(29, 300, 14), // "slotApplyStyle"
QT_MOC_LITERAL(30, 315, 7), // "newitem"
QT_MOC_LITERAL(31, 323, 7), // "olditem"
QT_MOC_LITERAL(32, 331, 23), // "slotDocSelectionChanged"
QT_MOC_LITERAL(33, 355, 20), // "slotDocStylesChanged"
QT_MOC_LITERAL(34, 376, 9), // "slotDirty"
QT_MOC_LITERAL(35, 386, 9), // "slotClean"
QT_MOC_LITERAL(36, 396, 9) // "keyString"

    },
    "StyleManager\0closed\0\0setDoc\0ScribusDoc*\0"
    "doc\0languageChange\0unitChange\0slotOk\0"
    "slotApply\0slotDelete\0slotImport\0"
    "slotEdit\0slotClone\0slotNew\0slotNewPopup\0"
    "QAction*\0slotScrap\0slotRightClick\0"
    "point\0slotDoubleClick\0QTreeWidgetItem*\0"
    "item\0column\0slotNameChanged\0name\0"
    "slotShortcutChanged\0shortcut\0"
    "slotSetupWidget\0slotApplyStyle\0newitem\0"
    "olditem\0slotDocSelectionChanged\0"
    "slotDocStylesChanged\0slotDirty\0slotClean\0"
    "keyString"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StyleManager[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      27,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  149,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,  150,    2, 0x0a /* Public */,
       6,    0,  153,    2, 0x0a /* Public */,
       7,    0,  154,    2, 0x0a /* Public */,
       8,    0,  155,    2, 0x08 /* Private */,
       9,    0,  156,    2, 0x08 /* Private */,
      10,    0,  157,    2, 0x08 /* Private */,
      11,    0,  158,    2, 0x08 /* Private */,
      12,    0,  159,    2, 0x08 /* Private */,
      13,    0,  160,    2, 0x08 /* Private */,
      14,    0,  161,    2, 0x08 /* Private */,
      15,    1,  162,    2, 0x08 /* Private */,
      15,    0,  165,    2, 0x08 /* Private */,
      17,    0,  166,    2, 0x08 /* Private */,
      18,    1,  167,    2, 0x08 /* Private */,
      20,    2,  170,    2, 0x08 /* Private */,
      24,    1,  175,    2, 0x08 /* Private */,
      26,    1,  178,    2, 0x08 /* Private */,
      28,    0,  181,    2, 0x08 /* Private */,
      29,    1,  182,    2, 0x08 /* Private */,
      29,    2,  185,    2, 0x08 /* Private */,
      29,    2,  190,    2, 0x08 /* Private */,
      32,    0,  195,    2, 0x08 /* Private */,
      33,    0,  196,    2, 0x08 /* Private */,
      34,    0,  197,    2, 0x08 /* Private */,
      35,    0,  198,    2, 0x08 /* Private */,
      29,    1,  199,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,   19,
    QMetaType::Void, 0x80000000 | 21, QMetaType::Int,   22,   23,
    QMetaType::Void, QMetaType::QString,   25,
    QMetaType::Void, QMetaType::QString,   27,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 21,   22,
    QMetaType::Void, 0x80000000 | 21, 0x80000000 | 21,   30,   31,
    QMetaType::Void, 0x80000000 | 21, QMetaType::Int,   22,   23,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   36,

       0        // eod
};

void StyleManager::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<StyleManager *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->closed(); break;
        case 1: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 2: _t->languageChange(); break;
        case 3: _t->unitChange(); break;
        case 4: _t->slotOk(); break;
        case 5: _t->slotApply(); break;
        case 6: _t->slotDelete(); break;
        case 7: _t->slotImport(); break;
        case 8: _t->slotEdit(); break;
        case 9: _t->slotClone(); break;
        case 10: _t->slotNew(); break;
        case 11: _t->slotNewPopup((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 12: _t->slotNewPopup(); break;
        case 13: _t->slotScrap(); break;
        case 14: _t->slotRightClick((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 15: _t->slotDoubleClick((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 16: _t->slotNameChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->slotShortcutChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 18: _t->slotSetupWidget(); break;
        case 19: _t->slotApplyStyle((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1]))); break;
        case 20: _t->slotApplyStyle((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< QTreeWidgetItem*(*)>(_a[2]))); break;
        case 21: _t->slotApplyStyle((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 22: _t->slotDocSelectionChanged(); break;
        case 23: _t->slotDocStylesChanged(); break;
        case 24: _t->slotDirty(); break;
        case 25: _t->slotClean(); break;
        case 26: _t->slotApplyStyle((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (StyleManager::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&StyleManager::closed)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject StyleManager::staticMetaObject = { {
    QMetaObject::SuperData::link<ScrPaletteBase::staticMetaObject>(),
    qt_meta_stringdata_StyleManager.data,
    qt_meta_data_StyleManager,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StyleManager::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StyleManager::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StyleManager.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::StyleManager"))
        return static_cast< Ui::StyleManager*>(this);
    return ScrPaletteBase::qt_metacast(_clname);
}

int StyleManager::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScrPaletteBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 27;
    }
    return _id;
}

// SIGNAL 0
void StyleManager::closed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
