/********************************************************************************
** Form generated from reading UI file 'cmykfwbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CMYKFWBASE_H
#define UI_CMYKFWBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/scrspinbox.h>
#include "ui/colorchart.h"
#include "ui/colorlistbox.h"
#include "ui/sccolorslider.h"
#include "ui/swatchcombo.h"

QT_BEGIN_NAMESPACE

class Ui_CMYKChooseBase
{
public:
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QLineEdit *ColorName;
    QLabel *label_2;
    QComboBox *ComboBox1;
    QCheckBox *Separations;
    QSpacerItem *verticalSpacer;
    QGridLayout *gridLayout;
    QLabel *label_4;
    QLabel *label_3;
    QLabel *OldC;
    QLabel *NewC;
    QHBoxLayout *horizontalLayout_5;
    QSpacerItem *horizontalSpacer;
    QPushButton *buttonOK;
    QSpacerItem *horizontalSpacer_3;
    QPushButton *buttonCancel;
    QSpacerItem *horizontalSpacer_2;
    QFrame *frame;
    QVBoxLayout *verticalLayout_2;
    SwatchCombo *Swatches;
    QStackedWidget *TabStack;
    QWidget *page;
    QHBoxLayout *horizontalLayout_6;
    QFrame *frame_2;
    QVBoxLayout *verticalLayout_3;
    QFrame *frame_3;
    QHBoxLayout *horizontalLayout_7;
    ColorChart *ColorMap;
    QWidget *page_2;
    QHBoxLayout *horizontalLayout_2;
    ColorListBox *ColorSwatch;
    QSpacerItem *verticalSpacer_2;
    QGridLayout *slidersLayout;
    ScColorSlider *CyanSL;
    ScColorSlider *BlackSL;
    QLabel *BlackT;
    ScColorSlider *MagentaSL;
    ScrSpinBox *BlackSp;
    ScColorSlider *YellowSL;
    QLabel *CyanT;
    ScrSpinBox *YellowSp;
    ScrSpinBox *MagentaSp;
    QLabel *MagentaT;
    ScrSpinBox *CyanSp;
    QLabel *YellowT;

    void setupUi(QDialog *CMYKChooseBase)
    {
        if (CMYKChooseBase->objectName().isEmpty())
            CMYKChooseBase->setObjectName(QString::fromUtf8("CMYKChooseBase"));
        CMYKChooseBase->resize(530, 319);
        horizontalLayout = new QHBoxLayout(CMYKChooseBase);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(9, 9, 9, 9);
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        label = new QLabel(CMYKChooseBase);
        label->setObjectName(QString::fromUtf8("label"));
        label->setMinimumSize(QSize(200, 22));

        verticalLayout->addWidget(label);

        ColorName = new QLineEdit(CMYKChooseBase);
        ColorName->setObjectName(QString::fromUtf8("ColorName"));
        ColorName->setMinimumSize(QSize(200, 22));

        verticalLayout->addWidget(ColorName);

        label_2 = new QLabel(CMYKChooseBase);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setMinimumSize(QSize(100, 22));

        verticalLayout->addWidget(label_2);

        ComboBox1 = new QComboBox(CMYKChooseBase);
        ComboBox1->setObjectName(QString::fromUtf8("ComboBox1"));

        verticalLayout->addWidget(ComboBox1);

        Separations = new QCheckBox(CMYKChooseBase);
        Separations->setObjectName(QString::fromUtf8("Separations"));

        verticalLayout->addWidget(Separations);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label_4 = new QLabel(CMYKChooseBase);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        label_4->setMinimumSize(QSize(50, 22));
        label_4->setMaximumSize(QSize(100, 22));
        label_4->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(label_4, 0, 1, 1, 1);

        label_3 = new QLabel(CMYKChooseBase);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        label_3->setMinimumSize(QSize(50, 22));
        label_3->setMaximumSize(QSize(100, 22));
        label_3->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        OldC = new QLabel(CMYKChooseBase);
        OldC->setObjectName(QString::fromUtf8("OldC"));
        OldC->setMinimumSize(QSize(50, 50));
        OldC->setMaximumSize(QSize(50, 50));
        OldC->setFrameShape(QFrame::WinPanel);
        OldC->setFrameShadow(QFrame::Sunken);
        OldC->setScaledContents(true);
        OldC->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(OldC, 1, 0, 1, 1);

        NewC = new QLabel(CMYKChooseBase);
        NewC->setObjectName(QString::fromUtf8("NewC"));
        NewC->setMinimumSize(QSize(50, 50));
        NewC->setMaximumSize(QSize(50, 50));
        NewC->setFrameShape(QFrame::WinPanel);
        NewC->setFrameShadow(QFrame::Sunken);
        NewC->setScaledContents(true);
        NewC->setAlignment(Qt::AlignCenter);

        gridLayout->addWidget(NewC, 1, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setSpacing(0);
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        horizontalLayout_5->setContentsMargins(0, 0, 0, 0);
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer);

        buttonOK = new QPushButton(CMYKChooseBase);
        buttonOK->setObjectName(QString::fromUtf8("buttonOK"));

        horizontalLayout_5->addWidget(buttonOK);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_3);

        buttonCancel = new QPushButton(CMYKChooseBase);
        buttonCancel->setObjectName(QString::fromUtf8("buttonCancel"));

        horizontalLayout_5->addWidget(buttonCancel);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout_5);


        horizontalLayout->addLayout(verticalLayout);

        frame = new QFrame(CMYKChooseBase);
        frame->setObjectName(QString::fromUtf8("frame"));
        frame->setFrameShape(QFrame::NoFrame);
        frame->setFrameShadow(QFrame::Raised);
        verticalLayout_2 = new QVBoxLayout(frame);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        Swatches = new SwatchCombo(frame);
        Swatches->setObjectName(QString::fromUtf8("Swatches"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Swatches->sizePolicy().hasHeightForWidth());
        Swatches->setSizePolicy(sizePolicy);
        Swatches->setPopupMode(QToolButton::MenuButtonPopup);

        verticalLayout_2->addWidget(Swatches);

        TabStack = new QStackedWidget(frame);
        TabStack->setObjectName(QString::fromUtf8("TabStack"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Maximum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(TabStack->sizePolicy().hasHeightForWidth());
        TabStack->setSizePolicy(sizePolicy1);
        TabStack->setFrameShape(QFrame::NoFrame);
        page = new QWidget();
        page->setObjectName(QString::fromUtf8("page"));
        horizontalLayout_6 = new QHBoxLayout(page);
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        horizontalLayout_6->setContentsMargins(0, 0, 0, 0);
        frame_2 = new QFrame(page);
        frame_2->setObjectName(QString::fromUtf8("frame_2"));
        frame_2->setFrameShape(QFrame::NoFrame);
        frame_2->setFrameShadow(QFrame::Sunken);
        verticalLayout_3 = new QVBoxLayout(frame_2);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        frame_3 = new QFrame(frame_2);
        frame_3->setObjectName(QString::fromUtf8("frame_3"));
        QSizePolicy sizePolicy2(QSizePolicy::Fixed, QSizePolicy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(frame_3->sizePolicy().hasHeightForWidth());
        frame_3->setSizePolicy(sizePolicy2);
        frame_3->setMinimumSize(QSize(182, 130));
        frame_3->setMaximumSize(QSize(182, 130));
        frame_3->setFrameShape(QFrame::WinPanel);
        frame_3->setFrameShadow(QFrame::Sunken);
        horizontalLayout_7 = new QHBoxLayout(frame_3);
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        horizontalLayout_7->setContentsMargins(0, 0, 0, 0);
        ColorMap = new ColorChart(frame_3);
        ColorMap->setObjectName(QString::fromUtf8("ColorMap"));
        ColorMap->setMinimumSize(QSize(180, 128));
        ColorMap->setMaximumSize(QSize(180, 128));

        horizontalLayout_7->addWidget(ColorMap);


        verticalLayout_3->addWidget(frame_3);


        horizontalLayout_6->addWidget(frame_2, 0, Qt::AlignHCenter|Qt::AlignVCenter);

        TabStack->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName(QString::fromUtf8("page_2"));
        horizontalLayout_2 = new QHBoxLayout(page_2);
        horizontalLayout_2->setSpacing(0);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        ColorSwatch = new ColorListBox(page_2);
        ColorSwatch->setObjectName(QString::fromUtf8("ColorSwatch"));

        horizontalLayout_2->addWidget(ColorSwatch);

        TabStack->addWidget(page_2);

        verticalLayout_2->addWidget(TabStack);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_2);

        slidersLayout = new QGridLayout();
        slidersLayout->setObjectName(QString::fromUtf8("slidersLayout"));
        CyanSL = new ScColorSlider(frame);
        CyanSL->setObjectName(QString::fromUtf8("CyanSL"));
        CyanSL->setMinimumSize(QSize(200, 16));
        CyanSL->setMinimum(0);
        CyanSL->setMaximum(100);
        CyanSL->setOrientation(Qt::Horizontal);

        slidersLayout->addWidget(CyanSL, 0, 1, 1, 1);

        BlackSL = new ScColorSlider(frame);
        BlackSL->setObjectName(QString::fromUtf8("BlackSL"));
        BlackSL->setMinimumSize(QSize(200, 16));
        BlackSL->setMinimum(0);
        BlackSL->setMaximum(100);
        BlackSL->setOrientation(Qt::Horizontal);

        slidersLayout->addWidget(BlackSL, 3, 1, 1, 1);

        BlackT = new QLabel(frame);
        BlackT->setObjectName(QString::fromUtf8("BlackT"));
        BlackT->setMinimumSize(QSize(20, 0));

        slidersLayout->addWidget(BlackT, 3, 0, 1, 1);

        MagentaSL = new ScColorSlider(frame);
        MagentaSL->setObjectName(QString::fromUtf8("MagentaSL"));
        MagentaSL->setMinimumSize(QSize(200, 16));
        MagentaSL->setMinimum(0);
        MagentaSL->setMaximum(100);
        MagentaSL->setOrientation(Qt::Horizontal);

        slidersLayout->addWidget(MagentaSL, 1, 1, 1, 1);

        BlackSp = new ScrSpinBox(frame);
        BlackSp->setObjectName(QString::fromUtf8("BlackSp"));
        BlackSp->setMinimumSize(QSize(70, 0));
        BlackSp->setMinimum(0.000000000000000);
        BlackSp->setMaximum(100.000000000000000);

        slidersLayout->addWidget(BlackSp, 3, 2, 1, 1);

        YellowSL = new ScColorSlider(frame);
        YellowSL->setObjectName(QString::fromUtf8("YellowSL"));
        YellowSL->setMinimumSize(QSize(200, 16));
        YellowSL->setMinimum(0);
        YellowSL->setMaximum(100);
        YellowSL->setOrientation(Qt::Horizontal);

        slidersLayout->addWidget(YellowSL, 2, 1, 1, 1);

        CyanT = new QLabel(frame);
        CyanT->setObjectName(QString::fromUtf8("CyanT"));
        CyanT->setMinimumSize(QSize(20, 0));

        slidersLayout->addWidget(CyanT, 0, 0, 1, 1);

        YellowSp = new ScrSpinBox(frame);
        YellowSp->setObjectName(QString::fromUtf8("YellowSp"));
        YellowSp->setMinimumSize(QSize(70, 0));
        YellowSp->setMinimum(0.000000000000000);
        YellowSp->setMaximum(100.000000000000000);

        slidersLayout->addWidget(YellowSp, 2, 2, 1, 1);

        MagentaSp = new ScrSpinBox(frame);
        MagentaSp->setObjectName(QString::fromUtf8("MagentaSp"));
        MagentaSp->setMinimumSize(QSize(70, 0));
        MagentaSp->setMinimum(0.000000000000000);
        MagentaSp->setMaximum(100.000000000000000);

        slidersLayout->addWidget(MagentaSp, 1, 2, 1, 1);

        MagentaT = new QLabel(frame);
        MagentaT->setObjectName(QString::fromUtf8("MagentaT"));
        MagentaT->setMinimumSize(QSize(20, 0));

        slidersLayout->addWidget(MagentaT, 1, 0, 1, 1);

        CyanSp = new ScrSpinBox(frame);
        CyanSp->setObjectName(QString::fromUtf8("CyanSp"));
        CyanSp->setMinimumSize(QSize(70, 0));
        CyanSp->setMinimum(0.000000000000000);
        CyanSp->setMaximum(100.000000000000000);

        slidersLayout->addWidget(CyanSp, 0, 2, 1, 1);

        YellowT = new QLabel(frame);
        YellowT->setObjectName(QString::fromUtf8("YellowT"));
        YellowT->setMinimumSize(QSize(20, 0));

        slidersLayout->addWidget(YellowT, 2, 0, 1, 1);


        verticalLayout_2->addLayout(slidersLayout);


        horizontalLayout->addWidget(frame);

#if QT_CONFIG(shortcut)
        label->setBuddy(ColorName);
        label_2->setBuddy(ComboBox1);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(ColorName, ComboBox1);
        QWidget::setTabOrder(ComboBox1, Separations);
        QWidget::setTabOrder(Separations, Swatches);
        QWidget::setTabOrder(Swatches, ColorSwatch);
        QWidget::setTabOrder(ColorSwatch, CyanSL);
        QWidget::setTabOrder(CyanSL, CyanSp);
        QWidget::setTabOrder(CyanSp, MagentaSL);
        QWidget::setTabOrder(MagentaSL, MagentaSp);
        QWidget::setTabOrder(MagentaSp, YellowSL);
        QWidget::setTabOrder(YellowSL, YellowSp);
        QWidget::setTabOrder(YellowSp, BlackSL);
        QWidget::setTabOrder(BlackSL, BlackSp);
        QWidget::setTabOrder(BlackSp, buttonOK);
        QWidget::setTabOrder(buttonOK, buttonCancel);

        retranslateUi(CMYKChooseBase);

        buttonOK->setDefault(true);
        TabStack->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(CMYKChooseBase);
    } // setupUi

    void retranslateUi(QDialog *CMYKChooseBase)
    {
        CMYKChooseBase->setWindowTitle(QCoreApplication::translate("CMYKChooseBase", "Edit Color", nullptr));
        label->setText(QCoreApplication::translate("CMYKChooseBase", "&Name:", nullptr));
        label_2->setText(QCoreApplication::translate("CMYKChooseBase", "Color &Model", nullptr));
        Separations->setText(QCoreApplication::translate("CMYKChooseBase", "Is Spot Color", nullptr));
        label_4->setText(QCoreApplication::translate("CMYKChooseBase", "New", nullptr));
        label_3->setText(QCoreApplication::translate("CMYKChooseBase", "Old", nullptr));
        OldC->setText(QString());
        NewC->setText(QString());
        buttonOK->setText(QCoreApplication::translate("CMYKChooseBase", "OK", nullptr));
        buttonCancel->setText(QCoreApplication::translate("CMYKChooseBase", "Cancel", nullptr));
        Swatches->setText(QString());
        BlackT->setText(QCoreApplication::translate("CMYKChooseBase", "K:", nullptr));
        BlackSp->setSuffix(QCoreApplication::translate("CMYKChooseBase", " %", nullptr));
        CyanT->setText(QCoreApplication::translate("CMYKChooseBase", "C:", nullptr));
        YellowSp->setSuffix(QCoreApplication::translate("CMYKChooseBase", " %", nullptr));
        MagentaSp->setSuffix(QCoreApplication::translate("CMYKChooseBase", " %", nullptr));
        MagentaT->setText(QCoreApplication::translate("CMYKChooseBase", "M:", nullptr));
        CyanSp->setSuffix(QCoreApplication::translate("CMYKChooseBase", " %", nullptr));
        YellowT->setText(QCoreApplication::translate("CMYKChooseBase", "Y:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CMYKChooseBase: public Ui_CMYKChooseBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CMYKFWBASE_H
