/****************************************************************************
** Meta object code from reading C++ file 'layers.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/layers.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'layers.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_LayerPalette_t {
    QByteArrayData data[29];
    char stringdata0[334];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_LayerPalette_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_LayerPalette_t qt_meta_stringdata_LayerPalette = {
    {
QT_MOC_LITERAL(0, 0, 12), // "LayerPalette"
QT_MOC_LITERAL(1, 13, 12), // "LayerChanged"
QT_MOC_LITERAL(2, 26, 0), // ""
QT_MOC_LITERAL(3, 27, 8), // "addLayer"
QT_MOC_LITERAL(4, 36, 8), // "dupLayer"
QT_MOC_LITERAL(5, 45, 11), // "removeLayer"
QT_MOC_LITERAL(6, 57, 7), // "upLayer"
QT_MOC_LITERAL(7, 65, 9), // "downLayer"
QT_MOC_LITERAL(8, 75, 10), // "changeName"
QT_MOC_LITERAL(9, 86, 3), // "row"
QT_MOC_LITERAL(10, 90, 3), // "col"
QT_MOC_LITERAL(11, 94, 12), // "visibleLayer"
QT_MOC_LITERAL(12, 107, 10), // "printLayer"
QT_MOC_LITERAL(13, 118, 9), // "lockLayer"
QT_MOC_LITERAL(14, 128, 15), // "flowToggleLayer"
QT_MOC_LITERAL(15, 144, 18), // "outlineToggleLayer"
QT_MOC_LITERAL(16, 163, 17), // "selectToggleLayer"
QT_MOC_LITERAL(17, 181, 19), // "toggleAllfromHeader"
QT_MOC_LITERAL(18, 201, 5), // "index"
QT_MOC_LITERAL(19, 207, 9), // "markLayer"
QT_MOC_LITERAL(20, 217, 13), // "changeOpacity"
QT_MOC_LITERAL(21, 231, 15), // "changeBlendMode"
QT_MOC_LITERAL(22, 247, 5), // "blend"
QT_MOC_LITERAL(23, 253, 14), // "setActiveLayer"
QT_MOC_LITERAL(24, 268, 12), // "clearContent"
QT_MOC_LITERAL(25, 281, 15), // "markActiveLayer"
QT_MOC_LITERAL(26, 297, 7), // "layerID"
QT_MOC_LITERAL(27, 305, 13), // "iconSetChange"
QT_MOC_LITERAL(28, 319, 14) // "languageChange"

    },
    "LayerPalette\0LayerChanged\0\0addLayer\0"
    "dupLayer\0removeLayer\0upLayer\0downLayer\0"
    "changeName\0row\0col\0visibleLayer\0"
    "printLayer\0lockLayer\0flowToggleLayer\0"
    "outlineToggleLayer\0selectToggleLayer\0"
    "toggleAllfromHeader\0index\0markLayer\0"
    "changeOpacity\0changeBlendMode\0blend\0"
    "setActiveLayer\0clearContent\0markActiveLayer\0"
    "layerID\0iconSetChange\0languageChange"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_LayerPalette[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      23,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  129,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,  130,    2, 0x0a /* Public */,
       4,    0,  131,    2, 0x0a /* Public */,
       5,    0,  132,    2, 0x0a /* Public */,
       6,    0,  133,    2, 0x0a /* Public */,
       7,    0,  134,    2, 0x0a /* Public */,
       8,    2,  135,    2, 0x0a /* Public */,
      11,    0,  140,    2, 0x0a /* Public */,
      12,    0,  141,    2, 0x0a /* Public */,
      13,    0,  142,    2, 0x0a /* Public */,
      14,    0,  143,    2, 0x0a /* Public */,
      15,    0,  144,    2, 0x0a /* Public */,
      16,    0,  145,    2, 0x0a /* Public */,
      17,    1,  146,    2, 0x0a /* Public */,
      19,    0,  149,    2, 0x0a /* Public */,
      20,    0,  150,    2, 0x0a /* Public */,
      21,    1,  151,    2, 0x0a /* Public */,
      23,    2,  154,    2, 0x0a /* Public */,
      24,    0,  159,    2, 0x0a /* Public */,
      25,    1,  160,    2, 0x0a /* Public */,
      25,    0,  163,    2, 0x2a /* Public | MethodCloned */,
      27,    0,  164,    2, 0x0a /* Public */,
      28,    0,  165,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    9,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    9,   10,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   26,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void LayerPalette::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<LayerPalette *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->LayerChanged(); break;
        case 1: _t->addLayer(); break;
        case 2: _t->dupLayer(); break;
        case 3: _t->removeLayer(); break;
        case 4: _t->upLayer(); break;
        case 5: _t->downLayer(); break;
        case 6: _t->changeName((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 7: _t->visibleLayer(); break;
        case 8: _t->printLayer(); break;
        case 9: _t->lockLayer(); break;
        case 10: _t->flowToggleLayer(); break;
        case 11: _t->outlineToggleLayer(); break;
        case 12: _t->selectToggleLayer(); break;
        case 13: _t->toggleAllfromHeader((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->markLayer(); break;
        case 15: _t->changeOpacity(); break;
        case 16: _t->changeBlendMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->setActiveLayer((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 18: _t->clearContent(); break;
        case 19: _t->markActiveLayer((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->markActiveLayer(); break;
        case 21: _t->iconSetChange(); break;
        case 22: _t->languageChange(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (LayerPalette::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&LayerPalette::LayerChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject LayerPalette::staticMetaObject = { {
    QMetaObject::SuperData::link<ScDockPalette::staticMetaObject>(),
    qt_meta_stringdata_LayerPalette.data,
    qt_meta_data_LayerPalette,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *LayerPalette::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *LayerPalette::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_LayerPalette.stringdata0))
        return static_cast<void*>(this);
    return ScDockPalette::qt_metacast(_clname);
}

int LayerPalette::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScDockPalette::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 23)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 23;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 23)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 23;
    }
    return _id;
}

// SIGNAL 0
void LayerPalette::LayerChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
