/********************************************************************************
** Form generated from reading UI file 'picsearch.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PICSEARCH_H
#define UI_PICSEARCH_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_PicSearch
{
public:
    QVBoxLayout *vboxLayout;
    QHBoxLayout *hboxLayout;
    QLabel *textLabel1;
    QLabel *fileNameLabel;
    QHBoxLayout *hboxLayout1;
    QListWidget *foundFilesBox;
    QVBoxLayout *vboxLayout1;
    QCheckBox *previewCheckBox;
    QSpacerItem *spacerItem;
    QVBoxLayout *vboxLayout2;
    QLabel *previewLabel;
    QSpacerItem *spacerItem1;
    QCheckBox *matchCheckBox;
    QLabel *matchWarningLabel;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *useButton;
    QPushButton *cancelButton;

    void setupUi(QDialog *PicSearch)
    {
        if (PicSearch->objectName().isEmpty())
            PicSearch->setObjectName(QString::fromUtf8("PicSearch"));
        PicSearch->resize(516, 292);
        vboxLayout = new QVBoxLayout(PicSearch);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        textLabel1 = new QLabel(PicSearch);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));
        textLabel1->setWordWrap(false);

        hboxLayout->addWidget(textLabel1);

        fileNameLabel = new QLabel(PicSearch);
        fileNameLabel->setObjectName(QString::fromUtf8("fileNameLabel"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(fileNameLabel->sizePolicy().hasHeightForWidth());
        fileNameLabel->setSizePolicy(sizePolicy);
        fileNameLabel->setFrameShape(QFrame::Box);
        fileNameLabel->setWordWrap(false);

        hboxLayout->addWidget(fileNameLabel);


        vboxLayout->addLayout(hboxLayout);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        foundFilesBox = new QListWidget(PicSearch);
        foundFilesBox->setObjectName(QString::fromUtf8("foundFilesBox"));

        hboxLayout1->addWidget(foundFilesBox);

        vboxLayout1 = new QVBoxLayout();
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        previewCheckBox = new QCheckBox(PicSearch);
        previewCheckBox->setObjectName(QString::fromUtf8("previewCheckBox"));

        vboxLayout1->addWidget(previewCheckBox);

        spacerItem = new QSpacerItem(21, 81, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout1->addItem(spacerItem);


        hboxLayout1->addLayout(vboxLayout1);

        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        previewLabel = new QLabel(PicSearch);
        previewLabel->setObjectName(QString::fromUtf8("previewLabel"));
        previewLabel->setMinimumSize(QSize(210, 210));
        previewLabel->setMaximumSize(QSize(210, 210));
        previewLabel->setFrameShape(QFrame::Box);
        previewLabel->setAlignment(Qt::AlignCenter);
        previewLabel->setWordWrap(false);

        vboxLayout2->addWidget(previewLabel);

        spacerItem1 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout2->addItem(spacerItem1);


        hboxLayout1->addLayout(vboxLayout2);


        vboxLayout->addLayout(hboxLayout1);

        matchCheckBox = new QCheckBox(PicSearch);
        matchCheckBox->setObjectName(QString::fromUtf8("matchCheckBox"));

        vboxLayout->addWidget(matchCheckBox);

        matchWarningLabel = new QLabel(PicSearch);
        matchWarningLabel->setObjectName(QString::fromUtf8("matchWarningLabel"));
        matchWarningLabel->setEnabled(true);
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(matchWarningLabel->sizePolicy().hasHeightForWidth());
        matchWarningLabel->setSizePolicy(sizePolicy1);
        matchWarningLabel->setWordWrap(true);

        vboxLayout->addWidget(matchWarningLabel);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        useButton = new QPushButton(PicSearch);
        useButton->setObjectName(QString::fromUtf8("useButton"));
        useButton->setEnabled(true);

        horizontalLayout->addWidget(useButton);

        cancelButton = new QPushButton(PicSearch);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        horizontalLayout->addWidget(cancelButton);


        vboxLayout->addLayout(horizontalLayout);


        retranslateUi(PicSearch);

        useButton->setDefault(true);


        QMetaObject::connectSlotsByName(PicSearch);
    } // setupUi

    void retranslateUi(QDialog *PicSearch)
    {
        PicSearch->setWindowTitle(QCoreApplication::translate("PicSearch", "Result", nullptr));
        textLabel1->setText(QCoreApplication::translate("PicSearch", "Search Results for: ", nullptr));
        fileNameLabel->setText(QString());
        previewCheckBox->setText(QCoreApplication::translate("PicSearch", "&Preview", nullptr));
#if QT_CONFIG(shortcut)
        previewCheckBox->setShortcut(QCoreApplication::translate("PicSearch", "Alt+P", nullptr));
#endif // QT_CONFIG(shortcut)
        matchCheckBox->setText(QCoreApplication::translate("PicSearch", "Apply to all &matching images", nullptr));
#if QT_CONFIG(shortcut)
        matchCheckBox->setShortcut(QCoreApplication::translate("PicSearch", "Alt+M", nullptr));
#endif // QT_CONFIG(shortcut)
        matchWarningLabel->setText(QCoreApplication::translate("PicSearch", "Since you selected an image with a broken link, only image frames with invalid links will be modified.", nullptr));
        useButton->setText(QCoreApplication::translate("PicSearch", "&Select", nullptr));
#if QT_CONFIG(shortcut)
        useButton->setShortcut(QCoreApplication::translate("PicSearch", "Alt+S", nullptr));
#endif // QT_CONFIG(shortcut)
        cancelButton->setText(QCoreApplication::translate("PicSearch", "Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PicSearch: public Ui_PicSearch {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PICSEARCH_H
