/****************************************************************************
** Meta object code from reading C++ file 'nodeeditpalette.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/nodeeditpalette.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'nodeeditpalette.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_NodePalette_t {
    QByteArrayData data[50];
    char stringdata0[474];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_NodePalette_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_NodePalette_t qt_meta_stringdata_NodePalette = {
    {
QT_MOC_LITERAL(0, 0, 11), // "NodePalette"
QT_MOC_LITERAL(1, 12, 10), // "DocChanged"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 13), // "paletteClosed"
QT_MOC_LITERAL(4, 38, 5), // "MoveK"
QT_MOC_LITERAL(5, 44, 4), // "AddN"
QT_MOC_LITERAL(6, 49, 4), // "DelN"
QT_MOC_LITERAL(7, 54, 9), // "MovePoint"
QT_MOC_LITERAL(8, 64, 6), // "SetSym"
QT_MOC_LITERAL(9, 71, 7), // "SetAsym"
QT_MOC_LITERAL(10, 79, 12), // "ResetControl"
QT_MOC_LITERAL(11, 92, 13), // "Reset1Control"
QT_MOC_LITERAL(12, 106, 12), // "ResetContour"
QT_MOC_LITERAL(13, 119, 23), // "ResetContourToImageClip"
QT_MOC_LITERAL(14, 143, 21), // "ResetShapeToImageClip"
QT_MOC_LITERAL(15, 165, 11), // "CloseBezier"
QT_MOC_LITERAL(16, 177, 9), // "SplitPoly"
QT_MOC_LITERAL(17, 187, 7), // "MirrorH"
QT_MOC_LITERAL(18, 195, 7), // "MirrorV"
QT_MOC_LITERAL(19, 203, 8), // "doRotCCW"
QT_MOC_LITERAL(20, 212, 7), // "doRotCW"
QT_MOC_LITERAL(21, 220, 8), // "doShrink"
QT_MOC_LITERAL(22, 229, 8), // "doExpand"
QT_MOC_LITERAL(23, 238, 8), // "doReduce"
QT_MOC_LITERAL(24, 247, 9), // "doEnlarge"
QT_MOC_LITERAL(25, 257, 6), // "ShearR"
QT_MOC_LITERAL(26, 264, 6), // "ShearL"
QT_MOC_LITERAL(27, 271, 6), // "ShearU"
QT_MOC_LITERAL(28, 278, 6), // "ShearD"
QT_MOC_LITERAL(29, 285, 13), // "ToggleAbsMode"
QT_MOC_LITERAL(30, 299, 13), // "ToggleConMode"
QT_MOC_LITERAL(31, 313, 13), // "TogglePreview"
QT_MOC_LITERAL(32, 327, 5), // "MoveN"
QT_MOC_LITERAL(33, 333, 5), // "SetXY"
QT_MOC_LITERAL(34, 339, 1), // "x"
QT_MOC_LITERAL(35, 341, 1), // "y"
QT_MOC_LITERAL(36, 343, 8), // "HaveNode"
QT_MOC_LITERAL(37, 352, 4), // "have"
QT_MOC_LITERAL(38, 357, 3), // "mov"
QT_MOC_LITERAL(39, 361, 6), // "IsOpen"
QT_MOC_LITERAL(40, 368, 10), // "PolyStatus"
QT_MOC_LITERAL(41, 379, 3), // "typ"
QT_MOC_LITERAL(42, 383, 4), // "size"
QT_MOC_LITERAL(43, 388, 13), // "iconSetChange"
QT_MOC_LITERAL(44, 402, 14), // "languageChange"
QT_MOC_LITERAL(45, 417, 10), // "unitChange"
QT_MOC_LITERAL(46, 428, 7), // "EndEdit"
QT_MOC_LITERAL(47, 436, 10), // "CancelEdit"
QT_MOC_LITERAL(48, 447, 19), // "ResetToEditDefaults"
QT_MOC_LITERAL(49, 467, 6) // "reject"

    },
    "NodePalette\0DocChanged\0\0paletteClosed\0"
    "MoveK\0AddN\0DelN\0MovePoint\0SetSym\0"
    "SetAsym\0ResetControl\0Reset1Control\0"
    "ResetContour\0ResetContourToImageClip\0"
    "ResetShapeToImageClip\0CloseBezier\0"
    "SplitPoly\0MirrorH\0MirrorV\0doRotCCW\0"
    "doRotCW\0doShrink\0doExpand\0doReduce\0"
    "doEnlarge\0ShearR\0ShearL\0ShearU\0ShearD\0"
    "ToggleAbsMode\0ToggleConMode\0TogglePreview\0"
    "MoveN\0SetXY\0x\0y\0HaveNode\0have\0mov\0"
    "IsOpen\0PolyStatus\0typ\0size\0iconSetChange\0"
    "languageChange\0unitChange\0EndEdit\0"
    "CancelEdit\0ResetToEditDefaults\0reject"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_NodePalette[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      42,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  224,    2, 0x06 /* Public */,
       3,    0,  225,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,  226,    2, 0x08 /* Private */,
       5,    0,  227,    2, 0x08 /* Private */,
       6,    0,  228,    2, 0x08 /* Private */,
       7,    0,  229,    2, 0x08 /* Private */,
       8,    0,  230,    2, 0x08 /* Private */,
       9,    0,  231,    2, 0x08 /* Private */,
      10,    0,  232,    2, 0x08 /* Private */,
      11,    0,  233,    2, 0x08 /* Private */,
      12,    0,  234,    2, 0x08 /* Private */,
      13,    0,  235,    2, 0x08 /* Private */,
      14,    0,  236,    2, 0x08 /* Private */,
      15,    0,  237,    2, 0x08 /* Private */,
      16,    0,  238,    2, 0x08 /* Private */,
      17,    0,  239,    2, 0x08 /* Private */,
      18,    0,  240,    2, 0x08 /* Private */,
      19,    0,  241,    2, 0x08 /* Private */,
      20,    0,  242,    2, 0x08 /* Private */,
      21,    0,  243,    2, 0x08 /* Private */,
      22,    0,  244,    2, 0x08 /* Private */,
      23,    0,  245,    2, 0x08 /* Private */,
      24,    0,  246,    2, 0x08 /* Private */,
      25,    0,  247,    2, 0x08 /* Private */,
      26,    0,  248,    2, 0x08 /* Private */,
      27,    0,  249,    2, 0x08 /* Private */,
      28,    0,  250,    2, 0x08 /* Private */,
      29,    0,  251,    2, 0x08 /* Private */,
      30,    0,  252,    2, 0x08 /* Private */,
      31,    0,  253,    2, 0x08 /* Private */,
      32,    0,  254,    2, 0x0a /* Public */,
      33,    2,  255,    2, 0x0a /* Public */,
      36,    2,  260,    2, 0x0a /* Public */,
      39,    0,  265,    2, 0x0a /* Public */,
      40,    2,  266,    2, 0x0a /* Public */,
      43,    0,  271,    2, 0x0a /* Public */,
      44,    0,  272,    2, 0x0a /* Public */,
      45,    0,  273,    2, 0x0a /* Public */,
      46,    0,  274,    2, 0x0a /* Public */,
      47,    0,  275,    2, 0x0a /* Public */,
      48,    0,  276,    2, 0x0a /* Public */,
      49,    0,  277,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,   34,   35,
    QMetaType::Void, QMetaType::Bool, QMetaType::Bool,   37,   38,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::UInt,   41,   42,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void NodePalette::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NodePalette *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->DocChanged(); break;
        case 1: _t->paletteClosed(); break;
        case 2: _t->MoveK(); break;
        case 3: _t->AddN(); break;
        case 4: _t->DelN(); break;
        case 5: _t->MovePoint(); break;
        case 6: _t->SetSym(); break;
        case 7: _t->SetAsym(); break;
        case 8: _t->ResetControl(); break;
        case 9: _t->Reset1Control(); break;
        case 10: _t->ResetContour(); break;
        case 11: _t->ResetContourToImageClip(); break;
        case 12: _t->ResetShapeToImageClip(); break;
        case 13: _t->CloseBezier(); break;
        case 14: _t->SplitPoly(); break;
        case 15: _t->MirrorH(); break;
        case 16: _t->MirrorV(); break;
        case 17: _t->doRotCCW(); break;
        case 18: _t->doRotCW(); break;
        case 19: _t->doShrink(); break;
        case 20: _t->doExpand(); break;
        case 21: _t->doReduce(); break;
        case 22: _t->doEnlarge(); break;
        case 23: _t->ShearR(); break;
        case 24: _t->ShearL(); break;
        case 25: _t->ShearU(); break;
        case 26: _t->ShearD(); break;
        case 27: _t->ToggleAbsMode(); break;
        case 28: _t->ToggleConMode(); break;
        case 29: _t->TogglePreview(); break;
        case 30: _t->MoveN(); break;
        case 31: _t->SetXY((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 32: _t->HaveNode((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 33: _t->IsOpen(); break;
        case 34: _t->PolyStatus((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< uint(*)>(_a[2]))); break;
        case 35: _t->iconSetChange(); break;
        case 36: _t->languageChange(); break;
        case 37: _t->unitChange(); break;
        case 38: _t->EndEdit(); break;
        case 39: _t->CancelEdit(); break;
        case 40: _t->ResetToEditDefaults(); break;
        case 41: _t->reject(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (NodePalette::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NodePalette::DocChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (NodePalette::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&NodePalette::paletteClosed)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject NodePalette::staticMetaObject = { {
    QMetaObject::SuperData::link<ScrPaletteBase::staticMetaObject>(),
    qt_meta_stringdata_NodePalette.data,
    qt_meta_data_NodePalette,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *NodePalette::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NodePalette::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_NodePalette.stringdata0))
        return static_cast<void*>(this);
    return ScrPaletteBase::qt_metacast(_clname);
}

int NodePalette::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScrPaletteBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 42)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 42;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 42)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 42;
    }
    return _id;
}

// SIGNAL 0
void NodePalette::DocChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void NodePalette::paletteClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
