/********************************************************************************
** Form generated from reading UI file 'osgeditor.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OSGEDITOR_H
#define UI_OSGEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "ui/AdapterWidget.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_OSGEditor
{
public:
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_4;
    QFrame *frame;
    QGridLayout *gridLayout_4;
    QSpacerItem *verticalSpacer;
    QSpacerItem *horizontalSpacer;
    ViewerQT *drawingarea;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer_2;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QPushButton *loadButton;
    QPushButton *clearButton;
    QGroupBox *groupBox_5;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_10;
    QComboBox *viewCombo;
    QGroupBox *groupBox_4;
    QGridLayout *gridLayout_2;
    QLabel *label_8;
    QComboBox *renderStyleCombo;
    QGridLayout *gridLayout;
    QLabel *label;
    QPushButton *buttonACcolor;
    QLabel *label_2;
    QPushButton *buttonFCcolor;
    QLabel *label_3;
    ScrSpinBox *transpSpin;
    QLabel *label_9;
    QComboBox *lightStyleCombo;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout_3;
    QLabel *label_7;
    ScrSpinBox *fovAngle;
    QHBoxLayout *horizontalLayout_3;
    QPushButton *buttonAddView;
    QPushButton *buttonRemoveView;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *OSGEditor)
    {
        if (OSGEditor->objectName().isEmpty())
            OSGEditor->setObjectName(QString::fromUtf8("OSGEditor"));
        OSGEditor->resize(658, 490);
        verticalLayout_3 = new QVBoxLayout(OSGEditor);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(9, 9, 9, 9);
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        frame = new QFrame(OSGEditor);
        frame->setObjectName(QString::fromUtf8("frame"));
        frame->setFrameShape(QFrame::StyledPanel);
        frame->setFrameShadow(QFrame::Sunken);
        frame->setLineWidth(2);
        gridLayout_4 = new QGridLayout(frame);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        verticalSpacer = new QSpacerItem(20, 118, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_4->addItem(verticalSpacer, 0, 1, 1, 1);

        horizontalSpacer = new QSpacerItem(105, 1, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_4->addItem(horizontalSpacer, 1, 0, 1, 1);

        drawingarea = new ViewerQT(frame);
        drawingarea->setObjectName(QString::fromUtf8("drawingarea"));
        QSizePolicy sizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(drawingarea->sizePolicy().hasHeightForWidth());
        drawingarea->setSizePolicy(sizePolicy);
        drawingarea->setMinimumSize(QSize(30, 30));
        drawingarea->setMaximumSize(QSize(400, 400));

        gridLayout_4->addWidget(drawingarea, 1, 1, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(104, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_4->addItem(horizontalSpacer_2, 1, 2, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 118, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_4->addItem(verticalSpacer_2, 2, 1, 1, 1);


        horizontalLayout_4->addWidget(frame);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        loadButton = new QPushButton(OSGEditor);
        loadButton->setObjectName(QString::fromUtf8("loadButton"));
        loadButton->setAutoDefault(false);

        horizontalLayout->addWidget(loadButton);

        clearButton = new QPushButton(OSGEditor);
        clearButton->setObjectName(QString::fromUtf8("clearButton"));
        clearButton->setAutoDefault(false);

        horizontalLayout->addWidget(clearButton);


        verticalLayout_2->addLayout(horizontalLayout);

        groupBox_5 = new QGroupBox(OSGEditor);
        groupBox_5->setObjectName(QString::fromUtf8("groupBox_5"));
        verticalLayout = new QVBoxLayout(groupBox_5);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        label_10 = new QLabel(groupBox_5);
        label_10->setObjectName(QString::fromUtf8("label_10"));

        horizontalLayout_2->addWidget(label_10);

        viewCombo = new QComboBox(groupBox_5);
        viewCombo->setObjectName(QString::fromUtf8("viewCombo"));
        viewCombo->setEditable(false);
        viewCombo->setInsertPolicy(QComboBox::InsertAtCurrent);

        horizontalLayout_2->addWidget(viewCombo);


        verticalLayout->addLayout(horizontalLayout_2);

        groupBox_4 = new QGroupBox(groupBox_5);
        groupBox_4->setObjectName(QString::fromUtf8("groupBox_4"));
        gridLayout_2 = new QGridLayout(groupBox_4);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        label_8 = new QLabel(groupBox_4);
        label_8->setObjectName(QString::fromUtf8("label_8"));
        QSizePolicy sizePolicy1(QSizePolicy::Maximum, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(label_8->sizePolicy().hasHeightForWidth());
        label_8->setSizePolicy(sizePolicy1);

        gridLayout_2->addWidget(label_8, 0, 0, 1, 1);

        renderStyleCombo = new QComboBox(groupBox_4);
        renderStyleCombo->addItem(QString());
        renderStyleCombo->addItem(QString());
        renderStyleCombo->addItem(QString());
        renderStyleCombo->addItem(QString());
        renderStyleCombo->addItem(QString());
        renderStyleCombo->addItem(QString());
        renderStyleCombo->addItem(QString());
        renderStyleCombo->addItem(QString());
        renderStyleCombo->addItem(QString());
        renderStyleCombo->addItem(QString());
        renderStyleCombo->addItem(QString());
        renderStyleCombo->addItem(QString());
        renderStyleCombo->addItem(QString());
        renderStyleCombo->addItem(QString());
        renderStyleCombo->addItem(QString());
        renderStyleCombo->setObjectName(QString::fromUtf8("renderStyleCombo"));

        gridLayout_2->addWidget(renderStyleCombo, 0, 1, 1, 1);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label = new QLabel(groupBox_4);
        label->setObjectName(QString::fromUtf8("label"));
        label->setEnabled(false);

        gridLayout->addWidget(label, 0, 0, 1, 1);

        buttonACcolor = new QPushButton(groupBox_4);
        buttonACcolor->setObjectName(QString::fromUtf8("buttonACcolor"));
        buttonACcolor->setEnabled(false);
        buttonACcolor->setMinimumSize(QSize(60, 20));
        buttonACcolor->setMaximumSize(QSize(60, 20));
        buttonACcolor->setAutoDefault(false);

        gridLayout->addWidget(buttonACcolor, 0, 1, 1, 1);

        label_2 = new QLabel(groupBox_4);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setEnabled(false);

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        buttonFCcolor = new QPushButton(groupBox_4);
        buttonFCcolor->setObjectName(QString::fromUtf8("buttonFCcolor"));
        buttonFCcolor->setEnabled(false);
        buttonFCcolor->setMinimumSize(QSize(60, 20));
        buttonFCcolor->setMaximumSize(QSize(60, 20));
        buttonFCcolor->setAutoDefault(false);

        gridLayout->addWidget(buttonFCcolor, 1, 1, 1, 1);

        label_3 = new QLabel(groupBox_4);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        label_3->setEnabled(false);

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        transpSpin = new ScrSpinBox(groupBox_4);
        transpSpin->setObjectName(QString::fromUtf8("transpSpin"));
        transpSpin->setEnabled(false);
        transpSpin->setMaximum(100);
        transpSpin->setSingleStep(10);

        gridLayout->addWidget(transpSpin, 2, 1, 1, 1);


        gridLayout_2->addLayout(gridLayout, 1, 1, 1, 1);

        label_9 = new QLabel(groupBox_4);
        label_9->setObjectName(QString::fromUtf8("label_9"));
        sizePolicy1.setHeightForWidth(label_9->sizePolicy().hasHeightForWidth());
        label_9->setSizePolicy(sizePolicy1);

        gridLayout_2->addWidget(label_9, 2, 0, 1, 1);

        lightStyleCombo = new QComboBox(groupBox_4);
        lightStyleCombo->addItem(QString());
        lightStyleCombo->addItem(QString());
        lightStyleCombo->addItem(QString());
        lightStyleCombo->addItem(QString());
        lightStyleCombo->addItem(QString());
        lightStyleCombo->addItem(QString());
        lightStyleCombo->addItem(QString());
        lightStyleCombo->addItem(QString());
        lightStyleCombo->addItem(QString());
        lightStyleCombo->addItem(QString());
        lightStyleCombo->addItem(QString());
        lightStyleCombo->addItem(QString());
        lightStyleCombo->setObjectName(QString::fromUtf8("lightStyleCombo"));

        gridLayout_2->addWidget(lightStyleCombo, 2, 1, 1, 1);


        verticalLayout->addWidget(groupBox_4);

        groupBox_3 = new QGroupBox(groupBox_5);
        groupBox_3->setObjectName(QString::fromUtf8("groupBox_3"));
        groupBox_3->setFlat(false);
        gridLayout_3 = new QGridLayout(groupBox_3);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        label_7 = new QLabel(groupBox_3);
        label_7->setObjectName(QString::fromUtf8("label_7"));
        label_7->setFrameShape(QFrame::NoFrame);

        gridLayout_3->addWidget(label_7, 0, 0, 1, 1);

        fovAngle = new ScrSpinBox(groupBox_3);
        fovAngle->setObjectName(QString::fromUtf8("fovAngle"));
        fovAngle->setMinimum(1.000000000000000);
        fovAngle->setMaximum(180.000000000000000);
        fovAngle->setValue(1.000000000000000);

        gridLayout_3->addWidget(fovAngle, 0, 1, 1, 1);


        verticalLayout->addWidget(groupBox_3);


        verticalLayout_2->addWidget(groupBox_5);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        buttonAddView = new QPushButton(OSGEditor);
        buttonAddView->setObjectName(QString::fromUtf8("buttonAddView"));
        buttonAddView->setAutoDefault(false);

        horizontalLayout_3->addWidget(buttonAddView);

        buttonRemoveView = new QPushButton(OSGEditor);
        buttonRemoveView->setObjectName(QString::fromUtf8("buttonRemoveView"));
        buttonRemoveView->setAutoDefault(false);

        horizontalLayout_3->addWidget(buttonRemoveView);


        verticalLayout_2->addLayout(horizontalLayout_3);


        horizontalLayout_4->addLayout(verticalLayout_2);


        verticalLayout_3->addLayout(horizontalLayout_4);

        buttonBox = new QDialogButtonBox(OSGEditor);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout_3->addWidget(buttonBox);


        retranslateUi(OSGEditor);
        QObject::connect(buttonBox, SIGNAL(accepted()), OSGEditor, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), OSGEditor, SLOT(reject()));

        lightStyleCombo->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(OSGEditor);
    } // setupUi

    void retranslateUi(QDialog *OSGEditor)
    {
        OSGEditor->setWindowTitle(QCoreApplication::translate("OSGEditor", "3D Properties", nullptr));
        loadButton->setText(QCoreApplication::translate("OSGEditor", "Load Model ...", nullptr));
        clearButton->setText(QCoreApplication::translate("OSGEditor", "Clear Model", nullptr));
        groupBox_5->setTitle(QCoreApplication::translate("OSGEditor", "Views", nullptr));
        label_10->setText(QCoreApplication::translate("OSGEditor", "Name:", nullptr));
        groupBox_4->setTitle(QCoreApplication::translate("OSGEditor", "Options", nullptr));
        label_8->setText(QCoreApplication::translate("OSGEditor", "Render Style:", nullptr));
        renderStyleCombo->setItemText(0, QCoreApplication::translate("OSGEditor", "Solid", nullptr));
        renderStyleCombo->setItemText(1, QCoreApplication::translate("OSGEditor", "Solid Wireframe", nullptr));
        renderStyleCombo->setItemText(2, QCoreApplication::translate("OSGEditor", "Transparent", nullptr));
        renderStyleCombo->setItemText(3, QCoreApplication::translate("OSGEditor", "Transparent Wireframe", nullptr));
        renderStyleCombo->setItemText(4, QCoreApplication::translate("OSGEditor", "Bounding Box", nullptr));
        renderStyleCombo->setItemText(5, QCoreApplication::translate("OSGEditor", "Transparent Bounding Box", nullptr));
        renderStyleCombo->setItemText(6, QCoreApplication::translate("OSGEditor", "Transparent Bounding Box Outline", nullptr));
        renderStyleCombo->setItemText(7, QCoreApplication::translate("OSGEditor", "Wireframe", nullptr));
        renderStyleCombo->setItemText(8, QCoreApplication::translate("OSGEditor", "Shaded Wireframe", nullptr));
        renderStyleCombo->setItemText(9, QCoreApplication::translate("OSGEditor", "Hidden Wireframe", nullptr));
        renderStyleCombo->setItemText(10, QCoreApplication::translate("OSGEditor", "Vertices", nullptr));
        renderStyleCombo->setItemText(11, QCoreApplication::translate("OSGEditor", "Shaded Vertices", nullptr));
        renderStyleCombo->setItemText(12, QCoreApplication::translate("OSGEditor", "Illustration", nullptr));
        renderStyleCombo->setItemText(13, QCoreApplication::translate("OSGEditor", "Solid Outline", nullptr));
        renderStyleCombo->setItemText(14, QCoreApplication::translate("OSGEditor", "Shaded Illustration", nullptr));

        label->setText(QCoreApplication::translate("OSGEditor", "Line Color:", nullptr));
        buttonACcolor->setText(QString());
        label_2->setText(QCoreApplication::translate("OSGEditor", "Face Color:", nullptr));
        buttonFCcolor->setText(QString());
        label_3->setText(QCoreApplication::translate("OSGEditor", "Opacity:", nullptr));
        transpSpin->setSuffix(QCoreApplication::translate("OSGEditor", " %", nullptr));
        label_9->setText(QCoreApplication::translate("OSGEditor", "Lighting:", nullptr));
        lightStyleCombo->setItemText(0, QCoreApplication::translate("OSGEditor", "None", nullptr));
        lightStyleCombo->setItemText(1, QCoreApplication::translate("OSGEditor", "Headlamp", nullptr));
        lightStyleCombo->setItemText(2, QCoreApplication::translate("OSGEditor", "White", nullptr));
        lightStyleCombo->setItemText(3, QCoreApplication::translate("OSGEditor", "Day", nullptr));
        lightStyleCombo->setItemText(4, QCoreApplication::translate("OSGEditor", "Night", nullptr));
        lightStyleCombo->setItemText(5, QCoreApplication::translate("OSGEditor", "Hard", nullptr));
        lightStyleCombo->setItemText(6, QCoreApplication::translate("OSGEditor", "Primary", nullptr));
        lightStyleCombo->setItemText(7, QCoreApplication::translate("OSGEditor", "Blue", nullptr));
        lightStyleCombo->setItemText(8, QCoreApplication::translate("OSGEditor", "Red", nullptr));
        lightStyleCombo->setItemText(9, QCoreApplication::translate("OSGEditor", "Cube", nullptr));
        lightStyleCombo->setItemText(10, QCoreApplication::translate("OSGEditor", "CAD", nullptr));
        lightStyleCombo->setItemText(11, QCoreApplication::translate("OSGEditor", "Artwork", nullptr));

        groupBox_3->setTitle(QCoreApplication::translate("OSGEditor", "Field of Vision", nullptr));
        label_7->setText(QCoreApplication::translate("OSGEditor", "Angle:", nullptr));
        buttonAddView->setText(QCoreApplication::translate("OSGEditor", "Add View...", nullptr));
        buttonRemoveView->setText(QCoreApplication::translate("OSGEditor", "Remove View", nullptr));
    } // retranslateUi

};

namespace Ui {
    class OSGEditor: public Ui_OSGEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OSGEDITOR_H
