/********************************************************************************
** Form generated from reading UI file 'prefs_hyphenatorbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_HYPHENATORBASE_H
#define UI_PREFS_HYPHENATORBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Prefs_Hyphenator
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QFrame *line;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *hyphSuggestionsCheckBox;
    QCheckBox *hyphAutoCheckBox;
    QSpacerItem *verticalSpacer_2;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_2;
    QGroupBox *groupBox_2;
    QGridLayout *_2;
    QListWidget *exceptionListWidget;
    QPushButton *exceptionAddButton;
    QPushButton *exceptionEditButton;
    QPushButton *exceptionRemoveButton;
    QSpacerItem *spacerItem;
    QGroupBox *groupBox_3;
    QGridLayout *_3;
    QListWidget *ignoreListWidget;
    QPushButton *ignoreAddButton;
    QPushButton *ignoreEditButton;
    QPushButton *ignoreRemoveButton;
    QSpacerItem *spacerItem1;

    void setupUi(QWidget *Prefs_Hyphenator)
    {
        if (Prefs_Hyphenator->objectName().isEmpty())
            Prefs_Hyphenator->setObjectName(QString::fromUtf8("Prefs_Hyphenator"));
        Prefs_Hyphenator->resize(711, 550);
        verticalLayout = new QVBoxLayout(Prefs_Hyphenator);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(Prefs_Hyphenator);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        verticalLayout->addWidget(label);

        line = new QFrame(Prefs_Hyphenator);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        tabWidget = new QTabWidget(Prefs_Hyphenator);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tab = new QWidget();
        tab->setObjectName(QString::fromUtf8("tab"));
        verticalLayout_2 = new QVBoxLayout(tab);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        hyphSuggestionsCheckBox = new QCheckBox(tab);
        hyphSuggestionsCheckBox->setObjectName(QString::fromUtf8("hyphSuggestionsCheckBox"));

        verticalLayout_2->addWidget(hyphSuggestionsCheckBox);

        hyphAutoCheckBox = new QCheckBox(tab);
        hyphAutoCheckBox->setObjectName(QString::fromUtf8("hyphAutoCheckBox"));

        verticalLayout_2->addWidget(hyphAutoCheckBox);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_2);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName(QString::fromUtf8("tab_2"));
        verticalLayout_3 = new QVBoxLayout(tab_2);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        groupBox_2 = new QGroupBox(tab_2);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        _2 = new QGridLayout(groupBox_2);
        _2->setObjectName(QString::fromUtf8("_2"));
        exceptionListWidget = new QListWidget(groupBox_2);
        exceptionListWidget->setObjectName(QString::fromUtf8("exceptionListWidget"));

        _2->addWidget(exceptionListWidget, 0, 0, 1, 4);

        exceptionAddButton = new QPushButton(groupBox_2);
        exceptionAddButton->setObjectName(QString::fromUtf8("exceptionAddButton"));

        _2->addWidget(exceptionAddButton, 1, 0, 1, 1);

        exceptionEditButton = new QPushButton(groupBox_2);
        exceptionEditButton->setObjectName(QString::fromUtf8("exceptionEditButton"));

        _2->addWidget(exceptionEditButton, 1, 1, 1, 1);

        exceptionRemoveButton = new QPushButton(groupBox_2);
        exceptionRemoveButton->setObjectName(QString::fromUtf8("exceptionRemoveButton"));

        _2->addWidget(exceptionRemoveButton, 1, 2, 1, 1);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _2->addItem(spacerItem, 1, 3, 1, 1);


        horizontalLayout_2->addWidget(groupBox_2);

        groupBox_3 = new QGroupBox(tab_2);
        groupBox_3->setObjectName(QString::fromUtf8("groupBox_3"));
        _3 = new QGridLayout(groupBox_3);
        _3->setObjectName(QString::fromUtf8("_3"));
        ignoreListWidget = new QListWidget(groupBox_3);
        ignoreListWidget->setObjectName(QString::fromUtf8("ignoreListWidget"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ignoreListWidget->sizePolicy().hasHeightForWidth());
        ignoreListWidget->setSizePolicy(sizePolicy);

        _3->addWidget(ignoreListWidget, 0, 0, 1, 4);

        ignoreAddButton = new QPushButton(groupBox_3);
        ignoreAddButton->setObjectName(QString::fromUtf8("ignoreAddButton"));

        _3->addWidget(ignoreAddButton, 1, 0, 1, 1);

        ignoreEditButton = new QPushButton(groupBox_3);
        ignoreEditButton->setObjectName(QString::fromUtf8("ignoreEditButton"));

        _3->addWidget(ignoreEditButton, 1, 1, 1, 1);

        ignoreRemoveButton = new QPushButton(groupBox_3);
        ignoreRemoveButton->setObjectName(QString::fromUtf8("ignoreRemoveButton"));

        _3->addWidget(ignoreRemoveButton, 1, 2, 1, 1);

        spacerItem1 = new QSpacerItem(21, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _3->addItem(spacerItem1, 1, 3, 1, 1);


        horizontalLayout_2->addWidget(groupBox_3);


        verticalLayout_3->addLayout(horizontalLayout_2);

        tabWidget->addTab(tab_2, QString());

        verticalLayout->addWidget(tabWidget);


        retranslateUi(Prefs_Hyphenator);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Prefs_Hyphenator);
    } // setupUi

    void retranslateUi(QWidget *Prefs_Hyphenator)
    {
        label->setText(QCoreApplication::translate("Prefs_Hyphenator", "Hyphenation", nullptr));
#if QT_CONFIG(tooltip)
        hyphSuggestionsCheckBox->setToolTip(QCoreApplication::translate("Prefs_Hyphenator", "A dialog box showing all possible hyphens for each word will show up when you use the Extras > Hyphenate Text option", nullptr));
#endif // QT_CONFIG(tooltip)
        hyphSuggestionsCheckBox->setText(QCoreApplication::translate("Prefs_Hyphenator", "&Hyphenation Suggestions", nullptr));
#if QT_CONFIG(tooltip)
        hyphAutoCheckBox->setToolTip(QCoreApplication::translate("Prefs_Hyphenator", "Enables automatic hyphenation of your text while typing", nullptr));
#endif // QT_CONFIG(tooltip)
        hyphAutoCheckBox->setText(QCoreApplication::translate("Prefs_Hyphenator", "Hyphenate Text Automatically &During Typing", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("Prefs_Hyphenator", "Behaviour", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("Prefs_Hyphenator", "Exceptions", nullptr));
        exceptionAddButton->setText(QString());
        exceptionEditButton->setText(QCoreApplication::translate("Prefs_Hyphenator", "Edit", nullptr));
        exceptionRemoveButton->setText(QString());
        groupBox_3->setTitle(QCoreApplication::translate("Prefs_Hyphenator", "Ignore List", nullptr));
        ignoreAddButton->setText(QString());
        ignoreEditButton->setText(QCoreApplication::translate("Prefs_Hyphenator", "Edit", nullptr));
        ignoreRemoveButton->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("Prefs_Hyphenator", "Word Lists", nullptr));
        (void)Prefs_Hyphenator;
    } // retranslateUi

};

namespace Ui {
    class Prefs_Hyphenator: public Ui_Prefs_Hyphenator {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_HYPHENATORBASE_H
