/********************************************************************************
** Form generated from reading UI file 'propertiespalette_groupbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTIESPALETTE_GROUPBASE_H
#define UI_PROPERTIESPALETTE_GROUPBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_PropertiesPalette_GroupBase
{
public:
    QVBoxLayout *verticalLayout_3;
    QCheckBox *clipGroups;
    QSpacerItem *verticalSpacer;
    QButtonGroup *textFlowBtnGroup;

    void setupUi(QWidget *PropertiesPalette_GroupBase)
    {
        if (PropertiesPalette_GroupBase->objectName().isEmpty())
            PropertiesPalette_GroupBase->setObjectName(QString::fromUtf8("PropertiesPalette_GroupBase"));
        PropertiesPalette_GroupBase->resize(212, 47);
        verticalLayout_3 = new QVBoxLayout(PropertiesPalette_GroupBase);
        verticalLayout_3->setSpacing(3);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(3, 3, 3, 3);
        clipGroups = new QCheckBox(PropertiesPalette_GroupBase);
        clipGroups->setObjectName(QString::fromUtf8("clipGroups"));

        verticalLayout_3->addWidget(clipGroups);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_3->addItem(verticalSpacer);


        retranslateUi(PropertiesPalette_GroupBase);

        QMetaObject::connectSlotsByName(PropertiesPalette_GroupBase);
    } // setupUi

    void retranslateUi(QWidget *PropertiesPalette_GroupBase)
    {
#if QT_CONFIG(tooltip)
        clipGroups->setToolTip(QCoreApplication::translate("PropertiesPalette_GroupBase", "When the group's bounding frame has been edited, clip the contents of the page items to the edited bounding frame", nullptr));
#endif // QT_CONFIG(tooltip)
        clipGroups->setText(QCoreApplication::translate("PropertiesPalette_GroupBase", "Group clips Contents", nullptr));
        (void)PropertiesPalette_GroupBase;
    } // retranslateUi

};

namespace Ui {
    class PropertiesPalette_GroupBase: public Ui_PropertiesPalette_GroupBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTIESPALETTE_GROUPBASE_H
