/********************************************************************************
** Form generated from reading UI file 'prefs_documentsetupbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_DOCUMENTSETUPBASE_H
#define UI_PREFS_DOCUMENTSETUPBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/newmarginwidget.h>
#include <ui/scrspinbox.h>

QT_BEGIN_NAMESPACE

class Ui_Prefs_DocumentSetup
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *titleLabel;
    QFrame *line;
    QTabWidget *tabWidget;
    QWidget *sizeTab;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_3;
    QComboBox *languageComboBox;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer;
    QLabel *pageSizeTitleLabel;
    QFrame *line_2;
    QFormLayout *formLayout;
    QLabel *pageSizeLabel;
    QHBoxLayout *pageSizeLayout;
    QComboBox *pageSizeComboBox;
    QToolButton *pageSizeLinkToolButton;
    QLabel *pageOrientatioLabel;
    QComboBox *pageOrientationComboBox;
    QLabel *pageUnitsLabel;
    QComboBox *pageUnitsComboBox;
    QLabel *pageWidthLabel;
    ScrSpinBox *pageWidthSpinBox;
    QLabel *pageHeightLabel;
    ScrSpinBox *pageHeightSpinBox;
    QCheckBox *applySizesToAllPagesCheckBox;
    QCheckBox *applySizesToAllMasterPagesCheckBox;
    QSpacerItem *verticalSpacer_5;
    QWidget *layoutTab;
    QVBoxLayout *verticalLayout_6;
    QLabel *pageLayoutTitleLabel;
    QFrame *line_3;
    QFormLayout *pageLayoutLayout;
    QLabel *layoutFirstPageIsLabel;
    QComboBox *layoutFirstPageIsComboBox;
    QRadioButton *fourFoldRadioButton;
    QRadioButton *threeFoldRadioButton;
    QRadioButton *facingPagesRadioButton;
    QRadioButton *singlePageRadioButton;
    QSpacerItem *verticalSpacer_8;
    QWidget *marginsTab;
    QVBoxLayout *verticalLayout_3;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_5;
    QLabel *marginsTitleLabel;
    QFrame *line_5;
    QHBoxLayout *horizontalLayout_6;
    NewMarginWidget *marginsWidget;
    QSpacerItem *horizontalSpacer_6;
    QCheckBox *applyMarginsToAllPagesCheckBox;
    QCheckBox *applyMarginsToAllMasterPagesCheckBox;
    QSpacerItem *verticalSpacer_3;
    QLabel *bleedsTitleLabel;
    QFrame *line_4;
    QHBoxLayout *horizontalLayout_7;
    NewMarginWidget *bleedsWidget;
    QSpacerItem *horizontalSpacer_7;
    QSpacerItem *verticalSpacer_4;
    QWidget *saveTab;
    QVBoxLayout *verticalLayout;
    QCheckBox *saveCompressedCheckBox;
    QCheckBox *emergencyCheckBox;
    QSpacerItem *verticalSpacer_6;
    QLabel *autosaveTitleLable;
    QFrame *line_6;
    QCheckBox *autosaveCheckBox;
    QHBoxLayout *horizontalLayout;
    QGridLayout *gridLayout;
    QLabel *label;
    QSpinBox *autosaveIntervalSpinBox;
    QLabel *label_2;
    QSpinBox *autosaveCountSpinBox;
    QSpacerItem *horizontalSpacer_2;
    QGridLayout *gridLayout_2;
    QRadioButton *autosaveDocRadio;
    QRadioButton *autosaveDirRadio;
    QLineEdit *autosaveDirEdit;
    QPushButton *changeAutoDir;
    QCheckBox *autosaveKeepCheckBox;
    QCheckBox *showAutosaveClockOnCanvasCheckBox;
    QSpacerItem *verticalSpacer_7;
    QLabel *undoRedoTitleLabel;
    QFrame *line_7;
    QCheckBox *undoCheckBox;
    QHBoxLayout *horizontalLayout_2;
    QLabel *undoLengthLabel;
    QSpinBox *undoLengthSpinBox;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer_2;
    QButtonGroup *pageLayoutButtonGroup;

    void setupUi(QWidget *Prefs_DocumentSetup)
    {
        if (Prefs_DocumentSetup->objectName().isEmpty())
            Prefs_DocumentSetup->setObjectName(QString::fromUtf8("Prefs_DocumentSetup"));
        Prefs_DocumentSetup->resize(498, 650);
        verticalLayout_2 = new QVBoxLayout(Prefs_DocumentSetup);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleLabel = new QLabel(Prefs_DocumentSetup);
        titleLabel->setObjectName(QString::fromUtf8("titleLabel"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        titleLabel->setFont(font);

        verticalLayout_2->addWidget(titleLabel);

        line = new QFrame(Prefs_DocumentSetup);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line);

        tabWidget = new QTabWidget(Prefs_DocumentSetup);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        sizeTab = new QWidget();
        sizeTab->setObjectName(QString::fromUtf8("sizeTab"));
        verticalLayout_4 = new QVBoxLayout(sizeTab);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        label_3 = new QLabel(sizeTab);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        horizontalLayout_3->addWidget(label_3);

        languageComboBox = new QComboBox(sizeTab);
        languageComboBox->setObjectName(QString::fromUtf8("languageComboBox"));

        horizontalLayout_3->addWidget(languageComboBox);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);


        verticalLayout_4->addLayout(horizontalLayout_3);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_4->addItem(verticalSpacer);

        pageSizeTitleLabel = new QLabel(sizeTab);
        pageSizeTitleLabel->setObjectName(QString::fromUtf8("pageSizeTitleLabel"));
        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
        pageSizeTitleLabel->setFont(font1);

        verticalLayout_4->addWidget(pageSizeTitleLabel);

        line_2 = new QFrame(sizeTab);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        verticalLayout_4->addWidget(line_2);

        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        pageSizeLabel = new QLabel(sizeTab);
        pageSizeLabel->setObjectName(QString::fromUtf8("pageSizeLabel"));

        formLayout->setWidget(0, QFormLayout::LabelRole, pageSizeLabel);

        pageSizeLayout = new QHBoxLayout();
        pageSizeLayout->setObjectName(QString::fromUtf8("pageSizeLayout"));
        pageSizeComboBox = new QComboBox(sizeTab);
        pageSizeComboBox->setObjectName(QString::fromUtf8("pageSizeComboBox"));

        pageSizeLayout->addWidget(pageSizeComboBox);

        pageSizeLinkToolButton = new QToolButton(sizeTab);
        pageSizeLinkToolButton->setObjectName(QString::fromUtf8("pageSizeLinkToolButton"));
        pageSizeLinkToolButton->setArrowType(Qt::NoArrow);

        pageSizeLayout->addWidget(pageSizeLinkToolButton);


        formLayout->setLayout(0, QFormLayout::FieldRole, pageSizeLayout);

        pageOrientatioLabel = new QLabel(sizeTab);
        pageOrientatioLabel->setObjectName(QString::fromUtf8("pageOrientatioLabel"));

        formLayout->setWidget(1, QFormLayout::LabelRole, pageOrientatioLabel);

        pageOrientationComboBox = new QComboBox(sizeTab);
        pageOrientationComboBox->setObjectName(QString::fromUtf8("pageOrientationComboBox"));

        formLayout->setWidget(1, QFormLayout::FieldRole, pageOrientationComboBox);

        pageUnitsLabel = new QLabel(sizeTab);
        pageUnitsLabel->setObjectName(QString::fromUtf8("pageUnitsLabel"));

        formLayout->setWidget(2, QFormLayout::LabelRole, pageUnitsLabel);

        pageUnitsComboBox = new QComboBox(sizeTab);
        pageUnitsComboBox->setObjectName(QString::fromUtf8("pageUnitsComboBox"));

        formLayout->setWidget(2, QFormLayout::FieldRole, pageUnitsComboBox);

        pageWidthLabel = new QLabel(sizeTab);
        pageWidthLabel->setObjectName(QString::fromUtf8("pageWidthLabel"));

        formLayout->setWidget(3, QFormLayout::LabelRole, pageWidthLabel);

        pageWidthSpinBox = new ScrSpinBox(sizeTab);
        pageWidthSpinBox->setObjectName(QString::fromUtf8("pageWidthSpinBox"));

        formLayout->setWidget(3, QFormLayout::FieldRole, pageWidthSpinBox);

        pageHeightLabel = new QLabel(sizeTab);
        pageHeightLabel->setObjectName(QString::fromUtf8("pageHeightLabel"));

        formLayout->setWidget(4, QFormLayout::LabelRole, pageHeightLabel);

        pageHeightSpinBox = new ScrSpinBox(sizeTab);
        pageHeightSpinBox->setObjectName(QString::fromUtf8("pageHeightSpinBox"));

        formLayout->setWidget(4, QFormLayout::FieldRole, pageHeightSpinBox);


        verticalLayout_4->addLayout(formLayout);

        applySizesToAllPagesCheckBox = new QCheckBox(sizeTab);
        applySizesToAllPagesCheckBox->setObjectName(QString::fromUtf8("applySizesToAllPagesCheckBox"));

        verticalLayout_4->addWidget(applySizesToAllPagesCheckBox);

        applySizesToAllMasterPagesCheckBox = new QCheckBox(sizeTab);
        applySizesToAllMasterPagesCheckBox->setObjectName(QString::fromUtf8("applySizesToAllMasterPagesCheckBox"));

        verticalLayout_4->addWidget(applySizesToAllMasterPagesCheckBox);

        verticalSpacer_5 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_5);

        tabWidget->addTab(sizeTab, QString());
        layoutTab = new QWidget();
        layoutTab->setObjectName(QString::fromUtf8("layoutTab"));
        verticalLayout_6 = new QVBoxLayout(layoutTab);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        pageLayoutTitleLabel = new QLabel(layoutTab);
        pageLayoutTitleLabel->setObjectName(QString::fromUtf8("pageLayoutTitleLabel"));
        pageLayoutTitleLabel->setFont(font1);

        verticalLayout_6->addWidget(pageLayoutTitleLabel);

        line_3 = new QFrame(layoutTab);
        line_3->setObjectName(QString::fromUtf8("line_3"));
        line_3->setFrameShape(QFrame::HLine);
        line_3->setFrameShadow(QFrame::Sunken);

        verticalLayout_6->addWidget(line_3);

        pageLayoutLayout = new QFormLayout();
        pageLayoutLayout->setObjectName(QString::fromUtf8("pageLayoutLayout"));
        layoutFirstPageIsLabel = new QLabel(layoutTab);
        layoutFirstPageIsLabel->setObjectName(QString::fromUtf8("layoutFirstPageIsLabel"));

        pageLayoutLayout->setWidget(4, QFormLayout::LabelRole, layoutFirstPageIsLabel);

        layoutFirstPageIsComboBox = new QComboBox(layoutTab);
        layoutFirstPageIsComboBox->setObjectName(QString::fromUtf8("layoutFirstPageIsComboBox"));
        layoutFirstPageIsComboBox->setSizeAdjustPolicy(QComboBox::AdjustToContents);

        pageLayoutLayout->setWidget(4, QFormLayout::FieldRole, layoutFirstPageIsComboBox);

        fourFoldRadioButton = new QRadioButton(layoutTab);
        pageLayoutButtonGroup = new QButtonGroup(Prefs_DocumentSetup);
        pageLayoutButtonGroup->setObjectName(QString::fromUtf8("pageLayoutButtonGroup"));
        pageLayoutButtonGroup->addButton(fourFoldRadioButton);
        fourFoldRadioButton->setObjectName(QString::fromUtf8("fourFoldRadioButton"));

        pageLayoutLayout->setWidget(3, QFormLayout::FieldRole, fourFoldRadioButton);

        threeFoldRadioButton = new QRadioButton(layoutTab);
        pageLayoutButtonGroup->addButton(threeFoldRadioButton);
        threeFoldRadioButton->setObjectName(QString::fromUtf8("threeFoldRadioButton"));

        pageLayoutLayout->setWidget(2, QFormLayout::FieldRole, threeFoldRadioButton);

        facingPagesRadioButton = new QRadioButton(layoutTab);
        pageLayoutButtonGroup->addButton(facingPagesRadioButton);
        facingPagesRadioButton->setObjectName(QString::fromUtf8("facingPagesRadioButton"));

        pageLayoutLayout->setWidget(1, QFormLayout::FieldRole, facingPagesRadioButton);

        singlePageRadioButton = new QRadioButton(layoutTab);
        pageLayoutButtonGroup->addButton(singlePageRadioButton);
        singlePageRadioButton->setObjectName(QString::fromUtf8("singlePageRadioButton"));
        singlePageRadioButton->setChecked(true);

        pageLayoutLayout->setWidget(0, QFormLayout::FieldRole, singlePageRadioButton);


        verticalLayout_6->addLayout(pageLayoutLayout);

        verticalSpacer_8 = new QSpacerItem(20, 353, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_6->addItem(verticalSpacer_8);

        tabWidget->addTab(layoutTab, QString());
        marginsTab = new QWidget();
        marginsTab->setObjectName(QString::fromUtf8("marginsTab"));
        verticalLayout_3 = new QVBoxLayout(marginsTab);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        scrollArea = new QScrollArea(marginsTab);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setLineWidth(0);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 458, 540));
        verticalLayout_5 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        marginsTitleLabel = new QLabel(scrollAreaWidgetContents);
        marginsTitleLabel->setObjectName(QString::fromUtf8("marginsTitleLabel"));
        marginsTitleLabel->setFont(font1);

        verticalLayout_5->addWidget(marginsTitleLabel);

        line_5 = new QFrame(scrollAreaWidgetContents);
        line_5->setObjectName(QString::fromUtf8("line_5"));
        line_5->setFrameShape(QFrame::HLine);
        line_5->setFrameShadow(QFrame::Sunken);

        verticalLayout_5->addWidget(line_5);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        horizontalLayout_6->setContentsMargins(0, 0, 0, 0);
        marginsWidget = new NewMarginWidget(scrollAreaWidgetContents);
        marginsWidget->setObjectName(QString::fromUtf8("marginsWidget"));

        horizontalLayout_6->addWidget(marginsWidget);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_6);


        verticalLayout_5->addLayout(horizontalLayout_6);

        applyMarginsToAllPagesCheckBox = new QCheckBox(scrollAreaWidgetContents);
        applyMarginsToAllPagesCheckBox->setObjectName(QString::fromUtf8("applyMarginsToAllPagesCheckBox"));

        verticalLayout_5->addWidget(applyMarginsToAllPagesCheckBox);

        applyMarginsToAllMasterPagesCheckBox = new QCheckBox(scrollAreaWidgetContents);
        applyMarginsToAllMasterPagesCheckBox->setObjectName(QString::fromUtf8("applyMarginsToAllMasterPagesCheckBox"));

        verticalLayout_5->addWidget(applyMarginsToAllMasterPagesCheckBox);

        verticalSpacer_3 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_5->addItem(verticalSpacer_3);

        bleedsTitleLabel = new QLabel(scrollAreaWidgetContents);
        bleedsTitleLabel->setObjectName(QString::fromUtf8("bleedsTitleLabel"));
        bleedsTitleLabel->setFont(font1);

        verticalLayout_5->addWidget(bleedsTitleLabel);

        line_4 = new QFrame(scrollAreaWidgetContents);
        line_4->setObjectName(QString::fromUtf8("line_4"));
        line_4->setFrameShape(QFrame::HLine);
        line_4->setFrameShadow(QFrame::Sunken);

        verticalLayout_5->addWidget(line_4);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        horizontalLayout_7->setContentsMargins(0, 0, 0, 0);
        bleedsWidget = new NewMarginWidget(scrollAreaWidgetContents);
        bleedsWidget->setObjectName(QString::fromUtf8("bleedsWidget"));

        horizontalLayout_7->addWidget(bleedsWidget);

        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_7);


        verticalLayout_5->addLayout(horizontalLayout_7);

        verticalSpacer_4 = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_4);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_3->addWidget(scrollArea);

        tabWidget->addTab(marginsTab, QString());
        saveTab = new QWidget();
        saveTab->setObjectName(QString::fromUtf8("saveTab"));
        verticalLayout = new QVBoxLayout(saveTab);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        saveCompressedCheckBox = new QCheckBox(saveTab);
        saveCompressedCheckBox->setObjectName(QString::fromUtf8("saveCompressedCheckBox"));

        verticalLayout->addWidget(saveCompressedCheckBox);

        emergencyCheckBox = new QCheckBox(saveTab);
        emergencyCheckBox->setObjectName(QString::fromUtf8("emergencyCheckBox"));

        verticalLayout->addWidget(emergencyCheckBox);

        verticalSpacer_6 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout->addItem(verticalSpacer_6);

        autosaveTitleLable = new QLabel(saveTab);
        autosaveTitleLable->setObjectName(QString::fromUtf8("autosaveTitleLable"));
        autosaveTitleLable->setFont(font1);

        verticalLayout->addWidget(autosaveTitleLable);

        line_6 = new QFrame(saveTab);
        line_6->setObjectName(QString::fromUtf8("line_6"));
        line_6->setFrameShape(QFrame::HLine);
        line_6->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line_6);

        autosaveCheckBox = new QCheckBox(saveTab);
        autosaveCheckBox->setObjectName(QString::fromUtf8("autosaveCheckBox"));

        verticalLayout->addWidget(autosaveCheckBox);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label = new QLabel(saveTab);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 0, 1, 1);

        autosaveIntervalSpinBox = new QSpinBox(saveTab);
        autosaveIntervalSpinBox->setObjectName(QString::fromUtf8("autosaveIntervalSpinBox"));
        autosaveIntervalSpinBox->setMinimum(1);

        gridLayout->addWidget(autosaveIntervalSpinBox, 0, 1, 1, 1);

        label_2 = new QLabel(saveTab);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        autosaveCountSpinBox = new QSpinBox(saveTab);
        autosaveCountSpinBox->setObjectName(QString::fromUtf8("autosaveCountSpinBox"));
        autosaveCountSpinBox->setMinimum(1);

        gridLayout->addWidget(autosaveCountSpinBox, 1, 1, 1, 1);


        horizontalLayout->addLayout(gridLayout);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        autosaveDocRadio = new QRadioButton(saveTab);
        autosaveDocRadio->setObjectName(QString::fromUtf8("autosaveDocRadio"));
        autosaveDocRadio->setChecked(true);

        gridLayout_2->addWidget(autosaveDocRadio, 0, 0, 1, 3);

        autosaveDirRadio = new QRadioButton(saveTab);
        autosaveDirRadio->setObjectName(QString::fromUtf8("autosaveDirRadio"));

        gridLayout_2->addWidget(autosaveDirRadio, 1, 0, 1, 1);

        autosaveDirEdit = new QLineEdit(saveTab);
        autosaveDirEdit->setObjectName(QString::fromUtf8("autosaveDirEdit"));

        gridLayout_2->addWidget(autosaveDirEdit, 1, 1, 1, 1);

        changeAutoDir = new QPushButton(saveTab);
        changeAutoDir->setObjectName(QString::fromUtf8("changeAutoDir"));
        changeAutoDir->setFlat(false);

        gridLayout_2->addWidget(changeAutoDir, 1, 2, 1, 1);


        verticalLayout->addLayout(gridLayout_2);

        autosaveKeepCheckBox = new QCheckBox(saveTab);
        autosaveKeepCheckBox->setObjectName(QString::fromUtf8("autosaveKeepCheckBox"));

        verticalLayout->addWidget(autosaveKeepCheckBox);

        showAutosaveClockOnCanvasCheckBox = new QCheckBox(saveTab);
        showAutosaveClockOnCanvasCheckBox->setObjectName(QString::fromUtf8("showAutosaveClockOnCanvasCheckBox"));

        verticalLayout->addWidget(showAutosaveClockOnCanvasCheckBox);

        verticalSpacer_7 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Maximum);

        verticalLayout->addItem(verticalSpacer_7);

        undoRedoTitleLabel = new QLabel(saveTab);
        undoRedoTitleLabel->setObjectName(QString::fromUtf8("undoRedoTitleLabel"));
        undoRedoTitleLabel->setFont(font1);

        verticalLayout->addWidget(undoRedoTitleLabel);

        line_7 = new QFrame(saveTab);
        line_7->setObjectName(QString::fromUtf8("line_7"));
        line_7->setFrameShape(QFrame::HLine);
        line_7->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line_7);

        undoCheckBox = new QCheckBox(saveTab);
        undoCheckBox->setObjectName(QString::fromUtf8("undoCheckBox"));

        verticalLayout->addWidget(undoCheckBox);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        undoLengthLabel = new QLabel(saveTab);
        undoLengthLabel->setObjectName(QString::fromUtf8("undoLengthLabel"));

        horizontalLayout_2->addWidget(undoLengthLabel);

        undoLengthSpinBox = new QSpinBox(saveTab);
        undoLengthSpinBox->setObjectName(QString::fromUtf8("undoLengthSpinBox"));

        horizontalLayout_2->addWidget(undoLengthSpinBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout_2);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);

        tabWidget->addTab(saveTab, QString());

        verticalLayout_2->addWidget(tabWidget);

#if QT_CONFIG(shortcut)
        pageSizeLabel->setBuddy(pageSizeComboBox);
        pageOrientatioLabel->setBuddy(pageOrientationComboBox);
        pageUnitsLabel->setBuddy(pageUnitsComboBox);
        pageWidthLabel->setBuddy(pageWidthSpinBox);
        pageHeightLabel->setBuddy(pageHeightSpinBox);
        layoutFirstPageIsLabel->setBuddy(layoutFirstPageIsComboBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(tabWidget, languageComboBox);
        QWidget::setTabOrder(languageComboBox, pageSizeComboBox);
        QWidget::setTabOrder(pageSizeComboBox, pageSizeLinkToolButton);
        QWidget::setTabOrder(pageSizeLinkToolButton, pageOrientationComboBox);
        QWidget::setTabOrder(pageOrientationComboBox, pageUnitsComboBox);
        QWidget::setTabOrder(pageUnitsComboBox, pageWidthSpinBox);
        QWidget::setTabOrder(pageWidthSpinBox, pageHeightSpinBox);
        QWidget::setTabOrder(pageHeightSpinBox, applySizesToAllPagesCheckBox);
        QWidget::setTabOrder(applySizesToAllPagesCheckBox, applySizesToAllMasterPagesCheckBox);
        QWidget::setTabOrder(applySizesToAllMasterPagesCheckBox, singlePageRadioButton);
        QWidget::setTabOrder(singlePageRadioButton, facingPagesRadioButton);
        QWidget::setTabOrder(facingPagesRadioButton, threeFoldRadioButton);
        QWidget::setTabOrder(threeFoldRadioButton, fourFoldRadioButton);
        QWidget::setTabOrder(fourFoldRadioButton, layoutFirstPageIsComboBox);
        QWidget::setTabOrder(layoutFirstPageIsComboBox, scrollArea);
        QWidget::setTabOrder(scrollArea, applyMarginsToAllPagesCheckBox);
        QWidget::setTabOrder(applyMarginsToAllPagesCheckBox, applyMarginsToAllMasterPagesCheckBox);
        QWidget::setTabOrder(applyMarginsToAllMasterPagesCheckBox, saveCompressedCheckBox);
        QWidget::setTabOrder(saveCompressedCheckBox, emergencyCheckBox);
        QWidget::setTabOrder(emergencyCheckBox, autosaveCheckBox);
        QWidget::setTabOrder(autosaveCheckBox, autosaveIntervalSpinBox);
        QWidget::setTabOrder(autosaveIntervalSpinBox, autosaveCountSpinBox);
        QWidget::setTabOrder(autosaveCountSpinBox, autosaveDocRadio);
        QWidget::setTabOrder(autosaveDocRadio, autosaveDirRadio);
        QWidget::setTabOrder(autosaveDirRadio, autosaveDirEdit);
        QWidget::setTabOrder(autosaveDirEdit, changeAutoDir);
        QWidget::setTabOrder(changeAutoDir, autosaveKeepCheckBox);
        QWidget::setTabOrder(autosaveKeepCheckBox, showAutosaveClockOnCanvasCheckBox);
        QWidget::setTabOrder(showAutosaveClockOnCanvasCheckBox, undoCheckBox);
        QWidget::setTabOrder(undoCheckBox, undoLengthSpinBox);

        retranslateUi(Prefs_DocumentSetup);
        QObject::connect(autosaveDocRadio, SIGNAL(clicked(bool)), autosaveDirEdit, SLOT(setDisabled(bool)));
        QObject::connect(autosaveDocRadio, SIGNAL(clicked(bool)), changeAutoDir, SLOT(setDisabled(bool)));
        QObject::connect(autosaveDirRadio, SIGNAL(clicked(bool)), autosaveDirEdit, SLOT(setEnabled(bool)));
        QObject::connect(autosaveDirRadio, SIGNAL(clicked(bool)), changeAutoDir, SLOT(setEnabled(bool)));

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Prefs_DocumentSetup);
    } // setupUi

    void retranslateUi(QWidget *Prefs_DocumentSetup)
    {
        titleLabel->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Document Setup", nullptr));
        label_3->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Language:", nullptr));
        pageSizeTitleLabel->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Page Size", nullptr));
        pageSizeLabel->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Size:", nullptr));
        pageOrientatioLabel->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Orientation:", nullptr));
        pageUnitsLabel->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Units:", nullptr));
        pageWidthLabel->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Width:", nullptr));
        pageHeightLabel->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Height:", nullptr));
        applySizesToAllPagesCheckBox->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Apply changes to all pages", nullptr));
        applySizesToAllMasterPagesCheckBox->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Apply changes to all master pages", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(sizeTab), QCoreApplication::translate("Prefs_DocumentSetup", "General", nullptr));
        pageLayoutTitleLabel->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Page Layout", nullptr));
        layoutFirstPageIsLabel->setText(QCoreApplication::translate("Prefs_DocumentSetup", "First Page is:", nullptr));
        fourFoldRadioButton->setText(QCoreApplication::translate("Prefs_DocumentSetup", "4-Fold", nullptr));
        threeFoldRadioButton->setText(QCoreApplication::translate("Prefs_DocumentSetup", "3-Fold", nullptr));
        facingPagesRadioButton->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Facing Pages", nullptr));
        singlePageRadioButton->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Single Page", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(layoutTab), QCoreApplication::translate("Prefs_DocumentSetup", "Layout", nullptr));
        marginsTitleLabel->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Margins", nullptr));
        applyMarginsToAllPagesCheckBox->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Apply changes to all pages", nullptr));
        applyMarginsToAllMasterPagesCheckBox->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Apply changes to all master pages", nullptr));
        bleedsTitleLabel->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Bleeds", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(marginsTab), QCoreApplication::translate("Prefs_DocumentSetup", "Margins && Bleeds", nullptr));
        saveCompressedCheckBox->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Save in Compressed Format", nullptr));
        emergencyCheckBox->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Try to save an \"Emergency File\" in case of a crash", nullptr));
        autosaveTitleLable->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Autosave", nullptr));
        autosaveCheckBox->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Enable automatic saving of documents", nullptr));
        label->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Interval:", nullptr));
        autosaveIntervalSpinBox->setSuffix(QCoreApplication::translate("Prefs_DocumentSetup", " min", nullptr));
        label_2->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Number of files to keep:", nullptr));
        autosaveDocRadio->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Save into the same directory as the document", nullptr));
        autosaveDirRadio->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Save into:", nullptr));
        changeAutoDir->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Change...", nullptr));
        autosaveKeepCheckBox->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Keep backup files after closing the document", nullptr));
        showAutosaveClockOnCanvasCheckBox->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Show autosave countdown clock on the canvas", nullptr));
        undoRedoTitleLabel->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Undo / Redo", nullptr));
        undoCheckBox->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Enable Undo/Redo System", nullptr));
        undoLengthLabel->setText(QCoreApplication::translate("Prefs_DocumentSetup", "Action History Length:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(saveTab), QCoreApplication::translate("Prefs_DocumentSetup", "Saving && Undo", nullptr));
        (void)Prefs_DocumentSetup;
    } // retranslateUi

};

namespace Ui {
    class Prefs_DocumentSetup: public Ui_Prefs_DocumentSetup {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_DOCUMENTSETUPBASE_H
