/*
This is base class for Scribus Barcode Generator. Always use designer for this file!
*/

/********************************************************************************
** Form generated from reading UI file 'barcodegenerator.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_BARCODEGENERATOR_H
#define UI_BARCODEGENERATOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_BarcodeGeneratorBase
{
public:
    QGridLayout *gridLayout;
    QGroupBox *colorBox;
    QGridLayout *gridLayout_4;
    QLabel *txtLabel;
    QLabel *linesLabel;
    QPushButton *txtColorButton;
    QLabel *bgLabel;
    QPushButton *bgColorButton;
    QPushButton *lnColorButton;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QPushButton *okButton;
    QPushButton *cancelButton;
    QGroupBox *bcodeBox;
    QGridLayout *gridLayout_3;
    QCheckBox *includecheckCheck;
    QLabel *formatLabel;
    QCheckBox *includecheckintextCheck;
    QCheckBox *guardwhitespaceCheck;
    QCheckBox *parseCheck;
    QLabel *eccLabel;
    QCheckBox *parsefncCheck;
    QCheckBox *includetextCheck;
    QComboBox *formatCombo;
    QComboBox *eccCombo;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_2;
    QLabel *bcFamilyLabel;
    QComboBox *bcFamilyCombo;
    QLabel *bcLabel;
    QComboBox *bcCombo;
    QLabel *codeLabel;
    QLineEdit *codeEdit;
    QPushButton *resetButton;
    QLabel *optionsLabel;
    QLineEdit *optionsEdit;
    QLabel *sampleLabel;

    void setupUi(QDialog *BarcodeGeneratorBase)
    {
        if (BarcodeGeneratorBase->objectName().isEmpty())
            BarcodeGeneratorBase->setObjectName(QString::fromUtf8("BarcodeGeneratorBase"));
        BarcodeGeneratorBase->resize(504, 541);
        gridLayout = new QGridLayout(BarcodeGeneratorBase);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        colorBox = new QGroupBox(BarcodeGeneratorBase);
        colorBox->setObjectName(QString::fromUtf8("colorBox"));
        gridLayout_4 = new QGridLayout(colorBox);
        gridLayout_4->setSpacing(6);
        gridLayout_4->setContentsMargins(11, 11, 11, 11);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        txtLabel = new QLabel(colorBox);
        txtLabel->setObjectName(QString::fromUtf8("txtLabel"));
        txtLabel->setMinimumSize(QSize(25, 25));
        txtLabel->setFrameShape(QFrame::Box);
        txtLabel->setScaledContents(true);
        txtLabel->setWordWrap(false);

        gridLayout_4->addWidget(txtLabel, 2, 1, 1, 1);

        linesLabel = new QLabel(colorBox);
        linesLabel->setObjectName(QString::fromUtf8("linesLabel"));
        linesLabel->setMinimumSize(QSize(25, 25));
        linesLabel->setFrameShape(QFrame::Box);
        linesLabel->setScaledContents(true);
        linesLabel->setWordWrap(false);

        gridLayout_4->addWidget(linesLabel, 1, 1, 1, 1);

        txtColorButton = new QPushButton(colorBox);
        txtColorButton->setObjectName(QString::fromUtf8("txtColorButton"));

        gridLayout_4->addWidget(txtColorButton, 2, 0, 1, 1);

        bgLabel = new QLabel(colorBox);
        bgLabel->setObjectName(QString::fromUtf8("bgLabel"));
        bgLabel->setMinimumSize(QSize(25, 25));
        bgLabel->setFrameShape(QFrame::Box);
        bgLabel->setScaledContents(true);
        bgLabel->setWordWrap(false);

        gridLayout_4->addWidget(bgLabel, 0, 1, 1, 1);

        bgColorButton = new QPushButton(colorBox);
        bgColorButton->setObjectName(QString::fromUtf8("bgColorButton"));

        gridLayout_4->addWidget(bgColorButton, 0, 0, 1, 1);

        lnColorButton = new QPushButton(colorBox);
        lnColorButton->setObjectName(QString::fromUtf8("lnColorButton"));

        gridLayout_4->addWidget(lnColorButton, 1, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_4->addItem(verticalSpacer, 3, 0, 1, 1);


        gridLayout->addWidget(colorBox, 4, 1, 1, 2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        okButton = new QPushButton(BarcodeGeneratorBase);
        okButton->setObjectName(QString::fromUtf8("okButton"));

        horizontalLayout->addWidget(okButton);

        cancelButton = new QPushButton(BarcodeGeneratorBase);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        horizontalLayout->addWidget(cancelButton);


        gridLayout->addLayout(horizontalLayout, 7, 0, 1, 3);

        bcodeBox = new QGroupBox(BarcodeGeneratorBase);
        bcodeBox->setObjectName(QString::fromUtf8("bcodeBox"));
        gridLayout_3 = new QGridLayout(bcodeBox);
        gridLayout_3->setSpacing(6);
        gridLayout_3->setContentsMargins(11, 11, 11, 11);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        includecheckCheck = new QCheckBox(bcodeBox);
        includecheckCheck->setObjectName(QString::fromUtf8("includecheckCheck"));

        gridLayout_3->addWidget(includecheckCheck, 1, 0, 1, 1);

        formatLabel = new QLabel(bcodeBox);
        formatLabel->setObjectName(QString::fromUtf8("formatLabel"));
        formatLabel->setWordWrap(false);

        gridLayout_3->addWidget(formatLabel, 3, 0, 1, 1);

        includecheckintextCheck = new QCheckBox(bcodeBox);
        includecheckintextCheck->setObjectName(QString::fromUtf8("includecheckintextCheck"));

        gridLayout_3->addWidget(includecheckintextCheck, 1, 1, 1, 1);

        guardwhitespaceCheck = new QCheckBox(bcodeBox);
        guardwhitespaceCheck->setObjectName(QString::fromUtf8("guardwhitespaceCheck"));

        gridLayout_3->addWidget(guardwhitespaceCheck, 0, 1, 1, 1);

        parseCheck = new QCheckBox(bcodeBox);
        parseCheck->setObjectName(QString::fromUtf8("parseCheck"));

        gridLayout_3->addWidget(parseCheck, 2, 0, 1, 1);

        eccLabel = new QLabel(bcodeBox);
        eccLabel->setObjectName(QString::fromUtf8("eccLabel"));
        eccLabel->setWordWrap(false);

        gridLayout_3->addWidget(eccLabel, 4, 0, 1, 1);

        parsefncCheck = new QCheckBox(bcodeBox);
        parsefncCheck->setObjectName(QString::fromUtf8("parsefncCheck"));

        gridLayout_3->addWidget(parsefncCheck, 2, 1, 1, 1);

        includetextCheck = new QCheckBox(bcodeBox);
        includetextCheck->setObjectName(QString::fromUtf8("includetextCheck"));
        includetextCheck->setChecked(false);

        gridLayout_3->addWidget(includetextCheck, 0, 0, 1, 1);

        formatCombo = new QComboBox(bcodeBox);
        formatCombo->setObjectName(QString::fromUtf8("formatCombo"));
        formatCombo->setMinimumSize(QSize(125, 0));

        gridLayout_3->addWidget(formatCombo, 3, 1, 1, 1);

        eccCombo = new QComboBox(bcodeBox);
        eccCombo->setObjectName(QString::fromUtf8("eccCombo"));
        eccCombo->setMinimumSize(QSize(125, 0));

        gridLayout_3->addWidget(eccCombo, 4, 1, 1, 1);


        gridLayout->addWidget(bcodeBox, 4, 0, 1, 1);

        groupBox = new QGroupBox(BarcodeGeneratorBase);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        groupBox->setMinimumSize(QSize(0, 142));
        gridLayout_2 = new QGridLayout(groupBox);
        gridLayout_2->setSpacing(6);
        gridLayout_2->setContentsMargins(11, 11, 11, 11);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        bcFamilyLabel = new QLabel(groupBox);
        bcFamilyLabel->setObjectName(QString::fromUtf8("bcFamilyLabel"));

        gridLayout_2->addWidget(bcFamilyLabel, 0, 0, 1, 1);

        bcFamilyCombo = new QComboBox(groupBox);
        bcFamilyCombo->setObjectName(QString::fromUtf8("bcFamilyCombo"));

        gridLayout_2->addWidget(bcFamilyCombo, 0, 1, 1, 1);

        bcLabel = new QLabel(groupBox);
        bcLabel->setObjectName(QString::fromUtf8("bcLabel"));
        bcLabel->setWordWrap(false);

        gridLayout_2->addWidget(bcLabel, 1, 0, 1, 1);

        bcCombo = new QComboBox(groupBox);
        bcCombo->setObjectName(QString::fromUtf8("bcCombo"));

        gridLayout_2->addWidget(bcCombo, 1, 1, 1, 1);

        codeLabel = new QLabel(groupBox);
        codeLabel->setObjectName(QString::fromUtf8("codeLabel"));
        codeLabel->setWordWrap(false);

        gridLayout_2->addWidget(codeLabel, 2, 0, 1, 1);

        codeEdit = new QLineEdit(groupBox);
        codeEdit->setObjectName(QString::fromUtf8("codeEdit"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(codeEdit->sizePolicy().hasHeightForWidth());
        codeEdit->setSizePolicy(sizePolicy);
        codeEdit->setMinimumSize(QSize(0, 21));

        gridLayout_2->addWidget(codeEdit, 2, 1, 1, 1);

        resetButton = new QPushButton(groupBox);
        resetButton->setObjectName(QString::fromUtf8("resetButton"));
        QSizePolicy sizePolicy1(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(resetButton->sizePolicy().hasHeightForWidth());
        resetButton->setSizePolicy(sizePolicy1);
        resetButton->setMaximumSize(QSize(24, 24));

        gridLayout_2->addWidget(resetButton, 2, 2, 1, 1);

        optionsLabel = new QLabel(groupBox);
        optionsLabel->setObjectName(QString::fromUtf8("optionsLabel"));
        optionsLabel->setTextInteractionFlags(Qt::LinksAccessibleByMouse);

        gridLayout_2->addWidget(optionsLabel, 3, 0, 1, 1);

        optionsEdit = new QLineEdit(groupBox);
        optionsEdit->setObjectName(QString::fromUtf8("optionsEdit"));
        optionsEdit->setMinimumSize(QSize(0, 21));

        gridLayout_2->addWidget(optionsEdit, 3, 1, 1, 1);


        gridLayout->addWidget(groupBox, 2, 0, 2, 3);

        sampleLabel = new QLabel(BarcodeGeneratorBase);
        sampleLabel->setObjectName(QString::fromUtf8("sampleLabel"));
        sampleLabel->setMinimumSize(QSize(450, 190));
        sampleLabel->setStyleSheet(QString::fromUtf8("background-color : #FFF"));
        sampleLabel->setFrameShape(QFrame::Box);
        sampleLabel->setAlignment(Qt::AlignCenter);
        sampleLabel->setWordWrap(true);

        gridLayout->addWidget(sampleLabel, 5, 0, 1, 3);

#if QT_CONFIG(shortcut)
        formatLabel->setBuddy(formatCombo);
        eccLabel->setBuddy(eccCombo);
        bcLabel->setBuddy(bcCombo);
        codeLabel->setBuddy(codeEdit);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(bgColorButton, lnColorButton);
        QWidget::setTabOrder(lnColorButton, txtColorButton);
        QWidget::setTabOrder(txtColorButton, okButton);
        QWidget::setTabOrder(okButton, cancelButton);

        retranslateUi(BarcodeGeneratorBase);

        QMetaObject::connectSlotsByName(BarcodeGeneratorBase);
    } // setupUi

    void retranslateUi(QDialog *BarcodeGeneratorBase)
    {
        BarcodeGeneratorBase->setWindowTitle(QCoreApplication::translate("BarcodeGeneratorBase", "Insert Barcode", nullptr));
#if QT_CONFIG(tooltip)
        BarcodeGeneratorBase->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        colorBox->setTitle(QCoreApplication::translate("BarcodeGeneratorBase", "Colors", nullptr));
#if QT_CONFIG(tooltip)
        txtColorButton->setToolTip(QCoreApplication::translate("BarcodeGeneratorBase", "Color of the text and numbers", nullptr));
#endif // QT_CONFIG(tooltip)
        txtColorButton->setText(QCoreApplication::translate("BarcodeGeneratorBase", "&Text", nullptr));
#if QT_CONFIG(shortcut)
        txtColorButton->setShortcut(QCoreApplication::translate("BarcodeGeneratorBase", "Alt+T", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        bgColorButton->setToolTip(QCoreApplication::translate("BarcodeGeneratorBase", "Background color - under the code lines", nullptr));
#endif // QT_CONFIG(tooltip)
        bgColorButton->setText(QCoreApplication::translate("BarcodeGeneratorBase", "&Background", nullptr));
#if QT_CONFIG(shortcut)
        bgColorButton->setShortcut(QCoreApplication::translate("BarcodeGeneratorBase", "Alt+B", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        lnColorButton->setToolTip(QCoreApplication::translate("BarcodeGeneratorBase", "Color of the lines in barcode", nullptr));
#endif // QT_CONFIG(tooltip)
        lnColorButton->setText(QCoreApplication::translate("BarcodeGeneratorBase", "&Lines", nullptr));
#if QT_CONFIG(shortcut)
        lnColorButton->setShortcut(QCoreApplication::translate("BarcodeGeneratorBase", "Alt+L", nullptr));
#endif // QT_CONFIG(shortcut)
        okButton->setText(QString());
        cancelButton->setText(QString());
        bcodeBox->setTitle(QCoreApplication::translate("BarcodeGeneratorBase", "Options", nullptr));
#if QT_CONFIG(tooltip)
        includecheckCheck->setToolTip(QCoreApplication::translate("BarcodeGeneratorBase", "Generate and include a checksum in barcode", nullptr));
#endif // QT_CONFIG(tooltip)
        includecheckCheck->setText(QCoreApplication::translate("BarcodeGeneratorBase", "Add checksum", nullptr));
#if QT_CONFIG(shortcut)
        includecheckCheck->setShortcut(QCoreApplication::translate("BarcodeGeneratorBase", "Alt+N", nullptr));
#endif // QT_CONFIG(shortcut)
        formatLabel->setText(QCoreApplication::translate("BarcodeGeneratorBase", "Version:", nullptr));
#if QT_CONFIG(tooltip)
        includecheckintextCheck->setToolTip(QCoreApplication::translate("BarcodeGeneratorBase", "Include the checksum digit in the barcode text", nullptr));
#endif // QT_CONFIG(tooltip)
        includecheckintextCheck->setText(QCoreApplication::translate("BarcodeGeneratorBase", "Display checksum", nullptr));
#if QT_CONFIG(shortcut)
        includecheckintextCheck->setShortcut(QCoreApplication::translate("BarcodeGeneratorBase", "Alt+U", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        guardwhitespaceCheck->setToolTip(QCoreApplication::translate("BarcodeGeneratorBase", "Draw arrows to be sure of space next the code", nullptr));
#endif // QT_CONFIG(tooltip)
        guardwhitespaceCheck->setText(QCoreApplication::translate("BarcodeGeneratorBase", "Guard whitespace", nullptr));
#if QT_CONFIG(shortcut)
        guardwhitespaceCheck->setShortcut(QCoreApplication::translate("BarcodeGeneratorBase", "Alt+G", nullptr));
#endif // QT_CONFIG(shortcut)
        parseCheck->setText(QCoreApplication::translate("BarcodeGeneratorBase", "^NNN as ASCII", nullptr));
        eccLabel->setText(QCoreApplication::translate("BarcodeGeneratorBase", "Error correction level:", nullptr));
        parsefncCheck->setText(QCoreApplication::translate("BarcodeGeneratorBase", "Parse specials", nullptr));
#if QT_CONFIG(tooltip)
        includetextCheck->setToolTip(QCoreApplication::translate("BarcodeGeneratorBase", "If checked, there will be numbers in the barcode too", nullptr));
#endif // QT_CONFIG(tooltip)
        includetextCheck->setText(QCoreApplication::translate("BarcodeGeneratorBase", "Show readable text", nullptr));
#if QT_CONFIG(shortcut)
        includetextCheck->setShortcut(QCoreApplication::translate("BarcodeGeneratorBase", "Alt+I", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        formatCombo->setToolTip(QCoreApplication::translate("BarcodeGeneratorBase", "Version of the symbol", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        eccCombo->setToolTip(QCoreApplication::translate("BarcodeGeneratorBase", "Error correction level", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox->setTitle(QCoreApplication::translate("BarcodeGeneratorBase", "Type", nullptr));
        bcFamilyLabel->setText(QCoreApplication::translate("BarcodeGeneratorBase", "Barcode Family:", nullptr));
        bcLabel->setText(QCoreApplication::translate("BarcodeGeneratorBase", "&Barcode:", nullptr));
#if QT_CONFIG(tooltip)
        bcCombo->setToolTip(QCoreApplication::translate("BarcodeGeneratorBase", "Select one of the available barcode types here", nullptr));
#endif // QT_CONFIG(tooltip)
        codeLabel->setText(QCoreApplication::translate("BarcodeGeneratorBase", "&Contents:", nullptr));
#if QT_CONFIG(tooltip)
        codeEdit->setToolTip(QCoreApplication::translate("BarcodeGeneratorBase", "The contents encoded in the barcode", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        resetButton->setToolTip(QCoreApplication::translate("BarcodeGeneratorBase", "Reset the barcode samples", nullptr));
#endif // QT_CONFIG(tooltip)
        resetButton->setText(QString());
        optionsLabel->setText(QCoreApplication::translate("BarcodeGeneratorBase", "Options:", nullptr));
#if QT_CONFIG(tooltip)
        optionsEdit->setToolTip(QCoreApplication::translate("BarcodeGeneratorBase", "Advanced options for the barcode", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class BarcodeGeneratorBase: public Ui_BarcodeGeneratorBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_BARCODEGENERATOR_H
