/****************************************************************************
** Meta object code from reading C++ file 'barcodegenerator.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../scribus-1.6.6/scribus/plugins/barcodegenerator/barcodegenerator.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'barcodegenerator.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_BarcodeGenerator_t {
    QByteArrayData data[25];
    char stringdata0[535];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_BarcodeGenerator_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_BarcodeGenerator_t qt_meta_stringdata_BarcodeGenerator = {
    {
QT_MOC_LITERAL(0, 0, 16), // "BarcodeGenerator"
QT_MOC_LITERAL(1, 17, 12), // "paintBarcode"
QT_MOC_LITERAL(2, 30, 0), // ""
QT_MOC_LITERAL(3, 31, 13), // "updatePreview"
QT_MOC_LITERAL(4, 45, 20), // "bcFamilyComboChanged"
QT_MOC_LITERAL(5, 66, 14), // "bcComboChanged"
QT_MOC_LITERAL(6, 81, 21), // "bgColorButton_pressed"
QT_MOC_LITERAL(7, 103, 21), // "lnColorButton_pressed"
QT_MOC_LITERAL(8, 125, 22), // "txtColorButton_pressed"
QT_MOC_LITERAL(9, 148, 20), // "codeEdit_textChanged"
QT_MOC_LITERAL(10, 169, 1), // "s"
QT_MOC_LITERAL(11, 171, 19), // "resetButton_clicked"
QT_MOC_LITERAL(12, 191, 16), // "okButton_pressed"
QT_MOC_LITERAL(13, 208, 20), // "cancelButton_pressed"
QT_MOC_LITERAL(14, 229, 32), // "on_includetextCheck_stateChanged"
QT_MOC_LITERAL(15, 262, 4), // "arg1"
QT_MOC_LITERAL(16, 267, 33), // "on_includecheckCheck_stateCha..."
QT_MOC_LITERAL(17, 301, 39), // "on_includecheckintextCheck_st..."
QT_MOC_LITERAL(18, 341, 26), // "on_parseCheck_stateChanged"
QT_MOC_LITERAL(19, 368, 29), // "on_parsefncCheck_stateChanged"
QT_MOC_LITERAL(20, 398, 34), // "on_formatCombo_currentIndexCh..."
QT_MOC_LITERAL(21, 433, 5), // "index"
QT_MOC_LITERAL(22, 439, 31), // "on_eccCombo_currentIndexChanged"
QT_MOC_LITERAL(23, 471, 36), // "on_guardwhitespaceCheck_state..."
QT_MOC_LITERAL(24, 508, 26) // "on_optionsEdit_textChanged"

    },
    "BarcodeGenerator\0paintBarcode\0\0"
    "updatePreview\0bcFamilyComboChanged\0"
    "bcComboChanged\0bgColorButton_pressed\0"
    "lnColorButton_pressed\0txtColorButton_pressed\0"
    "codeEdit_textChanged\0s\0resetButton_clicked\0"
    "okButton_pressed\0cancelButton_pressed\0"
    "on_includetextCheck_stateChanged\0arg1\0"
    "on_includecheckCheck_stateChanged\0"
    "on_includecheckintextCheck_stateChanged\0"
    "on_parseCheck_stateChanged\0"
    "on_parsefncCheck_stateChanged\0"
    "on_formatCombo_currentIndexChanged\0"
    "index\0on_eccCombo_currentIndexChanged\0"
    "on_guardwhitespaceCheck_stateChanged\0"
    "on_optionsEdit_textChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_BarcodeGenerator[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  119,    2, 0x09 /* Protected */,
       3,    1,  120,    2, 0x09 /* Protected */,
       4,    0,  123,    2, 0x09 /* Protected */,
       5,    0,  124,    2, 0x09 /* Protected */,
       5,    1,  125,    2, 0x09 /* Protected */,
       6,    0,  128,    2, 0x09 /* Protected */,
       7,    0,  129,    2, 0x09 /* Protected */,
       8,    0,  130,    2, 0x09 /* Protected */,
       9,    1,  131,    2, 0x09 /* Protected */,
      11,    0,  134,    2, 0x09 /* Protected */,
      12,    0,  135,    2, 0x09 /* Protected */,
      13,    0,  136,    2, 0x09 /* Protected */,
      14,    1,  137,    2, 0x08 /* Private */,
      16,    1,  140,    2, 0x08 /* Private */,
      17,    1,  143,    2, 0x08 /* Private */,
      18,    1,  146,    2, 0x08 /* Private */,
      19,    1,  149,    2, 0x08 /* Private */,
      20,    1,  152,    2, 0x08 /* Private */,
      22,    1,  155,    2, 0x08 /* Private */,
      23,    1,  158,    2, 0x08 /* Private */,
      24,    1,  161,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Void, QMetaType::Int,   15,
    QMetaType::Void, QMetaType::QString,   15,

       0        // eod
};

void BarcodeGenerator::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BarcodeGenerator *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->paintBarcode(); break;
        case 1: _t->updatePreview((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->bcFamilyComboChanged(); break;
        case 3: _t->bcComboChanged(); break;
        case 4: _t->bcComboChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->bgColorButton_pressed(); break;
        case 6: _t->lnColorButton_pressed(); break;
        case 7: _t->txtColorButton_pressed(); break;
        case 8: _t->codeEdit_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->resetButton_clicked(); break;
        case 10: _t->okButton_pressed(); break;
        case 11: _t->cancelButton_pressed(); break;
        case 12: _t->on_includetextCheck_stateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->on_includecheckCheck_stateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->on_includecheckintextCheck_stateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->on_parseCheck_stateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->on_parsefncCheck_stateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->on_formatCombo_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 18: _t->on_eccCombo_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->on_guardwhitespaceCheck_stateChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->on_optionsEdit_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject BarcodeGenerator::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_BarcodeGenerator.data,
    qt_meta_data_BarcodeGenerator,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *BarcodeGenerator::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BarcodeGenerator::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_BarcodeGenerator.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int BarcodeGenerator::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 21;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
