/****************************************************************************
** Meta object code from reading C++ file 'search.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/search.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'search.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SearchReplace_t {
    QByteArrayData data[29];
    char stringdata0[480];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SearchReplace_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SearchReplace_t qt_meta_stringdata_SearchReplace = {
    {
QT_MOC_LITERAL(0, 0, 13), // "SearchReplace"
QT_MOC_LITERAL(1, 14, 10), // "slotSearch"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 11), // "slotReplace"
QT_MOC_LITERAL(4, 38, 14), // "slotReplaceAll"
QT_MOC_LITERAL(5, 53, 14), // "enableTxSearch"
QT_MOC_LITERAL(6, 68, 17), // "enableStyleSearch"
QT_MOC_LITERAL(7, 86, 17), // "enableAlignSearch"
QT_MOC_LITERAL(8, 104, 16), // "enableFontSearch"
QT_MOC_LITERAL(9, 121, 16), // "enableSizeSearch"
QT_MOC_LITERAL(10, 138, 15), // "enableEffSearch"
QT_MOC_LITERAL(11, 154, 16), // "enableFillSearch"
QT_MOC_LITERAL(12, 171, 17), // "enableFillSSearch"
QT_MOC_LITERAL(13, 189, 18), // "enableStrokeSearch"
QT_MOC_LITERAL(14, 208, 19), // "enableStrokeSSearch"
QT_MOC_LITERAL(15, 228, 15), // "enableTxReplace"
QT_MOC_LITERAL(16, 244, 18), // "enableStyleReplace"
QT_MOC_LITERAL(17, 263, 18), // "enableAlignReplace"
QT_MOC_LITERAL(18, 282, 17), // "enableFontReplace"
QT_MOC_LITERAL(19, 300, 17), // "enableSizeReplace"
QT_MOC_LITERAL(20, 318, 16), // "enableEffReplace"
QT_MOC_LITERAL(21, 335, 17), // "enableFillReplace"
QT_MOC_LITERAL(22, 353, 18), // "enableFillSReplace"
QT_MOC_LITERAL(23, 372, 19), // "enableStrokeReplace"
QT_MOC_LITERAL(24, 392, 20), // "enableStrokeSReplace"
QT_MOC_LITERAL(25, 413, 25), // "updateReplaceButtonsState"
QT_MOC_LITERAL(26, 439, 23), // "updateSearchButtonState"
QT_MOC_LITERAL(27, 463, 10), // "writePrefs"
QT_MOC_LITERAL(28, 474, 5) // "clear"

    },
    "SearchReplace\0slotSearch\0\0slotReplace\0"
    "slotReplaceAll\0enableTxSearch\0"
    "enableStyleSearch\0enableAlignSearch\0"
    "enableFontSearch\0enableSizeSearch\0"
    "enableEffSearch\0enableFillSearch\0"
    "enableFillSSearch\0enableStrokeSearch\0"
    "enableStrokeSSearch\0enableTxReplace\0"
    "enableStyleReplace\0enableAlignReplace\0"
    "enableFontReplace\0enableSizeReplace\0"
    "enableEffReplace\0enableFillReplace\0"
    "enableFillSReplace\0enableStrokeReplace\0"
    "enableStrokeSReplace\0updateReplaceButtonsState\0"
    "updateSearchButtonState\0writePrefs\0"
    "clear"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SearchReplace[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      27,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  149,    2, 0x0a /* Public */,
       3,    0,  150,    2, 0x0a /* Public */,
       4,    0,  151,    2, 0x0a /* Public */,
       5,    0,  152,    2, 0x0a /* Public */,
       6,    0,  153,    2, 0x0a /* Public */,
       7,    0,  154,    2, 0x0a /* Public */,
       8,    0,  155,    2, 0x0a /* Public */,
       9,    0,  156,    2, 0x0a /* Public */,
      10,    0,  157,    2, 0x0a /* Public */,
      11,    0,  158,    2, 0x0a /* Public */,
      12,    0,  159,    2, 0x0a /* Public */,
      13,    0,  160,    2, 0x0a /* Public */,
      14,    0,  161,    2, 0x0a /* Public */,
      15,    0,  162,    2, 0x0a /* Public */,
      16,    0,  163,    2, 0x0a /* Public */,
      17,    0,  164,    2, 0x0a /* Public */,
      18,    0,  165,    2, 0x0a /* Public */,
      19,    0,  166,    2, 0x0a /* Public */,
      20,    0,  167,    2, 0x0a /* Public */,
      21,    0,  168,    2, 0x0a /* Public */,
      22,    0,  169,    2, 0x0a /* Public */,
      23,    0,  170,    2, 0x0a /* Public */,
      24,    0,  171,    2, 0x0a /* Public */,
      25,    0,  172,    2, 0x0a /* Public */,
      26,    0,  173,    2, 0x0a /* Public */,
      27,    0,  174,    2, 0x0a /* Public */,
      28,    0,  175,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SearchReplace::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SearchReplace *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->slotSearch(); break;
        case 1: _t->slotReplace(); break;
        case 2: _t->slotReplaceAll(); break;
        case 3: _t->enableTxSearch(); break;
        case 4: _t->enableStyleSearch(); break;
        case 5: _t->enableAlignSearch(); break;
        case 6: _t->enableFontSearch(); break;
        case 7: _t->enableSizeSearch(); break;
        case 8: _t->enableEffSearch(); break;
        case 9: _t->enableFillSearch(); break;
        case 10: _t->enableFillSSearch(); break;
        case 11: _t->enableStrokeSearch(); break;
        case 12: _t->enableStrokeSSearch(); break;
        case 13: _t->enableTxReplace(); break;
        case 14: _t->enableStyleReplace(); break;
        case 15: _t->enableAlignReplace(); break;
        case 16: _t->enableFontReplace(); break;
        case 17: _t->enableSizeReplace(); break;
        case 18: _t->enableEffReplace(); break;
        case 19: _t->enableFillReplace(); break;
        case 20: _t->enableFillSReplace(); break;
        case 21: _t->enableStrokeReplace(); break;
        case 22: _t->enableStrokeSReplace(); break;
        case 23: _t->updateReplaceButtonsState(); break;
        case 24: _t->updateSearchButtonState(); break;
        case 25: _t->writePrefs(); break;
        case 26: _t->clear(); break;
        default: ;
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SearchReplace::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_SearchReplace.data,
    qt_meta_data_SearchReplace,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SearchReplace::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SearchReplace::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SearchReplace.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int SearchReplace::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 27;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
