/****************************************************************************
** Meta object code from reading C++ file 'pconsole.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../scribus-1.6.6/scribus/plugins/scriptplugin/pconsole.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pconsole.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PythonConsole_t {
    QByteArrayData data[15];
    char stringdata0[207];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PythonConsole_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PythonConsole_t qt_meta_stringdata_PythonConsole = {
    {
QT_MOC_LITERAL(0, 0, 13), // "PythonConsole"
QT_MOC_LITERAL(1, 14, 12), // "paletteShown"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 10), // "runCommand"
QT_MOC_LITERAL(4, 39, 14), // "slot_runScript"
QT_MOC_LITERAL(5, 54, 23), // "slot_runScriptAsConsole"
QT_MOC_LITERAL(6, 78, 9), // "slot_open"
QT_MOC_LITERAL(7, 88, 9), // "slot_save"
QT_MOC_LITERAL(8, 98, 11), // "slot_saveAs"
QT_MOC_LITERAL(9, 110, 15), // "slot_saveOutput"
QT_MOC_LITERAL(10, 126, 9), // "slot_quit"
QT_MOC_LITERAL(11, 136, 33), // "commandEdit_cursorPositionCha..."
QT_MOC_LITERAL(12, 170, 14), // "languageChange"
QT_MOC_LITERAL(13, 185, 15), // "documentChanged"
QT_MOC_LITERAL(14, 201, 5) // "state"

    },
    "PythonConsole\0paletteShown\0\0runCommand\0"
    "slot_runScript\0slot_runScriptAsConsole\0"
    "slot_open\0slot_save\0slot_saveAs\0"
    "slot_saveOutput\0slot_quit\0"
    "commandEdit_cursorPositionChanged\0"
    "languageChange\0documentChanged\0state"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PythonConsole[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       3,    0,   77,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   78,    2, 0x0a /* Public */,
       5,    0,   79,    2, 0x0a /* Public */,
       6,    0,   80,    2, 0x0a /* Public */,
       7,    0,   81,    2, 0x0a /* Public */,
       8,    0,   82,    2, 0x0a /* Public */,
       9,    0,   83,    2, 0x0a /* Public */,
      10,    0,   84,    2, 0x0a /* Public */,
      11,    0,   85,    2, 0x0a /* Public */,
      12,    0,   86,    2, 0x09 /* Protected */,
      13,    1,   87,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   14,

       0        // eod
};

void PythonConsole::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PythonConsole *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->paletteShown((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->runCommand(); break;
        case 2: _t->slot_runScript(); break;
        case 3: _t->slot_runScriptAsConsole(); break;
        case 4: _t->slot_open(); break;
        case 5: _t->slot_save(); break;
        case 6: _t->slot_saveAs(); break;
        case 7: _t->slot_saveOutput(); break;
        case 8: _t->slot_quit(); break;
        case 9: _t->commandEdit_cursorPositionChanged(); break;
        case 10: _t->languageChange(); break;
        case 11: _t->documentChanged((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PythonConsole::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PythonConsole::paletteShown)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PythonConsole::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PythonConsole::runCommand)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PythonConsole::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_PythonConsole.data,
    qt_meta_data_PythonConsole,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PythonConsole::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PythonConsole::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PythonConsole.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PythonConsole"))
        return static_cast< Ui::PythonConsole*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int PythonConsole::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void PythonConsole::paletteShown(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PythonConsole::runCommand()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
struct qt_meta_stringdata_SyntaxHighlighter_t {
    QByteArrayData data[1];
    char stringdata0[18];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SyntaxHighlighter_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SyntaxHighlighter_t qt_meta_stringdata_SyntaxHighlighter = {
    {
QT_MOC_LITERAL(0, 0, 17) // "SyntaxHighlighter"

    },
    "SyntaxHighlighter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SyntaxHighlighter[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void SyntaxHighlighter::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SyntaxHighlighter::staticMetaObject = { {
    QMetaObject::SuperData::link<QSyntaxHighlighter::staticMetaObject>(),
    qt_meta_stringdata_SyntaxHighlighter.data,
    qt_meta_data_SyntaxHighlighter,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SyntaxHighlighter::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SyntaxHighlighter::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SyntaxHighlighter.stringdata0))
        return static_cast<void*>(this);
    return QSyntaxHighlighter::qt_metacast(_clname);
}

int SyntaxHighlighter::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QSyntaxHighlighter::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
