/****************************************************************************
** Meta object code from reading C++ file 'undogui.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/undogui.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'undogui.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_UndoGui_t {
    QByteArrayData data[14];
    char stringdata0[139];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_UndoGui_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_UndoGui_t qt_meta_stringdata_UndoGui = {
    {
QT_MOC_LITERAL(0, 0, 7), // "UndoGui"
QT_MOC_LITERAL(1, 8, 14), // "insertUndoItem"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 11), // "UndoObject*"
QT_MOC_LITERAL(4, 36, 6), // "target"
QT_MOC_LITERAL(5, 43, 10), // "UndoState*"
QT_MOC_LITERAL(6, 54, 5), // "state"
QT_MOC_LITERAL(7, 60, 14), // "insertRedoItem"
QT_MOC_LITERAL(8, 75, 10), // "updateUndo"
QT_MOC_LITERAL(9, 86, 5), // "steps"
QT_MOC_LITERAL(10, 92, 10), // "updateRedo"
QT_MOC_LITERAL(11, 103, 17), // "updateUndoActions"
QT_MOC_LITERAL(12, 121, 9), // "clearRedo"
QT_MOC_LITERAL(13, 131, 7) // "popBack"

    },
    "UndoGui\0insertUndoItem\0\0UndoObject*\0"
    "target\0UndoState*\0state\0insertRedoItem\0"
    "updateUndo\0steps\0updateRedo\0"
    "updateUndoActions\0clearRedo\0popBack"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_UndoGui[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       7,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    2,   49,    2, 0x0a /* Public */,
       7,    2,   54,    2, 0x0a /* Public */,
       8,    1,   59,    2, 0x0a /* Public */,
      10,    1,   62,    2, 0x0a /* Public */,
      11,    0,   65,    2, 0x0a /* Public */,
      12,    0,   66,    2, 0x0a /* Public */,
      13,    0,   67,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    4,    6,
    QMetaType::Void, 0x80000000 | 3, 0x80000000 | 5,    4,    6,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void, QMetaType::Int,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void UndoGui::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<UndoGui *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->insertUndoItem((*reinterpret_cast< UndoObject*(*)>(_a[1])),(*reinterpret_cast< UndoState*(*)>(_a[2]))); break;
        case 1: _t->insertRedoItem((*reinterpret_cast< UndoObject*(*)>(_a[1])),(*reinterpret_cast< UndoState*(*)>(_a[2]))); break;
        case 2: _t->updateUndo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->updateRedo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->updateUndoActions(); break;
        case 5: _t->clearRedo(); break;
        case 6: _t->popBack(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject UndoGui::staticMetaObject = { {
    QMetaObject::SuperData::link<ScDockPalette::staticMetaObject>(),
    qt_meta_stringdata_UndoGui.data,
    qt_meta_data_UndoGui,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *UndoGui::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UndoGui::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_UndoGui.stringdata0))
        return static_cast<void*>(this);
    return ScDockPalette::qt_metacast(_clname);
}

int UndoGui::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScDockPalette::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 7)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 7;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 7)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 7;
    }
    return _id;
}
struct qt_meta_stringdata_UndoWidget_t {
    QByteArrayData data[21];
    char stringdata0[202];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_UndoWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_UndoWidget_t qt_meta_stringdata_UndoWidget = {
    {
QT_MOC_LITERAL(0, 0, 10), // "UndoWidget"
QT_MOC_LITERAL(1, 11, 4), // "undo"
QT_MOC_LITERAL(2, 16, 0), // ""
QT_MOC_LITERAL(3, 17, 5), // "steps"
QT_MOC_LITERAL(4, 23, 4), // "redo"
QT_MOC_LITERAL(5, 28, 11), // "undoClicked"
QT_MOC_LITERAL(6, 40, 11), // "redoClicked"
QT_MOC_LITERAL(7, 52, 15), // "undoMenuClicked"
QT_MOC_LITERAL(8, 68, 8), // "QAction*"
QT_MOC_LITERAL(9, 77, 2), // "id"
QT_MOC_LITERAL(10, 80, 15), // "redoMenuClicked"
QT_MOC_LITERAL(11, 96, 14), // "insertUndoItem"
QT_MOC_LITERAL(12, 111, 11), // "UndoObject*"
QT_MOC_LITERAL(13, 123, 6), // "target"
QT_MOC_LITERAL(14, 130, 10), // "UndoState*"
QT_MOC_LITERAL(15, 141, 5), // "state"
QT_MOC_LITERAL(16, 147, 14), // "insertRedoItem"
QT_MOC_LITERAL(17, 162, 10), // "updateUndo"
QT_MOC_LITERAL(18, 173, 10), // "updateRedo"
QT_MOC_LITERAL(19, 184, 9), // "clearRedo"
QT_MOC_LITERAL(20, 194, 7) // "popBack"

    },
    "UndoWidget\0undo\0\0steps\0redo\0undoClicked\0"
    "redoClicked\0undoMenuClicked\0QAction*\0"
    "id\0redoMenuClicked\0insertUndoItem\0"
    "UndoObject*\0target\0UndoState*\0state\0"
    "insertRedoItem\0updateUndo\0updateRedo\0"
    "clearRedo\0popBack"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_UndoWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       4,    1,   77,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,   80,    2, 0x08 /* Private */,
       6,    0,   81,    2, 0x08 /* Private */,
       7,    1,   82,    2, 0x08 /* Private */,
      10,    1,   85,    2, 0x08 /* Private */,
      11,    2,   88,    2, 0x0a /* Public */,
      16,    2,   93,    2, 0x0a /* Public */,
      17,    1,   98,    2, 0x0a /* Public */,
      18,    1,  101,    2, 0x0a /* Public */,
      19,    0,  104,    2, 0x0a /* Public */,
      20,    0,  105,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void, 0x80000000 | 12, 0x80000000 | 14,   13,   15,
    QMetaType::Void, 0x80000000 | 12, 0x80000000 | 14,   13,   15,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void, QMetaType::Int,    3,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void UndoWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<UndoWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->undo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->redo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->undoClicked(); break;
        case 3: _t->redoClicked(); break;
        case 4: _t->undoMenuClicked((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 5: _t->redoMenuClicked((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 6: _t->insertUndoItem((*reinterpret_cast< UndoObject*(*)>(_a[1])),(*reinterpret_cast< UndoState*(*)>(_a[2]))); break;
        case 7: _t->insertRedoItem((*reinterpret_cast< UndoObject*(*)>(_a[1])),(*reinterpret_cast< UndoState*(*)>(_a[2]))); break;
        case 8: _t->updateUndo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->updateRedo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->clearRedo(); break;
        case 11: _t->popBack(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (UndoWidget::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&UndoWidget::undo)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (UndoWidget::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&UndoWidget::redo)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject UndoWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<UndoGui::staticMetaObject>(),
    qt_meta_stringdata_UndoWidget.data,
    qt_meta_data_UndoWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *UndoWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UndoWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_UndoWidget.stringdata0))
        return static_cast<void*>(this);
    return UndoGui::qt_metacast(_clname);
}

int UndoWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = UndoGui::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void UndoWidget::undo(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void UndoWidget::redo(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}
struct qt_meta_stringdata_UndoPalette_t {
    QByteArrayData data[28];
    char stringdata0[297];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_UndoPalette_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_UndoPalette_t qt_meta_stringdata_UndoPalette = {
    {
QT_MOC_LITERAL(0, 0, 11), // "UndoPalette"
QT_MOC_LITERAL(1, 12, 10), // "objectMode"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 9), // "isEnabled"
QT_MOC_LITERAL(4, 34, 4), // "undo"
QT_MOC_LITERAL(5, 39, 5), // "steps"
QT_MOC_LITERAL(6, 45, 4), // "redo"
QT_MOC_LITERAL(7, 50, 14), // "languageChange"
QT_MOC_LITERAL(8, 65, 14), // "insertUndoItem"
QT_MOC_LITERAL(9, 80, 11), // "UndoObject*"
QT_MOC_LITERAL(10, 92, 6), // "target"
QT_MOC_LITERAL(11, 99, 10), // "UndoState*"
QT_MOC_LITERAL(12, 110, 5), // "state"
QT_MOC_LITERAL(13, 116, 14), // "insertRedoItem"
QT_MOC_LITERAL(14, 131, 10), // "updateUndo"
QT_MOC_LITERAL(15, 142, 10), // "updateRedo"
QT_MOC_LITERAL(16, 153, 9), // "clearRedo"
QT_MOC_LITERAL(17, 163, 7), // "popBack"
QT_MOC_LITERAL(18, 171, 15), // "updateFromPrefs"
QT_MOC_LITERAL(19, 187, 11), // "undoClicked"
QT_MOC_LITERAL(20, 199, 11), // "redoClicked"
QT_MOC_LITERAL(21, 211, 15), // "undoListClicked"
QT_MOC_LITERAL(22, 227, 1), // "i"
QT_MOC_LITERAL(23, 229, 11), // "showToolTip"
QT_MOC_LITERAL(24, 241, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(25, 258, 13), // "removeToolTip"
QT_MOC_LITERAL(26, 272, 21), // "objectCheckBoxClicked"
QT_MOC_LITERAL(27, 294, 2) // "on"

    },
    "UndoPalette\0objectMode\0\0isEnabled\0"
    "undo\0steps\0redo\0languageChange\0"
    "insertUndoItem\0UndoObject*\0target\0"
    "UndoState*\0state\0insertRedoItem\0"
    "updateUndo\0updateRedo\0clearRedo\0popBack\0"
    "updateFromPrefs\0undoClicked\0redoClicked\0"
    "undoListClicked\0i\0showToolTip\0"
    "QListWidgetItem*\0removeToolTip\0"
    "objectCheckBoxClicked\0on"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_UndoPalette[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   99,    2, 0x06 /* Public */,
       4,    1,  102,    2, 0x06 /* Public */,
       6,    1,  105,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,  108,    2, 0x0a /* Public */,
       8,    2,  109,    2, 0x0a /* Public */,
      13,    2,  114,    2, 0x0a /* Public */,
      14,    1,  119,    2, 0x0a /* Public */,
      15,    1,  122,    2, 0x0a /* Public */,
      16,    0,  125,    2, 0x0a /* Public */,
      17,    0,  126,    2, 0x0a /* Public */,
      18,    0,  127,    2, 0x0a /* Public */,
      19,    0,  128,    2, 0x08 /* Private */,
      20,    0,  129,    2, 0x08 /* Private */,
      21,    1,  130,    2, 0x08 /* Private */,
      23,    1,  133,    2, 0x08 /* Private */,
      25,    0,  136,    2, 0x08 /* Private */,
      26,    1,  137,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Bool,    3,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9, 0x80000000 | 11,   10,   12,
    QMetaType::Void, 0x80000000 | 9, 0x80000000 | 11,   10,   12,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void, 0x80000000 | 24,   22,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   27,

       0        // eod
};

void UndoPalette::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<UndoPalette *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->objectMode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 1: _t->undo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->redo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->languageChange(); break;
        case 4: _t->insertUndoItem((*reinterpret_cast< UndoObject*(*)>(_a[1])),(*reinterpret_cast< UndoState*(*)>(_a[2]))); break;
        case 5: _t->insertRedoItem((*reinterpret_cast< UndoObject*(*)>(_a[1])),(*reinterpret_cast< UndoState*(*)>(_a[2]))); break;
        case 6: _t->updateUndo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->updateRedo((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->clearRedo(); break;
        case 9: _t->popBack(); break;
        case 10: _t->updateFromPrefs(); break;
        case 11: _t->undoClicked(); break;
        case 12: _t->redoClicked(); break;
        case 13: _t->undoListClicked((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->showToolTip((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 15: _t->removeToolTip(); break;
        case 16: _t->objectCheckBoxClicked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (UndoPalette::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&UndoPalette::objectMode)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (UndoPalette::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&UndoPalette::undo)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (UndoPalette::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&UndoPalette::redo)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject UndoPalette::staticMetaObject = { {
    QMetaObject::SuperData::link<UndoGui::staticMetaObject>(),
    qt_meta_stringdata_UndoPalette.data,
    qt_meta_data_UndoPalette,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *UndoPalette::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *UndoPalette::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_UndoPalette.stringdata0))
        return static_cast<void*>(this);
    return UndoGui::qt_metacast(_clname);
}

int UndoPalette::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = UndoGui::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 17;
    }
    return _id;
}

// SIGNAL 0
void UndoPalette::objectMode(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void UndoPalette::undo(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void UndoPalette::redo(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
