/********************************************************************************
** Form generated from reading UI file 'pagepalette_pagesbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAGEPALETTE_PAGESBASE_H
#define UI_PAGEPALETTE_PAGESBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/pagelayout.h"
#include "ui/pagepalette_widgets.h"

QT_BEGIN_NAMESPACE

class Ui_PagePalette_PagesBase
{
public:
    QVBoxLayout *verticalLayout;
    QSplitter *splitter;
    QWidget *widget;
    QVBoxLayout *verticalLayout_3;
    QLabel *masterPageLabel;
    SeList *masterPageList;
    QWidget *widget_2;
    QVBoxLayout *verticalLayout_4;
    QLabel *pageViewLabel;
    SeView *pageView;
    QHBoxLayout *horizontalLayout;
    PageLayouts *pageLayout;
    TrashBin *trash;

    void setupUi(QWidget *PagePalette_PagesBase)
    {
        if (PagePalette_PagesBase->objectName().isEmpty())
            PagePalette_PagesBase->setObjectName(QString::fromUtf8("PagePalette_PagesBase"));
        PagePalette_PagesBase->resize(220, 286);
        verticalLayout = new QVBoxLayout(PagePalette_PagesBase);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        splitter = new QSplitter(PagePalette_PagesBase);
        splitter->setObjectName(QString::fromUtf8("splitter"));
        splitter->setOrientation(Qt::Vertical);
        splitter->setChildrenCollapsible(false);
        widget = new QWidget(splitter);
        widget->setObjectName(QString::fromUtf8("widget"));
        verticalLayout_3 = new QVBoxLayout(widget);
        verticalLayout_3->setSpacing(5);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        masterPageLabel = new QLabel(widget);
        masterPageLabel->setObjectName(QString::fromUtf8("masterPageLabel"));

        verticalLayout_3->addWidget(masterPageLabel);

        masterPageList = new SeList(widget);
        masterPageList->setObjectName(QString::fromUtf8("masterPageList"));
        masterPageList->setIconSize(QSize(60, 60));

        verticalLayout_3->addWidget(masterPageList);

        splitter->addWidget(widget);
        widget_2 = new QWidget(splitter);
        widget_2->setObjectName(QString::fromUtf8("widget_2"));
        verticalLayout_4 = new QVBoxLayout(widget_2);
        verticalLayout_4->setSpacing(5);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        pageViewLabel = new QLabel(widget_2);
        pageViewLabel->setObjectName(QString::fromUtf8("pageViewLabel"));

        verticalLayout_4->addWidget(pageViewLabel);

        pageView = new SeView(widget_2);
        if (pageView->columnCount() < 1)
            pageView->setColumnCount(1);
        if (pageView->rowCount() < 1)
            pageView->setRowCount(1);
        pageView->setObjectName(QString::fromUtf8("pageView"));
        pageView->setSelectionMode(QAbstractItemView::NoSelection);
        pageView->setIconSize(QSize(60, 60));
        pageView->setRowCount(1);
        pageView->setColumnCount(1);
        pageView->horizontalHeader()->setVisible(false);
        pageView->verticalHeader()->setVisible(false);

        verticalLayout_4->addWidget(pageView);

        splitter->addWidget(widget_2);

        verticalLayout->addWidget(splitter);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        pageLayout = new PageLayouts(PagePalette_PagesBase);
        pageLayout->setObjectName(QString::fromUtf8("pageLayout"));

        horizontalLayout->addWidget(pageLayout);

        trash = new TrashBin(PagePalette_PagesBase);
        trash->setObjectName(QString::fromUtf8("trash"));
        trash->setMinimumSize(QSize(32, 32));
        trash->setMaximumSize(QSize(32, 32));

        horizontalLayout->addWidget(trash);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(PagePalette_PagesBase);

        QMetaObject::connectSlotsByName(PagePalette_PagesBase);
    } // setupUi

    void retranslateUi(QWidget *PagePalette_PagesBase)
    {
        PagePalette_PagesBase->setWindowTitle(QCoreApplication::translate("PagePalette_PagesBase", "Arrange Pages", nullptr));
        masterPageLabel->setText(QCoreApplication::translate("PagePalette_PagesBase", "Available Master Pages:", nullptr));
#if QT_CONFIG(tooltip)
        masterPageList->setToolTip(QCoreApplication::translate("PagePalette_PagesBase", "<qt>List of master pages in the document. Master page names may be dragged onto the page view below to apply master pages, or onto the empty space between pages to create new pages.</qt>", nullptr));
#endif // QT_CONFIG(tooltip)
        pageViewLabel->setText(QCoreApplication::translate("PagePalette_PagesBase", "Document Pages:", nullptr));
#if QT_CONFIG(tooltip)
        pageView->setToolTip(QCoreApplication::translate("PagePalette_PagesBase", "<qt>List of normal pages in the document, shown with the document layout. Pages may be dragged to rearrange or delete them.</qt>", nullptr));
#endif // QT_CONFIG(tooltip)
        pageLayout->setTitle(QCoreApplication::translate("PagePalette_PagesBase", "Document Layout", nullptr));
#if QT_CONFIG(tooltip)
        trash->setToolTip(QCoreApplication::translate("PagePalette_PagesBase", "<qt>Drag pages or master pages onto the trash to delete them</qt>", nullptr));
#endif // QT_CONFIG(tooltip)
        trash->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class PagePalette_PagesBase: public Ui_PagePalette_PagesBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PAGEPALETTE_PAGESBASE_H
