/****************************************************************************
** Meta object code from reading C++ file 'smfontfeatures.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/smfontfeatures.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smfontfeatures.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_StylisticSetsMenu_t {
    QByteArrayData data[1];
    char stringdata0[18];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_StylisticSetsMenu_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_StylisticSetsMenu_t qt_meta_stringdata_StylisticSetsMenu = {
    {
QT_MOC_LITERAL(0, 0, 17) // "StylisticSetsMenu"

    },
    "StylisticSetsMenu"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_StylisticSetsMenu[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void StylisticSetsMenu::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject StylisticSetsMenu::staticMetaObject = { {
    QMetaObject::SuperData::link<QMenu::staticMetaObject>(),
    qt_meta_stringdata_StylisticSetsMenu.data,
    qt_meta_data_StylisticSetsMenu,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *StylisticSetsMenu::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *StylisticSetsMenu::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_StylisticSetsMenu.stringdata0))
        return static_cast<void*>(this);
    return QMenu::qt_metacast(_clname);
}

int StylisticSetsMenu::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMenu::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_SMFontFeatures_t {
    QByteArrayData data[15];
    char stringdata0[248];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SMFontFeatures_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SMFontFeatures_t qt_meta_stringdata_SMFontFeatures = {
    {
QT_MOC_LITERAL(0, 0, 14), // "SMFontFeatures"
QT_MOC_LITERAL(1, 15, 7), // "changed"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 10), // "slotChange"
QT_MOC_LITERAL(4, 35, 19), // "slotContextualCheck"
QT_MOC_LITERAL(5, 55, 15), // "slotCommonCheck"
QT_MOC_LITERAL(6, 71, 22), // "slotDiscretionaryCheck"
QT_MOC_LITERAL(7, 94, 19), // "slotHistoricalCheck"
QT_MOC_LITERAL(8, 114, 20), // "slotCapitalsComboBox"
QT_MOC_LITERAL(9, 135, 20), // "slotPositionComboBox"
QT_MOC_LITERAL(10, 156, 19), // "slotNumeralComboBox"
QT_MOC_LITERAL(11, 176, 17), // "slotWidthComboBox"
QT_MOC_LITERAL(12, 194, 20), // "slotFractionComboBox"
QT_MOC_LITERAL(13, 215, 20), // "slotSlashedZeroCheck"
QT_MOC_LITERAL(14, 236, 11) // "slotSetMenu"

    },
    "SMFontFeatures\0changed\0\0slotChange\0"
    "slotContextualCheck\0slotCommonCheck\0"
    "slotDiscretionaryCheck\0slotHistoricalCheck\0"
    "slotCapitalsComboBox\0slotPositionComboBox\0"
    "slotNumeralComboBox\0slotWidthComboBox\0"
    "slotFractionComboBox\0slotSlashedZeroCheck\0"
    "slotSetMenu"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SMFontFeatures[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   79,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   80,    2, 0x08 /* Private */,
       4,    0,   81,    2, 0x08 /* Private */,
       5,    0,   82,    2, 0x08 /* Private */,
       6,    0,   83,    2, 0x08 /* Private */,
       7,    0,   84,    2, 0x08 /* Private */,
       8,    0,   85,    2, 0x08 /* Private */,
       9,    0,   86,    2, 0x08 /* Private */,
      10,    0,   87,    2, 0x08 /* Private */,
      11,    0,   88,    2, 0x08 /* Private */,
      12,    0,   89,    2, 0x08 /* Private */,
      13,    0,   90,    2, 0x08 /* Private */,
      14,    0,   91,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void SMFontFeatures::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SMFontFeatures *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->changed(); break;
        case 1: _t->slotChange(); break;
        case 2: _t->slotContextualCheck(); break;
        case 3: _t->slotCommonCheck(); break;
        case 4: _t->slotDiscretionaryCheck(); break;
        case 5: _t->slotHistoricalCheck(); break;
        case 6: _t->slotCapitalsComboBox(); break;
        case 7: _t->slotPositionComboBox(); break;
        case 8: _t->slotNumeralComboBox(); break;
        case 9: _t->slotWidthComboBox(); break;
        case 10: _t->slotFractionComboBox(); break;
        case 11: _t->slotSlashedZeroCheck(); break;
        case 12: _t->slotSetMenu(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SMFontFeatures::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SMFontFeatures::changed)) {
                *result = 0;
                return;
            }
        }
    }
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject SMFontFeatures::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_SMFontFeatures.data,
    qt_meta_data_SMFontFeatures,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SMFontFeatures::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SMFontFeatures::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SMFontFeatures.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::smFontFeaturesBase"))
        return static_cast< Ui::smFontFeaturesBase*>(this);
    return QWidget::qt_metacast(_clname);
}

int SMFontFeatures::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void SMFontFeatures::changed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
