/********************************************************************************
** Form generated from reading UI file 'prefs_typographybase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_TYPOGRAPHYBASE_H
#define UI_PREFS_TYPOGRAPHYBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDoubleSpinBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Prefs_Typography
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    QFrame *line;
    QHBoxLayout *horizontalLayout_4;
    QVBoxLayout *verticalLayout_5;
    QLabel *label_2;
    QFrame *line_3;
    QFormLayout *formLayout_3;
    QLabel *label_5;
    QSpinBox *subscriptDisplacementSpinBox;
    QSpinBox *subscriptScalingSpinBox;
    QLabel *label_6;
    QSpacerItem *verticalSpacer;
    QLabel *label_3;
    QFrame *line_2;
    QFormLayout *formLayout_2;
    QLabel *label_7;
    QSpinBox *superscriptDisplacementSpinBox;
    QLabel *label_8;
    QSpinBox *superscriptScalingSpinBox;
    QSpacerItem *verticalSpacer_2;
    QLabel *label_14;
    QFrame *line_5;
    QFormLayout *formLayout_5;
    QLabel *label_15;
    QSpinBox *smallcapsScalingSpinBox;
    QSpacerItem *horizontalSpacer;
    QVBoxLayout *verticalLayout_3;
    QLabel *label_4;
    QFrame *line_1;
    QFormLayout *formLayout_1;
    QLabel *label_9;
    QLabel *label_10;
    QDoubleSpinBox *underlineLineWidthSpinBox;
    QDoubleSpinBox *underlineDisplacementSpinBox;
    QSpacerItem *verticalSpacer_4;
    QLabel *label_11;
    QFrame *line_4;
    QFormLayout *formLayout_4;
    QLabel *label_12;
    QDoubleSpinBox *strikeoutDisplacementSpinBox;
    QLabel *label_13;
    QDoubleSpinBox *strikeoutLineWidthSpinBox;
    QSpacerItem *verticalSpacer_6;
    QLabel *label_17;
    QFrame *line_6;
    QFormLayout *formLayout_6;
    QLabel *label_19;
    QSpinBox *automaticLineSpacingSpinBox;
    QSpacerItem *verticalSpacer_3;

    void setupUi(QWidget *Prefs_Typography)
    {
        if (Prefs_Typography->objectName().isEmpty())
            Prefs_Typography->setObjectName(QString::fromUtf8("Prefs_Typography"));
        Prefs_Typography->resize(682, 449);
        verticalLayout_2 = new QVBoxLayout(Prefs_Typography);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(Prefs_Typography);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        verticalLayout_2->addWidget(label);

        line = new QFrame(Prefs_Typography);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        label_2 = new QLabel(Prefs_Typography);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
        label_2->setFont(font1);

        verticalLayout_5->addWidget(label_2);

        line_3 = new QFrame(Prefs_Typography);
        line_3->setObjectName(QString::fromUtf8("line_3"));
        line_3->setFrameShape(QFrame::HLine);
        line_3->setFrameShadow(QFrame::Sunken);

        verticalLayout_5->addWidget(line_3);

        formLayout_3 = new QFormLayout();
        formLayout_3->setObjectName(QString::fromUtf8("formLayout_3"));
        formLayout_3->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        formLayout_3->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_5 = new QLabel(Prefs_Typography);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        formLayout_3->setWidget(0, QFormLayout::LabelRole, label_5);

        subscriptDisplacementSpinBox = new QSpinBox(Prefs_Typography);
        subscriptDisplacementSpinBox->setObjectName(QString::fromUtf8("subscriptDisplacementSpinBox"));
        subscriptDisplacementSpinBox->setMaximum(100);

        formLayout_3->setWidget(0, QFormLayout::FieldRole, subscriptDisplacementSpinBox);

        subscriptScalingSpinBox = new QSpinBox(Prefs_Typography);
        subscriptScalingSpinBox->setObjectName(QString::fromUtf8("subscriptScalingSpinBox"));
        subscriptScalingSpinBox->setMinimum(1);
        subscriptScalingSpinBox->setMaximum(100);

        formLayout_3->setWidget(1, QFormLayout::FieldRole, subscriptScalingSpinBox);

        label_6 = new QLabel(Prefs_Typography);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        formLayout_3->setWidget(1, QFormLayout::LabelRole, label_6);


        verticalLayout_5->addLayout(formLayout_3);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_5->addItem(verticalSpacer);

        label_3 = new QLabel(Prefs_Typography);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        label_3->setFont(font1);

        verticalLayout_5->addWidget(label_3);

        line_2 = new QFrame(Prefs_Typography);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        verticalLayout_5->addWidget(line_2);

        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName(QString::fromUtf8("formLayout_2"));
        formLayout_2->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        formLayout_2->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_7 = new QLabel(Prefs_Typography);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        formLayout_2->setWidget(0, QFormLayout::LabelRole, label_7);

        superscriptDisplacementSpinBox = new QSpinBox(Prefs_Typography);
        superscriptDisplacementSpinBox->setObjectName(QString::fromUtf8("superscriptDisplacementSpinBox"));
        superscriptDisplacementSpinBox->setMaximum(100);

        formLayout_2->setWidget(0, QFormLayout::FieldRole, superscriptDisplacementSpinBox);

        label_8 = new QLabel(Prefs_Typography);
        label_8->setObjectName(QString::fromUtf8("label_8"));

        formLayout_2->setWidget(1, QFormLayout::LabelRole, label_8);

        superscriptScalingSpinBox = new QSpinBox(Prefs_Typography);
        superscriptScalingSpinBox->setObjectName(QString::fromUtf8("superscriptScalingSpinBox"));
        superscriptScalingSpinBox->setMinimum(1);
        superscriptScalingSpinBox->setMaximum(100);

        formLayout_2->setWidget(1, QFormLayout::FieldRole, superscriptScalingSpinBox);


        verticalLayout_5->addLayout(formLayout_2);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_5->addItem(verticalSpacer_2);

        label_14 = new QLabel(Prefs_Typography);
        label_14->setObjectName(QString::fromUtf8("label_14"));
        label_14->setFont(font1);

        verticalLayout_5->addWidget(label_14);

        line_5 = new QFrame(Prefs_Typography);
        line_5->setObjectName(QString::fromUtf8("line_5"));
        line_5->setFrameShape(QFrame::HLine);
        line_5->setFrameShadow(QFrame::Sunken);

        verticalLayout_5->addWidget(line_5);

        formLayout_5 = new QFormLayout();
        formLayout_5->setObjectName(QString::fromUtf8("formLayout_5"));
        formLayout_5->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        formLayout_5->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_15 = new QLabel(Prefs_Typography);
        label_15->setObjectName(QString::fromUtf8("label_15"));

        formLayout_5->setWidget(0, QFormLayout::LabelRole, label_15);

        smallcapsScalingSpinBox = new QSpinBox(Prefs_Typography);
        smallcapsScalingSpinBox->setObjectName(QString::fromUtf8("smallcapsScalingSpinBox"));
        smallcapsScalingSpinBox->setMaximum(100);

        formLayout_5->setWidget(0, QFormLayout::FieldRole, smallcapsScalingSpinBox);


        verticalLayout_5->addLayout(formLayout_5);


        horizontalLayout_4->addLayout(verticalLayout_5);

        horizontalSpacer = new QSpacerItem(9, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer);

        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        label_4 = new QLabel(Prefs_Typography);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        label_4->setFont(font1);

        verticalLayout_3->addWidget(label_4);

        line_1 = new QFrame(Prefs_Typography);
        line_1->setObjectName(QString::fromUtf8("line_1"));
        line_1->setFrameShape(QFrame::HLine);
        line_1->setFrameShadow(QFrame::Sunken);

        verticalLayout_3->addWidget(line_1);

        formLayout_1 = new QFormLayout();
        formLayout_1->setObjectName(QString::fromUtf8("formLayout_1"));
        formLayout_1->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        formLayout_1->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_9 = new QLabel(Prefs_Typography);
        label_9->setObjectName(QString::fromUtf8("label_9"));

        formLayout_1->setWidget(0, QFormLayout::LabelRole, label_9);

        label_10 = new QLabel(Prefs_Typography);
        label_10->setObjectName(QString::fromUtf8("label_10"));

        formLayout_1->setWidget(1, QFormLayout::LabelRole, label_10);

        underlineLineWidthSpinBox = new QDoubleSpinBox(Prefs_Typography);
        underlineLineWidthSpinBox->setObjectName(QString::fromUtf8("underlineLineWidthSpinBox"));
        underlineLineWidthSpinBox->setWrapping(true);
        underlineLineWidthSpinBox->setMinimum(-0.100000000000000);
        underlineLineWidthSpinBox->setMaximum(100.000000000000000);

        formLayout_1->setWidget(1, QFormLayout::FieldRole, underlineLineWidthSpinBox);

        underlineDisplacementSpinBox = new QDoubleSpinBox(Prefs_Typography);
        underlineDisplacementSpinBox->setObjectName(QString::fromUtf8("underlineDisplacementSpinBox"));
        underlineDisplacementSpinBox->setWrapping(true);
        underlineDisplacementSpinBox->setMinimum(-0.100000000000000);
        underlineDisplacementSpinBox->setMaximum(100.000000000000000);

        formLayout_1->setWidget(0, QFormLayout::FieldRole, underlineDisplacementSpinBox);


        verticalLayout_3->addLayout(formLayout_1);

        verticalSpacer_4 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_3->addItem(verticalSpacer_4);

        label_11 = new QLabel(Prefs_Typography);
        label_11->setObjectName(QString::fromUtf8("label_11"));
        label_11->setFont(font1);

        verticalLayout_3->addWidget(label_11);

        line_4 = new QFrame(Prefs_Typography);
        line_4->setObjectName(QString::fromUtf8("line_4"));
        line_4->setFrameShape(QFrame::HLine);
        line_4->setFrameShadow(QFrame::Sunken);

        verticalLayout_3->addWidget(line_4);

        formLayout_4 = new QFormLayout();
        formLayout_4->setObjectName(QString::fromUtf8("formLayout_4"));
        formLayout_4->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        formLayout_4->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_12 = new QLabel(Prefs_Typography);
        label_12->setObjectName(QString::fromUtf8("label_12"));

        formLayout_4->setWidget(0, QFormLayout::LabelRole, label_12);

        strikeoutDisplacementSpinBox = new QDoubleSpinBox(Prefs_Typography);
        strikeoutDisplacementSpinBox->setObjectName(QString::fromUtf8("strikeoutDisplacementSpinBox"));
        strikeoutDisplacementSpinBox->setWrapping(true);
        strikeoutDisplacementSpinBox->setMinimum(-0.100000000000000);
        strikeoutDisplacementSpinBox->setMaximum(100.000000000000000);

        formLayout_4->setWidget(0, QFormLayout::FieldRole, strikeoutDisplacementSpinBox);

        label_13 = new QLabel(Prefs_Typography);
        label_13->setObjectName(QString::fromUtf8("label_13"));

        formLayout_4->setWidget(1, QFormLayout::LabelRole, label_13);

        strikeoutLineWidthSpinBox = new QDoubleSpinBox(Prefs_Typography);
        strikeoutLineWidthSpinBox->setObjectName(QString::fromUtf8("strikeoutLineWidthSpinBox"));
        strikeoutLineWidthSpinBox->setWrapping(true);
        strikeoutLineWidthSpinBox->setMinimum(-0.100000000000000);
        strikeoutLineWidthSpinBox->setMaximum(100.000000000000000);

        formLayout_4->setWidget(1, QFormLayout::FieldRole, strikeoutLineWidthSpinBox);


        verticalLayout_3->addLayout(formLayout_4);

        verticalSpacer_6 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_3->addItem(verticalSpacer_6);

        label_17 = new QLabel(Prefs_Typography);
        label_17->setObjectName(QString::fromUtf8("label_17"));
        label_17->setFont(font1);

        verticalLayout_3->addWidget(label_17);

        line_6 = new QFrame(Prefs_Typography);
        line_6->setObjectName(QString::fromUtf8("line_6"));
        line_6->setFrameShape(QFrame::HLine);
        line_6->setFrameShadow(QFrame::Sunken);

        verticalLayout_3->addWidget(line_6);

        formLayout_6 = new QFormLayout();
        formLayout_6->setObjectName(QString::fromUtf8("formLayout_6"));
        formLayout_6->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        formLayout_6->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_19 = new QLabel(Prefs_Typography);
        label_19->setObjectName(QString::fromUtf8("label_19"));

        formLayout_6->setWidget(0, QFormLayout::LabelRole, label_19);

        automaticLineSpacingSpinBox = new QSpinBox(Prefs_Typography);
        automaticLineSpacingSpinBox->setObjectName(QString::fromUtf8("automaticLineSpacingSpinBox"));
        automaticLineSpacingSpinBox->setMinimum(1);
        automaticLineSpacingSpinBox->setMaximum(500);

        formLayout_6->setWidget(0, QFormLayout::FieldRole, automaticLineSpacingSpinBox);


        verticalLayout_3->addLayout(formLayout_6);


        horizontalLayout_4->addLayout(verticalLayout_3);


        verticalLayout_2->addLayout(horizontalLayout_4);

        verticalSpacer_3 = new QSpacerItem(20, 34, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_3);

        QWidget::setTabOrder(subscriptDisplacementSpinBox, subscriptScalingSpinBox);
        QWidget::setTabOrder(subscriptScalingSpinBox, superscriptDisplacementSpinBox);
        QWidget::setTabOrder(superscriptDisplacementSpinBox, superscriptScalingSpinBox);
        QWidget::setTabOrder(superscriptScalingSpinBox, smallcapsScalingSpinBox);
        QWidget::setTabOrder(smallcapsScalingSpinBox, underlineDisplacementSpinBox);
        QWidget::setTabOrder(underlineDisplacementSpinBox, underlineLineWidthSpinBox);
        QWidget::setTabOrder(underlineLineWidthSpinBox, strikeoutDisplacementSpinBox);
        QWidget::setTabOrder(strikeoutDisplacementSpinBox, strikeoutLineWidthSpinBox);
        QWidget::setTabOrder(strikeoutLineWidthSpinBox, automaticLineSpacingSpinBox);

        retranslateUi(Prefs_Typography);

        QMetaObject::connectSlotsByName(Prefs_Typography);
    } // setupUi

    void retranslateUi(QWidget *Prefs_Typography)
    {
        label->setText(QCoreApplication::translate("Prefs_Typography", "Typography", nullptr));
        label_2->setText(QCoreApplication::translate("Prefs_Typography", "Subscript", nullptr));
        label_5->setText(QCoreApplication::translate("Prefs_Typography", "Displacement:", nullptr));
        subscriptDisplacementSpinBox->setSuffix(QCoreApplication::translate("Prefs_Typography", " %", nullptr));
        subscriptScalingSpinBox->setSuffix(QCoreApplication::translate("Prefs_Typography", " %", nullptr));
        label_6->setText(QCoreApplication::translate("Prefs_Typography", "Scaling:", nullptr));
        label_3->setText(QCoreApplication::translate("Prefs_Typography", "Superscript", nullptr));
        label_7->setText(QCoreApplication::translate("Prefs_Typography", "Displacement:", nullptr));
        superscriptDisplacementSpinBox->setSuffix(QCoreApplication::translate("Prefs_Typography", " %", nullptr));
        label_8->setText(QCoreApplication::translate("Prefs_Typography", "Scaling:", nullptr));
        superscriptScalingSpinBox->setSuffix(QCoreApplication::translate("Prefs_Typography", " %", nullptr));
        label_14->setText(QCoreApplication::translate("Prefs_Typography", "Small Caps", nullptr));
        label_15->setText(QCoreApplication::translate("Prefs_Typography", "Scaling:", nullptr));
        smallcapsScalingSpinBox->setSuffix(QCoreApplication::translate("Prefs_Typography", " %", nullptr));
        label_4->setText(QCoreApplication::translate("Prefs_Typography", "Underline", nullptr));
        label_9->setText(QCoreApplication::translate("Prefs_Typography", "Displacement:", nullptr));
        label_10->setText(QCoreApplication::translate("Prefs_Typography", "Line Width:", nullptr));
        underlineLineWidthSpinBox->setSpecialValueText(QCoreApplication::translate("Prefs_Typography", "Auto", nullptr));
        underlineLineWidthSpinBox->setSuffix(QCoreApplication::translate("Prefs_Typography", " %", nullptr));
        underlineDisplacementSpinBox->setSpecialValueText(QCoreApplication::translate("Prefs_Typography", "Auto", nullptr));
        underlineDisplacementSpinBox->setSuffix(QCoreApplication::translate("Prefs_Typography", " %", nullptr));
        label_11->setText(QCoreApplication::translate("Prefs_Typography", "Strikeout", nullptr));
        label_12->setText(QCoreApplication::translate("Prefs_Typography", "Displacement:", nullptr));
        strikeoutDisplacementSpinBox->setSpecialValueText(QCoreApplication::translate("Prefs_Typography", "Auto", nullptr));
        strikeoutDisplacementSpinBox->setSuffix(QCoreApplication::translate("Prefs_Typography", " %", nullptr));
        label_13->setText(QCoreApplication::translate("Prefs_Typography", "Line Width:", nullptr));
        strikeoutLineWidthSpinBox->setSpecialValueText(QCoreApplication::translate("Prefs_Typography", "Auto", nullptr));
        strikeoutLineWidthSpinBox->setSuffix(QCoreApplication::translate("Prefs_Typography", " %", nullptr));
        label_17->setText(QCoreApplication::translate("Prefs_Typography", "Automatic Line Spacing", nullptr));
        label_19->setText(QCoreApplication::translate("Prefs_Typography", "Line Spacing:", nullptr));
        automaticLineSpacingSpinBox->setSuffix(QCoreApplication::translate("Prefs_Typography", " %", nullptr));
        (void)Prefs_Typography;
    } // retranslateUi

};

namespace Ui {
    class Prefs_Typography: public Ui_Prefs_Typography {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_TYPOGRAPHYBASE_H
