/********************************************************************************
** Form generated from reading UI file 'prefs_documentinformationbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_DOCUMENTINFORMATIONBASE_H
#define UI_PREFS_DOCUMENTINFORMATIONBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Prefs_DocumentInformation
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QFrame *line;
    QTabWidget *tabWidget;
    QWidget *tabDocument;
    QFormLayout *formLayout;
    QLabel *label_2;
    QLineEdit *titleLineEdit;
    QLabel *label_3;
    QLineEdit *authorLineEdit;
    QLabel *label_4;
    QLineEdit *subjectLineEdit;
    QLabel *label_5;
    QTextEdit *keywordsTextEdit;
    QLabel *label_6;
    QTextEdit *descriptionTextEdit;
    QWidget *tabFurtherInformation;
    QVBoxLayout *verticalLayout_3;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_2;
    QFormLayout *formLayout_2;
    QLabel *label_7;
    QLineEdit *publisherLineEdit;
    QLabel *label_8;
    QTextEdit *contributorsTextEdit;
    QLabel *label_9;
    QComboBox *dateLineEdit;
    QLabel *label_10;
    QComboBox *typeLineEdit;
    QLabel *label_11;
    QLineEdit *formatLineEdit;
    QLabel *label_12;
    QLineEdit *identifierLineEdit;
    QLabel *label_13;
    QLineEdit *sourceLineEdit;
    QLabel *label_14;
    QComboBox *languageLineEdit;
    QLabel *label_15;
    QLineEdit *relationLineEdit;
    QLabel *label_16;
    QComboBox *coverageLineEdit;
    QLabel *label_17;
    QComboBox *rightsLineEdit;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *Prefs_DocumentInformation)
    {
        if (Prefs_DocumentInformation->objectName().isEmpty())
            Prefs_DocumentInformation->setObjectName(QString::fromUtf8("Prefs_DocumentInformation"));
        Prefs_DocumentInformation->resize(688, 692);
        verticalLayout = new QVBoxLayout(Prefs_DocumentInformation);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(Prefs_DocumentInformation);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        verticalLayout->addWidget(label);

        line = new QFrame(Prefs_DocumentInformation);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        tabWidget = new QTabWidget(Prefs_DocumentInformation);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tabDocument = new QWidget();
        tabDocument->setObjectName(QString::fromUtf8("tabDocument"));
        formLayout = new QFormLayout(tabDocument);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        label_2 = new QLabel(tabDocument);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        formLayout->setWidget(0, QFormLayout::LabelRole, label_2);

        titleLineEdit = new QLineEdit(tabDocument);
        titleLineEdit->setObjectName(QString::fromUtf8("titleLineEdit"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(titleLineEdit->sizePolicy().hasHeightForWidth());
        titleLineEdit->setSizePolicy(sizePolicy);

        formLayout->setWidget(0, QFormLayout::FieldRole, titleLineEdit);

        label_3 = new QLabel(tabDocument);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        formLayout->setWidget(1, QFormLayout::LabelRole, label_3);

        authorLineEdit = new QLineEdit(tabDocument);
        authorLineEdit->setObjectName(QString::fromUtf8("authorLineEdit"));

        formLayout->setWidget(1, QFormLayout::FieldRole, authorLineEdit);

        label_4 = new QLabel(tabDocument);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        formLayout->setWidget(2, QFormLayout::LabelRole, label_4);

        subjectLineEdit = new QLineEdit(tabDocument);
        subjectLineEdit->setObjectName(QString::fromUtf8("subjectLineEdit"));

        formLayout->setWidget(2, QFormLayout::FieldRole, subjectLineEdit);

        label_5 = new QLabel(tabDocument);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        formLayout->setWidget(3, QFormLayout::LabelRole, label_5);

        keywordsTextEdit = new QTextEdit(tabDocument);
        keywordsTextEdit->setObjectName(QString::fromUtf8("keywordsTextEdit"));
        keywordsTextEdit->setTabChangesFocus(true);

        formLayout->setWidget(3, QFormLayout::FieldRole, keywordsTextEdit);

        label_6 = new QLabel(tabDocument);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        formLayout->setWidget(4, QFormLayout::LabelRole, label_6);

        descriptionTextEdit = new QTextEdit(tabDocument);
        descriptionTextEdit->setObjectName(QString::fromUtf8("descriptionTextEdit"));
        descriptionTextEdit->setTabChangesFocus(true);

        formLayout->setWidget(4, QFormLayout::FieldRole, descriptionTextEdit);

        tabWidget->addTab(tabDocument, QString());
        tabFurtherInformation = new QWidget();
        tabFurtherInformation->setObjectName(QString::fromUtf8("tabFurtherInformation"));
        verticalLayout_3 = new QVBoxLayout(tabFurtherInformation);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        scrollArea = new QScrollArea(tabFurtherInformation);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 618, 567));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName(QString::fromUtf8("formLayout_2"));
        label_7 = new QLabel(scrollAreaWidgetContents);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        formLayout_2->setWidget(0, QFormLayout::LabelRole, label_7);

        publisherLineEdit = new QLineEdit(scrollAreaWidgetContents);
        publisherLineEdit->setObjectName(QString::fromUtf8("publisherLineEdit"));

        formLayout_2->setWidget(0, QFormLayout::FieldRole, publisherLineEdit);

        label_8 = new QLabel(scrollAreaWidgetContents);
        label_8->setObjectName(QString::fromUtf8("label_8"));

        formLayout_2->setWidget(1, QFormLayout::LabelRole, label_8);

        contributorsTextEdit = new QTextEdit(scrollAreaWidgetContents);
        contributorsTextEdit->setObjectName(QString::fromUtf8("contributorsTextEdit"));
        contributorsTextEdit->setTabChangesFocus(true);

        formLayout_2->setWidget(1, QFormLayout::FieldRole, contributorsTextEdit);

        label_9 = new QLabel(scrollAreaWidgetContents);
        label_9->setObjectName(QString::fromUtf8("label_9"));

        formLayout_2->setWidget(2, QFormLayout::LabelRole, label_9);

        dateLineEdit = new QComboBox(scrollAreaWidgetContents);
        dateLineEdit->setObjectName(QString::fromUtf8("dateLineEdit"));
        dateLineEdit->setEditable(true);

        formLayout_2->setWidget(2, QFormLayout::FieldRole, dateLineEdit);

        label_10 = new QLabel(scrollAreaWidgetContents);
        label_10->setObjectName(QString::fromUtf8("label_10"));

        formLayout_2->setWidget(3, QFormLayout::LabelRole, label_10);

        typeLineEdit = new QComboBox(scrollAreaWidgetContents);
        typeLineEdit->setObjectName(QString::fromUtf8("typeLineEdit"));
        typeLineEdit->setEditable(true);

        formLayout_2->setWidget(3, QFormLayout::FieldRole, typeLineEdit);

        label_11 = new QLabel(scrollAreaWidgetContents);
        label_11->setObjectName(QString::fromUtf8("label_11"));

        formLayout_2->setWidget(4, QFormLayout::LabelRole, label_11);

        formatLineEdit = new QLineEdit(scrollAreaWidgetContents);
        formatLineEdit->setObjectName(QString::fromUtf8("formatLineEdit"));

        formLayout_2->setWidget(4, QFormLayout::FieldRole, formatLineEdit);

        label_12 = new QLabel(scrollAreaWidgetContents);
        label_12->setObjectName(QString::fromUtf8("label_12"));

        formLayout_2->setWidget(5, QFormLayout::LabelRole, label_12);

        identifierLineEdit = new QLineEdit(scrollAreaWidgetContents);
        identifierLineEdit->setObjectName(QString::fromUtf8("identifierLineEdit"));

        formLayout_2->setWidget(5, QFormLayout::FieldRole, identifierLineEdit);

        label_13 = new QLabel(scrollAreaWidgetContents);
        label_13->setObjectName(QString::fromUtf8("label_13"));

        formLayout_2->setWidget(6, QFormLayout::LabelRole, label_13);

        sourceLineEdit = new QLineEdit(scrollAreaWidgetContents);
        sourceLineEdit->setObjectName(QString::fromUtf8("sourceLineEdit"));

        formLayout_2->setWidget(6, QFormLayout::FieldRole, sourceLineEdit);

        label_14 = new QLabel(scrollAreaWidgetContents);
        label_14->setObjectName(QString::fromUtf8("label_14"));

        formLayout_2->setWidget(7, QFormLayout::LabelRole, label_14);

        languageLineEdit = new QComboBox(scrollAreaWidgetContents);
        languageLineEdit->setObjectName(QString::fromUtf8("languageLineEdit"));
        languageLineEdit->setEditable(true);

        formLayout_2->setWidget(7, QFormLayout::FieldRole, languageLineEdit);

        label_15 = new QLabel(scrollAreaWidgetContents);
        label_15->setObjectName(QString::fromUtf8("label_15"));

        formLayout_2->setWidget(8, QFormLayout::LabelRole, label_15);

        relationLineEdit = new QLineEdit(scrollAreaWidgetContents);
        relationLineEdit->setObjectName(QString::fromUtf8("relationLineEdit"));

        formLayout_2->setWidget(8, QFormLayout::FieldRole, relationLineEdit);

        label_16 = new QLabel(scrollAreaWidgetContents);
        label_16->setObjectName(QString::fromUtf8("label_16"));

        formLayout_2->setWidget(9, QFormLayout::LabelRole, label_16);

        coverageLineEdit = new QComboBox(scrollAreaWidgetContents);
        coverageLineEdit->setObjectName(QString::fromUtf8("coverageLineEdit"));
        coverageLineEdit->setEditable(true);

        formLayout_2->setWidget(9, QFormLayout::FieldRole, coverageLineEdit);

        label_17 = new QLabel(scrollAreaWidgetContents);
        label_17->setObjectName(QString::fromUtf8("label_17"));

        formLayout_2->setWidget(10, QFormLayout::LabelRole, label_17);

        rightsLineEdit = new QComboBox(scrollAreaWidgetContents);
        rightsLineEdit->setObjectName(QString::fromUtf8("rightsLineEdit"));
        rightsLineEdit->setEditable(true);

        formLayout_2->setWidget(10, QFormLayout::FieldRole, rightsLineEdit);


        verticalLayout_2->addLayout(formLayout_2);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_3->addWidget(scrollArea);

        tabWidget->addTab(tabFurtherInformation, QString());

        verticalLayout->addWidget(tabWidget);


        retranslateUi(Prefs_DocumentInformation);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Prefs_DocumentInformation);
    } // setupUi

    void retranslateUi(QWidget *Prefs_DocumentInformation)
    {
        label->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Document Information", nullptr));
        label_2->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Title:", nullptr));
        label_3->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Author:", nullptr));
        label_4->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Subject:", nullptr));
        label_5->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Keywords:", nullptr));
        label_6->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Description:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabDocument), QCoreApplication::translate("Prefs_DocumentInformation", "Document", nullptr));
        label_7->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Publisher:", nullptr));
        label_8->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Contributors:", nullptr));
        label_9->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Date:", nullptr));
        label_10->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Type:", nullptr));
        label_11->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Format:", nullptr));
        label_12->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Identifier:", nullptr));
        label_13->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Source:", nullptr));
        label_14->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Language:", nullptr));
        label_15->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Relation:", nullptr));
        label_16->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Coverage:", nullptr));
        label_17->setText(QCoreApplication::translate("Prefs_DocumentInformation", "Rights:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabFurtherInformation), QCoreApplication::translate("Prefs_DocumentInformation", "Further Information", nullptr));
        (void)Prefs_DocumentInformation;
    } // retranslateUi

};

namespace Ui {
    class Prefs_DocumentInformation: public Ui_Prefs_DocumentInformation {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_DOCUMENTINFORMATIONBASE_H
