/*
For general Scribus (>=1.3.2) copyright and licensing information please refer
to the COPYING file provided with the program. Following this notice may exist
a copyright and/or license notice that predates the release of Scribus 1.3.2
for which a new license (GPL+exception) is in place.
*/

#ifndef CXFUTILS_H
#define CXFUTILS_H

#include <QString>
#include "cxftypes.h"

CxfIlluminant qstrToCxfIlluminant(const QString& str);
CxfObserver   qstrToCxfObserver(const QString& str);
CxfMeasurementType qstrToCxfMeasurementType(const QString& str);
CxfCIEConversionMethod qstrToCxfCIEConversionMethod(const QString& str);
CxfColorType  qstrToCxfColorType(const QString& str);

#endif