/****************************************************************************
** Meta object code from reading C++ file 'annot.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/annot.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'annot.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ScAnnot_t {
    QByteArrayData data[41];
    char stringdata0[379];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ScAnnot_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ScAnnot_t qt_meta_stringdata_ScAnnot = {
    {
QT_MOC_LITERAL(0, 0, 7), // "ScAnnot"
QT_MOC_LITERAL(1, 8, 7), // "NewName"
QT_MOC_LITERAL(2, 16, 0), // ""
QT_MOC_LITERAL(3, 17, 6), // "IPlace"
QT_MOC_LITERAL(4, 24, 11), // "RemoveNIcon"
QT_MOC_LITERAL(5, 36, 11), // "RemovePIcon"
QT_MOC_LITERAL(6, 48, 11), // "RemoveRIcon"
QT_MOC_LITERAL(7, 60, 8), // "IconsEin"
QT_MOC_LITERAL(8, 69, 8), // "GetNIcon"
QT_MOC_LITERAL(9, 78, 8), // "GetPIcon"
QT_MOC_LITERAL(10, 87, 8), // "GetRIcon"
QT_MOC_LITERAL(11, 96, 12), // "SelectFelder"
QT_MOC_LITERAL(12, 109, 9), // "editKeySc"
QT_MOC_LITERAL(13, 119, 12), // "editFormatSc"
QT_MOC_LITERAL(14, 132, 11), // "editValidSc"
QT_MOC_LITERAL(15, 144, 10), // "editCalcSc"
QT_MOC_LITERAL(16, 155, 10), // "editJavaSc"
QT_MOC_LITERAL(17, 166, 10), // "SetFormNum"
QT_MOC_LITERAL(18, 177, 7), // "SetCurr"
QT_MOC_LITERAL(19, 185, 10), // "HandleVali"
QT_MOC_LITERAL(20, 196, 7), // "SetVali"
QT_MOC_LITERAL(21, 204, 10), // "HandleCalc"
QT_MOC_LITERAL(22, 215, 7), // "SetCalc"
QT_MOC_LITERAL(23, 223, 11), // "SetFoScript"
QT_MOC_LITERAL(24, 235, 2), // "it"
QT_MOC_LITERAL(25, 238, 9), // "SetCoords"
QT_MOC_LITERAL(26, 248, 1), // "x"
QT_MOC_LITERAL(27, 250, 1), // "y"
QT_MOC_LITERAL(28, 252, 7), // "SetPage"
QT_MOC_LITERAL(29, 260, 1), // "v"
QT_MOC_LITERAL(30, 262, 8), // "SetCross"
QT_MOC_LITERAL(31, 271, 9), // "SetValues"
QT_MOC_LITERAL(32, 281, 17), // "SetAnnotationType"
QT_MOC_LITERAL(33, 299, 1), // "i"
QT_MOC_LITERAL(34, 301, 7), // "GetFile"
QT_MOC_LITERAL(35, 309, 13), // "SetActionType"
QT_MOC_LITERAL(36, 323, 8), // "SetLimit"
QT_MOC_LITERAL(37, 332, 15), // "SetActionScript"
QT_MOC_LITERAL(38, 348, 13), // "setDateSample"
QT_MOC_LITERAL(39, 362, 2), // "ds"
QT_MOC_LITERAL(40, 365, 13) // "SetExternLink"

    },
    "ScAnnot\0NewName\0\0IPlace\0RemoveNIcon\0"
    "RemovePIcon\0RemoveRIcon\0IconsEin\0"
    "GetNIcon\0GetPIcon\0GetRIcon\0SelectFelder\0"
    "editKeySc\0editFormatSc\0editValidSc\0"
    "editCalcSc\0editJavaSc\0SetFormNum\0"
    "SetCurr\0HandleVali\0SetVali\0HandleCalc\0"
    "SetCalc\0SetFoScript\0it\0SetCoords\0x\0y\0"
    "SetPage\0v\0SetCross\0SetValues\0"
    "SetAnnotationType\0i\0GetFile\0SetActionType\0"
    "SetLimit\0SetActionScript\0setDateSample\0"
    "ds\0SetExternLink"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ScAnnot[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      33,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  179,    2, 0x0a /* Public */,
       3,    0,  180,    2, 0x0a /* Public */,
       4,    0,  181,    2, 0x0a /* Public */,
       5,    0,  182,    2, 0x0a /* Public */,
       6,    0,  183,    2, 0x0a /* Public */,
       7,    0,  184,    2, 0x0a /* Public */,
       8,    0,  185,    2, 0x0a /* Public */,
       9,    0,  186,    2, 0x0a /* Public */,
      10,    0,  187,    2, 0x0a /* Public */,
      11,    0,  188,    2, 0x0a /* Public */,
      12,    0,  189,    2, 0x0a /* Public */,
      13,    0,  190,    2, 0x0a /* Public */,
      14,    0,  191,    2, 0x0a /* Public */,
      15,    0,  192,    2, 0x0a /* Public */,
      16,    0,  193,    2, 0x0a /* Public */,
      17,    0,  194,    2, 0x0a /* Public */,
      18,    0,  195,    2, 0x0a /* Public */,
      19,    0,  196,    2, 0x0a /* Public */,
      20,    0,  197,    2, 0x0a /* Public */,
      21,    0,  198,    2, 0x0a /* Public */,
      22,    0,  199,    2, 0x0a /* Public */,
      23,    1,  200,    2, 0x0a /* Public */,
      25,    2,  203,    2, 0x0a /* Public */,
      28,    1,  208,    2, 0x0a /* Public */,
      30,    0,  211,    2, 0x0a /* Public */,
      31,    0,  212,    2, 0x0a /* Public */,
      32,    1,  213,    2, 0x0a /* Public */,
      34,    0,  216,    2, 0x0a /* Public */,
      35,    1,  217,    2, 0x0a /* Public */,
      36,    0,  220,    2, 0x0a /* Public */,
      37,    1,  221,    2, 0x0a /* Public */,
      38,    1,  224,    2, 0x0a /* Public */,
      40,    0,  227,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   24,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,   26,   27,
    QMetaType::Void, QMetaType::Int,   29,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   33,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   24,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   24,
    QMetaType::Void, QMetaType::QString,   39,
    QMetaType::Void,

       0        // eod
};

void ScAnnot::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ScAnnot *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->NewName(); break;
        case 1: _t->IPlace(); break;
        case 2: _t->RemoveNIcon(); break;
        case 3: _t->RemovePIcon(); break;
        case 4: _t->RemoveRIcon(); break;
        case 5: _t->IconsEin(); break;
        case 6: _t->GetNIcon(); break;
        case 7: _t->GetPIcon(); break;
        case 8: _t->GetRIcon(); break;
        case 9: _t->SelectFelder(); break;
        case 10: _t->editKeySc(); break;
        case 11: _t->editFormatSc(); break;
        case 12: _t->editValidSc(); break;
        case 13: _t->editCalcSc(); break;
        case 14: _t->editJavaSc(); break;
        case 15: _t->SetFormNum(); break;
        case 16: _t->SetCurr(); break;
        case 17: _t->HandleVali(); break;
        case 18: _t->SetVali(); break;
        case 19: _t->HandleCalc(); break;
        case 20: _t->SetCalc(); break;
        case 21: _t->SetFoScript((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 22: _t->SetCoords((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 23: _t->SetPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 24: _t->SetCross(); break;
        case 25: _t->SetValues(); break;
        case 26: _t->SetAnnotationType((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 27: _t->GetFile(); break;
        case 28: _t->SetActionType((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 29: _t->SetLimit(); break;
        case 30: _t->SetActionScript((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 31: _t->setDateSample((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 32: _t->SetExternLink(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ScAnnot::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_ScAnnot.data,
    qt_meta_data_ScAnnot,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ScAnnot::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ScAnnot::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ScAnnot.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::ScAnnot"))
        return static_cast< Ui::ScAnnot*>(this);
    return QDialog::qt_metacast(_clname);
}

int ScAnnot::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 33)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 33;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 33)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 33;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
