/****************************************************************************
** Meta object code from reading C++ file 'prefs_pdfexport.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/prefs_pdfexport.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'prefs_pdfexport.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_Prefs_PDFExport_t {
    QByteArrayData data[33];
    char stringdata0[407];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Prefs_PDFExport_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Prefs_PDFExport_t qt_meta_stringdata_Prefs_PDFExport = {
    {
QT_MOC_LITERAL(0, 0, 15), // "Prefs_PDFExport"
QT_MOC_LITERAL(1, 16, 6), // "noInfo"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 7), // "hasInfo"
QT_MOC_LITERAL(4, 32, 14), // "languageChange"
QT_MOC_LITERAL(5, 47, 10), // "unitChange"
QT_MOC_LITERAL(6, 58, 19), // "enableRangeControls"
QT_MOC_LITERAL(7, 78, 22), // "enableSecurityControls"
QT_MOC_LITERAL(8, 101, 19), // "enablePDFExportTabs"
QT_MOC_LITERAL(9, 121, 21), // "createPageNumberRange"
QT_MOC_LITERAL(10, 143, 20), // "setMaximumResolution"
QT_MOC_LITERAL(11, 164, 14), // "enableProfiles"
QT_MOC_LITERAL(12, 179, 9), // "enableLPI"
QT_MOC_LITERAL(13, 189, 10), // "enableLPI2"
QT_MOC_LITERAL(14, 200, 8), // "enablePG"
QT_MOC_LITERAL(15, 209, 9), // "enablePGI"
QT_MOC_LITERAL(16, 219, 10), // "enablePGI2"
QT_MOC_LITERAL(17, 230, 10), // "enablePDFX"
QT_MOC_LITERAL(18, 241, 9), // "SelLPIcol"
QT_MOC_LITERAL(19, 251, 12), // "DoDownsample"
QT_MOC_LITERAL(20, 264, 19), // "EmbeddingModeChange"
QT_MOC_LITERAL(21, 284, 12), // "RemoveSubset"
QT_MOC_LITERAL(22, 297, 11), // "PutToSubset"
QT_MOC_LITERAL(23, 309, 8), // "SelEFont"
QT_MOC_LITERAL(24, 318, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(25, 335, 8), // "SelSFont"
QT_MOC_LITERAL(26, 344, 8), // "EmbedAll"
QT_MOC_LITERAL(27, 353, 9), // "SubsetAll"
QT_MOC_LITERAL(28, 363, 10), // "OutlineAll"
QT_MOC_LITERAL(29, 374, 6), // "PagePr"
QT_MOC_LITERAL(30, 381, 11), // "doDocBleeds"
QT_MOC_LITERAL(31, 393, 10), // "SetEffOpts"
QT_MOC_LITERAL(32, 404, 2) // "nr"

    },
    "Prefs_PDFExport\0noInfo\0\0hasInfo\0"
    "languageChange\0unitChange\0enableRangeControls\0"
    "enableSecurityControls\0enablePDFExportTabs\0"
    "createPageNumberRange\0setMaximumResolution\0"
    "enableProfiles\0enableLPI\0enableLPI2\0"
    "enablePG\0enablePGI\0enablePGI2\0enablePDFX\0"
    "SelLPIcol\0DoDownsample\0EmbeddingModeChange\0"
    "RemoveSubset\0PutToSubset\0SelEFont\0"
    "QListWidgetItem*\0SelSFont\0EmbedAll\0"
    "SubsetAll\0OutlineAll\0PagePr\0doDocBleeds\0"
    "SetEffOpts\0nr"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Prefs_PDFExport[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      29,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  159,    2, 0x06 /* Public */,
       3,    0,  160,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,  161,    2, 0x0a /* Public */,
       5,    1,  162,    2, 0x0a /* Public */,
       6,    1,  165,    2, 0x09 /* Protected */,
       7,    1,  168,    2, 0x09 /* Protected */,
       8,    1,  171,    2, 0x09 /* Protected */,
       9,    0,  174,    2, 0x09 /* Protected */,
      10,    0,  175,    2, 0x09 /* Protected */,
      11,    1,  176,    2, 0x09 /* Protected */,
      12,    1,  179,    2, 0x09 /* Protected */,
      13,    0,  182,    2, 0x09 /* Protected */,
      14,    0,  183,    2, 0x09 /* Protected */,
      15,    0,  184,    2, 0x09 /* Protected */,
      16,    0,  185,    2, 0x09 /* Protected */,
      17,    1,  186,    2, 0x09 /* Protected */,
      18,    1,  189,    2, 0x09 /* Protected */,
      19,    0,  192,    2, 0x09 /* Protected */,
      20,    0,  193,    2, 0x09 /* Protected */,
      21,    0,  194,    2, 0x09 /* Protected */,
      22,    0,  195,    2, 0x09 /* Protected */,
      23,    1,  196,    2, 0x09 /* Protected */,
      25,    1,  199,    2, 0x09 /* Protected */,
      26,    0,  202,    2, 0x09 /* Protected */,
      27,    0,  203,    2, 0x09 /* Protected */,
      28,    0,  204,    2, 0x09 /* Protected */,
      29,    0,  205,    2, 0x09 /* Protected */,
      30,    0,  206,    2, 0x09 /* Protected */,
      31,    1,  207,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 24,    2,
    QMetaType::Void, 0x80000000 | 24,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   32,

       0        // eod
};

void Prefs_PDFExport::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Prefs_PDFExport *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->noInfo(); break;
        case 1: _t->hasInfo(); break;
        case 2: _t->languageChange(); break;
        case 3: _t->unitChange((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->enableRangeControls((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->enableSecurityControls((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->enablePDFExportTabs((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 7: _t->createPageNumberRange(); break;
        case 8: _t->setMaximumResolution(); break;
        case 9: _t->enableProfiles((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->enableLPI((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->enableLPI2(); break;
        case 12: _t->enablePG(); break;
        case 13: _t->enablePGI(); break;
        case 14: _t->enablePGI2(); break;
        case 15: _t->enablePDFX((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->SelLPIcol((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->DoDownsample(); break;
        case 18: _t->EmbeddingModeChange(); break;
        case 19: _t->RemoveSubset(); break;
        case 20: _t->PutToSubset(); break;
        case 21: _t->SelEFont((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 22: _t->SelSFont((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 23: _t->EmbedAll(); break;
        case 24: _t->SubsetAll(); break;
        case 25: _t->OutlineAll(); break;
        case 26: _t->PagePr(); break;
        case 27: _t->doDocBleeds(); break;
        case 28: _t->SetEffOpts((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Prefs_PDFExport::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Prefs_PDFExport::noInfo)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Prefs_PDFExport::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Prefs_PDFExport::hasInfo)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Prefs_PDFExport::staticMetaObject = { {
    QMetaObject::SuperData::link<Prefs_Pane::staticMetaObject>(),
    qt_meta_stringdata_Prefs_PDFExport.data,
    qt_meta_data_Prefs_PDFExport,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Prefs_PDFExport::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Prefs_PDFExport::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Prefs_PDFExport.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::Prefs_PDFExport"))
        return static_cast< Ui::Prefs_PDFExport*>(this);
    return Prefs_Pane::qt_metacast(_clname);
}

int Prefs_PDFExport::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = Prefs_Pane::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 29)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 29;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 29)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 29;
    }
    return _id;
}

// SIGNAL 0
void Prefs_PDFExport::noInfo()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Prefs_PDFExport::hasInfo()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
