/********************************************************************************
** Form generated from reading UI file 'smtablestylewidget.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SMTABLESTYLEWIDGET_H
#define UI_SMTABLESTYLEWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/smcolorcombo.h"
#include "ui/smshadebutton.h"

QT_BEGIN_NAMESPACE

class Ui_SMTableStyleWidget
{
public:
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QLabel *parentLabel;
    QComboBox *parentCombo;
    QGroupBox *smColorGroup;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *fillColorIcon;
    SMColorCombo *fillColor;
    QLabel *fillShadeLabel;
    SMShadeButton *fillShade;
    QSpacerItem *spacerItem;
    QSpacerItem *spacerItem1;

    void setupUi(QWidget *SMTableStyleWidget)
    {
        if (SMTableStyleWidget->objectName().isEmpty())
            SMTableStyleWidget->setObjectName(QString::fromUtf8("SMTableStyleWidget"));
        SMTableStyleWidget->resize(378, 486);
        verticalLayout_2 = new QVBoxLayout(SMTableStyleWidget);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        parentLabel = new QLabel(SMTableStyleWidget);
        parentLabel->setObjectName(QString::fromUtf8("parentLabel"));

        horizontalLayout_2->addWidget(parentLabel);

        parentCombo = new QComboBox(SMTableStyleWidget);
        parentCombo->setObjectName(QString::fromUtf8("parentCombo"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(parentCombo->sizePolicy().hasHeightForWidth());
        parentCombo->setSizePolicy(sizePolicy);

        horizontalLayout_2->addWidget(parentCombo);


        verticalLayout_2->addLayout(horizontalLayout_2);

        smColorGroup = new QGroupBox(SMTableStyleWidget);
        smColorGroup->setObjectName(QString::fromUtf8("smColorGroup"));
        verticalLayout = new QVBoxLayout(smColorGroup);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        fillColorIcon = new QLabel(smColorGroup);
        fillColorIcon->setObjectName(QString::fromUtf8("fillColorIcon"));
        fillColorIcon->setMaximumSize(QSize(22, 22));

        horizontalLayout->addWidget(fillColorIcon);

        fillColor = new SMColorCombo(smColorGroup);
        fillColor->setObjectName(QString::fromUtf8("fillColor"));

        horizontalLayout->addWidget(fillColor);

        fillShadeLabel = new QLabel(smColorGroup);
        fillShadeLabel->setObjectName(QString::fromUtf8("fillShadeLabel"));

        horizontalLayout->addWidget(fillShadeLabel);

        fillShade = new SMShadeButton(smColorGroup);
        fillShade->setObjectName(QString::fromUtf8("fillShade"));

        horizontalLayout->addWidget(fillShade);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(spacerItem);


        verticalLayout->addLayout(horizontalLayout);


        verticalLayout_2->addWidget(smColorGroup);

        spacerItem1 = new QSpacerItem(72, 356, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

        verticalLayout_2->addItem(spacerItem1);

#if QT_CONFIG(shortcut)
        fillColorIcon->setBuddy(fillColor);
#endif // QT_CONFIG(shortcut)

        retranslateUi(SMTableStyleWidget);

        QMetaObject::connectSlotsByName(SMTableStyleWidget);
    } // setupUi

    void retranslateUi(QWidget *SMTableStyleWidget)
    {
        parentLabel->setText(QCoreApplication::translate("SMTableStyleWidget", "Based On:", nullptr));
        smColorGroup->setTitle(QCoreApplication::translate("SMTableStyleWidget", "Colors", nullptr));
        fillColorIcon->setText(QString());
#if QT_CONFIG(tooltip)
        fillColor->setToolTip(QCoreApplication::translate("SMTableStyleWidget", "Fill Color", nullptr));
#endif // QT_CONFIG(tooltip)
        fillShadeLabel->setText(QString());
#if QT_CONFIG(tooltip)
        fillShade->setToolTip(QCoreApplication::translate("SMTableStyleWidget", "Fill Shade", nullptr));
#endif // QT_CONFIG(tooltip)
        fillShade->setText(QString());
        (void)SMTableStyleWidget;
    } // retranslateUi

};

namespace Ui {
    class SMTableStyleWidget: public Ui_SMTableStyleWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SMTABLESTYLEWIDGET_H
