/********************************************************************************
** Form generated from reading UI file 'resourcemanagerbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RESOURCEMANAGERBASE_H
#define UI_RESOURCEMANAGERBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ResourceManagerBase
{
public:
    QVBoxLayout *verticalLayout_5;
    QHBoxLayout *horizontalLayout;
    QLabel *categoryLabel;
    QComboBox *categoryComboBox;
    QSpacerItem *horizontalSpacer;
    QGridLayout *gridLayout;
    QVBoxLayout *verticalLayout_3;
    QLabel *label_2;
    QTableWidget *installedTableWidget;
    QVBoxLayout *verticalLayout;
    QSpacerItem *verticalSpacer_4;
    QPushButton *showInFSButton;
    QPushButton *deleteSelectedButton;
    QSpacerItem *verticalSpacer;
    QVBoxLayout *verticalLayout_4;
    QLabel *label_3;
    QTableWidget *availableTableWidget;
    QVBoxLayout *verticalLayout_2;
    QSpacerItem *verticalSpacer_3;
    QPushButton *updateAvailableButton;
    QPushButton *showLicenseButton;
    QPushButton *downloadButton;
    QSpacerItem *verticalSpacer_2;
    QProgressBar *downloadProgressBar;
    QLabel *dataReceivedLabel;

    void setupUi(QDialog *ResourceManagerBase)
    {
        if (ResourceManagerBase->objectName().isEmpty())
            ResourceManagerBase->setObjectName(QString::fromUtf8("ResourceManagerBase"));
        ResourceManagerBase->resize(785, 716);
        verticalLayout_5 = new QVBoxLayout(ResourceManagerBase);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        verticalLayout_5->setContentsMargins(9, 9, 9, 9);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        categoryLabel = new QLabel(ResourceManagerBase);
        categoryLabel->setObjectName(QString::fromUtf8("categoryLabel"));

        horizontalLayout->addWidget(categoryLabel);

        categoryComboBox = new QComboBox(ResourceManagerBase);
        categoryComboBox->setObjectName(QString::fromUtf8("categoryComboBox"));

        horizontalLayout->addWidget(categoryComboBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_5->addLayout(horizontalLayout);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        verticalLayout_3 = new QVBoxLayout();
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setSizeConstraint(QLayout::SetDefaultConstraint);
        label_2 = new QLabel(ResourceManagerBase);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        QFont font;
        font.setBold(false);
        font.setWeight(50);
        label_2->setFont(font);
        label_2->setTextFormat(Qt::AutoText);

        verticalLayout_3->addWidget(label_2);

        installedTableWidget = new QTableWidget(ResourceManagerBase);
        if (installedTableWidget->columnCount() < 3)
            installedTableWidget->setColumnCount(3);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        installedTableWidget->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        installedTableWidget->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        installedTableWidget->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        installedTableWidget->setObjectName(QString::fromUtf8("installedTableWidget"));
        installedTableWidget->setAlternatingRowColors(true);
        installedTableWidget->setSortingEnabled(true);
        installedTableWidget->horizontalHeader()->setStretchLastSection(true);
        installedTableWidget->verticalHeader()->setVisible(false);

        verticalLayout_3->addWidget(installedTableWidget);


        gridLayout->addLayout(verticalLayout_3, 0, 0, 1, 1);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalSpacer_4 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer_4);

        showInFSButton = new QPushButton(ResourceManagerBase);
        showInFSButton->setObjectName(QString::fromUtf8("showInFSButton"));
        showInFSButton->setAutoDefault(false);

        verticalLayout->addWidget(showInFSButton);

        deleteSelectedButton = new QPushButton(ResourceManagerBase);
        deleteSelectedButton->setObjectName(QString::fromUtf8("deleteSelectedButton"));
        deleteSelectedButton->setEnabled(false);
        deleteSelectedButton->setAutoDefault(false);

        verticalLayout->addWidget(deleteSelectedButton);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        gridLayout->addLayout(verticalLayout, 0, 1, 1, 1);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        label_3 = new QLabel(ResourceManagerBase);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        verticalLayout_4->addWidget(label_3);

        availableTableWidget = new QTableWidget(ResourceManagerBase);
        if (availableTableWidget->columnCount() < 4)
            availableTableWidget->setColumnCount(4);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        availableTableWidget->setHorizontalHeaderItem(0, __qtablewidgetitem3);
        QTableWidgetItem *__qtablewidgetitem4 = new QTableWidgetItem();
        availableTableWidget->setHorizontalHeaderItem(1, __qtablewidgetitem4);
        QTableWidgetItem *__qtablewidgetitem5 = new QTableWidgetItem();
        availableTableWidget->setHorizontalHeaderItem(2, __qtablewidgetitem5);
        QTableWidgetItem *__qtablewidgetitem6 = new QTableWidgetItem();
        availableTableWidget->setHorizontalHeaderItem(3, __qtablewidgetitem6);
        availableTableWidget->setObjectName(QString::fromUtf8("availableTableWidget"));
        availableTableWidget->setAlternatingRowColors(true);
        availableTableWidget->setSortingEnabled(true);
        availableTableWidget->setRowCount(0);
        availableTableWidget->setColumnCount(4);
        availableTableWidget->horizontalHeader()->setStretchLastSection(true);
        availableTableWidget->verticalHeader()->setVisible(false);

        verticalLayout_4->addWidget(availableTableWidget);


        gridLayout->addLayout(verticalLayout_4, 1, 0, 1, 1);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_3);

        updateAvailableButton = new QPushButton(ResourceManagerBase);
        updateAvailableButton->setObjectName(QString::fromUtf8("updateAvailableButton"));
        updateAvailableButton->setAutoDefault(false);

        verticalLayout_2->addWidget(updateAvailableButton);

        showLicenseButton = new QPushButton(ResourceManagerBase);
        showLicenseButton->setObjectName(QString::fromUtf8("showLicenseButton"));
        showLicenseButton->setAutoDefault(false);

        verticalLayout_2->addWidget(showLicenseButton);

        downloadButton = new QPushButton(ResourceManagerBase);
        downloadButton->setObjectName(QString::fromUtf8("downloadButton"));
        downloadButton->setEnabled(false);
        downloadButton->setAutoDefault(false);

        verticalLayout_2->addWidget(downloadButton);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_2);

        downloadProgressBar = new QProgressBar(ResourceManagerBase);
        downloadProgressBar->setObjectName(QString::fromUtf8("downloadProgressBar"));
        downloadProgressBar->setValue(24);

        verticalLayout_2->addWidget(downloadProgressBar);

        dataReceivedLabel = new QLabel(ResourceManagerBase);
        dataReceivedLabel->setObjectName(QString::fromUtf8("dataReceivedLabel"));
        dataReceivedLabel->setText(QString::fromUtf8(""));
        dataReceivedLabel->setAlignment(Qt::AlignCenter);

        verticalLayout_2->addWidget(dataReceivedLabel);


        gridLayout->addLayout(verticalLayout_2, 1, 1, 1, 1);

        gridLayout->setColumnStretch(0, 1);

        verticalLayout_5->addLayout(gridLayout);


        retranslateUi(ResourceManagerBase);

        QMetaObject::connectSlotsByName(ResourceManagerBase);
    } // setupUi

    void retranslateUi(QDialog *ResourceManagerBase)
    {
        ResourceManagerBase->setWindowTitle(QCoreApplication::translate("ResourceManagerBase", "Resource Manager", nullptr));
        categoryLabel->setText(QCoreApplication::translate("ResourceManagerBase", "Category:", nullptr));
        label_2->setText(QCoreApplication::translate("ResourceManagerBase", "Installed", nullptr));
        QTableWidgetItem *___qtablewidgetitem = installedTableWidget->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("ResourceManagerBase", "Description", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = installedTableWidget->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("ResourceManagerBase", "ID", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = installedTableWidget->horizontalHeaderItem(2);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("ResourceManagerBase", "Location", nullptr));
        showInFSButton->setText(QCoreApplication::translate("ResourceManagerBase", "Show in Filesystem", nullptr));
        deleteSelectedButton->setText(QCoreApplication::translate("ResourceManagerBase", "Delete Selected", nullptr));
        label_3->setText(QCoreApplication::translate("ResourceManagerBase", "Available", nullptr));
        QTableWidgetItem *___qtablewidgetitem3 = availableTableWidget->horizontalHeaderItem(0);
        ___qtablewidgetitem3->setText(QCoreApplication::translate("ResourceManagerBase", "Description", nullptr));
        QTableWidgetItem *___qtablewidgetitem4 = availableTableWidget->horizontalHeaderItem(1);
        ___qtablewidgetitem4->setText(QCoreApplication::translate("ResourceManagerBase", "ID", nullptr));
        QTableWidgetItem *___qtablewidgetitem5 = availableTableWidget->horizontalHeaderItem(2);
        ___qtablewidgetitem5->setText(QCoreApplication::translate("ResourceManagerBase", "Location", nullptr));
        QTableWidgetItem *___qtablewidgetitem6 = availableTableWidget->horizontalHeaderItem(3);
        ___qtablewidgetitem6->setText(QCoreApplication::translate("ResourceManagerBase", "Download", nullptr));
        updateAvailableButton->setText(QCoreApplication::translate("ResourceManagerBase", "Update Available List", nullptr));
        showLicenseButton->setText(QCoreApplication::translate("ResourceManagerBase", "Show License", nullptr));
        downloadButton->setText(QCoreApplication::translate("ResourceManagerBase", "Download", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ResourceManagerBase: public Ui_ResourceManagerBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RESOURCEMANAGERBASE_H
