/****************************************************************************
** Meta object code from reading C++ file 'propertywidget_pareffect.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/propertywidget_pareffect.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertywidget_pareffect.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PropertyWidget_ParEffect_t {
    QByteArrayData data[35];
    char stringdata0[507];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PropertyWidget_ParEffect_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PropertyWidget_ParEffect_t qt_meta_stringdata_PropertyWidget_ParEffect = {
    {
QT_MOC_LITERAL(0, 0, 24), // "PropertyWidget_ParEffect"
QT_MOC_LITERAL(1, 25, 13), // "needsRelayout"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 13), // "setMainWindow"
QT_MOC_LITERAL(4, 54, 18), // "ScribusMainWindow*"
QT_MOC_LITERAL(5, 73, 2), // "mw"
QT_MOC_LITERAL(6, 76, 6), // "setDoc"
QT_MOC_LITERAL(7, 83, 11), // "ScribusDoc*"
QT_MOC_LITERAL(8, 95, 3), // "doc"
QT_MOC_LITERAL(9, 99, 20), // "handleAppModeChanged"
QT_MOC_LITERAL(10, 120, 7), // "oldMode"
QT_MOC_LITERAL(11, 128, 4), // "mode"
QT_MOC_LITERAL(12, 133, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(13, 156, 19), // "handleUpdateRequest"
QT_MOC_LITERAL(14, 176, 13), // "iconSetChange"
QT_MOC_LITERAL(15, 190, 14), // "languageChange"
QT_MOC_LITERAL(16, 205, 10), // "unitChange"
QT_MOC_LITERAL(17, 216, 12), // "localeChange"
QT_MOC_LITERAL(18, 229, 18), // "handleParEffectUse"
QT_MOC_LITERAL(19, 248, 15), // "handleBulletStr"
QT_MOC_LITERAL(20, 264, 18), // "handleDropCapLines"
QT_MOC_LITERAL(21, 283, 13), // "handleNumName"
QT_MOC_LITERAL(22, 297, 15), // "handleNumFormat"
QT_MOC_LITERAL(23, 313, 14), // "handleNumLevel"
QT_MOC_LITERAL(24, 328, 15), // "handleNumPrefix"
QT_MOC_LITERAL(25, 344, 15), // "handleNumSuffix"
QT_MOC_LITERAL(26, 360, 14), // "handleNumStart"
QT_MOC_LITERAL(27, 375, 14), // "handlePEOffset"
QT_MOC_LITERAL(28, 390, 14), // "handlePEIndent"
QT_MOC_LITERAL(29, 405, 17), // "handlePECharStyle"
QT_MOC_LITERAL(30, 423, 32), // "on_bulletCharTableButton_toggled"
QT_MOC_LITERAL(31, 456, 7), // "checked"
QT_MOC_LITERAL(32, 464, 18), // "insertSpecialChars"
QT_MOC_LITERAL(33, 483, 13), // "QVector<uint>"
QT_MOC_LITERAL(34, 497, 9) // "charCodes"

    },
    "PropertyWidget_ParEffect\0needsRelayout\0"
    "\0setMainWindow\0ScribusMainWindow*\0mw\0"
    "setDoc\0ScribusDoc*\0doc\0handleAppModeChanged\0"
    "oldMode\0mode\0handleSelectionChanged\0"
    "handleUpdateRequest\0iconSetChange\0"
    "languageChange\0unitChange\0localeChange\0"
    "handleParEffectUse\0handleBulletStr\0"
    "handleDropCapLines\0handleNumName\0"
    "handleNumFormat\0handleNumLevel\0"
    "handleNumPrefix\0handleNumSuffix\0"
    "handleNumStart\0handlePEOffset\0"
    "handlePEIndent\0handlePECharStyle\0"
    "on_bulletCharTableButton_toggled\0"
    "checked\0insertSpecialChars\0QVector<uint>\0"
    "charCodes"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PropertyWidget_ParEffect[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      24,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  134,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,  135,    2, 0x0a /* Public */,
       6,    1,  138,    2, 0x0a /* Public */,
       9,    2,  141,    2, 0x0a /* Public */,
      12,    0,  146,    2, 0x0a /* Public */,
      13,    1,  147,    2, 0x0a /* Public */,
      14,    0,  150,    2, 0x0a /* Public */,
      15,    0,  151,    2, 0x0a /* Public */,
      16,    0,  152,    2, 0x0a /* Public */,
      17,    0,  153,    2, 0x0a /* Public */,
      18,    0,  154,    2, 0x0a /* Public */,
      19,    1,  155,    2, 0x0a /* Public */,
      20,    1,  158,    2, 0x0a /* Public */,
      21,    1,  161,    2, 0x0a /* Public */,
      22,    1,  164,    2, 0x0a /* Public */,
      23,    1,  167,    2, 0x0a /* Public */,
      24,    1,  170,    2, 0x0a /* Public */,
      25,    1,  173,    2, 0x0a /* Public */,
      26,    1,  176,    2, 0x0a /* Public */,
      27,    1,  179,    2, 0x0a /* Public */,
      28,    1,  182,    2, 0x0a /* Public */,
      29,    1,  185,    2, 0x0a /* Public */,
      30,    1,  188,    2, 0x08 /* Private */,
      32,    1,  191,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,   10,   11,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Bool,   31,
    QMetaType::Void, 0x80000000 | 33,   34,

       0        // eod
};

void PropertyWidget_ParEffect::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertyWidget_ParEffect *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->needsRelayout(); break;
        case 1: _t->setMainWindow((*reinterpret_cast< ScribusMainWindow*(*)>(_a[1]))); break;
        case 2: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 3: _t->handleAppModeChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 4: _t->handleSelectionChanged(); break;
        case 5: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->iconSetChange(); break;
        case 7: _t->languageChange(); break;
        case 8: _t->unitChange(); break;
        case 9: _t->localeChange(); break;
        case 10: _t->handleParEffectUse(); break;
        case 11: _t->handleBulletStr((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 12: _t->handleDropCapLines((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->handleNumName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->handleNumFormat((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->handleNumLevel((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->handleNumPrefix((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 17: _t->handleNumSuffix((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 18: _t->handleNumStart((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->handlePEOffset((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 20: _t->handlePEIndent((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->handlePECharStyle((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 22: _t->on_bulletCharTableButton_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 23: _t->insertSpecialChars((*reinterpret_cast< const QVector<uint>(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<uint> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PropertyWidget_ParEffect::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PropertyWidget_ParEffect::needsRelayout)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PropertyWidget_ParEffect::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_PropertyWidget_ParEffect.data,
    qt_meta_data_PropertyWidget_ParEffect,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PropertyWidget_ParEffect::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PropertyWidget_ParEffect::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PropertyWidget_ParEffect.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "PropertyWidgetBase"))
        return static_cast< PropertyWidgetBase*>(this);
    return QFrame::qt_metacast(_clname);
}

int PropertyWidget_ParEffect::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    }
    return _id;
}

// SIGNAL 0
void PropertyWidget_ParEffect::needsRelayout()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
