/********************************************************************************
** Form generated from reading UI file 'prefs_scripterbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_SCRIPTERBASE_H
#define UI_PREFS_SCRIPTERBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Prefs_Scripter
{
public:
    QVBoxLayout *verticalLayout_3;
    QLabel *titleLabel;
    QFrame *line;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout;
    QLabel *extensionsTitleLabel;
    QFrame *line_2;
    QCheckBox *extensionScriptsChk;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_2;
    QLineEdit *startupScriptEdit;
    QPushButton *startupScriptChangeButton;
    QSpacerItem *verticalSpacer_2;
    QLabel *consoleTitleLabel;
    QFrame *line_3;
    QHBoxLayout *horizontalLayout;
    QFormLayout *formLayout;
    QLabel *label_4;
    QPushButton *textButton;
    QLabel *label_5;
    QPushButton *commentButton;
    QLabel *label_6;
    QPushButton *keywordButton;
    QLabel *label_7;
    QPushButton *signButton;
    QLabel *label_8;
    QPushButton *errorButton;
    QLabel *label_9;
    QPushButton *stringButton;
    QLabel *label_10;
    QPushButton *numberButton;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *Prefs_Scripter)
    {
        if (Prefs_Scripter->objectName().isEmpty())
            Prefs_Scripter->setObjectName(QString::fromUtf8("Prefs_Scripter"));
        Prefs_Scripter->resize(770, 608);
        verticalLayout_3 = new QVBoxLayout(Prefs_Scripter);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        titleLabel = new QLabel(Prefs_Scripter);
        titleLabel->setObjectName(QString::fromUtf8("titleLabel"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        titleLabel->setFont(font);

        verticalLayout_3->addWidget(titleLabel);

        line = new QFrame(Prefs_Scripter);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout_3->addWidget(line);

        scrollArea = new QScrollArea(Prefs_Scripter);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 742, 538));
        verticalLayout = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, -1);
        extensionsTitleLabel = new QLabel(scrollAreaWidgetContents);
        extensionsTitleLabel->setObjectName(QString::fromUtf8("extensionsTitleLabel"));
        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
        extensionsTitleLabel->setFont(font1);

        verticalLayout->addWidget(extensionsTitleLabel);

        line_2 = new QFrame(scrollAreaWidgetContents);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line_2);

        extensionScriptsChk = new QCheckBox(scrollAreaWidgetContents);
        extensionScriptsChk->setObjectName(QString::fromUtf8("extensionScriptsChk"));

        verticalLayout->addWidget(extensionScriptsChk);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        label_2 = new QLabel(scrollAreaWidgetContents);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        horizontalLayout_3->addWidget(label_2);

        startupScriptEdit = new QLineEdit(scrollAreaWidgetContents);
        startupScriptEdit->setObjectName(QString::fromUtf8("startupScriptEdit"));

        horizontalLayout_3->addWidget(startupScriptEdit);

        startupScriptChangeButton = new QPushButton(scrollAreaWidgetContents);
        startupScriptChangeButton->setObjectName(QString::fromUtf8("startupScriptChangeButton"));

        horizontalLayout_3->addWidget(startupScriptChangeButton);


        verticalLayout->addLayout(horizontalLayout_3);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout->addItem(verticalSpacer_2);

        consoleTitleLabel = new QLabel(scrollAreaWidgetContents);
        consoleTitleLabel->setObjectName(QString::fromUtf8("consoleTitleLabel"));
        consoleTitleLabel->setFont(font1);

        verticalLayout->addWidget(consoleTitleLabel);

        line_3 = new QFrame(scrollAreaWidgetContents);
        line_3->setObjectName(QString::fromUtf8("line_3"));
        line_3->setFrameShape(QFrame::HLine);
        line_3->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line_3);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_4 = new QLabel(scrollAreaWidgetContents);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        formLayout->setWidget(0, QFormLayout::LabelRole, label_4);

        textButton = new QPushButton(scrollAreaWidgetContents);
        textButton->setObjectName(QString::fromUtf8("textButton"));

        formLayout->setWidget(0, QFormLayout::FieldRole, textButton);

        label_5 = new QLabel(scrollAreaWidgetContents);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        formLayout->setWidget(1, QFormLayout::LabelRole, label_5);

        commentButton = new QPushButton(scrollAreaWidgetContents);
        commentButton->setObjectName(QString::fromUtf8("commentButton"));

        formLayout->setWidget(1, QFormLayout::FieldRole, commentButton);

        label_6 = new QLabel(scrollAreaWidgetContents);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        formLayout->setWidget(2, QFormLayout::LabelRole, label_6);

        keywordButton = new QPushButton(scrollAreaWidgetContents);
        keywordButton->setObjectName(QString::fromUtf8("keywordButton"));

        formLayout->setWidget(2, QFormLayout::FieldRole, keywordButton);

        label_7 = new QLabel(scrollAreaWidgetContents);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        formLayout->setWidget(3, QFormLayout::LabelRole, label_7);

        signButton = new QPushButton(scrollAreaWidgetContents);
        signButton->setObjectName(QString::fromUtf8("signButton"));

        formLayout->setWidget(3, QFormLayout::FieldRole, signButton);

        label_8 = new QLabel(scrollAreaWidgetContents);
        label_8->setObjectName(QString::fromUtf8("label_8"));

        formLayout->setWidget(4, QFormLayout::LabelRole, label_8);

        errorButton = new QPushButton(scrollAreaWidgetContents);
        errorButton->setObjectName(QString::fromUtf8("errorButton"));

        formLayout->setWidget(4, QFormLayout::FieldRole, errorButton);

        label_9 = new QLabel(scrollAreaWidgetContents);
        label_9->setObjectName(QString::fromUtf8("label_9"));

        formLayout->setWidget(5, QFormLayout::LabelRole, label_9);

        stringButton = new QPushButton(scrollAreaWidgetContents);
        stringButton->setObjectName(QString::fromUtf8("stringButton"));

        formLayout->setWidget(5, QFormLayout::FieldRole, stringButton);

        label_10 = new QLabel(scrollAreaWidgetContents);
        label_10->setObjectName(QString::fromUtf8("label_10"));

        formLayout->setWidget(6, QFormLayout::LabelRole, label_10);

        numberButton = new QPushButton(scrollAreaWidgetContents);
        numberButton->setObjectName(QString::fromUtf8("numberButton"));

        formLayout->setWidget(6, QFormLayout::FieldRole, numberButton);


        horizontalLayout->addLayout(formLayout);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addLayout(horizontalLayout);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

        verticalLayout->addItem(verticalSpacer);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_3->addWidget(scrollArea);


        retranslateUi(Prefs_Scripter);

        QMetaObject::connectSlotsByName(Prefs_Scripter);
    } // setupUi

    void retranslateUi(QWidget *Prefs_Scripter)
    {
        titleLabel->setText(QCoreApplication::translate("Prefs_Scripter", "Scripter", nullptr));
        extensionsTitleLabel->setText(QCoreApplication::translate("Prefs_Scripter", "Extensions", nullptr));
        extensionScriptsChk->setText(QCoreApplication::translate("Prefs_Scripter", "Enable Extension Scripts", nullptr));
        label_2->setText(QCoreApplication::translate("Prefs_Scripter", "Startup Script:", nullptr));
        startupScriptChangeButton->setText(QCoreApplication::translate("Prefs_Scripter", "Change...", nullptr));
        consoleTitleLabel->setText(QCoreApplication::translate("Prefs_Scripter", "Console", nullptr));
        label_4->setText(QCoreApplication::translate("Prefs_Scripter", "Base Texts:", nullptr));
        textButton->setText(QString());
        label_5->setText(QCoreApplication::translate("Prefs_Scripter", "Comments:", nullptr));
        commentButton->setText(QString());
        label_6->setText(QCoreApplication::translate("Prefs_Scripter", "Keywords:", nullptr));
        keywordButton->setText(QString());
        label_7->setText(QCoreApplication::translate("Prefs_Scripter", "Signs:", nullptr));
        signButton->setText(QString());
        label_8->setText(QCoreApplication::translate("Prefs_Scripter", "Errors:", nullptr));
        errorButton->setText(QString());
        label_9->setText(QCoreApplication::translate("Prefs_Scripter", "Strings:", nullptr));
        stringButton->setText(QString());
        label_10->setText(QCoreApplication::translate("Prefs_Scripter", "Numbers:", nullptr));
        numberButton->setText(QString());
        (void)Prefs_Scripter;
    } // retranslateUi

};

namespace Ui {
    class Prefs_Scripter: public Ui_Prefs_Scripter {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_SCRIPTERBASE_H
