/********************************************************************************
** Form generated from reading UI file 'tablerowheightsdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TABLEROWHEIGHTSDIALOG_H
#define UI_TABLEROWHEIGHTSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_TableRowHeightsDialog
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *tableRowHeightLabel;
    ScrSpinBox *tableRowHeight;
    QDialogButtonBox *dialogButtonBox;

    void setupUi(QDialog *TableRowHeightsDialog)
    {
        if (TableRowHeightsDialog->objectName().isEmpty())
            TableRowHeightsDialog->setObjectName(QString::fromUtf8("TableRowHeightsDialog"));
        TableRowHeightsDialog->resize(227, 61);
        TableRowHeightsDialog->setModal(true);
        verticalLayout = new QVBoxLayout(TableRowHeightsDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        tableRowHeightLabel = new QLabel(TableRowHeightsDialog);
        tableRowHeightLabel->setObjectName(QString::fromUtf8("tableRowHeightLabel"));

        formLayout->setWidget(0, QFormLayout::LabelRole, tableRowHeightLabel);

        tableRowHeight = new ScrSpinBox(TableRowHeightsDialog);
        tableRowHeight->setObjectName(QString::fromUtf8("tableRowHeight"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tableRowHeight->sizePolicy().hasHeightForWidth());
        tableRowHeight->setSizePolicy(sizePolicy);
        tableRowHeight->setMinimumSize(QSize(140, 0));
        tableRowHeight->setMaximum(1000000.000000000000000);

        formLayout->setWidget(0, QFormLayout::FieldRole, tableRowHeight);


        verticalLayout->addLayout(formLayout);

        dialogButtonBox = new QDialogButtonBox(TableRowHeightsDialog);
        dialogButtonBox->setObjectName(QString::fromUtf8("dialogButtonBox"));
        dialogButtonBox->setOrientation(Qt::Horizontal);
        dialogButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(dialogButtonBox);

#if QT_CONFIG(shortcut)
        tableRowHeightLabel->setBuddy(tableRowHeight);
#endif // QT_CONFIG(shortcut)

        retranslateUi(TableRowHeightsDialog);
        QObject::connect(dialogButtonBox, SIGNAL(accepted()), TableRowHeightsDialog, SLOT(accept()));
        QObject::connect(dialogButtonBox, SIGNAL(rejected()), TableRowHeightsDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(TableRowHeightsDialog);
    } // setupUi

    void retranslateUi(QDialog *TableRowHeightsDialog)
    {
        TableRowHeightsDialog->setWindowTitle(QCoreApplication::translate("TableRowHeightsDialog", "Set Row Heights", nullptr));
        tableRowHeightLabel->setText(QCoreApplication::translate("TableRowHeightsDialog", "Row Height:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TableRowHeightsDialog: public Ui_TableRowHeightsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TABLEROWHEIGHTSDIALOG_H
