/****************************************************************************
** Meta object code from reading C++ file 'propertywidget_hyphenation.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/propertywidget_hyphenation.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertywidget_hyphenation.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PropertyWidget_Hyphenation_t {
    QByteArrayData data[23];
    char stringdata0[297];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PropertyWidget_Hyphenation_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PropertyWidget_Hyphenation_t qt_meta_stringdata_PropertyWidget_Hyphenation = {
    {
QT_MOC_LITERAL(0, 0, 26), // "PropertyWidget_Hyphenation"
QT_MOC_LITERAL(1, 27, 13), // "setMainWindow"
QT_MOC_LITERAL(2, 41, 0), // ""
QT_MOC_LITERAL(3, 42, 18), // "ScribusMainWindow*"
QT_MOC_LITERAL(4, 61, 2), // "mw"
QT_MOC_LITERAL(5, 64, 6), // "setDoc"
QT_MOC_LITERAL(6, 71, 11), // "ScribusDoc*"
QT_MOC_LITERAL(7, 83, 1), // "d"
QT_MOC_LITERAL(8, 85, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(9, 108, 14), // "languageChange"
QT_MOC_LITERAL(10, 123, 10), // "unitChange"
QT_MOC_LITERAL(11, 134, 15), // "updateCharStyle"
QT_MOC_LITERAL(12, 150, 9), // "CharStyle"
QT_MOC_LITERAL(13, 160, 9), // "charStyle"
QT_MOC_LITERAL(14, 170, 11), // "updateStyle"
QT_MOC_LITERAL(15, 182, 14), // "ParagraphStyle"
QT_MOC_LITERAL(16, 197, 9), // "paraStyle"
QT_MOC_LITERAL(17, 207, 13), // "handleWordMin"
QT_MOC_LITERAL(18, 221, 7), // "minWord"
QT_MOC_LITERAL(19, 229, 22), // "handleConsecutiveLines"
QT_MOC_LITERAL(20, 252, 16), // "consecutiveLines"
QT_MOC_LITERAL(21, 269, 16), // "handleHyphenChar"
QT_MOC_LITERAL(22, 286, 10) // "hyphenText"

    },
    "PropertyWidget_Hyphenation\0setMainWindow\0"
    "\0ScribusMainWindow*\0mw\0setDoc\0ScribusDoc*\0"
    "d\0handleSelectionChanged\0languageChange\0"
    "unitChange\0updateCharStyle\0CharStyle\0"
    "charStyle\0updateStyle\0ParagraphStyle\0"
    "paraStyle\0handleWordMin\0minWord\0"
    "handleConsecutiveLines\0consecutiveLines\0"
    "handleHyphenChar\0hyphenText"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PropertyWidget_Hyphenation[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   64,    2, 0x0a /* Public */,
       5,    1,   67,    2, 0x0a /* Public */,
       8,    0,   70,    2, 0x0a /* Public */,
       9,    0,   71,    2, 0x0a /* Public */,
      10,    0,   72,    2, 0x0a /* Public */,
      11,    1,   73,    2, 0x0a /* Public */,
      14,    1,   76,    2, 0x0a /* Public */,
      17,    1,   79,    2, 0x08 /* Private */,
      19,    1,   82,    2, 0x08 /* Private */,
      21,    1,   85,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    QMetaType::Void, 0x80000000 | 15,   16,
    QMetaType::Void, QMetaType::Int,   18,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::QString,   22,

       0        // eod
};

void PropertyWidget_Hyphenation::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertyWidget_Hyphenation *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setMainWindow((*reinterpret_cast< ScribusMainWindow*(*)>(_a[1]))); break;
        case 1: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 2: _t->handleSelectionChanged(); break;
        case 3: _t->languageChange(); break;
        case 4: _t->unitChange(); break;
        case 5: _t->updateCharStyle((*reinterpret_cast< const CharStyle(*)>(_a[1]))); break;
        case 6: _t->updateStyle((*reinterpret_cast< const ParagraphStyle(*)>(_a[1]))); break;
        case 7: _t->handleWordMin((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->handleConsecutiveLines((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->handleHyphenChar((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PropertyWidget_Hyphenation::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_PropertyWidget_Hyphenation.data,
    qt_meta_data_PropertyWidget_Hyphenation,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PropertyWidget_Hyphenation::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PropertyWidget_Hyphenation::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PropertyWidget_Hyphenation.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PropertyWidget_HyphenationBase"))
        return static_cast< Ui::PropertyWidget_HyphenationBase*>(this);
    if (!strcmp(_clname, "PropertyWidgetBase"))
        return static_cast< PropertyWidgetBase*>(this);
    return QFrame::qt_metacast(_clname);
}

int PropertyWidget_Hyphenation::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
