/********************************************************************************
** Form generated from reading UI file 'arcwidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ARCWIDGETBASE_H
#define UI_ARCWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_ArcWidgetBase
{
public:
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *startAngleLabel;
    ScrSpinBox *startAngle;
    QLabel *endAngleLabel;
    ScrSpinBox *endAngle;
    QSpacerItem *verticalSpacer;
    QLabel *Preview;

    void setupUi(QWidget *ArcWidgetBase)
    {
        if (ArcWidgetBase->objectName().isEmpty())
            ArcWidgetBase->setObjectName(QString::fromUtf8("ArcWidgetBase"));
        ArcWidgetBase->resize(425, 288);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(ArcWidgetBase->sizePolicy().hasHeightForWidth());
        ArcWidgetBase->setSizePolicy(sizePolicy);
        ArcWidgetBase->setMinimumSize(QSize(425, 288));
        horizontalLayout = new QHBoxLayout(ArcWidgetBase);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        startAngleLabel = new QLabel(ArcWidgetBase);
        startAngleLabel->setObjectName(QString::fromUtf8("startAngleLabel"));

        gridLayout->addWidget(startAngleLabel, 0, 0, 1, 1);

        startAngle = new ScrSpinBox(ArcWidgetBase);
        startAngle->setObjectName(QString::fromUtf8("startAngle"));
        startAngle->setDecimals(1);
        startAngle->setMinimum(-360.000000000000000);
        startAngle->setMaximum(360.000000000000000);

        gridLayout->addWidget(startAngle, 0, 1, 1, 1);

        endAngleLabel = new QLabel(ArcWidgetBase);
        endAngleLabel->setObjectName(QString::fromUtf8("endAngleLabel"));

        gridLayout->addWidget(endAngleLabel, 1, 0, 1, 1);

        endAngle = new ScrSpinBox(ArcWidgetBase);
        endAngle->setObjectName(QString::fromUtf8("endAngle"));
        endAngle->setDecimals(1);
        endAngle->setMinimum(-360.000000000000000);
        endAngle->setMaximum(360.000000000000000);

        gridLayout->addWidget(endAngle, 1, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout);

        Preview = new QLabel(ArcWidgetBase);
        Preview->setObjectName(QString::fromUtf8("Preview"));
        QSizePolicy sizePolicy1(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(Preview->sizePolicy().hasHeightForWidth());
        Preview->setSizePolicy(sizePolicy1);
        Preview->setMinimumSize(QSize(280, 280));
        Preview->setMaximumSize(QSize(280, 280));
        Preview->setSizeIncrement(QSize(0, 0));
        Preview->setFrameShape(QFrame::Panel);
        Preview->setFrameShadow(QFrame::Sunken);
        Preview->setLineWidth(2);
        Preview->setAlignment(Qt::AlignCenter);

        horizontalLayout->addWidget(Preview);


        retranslateUi(ArcWidgetBase);

        QMetaObject::connectSlotsByName(ArcWidgetBase);
    } // setupUi

    void retranslateUi(QWidget *ArcWidgetBase)
    {
        startAngleLabel->setText(QCoreApplication::translate("ArcWidgetBase", "Start Angle:", nullptr));
        endAngleLabel->setText(QCoreApplication::translate("ArcWidgetBase", "End Angle:", nullptr));
#if QT_CONFIG(tooltip)
        Preview->setToolTip(QCoreApplication::translate("ArcWidgetBase", "Sample Polygon", nullptr));
#endif // QT_CONFIG(tooltip)
        Preview->setText(QString());
        (void)ArcWidgetBase;
    } // retranslateUi

};

namespace Ui {
    class ArcWidgetBase: public Ui_ArcWidgetBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ARCWIDGETBASE_H
