/****************************************************************************
** Meta object code from reading C++ file 'propertywidget_textcolor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/propertywidget_textcolor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertywidget_textcolor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PropertyWidget_TextColor_t {
    QByteArrayData data[46];
    char stringdata0[559];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PropertyWidget_TextColor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PropertyWidget_TextColor_t qt_meta_stringdata_PropertyWidget_TextColor = {
    {
QT_MOC_LITERAL(0, 0, 24), // "PropertyWidget_TextColor"
QT_MOC_LITERAL(1, 25, 13), // "setMainWindow"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 18), // "ScribusMainWindow*"
QT_MOC_LITERAL(4, 59, 2), // "mw"
QT_MOC_LITERAL(5, 62, 6), // "setDoc"
QT_MOC_LITERAL(6, 69, 11), // "ScribusDoc*"
QT_MOC_LITERAL(7, 81, 1), // "d"
QT_MOC_LITERAL(8, 83, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(9, 106, 19), // "handleUpdateRequest"
QT_MOC_LITERAL(10, 126, 13), // "iconSetChange"
QT_MOC_LITERAL(11, 140, 14), // "languageChange"
QT_MOC_LITERAL(12, 155, 10), // "unitChange"
QT_MOC_LITERAL(13, 166, 12), // "localeChange"
QT_MOC_LITERAL(14, 179, 12), // "showOutlineW"
QT_MOC_LITERAL(15, 192, 1), // "x"
QT_MOC_LITERAL(16, 194, 16), // "showShadowOffset"
QT_MOC_LITERAL(17, 211, 1), // "y"
QT_MOC_LITERAL(18, 213, 14), // "showStrikeThru"
QT_MOC_LITERAL(19, 228, 1), // "p"
QT_MOC_LITERAL(20, 230, 1), // "w"
QT_MOC_LITERAL(21, 232, 14), // "showTextColors"
QT_MOC_LITERAL(22, 247, 9), // "strokeCol"
QT_MOC_LITERAL(23, 257, 7), // "fillCol"
QT_MOC_LITERAL(24, 265, 7), // "backCol"
QT_MOC_LITERAL(25, 273, 9), // "strokeShd"
QT_MOC_LITERAL(26, 283, 7), // "fillShd"
QT_MOC_LITERAL(27, 291, 7), // "backShd"
QT_MOC_LITERAL(28, 299, 15), // "showTextEffects"
QT_MOC_LITERAL(29, 315, 1), // "s"
QT_MOC_LITERAL(30, 317, 13), // "showUnderline"
QT_MOC_LITERAL(31, 331, 15), // "updateCharStyle"
QT_MOC_LITERAL(32, 347, 9), // "CharStyle"
QT_MOC_LITERAL(33, 357, 9), // "charStyle"
QT_MOC_LITERAL(34, 367, 11), // "updateStyle"
QT_MOC_LITERAL(35, 379, 14), // "ParagraphStyle"
QT_MOC_LITERAL(36, 394, 10), // "newCurrent"
QT_MOC_LITERAL(37, 405, 18), // "handleOutlineWidth"
QT_MOC_LITERAL(38, 424, 16), // "handleShadowOffs"
QT_MOC_LITERAL(39, 441, 16), // "handleStrikeThru"
QT_MOC_LITERAL(40, 458, 14), // "handleTextFill"
QT_MOC_LITERAL(41, 473, 15), // "handleTextShade"
QT_MOC_LITERAL(42, 489, 16), // "handleTextStroke"
QT_MOC_LITERAL(43, 506, 20), // "handleTextBackground"
QT_MOC_LITERAL(44, 527, 15), // "handleTypeStyle"
QT_MOC_LITERAL(45, 543, 15) // "handleUnderline"

    },
    "PropertyWidget_TextColor\0setMainWindow\0"
    "\0ScribusMainWindow*\0mw\0setDoc\0ScribusDoc*\0"
    "d\0handleSelectionChanged\0handleUpdateRequest\0"
    "iconSetChange\0languageChange\0unitChange\0"
    "localeChange\0showOutlineW\0x\0"
    "showShadowOffset\0y\0showStrikeThru\0p\0"
    "w\0showTextColors\0strokeCol\0fillCol\0"
    "backCol\0strokeShd\0fillShd\0backShd\0"
    "showTextEffects\0s\0showUnderline\0"
    "updateCharStyle\0CharStyle\0charStyle\0"
    "updateStyle\0ParagraphStyle\0newCurrent\0"
    "handleOutlineWidth\0handleShadowOffs\0"
    "handleStrikeThru\0handleTextFill\0"
    "handleTextShade\0handleTextStroke\0"
    "handleTextBackground\0handleTypeStyle\0"
    "handleUnderline"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PropertyWidget_TextColor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      25,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  139,    2, 0x0a /* Public */,
       5,    1,  142,    2, 0x0a /* Public */,
       8,    0,  145,    2, 0x0a /* Public */,
       9,    1,  146,    2, 0x0a /* Public */,
      10,    0,  149,    2, 0x0a /* Public */,
      11,    0,  150,    2, 0x0a /* Public */,
      12,    0,  151,    2, 0x0a /* Public */,
      13,    0,  152,    2, 0x0a /* Public */,
      14,    1,  153,    2, 0x0a /* Public */,
      16,    2,  156,    2, 0x0a /* Public */,
      18,    2,  161,    2, 0x0a /* Public */,
      21,    6,  166,    2, 0x0a /* Public */,
      28,    1,  179,    2, 0x0a /* Public */,
      30,    2,  182,    2, 0x0a /* Public */,
      31,    1,  187,    2, 0x0a /* Public */,
      34,    1,  190,    2, 0x0a /* Public */,
      37,    0,  193,    2, 0x08 /* Private */,
      38,    0,  194,    2, 0x08 /* Private */,
      39,    0,  195,    2, 0x08 /* Private */,
      40,    0,  196,    2, 0x08 /* Private */,
      41,    0,  197,    2, 0x08 /* Private */,
      42,    0,  198,    2, 0x08 /* Private */,
      43,    0,  199,    2, 0x08 /* Private */,
      44,    1,  200,    2, 0x08 /* Private */,
      45,    0,  203,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,   15,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,   15,   17,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,   19,   20,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::QString, QMetaType::Double, QMetaType::Double, QMetaType::Double,   22,   23,   24,   25,   26,   27,
    QMetaType::Void, QMetaType::Int,   29,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,   19,   20,
    QMetaType::Void, 0x80000000 | 32,   33,
    QMetaType::Void, 0x80000000 | 35,   36,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   29,
    QMetaType::Void,

       0        // eod
};

void PropertyWidget_TextColor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertyWidget_TextColor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setMainWindow((*reinterpret_cast< ScribusMainWindow*(*)>(_a[1]))); break;
        case 1: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 2: _t->handleSelectionChanged(); break;
        case 3: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->iconSetChange(); break;
        case 5: _t->languageChange(); break;
        case 6: _t->unitChange(); break;
        case 7: _t->localeChange(); break;
        case 8: _t->showOutlineW((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 9: _t->showShadowOffset((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 10: _t->showStrikeThru((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 11: _t->showTextColors((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< const QString(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6]))); break;
        case 12: _t->showTextEffects((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->showUnderline((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 14: _t->updateCharStyle((*reinterpret_cast< const CharStyle(*)>(_a[1]))); break;
        case 15: _t->updateStyle((*reinterpret_cast< const ParagraphStyle(*)>(_a[1]))); break;
        case 16: _t->handleOutlineWidth(); break;
        case 17: _t->handleShadowOffs(); break;
        case 18: _t->handleStrikeThru(); break;
        case 19: _t->handleTextFill(); break;
        case 20: _t->handleTextShade(); break;
        case 21: _t->handleTextStroke(); break;
        case 22: _t->handleTextBackground(); break;
        case 23: _t->handleTypeStyle((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 24: _t->handleUnderline(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PropertyWidget_TextColor::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_PropertyWidget_TextColor.data,
    qt_meta_data_PropertyWidget_TextColor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PropertyWidget_TextColor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PropertyWidget_TextColor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PropertyWidget_TextColor.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PropertyWidget_TextColorBase"))
        return static_cast< Ui::PropertyWidget_TextColorBase*>(this);
    if (!strcmp(_clname, "PropertyWidgetBase"))
        return static_cast< PropertyWidgetBase*>(this);
    return QFrame::qt_metacast(_clname);
}

int PropertyWidget_TextColor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 25;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
