/********************************************************************************
** Form generated from reading UI file 'propertywidget_fontfeaturesbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYWIDGET_FONTFEATURESBASE_H
#define UI_PROPERTYWIDGET_FONTFEATURESBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_PropertyWidget_FontFeatures
{
public:
    QVBoxLayout *verticalLayout;
    QCheckBox *CommonCheck;
    QCheckBox *ContextualCheck;
    QCheckBox *DiscretionaryCheck;
    QCheckBox *HistoricalCheck;
    QFrame *line;
    QRadioButton *NormalCapRadio;
    QRadioButton *SmallRadio;
    QRadioButton *SmallFromCRadio;
    QRadioButton *AllSmallCapsRadio;
    QRadioButton *PetiteRadio;
    QRadioButton *PetiteCapRadio;
    QRadioButton *AllPetiteCapsRadio;
    QRadioButton *UnicaseRadio;
    QRadioButton *TiltingRadio;
    QFrame *line_2;
    QRadioButton *DefaultStyleRadio;
    QRadioButton *LiningRadio;
    QRadioButton *OldStyleRadio;
    QFrame *line_3;
    QRadioButton *DefaultWidthRadio;
    QRadioButton *ProportionalRadio;
    QRadioButton *TabularRadio;
    QFrame *line_4;
    QRadioButton *DefaultFractionsRadio;
    QRadioButton *DiagonalRadio;
    QRadioButton *StackedRadio;
    QFrame *line_5;
    QRadioButton *DefaultPosRadio;
    QRadioButton *SubscriptRadio;
    QRadioButton *SuperscriptRadio;
    QRadioButton *OrdinalCheck;
    QFrame *line_6;
    QCheckBox *SlashedZeroCheck;
    QFrame *line_7;
    QGroupBox *groupBox_7;
    QGridLayout *gridLayout;
    QCheckBox *StyleSet01;
    QCheckBox *StyleSet05;
    QCheckBox *StyleSet09;
    QCheckBox *StyleSet13;
    QCheckBox *StyleSet17;
    QCheckBox *StyleSet02;
    QCheckBox *StyleSet06;
    QCheckBox *StyleSet10;
    QCheckBox *StyleSet14;
    QCheckBox *StyleSet18;
    QCheckBox *StyleSet03;
    QCheckBox *StyleSet07;
    QCheckBox *StyleSet11;
    QCheckBox *StyleSet15;
    QCheckBox *StyleSet19;
    QCheckBox *StyleSet04;
    QCheckBox *StyleSet08;
    QCheckBox *StyleSet12;
    QCheckBox *StyleSet16;
    QCheckBox *StyleSet20;
    QButtonGroup *PositionGroup;
    QButtonGroup *CapitalsGroup;
    QButtonGroup *FractionsGroup;
    QButtonGroup *NumeralStyleGroup;
    QButtonGroup *NumeralWidthGroup;

    void setupUi(QFrame *PropertyWidget_FontFeatures)
    {
        if (PropertyWidget_FontFeatures->objectName().isEmpty())
            PropertyWidget_FontFeatures->setObjectName(QString::fromUtf8("PropertyWidget_FontFeatures"));
        PropertyWidget_FontFeatures->resize(509, 758);
        verticalLayout = new QVBoxLayout(PropertyWidget_FontFeatures);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        CommonCheck = new QCheckBox(PropertyWidget_FontFeatures);
        CommonCheck->setObjectName(QString::fromUtf8("CommonCheck"));
        CommonCheck->setChecked(true);

        verticalLayout->addWidget(CommonCheck);

        ContextualCheck = new QCheckBox(PropertyWidget_FontFeatures);
        ContextualCheck->setObjectName(QString::fromUtf8("ContextualCheck"));
        ContextualCheck->setChecked(true);
        ContextualCheck->setAutoRepeat(false);
        ContextualCheck->setAutoExclusive(false);
        ContextualCheck->setTristate(false);

        verticalLayout->addWidget(ContextualCheck);

        DiscretionaryCheck = new QCheckBox(PropertyWidget_FontFeatures);
        DiscretionaryCheck->setObjectName(QString::fromUtf8("DiscretionaryCheck"));
        DiscretionaryCheck->setAutoExclusive(false);

        verticalLayout->addWidget(DiscretionaryCheck);

        HistoricalCheck = new QCheckBox(PropertyWidget_FontFeatures);
        HistoricalCheck->setObjectName(QString::fromUtf8("HistoricalCheck"));
        HistoricalCheck->setChecked(false);
        HistoricalCheck->setAutoExclusive(true);

        verticalLayout->addWidget(HistoricalCheck);

        line = new QFrame(PropertyWidget_FontFeatures);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        NormalCapRadio = new QRadioButton(PropertyWidget_FontFeatures);
        CapitalsGroup = new QButtonGroup(PropertyWidget_FontFeatures);
        CapitalsGroup->setObjectName(QString::fromUtf8("CapitalsGroup"));
        CapitalsGroup->addButton(NormalCapRadio);
        NormalCapRadio->setObjectName(QString::fromUtf8("NormalCapRadio"));
        NormalCapRadio->setChecked(true);

        verticalLayout->addWidget(NormalCapRadio);

        SmallRadio = new QRadioButton(PropertyWidget_FontFeatures);
        CapitalsGroup->addButton(SmallRadio);
        SmallRadio->setObjectName(QString::fromUtf8("SmallRadio"));

        verticalLayout->addWidget(SmallRadio);

        SmallFromCRadio = new QRadioButton(PropertyWidget_FontFeatures);
        CapitalsGroup->addButton(SmallFromCRadio);
        SmallFromCRadio->setObjectName(QString::fromUtf8("SmallFromCRadio"));

        verticalLayout->addWidget(SmallFromCRadio);

        AllSmallCapsRadio = new QRadioButton(PropertyWidget_FontFeatures);
        CapitalsGroup->addButton(AllSmallCapsRadio);
        AllSmallCapsRadio->setObjectName(QString::fromUtf8("AllSmallCapsRadio"));

        verticalLayout->addWidget(AllSmallCapsRadio);

        PetiteRadio = new QRadioButton(PropertyWidget_FontFeatures);
        CapitalsGroup->addButton(PetiteRadio);
        PetiteRadio->setObjectName(QString::fromUtf8("PetiteRadio"));

        verticalLayout->addWidget(PetiteRadio);

        PetiteCapRadio = new QRadioButton(PropertyWidget_FontFeatures);
        CapitalsGroup->addButton(PetiteCapRadio);
        PetiteCapRadio->setObjectName(QString::fromUtf8("PetiteCapRadio"));

        verticalLayout->addWidget(PetiteCapRadio);

        AllPetiteCapsRadio = new QRadioButton(PropertyWidget_FontFeatures);
        CapitalsGroup->addButton(AllPetiteCapsRadio);
        AllPetiteCapsRadio->setObjectName(QString::fromUtf8("AllPetiteCapsRadio"));

        verticalLayout->addWidget(AllPetiteCapsRadio);

        UnicaseRadio = new QRadioButton(PropertyWidget_FontFeatures);
        CapitalsGroup->addButton(UnicaseRadio);
        UnicaseRadio->setObjectName(QString::fromUtf8("UnicaseRadio"));

        verticalLayout->addWidget(UnicaseRadio);

        TiltingRadio = new QRadioButton(PropertyWidget_FontFeatures);
        CapitalsGroup->addButton(TiltingRadio);
        TiltingRadio->setObjectName(QString::fromUtf8("TiltingRadio"));

        verticalLayout->addWidget(TiltingRadio);

        line_2 = new QFrame(PropertyWidget_FontFeatures);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line_2);

        DefaultStyleRadio = new QRadioButton(PropertyWidget_FontFeatures);
        NumeralStyleGroup = new QButtonGroup(PropertyWidget_FontFeatures);
        NumeralStyleGroup->setObjectName(QString::fromUtf8("NumeralStyleGroup"));
        NumeralStyleGroup->addButton(DefaultStyleRadio);
        DefaultStyleRadio->setObjectName(QString::fromUtf8("DefaultStyleRadio"));
        DefaultStyleRadio->setEnabled(true);
        DefaultStyleRadio->setChecked(true);

        verticalLayout->addWidget(DefaultStyleRadio);

        LiningRadio = new QRadioButton(PropertyWidget_FontFeatures);
        NumeralStyleGroup->addButton(LiningRadio);
        LiningRadio->setObjectName(QString::fromUtf8("LiningRadio"));
        LiningRadio->setEnabled(true);

        verticalLayout->addWidget(LiningRadio);

        OldStyleRadio = new QRadioButton(PropertyWidget_FontFeatures);
        NumeralStyleGroup->addButton(OldStyleRadio);
        OldStyleRadio->setObjectName(QString::fromUtf8("OldStyleRadio"));
        OldStyleRadio->setEnabled(true);

        verticalLayout->addWidget(OldStyleRadio);

        line_3 = new QFrame(PropertyWidget_FontFeatures);
        line_3->setObjectName(QString::fromUtf8("line_3"));
        line_3->setFrameShape(QFrame::HLine);
        line_3->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line_3);

        DefaultWidthRadio = new QRadioButton(PropertyWidget_FontFeatures);
        NumeralWidthGroup = new QButtonGroup(PropertyWidget_FontFeatures);
        NumeralWidthGroup->setObjectName(QString::fromUtf8("NumeralWidthGroup"));
        NumeralWidthGroup->addButton(DefaultWidthRadio);
        DefaultWidthRadio->setObjectName(QString::fromUtf8("DefaultWidthRadio"));
        DefaultWidthRadio->setEnabled(true);
        DefaultWidthRadio->setChecked(true);

        verticalLayout->addWidget(DefaultWidthRadio);

        ProportionalRadio = new QRadioButton(PropertyWidget_FontFeatures);
        NumeralWidthGroup->addButton(ProportionalRadio);
        ProportionalRadio->setObjectName(QString::fromUtf8("ProportionalRadio"));
        ProportionalRadio->setEnabled(true);

        verticalLayout->addWidget(ProportionalRadio);

        TabularRadio = new QRadioButton(PropertyWidget_FontFeatures);
        NumeralWidthGroup->addButton(TabularRadio);
        TabularRadio->setObjectName(QString::fromUtf8("TabularRadio"));
        TabularRadio->setEnabled(true);

        verticalLayout->addWidget(TabularRadio);

        line_4 = new QFrame(PropertyWidget_FontFeatures);
        line_4->setObjectName(QString::fromUtf8("line_4"));
        line_4->setFrameShape(QFrame::HLine);
        line_4->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line_4);

        DefaultFractionsRadio = new QRadioButton(PropertyWidget_FontFeatures);
        FractionsGroup = new QButtonGroup(PropertyWidget_FontFeatures);
        FractionsGroup->setObjectName(QString::fromUtf8("FractionsGroup"));
        FractionsGroup->addButton(DefaultFractionsRadio);
        DefaultFractionsRadio->setObjectName(QString::fromUtf8("DefaultFractionsRadio"));
        DefaultFractionsRadio->setEnabled(true);
        DefaultFractionsRadio->setChecked(true);

        verticalLayout->addWidget(DefaultFractionsRadio);

        DiagonalRadio = new QRadioButton(PropertyWidget_FontFeatures);
        FractionsGroup->addButton(DiagonalRadio);
        DiagonalRadio->setObjectName(QString::fromUtf8("DiagonalRadio"));
        DiagonalRadio->setEnabled(true);

        verticalLayout->addWidget(DiagonalRadio);

        StackedRadio = new QRadioButton(PropertyWidget_FontFeatures);
        FractionsGroup->addButton(StackedRadio);
        StackedRadio->setObjectName(QString::fromUtf8("StackedRadio"));
        StackedRadio->setEnabled(true);

        verticalLayout->addWidget(StackedRadio);

        line_5 = new QFrame(PropertyWidget_FontFeatures);
        line_5->setObjectName(QString::fromUtf8("line_5"));
        line_5->setFrameShape(QFrame::HLine);
        line_5->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line_5);

        DefaultPosRadio = new QRadioButton(PropertyWidget_FontFeatures);
        PositionGroup = new QButtonGroup(PropertyWidget_FontFeatures);
        PositionGroup->setObjectName(QString::fromUtf8("PositionGroup"));
        PositionGroup->addButton(DefaultPosRadio);
        DefaultPosRadio->setObjectName(QString::fromUtf8("DefaultPosRadio"));
        DefaultPosRadio->setChecked(true);

        verticalLayout->addWidget(DefaultPosRadio);

        SubscriptRadio = new QRadioButton(PropertyWidget_FontFeatures);
        PositionGroup->addButton(SubscriptRadio);
        SubscriptRadio->setObjectName(QString::fromUtf8("SubscriptRadio"));

        verticalLayout->addWidget(SubscriptRadio);

        SuperscriptRadio = new QRadioButton(PropertyWidget_FontFeatures);
        PositionGroup->addButton(SuperscriptRadio);
        SuperscriptRadio->setObjectName(QString::fromUtf8("SuperscriptRadio"));

        verticalLayout->addWidget(SuperscriptRadio);

        OrdinalCheck = new QRadioButton(PropertyWidget_FontFeatures);
        PositionGroup->addButton(OrdinalCheck);
        OrdinalCheck->setObjectName(QString::fromUtf8("OrdinalCheck"));
        OrdinalCheck->setEnabled(true);

        verticalLayout->addWidget(OrdinalCheck);

        line_6 = new QFrame(PropertyWidget_FontFeatures);
        line_6->setObjectName(QString::fromUtf8("line_6"));
        line_6->setFrameShape(QFrame::HLine);
        line_6->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line_6);

        SlashedZeroCheck = new QCheckBox(PropertyWidget_FontFeatures);
        SlashedZeroCheck->setObjectName(QString::fromUtf8("SlashedZeroCheck"));
        SlashedZeroCheck->setEnabled(true);

        verticalLayout->addWidget(SlashedZeroCheck);

        line_7 = new QFrame(PropertyWidget_FontFeatures);
        line_7->setObjectName(QString::fromUtf8("line_7"));
        line_7->setFrameShape(QFrame::HLine);
        line_7->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line_7);

        groupBox_7 = new QGroupBox(PropertyWidget_FontFeatures);
        groupBox_7->setObjectName(QString::fromUtf8("groupBox_7"));
        gridLayout = new QGridLayout(groupBox_7);
        gridLayout->setSpacing(3);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(6, 6, 6, 6);
        StyleSet01 = new QCheckBox(groupBox_7);
        StyleSet01->setObjectName(QString::fromUtf8("StyleSet01"));

        gridLayout->addWidget(StyleSet01, 0, 0, 1, 1);

        StyleSet05 = new QCheckBox(groupBox_7);
        StyleSet05->setObjectName(QString::fromUtf8("StyleSet05"));

        gridLayout->addWidget(StyleSet05, 0, 1, 1, 1);

        StyleSet09 = new QCheckBox(groupBox_7);
        StyleSet09->setObjectName(QString::fromUtf8("StyleSet09"));

        gridLayout->addWidget(StyleSet09, 0, 2, 1, 1);

        StyleSet13 = new QCheckBox(groupBox_7);
        StyleSet13->setObjectName(QString::fromUtf8("StyleSet13"));

        gridLayout->addWidget(StyleSet13, 0, 3, 1, 1);

        StyleSet17 = new QCheckBox(groupBox_7);
        StyleSet17->setObjectName(QString::fromUtf8("StyleSet17"));

        gridLayout->addWidget(StyleSet17, 0, 4, 1, 1);

        StyleSet02 = new QCheckBox(groupBox_7);
        StyleSet02->setObjectName(QString::fromUtf8("StyleSet02"));

        gridLayout->addWidget(StyleSet02, 1, 0, 1, 1);

        StyleSet06 = new QCheckBox(groupBox_7);
        StyleSet06->setObjectName(QString::fromUtf8("StyleSet06"));

        gridLayout->addWidget(StyleSet06, 1, 1, 1, 1);

        StyleSet10 = new QCheckBox(groupBox_7);
        StyleSet10->setObjectName(QString::fromUtf8("StyleSet10"));

        gridLayout->addWidget(StyleSet10, 1, 2, 1, 1);

        StyleSet14 = new QCheckBox(groupBox_7);
        StyleSet14->setObjectName(QString::fromUtf8("StyleSet14"));

        gridLayout->addWidget(StyleSet14, 1, 3, 1, 1);

        StyleSet18 = new QCheckBox(groupBox_7);
        StyleSet18->setObjectName(QString::fromUtf8("StyleSet18"));

        gridLayout->addWidget(StyleSet18, 1, 4, 1, 1);

        StyleSet03 = new QCheckBox(groupBox_7);
        StyleSet03->setObjectName(QString::fromUtf8("StyleSet03"));

        gridLayout->addWidget(StyleSet03, 2, 0, 1, 1);

        StyleSet07 = new QCheckBox(groupBox_7);
        StyleSet07->setObjectName(QString::fromUtf8("StyleSet07"));

        gridLayout->addWidget(StyleSet07, 2, 1, 1, 1);

        StyleSet11 = new QCheckBox(groupBox_7);
        StyleSet11->setObjectName(QString::fromUtf8("StyleSet11"));

        gridLayout->addWidget(StyleSet11, 2, 2, 1, 1);

        StyleSet15 = new QCheckBox(groupBox_7);
        StyleSet15->setObjectName(QString::fromUtf8("StyleSet15"));

        gridLayout->addWidget(StyleSet15, 2, 3, 1, 1);

        StyleSet19 = new QCheckBox(groupBox_7);
        StyleSet19->setObjectName(QString::fromUtf8("StyleSet19"));

        gridLayout->addWidget(StyleSet19, 2, 4, 1, 1);

        StyleSet04 = new QCheckBox(groupBox_7);
        StyleSet04->setObjectName(QString::fromUtf8("StyleSet04"));

        gridLayout->addWidget(StyleSet04, 3, 0, 1, 1);

        StyleSet08 = new QCheckBox(groupBox_7);
        StyleSet08->setObjectName(QString::fromUtf8("StyleSet08"));

        gridLayout->addWidget(StyleSet08, 3, 1, 1, 1);

        StyleSet12 = new QCheckBox(groupBox_7);
        StyleSet12->setObjectName(QString::fromUtf8("StyleSet12"));

        gridLayout->addWidget(StyleSet12, 3, 2, 1, 1);

        StyleSet16 = new QCheckBox(groupBox_7);
        StyleSet16->setObjectName(QString::fromUtf8("StyleSet16"));

        gridLayout->addWidget(StyleSet16, 3, 3, 1, 1);

        StyleSet20 = new QCheckBox(groupBox_7);
        StyleSet20->setObjectName(QString::fromUtf8("StyleSet20"));

        gridLayout->addWidget(StyleSet20, 3, 4, 1, 1);


        verticalLayout->addWidget(groupBox_7);


        retranslateUi(PropertyWidget_FontFeatures);

        QMetaObject::connectSlotsByName(PropertyWidget_FontFeatures);
    } // setupUi

    void retranslateUi(QFrame *PropertyWidget_FontFeatures)
    {
#if QT_CONFIG(tooltip)
        CommonCheck->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Common Ligatures (liga)", nullptr));
#endif // QT_CONFIG(tooltip)
        CommonCheck->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Common Ligatures", nullptr));
#if QT_CONFIG(tooltip)
        ContextualCheck->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Contextual Ligatures (clig)", nullptr));
#endif // QT_CONFIG(tooltip)
        ContextualCheck->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Contextual Ligatures", nullptr));
#if QT_CONFIG(tooltip)
        DiscretionaryCheck->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Discretionary Ligatures (dlig)", nullptr));
#endif // QT_CONFIG(tooltip)
        DiscretionaryCheck->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Discretionary Ligatures", nullptr));
#if QT_CONFIG(tooltip)
        HistoricalCheck->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Historical Ligatures (hlig)", nullptr));
#endif // QT_CONFIG(tooltip)
        HistoricalCheck->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Historical Ligatures", nullptr));
#if QT_CONFIG(tooltip)
        NormalCapRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Normal capitals", nullptr));
#endif // QT_CONFIG(tooltip)
        NormalCapRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Default Capitals", nullptr));
#if QT_CONFIG(tooltip)
        SmallRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Small capitals (smcp)", nullptr));
#endif // QT_CONFIG(tooltip)
        SmallRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Small Capitals", nullptr));
#if QT_CONFIG(tooltip)
        SmallFromCRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Small capitals from capitals (c2sc)", nullptr));
#endif // QT_CONFIG(tooltip)
        SmallFromCRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Small Capitals from Capitals", nullptr));
        AllSmallCapsRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "All Small Caps", nullptr));
#if QT_CONFIG(tooltip)
        PetiteRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Petite capitals (pcap)", nullptr));
#endif // QT_CONFIG(tooltip)
        PetiteRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Petite Capitals", nullptr));
#if QT_CONFIG(tooltip)
        PetiteCapRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Petite capitals from capitals (c2pc)", nullptr));
#endif // QT_CONFIG(tooltip)
        PetiteCapRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Petite Capitals from Capitals", nullptr));
        AllPetiteCapsRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "All Petite Caps", nullptr));
#if QT_CONFIG(tooltip)
        UnicaseRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Unicase (unic)", nullptr));
#endif // QT_CONFIG(tooltip)
        UnicaseRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Unicase", nullptr));
#if QT_CONFIG(tooltip)
        TiltingRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Titling (titl)", nullptr));
#endif // QT_CONFIG(tooltip)
        TiltingRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Titling", nullptr));
#if QT_CONFIG(tooltip)
        DefaultStyleRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Default numbers style", nullptr));
#endif // QT_CONFIG(tooltip)
        DefaultStyleRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Default Numerals", nullptr));
#if QT_CONFIG(tooltip)
        LiningRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Lining Figures (lnum)", nullptr));
#endif // QT_CONFIG(tooltip)
        LiningRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Lining", nullptr));
#if QT_CONFIG(tooltip)
        OldStyleRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Oldstyle Figures (onum)", nullptr));
#endif // QT_CONFIG(tooltip)
        OldStyleRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Old Style", nullptr));
#if QT_CONFIG(tooltip)
        DefaultWidthRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Default numbers width", nullptr));
#endif // QT_CONFIG(tooltip)
        DefaultWidthRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Default Numeral Width", nullptr));
#if QT_CONFIG(tooltip)
        ProportionalRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Proportional Figures (pnum)", nullptr));
#endif // QT_CONFIG(tooltip)
        ProportionalRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Proportional", nullptr));
#if QT_CONFIG(tooltip)
        TabularRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Tabular Figures (tnum)", nullptr));
#endif // QT_CONFIG(tooltip)
        TabularRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Tabular", nullptr));
#if QT_CONFIG(tooltip)
        DefaultFractionsRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "No fractions", nullptr));
#endif // QT_CONFIG(tooltip)
        DefaultFractionsRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "No Fractions", nullptr));
#if QT_CONFIG(tooltip)
        DiagonalRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Fractions (frac)", nullptr));
#endif // QT_CONFIG(tooltip)
        DiagonalRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Diagonal Fractions", nullptr));
#if QT_CONFIG(tooltip)
        StackedRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Alternative Fractions (afrc)", nullptr));
#endif // QT_CONFIG(tooltip)
        StackedRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Stacked Fractions", nullptr));
#if QT_CONFIG(tooltip)
        DefaultPosRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Normal position", nullptr));
#endif // QT_CONFIG(tooltip)
        DefaultPosRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Default Position", nullptr));
#if QT_CONFIG(tooltip)
        SubscriptRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Subscript (subs)", nullptr));
#endif // QT_CONFIG(tooltip)
        SubscriptRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Subscript", nullptr));
#if QT_CONFIG(tooltip)
        SuperscriptRadio->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Superscript (sups)", nullptr));
#endif // QT_CONFIG(tooltip)
        SuperscriptRadio->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Superscript", nullptr));
#if QT_CONFIG(tooltip)
        OrdinalCheck->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Ordinals (ordn)", nullptr));
#endif // QT_CONFIG(tooltip)
        OrdinalCheck->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Ordinals", nullptr));
#if QT_CONFIG(tooltip)
        SlashedZeroCheck->setToolTip(QCoreApplication::translate("PropertyWidget_FontFeatures", "Slashed Zero (zero)", nullptr));
#endif // QT_CONFIG(tooltip)
        SlashedZeroCheck->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "Slashed Zero", nullptr));
        groupBox_7->setTitle(QCoreApplication::translate("PropertyWidget_FontFeatures", "Stylistic sets", nullptr));
        StyleSet01->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "1", nullptr));
        StyleSet05->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "5", nullptr));
        StyleSet09->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "9", nullptr));
        StyleSet13->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "13", nullptr));
        StyleSet17->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "17", nullptr));
        StyleSet02->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "2", nullptr));
        StyleSet06->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "6", nullptr));
        StyleSet10->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "10", nullptr));
        StyleSet14->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "14", nullptr));
        StyleSet18->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "18", nullptr));
        StyleSet03->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "3", nullptr));
        StyleSet07->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "7", nullptr));
        StyleSet11->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "11", nullptr));
        StyleSet15->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "15", nullptr));
        StyleSet19->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "19", nullptr));
        StyleSet04->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "4", nullptr));
        StyleSet08->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "8", nullptr));
        StyleSet12->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "12", nullptr));
        StyleSet16->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "16", nullptr));
        StyleSet20->setText(QCoreApplication::translate("PropertyWidget_FontFeatures", "20", nullptr));
        (void)PropertyWidget_FontFeatures;
    } // retranslateUi

};

namespace Ui {
    class PropertyWidget_FontFeatures: public Ui_PropertyWidget_FontFeatures {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTYWIDGET_FONTFEATURESBASE_H
