/********************************************************************************
** Form generated from reading UI file 'printpreviewbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PRINTPREVIEWBASE_H
#define UI_PRINTPREVIEWBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/pageselector.h"
#include "ui/printlanguagecombo.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_PrintPreviewBase
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *topHLayout;
    QScrollArea *previewArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *settingsBarLayout;
    QWidget *optionWidget;
    QVBoxLayout *verticalLayout_4;
    QGroupBox *previewSettings;
    QVBoxLayout *verticalLayout_2;
    PrintLanguageCombo *printLanguageCombo;
    QComboBox *colorOutputMode;
    QCheckBox *mirrorH;
    QCheckBox *mirrorV;
    QCheckBox *clipToMargins;
    QCheckBox *convertSpots;
    QCheckBox *enableGCR;
    QGroupBox *displaySettings;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *antiAliasing;
    QCheckBox *showTransparency;
    QCheckBox *enableCMYK;
    QTableWidget *inkTable;
    QCheckBox *displayInkCoverage;
    QHBoxLayout *horizontalLayout;
    QLabel *coverThresholdLabel;
    ScrSpinBox *coverThresholdValue;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *bottomHLayout;
    QLabel *scaleLabel;
    QComboBox *scaleBox;
    QSpacerItem *horizontalSpacer;
    PageSelector *pageSelector;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *closeButton;
    QPushButton *printButton;

    void setupUi(QDialog *PrintPreviewBase)
    {
        if (PrintPreviewBase->objectName().isEmpty())
            PrintPreviewBase->setObjectName(QString::fromUtf8("PrintPreviewBase"));
        PrintPreviewBase->resize(982, 733);
        verticalLayout = new QVBoxLayout(PrintPreviewBase);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        topHLayout = new QHBoxLayout();
        topHLayout->setObjectName(QString::fromUtf8("topHLayout"));
        previewArea = new QScrollArea(PrintPreviewBase);
        previewArea->setObjectName(QString::fromUtf8("previewArea"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(previewArea->sizePolicy().hasHeightForWidth());
        previewArea->setSizePolicy(sizePolicy);
        previewArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 744, 663));
        previewArea->setWidget(scrollAreaWidgetContents);

        topHLayout->addWidget(previewArea);

        settingsBarLayout = new QVBoxLayout();
        settingsBarLayout->setObjectName(QString::fromUtf8("settingsBarLayout"));
        optionWidget = new QWidget(PrintPreviewBase);
        optionWidget->setObjectName(QString::fromUtf8("optionWidget"));
        QSizePolicy sizePolicy1(QSizePolicy::Maximum, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(optionWidget->sizePolicy().hasHeightForWidth());
        optionWidget->setSizePolicy(sizePolicy1);
        verticalLayout_4 = new QVBoxLayout(optionWidget);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        previewSettings = new QGroupBox(optionWidget);
        previewSettings->setObjectName(QString::fromUtf8("previewSettings"));
        verticalLayout_2 = new QVBoxLayout(previewSettings);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        printLanguageCombo = new PrintLanguageCombo(previewSettings);
        printLanguageCombo->setObjectName(QString::fromUtf8("printLanguageCombo"));

        verticalLayout_2->addWidget(printLanguageCombo);

        colorOutputMode = new QComboBox(previewSettings);
        colorOutputMode->addItem(QString());
        colorOutputMode->addItem(QString());
        colorOutputMode->setObjectName(QString::fromUtf8("colorOutputMode"));

        verticalLayout_2->addWidget(colorOutputMode);

        mirrorH = new QCheckBox(previewSettings);
        mirrorH->setObjectName(QString::fromUtf8("mirrorH"));

        verticalLayout_2->addWidget(mirrorH);

        mirrorV = new QCheckBox(previewSettings);
        mirrorV->setObjectName(QString::fromUtf8("mirrorV"));

        verticalLayout_2->addWidget(mirrorV);

        clipToMargins = new QCheckBox(previewSettings);
        clipToMargins->setObjectName(QString::fromUtf8("clipToMargins"));

        verticalLayout_2->addWidget(clipToMargins);

        convertSpots = new QCheckBox(previewSettings);
        convertSpots->setObjectName(QString::fromUtf8("convertSpots"));

        verticalLayout_2->addWidget(convertSpots);

        enableGCR = new QCheckBox(previewSettings);
        enableGCR->setObjectName(QString::fromUtf8("enableGCR"));

        verticalLayout_2->addWidget(enableGCR);


        verticalLayout_4->addWidget(previewSettings, 0, Qt::AlignTop);

        displaySettings = new QGroupBox(optionWidget);
        displaySettings->setObjectName(QString::fromUtf8("displaySettings"));
        verticalLayout_3 = new QVBoxLayout(displaySettings);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        antiAliasing = new QCheckBox(displaySettings);
        antiAliasing->setObjectName(QString::fromUtf8("antiAliasing"));

        verticalLayout_3->addWidget(antiAliasing);

        showTransparency = new QCheckBox(displaySettings);
        showTransparency->setObjectName(QString::fromUtf8("showTransparency"));

        verticalLayout_3->addWidget(showTransparency);

        enableCMYK = new QCheckBox(displaySettings);
        enableCMYK->setObjectName(QString::fromUtf8("enableCMYK"));

        verticalLayout_3->addWidget(enableCMYK);

        inkTable = new QTableWidget(displaySettings);
        inkTable->setObjectName(QString::fromUtf8("inkTable"));
        inkTable->setFocusPolicy(Qt::NoFocus);
        inkTable->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContentsOnFirstShow);
        inkTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
        inkTable->setSelectionMode(QAbstractItemView::NoSelection);
        inkTable->horizontalHeader()->setStretchLastSection(true);

        verticalLayout_3->addWidget(inkTable);

        displayInkCoverage = new QCheckBox(displaySettings);
        displayInkCoverage->setObjectName(QString::fromUtf8("displayInkCoverage"));

        verticalLayout_3->addWidget(displayInkCoverage);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        coverThresholdLabel = new QLabel(displaySettings);
        coverThresholdLabel->setObjectName(QString::fromUtf8("coverThresholdLabel"));

        horizontalLayout->addWidget(coverThresholdLabel);

        coverThresholdValue = new ScrSpinBox(displaySettings);
        coverThresholdValue->setObjectName(QString::fromUtf8("coverThresholdValue"));
        coverThresholdValue->setDecimals(0);
        coverThresholdValue->setMaximum(400.000000000000000);
        coverThresholdValue->setSingleStep(10.000000000000000);
        coverThresholdValue->setValue(300.000000000000000);

        horizontalLayout->addWidget(coverThresholdValue);


        verticalLayout_3->addLayout(horizontalLayout);


        verticalLayout_4->addWidget(displaySettings);


        settingsBarLayout->addWidget(optionWidget);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        settingsBarLayout->addItem(verticalSpacer);


        topHLayout->addLayout(settingsBarLayout);


        verticalLayout->addLayout(topHLayout);

        bottomHLayout = new QHBoxLayout();
        bottomHLayout->setObjectName(QString::fromUtf8("bottomHLayout"));
        scaleLabel = new QLabel(PrintPreviewBase);
        scaleLabel->setObjectName(QString::fromUtf8("scaleLabel"));

        bottomHLayout->addWidget(scaleLabel);

        scaleBox = new QComboBox(PrintPreviewBase);
        scaleBox->addItem(QString());
        scaleBox->addItem(QString());
        scaleBox->addItem(QString());
        scaleBox->addItem(QString());
        scaleBox->addItem(QString());
        scaleBox->addItem(QString());
        scaleBox->addItem(QString());
        scaleBox->setObjectName(QString::fromUtf8("scaleBox"));
        scaleBox->setFocusPolicy(Qt::NoFocus);

        bottomHLayout->addWidget(scaleBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        bottomHLayout->addItem(horizontalSpacer);

        pageSelector = new PageSelector(PrintPreviewBase);
        pageSelector->setObjectName(QString::fromUtf8("pageSelector"));
        QSizePolicy sizePolicy2(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(pageSelector->sizePolicy().hasHeightForWidth());
        pageSelector->setSizePolicy(sizePolicy2);

        bottomHLayout->addWidget(pageSelector);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        bottomHLayout->addItem(horizontalSpacer_2);

        closeButton = new QPushButton(PrintPreviewBase);
        closeButton->setObjectName(QString::fromUtf8("closeButton"));
        closeButton->setAutoDefault(false);

        bottomHLayout->addWidget(closeButton);

        printButton = new QPushButton(PrintPreviewBase);
        printButton->setObjectName(QString::fromUtf8("printButton"));
        printButton->setAutoDefault(false);

        bottomHLayout->addWidget(printButton);


        verticalLayout->addLayout(bottomHLayout);


        retranslateUi(PrintPreviewBase);

        QMetaObject::connectSlotsByName(PrintPreviewBase);
    } // setupUi

    void retranslateUi(QDialog *PrintPreviewBase)
    {
        PrintPreviewBase->setWindowTitle(QCoreApplication::translate("PrintPreviewBase", "Print Preview", nullptr));
        previewSettings->setTitle(QCoreApplication::translate("PrintPreviewBase", "Preview Settings", nullptr));
        colorOutputMode->setItemText(0, QCoreApplication::translate("PrintPreviewBase", "Color", nullptr));
        colorOutputMode->setItemText(1, QCoreApplication::translate("PrintPreviewBase", "Grayscale", nullptr));

        mirrorH->setText(QCoreApplication::translate("PrintPreviewBase", "Mirror Page(s) Horizontally", nullptr));
        mirrorV->setText(QCoreApplication::translate("PrintPreviewBase", "Mirror Page(s) Vertically", nullptr));
        clipToMargins->setText(QCoreApplication::translate("PrintPreviewBase", "Clip to Printer Margins", nullptr));
        convertSpots->setText(QCoreApplication::translate("PrintPreviewBase", "Convert Spot Colors", nullptr));
        enableGCR->setText(QCoreApplication::translate("PrintPreviewBase", "&Under Color Removal", nullptr));
        displaySettings->setTitle(QCoreApplication::translate("PrintPreviewBase", "Display Settings", nullptr));
        antiAliasing->setText(QCoreApplication::translate("PrintPreviewBase", "Enable &Antialiasing", nullptr));
        showTransparency->setText(QCoreApplication::translate("PrintPreviewBase", "Display Trans&parency", nullptr));
        enableCMYK->setText(QCoreApplication::translate("PrintPreviewBase", "&Display CMYK", nullptr));
        displayInkCoverage->setText(QCoreApplication::translate("PrintPreviewBase", "Display Ink Coverage", nullptr));
        coverThresholdLabel->setText(QCoreApplication::translate("PrintPreviewBase", "Threshold:", nullptr));
        scaleLabel->setText(QCoreApplication::translate("PrintPreviewBase", "Scaling:", nullptr));
        scaleBox->setItemText(0, QCoreApplication::translate("PrintPreviewBase", "50%", nullptr));
        scaleBox->setItemText(1, QCoreApplication::translate("PrintPreviewBase", "100%", nullptr));
        scaleBox->setItemText(2, QCoreApplication::translate("PrintPreviewBase", "150%", nullptr));
        scaleBox->setItemText(3, QCoreApplication::translate("PrintPreviewBase", "200%", nullptr));
        scaleBox->setItemText(4, QCoreApplication::translate("PrintPreviewBase", "Fit to Width", nullptr));
        scaleBox->setItemText(5, QCoreApplication::translate("PrintPreviewBase", "Fit to Height", nullptr));
        scaleBox->setItemText(6, QCoreApplication::translate("PrintPreviewBase", "Fit to Page", nullptr));

#if QT_CONFIG(tooltip)
        scaleBox->setToolTip(QCoreApplication::translate("PrintPreviewBase", "Resize the scale of the page", nullptr));
#endif // QT_CONFIG(tooltip)
        closeButton->setText(QCoreApplication::translate("PrintPreviewBase", "Close", nullptr));
        printButton->setText(QCoreApplication::translate("PrintPreviewBase", "Print...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PrintPreviewBase: public Ui_PrintPreviewBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PRINTPREVIEWBASE_H
