/********************************************************************************
** Form generated from reading UI file 'propertiespalette_shapebase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTIESPALETTE_SHAPEBASE_H
#define UI_PROPERTIESPALETTE_SHAPEBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/autoform.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_PropertiesPalette_ShapeBase
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *label_2;
    QHBoxLayout *horizontalLayout_5;
    QToolButton *textFlowDisabled;
    QToolButton *textFlowUsesFrameShape;
    QToolButton *textFlowUsesBoundingBox;
    QToolButton *textFlowUsesContourLine;
    QToolButton *textFlowUsesImageClipping;
    QSpacerItem *horizontalSpacer_4;
    QHBoxLayout *horizontalLayout_6;
    Autoforms *customShape;
    QToolButton *editShape;
    QSpacerItem *horizontalSpacer;
    QLabel *roundRectIcon;
    ScrSpinBox *roundRect;
    QLabel *label_3;
    QLabel *label_4;
    QHBoxLayout *horizontalLayout_2;
    QToolButton *evenOdd;
    QToolButton *nonZero;
    QSpacerItem *horizontalSpacer_3;
    QSpacerItem *verticalSpacer;
    QButtonGroup *textFlowBtnGroup;
    QButtonGroup *fillRuleBtnGroup;

    void setupUi(QWidget *PropertiesPalette_ShapeBase)
    {
        if (PropertiesPalette_ShapeBase->objectName().isEmpty())
            PropertiesPalette_ShapeBase->setObjectName(QString::fromUtf8("PropertiesPalette_ShapeBase"));
        PropertiesPalette_ShapeBase->resize(280, 95);
        verticalLayout = new QVBoxLayout(PropertiesPalette_ShapeBase);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(3);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label_2 = new QLabel(PropertiesPalette_ShapeBase);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setSpacing(3);
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        textFlowDisabled = new QToolButton(PropertiesPalette_ShapeBase);
        textFlowBtnGroup = new QButtonGroup(PropertiesPalette_ShapeBase);
        textFlowBtnGroup->setObjectName(QString::fromUtf8("textFlowBtnGroup"));
        textFlowBtnGroup->addButton(textFlowDisabled);
        textFlowDisabled->setObjectName(QString::fromUtf8("textFlowDisabled"));
        textFlowDisabled->setCheckable(true);
        textFlowDisabled->setAutoExclusive(true);
        textFlowDisabled->setToolButtonStyle(Qt::ToolButtonIconOnly);

        horizontalLayout_5->addWidget(textFlowDisabled);

        textFlowUsesFrameShape = new QToolButton(PropertiesPalette_ShapeBase);
        textFlowBtnGroup->addButton(textFlowUsesFrameShape);
        textFlowUsesFrameShape->setObjectName(QString::fromUtf8("textFlowUsesFrameShape"));
        textFlowUsesFrameShape->setCheckable(true);
        textFlowUsesFrameShape->setAutoExclusive(true);
        textFlowUsesFrameShape->setToolButtonStyle(Qt::ToolButtonIconOnly);

        horizontalLayout_5->addWidget(textFlowUsesFrameShape);

        textFlowUsesBoundingBox = new QToolButton(PropertiesPalette_ShapeBase);
        textFlowBtnGroup->addButton(textFlowUsesBoundingBox);
        textFlowUsesBoundingBox->setObjectName(QString::fromUtf8("textFlowUsesBoundingBox"));
        textFlowUsesBoundingBox->setCheckable(true);
        textFlowUsesBoundingBox->setAutoExclusive(true);
        textFlowUsesBoundingBox->setToolButtonStyle(Qt::ToolButtonIconOnly);

        horizontalLayout_5->addWidget(textFlowUsesBoundingBox);

        textFlowUsesContourLine = new QToolButton(PropertiesPalette_ShapeBase);
        textFlowBtnGroup->addButton(textFlowUsesContourLine);
        textFlowUsesContourLine->setObjectName(QString::fromUtf8("textFlowUsesContourLine"));
        textFlowUsesContourLine->setCheckable(true);
        textFlowUsesContourLine->setAutoExclusive(true);
        textFlowUsesContourLine->setToolButtonStyle(Qt::ToolButtonIconOnly);

        horizontalLayout_5->addWidget(textFlowUsesContourLine);

        textFlowUsesImageClipping = new QToolButton(PropertiesPalette_ShapeBase);
        textFlowBtnGroup->addButton(textFlowUsesImageClipping);
        textFlowUsesImageClipping->setObjectName(QString::fromUtf8("textFlowUsesImageClipping"));
        textFlowUsesImageClipping->setCheckable(true);
        textFlowUsesImageClipping->setAutoExclusive(true);
        textFlowUsesImageClipping->setToolButtonStyle(Qt::ToolButtonIconOnly);

        horizontalLayout_5->addWidget(textFlowUsesImageClipping);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_4);


        gridLayout->addLayout(horizontalLayout_5, 3, 1, 1, 1);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setSpacing(3);
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        customShape = new Autoforms(PropertiesPalette_ShapeBase);
        customShape->setObjectName(QString::fromUtf8("customShape"));

        horizontalLayout_6->addWidget(customShape);

        editShape = new QToolButton(PropertiesPalette_ShapeBase);
        editShape->setObjectName(QString::fromUtf8("editShape"));
        editShape->setMaximumSize(QSize(16777215, 23));

        horizontalLayout_6->addWidget(editShape);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer);

        roundRectIcon = new QLabel(PropertiesPalette_ShapeBase);
        roundRectIcon->setObjectName(QString::fromUtf8("roundRectIcon"));
        roundRectIcon->setMinimumSize(QSize(16, 16));
#if QT_CONFIG(tooltip)
        roundRectIcon->setToolTip(QString::fromUtf8(""));
#endif // QT_CONFIG(tooltip)

        horizontalLayout_6->addWidget(roundRectIcon);

        roundRect = new ScrSpinBox(PropertiesPalette_ShapeBase);
        roundRect->setObjectName(QString::fromUtf8("roundRect"));

        horizontalLayout_6->addWidget(roundRect);


        gridLayout->addLayout(horizontalLayout_6, 1, 1, 1, 1);

        label_3 = new QLabel(PropertiesPalette_ShapeBase);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 3, 0, 1, 1);

        label_4 = new QLabel(PropertiesPalette_ShapeBase);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout->addWidget(label_4, 5, 0, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(3);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        evenOdd = new QToolButton(PropertiesPalette_ShapeBase);
        fillRuleBtnGroup = new QButtonGroup(PropertiesPalette_ShapeBase);
        fillRuleBtnGroup->setObjectName(QString::fromUtf8("fillRuleBtnGroup"));
        fillRuleBtnGroup->addButton(evenOdd);
        evenOdd->setObjectName(QString::fromUtf8("evenOdd"));
        evenOdd->setCheckable(true);
        evenOdd->setAutoExclusive(true);

        horizontalLayout_2->addWidget(evenOdd);

        nonZero = new QToolButton(PropertiesPalette_ShapeBase);
        fillRuleBtnGroup->addButton(nonZero);
        nonZero->setObjectName(QString::fromUtf8("nonZero"));
        nonZero->setCheckable(true);
        nonZero->setAutoExclusive(true);

        horizontalLayout_2->addWidget(nonZero);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_3);


        gridLayout->addLayout(horizontalLayout_2, 5, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);


        retranslateUi(PropertiesPalette_ShapeBase);

        QMetaObject::connectSlotsByName(PropertiesPalette_ShapeBase);
    } // setupUi

    void retranslateUi(QWidget *PropertiesPalette_ShapeBase)
    {
        label_2->setText(QCoreApplication::translate("PropertiesPalette_ShapeBase", "Shape:", nullptr));
#if QT_CONFIG(tooltip)
        textFlowDisabled->setToolTip(QCoreApplication::translate("PropertiesPalette_ShapeBase", "Text flow around frame disabled", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        textFlowUsesFrameShape->setToolTip(QCoreApplication::translate("PropertiesPalette_ShapeBase", "Text flow around frame shape", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        textFlowUsesBoundingBox->setToolTip(QCoreApplication::translate("PropertiesPalette_ShapeBase", "Text flow around bounding box", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        textFlowUsesContourLine->setToolTip(QCoreApplication::translate("PropertiesPalette_ShapeBase", "Text flow around contour line", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        textFlowUsesImageClipping->setToolTip(QCoreApplication::translate("PropertiesPalette_ShapeBase", "Text flow around image clip path", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        customShape->setToolTip(QCoreApplication::translate("PropertiesPalette_ShapeBase", "Choose the shape of frame...", nullptr));
#endif // QT_CONFIG(tooltip)
        editShape->setText(QCoreApplication::translate("PropertiesPalette_ShapeBase", "&Edit...", nullptr));
        roundRectIcon->setText(QString());
#if QT_CONFIG(tooltip)
        roundRect->setToolTip(QCoreApplication::translate("PropertiesPalette_ShapeBase", "Set radius of corner rounding", nullptr));
#endif // QT_CONFIG(tooltip)
        label_3->setText(QCoreApplication::translate("PropertiesPalette_ShapeBase", "Text Flow:", nullptr));
        label_4->setText(QCoreApplication::translate("PropertiesPalette_ShapeBase", "Fill Rule:", nullptr));
#if QT_CONFIG(tooltip)
        evenOdd->setToolTip(QCoreApplication::translate("PropertiesPalette_ShapeBase", "Any path self-intersections or subpaths create holes in the fill (fill-rule: evenodd)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        nonZero->setToolTip(QCoreApplication::translate("PropertiesPalette_ShapeBase", "Fill is solid unless a subpath is counterdirectional  (fill-rule: nonzero)", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)PropertiesPalette_ShapeBase;
    } // retranslateUi

};

namespace Ui {
    class PropertiesPalette_ShapeBase: public Ui_PropertiesPalette_ShapeBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTIESPALETTE_SHAPEBASE_H
