/****************************************************************************
** Meta object code from reading C++ file 'newdocdialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/newdocdialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'newdocdialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PageLayoutsWidget_t {
    QByteArrayData data[1];
    char stringdata0[18];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PageLayoutsWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PageLayoutsWidget_t qt_meta_stringdata_PageLayoutsWidget = {
    {
QT_MOC_LITERAL(0, 0, 17) // "PageLayoutsWidget"

    },
    "PageLayoutsWidget"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PageLayoutsWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void PageLayoutsWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject PageLayoutsWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QListWidget::staticMetaObject>(),
    qt_meta_stringdata_PageLayoutsWidget.data,
    qt_meta_data_PageLayoutsWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PageLayoutsWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageLayoutsWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PageLayoutsWidget.stringdata0))
        return static_cast<void*>(this);
    return QListWidget::qt_metacast(_clname);
}

int PageLayoutsWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListWidget::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_NewDocDialog_t {
    QByteArrayData data[32];
    char stringdata0[371];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_NewDocDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_NewDocDialog_t qt_meta_stringdata_NewDocDialog = {
    {
QT_MOC_LITERAL(0, 0, 12), // "NewDocDialog"
QT_MOC_LITERAL(1, 13, 9), // "setHeight"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 1), // "v"
QT_MOC_LITERAL(4, 26, 8), // "setWidth"
QT_MOC_LITERAL(5, 35, 10), // "selectItem"
QT_MOC_LITERAL(6, 46, 2), // "nr"
QT_MOC_LITERAL(7, 49, 12), // "itemSelected"
QT_MOC_LITERAL(8, 62, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(9, 79, 2), // "ic"
QT_MOC_LITERAL(10, 82, 15), // "handleAutoFrame"
QT_MOC_LITERAL(11, 98, 11), // "setDistance"
QT_MOC_LITERAL(12, 110, 7), // "setUnit"
QT_MOC_LITERAL(13, 118, 1), // "u"
QT_MOC_LITERAL(14, 120, 6), // "ExitOK"
QT_MOC_LITERAL(15, 127, 14), // "setOrientation"
QT_MOC_LITERAL(16, 142, 3), // "ori"
QT_MOC_LITERAL(17, 146, 11), // "setPageSize"
QT_MOC_LITERAL(18, 158, 12), // "setDocLayout"
QT_MOC_LITERAL(19, 171, 6), // "layout"
QT_MOC_LITERAL(20, 178, 30), // "recentDocListBox_doubleClicked"
QT_MOC_LITERAL(21, 209, 8), // "openFile"
QT_MOC_LITERAL(22, 218, 12), // "adjustTitles"
QT_MOC_LITERAL(23, 231, 3), // "tab"
QT_MOC_LITERAL(24, 235, 15), // "locationDropped"
QT_MOC_LITERAL(25, 251, 7), // "fileUrl"
QT_MOC_LITERAL(26, 259, 19), // "gotoParentDirectory"
QT_MOC_LITERAL(27, 279, 21), // "gotoSelectedDirectory"
QT_MOC_LITERAL(28, 301, 20), // "gotoDesktopDirectory"
QT_MOC_LITERAL(29, 322, 17), // "gotoHomeDirectory"
QT_MOC_LITERAL(30, 340, 25), // "openFileDialogFileClicked"
QT_MOC_LITERAL(31, 366, 4) // "path"

    },
    "NewDocDialog\0setHeight\0\0v\0setWidth\0"
    "selectItem\0nr\0itemSelected\0QListWidgetItem*\0"
    "ic\0handleAutoFrame\0setDistance\0setUnit\0"
    "u\0ExitOK\0setOrientation\0ori\0setPageSize\0"
    "setDocLayout\0layout\0recentDocListBox_doubleClicked\0"
    "openFile\0adjustTitles\0tab\0locationDropped\0"
    "fileUrl\0gotoParentDirectory\0"
    "gotoSelectedDirectory\0gotoDesktopDirectory\0"
    "gotoHomeDirectory\0openFileDialogFileClicked\0"
    "path"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_NewDocDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      20,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  114,    2, 0x0a /* Public */,
       4,    1,  117,    2, 0x0a /* Public */,
       5,    1,  120,    2, 0x0a /* Public */,
       7,    1,  123,    2, 0x0a /* Public */,
      10,    0,  126,    2, 0x0a /* Public */,
      11,    1,  127,    2, 0x0a /* Public */,
      12,    1,  130,    2, 0x0a /* Public */,
      14,    0,  133,    2, 0x0a /* Public */,
      15,    1,  134,    2, 0x0a /* Public */,
      17,    1,  137,    2, 0x0a /* Public */,
      18,    1,  140,    2, 0x0a /* Public */,
      20,    0,  143,    2, 0x0a /* Public */,
      21,    0,  144,    2, 0x0a /* Public */,
      22,    1,  145,    2, 0x0a /* Public */,
      24,    1,  148,    2, 0x0a /* Public */,
      26,    0,  151,    2, 0x0a /* Public */,
      27,    0,  152,    2, 0x0a /* Public */,
      28,    0,  153,    2, 0x0a /* Public */,
      29,    0,  154,    2, 0x0a /* Public */,
      30,    1,  155,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, QMetaType::Double,    3,
    QMetaType::Void, QMetaType::Double,    3,
    QMetaType::Void, QMetaType::UInt,    6,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,    3,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Int,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   23,
    QMetaType::Void, QMetaType::QString,   25,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   31,

       0        // eod
};

void NewDocDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NewDocDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setHeight((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 1: _t->setWidth((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 2: _t->selectItem((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 3: _t->itemSelected((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 4: _t->handleAutoFrame(); break;
        case 5: _t->setDistance((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 6: _t->setUnit((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->ExitOK(); break;
        case 8: _t->setOrientation((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->setPageSize((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->setDocLayout((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->recentDocListBox_doubleClicked(); break;
        case 12: _t->openFile(); break;
        case 13: _t->adjustTitles((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->locationDropped((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: _t->gotoParentDirectory(); break;
        case 16: _t->gotoSelectedDirectory(); break;
        case 17: _t->gotoDesktopDirectory(); break;
        case 18: _t->gotoHomeDirectory(); break;
        case 19: _t->openFileDialogFileClicked((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject NewDocDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_NewDocDialog.data,
    qt_meta_data_NewDocDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *NewDocDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NewDocDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_NewDocDialog.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int NewDocDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 20)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 20;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 20)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 20;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
