/********************************************************************************
** Form generated from reading UI file 'nftwidget.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NFTWIDGET_H
#define UI_NFTWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QToolBox>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_nftwidget
{
public:
    QHBoxLayout *horizontalLayout;
    QListWidget *categoryList;
    QListWidget *tnailGrid;
    QToolBox *toolBox;
    QWidget *aboutPage;
    QGridLayout *gridLayout;
    QTextBrowser *textBrowser;
    QWidget *previewPage;
    QGridLayout *_2;
    QListWidget *imageView;
    QWidget *helpPage;
    QGridLayout *_3;
    QTextBrowser *helpBrowser;

    void setupUi(QWidget *nftwidget)
    {
        if (nftwidget->objectName().isEmpty())
            nftwidget->setObjectName(QString::fromUtf8("nftwidget"));
        nftwidget->resize(628, 313);
        horizontalLayout = new QHBoxLayout(nftwidget);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(0, 0, 0, 0);
        categoryList = new QListWidget(nftwidget);
        categoryList->setObjectName(QString::fromUtf8("categoryList"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(categoryList->sizePolicy().hasHeightForWidth());
        categoryList->setSizePolicy(sizePolicy);
        categoryList->setMinimumSize(QSize(130, 200));
        categoryList->setMaximumSize(QSize(140, 16777215));

        horizontalLayout->addWidget(categoryList);

        tnailGrid = new QListWidget(nftwidget);
        tnailGrid->setObjectName(QString::fromUtf8("tnailGrid"));
        tnailGrid->setMinimumSize(QSize(250, 0));
        tnailGrid->setContextMenuPolicy(Qt::ActionsContextMenu);
        tnailGrid->setSpacing(5);
        tnailGrid->setViewMode(QListView::IconMode);

        horizontalLayout->addWidget(tnailGrid);

        toolBox = new QToolBox(nftwidget);
        toolBox->setObjectName(QString::fromUtf8("toolBox"));
        toolBox->setMinimumSize(QSize(220, 16));
        aboutPage = new QWidget();
        aboutPage->setObjectName(QString::fromUtf8("aboutPage"));
        aboutPage->setGeometry(QRect(0, 0, 220, 202));
        gridLayout = new QGridLayout(aboutPage);
        gridLayout->setSpacing(6);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        textBrowser = new QTextBrowser(aboutPage);
        textBrowser->setObjectName(QString::fromUtf8("textBrowser"));

        gridLayout->addWidget(textBrowser, 0, 0, 1, 1);

        toolBox->addItem(aboutPage, QString::fromUtf8("&About"));
        previewPage = new QWidget();
        previewPage->setObjectName(QString::fromUtf8("previewPage"));
        previewPage->setGeometry(QRect(0, 0, 112, 112));
        _2 = new QGridLayout(previewPage);
        _2->setSpacing(6);
        _2->setObjectName(QString::fromUtf8("_2"));
        _2->setContentsMargins(9, 9, 9, 9);
        imageView = new QListWidget(previewPage);
        imageView->setObjectName(QString::fromUtf8("imageView"));
        imageView->setViewMode(QListView::IconMode);

        _2->addWidget(imageView, 0, 0, 1, 1);

        toolBox->addItem(previewPage, QString::fromUtf8("&Preview"));
        helpPage = new QWidget();
        helpPage->setObjectName(QString::fromUtf8("helpPage"));
        helpPage->setGeometry(QRect(0, 0, 112, 112));
        _3 = new QGridLayout(helpPage);
        _3->setSpacing(6);
        _3->setObjectName(QString::fromUtf8("_3"));
        _3->setContentsMargins(9, 9, 9, 9);
        helpBrowser = new QTextBrowser(helpPage);
        helpBrowser->setObjectName(QString::fromUtf8("helpBrowser"));

        _3->addWidget(helpBrowser, 0, 0, 1, 1);

        toolBox->addItem(helpPage, QString::fromUtf8("&Help"));

        horizontalLayout->addWidget(toolBox);


        retranslateUi(nftwidget);

        toolBox->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(nftwidget);
    } // setupUi

    void retranslateUi(QWidget *nftwidget)
    {
        toolBox->setItemText(toolBox->indexOf(aboutPage), QCoreApplication::translate("nftwidget", "&About", nullptr));
        toolBox->setItemText(toolBox->indexOf(previewPage), QCoreApplication::translate("nftwidget", "&Preview", nullptr));
        toolBox->setItemText(toolBox->indexOf(helpPage), QCoreApplication::translate("nftwidget", "&Help", nullptr));
        (void)nftwidget;
    } // retranslateUi

};

namespace Ui {
    class nftwidget: public Ui_nftwidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_NFTWIDGET_H
