/****************************************************************************
** Meta object code from reading C++ file 'propertiespalette_text.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/propertiespalette_text.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertiespalette_text.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PropertiesPalette_Text_t {
    QByteArrayData data[59];
    char stringdata0[741];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PropertiesPalette_Text_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PropertiesPalette_Text_t qt_meta_stringdata_PropertiesPalette_Text = {
    {
QT_MOC_LITERAL(0, 0, 22), // "PropertiesPalette_Text"
QT_MOC_LITERAL(1, 23, 13), // "setMainWindow"
QT_MOC_LITERAL(2, 37, 0), // ""
QT_MOC_LITERAL(3, 38, 18), // "ScribusMainWindow*"
QT_MOC_LITERAL(4, 57, 2), // "mw"
QT_MOC_LITERAL(5, 60, 6), // "setDoc"
QT_MOC_LITERAL(6, 67, 11), // "ScribusDoc*"
QT_MOC_LITERAL(7, 79, 1), // "d"
QT_MOC_LITERAL(8, 81, 14), // "setCurrentItem"
QT_MOC_LITERAL(9, 96, 9), // "PageItem*"
QT_MOC_LITERAL(10, 106, 4), // "item"
QT_MOC_LITERAL(11, 111, 8), // "unsetDoc"
QT_MOC_LITERAL(12, 120, 9), // "unsetItem"
QT_MOC_LITERAL(13, 130, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(14, 153, 19), // "handleUpdateRequest"
QT_MOC_LITERAL(15, 173, 13), // "iconSetChange"
QT_MOC_LITERAL(16, 187, 14), // "languageChange"
QT_MOC_LITERAL(17, 202, 10), // "unitChange"
QT_MOC_LITERAL(18, 213, 12), // "localeChange"
QT_MOC_LITERAL(19, 226, 10), // "changeLang"
QT_MOC_LITERAL(20, 237, 2), // "id"
QT_MOC_LITERAL(21, 240, 13), // "showAlignment"
QT_MOC_LITERAL(22, 254, 1), // "e"
QT_MOC_LITERAL(23, 256, 13), // "showDirection"
QT_MOC_LITERAL(24, 270, 13), // "showCharStyle"
QT_MOC_LITERAL(25, 284, 4), // "name"
QT_MOC_LITERAL(26, 289, 12), // "showFontFace"
QT_MOC_LITERAL(27, 302, 12), // "showFontSize"
QT_MOC_LITERAL(28, 315, 1), // "s"
QT_MOC_LITERAL(29, 317, 12), // "showLanguage"
QT_MOC_LITERAL(30, 330, 1), // "w"
QT_MOC_LITERAL(31, 332, 19), // "showFirstLinePolicy"
QT_MOC_LITERAL(32, 352, 21), // "FirstLineOffsetPolicy"
QT_MOC_LITERAL(33, 374, 15), // "showLineSpacing"
QT_MOC_LITERAL(34, 390, 1), // "r"
QT_MOC_LITERAL(35, 392, 12), // "showParStyle"
QT_MOC_LITERAL(36, 405, 23), // "setupLineSpacingSpinbox"
QT_MOC_LITERAL(37, 429, 4), // "mode"
QT_MOC_LITERAL(38, 434, 5), // "value"
QT_MOC_LITERAL(39, 440, 15), // "updateCharStyle"
QT_MOC_LITERAL(40, 456, 9), // "CharStyle"
QT_MOC_LITERAL(41, 466, 9), // "charStyle"
QT_MOC_LITERAL(42, 476, 11), // "updateStyle"
QT_MOC_LITERAL(43, 488, 14), // "ParagraphStyle"
QT_MOC_LITERAL(44, 503, 10), // "newCurrent"
QT_MOC_LITERAL(45, 514, 16), // "updateCharStyles"
QT_MOC_LITERAL(46, 531, 21), // "updateParagraphStyles"
QT_MOC_LITERAL(47, 553, 16), // "updateTextStyles"
QT_MOC_LITERAL(48, 570, 21), // "handleLineSpacingMode"
QT_MOC_LITERAL(49, 592, 14), // "handleTextFont"
QT_MOC_LITERAL(50, 607, 16), // "updateTreeLayout"
QT_MOC_LITERAL(51, 624, 15), // "handleAlignment"
QT_MOC_LITERAL(52, 640, 1), // "a"
QT_MOC_LITERAL(53, 642, 15), // "handleDirection"
QT_MOC_LITERAL(54, 658, 21), // "handleFirstLinePolicy"
QT_MOC_LITERAL(55, 680, 14), // "handleFontSize"
QT_MOC_LITERAL(56, 695, 17), // "handleLineSpacing"
QT_MOC_LITERAL(57, 713, 13), // "doClearCStyle"
QT_MOC_LITERAL(58, 727, 13) // "doClearPStyle"

    },
    "PropertiesPalette_Text\0setMainWindow\0"
    "\0ScribusMainWindow*\0mw\0setDoc\0ScribusDoc*\0"
    "d\0setCurrentItem\0PageItem*\0item\0"
    "unsetDoc\0unsetItem\0handleSelectionChanged\0"
    "handleUpdateRequest\0iconSetChange\0"
    "languageChange\0unitChange\0localeChange\0"
    "changeLang\0id\0showAlignment\0e\0"
    "showDirection\0showCharStyle\0name\0"
    "showFontFace\0showFontSize\0s\0showLanguage\0"
    "w\0showFirstLinePolicy\0FirstLineOffsetPolicy\0"
    "showLineSpacing\0r\0showParStyle\0"
    "setupLineSpacingSpinbox\0mode\0value\0"
    "updateCharStyle\0CharStyle\0charStyle\0"
    "updateStyle\0ParagraphStyle\0newCurrent\0"
    "updateCharStyles\0updateParagraphStyles\0"
    "updateTextStyles\0handleLineSpacingMode\0"
    "handleTextFont\0updateTreeLayout\0"
    "handleAlignment\0a\0handleDirection\0"
    "handleFirstLinePolicy\0handleFontSize\0"
    "handleLineSpacing\0doClearCStyle\0"
    "doClearPStyle"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PropertiesPalette_Text[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      37,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  199,    2, 0x0a /* Public */,
       5,    1,  202,    2, 0x0a /* Public */,
       8,    1,  205,    2, 0x0a /* Public */,
      11,    0,  208,    2, 0x0a /* Public */,
      12,    0,  209,    2, 0x0a /* Public */,
      13,    0,  210,    2, 0x0a /* Public */,
      14,    1,  211,    2, 0x0a /* Public */,
      15,    0,  214,    2, 0x0a /* Public */,
      16,    0,  215,    2, 0x0a /* Public */,
      17,    0,  216,    2, 0x0a /* Public */,
      18,    0,  217,    2, 0x0a /* Public */,
      19,    1,  218,    2, 0x0a /* Public */,
      21,    1,  221,    2, 0x0a /* Public */,
      23,    1,  224,    2, 0x0a /* Public */,
      24,    1,  227,    2, 0x0a /* Public */,
      26,    1,  230,    2, 0x0a /* Public */,
      27,    1,  233,    2, 0x0a /* Public */,
      29,    1,  236,    2, 0x0a /* Public */,
      31,    1,  239,    2, 0x0a /* Public */,
      33,    1,  242,    2, 0x0a /* Public */,
      35,    1,  245,    2, 0x0a /* Public */,
      36,    2,  248,    2, 0x0a /* Public */,
      39,    1,  253,    2, 0x0a /* Public */,
      42,    1,  256,    2, 0x0a /* Public */,
      45,    0,  259,    2, 0x0a /* Public */,
      46,    0,  260,    2, 0x0a /* Public */,
      47,    0,  261,    2, 0x0a /* Public */,
      48,    1,  262,    2, 0x0a /* Public */,
      49,    1,  265,    2, 0x0a /* Public */,
      50,    0,  268,    2, 0x0a /* Public */,
      51,    1,  269,    2, 0x08 /* Private */,
      53,    1,  272,    2, 0x08 /* Private */,
      54,    1,  275,    2, 0x08 /* Private */,
      55,    0,  278,    2, 0x08 /* Private */,
      56,    0,  279,    2, 0x08 /* Private */,
      57,    0,  280,    2, 0x08 /* Private */,
      58,    0,  281,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void, QMetaType::Int,   22,
    QMetaType::Void, QMetaType::QString,   25,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Double,   28,
    QMetaType::Void, QMetaType::QString,   30,
    QMetaType::Void, 0x80000000 | 32,    2,
    QMetaType::Void, QMetaType::Double,   34,
    QMetaType::Void, QMetaType::QString,   25,
    QMetaType::Void, QMetaType::Int, QMetaType::Double,   37,   38,
    QMetaType::Void, 0x80000000 | 40,   41,
    QMetaType::Void, 0x80000000 | 43,   44,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   20,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   52,
    QMetaType::Void, QMetaType::Int,    7,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PropertiesPalette_Text::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertiesPalette_Text *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setMainWindow((*reinterpret_cast< ScribusMainWindow*(*)>(_a[1]))); break;
        case 1: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 2: _t->setCurrentItem((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 3: _t->unsetDoc(); break;
        case 4: _t->unsetItem(); break;
        case 5: _t->handleSelectionChanged(); break;
        case 6: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->iconSetChange(); break;
        case 8: _t->languageChange(); break;
        case 9: _t->unitChange(); break;
        case 10: _t->localeChange(); break;
        case 11: _t->changeLang((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->showAlignment((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->showDirection((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->showCharStyle((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: _t->showFontFace((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 16: _t->showFontSize((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 17: _t->showLanguage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 18: _t->showFirstLinePolicy((*reinterpret_cast< FirstLineOffsetPolicy(*)>(_a[1]))); break;
        case 19: _t->showLineSpacing((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 20: _t->showParStyle((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 21: _t->setupLineSpacingSpinbox((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 22: _t->updateCharStyle((*reinterpret_cast< const CharStyle(*)>(_a[1]))); break;
        case 23: _t->updateStyle((*reinterpret_cast< const ParagraphStyle(*)>(_a[1]))); break;
        case 24: _t->updateCharStyles(); break;
        case 25: _t->updateParagraphStyles(); break;
        case 26: _t->updateTextStyles(); break;
        case 27: _t->handleLineSpacingMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 28: _t->handleTextFont((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 29: _t->updateTreeLayout(); break;
        case 30: _t->handleAlignment((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 31: _t->handleDirection((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 32: _t->handleFirstLinePolicy((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 33: _t->handleFontSize(); break;
        case 34: _t->handleLineSpacing(); break;
        case 35: _t->doClearCStyle(); break;
        case 36: _t->doClearPStyle(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PropertiesPalette_Text::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_PropertiesPalette_Text.data,
    qt_meta_data_PropertiesPalette_Text,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PropertiesPalette_Text::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PropertiesPalette_Text::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PropertiesPalette_Text.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PropertiesPalette_TextBase"))
        return static_cast< Ui::PropertiesPalette_TextBase*>(this);
    return QWidget::qt_metacast(_clname);
}

int PropertiesPalette_Text::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 37)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 37;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 37)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 37;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
