/********************************************************************************
** Form generated from reading UI file 'selectobjects.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SELECTOBJECTS_H
#define UI_SELECTOBJECTS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>
#include "ui/colorcombo.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_selectDialog
{
public:
    QVBoxLayout *vboxLayout;
    QGroupBox *rangeGroup;
    QHBoxLayout *hboxLayout;
    QRadioButton *buttonCurrentPage;
    QRadioButton *buttonCurrentLayer;
    QRadioButton *buttonWorkspace;
    QGroupBox *attributeGroup;
    QGridLayout *gridLayout;
    QCheckBox *useItemType;
    QComboBox *itemTypeCombo;
    QCheckBox *useFillColor;
    ColorCombo *backgroundCombo;
    QCheckBox *useLineColor;
    ColorCombo *lineCombo;
    QCheckBox *useLineWidth;
    ScrSpinBox *lineWidthSpin;
    QCheckBox *usePrintState;
    QComboBox *printStateCombo;
    QCheckBox *useLockedState;
    QComboBox *lockStateCombo;
    QCheckBox *useResizeState;
    QComboBox *resizeStateCombo;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *selectDialog)
    {
        if (selectDialog->objectName().isEmpty())
            selectDialog->setObjectName(QString::fromUtf8("selectDialog"));
        selectDialog->resize(423, 321);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(selectDialog->sizePolicy().hasHeightForWidth());
        selectDialog->setSizePolicy(sizePolicy);
        selectDialog->setModal(true);
        vboxLayout = new QVBoxLayout(selectDialog);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        rangeGroup = new QGroupBox(selectDialog);
        rangeGroup->setObjectName(QString::fromUtf8("rangeGroup"));
        hboxLayout = new QHBoxLayout(rangeGroup);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        buttonCurrentPage = new QRadioButton(rangeGroup);
        buttonCurrentPage->setObjectName(QString::fromUtf8("buttonCurrentPage"));
        buttonCurrentPage->setChecked(true);

        hboxLayout->addWidget(buttonCurrentPage);

        buttonCurrentLayer = new QRadioButton(rangeGroup);
        buttonCurrentLayer->setObjectName(QString::fromUtf8("buttonCurrentLayer"));

        hboxLayout->addWidget(buttonCurrentLayer);

        buttonWorkspace = new QRadioButton(rangeGroup);
        buttonWorkspace->setObjectName(QString::fromUtf8("buttonWorkspace"));

        hboxLayout->addWidget(buttonWorkspace);


        vboxLayout->addWidget(rangeGroup);

        attributeGroup = new QGroupBox(selectDialog);
        attributeGroup->setObjectName(QString::fromUtf8("attributeGroup"));
        attributeGroup->setCheckable(true);
        attributeGroup->setChecked(false);
        gridLayout = new QGridLayout(attributeGroup);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        useItemType = new QCheckBox(attributeGroup);
        useItemType->setObjectName(QString::fromUtf8("useItemType"));

        gridLayout->addWidget(useItemType, 0, 0, 1, 1);

        itemTypeCombo = new QComboBox(attributeGroup);
        itemTypeCombo->addItem(QString());
        itemTypeCombo->addItem(QString());
        itemTypeCombo->addItem(QString());
        itemTypeCombo->addItem(QString());
        itemTypeCombo->addItem(QString());
        itemTypeCombo->addItem(QString());
        itemTypeCombo->addItem(QString());
        itemTypeCombo->setObjectName(QString::fromUtf8("itemTypeCombo"));
        itemTypeCombo->setEnabled(false);

        gridLayout->addWidget(itemTypeCombo, 0, 1, 1, 1);

        useFillColor = new QCheckBox(attributeGroup);
        useFillColor->setObjectName(QString::fromUtf8("useFillColor"));

        gridLayout->addWidget(useFillColor, 1, 0, 1, 1);

        backgroundCombo = new ColorCombo(attributeGroup);
        backgroundCombo->setObjectName(QString::fromUtf8("backgroundCombo"));
        backgroundCombo->setEnabled(false);

        gridLayout->addWidget(backgroundCombo, 1, 1, 1, 1);

        useLineColor = new QCheckBox(attributeGroup);
        useLineColor->setObjectName(QString::fromUtf8("useLineColor"));

        gridLayout->addWidget(useLineColor, 2, 0, 1, 1);

        lineCombo = new ColorCombo(attributeGroup);
        lineCombo->setObjectName(QString::fromUtf8("lineCombo"));
        lineCombo->setEnabled(false);

        gridLayout->addWidget(lineCombo, 2, 1, 1, 1);

        useLineWidth = new QCheckBox(attributeGroup);
        useLineWidth->setObjectName(QString::fromUtf8("useLineWidth"));

        gridLayout->addWidget(useLineWidth, 3, 0, 1, 1);

        lineWidthSpin = new ScrSpinBox(attributeGroup);
        lineWidthSpin->setObjectName(QString::fromUtf8("lineWidthSpin"));
        lineWidthSpin->setEnabled(false);

        gridLayout->addWidget(lineWidthSpin, 3, 1, 1, 1);

        usePrintState = new QCheckBox(attributeGroup);
        usePrintState->setObjectName(QString::fromUtf8("usePrintState"));

        gridLayout->addWidget(usePrintState, 4, 0, 1, 1);

        printStateCombo = new QComboBox(attributeGroup);
        printStateCombo->addItem(QString());
        printStateCombo->addItem(QString());
        printStateCombo->setObjectName(QString::fromUtf8("printStateCombo"));
        printStateCombo->setEnabled(false);

        gridLayout->addWidget(printStateCombo, 4, 1, 1, 1);

        useLockedState = new QCheckBox(attributeGroup);
        useLockedState->setObjectName(QString::fromUtf8("useLockedState"));

        gridLayout->addWidget(useLockedState, 5, 0, 1, 1);

        lockStateCombo = new QComboBox(attributeGroup);
        lockStateCombo->addItem(QString());
        lockStateCombo->addItem(QString());
        lockStateCombo->setObjectName(QString::fromUtf8("lockStateCombo"));
        lockStateCombo->setEnabled(false);

        gridLayout->addWidget(lockStateCombo, 5, 1, 1, 1);

        useResizeState = new QCheckBox(attributeGroup);
        useResizeState->setObjectName(QString::fromUtf8("useResizeState"));

        gridLayout->addWidget(useResizeState, 6, 0, 1, 1);

        resizeStateCombo = new QComboBox(attributeGroup);
        resizeStateCombo->addItem(QString());
        resizeStateCombo->addItem(QString());
        resizeStateCombo->setObjectName(QString::fromUtf8("resizeStateCombo"));
        resizeStateCombo->setEnabled(false);

        gridLayout->addWidget(resizeStateCombo, 6, 1, 1, 1);


        vboxLayout->addWidget(attributeGroup);

        buttonBox = new QDialogButtonBox(selectDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(selectDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), selectDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), selectDialog, SLOT(reject()));
        QObject::connect(useItemType, SIGNAL(clicked(bool)), itemTypeCombo, SLOT(setEnabled(bool)));
        QObject::connect(useFillColor, SIGNAL(clicked(bool)), backgroundCombo, SLOT(setEnabled(bool)));
        QObject::connect(useLineColor, SIGNAL(clicked(bool)), lineCombo, SLOT(setEnabled(bool)));
        QObject::connect(useLineWidth, SIGNAL(clicked(bool)), lineWidthSpin, SLOT(setEnabled(bool)));
        QObject::connect(usePrintState, SIGNAL(clicked(bool)), printStateCombo, SLOT(setEnabled(bool)));
        QObject::connect(useLockedState, SIGNAL(clicked(bool)), lockStateCombo, SLOT(setEnabled(bool)));
        QObject::connect(useResizeState, SIGNAL(clicked(bool)), resizeStateCombo, SLOT(setEnabled(bool)));

        lockStateCombo->setCurrentIndex(1);


        QMetaObject::connectSlotsByName(selectDialog);
    } // setupUi

    void retranslateUi(QDialog *selectDialog)
    {
        selectDialog->setWindowTitle(QCoreApplication::translate("selectDialog", "Select All Items", nullptr));
        rangeGroup->setTitle(QCoreApplication::translate("selectDialog", "Select All Items", nullptr));
#if QT_CONFIG(tooltip)
        buttonCurrentPage->setToolTip(QCoreApplication::translate("selectDialog", "Select all items on the current page", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonCurrentPage->setText(QCoreApplication::translate("selectDialog", "on Current Page", nullptr));
#if QT_CONFIG(tooltip)
        buttonCurrentLayer->setToolTip(QCoreApplication::translate("selectDialog", "Select all items on the current layer on all pages", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonCurrentLayer->setText(QCoreApplication::translate("selectDialog", "on Current Layer", nullptr));
#if QT_CONFIG(tooltip)
        buttonWorkspace->setToolTip(QCoreApplication::translate("selectDialog", "Select all items not on a page", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonWorkspace->setText(QCoreApplication::translate("selectDialog", "on the Scratch Space", nullptr));
#if QT_CONFIG(tooltip)
        attributeGroup->setToolTip(QCoreApplication::translate("selectDialog", "Narrow the selection of items based on various item properties", nullptr));
#endif // QT_CONFIG(tooltip)
        attributeGroup->setTitle(QCoreApplication::translate("selectDialog", "With the Following Attributes", nullptr));
#if QT_CONFIG(tooltip)
        useItemType->setToolTip(QCoreApplication::translate("selectDialog", "Select based on item type", nullptr));
#endif // QT_CONFIG(tooltip)
        useItemType->setText(QCoreApplication::translate("selectDialog", "Item Type", nullptr));
        itemTypeCombo->setItemText(0, QCoreApplication::translate("selectDialog", "Text Frame", nullptr));
        itemTypeCombo->setItemText(1, QCoreApplication::translate("selectDialog", "Image Frame", nullptr));
        itemTypeCombo->setItemText(2, QCoreApplication::translate("selectDialog", "Shape", nullptr));
        itemTypeCombo->setItemText(3, QCoreApplication::translate("selectDialog", "Polyline", nullptr));
        itemTypeCombo->setItemText(4, QCoreApplication::translate("selectDialog", "Line", nullptr));
        itemTypeCombo->setItemText(5, QCoreApplication::translate("selectDialog", "Render Frame", nullptr));
        itemTypeCombo->setItemText(6, QCoreApplication::translate("selectDialog", "Symbol", nullptr));

#if QT_CONFIG(tooltip)
        useFillColor->setToolTip(QCoreApplication::translate("selectDialog", "Select based on the color that the item is filled with", nullptr));
#endif // QT_CONFIG(tooltip)
        useFillColor->setText(QCoreApplication::translate("selectDialog", "Fill Color", nullptr));
#if QT_CONFIG(tooltip)
        useLineColor->setToolTip(QCoreApplication::translate("selectDialog", "Select based on the color of the line or outline", nullptr));
#endif // QT_CONFIG(tooltip)
        useLineColor->setText(QCoreApplication::translate("selectDialog", "Line Color", nullptr));
#if QT_CONFIG(tooltip)
        useLineWidth->setToolTip(QCoreApplication::translate("selectDialog", "Select based on the width of the line of the item", nullptr));
#endif // QT_CONFIG(tooltip)
        useLineWidth->setText(QCoreApplication::translate("selectDialog", "Line Width", nullptr));
#if QT_CONFIG(tooltip)
        usePrintState->setToolTip(QCoreApplication::translate("selectDialog", "Select items based on whether they will be printed or not", nullptr));
#endif // QT_CONFIG(tooltip)
        usePrintState->setText(QCoreApplication::translate("selectDialog", "Printable", nullptr));
        printStateCombo->setItemText(0, QCoreApplication::translate("selectDialog", "Yes", nullptr));
        printStateCombo->setItemText(1, QCoreApplication::translate("selectDialog", "No", nullptr));

#if QT_CONFIG(tooltip)
        useLockedState->setToolTip(QCoreApplication::translate("selectDialog", "Select items based on their locked status", nullptr));
#endif // QT_CONFIG(tooltip)
        useLockedState->setText(QCoreApplication::translate("selectDialog", "Locked", nullptr));
        lockStateCombo->setItemText(0, QCoreApplication::translate("selectDialog", "Yes", nullptr));
        lockStateCombo->setItemText(1, QCoreApplication::translate("selectDialog", "No", nullptr));

#if QT_CONFIG(tooltip)
        useResizeState->setToolTip(QCoreApplication::translate("selectDialog", "Select items based on whether they have their size locked or not", nullptr));
#endif // QT_CONFIG(tooltip)
        useResizeState->setText(QCoreApplication::translate("selectDialog", "Resizable", nullptr));
        resizeStateCombo->setItemText(0, QCoreApplication::translate("selectDialog", "Yes", nullptr));
        resizeStateCombo->setItemText(1, QCoreApplication::translate("selectDialog", "No", nullptr));

    } // retranslateUi

};

namespace Ui {
    class selectDialog: public Ui_selectDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SELECTOBJECTS_H
