/****************************************************************************
** Meta object code from reading C++ file 'propertiespalette_table.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/propertiespalette_table.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertiespalette_table.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PropertiesPalette_Table_t {
    QByteArrayData data[42];
    char stringdata0[739];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PropertiesPalette_Table_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PropertiesPalette_Table_t qt_meta_stringdata_PropertiesPalette_Table = {
    {
QT_MOC_LITERAL(0, 0, 23), // "PropertiesPalette_Table"
QT_MOC_LITERAL(1, 24, 13), // "setMainWindow"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 18), // "ScribusMainWindow*"
QT_MOC_LITERAL(4, 58, 10), // "mainWindow"
QT_MOC_LITERAL(5, 69, 11), // "setDocument"
QT_MOC_LITERAL(6, 81, 11), // "ScribusDoc*"
QT_MOC_LITERAL(7, 93, 3), // "doc"
QT_MOC_LITERAL(8, 97, 13), // "unsetDocument"
QT_MOC_LITERAL(9, 111, 7), // "setItem"
QT_MOC_LITERAL(10, 119, 9), // "PageItem*"
QT_MOC_LITERAL(11, 129, 4), // "item"
QT_MOC_LITERAL(12, 134, 9), // "unsetItem"
QT_MOC_LITERAL(13, 144, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(14, 167, 26), // "handleCellSelectionChanged"
QT_MOC_LITERAL(15, 194, 19), // "handleUpdateRequest"
QT_MOC_LITERAL(16, 214, 11), // "updateFlags"
QT_MOC_LITERAL(17, 226, 13), // "iconSetChange"
QT_MOC_LITERAL(18, 240, 14), // "languageChange"
QT_MOC_LITERAL(19, 255, 10), // "unitChange"
QT_MOC_LITERAL(20, 266, 18), // "updateFillControls"
QT_MOC_LITERAL(21, 285, 19), // "updateStyleControls"
QT_MOC_LITERAL(22, 305, 13), // "setTableStyle"
QT_MOC_LITERAL(23, 319, 4), // "name"
QT_MOC_LITERAL(24, 324, 12), // "setCellStyle"
QT_MOC_LITERAL(25, 337, 32), // "on_sideSelector_selectionChanged"
QT_MOC_LITERAL(26, 370, 35), // "on_borderLineList_currentRowC..."
QT_MOC_LITERAL(27, 406, 3), // "row"
QT_MOC_LITERAL(28, 410, 30), // "on_addBorderLineButton_clicked"
QT_MOC_LITERAL(29, 441, 33), // "on_removeBorderLineButton_cli..."
QT_MOC_LITERAL(30, 475, 31), // "on_borderLineShade_valueChanged"
QT_MOC_LITERAL(31, 507, 5), // "shade"
QT_MOC_LITERAL(32, 513, 31), // "on_borderLineWidth_valueChanged"
QT_MOC_LITERAL(33, 545, 5), // "width"
QT_MOC_LITERAL(34, 551, 28), // "on_borderLineColor_activated"
QT_MOC_LITERAL(35, 580, 9), // "colorName"
QT_MOC_LITERAL(36, 590, 28), // "on_borderLineStyle_activated"
QT_MOC_LITERAL(37, 619, 5), // "style"
QT_MOC_LITERAL(38, 625, 22), // "on_fillColor_activated"
QT_MOC_LITERAL(39, 648, 25), // "on_fillShade_valueChanged"
QT_MOC_LITERAL(40, 674, 32), // "on_buttonClearTableStyle_clicked"
QT_MOC_LITERAL(41, 707, 31) // "on_buttonClearCellStyle_clicked"

    },
    "PropertiesPalette_Table\0setMainWindow\0"
    "\0ScribusMainWindow*\0mainWindow\0"
    "setDocument\0ScribusDoc*\0doc\0unsetDocument\0"
    "setItem\0PageItem*\0item\0unsetItem\0"
    "handleSelectionChanged\0"
    "handleCellSelectionChanged\0"
    "handleUpdateRequest\0updateFlags\0"
    "iconSetChange\0languageChange\0unitChange\0"
    "updateFillControls\0updateStyleControls\0"
    "setTableStyle\0name\0setCellStyle\0"
    "on_sideSelector_selectionChanged\0"
    "on_borderLineList_currentRowChanged\0"
    "row\0on_addBorderLineButton_clicked\0"
    "on_removeBorderLineButton_clicked\0"
    "on_borderLineShade_valueChanged\0shade\0"
    "on_borderLineWidth_valueChanged\0width\0"
    "on_borderLineColor_activated\0colorName\0"
    "on_borderLineStyle_activated\0style\0"
    "on_fillColor_activated\0on_fillShade_valueChanged\0"
    "on_buttonClearTableStyle_clicked\0"
    "on_buttonClearCellStyle_clicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PropertiesPalette_Table[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      27,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  149,    2, 0x0a /* Public */,
       5,    1,  152,    2, 0x0a /* Public */,
       8,    0,  155,    2, 0x0a /* Public */,
       9,    1,  156,    2, 0x0a /* Public */,
      12,    0,  159,    2, 0x0a /* Public */,
      13,    0,  160,    2, 0x0a /* Public */,
      14,    0,  161,    2, 0x0a /* Public */,
      15,    1,  162,    2, 0x0a /* Public */,
      17,    0,  165,    2, 0x0a /* Public */,
      18,    0,  166,    2, 0x0a /* Public */,
      19,    0,  167,    2, 0x0a /* Public */,
      20,    0,  168,    2, 0x0a /* Public */,
      21,    0,  169,    2, 0x0a /* Public */,
      22,    1,  170,    2, 0x08 /* Private */,
      24,    1,  173,    2, 0x08 /* Private */,
      25,    0,  176,    2, 0x08 /* Private */,
      26,    1,  177,    2, 0x08 /* Private */,
      28,    0,  180,    2, 0x08 /* Private */,
      29,    0,  181,    2, 0x08 /* Private */,
      30,    1,  182,    2, 0x08 /* Private */,
      32,    1,  185,    2, 0x08 /* Private */,
      34,    1,  188,    2, 0x08 /* Private */,
      36,    1,  191,    2, 0x08 /* Private */,
      38,    1,  194,    2, 0x08 /* Private */,
      39,    1,  197,    2, 0x08 /* Private */,
      40,    0,  200,    2, 0x08 /* Private */,
      41,    0,  201,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   23,
    QMetaType::Void, QMetaType::QString,   23,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   27,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,   31,
    QMetaType::Void, QMetaType::Double,   33,
    QMetaType::Void, QMetaType::QString,   35,
    QMetaType::Void, QMetaType::Int,   37,
    QMetaType::Void, QMetaType::QString,   35,
    QMetaType::Void, QMetaType::Double,   31,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PropertiesPalette_Table::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertiesPalette_Table *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setMainWindow((*reinterpret_cast< ScribusMainWindow*(*)>(_a[1]))); break;
        case 1: _t->setDocument((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 2: _t->unsetDocument(); break;
        case 3: _t->setItem((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 4: _t->unsetItem(); break;
        case 5: _t->handleSelectionChanged(); break;
        case 6: _t->handleCellSelectionChanged(); break;
        case 7: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->iconSetChange(); break;
        case 9: _t->languageChange(); break;
        case 10: _t->unitChange(); break;
        case 11: _t->updateFillControls(); break;
        case 12: _t->updateStyleControls(); break;
        case 13: _t->setTableStyle((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 14: _t->setCellStyle((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: _t->on_sideSelector_selectionChanged(); break;
        case 16: _t->on_borderLineList_currentRowChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->on_addBorderLineButton_clicked(); break;
        case 18: _t->on_removeBorderLineButton_clicked(); break;
        case 19: _t->on_borderLineShade_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 20: _t->on_borderLineWidth_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 21: _t->on_borderLineColor_activated((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 22: _t->on_borderLineStyle_activated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 23: _t->on_fillColor_activated((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 24: _t->on_fillShade_valueChanged((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 25: _t->on_buttonClearTableStyle_clicked(); break;
        case 26: _t->on_buttonClearCellStyle_clicked(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PropertiesPalette_Table::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_PropertiesPalette_Table.data,
    qt_meta_data_PropertiesPalette_Table,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PropertiesPalette_Table::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PropertiesPalette_Table::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PropertiesPalette_Table.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PropertiesPalette_TableBase"))
        return static_cast< Ui::PropertiesPalette_TableBase*>(this);
    return QWidget::qt_metacast(_clname);
}

int PropertiesPalette_Table::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 27)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 27;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 27)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 27;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
