/********************************************************************************
** Form generated from reading UI file 'actionsearchdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ACTIONSEARCHDIALOG_H
#define UI_ACTIONSEARCHDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ActionSearchDialog
{
public:
    QVBoxLayout *verticalLayout;
    QLineEdit *filterLineEdit;
    QListWidget *actionsListWidget;

    void setupUi(QDialog *ActionSearchDialog)
    {
        if (ActionSearchDialog->objectName().isEmpty())
            ActionSearchDialog->setObjectName(QString::fromUtf8("ActionSearchDialog"));
        ActionSearchDialog->resize(400, 300);
        verticalLayout = new QVBoxLayout(ActionSearchDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        filterLineEdit = new QLineEdit(ActionSearchDialog);
        filterLineEdit->setObjectName(QString::fromUtf8("filterLineEdit"));

        verticalLayout->addWidget(filterLineEdit);

        actionsListWidget = new QListWidget(ActionSearchDialog);
        actionsListWidget->setObjectName(QString::fromUtf8("actionsListWidget"));

        verticalLayout->addWidget(actionsListWidget);


        retranslateUi(ActionSearchDialog);

        QMetaObject::connectSlotsByName(ActionSearchDialog);
    } // setupUi

    void retranslateUi(QDialog *ActionSearchDialog)
    {
        ActionSearchDialog->setWindowTitle(QCoreApplication::translate("ActionSearchDialog", "Action Search", nullptr));
        filterLineEdit->setPlaceholderText(QCoreApplication::translate("ActionSearchDialog", "Search for action...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ActionSearchDialog: public Ui_ActionSearchDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ACTIONSEARCHDIALOG_H
