/********************************************************************************
** Form generated from reading UI file 'propertiespalette_tablebase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTIESPALETTE_TABLEBASE_H
#define UI_PROPERTIESPALETTE_TABLEBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/colorcombo.h"
#include "ui/linecombo.h"
#include "ui/scrspinbox.h"
#include "ui/stylecombos.h"
#include "ui/tablesideselector.h"

QT_BEGIN_NAMESPACE

class Ui_PropertiesPalette_TableBase
{
public:
    QVBoxLayout *verticalLayout_2;
    QGroupBox *groupBox;
    QGridLayout *gridLayout;
    QLabel *labelTable;
    TableStyleComboBox *tableStyleCombo;
    QToolButton *buttonClearTableStyle;
    QLabel *labelCells;
    CellStyleComboBox *cellStyleCombo;
    QToolButton *buttonClearCellStyle;
    QGroupBox *fillLayout;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *fillLayout1;
    QFormLayout *fillLayout1_1;
    QLabel *fillColorLabel;
    ColorCombo *fillColor;
    QFormLayout *fillLayout1_2;
    QLabel *fillShadeLabel;
    ScrSpinBox *fillShade;
    QGroupBox *bordersLayout;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *bordesLayout1;
    QVBoxLayout *bordersLayout1_1;
    QToolButton *addBorderLineButton;
    QToolButton *removeBorderLineButton;
    QSpacerItem *buttonSpacer;
    QListWidget *borderLineList;
    TableSideSelector *sideSelector;
    QHBoxLayout *bordersLayout2;
    QFormLayout *bordersLayout2_1;
    QLabel *borderLineWidthLabel;
    QLabel *borderLineColorLabel;
    ColorCombo *borderLineColor;
    ScrSpinBox *borderLineWidth;
    QFormLayout *bordersLayout2_2;
    QLabel *borderLineStyleLabel;
    LineCombo *borderLineStyle;
    QLabel *borderLineShadeLabel;
    ScrSpinBox *borderLineShade;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *PropertiesPalette_TableBase)
    {
        if (PropertiesPalette_TableBase->objectName().isEmpty())
            PropertiesPalette_TableBase->setObjectName(QString::fromUtf8("PropertiesPalette_TableBase"));
        PropertiesPalette_TableBase->resize(352, 398);
        verticalLayout_2 = new QVBoxLayout(PropertiesPalette_TableBase);
        verticalLayout_2->setSpacing(3);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(3, 3, 3, 3);
        groupBox = new QGroupBox(PropertiesPalette_TableBase);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        gridLayout = new QGridLayout(groupBox);
        gridLayout->setSpacing(3);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(6, 6, 6, 6);
        labelTable = new QLabel(groupBox);
        labelTable->setObjectName(QString::fromUtf8("labelTable"));

        gridLayout->addWidget(labelTable, 0, 0, 1, 1);

        tableStyleCombo = new TableStyleComboBox(groupBox);
        tableStyleCombo->setObjectName(QString::fromUtf8("tableStyleCombo"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tableStyleCombo->sizePolicy().hasHeightForWidth());
        tableStyleCombo->setSizePolicy(sizePolicy);

        gridLayout->addWidget(tableStyleCombo, 0, 1, 1, 1);

        buttonClearTableStyle = new QToolButton(groupBox);
        buttonClearTableStyle->setObjectName(QString::fromUtf8("buttonClearTableStyle"));
        buttonClearTableStyle->setIconSize(QSize(16, 16));

        gridLayout->addWidget(buttonClearTableStyle, 0, 2, 1, 1);

        labelCells = new QLabel(groupBox);
        labelCells->setObjectName(QString::fromUtf8("labelCells"));

        gridLayout->addWidget(labelCells, 1, 0, 1, 1);

        cellStyleCombo = new CellStyleComboBox(groupBox);
        cellStyleCombo->setObjectName(QString::fromUtf8("cellStyleCombo"));
        sizePolicy.setHeightForWidth(cellStyleCombo->sizePolicy().hasHeightForWidth());
        cellStyleCombo->setSizePolicy(sizePolicy);

        gridLayout->addWidget(cellStyleCombo, 1, 1, 1, 1);

        buttonClearCellStyle = new QToolButton(groupBox);
        buttonClearCellStyle->setObjectName(QString::fromUtf8("buttonClearCellStyle"));
        buttonClearCellStyle->setIconSize(QSize(16, 16));

        gridLayout->addWidget(buttonClearCellStyle, 1, 2, 1, 1);


        verticalLayout_2->addWidget(groupBox);

        fillLayout = new QGroupBox(PropertiesPalette_TableBase);
        fillLayout->setObjectName(QString::fromUtf8("fillLayout"));
        verticalLayout_3 = new QVBoxLayout(fillLayout);
        verticalLayout_3->setSpacing(3);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(6, 6, 6, 6);
        fillLayout1 = new QHBoxLayout();
        fillLayout1->setSpacing(3);
        fillLayout1->setObjectName(QString::fromUtf8("fillLayout1"));
        fillLayout1_1 = new QFormLayout();
        fillLayout1_1->setObjectName(QString::fromUtf8("fillLayout1_1"));
        fillLayout1_1->setHorizontalSpacing(3);
        fillLayout1_1->setVerticalSpacing(3);
        fillColorLabel = new QLabel(fillLayout);
        fillColorLabel->setObjectName(QString::fromUtf8("fillColorLabel"));
        fillColorLabel->setEnabled(false);

        fillLayout1_1->setWidget(0, QFormLayout::LabelRole, fillColorLabel);

        fillColor = new ColorCombo(fillLayout);
        fillColor->setObjectName(QString::fromUtf8("fillColor"));
        fillColor->setEnabled(false);
        QSizePolicy sizePolicy1(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(fillColor->sizePolicy().hasHeightForWidth());
        fillColor->setSizePolicy(sizePolicy1);
        fillColor->setMinimumSize(QSize(100, 0));

        fillLayout1_1->setWidget(0, QFormLayout::FieldRole, fillColor);


        fillLayout1->addLayout(fillLayout1_1);

        fillLayout1_2 = new QFormLayout();
        fillLayout1_2->setObjectName(QString::fromUtf8("fillLayout1_2"));
        fillLayout1_2->setHorizontalSpacing(3);
        fillLayout1_2->setVerticalSpacing(3);
        fillShadeLabel = new QLabel(fillLayout);
        fillShadeLabel->setObjectName(QString::fromUtf8("fillShadeLabel"));
        fillShadeLabel->setEnabled(false);

        fillLayout1_2->setWidget(0, QFormLayout::LabelRole, fillShadeLabel);

        fillShade = new ScrSpinBox(fillLayout);
        fillShade->setObjectName(QString::fromUtf8("fillShade"));
        fillShade->setEnabled(false);
        sizePolicy1.setHeightForWidth(fillShade->sizePolicy().hasHeightForWidth());
        fillShade->setSizePolicy(sizePolicy1);
        fillShade->setMinimumSize(QSize(100, 0));
        fillShade->setDecimals(0);
        fillShade->setMaximum(100.000000000000000);
        fillShade->setValue(100.000000000000000);

        fillLayout1_2->setWidget(0, QFormLayout::FieldRole, fillShade);


        fillLayout1->addLayout(fillLayout1_2);


        verticalLayout_3->addLayout(fillLayout1);


        verticalLayout_2->addWidget(fillLayout);

        bordersLayout = new QGroupBox(PropertiesPalette_TableBase);
        bordersLayout->setObjectName(QString::fromUtf8("bordersLayout"));
        verticalLayout = new QVBoxLayout(bordersLayout);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(6, 6, 6, 6);
        bordesLayout1 = new QHBoxLayout();
        bordesLayout1->setSpacing(3);
        bordesLayout1->setObjectName(QString::fromUtf8("bordesLayout1"));
        bordersLayout1_1 = new QVBoxLayout();
        bordersLayout1_1->setSpacing(3);
        bordersLayout1_1->setObjectName(QString::fromUtf8("bordersLayout1_1"));
        addBorderLineButton = new QToolButton(bordersLayout);
        addBorderLineButton->setObjectName(QString::fromUtf8("addBorderLineButton"));
        addBorderLineButton->setEnabled(true);
        QSizePolicy sizePolicy2(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(addBorderLineButton->sizePolicy().hasHeightForWidth());
        addBorderLineButton->setSizePolicy(sizePolicy2);

        bordersLayout1_1->addWidget(addBorderLineButton);

        removeBorderLineButton = new QToolButton(bordersLayout);
        removeBorderLineButton->setObjectName(QString::fromUtf8("removeBorderLineButton"));
        removeBorderLineButton->setEnabled(true);
        sizePolicy2.setHeightForWidth(removeBorderLineButton->sizePolicy().hasHeightForWidth());
        removeBorderLineButton->setSizePolicy(sizePolicy2);

        bordersLayout1_1->addWidget(removeBorderLineButton);

        buttonSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Minimum);

        bordersLayout1_1->addItem(buttonSpacer);


        bordesLayout1->addLayout(bordersLayout1_1);

        borderLineList = new QListWidget(bordersLayout);
        borderLineList->setObjectName(QString::fromUtf8("borderLineList"));
        borderLineList->setEnabled(true);
        QSizePolicy sizePolicy3(QSizePolicy::Preferred, QSizePolicy::Minimum);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(borderLineList->sizePolicy().hasHeightForWidth());
        borderLineList->setSizePolicy(sizePolicy3);
        borderLineList->setMinimumSize(QSize(0, 50));

        bordesLayout1->addWidget(borderLineList);

        sideSelector = new TableSideSelector(bordersLayout);
        sideSelector->setObjectName(QString::fromUtf8("sideSelector"));
        sizePolicy2.setHeightForWidth(sideSelector->sizePolicy().hasHeightForWidth());
        sideSelector->setSizePolicy(sizePolicy2);
        sideSelector->setMinimumSize(QSize(80, 80));
        sideSelector->setFrameShape(QFrame::StyledPanel);
        sideSelector->setFrameShadow(QFrame::Sunken);

        bordesLayout1->addWidget(sideSelector);


        verticalLayout->addLayout(bordesLayout1);

        bordersLayout2 = new QHBoxLayout();
        bordersLayout2->setSpacing(3);
        bordersLayout2->setObjectName(QString::fromUtf8("bordersLayout2"));
        bordersLayout2_1 = new QFormLayout();
        bordersLayout2_1->setObjectName(QString::fromUtf8("bordersLayout2_1"));
        bordersLayout2_1->setFieldGrowthPolicy(QFormLayout::ExpandingFieldsGrow);
        bordersLayout2_1->setHorizontalSpacing(3);
        bordersLayout2_1->setVerticalSpacing(3);
        borderLineWidthLabel = new QLabel(bordersLayout);
        borderLineWidthLabel->setObjectName(QString::fromUtf8("borderLineWidthLabel"));
        borderLineWidthLabel->setEnabled(false);

        bordersLayout2_1->setWidget(0, QFormLayout::LabelRole, borderLineWidthLabel);

        borderLineColorLabel = new QLabel(bordersLayout);
        borderLineColorLabel->setObjectName(QString::fromUtf8("borderLineColorLabel"));
        borderLineColorLabel->setEnabled(false);

        bordersLayout2_1->setWidget(1, QFormLayout::LabelRole, borderLineColorLabel);

        borderLineColor = new ColorCombo(bordersLayout);
        borderLineColor->setObjectName(QString::fromUtf8("borderLineColor"));
        borderLineColor->setEnabled(false);
        sizePolicy1.setHeightForWidth(borderLineColor->sizePolicy().hasHeightForWidth());
        borderLineColor->setSizePolicy(sizePolicy1);
        borderLineColor->setMinimumSize(QSize(100, 0));

        bordersLayout2_1->setWidget(1, QFormLayout::FieldRole, borderLineColor);

        borderLineWidth = new ScrSpinBox(bordersLayout);
        borderLineWidth->setObjectName(QString::fromUtf8("borderLineWidth"));
        borderLineWidth->setEnabled(false);
        sizePolicy1.setHeightForWidth(borderLineWidth->sizePolicy().hasHeightForWidth());
        borderLineWidth->setSizePolicy(sizePolicy1);
        borderLineWidth->setMinimumSize(QSize(100, 0));

        bordersLayout2_1->setWidget(0, QFormLayout::FieldRole, borderLineWidth);


        bordersLayout2->addLayout(bordersLayout2_1);

        bordersLayout2_2 = new QFormLayout();
        bordersLayout2_2->setObjectName(QString::fromUtf8("bordersLayout2_2"));
        bordersLayout2_2->setHorizontalSpacing(3);
        bordersLayout2_2->setVerticalSpacing(3);
        borderLineStyleLabel = new QLabel(bordersLayout);
        borderLineStyleLabel->setObjectName(QString::fromUtf8("borderLineStyleLabel"));
        borderLineStyleLabel->setEnabled(false);

        bordersLayout2_2->setWidget(0, QFormLayout::LabelRole, borderLineStyleLabel);

        borderLineStyle = new LineCombo(bordersLayout);
        borderLineStyle->setObjectName(QString::fromUtf8("borderLineStyle"));
        borderLineStyle->setEnabled(false);
        sizePolicy1.setHeightForWidth(borderLineStyle->sizePolicy().hasHeightForWidth());
        borderLineStyle->setSizePolicy(sizePolicy1);
        borderLineStyle->setMinimumSize(QSize(100, 0));

        bordersLayout2_2->setWidget(0, QFormLayout::FieldRole, borderLineStyle);

        borderLineShadeLabel = new QLabel(bordersLayout);
        borderLineShadeLabel->setObjectName(QString::fromUtf8("borderLineShadeLabel"));
        borderLineShadeLabel->setEnabled(false);

        bordersLayout2_2->setWidget(1, QFormLayout::LabelRole, borderLineShadeLabel);

        borderLineShade = new ScrSpinBox(bordersLayout);
        borderLineShade->setObjectName(QString::fromUtf8("borderLineShade"));
        borderLineShade->setEnabled(false);
        sizePolicy1.setHeightForWidth(borderLineShade->sizePolicy().hasHeightForWidth());
        borderLineShade->setSizePolicy(sizePolicy1);
        borderLineShade->setMinimumSize(QSize(100, 0));
        borderLineShade->setDecimals(0);
        borderLineShade->setMaximum(100.000000000000000);
        borderLineShade->setValue(100.000000000000000);

        bordersLayout2_2->setWidget(1, QFormLayout::FieldRole, borderLineShade);


        bordersLayout2->addLayout(bordersLayout2_2);


        verticalLayout->addLayout(bordersLayout2);


        verticalLayout_2->addWidget(bordersLayout);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        borderLineWidthLabel->setBuddy(borderLineWidth);
        borderLineColorLabel->setBuddy(borderLineColor);
        borderLineStyleLabel->setBuddy(borderLineStyle);
        borderLineShadeLabel->setBuddy(borderLineShade);
#endif // QT_CONFIG(shortcut)

        retranslateUi(PropertiesPalette_TableBase);

        QMetaObject::connectSlotsByName(PropertiesPalette_TableBase);
    } // setupUi

    void retranslateUi(QWidget *PropertiesPalette_TableBase)
    {
        groupBox->setTitle(QCoreApplication::translate("PropertiesPalette_TableBase", "Styles", nullptr));
        labelTable->setText(QCoreApplication::translate("PropertiesPalette_TableBase", "Table:", nullptr));
#if QT_CONFIG(tooltip)
        tableStyleCombo->setToolTip(QCoreApplication::translate("PropertiesPalette_TableBase", "Table style of currently selected table", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonClearTableStyle->setToolTip(QCoreApplication::translate("PropertiesPalette_TableBase", "Remove Direct Table Formatting", nullptr));
#endif // QT_CONFIG(tooltip)
        labelCells->setText(QCoreApplication::translate("PropertiesPalette_TableBase", "Cells:", nullptr));
#if QT_CONFIG(tooltip)
        cellStyleCombo->setToolTip(QCoreApplication::translate("PropertiesPalette_TableBase", "Cell style of currently selected cell", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        buttonClearCellStyle->setToolTip(QCoreApplication::translate("PropertiesPalette_TableBase", "Remove Direct Cell Formatting", nullptr));
#endif // QT_CONFIG(tooltip)
        fillLayout->setTitle(QCoreApplication::translate("PropertiesPalette_TableBase", "Fill", nullptr));
        fillColorLabel->setText(QCoreApplication::translate("PropertiesPalette_TableBase", "Color:", nullptr));
        fillShadeLabel->setText(QCoreApplication::translate("PropertiesPalette_TableBase", "Shade:", nullptr));
        fillShade->setSuffix(QCoreApplication::translate("PropertiesPalette_TableBase", " %", nullptr));
        bordersLayout->setTitle(QCoreApplication::translate("PropertiesPalette_TableBase", "Borders", nullptr));
        borderLineWidthLabel->setText(QCoreApplication::translate("PropertiesPalette_TableBase", "Width:", nullptr));
        borderLineColorLabel->setText(QCoreApplication::translate("PropertiesPalette_TableBase", "Color:", nullptr));
        borderLineStyleLabel->setText(QCoreApplication::translate("PropertiesPalette_TableBase", "Type:", nullptr));
        borderLineShadeLabel->setText(QCoreApplication::translate("PropertiesPalette_TableBase", "Shade:", nullptr));
        borderLineShade->setSuffix(QCoreApplication::translate("PropertiesPalette_TableBase", " %", nullptr));
        (void)PropertiesPalette_TableBase;
    } // retranslateUi

};

namespace Ui {
    class PropertiesPalette_TableBase: public Ui_PropertiesPalette_TableBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTIESPALETTE_TABLEBASE_H
