/********************************************************************************
** Form generated from reading UI file 'cxfimportdialogbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CXFIMPORTDIALOGBASE_H
#define UI_CXFIMPORTDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_CxfImportDialogBase
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QListWidget *colorspaceListBox;
    QVBoxLayout *verticalLayout_2;
    QPushButton *buttonUp;
    QPushButton *buttonDown;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *CxfImportDialogBase)
    {
        if (CxfImportDialogBase->objectName().isEmpty())
            CxfImportDialogBase->setObjectName(QString::fromUtf8("CxfImportDialogBase"));
        CxfImportDialogBase->resize(372, 300);
        verticalLayout = new QVBoxLayout(CxfImportDialogBase);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        label = new QLabel(CxfImportDialogBase);
        label->setObjectName(QString::fromUtf8("label"));
        label->setTextFormat(Qt::PlainText);
        label->setWordWrap(true);

        verticalLayout->addWidget(label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        colorspaceListBox = new QListWidget(CxfImportDialogBase);
        colorspaceListBox->setObjectName(QString::fromUtf8("colorspaceListBox"));

        horizontalLayout->addWidget(colorspaceListBox);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        buttonUp = new QPushButton(CxfImportDialogBase);
        buttonUp->setObjectName(QString::fromUtf8("buttonUp"));
        buttonUp->setEnabled(false);
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(buttonUp->sizePolicy().hasHeightForWidth());
        buttonUp->setSizePolicy(sizePolicy);

        verticalLayout_2->addWidget(buttonUp);

        buttonDown = new QPushButton(CxfImportDialogBase);
        buttonDown->setObjectName(QString::fromUtf8("buttonDown"));
        buttonDown->setEnabled(false);
        sizePolicy.setHeightForWidth(buttonDown->sizePolicy().hasHeightForWidth());
        buttonDown->setSizePolicy(sizePolicy);

        verticalLayout_2->addWidget(buttonDown);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        horizontalLayout->addLayout(verticalLayout_2);


        verticalLayout->addLayout(horizontalLayout);

        buttonBox = new QDialogButtonBox(CxfImportDialogBase);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(CxfImportDialogBase);
        QObject::connect(buttonBox, SIGNAL(accepted()), CxfImportDialogBase, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), CxfImportDialogBase, SLOT(reject()));

        QMetaObject::connectSlotsByName(CxfImportDialogBase);
    } // setupUi

    void retranslateUi(QDialog *CxfImportDialogBase)
    {
        CxfImportDialogBase->setWindowTitle(QCoreApplication::translate("CxfImportDialogBase", "CxF Import", nullptr));
        label->setText(QCoreApplication::translate("CxfImportDialogBase", "The file you are trying to import contains multiple definitions for colors. Please select how you would like colors to be imported by priority:", nullptr));
        buttonUp->setText(QString());
        buttonDown->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class CxfImportDialogBase: public Ui_CxfImportDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CXFIMPORTDIALOGBASE_H
