/********************************************************************************
** Form generated from reading UI file 'multipleduplicate.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MULTIPLEDUPLICATE_H
#define UI_MULTIPLEDUPLICATE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/scrspinbox.h>
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_MultipleDuplicate
{
public:
    QVBoxLayout *vboxLayout;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout_2;
    ScrSpinBox *rotationSpinBox;
    QLabel *numberOfCopiesLabel;
    QRadioButton *shiftCreatedItemsRadioButton;
    QSpinBox *numberOfCopiesSpinBox;
    ScrSpinBox *vertShiftSpinBox;
    ScrSpinBox *horizShiftSpinBox;
    QLabel *vertShiftLabel;
    QLabel *rotationLabel;
    QRadioButton *createGapRadioButton;
    QLabel *horizShiftLabel;
    QSpacerItem *verticalSpacer_2;
    QWidget *tab2;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout;
    QLabel *numberOfColumnsLabel;
    ScrSpinBox *horizRCGapSpinBox;
    QLabel *numberOfRowsLabel;
    QSpinBox *gridRowsSpinBox;
    QSpinBox *gridColsSpinBox;
    QLabel *horizRCLabel;
    QLabel *vertRCLabel;
    ScrSpinBox *vertRCGapSpinBox;
    QSpacerItem *verticalSpacer;
    QWidget *tab3;
    QVBoxLayout *verticalLayout_8;
    QVBoxLayout *verticalLayout_7;
    QRadioButton *radioButtonPageAll;
    QRadioButton *radioButtonPageEven;
    QRadioButton *radioButtonPageOdd;
    QRadioButton *radioButtonPageRange;
    QHBoxLayout *horizontalLayout;
    QLineEdit *lineEditPageRange;
    QToolButton *toolButtonPageRange;
    QVBoxLayout *verticalLayout_9;
    QCheckBox *checkBoxPageLinkText;
    QSpacerItem *verticalSpacer_5;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *buttonOk;
    QPushButton *buttonCancel;
    QButtonGroup *buttonGroup;

    void setupUi(QDialog *MultipleDuplicate)
    {
        if (MultipleDuplicate->objectName().isEmpty())
            MultipleDuplicate->setObjectName(QString::fromUtf8("MultipleDuplicate"));
        MultipleDuplicate->setEnabled(true);
        MultipleDuplicate->resize(426, 291);
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(MultipleDuplicate->sizePolicy().hasHeightForWidth());
        MultipleDuplicate->setSizePolicy(sizePolicy);
        MultipleDuplicate->setSizeGripEnabled(false);
        vboxLayout = new QVBoxLayout(MultipleDuplicate);
        vboxLayout->setSpacing(6);
        vboxLayout->setContentsMargins(11, 11, 11, 11);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setSizeConstraint(QLayout::SetMinimumSize);
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        tabWidget = new QTabWidget(MultipleDuplicate);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::MinimumExpanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(tabWidget->sizePolicy().hasHeightForWidth());
        tabWidget->setSizePolicy(sizePolicy1);
        tabWidget->setTabPosition(QTabWidget::North);
        tabWidget->setElideMode(Qt::ElideNone);
        tabWidget->setUsesScrollButtons(false);
        tab = new QWidget();
        tab->setObjectName(QString::fromUtf8("tab"));
        verticalLayout = new QVBoxLayout(tab);
        verticalLayout->setSpacing(6);
        verticalLayout->setContentsMargins(11, 11, 11, 11);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setSpacing(6);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        rotationSpinBox = new ScrSpinBox(tab);
        rotationSpinBox->setObjectName(QString::fromUtf8("rotationSpinBox"));
        rotationSpinBox->setMinimumSize(QSize(0, 23));

        gridLayout_2->addWidget(rotationSpinBox, 5, 1, 1, 1);

        numberOfCopiesLabel = new QLabel(tab);
        numberOfCopiesLabel->setObjectName(QString::fromUtf8("numberOfCopiesLabel"));
        numberOfCopiesLabel->setWordWrap(false);

        gridLayout_2->addWidget(numberOfCopiesLabel, 0, 0, 1, 1);

        shiftCreatedItemsRadioButton = new QRadioButton(tab);
        shiftCreatedItemsRadioButton->setObjectName(QString::fromUtf8("shiftCreatedItemsRadioButton"));

        gridLayout_2->addWidget(shiftCreatedItemsRadioButton, 2, 0, 1, 2);

        numberOfCopiesSpinBox = new QSpinBox(tab);
        numberOfCopiesSpinBox->setObjectName(QString::fromUtf8("numberOfCopiesSpinBox"));
        numberOfCopiesSpinBox->setMinimumSize(QSize(0, 23));
        numberOfCopiesSpinBox->setMinimum(1);
        numberOfCopiesSpinBox->setMaximum(999);

        gridLayout_2->addWidget(numberOfCopiesSpinBox, 0, 1, 1, 1);

        vertShiftSpinBox = new ScrSpinBox(tab);
        vertShiftSpinBox->setObjectName(QString::fromUtf8("vertShiftSpinBox"));
        vertShiftSpinBox->setMinimumSize(QSize(0, 23));

        gridLayout_2->addWidget(vertShiftSpinBox, 4, 1, 1, 1);

        horizShiftSpinBox = new ScrSpinBox(tab);
        horizShiftSpinBox->setObjectName(QString::fromUtf8("horizShiftSpinBox"));
        horizShiftSpinBox->setMinimumSize(QSize(0, 23));

        gridLayout_2->addWidget(horizShiftSpinBox, 3, 1, 1, 1);

        vertShiftLabel = new QLabel(tab);
        vertShiftLabel->setObjectName(QString::fromUtf8("vertShiftLabel"));
        vertShiftLabel->setWordWrap(false);

        gridLayout_2->addWidget(vertShiftLabel, 4, 0, 1, 1);

        rotationLabel = new QLabel(tab);
        rotationLabel->setObjectName(QString::fromUtf8("rotationLabel"));
        rotationLabel->setWordWrap(false);

        gridLayout_2->addWidget(rotationLabel, 5, 0, 1, 1);

        createGapRadioButton = new QRadioButton(tab);
        createGapRadioButton->setObjectName(QString::fromUtf8("createGapRadioButton"));

        gridLayout_2->addWidget(createGapRadioButton, 1, 0, 1, 2);

        horizShiftLabel = new QLabel(tab);
        horizShiftLabel->setObjectName(QString::fromUtf8("horizShiftLabel"));
        horizShiftLabel->setWordWrap(false);

        gridLayout_2->addWidget(horizShiftLabel, 3, 0, 1, 1);


        verticalLayout->addLayout(gridLayout_2);

        verticalSpacer_2 = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);

        tabWidget->addTab(tab, QString());
        tab2 = new QWidget();
        tab2->setObjectName(QString::fromUtf8("tab2"));
        verticalLayout_2 = new QVBoxLayout(tab2);
        verticalLayout_2->setSpacing(6);
        verticalLayout_2->setContentsMargins(11, 11, 11, 11);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(6);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        numberOfColumnsLabel = new QLabel(tab2);
        numberOfColumnsLabel->setObjectName(QString::fromUtf8("numberOfColumnsLabel"));
        numberOfColumnsLabel->setWordWrap(false);

        gridLayout->addWidget(numberOfColumnsLabel, 1, 0, 1, 1);

        horizRCGapSpinBox = new ScrSpinBox(tab2);
        horizRCGapSpinBox->setObjectName(QString::fromUtf8("horizRCGapSpinBox"));
        QSizePolicy sizePolicy2(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(horizRCGapSpinBox->sizePolicy().hasHeightForWidth());
        horizRCGapSpinBox->setSizePolicy(sizePolicy2);
        horizRCGapSpinBox->setMinimumSize(QSize(0, 23));

        gridLayout->addWidget(horizRCGapSpinBox, 2, 1, 1, 1);

        numberOfRowsLabel = new QLabel(tab2);
        numberOfRowsLabel->setObjectName(QString::fromUtf8("numberOfRowsLabel"));
        numberOfRowsLabel->setWordWrap(false);

        gridLayout->addWidget(numberOfRowsLabel, 0, 0, 1, 1);

        gridRowsSpinBox = new QSpinBox(tab2);
        gridRowsSpinBox->setObjectName(QString::fromUtf8("gridRowsSpinBox"));
        gridRowsSpinBox->setMinimumSize(QSize(0, 23));
        gridRowsSpinBox->setMinimum(1);
        gridRowsSpinBox->setMaximum(999);

        gridLayout->addWidget(gridRowsSpinBox, 0, 1, 1, 1);

        gridColsSpinBox = new QSpinBox(tab2);
        gridColsSpinBox->setObjectName(QString::fromUtf8("gridColsSpinBox"));
        gridColsSpinBox->setMinimumSize(QSize(0, 23));
        gridColsSpinBox->setMinimum(1);
        gridColsSpinBox->setMaximum(999);

        gridLayout->addWidget(gridColsSpinBox, 1, 1, 1, 1);

        horizRCLabel = new QLabel(tab2);
        horizRCLabel->setObjectName(QString::fromUtf8("horizRCLabel"));
        horizRCLabel->setWordWrap(false);

        gridLayout->addWidget(horizRCLabel, 2, 0, 1, 1);

        vertRCLabel = new QLabel(tab2);
        vertRCLabel->setObjectName(QString::fromUtf8("vertRCLabel"));
        vertRCLabel->setWordWrap(false);

        gridLayout->addWidget(vertRCLabel, 3, 0, 1, 1);

        vertRCGapSpinBox = new ScrSpinBox(tab2);
        vertRCGapSpinBox->setObjectName(QString::fromUtf8("vertRCGapSpinBox"));
        vertRCGapSpinBox->setMinimumSize(QSize(0, 23));

        gridLayout->addWidget(vertRCGapSpinBox, 3, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        tabWidget->addTab(tab2, QString());
        tab3 = new QWidget();
        tab3->setObjectName(QString::fromUtf8("tab3"));
        verticalLayout_8 = new QVBoxLayout(tab3);
        verticalLayout_8->setSpacing(6);
        verticalLayout_8->setContentsMargins(11, 11, 11, 11);
        verticalLayout_8->setObjectName(QString::fromUtf8("verticalLayout_8"));
        verticalLayout_7 = new QVBoxLayout();
        verticalLayout_7->setSpacing(6);
        verticalLayout_7->setObjectName(QString::fromUtf8("verticalLayout_7"));
        radioButtonPageAll = new QRadioButton(tab3);
        buttonGroup = new QButtonGroup(MultipleDuplicate);
        buttonGroup->setObjectName(QString::fromUtf8("buttonGroup"));
        buttonGroup->addButton(radioButtonPageAll);
        radioButtonPageAll->setObjectName(QString::fromUtf8("radioButtonPageAll"));

        verticalLayout_7->addWidget(radioButtonPageAll);

        radioButtonPageEven = new QRadioButton(tab3);
        buttonGroup->addButton(radioButtonPageEven);
        radioButtonPageEven->setObjectName(QString::fromUtf8("radioButtonPageEven"));

        verticalLayout_7->addWidget(radioButtonPageEven);

        radioButtonPageOdd = new QRadioButton(tab3);
        buttonGroup->addButton(radioButtonPageOdd);
        radioButtonPageOdd->setObjectName(QString::fromUtf8("radioButtonPageOdd"));

        verticalLayout_7->addWidget(radioButtonPageOdd);

        radioButtonPageRange = new QRadioButton(tab3);
        buttonGroup->addButton(radioButtonPageRange);
        radioButtonPageRange->setObjectName(QString::fromUtf8("radioButtonPageRange"));

        verticalLayout_7->addWidget(radioButtonPageRange);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(6);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        lineEditPageRange = new QLineEdit(tab3);
        lineEditPageRange->setObjectName(QString::fromUtf8("lineEditPageRange"));

        horizontalLayout->addWidget(lineEditPageRange);

        toolButtonPageRange = new QToolButton(tab3);
        toolButtonPageRange->setObjectName(QString::fromUtf8("toolButtonPageRange"));

        horizontalLayout->addWidget(toolButtonPageRange);


        verticalLayout_7->addLayout(horizontalLayout);


        verticalLayout_8->addLayout(verticalLayout_7);

        verticalLayout_9 = new QVBoxLayout();
        verticalLayout_9->setSpacing(6);
        verticalLayout_9->setObjectName(QString::fromUtf8("verticalLayout_9"));
        checkBoxPageLinkText = new QCheckBox(tab3);
        checkBoxPageLinkText->setObjectName(QString::fromUtf8("checkBoxPageLinkText"));

        verticalLayout_9->addWidget(checkBoxPageLinkText);


        verticalLayout_8->addLayout(verticalLayout_9);

        verticalSpacer_5 = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_8->addItem(verticalSpacer_5);

        tabWidget->addTab(tab3, QString());

        vboxLayout->addWidget(tabWidget);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        spacerItem = new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        buttonOk = new QPushButton(MultipleDuplicate);
        buttonOk->setObjectName(QString::fromUtf8("buttonOk"));
        buttonOk->setAutoDefault(true);

        hboxLayout->addWidget(buttonOk);

        buttonCancel = new QPushButton(MultipleDuplicate);
        buttonCancel->setObjectName(QString::fromUtf8("buttonCancel"));
        buttonCancel->setAutoDefault(true);

        hboxLayout->addWidget(buttonCancel);


        vboxLayout->addLayout(hboxLayout);

#if QT_CONFIG(shortcut)
        numberOfCopiesLabel->setBuddy(numberOfCopiesSpinBox);
        vertShiftLabel->setBuddy(vertShiftSpinBox);
        horizShiftLabel->setBuddy(horizShiftSpinBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(tabWidget, numberOfCopiesSpinBox);
        QWidget::setTabOrder(numberOfCopiesSpinBox, horizShiftSpinBox);
        QWidget::setTabOrder(horizShiftSpinBox, vertShiftSpinBox);
        QWidget::setTabOrder(vertShiftSpinBox, rotationSpinBox);
        QWidget::setTabOrder(rotationSpinBox, gridRowsSpinBox);
        QWidget::setTabOrder(gridRowsSpinBox, gridColsSpinBox);
        QWidget::setTabOrder(gridColsSpinBox, horizRCGapSpinBox);
        QWidget::setTabOrder(horizRCGapSpinBox, vertRCGapSpinBox);
        QWidget::setTabOrder(vertRCGapSpinBox, buttonOk);
        QWidget::setTabOrder(buttonOk, buttonCancel);

        retranslateUi(MultipleDuplicate);
        QObject::connect(buttonOk, SIGNAL(clicked()), MultipleDuplicate, SLOT(accept()));
        QObject::connect(buttonCancel, SIGNAL(clicked()), MultipleDuplicate, SLOT(reject()));

        tabWidget->setCurrentIndex(0);
        buttonOk->setDefault(true);


        QMetaObject::connectSlotsByName(MultipleDuplicate);
    } // setupUi

    void retranslateUi(QDialog *MultipleDuplicate)
    {
        MultipleDuplicate->setWindowTitle(QCoreApplication::translate("MultipleDuplicate", "Multiple Duplicate", nullptr));
        numberOfCopiesLabel->setText(QCoreApplication::translate("MultipleDuplicate", "&Number of Copies:", nullptr));
        shiftCreatedItemsRadioButton->setText(QCoreApplication::translate("MultipleDuplicate", "&Shift Created Items By", nullptr));
#if QT_CONFIG(shortcut)
        shiftCreatedItemsRadioButton->setShortcut(QCoreApplication::translate("MultipleDuplicate", "Alt+S", nullptr));
#endif // QT_CONFIG(shortcut)
        vertShiftLabel->setText(QCoreApplication::translate("MultipleDuplicate", "&Vertical Shift:", nullptr));
        rotationLabel->setText(QCoreApplication::translate("MultipleDuplicate", "Rotation:", nullptr));
        createGapRadioButton->setText(QCoreApplication::translate("MultipleDuplicate", "Create &Gap Between Items Of", nullptr));
#if QT_CONFIG(shortcut)
        createGapRadioButton->setShortcut(QCoreApplication::translate("MultipleDuplicate", "Alt+G", nullptr));
#endif // QT_CONFIG(shortcut)
        horizShiftLabel->setText(QCoreApplication::translate("MultipleDuplicate", "&Horizontal Shift:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("MultipleDuplicate", "&By Number of Copies", nullptr));
        numberOfColumnsLabel->setText(QCoreApplication::translate("MultipleDuplicate", "Number of Columns:", nullptr));
        numberOfRowsLabel->setText(QCoreApplication::translate("MultipleDuplicate", "Number of Rows:", nullptr));
        horizRCLabel->setText(QCoreApplication::translate("MultipleDuplicate", "Horizontal Gap:", nullptr));
        vertRCLabel->setText(QCoreApplication::translate("MultipleDuplicate", "Vertical Gap:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab2), QCoreApplication::translate("MultipleDuplicate", "By &Rows && Columns", nullptr));
        radioButtonPageAll->setText(QCoreApplication::translate("MultipleDuplicate", "&Following Pages", nullptr));
        radioButtonPageEven->setText(QCoreApplication::translate("MultipleDuplicate", "Following &Even Pages", nullptr));
        radioButtonPageOdd->setText(QCoreApplication::translate("MultipleDuplicate", "Following &Odd Pages", nullptr));
        radioButtonPageRange->setText(QCoreApplication::translate("MultipleDuplicate", "Ran&ge of Pages:", nullptr));
        checkBoxPageLinkText->setText(QCoreApplication::translate("MultipleDuplicate", "&Link Text Frames", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab3), QCoreApplication::translate("MultipleDuplicate", "By &Page", nullptr));
        buttonOk->setText(QCoreApplication::translate("MultipleDuplicate", "&OK", nullptr));
#if QT_CONFIG(shortcut)
        buttonOk->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        buttonCancel->setText(QCoreApplication::translate("MultipleDuplicate", "&Cancel", nullptr));
#if QT_CONFIG(shortcut)
        buttonCancel->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class MultipleDuplicate: public Ui_MultipleDuplicate {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MULTIPLEDUPLICATE_H
