/********************************************************************************
** Form generated from reading UI file 'propertiespalette_xyzbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTIESPALETTE_XYZBASE_H
#define UI_PROPERTIESPALETTE_XYZBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/basepointwidget.h"
#include "ui/linkbutton.h"
#include "ui/propertiespalette_utils.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_PropertiesPalette_XYZBase
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *nameGroup;
    QHBoxLayout *horizontalLayout;
    NameWidget *nameEdit;
    QGroupBox *geoGroup;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *geoGroupLayout;
    QLabel *xposLabel;
    ScrSpinBox *xposSpin;
    QLabel *widthLabel;
    QLabel *yposLabel;
    ScrSpinBox *yposSpin;
    ScrSpinBox *widthSpin;
    QLabel *heightLabel;
    ScrSpinBox *heightSpin;
    QLabel *rotationLabel;
    ScrSpinBox *rotationSpin;
    QLabel *basepointLabel;
    BasePointWidget *basePointWidget;
    LinkButton *keepFrameWHRatioButton;
    QHBoxLayout *horizontalLayout_2;
    QGroupBox *levelGroup;
    QGridLayout *gridLayout;
    QToolButton *levelUp;
    QToolButton *levelDown;
    QToolButton *levelTop;
    QToolButton *levelBottom;
    QLabel *levelLabel;
    QSpacerItem *horizontalSpacer;
    QGridLayout *gridLayout_2;
    QToolButton *doGroup;
    QToolButton *doUnGroup;
    QToolButton *flipH;
    QToolButton *flipV;
    QToolButton *doLock;
    QToolButton *noPrint;
    QToolButton *noResize;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *PropertiesPalette_XYZBase)
    {
        if (PropertiesPalette_XYZBase->objectName().isEmpty())
            PropertiesPalette_XYZBase->setObjectName(QString::fromUtf8("PropertiesPalette_XYZBase"));
        PropertiesPalette_XYZBase->resize(280, 371);
        verticalLayout = new QVBoxLayout(PropertiesPalette_XYZBase);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        nameGroup = new QGroupBox(PropertiesPalette_XYZBase);
        nameGroup->setObjectName(QString::fromUtf8("nameGroup"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(nameGroup->sizePolicy().hasHeightForWidth());
        nameGroup->setSizePolicy(sizePolicy);
        horizontalLayout = new QHBoxLayout(nameGroup);
        horizontalLayout->setSpacing(3);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(6, 6, 6, 6);
        nameEdit = new NameWidget(nameGroup);
        nameEdit->setObjectName(QString::fromUtf8("nameEdit"));
        nameEdit->setFocusPolicy(Qt::ClickFocus);

        horizontalLayout->addWidget(nameEdit);


        verticalLayout->addWidget(nameGroup);

        geoGroup = new QGroupBox(PropertiesPalette_XYZBase);
        geoGroup->setObjectName(QString::fromUtf8("geoGroup"));
        sizePolicy.setHeightForWidth(geoGroup->sizePolicy().hasHeightForWidth());
        geoGroup->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(geoGroup);
        verticalLayout_2->setSpacing(3);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(6, 6, 6, 6);
        geoGroupLayout = new QGridLayout();
        geoGroupLayout->setSpacing(3);
        geoGroupLayout->setObjectName(QString::fromUtf8("geoGroupLayout"));
        geoGroupLayout->setContentsMargins(0, 0, 0, 0);
        xposLabel = new QLabel(geoGroup);
        xposLabel->setObjectName(QString::fromUtf8("xposLabel"));

        geoGroupLayout->addWidget(xposLabel, 0, 0, 1, 1);

        xposSpin = new ScrSpinBox(geoGroup);
        xposSpin->setObjectName(QString::fromUtf8("xposSpin"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(xposSpin->sizePolicy().hasHeightForWidth());
        xposSpin->setSizePolicy(sizePolicy1);

        geoGroupLayout->addWidget(xposSpin, 0, 1, 1, 1);

        widthLabel = new QLabel(geoGroup);
        widthLabel->setObjectName(QString::fromUtf8("widthLabel"));

        geoGroupLayout->addWidget(widthLabel, 2, 0, 1, 1);

        yposLabel = new QLabel(geoGroup);
        yposLabel->setObjectName(QString::fromUtf8("yposLabel"));

        geoGroupLayout->addWidget(yposLabel, 1, 0, 1, 1);

        yposSpin = new ScrSpinBox(geoGroup);
        yposSpin->setObjectName(QString::fromUtf8("yposSpin"));

        geoGroupLayout->addWidget(yposSpin, 1, 1, 1, 1);

        widthSpin = new ScrSpinBox(geoGroup);
        widthSpin->setObjectName(QString::fromUtf8("widthSpin"));

        geoGroupLayout->addWidget(widthSpin, 2, 1, 1, 1);

        heightLabel = new QLabel(geoGroup);
        heightLabel->setObjectName(QString::fromUtf8("heightLabel"));

        geoGroupLayout->addWidget(heightLabel, 3, 0, 1, 1);

        heightSpin = new ScrSpinBox(geoGroup);
        heightSpin->setObjectName(QString::fromUtf8("heightSpin"));

        geoGroupLayout->addWidget(heightSpin, 3, 1, 1, 1);

        rotationLabel = new QLabel(geoGroup);
        rotationLabel->setObjectName(QString::fromUtf8("rotationLabel"));

        geoGroupLayout->addWidget(rotationLabel, 4, 0, 1, 1);

        rotationSpin = new ScrSpinBox(geoGroup);
        rotationSpin->setObjectName(QString::fromUtf8("rotationSpin"));
        rotationSpin->setWrapping(true);

        geoGroupLayout->addWidget(rotationSpin, 4, 1, 1, 1);

        basepointLabel = new QLabel(geoGroup);
        basepointLabel->setObjectName(QString::fromUtf8("basepointLabel"));

        geoGroupLayout->addWidget(basepointLabel, 5, 0, 1, 1);

        basePointWidget = new BasePointWidget(geoGroup);
        basePointWidget->setObjectName(QString::fromUtf8("basePointWidget"));

        geoGroupLayout->addWidget(basePointWidget, 5, 1, 1, 1);

        keepFrameWHRatioButton = new LinkButton(geoGroup);
        keepFrameWHRatioButton->setObjectName(QString::fromUtf8("keepFrameWHRatioButton"));
        keepFrameWHRatioButton->setMaximumSize(QSize(15, 32767));
        keepFrameWHRatioButton->setCheckable(true);
        keepFrameWHRatioButton->setAutoRaise(true);

        geoGroupLayout->addWidget(keepFrameWHRatioButton, 2, 2, 2, 1);


        verticalLayout_2->addLayout(geoGroupLayout);


        verticalLayout->addWidget(geoGroup);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(3);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        levelGroup = new QGroupBox(PropertiesPalette_XYZBase);
        levelGroup->setObjectName(QString::fromUtf8("levelGroup"));
        gridLayout = new QGridLayout(levelGroup);
        gridLayout->setSpacing(3);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(6, 6, 6, 6);
        levelUp = new QToolButton(levelGroup);
        levelUp->setObjectName(QString::fromUtf8("levelUp"));

        gridLayout->addWidget(levelUp, 0, 0, 1, 1);

        levelDown = new QToolButton(levelGroup);
        levelDown->setObjectName(QString::fromUtf8("levelDown"));

        gridLayout->addWidget(levelDown, 1, 0, 1, 1);

        levelTop = new QToolButton(levelGroup);
        levelTop->setObjectName(QString::fromUtf8("levelTop"));

        gridLayout->addWidget(levelTop, 0, 1, 1, 1);

        levelBottom = new QToolButton(levelGroup);
        levelBottom->setObjectName(QString::fromUtf8("levelBottom"));

        gridLayout->addWidget(levelBottom, 1, 1, 1, 1);

        levelLabel = new QLabel(levelGroup);
        levelLabel->setObjectName(QString::fromUtf8("levelLabel"));

        gridLayout->addWidget(levelLabel, 0, 2, 2, 1);


        horizontalLayout_2->addWidget(levelGroup);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setSpacing(3);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        doGroup = new QToolButton(PropertiesPalette_XYZBase);
        doGroup->setObjectName(QString::fromUtf8("doGroup"));

        gridLayout_2->addWidget(doGroup, 0, 0, 1, 1);

        doUnGroup = new QToolButton(PropertiesPalette_XYZBase);
        doUnGroup->setObjectName(QString::fromUtf8("doUnGroup"));

        gridLayout_2->addWidget(doUnGroup, 1, 0, 1, 1);

        flipH = new QToolButton(PropertiesPalette_XYZBase);
        flipH->setObjectName(QString::fromUtf8("flipH"));

        gridLayout_2->addWidget(flipH, 0, 1, 1, 1);

        flipV = new QToolButton(PropertiesPalette_XYZBase);
        flipV->setObjectName(QString::fromUtf8("flipV"));

        gridLayout_2->addWidget(flipV, 1, 1, 1, 1);

        doLock = new QToolButton(PropertiesPalette_XYZBase);
        doLock->setObjectName(QString::fromUtf8("doLock"));

        gridLayout_2->addWidget(doLock, 0, 2, 1, 1);

        noPrint = new QToolButton(PropertiesPalette_XYZBase);
        noPrint->setObjectName(QString::fromUtf8("noPrint"));

        gridLayout_2->addWidget(noPrint, 1, 2, 1, 1);

        noResize = new QToolButton(PropertiesPalette_XYZBase);
        noResize->setObjectName(QString::fromUtf8("noResize"));

        gridLayout_2->addWidget(noResize, 0, 3, 1, 1);


        horizontalLayout_2->addLayout(gridLayout_2);


        verticalLayout->addLayout(horizontalLayout_2);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);

#if QT_CONFIG(shortcut)
        xposLabel->setBuddy(xposSpin);
        widthLabel->setBuddy(widthSpin);
        yposLabel->setBuddy(yposSpin);
        heightLabel->setBuddy(heightSpin);
#endif // QT_CONFIG(shortcut)

        retranslateUi(PropertiesPalette_XYZBase);

        QMetaObject::connectSlotsByName(PropertiesPalette_XYZBase);
    } // setupUi

    void retranslateUi(QWidget *PropertiesPalette_XYZBase)
    {
        nameGroup->setTitle(QCoreApplication::translate("PropertiesPalette_XYZBase", "Name", nullptr));
#if QT_CONFIG(tooltip)
        nameEdit->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Name of selected object", nullptr));
#endif // QT_CONFIG(tooltip)
        geoGroup->setTitle(QCoreApplication::translate("PropertiesPalette_XYZBase", "Geometry", nullptr));
        xposLabel->setText(QCoreApplication::translate("PropertiesPalette_XYZBase", "&X-Pos:", nullptr));
#if QT_CONFIG(tooltip)
        xposSpin->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Horizontal position of current basepoint", nullptr));
#endif // QT_CONFIG(tooltip)
        widthLabel->setText(QCoreApplication::translate("PropertiesPalette_XYZBase", "&Width:", nullptr));
        yposLabel->setText(QCoreApplication::translate("PropertiesPalette_XYZBase", "&Y-Pos:", nullptr));
#if QT_CONFIG(tooltip)
        yposSpin->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Vertical position of current basepoint", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        widthSpin->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Width of current selection", nullptr));
#endif // QT_CONFIG(tooltip)
        heightLabel->setText(QCoreApplication::translate("PropertiesPalette_XYZBase", "&Height:", nullptr));
#if QT_CONFIG(tooltip)
        heightSpin->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Height of current selection", nullptr));
#endif // QT_CONFIG(tooltip)
        rotationLabel->setText(QCoreApplication::translate("PropertiesPalette_XYZBase", "&Rotation:", nullptr));
#if QT_CONFIG(tooltip)
        rotationSpin->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Rotation of object at current basepoint", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        basepointLabel->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Point from which measurements or rotation angles are referenced", nullptr));
#endif // QT_CONFIG(tooltip)
        basepointLabel->setText(QCoreApplication::translate("PropertiesPalette_XYZBase", "Basepoint:", nullptr));
#if QT_CONFIG(tooltip)
        keepFrameWHRatioButton->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Keep the aspect ratio", nullptr));
#endif // QT_CONFIG(tooltip)
        levelGroup->setTitle(QCoreApplication::translate("PropertiesPalette_XYZBase", "Level", nullptr));
#if QT_CONFIG(tooltip)
        levelUp->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Move one level up", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        levelDown->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Move one level down", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        levelTop->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Move to front", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        levelBottom->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Move to back", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        levelLabel->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Indicates the level the object is on, 1 means the object is at the bottom", nullptr));
#endif // QT_CONFIG(tooltip)
        levelLabel->setText(QCoreApplication::translate("PropertiesPalette_XYZBase", "0", nullptr));
#if QT_CONFIG(tooltip)
        doGroup->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Group the selected objects", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        doUnGroup->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Ungroup the selected group", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        flipH->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Flip horizontally", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        flipV->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Flip vertically", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        doLock->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Lock or unlock the object", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        noPrint->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Enable or disable exporting of the object", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        noResize->setToolTip(QCoreApplication::translate("PropertiesPalette_XYZBase", "Lock or unlock the size of the object", nullptr));
#endif // QT_CONFIG(tooltip)
        (void)PropertiesPalette_XYZBase;
    } // retranslateUi

};

namespace Ui {
    class PropertiesPalette_XYZBase: public Ui_PropertiesPalette_XYZBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTIESPALETTE_XYZBASE_H
