/********************************************************************************
** Form generated from reading UI file 'weldeditbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WELDEDITBASE_H
#define UI_WELDEDITBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>
#include "ui/scrpalettebase.h"

QT_BEGIN_NAMESPACE

class Ui_weldEditBase
{
public:
    QVBoxLayout *verticalLayout;
    QRadioButton *moveWeldPoint;
    QRadioButton *moveObject;
    QPushButton *buttonEndEdit;

    void setupUi(ScrPaletteBase *weldEditBase)
    {
        if (weldEditBase->objectName().isEmpty())
            weldEditBase->setObjectName(QString::fromUtf8("weldEditBase"));
        weldEditBase->resize(151, 95);
        verticalLayout = new QVBoxLayout(weldEditBase);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        moveWeldPoint = new QRadioButton(weldEditBase);
        moveWeldPoint->setObjectName(QString::fromUtf8("moveWeldPoint"));
        moveWeldPoint->setChecked(true);

        verticalLayout->addWidget(moveWeldPoint);

        moveObject = new QRadioButton(weldEditBase);
        moveObject->setObjectName(QString::fromUtf8("moveObject"));

        verticalLayout->addWidget(moveObject);

        buttonEndEdit = new QPushButton(weldEditBase);
        buttonEndEdit->setObjectName(QString::fromUtf8("buttonEndEdit"));

        verticalLayout->addWidget(buttonEndEdit);


        retranslateUi(weldEditBase);

        QMetaObject::connectSlotsByName(weldEditBase);
    } // setupUi

    void retranslateUi(ScrPaletteBase *weldEditBase)
    {
        weldEditBase->setWindowTitle(QCoreApplication::translate("weldEditBase", "Edit Weld", nullptr));
        moveWeldPoint->setText(QCoreApplication::translate("weldEditBase", "Move Weld Point", nullptr));
        moveObject->setText(QCoreApplication::translate("weldEditBase", "Move Object", nullptr));
        buttonEndEdit->setText(QCoreApplication::translate("weldEditBase", "End Edit", nullptr));
    } // retranslateUi

};

namespace Ui {
    class weldEditBase: public Ui_weldEditBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_WELDEDITBASE_H
