/****************************************************************************
** Meta object code from reading C++ file 'scriptercore.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../scribus-1.6.6/scribus/plugins/scriptplugin/scriptercore.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scriptercore.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ScripterCore_t {
    QByteArrayData data[28];
    char stringdata0[396];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ScripterCore_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ScripterCore_t qt_meta_stringdata_ScripterCore = {
    {
QT_MOC_LITERAL(0, 0, 12), // "ScripterCore"
QT_MOC_LITERAL(1, 13, 15), // "runScriptDialog"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 9), // "StdScript"
QT_MOC_LITERAL(4, 40, 12), // "baseFilename"
QT_MOC_LITERAL(5, 53, 12), // "RecentScript"
QT_MOC_LITERAL(6, 66, 2), // "fn"
QT_MOC_LITERAL(7, 69, 17), // "slotRunScriptFile"
QT_MOC_LITERAL(8, 87, 8), // "fileName"
QT_MOC_LITERAL(9, 96, 17), // "inMainInterpreter"
QT_MOC_LITERAL(10, 114, 9), // "arguments"
QT_MOC_LITERAL(11, 124, 19), // "slotRunPythonScript"
QT_MOC_LITERAL(12, 144, 13), // "slotRunScript"
QT_MOC_LITERAL(13, 158, 6), // "script"
QT_MOC_LITERAL(14, 165, 21), // "slotInteractiveScript"
QT_MOC_LITERAL(15, 187, 11), // "slotExecute"
QT_MOC_LITERAL(16, 199, 11), // "aboutScript"
QT_MOC_LITERAL(17, 211, 20), // "setupMainInterpreter"
QT_MOC_LITERAL(18, 232, 20), // "initExtensionScripts"
QT_MOC_LITERAL(19, 253, 16), // "runStartupScript"
QT_MOC_LITERAL(20, 270, 14), // "languageChange"
QT_MOC_LITERAL(21, 285, 13), // "startupScript"
QT_MOC_LITERAL(22, 299, 17), // "extensionsEnabled"
QT_MOC_LITERAL(23, 317, 16), // "setStartupScript"
QT_MOC_LITERAL(24, 334, 9), // "newScript"
QT_MOC_LITERAL(25, 344, 20), // "setExtensionsEnabled"
QT_MOC_LITERAL(26, 365, 6), // "enable"
QT_MOC_LITERAL(27, 372, 23) // "updateSyntaxHighlighter"

    },
    "ScripterCore\0runScriptDialog\0\0StdScript\0"
    "baseFilename\0RecentScript\0fn\0"
    "slotRunScriptFile\0fileName\0inMainInterpreter\0"
    "arguments\0slotRunPythonScript\0"
    "slotRunScript\0script\0slotInteractiveScript\0"
    "slotExecute\0aboutScript\0setupMainInterpreter\0"
    "initExtensionScripts\0runStartupScript\0"
    "languageChange\0startupScript\0"
    "extensionsEnabled\0setStartupScript\0"
    "newScript\0setExtensionsEnabled\0enable\0"
    "updateSyntaxHighlighter"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ScripterCore[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  119,    2, 0x0a /* Public */,
       3,    1,  120,    2, 0x0a /* Public */,
       5,    1,  123,    2, 0x0a /* Public */,
       7,    2,  126,    2, 0x0a /* Public */,
       7,    1,  131,    2, 0x2a /* Public | MethodCloned */,
       7,    3,  134,    2, 0x0a /* Public */,
       7,    2,  141,    2, 0x2a /* Public | MethodCloned */,
      11,    0,  146,    2, 0x0a /* Public */,
      12,    1,  147,    2, 0x0a /* Public */,
      14,    1,  150,    2, 0x0a /* Public */,
      15,    0,  153,    2, 0x0a /* Public */,
      16,    0,  154,    2, 0x0a /* Public */,
      17,    0,  155,    2, 0x0a /* Public */,
      18,    0,  156,    2, 0x0a /* Public */,
      19,    0,  157,    2, 0x0a /* Public */,
      20,    0,  158,    2, 0x0a /* Public */,
      21,    0,  159,    2, 0x0a /* Public */,
      22,    0,  160,    2, 0x0a /* Public */,
      23,    1,  161,    2, 0x0a /* Public */,
      25,    1,  164,    2, 0x0a /* Public */,
      27,    0,  167,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void, QMetaType::QString,    6,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,    8,    9,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void, QMetaType::QString, QMetaType::QStringList, QMetaType::Bool,    8,   10,    9,
    QMetaType::Void, QMetaType::QString, QMetaType::QStringList,    8,   10,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   13,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::QString,   24,
    QMetaType::Void, QMetaType::Bool,   26,
    QMetaType::Void,

       0        // eod
};

void ScripterCore::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ScripterCore *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->runScriptDialog(); break;
        case 1: _t->StdScript((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->RecentScript((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->slotRunScriptFile((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 4: _t->slotRunScriptFile((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->slotRunScriptFile((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QStringList(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 6: _t->slotRunScriptFile((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< QStringList(*)>(_a[2]))); break;
        case 7: _t->slotRunPythonScript(); break;
        case 8: _t->slotRunScript((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->slotInteractiveScript((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->slotExecute(); break;
        case 11: _t->aboutScript(); break;
        case 12: { bool _r = _t->setupMainInterpreter();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 13: _t->initExtensionScripts(); break;
        case 14: _t->runStartupScript(); break;
        case 15: _t->languageChange(); break;
        case 16: _t->startupScript(); break;
        case 17: { bool _r = _t->extensionsEnabled();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 18: _t->setStartupScript((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 19: _t->setExtensionsEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->updateSyntaxHighlighter(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ScripterCore::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ScripterCore.data,
    qt_meta_data_ScripterCore,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ScripterCore::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ScripterCore::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ScripterCore.stringdata0))
        return static_cast<void*>(this);
    return QObject::qt_metacast(_clname);
}

int ScripterCore::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 21;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
