/********************************************************************************
** Form generated from reading UI file 'picturebrowser.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PICTUREBROWSER_H
#define UI_PICTUREBROWSER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDateTimeEdit>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTreeView>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "multicombobox.h"
#include "pictview.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_PictureBrowser
{
public:
    QGridLayout *gridLayout;
    QSplitter *splitter_2;
    QWidget *widget_3;
    QGridLayout *gridLayout1;
    QSplitter *splitter;
    QWidget *widget;
    QVBoxLayout *verticalLayout_3;
    QComboBox *navigationBox;
    QStackedWidget *navigationStack;
    QWidget *folderpage;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *subdirsCheckbox;
    QTreeView *folderView;
    QWidget *collectionspage;
    QVBoxLayout *vboxLayout;
    QStackedWidget *collectionsStackedwidget;
    QWidget *page_5;
    QHBoxLayout *horizontalLayout_2;
    QPushButton *collectionsNewCategoryButton;
    QPushButton *collectionsNewButton;
    QPushButton *collectionsDeleteButton;
    QPushButton *collectionsRenameButton;
    QSpacerItem *spacerItem;
    QPushButton *collectionsAddImagesButton;
    QPushButton *collectionsRemoveImagesButton;
    QPushButton *collectionsTagImagesButton;
    QSpacerItem *spacerItem1;
    QPushButton *collectionsImportButton;
    QPushButton *collectionsExportButton;
    QWidget *page_6;
    QHBoxLayout *hboxLayout;
    QLabel *label_8;
    QPushButton *collectionsAddImagesOkButton;
    QPushButton *collectionsAddImagesCancelButton;
    QTreeWidget *collectionsWidget;
    QWidget *documentpage;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout1;
    QPushButton *gotoPageButton;
    QSpacerItem *spacerItem2;
    QTreeWidget *documentWidget;
    QHBoxLayout *hboxLayout2;
    QPushButton *moreButton;
    QSpacerItem *spacerItem3;
    QCheckBox *alwaysOnTopCheckbox;
    QWidget *widget_2;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QComboBox *sortCombobox;
    QPushButton *sortOrderButton;
    QSpacerItem *spacerItem4;
    QPushButton *zoomPlusButton;
    QPushButton *zoomMinusButton;
    QSpacerItem *spacerItem5;
    QComboBox *previewModeCombobox;
    QSpacerItem *spacerItem6;
    QComboBox *actionsCombobox;
    QPushButton *actionsGoButton;
    PictView *imageViewArea;
    QHBoxLayout *hboxLayout3;
    QLabel *statusLabel;
    QSpacerItem *spacerItem7;
    QCheckBox *saveSettingsCheckbox;
    QPushButton *resetSettingsButton;
    QPushButton *closeButton;
    QTabWidget *tabWidget;
    QWidget *tab;
    QHBoxLayout *horizontalLayout_3;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout2;
    QLabel *label_10;
    QLabel *informationFileNameLabel;
    QLabel *label_11;
    QHBoxLayout *horizontalLayout_5;
    QLabel *informationFilePathLabel;
    QPushButton *jumpToImageButton;
    QLabel *label_22;
    QLabel *informationFileSizeLabel;
    QLabel *label_23;
    QLabel *informationFileDateLabel;
    QSpacerItem *verticalSpacer_2;
    QGroupBox *groupBox;
    QGridLayout *gridLayout3;
    QLabel *informationEmbeddedLabel;
    QLabel *informationDpiLabel;
    QLabel *informationLayersLabel;
    QLabel *label_9;
    QLabel *informationHeightLabel;
    QLabel *label_15;
    QLabel *informationProfileNameLabel;
    QLabel *label_3;
    QLabel *informationFormatLabel;
    QLabel *informationWidthLabel;
    QLabel *informationColorspaceLabel;
    QLabel *label_12;
    QLabel *label_13;
    QLabel *label_5;
    QLabel *label_14;
    QLabel *label_4;
    QSpacerItem *verticalSpacer;
    QWidget *tab_3;
    QGridLayout *gridLayout4;
    QVBoxLayout *vboxLayout2;
    QLabel *label_6;
    QListWidget *filterFiltersListwidget;
    QComboBox *filterTargetCombobox;
    QStackedWidget *filterTargetStackedWidget;
    QWidget *page_10;
    QWidget *page_8;
    QHBoxLayout *hboxLayout4;
    QLineEdit *filterSearchLineedit;
    QPushButton *filterSearchDirButton;
    QWidget *page_9;
    QHBoxLayout *hboxLayout5;
    MultiCombobox *filterCollectionsCombobox;
    QGroupBox *groupBox_3;
    QGridLayout *gridLayout5;
    QPushButton *filterAddCriterionButton;
    QComboBox *filterCriteriaCombobox;
    QStackedWidget *filterStackedwidget;
    QWidget *page;
    QHBoxLayout *hboxLayout6;
    QComboBox *filterNameCombobox;
    QLineEdit *filterNameLineedit;
    QWidget *page_2;
    QHBoxLayout *hboxLayout7;
    QComboBox *filterDateCombobox;
    QDateTimeEdit *filterDateDatetimeedit;
    QWidget *page_3;
    QHBoxLayout *hboxLayout8;
    QComboBox *filterSizeCombobox;
    QSpinBox *filterSizeSpinbox;
    QWidget *page_4;
    QHBoxLayout *hboxLayout9;
    MultiCombobox *filterTypeCombobox;
    QWidget *page_7;
    QHBoxLayout *hboxLayout10;
    MultiCombobox *filterTagsCombobox;
    QLineEdit *filterTagLineedit;
    QPushButton *filterAddTagButton;
    QPushButton *filterFilterButton;
    QPushButton *filterClearButton;
    QWidget *tab_4;
    QGridLayout *gridLayout6;
    QGroupBox *groupBox_5;
    QGridLayout *gridLayout7;
    QLabel *label;
    MultiCombobox *insertPagesCombobox;
    QLabel *label_2;
    MultiCombobox *insertFramesCombobox;
    QGroupBox *groupBox_6;
    QGridLayout *gridLayout8;
    QComboBox *insertPositionCombobox;
    QLabel *insertPositionYLabel;
    QLabel *insertPositionXLabel;
    ScrSpinBox *insertPositionXSpinbox;
    ScrSpinBox *insertPositionYSpinbox;
    QGroupBox *groupBox_7;
    QGridLayout *gridLayout9;
    QLabel *insertWidthLabel;
    QLabel *insertHeightLabel;
    ScrSpinBox *insertHeightSpinbox;
    ScrSpinBox *insertWidthSpinbox;
    QComboBox *insertSizeCombobox;
    QVBoxLayout *vboxLayout3;
    QLabel *label_25;
    QLabel *label_26;
    QLabel *label_27;
    QPushButton *insertImageButton;
    QWidget *tab_5;
    QGridLayout *gridLayout10;
    QLabel *label_7;
    QSpacerItem *spacerItem8;
    QListWidget *collectionsSelectedImagesListwidget;
    QGroupBox *groupBox_9;
    QGridLayout *gridLayout11;
    MultiCombobox *collectionsTagImagesCombobox;
    QLineEdit *collectionsAddNewTagLineedit;
    QPushButton *collectionsAddNewTagButton;
    QPushButton *collectionsSetTagsButton;

    void setupUi(QDialog *PictureBrowser)
    {
        if (PictureBrowser->objectName().isEmpty())
            PictureBrowser->setObjectName(QString::fromUtf8("PictureBrowser"));
        PictureBrowser->resize(967, 482);
        gridLayout = new QGridLayout(PictureBrowser);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        splitter_2 = new QSplitter(PictureBrowser);
        splitter_2->setObjectName(QString::fromUtf8("splitter_2"));
        splitter_2->setOrientation(Qt::Vertical);
        widget_3 = new QWidget(splitter_2);
        widget_3->setObjectName(QString::fromUtf8("widget_3"));
        gridLayout1 = new QGridLayout(widget_3);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        gridLayout1->setContentsMargins(0, 0, 0, 0);
        splitter = new QSplitter(widget_3);
        splitter->setObjectName(QString::fromUtf8("splitter"));
        splitter->setOrientation(Qt::Horizontal);
        splitter->setHandleWidth(9);
        widget = new QWidget(splitter);
        widget->setObjectName(QString::fromUtf8("widget"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(1);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(widget->sizePolicy().hasHeightForWidth());
        widget->setSizePolicy(sizePolicy);
        verticalLayout_3 = new QVBoxLayout(widget);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        navigationBox = new QComboBox(widget);
        navigationBox->addItem(QString());
        navigationBox->addItem(QString());
        navigationBox->addItem(QString());
        navigationBox->setObjectName(QString::fromUtf8("navigationBox"));

        verticalLayout_3->addWidget(navigationBox);

        navigationStack = new QStackedWidget(widget);
        navigationStack->setObjectName(QString::fromUtf8("navigationStack"));
        folderpage = new QWidget();
        folderpage->setObjectName(QString::fromUtf8("folderpage"));
        verticalLayout_2 = new QVBoxLayout(folderpage);
        verticalLayout_2->setSpacing(3);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        subdirsCheckbox = new QCheckBox(folderpage);
        subdirsCheckbox->setObjectName(QString::fromUtf8("subdirsCheckbox"));

        verticalLayout_2->addWidget(subdirsCheckbox);

        folderView = new QTreeView(folderpage);
        folderView->setObjectName(QString::fromUtf8("folderView"));

        verticalLayout_2->addWidget(folderView);

        navigationStack->addWidget(folderpage);
        collectionspage = new QWidget();
        collectionspage->setObjectName(QString::fromUtf8("collectionspage"));
        vboxLayout = new QVBoxLayout(collectionspage);
        vboxLayout->setSpacing(3);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        collectionsStackedwidget = new QStackedWidget(collectionspage);
        collectionsStackedwidget->setObjectName(QString::fromUtf8("collectionsStackedwidget"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(collectionsStackedwidget->sizePolicy().hasHeightForWidth());
        collectionsStackedwidget->setSizePolicy(sizePolicy1);
        collectionsStackedwidget->setMinimumSize(QSize(0, 0));
        page_5 = new QWidget();
        page_5->setObjectName(QString::fromUtf8("page_5"));
        horizontalLayout_2 = new QHBoxLayout(page_5);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        horizontalLayout_2->setContentsMargins(0, 0, 0, 0);
        collectionsNewCategoryButton = new QPushButton(page_5);
        collectionsNewCategoryButton->setObjectName(QString::fromUtf8("collectionsNewCategoryButton"));
        collectionsNewCategoryButton->setMinimumSize(QSize(23, 23));
        collectionsNewCategoryButton->setMaximumSize(QSize(23, 23));

        horizontalLayout_2->addWidget(collectionsNewCategoryButton);

        collectionsNewButton = new QPushButton(page_5);
        collectionsNewButton->setObjectName(QString::fromUtf8("collectionsNewButton"));
        collectionsNewButton->setMinimumSize(QSize(23, 23));
        collectionsNewButton->setMaximumSize(QSize(23, 23));

        horizontalLayout_2->addWidget(collectionsNewButton);

        collectionsDeleteButton = new QPushButton(page_5);
        collectionsDeleteButton->setObjectName(QString::fromUtf8("collectionsDeleteButton"));
        collectionsDeleteButton->setMinimumSize(QSize(23, 23));
        collectionsDeleteButton->setMaximumSize(QSize(23, 23));

        horizontalLayout_2->addWidget(collectionsDeleteButton);

        collectionsRenameButton = new QPushButton(page_5);
        collectionsRenameButton->setObjectName(QString::fromUtf8("collectionsRenameButton"));
        collectionsRenameButton->setMinimumSize(QSize(23, 23));
        collectionsRenameButton->setMaximumSize(QSize(23, 23));

        horizontalLayout_2->addWidget(collectionsRenameButton);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(spacerItem);

        collectionsAddImagesButton = new QPushButton(page_5);
        collectionsAddImagesButton->setObjectName(QString::fromUtf8("collectionsAddImagesButton"));
        collectionsAddImagesButton->setMinimumSize(QSize(23, 23));
        collectionsAddImagesButton->setMaximumSize(QSize(23, 23));

        horizontalLayout_2->addWidget(collectionsAddImagesButton);

        collectionsRemoveImagesButton = new QPushButton(page_5);
        collectionsRemoveImagesButton->setObjectName(QString::fromUtf8("collectionsRemoveImagesButton"));
        collectionsRemoveImagesButton->setMinimumSize(QSize(23, 23));
        collectionsRemoveImagesButton->setMaximumSize(QSize(23, 23));

        horizontalLayout_2->addWidget(collectionsRemoveImagesButton);

        collectionsTagImagesButton = new QPushButton(page_5);
        collectionsTagImagesButton->setObjectName(QString::fromUtf8("collectionsTagImagesButton"));
        collectionsTagImagesButton->setMinimumSize(QSize(23, 23));
        collectionsTagImagesButton->setMaximumSize(QSize(23, 23));

        horizontalLayout_2->addWidget(collectionsTagImagesButton);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(spacerItem1);

        collectionsImportButton = new QPushButton(page_5);
        collectionsImportButton->setObjectName(QString::fromUtf8("collectionsImportButton"));
        collectionsImportButton->setMinimumSize(QSize(23, 23));
        collectionsImportButton->setMaximumSize(QSize(23, 23));

        horizontalLayout_2->addWidget(collectionsImportButton);

        collectionsExportButton = new QPushButton(page_5);
        collectionsExportButton->setObjectName(QString::fromUtf8("collectionsExportButton"));
        collectionsExportButton->setMinimumSize(QSize(23, 23));
        collectionsExportButton->setMaximumSize(QSize(23, 23));

        horizontalLayout_2->addWidget(collectionsExportButton);

        collectionsStackedwidget->addWidget(page_5);
        page_6 = new QWidget();
        page_6->setObjectName(QString::fromUtf8("page_6"));
        hboxLayout = new QHBoxLayout(page_6);
        hboxLayout->setSpacing(3);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        label_8 = new QLabel(page_6);
        label_8->setObjectName(QString::fromUtf8("label_8"));

        hboxLayout->addWidget(label_8);

        collectionsAddImagesOkButton = new QPushButton(page_6);
        collectionsAddImagesOkButton->setObjectName(QString::fromUtf8("collectionsAddImagesOkButton"));
        collectionsAddImagesOkButton->setMinimumSize(QSize(23, 23));
        collectionsAddImagesOkButton->setMaximumSize(QSize(23, 23));

        hboxLayout->addWidget(collectionsAddImagesOkButton);

        collectionsAddImagesCancelButton = new QPushButton(page_6);
        collectionsAddImagesCancelButton->setObjectName(QString::fromUtf8("collectionsAddImagesCancelButton"));
        collectionsAddImagesCancelButton->setMinimumSize(QSize(23, 23));
        collectionsAddImagesCancelButton->setMaximumSize(QSize(23, 23));

        hboxLayout->addWidget(collectionsAddImagesCancelButton);

        collectionsStackedwidget->addWidget(page_6);

        vboxLayout->addWidget(collectionsStackedwidget);

        collectionsWidget = new QTreeWidget(collectionspage);
        collectionsWidget->setObjectName(QString::fromUtf8("collectionsWidget"));
        collectionsWidget->setColumnCount(0);

        vboxLayout->addWidget(collectionsWidget);

        navigationStack->addWidget(collectionspage);
        documentpage = new QWidget();
        documentpage->setObjectName(QString::fromUtf8("documentpage"));
        vboxLayout1 = new QVBoxLayout(documentpage);
        vboxLayout1->setSpacing(3);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        gotoPageButton = new QPushButton(documentpage);
        gotoPageButton->setObjectName(QString::fromUtf8("gotoPageButton"));
        sizePolicy1.setHeightForWidth(gotoPageButton->sizePolicy().hasHeightForWidth());
        gotoPageButton->setSizePolicy(sizePolicy1);
        gotoPageButton->setMinimumSize(QSize(0, 23));
        gotoPageButton->setMaximumSize(QSize(16777215, 23));

        hboxLayout1->addWidget(gotoPageButton);

        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem2);


        vboxLayout1->addLayout(hboxLayout1);

        documentWidget = new QTreeWidget(documentpage);
        documentWidget->setObjectName(QString::fromUtf8("documentWidget"));
        documentWidget->setColumnCount(0);

        vboxLayout1->addWidget(documentWidget);

        navigationStack->addWidget(documentpage);

        verticalLayout_3->addWidget(navigationStack);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        moreButton = new QPushButton(widget);
        moreButton->setObjectName(QString::fromUtf8("moreButton"));
        moreButton->setEnabled(true);
        moreButton->setIconSize(QSize(15, 15));

        hboxLayout2->addWidget(moreButton);

        spacerItem3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout2->addItem(spacerItem3);

        alwaysOnTopCheckbox = new QCheckBox(widget);
        alwaysOnTopCheckbox->setObjectName(QString::fromUtf8("alwaysOnTopCheckbox"));

        hboxLayout2->addWidget(alwaysOnTopCheckbox);


        verticalLayout_3->addLayout(hboxLayout2);

        splitter->addWidget(widget);
        widget_2 = new QWidget(splitter);
        widget_2->setObjectName(QString::fromUtf8("widget_2"));
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy2.setHorizontalStretch(3);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(widget_2->sizePolicy().hasHeightForWidth());
        widget_2->setSizePolicy(sizePolicy2);
        verticalLayout = new QVBoxLayout(widget_2);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        sortCombobox = new QComboBox(widget_2);
        sortCombobox->addItem(QString());
        sortCombobox->addItem(QString());
        sortCombobox->addItem(QString());
        sortCombobox->addItem(QString());
        sortCombobox->setObjectName(QString::fromUtf8("sortCombobox"));

        horizontalLayout->addWidget(sortCombobox);

        sortOrderButton = new QPushButton(widget_2);
        sortOrderButton->setObjectName(QString::fromUtf8("sortOrderButton"));
        sortOrderButton->setMinimumSize(QSize(23, 23));
        sortOrderButton->setMaximumSize(QSize(23, 23));
        sortOrderButton->setIconSize(QSize(15, 15));

        horizontalLayout->addWidget(sortOrderButton);

        spacerItem4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(spacerItem4);

        zoomPlusButton = new QPushButton(widget_2);
        zoomPlusButton->setObjectName(QString::fromUtf8("zoomPlusButton"));
        zoomPlusButton->setMinimumSize(QSize(23, 23));
        zoomPlusButton->setMaximumSize(QSize(23, 23));
        zoomPlusButton->setIconSize(QSize(20, 20));

        horizontalLayout->addWidget(zoomPlusButton);

        zoomMinusButton = new QPushButton(widget_2);
        zoomMinusButton->setObjectName(QString::fromUtf8("zoomMinusButton"));
        zoomMinusButton->setMinimumSize(QSize(23, 23));
        zoomMinusButton->setMaximumSize(QSize(23, 23));
        zoomMinusButton->setIconSize(QSize(20, 20));

        horizontalLayout->addWidget(zoomMinusButton);

        spacerItem5 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(spacerItem5);

        previewModeCombobox = new QComboBox(widget_2);
        previewModeCombobox->addItem(QString());
        previewModeCombobox->addItem(QString());
        previewModeCombobox->setObjectName(QString::fromUtf8("previewModeCombobox"));

        horizontalLayout->addWidget(previewModeCombobox);

        spacerItem6 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(spacerItem6);

        actionsCombobox = new QComboBox(widget_2);
        actionsCombobox->addItem(QString());
        actionsCombobox->addItem(QString());
        actionsCombobox->addItem(QString());
        actionsCombobox->setObjectName(QString::fromUtf8("actionsCombobox"));

        horizontalLayout->addWidget(actionsCombobox);

        actionsGoButton = new QPushButton(widget_2);
        actionsGoButton->setObjectName(QString::fromUtf8("actionsGoButton"));
        actionsGoButton->setMinimumSize(QSize(23, 23));
        actionsGoButton->setMaximumSize(QSize(23, 23));
        actionsGoButton->setLayoutDirection(Qt::LeftToRight);
        actionsGoButton->setIconSize(QSize(20, 20));

        horizontalLayout->addWidget(actionsGoButton);


        verticalLayout->addLayout(horizontalLayout);

        imageViewArea = new PictView(widget_2);
        imageViewArea->setObjectName(QString::fromUtf8("imageViewArea"));
        imageViewArea->setDragEnabled(true);
        imageViewArea->setDragDropMode(QAbstractItemView::DragOnly);
        imageViewArea->setSelectionMode(QAbstractItemView::SingleSelection);

        verticalLayout->addWidget(imageViewArea);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setObjectName(QString::fromUtf8("hboxLayout3"));
        statusLabel = new QLabel(widget_2);
        statusLabel->setObjectName(QString::fromUtf8("statusLabel"));

        hboxLayout3->addWidget(statusLabel);

        spacerItem7 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout3->addItem(spacerItem7);

        saveSettingsCheckbox = new QCheckBox(widget_2);
        saveSettingsCheckbox->setObjectName(QString::fromUtf8("saveSettingsCheckbox"));
        saveSettingsCheckbox->setChecked(false);
        saveSettingsCheckbox->setTristate(false);

        hboxLayout3->addWidget(saveSettingsCheckbox);

        resetSettingsButton = new QPushButton(widget_2);
        resetSettingsButton->setObjectName(QString::fromUtf8("resetSettingsButton"));

        hboxLayout3->addWidget(resetSettingsButton);

        closeButton = new QPushButton(widget_2);
        closeButton->setObjectName(QString::fromUtf8("closeButton"));

        hboxLayout3->addWidget(closeButton);


        verticalLayout->addLayout(hboxLayout3);

        splitter->addWidget(widget_2);

        gridLayout1->addWidget(splitter, 0, 0, 1, 1);

        splitter_2->addWidget(widget_3);
        tabWidget = new QTabWidget(splitter_2);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        QSizePolicy sizePolicy3(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(tabWidget->sizePolicy().hasHeightForWidth());
        tabWidget->setSizePolicy(sizePolicy3);
        tabWidget->setMinimumSize(QSize(866, 191));
        tab = new QWidget();
        tab->setObjectName(QString::fromUtf8("tab"));
        horizontalLayout_3 = new QHBoxLayout(tab);
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        groupBox_2 = new QGroupBox(tab);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        sizePolicy1.setHeightForWidth(groupBox_2->sizePolicy().hasHeightForWidth());
        groupBox_2->setSizePolicy(sizePolicy1);
        gridLayout2 = new QGridLayout(groupBox_2);
        gridLayout2->setObjectName(QString::fromUtf8("gridLayout2"));
        label_10 = new QLabel(groupBox_2);
        label_10->setObjectName(QString::fromUtf8("label_10"));

        gridLayout2->addWidget(label_10, 0, 0, 1, 1);

        informationFileNameLabel = new QLabel(groupBox_2);
        informationFileNameLabel->setObjectName(QString::fromUtf8("informationFileNameLabel"));

        gridLayout2->addWidget(informationFileNameLabel, 0, 1, 1, 1);

        label_11 = new QLabel(groupBox_2);
        label_11->setObjectName(QString::fromUtf8("label_11"));
        QSizePolicy sizePolicy4(QSizePolicy::Maximum, QSizePolicy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(label_11->sizePolicy().hasHeightForWidth());
        label_11->setSizePolicy(sizePolicy4);

        gridLayout2->addWidget(label_11, 1, 0, 1, 1);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        informationFilePathLabel = new QLabel(groupBox_2);
        informationFilePathLabel->setObjectName(QString::fromUtf8("informationFilePathLabel"));

        horizontalLayout_5->addWidget(informationFilePathLabel);

        jumpToImageButton = new QPushButton(groupBox_2);
        jumpToImageButton->setObjectName(QString::fromUtf8("jumpToImageButton"));
        QSizePolicy sizePolicy5(QSizePolicy::Maximum, QSizePolicy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(jumpToImageButton->sizePolicy().hasHeightForWidth());
        jumpToImageButton->setSizePolicy(sizePolicy5);

        horizontalLayout_5->addWidget(jumpToImageButton);


        gridLayout2->addLayout(horizontalLayout_5, 1, 1, 1, 1);

        label_22 = new QLabel(groupBox_2);
        label_22->setObjectName(QString::fromUtf8("label_22"));
        sizePolicy4.setHeightForWidth(label_22->sizePolicy().hasHeightForWidth());
        label_22->setSizePolicy(sizePolicy4);

        gridLayout2->addWidget(label_22, 2, 0, 1, 1);

        informationFileSizeLabel = new QLabel(groupBox_2);
        informationFileSizeLabel->setObjectName(QString::fromUtf8("informationFileSizeLabel"));

        gridLayout2->addWidget(informationFileSizeLabel, 2, 1, 1, 1);

        label_23 = new QLabel(groupBox_2);
        label_23->setObjectName(QString::fromUtf8("label_23"));
        sizePolicy4.setHeightForWidth(label_23->sizePolicy().hasHeightForWidth());
        label_23->setSizePolicy(sizePolicy4);

        gridLayout2->addWidget(label_23, 3, 0, 1, 1);

        informationFileDateLabel = new QLabel(groupBox_2);
        informationFileDateLabel->setObjectName(QString::fromUtf8("informationFileDateLabel"));

        gridLayout2->addWidget(informationFileDateLabel, 3, 1, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout2->addItem(verticalSpacer_2, 4, 0, 1, 1);


        horizontalLayout_3->addWidget(groupBox_2);

        groupBox = new QGroupBox(tab);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        sizePolicy1.setHeightForWidth(groupBox->sizePolicy().hasHeightForWidth());
        groupBox->setSizePolicy(sizePolicy1);
        groupBox->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        gridLayout3 = new QGridLayout(groupBox);
        gridLayout3->setObjectName(QString::fromUtf8("gridLayout3"));
        informationEmbeddedLabel = new QLabel(groupBox);
        informationEmbeddedLabel->setObjectName(QString::fromUtf8("informationEmbeddedLabel"));

        gridLayout3->addWidget(informationEmbeddedLabel, 3, 1, 1, 1);

        informationDpiLabel = new QLabel(groupBox);
        informationDpiLabel->setObjectName(QString::fromUtf8("informationDpiLabel"));

        gridLayout3->addWidget(informationDpiLabel, 2, 1, 1, 1);

        informationLayersLabel = new QLabel(groupBox);
        informationLayersLabel->setObjectName(QString::fromUtf8("informationLayersLabel"));

        gridLayout3->addWidget(informationLayersLabel, 2, 3, 1, 1);

        label_9 = new QLabel(groupBox);
        label_9->setObjectName(QString::fromUtf8("label_9"));

        gridLayout3->addWidget(label_9, 3, 0, 1, 1);

        informationHeightLabel = new QLabel(groupBox);
        informationHeightLabel->setObjectName(QString::fromUtf8("informationHeightLabel"));

        gridLayout3->addWidget(informationHeightLabel, 1, 3, 1, 1);

        label_15 = new QLabel(groupBox);
        label_15->setObjectName(QString::fromUtf8("label_15"));

        gridLayout3->addWidget(label_15, 3, 2, 1, 1);

        informationProfileNameLabel = new QLabel(groupBox);
        informationProfileNameLabel->setObjectName(QString::fromUtf8("informationProfileNameLabel"));

        gridLayout3->addWidget(informationProfileNameLabel, 3, 3, 1, 1);

        label_3 = new QLabel(groupBox);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout3->addWidget(label_3, 0, 2, 1, 1);

        informationFormatLabel = new QLabel(groupBox);
        informationFormatLabel->setObjectName(QString::fromUtf8("informationFormatLabel"));

        gridLayout3->addWidget(informationFormatLabel, 0, 1, 1, 1);

        informationWidthLabel = new QLabel(groupBox);
        informationWidthLabel->setObjectName(QString::fromUtf8("informationWidthLabel"));

        gridLayout3->addWidget(informationWidthLabel, 0, 3, 1, 1);

        informationColorspaceLabel = new QLabel(groupBox);
        informationColorspaceLabel->setObjectName(QString::fromUtf8("informationColorspaceLabel"));

        gridLayout3->addWidget(informationColorspaceLabel, 1, 1, 1, 1);

        label_12 = new QLabel(groupBox);
        label_12->setObjectName(QString::fromUtf8("label_12"));

        gridLayout3->addWidget(label_12, 0, 0, 1, 1);

        label_13 = new QLabel(groupBox);
        label_13->setObjectName(QString::fromUtf8("label_13"));

        gridLayout3->addWidget(label_13, 1, 0, 1, 1);

        label_5 = new QLabel(groupBox);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        gridLayout3->addWidget(label_5, 2, 2, 1, 1);

        label_14 = new QLabel(groupBox);
        label_14->setObjectName(QString::fromUtf8("label_14"));

        gridLayout3->addWidget(label_14, 2, 0, 1, 1);

        label_4 = new QLabel(groupBox);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout3->addWidget(label_4, 1, 2, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout3->addItem(verticalSpacer, 4, 0, 1, 1);


        horizontalLayout_3->addWidget(groupBox);

        tabWidget->addTab(tab, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName(QString::fromUtf8("tab_3"));
        gridLayout4 = new QGridLayout(tab_3);
        gridLayout4->setObjectName(QString::fromUtf8("gridLayout4"));
        vboxLayout2 = new QVBoxLayout();
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        label_6 = new QLabel(tab_3);
        label_6->setObjectName(QString::fromUtf8("label_6"));
        sizePolicy1.setHeightForWidth(label_6->sizePolicy().hasHeightForWidth());
        label_6->setSizePolicy(sizePolicy1);

        vboxLayout2->addWidget(label_6);

        filterFiltersListwidget = new QListWidget(tab_3);
        filterFiltersListwidget->setObjectName(QString::fromUtf8("filterFiltersListwidget"));

        vboxLayout2->addWidget(filterFiltersListwidget);


        gridLayout4->addLayout(vboxLayout2, 0, 0, 3, 1);

        filterTargetCombobox = new QComboBox(tab_3);
        filterTargetCombobox->addItem(QString());
        filterTargetCombobox->addItem(QString());
        filterTargetCombobox->addItem(QString());
        filterTargetCombobox->setObjectName(QString::fromUtf8("filterTargetCombobox"));
        QSizePolicy sizePolicy6(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(filterTargetCombobox->sizePolicy().hasHeightForWidth());
        filterTargetCombobox->setSizePolicy(sizePolicy6);

        gridLayout4->addWidget(filterTargetCombobox, 0, 1, 1, 1);

        filterTargetStackedWidget = new QStackedWidget(tab_3);
        filterTargetStackedWidget->setObjectName(QString::fromUtf8("filterTargetStackedWidget"));
        sizePolicy1.setHeightForWidth(filterTargetStackedWidget->sizePolicy().hasHeightForWidth());
        filterTargetStackedWidget->setSizePolicy(sizePolicy1);
        page_10 = new QWidget();
        page_10->setObjectName(QString::fromUtf8("page_10"));
        filterTargetStackedWidget->addWidget(page_10);
        page_8 = new QWidget();
        page_8->setObjectName(QString::fromUtf8("page_8"));
        hboxLayout4 = new QHBoxLayout(page_8);
        hboxLayout4->setObjectName(QString::fromUtf8("hboxLayout4"));
        hboxLayout4->setContentsMargins(0, 0, 0, 0);
        filterSearchLineedit = new QLineEdit(page_8);
        filterSearchLineedit->setObjectName(QString::fromUtf8("filterSearchLineedit"));
        filterSearchLineedit->setEnabled(true);

        hboxLayout4->addWidget(filterSearchLineedit);

        filterSearchDirButton = new QPushButton(page_8);
        filterSearchDirButton->setObjectName(QString::fromUtf8("filterSearchDirButton"));
        filterSearchDirButton->setEnabled(true);
        sizePolicy6.setHeightForWidth(filterSearchDirButton->sizePolicy().hasHeightForWidth());
        filterSearchDirButton->setSizePolicy(sizePolicy6);

        hboxLayout4->addWidget(filterSearchDirButton);

        filterTargetStackedWidget->addWidget(page_8);
        page_9 = new QWidget();
        page_9->setObjectName(QString::fromUtf8("page_9"));
        hboxLayout5 = new QHBoxLayout(page_9);
        hboxLayout5->setObjectName(QString::fromUtf8("hboxLayout5"));
        hboxLayout5->setContentsMargins(0, 0, 0, 0);
        filterCollectionsCombobox = new MultiCombobox(page_9);
        filterCollectionsCombobox->setObjectName(QString::fromUtf8("filterCollectionsCombobox"));
        filterCollectionsCombobox->setEnabled(true);

        hboxLayout5->addWidget(filterCollectionsCombobox);

        filterTargetStackedWidget->addWidget(page_9);

        gridLayout4->addWidget(filterTargetStackedWidget, 0, 2, 1, 1);

        groupBox_3 = new QGroupBox(tab_3);
        groupBox_3->setObjectName(QString::fromUtf8("groupBox_3"));
        sizePolicy1.setHeightForWidth(groupBox_3->sizePolicy().hasHeightForWidth());
        groupBox_3->setSizePolicy(sizePolicy1);
        gridLayout5 = new QGridLayout(groupBox_3);
        gridLayout5->setObjectName(QString::fromUtf8("gridLayout5"));
        filterAddCriterionButton = new QPushButton(groupBox_3);
        filterAddCriterionButton->setObjectName(QString::fromUtf8("filterAddCriterionButton"));
        sizePolicy6.setHeightForWidth(filterAddCriterionButton->sizePolicy().hasHeightForWidth());
        filterAddCriterionButton->setSizePolicy(sizePolicy6);

        gridLayout5->addWidget(filterAddCriterionButton, 0, 0, 1, 1);

        filterCriteriaCombobox = new QComboBox(groupBox_3);
        filterCriteriaCombobox->addItem(QString());
        filterCriteriaCombobox->addItem(QString());
        filterCriteriaCombobox->addItem(QString());
        filterCriteriaCombobox->addItem(QString());
        filterCriteriaCombobox->addItem(QString());
        filterCriteriaCombobox->setObjectName(QString::fromUtf8("filterCriteriaCombobox"));

        gridLayout5->addWidget(filterCriteriaCombobox, 0, 1, 1, 1);

        filterStackedwidget = new QStackedWidget(groupBox_3);
        filterStackedwidget->setObjectName(QString::fromUtf8("filterStackedwidget"));
        sizePolicy1.setHeightForWidth(filterStackedwidget->sizePolicy().hasHeightForWidth());
        filterStackedwidget->setSizePolicy(sizePolicy1);
        page = new QWidget();
        page->setObjectName(QString::fromUtf8("page"));
        hboxLayout6 = new QHBoxLayout(page);
        hboxLayout6->setObjectName(QString::fromUtf8("hboxLayout6"));
        hboxLayout6->setContentsMargins(0, 0, 0, 0);
        filterNameCombobox = new QComboBox(page);
        filterNameCombobox->addItem(QString());
        filterNameCombobox->addItem(QString());
        filterNameCombobox->setObjectName(QString::fromUtf8("filterNameCombobox"));

        hboxLayout6->addWidget(filterNameCombobox);

        filterNameLineedit = new QLineEdit(page);
        filterNameLineedit->setObjectName(QString::fromUtf8("filterNameLineedit"));

        hboxLayout6->addWidget(filterNameLineedit);

        filterStackedwidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName(QString::fromUtf8("page_2"));
        hboxLayout7 = new QHBoxLayout(page_2);
        hboxLayout7->setObjectName(QString::fromUtf8("hboxLayout7"));
        hboxLayout7->setContentsMargins(0, 0, 0, 0);
        filterDateCombobox = new QComboBox(page_2);
        filterDateCombobox->addItem(QString());
        filterDateCombobox->addItem(QString());
        filterDateCombobox->setObjectName(QString::fromUtf8("filterDateCombobox"));

        hboxLayout7->addWidget(filterDateCombobox);

        filterDateDatetimeedit = new QDateTimeEdit(page_2);
        filterDateDatetimeedit->setObjectName(QString::fromUtf8("filterDateDatetimeedit"));

        hboxLayout7->addWidget(filterDateDatetimeedit);

        filterStackedwidget->addWidget(page_2);
        page_3 = new QWidget();
        page_3->setObjectName(QString::fromUtf8("page_3"));
        hboxLayout8 = new QHBoxLayout(page_3);
        hboxLayout8->setObjectName(QString::fromUtf8("hboxLayout8"));
        hboxLayout8->setContentsMargins(0, 0, 0, 0);
        filterSizeCombobox = new QComboBox(page_3);
        filterSizeCombobox->addItem(QString());
        filterSizeCombobox->addItem(QString());
        filterSizeCombobox->setObjectName(QString::fromUtf8("filterSizeCombobox"));

        hboxLayout8->addWidget(filterSizeCombobox);

        filterSizeSpinbox = new QSpinBox(page_3);
        filterSizeSpinbox->setObjectName(QString::fromUtf8("filterSizeSpinbox"));
        filterSizeSpinbox->setMinimum(1);
        filterSizeSpinbox->setMaximum(60000);
        filterSizeSpinbox->setValue(1000);

        hboxLayout8->addWidget(filterSizeSpinbox);

        filterStackedwidget->addWidget(page_3);
        page_4 = new QWidget();
        page_4->setObjectName(QString::fromUtf8("page_4"));
        hboxLayout9 = new QHBoxLayout(page_4);
        hboxLayout9->setObjectName(QString::fromUtf8("hboxLayout9"));
        hboxLayout9->setContentsMargins(0, 0, 0, 0);
        filterTypeCombobox = new MultiCombobox(page_4);
        filterTypeCombobox->setObjectName(QString::fromUtf8("filterTypeCombobox"));

        hboxLayout9->addWidget(filterTypeCombobox);

        filterStackedwidget->addWidget(page_4);
        page_7 = new QWidget();
        page_7->setObjectName(QString::fromUtf8("page_7"));
        hboxLayout10 = new QHBoxLayout(page_7);
        hboxLayout10->setObjectName(QString::fromUtf8("hboxLayout10"));
        filterTagsCombobox = new MultiCombobox(page_7);
        filterTagsCombobox->setObjectName(QString::fromUtf8("filterTagsCombobox"));

        hboxLayout10->addWidget(filterTagsCombobox);

        filterTagLineedit = new QLineEdit(page_7);
        filterTagLineedit->setObjectName(QString::fromUtf8("filterTagLineedit"));

        hboxLayout10->addWidget(filterTagLineedit);

        filterAddTagButton = new QPushButton(page_7);
        filterAddTagButton->setObjectName(QString::fromUtf8("filterAddTagButton"));

        hboxLayout10->addWidget(filterAddTagButton);

        filterStackedwidget->addWidget(page_7);

        gridLayout5->addWidget(filterStackedwidget, 1, 0, 1, 2);


        gridLayout4->addWidget(groupBox_3, 1, 1, 2, 2);

        filterFilterButton = new QPushButton(tab_3);
        filterFilterButton->setObjectName(QString::fromUtf8("filterFilterButton"));
        sizePolicy6.setHeightForWidth(filterFilterButton->sizePolicy().hasHeightForWidth());
        filterFilterButton->setSizePolicy(sizePolicy6);

        gridLayout4->addWidget(filterFilterButton, 1, 3, 1, 1);

        filterClearButton = new QPushButton(tab_3);
        filterClearButton->setObjectName(QString::fromUtf8("filterClearButton"));
        sizePolicy6.setHeightForWidth(filterClearButton->sizePolicy().hasHeightForWidth());
        filterClearButton->setSizePolicy(sizePolicy6);

        gridLayout4->addWidget(filterClearButton, 2, 3, 1, 1);

        tabWidget->addTab(tab_3, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName(QString::fromUtf8("tab_4"));
        gridLayout6 = new QGridLayout(tab_4);
        gridLayout6->setObjectName(QString::fromUtf8("gridLayout6"));
        groupBox_5 = new QGroupBox(tab_4);
        groupBox_5->setObjectName(QString::fromUtf8("groupBox_5"));
        sizePolicy3.setHeightForWidth(groupBox_5->sizePolicy().hasHeightForWidth());
        groupBox_5->setSizePolicy(sizePolicy3);
        gridLayout7 = new QGridLayout(groupBox_5);
        gridLayout7->setObjectName(QString::fromUtf8("gridLayout7"));
        label = new QLabel(groupBox_5);
        label->setObjectName(QString::fromUtf8("label"));
        sizePolicy1.setHeightForWidth(label->sizePolicy().hasHeightForWidth());
        label->setSizePolicy(sizePolicy1);

        gridLayout7->addWidget(label, 0, 0, 1, 1);

        insertPagesCombobox = new MultiCombobox(groupBox_5);
        insertPagesCombobox->setObjectName(QString::fromUtf8("insertPagesCombobox"));

        gridLayout7->addWidget(insertPagesCombobox, 1, 0, 1, 1);

        label_2 = new QLabel(groupBox_5);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        sizePolicy1.setHeightForWidth(label_2->sizePolicy().hasHeightForWidth());
        label_2->setSizePolicy(sizePolicy1);

        gridLayout7->addWidget(label_2, 2, 0, 1, 1);

        insertFramesCombobox = new MultiCombobox(groupBox_5);
        insertFramesCombobox->setObjectName(QString::fromUtf8("insertFramesCombobox"));

        gridLayout7->addWidget(insertFramesCombobox, 3, 0, 1, 1);


        gridLayout6->addWidget(groupBox_5, 1, 0, 1, 1);

        groupBox_6 = new QGroupBox(tab_4);
        groupBox_6->setObjectName(QString::fromUtf8("groupBox_6"));
        sizePolicy3.setHeightForWidth(groupBox_6->sizePolicy().hasHeightForWidth());
        groupBox_6->setSizePolicy(sizePolicy3);
        gridLayout8 = new QGridLayout(groupBox_6);
        gridLayout8->setObjectName(QString::fromUtf8("gridLayout8"));
        insertPositionCombobox = new QComboBox(groupBox_6);
        insertPositionCombobox->addItem(QString());
        insertPositionCombobox->addItem(QString());
        insertPositionCombobox->addItem(QString());
        insertPositionCombobox->addItem(QString());
        insertPositionCombobox->setObjectName(QString::fromUtf8("insertPositionCombobox"));

        gridLayout8->addWidget(insertPositionCombobox, 0, 0, 1, 2);

        insertPositionYLabel = new QLabel(groupBox_6);
        insertPositionYLabel->setObjectName(QString::fromUtf8("insertPositionYLabel"));
        insertPositionYLabel->setEnabled(false);

        gridLayout8->addWidget(insertPositionYLabel, 2, 0, 1, 1);

        insertPositionXLabel = new QLabel(groupBox_6);
        insertPositionXLabel->setObjectName(QString::fromUtf8("insertPositionXLabel"));
        insertPositionXLabel->setEnabled(false);

        gridLayout8->addWidget(insertPositionXLabel, 1, 0, 1, 1);

        insertPositionXSpinbox = new ScrSpinBox(groupBox_6);
        insertPositionXSpinbox->setObjectName(QString::fromUtf8("insertPositionXSpinbox"));
        insertPositionXSpinbox->setEnabled(false);
        insertPositionXSpinbox->setMaximum(9999.000000000000000);

        gridLayout8->addWidget(insertPositionXSpinbox, 1, 1, 1, 1);

        insertPositionYSpinbox = new ScrSpinBox(groupBox_6);
        insertPositionYSpinbox->setObjectName(QString::fromUtf8("insertPositionYSpinbox"));
        insertPositionYSpinbox->setEnabled(false);
        insertPositionYSpinbox->setMaximum(9999.000000000000000);

        gridLayout8->addWidget(insertPositionYSpinbox, 2, 1, 1, 1);


        gridLayout6->addWidget(groupBox_6, 1, 1, 1, 1);

        groupBox_7 = new QGroupBox(tab_4);
        groupBox_7->setObjectName(QString::fromUtf8("groupBox_7"));
        sizePolicy3.setHeightForWidth(groupBox_7->sizePolicy().hasHeightForWidth());
        groupBox_7->setSizePolicy(sizePolicy3);
        gridLayout9 = new QGridLayout(groupBox_7);
        gridLayout9->setObjectName(QString::fromUtf8("gridLayout9"));
        insertWidthLabel = new QLabel(groupBox_7);
        insertWidthLabel->setObjectName(QString::fromUtf8("insertWidthLabel"));
        insertWidthLabel->setEnabled(false);

        gridLayout9->addWidget(insertWidthLabel, 1, 0, 1, 1);

        insertHeightLabel = new QLabel(groupBox_7);
        insertHeightLabel->setObjectName(QString::fromUtf8("insertHeightLabel"));
        insertHeightLabel->setEnabled(false);

        gridLayout9->addWidget(insertHeightLabel, 2, 0, 1, 1);

        insertHeightSpinbox = new ScrSpinBox(groupBox_7);
        insertHeightSpinbox->setObjectName(QString::fromUtf8("insertHeightSpinbox"));
        insertHeightSpinbox->setEnabled(false);
        insertHeightSpinbox->setMaximum(60000.000000000000000);

        gridLayout9->addWidget(insertHeightSpinbox, 2, 1, 1, 1);

        insertWidthSpinbox = new ScrSpinBox(groupBox_7);
        insertWidthSpinbox->setObjectName(QString::fromUtf8("insertWidthSpinbox"));
        insertWidthSpinbox->setEnabled(false);
        insertWidthSpinbox->setMaximum(60000.000000000000000);

        gridLayout9->addWidget(insertWidthSpinbox, 1, 1, 1, 1);

        insertSizeCombobox = new QComboBox(groupBox_7);
        insertSizeCombobox->addItem(QString());
        insertSizeCombobox->addItem(QString());
        insertSizeCombobox->addItem(QString());
        insertSizeCombobox->addItem(QString());
        insertSizeCombobox->addItem(QString());
        insertSizeCombobox->setObjectName(QString::fromUtf8("insertSizeCombobox"));

        gridLayout9->addWidget(insertSizeCombobox, 0, 0, 1, 2);


        gridLayout6->addWidget(groupBox_7, 1, 2, 1, 1);

        vboxLayout3 = new QVBoxLayout();
        vboxLayout3->setObjectName(QString::fromUtf8("vboxLayout3"));
        label_25 = new QLabel(tab_4);
        label_25->setObjectName(QString::fromUtf8("label_25"));
        QSizePolicy sizePolicy7(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy7.setHorizontalStretch(0);
        sizePolicy7.setVerticalStretch(0);
        sizePolicy7.setHeightForWidth(label_25->sizePolicy().hasHeightForWidth());
        label_25->setSizePolicy(sizePolicy7);
        label_25->setWordWrap(true);

        vboxLayout3->addWidget(label_25);

        label_26 = new QLabel(tab_4);
        label_26->setObjectName(QString::fromUtf8("label_26"));
        sizePolicy1.setHeightForWidth(label_26->sizePolicy().hasHeightForWidth());
        label_26->setSizePolicy(sizePolicy1);

        vboxLayout3->addWidget(label_26);

        label_27 = new QLabel(tab_4);
        label_27->setObjectName(QString::fromUtf8("label_27"));
        sizePolicy1.setHeightForWidth(label_27->sizePolicy().hasHeightForWidth());
        label_27->setSizePolicy(sizePolicy1);

        vboxLayout3->addWidget(label_27);

        insertImageButton = new QPushButton(tab_4);
        insertImageButton->setObjectName(QString::fromUtf8("insertImageButton"));
        sizePolicy6.setHeightForWidth(insertImageButton->sizePolicy().hasHeightForWidth());
        insertImageButton->setSizePolicy(sizePolicy6);

        vboxLayout3->addWidget(insertImageButton);


        gridLayout6->addLayout(vboxLayout3, 1, 3, 1, 1);

        tabWidget->addTab(tab_4, QString());
        tab_5 = new QWidget();
        tab_5->setObjectName(QString::fromUtf8("tab_5"));
        gridLayout10 = new QGridLayout(tab_5);
        gridLayout10->setObjectName(QString::fromUtf8("gridLayout10"));
        label_7 = new QLabel(tab_5);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        gridLayout10->addWidget(label_7, 0, 0, 1, 1);

        spacerItem8 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout10->addItem(spacerItem8, 0, 2, 1, 1);

        collectionsSelectedImagesListwidget = new QListWidget(tab_5);
        collectionsSelectedImagesListwidget->setObjectName(QString::fromUtf8("collectionsSelectedImagesListwidget"));

        gridLayout10->addWidget(collectionsSelectedImagesListwidget, 1, 0, 1, 1);

        groupBox_9 = new QGroupBox(tab_5);
        groupBox_9->setObjectName(QString::fromUtf8("groupBox_9"));
        sizePolicy1.setHeightForWidth(groupBox_9->sizePolicy().hasHeightForWidth());
        groupBox_9->setSizePolicy(sizePolicy1);
        gridLayout11 = new QGridLayout(groupBox_9);
        gridLayout11->setObjectName(QString::fromUtf8("gridLayout11"));
        collectionsTagImagesCombobox = new MultiCombobox(groupBox_9);
        collectionsTagImagesCombobox->setObjectName(QString::fromUtf8("collectionsTagImagesCombobox"));

        gridLayout11->addWidget(collectionsTagImagesCombobox, 0, 0, 1, 2);

        collectionsAddNewTagLineedit = new QLineEdit(groupBox_9);
        collectionsAddNewTagLineedit->setObjectName(QString::fromUtf8("collectionsAddNewTagLineedit"));

        gridLayout11->addWidget(collectionsAddNewTagLineedit, 1, 0, 1, 1);

        collectionsAddNewTagButton = new QPushButton(groupBox_9);
        collectionsAddNewTagButton->setObjectName(QString::fromUtf8("collectionsAddNewTagButton"));

        gridLayout11->addWidget(collectionsAddNewTagButton, 1, 1, 1, 1);

        collectionsSetTagsButton = new QPushButton(groupBox_9);
        collectionsSetTagsButton->setObjectName(QString::fromUtf8("collectionsSetTagsButton"));

        gridLayout11->addWidget(collectionsSetTagsButton, 2, 0, 1, 1);


        gridLayout10->addWidget(groupBox_9, 1, 1, 1, 1);

        tabWidget->addTab(tab_5, QString());
        splitter_2->addWidget(tabWidget);

        gridLayout->addWidget(splitter_2, 0, 0, 1, 1);

#if QT_CONFIG(shortcut)
        label->setBuddy(insertPagesCombobox);
        label_2->setBuddy(insertFramesCombobox);
#endif // QT_CONFIG(shortcut)

        retranslateUi(PictureBrowser);

        navigationStack->setCurrentIndex(0);
        collectionsStackedwidget->setCurrentIndex(0);
        sortCombobox->setCurrentIndex(0);
        tabWidget->setCurrentIndex(0);
        filterTargetStackedWidget->setCurrentIndex(0);
        filterStackedwidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(PictureBrowser);
    } // setupUi

    void retranslateUi(QDialog *PictureBrowser)
    {
        PictureBrowser->setWindowTitle(QCoreApplication::translate("PictureBrowser", "Picture Browser", nullptr));
        navigationBox->setItemText(0, QCoreApplication::translate("PictureBrowser", "Folder Browser", nullptr));
        navigationBox->setItemText(1, QCoreApplication::translate("PictureBrowser", "Collection Browser", nullptr));
        navigationBox->setItemText(2, QCoreApplication::translate("PictureBrowser", "Document Browser", nullptr));

        subdirsCheckbox->setText(QCoreApplication::translate("PictureBrowser", "Include Subdirectories", nullptr));
#if QT_CONFIG(tooltip)
        collectionsNewCategoryButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Create new category", nullptr));
#endif // QT_CONFIG(tooltip)
        collectionsNewCategoryButton->setText(QString());
#if QT_CONFIG(tooltip)
        collectionsNewButton->setToolTip(QCoreApplication::translate("PictureBrowser", "New collection", nullptr));
#endif // QT_CONFIG(tooltip)
        collectionsNewButton->setText(QString());
#if QT_CONFIG(tooltip)
        collectionsDeleteButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Delete selected collection", nullptr));
#endif // QT_CONFIG(tooltip)
        collectionsDeleteButton->setText(QString());
#if QT_CONFIG(tooltip)
        collectionsRenameButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Edit selected collection", nullptr));
#endif // QT_CONFIG(tooltip)
        collectionsRenameButton->setText(QString());
#if QT_CONFIG(tooltip)
        collectionsAddImagesButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Add selected images to collection(s)", nullptr));
#endif // QT_CONFIG(tooltip)
        collectionsAddImagesButton->setText(QString());
#if QT_CONFIG(tooltip)
        collectionsRemoveImagesButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Remove selected images from current collection", nullptr));
#endif // QT_CONFIG(tooltip)
        collectionsRemoveImagesButton->setText(QString());
#if QT_CONFIG(tooltip)
        collectionsTagImagesButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Tag selected images", nullptr));
#endif // QT_CONFIG(tooltip)
        collectionsTagImagesButton->setText(QString());
#if QT_CONFIG(tooltip)
        collectionsImportButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Import collection/collectionset", nullptr));
#endif // QT_CONFIG(tooltip)
        collectionsImportButton->setText(QString());
#if QT_CONFIG(tooltip)
        collectionsExportButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Export collection/collectionset", nullptr));
#endif // QT_CONFIG(tooltip)
        collectionsExportButton->setText(QString());
        label_8->setText(QCoreApplication::translate("PictureBrowser", "add selected images to collections", nullptr));
        collectionsAddImagesOkButton->setText(QString());
        collectionsAddImagesCancelButton->setText(QString());
#if QT_CONFIG(tooltip)
        gotoPageButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Move the document view to the selected page", nullptr));
#endif // QT_CONFIG(tooltip)
        gotoPageButton->setText(QCoreApplication::translate("PictureBrowser", "Go to Page", nullptr));
#if QT_CONFIG(tooltip)
        moreButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Show/Hide advanced tools", nullptr));
#endif // QT_CONFIG(tooltip)
        moreButton->setText(QCoreApplication::translate("PictureBrowser", "More", nullptr));
#if QT_CONFIG(tooltip)
        alwaysOnTopCheckbox->setToolTip(QCoreApplication::translate("PictureBrowser", "Always keep Picture Browser on top of other windows", nullptr));
#endif // QT_CONFIG(tooltip)
        alwaysOnTopCheckbox->setText(QCoreApplication::translate("PictureBrowser", "Always on Top", nullptr));
        sortCombobox->setItemText(0, QCoreApplication::translate("PictureBrowser", "Sort by Name", nullptr));
        sortCombobox->setItemText(1, QCoreApplication::translate("PictureBrowser", "Sort by Date", nullptr));
        sortCombobox->setItemText(2, QCoreApplication::translate("PictureBrowser", "Sort by Size", nullptr));
        sortCombobox->setItemText(3, QCoreApplication::translate("PictureBrowser", "Sort by Type", nullptr));

#if QT_CONFIG(tooltip)
        sortOrderButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Change sort order", nullptr));
#endif // QT_CONFIG(tooltip)
        sortOrderButton->setText(QString());
#if QT_CONFIG(tooltip)
        zoomPlusButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Zoom in", nullptr));
#endif // QT_CONFIG(tooltip)
        zoomPlusButton->setText(QString());
#if QT_CONFIG(tooltip)
        zoomMinusButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Zoom out", nullptr));
#endif // QT_CONFIG(tooltip)
        zoomMinusButton->setText(QString());
        previewModeCombobox->setItemText(0, QCoreApplication::translate("PictureBrowser", "Preview Images Only", nullptr));
        previewModeCombobox->setItemText(1, QCoreApplication::translate("PictureBrowser", "Preview with Information", nullptr));

#if QT_CONFIG(tooltip)
        previewModeCombobox->setToolTip(QCoreApplication::translate("PictureBrowser", "Change preview settings", nullptr));
#endif // QT_CONFIG(tooltip)
        actionsCombobox->setItemText(0, QCoreApplication::translate("PictureBrowser", "Quick Insert", nullptr));
        actionsCombobox->setItemText(1, QCoreApplication::translate("PictureBrowser", "Custom Insert", nullptr));
        actionsCombobox->setItemText(2, QCoreApplication::translate("PictureBrowser", "Add to Collection", nullptr));

#if QT_CONFIG(tooltip)
        actionsGoButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Go", nullptr));
#endif // QT_CONFIG(tooltip)
        actionsGoButton->setText(QString());
        statusLabel->setText(QCoreApplication::translate("PictureBrowser", "0 Image(s) Displayed, 0 Image(s) Filtered", nullptr));
#if QT_CONFIG(tooltip)
        saveSettingsCheckbox->setToolTip(QCoreApplication::translate("PictureBrowser", "Enable Picture Browser autosave settings", nullptr));
#endif // QT_CONFIG(tooltip)
        saveSettingsCheckbox->setText(QCoreApplication::translate("PictureBrowser", "Autosave Settings", nullptr));
#if QT_CONFIG(tooltip)
        resetSettingsButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Reset Picture Browser to default settings", nullptr));
#endif // QT_CONFIG(tooltip)
        resetSettingsButton->setText(QCoreApplication::translate("PictureBrowser", "Reset Settings", nullptr));
        closeButton->setText(QCoreApplication::translate("PictureBrowser", "Close", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("PictureBrowser", "File", nullptr));
        label_10->setText(QCoreApplication::translate("PictureBrowser", "Name:", nullptr));
        informationFileNameLabel->setText(QString());
        label_11->setText(QCoreApplication::translate("PictureBrowser", "Path:", nullptr));
        informationFilePathLabel->setText(QString());
        jumpToImageButton->setText(QCoreApplication::translate("PictureBrowser", "Go", nullptr));
        label_22->setText(QCoreApplication::translate("PictureBrowser", "Size:", nullptr));
        informationFileSizeLabel->setText(QString());
        label_23->setText(QCoreApplication::translate("PictureBrowser", "Date:", nullptr));
        informationFileDateLabel->setText(QString());
        groupBox->setTitle(QCoreApplication::translate("PictureBrowser", "Image", nullptr));
        informationEmbeddedLabel->setText(QString());
        informationDpiLabel->setText(QString());
        informationLayersLabel->setText(QString());
        label_9->setText(QCoreApplication::translate("PictureBrowser", "Embedded Profile:", nullptr));
        informationHeightLabel->setText(QString());
        label_15->setText(QCoreApplication::translate("PictureBrowser", "Profile Name:", nullptr));
        informationProfileNameLabel->setText(QString());
        label_3->setText(QCoreApplication::translate("PictureBrowser", "Width:", nullptr));
        informationFormatLabel->setText(QString());
        informationWidthLabel->setText(QString());
        informationColorspaceLabel->setText(QString());
        label_12->setText(QCoreApplication::translate("PictureBrowser", "Format:", nullptr));
        label_13->setText(QCoreApplication::translate("PictureBrowser", "Colorspace:", nullptr));
        label_5->setText(QCoreApplication::translate("PictureBrowser", "Layers:", nullptr));
        label_14->setText(QCoreApplication::translate("PictureBrowser", "DPI:", nullptr));
        label_4->setText(QCoreApplication::translate("PictureBrowser", "Height:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("PictureBrowser", "Information", nullptr));
        label_6->setText(QCoreApplication::translate("PictureBrowser", "Filters:", nullptr));
        filterTargetCombobox->setItemText(0, QCoreApplication::translate("PictureBrowser", "Filter Displayed Images", nullptr));
        filterTargetCombobox->setItemText(1, QCoreApplication::translate("PictureBrowser", "Search Images", nullptr));
        filterTargetCombobox->setItemText(2, QCoreApplication::translate("PictureBrowser", "Filter Collections", nullptr));

#if QT_CONFIG(tooltip)
        filterSearchDirButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Select a directory", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox_3->setTitle(QCoreApplication::translate("PictureBrowser", "Criteria", nullptr));
#if QT_CONFIG(tooltip)
        filterAddCriterionButton->setToolTip(QCoreApplication::translate("PictureBrowser", "Add filter", nullptr));
#endif // QT_CONFIG(tooltip)
        filterAddCriterionButton->setText(QCoreApplication::translate("PictureBrowser", "<< Add", nullptr));
        filterCriteriaCombobox->setItemText(0, QCoreApplication::translate("PictureBrowser", "Name (Wildcards Allowed)", nullptr));
        filterCriteriaCombobox->setItemText(1, QCoreApplication::translate("PictureBrowser", "Date", nullptr));
        filterCriteriaCombobox->setItemText(2, QCoreApplication::translate("PictureBrowser", "Size", nullptr));
        filterCriteriaCombobox->setItemText(3, QCoreApplication::translate("PictureBrowser", "Type", nullptr));
        filterCriteriaCombobox->setItemText(4, QCoreApplication::translate("PictureBrowser", "Tag", nullptr));

#if QT_CONFIG(tooltip)
        filterCriteriaCombobox->setToolTip(QCoreApplication::translate("PictureBrowser", "Filter criteria", nullptr));
#endif // QT_CONFIG(tooltip)
        filterNameCombobox->setItemText(0, QCoreApplication::translate("PictureBrowser", "Contains", nullptr));
        filterNameCombobox->setItemText(1, QCoreApplication::translate("PictureBrowser", "Doesn't Contain", nullptr));

#if QT_CONFIG(tooltip)
        filterNameCombobox->setToolTip(QCoreApplication::translate("PictureBrowser", "Search criteria", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        filterNameLineedit->setToolTip(QCoreApplication::translate("PictureBrowser", "Type name", nullptr));
#endif // QT_CONFIG(tooltip)
        filterDateCombobox->setItemText(0, QCoreApplication::translate("PictureBrowser", "Newer than", nullptr));
        filterDateCombobox->setItemText(1, QCoreApplication::translate("PictureBrowser", "Older than", nullptr));

#if QT_CONFIG(tooltip)
        filterDateCombobox->setToolTip(QCoreApplication::translate("PictureBrowser", "Search criteria", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        filterDateDatetimeedit->setToolTip(QCoreApplication::translate("PictureBrowser", "Set date", nullptr));
#endif // QT_CONFIG(tooltip)
        filterSizeCombobox->setItemText(0, QCoreApplication::translate("PictureBrowser", "Smaller than", nullptr));
        filterSizeCombobox->setItemText(1, QCoreApplication::translate("PictureBrowser", "Bigger than", nullptr));

#if QT_CONFIG(tooltip)
        filterSizeCombobox->setToolTip(QCoreApplication::translate("PictureBrowser", "Search criteria", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        filterSizeSpinbox->setToolTip(QCoreApplication::translate("PictureBrowser", "Set size", nullptr));
#endif // QT_CONFIG(tooltip)
        filterSizeSpinbox->setSuffix(QCoreApplication::translate("PictureBrowser", " KB", nullptr));
#if QT_CONFIG(tooltip)
        filterTypeCombobox->setToolTip(QCoreApplication::translate("PictureBrowser", "Select file type(s)", nullptr));
#endif // QT_CONFIG(tooltip)
        filterAddTagButton->setText(QCoreApplication::translate("PictureBrowser", "add", nullptr));
        filterFilterButton->setText(QCoreApplication::translate("PictureBrowser", "Apply Filters", nullptr));
        filterClearButton->setText(QCoreApplication::translate("PictureBrowser", "Clear Filters", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("PictureBrowser", "Filter / Search", nullptr));
        groupBox_5->setTitle(QCoreApplication::translate("PictureBrowser", "Image Placement", nullptr));
        label->setText(QCoreApplication::translate("PictureBrowser", "Page(s):", nullptr));
#if QT_CONFIG(tooltip)
        insertPagesCombobox->setToolTip(QCoreApplication::translate("PictureBrowser", "Select the page(s) where to insert the image(s)", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(QCoreApplication::translate("PictureBrowser", "Image Frame(s):", nullptr));
#if QT_CONFIG(tooltip)
        insertFramesCombobox->setToolTip(QCoreApplication::translate("PictureBrowser", "Select which frame(s) to use", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox_6->setTitle(QCoreApplication::translate("PictureBrowser", "Position of New Frame(s)", nullptr));
        insertPositionCombobox->setItemText(0, QCoreApplication::translate("PictureBrowser", "Top Left of Margins", nullptr));
        insertPositionCombobox->setItemText(1, QCoreApplication::translate("PictureBrowser", "Top Left of Page", nullptr));
        insertPositionCombobox->setItemText(2, QCoreApplication::translate("PictureBrowser", "Top Left of Bleed", nullptr));
        insertPositionCombobox->setItemText(3, QCoreApplication::translate("PictureBrowser", "Custom Position", nullptr));

#if QT_CONFIG(tooltip)
        insertPositionCombobox->setToolTip(QCoreApplication::translate("PictureBrowser", "Set the position of the image frame(s) to insert", nullptr));
#endif // QT_CONFIG(tooltip)
        insertPositionYLabel->setText(QCoreApplication::translate("PictureBrowser", "Y:", nullptr));
        insertPositionXLabel->setText(QCoreApplication::translate("PictureBrowser", "X:", nullptr));
#if QT_CONFIG(tooltip)
        insertPositionXSpinbox->setToolTip(QCoreApplication::translate("PictureBrowser", "Horizontal position", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        insertPositionYSpinbox->setToolTip(QCoreApplication::translate("PictureBrowser", "Vertical position", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox_7->setTitle(QCoreApplication::translate("PictureBrowser", "Size", nullptr));
        insertWidthLabel->setText(QCoreApplication::translate("PictureBrowser", "Width:", nullptr));
        insertHeightLabel->setText(QCoreApplication::translate("PictureBrowser", "Height:", nullptr));
#if QT_CONFIG(tooltip)
        insertHeightSpinbox->setToolTip(QCoreApplication::translate("PictureBrowser", "Height", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        insertWidthSpinbox->setToolTip(QCoreApplication::translate("PictureBrowser", "Width", nullptr));
#endif // QT_CONFIG(tooltip)
        insertSizeCombobox->setItemText(0, QCoreApplication::translate("PictureBrowser", "Same as Page Margins", nullptr));
        insertSizeCombobox->setItemText(1, QCoreApplication::translate("PictureBrowser", "Same as Page", nullptr));
        insertSizeCombobox->setItemText(2, QCoreApplication::translate("PictureBrowser", "Same as Bleed", nullptr));
        insertSizeCombobox->setItemText(3, QCoreApplication::translate("PictureBrowser", "Same as Image", nullptr));
        insertSizeCombobox->setItemText(4, QCoreApplication::translate("PictureBrowser", "Custom Size", nullptr));

#if QT_CONFIG(tooltip)
        insertSizeCombobox->setToolTip(QCoreApplication::translate("PictureBrowser", "Set the size of the image frame(s) to insert", nullptr));
#endif // QT_CONFIG(tooltip)
        label_25->setText(QCoreApplication::translate("PictureBrowser", "Insert Image into", nullptr));
        label_26->setText(QCoreApplication::translate("PictureBrowser", "0 Page(s)", nullptr));
        label_27->setText(QCoreApplication::translate("PictureBrowser", "0 Frame(s)", nullptr));
        insertImageButton->setText(QCoreApplication::translate("PictureBrowser", "Insert", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), QCoreApplication::translate("PictureBrowser", "Insert Image", nullptr));
        label_7->setText(QCoreApplication::translate("PictureBrowser", "Selected Images:", nullptr));
        groupBox_9->setTitle(QCoreApplication::translate("PictureBrowser", "Tag Selected Image(s)", nullptr));
#if QT_CONFIG(tooltip)
        collectionsAddNewTagLineedit->setToolTip(QCoreApplication::translate("PictureBrowser", "Type Tag name", nullptr));
#endif // QT_CONFIG(tooltip)
        collectionsAddNewTagButton->setText(QCoreApplication::translate("PictureBrowser", "Add New Tag", nullptr));
        collectionsSetTagsButton->setText(QCoreApplication::translate("PictureBrowser", "Set Tags", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_5), QCoreApplication::translate("PictureBrowser", "Tag Image(s)", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PictureBrowser: public Ui_PictureBrowser {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PICTUREBROWSER_H
