/********************************************************************************
** Form generated from reading UI file 'documentlogviewerbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_DOCUMENTLOGVIEWERBASE_H
#define UI_DOCUMENTLOGVIEWERBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_DocumentLogViewerBase
{
public:
    QVBoxLayout *verticalLayout;
    QTableWidget *logTableWidget;
    QPushButton *clearPushButton;

    void setupUi(QDialog *DocumentLogViewerBase)
    {
        if (DocumentLogViewerBase->objectName().isEmpty())
            DocumentLogViewerBase->setObjectName(QString::fromUtf8("DocumentLogViewerBase"));
        DocumentLogViewerBase->resize(400, 300);
        verticalLayout = new QVBoxLayout(DocumentLogViewerBase);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        logTableWidget = new QTableWidget(DocumentLogViewerBase);
        if (logTableWidget->columnCount() < 4)
            logTableWidget->setColumnCount(4);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        logTableWidget->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        logTableWidget->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        logTableWidget->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        logTableWidget->setHorizontalHeaderItem(3, __qtablewidgetitem3);
        logTableWidget->setObjectName(QString::fromUtf8("logTableWidget"));
        logTableWidget->setEditTriggers(QAbstractItemView::NoEditTriggers);
        logTableWidget->setSelectionBehavior(QAbstractItemView::SelectRows);
        logTableWidget->horizontalHeader()->setStretchLastSection(true);

        verticalLayout->addWidget(logTableWidget);

        clearPushButton = new QPushButton(DocumentLogViewerBase);
        clearPushButton->setObjectName(QString::fromUtf8("clearPushButton"));

        verticalLayout->addWidget(clearPushButton);


        retranslateUi(DocumentLogViewerBase);

        QMetaObject::connectSlotsByName(DocumentLogViewerBase);
    } // setupUi

    void retranslateUi(QDialog *DocumentLogViewerBase)
    {
        DocumentLogViewerBase->setWindowTitle(QCoreApplication::translate("DocumentLogViewerBase", "Document Log", nullptr));
        QTableWidgetItem *___qtablewidgetitem = logTableWidget->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("DocumentLogViewerBase", "Time", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = logTableWidget->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("DocumentLogViewerBase", "Level", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = logTableWidget->horizontalHeaderItem(2);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("DocumentLogViewerBase", "Source", nullptr));
        QTableWidgetItem *___qtablewidgetitem3 = logTableWidget->horizontalHeaderItem(3);
        ___qtablewidgetitem3->setText(QCoreApplication::translate("DocumentLogViewerBase", "Message", nullptr));
        clearPushButton->setText(QCoreApplication::translate("DocumentLogViewerBase", "Clear Log", nullptr));
    } // retranslateUi

};

namespace Ui {
    class DocumentLogViewerBase: public Ui_DocumentLogViewerBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_DOCUMENTLOGVIEWERBASE_H
