/****************************************************************************
** Meta object code from reading C++ file 'propertiespalette.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/propertiespalette.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertiespalette.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PropertiesPalette_t {
    QByteArrayData data[40];
    char stringdata0[386];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PropertiesPalette_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PropertiesPalette_t qt_meta_stringdata_PropertiesPalette = {
    {
QT_MOC_LITERAL(0, 0, 17), // "PropertiesPalette"
QT_MOC_LITERAL(1, 18, 13), // "setMainWindow"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 18), // "ScribusMainWindow*"
QT_MOC_LITERAL(4, 52, 2), // "mw"
QT_MOC_LITERAL(5, 55, 6), // "setDoc"
QT_MOC_LITERAL(6, 62, 11), // "ScribusDoc*"
QT_MOC_LITERAL(7, 74, 1), // "d"
QT_MOC_LITERAL(8, 76, 14), // "AppModeChanged"
QT_MOC_LITERAL(9, 91, 14), // "setCurrentItem"
QT_MOC_LITERAL(10, 106, 9), // "PageItem*"
QT_MOC_LITERAL(11, 116, 4), // "item"
QT_MOC_LITERAL(12, 121, 8), // "unsetDoc"
QT_MOC_LITERAL(13, 130, 9), // "unsetItem"
QT_MOC_LITERAL(14, 140, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(15, 163, 10), // "unitChange"
QT_MOC_LITERAL(16, 174, 14), // "languageChange"
QT_MOC_LITERAL(17, 189, 9), // "setLocked"
QT_MOC_LITERAL(18, 199, 6), // "SelTab"
QT_MOC_LITERAL(19, 206, 1), // "t"
QT_MOC_LITERAL(20, 208, 11), // "NewLineMode"
QT_MOC_LITERAL(21, 220, 4), // "mode"
QT_MOC_LITERAL(22, 225, 21), // "handleGradientChanged"
QT_MOC_LITERAL(23, 247, 14), // "handleNewShape"
QT_MOC_LITERAL(24, 262, 9), // "frameType"
QT_MOC_LITERAL(25, 272, 13), // "NewSpGradient"
QT_MOC_LITERAL(26, 286, 2), // "x1"
QT_MOC_LITERAL(27, 289, 2), // "y1"
QT_MOC_LITERAL(28, 292, 2), // "x2"
QT_MOC_LITERAL(29, 295, 2), // "y2"
QT_MOC_LITERAL(30, 298, 2), // "fx"
QT_MOC_LITERAL(31, 301, 2), // "fy"
QT_MOC_LITERAL(32, 304, 2), // "sg"
QT_MOC_LITERAL(33, 307, 2), // "sk"
QT_MOC_LITERAL(34, 310, 2), // "cx"
QT_MOC_LITERAL(35, 313, 2), // "cy"
QT_MOC_LITERAL(36, 316, 18), // "toggleGradientEdit"
QT_MOC_LITERAL(37, 335, 14), // "NewSpGradientM"
QT_MOC_LITERAL(38, 350, 19), // "toggleGradientEditM"
QT_MOC_LITERAL(39, 370, 15) // "handleShapeEdit"

    },
    "PropertiesPalette\0setMainWindow\0\0"
    "ScribusMainWindow*\0mw\0setDoc\0ScribusDoc*\0"
    "d\0AppModeChanged\0setCurrentItem\0"
    "PageItem*\0item\0unsetDoc\0unsetItem\0"
    "handleSelectionChanged\0unitChange\0"
    "languageChange\0setLocked\0SelTab\0t\0"
    "NewLineMode\0mode\0handleGradientChanged\0"
    "handleNewShape\0frameType\0NewSpGradient\0"
    "x1\0y1\0x2\0y2\0fx\0fy\0sg\0sk\0cx\0cy\0"
    "toggleGradientEdit\0NewSpGradientM\0"
    "toggleGradientEditM\0handleShapeEdit"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PropertiesPalette[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  109,    2, 0x0a /* Public */,
       5,    1,  112,    2, 0x0a /* Public */,
       8,    0,  115,    2, 0x0a /* Public */,
       9,    1,  116,    2, 0x0a /* Public */,
      12,    0,  119,    2, 0x0a /* Public */,
      13,    0,  120,    2, 0x0a /* Public */,
      14,    0,  121,    2, 0x0a /* Public */,
      15,    0,  122,    2, 0x0a /* Public */,
      16,    0,  123,    2, 0x0a /* Public */,
      17,    1,  124,    2, 0x0a /* Public */,
      18,    1,  127,    2, 0x08 /* Private */,
      20,    1,  130,    2, 0x08 /* Private */,
      22,    0,  133,    2, 0x08 /* Private */,
      23,    1,  134,    2, 0x08 /* Private */,
      25,   10,  137,    2, 0x08 /* Private */,
      36,    1,  158,    2, 0x08 /* Private */,
      37,    8,  161,    2, 0x08 /* Private */,
      38,    0,  178,    2, 0x08 /* Private */,
      39,    0,  179,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Int,   19,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   24,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double,   26,   27,   28,   29,   30,   31,   32,   33,   34,   35,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double,   26,   27,   28,   29,   30,   31,   32,   33,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PropertiesPalette::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertiesPalette *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setMainWindow((*reinterpret_cast< ScribusMainWindow*(*)>(_a[1]))); break;
        case 1: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 2: _t->AppModeChanged(); break;
        case 3: _t->setCurrentItem((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 4: _t->unsetDoc(); break;
        case 5: _t->unsetItem(); break;
        case 6: _t->handleSelectionChanged(); break;
        case 7: _t->unitChange(); break;
        case 8: _t->languageChange(); break;
        case 9: _t->setLocked((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->SelTab((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->NewLineMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->handleGradientChanged(); break;
        case 13: _t->handleNewShape((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->NewSpGradient((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6])),(*reinterpret_cast< double(*)>(_a[7])),(*reinterpret_cast< double(*)>(_a[8])),(*reinterpret_cast< double(*)>(_a[9])),(*reinterpret_cast< double(*)>(_a[10]))); break;
        case 15: _t->toggleGradientEdit((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->NewSpGradientM((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6])),(*reinterpret_cast< double(*)>(_a[7])),(*reinterpret_cast< double(*)>(_a[8]))); break;
        case 17: _t->toggleGradientEditM(); break;
        case 18: _t->handleShapeEdit(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PropertiesPalette::staticMetaObject = { {
    QMetaObject::SuperData::link<ScDockPalette::staticMetaObject>(),
    qt_meta_stringdata_PropertiesPalette.data,
    qt_meta_data_PropertiesPalette,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PropertiesPalette::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PropertiesPalette::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PropertiesPalette.stringdata0))
        return static_cast<void*>(this);
    return ScDockPalette::qt_metacast(_clname);
}

int PropertiesPalette::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScDockPalette::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
