/********************************************************************************
** Form generated from reading UI file 'outputpreviewbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OUTPUTPREVIEWBASE_H
#define UI_OUTPUTPREVIEWBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/pageselector.h"

QT_BEGIN_NAMESPACE

class Ui_OutputPreviewBase
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *topHLayout;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *settingsBarLayout;
    QWidget *optionWidget;
    QSpacerItem *verticalSpacer;
    QHBoxLayout *bottomHLayout;
    QLabel *scaleLabel;
    QComboBox *scaleBox;
    QSpacerItem *horizontalSpacer;
    PageSelector *pageSelector;
    QSpacerItem *horizontalSpacer_2;
    QPushButton *closeButton;
    QPushButton *exportButton;

    void setupUi(QDialog *OutputPreviewBase)
    {
        if (OutputPreviewBase->objectName().isEmpty())
            OutputPreviewBase->setObjectName(QString::fromUtf8("OutputPreviewBase"));
        OutputPreviewBase->resize(842, 733);
        verticalLayout = new QVBoxLayout(OutputPreviewBase);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        topHLayout = new QHBoxLayout();
        topHLayout->setObjectName(QString::fromUtf8("topHLayout"));
        scrollArea = new QScrollArea(OutputPreviewBase);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 810, 702));
        scrollArea->setWidget(scrollAreaWidgetContents);

        topHLayout->addWidget(scrollArea);

        settingsBarLayout = new QVBoxLayout();
        settingsBarLayout->setObjectName(QString::fromUtf8("settingsBarLayout"));
        optionWidget = new QWidget(OutputPreviewBase);
        optionWidget->setObjectName(QString::fromUtf8("optionWidget"));
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(optionWidget->sizePolicy().hasHeightForWidth());
        optionWidget->setSizePolicy(sizePolicy);

        settingsBarLayout->addWidget(optionWidget);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        settingsBarLayout->addItem(verticalSpacer);


        topHLayout->addLayout(settingsBarLayout);


        verticalLayout->addLayout(topHLayout);

        bottomHLayout = new QHBoxLayout();
        bottomHLayout->setObjectName(QString::fromUtf8("bottomHLayout"));
        scaleLabel = new QLabel(OutputPreviewBase);
        scaleLabel->setObjectName(QString::fromUtf8("scaleLabel"));

        bottomHLayout->addWidget(scaleLabel);

        scaleBox = new QComboBox(OutputPreviewBase);
        scaleBox->addItem(QString());
        scaleBox->addItem(QString());
        scaleBox->addItem(QString());
        scaleBox->addItem(QString());
        scaleBox->addItem(QString());
        scaleBox->addItem(QString());
        scaleBox->addItem(QString());
        scaleBox->setObjectName(QString::fromUtf8("scaleBox"));
        scaleBox->setFocusPolicy(Qt::NoFocus);

        bottomHLayout->addWidget(scaleBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        bottomHLayout->addItem(horizontalSpacer);

        pageSelector = new PageSelector(OutputPreviewBase);
        pageSelector->setObjectName(QString::fromUtf8("pageSelector"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(pageSelector->sizePolicy().hasHeightForWidth());
        pageSelector->setSizePolicy(sizePolicy1);

        bottomHLayout->addWidget(pageSelector);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        bottomHLayout->addItem(horizontalSpacer_2);

        closeButton = new QPushButton(OutputPreviewBase);
        closeButton->setObjectName(QString::fromUtf8("closeButton"));
        closeButton->setAutoDefault(false);

        bottomHLayout->addWidget(closeButton);

        exportButton = new QPushButton(OutputPreviewBase);
        exportButton->setObjectName(QString::fromUtf8("exportButton"));
        exportButton->setAutoDefault(false);

        bottomHLayout->addWidget(exportButton);


        verticalLayout->addLayout(bottomHLayout);


        retranslateUi(OutputPreviewBase);

        QMetaObject::connectSlotsByName(OutputPreviewBase);
    } // setupUi

    void retranslateUi(QDialog *OutputPreviewBase)
    {
        OutputPreviewBase->setWindowTitle(QCoreApplication::translate("OutputPreviewBase", "Dialog", nullptr));
        scaleLabel->setText(QCoreApplication::translate("OutputPreviewBase", "Scaling:", nullptr));
        scaleBox->setItemText(0, QCoreApplication::translate("OutputPreviewBase", "50%", nullptr));
        scaleBox->setItemText(1, QCoreApplication::translate("OutputPreviewBase", "100%", nullptr));
        scaleBox->setItemText(2, QCoreApplication::translate("OutputPreviewBase", "150%", nullptr));
        scaleBox->setItemText(3, QCoreApplication::translate("OutputPreviewBase", "200%", nullptr));
        scaleBox->setItemText(4, QCoreApplication::translate("OutputPreviewBase", "Fit to Width", nullptr));
        scaleBox->setItemText(5, QCoreApplication::translate("OutputPreviewBase", "Fit to Height", nullptr));
        scaleBox->setItemText(6, QCoreApplication::translate("OutputPreviewBase", "Fit to Page", nullptr));

#if QT_CONFIG(tooltip)
        scaleBox->setToolTip(QCoreApplication::translate("OutputPreviewBase", "Resize the scale of the page", nullptr));
#endif // QT_CONFIG(tooltip)
        closeButton->setText(QCoreApplication::translate("OutputPreviewBase", "Close", nullptr));
        exportButton->setText(QCoreApplication::translate("OutputPreviewBase", "Export...", nullptr));
    } // retranslateUi

};

namespace Ui {
    class OutputPreviewBase: public Ui_OutputPreviewBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OUTPUTPREVIEWBASE_H
