/********************************************************************************
** Form generated from reading UI file 'smcstylewidget.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SMCSTYLEWIDGET_H
#define UI_SMCSTYLEWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/smcolorcombo.h"
#include "ui/smfontcomboh.h"
#include "ui/smfontfeatures.h"
#include "ui/smlineedit.h"
#include "ui/smsccombobox.h"
#include "ui/smscrspinbox.h"
#include "ui/smshadebutton.h"
#include "ui/smspinbox.h"
#include "ui/smstyleselect.h"

QT_BEGIN_NAMESPACE

class Ui_SMCStyleWidget
{
public:
    QVBoxLayout *verticalLayout_3;
    QTabWidget *tabwidget;
    QWidget *tabProperties;
    QVBoxLayout *verticalLayout_6;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *_3;
    QLabel *parentLabel;
    QComboBox *parentCombo;
    QGroupBox *basicGroup;
    QVBoxLayout *_4;
    SMFontComboH *fontFace_;
    QHBoxLayout *_5;
    QLabel *fontSizeLabel_;
    SMScrSpinBox *fontSize_;
    QLabel *trackingLabel_;
    SMScrSpinBox *tracking_;
    QLabel *widthSpaceLabel;
    SMScrSpinBox *widthSpaceSpin;
    QSpacerItem *spacerItem;
    QHBoxLayout *_6;
    SMStyleSelect *effects_;
    QSpacerItem *spacerItem1;
    QGroupBox *advGroup;
    QVBoxLayout *_7;
    QHBoxLayout *_8;
    QLabel *hscaleLabel_;
    SMScrSpinBox *fontHScale_;
    QLabel *vscaleLabel_;
    SMScrSpinBox *fontVScale_;
    QLabel *baselineOffsetLabel_;
    SMScrSpinBox *baselineOffset_;
    QSpacerItem *spacerItem2;
    QHBoxLayout *_9;
    QLabel *languageLabel_;
    SMScComboBox *language_;
    QSpacerItem *spacerItem3;
    QGridLayout *gridLayout;
    QGroupBox *smColorGroup;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *_2;
    QLabel *FillIcon;
    SMColorCombo *fillColor_;
    QLabel *fillShadeLabel;
    SMShadeButton *fillShade_;
    QSpacerItem *spacerItem4;
    QHBoxLayout *_10;
    QLabel *StrokeIcon;
    SMColorCombo *strokeColor_;
    QLabel *strokeShadeLabel;
    SMShadeButton *strokeShade_;
    QSpacerItem *spacerItem5;
    QHBoxLayout *_11;
    QLabel *backIcon;
    SMColorCombo *backColor_;
    QLabel *backShadeLabel;
    SMShadeButton *backShade_;
    QSpacerItem *spacerItem6;
    QGroupBox *hyphenGroup;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_4;
    SMLineEdit *hyphenCharLineEdit;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout;
    QLabel *smallestWordLabel;
    SMSpinBox *smallestWordSpinBox;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *spacerItem7;
    QSpacerItem *verticalSpacer;
    QWidget *tabFontFeatures;
    QVBoxLayout *verticalLayout_5;
    SMFontFeatures *fontfeaturesSetting;

    void setupUi(QWidget *SMCStyleWidget)
    {
        if (SMCStyleWidget->objectName().isEmpty())
            SMCStyleWidget->setObjectName(QString::fromUtf8("SMCStyleWidget"));
        SMCStyleWidget->resize(531, 492);
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(SMCStyleWidget->sizePolicy().hasHeightForWidth());
        SMCStyleWidget->setSizePolicy(sizePolicy);
        verticalLayout_3 = new QVBoxLayout(SMCStyleWidget);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        tabwidget = new QTabWidget(SMCStyleWidget);
        tabwidget->setObjectName(QString::fromUtf8("tabwidget"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(tabwidget->sizePolicy().hasHeightForWidth());
        tabwidget->setSizePolicy(sizePolicy1);
        tabProperties = new QWidget();
        tabProperties->setObjectName(QString::fromUtf8("tabProperties"));
        sizePolicy1.setHeightForWidth(tabProperties->sizePolicy().hasHeightForWidth());
        tabProperties->setSizePolicy(sizePolicy1);
        verticalLayout_6 = new QVBoxLayout(tabProperties);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        _3 = new QHBoxLayout();
        _3->setObjectName(QString::fromUtf8("_3"));
        parentLabel = new QLabel(tabProperties);
        parentLabel->setObjectName(QString::fromUtf8("parentLabel"));
        parentLabel->setWordWrap(false);

        _3->addWidget(parentLabel);

        parentCombo = new QComboBox(tabProperties);
        parentCombo->setObjectName(QString::fromUtf8("parentCombo"));
        QSizePolicy sizePolicy2(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(5);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(parentCombo->sizePolicy().hasHeightForWidth());
        parentCombo->setSizePolicy(sizePolicy2);

        _3->addWidget(parentCombo);


        verticalLayout_2->addLayout(_3);

        basicGroup = new QGroupBox(tabProperties);
        basicGroup->setObjectName(QString::fromUtf8("basicGroup"));
        _4 = new QVBoxLayout(basicGroup);
        _4->setObjectName(QString::fromUtf8("_4"));
        fontFace_ = new SMFontComboH(basicGroup);
        fontFace_->setObjectName(QString::fromUtf8("fontFace_"));
        sizePolicy1.setHeightForWidth(fontFace_->sizePolicy().hasHeightForWidth());
        fontFace_->setSizePolicy(sizePolicy1);
        fontFace_->setMinimumSize(QSize(0, 20));

        _4->addWidget(fontFace_);

        _5 = new QHBoxLayout();
        _5->setObjectName(QString::fromUtf8("_5"));
        fontSizeLabel_ = new QLabel(basicGroup);
        fontSizeLabel_->setObjectName(QString::fromUtf8("fontSizeLabel_"));
        fontSizeLabel_->setMinimumSize(QSize(20, 0));
        fontSizeLabel_->setMaximumSize(QSize(22, 22));

        _5->addWidget(fontSizeLabel_);

        fontSize_ = new SMScrSpinBox(basicGroup);
        fontSize_->setObjectName(QString::fromUtf8("fontSize_"));
        fontSize_->setMinimum(1.000000000000000);
        fontSize_->setMaximum(2048.000000000000000);

        _5->addWidget(fontSize_);

        trackingLabel_ = new QLabel(basicGroup);
        trackingLabel_->setObjectName(QString::fromUtf8("trackingLabel_"));
        trackingLabel_->setMinimumSize(QSize(20, 0));
        trackingLabel_->setMaximumSize(QSize(22, 22));

        _5->addWidget(trackingLabel_);

        tracking_ = new SMScrSpinBox(basicGroup);
        tracking_->setObjectName(QString::fromUtf8("tracking_"));
        tracking_->setMinimum(-300.000000000000000);
        tracking_->setMaximum(300.000000000000000);

        _5->addWidget(tracking_);

        widthSpaceLabel = new QLabel(basicGroup);
        widthSpaceLabel->setObjectName(QString::fromUtf8("widthSpaceLabel"));
        widthSpaceLabel->setMinimumSize(QSize(20, 0));
        widthSpaceLabel->setMaximumSize(QSize(22, 22));
        widthSpaceLabel->setTextInteractionFlags(Qt::NoTextInteraction);

        _5->addWidget(widthSpaceLabel);

        widthSpaceSpin = new SMScrSpinBox(basicGroup);
        widthSpaceSpin->setObjectName(QString::fromUtf8("widthSpaceSpin"));
        widthSpaceSpin->setMinimum(1.000000000000000);
        widthSpaceSpin->setMaximum(199.000000000000000);
        widthSpaceSpin->setValue(100.000000000000000);

        _5->addWidget(widthSpaceSpin);

        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _5->addItem(spacerItem);


        _4->addLayout(_5);

        _6 = new QHBoxLayout();
        _6->setObjectName(QString::fromUtf8("_6"));
        effects_ = new SMStyleSelect(basicGroup);
        effects_->setObjectName(QString::fromUtf8("effects_"));
        QSizePolicy sizePolicy3(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(effects_->sizePolicy().hasHeightForWidth());
        effects_->setSizePolicy(sizePolicy3);
        effects_->setMinimumSize(QSize(30, 0));

        _6->addWidget(effects_);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _6->addItem(spacerItem1);


        _4->addLayout(_6);


        verticalLayout_2->addWidget(basicGroup);

        advGroup = new QGroupBox(tabProperties);
        advGroup->setObjectName(QString::fromUtf8("advGroup"));
        _7 = new QVBoxLayout(advGroup);
        _7->setObjectName(QString::fromUtf8("_7"));
        _8 = new QHBoxLayout();
        _8->setObjectName(QString::fromUtf8("_8"));
        hscaleLabel_ = new QLabel(advGroup);
        hscaleLabel_->setObjectName(QString::fromUtf8("hscaleLabel_"));
        hscaleLabel_->setMinimumSize(QSize(20, 0));
        hscaleLabel_->setMaximumSize(QSize(22, 22));

        _8->addWidget(hscaleLabel_);

        fontHScale_ = new SMScrSpinBox(advGroup);
        fontHScale_->setObjectName(QString::fromUtf8("fontHScale_"));
        fontHScale_->setMinimum(10.000000000000000);
        fontHScale_->setMaximum(400.000000000000000);

        _8->addWidget(fontHScale_);

        vscaleLabel_ = new QLabel(advGroup);
        vscaleLabel_->setObjectName(QString::fromUtf8("vscaleLabel_"));
        vscaleLabel_->setMinimumSize(QSize(20, 0));
        vscaleLabel_->setMaximumSize(QSize(22, 22));

        _8->addWidget(vscaleLabel_);

        fontVScale_ = new SMScrSpinBox(advGroup);
        fontVScale_->setObjectName(QString::fromUtf8("fontVScale_"));
        fontVScale_->setMinimum(10.000000000000000);
        fontVScale_->setMaximum(400.000000000000000);

        _8->addWidget(fontVScale_);

        baselineOffsetLabel_ = new QLabel(advGroup);
        baselineOffsetLabel_->setObjectName(QString::fromUtf8("baselineOffsetLabel_"));
        baselineOffsetLabel_->setMinimumSize(QSize(20, 0));
        baselineOffsetLabel_->setMaximumSize(QSize(22, 22));

        _8->addWidget(baselineOffsetLabel_);

        baselineOffset_ = new SMScrSpinBox(advGroup);
        baselineOffset_->setObjectName(QString::fromUtf8("baselineOffset_"));
        baselineOffset_->setMinimum(-100.000000000000000);
        baselineOffset_->setMaximum(100.000000000000000);

        _8->addWidget(baselineOffset_);

        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _8->addItem(spacerItem2);


        _7->addLayout(_8);

        _9 = new QHBoxLayout();
        _9->setObjectName(QString::fromUtf8("_9"));
        languageLabel_ = new QLabel(advGroup);
        languageLabel_->setObjectName(QString::fromUtf8("languageLabel_"));

        _9->addWidget(languageLabel_);

        language_ = new SMScComboBox(advGroup);
        language_->setObjectName(QString::fromUtf8("language_"));
        language_->setEditable(true);

        _9->addWidget(language_);

        spacerItem3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _9->addItem(spacerItem3);


        _7->addLayout(_9);


        verticalLayout_2->addWidget(advGroup);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        smColorGroup = new QGroupBox(tabProperties);
        smColorGroup->setObjectName(QString::fromUtf8("smColorGroup"));
        verticalLayout = new QVBoxLayout(smColorGroup);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        _2 = new QHBoxLayout();
        _2->setObjectName(QString::fromUtf8("_2"));
        FillIcon = new QLabel(smColorGroup);
        FillIcon->setObjectName(QString::fromUtf8("FillIcon"));
        FillIcon->setMinimumSize(QSize(20, 0));
        FillIcon->setMaximumSize(QSize(22, 22));

        _2->addWidget(FillIcon);

        fillColor_ = new SMColorCombo(smColorGroup);
        fillColor_->setObjectName(QString::fromUtf8("fillColor_"));

        _2->addWidget(fillColor_);

        fillShadeLabel = new QLabel(smColorGroup);
        fillShadeLabel->setObjectName(QString::fromUtf8("fillShadeLabel"));
        fillShadeLabel->setMinimumSize(QSize(20, 0));
        fillShadeLabel->setMaximumSize(QSize(22, 22));

        _2->addWidget(fillShadeLabel);

        fillShade_ = new SMShadeButton(smColorGroup);
        fillShade_->setObjectName(QString::fromUtf8("fillShade_"));

        _2->addWidget(fillShade_);

        spacerItem4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _2->addItem(spacerItem4);


        verticalLayout->addLayout(_2);

        _10 = new QHBoxLayout();
        _10->setObjectName(QString::fromUtf8("_10"));
        StrokeIcon = new QLabel(smColorGroup);
        StrokeIcon->setObjectName(QString::fromUtf8("StrokeIcon"));
        StrokeIcon->setMinimumSize(QSize(20, 0));
        StrokeIcon->setMaximumSize(QSize(22, 22));

        _10->addWidget(StrokeIcon);

        strokeColor_ = new SMColorCombo(smColorGroup);
        strokeColor_->setObjectName(QString::fromUtf8("strokeColor_"));

        _10->addWidget(strokeColor_);

        strokeShadeLabel = new QLabel(smColorGroup);
        strokeShadeLabel->setObjectName(QString::fromUtf8("strokeShadeLabel"));
        strokeShadeLabel->setMinimumSize(QSize(20, 0));
        strokeShadeLabel->setMaximumSize(QSize(22, 22));

        _10->addWidget(strokeShadeLabel);

        strokeShade_ = new SMShadeButton(smColorGroup);
        strokeShade_->setObjectName(QString::fromUtf8("strokeShade_"));

        _10->addWidget(strokeShade_);

        spacerItem5 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _10->addItem(spacerItem5);


        verticalLayout->addLayout(_10);

        _11 = new QHBoxLayout();
        _11->setObjectName(QString::fromUtf8("_11"));
        backIcon = new QLabel(smColorGroup);
        backIcon->setObjectName(QString::fromUtf8("backIcon"));
        backIcon->setMinimumSize(QSize(20, 0));
        backIcon->setMaximumSize(QSize(22, 22));

        _11->addWidget(backIcon);

        backColor_ = new SMColorCombo(smColorGroup);
        backColor_->setObjectName(QString::fromUtf8("backColor_"));

        _11->addWidget(backColor_);

        backShadeLabel = new QLabel(smColorGroup);
        backShadeLabel->setObjectName(QString::fromUtf8("backShadeLabel"));
        backShadeLabel->setMinimumSize(QSize(20, 0));
        backShadeLabel->setMaximumSize(QSize(22, 22));

        _11->addWidget(backShadeLabel);

        backShade_ = new SMShadeButton(smColorGroup);
        backShade_->setObjectName(QString::fromUtf8("backShade_"));

        _11->addWidget(backShade_);

        spacerItem6 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _11->addItem(spacerItem6);


        verticalLayout->addLayout(_11);


        gridLayout->addWidget(smColorGroup, 0, 0, 1, 1);

        hyphenGroup = new QGroupBox(tabProperties);
        hyphenGroup->setObjectName(QString::fromUtf8("hyphenGroup"));
        verticalLayout_4 = new QVBoxLayout(hyphenGroup);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        label_4 = new QLabel(hyphenGroup);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        horizontalLayout_3->addWidget(label_4);

        hyphenCharLineEdit = new SMLineEdit(hyphenGroup);
        hyphenCharLineEdit->setObjectName(QString::fromUtf8("hyphenCharLineEdit"));
        hyphenCharLineEdit->setMaximumSize(QSize(60, 16777215));

        horizontalLayout_3->addWidget(hyphenCharLineEdit);

        horizontalSpacer_2 = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_2);


        verticalLayout_4->addLayout(horizontalLayout_3);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        smallestWordLabel = new QLabel(hyphenGroup);
        smallestWordLabel->setObjectName(QString::fromUtf8("smallestWordLabel"));

        horizontalLayout->addWidget(smallestWordLabel);

        smallestWordSpinBox = new SMSpinBox(hyphenGroup);
        smallestWordSpinBox->setObjectName(QString::fromUtf8("smallestWordSpinBox"));
        smallestWordSpinBox->setMinimum(3);

        horizontalLayout->addWidget(smallestWordSpinBox);

        horizontalSpacer = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_4->addLayout(horizontalLayout);

        spacerItem7 = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::MinimumExpanding);

        verticalLayout_4->addItem(spacerItem7);


        gridLayout->addWidget(hyphenGroup, 0, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout);


        verticalLayout_6->addLayout(verticalLayout_2);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_6->addItem(verticalSpacer);

        tabwidget->addTab(tabProperties, QString());
        tabFontFeatures = new QWidget();
        tabFontFeatures->setObjectName(QString::fromUtf8("tabFontFeatures"));
        sizePolicy1.setHeightForWidth(tabFontFeatures->sizePolicy().hasHeightForWidth());
        tabFontFeatures->setSizePolicy(sizePolicy1);
        verticalLayout_5 = new QVBoxLayout(tabFontFeatures);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        fontfeaturesSetting = new SMFontFeatures(tabFontFeatures);
        fontfeaturesSetting->setObjectName(QString::fromUtf8("fontfeaturesSetting"));

        verticalLayout_5->addWidget(fontfeaturesSetting);

        tabwidget->addTab(tabFontFeatures, QString());

        verticalLayout_3->addWidget(tabwidget);

#if QT_CONFIG(shortcut)
        parentLabel->setBuddy(parentCombo);
        fontSizeLabel_->setBuddy(fontSize_);
        trackingLabel_->setBuddy(tracking_);
        widthSpaceLabel->setBuddy(widthSpaceSpin);
        hscaleLabel_->setBuddy(fontHScale_);
        vscaleLabel_->setBuddy(fontVScale_);
        baselineOffsetLabel_->setBuddy(baselineOffset_);
        languageLabel_->setBuddy(language_);
        FillIcon->setBuddy(fillColor_);
        fillShadeLabel->setBuddy(fillShade_);
        StrokeIcon->setBuddy(strokeColor_);
        strokeShadeLabel->setBuddy(strokeShade_);
        backIcon->setBuddy(fillColor_);
        backShadeLabel->setBuddy(fillShade_);
#endif // QT_CONFIG(shortcut)

        retranslateUi(SMCStyleWidget);

        tabwidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(SMCStyleWidget);
    } // setupUi

    void retranslateUi(QWidget *SMCStyleWidget)
    {
        parentLabel->setText(QCoreApplication::translate("SMCStyleWidget", "Based On:", nullptr));
#if QT_CONFIG(tooltip)
        parentCombo->setToolTip(QCoreApplication::translate("SMCStyleWidget", "Parent Style", nullptr));
#endif // QT_CONFIG(tooltip)
        basicGroup->setTitle(QCoreApplication::translate("SMCStyleWidget", "Basic Formatting", nullptr));
        fontSizeLabel_->setText(QString());
#if QT_CONFIG(tooltip)
        fontSize_->setToolTip(QCoreApplication::translate("SMCStyleWidget", "Font Size", nullptr));
#endif // QT_CONFIG(tooltip)
        trackingLabel_->setText(QString());
#if QT_CONFIG(tooltip)
        tracking_->setToolTip(QCoreApplication::translate("SMCStyleWidget", "Tracking", nullptr));
#endif // QT_CONFIG(tooltip)
        tracking_->setSuffix(QCoreApplication::translate("SMCStyleWidget", " %", nullptr));
#if QT_CONFIG(tooltip)
        widthSpaceSpin->setToolTip(QCoreApplication::translate("SMCStyleWidget", "Default width for space", nullptr));
#endif // QT_CONFIG(tooltip)
        widthSpaceSpin->setSuffix(QCoreApplication::translate("SMCStyleWidget", " %", nullptr));
        advGroup->setTitle(QCoreApplication::translate("SMCStyleWidget", "Advanced Formatting", nullptr));
        hscaleLabel_->setText(QString());
#if QT_CONFIG(tooltip)
        fontHScale_->setToolTip(QCoreApplication::translate("SMCStyleWidget", "Horizontal Scaling", nullptr));
#endif // QT_CONFIG(tooltip)
        fontHScale_->setSuffix(QCoreApplication::translate("SMCStyleWidget", " %", nullptr));
        vscaleLabel_->setText(QString());
#if QT_CONFIG(tooltip)
        fontVScale_->setToolTip(QCoreApplication::translate("SMCStyleWidget", "Vertical Scaling", nullptr));
#endif // QT_CONFIG(tooltip)
        fontVScale_->setSuffix(QCoreApplication::translate("SMCStyleWidget", " %", nullptr));
        baselineOffsetLabel_->setText(QString());
#if QT_CONFIG(tooltip)
        baselineOffset_->setToolTip(QCoreApplication::translate("SMCStyleWidget", "Baseline Offset", nullptr));
#endif // QT_CONFIG(tooltip)
        baselineOffset_->setSuffix(QCoreApplication::translate("SMCStyleWidget", " %", nullptr));
        languageLabel_->setText(QCoreApplication::translate("SMCStyleWidget", "Language:", nullptr));
#if QT_CONFIG(tooltip)
        language_->setToolTip(QCoreApplication::translate("SMCStyleWidget", "Language", nullptr));
#endif // QT_CONFIG(tooltip)
        smColorGroup->setTitle(QCoreApplication::translate("SMCStyleWidget", "Colors", nullptr));
        FillIcon->setText(QString());
#if QT_CONFIG(tooltip)
        fillColor_->setToolTip(QCoreApplication::translate("SMCStyleWidget", "Fill Color", nullptr));
#endif // QT_CONFIG(tooltip)
        fillShadeLabel->setText(QString());
#if QT_CONFIG(tooltip)
        fillShade_->setToolTip(QCoreApplication::translate("SMCStyleWidget", "Fill Shade", nullptr));
#endif // QT_CONFIG(tooltip)
        fillShade_->setText(QString());
        StrokeIcon->setText(QString());
#if QT_CONFIG(tooltip)
        strokeColor_->setToolTip(QCoreApplication::translate("SMCStyleWidget", "Stroke Color", nullptr));
#endif // QT_CONFIG(tooltip)
        strokeShadeLabel->setText(QString());
#if QT_CONFIG(tooltip)
        strokeShade_->setToolTip(QCoreApplication::translate("SMCStyleWidget", "Stroke Shade", nullptr));
#endif // QT_CONFIG(tooltip)
        strokeShade_->setText(QString());
        backIcon->setText(QString());
#if QT_CONFIG(tooltip)
        backColor_->setToolTip(QCoreApplication::translate("SMCStyleWidget", "Background Color", nullptr));
#endif // QT_CONFIG(tooltip)
        backShadeLabel->setText(QString());
#if QT_CONFIG(tooltip)
        backShade_->setToolTip(QCoreApplication::translate("SMCStyleWidget", "Background Shade", nullptr));
#endif // QT_CONFIG(tooltip)
        backShade_->setText(QString());
        hyphenGroup->setTitle(QCoreApplication::translate("SMCStyleWidget", "Hyphenation", nullptr));
        label_4->setText(QCoreApplication::translate("SMCStyleWidget", "Hyphenation Character:", nullptr));
        smallestWordLabel->setText(QCoreApplication::translate("SMCStyleWidget", "Smallest Word:", nullptr));
        smallestWordSpinBox->setSuffix(QCoreApplication::translate("SMCStyleWidget", " characters", nullptr));
        tabwidget->setTabText(tabwidget->indexOf(tabProperties), QCoreApplication::translate("SMCStyleWidget", "Properties", nullptr));
        tabwidget->setTabText(tabwidget->indexOf(tabFontFeatures), QCoreApplication::translate("SMCStyleWidget", "Font Features", nullptr));
        (void)SMCStyleWidget;
    } // retranslateUi

};

namespace Ui {
    class SMCStyleWidget: public Ui_SMCStyleWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SMCSTYLEWIDGET_H
