/********************************************************************************
** Form generated from reading UI file 'swdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SWDIALOG_H
#define UI_SWDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_SWDialog
{
public:
    QGridLayout *gridLayout_3;
    QGroupBox *buttonGroup;
    QGridLayout *gridLayout;
    QRadioButton *frameRadio;
    QRadioButton *pageRadio;
    QRadioButton *allRadio;
    QGroupBox *languageGroup;
    QGridLayout *gridLayout_2;
    QCheckBox *styleCheckBox;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QComboBox *languageComboBox;
    QSpacerItem *verticalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *SWDialog)
    {
        if (SWDialog->objectName().isEmpty())
            SWDialog->setObjectName(QString::fromUtf8("SWDialog"));
        SWDialog->resize(400, 297);
        gridLayout_3 = new QGridLayout(SWDialog);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        gridLayout_3->setContentsMargins(9, 9, 9, 9);
        buttonGroup = new QGroupBox(SWDialog);
        buttonGroup->setObjectName(QString::fromUtf8("buttonGroup"));
        gridLayout = new QGridLayout(buttonGroup);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        frameRadio = new QRadioButton(buttonGroup);
        frameRadio->setObjectName(QString::fromUtf8("frameRadio"));

        gridLayout->addWidget(frameRadio, 0, 0, 1, 1);

        pageRadio = new QRadioButton(buttonGroup);
        pageRadio->setObjectName(QString::fromUtf8("pageRadio"));

        gridLayout->addWidget(pageRadio, 1, 0, 1, 1);

        allRadio = new QRadioButton(buttonGroup);
        allRadio->setObjectName(QString::fromUtf8("allRadio"));

        gridLayout->addWidget(allRadio, 2, 0, 1, 1);


        gridLayout_3->addWidget(buttonGroup, 0, 0, 1, 1);

        languageGroup = new QGroupBox(SWDialog);
        languageGroup->setObjectName(QString::fromUtf8("languageGroup"));
        gridLayout_2 = new QGridLayout(languageGroup);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        styleCheckBox = new QCheckBox(languageGroup);
        styleCheckBox->setObjectName(QString::fromUtf8("styleCheckBox"));

        gridLayout_2->addWidget(styleCheckBox, 0, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label = new QLabel(languageGroup);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout->addWidget(label);

        languageComboBox = new QComboBox(languageGroup);
        languageComboBox->setObjectName(QString::fromUtf8("languageComboBox"));

        horizontalLayout->addWidget(languageComboBox);


        gridLayout_2->addLayout(horizontalLayout, 1, 0, 1, 1);


        gridLayout_3->addWidget(languageGroup, 1, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 1, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_3->addItem(verticalSpacer, 2, 0, 1, 1);

        buttonBox = new QDialogButtonBox(SWDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout_3->addWidget(buttonBox, 3, 0, 1, 1);

#if QT_CONFIG(shortcut)
        label->setBuddy(languageComboBox);
#endif // QT_CONFIG(shortcut)

        retranslateUi(SWDialog);
        QObject::connect(buttonBox, SIGNAL(accepted()), SWDialog, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), SWDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(SWDialog);
    } // setupUi

    void retranslateUi(QDialog *SWDialog)
    {
        SWDialog->setWindowTitle(QCoreApplication::translate("SWDialog", "Short Words", nullptr));
        buttonGroup->setTitle(QCoreApplication::translate("SWDialog", "Apply Unbreakable Space To:", nullptr));
        frameRadio->setText(QCoreApplication::translate("SWDialog", "&Selected Frames", nullptr));
        pageRadio->setText(QCoreApplication::translate("SWDialog", "Active &Page", nullptr));
        allRadio->setText(QCoreApplication::translate("SWDialog", "&All Items", nullptr));
        languageGroup->setTitle(QCoreApplication::translate("SWDialog", "Language Settings", nullptr));
        styleCheckBox->setText(QCoreApplication::translate("SWDialog", "&Use Language from Style Definition", nullptr));
        label->setText(QCoreApplication::translate("SWDialog", "&Languages:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class SWDialog: public Ui_SWDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SWDIALOG_H
