/****************************************************************************
** Meta object code from reading C++ file 'propertiespalette_shape.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/propertiespalette_shape.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertiespalette_shape.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PropertiesPalette_Shape_t {
    QByteArrayData data[26];
    char stringdata0[335];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PropertiesPalette_Shape_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PropertiesPalette_Shape_t qt_meta_stringdata_PropertiesPalette_Shape = {
    {
QT_MOC_LITERAL(0, 0, 23), // "PropertiesPalette_Shape"
QT_MOC_LITERAL(1, 24, 13), // "setMainWindow"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 18), // "ScribusMainWindow*"
QT_MOC_LITERAL(4, 58, 2), // "mw"
QT_MOC_LITERAL(5, 61, 6), // "setDoc"
QT_MOC_LITERAL(6, 68, 11), // "ScribusDoc*"
QT_MOC_LITERAL(7, 80, 1), // "d"
QT_MOC_LITERAL(8, 82, 14), // "setCurrentItem"
QT_MOC_LITERAL(9, 97, 9), // "PageItem*"
QT_MOC_LITERAL(10, 107, 4), // "item"
QT_MOC_LITERAL(11, 112, 8), // "unsetDoc"
QT_MOC_LITERAL(12, 121, 9), // "unsetItem"
QT_MOC_LITERAL(13, 131, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(14, 154, 19), // "handleUpdateRequest"
QT_MOC_LITERAL(15, 174, 13), // "iconSetChange"
QT_MOC_LITERAL(16, 188, 14), // "languageChange"
QT_MOC_LITERAL(17, 203, 10), // "unitChange"
QT_MOC_LITERAL(18, 214, 12), // "localeChange"
QT_MOC_LITERAL(19, 227, 14), // "handleTextFlow"
QT_MOC_LITERAL(20, 242, 15), // "handleShapeEdit"
QT_MOC_LITERAL(21, 258, 20), // "handleShapeEditEnded"
QT_MOC_LITERAL(22, 279, 18), // "handleCornerRadius"
QT_MOC_LITERAL(23, 298, 14), // "handleFillRule"
QT_MOC_LITERAL(24, 313, 14), // "handleNewShape"
QT_MOC_LITERAL(25, 328, 6) // "qreal*"

    },
    "PropertiesPalette_Shape\0setMainWindow\0"
    "\0ScribusMainWindow*\0mw\0setDoc\0ScribusDoc*\0"
    "d\0setCurrentItem\0PageItem*\0item\0"
    "unsetDoc\0unsetItem\0handleSelectionChanged\0"
    "handleUpdateRequest\0iconSetChange\0"
    "languageChange\0unitChange\0localeChange\0"
    "handleTextFlow\0handleShapeEdit\0"
    "handleShapeEditEnded\0handleCornerRadius\0"
    "handleFillRule\0handleNewShape\0qreal*"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PropertiesPalette_Shape[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      17,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   99,    2, 0x0a /* Public */,
       5,    1,  102,    2, 0x0a /* Public */,
       8,    1,  105,    2, 0x0a /* Public */,
      11,    0,  108,    2, 0x0a /* Public */,
      12,    0,  109,    2, 0x0a /* Public */,
      13,    0,  110,    2, 0x0a /* Public */,
      14,    1,  111,    2, 0x0a /* Public */,
      15,    0,  114,    2, 0x0a /* Public */,
      16,    0,  115,    2, 0x0a /* Public */,
      17,    0,  116,    2, 0x0a /* Public */,
      18,    0,  117,    2, 0x0a /* Public */,
      19,    0,  118,    2, 0x08 /* Private */,
      20,    0,  119,    2, 0x08 /* Private */,
      21,    0,  120,    2, 0x08 /* Private */,
      22,    0,  121,    2, 0x08 /* Private */,
      23,    0,  122,    2, 0x08 /* Private */,
      24,    3,  123,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, 0x80000000 | 25,    2,    2,    2,

       0        // eod
};

void PropertiesPalette_Shape::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertiesPalette_Shape *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setMainWindow((*reinterpret_cast< ScribusMainWindow*(*)>(_a[1]))); break;
        case 1: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 2: _t->setCurrentItem((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 3: _t->unsetDoc(); break;
        case 4: _t->unsetItem(); break;
        case 5: _t->handleSelectionChanged(); break;
        case 6: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->iconSetChange(); break;
        case 8: _t->languageChange(); break;
        case 9: _t->unitChange(); break;
        case 10: _t->localeChange(); break;
        case 11: _t->handleTextFlow(); break;
        case 12: _t->handleShapeEdit(); break;
        case 13: _t->handleShapeEditEnded(); break;
        case 14: _t->handleCornerRadius(); break;
        case 15: _t->handleFillRule(); break;
        case 16: _t->handleNewShape((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< qreal*(*)>(_a[3]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PropertiesPalette_Shape::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_PropertiesPalette_Shape.data,
    qt_meta_data_PropertiesPalette_Shape,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PropertiesPalette_Shape::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PropertiesPalette_Shape::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PropertiesPalette_Shape.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PropertiesPalette_ShapeBase"))
        return static_cast< Ui::PropertiesPalette_ShapeBase*>(this);
    return QWidget::qt_metacast(_clname);
}

int PropertiesPalette_Shape::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 17)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 17;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
