/********************************************************************************
** Form generated from reading UI file 'stylemanager.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_STYLEMANAGER_H
#define UI_STYLEMANAGER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "ui/scrpalettebase.h"
#include "ui/styleview.h"

QT_BEGIN_NAMESPACE

class Ui_StyleManager
{
public:
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout;
    QVBoxLayout *verticalLayout;
    StyleView *styleView;
    QFrame *leftFrame;
    QGridLayout *gridLayout;
    QPushButton *newButton;
    QPushButton *cloneButton;
    QPushButton *importButton;
    QPushButton *deleteButton;
    QFrame *rightFrame;
    QVBoxLayout *vboxLayout;
    QFrame *editFrame;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *hboxLayout;
    QLabel *nameLabel;
    QLineEdit *nameEdit;
    QLabel *uniqueLabel;
    QFrame *mainFrame;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *horizontalSpacer;
    QHBoxLayout *hboxLayout1;
    QPushButton *okButton;
    QPushButton *applyButton;
    QPushButton *resetButton;

    void setupUi(QDialog *StyleManager)
    {
        if (StyleManager->objectName().isEmpty())
            StyleManager->setObjectName(QString::fromUtf8("StyleManager"));
        StyleManager->setEnabled(true);
        StyleManager->resize(772, 422);
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(StyleManager->sizePolicy().hasHeightForWidth());
        StyleManager->setSizePolicy(sizePolicy);
        verticalLayout_2 = new QVBoxLayout(StyleManager);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(9, 9, 9, 9);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(9);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        verticalLayout = new QVBoxLayout();
        verticalLayout->setSpacing(9);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        styleView = new StyleView(StyleManager);
        styleView->setObjectName(QString::fromUtf8("styleView"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(styleView->sizePolicy().hasHeightForWidth());
        styleView->setSizePolicy(sizePolicy1);
        styleView->setContextMenuPolicy(Qt::CustomContextMenu);
        styleView->setAlternatingRowColors(true);
        styleView->setAllColumnsShowFocus(true);

        verticalLayout->addWidget(styleView);

        leftFrame = new QFrame(StyleManager);
        leftFrame->setObjectName(QString::fromUtf8("leftFrame"));
        QSizePolicy sizePolicy2(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(leftFrame->sizePolicy().hasHeightForWidth());
        leftFrame->setSizePolicy(sizePolicy2);
        leftFrame->setMinimumSize(QSize(190, 0));
        leftFrame->setMaximumSize(QSize(32767, 32767));
        leftFrame->setFrameShape(QFrame::NoFrame);
        leftFrame->setFrameShadow(QFrame::Plain);
        leftFrame->setLineWidth(0);
        gridLayout = new QGridLayout(leftFrame);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(0, 0, 0, 0);
        newButton = new QPushButton(leftFrame);
        newButton->setObjectName(QString::fromUtf8("newButton"));
        QSizePolicy sizePolicy3(QSizePolicy::Preferred, QSizePolicy::Minimum);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(newButton->sizePolicy().hasHeightForWidth());
        newButton->setSizePolicy(sizePolicy3);
        newButton->setMinimumSize(QSize(0, 0));
        newButton->setMaximumSize(QSize(1000, 32767));
        newButton->setAutoDefault(false);

        gridLayout->addWidget(newButton, 0, 0, 1, 1);

        cloneButton = new QPushButton(leftFrame);
        cloneButton->setObjectName(QString::fromUtf8("cloneButton"));
        sizePolicy3.setHeightForWidth(cloneButton->sizePolicy().hasHeightForWidth());
        cloneButton->setSizePolicy(sizePolicy3);
        cloneButton->setMinimumSize(QSize(0, 0));
        cloneButton->setMaximumSize(QSize(1000, 32767));
        cloneButton->setAutoDefault(false);

        gridLayout->addWidget(cloneButton, 0, 1, 1, 1);

        importButton = new QPushButton(leftFrame);
        importButton->setObjectName(QString::fromUtf8("importButton"));
        sizePolicy3.setHeightForWidth(importButton->sizePolicy().hasHeightForWidth());
        importButton->setSizePolicy(sizePolicy3);
        importButton->setAutoDefault(false);

        gridLayout->addWidget(importButton, 1, 0, 1, 1);

        deleteButton = new QPushButton(leftFrame);
        deleteButton->setObjectName(QString::fromUtf8("deleteButton"));
        sizePolicy3.setHeightForWidth(deleteButton->sizePolicy().hasHeightForWidth());
        deleteButton->setSizePolicy(sizePolicy3);
        deleteButton->setAutoDefault(false);

        gridLayout->addWidget(deleteButton, 1, 1, 1, 1);


        verticalLayout->addWidget(leftFrame);


        horizontalLayout->addLayout(verticalLayout);

        rightFrame = new QFrame(StyleManager);
        rightFrame->setObjectName(QString::fromUtf8("rightFrame"));
        rightFrame->setMinimumSize(QSize(472, 0));
        rightFrame->setFrameShape(QFrame::NoFrame);
        rightFrame->setFrameShadow(QFrame::Plain);
        rightFrame->setLineWidth(0);
        vboxLayout = new QVBoxLayout(rightFrame);
        vboxLayout->setSpacing(0);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        editFrame = new QFrame(rightFrame);
        editFrame->setObjectName(QString::fromUtf8("editFrame"));
        QSizePolicy sizePolicy4(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(editFrame->sizePolicy().hasHeightForWidth());
        editFrame->setSizePolicy(sizePolicy4);
        editFrame->setFrameShape(QFrame::NoFrame);
        editFrame->setFrameShadow(QFrame::Plain);
        editFrame->setLineWidth(0);
        vboxLayout1 = new QVBoxLayout(editFrame);
        vboxLayout1->setSpacing(9);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        vboxLayout1->setContentsMargins(0, 0, 0, 0);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        nameLabel = new QLabel(editFrame);
        nameLabel->setObjectName(QString::fromUtf8("nameLabel"));
        nameLabel->setWordWrap(false);

        hboxLayout->addWidget(nameLabel);

        nameEdit = new QLineEdit(editFrame);
        nameEdit->setObjectName(QString::fromUtf8("nameEdit"));
        nameEdit->setEnabled(false);

        hboxLayout->addWidget(nameEdit);


        vboxLayout1->addLayout(hboxLayout);

        uniqueLabel = new QLabel(editFrame);
        uniqueLabel->setObjectName(QString::fromUtf8("uniqueLabel"));
        uniqueLabel->setMaximumSize(QSize(32767, 30));
        QFont font;
        font.setBold(true);
        font.setWeight(75);
        uniqueLabel->setFont(font);
        uniqueLabel->setAlignment(Qt::AlignCenter);
        uniqueLabel->setWordWrap(false);

        vboxLayout1->addWidget(uniqueLabel);

        mainFrame = new QFrame(editFrame);
        mainFrame->setObjectName(QString::fromUtf8("mainFrame"));
        QSizePolicy sizePolicy5(QSizePolicy::Expanding, QSizePolicy::MinimumExpanding);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(mainFrame->sizePolicy().hasHeightForWidth());
        mainFrame->setSizePolicy(sizePolicy5);

        vboxLayout1->addWidget(mainFrame);


        vboxLayout->addWidget(editFrame);


        horizontalLayout->addWidget(rightFrame);


        verticalLayout_2->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        hboxLayout1->setContentsMargins(0, 0, 0, 0);
        okButton = new QPushButton(StyleManager);
        okButton->setObjectName(QString::fromUtf8("okButton"));
        QSizePolicy sizePolicy6(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(okButton->sizePolicy().hasHeightForWidth());
        okButton->setSizePolicy(sizePolicy6);
        okButton->setMinimumSize(QSize(85, 24));
        okButton->setAutoDefault(false);

        hboxLayout1->addWidget(okButton);

        applyButton = new QPushButton(StyleManager);
        applyButton->setObjectName(QString::fromUtf8("applyButton"));
        sizePolicy6.setHeightForWidth(applyButton->sizePolicy().hasHeightForWidth());
        applyButton->setSizePolicy(sizePolicy6);
        applyButton->setMinimumSize(QSize(85, 24));
        applyButton->setAutoDefault(false);

        hboxLayout1->addWidget(applyButton);

        resetButton = new QPushButton(StyleManager);
        resetButton->setObjectName(QString::fromUtf8("resetButton"));
        sizePolicy6.setHeightForWidth(resetButton->sizePolicy().hasHeightForWidth());
        resetButton->setSizePolicy(sizePolicy6);
        resetButton->setMinimumSize(QSize(85, 24));
        resetButton->setAutoDefault(false);

        hboxLayout1->addWidget(resetButton);


        horizontalLayout_2->addLayout(hboxLayout1);


        verticalLayout_2->addLayout(horizontalLayout_2);


        retranslateUi(StyleManager);

        QMetaObject::connectSlotsByName(StyleManager);
    } // setupUi

    void retranslateUi(QDialog *StyleManager)
    {
        StyleManager->setWindowTitle(QCoreApplication::translate("StyleManager", "Style Manager", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = styleView->headerItem();
        ___qtreewidgetitem->setText(1, QCoreApplication::translate("StyleManager", "Shortcut", nullptr));
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("StyleManager", "Name", nullptr));
        newButton->setText(QCoreApplication::translate("StyleManager", "&New", nullptr));
#if QT_CONFIG(shortcut)
        newButton->setShortcut(QCoreApplication::translate("StyleManager", "Alt+N", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        cloneButton->setToolTip(QCoreApplication::translate("StyleManager", "Clone copies the style to make similar styles easily", nullptr));
#endif // QT_CONFIG(tooltip)
        cloneButton->setText(QCoreApplication::translate("StyleManager", "&Clone", nullptr));
#if QT_CONFIG(shortcut)
        cloneButton->setShortcut(QCoreApplication::translate("StyleManager", "Alt+C", nullptr));
#endif // QT_CONFIG(shortcut)
        importButton->setText(QCoreApplication::translate("StyleManager", "&Import", nullptr));
#if QT_CONFIG(shortcut)
        importButton->setShortcut(QCoreApplication::translate("StyleManager", "Alt+I", nullptr));
#endif // QT_CONFIG(shortcut)
        deleteButton->setText(QCoreApplication::translate("StyleManager", "&Delete", nullptr));
#if QT_CONFIG(shortcut)
        deleteButton->setShortcut(QCoreApplication::translate("StyleManager", "Alt+D", nullptr));
#endif // QT_CONFIG(shortcut)
        nameLabel->setText(QCoreApplication::translate("StyleManager", "Name:", nullptr));
        uniqueLabel->setText(QCoreApplication::translate("StyleManager", "Please select a unique name for the style", nullptr));
        okButton->setText(QCoreApplication::translate("StyleManager", "<< &Done", nullptr));
#if QT_CONFIG(shortcut)
        okButton->setShortcut(QCoreApplication::translate("StyleManager", "Alt+D", nullptr));
#endif // QT_CONFIG(shortcut)
        applyButton->setText(QCoreApplication::translate("StyleManager", "&Apply", nullptr));
#if QT_CONFIG(shortcut)
        applyButton->setShortcut(QCoreApplication::translate("StyleManager", "Alt+A", nullptr));
#endif // QT_CONFIG(shortcut)
        resetButton->setText(QCoreApplication::translate("StyleManager", "&Reset", nullptr));
#if QT_CONFIG(shortcut)
        resetButton->setShortcut(QCoreApplication::translate("StyleManager", "Alt+R", nullptr));
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class StyleManager: public Ui_StyleManager {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_STYLEMANAGER_H
