/********************************************************************************
** Form generated from reading UI file 'smfontfeaturesbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SMFONTFEATURESBASE_H
#define UI_SMFONTFEATURESBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_smFontFeaturesBase
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *statusLabel;
    QGroupBox *ligaturesGroupBox;
    QFormLayout *formLayout;
    QCheckBox *CommonCheck;
    QCheckBox *DiscretionaryCheck;
    QCheckBox *ContextualCheck;
    QCheckBox *HistoricalCheck;
    QGroupBox *capitalsGroupBox;
    QVBoxLayout *verticalLayout_3;
    QGridLayout *gridLayout;
    QComboBox *capitalsComboBox;
    QLabel *capitalsLabel;
    QSpacerItem *capitalsSpacer;
    QLabel *positionLabel;
    QComboBox *positionComboBox;
    QSpacerItem *positionSpacer;
    QGroupBox *numeralsGroupBox;
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout_2;
    QComboBox *numeralComboBox;
    QSpacerItem *numeralSpacer;
    QLabel *styleLabel;
    QLabel *widthLabel;
    QSpacerItem *widthSpacer;
    QComboBox *widthComboBox;
    QLabel *fractionLabel;
    QComboBox *fractionComboBox;
    QSpacerItem *fractionSpacer;
    QCheckBox *SlashedZeroCheck;
    QHBoxLayout *horizontalLayout_6;
    QLabel *stylisticSetsLabel;
    QPushButton *stylisticSetsButton;
    QSpacerItem *horizontalSpacer_6;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *smFontFeaturesBase)
    {
        if (smFontFeaturesBase->objectName().isEmpty())
            smFontFeaturesBase->setObjectName(QString::fromUtf8("smFontFeaturesBase"));
        smFontFeaturesBase->resize(415, 488);
        verticalLayout_2 = new QVBoxLayout(smFontFeaturesBase);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        statusLabel = new QLabel(smFontFeaturesBase);
        statusLabel->setObjectName(QString::fromUtf8("statusLabel"));
        statusLabel->setEnabled(true);
        statusLabel->setTextFormat(Qt::RichText);
        statusLabel->setWordWrap(true);

        verticalLayout_2->addWidget(statusLabel);

        ligaturesGroupBox = new QGroupBox(smFontFeaturesBase);
        ligaturesGroupBox->setObjectName(QString::fromUtf8("ligaturesGroupBox"));
        formLayout = new QFormLayout(ligaturesGroupBox);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        CommonCheck = new QCheckBox(ligaturesGroupBox);
        CommonCheck->setObjectName(QString::fromUtf8("CommonCheck"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(CommonCheck->sizePolicy().hasHeightForWidth());
        CommonCheck->setSizePolicy(sizePolicy);
        CommonCheck->setChecked(false);

        formLayout->setWidget(0, QFormLayout::LabelRole, CommonCheck);

        DiscretionaryCheck = new QCheckBox(ligaturesGroupBox);
        DiscretionaryCheck->setObjectName(QString::fromUtf8("DiscretionaryCheck"));
        sizePolicy.setHeightForWidth(DiscretionaryCheck->sizePolicy().hasHeightForWidth());
        DiscretionaryCheck->setSizePolicy(sizePolicy);
        DiscretionaryCheck->setAutoExclusive(false);

        formLayout->setWidget(0, QFormLayout::FieldRole, DiscretionaryCheck);

        ContextualCheck = new QCheckBox(ligaturesGroupBox);
        ContextualCheck->setObjectName(QString::fromUtf8("ContextualCheck"));
        sizePolicy.setHeightForWidth(ContextualCheck->sizePolicy().hasHeightForWidth());
        ContextualCheck->setSizePolicy(sizePolicy);
        ContextualCheck->setChecked(false);
        ContextualCheck->setAutoRepeat(false);
        ContextualCheck->setAutoExclusive(false);
        ContextualCheck->setTristate(false);

        formLayout->setWidget(1, QFormLayout::LabelRole, ContextualCheck);

        HistoricalCheck = new QCheckBox(ligaturesGroupBox);
        HistoricalCheck->setObjectName(QString::fromUtf8("HistoricalCheck"));
        sizePolicy.setHeightForWidth(HistoricalCheck->sizePolicy().hasHeightForWidth());
        HistoricalCheck->setSizePolicy(sizePolicy);
        HistoricalCheck->setChecked(false);
        HistoricalCheck->setAutoExclusive(true);

        formLayout->setWidget(1, QFormLayout::FieldRole, HistoricalCheck);


        verticalLayout_2->addWidget(ligaturesGroupBox);

        capitalsGroupBox = new QGroupBox(smFontFeaturesBase);
        capitalsGroupBox->setObjectName(QString::fromUtf8("capitalsGroupBox"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Maximum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(capitalsGroupBox->sizePolicy().hasHeightForWidth());
        capitalsGroupBox->setSizePolicy(sizePolicy1);
        verticalLayout_3 = new QVBoxLayout(capitalsGroupBox);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        capitalsComboBox = new QComboBox(capitalsGroupBox);
        capitalsComboBox->addItem(QString());
        capitalsComboBox->addItem(QString());
        capitalsComboBox->addItem(QString());
        capitalsComboBox->addItem(QString());
        capitalsComboBox->addItem(QString());
        capitalsComboBox->addItem(QString());
        capitalsComboBox->addItem(QString());
        capitalsComboBox->setObjectName(QString::fromUtf8("capitalsComboBox"));

        gridLayout->addWidget(capitalsComboBox, 0, 1, 1, 1);

        capitalsLabel = new QLabel(capitalsGroupBox);
        capitalsLabel->setObjectName(QString::fromUtf8("capitalsLabel"));

        gridLayout->addWidget(capitalsLabel, 0, 0, 1, 1);

        capitalsSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(capitalsSpacer, 0, 2, 1, 1);

        positionLabel = new QLabel(capitalsGroupBox);
        positionLabel->setObjectName(QString::fromUtf8("positionLabel"));

        gridLayout->addWidget(positionLabel, 1, 0, 1, 1);

        positionComboBox = new QComboBox(capitalsGroupBox);
        positionComboBox->addItem(QString());
        positionComboBox->addItem(QString());
        positionComboBox->addItem(QString());
        positionComboBox->addItem(QString());
        positionComboBox->setObjectName(QString::fromUtf8("positionComboBox"));

        gridLayout->addWidget(positionComboBox, 1, 1, 1, 1);

        positionSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(positionSpacer, 1, 2, 1, 1);


        verticalLayout_3->addLayout(gridLayout);


        verticalLayout_2->addWidget(capitalsGroupBox);

        numeralsGroupBox = new QGroupBox(smFontFeaturesBase);
        numeralsGroupBox->setObjectName(QString::fromUtf8("numeralsGroupBox"));
        sizePolicy1.setHeightForWidth(numeralsGroupBox->sizePolicy().hasHeightForWidth());
        numeralsGroupBox->setSizePolicy(sizePolicy1);
        verticalLayout = new QVBoxLayout(numeralsGroupBox);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        numeralComboBox = new QComboBox(numeralsGroupBox);
        numeralComboBox->addItem(QString());
        numeralComboBox->addItem(QString());
        numeralComboBox->addItem(QString());
        numeralComboBox->setObjectName(QString::fromUtf8("numeralComboBox"));

        gridLayout_2->addWidget(numeralComboBox, 0, 1, 1, 1);

        numeralSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_2->addItem(numeralSpacer, 0, 2, 1, 1);

        styleLabel = new QLabel(numeralsGroupBox);
        styleLabel->setObjectName(QString::fromUtf8("styleLabel"));

        gridLayout_2->addWidget(styleLabel, 0, 0, 1, 1);

        widthLabel = new QLabel(numeralsGroupBox);
        widthLabel->setObjectName(QString::fromUtf8("widthLabel"));

        gridLayout_2->addWidget(widthLabel, 1, 0, 1, 1);

        widthSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_2->addItem(widthSpacer, 1, 2, 1, 1);

        widthComboBox = new QComboBox(numeralsGroupBox);
        widthComboBox->addItem(QString());
        widthComboBox->addItem(QString());
        widthComboBox->addItem(QString());
        widthComboBox->setObjectName(QString::fromUtf8("widthComboBox"));

        gridLayout_2->addWidget(widthComboBox, 1, 1, 1, 1);

        fractionLabel = new QLabel(numeralsGroupBox);
        fractionLabel->setObjectName(QString::fromUtf8("fractionLabel"));

        gridLayout_2->addWidget(fractionLabel, 2, 0, 1, 1);

        fractionComboBox = new QComboBox(numeralsGroupBox);
        fractionComboBox->addItem(QString());
        fractionComboBox->addItem(QString());
        fractionComboBox->addItem(QString());
        fractionComboBox->setObjectName(QString::fromUtf8("fractionComboBox"));

        gridLayout_2->addWidget(fractionComboBox, 2, 1, 1, 1);

        fractionSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_2->addItem(fractionSpacer, 2, 2, 1, 1);


        verticalLayout->addLayout(gridLayout_2);

        SlashedZeroCheck = new QCheckBox(numeralsGroupBox);
        SlashedZeroCheck->setObjectName(QString::fromUtf8("SlashedZeroCheck"));
        SlashedZeroCheck->setEnabled(true);

        verticalLayout->addWidget(SlashedZeroCheck);


        verticalLayout_2->addWidget(numeralsGroupBox);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        stylisticSetsLabel = new QLabel(smFontFeaturesBase);
        stylisticSetsLabel->setObjectName(QString::fromUtf8("stylisticSetsLabel"));

        horizontalLayout_6->addWidget(stylisticSetsLabel);

        stylisticSetsButton = new QPushButton(smFontFeaturesBase);
        stylisticSetsButton->setObjectName(QString::fromUtf8("stylisticSetsButton"));
        sizePolicy.setHeightForWidth(stylisticSetsButton->sizePolicy().hasHeightForWidth());
        stylisticSetsButton->setSizePolicy(sizePolicy);
        stylisticSetsButton->setMinimumSize(QSize(168, 0));
        stylisticSetsButton->setCheckable(false);

        horizontalLayout_6->addWidget(stylisticSetsButton);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_6);


        verticalLayout_2->addLayout(horizontalLayout_6);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        retranslateUi(smFontFeaturesBase);

        QMetaObject::connectSlotsByName(smFontFeaturesBase);
    } // setupUi

    void retranslateUi(QWidget *smFontFeaturesBase)
    {
        statusLabel->setText(QCoreApplication::translate("smFontFeaturesBase", "<html><head/><body><p><span style=\" font-weight:600; color:#008000;\">The current selected font does not have any font features.</span></p></body></html>", nullptr));
        ligaturesGroupBox->setTitle(QCoreApplication::translate("smFontFeaturesBase", "Ligatures", nullptr));
#if QT_CONFIG(tooltip)
        CommonCheck->setToolTip(QCoreApplication::translate("smFontFeaturesBase", "Common Ligatures (liga)", nullptr));
#endif // QT_CONFIG(tooltip)
        CommonCheck->setText(QCoreApplication::translate("smFontFeaturesBase", "Common", nullptr));
#if QT_CONFIG(tooltip)
        DiscretionaryCheck->setToolTip(QCoreApplication::translate("smFontFeaturesBase", "Discretionary Ligatures (dlig)", nullptr));
#endif // QT_CONFIG(tooltip)
        DiscretionaryCheck->setText(QCoreApplication::translate("smFontFeaturesBase", "Discretionary", nullptr));
#if QT_CONFIG(tooltip)
        ContextualCheck->setToolTip(QCoreApplication::translate("smFontFeaturesBase", "Contextual Ligatures (clig)", nullptr));
#endif // QT_CONFIG(tooltip)
        ContextualCheck->setText(QCoreApplication::translate("smFontFeaturesBase", "Contextual", nullptr));
#if QT_CONFIG(tooltip)
        HistoricalCheck->setToolTip(QCoreApplication::translate("smFontFeaturesBase", "Historical Ligatures (hlig)", nullptr));
#endif // QT_CONFIG(tooltip)
        HistoricalCheck->setText(QCoreApplication::translate("smFontFeaturesBase", "Historical", nullptr));
        capitalsGroupBox->setTitle(QCoreApplication::translate("smFontFeaturesBase", "Capitals && Position", nullptr));
        capitalsComboBox->setItemText(0, QCoreApplication::translate("smFontFeaturesBase", "Default Capitals", nullptr));
        capitalsComboBox->setItemText(1, QCoreApplication::translate("smFontFeaturesBase", "Small Capitals", nullptr));
        capitalsComboBox->setItemText(2, QCoreApplication::translate("smFontFeaturesBase", "Petite Capitals", nullptr));
        capitalsComboBox->setItemText(3, QCoreApplication::translate("smFontFeaturesBase", "Unicase", nullptr));
        capitalsComboBox->setItemText(4, QCoreApplication::translate("smFontFeaturesBase", "Titling", nullptr));
        capitalsComboBox->setItemText(5, QCoreApplication::translate("smFontFeaturesBase", "Petite Capitals from Capitals", nullptr));
        capitalsComboBox->setItemText(6, QCoreApplication::translate("smFontFeaturesBase", "Small Capitals from Capitals", nullptr));

        capitalsLabel->setText(QCoreApplication::translate("smFontFeaturesBase", "Capitals:", nullptr));
        positionLabel->setText(QCoreApplication::translate("smFontFeaturesBase", "Position:", nullptr));
        positionComboBox->setItemText(0, QCoreApplication::translate("smFontFeaturesBase", "Default Position", nullptr));
        positionComboBox->setItemText(1, QCoreApplication::translate("smFontFeaturesBase", "Subscript", nullptr));
        positionComboBox->setItemText(2, QCoreApplication::translate("smFontFeaturesBase", "Superscript", nullptr));
        positionComboBox->setItemText(3, QCoreApplication::translate("smFontFeaturesBase", "Ordinals", nullptr));

        numeralsGroupBox->setTitle(QCoreApplication::translate("smFontFeaturesBase", "Numerals", nullptr));
        numeralComboBox->setItemText(0, QCoreApplication::translate("smFontFeaturesBase", "Default Numerals", nullptr));
        numeralComboBox->setItemText(1, QCoreApplication::translate("smFontFeaturesBase", "Lining", nullptr));
        numeralComboBox->setItemText(2, QCoreApplication::translate("smFontFeaturesBase", "Old Style", nullptr));

        styleLabel->setText(QCoreApplication::translate("smFontFeaturesBase", "Style:", nullptr));
        widthLabel->setText(QCoreApplication::translate("smFontFeaturesBase", "Width:", nullptr));
        widthComboBox->setItemText(0, QCoreApplication::translate("smFontFeaturesBase", "Default Numeral Width", nullptr));
        widthComboBox->setItemText(1, QCoreApplication::translate("smFontFeaturesBase", "Proportional", nullptr));
        widthComboBox->setItemText(2, QCoreApplication::translate("smFontFeaturesBase", "Tabular", nullptr));

        fractionLabel->setText(QCoreApplication::translate("smFontFeaturesBase", "Fraction:", nullptr));
        fractionComboBox->setItemText(0, QCoreApplication::translate("smFontFeaturesBase", "No Fractions", nullptr));
        fractionComboBox->setItemText(1, QCoreApplication::translate("smFontFeaturesBase", "Diagonal Fractions", nullptr));
        fractionComboBox->setItemText(2, QCoreApplication::translate("smFontFeaturesBase", "Stacked Fractions", nullptr));

#if QT_CONFIG(tooltip)
        SlashedZeroCheck->setToolTip(QCoreApplication::translate("smFontFeaturesBase", "Slashed Zero (zero)", nullptr));
#endif // QT_CONFIG(tooltip)
        SlashedZeroCheck->setText(QCoreApplication::translate("smFontFeaturesBase", "Slashed Zero", nullptr));
        stylisticSetsLabel->setText(QCoreApplication::translate("smFontFeaturesBase", "Stylistic Sets:", nullptr));
        stylisticSetsButton->setText(QCoreApplication::translate("smFontFeaturesBase", "click to select", nullptr));
        (void)smFontFeaturesBase;
    } // retranslateUi

};

namespace Ui {
    class smFontFeaturesBase: public Ui_smFontFeaturesBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SMFONTFEATURESBASE_H
