/********************************************************************************
** Form generated from reading UI file 'helpbrowser.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HELPBROWSER_H
#define UI_HELPBROWSER_H

#include <QtCore/QVariant>
#include <QtWidgets/QAction>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QMainWindow>
#include <QtWidgets/QMenuBar>
#include <QtWidgets/QStatusBar>
#include <QtWidgets/QToolBar>
#include <QtWidgets/QWidget>
#include "ui/sctextbrowser.h"

QT_BEGIN_NAMESPACE

class Ui_HelpBrowser
{
public:
    QAction *action_Print;
    QAction *action_Exit;
    QAction *action_Find;
    QAction *actionFind_Next;
    QAction *actionFind_Previous;
    QAction *action_Add;
    QAction *action_Delete;
    QAction *actionD_elete_All;
    QWidget *centralwidget;
    QGridLayout *gridLayout;
    ScTextBrowser *textBrowser;
    QMenuBar *menubar;
    QStatusBar *statusbar;
    QToolBar *toolBar;

    void setupUi(QMainWindow *HelpBrowser)
    {
        if (HelpBrowser->objectName().isEmpty())
            HelpBrowser->setObjectName(QString::fromUtf8("HelpBrowser"));
        HelpBrowser->resize(611, 535);
        action_Print = new QAction(HelpBrowser);
        action_Print->setObjectName(QString::fromUtf8("action_Print"));
        action_Exit = new QAction(HelpBrowser);
        action_Exit->setObjectName(QString::fromUtf8("action_Exit"));
        action_Find = new QAction(HelpBrowser);
        action_Find->setObjectName(QString::fromUtf8("action_Find"));
        actionFind_Next = new QAction(HelpBrowser);
        actionFind_Next->setObjectName(QString::fromUtf8("actionFind_Next"));
        actionFind_Previous = new QAction(HelpBrowser);
        actionFind_Previous->setObjectName(QString::fromUtf8("actionFind_Previous"));
        action_Add = new QAction(HelpBrowser);
        action_Add->setObjectName(QString::fromUtf8("action_Add"));
        action_Delete = new QAction(HelpBrowser);
        action_Delete->setObjectName(QString::fromUtf8("action_Delete"));
        actionD_elete_All = new QAction(HelpBrowser);
        actionD_elete_All->setObjectName(QString::fromUtf8("actionD_elete_All"));
        centralwidget = new QWidget(HelpBrowser);
        centralwidget->setObjectName(QString::fromUtf8("centralwidget"));
        gridLayout = new QGridLayout(centralwidget);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        textBrowser = new ScTextBrowser(centralwidget);
        textBrowser->setObjectName(QString::fromUtf8("textBrowser"));

        gridLayout->addWidget(textBrowser, 0, 0, 1, 1);

        HelpBrowser->setCentralWidget(centralwidget);
        menubar = new QMenuBar(HelpBrowser);
        menubar->setObjectName(QString::fromUtf8("menubar"));
        menubar->setGeometry(QRect(0, 0, 611, 26));
        HelpBrowser->setMenuBar(menubar);
        statusbar = new QStatusBar(HelpBrowser);
        statusbar->setObjectName(QString::fromUtf8("statusbar"));
        HelpBrowser->setStatusBar(statusbar);
        toolBar = new QToolBar(HelpBrowser);
        toolBar->setObjectName(QString::fromUtf8("toolBar"));
        HelpBrowser->addToolBar(Qt::TopToolBarArea, toolBar);

        retranslateUi(HelpBrowser);

        QMetaObject::connectSlotsByName(HelpBrowser);
    } // setupUi

    void retranslateUi(QMainWindow *HelpBrowser)
    {
        HelpBrowser->setWindowTitle(QCoreApplication::translate("HelpBrowser", "Scribus Help", nullptr));
        action_Print->setText(QCoreApplication::translate("HelpBrowser", "&Print...", nullptr));
        action_Exit->setText(QCoreApplication::translate("HelpBrowser", "&Exit", nullptr));
        action_Find->setText(QCoreApplication::translate("HelpBrowser", "&Find...", nullptr));
        actionFind_Next->setText(QCoreApplication::translate("HelpBrowser", "Find &Next...", nullptr));
        actionFind_Previous->setText(QCoreApplication::translate("HelpBrowser", "Find &Previous...", nullptr));
        action_Add->setText(QCoreApplication::translate("HelpBrowser", "&Add", nullptr));
        action_Delete->setText(QCoreApplication::translate("HelpBrowser", "&Delete", nullptr));
        actionD_elete_All->setText(QCoreApplication::translate("HelpBrowser", "D&elete All", nullptr));
        toolBar->setWindowTitle(QCoreApplication::translate("HelpBrowser", "Scribus Help", nullptr));
    } // retranslateUi

};

namespace Ui {
    class HelpBrowser: public Ui_HelpBrowser {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_HELPBROWSER_H
