/********************************************************************************
** Form generated from reading UI file 'propertiespalette_textbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTIESPALETTE_TEXTBASE_H
#define UI_PROPERTIESPALETTE_TEXTBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/alignselect.h"
#include "ui/directionselect.h"
#include "ui/fontcombo.h"
#include "ui/scrspinbox.h"
#include "ui/sctreewidget.h"
#include "ui/stylecombos.h"

QT_BEGIN_NAMESPACE

class Ui_PropertiesPalette_TextBase
{
public:
    QVBoxLayout *verticalLayout;
    FontComboH *fonts;
    QHBoxLayout *horizontalLayout;
    QLabel *fontSizeLabel;
    ScrSpinBox *fontSize;
    QHBoxLayout *horizontalLayout_2;
    QLabel *lineSpacingLabel;
    QComboBox *lineSpacingModeCombo;
    ScrSpinBox *lineSpacing;
    QHBoxLayout *horizontalLayout_5;
    QLabel *textAlignmentLabel;
    AlignSelect *textAlignment;
    DirectionSelect *textDirection;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout_6;
    QLabel *langLabel;
    QComboBox *langCombo;
    QHBoxLayout *horizontalLayout_3;
    QLabel *paraStyleLabel;
    ParaStyleComboBox *paraStyleCombo;
    QToolButton *paraStyleClear;
    QHBoxLayout *horizontalLayout_4;
    QLabel *charStyleLabel;
    CharStyleComboBox *charStyleCombo;
    QToolButton *charStyleClear;
    ScTreeWidget *textTree;

    void setupUi(QWidget *PropertiesPalette_TextBase)
    {
        if (PropertiesPalette_TextBase->objectName().isEmpty())
            PropertiesPalette_TextBase->setObjectName(QString::fromUtf8("PropertiesPalette_TextBase"));
        PropertiesPalette_TextBase->resize(308, 390);
        verticalLayout = new QVBoxLayout(PropertiesPalette_TextBase);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        fonts = new FontComboH(PropertiesPalette_TextBase);
        fonts->setObjectName(QString::fromUtf8("fonts"));

        verticalLayout->addWidget(fonts);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(3);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        fontSizeLabel = new QLabel(PropertiesPalette_TextBase);
        fontSizeLabel->setObjectName(QString::fromUtf8("fontSizeLabel"));
        fontSizeLabel->setMinimumSize(QSize(20, 0));
        fontSizeLabel->setMaximumSize(QSize(22, 22));
        fontSizeLabel->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);

        horizontalLayout->addWidget(fontSizeLabel);

        fontSize = new ScrSpinBox(PropertiesPalette_TextBase);
        fontSize->setObjectName(QString::fromUtf8("fontSize"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(fontSize->sizePolicy().hasHeightForWidth());
        fontSize->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(fontSize);


        verticalLayout->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(3);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        lineSpacingLabel = new QLabel(PropertiesPalette_TextBase);
        lineSpacingLabel->setObjectName(QString::fromUtf8("lineSpacingLabel"));
        lineSpacingLabel->setMinimumSize(QSize(20, 0));
        lineSpacingLabel->setMaximumSize(QSize(22, 22));

        horizontalLayout_2->addWidget(lineSpacingLabel);

        lineSpacingModeCombo = new QComboBox(PropertiesPalette_TextBase);
        lineSpacingModeCombo->setObjectName(QString::fromUtf8("lineSpacingModeCombo"));
        sizePolicy.setHeightForWidth(lineSpacingModeCombo->sizePolicy().hasHeightForWidth());
        lineSpacingModeCombo->setSizePolicy(sizePolicy);

        horizontalLayout_2->addWidget(lineSpacingModeCombo);

        lineSpacing = new ScrSpinBox(PropertiesPalette_TextBase);
        lineSpacing->setObjectName(QString::fromUtf8("lineSpacing"));
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(lineSpacing->sizePolicy().hasHeightForWidth());
        lineSpacing->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(lineSpacing);


        verticalLayout->addLayout(horizontalLayout_2);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setSpacing(3);
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        textAlignmentLabel = new QLabel(PropertiesPalette_TextBase);
        textAlignmentLabel->setObjectName(QString::fromUtf8("textAlignmentLabel"));
        textAlignmentLabel->setMinimumSize(QSize(20, 0));
        textAlignmentLabel->setMaximumSize(QSize(22, 22));

        horizontalLayout_5->addWidget(textAlignmentLabel);

        textAlignment = new AlignSelect(PropertiesPalette_TextBase);
        textAlignment->setObjectName(QString::fromUtf8("textAlignment"));

        horizontalLayout_5->addWidget(textAlignment);

        textDirection = new DirectionSelect(PropertiesPalette_TextBase);
        textDirection->setObjectName(QString::fromUtf8("textDirection"));

        horizontalLayout_5->addWidget(textDirection, 0, Qt::AlignLeft);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout_5);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setSpacing(3);
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        langLabel = new QLabel(PropertiesPalette_TextBase);
        langLabel->setObjectName(QString::fromUtf8("langLabel"));
        langLabel->setMinimumSize(QSize(20, 0));
        langLabel->setMaximumSize(QSize(22, 22));

        horizontalLayout_6->addWidget(langLabel);

        langCombo = new QComboBox(PropertiesPalette_TextBase);
        langCombo->setObjectName(QString::fromUtf8("langCombo"));
        QSizePolicy sizePolicy2(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(langCombo->sizePolicy().hasHeightForWidth());
        langCombo->setSizePolicy(sizePolicy2);
        langCombo->setEditable(true);

        horizontalLayout_6->addWidget(langCombo);


        verticalLayout->addLayout(horizontalLayout_6);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(3);
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        paraStyleLabel = new QLabel(PropertiesPalette_TextBase);
        paraStyleLabel->setObjectName(QString::fromUtf8("paraStyleLabel"));
        paraStyleLabel->setMinimumSize(QSize(20, 0));
        paraStyleLabel->setMaximumSize(QSize(22, 22));

        horizontalLayout_3->addWidget(paraStyleLabel);

        paraStyleCombo = new ParaStyleComboBox(PropertiesPalette_TextBase);
        paraStyleCombo->setObjectName(QString::fromUtf8("paraStyleCombo"));
        sizePolicy.setHeightForWidth(paraStyleCombo->sizePolicy().hasHeightForWidth());
        paraStyleCombo->setSizePolicy(sizePolicy);

        horizontalLayout_3->addWidget(paraStyleCombo);

        paraStyleClear = new QToolButton(PropertiesPalette_TextBase);
        paraStyleClear->setObjectName(QString::fromUtf8("paraStyleClear"));

        horizontalLayout_3->addWidget(paraStyleClear);


        verticalLayout->addLayout(horizontalLayout_3);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(3);
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        charStyleLabel = new QLabel(PropertiesPalette_TextBase);
        charStyleLabel->setObjectName(QString::fromUtf8("charStyleLabel"));
        charStyleLabel->setMinimumSize(QSize(20, 0));
        charStyleLabel->setMaximumSize(QSize(22, 22));

        horizontalLayout_4->addWidget(charStyleLabel);

        charStyleCombo = new CharStyleComboBox(PropertiesPalette_TextBase);
        charStyleCombo->setObjectName(QString::fromUtf8("charStyleCombo"));
        sizePolicy.setHeightForWidth(charStyleCombo->sizePolicy().hasHeightForWidth());
        charStyleCombo->setSizePolicy(sizePolicy);

        horizontalLayout_4->addWidget(charStyleCombo);

        charStyleClear = new QToolButton(PropertiesPalette_TextBase);
        charStyleClear->setObjectName(QString::fromUtf8("charStyleClear"));

        horizontalLayout_4->addWidget(charStyleClear);


        verticalLayout->addLayout(horizontalLayout_4);

        textTree = new ScTreeWidget(PropertiesPalette_TextBase);
        QTreeWidgetItem *__qtreewidgetitem = new QTreeWidgetItem();
        __qtreewidgetitem->setText(0, QString::fromUtf8("1"));
        textTree->setHeaderItem(__qtreewidgetitem);
        textTree->setObjectName(QString::fromUtf8("textTree"));

        verticalLayout->addWidget(textTree);


        retranslateUi(PropertiesPalette_TextBase);

        QMetaObject::connectSlotsByName(PropertiesPalette_TextBase);
    } // setupUi

    void retranslateUi(QWidget *PropertiesPalette_TextBase)
    {
        fontSizeLabel->setText(QString());
#if QT_CONFIG(tooltip)
        fontSize->setToolTip(QCoreApplication::translate("PropertiesPalette_TextBase", "Font Size", nullptr));
#endif // QT_CONFIG(tooltip)
        fontSize->setSuffix(QCoreApplication::translate("PropertiesPalette_TextBase", " pt", nullptr));
        lineSpacingLabel->setText(QString());
#if QT_CONFIG(tooltip)
        lineSpacingModeCombo->setToolTip(QCoreApplication::translate("PropertiesPalette_TextBase", "Select the line spacing mode", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        lineSpacing->setToolTip(QCoreApplication::translate("PropertiesPalette_TextBase", "Line Spacing", nullptr));
#endif // QT_CONFIG(tooltip)
        lineSpacing->setSuffix(QCoreApplication::translate("PropertiesPalette_TextBase", " pt", nullptr));
        textAlignmentLabel->setText(QString());
        langLabel->setText(QString());
#if QT_CONFIG(tooltip)
        langCombo->setToolTip(QCoreApplication::translate("PropertiesPalette_TextBase", "Text Language", nullptr));
#endif // QT_CONFIG(tooltip)
        paraStyleLabel->setText(QString());
#if QT_CONFIG(tooltip)
        paraStyleCombo->setToolTip(QCoreApplication::translate("PropertiesPalette_TextBase", "Paragraph style of currently selected text or paragraph", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        paraStyleClear->setToolTip(QCoreApplication::translate("PropertiesPalette_TextBase", "Remove Direct Paragraph Formatting", nullptr));
#endif // QT_CONFIG(tooltip)
        paraStyleClear->setText(QString());
        charStyleLabel->setText(QString());
#if QT_CONFIG(tooltip)
        charStyleCombo->setToolTip(QCoreApplication::translate("PropertiesPalette_TextBase", "Character style of currently selected text or paragraph", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        charStyleClear->setToolTip(QCoreApplication::translate("PropertiesPalette_TextBase", "Remove Direct Character Formatting", nullptr));
#endif // QT_CONFIG(tooltip)
        charStyleClear->setText(QString());
        (void)PropertiesPalette_TextBase;
    } // retranslateUi

};

namespace Ui {
    class PropertiesPalette_TextBase: public Ui_PropertiesPalette_TextBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTIESPALETTE_TEXTBASE_H
