/********************************************************************************
** Form generated from reading UI file 'createrange.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CREATERANGE_H
#define UI_CREATERANGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QWidget>
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_CreateRange
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *hboxLayout;
    QLabel *pageCountLabel;
    QLabel *pageCountValueLabel;
    QSpacerItem *spacerItem;
    QSpacerItem *spacerItem1;
    QTabWidget *tabWidget;
    QWidget *tabBasic;
    QGridLayout *gridLayout1;
    QGroupBox *basicAddARangeButtonGroup;
    QGridLayout *gridLayout2;
    QRadioButton *basicConsecutiveRadioButton;
    ScrSpinBox *basicConsecutiveFromSpinBox;
    QLineEdit *basicCommaSepLineEdit;
    QRadioButton *basicOddRadioButton;
    ScrSpinBox *basicConsecutiveToSpinBox;
    QLabel *basicConsecutiveFromLabel;
    QHBoxLayout *hboxLayout1;
    QSpacerItem *spacerItem2;
    QPushButton *basicRangeAddButton;
    QSpacerItem *spacerItem3;
    QRadioButton *basicEvenRadioButton;
    QSpacerItem *spacerItem4;
    QLabel *basicConsecutiveToLabel;
    QRadioButton *basicCommaSepRadioButton;
    QSpacerItem *verticalSpacer;
    QGroupBox *groupBox1;
    QGridLayout *gridLayout3;
    QPushButton *basicRangeDelButton;
    QSpacerItem *spacerItem5;
    QPushButton *basicRangeDownButton;
    QPushButton *basicRangeUpButton;
    QListWidget *basicRangeListBox;
    QWidget *tabAdvanced;
    QGridLayout *gridLayout4;
    QSpacerItem *spacerItem6;
    QSpacerItem *spacerItem7;
    QLabel *advSampleOrderExampleLabel;
    QLabel *advSampleOrderLabel;
    QSpacerItem *spacerItem8;
    ScrSpinBox *advPageGroupSizeSpinBox;
    QLabel *advPageGroupSizeLabel;
    QHBoxLayout *hboxLayout2;
    QSpacerItem *spacerItem9;
    QPushButton *okButton;
    QPushButton *cancelButton;

    void setupUi(QDialog *CreateRange)
    {
        if (CreateRange->objectName().isEmpty())
            CreateRange->setObjectName(QString::fromUtf8("CreateRange"));
        CreateRange->resize(605, 386);
        gridLayout = new QGridLayout(CreateRange);
        gridLayout->setSpacing(6);
        gridLayout->setContentsMargins(11, 11, 11, 11);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setSpacing(6);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        pageCountLabel = new QLabel(CreateRange);
        pageCountLabel->setObjectName(QString::fromUtf8("pageCountLabel"));
        pageCountLabel->setWordWrap(false);

        hboxLayout->addWidget(pageCountLabel);

        pageCountValueLabel = new QLabel(CreateRange);
        pageCountValueLabel->setObjectName(QString::fromUtf8("pageCountValueLabel"));
        pageCountValueLabel->setWordWrap(false);

        hboxLayout->addWidget(pageCountValueLabel);

        spacerItem = new QSpacerItem(80, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        spacerItem1 = new QSpacerItem(72, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem1);


        gridLayout->addLayout(hboxLayout, 0, 0, 1, 1);

        tabWidget = new QTabWidget(CreateRange);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tabBasic = new QWidget();
        tabBasic->setObjectName(QString::fromUtf8("tabBasic"));
        gridLayout1 = new QGridLayout(tabBasic);
        gridLayout1->setSpacing(6);
        gridLayout1->setContentsMargins(11, 11, 11, 11);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        basicAddARangeButtonGroup = new QGroupBox(tabBasic);
        basicAddARangeButtonGroup->setObjectName(QString::fromUtf8("basicAddARangeButtonGroup"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(basicAddARangeButtonGroup->sizePolicy().hasHeightForWidth());
        basicAddARangeButtonGroup->setSizePolicy(sizePolicy);
        gridLayout2 = new QGridLayout(basicAddARangeButtonGroup);
        gridLayout2->setSpacing(6);
        gridLayout2->setContentsMargins(11, 11, 11, 11);
        gridLayout2->setObjectName(QString::fromUtf8("gridLayout2"));
        basicConsecutiveRadioButton = new QRadioButton(basicAddARangeButtonGroup);
        basicConsecutiveRadioButton->setObjectName(QString::fromUtf8("basicConsecutiveRadioButton"));
        basicConsecutiveRadioButton->setChecked(true);

        gridLayout2->addWidget(basicConsecutiveRadioButton, 0, 0, 1, 5);

        basicConsecutiveFromSpinBox = new ScrSpinBox(basicAddARangeButtonGroup);
        basicConsecutiveFromSpinBox->setObjectName(QString::fromUtf8("basicConsecutiveFromSpinBox"));

        gridLayout2->addWidget(basicConsecutiveFromSpinBox, 1, 1, 1, 1);

        basicCommaSepLineEdit = new QLineEdit(basicAddARangeButtonGroup);
        basicCommaSepLineEdit->setObjectName(QString::fromUtf8("basicCommaSepLineEdit"));

        gridLayout2->addWidget(basicCommaSepLineEdit, 3, 0, 1, 5);

        basicOddRadioButton = new QRadioButton(basicAddARangeButtonGroup);
        basicOddRadioButton->setObjectName(QString::fromUtf8("basicOddRadioButton"));

        gridLayout2->addWidget(basicOddRadioButton, 5, 0, 1, 5);

        basicConsecutiveToSpinBox = new ScrSpinBox(basicAddARangeButtonGroup);
        basicConsecutiveToSpinBox->setObjectName(QString::fromUtf8("basicConsecutiveToSpinBox"));

        gridLayout2->addWidget(basicConsecutiveToSpinBox, 1, 3, 1, 1);

        basicConsecutiveFromLabel = new QLabel(basicAddARangeButtonGroup);
        basicConsecutiveFromLabel->setObjectName(QString::fromUtf8("basicConsecutiveFromLabel"));
        basicConsecutiveFromLabel->setWordWrap(false);

        gridLayout2->addWidget(basicConsecutiveFromLabel, 1, 0, 1, 1);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setSpacing(6);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        spacerItem2 = new QSpacerItem(38, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem2);

        basicRangeAddButton = new QPushButton(basicAddARangeButtonGroup);
        basicRangeAddButton->setObjectName(QString::fromUtf8("basicRangeAddButton"));

        hboxLayout1->addWidget(basicRangeAddButton);

        spacerItem3 = new QSpacerItem(38, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem3);


        gridLayout2->addLayout(hboxLayout1, 6, 0, 1, 5);

        basicEvenRadioButton = new QRadioButton(basicAddARangeButtonGroup);
        basicEvenRadioButton->setObjectName(QString::fromUtf8("basicEvenRadioButton"));

        gridLayout2->addWidget(basicEvenRadioButton, 4, 0, 1, 5);

        spacerItem4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout2->addItem(spacerItem4, 1, 4, 1, 1);

        basicConsecutiveToLabel = new QLabel(basicAddARangeButtonGroup);
        basicConsecutiveToLabel->setObjectName(QString::fromUtf8("basicConsecutiveToLabel"));
        basicConsecutiveToLabel->setWordWrap(false);

        gridLayout2->addWidget(basicConsecutiveToLabel, 1, 2, 1, 1);

        basicCommaSepRadioButton = new QRadioButton(basicAddARangeButtonGroup);
        basicCommaSepRadioButton->setObjectName(QString::fromUtf8("basicCommaSepRadioButton"));

        gridLayout2->addWidget(basicCommaSepRadioButton, 2, 0, 1, 5);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout2->addItem(verticalSpacer, 7, 0, 1, 1);


        gridLayout1->addWidget(basicAddARangeButtonGroup, 0, 0, 1, 1);

        groupBox1 = new QGroupBox(tabBasic);
        groupBox1->setObjectName(QString::fromUtf8("groupBox1"));
        gridLayout3 = new QGridLayout(groupBox1);
        gridLayout3->setSpacing(6);
        gridLayout3->setContentsMargins(11, 11, 11, 11);
        gridLayout3->setObjectName(QString::fromUtf8("gridLayout3"));
        basicRangeDelButton = new QPushButton(groupBox1);
        basicRangeDelButton->setObjectName(QString::fromUtf8("basicRangeDelButton"));

        gridLayout3->addWidget(basicRangeDelButton, 2, 1, 1, 1);

        spacerItem5 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout3->addItem(spacerItem5, 3, 1, 1, 1);

        basicRangeDownButton = new QPushButton(groupBox1);
        basicRangeDownButton->setObjectName(QString::fromUtf8("basicRangeDownButton"));

        gridLayout3->addWidget(basicRangeDownButton, 1, 1, 1, 1);

        basicRangeUpButton = new QPushButton(groupBox1);
        basicRangeUpButton->setObjectName(QString::fromUtf8("basicRangeUpButton"));

        gridLayout3->addWidget(basicRangeUpButton, 0, 1, 1, 1);

        basicRangeListBox = new QListWidget(groupBox1);
        basicRangeListBox->setObjectName(QString::fromUtf8("basicRangeListBox"));

        gridLayout3->addWidget(basicRangeListBox, 0, 0, 4, 1);


        gridLayout1->addWidget(groupBox1, 0, 1, 1, 1);

        tabWidget->addTab(tabBasic, QString());
        tabAdvanced = new QWidget();
        tabAdvanced->setObjectName(QString::fromUtf8("tabAdvanced"));
        gridLayout4 = new QGridLayout(tabAdvanced);
        gridLayout4->setSpacing(6);
        gridLayout4->setContentsMargins(11, 11, 11, 11);
        gridLayout4->setObjectName(QString::fromUtf8("gridLayout4"));
        spacerItem6 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout4->addItem(spacerItem6, 2, 0, 1, 5);

        spacerItem7 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout4->addItem(spacerItem7, 1, 4, 1, 1);

        advSampleOrderExampleLabel = new QLabel(tabAdvanced);
        advSampleOrderExampleLabel->setObjectName(QString::fromUtf8("advSampleOrderExampleLabel"));
        advSampleOrderExampleLabel->setWordWrap(false);

        gridLayout4->addWidget(advSampleOrderExampleLabel, 1, 2, 1, 2);

        advSampleOrderLabel = new QLabel(tabAdvanced);
        advSampleOrderLabel->setObjectName(QString::fromUtf8("advSampleOrderLabel"));
        advSampleOrderLabel->setWordWrap(false);

        gridLayout4->addWidget(advSampleOrderLabel, 1, 0, 1, 2);

        spacerItem8 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout4->addItem(spacerItem8, 0, 3, 1, 2);

        advPageGroupSizeSpinBox = new ScrSpinBox(tabAdvanced);
        advPageGroupSizeSpinBox->setObjectName(QString::fromUtf8("advPageGroupSizeSpinBox"));

        gridLayout4->addWidget(advPageGroupSizeSpinBox, 0, 1, 1, 2);

        advPageGroupSizeLabel = new QLabel(tabAdvanced);
        advPageGroupSizeLabel->setObjectName(QString::fromUtf8("advPageGroupSizeLabel"));
        advPageGroupSizeLabel->setWordWrap(false);

        gridLayout4->addWidget(advPageGroupSizeLabel, 0, 0, 1, 1);

        tabWidget->addTab(tabAdvanced, QString());

        gridLayout->addWidget(tabWidget, 1, 0, 1, 1);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setSpacing(6);
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        spacerItem9 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout2->addItem(spacerItem9);

        okButton = new QPushButton(CreateRange);
        okButton->setObjectName(QString::fromUtf8("okButton"));

        hboxLayout2->addWidget(okButton);

        cancelButton = new QPushButton(CreateRange);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        hboxLayout2->addWidget(cancelButton);


        gridLayout->addLayout(hboxLayout2, 2, 0, 1, 1);


        retranslateUi(CreateRange);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(CreateRange);
    } // setupUi

    void retranslateUi(QDialog *CreateRange)
    {
        CreateRange->setWindowTitle(QCoreApplication::translate("CreateRange", "Create Range", nullptr));
        pageCountLabel->setText(QCoreApplication::translate("CreateRange", "Number of Pages in Document:", nullptr));
        pageCountValueLabel->setText(QString());
        basicAddARangeButtonGroup->setTitle(QCoreApplication::translate("CreateRange", "Add a Range of Pages", nullptr));
        basicConsecutiveRadioButton->setText(QCoreApplication::translate("CreateRange", "Consecutive Pages", nullptr));
        basicOddRadioButton->setText(QCoreApplication::translate("CreateRange", "Odd Pages", nullptr));
        basicConsecutiveFromLabel->setText(QCoreApplication::translate("CreateRange", "From:", nullptr));
        basicRangeAddButton->setText(QCoreApplication::translate("CreateRange", "&Add To Range", nullptr));
#if QT_CONFIG(shortcut)
        basicRangeAddButton->setShortcut(QCoreApplication::translate("CreateRange", "Alt+A", nullptr));
#endif // QT_CONFIG(shortcut)
        basicEvenRadioButton->setText(QCoreApplication::translate("CreateRange", "Even Pages", nullptr));
        basicConsecutiveToLabel->setText(QCoreApplication::translate("CreateRange", "To:", nullptr));
        basicCommaSepRadioButton->setText(QCoreApplication::translate("CreateRange", "Comma Separated List", nullptr));
        groupBox1->setTitle(QCoreApplication::translate("CreateRange", "Range of Pages", nullptr));
        basicRangeDelButton->setText(QCoreApplication::translate("CreateRange", "De&lete", nullptr));
#if QT_CONFIG(shortcut)
        basicRangeDelButton->setShortcut(QCoreApplication::translate("CreateRange", "Alt+L", nullptr));
#endif // QT_CONFIG(shortcut)
        basicRangeDownButton->setText(QCoreApplication::translate("CreateRange", "Move &Down", nullptr));
#if QT_CONFIG(shortcut)
        basicRangeDownButton->setShortcut(QCoreApplication::translate("CreateRange", "Alt+D", nullptr));
#endif // QT_CONFIG(shortcut)
        basicRangeUpButton->setText(QCoreApplication::translate("CreateRange", "Move &Up", nullptr));
#if QT_CONFIG(shortcut)
        basicRangeUpButton->setShortcut(QCoreApplication::translate("CreateRange", "Alt+U", nullptr));
#endif // QT_CONFIG(shortcut)
        tabWidget->setTabText(tabWidget->indexOf(tabBasic), QCoreApplication::translate("CreateRange", "Basic Range Selection", nullptr));
        advSampleOrderExampleLabel->setText(QString());
        advSampleOrderLabel->setText(QCoreApplication::translate("CreateRange", "Sample Page Order:", nullptr));
        advPageGroupSizeLabel->setText(QCoreApplication::translate("CreateRange", "Page Group Size:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabAdvanced), QCoreApplication::translate("CreateRange", "Advanced Reordering", nullptr));
        okButton->setText(QCoreApplication::translate("CreateRange", "&OK", nullptr));
#if QT_CONFIG(shortcut)
        okButton->setShortcut(QCoreApplication::translate("CreateRange", "Alt+O", nullptr));
#endif // QT_CONFIG(shortcut)
        cancelButton->setText(QCoreApplication::translate("CreateRange", "&Cancel", nullptr));
#if QT_CONFIG(shortcut)
        cancelButton->setShortcut(QCoreApplication::translate("CreateRange", "Alt+C", nullptr));
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class CreateRange: public Ui_CreateRange {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CREATERANGE_H
