/********************************************************************************
** Form generated from reading UI file 'smreplacedia.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SMREPLACEDIA_H
#define UI_SMREPLACEDIA_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_SMReplaceDia
{
public:
    QVBoxLayout *vboxLayout;
    QFrame *mainFrame;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *buttonOk;
    QPushButton *buttonCancel;

    void setupUi(QDialog *SMReplaceDia)
    {
        if (SMReplaceDia->objectName().isEmpty())
            SMReplaceDia->setObjectName(QString::fromUtf8("SMReplaceDia"));
        SMReplaceDia->resize(235, 271);
        SMReplaceDia->setSizeGripEnabled(false);
        vboxLayout = new QVBoxLayout(SMReplaceDia);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        mainFrame = new QFrame(SMReplaceDia);
        mainFrame->setObjectName(QString::fromUtf8("mainFrame"));
        mainFrame->setFrameShape(QFrame::NoFrame);
        mainFrame->setFrameShadow(QFrame::Plain);
        mainFrame->setLineWidth(0);

        vboxLayout->addWidget(mainFrame);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        hboxLayout->setContentsMargins(0, 0, 0, 0);
        spacerItem = new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        buttonOk = new QPushButton(SMReplaceDia);
        buttonOk->setObjectName(QString::fromUtf8("buttonOk"));
        buttonOk->setAutoDefault(true);

        hboxLayout->addWidget(buttonOk);

        buttonCancel = new QPushButton(SMReplaceDia);
        buttonCancel->setObjectName(QString::fromUtf8("buttonCancel"));
        buttonCancel->setAutoDefault(true);

        hboxLayout->addWidget(buttonCancel);


        vboxLayout->addLayout(hboxLayout);


        retranslateUi(SMReplaceDia);
        QObject::connect(buttonOk, SIGNAL(clicked()), SMReplaceDia, SLOT(accept()));
        QObject::connect(buttonCancel, SIGNAL(clicked()), SMReplaceDia, SLOT(reject()));

        buttonOk->setDefault(true);


        QMetaObject::connectSlotsByName(SMReplaceDia);
    } // setupUi

    void retranslateUi(QDialog *SMReplaceDia)
    {
        SMReplaceDia->setWindowTitle(QCoreApplication::translate("SMReplaceDia", "Delete Styles", nullptr));
        buttonOk->setText(QCoreApplication::translate("SMReplaceDia", "&OK", nullptr));
#if QT_CONFIG(shortcut)
        buttonOk->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        buttonCancel->setText(QCoreApplication::translate("SMReplaceDia", "Ca&ncel", nullptr));
#if QT_CONFIG(shortcut)
        buttonCancel->setShortcut(QCoreApplication::translate("SMReplaceDia", "Alt+N", nullptr));
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class SMReplaceDia: public Ui_SMReplaceDia {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SMREPLACEDIA_H
