/********************************************************************************
** Form generated from reading UI file 'aligndistribute.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ALIGNDISTRIBUTE_H
#define UI_ALIGNDISTRIBUTE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/scdockpalette.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_AlignDistribute
{
public:
    QWidget *dockWidgetContents;
    QVBoxLayout *verticalLayout_3;
    QTabWidget *tabWidget;
    QWidget *tabAlign;
    QVBoxLayout *vboxLayout;
    QHBoxLayout *_2;
    QSpacerItem *spacerItem;
    QGridLayout *_3;
    QLabel *alignGuideLabel;
    QLabel *alignRelativeToLabel;
    QLineEdit *alignGuideLineEdit;
    QComboBox *alignRelativeToCombo;
    QLabel *alignMoveOrResizeLabel;
    QComboBox *alignMoveOrResizeCombo;
    QSpacerItem *spacerItem1;
    QHBoxLayout *_4;
    QSpacerItem *spacerItem2;
    QGridLayout *_5;
    QToolButton *alignBottomInToolButton;
    QToolButton *alignTopInToolButton;
    QToolButton *alignLeftInToolButton;
    QToolButton *alignRightInToolButton;
    QToolButton *alignBottomOutToolButton;
    QToolButton *alignTopOutToolButton;
    QToolButton *alignCenterVerToolButton;
    QToolButton *alignLeftOutToolButton;
    QToolButton *alignRightOutToolButton;
    QToolButton *alignCenterHorToolButton;
    QSpacerItem *spacerItem3;
    QSpacerItem *spacerItem4;
    QWidget *tabDistribute;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *_6;
    QSpacerItem *spacerItem5;
    QGridLayout *_7;
    QToolButton *distributeDistHToolButton;
    QToolButton *distributeTopToolButton;
    QToolButton *distributeCenterVToolButton;
    QToolButton *distributeRightToolButton;
    QToolButton *distributeAcrossMarginsToolButton;
    QToolButton *distributeLeftToolButton;
    QToolButton *distributeAcrossPageToolButton;
    QToolButton *distributeDistValueHToolButton;
    QToolButton *distributeDownMarginsToolButton;
    QToolButton *distributeCenterHToolButton;
    QToolButton *distributeDownPageToolButton;
    QToolButton *distributeDistValueVToolButton;
    QToolButton *distributeDistVToolButton;
    QToolButton *distributeBottomToolButton;
    QSpacerItem *spacerItem6;
    QHBoxLayout *_8;
    QSpacerItem *spacerItem7;
    QHBoxLayout *_9;
    QLabel *distributeDistLabel;
    ScrSpinBox *distributeDistSpinBox;
    QSpacerItem *spacerItem8;
    QHBoxLayout *horizontalLayout_2;
    QSpacerItem *spacer;
    QCheckBox *reverseDistributionCheckBox;
    QSpacerItem *spacer_2;
    QSpacerItem *spacerItem9;
    QWidget *tabSwap;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QToolButton *swapLeftToolButton;
    QToolButton *swapRightToolButton;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *verticalSpacer;

    void setupUi(ScDockPalette *AlignDistribute)
    {
        if (AlignDistribute->objectName().isEmpty())
            AlignDistribute->setObjectName(QString::fromUtf8("AlignDistribute"));
        AlignDistribute->resize(269, 308);
        dockWidgetContents = new QWidget();
        dockWidgetContents->setObjectName(QString::fromUtf8("dockWidgetContents"));
        verticalLayout_3 = new QVBoxLayout(dockWidgetContents);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(3, 3, 3, 3);
        tabWidget = new QTabWidget(dockWidgetContents);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tabAlign = new QWidget();
        tabAlign->setObjectName(QString::fromUtf8("tabAlign"));
        vboxLayout = new QVBoxLayout(tabAlign);
        vboxLayout->setSpacing(3);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(6, 6, 6, 6);
        _2 = new QHBoxLayout();
        _2->setSpacing(3);
        _2->setObjectName(QString::fromUtf8("_2"));
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _2->addItem(spacerItem);

        _3 = new QGridLayout();
        _3->setObjectName(QString::fromUtf8("_3"));
        alignGuideLabel = new QLabel(tabAlign);
        alignGuideLabel->setObjectName(QString::fromUtf8("alignGuideLabel"));
        alignGuideLabel->setWordWrap(false);

        _3->addWidget(alignGuideLabel, 1, 0, 1, 1);

        alignRelativeToLabel = new QLabel(tabAlign);
        alignRelativeToLabel->setObjectName(QString::fromUtf8("alignRelativeToLabel"));
        alignRelativeToLabel->setWordWrap(false);

        _3->addWidget(alignRelativeToLabel, 0, 0, 1, 1);

        alignGuideLineEdit = new QLineEdit(tabAlign);
        alignGuideLineEdit->setObjectName(QString::fromUtf8("alignGuideLineEdit"));

        _3->addWidget(alignGuideLineEdit, 1, 1, 1, 1);

        alignRelativeToCombo = new QComboBox(tabAlign);
        alignRelativeToCombo->setObjectName(QString::fromUtf8("alignRelativeToCombo"));

        _3->addWidget(alignRelativeToCombo, 0, 1, 1, 1);

        alignMoveOrResizeLabel = new QLabel(tabAlign);
        alignMoveOrResizeLabel->setObjectName(QString::fromUtf8("alignMoveOrResizeLabel"));
        alignMoveOrResizeLabel->setWordWrap(false);

        _3->addWidget(alignMoveOrResizeLabel, 2, 0, 1, 1);

        alignMoveOrResizeCombo = new QComboBox(tabAlign);
        alignMoveOrResizeCombo->setObjectName(QString::fromUtf8("alignMoveOrResizeCombo"));

        _3->addWidget(alignMoveOrResizeCombo, 2, 1, 1, 1);


        _2->addLayout(_3);

        spacerItem1 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _2->addItem(spacerItem1);


        vboxLayout->addLayout(_2);

        _4 = new QHBoxLayout();
        _4->setSpacing(3);
        _4->setObjectName(QString::fromUtf8("_4"));
        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _4->addItem(spacerItem2);

        _5 = new QGridLayout();
        _5->setObjectName(QString::fromUtf8("_5"));
        alignBottomInToolButton = new QToolButton(tabAlign);
        alignBottomInToolButton->setObjectName(QString::fromUtf8("alignBottomInToolButton"));
        alignBottomInToolButton->setMinimumSize(QSize(24, 24));
        alignBottomInToolButton->setMaximumSize(QSize(24, 24));
        alignBottomInToolButton->setIconSize(QSize(22, 22));

        _5->addWidget(alignBottomInToolButton, 1, 3, 1, 1);

        alignTopInToolButton = new QToolButton(tabAlign);
        alignTopInToolButton->setObjectName(QString::fromUtf8("alignTopInToolButton"));
        alignTopInToolButton->setMinimumSize(QSize(24, 24));
        alignTopInToolButton->setMaximumSize(QSize(24, 24));
        alignTopInToolButton->setIconSize(QSize(22, 22));

        _5->addWidget(alignTopInToolButton, 1, 1, 1, 1);

        alignLeftInToolButton = new QToolButton(tabAlign);
        alignLeftInToolButton->setObjectName(QString::fromUtf8("alignLeftInToolButton"));
        alignLeftInToolButton->setMinimumSize(QSize(24, 24));
        alignLeftInToolButton->setMaximumSize(QSize(24, 24));
        alignLeftInToolButton->setIconSize(QSize(22, 22));

        _5->addWidget(alignLeftInToolButton, 0, 1, 1, 1);

        alignRightInToolButton = new QToolButton(tabAlign);
        alignRightInToolButton->setObjectName(QString::fromUtf8("alignRightInToolButton"));
        alignRightInToolButton->setMinimumSize(QSize(24, 24));
        alignRightInToolButton->setMaximumSize(QSize(24, 24));
        alignRightInToolButton->setIconSize(QSize(22, 22));

        _5->addWidget(alignRightInToolButton, 0, 3, 1, 1);

        alignBottomOutToolButton = new QToolButton(tabAlign);
        alignBottomOutToolButton->setObjectName(QString::fromUtf8("alignBottomOutToolButton"));
        alignBottomOutToolButton->setMinimumSize(QSize(24, 24));
        alignBottomOutToolButton->setMaximumSize(QSize(24, 24));
        alignBottomOutToolButton->setIconSize(QSize(22, 22));

        _5->addWidget(alignBottomOutToolButton, 1, 4, 1, 1);

        alignTopOutToolButton = new QToolButton(tabAlign);
        alignTopOutToolButton->setObjectName(QString::fromUtf8("alignTopOutToolButton"));
        alignTopOutToolButton->setMinimumSize(QSize(24, 24));
        alignTopOutToolButton->setMaximumSize(QSize(24, 24));
        alignTopOutToolButton->setIconSize(QSize(22, 22));

        _5->addWidget(alignTopOutToolButton, 1, 0, 1, 1);

        alignCenterVerToolButton = new QToolButton(tabAlign);
        alignCenterVerToolButton->setObjectName(QString::fromUtf8("alignCenterVerToolButton"));
        alignCenterVerToolButton->setMinimumSize(QSize(24, 24));
        alignCenterVerToolButton->setMaximumSize(QSize(24, 24));
        alignCenterVerToolButton->setIconSize(QSize(22, 22));

        _5->addWidget(alignCenterVerToolButton, 1, 2, 1, 1);

        alignLeftOutToolButton = new QToolButton(tabAlign);
        alignLeftOutToolButton->setObjectName(QString::fromUtf8("alignLeftOutToolButton"));
        alignLeftOutToolButton->setMinimumSize(QSize(24, 24));
        alignLeftOutToolButton->setMaximumSize(QSize(24, 24));
        alignLeftOutToolButton->setIconSize(QSize(22, 22));

        _5->addWidget(alignLeftOutToolButton, 0, 0, 1, 1);

        alignRightOutToolButton = new QToolButton(tabAlign);
        alignRightOutToolButton->setObjectName(QString::fromUtf8("alignRightOutToolButton"));
        alignRightOutToolButton->setMinimumSize(QSize(24, 24));
        alignRightOutToolButton->setMaximumSize(QSize(24, 24));
        alignRightOutToolButton->setIconSize(QSize(22, 22));

        _5->addWidget(alignRightOutToolButton, 0, 4, 1, 1);

        alignCenterHorToolButton = new QToolButton(tabAlign);
        alignCenterHorToolButton->setObjectName(QString::fromUtf8("alignCenterHorToolButton"));
        alignCenterHorToolButton->setMinimumSize(QSize(24, 24));
        alignCenterHorToolButton->setMaximumSize(QSize(24, 24));
        alignCenterHorToolButton->setIconSize(QSize(22, 22));

        _5->addWidget(alignCenterHorToolButton, 0, 2, 1, 1);


        _4->addLayout(_5);

        spacerItem3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _4->addItem(spacerItem3);


        vboxLayout->addLayout(_4);

        spacerItem4 = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout->addItem(spacerItem4);

        tabWidget->addTab(tabAlign, QString());
        tabDistribute = new QWidget();
        tabDistribute->setObjectName(QString::fromUtf8("tabDistribute"));
        verticalLayout_2 = new QVBoxLayout(tabDistribute);
        verticalLayout_2->setSpacing(3);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(6, 6, 6, 6);
        _6 = new QHBoxLayout();
        _6->setSpacing(3);
        _6->setObjectName(QString::fromUtf8("_6"));
        spacerItem5 = new QSpacerItem(63, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _6->addItem(spacerItem5);

        _7 = new QGridLayout();
        _7->setObjectName(QString::fromUtf8("_7"));
        distributeDistHToolButton = new QToolButton(tabDistribute);
        distributeDistHToolButton->setObjectName(QString::fromUtf8("distributeDistHToolButton"));
        distributeDistHToolButton->setMinimumSize(QSize(24, 24));
        distributeDistHToolButton->setMaximumSize(QSize(24, 24));
        distributeDistHToolButton->setIconSize(QSize(22, 22));

        _7->addWidget(distributeDistHToolButton, 0, 3, 1, 1);

        distributeTopToolButton = new QToolButton(tabDistribute);
        distributeTopToolButton->setObjectName(QString::fromUtf8("distributeTopToolButton"));
        distributeTopToolButton->setMinimumSize(QSize(24, 24));
        distributeTopToolButton->setMaximumSize(QSize(24, 24));
        distributeTopToolButton->setIconSize(QSize(22, 22));

        _7->addWidget(distributeTopToolButton, 1, 2, 1, 1);

        distributeCenterVToolButton = new QToolButton(tabDistribute);
        distributeCenterVToolButton->setObjectName(QString::fromUtf8("distributeCenterVToolButton"));
        distributeCenterVToolButton->setMinimumSize(QSize(24, 24));
        distributeCenterVToolButton->setMaximumSize(QSize(24, 24));
        distributeCenterVToolButton->setIconSize(QSize(22, 22));

        _7->addWidget(distributeCenterVToolButton, 1, 1, 1, 1);

        distributeRightToolButton = new QToolButton(tabDistribute);
        distributeRightToolButton->setObjectName(QString::fromUtf8("distributeRightToolButton"));
        distributeRightToolButton->setMinimumSize(QSize(24, 24));
        distributeRightToolButton->setMaximumSize(QSize(24, 24));
        distributeRightToolButton->setIconSize(QSize(22, 22));

        _7->addWidget(distributeRightToolButton, 0, 2, 1, 1);

        distributeAcrossMarginsToolButton = new QToolButton(tabDistribute);
        distributeAcrossMarginsToolButton->setObjectName(QString::fromUtf8("distributeAcrossMarginsToolButton"));
        distributeAcrossMarginsToolButton->setEnabled(true);
        distributeAcrossMarginsToolButton->setMinimumSize(QSize(24, 24));
        distributeAcrossMarginsToolButton->setMaximumSize(QSize(24, 24));
        distributeAcrossMarginsToolButton->setIconSize(QSize(22, 22));

        _7->addWidget(distributeAcrossMarginsToolButton, 2, 2, 1, 1);

        distributeLeftToolButton = new QToolButton(tabDistribute);
        distributeLeftToolButton->setObjectName(QString::fromUtf8("distributeLeftToolButton"));
        distributeLeftToolButton->setMinimumSize(QSize(24, 24));
        distributeLeftToolButton->setMaximumSize(QSize(24, 24));
        distributeLeftToolButton->setIconSize(QSize(22, 22));

        _7->addWidget(distributeLeftToolButton, 0, 0, 1, 1);

        distributeAcrossPageToolButton = new QToolButton(tabDistribute);
        distributeAcrossPageToolButton->setObjectName(QString::fromUtf8("distributeAcrossPageToolButton"));
        distributeAcrossPageToolButton->setEnabled(true);
        distributeAcrossPageToolButton->setMinimumSize(QSize(24, 24));
        distributeAcrossPageToolButton->setMaximumSize(QSize(24, 24));
        distributeAcrossPageToolButton->setIconSize(QSize(22, 22));

        _7->addWidget(distributeAcrossPageToolButton, 2, 0, 1, 1);

        distributeDistValueHToolButton = new QToolButton(tabDistribute);
        distributeDistValueHToolButton->setObjectName(QString::fromUtf8("distributeDistValueHToolButton"));
        distributeDistValueHToolButton->setMinimumSize(QSize(24, 24));
        distributeDistValueHToolButton->setMaximumSize(QSize(24, 24));
        distributeDistValueHToolButton->setIconSize(QSize(22, 22));

        _7->addWidget(distributeDistValueHToolButton, 3, 1, 1, 1);

        distributeDownMarginsToolButton = new QToolButton(tabDistribute);
        distributeDownMarginsToolButton->setObjectName(QString::fromUtf8("distributeDownMarginsToolButton"));
        distributeDownMarginsToolButton->setEnabled(true);
        distributeDownMarginsToolButton->setMinimumSize(QSize(24, 24));
        distributeDownMarginsToolButton->setMaximumSize(QSize(24, 24));
        distributeDownMarginsToolButton->setIconSize(QSize(22, 22));

        _7->addWidget(distributeDownMarginsToolButton, 2, 3, 1, 1);

        distributeCenterHToolButton = new QToolButton(tabDistribute);
        distributeCenterHToolButton->setObjectName(QString::fromUtf8("distributeCenterHToolButton"));
        distributeCenterHToolButton->setMinimumSize(QSize(24, 24));
        distributeCenterHToolButton->setMaximumSize(QSize(24, 24));
        distributeCenterHToolButton->setIconSize(QSize(22, 22));

        _7->addWidget(distributeCenterHToolButton, 0, 1, 1, 1);

        distributeDownPageToolButton = new QToolButton(tabDistribute);
        distributeDownPageToolButton->setObjectName(QString::fromUtf8("distributeDownPageToolButton"));
        distributeDownPageToolButton->setEnabled(true);
        distributeDownPageToolButton->setMinimumSize(QSize(24, 24));
        distributeDownPageToolButton->setMaximumSize(QSize(24, 24));
        distributeDownPageToolButton->setIconSize(QSize(22, 22));

        _7->addWidget(distributeDownPageToolButton, 2, 1, 1, 1);

        distributeDistValueVToolButton = new QToolButton(tabDistribute);
        distributeDistValueVToolButton->setObjectName(QString::fromUtf8("distributeDistValueVToolButton"));
        distributeDistValueVToolButton->setMinimumSize(QSize(24, 24));
        distributeDistValueVToolButton->setMaximumSize(QSize(24, 24));
        distributeDistValueVToolButton->setIconSize(QSize(22, 22));

        _7->addWidget(distributeDistValueVToolButton, 3, 2, 1, 1);

        distributeDistVToolButton = new QToolButton(tabDistribute);
        distributeDistVToolButton->setObjectName(QString::fromUtf8("distributeDistVToolButton"));
        distributeDistVToolButton->setMinimumSize(QSize(24, 24));
        distributeDistVToolButton->setMaximumSize(QSize(24, 24));
        distributeDistVToolButton->setIconSize(QSize(22, 22));

        _7->addWidget(distributeDistVToolButton, 1, 3, 1, 1);

        distributeBottomToolButton = new QToolButton(tabDistribute);
        distributeBottomToolButton->setObjectName(QString::fromUtf8("distributeBottomToolButton"));
        distributeBottomToolButton->setMinimumSize(QSize(24, 24));
        distributeBottomToolButton->setMaximumSize(QSize(24, 24));
        distributeBottomToolButton->setIconSize(QSize(22, 22));

        _7->addWidget(distributeBottomToolButton, 1, 0, 1, 1);


        _6->addLayout(_7);

        spacerItem6 = new QSpacerItem(62, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _6->addItem(spacerItem6);


        verticalLayout_2->addLayout(_6);

        _8 = new QHBoxLayout();
        _8->setObjectName(QString::fromUtf8("_8"));
        spacerItem7 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _8->addItem(spacerItem7);

        _9 = new QHBoxLayout();
        _9->setSpacing(3);
        _9->setObjectName(QString::fromUtf8("_9"));
        distributeDistLabel = new QLabel(tabDistribute);
        distributeDistLabel->setObjectName(QString::fromUtf8("distributeDistLabel"));
        distributeDistLabel->setWordWrap(false);
        distributeDistLabel->setIndent(-1);

        _9->addWidget(distributeDistLabel);

        distributeDistSpinBox = new ScrSpinBox(tabDistribute);
        distributeDistSpinBox->setObjectName(QString::fromUtf8("distributeDistSpinBox"));
        distributeDistSpinBox->setMinimumSize(QSize(50, 24));

        _9->addWidget(distributeDistSpinBox);


        _8->addLayout(_9);

        spacerItem8 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _8->addItem(spacerItem8);


        verticalLayout_2->addLayout(_8);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(3);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        spacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(spacer);

        reverseDistributionCheckBox = new QCheckBox(tabDistribute);
        reverseDistributionCheckBox->setObjectName(QString::fromUtf8("reverseDistributionCheckBox"));

        horizontalLayout_2->addWidget(reverseDistributionCheckBox);

        spacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(spacer_2);


        verticalLayout_2->addLayout(horizontalLayout_2);

        spacerItem9 = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(spacerItem9);

        tabWidget->addTab(tabDistribute, QString());
        tabSwap = new QWidget();
        tabSwap->setObjectName(QString::fromUtf8("tabSwap"));
        verticalLayout = new QVBoxLayout(tabSwap);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(6, 6, 6, 6);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(3);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        swapLeftToolButton = new QToolButton(tabSwap);
        swapLeftToolButton->setObjectName(QString::fromUtf8("swapLeftToolButton"));
        swapLeftToolButton->setMinimumSize(QSize(24, 24));
        swapLeftToolButton->setMaximumSize(QSize(24, 24));
        swapLeftToolButton->setIconSize(QSize(22, 22));

        horizontalLayout->addWidget(swapLeftToolButton);

        swapRightToolButton = new QToolButton(tabSwap);
        swapRightToolButton->setObjectName(QString::fromUtf8("swapRightToolButton"));
        swapRightToolButton->setMinimumSize(QSize(24, 24));
        swapRightToolButton->setMaximumSize(QSize(24, 24));
        swapRightToolButton->setIconSize(QSize(22, 22));

        horizontalLayout->addWidget(swapRightToolButton);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer_2);


        verticalLayout->addLayout(horizontalLayout);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        tabWidget->addTab(tabSwap, QString());

        verticalLayout_3->addWidget(tabWidget);

        AlignDistribute->setWidget(dockWidgetContents);
#if QT_CONFIG(shortcut)
        alignGuideLabel->setBuddy(alignGuideLineEdit);
        alignRelativeToLabel->setBuddy(alignRelativeToCombo);
        alignMoveOrResizeLabel->setBuddy(alignMoveOrResizeCombo);
        distributeDistLabel->setBuddy(distributeDistSpinBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(alignRelativeToCombo, alignGuideLineEdit);
        QWidget::setTabOrder(alignGuideLineEdit, alignMoveOrResizeCombo);
        QWidget::setTabOrder(alignMoveOrResizeCombo, alignLeftOutToolButton);
        QWidget::setTabOrder(alignLeftOutToolButton, alignLeftInToolButton);
        QWidget::setTabOrder(alignLeftInToolButton, alignCenterHorToolButton);
        QWidget::setTabOrder(alignCenterHorToolButton, alignRightInToolButton);
        QWidget::setTabOrder(alignRightInToolButton, alignRightOutToolButton);
        QWidget::setTabOrder(alignRightOutToolButton, alignTopOutToolButton);
        QWidget::setTabOrder(alignTopOutToolButton, alignTopInToolButton);
        QWidget::setTabOrder(alignTopInToolButton, alignCenterVerToolButton);
        QWidget::setTabOrder(alignCenterVerToolButton, alignBottomInToolButton);
        QWidget::setTabOrder(alignBottomInToolButton, alignBottomOutToolButton);
        QWidget::setTabOrder(alignBottomOutToolButton, distributeLeftToolButton);
        QWidget::setTabOrder(distributeLeftToolButton, distributeCenterHToolButton);
        QWidget::setTabOrder(distributeCenterHToolButton, distributeRightToolButton);
        QWidget::setTabOrder(distributeRightToolButton, distributeDistHToolButton);
        QWidget::setTabOrder(distributeDistHToolButton, distributeBottomToolButton);
        QWidget::setTabOrder(distributeBottomToolButton, distributeCenterVToolButton);
        QWidget::setTabOrder(distributeCenterVToolButton, distributeTopToolButton);
        QWidget::setTabOrder(distributeTopToolButton, distributeDistVToolButton);
        QWidget::setTabOrder(distributeDistVToolButton, distributeAcrossPageToolButton);
        QWidget::setTabOrder(distributeAcrossPageToolButton, distributeDownPageToolButton);
        QWidget::setTabOrder(distributeDownPageToolButton, distributeAcrossMarginsToolButton);
        QWidget::setTabOrder(distributeAcrossMarginsToolButton, distributeDownMarginsToolButton);
        QWidget::setTabOrder(distributeDownMarginsToolButton, distributeDistValueHToolButton);
        QWidget::setTabOrder(distributeDistValueHToolButton, distributeDistValueVToolButton);
        QWidget::setTabOrder(distributeDistValueVToolButton, distributeDistSpinBox);
        QWidget::setTabOrder(distributeDistSpinBox, reverseDistributionCheckBox);
        QWidget::setTabOrder(reverseDistributionCheckBox, swapLeftToolButton);
        QWidget::setTabOrder(swapLeftToolButton, swapRightToolButton);
        QWidget::setTabOrder(swapRightToolButton, tabWidget);

        retranslateUi(AlignDistribute);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(AlignDistribute);
    } // setupUi

    void retranslateUi(ScDockPalette *AlignDistribute)
    {
        AlignDistribute->setWindowTitle(QCoreApplication::translate("AlignDistribute", "Align and Distribute", nullptr));
        alignGuideLabel->setText(QCoreApplication::translate("AlignDistribute", "&Selected Guide:", nullptr));
        alignRelativeToLabel->setText(QCoreApplication::translate("AlignDistribute", "&Relative To:", nullptr));
        alignMoveOrResizeLabel->setText(QCoreApplication::translate("AlignDistribute", "&Align Sides By:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabAlign), QCoreApplication::translate("AlignDistribute", "Align", nullptr));
        distributeDistLabel->setText(QCoreApplication::translate("AlignDistribute", "&Distance:", nullptr));
        reverseDistributionCheckBox->setText(QCoreApplication::translate("AlignDistribute", "Reverse Distribution", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabDistribute), QCoreApplication::translate("AlignDistribute", "Distribute", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabSwap), QCoreApplication::translate("AlignDistribute", "Swap", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AlignDistribute: public Ui_AlignDistribute {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ALIGNDISTRIBUTE_H
