/********************************************************************************
** Form generated from reading UI file 'propertiespalette_imagebase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTIESPALETTE_IMAGEBASE_H
#define UI_PROPERTIESPALETTE_IMAGEBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/linkbutton.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_PropertiesPalette_ImageBase
{
public:
    QVBoxLayout *verticalLayout;
    QGridLayout *gridLayout;
    QLabel *xposImgLabel;
    ScrSpinBox *imageXOffsetSpinBox;
    QLabel *imageRotationLabel;
    ScrSpinBox *imageRotation;
    QLabel *imagePageNumberLabel;
    QLabel *yposImgLabel;
    ScrSpinBox *imageYOffsetSpinBox;
    ScrSpinBox *imagePageNumber;
    QGroupBox *scalingGroupBox;
    QGridLayout *gridLayout_2;
    LinkButton *keepImageWHRatioButton;
    QLabel *xscaleLabel;
    ScrSpinBox *imageXScaleSpinBox;
    QLabel *yscaleLabel;
    QLabel *imgDPIXLabel;
    ScrSpinBox *imgDpiX;
    QLabel *imgDPIYLabel;
    ScrSpinBox *imgDpiY;
    LinkButton *keepImageDPIRatioButton;
    ScrSpinBox *imageYScaleSpinBox;
    QRadioButton *frameScale;
    QRadioButton *freeScale;
    QCheckBox *cbProportional;
    QGroupBox *colorMgmtGroup;
    QGridLayout *gridLayout_4;
    QLabel *inputProfLabel;
    QComboBox *inputProfiles;
    QLabel *renderIntentLabel;
    QComboBox *renderIntent;
    QGroupBox *pdfCompressionGroup;
    QGridLayout *gridLayout_3;
    QLabel *compressionMethodLabel;
    QComboBox *compressionMethod;
    QLabel *compressionQualityLabel;
    QComboBox *compressionQuality;
    QHBoxLayout *horizontalLayout;
    QToolButton *imgEffectsButton;
    QToolButton *imgExtProperties;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *PropertiesPalette_ImageBase)
    {
        if (PropertiesPalette_ImageBase->objectName().isEmpty())
            PropertiesPalette_ImageBase->setObjectName(QString::fromUtf8("PropertiesPalette_ImageBase"));
        PropertiesPalette_ImageBase->resize(286, 617);
        verticalLayout = new QVBoxLayout(PropertiesPalette_ImageBase);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(3);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        xposImgLabel = new QLabel(PropertiesPalette_ImageBase);
        xposImgLabel->setObjectName(QString::fromUtf8("xposImgLabel"));

        gridLayout->addWidget(xposImgLabel, 2, 0, 1, 1);

        imageXOffsetSpinBox = new ScrSpinBox(PropertiesPalette_ImageBase);
        imageXOffsetSpinBox->setObjectName(QString::fromUtf8("imageXOffsetSpinBox"));
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(imageXOffsetSpinBox->sizePolicy().hasHeightForWidth());
        imageXOffsetSpinBox->setSizePolicy(sizePolicy);

        gridLayout->addWidget(imageXOffsetSpinBox, 2, 1, 1, 1);

        imageRotationLabel = new QLabel(PropertiesPalette_ImageBase);
        imageRotationLabel->setObjectName(QString::fromUtf8("imageRotationLabel"));

        gridLayout->addWidget(imageRotationLabel, 4, 0, 1, 1);

        imageRotation = new ScrSpinBox(PropertiesPalette_ImageBase);
        imageRotation->setObjectName(QString::fromUtf8("imageRotation"));
        sizePolicy.setHeightForWidth(imageRotation->sizePolicy().hasHeightForWidth());
        imageRotation->setSizePolicy(sizePolicy);

        gridLayout->addWidget(imageRotation, 4, 1, 1, 1);

        imagePageNumberLabel = new QLabel(PropertiesPalette_ImageBase);
        imagePageNumberLabel->setObjectName(QString::fromUtf8("imagePageNumberLabel"));

        gridLayout->addWidget(imagePageNumberLabel, 1, 0, 1, 1);

        yposImgLabel = new QLabel(PropertiesPalette_ImageBase);
        yposImgLabel->setObjectName(QString::fromUtf8("yposImgLabel"));

        gridLayout->addWidget(yposImgLabel, 3, 0, 1, 1);

        imageYOffsetSpinBox = new ScrSpinBox(PropertiesPalette_ImageBase);
        imageYOffsetSpinBox->setObjectName(QString::fromUtf8("imageYOffsetSpinBox"));
        sizePolicy.setHeightForWidth(imageYOffsetSpinBox->sizePolicy().hasHeightForWidth());
        imageYOffsetSpinBox->setSizePolicy(sizePolicy);

        gridLayout->addWidget(imageYOffsetSpinBox, 3, 1, 1, 1);

        imagePageNumber = new ScrSpinBox(PropertiesPalette_ImageBase);
        imagePageNumber->setObjectName(QString::fromUtf8("imagePageNumber"));
        sizePolicy.setHeightForWidth(imagePageNumber->sizePolicy().hasHeightForWidth());
        imagePageNumber->setSizePolicy(sizePolicy);

        gridLayout->addWidget(imagePageNumber, 1, 1, 1, 1);


        verticalLayout->addLayout(gridLayout);

        scalingGroupBox = new QGroupBox(PropertiesPalette_ImageBase);
        scalingGroupBox->setObjectName(QString::fromUtf8("scalingGroupBox"));
        gridLayout_2 = new QGridLayout(scalingGroupBox);
        gridLayout_2->setSpacing(3);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(6, 6, 6, 6);
        keepImageWHRatioButton = new LinkButton(scalingGroupBox);
        keepImageWHRatioButton->setObjectName(QString::fromUtf8("keepImageWHRatioButton"));
        keepImageWHRatioButton->setMaximumSize(QSize(15, 32767));
        keepImageWHRatioButton->setCheckable(true);
        keepImageWHRatioButton->setChecked(true);
        keepImageWHRatioButton->setAutoRaise(true);

        gridLayout_2->addWidget(keepImageWHRatioButton, 2, 2, 2, 1);

        xscaleLabel = new QLabel(scalingGroupBox);
        xscaleLabel->setObjectName(QString::fromUtf8("xscaleLabel"));

        gridLayout_2->addWidget(xscaleLabel, 2, 0, 1, 1);

        imageXScaleSpinBox = new ScrSpinBox(scalingGroupBox);
        imageXScaleSpinBox->setObjectName(QString::fromUtf8("imageXScaleSpinBox"));
        sizePolicy.setHeightForWidth(imageXScaleSpinBox->sizePolicy().hasHeightForWidth());
        imageXScaleSpinBox->setSizePolicy(sizePolicy);

        gridLayout_2->addWidget(imageXScaleSpinBox, 2, 1, 1, 1);

        yscaleLabel = new QLabel(scalingGroupBox);
        yscaleLabel->setObjectName(QString::fromUtf8("yscaleLabel"));

        gridLayout_2->addWidget(yscaleLabel, 3, 0, 1, 1);

        imgDPIXLabel = new QLabel(scalingGroupBox);
        imgDPIXLabel->setObjectName(QString::fromUtf8("imgDPIXLabel"));

        gridLayout_2->addWidget(imgDPIXLabel, 4, 0, 1, 1);

        imgDpiX = new ScrSpinBox(scalingGroupBox);
        imgDpiX->setObjectName(QString::fromUtf8("imgDpiX"));
        sizePolicy.setHeightForWidth(imgDpiX->sizePolicy().hasHeightForWidth());
        imgDpiX->setSizePolicy(sizePolicy);

        gridLayout_2->addWidget(imgDpiX, 4, 1, 1, 1);

        imgDPIYLabel = new QLabel(scalingGroupBox);
        imgDPIYLabel->setObjectName(QString::fromUtf8("imgDPIYLabel"));

        gridLayout_2->addWidget(imgDPIYLabel, 5, 0, 1, 1);

        imgDpiY = new ScrSpinBox(scalingGroupBox);
        imgDpiY->setObjectName(QString::fromUtf8("imgDpiY"));
        sizePolicy.setHeightForWidth(imgDpiY->sizePolicy().hasHeightForWidth());
        imgDpiY->setSizePolicy(sizePolicy);

        gridLayout_2->addWidget(imgDpiY, 5, 1, 1, 1);

        keepImageDPIRatioButton = new LinkButton(scalingGroupBox);
        keepImageDPIRatioButton->setObjectName(QString::fromUtf8("keepImageDPIRatioButton"));
        keepImageDPIRatioButton->setMaximumSize(QSize(15, 32767));
        keepImageDPIRatioButton->setCheckable(true);
        keepImageDPIRatioButton->setChecked(true);
        keepImageDPIRatioButton->setAutoRaise(true);

        gridLayout_2->addWidget(keepImageDPIRatioButton, 4, 2, 2, 1);

        imageYScaleSpinBox = new ScrSpinBox(scalingGroupBox);
        imageYScaleSpinBox->setObjectName(QString::fromUtf8("imageYScaleSpinBox"));
        sizePolicy.setHeightForWidth(imageYScaleSpinBox->sizePolicy().hasHeightForWidth());
        imageYScaleSpinBox->setSizePolicy(sizePolicy);

        gridLayout_2->addWidget(imageYScaleSpinBox, 3, 1, 1, 1);

        frameScale = new QRadioButton(scalingGroupBox);
        frameScale->setObjectName(QString::fromUtf8("frameScale"));

        gridLayout_2->addWidget(frameScale, 6, 0, 1, 3);

        freeScale = new QRadioButton(scalingGroupBox);
        freeScale->setObjectName(QString::fromUtf8("freeScale"));

        gridLayout_2->addWidget(freeScale, 0, 0, 1, 3);

        cbProportional = new QCheckBox(scalingGroupBox);
        cbProportional->setObjectName(QString::fromUtf8("cbProportional"));

        gridLayout_2->addWidget(cbProportional, 7, 1, 1, 1);


        verticalLayout->addWidget(scalingGroupBox);

        colorMgmtGroup = new QGroupBox(PropertiesPalette_ImageBase);
        colorMgmtGroup->setObjectName(QString::fromUtf8("colorMgmtGroup"));
        gridLayout_4 = new QGridLayout(colorMgmtGroup);
        gridLayout_4->setSpacing(3);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        gridLayout_4->setContentsMargins(6, 6, 6, 6);
        inputProfLabel = new QLabel(colorMgmtGroup);
        inputProfLabel->setObjectName(QString::fromUtf8("inputProfLabel"));

        gridLayout_4->addWidget(inputProfLabel, 0, 0, 1, 1);

        inputProfiles = new QComboBox(colorMgmtGroup);
        inputProfiles->setObjectName(QString::fromUtf8("inputProfiles"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(inputProfiles->sizePolicy().hasHeightForWidth());
        inputProfiles->setSizePolicy(sizePolicy1);

        gridLayout_4->addWidget(inputProfiles, 0, 1, 1, 1);

        renderIntentLabel = new QLabel(colorMgmtGroup);
        renderIntentLabel->setObjectName(QString::fromUtf8("renderIntentLabel"));

        gridLayout_4->addWidget(renderIntentLabel, 1, 0, 1, 1);

        renderIntent = new QComboBox(colorMgmtGroup);
        renderIntent->setObjectName(QString::fromUtf8("renderIntent"));
        sizePolicy1.setHeightForWidth(renderIntent->sizePolicy().hasHeightForWidth());
        renderIntent->setSizePolicy(sizePolicy1);

        gridLayout_4->addWidget(renderIntent, 1, 1, 1, 1);


        verticalLayout->addWidget(colorMgmtGroup);

        pdfCompressionGroup = new QGroupBox(PropertiesPalette_ImageBase);
        pdfCompressionGroup->setObjectName(QString::fromUtf8("pdfCompressionGroup"));
        gridLayout_3 = new QGridLayout(pdfCompressionGroup);
        gridLayout_3->setSpacing(3);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        gridLayout_3->setContentsMargins(6, 6, 6, 6);
        compressionMethodLabel = new QLabel(pdfCompressionGroup);
        compressionMethodLabel->setObjectName(QString::fromUtf8("compressionMethodLabel"));

        gridLayout_3->addWidget(compressionMethodLabel, 0, 0, 1, 1);

        compressionMethod = new QComboBox(pdfCompressionGroup);
        compressionMethod->setObjectName(QString::fromUtf8("compressionMethod"));
        sizePolicy1.setHeightForWidth(compressionMethod->sizePolicy().hasHeightForWidth());
        compressionMethod->setSizePolicy(sizePolicy1);

        gridLayout_3->addWidget(compressionMethod, 0, 1, 1, 1);

        compressionQualityLabel = new QLabel(pdfCompressionGroup);
        compressionQualityLabel->setObjectName(QString::fromUtf8("compressionQualityLabel"));

        gridLayout_3->addWidget(compressionQualityLabel, 1, 0, 1, 1);

        compressionQuality = new QComboBox(pdfCompressionGroup);
        compressionQuality->setObjectName(QString::fromUtf8("compressionQuality"));
        sizePolicy1.setHeightForWidth(compressionQuality->sizePolicy().hasHeightForWidth());
        compressionQuality->setSizePolicy(sizePolicy1);

        gridLayout_3->addWidget(compressionQuality, 1, 1, 1, 1);


        verticalLayout->addWidget(pdfCompressionGroup);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(3);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        imgEffectsButton = new QToolButton(PropertiesPalette_ImageBase);
        imgEffectsButton->setObjectName(QString::fromUtf8("imgEffectsButton"));
        QSizePolicy sizePolicy2(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(imgEffectsButton->sizePolicy().hasHeightForWidth());
        imgEffectsButton->setSizePolicy(sizePolicy2);

        horizontalLayout->addWidget(imgEffectsButton);

        imgExtProperties = new QToolButton(PropertiesPalette_ImageBase);
        imgExtProperties->setObjectName(QString::fromUtf8("imgExtProperties"));
        sizePolicy2.setHeightForWidth(imgExtProperties->sizePolicy().hasHeightForWidth());
        imgExtProperties->setSizePolicy(sizePolicy2);

        horizontalLayout->addWidget(imgExtProperties);


        verticalLayout->addLayout(horizontalLayout);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        QWidget::setTabOrder(imagePageNumber, imageXOffsetSpinBox);
        QWidget::setTabOrder(imageXOffsetSpinBox, imageYOffsetSpinBox);
        QWidget::setTabOrder(imageYOffsetSpinBox, imageRotation);
        QWidget::setTabOrder(imageRotation, freeScale);
        QWidget::setTabOrder(freeScale, imageXScaleSpinBox);
        QWidget::setTabOrder(imageXScaleSpinBox, imageYScaleSpinBox);
        QWidget::setTabOrder(imageYScaleSpinBox, imgDpiX);
        QWidget::setTabOrder(imgDpiX, imgDpiY);
        QWidget::setTabOrder(imgDpiY, keepImageWHRatioButton);
        QWidget::setTabOrder(keepImageWHRatioButton, keepImageDPIRatioButton);
        QWidget::setTabOrder(keepImageDPIRatioButton, frameScale);
        QWidget::setTabOrder(frameScale, cbProportional);
        QWidget::setTabOrder(cbProportional, inputProfiles);
        QWidget::setTabOrder(inputProfiles, renderIntent);
        QWidget::setTabOrder(renderIntent, compressionMethod);
        QWidget::setTabOrder(compressionMethod, compressionQuality);
        QWidget::setTabOrder(compressionQuality, imgEffectsButton);
        QWidget::setTabOrder(imgEffectsButton, imgExtProperties);

        retranslateUi(PropertiesPalette_ImageBase);

        QMetaObject::connectSlotsByName(PropertiesPalette_ImageBase);
    } // setupUi

    void retranslateUi(QWidget *PropertiesPalette_ImageBase)
    {
        xposImgLabel->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "&X-Pos:", nullptr));
#if QT_CONFIG(tooltip)
        imageXOffsetSpinBox->setToolTip(QCoreApplication::translate("PropertiesPalette_ImageBase", "Horizontal offset of image within frame", nullptr));
#endif // QT_CONFIG(tooltip)
        imageRotationLabel->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "Rotation:", nullptr));
        imagePageNumberLabel->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "&Page Number:", nullptr));
        yposImgLabel->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "&Y-Pos:", nullptr));
#if QT_CONFIG(tooltip)
        imageYOffsetSpinBox->setToolTip(QCoreApplication::translate("PropertiesPalette_ImageBase", "Vertical offset of image within frame", nullptr));
#endif // QT_CONFIG(tooltip)
        imagePageNumber->setSpecialValueText(QCoreApplication::translate("PropertiesPalette_ImageBase", "Auto", nullptr));
        scalingGroupBox->setTitle(QCoreApplication::translate("PropertiesPalette_ImageBase", "Scaling", nullptr));
#if QT_CONFIG(tooltip)
        keepImageWHRatioButton->setToolTip(QCoreApplication::translate("PropertiesPalette_ImageBase", "Keep the X and Y scaling the same", nullptr));
#endif // QT_CONFIG(tooltip)
        xscaleLabel->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "X-Sc&ale:", nullptr));
#if QT_CONFIG(tooltip)
        imageXScaleSpinBox->setToolTip(QCoreApplication::translate("PropertiesPalette_ImageBase", "Resize the image horizontally", nullptr));
#endif // QT_CONFIG(tooltip)
        yscaleLabel->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "Y-Scal&e:", nullptr));
        imgDPIXLabel->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "Actual X-DPI:", nullptr));
#if QT_CONFIG(tooltip)
        imgDpiX->setToolTip(QCoreApplication::translate("PropertiesPalette_ImageBase", "Effective horizontal DPI of the image after scaling", nullptr));
#endif // QT_CONFIG(tooltip)
        imgDPIYLabel->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "Actual Y-DPI:", nullptr));
#if QT_CONFIG(tooltip)
        imgDpiY->setToolTip(QCoreApplication::translate("PropertiesPalette_ImageBase", "Effective vertical DPI of the image after scaling", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        imageYScaleSpinBox->setToolTip(QCoreApplication::translate("PropertiesPalette_ImageBase", "Resize the image vertically", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        frameScale->setToolTip(QCoreApplication::translate("PropertiesPalette_ImageBase", "Make the image fit within the size of the frame", nullptr));
#endif // QT_CONFIG(tooltip)
        frameScale->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "&To Frame Size", nullptr));
#if QT_CONFIG(tooltip)
        freeScale->setToolTip(QCoreApplication::translate("PropertiesPalette_ImageBase", "Allow the image to be a different size to the frame", nullptr));
#endif // QT_CONFIG(tooltip)
        freeScale->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "&Free Scaling", nullptr));
#if QT_CONFIG(tooltip)
        cbProportional->setToolTip(QCoreApplication::translate("PropertiesPalette_ImageBase", "Use image proportions rather than those of the frame", nullptr));
#endif // QT_CONFIG(tooltip)
        cbProportional->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "Proportional", nullptr));
        colorMgmtGroup->setTitle(QCoreApplication::translate("PropertiesPalette_ImageBase", "Color Management", nullptr));
        inputProfLabel->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "Input Profile:", nullptr));
#if QT_CONFIG(tooltip)
        inputProfiles->setToolTip(QCoreApplication::translate("PropertiesPalette_ImageBase", "Source profile of the image", nullptr));
#endif // QT_CONFIG(tooltip)
        renderIntentLabel->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "Rendering Intent:", nullptr));
#if QT_CONFIG(tooltip)
        renderIntent->setToolTip(QCoreApplication::translate("PropertiesPalette_ImageBase", "Rendering intent for the image", nullptr));
#endif // QT_CONFIG(tooltip)
        pdfCompressionGroup->setTitle(QCoreApplication::translate("PropertiesPalette_ImageBase", "PDF Compression", nullptr));
        compressionMethodLabel->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "Method:", nullptr));
#if QT_CONFIG(tooltip)
        compressionMethod->setToolTip(QCoreApplication::translate("PropertiesPalette_ImageBase", "Compression method used in PDF export for the image", nullptr));
#endif // QT_CONFIG(tooltip)
        compressionQualityLabel->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "Quality:", nullptr));
#if QT_CONFIG(tooltip)
        compressionQuality->setToolTip(QCoreApplication::translate("PropertiesPalette_ImageBase", "Compression quality used in PDF export for the image", nullptr));
#endif // QT_CONFIG(tooltip)
        imgEffectsButton->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "Image Effects", nullptr));
        imgExtProperties->setText(QCoreApplication::translate("PropertiesPalette_ImageBase", "Extended Properties", nullptr));
        (void)PropertiesPalette_ImageBase;
    } // retranslateUi

};

namespace Ui {
    class PropertiesPalette_ImageBase: public Ui_PropertiesPalette_ImageBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTIESPALETTE_IMAGEBASE_H
