/*
Use designer to update source ui file. All manual changes in cpp/h files will be lost
*/

/********************************************************************************
** Form generated from reading UI file 'guidemanager.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GUIDEMANAGER_H
#define UI_GUIDEMANAGER_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QWidget>
#include <ui/scrspinbox.h>
#include "guidesview.h"
#include "ui/scrpalettebase.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_GuideManager
{
public:
    QGridLayout *gridLayout;
    QTabWidget *tabWidget;
    QWidget *standardPage;
    QGridLayout *gridLayout1;
    QHBoxLayout *hboxLayout;
    QGroupBox *horizontalGroupBox;
    QGridLayout *gridLayout2;
    GuidesView *horizontalView;
    QPushButton *addHorButton;
    QPushButton *delHorButton;
    QGroupBox *verticalGroupBox;
    QGridLayout *gridLayout3;
    GuidesView *verticalView;
    QPushButton *addVerButton;
    QPushButton *delVerButton;
    QCheckBox *lockCheck;
    QPushButton *applyToAllStdButton;
    QWidget *autoPage;
    QGridLayout *gridLayout4;
    QPushButton *applyToAllAutoButton;
    QGroupBox *horizontalAutoGroup;
    QGridLayout *gridLayout5;
    QLabel *textLabel1;
    ScrSpinBox *horizontalAutoCountSpin;
    QCheckBox *horizontalAutoGapCheck;
    ScrSpinBox *horizontalAutoGapSpin;
    QGroupBox *groupBox;
    QGridLayout *gridLayout6;
    QRadioButton *horizontalPageAutoButton;
    QRadioButton *horizontalMarginsAutoButton;
    QRadioButton *horizontalSelectionAutoButton;
    QGroupBox *verticalAutoGroup;
    QGridLayout *gridLayout7;
    QLabel *textLabel1_2;
    ScrSpinBox *verticalAutoCountSpin;
    QCheckBox *verticalAutoGapCheck;
    ScrSpinBox *verticalAutoGapSpin;
    QGroupBox *groupBox_2;
    QGridLayout *gridLayout8;
    QRadioButton *verticalPageAutoButton;
    QRadioButton *verticalMarginsAutoButton;
    QRadioButton *verticalSelectionAutoButton;
    QSpacerItem *verticalSpacer;
    QWidget *miscPage;
    QGridLayout *gridLayout9;
    QPushButton *deletePageButton;
    QPushButton *deleteAllGuides;
    QSpacerItem *spacerItem;

    void setupUi(ScrPaletteBase *GuideManager)
    {
        if (GuideManager->objectName().isEmpty())
            GuideManager->setObjectName(QString::fromUtf8("GuideManager"));
        GuideManager->resize(428, 368);
        gridLayout = new QGridLayout(GuideManager);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        tabWidget = new QTabWidget(GuideManager);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        standardPage = new QWidget();
        standardPage->setObjectName(QString::fromUtf8("standardPage"));
        gridLayout1 = new QGridLayout(standardPage);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        gridLayout1->setContentsMargins(9, 9, 9, 9);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        horizontalGroupBox = new QGroupBox(standardPage);
        horizontalGroupBox->setObjectName(QString::fromUtf8("horizontalGroupBox"));
        gridLayout2 = new QGridLayout(horizontalGroupBox);
        gridLayout2->setObjectName(QString::fromUtf8("gridLayout2"));
        horizontalView = new GuidesView(horizontalGroupBox);
        horizontalView->setObjectName(QString::fromUtf8("horizontalView"));
        horizontalView->setAlternatingRowColors(true);
        horizontalView->setSelectionBehavior(QAbstractItemView::SelectRows);

        gridLayout2->addWidget(horizontalView, 0, 0, 1, 2);

        addHorButton = new QPushButton(horizontalGroupBox);
        addHorButton->setObjectName(QString::fromUtf8("addHorButton"));

        gridLayout2->addWidget(addHorButton, 1, 0, 1, 1);

        delHorButton = new QPushButton(horizontalGroupBox);
        delHorButton->setObjectName(QString::fromUtf8("delHorButton"));

        gridLayout2->addWidget(delHorButton, 1, 1, 1, 1);


        hboxLayout->addWidget(horizontalGroupBox);

        verticalGroupBox = new QGroupBox(standardPage);
        verticalGroupBox->setObjectName(QString::fromUtf8("verticalGroupBox"));
        gridLayout3 = new QGridLayout(verticalGroupBox);
        gridLayout3->setObjectName(QString::fromUtf8("gridLayout3"));
        verticalView = new GuidesView(verticalGroupBox);
        verticalView->setObjectName(QString::fromUtf8("verticalView"));
        verticalView->setAlternatingRowColors(true);
        verticalView->setSelectionBehavior(QAbstractItemView::SelectRows);

        gridLayout3->addWidget(verticalView, 0, 0, 1, 2);

        addVerButton = new QPushButton(verticalGroupBox);
        addVerButton->setObjectName(QString::fromUtf8("addVerButton"));

        gridLayout3->addWidget(addVerButton, 1, 0, 1, 1);

        delVerButton = new QPushButton(verticalGroupBox);
        delVerButton->setObjectName(QString::fromUtf8("delVerButton"));

        gridLayout3->addWidget(delVerButton, 1, 1, 1, 1);


        hboxLayout->addWidget(verticalGroupBox);


        gridLayout1->addLayout(hboxLayout, 0, 0, 1, 2);

        lockCheck = new QCheckBox(standardPage);
        lockCheck->setObjectName(QString::fromUtf8("lockCheck"));

        gridLayout1->addWidget(lockCheck, 1, 0, 1, 1);

        applyToAllStdButton = new QPushButton(standardPage);
        applyToAllStdButton->setObjectName(QString::fromUtf8("applyToAllStdButton"));

        gridLayout1->addWidget(applyToAllStdButton, 1, 1, 1, 1);

        tabWidget->addTab(standardPage, QString());
        autoPage = new QWidget();
        autoPage->setObjectName(QString::fromUtf8("autoPage"));
        gridLayout4 = new QGridLayout(autoPage);
        gridLayout4->setObjectName(QString::fromUtf8("gridLayout4"));
        gridLayout4->setContentsMargins(9, 9, 9, 9);
        applyToAllAutoButton = new QPushButton(autoPage);
        applyToAllAutoButton->setObjectName(QString::fromUtf8("applyToAllAutoButton"));

        gridLayout4->addWidget(applyToAllAutoButton, 2, 0, 1, 2);

        horizontalAutoGroup = new QGroupBox(autoPage);
        horizontalAutoGroup->setObjectName(QString::fromUtf8("horizontalAutoGroup"));
        gridLayout5 = new QGridLayout(horizontalAutoGroup);
        gridLayout5->setObjectName(QString::fromUtf8("gridLayout5"));
        textLabel1 = new QLabel(horizontalAutoGroup);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));
        textLabel1->setWordWrap(false);

        gridLayout5->addWidget(textLabel1, 0, 0, 1, 1);

        horizontalAutoCountSpin = new ScrSpinBox(horizontalAutoGroup);
        horizontalAutoCountSpin->setObjectName(QString::fromUtf8("horizontalAutoCountSpin"));
        horizontalAutoCountSpin->setMinimum(0);
        horizontalAutoCountSpin->setValue(0);

        gridLayout5->addWidget(horizontalAutoCountSpin, 0, 1, 1, 1);

        horizontalAutoGapCheck = new QCheckBox(horizontalAutoGroup);
        horizontalAutoGapCheck->setObjectName(QString::fromUtf8("horizontalAutoGapCheck"));

        gridLayout5->addWidget(horizontalAutoGapCheck, 1, 0, 1, 2);

        horizontalAutoGapSpin = new ScrSpinBox(horizontalAutoGroup);
        horizontalAutoGapSpin->setObjectName(QString::fromUtf8("horizontalAutoGapSpin"));

        gridLayout5->addWidget(horizontalAutoGapSpin, 2, 0, 1, 2);

        groupBox = new QGroupBox(horizontalAutoGroup);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        gridLayout6 = new QGridLayout(groupBox);
        gridLayout6->setObjectName(QString::fromUtf8("gridLayout6"));
        horizontalPageAutoButton = new QRadioButton(groupBox);
        horizontalPageAutoButton->setObjectName(QString::fromUtf8("horizontalPageAutoButton"));
        horizontalPageAutoButton->setChecked(true);

        gridLayout6->addWidget(horizontalPageAutoButton, 0, 0, 1, 1);

        horizontalMarginsAutoButton = new QRadioButton(groupBox);
        horizontalMarginsAutoButton->setObjectName(QString::fromUtf8("horizontalMarginsAutoButton"));

        gridLayout6->addWidget(horizontalMarginsAutoButton, 1, 0, 1, 1);

        horizontalSelectionAutoButton = new QRadioButton(groupBox);
        horizontalSelectionAutoButton->setObjectName(QString::fromUtf8("horizontalSelectionAutoButton"));

        gridLayout6->addWidget(horizontalSelectionAutoButton, 2, 0, 1, 1);


        gridLayout5->addWidget(groupBox, 3, 0, 1, 2);


        gridLayout4->addWidget(horizontalAutoGroup, 0, 0, 1, 1);

        verticalAutoGroup = new QGroupBox(autoPage);
        verticalAutoGroup->setObjectName(QString::fromUtf8("verticalAutoGroup"));
        gridLayout7 = new QGridLayout(verticalAutoGroup);
        gridLayout7->setObjectName(QString::fromUtf8("gridLayout7"));
        textLabel1_2 = new QLabel(verticalAutoGroup);
        textLabel1_2->setObjectName(QString::fromUtf8("textLabel1_2"));
        textLabel1_2->setWordWrap(false);

        gridLayout7->addWidget(textLabel1_2, 0, 0, 1, 1);

        verticalAutoCountSpin = new ScrSpinBox(verticalAutoGroup);
        verticalAutoCountSpin->setObjectName(QString::fromUtf8("verticalAutoCountSpin"));
        verticalAutoCountSpin->setMinimum(0);
        verticalAutoCountSpin->setValue(0);

        gridLayout7->addWidget(verticalAutoCountSpin, 0, 1, 1, 1);

        verticalAutoGapCheck = new QCheckBox(verticalAutoGroup);
        verticalAutoGapCheck->setObjectName(QString::fromUtf8("verticalAutoGapCheck"));

        gridLayout7->addWidget(verticalAutoGapCheck, 1, 0, 1, 2);

        verticalAutoGapSpin = new ScrSpinBox(verticalAutoGroup);
        verticalAutoGapSpin->setObjectName(QString::fromUtf8("verticalAutoGapSpin"));

        gridLayout7->addWidget(verticalAutoGapSpin, 2, 0, 1, 2);

        groupBox_2 = new QGroupBox(verticalAutoGroup);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        gridLayout8 = new QGridLayout(groupBox_2);
        gridLayout8->setObjectName(QString::fromUtf8("gridLayout8"));
        verticalPageAutoButton = new QRadioButton(groupBox_2);
        verticalPageAutoButton->setObjectName(QString::fromUtf8("verticalPageAutoButton"));
        verticalPageAutoButton->setChecked(true);

        gridLayout8->addWidget(verticalPageAutoButton, 0, 0, 1, 1);

        verticalMarginsAutoButton = new QRadioButton(groupBox_2);
        verticalMarginsAutoButton->setObjectName(QString::fromUtf8("verticalMarginsAutoButton"));

        gridLayout8->addWidget(verticalMarginsAutoButton, 1, 0, 1, 1);

        verticalSelectionAutoButton = new QRadioButton(groupBox_2);
        verticalSelectionAutoButton->setObjectName(QString::fromUtf8("verticalSelectionAutoButton"));

        gridLayout8->addWidget(verticalSelectionAutoButton, 2, 0, 1, 1);


        gridLayout7->addWidget(groupBox_2, 3, 0, 1, 2);


        gridLayout4->addWidget(verticalAutoGroup, 0, 1, 1, 1);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout4->addItem(verticalSpacer, 1, 0, 1, 1);

        tabWidget->addTab(autoPage, QString());
        miscPage = new QWidget();
        miscPage->setObjectName(QString::fromUtf8("miscPage"));
        gridLayout9 = new QGridLayout(miscPage);
        gridLayout9->setObjectName(QString::fromUtf8("gridLayout9"));
        gridLayout9->setContentsMargins(9, 9, 9, 9);
        deletePageButton = new QPushButton(miscPage);
        deletePageButton->setObjectName(QString::fromUtf8("deletePageButton"));

        gridLayout9->addWidget(deletePageButton, 0, 0, 1, 1);

        deleteAllGuides = new QPushButton(miscPage);
        deleteAllGuides->setObjectName(QString::fromUtf8("deleteAllGuides"));

        gridLayout9->addWidget(deleteAllGuides, 1, 0, 1, 1);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout9->addItem(spacerItem, 2, 0, 1, 1);

        tabWidget->addTab(miscPage, QString());

        gridLayout->addWidget(tabWidget, 0, 0, 1, 1);

#if QT_CONFIG(shortcut)
        textLabel1->setBuddy(horizontalAutoCountSpin);
        textLabel1_2->setBuddy(verticalAutoCountSpin);
#endif // QT_CONFIG(shortcut)

        retranslateUi(GuideManager);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(GuideManager);
    } // setupUi

    void retranslateUi(ScrPaletteBase *GuideManager)
    {
        GuideManager->setWindowTitle(QCoreApplication::translate("GuideManager", "Guide Manager", nullptr));
        horizontalGroupBox->setTitle(QCoreApplication::translate("GuideManager", "Horizontals", nullptr));
#if QT_CONFIG(tooltip)
        addHorButton->setToolTip(QCoreApplication::translate("GuideManager", "Add a new horizontal guide", nullptr));
#endif // QT_CONFIG(tooltip)
        addHorButton->setText(QCoreApplication::translate("GuideManager", "&Add", nullptr));
#if QT_CONFIG(shortcut)
        addHorButton->setShortcut(QCoreApplication::translate("GuideManager", "Alt+A", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        delHorButton->setToolTip(QCoreApplication::translate("GuideManager", "Delete the selected horizontal guide", nullptr));
#endif // QT_CONFIG(tooltip)
        delHorButton->setText(QCoreApplication::translate("GuideManager", "D&elete", nullptr));
#if QT_CONFIG(shortcut)
        delHorButton->setShortcut(QCoreApplication::translate("GuideManager", "Alt+E", nullptr));
#endif // QT_CONFIG(shortcut)
        verticalGroupBox->setTitle(QCoreApplication::translate("GuideManager", "Verticals", nullptr));
#if QT_CONFIG(tooltip)
        addVerButton->setToolTip(QCoreApplication::translate("GuideManager", "Add a new vertical guide", nullptr));
#endif // QT_CONFIG(tooltip)
        addVerButton->setText(QCoreApplication::translate("GuideManager", "A&dd", nullptr));
#if QT_CONFIG(shortcut)
        addVerButton->setShortcut(QCoreApplication::translate("GuideManager", "Alt+D", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        delVerButton->setToolTip(QCoreApplication::translate("GuideManager", "Delete the selected vertical guide", nullptr));
#endif // QT_CONFIG(tooltip)
        delVerButton->setText(QCoreApplication::translate("GuideManager", "De&lete", nullptr));
#if QT_CONFIG(shortcut)
        delVerButton->setShortcut(QCoreApplication::translate("GuideManager", "Alt+L", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        lockCheck->setToolTip(QCoreApplication::translate("GuideManager", "Lock the guides", nullptr));
#endif // QT_CONFIG(tooltip)
        lockCheck->setText(QCoreApplication::translate("GuideManager", "&Lock Guides", nullptr));
#if QT_CONFIG(shortcut)
        lockCheck->setShortcut(QCoreApplication::translate("GuideManager", "Alt+L", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        applyToAllStdButton->setToolTip(QCoreApplication::translate("GuideManager", "Apply to all pages", nullptr));
#endif // QT_CONFIG(tooltip)
        applyToAllStdButton->setText(QCoreApplication::translate("GuideManager", "Appl&y to All Pages", nullptr));
#if QT_CONFIG(shortcut)
        applyToAllStdButton->setShortcut(QCoreApplication::translate("GuideManager", "Alt+Y", nullptr));
#endif // QT_CONFIG(shortcut)
        tabWidget->setTabText(tabWidget->indexOf(standardPage), QCoreApplication::translate("GuideManager", "&Single", nullptr));
#if QT_CONFIG(tooltip)
        applyToAllAutoButton->setToolTip(QCoreApplication::translate("GuideManager", "Apply the shown guides to all pages in the document", nullptr));
#endif // QT_CONFIG(tooltip)
        applyToAllAutoButton->setText(QCoreApplication::translate("GuideManager", "Appl&y to All Pages", nullptr));
#if QT_CONFIG(shortcut)
        applyToAllAutoButton->setShortcut(QCoreApplication::translate("GuideManager", "Alt+Y", nullptr));
#endif // QT_CONFIG(shortcut)
        horizontalAutoGroup->setTitle(QCoreApplication::translate("GuideManager", "Horizontals", nullptr));
        textLabel1->setText(QCoreApplication::translate("GuideManager", "&Number:", nullptr));
#if QT_CONFIG(tooltip)
        horizontalAutoCountSpin->setToolTip(QCoreApplication::translate("GuideManager", "Number of horizontal guides to create", nullptr));
#endif // QT_CONFIG(tooltip)
        horizontalAutoCountSpin->setSuffix(QString());
#if QT_CONFIG(tooltip)
        horizontalAutoGapCheck->setToolTip(QCoreApplication::translate("GuideManager", "Create rows with guides, with an additional gap between the rows", nullptr));
#endif // QT_CONFIG(tooltip)
        horizontalAutoGapCheck->setText(QCoreApplication::translate("GuideManager", "U&se Gap:", nullptr));
#if QT_CONFIG(shortcut)
        horizontalAutoGapCheck->setShortcut(QCoreApplication::translate("GuideManager", "Alt+S", nullptr));
#endif // QT_CONFIG(shortcut)
        groupBox->setTitle(QCoreApplication::translate("GuideManager", "Refer to", nullptr));
#if QT_CONFIG(tooltip)
        horizontalPageAutoButton->setToolTip(QCoreApplication::translate("GuideManager", "Create the selected number of horizontal guides relative to the current page", nullptr));
#endif // QT_CONFIG(tooltip)
        horizontalPageAutoButton->setText(QCoreApplication::translate("GuideManager", "&Page", nullptr));
#if QT_CONFIG(shortcut)
        horizontalPageAutoButton->setShortcut(QCoreApplication::translate("GuideManager", "Alt+P", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        horizontalMarginsAutoButton->setToolTip(QCoreApplication::translate("GuideManager", "Create the selected number of horizontal guides relative to the current page's margins", nullptr));
#endif // QT_CONFIG(tooltip)
        horizontalMarginsAutoButton->setText(QCoreApplication::translate("GuideManager", "M&argins", nullptr));
#if QT_CONFIG(shortcut)
        horizontalMarginsAutoButton->setShortcut(QCoreApplication::translate("GuideManager", "Alt+A", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        horizontalSelectionAutoButton->setToolTip(QCoreApplication::translate("GuideManager", "Create the selected number of horizontal guides relative to the current selection of items", nullptr));
#endif // QT_CONFIG(tooltip)
        horizontalSelectionAutoButton->setText(QCoreApplication::translate("GuideManager", "S&election", nullptr));
#if QT_CONFIG(shortcut)
        horizontalSelectionAutoButton->setShortcut(QCoreApplication::translate("GuideManager", "Alt+E", nullptr));
#endif // QT_CONFIG(shortcut)
        verticalAutoGroup->setTitle(QCoreApplication::translate("GuideManager", "Verticals", nullptr));
        textLabel1_2->setText(QCoreApplication::translate("GuideManager", "Nu&mber:", nullptr));
#if QT_CONFIG(tooltip)
        verticalAutoCountSpin->setToolTip(QCoreApplication::translate("GuideManager", "Number of vertical guides to create", nullptr));
#endif // QT_CONFIG(tooltip)
        verticalAutoCountSpin->setSuffix(QString());
#if QT_CONFIG(tooltip)
        verticalAutoGapCheck->setToolTip(QCoreApplication::translate("GuideManager", "Create columns with guides, with an additional gap between the columns", nullptr));
#endif // QT_CONFIG(tooltip)
        verticalAutoGapCheck->setText(QCoreApplication::translate("GuideManager", "Use &Gap:", nullptr));
#if QT_CONFIG(shortcut)
        verticalAutoGapCheck->setShortcut(QCoreApplication::translate("GuideManager", "Alt+G", nullptr));
#endif // QT_CONFIG(shortcut)
        groupBox_2->setTitle(QCoreApplication::translate("GuideManager", "Refer to", nullptr));
#if QT_CONFIG(tooltip)
        verticalPageAutoButton->setToolTip(QCoreApplication::translate("GuideManager", "Create the selected number of vertical guides relative to the current page", nullptr));
#endif // QT_CONFIG(tooltip)
        verticalPageAutoButton->setText(QCoreApplication::translate("GuideManager", "&Page", nullptr));
#if QT_CONFIG(shortcut)
        verticalPageAutoButton->setShortcut(QCoreApplication::translate("GuideManager", "Alt+P", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        verticalMarginsAutoButton->setToolTip(QCoreApplication::translate("GuideManager", "Create the selected number of vertical guides relative to the current page's margins", nullptr));
#endif // QT_CONFIG(tooltip)
        verticalMarginsAutoButton->setText(QCoreApplication::translate("GuideManager", "M&argins", nullptr));
#if QT_CONFIG(shortcut)
        verticalMarginsAutoButton->setShortcut(QCoreApplication::translate("GuideManager", "Alt+A", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        verticalSelectionAutoButton->setToolTip(QCoreApplication::translate("GuideManager", "Create the selected number of vertical guides relative to the current selection of items", nullptr));
#endif // QT_CONFIG(tooltip)
        verticalSelectionAutoButton->setText(QCoreApplication::translate("GuideManager", "S&election", nullptr));
#if QT_CONFIG(shortcut)
        verticalSelectionAutoButton->setShortcut(QCoreApplication::translate("GuideManager", "Alt+E", nullptr));
#endif // QT_CONFIG(shortcut)
        tabWidget->setTabText(tabWidget->indexOf(autoPage), QCoreApplication::translate("GuideManager", "&Column/Row", nullptr));
#if QT_CONFIG(tooltip)
        deletePageButton->setToolTip(QCoreApplication::translate("GuideManager", "Delete all guides from the current page", nullptr));
#endif // QT_CONFIG(tooltip)
        deletePageButton->setText(QCoreApplication::translate("GuideManager", "Delete Guides from Current &Page", nullptr));
#if QT_CONFIG(shortcut)
        deletePageButton->setShortcut(QCoreApplication::translate("GuideManager", "Alt+P", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        deleteAllGuides->setToolTip(QCoreApplication::translate("GuideManager", "Delete all guides from the current document", nullptr));
#endif // QT_CONFIG(tooltip)
        deleteAllGuides->setText(QCoreApplication::translate("GuideManager", "Delete Guides from &All Pages", nullptr));
#if QT_CONFIG(shortcut)
        deleteAllGuides->setShortcut(QCoreApplication::translate("GuideManager", "Alt+A", nullptr));
#endif // QT_CONFIG(shortcut)
        tabWidget->setTabText(tabWidget->indexOf(miscPage), QCoreApplication::translate("GuideManager", "&Misc", nullptr));
    } // retranslateUi

};

namespace Ui {
    class GuideManager: public Ui_GuideManager {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_GUIDEMANAGER_H
