/********************************************************************************
** Form generated from reading UI file 'mark2mark.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MARK2MARK_H
#define UI_MARK2MARK_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_Mark2MarkDlg
{
public:
    QVBoxLayout *verticalLayout;
    QSplitter *splitter;
    QLabel *label_2;
    QLineEdit *labelEdit;
    QSplitter *splitter_2;
    QLabel *label;
    QComboBox *LabelList;
    QHBoxLayout *horizontalLayout;
    QSpacerItem *horizontalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *Mark2MarkDlg)
    {
        if (Mark2MarkDlg->objectName().isEmpty())
            Mark2MarkDlg->setObjectName(QString::fromUtf8("Mark2MarkDlg"));
        Mark2MarkDlg->resize(362, 103);
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(Mark2MarkDlg->sizePolicy().hasHeightForWidth());
        Mark2MarkDlg->setSizePolicy(sizePolicy);
        Mark2MarkDlg->setMinimumSize(QSize(0, 103));
        Mark2MarkDlg->setMaximumSize(QSize(16777215, 103));
        verticalLayout = new QVBoxLayout(Mark2MarkDlg);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        splitter = new QSplitter(Mark2MarkDlg);
        splitter->setObjectName(QString::fromUtf8("splitter"));
        splitter->setOrientation(Qt::Horizontal);
        label_2 = new QLabel(splitter);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        splitter->addWidget(label_2);
        labelEdit = new QLineEdit(splitter);
        labelEdit->setObjectName(QString::fromUtf8("labelEdit"));
        splitter->addWidget(labelEdit);

        verticalLayout->addWidget(splitter);

        splitter_2 = new QSplitter(Mark2MarkDlg);
        splitter_2->setObjectName(QString::fromUtf8("splitter_2"));
        splitter_2->setOrientation(Qt::Horizontal);
        label = new QLabel(splitter_2);
        label->setObjectName(QString::fromUtf8("label"));
        splitter_2->addWidget(label);
        LabelList = new QComboBox(splitter_2);
        LabelList->setObjectName(QString::fromUtf8("LabelList"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(LabelList->sizePolicy().hasHeightForWidth());
        LabelList->setSizePolicy(sizePolicy1);
        LabelList->setEditable(false);
        LabelList->setDuplicatesEnabled(false);
        splitter_2->addWidget(LabelList);

        verticalLayout->addWidget(splitter_2);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalSpacer = new QSpacerItem(166, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        buttonBox = new QDialogButtonBox(Mark2MarkDlg);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        QSizePolicy sizePolicy2(QSizePolicy::Fixed, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(buttonBox->sizePolicy().hasHeightForWidth());
        buttonBox->setSizePolicy(sizePolicy2);
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        horizontalLayout->addWidget(buttonBox);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(Mark2MarkDlg);
        QObject::connect(buttonBox, SIGNAL(accepted()), Mark2MarkDlg, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), Mark2MarkDlg, SLOT(reject()));

        LabelList->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(Mark2MarkDlg);
    } // setupUi

    void retranslateUi(QDialog *Mark2MarkDlg)
    {
        Mark2MarkDlg->setWindowTitle(QCoreApplication::translate("Mark2MarkDlg", "Dialog", nullptr));
        label_2->setText(QCoreApplication::translate("Mark2MarkDlg", "Label:", nullptr));
        label->setText(QCoreApplication::translate("Mark2MarkDlg", "Select marker:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class Mark2MarkDlg: public Ui_Mark2MarkDlg {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MARK2MARK_H
