/********************************************************************************
** Form generated from reading UI file 'exportform.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_EXPORTFORM_H
#define UI_EXPORTFORM_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ExportForm
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *hboxLayout;
    QLabel *label;
    QLineEdit *outputDirectory;
    QPushButton *outputDirectoryButton;
    QGridLayout *gridLayout;
    QGroupBox *groupBox_2;
    QHBoxLayout *horizontalLayout;
    QLabel *label_7;
    QLineEdit *prefixLineEdit;
    QGroupBox *rangeGroupBox;
    QGridLayout *rangeLayout;
    QRadioButton *intervalPagesRadio;
    QRadioButton *allPagesRadio;
    QPushButton *pageNrButton;
    QRadioButton *onePageRadio;
    QLineEdit *rangeVal;
    QSpacerItem *spacerItem;
    QGroupBox *groupBox;
    QGridLayout *gridLayout1;
    QSpinBox *DPIBox;
    QLabel *label_2;
    QSpinBox *qualityBox;
    QLabel *label_5;
    QSpacerItem *verticalSpacer;
    QLabel *label_6;
    QLabel *label_3;
    QCheckBox *noBackground;
    QLabel *label_4;
    QComboBox *bitmapType;
    QLabel *imageSizeLabel;
    QSpinBox *enlargementBox;
    QSpacerItem *verticalSpacer_2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *ExportForm)
    {
        if (ExportForm->objectName().isEmpty())
            ExportForm->setObjectName(QString::fromUtf8("ExportForm"));
        ExportForm->resize(520, 324);
        verticalLayout = new QVBoxLayout(ExportForm);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        label = new QLabel(ExportForm);
        label->setObjectName(QString::fromUtf8("label"));

        hboxLayout->addWidget(label);

        outputDirectory = new QLineEdit(ExportForm);
        outputDirectory->setObjectName(QString::fromUtf8("outputDirectory"));

        hboxLayout->addWidget(outputDirectory);

        outputDirectoryButton = new QPushButton(ExportForm);
        outputDirectoryButton->setObjectName(QString::fromUtf8("outputDirectoryButton"));

        hboxLayout->addWidget(outputDirectoryButton);


        verticalLayout->addLayout(hboxLayout);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        groupBox_2 = new QGroupBox(ExportForm);
        groupBox_2->setObjectName(QString::fromUtf8("groupBox_2"));
        horizontalLayout = new QHBoxLayout(groupBox_2);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label_7 = new QLabel(groupBox_2);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        horizontalLayout->addWidget(label_7);

        prefixLineEdit = new QLineEdit(groupBox_2);
        prefixLineEdit->setObjectName(QString::fromUtf8("prefixLineEdit"));

        horizontalLayout->addWidget(prefixLineEdit);


        gridLayout->addWidget(groupBox_2, 0, 0, 1, 1);

        rangeGroupBox = new QGroupBox(ExportForm);
        rangeGroupBox->setObjectName(QString::fromUtf8("rangeGroupBox"));
        rangeLayout = new QGridLayout(rangeGroupBox);
        rangeLayout->setObjectName(QString::fromUtf8("rangeLayout"));
        intervalPagesRadio = new QRadioButton(rangeGroupBox);
        intervalPagesRadio->setObjectName(QString::fromUtf8("intervalPagesRadio"));

        rangeLayout->addWidget(intervalPagesRadio, 2, 0, 1, 1);

        allPagesRadio = new QRadioButton(rangeGroupBox);
        allPagesRadio->setObjectName(QString::fromUtf8("allPagesRadio"));

        rangeLayout->addWidget(allPagesRadio, 1, 0, 1, 1);

        pageNrButton = new QPushButton(rangeGroupBox);
        pageNrButton->setObjectName(QString::fromUtf8("pageNrButton"));

        rangeLayout->addWidget(pageNrButton, 2, 2, 1, 1);

        onePageRadio = new QRadioButton(rangeGroupBox);
        onePageRadio->setObjectName(QString::fromUtf8("onePageRadio"));

        rangeLayout->addWidget(onePageRadio, 0, 0, 1, 3);

        rangeVal = new QLineEdit(rangeGroupBox);
        rangeVal->setObjectName(QString::fromUtf8("rangeVal"));

        rangeLayout->addWidget(rangeVal, 2, 1, 1, 1);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        rangeLayout->addItem(spacerItem, 3, 0, 1, 1);


        gridLayout->addWidget(rangeGroupBox, 1, 0, 1, 1);

        groupBox = new QGroupBox(ExportForm);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        gridLayout1 = new QGridLayout(groupBox);
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        DPIBox = new QSpinBox(groupBox);
        DPIBox->setObjectName(QString::fromUtf8("DPIBox"));
        DPIBox->setMinimum(72);
        DPIBox->setMaximum(2400);
        DPIBox->setValue(72);

        gridLayout1->addWidget(DPIBox, 2, 1, 1, 1);

        label_2 = new QLabel(groupBox);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout1->addWidget(label_2, 0, 0, 1, 1);

        qualityBox = new QSpinBox(groupBox);
        qualityBox->setObjectName(QString::fromUtf8("qualityBox"));
        qualityBox->setMinimum(-1);
        qualityBox->setMaximum(100);
        qualityBox->setValue(-1);

        gridLayout1->addWidget(qualityBox, 1, 1, 1, 1);

        label_5 = new QLabel(groupBox);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        gridLayout1->addWidget(label_5, 3, 0, 1, 1);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout1->addItem(verticalSpacer, 8, 0, 1, 1);

        label_6 = new QLabel(groupBox);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        gridLayout1->addWidget(label_6, 6, 0, 1, 2);

        label_3 = new QLabel(groupBox);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout1->addWidget(label_3, 1, 0, 1, 1);

        noBackground = new QCheckBox(groupBox);
        noBackground->setObjectName(QString::fromUtf8("noBackground"));

        gridLayout1->addWidget(noBackground, 4, 0, 1, 2);

        label_4 = new QLabel(groupBox);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout1->addWidget(label_4, 2, 0, 1, 1);

        bitmapType = new QComboBox(groupBox);
        bitmapType->setObjectName(QString::fromUtf8("bitmapType"));

        gridLayout1->addWidget(bitmapType, 0, 1, 1, 1);

        imageSizeLabel = new QLabel(groupBox);
        imageSizeLabel->setObjectName(QString::fromUtf8("imageSizeLabel"));

        gridLayout1->addWidget(imageSizeLabel, 7, 0, 1, 2);

        enlargementBox = new QSpinBox(groupBox);
        enlargementBox->setObjectName(QString::fromUtf8("enlargementBox"));
        enlargementBox->setMinimum(1);
        enlargementBox->setMaximum(2400);
        enlargementBox->setValue(100);

        gridLayout1->addWidget(enlargementBox, 3, 1, 1, 1);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout1->addItem(verticalSpacer_2, 5, 0, 1, 1);


        gridLayout->addWidget(groupBox, 0, 1, 2, 1);


        verticalLayout->addLayout(gridLayout);

        buttonBox = new QDialogButtonBox(ExportForm);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);

#if QT_CONFIG(shortcut)
        label->setBuddy(outputDirectory);
        label_2->setBuddy(bitmapType);
        label_5->setBuddy(enlargementBox);
        label_3->setBuddy(qualityBox);
        label_4->setBuddy(DPIBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(outputDirectory, outputDirectoryButton);
        QWidget::setTabOrder(outputDirectoryButton, prefixLineEdit);
        QWidget::setTabOrder(prefixLineEdit, onePageRadio);
        QWidget::setTabOrder(onePageRadio, allPagesRadio);
        QWidget::setTabOrder(allPagesRadio, intervalPagesRadio);
        QWidget::setTabOrder(intervalPagesRadio, rangeVal);
        QWidget::setTabOrder(rangeVal, pageNrButton);
        QWidget::setTabOrder(pageNrButton, bitmapType);
        QWidget::setTabOrder(bitmapType, qualityBox);
        QWidget::setTabOrder(qualityBox, DPIBox);
        QWidget::setTabOrder(DPIBox, enlargementBox);
        QWidget::setTabOrder(enlargementBox, noBackground);

        retranslateUi(ExportForm);
        QObject::connect(buttonBox, SIGNAL(accepted()), ExportForm, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), ExportForm, SLOT(reject()));

        QMetaObject::connectSlotsByName(ExportForm);
    } // setupUi

    void retranslateUi(QDialog *ExportForm)
    {
        ExportForm->setWindowTitle(QCoreApplication::translate("ExportForm", "Export as Image(s)", nullptr));
        label->setText(QCoreApplication::translate("ExportForm", "&Export to Directory:", nullptr));
        outputDirectoryButton->setText(QCoreApplication::translate("ExportForm", "C&hange...", nullptr));
        groupBox_2->setTitle(QCoreApplication::translate("ExportForm", "Naming of exported files:", nullptr));
        label_7->setText(QCoreApplication::translate("ExportForm", "Prefix:", nullptr));
        rangeGroupBox->setTitle(QCoreApplication::translate("ExportForm", "Range", nullptr));
        intervalPagesRadio->setText(QCoreApplication::translate("ExportForm", "&Range", nullptr));
        allPagesRadio->setText(QCoreApplication::translate("ExportForm", "&All pages", nullptr));
        pageNrButton->setText(QString());
        onePageRadio->setText(QCoreApplication::translate("ExportForm", "&Current page", nullptr));
        groupBox->setTitle(QCoreApplication::translate("ExportForm", "Options", nullptr));
        DPIBox->setSuffix(QCoreApplication::translate("ExportForm", " dpi", nullptr));
        label_2->setText(QCoreApplication::translate("ExportForm", "Image &Type:", nullptr));
        qualityBox->setSuffix(QCoreApplication::translate("ExportForm", " %", nullptr));
        label_5->setText(QCoreApplication::translate("ExportForm", "&Size:", nullptr));
        label_6->setText(QCoreApplication::translate("ExportForm", "Image size in Pixels", nullptr));
        label_3->setText(QCoreApplication::translate("ExportForm", "&Quality:", nullptr));
        noBackground->setText(QCoreApplication::translate("ExportForm", "No Background", nullptr));
        label_4->setText(QCoreApplication::translate("ExportForm", "&Resolution:", nullptr));
        imageSizeLabel->setText(QString());
        enlargementBox->setSuffix(QCoreApplication::translate("ExportForm", " %", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ExportForm: public Ui_ExportForm {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_EXPORTFORM_H
