/********************************************************************************
** Form generated from reading UI file 'prefs_userinterfacebase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_USERINTERFACEBASE_H
#define UI_PREFS_USERINTERFACEBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Prefs_UserInterface
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *titleLabel;
    QFrame *line;
    QTabWidget *tabWidget;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_3;
    QLabel *label;
    QFrame *line_3;
    QFormLayout *formLayout;
    QLabel *themeLabel;
    QComboBox *themeComboBox;
    QLabel *label_3;
    QComboBox *iconSetComboBox;
    QCheckBox *useSmallWidgetsCheckBox;
    QCheckBox *useTabsForDocumentsCheckBox;
    QLabel *recentDocumentsLabel;
    QSpinBox *recentDocumentsSpinBox;
    QSpacerItem *verticalSpacer_6;
    QLabel *label_4;
    QFrame *line_4;
    QFormLayout *formLayout_6;
    QLabel *languageLabel;
    QComboBox *languageComboBox;
    QLabel *numberFormatLabel;
    QComboBox *numberFormatComboBox;
    QSpacerItem *verticalSpacer_4;
    QLabel *label_2;
    QFrame *line_2;
    QFormLayout *formLayout_2;
    QLabel *fontSizeMenusLabel;
    QSpinBox *fontSizeMenuSpinBox;
    QLabel *fontSizePalettesLabel;
    QSpinBox *fontSizePaletteSpinBox;
    QSpacerItem *verticalSpacer_3;
    QWidget *tab_4;
    QVBoxLayout *verticalLayout_5;
    QFormLayout *formLayout_3;
    QLabel *storyEditorFontLabel;
    QPushButton *storyEditorFontPushButton;
    QCheckBox *storyEditorUseSmartSelectionCheckBox;
    QSpacerItem *verticalSpacer_5;
    QWidget *tab;
    QVBoxLayout *verticalLayout_4;
    QFormLayout *formLayout_4;
    QLabel *resizeMoveDelayLabel;
    QSpinBox *resizeMoveDelaySpinBox;
    QLabel *wheelJumpLabel;
    QSpinBox *wheelJumpSpinBox;
    QSpacerItem *verticalSpacer_2;
    QWidget *tab_5;
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout_5;
    QCheckBox *showStartupDialogCheckBox;
    QCheckBox *showSplashCheckBox;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *Prefs_UserInterface)
    {
        if (Prefs_UserInterface->objectName().isEmpty())
            Prefs_UserInterface->setObjectName(QString::fromUtf8("Prefs_UserInterface"));
        Prefs_UserInterface->resize(803, 744);
        verticalLayout_2 = new QVBoxLayout(Prefs_UserInterface);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        titleLabel = new QLabel(Prefs_UserInterface);
        titleLabel->setObjectName(QString::fromUtf8("titleLabel"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        titleLabel->setFont(font);

        verticalLayout_2->addWidget(titleLabel);

        line = new QFrame(Prefs_UserInterface);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line);

        tabWidget = new QTabWidget(Prefs_UserInterface);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tab_3 = new QWidget();
        tab_3->setObjectName(QString::fromUtf8("tab_3"));
        verticalLayout_3 = new QVBoxLayout(tab_3);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        label = new QLabel(tab_3);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
        label->setFont(font1);

        verticalLayout_3->addWidget(label);

        line_3 = new QFrame(tab_3);
        line_3->setObjectName(QString::fromUtf8("line_3"));
        line_3->setFrameShape(QFrame::HLine);
        line_3->setFrameShadow(QFrame::Sunken);

        verticalLayout_3->addWidget(line_3);

        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        themeLabel = new QLabel(tab_3);
        themeLabel->setObjectName(QString::fromUtf8("themeLabel"));

        formLayout->setWidget(0, QFormLayout::LabelRole, themeLabel);

        themeComboBox = new QComboBox(tab_3);
        themeComboBox->setObjectName(QString::fromUtf8("themeComboBox"));

        formLayout->setWidget(0, QFormLayout::FieldRole, themeComboBox);

        label_3 = new QLabel(tab_3);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        formLayout->setWidget(1, QFormLayout::LabelRole, label_3);

        iconSetComboBox = new QComboBox(tab_3);
        iconSetComboBox->setObjectName(QString::fromUtf8("iconSetComboBox"));

        formLayout->setWidget(1, QFormLayout::FieldRole, iconSetComboBox);

        useSmallWidgetsCheckBox = new QCheckBox(tab_3);
        useSmallWidgetsCheckBox->setObjectName(QString::fromUtf8("useSmallWidgetsCheckBox"));

        formLayout->setWidget(2, QFormLayout::FieldRole, useSmallWidgetsCheckBox);

        useTabsForDocumentsCheckBox = new QCheckBox(tab_3);
        useTabsForDocumentsCheckBox->setObjectName(QString::fromUtf8("useTabsForDocumentsCheckBox"));

        formLayout->setWidget(3, QFormLayout::FieldRole, useTabsForDocumentsCheckBox);

        recentDocumentsLabel = new QLabel(tab_3);
        recentDocumentsLabel->setObjectName(QString::fromUtf8("recentDocumentsLabel"));

        formLayout->setWidget(4, QFormLayout::LabelRole, recentDocumentsLabel);

        recentDocumentsSpinBox = new QSpinBox(tab_3);
        recentDocumentsSpinBox->setObjectName(QString::fromUtf8("recentDocumentsSpinBox"));

        formLayout->setWidget(4, QFormLayout::FieldRole, recentDocumentsSpinBox);


        verticalLayout_3->addLayout(formLayout);

        verticalSpacer_6 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_3->addItem(verticalSpacer_6);

        label_4 = new QLabel(tab_3);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        label_4->setFont(font1);

        verticalLayout_3->addWidget(label_4);

        line_4 = new QFrame(tab_3);
        line_4->setObjectName(QString::fromUtf8("line_4"));
        line_4->setFrameShape(QFrame::HLine);
        line_4->setFrameShadow(QFrame::Sunken);

        verticalLayout_3->addWidget(line_4);

        formLayout_6 = new QFormLayout();
        formLayout_6->setObjectName(QString::fromUtf8("formLayout_6"));
        languageLabel = new QLabel(tab_3);
        languageLabel->setObjectName(QString::fromUtf8("languageLabel"));

        formLayout_6->setWidget(0, QFormLayout::LabelRole, languageLabel);

        languageComboBox = new QComboBox(tab_3);
        languageComboBox->setObjectName(QString::fromUtf8("languageComboBox"));

        formLayout_6->setWidget(0, QFormLayout::FieldRole, languageComboBox);

        numberFormatLabel = new QLabel(tab_3);
        numberFormatLabel->setObjectName(QString::fromUtf8("numberFormatLabel"));

        formLayout_6->setWidget(1, QFormLayout::LabelRole, numberFormatLabel);

        numberFormatComboBox = new QComboBox(tab_3);
        numberFormatComboBox->setObjectName(QString::fromUtf8("numberFormatComboBox"));

        formLayout_6->setWidget(1, QFormLayout::FieldRole, numberFormatComboBox);


        verticalLayout_3->addLayout(formLayout_6);

        verticalSpacer_4 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_3->addItem(verticalSpacer_4);

        label_2 = new QLabel(tab_3);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setFont(font1);

        verticalLayout_3->addWidget(label_2);

        line_2 = new QFrame(tab_3);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        verticalLayout_3->addWidget(line_2);

        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName(QString::fromUtf8("formLayout_2"));
        fontSizeMenusLabel = new QLabel(tab_3);
        fontSizeMenusLabel->setObjectName(QString::fromUtf8("fontSizeMenusLabel"));

        formLayout_2->setWidget(0, QFormLayout::LabelRole, fontSizeMenusLabel);

        fontSizeMenuSpinBox = new QSpinBox(tab_3);
        fontSizeMenuSpinBox->setObjectName(QString::fromUtf8("fontSizeMenuSpinBox"));

        formLayout_2->setWidget(0, QFormLayout::FieldRole, fontSizeMenuSpinBox);

        fontSizePalettesLabel = new QLabel(tab_3);
        fontSizePalettesLabel->setObjectName(QString::fromUtf8("fontSizePalettesLabel"));

        formLayout_2->setWidget(1, QFormLayout::LabelRole, fontSizePalettesLabel);

        fontSizePaletteSpinBox = new QSpinBox(tab_3);
        fontSizePaletteSpinBox->setObjectName(QString::fromUtf8("fontSizePaletteSpinBox"));

        formLayout_2->setWidget(1, QFormLayout::FieldRole, fontSizePaletteSpinBox);


        verticalLayout_3->addLayout(formLayout_2);

        verticalSpacer_3 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        tabWidget->addTab(tab_3, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName(QString::fromUtf8("tab_4"));
        verticalLayout_5 = new QVBoxLayout(tab_4);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        formLayout_3 = new QFormLayout();
        formLayout_3->setObjectName(QString::fromUtf8("formLayout_3"));
        formLayout_3->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        storyEditorFontLabel = new QLabel(tab_4);
        storyEditorFontLabel->setObjectName(QString::fromUtf8("storyEditorFontLabel"));

        formLayout_3->setWidget(0, QFormLayout::LabelRole, storyEditorFontLabel);

        storyEditorFontPushButton = new QPushButton(tab_4);
        storyEditorFontPushButton->setObjectName(QString::fromUtf8("storyEditorFontPushButton"));

        formLayout_3->setWidget(0, QFormLayout::FieldRole, storyEditorFontPushButton);

        storyEditorUseSmartSelectionCheckBox = new QCheckBox(tab_4);
        storyEditorUseSmartSelectionCheckBox->setObjectName(QString::fromUtf8("storyEditorUseSmartSelectionCheckBox"));

        formLayout_3->setWidget(1, QFormLayout::FieldRole, storyEditorUseSmartSelectionCheckBox);


        verticalLayout_5->addLayout(formLayout_3);

        verticalSpacer_5 = new QSpacerItem(20, 251, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_5);

        tabWidget->addTab(tab_4, QString());
        tab = new QWidget();
        tab->setObjectName(QString::fromUtf8("tab"));
        verticalLayout_4 = new QVBoxLayout(tab);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        formLayout_4 = new QFormLayout();
        formLayout_4->setObjectName(QString::fromUtf8("formLayout_4"));
        formLayout_4->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        resizeMoveDelayLabel = new QLabel(tab);
        resizeMoveDelayLabel->setObjectName(QString::fromUtf8("resizeMoveDelayLabel"));

        formLayout_4->setWidget(0, QFormLayout::LabelRole, resizeMoveDelayLabel);

        resizeMoveDelaySpinBox = new QSpinBox(tab);
        resizeMoveDelaySpinBox->setObjectName(QString::fromUtf8("resizeMoveDelaySpinBox"));
        resizeMoveDelaySpinBox->setMaximum(9999);

        formLayout_4->setWidget(0, QFormLayout::FieldRole, resizeMoveDelaySpinBox);

        wheelJumpLabel = new QLabel(tab);
        wheelJumpLabel->setObjectName(QString::fromUtf8("wheelJumpLabel"));

        formLayout_4->setWidget(1, QFormLayout::LabelRole, wheelJumpLabel);

        wheelJumpSpinBox = new QSpinBox(tab);
        wheelJumpSpinBox->setObjectName(QString::fromUtf8("wheelJumpSpinBox"));

        formLayout_4->setWidget(1, QFormLayout::FieldRole, wheelJumpSpinBox);


        verticalLayout_4->addLayout(formLayout_4);

        verticalSpacer_2 = new QSpacerItem(20, 266, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_2);

        tabWidget->addTab(tab, QString());
        tab_5 = new QWidget();
        tab_5->setObjectName(QString::fromUtf8("tab_5"));
        verticalLayout = new QVBoxLayout(tab_5);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        formLayout_5 = new QFormLayout();
        formLayout_5->setObjectName(QString::fromUtf8("formLayout_5"));
        showStartupDialogCheckBox = new QCheckBox(tab_5);
        showStartupDialogCheckBox->setObjectName(QString::fromUtf8("showStartupDialogCheckBox"));

        formLayout_5->setWidget(2, QFormLayout::FieldRole, showStartupDialogCheckBox);

        showSplashCheckBox = new QCheckBox(tab_5);
        showSplashCheckBox->setObjectName(QString::fromUtf8("showSplashCheckBox"));

        formLayout_5->setWidget(1, QFormLayout::FieldRole, showSplashCheckBox);


        verticalLayout->addLayout(formLayout_5);

        verticalSpacer = new QSpacerItem(20, 296, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        tabWidget->addTab(tab_5, QString());

        verticalLayout_2->addWidget(tabWidget);

#if QT_CONFIG(shortcut)
        themeLabel->setBuddy(themeComboBox);
        recentDocumentsLabel->setBuddy(recentDocumentsSpinBox);
        languageLabel->setBuddy(languageComboBox);
        numberFormatLabel->setBuddy(numberFormatComboBox);
        fontSizeMenusLabel->setBuddy(fontSizeMenuSpinBox);
        fontSizePalettesLabel->setBuddy(fontSizePaletteSpinBox);
        resizeMoveDelayLabel->setBuddy(resizeMoveDelaySpinBox);
        wheelJumpLabel->setBuddy(wheelJumpSpinBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(tabWidget, themeComboBox);
        QWidget::setTabOrder(themeComboBox, iconSetComboBox);
        QWidget::setTabOrder(iconSetComboBox, useSmallWidgetsCheckBox);
        QWidget::setTabOrder(useSmallWidgetsCheckBox, useTabsForDocumentsCheckBox);
        QWidget::setTabOrder(useTabsForDocumentsCheckBox, recentDocumentsSpinBox);
        QWidget::setTabOrder(recentDocumentsSpinBox, fontSizeMenuSpinBox);
        QWidget::setTabOrder(fontSizeMenuSpinBox, fontSizePaletteSpinBox);
        QWidget::setTabOrder(fontSizePaletteSpinBox, storyEditorFontPushButton);
        QWidget::setTabOrder(storyEditorFontPushButton, storyEditorUseSmartSelectionCheckBox);
        QWidget::setTabOrder(storyEditorUseSmartSelectionCheckBox, resizeMoveDelaySpinBox);
        QWidget::setTabOrder(resizeMoveDelaySpinBox, wheelJumpSpinBox);
        QWidget::setTabOrder(wheelJumpSpinBox, showSplashCheckBox);
        QWidget::setTabOrder(showSplashCheckBox, showStartupDialogCheckBox);

        retranslateUi(Prefs_UserInterface);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Prefs_UserInterface);
    } // setupUi

    void retranslateUi(QWidget *Prefs_UserInterface)
    {
        titleLabel->setText(QCoreApplication::translate("Prefs_UserInterface", "User Interface", nullptr));
        label->setText(QCoreApplication::translate("Prefs_UserInterface", "Appearance", nullptr));
        themeLabel->setText(QCoreApplication::translate("Prefs_UserInterface", "&Theme:", nullptr));
        label_3->setText(QCoreApplication::translate("Prefs_UserInterface", "Icon Set:", nullptr));
        useSmallWidgetsCheckBox->setText(QCoreApplication::translate("Prefs_UserInterface", "Use Small Widgets on Palettes", nullptr));
        useTabsForDocumentsCheckBox->setText(QCoreApplication::translate("Prefs_UserInterface", "Use Tabs for Documents", nullptr));
        recentDocumentsLabel->setText(QCoreApplication::translate("Prefs_UserInterface", "&Recent Documents:", nullptr));
        label_4->setText(QCoreApplication::translate("Prefs_UserInterface", "Language and Regionalization", nullptr));
        languageLabel->setText(QCoreApplication::translate("Prefs_UserInterface", "&Language:", nullptr));
        numberFormatLabel->setText(QCoreApplication::translate("Prefs_UserInterface", "Number Format:", nullptr));
        label_2->setText(QCoreApplication::translate("Prefs_UserInterface", "Font Sizes", nullptr));
        fontSizeMenusLabel->setText(QCoreApplication::translate("Prefs_UserInterface", "&Menus:", nullptr));
        fontSizeMenuSpinBox->setSuffix(QCoreApplication::translate("Prefs_UserInterface", " pt", nullptr));
        fontSizePalettesLabel->setText(QCoreApplication::translate("Prefs_UserInterface", "&Palettes:", nullptr));
        fontSizePaletteSpinBox->setSuffix(QCoreApplication::translate("Prefs_UserInterface", " pt", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("Prefs_UserInterface", "Main Window", nullptr));
        storyEditorFontLabel->setText(QCoreApplication::translate("Prefs_UserInterface", "Font:", nullptr));
        storyEditorFontPushButton->setText(QString());
        storyEditorUseSmartSelectionCheckBox->setText(QCoreApplication::translate("Prefs_UserInterface", "Use Smart Text Selection", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), QCoreApplication::translate("Prefs_UserInterface", "Story Editor", nullptr));
        resizeMoveDelayLabel->setText(QCoreApplication::translate("Prefs_UserInterface", "Time before resize or move starts:", nullptr));
        resizeMoveDelaySpinBox->setSuffix(QCoreApplication::translate("Prefs_UserInterface", " ms", nullptr));
        wheelJumpLabel->setText(QCoreApplication::translate("Prefs_UserInterface", "&Wheel Jump:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("Prefs_UserInterface", "Interactivity", nullptr));
        showStartupDialogCheckBox->setText(QCoreApplication::translate("Prefs_UserInterface", "Show Startup Dialog", nullptr));
        showSplashCheckBox->setText(QCoreApplication::translate("Prefs_UserInterface", "Show Splashscreen", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_5), QCoreApplication::translate("Prefs_UserInterface", "Start Up", nullptr));
        (void)Prefs_UserInterface;
    } // retranslateUi

};

namespace Ui {
    class Prefs_UserInterface: public Ui_Prefs_UserInterface {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_USERINTERFACEBASE_H
