/****************************************************************************
** Meta object code from reading C++ file 'gradientvectordialog.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/gradientvectordialog.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'gradientvectordialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_GradientVectorDialog_t {
    QByteArrayData data[38];
    char stringdata0[435];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_GradientVectorDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_GradientVectorDialog_t qt_meta_stringdata_GradientVectorDialog = {
    {
QT_MOC_LITERAL(0, 0, 20), // "GradientVectorDialog"
QT_MOC_LITERAL(1, 21, 10), // "NewSpecial"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 12), // "editGradient"
QT_MOC_LITERAL(4, 46, 13), // "createNewMesh"
QT_MOC_LITERAL(5, 60, 9), // "resetMesh"
QT_MOC_LITERAL(6, 70, 11), // "meshToShape"
QT_MOC_LITERAL(7, 82, 13), // "reset1Control"
QT_MOC_LITERAL(8, 96, 15), // "resetAllControl"
QT_MOC_LITERAL(9, 112, 11), // "removePatch"
QT_MOC_LITERAL(10, 124, 11), // "snapToMGrid"
QT_MOC_LITERAL(11, 136, 14), // "languageChange"
QT_MOC_LITERAL(12, 151, 16), // "handleEditButton"
QT_MOC_LITERAL(13, 168, 23), // "handleEditControlButton"
QT_MOC_LITERAL(14, 192, 17), // "handlePEditButton"
QT_MOC_LITERAL(15, 210, 24), // "handlePEditControlButton"
QT_MOC_LITERAL(16, 235, 16), // "handlePAddButton"
QT_MOC_LITERAL(17, 252, 19), // "handleSnapToGridBox"
QT_MOC_LITERAL(18, 272, 13), // "endPAddButton"
QT_MOC_LITERAL(19, 286, 23), // "changebuttonRemovePatch"
QT_MOC_LITERAL(20, 310, 3), // "val"
QT_MOC_LITERAL(21, 314, 9), // "setValues"
QT_MOC_LITERAL(22, 324, 2), // "x1"
QT_MOC_LITERAL(23, 327, 2), // "y1"
QT_MOC_LITERAL(24, 330, 2), // "x2"
QT_MOC_LITERAL(25, 333, 2), // "y2"
QT_MOC_LITERAL(26, 336, 2), // "fx"
QT_MOC_LITERAL(27, 339, 2), // "fy"
QT_MOC_LITERAL(28, 342, 2), // "sg"
QT_MOC_LITERAL(29, 345, 2), // "sk"
QT_MOC_LITERAL(30, 348, 2), // "cx"
QT_MOC_LITERAL(31, 351, 2), // "cy"
QT_MOC_LITERAL(32, 354, 14), // "changeSpecialL"
QT_MOC_LITERAL(33, 369, 14), // "changeSpecialR"
QT_MOC_LITERAL(34, 384, 14), // "changeSpecialF"
QT_MOC_LITERAL(35, 399, 14), // "changeSpecialD"
QT_MOC_LITERAL(36, 414, 10), // "unitChange"
QT_MOC_LITERAL(37, 425, 9) // "unitIndex"

    },
    "GradientVectorDialog\0NewSpecial\0\0"
    "editGradient\0createNewMesh\0resetMesh\0"
    "meshToShape\0reset1Control\0resetAllControl\0"
    "removePatch\0snapToMGrid\0languageChange\0"
    "handleEditButton\0handleEditControlButton\0"
    "handlePEditButton\0handlePEditControlButton\0"
    "handlePAddButton\0handleSnapToGridBox\0"
    "endPAddButton\0changebuttonRemovePatch\0"
    "val\0setValues\0x1\0y1\0x2\0y2\0fx\0fy\0sg\0"
    "sk\0cx\0cy\0changeSpecialL\0changeSpecialR\0"
    "changeSpecialF\0changeSpecialD\0unitChange\0"
    "unitIndex"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_GradientVectorDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      24,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       9,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,   10,  134,    2, 0x06 /* Public */,
       3,    1,  155,    2, 0x06 /* Public */,
       4,    0,  158,    2, 0x06 /* Public */,
       5,    0,  159,    2, 0x06 /* Public */,
       6,    0,  160,    2, 0x06 /* Public */,
       7,    0,  161,    2, 0x06 /* Public */,
       8,    0,  162,    2, 0x06 /* Public */,
       9,    0,  163,    2, 0x06 /* Public */,
      10,    1,  164,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    0,  167,    2, 0x0a /* Public */,
      12,    0,  168,    2, 0x0a /* Public */,
      13,    0,  169,    2, 0x0a /* Public */,
      14,    0,  170,    2, 0x0a /* Public */,
      15,    0,  171,    2, 0x0a /* Public */,
      16,    0,  172,    2, 0x0a /* Public */,
      17,    0,  173,    2, 0x0a /* Public */,
      18,    0,  174,    2, 0x0a /* Public */,
      19,    1,  175,    2, 0x0a /* Public */,
      21,   10,  178,    2, 0x0a /* Public */,
      32,    0,  199,    2, 0x0a /* Public */,
      33,    0,  200,    2, 0x0a /* Public */,
      34,    0,  201,    2, 0x0a /* Public */,
      35,    0,  202,    2, 0x0a /* Public */,
      36,    1,  203,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   20,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double,   22,   23,   24,   25,   26,   27,   28,   29,   30,   31,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   37,

       0        // eod
};

void GradientVectorDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<GradientVectorDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->NewSpecial((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6])),(*reinterpret_cast< double(*)>(_a[7])),(*reinterpret_cast< double(*)>(_a[8])),(*reinterpret_cast< double(*)>(_a[9])),(*reinterpret_cast< double(*)>(_a[10]))); break;
        case 1: _t->editGradient((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->createNewMesh(); break;
        case 3: _t->resetMesh(); break;
        case 4: _t->meshToShape(); break;
        case 5: _t->reset1Control(); break;
        case 6: _t->resetAllControl(); break;
        case 7: _t->removePatch(); break;
        case 8: _t->snapToMGrid((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 9: _t->languageChange(); break;
        case 10: _t->handleEditButton(); break;
        case 11: _t->handleEditControlButton(); break;
        case 12: _t->handlePEditButton(); break;
        case 13: _t->handlePEditControlButton(); break;
        case 14: _t->handlePAddButton(); break;
        case 15: _t->handleSnapToGridBox(); break;
        case 16: _t->endPAddButton(); break;
        case 17: _t->changebuttonRemovePatch((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: _t->setValues((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6])),(*reinterpret_cast< double(*)>(_a[7])),(*reinterpret_cast< double(*)>(_a[8])),(*reinterpret_cast< double(*)>(_a[9])),(*reinterpret_cast< double(*)>(_a[10]))); break;
        case 19: _t->changeSpecialL(); break;
        case 20: _t->changeSpecialR(); break;
        case 21: _t->changeSpecialF(); break;
        case 22: _t->changeSpecialD(); break;
        case 23: _t->unitChange((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (GradientVectorDialog::*)(double , double , double , double , double , double , double , double , double , double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GradientVectorDialog::NewSpecial)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (GradientVectorDialog::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GradientVectorDialog::editGradient)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (GradientVectorDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GradientVectorDialog::createNewMesh)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (GradientVectorDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GradientVectorDialog::resetMesh)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (GradientVectorDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GradientVectorDialog::meshToShape)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (GradientVectorDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GradientVectorDialog::reset1Control)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (GradientVectorDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GradientVectorDialog::resetAllControl)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (GradientVectorDialog::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GradientVectorDialog::removePatch)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (GradientVectorDialog::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&GradientVectorDialog::snapToMGrid)) {
                *result = 8;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject GradientVectorDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<ScrPaletteBase::staticMetaObject>(),
    qt_meta_stringdata_GradientVectorDialog.data,
    qt_meta_data_GradientVectorDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *GradientVectorDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GradientVectorDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_GradientVectorDialog.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::GradientVectorBase"))
        return static_cast< Ui::GradientVectorBase*>(this);
    return ScrPaletteBase::qt_metacast(_clname);
}

int GradientVectorDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScrPaletteBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 24;
    }
    return _id;
}

// SIGNAL 0
void GradientVectorDialog::NewSpecial(double _t1, double _t2, double _t3, double _t4, double _t5, double _t6, double _t7, double _t8, double _t9, double _t10)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t6))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t7))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t8))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t9))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t10))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void GradientVectorDialog::editGradient(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void GradientVectorDialog::createNewMesh()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void GradientVectorDialog::resetMesh()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void GradientVectorDialog::meshToShape()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void GradientVectorDialog::reset1Control()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void GradientVectorDialog::resetAllControl()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}

// SIGNAL 7
void GradientVectorDialog::removePatch()
{
    QMetaObject::activate(this, &staticMetaObject, 7, nullptr);
}

// SIGNAL 8
void GradientVectorDialog::snapToMGrid(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
