/********************************************************************************
** Form generated from reading UI file 'propertywidget_advancedbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYWIDGET_ADVANCEDBASE_H
#define UI_PROPERTYWIDGET_ADVANCEDBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_PropertyWidget_AdvancedBase
{
public:
    QGridLayout *gridLayout;
    QLabel *textBaseLabel;
    ScrSpinBox *textBase;
    QLabel *trackingLabel;
    ScrSpinBox *tracking;
    QLabel *scaleHLabel;
    ScrSpinBox *scaleH;
    QLabel *scaleVLabel;
    ScrSpinBox *scaleV;
    QLabel *wordTrackingLabel;
    QLabel *minWordTrackingLabel;
    ScrSpinBox *minWordTrackingSpinBox;
    ScrSpinBox *normWordTrackingSpinBox;
    QLabel *normWordTrackingLabel;
    QLabel *glyphExtensionLabel;
    QLabel *minGlyphExtensionLabel;
    ScrSpinBox *minGlyphExtSpinBox;
    ScrSpinBox *maxGlyphExtSpinBox;
    QLabel *maxGlyphExtensionLabel;

    void setupUi(QFrame *PropertyWidget_AdvancedBase)
    {
        if (PropertyWidget_AdvancedBase->objectName().isEmpty())
            PropertyWidget_AdvancedBase->setObjectName(QString::fromUtf8("PropertyWidget_AdvancedBase"));
        PropertyWidget_AdvancedBase->resize(278, 161);
        PropertyWidget_AdvancedBase->setFrameShape(QFrame::StyledPanel);
        PropertyWidget_AdvancedBase->setFrameShadow(QFrame::Raised);
        gridLayout = new QGridLayout(PropertyWidget_AdvancedBase);
        gridLayout->setSpacing(3);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(3, 3, 3, 3);
        textBaseLabel = new QLabel(PropertyWidget_AdvancedBase);
        textBaseLabel->setObjectName(QString::fromUtf8("textBaseLabel"));
        textBaseLabel->setMinimumSize(QSize(20, 0));
        textBaseLabel->setMaximumSize(QSize(16777215, 22));
        textBaseLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(textBaseLabel, 0, 0, 1, 1);

        textBase = new ScrSpinBox(PropertyWidget_AdvancedBase);
        textBase->setObjectName(QString::fromUtf8("textBase"));
        textBase->setMinimum(-100.000000000000000);
        textBase->setMaximum(100.000000000000000);

        gridLayout->addWidget(textBase, 0, 1, 1, 1);

        trackingLabel = new QLabel(PropertyWidget_AdvancedBase);
        trackingLabel->setObjectName(QString::fromUtf8("trackingLabel"));
        trackingLabel->setMinimumSize(QSize(20, 0));
        trackingLabel->setMaximumSize(QSize(16777215, 22));
        trackingLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(trackingLabel, 0, 2, 1, 1);

        tracking = new ScrSpinBox(PropertyWidget_AdvancedBase);
        tracking->setObjectName(QString::fromUtf8("tracking"));

        gridLayout->addWidget(tracking, 0, 3, 1, 1);

        scaleHLabel = new QLabel(PropertyWidget_AdvancedBase);
        scaleHLabel->setObjectName(QString::fromUtf8("scaleHLabel"));
        scaleHLabel->setMinimumSize(QSize(20, 0));
        scaleHLabel->setMaximumSize(QSize(16777215, 22));
        scaleHLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(scaleHLabel, 1, 0, 1, 1);

        scaleH = new ScrSpinBox(PropertyWidget_AdvancedBase);
        scaleH->setObjectName(QString::fromUtf8("scaleH"));

        gridLayout->addWidget(scaleH, 1, 1, 1, 1);

        scaleVLabel = new QLabel(PropertyWidget_AdvancedBase);
        scaleVLabel->setObjectName(QString::fromUtf8("scaleVLabel"));
        scaleVLabel->setMinimumSize(QSize(20, 0));
        scaleVLabel->setMaximumSize(QSize(16777215, 22));
        scaleVLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        gridLayout->addWidget(scaleVLabel, 1, 2, 1, 1);

        scaleV = new ScrSpinBox(PropertyWidget_AdvancedBase);
        scaleV->setObjectName(QString::fromUtf8("scaleV"));

        gridLayout->addWidget(scaleV, 1, 3, 1, 1);

        wordTrackingLabel = new QLabel(PropertyWidget_AdvancedBase);
        wordTrackingLabel->setObjectName(QString::fromUtf8("wordTrackingLabel"));

        gridLayout->addWidget(wordTrackingLabel, 2, 0, 1, 2);

        minWordTrackingLabel = new QLabel(PropertyWidget_AdvancedBase);
        minWordTrackingLabel->setObjectName(QString::fromUtf8("minWordTrackingLabel"));

        gridLayout->addWidget(minWordTrackingLabel, 3, 0, 1, 1);

        minWordTrackingSpinBox = new ScrSpinBox(PropertyWidget_AdvancedBase);
        minWordTrackingSpinBox->setObjectName(QString::fromUtf8("minWordTrackingSpinBox"));
        minWordTrackingSpinBox->setMinimum(1.000000000000000);
        minWordTrackingSpinBox->setMaximum(200.000000000000000);
        minWordTrackingSpinBox->setValue(100.000000000000000);

        gridLayout->addWidget(minWordTrackingSpinBox, 3, 1, 1, 1);

        normWordTrackingSpinBox = new ScrSpinBox(PropertyWidget_AdvancedBase);
        normWordTrackingSpinBox->setObjectName(QString::fromUtf8("normWordTrackingSpinBox"));
        normWordTrackingSpinBox->setMinimum(1.000000000000000);
        normWordTrackingSpinBox->setMaximum(2000.000000000000000);
        normWordTrackingSpinBox->setValue(100.000000000000000);

        gridLayout->addWidget(normWordTrackingSpinBox, 3, 3, 1, 1);

        normWordTrackingLabel = new QLabel(PropertyWidget_AdvancedBase);
        normWordTrackingLabel->setObjectName(QString::fromUtf8("normWordTrackingLabel"));

        gridLayout->addWidget(normWordTrackingLabel, 3, 2, 1, 1);

        glyphExtensionLabel = new QLabel(PropertyWidget_AdvancedBase);
        glyphExtensionLabel->setObjectName(QString::fromUtf8("glyphExtensionLabel"));

        gridLayout->addWidget(glyphExtensionLabel, 4, 0, 1, 2);

        minGlyphExtensionLabel = new QLabel(PropertyWidget_AdvancedBase);
        minGlyphExtensionLabel->setObjectName(QString::fromUtf8("minGlyphExtensionLabel"));

        gridLayout->addWidget(minGlyphExtensionLabel, 5, 0, 1, 1);

        minGlyphExtSpinBox = new ScrSpinBox(PropertyWidget_AdvancedBase);
        minGlyphExtSpinBox->setObjectName(QString::fromUtf8("minGlyphExtSpinBox"));
        minGlyphExtSpinBox->setMinimum(90.000000000000000);
        minGlyphExtSpinBox->setMaximum(110.000000000000000);
        minGlyphExtSpinBox->setValue(100.000000000000000);

        gridLayout->addWidget(minGlyphExtSpinBox, 5, 1, 1, 1);

        maxGlyphExtSpinBox = new ScrSpinBox(PropertyWidget_AdvancedBase);
        maxGlyphExtSpinBox->setObjectName(QString::fromUtf8("maxGlyphExtSpinBox"));
        maxGlyphExtSpinBox->setMinimum(90.000000000000000);
        maxGlyphExtSpinBox->setMaximum(110.000000000000000);
        maxGlyphExtSpinBox->setValue(100.000000000000000);

        gridLayout->addWidget(maxGlyphExtSpinBox, 5, 3, 1, 1);

        maxGlyphExtensionLabel = new QLabel(PropertyWidget_AdvancedBase);
        maxGlyphExtensionLabel->setObjectName(QString::fromUtf8("maxGlyphExtensionLabel"));

        gridLayout->addWidget(maxGlyphExtensionLabel, 5, 2, 1, 1);


        retranslateUi(PropertyWidget_AdvancedBase);

        QMetaObject::connectSlotsByName(PropertyWidget_AdvancedBase);
    } // setupUi

    void retranslateUi(QFrame *PropertyWidget_AdvancedBase)
    {
        textBaseLabel->setText(QString());
#if QT_CONFIG(tooltip)
        textBase->setToolTip(QCoreApplication::translate("PropertyWidget_AdvancedBase", "Offset to baseline of characters", nullptr));
#endif // QT_CONFIG(tooltip)
        textBase->setSuffix(QCoreApplication::translate("PropertyWidget_AdvancedBase", " %", nullptr));
        trackingLabel->setText(QString());
#if QT_CONFIG(tooltip)
        tracking->setToolTip(QCoreApplication::translate("PropertyWidget_AdvancedBase", "Manual Tracking", nullptr));
#endif // QT_CONFIG(tooltip)
        tracking->setSuffix(QCoreApplication::translate("PropertyWidget_AdvancedBase", " %", nullptr));
        scaleHLabel->setText(QString());
#if QT_CONFIG(tooltip)
        scaleH->setToolTip(QCoreApplication::translate("PropertyWidget_AdvancedBase", "Scaling width of characters", nullptr));
#endif // QT_CONFIG(tooltip)
        scaleH->setSuffix(QCoreApplication::translate("PropertyWidget_AdvancedBase", " %", nullptr));
        scaleVLabel->setText(QString());
#if QT_CONFIG(tooltip)
        scaleV->setToolTip(QCoreApplication::translate("PropertyWidget_AdvancedBase", "Scaling height of characters", nullptr));
#endif // QT_CONFIG(tooltip)
        scaleV->setSuffix(QCoreApplication::translate("PropertyWidget_AdvancedBase", " %", nullptr));
        wordTrackingLabel->setText(QCoreApplication::translate("PropertyWidget_AdvancedBase", "Word Tracking", nullptr));
        minWordTrackingLabel->setText(QCoreApplication::translate("PropertyWidget_AdvancedBase", "Min:", nullptr));
#if QT_CONFIG(tooltip)
        minWordTrackingSpinBox->setToolTip(QCoreApplication::translate("PropertyWidget_AdvancedBase", "Minimal width of spaces between words", nullptr));
#endif // QT_CONFIG(tooltip)
        minWordTrackingSpinBox->setSuffix(QCoreApplication::translate("PropertyWidget_AdvancedBase", " %", nullptr));
#if QT_CONFIG(tooltip)
        normWordTrackingSpinBox->setToolTip(QCoreApplication::translate("PropertyWidget_AdvancedBase", "Normal width of spaces between words", nullptr));
#endif // QT_CONFIG(tooltip)
        normWordTrackingSpinBox->setSuffix(QCoreApplication::translate("PropertyWidget_AdvancedBase", " %", nullptr));
        normWordTrackingLabel->setText(QCoreApplication::translate("PropertyWidget_AdvancedBase", "Norm:", nullptr));
        glyphExtensionLabel->setText(QCoreApplication::translate("PropertyWidget_AdvancedBase", "Glyph Extension", nullptr));
        minGlyphExtensionLabel->setText(QCoreApplication::translate("PropertyWidget_AdvancedBase", "Min:", nullptr));
#if QT_CONFIG(tooltip)
        minGlyphExtSpinBox->setToolTip(QCoreApplication::translate("PropertyWidget_AdvancedBase", "Minimal shrinkage of glyphs for justification", nullptr));
#endif // QT_CONFIG(tooltip)
        minGlyphExtSpinBox->setSuffix(QCoreApplication::translate("PropertyWidget_AdvancedBase", " %", nullptr));
#if QT_CONFIG(tooltip)
        maxGlyphExtSpinBox->setToolTip(QCoreApplication::translate("PropertyWidget_AdvancedBase", "Maximal extension of glyphs for justification", nullptr));
#endif // QT_CONFIG(tooltip)
        maxGlyphExtSpinBox->setSuffix(QCoreApplication::translate("PropertyWidget_AdvancedBase", " %", nullptr));
        maxGlyphExtensionLabel->setText(QCoreApplication::translate("PropertyWidget_AdvancedBase", "Max:", nullptr));
        (void)PropertyWidget_AdvancedBase;
    } // retranslateUi

};

namespace Ui {
    class PropertyWidget_AdvancedBase: public Ui_PropertyWidget_AdvancedBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTYWIDGET_ADVANCEDBASE_H
