/****************************************************************************
** Meta object code from reading C++ file 'helpbrowser.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/helpbrowser.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'helpbrowser.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_HelpBrowser_t {
    QByteArrayData data[29];
    char stringdata0[399];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_HelpBrowser_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_HelpBrowser_t qt_meta_stringdata_HelpBrowser = {
    {
QT_MOC_LITERAL(0, 0, 11), // "HelpBrowser"
QT_MOC_LITERAL(1, 12, 6), // "closed"
QT_MOC_LITERAL(2, 19, 0), // ""
QT_MOC_LITERAL(3, 20, 17), // "jumpToHelpSection"
QT_MOC_LITERAL(4, 38, 13), // "jumpToSection"
QT_MOC_LITERAL(5, 52, 10), // "jumpToFile"
QT_MOC_LITERAL(6, 63, 25), // "dontChangeIfAlreadyLoaded"
QT_MOC_LITERAL(7, 89, 14), // "languageChange"
QT_MOC_LITERAL(8, 104, 10), // "histChosen"
QT_MOC_LITERAL(9, 115, 8), // "QAction*"
QT_MOC_LITERAL(10, 124, 1), // "i"
QT_MOC_LITERAL(11, 126, 8), // "loadHelp"
QT_MOC_LITERAL(12, 135, 8), // "fileName"
QT_MOC_LITERAL(13, 144, 8), // "loadMenu"
QT_MOC_LITERAL(14, 153, 12), // "itemSelected"
QT_MOC_LITERAL(15, 166, 14), // "QItemSelection"
QT_MOC_LITERAL(16, 181, 8), // "selected"
QT_MOC_LITERAL(17, 190, 10), // "deselected"
QT_MOC_LITERAL(18, 201, 18), // "itemSearchSelected"
QT_MOC_LITERAL(19, 220, 16), // "QTreeWidgetItem*"
QT_MOC_LITERAL(20, 237, 20), // "itemBookmarkSelected"
QT_MOC_LITERAL(21, 258, 23), // "searchingButton_clicked"
QT_MOC_LITERAL(22, 282, 4), // "find"
QT_MOC_LITERAL(23, 287, 8), // "findNext"
QT_MOC_LITERAL(24, 296, 12), // "findPrevious"
QT_MOC_LITERAL(25, 309, 5), // "print"
QT_MOC_LITERAL(26, 315, 22), // "bookmarkButton_clicked"
QT_MOC_LITERAL(27, 338, 28), // "deleteBookmarkButton_clicked"
QT_MOC_LITERAL(28, 367, 31) // "deleteAllBookmarkButton_clicked"

    },
    "HelpBrowser\0closed\0\0jumpToHelpSection\0"
    "jumpToSection\0jumpToFile\0"
    "dontChangeIfAlreadyLoaded\0languageChange\0"
    "histChosen\0QAction*\0i\0loadHelp\0fileName\0"
    "loadMenu\0itemSelected\0QItemSelection\0"
    "selected\0deselected\0itemSearchSelected\0"
    "QTreeWidgetItem*\0itemBookmarkSelected\0"
    "searchingButton_clicked\0find\0findNext\0"
    "findPrevious\0print\0bookmarkButton_clicked\0"
    "deleteBookmarkButton_clicked\0"
    "deleteAllBookmarkButton_clicked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_HelpBrowser[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      19,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  109,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    3,  110,    2, 0x0a /* Public */,
       3,    2,  117,    2, 0x2a /* Public | MethodCloned */,
       3,    1,  122,    2, 0x2a /* Public | MethodCloned */,
       7,    0,  125,    2, 0x09 /* Protected */,
       8,    1,  126,    2, 0x09 /* Protected */,
      11,    1,  129,    2, 0x09 /* Protected */,
      13,    0,  132,    2, 0x09 /* Protected */,
      14,    2,  133,    2, 0x09 /* Protected */,
      18,    2,  138,    2, 0x09 /* Protected */,
      20,    2,  143,    2, 0x09 /* Protected */,
      21,    0,  148,    2, 0x09 /* Protected */,
      22,    0,  149,    2, 0x09 /* Protected */,
      23,    0,  150,    2, 0x09 /* Protected */,
      24,    0,  151,    2, 0x09 /* Protected */,
      25,    0,  152,    2, 0x09 /* Protected */,
      26,    0,  153,    2, 0x09 /* Protected */,
      27,    0,  154,    2, 0x09 /* Protected */,
      28,    0,  155,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Bool,    4,    5,    6,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,    4,    5,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 15, 0x80000000 | 15,   16,   17,
    QMetaType::Void, 0x80000000 | 19, QMetaType::Int,    2,    2,
    QMetaType::Void, 0x80000000 | 19, QMetaType::Int,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void HelpBrowser::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<HelpBrowser *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->closed(); break;
        case 1: _t->jumpToHelpSection((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 2: _t->jumpToHelpSection((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 3: _t->jumpToHelpSection((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->languageChange(); break;
        case 5: _t->histChosen((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 6: _t->loadHelp((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 7: _t->loadMenu(); break;
        case 8: _t->itemSelected((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 9: _t->itemSearchSelected((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 10: _t->itemBookmarkSelected((*reinterpret_cast< QTreeWidgetItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 11: _t->searchingButton_clicked(); break;
        case 12: _t->find(); break;
        case 13: _t->findNext(); break;
        case 14: _t->findPrevious(); break;
        case 15: _t->print(); break;
        case 16: _t->bookmarkButton_clicked(); break;
        case 17: _t->deleteBookmarkButton_clicked(); break;
        case 18: _t->deleteAllBookmarkButton_clicked(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QAction* >(); break;
            }
            break;
        case 8:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QItemSelection >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (HelpBrowser::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&HelpBrowser::closed)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject HelpBrowser::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_HelpBrowser.data,
    qt_meta_data_HelpBrowser,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *HelpBrowser::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HelpBrowser::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_HelpBrowser.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::HelpBrowser"))
        return static_cast< Ui::HelpBrowser*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int HelpBrowser::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 19)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 19;
    }
    return _id;
}

// SIGNAL 0
void HelpBrowser::closed()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
