/****************************************************************************
** Meta object code from reading C++ file 'notesstyleseditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/notesstyleseditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'notesstyleseditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_NotesStylesEditor_t {
    QByteArrayData data[35];
    char stringdata0[681];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_NotesStylesEditor_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_NotesStylesEditor_t qt_meta_stringdata_NotesStylesEditor = {
    {
QT_MOC_LITERAL(0, 0, 17), // "NotesStylesEditor"
QT_MOC_LITERAL(1, 18, 6), // "setDoc"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 11), // "ScribusDoc*"
QT_MOC_LITERAL(4, 38, 3), // "doc"
QT_MOC_LITERAL(5, 42, 19), // "handleUpdateRequest"
QT_MOC_LITERAL(6, 62, 11), // "updateFlags"
QT_MOC_LITERAL(7, 74, 14), // "languageChange"
QT_MOC_LITERAL(8, 89, 13), // "setNotesStyle"
QT_MOC_LITERAL(9, 103, 11), // "NotesStyle*"
QT_MOC_LITERAL(10, 115, 2), // "NS"
QT_MOC_LITERAL(11, 118, 32), // "on_NSlistBox_currentIndexChanged"
QT_MOC_LITERAL(12, 151, 4), // "arg1"
QT_MOC_LITERAL(13, 156, 22), // "on_ApplyButton_clicked"
QT_MOC_LITERAL(14, 179, 23), // "on_DeleteButton_clicked"
QT_MOC_LITERAL(15, 203, 19), // "on_OKButton_clicked"
QT_MOC_LITERAL(16, 223, 26), // "on_NewNameEdit_textChanged"
QT_MOC_LITERAL(17, 250, 20), // "on_FootRadio_toggled"
QT_MOC_LITERAL(18, 271, 7), // "checked"
QT_MOC_LITERAL(19, 279, 19), // "on_EndRadio_toggled"
QT_MOC_LITERAL(20, 299, 35), // "on_NumberingBox_currentIndexC..."
QT_MOC_LITERAL(21, 335, 5), // "index"
QT_MOC_LITERAL(22, 341, 31), // "on_RangeBox_currentIndexChanged"
QT_MOC_LITERAL(23, 373, 28), // "on_StartSpinBox_valueChanged"
QT_MOC_LITERAL(24, 402, 25), // "on_PrefixEdit_textChanged"
QT_MOC_LITERAL(25, 428, 25), // "on_SuffixEdit_textChanged"
QT_MOC_LITERAL(26, 454, 27), // "on_SuperMasterCheck_toggled"
QT_MOC_LITERAL(27, 482, 25), // "on_SuperNoteCheck_toggled"
QT_MOC_LITERAL(28, 508, 16), // "on_AutoH_toggled"
QT_MOC_LITERAL(29, 525, 16), // "on_AutoW_toggled"
QT_MOC_LITERAL(30, 542, 19), // "on_AutoWeld_toggled"
QT_MOC_LITERAL(31, 562, 21), // "on_AutoRemove_toggled"
QT_MOC_LITERAL(32, 584, 20), // "on_NewButton_clicked"
QT_MOC_LITERAL(33, 605, 37), // "on_paraStyleCombo_currentInde..."
QT_MOC_LITERAL(34, 643, 37) // "on_charStyleCombo_currentInde..."

    },
    "NotesStylesEditor\0setDoc\0\0ScribusDoc*\0"
    "doc\0handleUpdateRequest\0updateFlags\0"
    "languageChange\0setNotesStyle\0NotesStyle*\0"
    "NS\0on_NSlistBox_currentIndexChanged\0"
    "arg1\0on_ApplyButton_clicked\0"
    "on_DeleteButton_clicked\0on_OKButton_clicked\0"
    "on_NewNameEdit_textChanged\0"
    "on_FootRadio_toggled\0checked\0"
    "on_EndRadio_toggled\0"
    "on_NumberingBox_currentIndexChanged\0"
    "index\0on_RangeBox_currentIndexChanged\0"
    "on_StartSpinBox_valueChanged\0"
    "on_PrefixEdit_textChanged\0"
    "on_SuffixEdit_textChanged\0"
    "on_SuperMasterCheck_toggled\0"
    "on_SuperNoteCheck_toggled\0on_AutoH_toggled\0"
    "on_AutoW_toggled\0on_AutoWeld_toggled\0"
    "on_AutoRemove_toggled\0on_NewButton_clicked\0"
    "on_paraStyleCombo_currentIndexChanged\0"
    "on_charStyleCombo_currentIndexChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_NotesStylesEditor[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      25,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  139,    2, 0x0a /* Public */,
       5,    1,  142,    2, 0x0a /* Public */,
       7,    0,  145,    2, 0x0a /* Public */,
       8,    1,  146,    2, 0x0a /* Public */,
      11,    1,  149,    2, 0x08 /* Private */,
      13,    0,  152,    2, 0x08 /* Private */,
      14,    0,  153,    2, 0x08 /* Private */,
      15,    0,  154,    2, 0x08 /* Private */,
      16,    1,  155,    2, 0x08 /* Private */,
      17,    1,  158,    2, 0x08 /* Private */,
      19,    1,  161,    2, 0x08 /* Private */,
      20,    1,  164,    2, 0x08 /* Private */,
      22,    1,  167,    2, 0x08 /* Private */,
      23,    1,  170,    2, 0x08 /* Private */,
      24,    1,  173,    2, 0x08 /* Private */,
      25,    1,  176,    2, 0x08 /* Private */,
      26,    1,  179,    2, 0x08 /* Private */,
      27,    1,  182,    2, 0x08 /* Private */,
      28,    1,  185,    2, 0x08 /* Private */,
      29,    1,  188,    2, 0x08 /* Private */,
      30,    1,  191,    2, 0x08 /* Private */,
      31,    1,  194,    2, 0x08 /* Private */,
      32,    0,  197,    2, 0x08 /* Private */,
      33,    1,  198,    2, 0x08 /* Private */,
      34,    1,  201,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9,   10,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Void, QMetaType::Int,   21,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::QString,   12,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   12,
    QMetaType::Void, QMetaType::Int,   12,

       0        // eod
};

void NotesStylesEditor::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<NotesStylesEditor *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 1: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->languageChange(); break;
        case 3: _t->setNotesStyle((*reinterpret_cast< NotesStyle*(*)>(_a[1]))); break;
        case 4: _t->on_NSlistBox_currentIndexChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->on_ApplyButton_clicked(); break;
        case 6: _t->on_DeleteButton_clicked(); break;
        case 7: _t->on_OKButton_clicked(); break;
        case 8: _t->on_NewNameEdit_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->on_FootRadio_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->on_EndRadio_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 11: _t->on_NumberingBox_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 12: _t->on_RangeBox_currentIndexChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->on_StartSpinBox_valueChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->on_PrefixEdit_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 15: _t->on_SuffixEdit_textChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 16: _t->on_SuperMasterCheck_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 17: _t->on_SuperNoteCheck_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 18: _t->on_AutoH_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 19: _t->on_AutoW_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 20: _t->on_AutoWeld_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 21: _t->on_AutoRemove_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 22: _t->on_NewButton_clicked(); break;
        case 23: _t->on_paraStyleCombo_currentIndexChanged((*reinterpret_cast< const int(*)>(_a[1]))); break;
        case 24: _t->on_charStyleCombo_currentIndexChanged((*reinterpret_cast< const int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject NotesStylesEditor::staticMetaObject = { {
    QMetaObject::SuperData::link<ScrPaletteBase::staticMetaObject>(),
    qt_meta_stringdata_NotesStylesEditor.data,
    qt_meta_data_NotesStylesEditor,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *NotesStylesEditor::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *NotesStylesEditor::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_NotesStylesEditor.stringdata0))
        return static_cast<void*>(this);
    return ScrPaletteBase::qt_metacast(_clname);
}

int NotesStylesEditor::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScrPaletteBase::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 25;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
