/********************************************************************************
** Form generated from reading UI file 'insertaframe.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_INSERTAFRAME_H
#define UI_INSERTAFRAME_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_InsertAFrame
{
public:
    QVBoxLayout *vboxLayout;
    QTabWidget *tabWidget;
    QWidget *typeTab;
    QHBoxLayout *hboxLayout;
    QVBoxLayout *typeGroupBox;
    QRadioButton *radioButtonTextFrame;
    QRadioButton *radioButtonImageFrame;
    QSpacerItem *spacerItem;
    QTextEdit *typeTextEdit;
    QWidget *locationTab;
    QHBoxLayout *hboxLayout1;
    QGroupBox *pagePlacementGroupBox;
    QVBoxLayout *vboxLayout1;
    QRadioButton *radioButtonCurrentPage;
    QRadioButton *radioButtonAllPages;
    QRadioButton *radioButtonRangeOfPages;
    QHBoxLayout *hboxLayout2;
    QLineEdit *placementPagesLineEdit;
    QToolButton *placementPagesRangeButton;
    QSpacerItem *spacerItem1;
    QGroupBox *framePositionGroupBox;
    QVBoxLayout *vboxLayout2;
    QRadioButton *radioButtonTopLeftOfMargins;
    QRadioButton *radioButtonTopLeftOfPage;
    QRadioButton *radioButtonTopLeftOfBleed;
    QRadioButton *radioButtonCustomPosition;
    QHBoxLayout *hboxLayout3;
    QSpacerItem *spacerItem2;
    QGridLayout *gridLayout;
    QLabel *label_2;
    QLabel *label;
    ScrSpinBox *yPosScrSpinBox;
    ScrSpinBox *xPosScrSpinBox;
    QSpacerItem *spacerItem3;
    QSpacerItem *spacerItem4;
    QWidget *sizeTab;
    QHBoxLayout *hboxLayout4;
    QVBoxLayout *sizeGroupBox;
    QRadioButton *radioButtonPageMarginSize;
    QRadioButton *radioButtonPageSize;
    QRadioButton *radioButtonBleedSize;
    QRadioButton *radioButtonImageSize;
    QRadioButton *radioButtonCustomSize;
    QHBoxLayout *hboxLayout5;
    QSpacerItem *spacerItem5;
    QGridLayout *gridLayout1;
    QLabel *label_3;
    QLabel *label_4;
    ScrSpinBox *widthScrSpinBox;
    ScrSpinBox *heightScrSpinBox;
    QSpacerItem *spacerItem6;
    QSpacerItem *spacerItem7;
    QWidget *optionsTab;
    QVBoxLayout *vboxLayout3;
    QStackedWidget *optionsStackedWidget;
    QWidget *page;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *hboxLayout6;
    QGridLayout *gridLayout2;
    QSpinBox *textColumnCountSpinBox;
    QLabel *textColumnGapLabel;
    ScrSpinBox *textColumnGapScrSpinBox;
    QLabel *textColumnCountLabel;
    QSpacerItem *spacerItem8;
    QCheckBox *checkBoxLinkCreatedTextFrames;
    QHBoxLayout *horizontalLayout;
    QCheckBox *checkBoxLinkToExistingFrame;
    QComboBox *comboBoxLinkToExistingFrameName;
    QHBoxLayout *hboxLayout7;
    QLabel *sourceDocLabel;
    QLineEdit *sourceDocLineEdit;
    QToolButton *selectDocFileButton;
    QSpacerItem *spacerItem9;
    QWidget *page_2;
    QVBoxLayout *vboxLayout4;
    QHBoxLayout *hboxLayout8;
    QLabel *sourceImageLabel;
    QLineEdit *sourceImageLineEdit;
    QToolButton *selectImageFileButton;
    QSpacerItem *spacerItem10;
    QWidget *page_3;
    QVBoxLayout *vboxLayout5;
    QLabel *label_9;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *InsertAFrame)
    {
        if (InsertAFrame->objectName().isEmpty())
            InsertAFrame->setObjectName(QString::fromUtf8("InsertAFrame"));
        InsertAFrame->resize(467, 328);
        vboxLayout = new QVBoxLayout(InsertAFrame);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        tabWidget = new QTabWidget(InsertAFrame);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        typeTab = new QWidget();
        typeTab->setObjectName(QString::fromUtf8("typeTab"));
        hboxLayout = new QHBoxLayout(typeTab);
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        typeGroupBox = new QVBoxLayout();
        typeGroupBox->setObjectName(QString::fromUtf8("typeGroupBox"));
        radioButtonTextFrame = new QRadioButton(typeTab);
        radioButtonTextFrame->setObjectName(QString::fromUtf8("radioButtonTextFrame"));

        typeGroupBox->addWidget(radioButtonTextFrame);

        radioButtonImageFrame = new QRadioButton(typeTab);
        radioButtonImageFrame->setObjectName(QString::fromUtf8("radioButtonImageFrame"));

        typeGroupBox->addWidget(radioButtonImageFrame);

        spacerItem = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        typeGroupBox->addItem(spacerItem);


        hboxLayout->addLayout(typeGroupBox);

        typeTextEdit = new QTextEdit(typeTab);
        typeTextEdit->setObjectName(QString::fromUtf8("typeTextEdit"));
        typeTextEdit->setReadOnly(true);

        hboxLayout->addWidget(typeTextEdit);

        tabWidget->addTab(typeTab, QString());
        locationTab = new QWidget();
        locationTab->setObjectName(QString::fromUtf8("locationTab"));
        hboxLayout1 = new QHBoxLayout(locationTab);
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        pagePlacementGroupBox = new QGroupBox(locationTab);
        pagePlacementGroupBox->setObjectName(QString::fromUtf8("pagePlacementGroupBox"));
        vboxLayout1 = new QVBoxLayout(pagePlacementGroupBox);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        radioButtonCurrentPage = new QRadioButton(pagePlacementGroupBox);
        radioButtonCurrentPage->setObjectName(QString::fromUtf8("radioButtonCurrentPage"));

        vboxLayout1->addWidget(radioButtonCurrentPage);

        radioButtonAllPages = new QRadioButton(pagePlacementGroupBox);
        radioButtonAllPages->setObjectName(QString::fromUtf8("radioButtonAllPages"));

        vboxLayout1->addWidget(radioButtonAllPages);

        radioButtonRangeOfPages = new QRadioButton(pagePlacementGroupBox);
        radioButtonRangeOfPages->setObjectName(QString::fromUtf8("radioButtonRangeOfPages"));

        vboxLayout1->addWidget(radioButtonRangeOfPages);

        hboxLayout2 = new QHBoxLayout();
        hboxLayout2->setObjectName(QString::fromUtf8("hboxLayout2"));
        placementPagesLineEdit = new QLineEdit(pagePlacementGroupBox);
        placementPagesLineEdit->setObjectName(QString::fromUtf8("placementPagesLineEdit"));

        hboxLayout2->addWidget(placementPagesLineEdit);

        placementPagesRangeButton = new QToolButton(pagePlacementGroupBox);
        placementPagesRangeButton->setObjectName(QString::fromUtf8("placementPagesRangeButton"));

        hboxLayout2->addWidget(placementPagesRangeButton);


        vboxLayout1->addLayout(hboxLayout2);

        spacerItem1 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout1->addItem(spacerItem1);


        hboxLayout1->addWidget(pagePlacementGroupBox);

        framePositionGroupBox = new QGroupBox(locationTab);
        framePositionGroupBox->setObjectName(QString::fromUtf8("framePositionGroupBox"));
        vboxLayout2 = new QVBoxLayout(framePositionGroupBox);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        radioButtonTopLeftOfMargins = new QRadioButton(framePositionGroupBox);
        radioButtonTopLeftOfMargins->setObjectName(QString::fromUtf8("radioButtonTopLeftOfMargins"));

        vboxLayout2->addWidget(radioButtonTopLeftOfMargins);

        radioButtonTopLeftOfPage = new QRadioButton(framePositionGroupBox);
        radioButtonTopLeftOfPage->setObjectName(QString::fromUtf8("radioButtonTopLeftOfPage"));

        vboxLayout2->addWidget(radioButtonTopLeftOfPage);

        radioButtonTopLeftOfBleed = new QRadioButton(framePositionGroupBox);
        radioButtonTopLeftOfBleed->setObjectName(QString::fromUtf8("radioButtonTopLeftOfBleed"));

        vboxLayout2->addWidget(radioButtonTopLeftOfBleed);

        radioButtonCustomPosition = new QRadioButton(framePositionGroupBox);
        radioButtonCustomPosition->setObjectName(QString::fromUtf8("radioButtonCustomPosition"));

        vboxLayout2->addWidget(radioButtonCustomPosition);

        hboxLayout3 = new QHBoxLayout();
        hboxLayout3->setObjectName(QString::fromUtf8("hboxLayout3"));
        spacerItem2 = new QSpacerItem(30, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout3->addItem(spacerItem2);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label_2 = new QLabel(framePositionGroupBox);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        label = new QLabel(framePositionGroupBox);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 0, 1, 1);

        yPosScrSpinBox = new ScrSpinBox(framePositionGroupBox);
        yPosScrSpinBox->setObjectName(QString::fromUtf8("yPosScrSpinBox"));

        gridLayout->addWidget(yPosScrSpinBox, 1, 1, 1, 1);

        xPosScrSpinBox = new ScrSpinBox(framePositionGroupBox);
        xPosScrSpinBox->setObjectName(QString::fromUtf8("xPosScrSpinBox"));

        gridLayout->addWidget(xPosScrSpinBox, 0, 1, 1, 1);


        hboxLayout3->addLayout(gridLayout);

        spacerItem3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout3->addItem(spacerItem3);


        vboxLayout2->addLayout(hboxLayout3);

        spacerItem4 = new QSpacerItem(20, 31, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout2->addItem(spacerItem4);


        hboxLayout1->addWidget(framePositionGroupBox);

        tabWidget->addTab(locationTab, QString());
        sizeTab = new QWidget();
        sizeTab->setObjectName(QString::fromUtf8("sizeTab"));
        hboxLayout4 = new QHBoxLayout(sizeTab);
        hboxLayout4->setObjectName(QString::fromUtf8("hboxLayout4"));
        sizeGroupBox = new QVBoxLayout();
        sizeGroupBox->setObjectName(QString::fromUtf8("sizeGroupBox"));
        sizeGroupBox->setContentsMargins(0, 0, 0, 0);
        radioButtonPageMarginSize = new QRadioButton(sizeTab);
        radioButtonPageMarginSize->setObjectName(QString::fromUtf8("radioButtonPageMarginSize"));

        sizeGroupBox->addWidget(radioButtonPageMarginSize);

        radioButtonPageSize = new QRadioButton(sizeTab);
        radioButtonPageSize->setObjectName(QString::fromUtf8("radioButtonPageSize"));

        sizeGroupBox->addWidget(radioButtonPageSize);

        radioButtonBleedSize = new QRadioButton(sizeTab);
        radioButtonBleedSize->setObjectName(QString::fromUtf8("radioButtonBleedSize"));

        sizeGroupBox->addWidget(radioButtonBleedSize);

        radioButtonImageSize = new QRadioButton(sizeTab);
        radioButtonImageSize->setObjectName(QString::fromUtf8("radioButtonImageSize"));

        sizeGroupBox->addWidget(radioButtonImageSize);

        radioButtonCustomSize = new QRadioButton(sizeTab);
        radioButtonCustomSize->setObjectName(QString::fromUtf8("radioButtonCustomSize"));

        sizeGroupBox->addWidget(radioButtonCustomSize);

        hboxLayout5 = new QHBoxLayout();
        hboxLayout5->setObjectName(QString::fromUtf8("hboxLayout5"));
        spacerItem5 = new QSpacerItem(30, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        hboxLayout5->addItem(spacerItem5);

        gridLayout1 = new QGridLayout();
        gridLayout1->setObjectName(QString::fromUtf8("gridLayout1"));
        label_3 = new QLabel(sizeTab);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout1->addWidget(label_3, 1, 0, 1, 1);

        label_4 = new QLabel(sizeTab);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        gridLayout1->addWidget(label_4, 0, 0, 1, 1);

        widthScrSpinBox = new ScrSpinBox(sizeTab);
        widthScrSpinBox->setObjectName(QString::fromUtf8("widthScrSpinBox"));

        gridLayout1->addWidget(widthScrSpinBox, 0, 1, 1, 1);

        heightScrSpinBox = new ScrSpinBox(sizeTab);
        heightScrSpinBox->setObjectName(QString::fromUtf8("heightScrSpinBox"));

        gridLayout1->addWidget(heightScrSpinBox, 1, 1, 1, 1);


        hboxLayout5->addLayout(gridLayout1);

        spacerItem6 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout5->addItem(spacerItem6);


        sizeGroupBox->addLayout(hboxLayout5);

        spacerItem7 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        sizeGroupBox->addItem(spacerItem7);


        hboxLayout4->addLayout(sizeGroupBox);

        tabWidget->addTab(sizeTab, QString());
        optionsTab = new QWidget();
        optionsTab->setObjectName(QString::fromUtf8("optionsTab"));
        vboxLayout3 = new QVBoxLayout(optionsTab);
        vboxLayout3->setObjectName(QString::fromUtf8("vboxLayout3"));
        optionsStackedWidget = new QStackedWidget(optionsTab);
        optionsStackedWidget->setObjectName(QString::fromUtf8("optionsStackedWidget"));
        page = new QWidget();
        page->setObjectName(QString::fromUtf8("page"));
        verticalLayout = new QVBoxLayout(page);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        hboxLayout6 = new QHBoxLayout();
        hboxLayout6->setObjectName(QString::fromUtf8("hboxLayout6"));
        gridLayout2 = new QGridLayout();
        gridLayout2->setObjectName(QString::fromUtf8("gridLayout2"));
        textColumnCountSpinBox = new QSpinBox(page);
        textColumnCountSpinBox->setObjectName(QString::fromUtf8("textColumnCountSpinBox"));
        textColumnCountSpinBox->setMinimum(1);

        gridLayout2->addWidget(textColumnCountSpinBox, 0, 1, 1, 1);

        textColumnGapLabel = new QLabel(page);
        textColumnGapLabel->setObjectName(QString::fromUtf8("textColumnGapLabel"));

        gridLayout2->addWidget(textColumnGapLabel, 1, 0, 1, 1);

        textColumnGapScrSpinBox = new ScrSpinBox(page);
        textColumnGapScrSpinBox->setObjectName(QString::fromUtf8("textColumnGapScrSpinBox"));

        gridLayout2->addWidget(textColumnGapScrSpinBox, 1, 1, 1, 1);

        textColumnCountLabel = new QLabel(page);
        textColumnCountLabel->setObjectName(QString::fromUtf8("textColumnCountLabel"));

        gridLayout2->addWidget(textColumnCountLabel, 0, 0, 1, 1);


        hboxLayout6->addLayout(gridLayout2);

        spacerItem8 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout6->addItem(spacerItem8);


        verticalLayout->addLayout(hboxLayout6);

        checkBoxLinkCreatedTextFrames = new QCheckBox(page);
        checkBoxLinkCreatedTextFrames->setObjectName(QString::fromUtf8("checkBoxLinkCreatedTextFrames"));

        verticalLayout->addWidget(checkBoxLinkCreatedTextFrames);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        checkBoxLinkToExistingFrame = new QCheckBox(page);
        checkBoxLinkToExistingFrame->setObjectName(QString::fromUtf8("checkBoxLinkToExistingFrame"));

        horizontalLayout->addWidget(checkBoxLinkToExistingFrame);

        comboBoxLinkToExistingFrameName = new QComboBox(page);
        comboBoxLinkToExistingFrameName->setObjectName(QString::fromUtf8("comboBoxLinkToExistingFrameName"));
        comboBoxLinkToExistingFrameName->setEnabled(false);
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(comboBoxLinkToExistingFrameName->sizePolicy().hasHeightForWidth());
        comboBoxLinkToExistingFrameName->setSizePolicy(sizePolicy);

        horizontalLayout->addWidget(comboBoxLinkToExistingFrameName);


        verticalLayout->addLayout(horizontalLayout);

        hboxLayout7 = new QHBoxLayout();
        hboxLayout7->setObjectName(QString::fromUtf8("hboxLayout7"));
        sourceDocLabel = new QLabel(page);
        sourceDocLabel->setObjectName(QString::fromUtf8("sourceDocLabel"));

        hboxLayout7->addWidget(sourceDocLabel);

        sourceDocLineEdit = new QLineEdit(page);
        sourceDocLineEdit->setObjectName(QString::fromUtf8("sourceDocLineEdit"));

        hboxLayout7->addWidget(sourceDocLineEdit);

        selectDocFileButton = new QToolButton(page);
        selectDocFileButton->setObjectName(QString::fromUtf8("selectDocFileButton"));

        hboxLayout7->addWidget(selectDocFileButton);


        verticalLayout->addLayout(hboxLayout7);

        spacerItem9 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(spacerItem9);

        optionsStackedWidget->addWidget(page);
        page_2 = new QWidget();
        page_2->setObjectName(QString::fromUtf8("page_2"));
        vboxLayout4 = new QVBoxLayout(page_2);
        vboxLayout4->setObjectName(QString::fromUtf8("vboxLayout4"));
        hboxLayout8 = new QHBoxLayout();
        hboxLayout8->setObjectName(QString::fromUtf8("hboxLayout8"));
        sourceImageLabel = new QLabel(page_2);
        sourceImageLabel->setObjectName(QString::fromUtf8("sourceImageLabel"));

        hboxLayout8->addWidget(sourceImageLabel);

        sourceImageLineEdit = new QLineEdit(page_2);
        sourceImageLineEdit->setObjectName(QString::fromUtf8("sourceImageLineEdit"));

        hboxLayout8->addWidget(sourceImageLineEdit);

        selectImageFileButton = new QToolButton(page_2);
        selectImageFileButton->setObjectName(QString::fromUtf8("selectImageFileButton"));

        hboxLayout8->addWidget(selectImageFileButton);


        vboxLayout4->addLayout(hboxLayout8);

        spacerItem10 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        vboxLayout4->addItem(spacerItem10);

        optionsStackedWidget->addWidget(page_2);
        page_3 = new QWidget();
        page_3->setObjectName(QString::fromUtf8("page_3"));
        vboxLayout5 = new QVBoxLayout(page_3);
        vboxLayout5->setObjectName(QString::fromUtf8("vboxLayout5"));
        label_9 = new QLabel(page_3);
        label_9->setObjectName(QString::fromUtf8("label_9"));
        label_9->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);

        vboxLayout5->addWidget(label_9);

        optionsStackedWidget->addWidget(page_3);

        vboxLayout3->addWidget(optionsStackedWidget);

        tabWidget->addTab(optionsTab, QString());

        vboxLayout->addWidget(tabWidget);

        buttonBox = new QDialogButtonBox(InsertAFrame);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(InsertAFrame);
        QObject::connect(buttonBox, SIGNAL(accepted()), InsertAFrame, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), InsertAFrame, SLOT(reject()));

        tabWidget->setCurrentIndex(0);
        optionsStackedWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(InsertAFrame);
    } // setupUi

    void retranslateUi(QDialog *InsertAFrame)
    {
        InsertAFrame->setWindowTitle(QCoreApplication::translate("InsertAFrame", "Insert Frames", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonTextFrame->setToolTip(QCoreApplication::translate("InsertAFrame", "Insert one or more text frames", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonTextFrame->setText(QCoreApplication::translate("InsertAFrame", "&Text Frame", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonImageFrame->setToolTip(QCoreApplication::translate("InsertAFrame", "Insert one or more image frames", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonImageFrame->setText(QCoreApplication::translate("InsertAFrame", "&Image Frame", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(typeTab), QCoreApplication::translate("InsertAFrame", "T&ype", nullptr));
#if QT_CONFIG(tooltip)
        pagePlacementGroupBox->setToolTip(QCoreApplication::translate("InsertAFrame", "Place the new frames on the current page, on all pages or on a selected range", nullptr));
#endif // QT_CONFIG(tooltip)
        pagePlacementGroupBox->setTitle(QCoreApplication::translate("InsertAFrame", "Page Placement", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonCurrentPage->setToolTip(QCoreApplication::translate("InsertAFrame", "Insert the frame on the current page only", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonCurrentPage->setText(QCoreApplication::translate("InsertAFrame", "Current Page", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonAllPages->setToolTip(QCoreApplication::translate("InsertAFrame", "Insert one frame for each existing page", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonAllPages->setText(QCoreApplication::translate("InsertAFrame", "All Pages", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonRangeOfPages->setToolTip(QCoreApplication::translate("InsertAFrame", "Insert frames on a range of pages", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonRangeOfPages->setText(QCoreApplication::translate("InsertAFrame", "Range of Pages", nullptr));
#if QT_CONFIG(tooltip)
        placementPagesLineEdit->setToolTip(QCoreApplication::translate("InsertAFrame", "Range of pages to insert frames on", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        placementPagesRangeButton->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        framePositionGroupBox->setToolTip(QCoreApplication::translate("InsertAFrame", "Position the new frame in relation to the page", nullptr));
#endif // QT_CONFIG(tooltip)
        framePositionGroupBox->setTitle(QCoreApplication::translate("InsertAFrame", "Position of Frame", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonTopLeftOfMargins->setToolTip(QCoreApplication::translate("InsertAFrame", "Insert the frame at the top left of the page margins", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonTopLeftOfMargins->setText(QCoreApplication::translate("InsertAFrame", "Top Left of Margins", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonTopLeftOfPage->setToolTip(QCoreApplication::translate("InsertAFrame", "Insert the frame at the top left of the page", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonTopLeftOfPage->setText(QCoreApplication::translate("InsertAFrame", "Top Left of Page", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonTopLeftOfBleed->setToolTip(QCoreApplication::translate("InsertAFrame", "Insert the frame at the top left of the page bleed", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonTopLeftOfBleed->setText(QCoreApplication::translate("InsertAFrame", "Top Left of Bleed", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonCustomPosition->setToolTip(QCoreApplication::translate("InsertAFrame", "Insert the frame at a custom position on the page", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonCustomPosition->setText(QCoreApplication::translate("InsertAFrame", "Custom Position", nullptr));
        label_2->setText(QCoreApplication::translate("InsertAFrame", "Y:", nullptr));
        label->setText(QCoreApplication::translate("InsertAFrame", "X:", nullptr));
#if QT_CONFIG(tooltip)
        yPosScrSpinBox->setToolTip(QCoreApplication::translate("InsertAFrame", "Top position of the inserted frame", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        xPosScrSpinBox->setToolTip(QCoreApplication::translate("InsertAFrame", "Left position of the inserted frame", nullptr));
#endif // QT_CONFIG(tooltip)
        tabWidget->setTabText(tabWidget->indexOf(locationTab), QCoreApplication::translate("InsertAFrame", "&Location", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonPageMarginSize->setToolTip(QCoreApplication::translate("InsertAFrame", "Insert the new frame with the same dimensions as the page margins", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonPageMarginSize->setText(QCoreApplication::translate("InsertAFrame", "Same as the Page Margins", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonPageSize->setToolTip(QCoreApplication::translate("InsertAFrame", "Insert the new frame with the same dimensions as the page", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonPageSize->setText(QCoreApplication::translate("InsertAFrame", "Same as the Page", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonBleedSize->setToolTip(QCoreApplication::translate("InsertAFrame", "Insert the new frame with the same dimensions as the bleed area outside the boundary of the page itself", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonBleedSize->setText(QCoreApplication::translate("InsertAFrame", "Same as the Bleed", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonImageSize->setToolTip(QCoreApplication::translate("InsertAFrame", "Insert the new frame with the same dimensions as the image that will be imported", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonImageSize->setText(QCoreApplication::translate("InsertAFrame", "Same as the Imported Image", nullptr));
#if QT_CONFIG(tooltip)
        radioButtonCustomSize->setToolTip(QCoreApplication::translate("InsertAFrame", "Insert the new frame with a custom size", nullptr));
#endif // QT_CONFIG(tooltip)
        radioButtonCustomSize->setText(QCoreApplication::translate("InsertAFrame", "Custom Size", nullptr));
        label_3->setText(QCoreApplication::translate("InsertAFrame", "Height:", nullptr));
        label_4->setText(QCoreApplication::translate("InsertAFrame", "Width:", nullptr));
#if QT_CONFIG(tooltip)
        widthScrSpinBox->setToolTip(QCoreApplication::translate("InsertAFrame", "Width of the inserted frame", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        heightScrSpinBox->setToolTip(QCoreApplication::translate("InsertAFrame", "Height of the inserted frame", nullptr));
#endif // QT_CONFIG(tooltip)
        tabWidget->setTabText(tabWidget->indexOf(sizeTab), QCoreApplication::translate("InsertAFrame", "&Size", nullptr));
#if QT_CONFIG(tooltip)
        textColumnCountSpinBox->setToolTip(QCoreApplication::translate("InsertAFrame", "Number of columns for the inserted text frame", nullptr));
#endif // QT_CONFIG(tooltip)
        textColumnGapLabel->setText(QCoreApplication::translate("InsertAFrame", "Gap:", nullptr));
#if QT_CONFIG(tooltip)
        textColumnGapScrSpinBox->setToolTip(QCoreApplication::translate("InsertAFrame", "Distance between the columns in the text frame", nullptr));
#endif // QT_CONFIG(tooltip)
        textColumnCountLabel->setText(QCoreApplication::translate("InsertAFrame", "Columns:", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxLinkCreatedTextFrames->setToolTip(QCoreApplication::translate("InsertAFrame", "Link the inserted text frames together to form a chain of frames", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxLinkCreatedTextFrames->setText(QCoreApplication::translate("InsertAFrame", "Link Inserted Frames", nullptr));
#if QT_CONFIG(tooltip)
        checkBoxLinkToExistingFrame->setToolTip(QCoreApplication::translate("InsertAFrame", "Link the first inserted frame to a preexisting text frame", nullptr));
#endif // QT_CONFIG(tooltip)
        checkBoxLinkToExistingFrame->setText(QCoreApplication::translate("InsertAFrame", "Link to Existing Frame", nullptr));
#if QT_CONFIG(tooltip)
        comboBoxLinkToExistingFrameName->setToolTip(QCoreApplication::translate("InsertAFrame", "Name of existing text frame to link to", nullptr));
#endif // QT_CONFIG(tooltip)
        sourceDocLabel->setText(QCoreApplication::translate("InsertAFrame", "Source Document:", nullptr));
#if QT_CONFIG(tooltip)
        sourceDocLineEdit->setToolTip(QCoreApplication::translate("InsertAFrame", "Source document to load into the text frame", nullptr));
#endif // QT_CONFIG(tooltip)
        sourceImageLabel->setText(QCoreApplication::translate("InsertAFrame", "Source Image:", nullptr));
#if QT_CONFIG(tooltip)
        sourceImageLineEdit->setToolTip(QCoreApplication::translate("InsertAFrame", "Source image to load into the inserted image frame", nullptr));
#endif // QT_CONFIG(tooltip)
        label_9->setText(QCoreApplication::translate("InsertAFrame", "There are no options for this type of frame", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(optionsTab), QCoreApplication::translate("InsertAFrame", "&Options", nullptr));
    } // retranslateUi

};

namespace Ui {
    class InsertAFrame: public Ui_InsertAFrame {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_INSERTAFRAME_H
