/********************************************************************************
** Form generated from reading UI file 'propertywidget_textcolorbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYWIDGET_TEXTCOLORBASE_H
#define UI_PROPERTYWIDGET_TEXTCOLORBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "ui/colorcombo.h"
#include "ui/shadebutton.h"
#include "ui/styleselect.h"

QT_BEGIN_NAMESPACE

class Ui_PropertyWidget_TextColorBase
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *fillLayout;
    QLabel *fillIcon;
    ColorCombo *fillColor;
    QLabel *fillShadeLabel;
    ShadeButton *fillShade;
    QHBoxLayout *strokeLayout;
    QLabel *strokeIcon;
    ColorCombo *strokeColor;
    QLabel *strokeShadeLabel;
    ShadeButton *strokeShade;
    QHBoxLayout *backLayout;
    QLabel *backIcon;
    ColorCombo *backColor;
    QLabel *backShadeLabel;
    ShadeButton *backShade;
    QHBoxLayout *effectsLayout;
    QSpacerItem *horizontalSpacer_3;
    StyleSelect *textEffects;

    void setupUi(QFrame *PropertyWidget_TextColorBase)
    {
        if (PropertyWidget_TextColorBase->objectName().isEmpty())
            PropertyWidget_TextColorBase->setObjectName(QString::fromUtf8("PropertyWidget_TextColorBase"));
        PropertyWidget_TextColorBase->resize(167, 135);
        verticalLayout = new QVBoxLayout(PropertyWidget_TextColorBase);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        fillLayout = new QHBoxLayout();
        fillLayout->setSpacing(3);
        fillLayout->setObjectName(QString::fromUtf8("fillLayout"));
        fillIcon = new QLabel(PropertyWidget_TextColorBase);
        fillIcon->setObjectName(QString::fromUtf8("fillIcon"));
        fillIcon->setMinimumSize(QSize(20, 0));
        fillIcon->setMaximumSize(QSize(22, 22));

        fillLayout->addWidget(fillIcon);

        fillColor = new ColorCombo(PropertyWidget_TextColorBase);
        fillColor->setObjectName(QString::fromUtf8("fillColor"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(fillColor->sizePolicy().hasHeightForWidth());
        fillColor->setSizePolicy(sizePolicy);

        fillLayout->addWidget(fillColor);

        fillShadeLabel = new QLabel(PropertyWidget_TextColorBase);
        fillShadeLabel->setObjectName(QString::fromUtf8("fillShadeLabel"));
        fillShadeLabel->setMinimumSize(QSize(20, 0));
        fillShadeLabel->setMaximumSize(QSize(22, 22));

        fillLayout->addWidget(fillShadeLabel);

        fillShade = new ShadeButton(PropertyWidget_TextColorBase);
        fillShade->setObjectName(QString::fromUtf8("fillShade"));

        fillLayout->addWidget(fillShade);


        verticalLayout->addLayout(fillLayout);

        strokeLayout = new QHBoxLayout();
        strokeLayout->setSpacing(3);
        strokeLayout->setObjectName(QString::fromUtf8("strokeLayout"));
        strokeIcon = new QLabel(PropertyWidget_TextColorBase);
        strokeIcon->setObjectName(QString::fromUtf8("strokeIcon"));
        strokeIcon->setMinimumSize(QSize(20, 0));
        strokeIcon->setMaximumSize(QSize(22, 22));

        strokeLayout->addWidget(strokeIcon);

        strokeColor = new ColorCombo(PropertyWidget_TextColorBase);
        strokeColor->setObjectName(QString::fromUtf8("strokeColor"));
        sizePolicy.setHeightForWidth(strokeColor->sizePolicy().hasHeightForWidth());
        strokeColor->setSizePolicy(sizePolicy);
        strokeColor->setTabletTracking(true);

        strokeLayout->addWidget(strokeColor);

        strokeShadeLabel = new QLabel(PropertyWidget_TextColorBase);
        strokeShadeLabel->setObjectName(QString::fromUtf8("strokeShadeLabel"));
        strokeShadeLabel->setMinimumSize(QSize(20, 0));
        strokeShadeLabel->setMaximumSize(QSize(22, 22));

        strokeLayout->addWidget(strokeShadeLabel);

        strokeShade = new ShadeButton(PropertyWidget_TextColorBase);
        strokeShade->setObjectName(QString::fromUtf8("strokeShade"));

        strokeLayout->addWidget(strokeShade);


        verticalLayout->addLayout(strokeLayout);

        backLayout = new QHBoxLayout();
        backLayout->setSpacing(3);
        backLayout->setObjectName(QString::fromUtf8("backLayout"));
        backIcon = new QLabel(PropertyWidget_TextColorBase);
        backIcon->setObjectName(QString::fromUtf8("backIcon"));
        backIcon->setMinimumSize(QSize(20, 0));
        backIcon->setMaximumSize(QSize(22, 22));

        backLayout->addWidget(backIcon);

        backColor = new ColorCombo(PropertyWidget_TextColorBase);
        backColor->setObjectName(QString::fromUtf8("backColor"));
        sizePolicy.setHeightForWidth(backColor->sizePolicy().hasHeightForWidth());
        backColor->setSizePolicy(sizePolicy);

        backLayout->addWidget(backColor);

        backShadeLabel = new QLabel(PropertyWidget_TextColorBase);
        backShadeLabel->setObjectName(QString::fromUtf8("backShadeLabel"));
        backShadeLabel->setMinimumSize(QSize(20, 0));
        backShadeLabel->setMaximumSize(QSize(22, 22));

        backLayout->addWidget(backShadeLabel);

        backShade = new ShadeButton(PropertyWidget_TextColorBase);
        backShade->setObjectName(QString::fromUtf8("backShade"));

        backLayout->addWidget(backShade);


        verticalLayout->addLayout(backLayout);

        effectsLayout = new QHBoxLayout();
        effectsLayout->setSpacing(3);
        effectsLayout->setObjectName(QString::fromUtf8("effectsLayout"));
        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        effectsLayout->addItem(horizontalSpacer_3);

        textEffects = new StyleSelect(PropertyWidget_TextColorBase);
        textEffects->setObjectName(QString::fromUtf8("textEffects"));

        effectsLayout->addWidget(textEffects);


        verticalLayout->addLayout(effectsLayout);


        retranslateUi(PropertyWidget_TextColorBase);

        QMetaObject::connectSlotsByName(PropertyWidget_TextColorBase);
    } // setupUi

    void retranslateUi(QFrame *PropertyWidget_TextColorBase)
    {
        fillIcon->setText(QString());
#if QT_CONFIG(tooltip)
        fillColor->setToolTip(QCoreApplication::translate("PropertyWidget_TextColorBase", "Color of selected text. If Outline text decoration is enabled, this color will be the fill color.<br/>If Drop Shadow Text is enabled, then this will be the top most color.", nullptr));
#endif // QT_CONFIG(tooltip)
        fillShadeLabel->setText(QString());
#if QT_CONFIG(tooltip)
        fillShade->setToolTip(QCoreApplication::translate("PropertyWidget_TextColorBase", "Saturation of color of text fill", nullptr));
#endif // QT_CONFIG(tooltip)
        fillShade->setText(QString());
        strokeIcon->setText(QString());
#if QT_CONFIG(tooltip)
        strokeColor->setToolTip(QCoreApplication::translate("PropertyWidget_TextColorBase", "Color of text stroke and/or drop shadow, depending which is chosen.<br/>If both are chosen, then they share the same color.", nullptr));
#endif // QT_CONFIG(tooltip)
        strokeShadeLabel->setText(QString());
#if QT_CONFIG(tooltip)
        strokeShade->setToolTip(QCoreApplication::translate("PropertyWidget_TextColorBase", "Saturation of color of text stroke", nullptr));
#endif // QT_CONFIG(tooltip)
        strokeShade->setText(QString());
        backIcon->setText(QString());
#if QT_CONFIG(tooltip)
        backColor->setToolTip(QCoreApplication::translate("PropertyWidget_TextColorBase", "Background color of selected text", nullptr));
#endif // QT_CONFIG(tooltip)
        backShadeLabel->setText(QString());
#if QT_CONFIG(tooltip)
        backShade->setToolTip(QCoreApplication::translate("PropertyWidget_TextColorBase", "Saturation of color of text background", nullptr));
#endif // QT_CONFIG(tooltip)
        backShade->setText(QString());
        (void)PropertyWidget_TextColorBase;
    } // retranslateUi

};

namespace Ui {
    class PropertyWidget_TextColorBase: public Ui_PropertyWidget_TextColorBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTYWIDGET_TEXTCOLORBASE_H
