/********************************************************************************
** Form generated from reading UI file 'prefs_displaybase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_DISPLAYBASE_H
#define UI_PREFS_DISPLAYBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSlider>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/scrspinbox.h>

QT_BEGIN_NAMESPACE

class Ui_Prefs_Display
{
public:
    QVBoxLayout *verticalLayout_3;
    QLabel *label;
    QFrame *line;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *showRulersCheckBox;
    QCheckBox *showRulersRelativeToPageCheckBox;
    QCheckBox *showFramesCheckBox;
    QCheckBox *showImagesCheckBox;
    QCheckBox *showTextChainsCheckBox;
    QCheckBox *showControlCharsCheckBox;
    QCheckBox *showLayerIndicatorsCheckBox;
    QCheckBox *showUnprintableAreaInMarginColorCheckBox;
    QCheckBox *showBleedAreaCheckBox;
    QCheckBox *showPageShadowCheckBox;
    QCheckBox *showVerifierWarningsOnCanvasCheckBox;
    QSpacerItem *verticalSpacer_8;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_5;
    QFormLayout *formLayout;
    QLabel *label_4;
    ScrSpinBox *scratchSpaceLeftSpinBox;
    QLabel *label_5;
    ScrSpinBox *scratchSpaceRightSpinBox;
    QLabel *label_6;
    ScrSpinBox *scratchSpaceTopSpinBox;
    QLabel *label_7;
    ScrSpinBox *scratchSpaceBottomSpinBox;
    QSpacerItem *verticalSpacer;
    QLabel *label_8;
    QFrame *line_4;
    QFormLayout *formLayout_4;
    QLabel *label_11;
    ScrSpinBox *pageGapHorizontalSpinBox;
    QLabel *label_12;
    ScrSpinBox *pageGapVerticalSpinBox;
    QSpacerItem *verticalSpacer_7;
    QWidget *tab_3;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout;
    QSlider *adjustDisplaySlider;
    QLabel *displayDPI;
    QToolButton *buttonRestoreDPI;
    QHBoxLayout *horizontalLayout_2;
    QLabel *displaySizeRuler;
    QComboBox *rulerUnitComboBox;
    QSpacerItem *verticalSpacer_6;
    QWidget *tab_4;
    QVBoxLayout *verticalLayout;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QFormLayout *formLayout_2;
    QLabel *label_15;
    QPushButton *pageFillColorButton;
    QLabel *label_2;
    QPushButton *scratchSpaceColorButton;
    QLabel *label_13;
    QPushButton *selectedPageBorderButton;
    QLabel *label_14;
    QPushButton *frameColorButton;
    QLabel *label_16;
    QPushButton *frameLockedColorButton;
    QLabel *label_17;
    QPushButton *frameSelectedColorButton;
    QLabel *label_31;
    QPushButton *frameLinkedColorButton;
    QLabel *label_30;
    QPushButton *frameGroupedColorButton;
    QLabel *label_29;
    QPushButton *frameAnnotationColorButton;
    QLabel *label_32;
    QPushButton *textControlCharsButton;

    void setupUi(QWidget *Prefs_Display)
    {
        if (Prefs_Display->objectName().isEmpty())
            Prefs_Display->setObjectName(QString::fromUtf8("Prefs_Display"));
        Prefs_Display->resize(521, 415);
        verticalLayout_3 = new QVBoxLayout(Prefs_Display);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(Prefs_Display);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        verticalLayout_3->addWidget(label);

        line = new QFrame(Prefs_Display);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout_3->addWidget(line);

        tabWidget = new QTabWidget(Prefs_Display);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tab = new QWidget();
        tab->setObjectName(QString::fromUtf8("tab"));
        verticalLayout_2 = new QVBoxLayout(tab);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        showRulersCheckBox = new QCheckBox(tab);
        showRulersCheckBox->setObjectName(QString::fromUtf8("showRulersCheckBox"));

        verticalLayout_2->addWidget(showRulersCheckBox);

        showRulersRelativeToPageCheckBox = new QCheckBox(tab);
        showRulersRelativeToPageCheckBox->setObjectName(QString::fromUtf8("showRulersRelativeToPageCheckBox"));

        verticalLayout_2->addWidget(showRulersRelativeToPageCheckBox);

        showFramesCheckBox = new QCheckBox(tab);
        showFramesCheckBox->setObjectName(QString::fromUtf8("showFramesCheckBox"));

        verticalLayout_2->addWidget(showFramesCheckBox);

        showImagesCheckBox = new QCheckBox(tab);
        showImagesCheckBox->setObjectName(QString::fromUtf8("showImagesCheckBox"));

        verticalLayout_2->addWidget(showImagesCheckBox);

        showTextChainsCheckBox = new QCheckBox(tab);
        showTextChainsCheckBox->setObjectName(QString::fromUtf8("showTextChainsCheckBox"));

        verticalLayout_2->addWidget(showTextChainsCheckBox);

        showControlCharsCheckBox = new QCheckBox(tab);
        showControlCharsCheckBox->setObjectName(QString::fromUtf8("showControlCharsCheckBox"));

        verticalLayout_2->addWidget(showControlCharsCheckBox);

        showLayerIndicatorsCheckBox = new QCheckBox(tab);
        showLayerIndicatorsCheckBox->setObjectName(QString::fromUtf8("showLayerIndicatorsCheckBox"));

        verticalLayout_2->addWidget(showLayerIndicatorsCheckBox);

        showUnprintableAreaInMarginColorCheckBox = new QCheckBox(tab);
        showUnprintableAreaInMarginColorCheckBox->setObjectName(QString::fromUtf8("showUnprintableAreaInMarginColorCheckBox"));

        verticalLayout_2->addWidget(showUnprintableAreaInMarginColorCheckBox);

        showBleedAreaCheckBox = new QCheckBox(tab);
        showBleedAreaCheckBox->setObjectName(QString::fromUtf8("showBleedAreaCheckBox"));

        verticalLayout_2->addWidget(showBleedAreaCheckBox);

        showPageShadowCheckBox = new QCheckBox(tab);
        showPageShadowCheckBox->setObjectName(QString::fromUtf8("showPageShadowCheckBox"));

        verticalLayout_2->addWidget(showPageShadowCheckBox);

        showVerifierWarningsOnCanvasCheckBox = new QCheckBox(tab);
        showVerifierWarningsOnCanvasCheckBox->setObjectName(QString::fromUtf8("showVerifierWarningsOnCanvasCheckBox"));

        verticalLayout_2->addWidget(showVerifierWarningsOnCanvasCheckBox);

        verticalSpacer_8 = new QSpacerItem(20, 4, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_8);

        tabWidget->addTab(tab, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName(QString::fromUtf8("tab_2"));
        verticalLayout_5 = new QVBoxLayout(tab_2);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        label_4 = new QLabel(tab_2);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        formLayout->setWidget(0, QFormLayout::LabelRole, label_4);

        scratchSpaceLeftSpinBox = new ScrSpinBox(tab_2);
        scratchSpaceLeftSpinBox->setObjectName(QString::fromUtf8("scratchSpaceLeftSpinBox"));

        formLayout->setWidget(0, QFormLayout::FieldRole, scratchSpaceLeftSpinBox);

        label_5 = new QLabel(tab_2);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        formLayout->setWidget(1, QFormLayout::LabelRole, label_5);

        scratchSpaceRightSpinBox = new ScrSpinBox(tab_2);
        scratchSpaceRightSpinBox->setObjectName(QString::fromUtf8("scratchSpaceRightSpinBox"));

        formLayout->setWidget(1, QFormLayout::FieldRole, scratchSpaceRightSpinBox);

        label_6 = new QLabel(tab_2);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        formLayout->setWidget(2, QFormLayout::LabelRole, label_6);

        scratchSpaceTopSpinBox = new ScrSpinBox(tab_2);
        scratchSpaceTopSpinBox->setObjectName(QString::fromUtf8("scratchSpaceTopSpinBox"));

        formLayout->setWidget(2, QFormLayout::FieldRole, scratchSpaceTopSpinBox);

        label_7 = new QLabel(tab_2);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        formLayout->setWidget(3, QFormLayout::LabelRole, label_7);

        scratchSpaceBottomSpinBox = new ScrSpinBox(tab_2);
        scratchSpaceBottomSpinBox->setObjectName(QString::fromUtf8("scratchSpaceBottomSpinBox"));

        formLayout->setWidget(3, QFormLayout::FieldRole, scratchSpaceBottomSpinBox);


        verticalLayout_5->addLayout(formLayout);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_5->addItem(verticalSpacer);

        label_8 = new QLabel(tab_2);
        label_8->setObjectName(QString::fromUtf8("label_8"));
        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
        label_8->setFont(font1);

        verticalLayout_5->addWidget(label_8);

        line_4 = new QFrame(tab_2);
        line_4->setObjectName(QString::fromUtf8("line_4"));
        line_4->setFrameShape(QFrame::HLine);
        line_4->setFrameShadow(QFrame::Sunken);

        verticalLayout_5->addWidget(line_4);

        formLayout_4 = new QFormLayout();
        formLayout_4->setObjectName(QString::fromUtf8("formLayout_4"));
        label_11 = new QLabel(tab_2);
        label_11->setObjectName(QString::fromUtf8("label_11"));

        formLayout_4->setWidget(0, QFormLayout::LabelRole, label_11);

        pageGapHorizontalSpinBox = new ScrSpinBox(tab_2);
        pageGapHorizontalSpinBox->setObjectName(QString::fromUtf8("pageGapHorizontalSpinBox"));

        formLayout_4->setWidget(0, QFormLayout::FieldRole, pageGapHorizontalSpinBox);

        label_12 = new QLabel(tab_2);
        label_12->setObjectName(QString::fromUtf8("label_12"));

        formLayout_4->setWidget(1, QFormLayout::LabelRole, label_12);

        pageGapVerticalSpinBox = new ScrSpinBox(tab_2);
        pageGapVerticalSpinBox->setObjectName(QString::fromUtf8("pageGapVerticalSpinBox"));

        formLayout_4->setWidget(1, QFormLayout::FieldRole, pageGapVerticalSpinBox);


        verticalLayout_5->addLayout(formLayout_4);

        verticalSpacer_7 = new QSpacerItem(20, 195, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_5->addItem(verticalSpacer_7);

        tabWidget->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName(QString::fromUtf8("tab_3"));
        verticalLayout_4 = new QVBoxLayout(tab_3);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        adjustDisplaySlider = new QSlider(tab_3);
        adjustDisplaySlider->setObjectName(QString::fromUtf8("adjustDisplaySlider"));
        adjustDisplaySlider->setMinimum(-150);
        adjustDisplaySlider->setMaximum(500);
        adjustDisplaySlider->setOrientation(Qt::Horizontal);
        adjustDisplaySlider->setTickInterval(10);

        horizontalLayout->addWidget(adjustDisplaySlider);

        displayDPI = new QLabel(tab_3);
        displayDPI->setObjectName(QString::fromUtf8("displayDPI"));
        displayDPI->setWordWrap(false);

        horizontalLayout->addWidget(displayDPI);

        buttonRestoreDPI = new QToolButton(tab_3);
        buttonRestoreDPI->setObjectName(QString::fromUtf8("buttonRestoreDPI"));

        horizontalLayout->addWidget(buttonRestoreDPI);


        verticalLayout_4->addLayout(horizontalLayout);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        displaySizeRuler = new QLabel(tab_3);
        displaySizeRuler->setObjectName(QString::fromUtf8("displaySizeRuler"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(displaySizeRuler->sizePolicy().hasHeightForWidth());
        displaySizeRuler->setSizePolicy(sizePolicy);
        displaySizeRuler->setMinimumSize(QSize(20, 20));
        displaySizeRuler->setFrameShape(QFrame::Box);
        displaySizeRuler->setFrameShadow(QFrame::Sunken);
        displaySizeRuler->setWordWrap(false);

        horizontalLayout_2->addWidget(displaySizeRuler);

        rulerUnitComboBox = new QComboBox(tab_3);
        rulerUnitComboBox->setObjectName(QString::fromUtf8("rulerUnitComboBox"));

        horizontalLayout_2->addWidget(rulerUnitComboBox);


        verticalLayout_4->addLayout(horizontalLayout_2);

        verticalSpacer_6 = new QSpacerItem(20, 263, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_4->addItem(verticalSpacer_6);

        tabWidget->addTab(tab_3, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName(QString::fromUtf8("tab_4"));
        verticalLayout = new QVBoxLayout(tab_4);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        scrollArea = new QScrollArea(tab_4);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 212, 292));
        formLayout_2 = new QFormLayout(scrollAreaWidgetContents);
        formLayout_2->setObjectName(QString::fromUtf8("formLayout_2"));
        formLayout_2->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
        formLayout_2->setContentsMargins(0, 0, 0, 0);
        label_15 = new QLabel(scrollAreaWidgetContents);
        label_15->setObjectName(QString::fromUtf8("label_15"));

        formLayout_2->setWidget(0, QFormLayout::LabelRole, label_15);

        pageFillColorButton = new QPushButton(scrollAreaWidgetContents);
        pageFillColorButton->setObjectName(QString::fromUtf8("pageFillColorButton"));

        formLayout_2->setWidget(0, QFormLayout::FieldRole, pageFillColorButton);

        label_2 = new QLabel(scrollAreaWidgetContents);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        formLayout_2->setWidget(1, QFormLayout::LabelRole, label_2);

        scratchSpaceColorButton = new QPushButton(scrollAreaWidgetContents);
        scratchSpaceColorButton->setObjectName(QString::fromUtf8("scratchSpaceColorButton"));

        formLayout_2->setWidget(1, QFormLayout::FieldRole, scratchSpaceColorButton);

        label_13 = new QLabel(scrollAreaWidgetContents);
        label_13->setObjectName(QString::fromUtf8("label_13"));

        formLayout_2->setWidget(2, QFormLayout::LabelRole, label_13);

        selectedPageBorderButton = new QPushButton(scrollAreaWidgetContents);
        selectedPageBorderButton->setObjectName(QString::fromUtf8("selectedPageBorderButton"));

        formLayout_2->setWidget(2, QFormLayout::FieldRole, selectedPageBorderButton);

        label_14 = new QLabel(scrollAreaWidgetContents);
        label_14->setObjectName(QString::fromUtf8("label_14"));

        formLayout_2->setWidget(3, QFormLayout::LabelRole, label_14);

        frameColorButton = new QPushButton(scrollAreaWidgetContents);
        frameColorButton->setObjectName(QString::fromUtf8("frameColorButton"));

        formLayout_2->setWidget(3, QFormLayout::FieldRole, frameColorButton);

        label_16 = new QLabel(scrollAreaWidgetContents);
        label_16->setObjectName(QString::fromUtf8("label_16"));

        formLayout_2->setWidget(4, QFormLayout::LabelRole, label_16);

        frameLockedColorButton = new QPushButton(scrollAreaWidgetContents);
        frameLockedColorButton->setObjectName(QString::fromUtf8("frameLockedColorButton"));

        formLayout_2->setWidget(4, QFormLayout::FieldRole, frameLockedColorButton);

        label_17 = new QLabel(scrollAreaWidgetContents);
        label_17->setObjectName(QString::fromUtf8("label_17"));

        formLayout_2->setWidget(5, QFormLayout::LabelRole, label_17);

        frameSelectedColorButton = new QPushButton(scrollAreaWidgetContents);
        frameSelectedColorButton->setObjectName(QString::fromUtf8("frameSelectedColorButton"));

        formLayout_2->setWidget(5, QFormLayout::FieldRole, frameSelectedColorButton);

        label_31 = new QLabel(scrollAreaWidgetContents);
        label_31->setObjectName(QString::fromUtf8("label_31"));

        formLayout_2->setWidget(6, QFormLayout::LabelRole, label_31);

        frameLinkedColorButton = new QPushButton(scrollAreaWidgetContents);
        frameLinkedColorButton->setObjectName(QString::fromUtf8("frameLinkedColorButton"));

        formLayout_2->setWidget(6, QFormLayout::FieldRole, frameLinkedColorButton);

        label_30 = new QLabel(scrollAreaWidgetContents);
        label_30->setObjectName(QString::fromUtf8("label_30"));

        formLayout_2->setWidget(7, QFormLayout::LabelRole, label_30);

        frameGroupedColorButton = new QPushButton(scrollAreaWidgetContents);
        frameGroupedColorButton->setObjectName(QString::fromUtf8("frameGroupedColorButton"));

        formLayout_2->setWidget(7, QFormLayout::FieldRole, frameGroupedColorButton);

        label_29 = new QLabel(scrollAreaWidgetContents);
        label_29->setObjectName(QString::fromUtf8("label_29"));

        formLayout_2->setWidget(8, QFormLayout::LabelRole, label_29);

        frameAnnotationColorButton = new QPushButton(scrollAreaWidgetContents);
        frameAnnotationColorButton->setObjectName(QString::fromUtf8("frameAnnotationColorButton"));

        formLayout_2->setWidget(8, QFormLayout::FieldRole, frameAnnotationColorButton);

        label_32 = new QLabel(scrollAreaWidgetContents);
        label_32->setObjectName(QString::fromUtf8("label_32"));

        formLayout_2->setWidget(9, QFormLayout::LabelRole, label_32);

        textControlCharsButton = new QPushButton(scrollAreaWidgetContents);
        textControlCharsButton->setObjectName(QString::fromUtf8("textControlCharsButton"));

        formLayout_2->setWidget(9, QFormLayout::FieldRole, textControlCharsButton);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout->addWidget(scrollArea);

        tabWidget->addTab(tab_4, QString());

        verticalLayout_3->addWidget(tabWidget);


        retranslateUi(Prefs_Display);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Prefs_Display);
    } // setupUi

    void retranslateUi(QWidget *Prefs_Display)
    {
        label->setText(QCoreApplication::translate("Prefs_Display", "Display", nullptr));
        showRulersCheckBox->setText(QCoreApplication::translate("Prefs_Display", "Show Rulers", nullptr));
        showRulersRelativeToPageCheckBox->setText(QCoreApplication::translate("Prefs_Display", "Show Rulers Relative to Page", nullptr));
        showFramesCheckBox->setText(QCoreApplication::translate("Prefs_Display", "Show Frames", nullptr));
        showImagesCheckBox->setText(QCoreApplication::translate("Prefs_Display", "Show Images", nullptr));
        showTextChainsCheckBox->setText(QCoreApplication::translate("Prefs_Display", "Show Text Chains", nullptr));
        showControlCharsCheckBox->setText(QCoreApplication::translate("Prefs_Display", "Show Text Control Characters", nullptr));
        showLayerIndicatorsCheckBox->setText(QCoreApplication::translate("Prefs_Display", "Show Layer Indicators", nullptr));
        showUnprintableAreaInMarginColorCheckBox->setText(QCoreApplication::translate("Prefs_Display", "Show Unprintable Area in Margin Color", nullptr));
        showBleedAreaCheckBox->setText(QCoreApplication::translate("Prefs_Display", "Show Bleed Area", nullptr));
        showPageShadowCheckBox->setText(QCoreApplication::translate("Prefs_Display", "Show Page Shadow", nullptr));
        showVerifierWarningsOnCanvasCheckBox->setText(QCoreApplication::translate("Prefs_Display", "Show Preflight Verifier Warnings on the Page", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("Prefs_Display", "Pages", nullptr));
        label_4->setText(QCoreApplication::translate("Prefs_Display", "Left:", nullptr));
        label_5->setText(QCoreApplication::translate("Prefs_Display", "Right:", nullptr));
        label_6->setText(QCoreApplication::translate("Prefs_Display", "Top:", nullptr));
        label_7->setText(QCoreApplication::translate("Prefs_Display", "Bottom:", nullptr));
        label_8->setText(QCoreApplication::translate("Prefs_Display", "Gaps Between Pages", nullptr));
        label_11->setText(QCoreApplication::translate("Prefs_Display", "Horizontal:", nullptr));
        label_12->setText(QCoreApplication::translate("Prefs_Display", "Vertical:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("Prefs_Display", "Scratch Space", nullptr));
        displayDPI->setText(QCoreApplication::translate("Prefs_Display", "Scale%", nullptr));
#if QT_CONFIG(tooltip)
        buttonRestoreDPI->setToolTip(QCoreApplication::translate("Prefs_Display", "Resets the scale to the default dpi of your display", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonRestoreDPI->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("Prefs_Display", "Display Scaling", nullptr));
        label_15->setText(QCoreApplication::translate("Prefs_Display", "Page Fill:", nullptr));
        pageFillColorButton->setText(QString());
        label_2->setText(QCoreApplication::translate("Prefs_Display", "Scratch Space:", nullptr));
        scratchSpaceColorButton->setText(QString());
        label_13->setText(QCoreApplication::translate("Prefs_Display", "Selected Page Border:", nullptr));
        selectedPageBorderButton->setText(QString());
        label_14->setText(QCoreApplication::translate("Prefs_Display", "Frames:", nullptr));
        frameColorButton->setText(QString());
        label_16->setText(QCoreApplication::translate("Prefs_Display", "Locked Frames:", nullptr));
        frameLockedColorButton->setText(QString());
        label_17->setText(QCoreApplication::translate("Prefs_Display", "Selected Frames:", nullptr));
        frameSelectedColorButton->setText(QString());
        label_31->setText(QCoreApplication::translate("Prefs_Display", "Linked Frames:", nullptr));
        frameLinkedColorButton->setText(QString());
        label_30->setText(QCoreApplication::translate("Prefs_Display", "Grouped Frames:", nullptr));
        frameGroupedColorButton->setText(QString());
        label_29->setText(QCoreApplication::translate("Prefs_Display", "Annotation Frames:", nullptr));
        frameAnnotationColorButton->setText(QString());
        label_32->setText(QCoreApplication::translate("Prefs_Display", "Text Control Characters:", nullptr));
        textControlCharsButton->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(tab_4), QCoreApplication::translate("Prefs_Display", "Colors", nullptr));
        (void)Prefs_Display;
    } // retranslateUi

};

namespace Ui {
    class Prefs_Display: public Ui_Prefs_Display {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_DISPLAYBASE_H
