/********************************************************************************
** Form generated from reading UI file 'picstatus.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PICSTATUS_H
#define UI_PICSTATUS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_PicStatus
{
public:
    QVBoxLayout *vboxLayout;
    QListWidget *imageViewArea;
    QTabWidget *workTab;
    QWidget *tabInfo;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_3;
    QFormLayout *formLayout_4;
    QLabel *textLabel1;
    QLabel *displayName;
    QLabel *textLabel3;
    QLabel *displayPath;
    QPushButton *searchButton;
    QPushButton *fileManagerButton;
    QHBoxLayout *horizontalLayout_2;
    QGroupBox *groupImage;
    QFormLayout *formLayout;
    QLabel *textLabel6;
    QLabel *displayFormat;
    QLabel *textLabel5_2;
    QLabel *displayColorspace;
    QLabel *textLabel1_2;
    QLabel *displayDPI;
    QGroupBox *groupSize;
    QFormLayout *formLayout_2;
    QLabel *textLabel9;
    QLabel *displaySizePixel;
    QLabel *textLabel10;
    QLabel *displayScale;
    QLabel *textLabel1_4;
    QLabel *displayPrintSize;
    QGroupBox *groupLayout;
    QHBoxLayout *horizontalLayout;
    QFormLayout *formLayout_3;
    QLabel *textLabel5;
    QLabel *displayPage;
    QLabel *textLabel1_3;
    QLabel *displayObjekt;
    QLabel *textLabel2;
    QLabel *displayEffDPI;
    QVBoxLayout *verticalLayout;
    QPushButton *goPageButton;
    QPushButton *selectButton;
    QWidget *tabTasks;
    QGridLayout *gridLayout;
    QCheckBox *isPrinting;
    QPushButton *buttonLayers;
    QPushButton *buttonEdit;
    QCheckBox *isVisibleCheck;
    QPushButton *effectsButton;
    QSpacerItem *spacerItem;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem1;
    QPushButton *closeButton;

    void setupUi(QDialog *PicStatus)
    {
        if (PicStatus->objectName().isEmpty())
            PicStatus->setObjectName(QString::fromUtf8("PicStatus"));
        PicStatus->setWindowModality(Qt::WindowModal);
        PicStatus->resize(720, 464);
        vboxLayout = new QVBoxLayout(PicStatus);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        imageViewArea = new QListWidget(PicStatus);
        imageViewArea->setObjectName(QString::fromUtf8("imageViewArea"));
        imageViewArea->setMinimumSize(QSize(0, 150));
        imageViewArea->setDragDropMode(QAbstractItemView::NoDragDrop);
        imageViewArea->setMovement(QListView::Static);
        imageViewArea->setFlow(QListView::LeftToRight);
        imageViewArea->setProperty("isWrapping", QVariant(true));
        imageViewArea->setResizeMode(QListView::Adjust);
        imageViewArea->setSpacing(6);
        imageViewArea->setViewMode(QListView::IconMode);
        imageViewArea->setWordWrap(true);

        vboxLayout->addWidget(imageViewArea);

        workTab = new QTabWidget(PicStatus);
        workTab->setObjectName(QString::fromUtf8("workTab"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Minimum);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(workTab->sizePolicy().hasHeightForWidth());
        workTab->setSizePolicy(sizePolicy);
        tabInfo = new QWidget();
        tabInfo->setObjectName(QString::fromUtf8("tabInfo"));
        verticalLayout_2 = new QVBoxLayout(tabInfo);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        formLayout_4 = new QFormLayout();
        formLayout_4->setObjectName(QString::fromUtf8("formLayout_4"));
        textLabel1 = new QLabel(tabInfo);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));
        QSizePolicy sizePolicy1(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(textLabel1->sizePolicy().hasHeightForWidth());
        textLabel1->setSizePolicy(sizePolicy1);
        textLabel1->setMinimumSize(QSize(70, 0));
        textLabel1->setWordWrap(false);

        formLayout_4->setWidget(0, QFormLayout::LabelRole, textLabel1);

        displayName = new QLabel(tabInfo);
        displayName->setObjectName(QString::fromUtf8("displayName"));
        QSizePolicy sizePolicy2(QSizePolicy::Maximum, QSizePolicy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(displayName->sizePolicy().hasHeightForWidth());
        displayName->setSizePolicy(sizePolicy2);
        displayName->setMinimumSize(QSize(220, 0));
        displayName->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        displayName->setWordWrap(false);

        formLayout_4->setWidget(0, QFormLayout::FieldRole, displayName);

        textLabel3 = new QLabel(tabInfo);
        textLabel3->setObjectName(QString::fromUtf8("textLabel3"));
        sizePolicy1.setHeightForWidth(textLabel3->sizePolicy().hasHeightForWidth());
        textLabel3->setSizePolicy(sizePolicy1);
        textLabel3->setMinimumSize(QSize(70, 0));
        textLabel3->setWordWrap(false);

        formLayout_4->setWidget(1, QFormLayout::LabelRole, textLabel3);

        displayPath = new QLabel(tabInfo);
        displayPath->setObjectName(QString::fromUtf8("displayPath"));
        sizePolicy2.setHeightForWidth(displayPath->sizePolicy().hasHeightForWidth());
        displayPath->setSizePolicy(sizePolicy2);
        displayPath->setMinimumSize(QSize(220, 0));
        displayPath->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        displayPath->setWordWrap(false);

        formLayout_4->setWidget(1, QFormLayout::FieldRole, displayPath);


        horizontalLayout_3->addLayout(formLayout_4);

        searchButton = new QPushButton(tabInfo);
        searchButton->setObjectName(QString::fromUtf8("searchButton"));
        searchButton->setAutoDefault(false);
        searchButton->setFlat(false);

        horizontalLayout_3->addWidget(searchButton);

        fileManagerButton = new QPushButton(tabInfo);
        fileManagerButton->setObjectName(QString::fromUtf8("fileManagerButton"));
        fileManagerButton->setAutoDefault(false);
        fileManagerButton->setFlat(false);

        horizontalLayout_3->addWidget(fileManagerButton);


        verticalLayout_2->addLayout(horizontalLayout_3);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        groupImage = new QGroupBox(tabInfo);
        groupImage->setObjectName(QString::fromUtf8("groupImage"));
        formLayout = new QFormLayout(groupImage);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        textLabel6 = new QLabel(groupImage);
        textLabel6->setObjectName(QString::fromUtf8("textLabel6"));
        textLabel6->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        textLabel6->setWordWrap(false);

        formLayout->setWidget(0, QFormLayout::LabelRole, textLabel6);

        displayFormat = new QLabel(groupImage);
        displayFormat->setObjectName(QString::fromUtf8("displayFormat"));
        displayFormat->setMinimumSize(QSize(70, 0));
        displayFormat->setWordWrap(false);

        formLayout->setWidget(0, QFormLayout::FieldRole, displayFormat);

        textLabel5_2 = new QLabel(groupImage);
        textLabel5_2->setObjectName(QString::fromUtf8("textLabel5_2"));
        textLabel5_2->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        textLabel5_2->setWordWrap(false);

        formLayout->setWidget(1, QFormLayout::LabelRole, textLabel5_2);

        displayColorspace = new QLabel(groupImage);
        displayColorspace->setObjectName(QString::fromUtf8("displayColorspace"));
        displayColorspace->setMinimumSize(QSize(70, 0));
        displayColorspace->setWordWrap(false);

        formLayout->setWidget(1, QFormLayout::FieldRole, displayColorspace);

        textLabel1_2 = new QLabel(groupImage);
        textLabel1_2->setObjectName(QString::fromUtf8("textLabel1_2"));
        textLabel1_2->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        textLabel1_2->setWordWrap(false);

        formLayout->setWidget(2, QFormLayout::LabelRole, textLabel1_2);

        displayDPI = new QLabel(groupImage);
        displayDPI->setObjectName(QString::fromUtf8("displayDPI"));
        displayDPI->setMinimumSize(QSize(70, 0));
        displayDPI->setWordWrap(false);

        formLayout->setWidget(2, QFormLayout::FieldRole, displayDPI);


        horizontalLayout_2->addWidget(groupImage);

        groupSize = new QGroupBox(tabInfo);
        groupSize->setObjectName(QString::fromUtf8("groupSize"));
        formLayout_2 = new QFormLayout(groupSize);
        formLayout_2->setObjectName(QString::fromUtf8("formLayout_2"));
        textLabel9 = new QLabel(groupSize);
        textLabel9->setObjectName(QString::fromUtf8("textLabel9"));
        textLabel9->setWordWrap(false);

        formLayout_2->setWidget(0, QFormLayout::LabelRole, textLabel9);

        displaySizePixel = new QLabel(groupSize);
        displaySizePixel->setObjectName(QString::fromUtf8("displaySizePixel"));
        displaySizePixel->setMinimumSize(QSize(120, 0));
        displaySizePixel->setWordWrap(false);

        formLayout_2->setWidget(0, QFormLayout::FieldRole, displaySizePixel);

        textLabel10 = new QLabel(groupSize);
        textLabel10->setObjectName(QString::fromUtf8("textLabel10"));
        textLabel10->setWordWrap(false);

        formLayout_2->setWidget(1, QFormLayout::LabelRole, textLabel10);

        displayScale = new QLabel(groupSize);
        displayScale->setObjectName(QString::fromUtf8("displayScale"));
        displayScale->setMinimumSize(QSize(120, 0));
        displayScale->setWordWrap(false);

        formLayout_2->setWidget(1, QFormLayout::FieldRole, displayScale);

        textLabel1_4 = new QLabel(groupSize);
        textLabel1_4->setObjectName(QString::fromUtf8("textLabel1_4"));
        textLabel1_4->setWordWrap(false);

        formLayout_2->setWidget(2, QFormLayout::LabelRole, textLabel1_4);

        displayPrintSize = new QLabel(groupSize);
        displayPrintSize->setObjectName(QString::fromUtf8("displayPrintSize"));
        QSizePolicy sizePolicy3(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(displayPrintSize->sizePolicy().hasHeightForWidth());
        displayPrintSize->setSizePolicy(sizePolicy3);
        displayPrintSize->setMinimumSize(QSize(120, 0));
        displayPrintSize->setWordWrap(false);

        formLayout_2->setWidget(2, QFormLayout::FieldRole, displayPrintSize);


        horizontalLayout_2->addWidget(groupSize);

        groupLayout = new QGroupBox(tabInfo);
        groupLayout->setObjectName(QString::fromUtf8("groupLayout"));
        horizontalLayout = new QHBoxLayout(groupLayout);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        formLayout_3 = new QFormLayout();
        formLayout_3->setObjectName(QString::fromUtf8("formLayout_3"));
        textLabel5 = new QLabel(groupLayout);
        textLabel5->setObjectName(QString::fromUtf8("textLabel5"));
        textLabel5->setWordWrap(false);

        formLayout_3->setWidget(0, QFormLayout::LabelRole, textLabel5);

        displayPage = new QLabel(groupLayout);
        displayPage->setObjectName(QString::fromUtf8("displayPage"));
        displayPage->setMinimumSize(QSize(70, 0));
        displayPage->setWordWrap(false);

        formLayout_3->setWidget(0, QFormLayout::FieldRole, displayPage);

        textLabel1_3 = new QLabel(groupLayout);
        textLabel1_3->setObjectName(QString::fromUtf8("textLabel1_3"));
        textLabel1_3->setWordWrap(false);

        formLayout_3->setWidget(1, QFormLayout::LabelRole, textLabel1_3);

        displayObjekt = new QLabel(groupLayout);
        displayObjekt->setObjectName(QString::fromUtf8("displayObjekt"));
        displayObjekt->setMinimumSize(QSize(70, 0));
        displayObjekt->setWordWrap(false);

        formLayout_3->setWidget(1, QFormLayout::FieldRole, displayObjekt);

        textLabel2 = new QLabel(groupLayout);
        textLabel2->setObjectName(QString::fromUtf8("textLabel2"));
        textLabel2->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignVCenter);
        textLabel2->setWordWrap(false);

        formLayout_3->setWidget(2, QFormLayout::LabelRole, textLabel2);

        displayEffDPI = new QLabel(groupLayout);
        displayEffDPI->setObjectName(QString::fromUtf8("displayEffDPI"));
        displayEffDPI->setMinimumSize(QSize(70, 0));
        displayEffDPI->setWordWrap(false);

        formLayout_3->setWidget(2, QFormLayout::FieldRole, displayEffDPI);


        horizontalLayout->addLayout(formLayout_3);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        goPageButton = new QPushButton(groupLayout);
        goPageButton->setObjectName(QString::fromUtf8("goPageButton"));
        goPageButton->setAutoDefault(false);
        goPageButton->setFlat(false);

        verticalLayout->addWidget(goPageButton);

        selectButton = new QPushButton(groupLayout);
        selectButton->setObjectName(QString::fromUtf8("selectButton"));
        selectButton->setAutoDefault(false);
        selectButton->setFlat(false);

        verticalLayout->addWidget(selectButton);


        horizontalLayout->addLayout(verticalLayout);


        horizontalLayout_2->addWidget(groupLayout);


        verticalLayout_2->addLayout(horizontalLayout_2);

        workTab->addTab(tabInfo, QString());
        tabTasks = new QWidget();
        tabTasks->setObjectName(QString::fromUtf8("tabTasks"));
        gridLayout = new QGridLayout(tabTasks);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        isPrinting = new QCheckBox(tabTasks);
        isPrinting->setObjectName(QString::fromUtf8("isPrinting"));

        gridLayout->addWidget(isPrinting, 0, 0, 1, 1);

        buttonLayers = new QPushButton(tabTasks);
        buttonLayers->setObjectName(QString::fromUtf8("buttonLayers"));

        gridLayout->addWidget(buttonLayers, 1, 1, 1, 1);

        buttonEdit = new QPushButton(tabTasks);
        buttonEdit->setObjectName(QString::fromUtf8("buttonEdit"));

        gridLayout->addWidget(buttonEdit, 0, 1, 1, 1);

        isVisibleCheck = new QCheckBox(tabTasks);
        isVisibleCheck->setObjectName(QString::fromUtf8("isVisibleCheck"));

        gridLayout->addWidget(isVisibleCheck, 1, 0, 1, 1);

        effectsButton = new QPushButton(tabTasks);
        effectsButton->setObjectName(QString::fromUtf8("effectsButton"));

        gridLayout->addWidget(effectsButton, 0, 2, 1, 1);

        spacerItem = new QSpacerItem(21, 90, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout->addItem(spacerItem, 2, 1, 1, 1);

        workTab->addTab(tabTasks, QString());

        vboxLayout->addWidget(workTab);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        spacerItem1 = new QSpacerItem(131, 21, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem1);

        closeButton = new QPushButton(PicStatus);
        closeButton->setObjectName(QString::fromUtf8("closeButton"));

        hboxLayout->addWidget(closeButton);


        vboxLayout->addLayout(hboxLayout);


        retranslateUi(PicStatus);

        workTab->setCurrentIndex(1);
        closeButton->setDefault(true);


        QMetaObject::connectSlotsByName(PicStatus);
    } // setupUi

    void retranslateUi(QDialog *PicStatus)
    {
        PicStatus->setWindowTitle(QCoreApplication::translate("PicStatus", "Manage Images", nullptr));
#if QT_CONFIG(tooltip)
        workTab->setToolTip(QString());
#endif // QT_CONFIG(tooltip)
        textLabel1->setText(QCoreApplication::translate("PicStatus", "Name:", nullptr));
#if QT_CONFIG(tooltip)
        displayName->setToolTip(QCoreApplication::translate("PicStatus", "Name of the image file", nullptr));
#endif // QT_CONFIG(tooltip)
        displayName->setText(QString());
        textLabel3->setText(QCoreApplication::translate("PicStatus", "Path:", nullptr));
#if QT_CONFIG(tooltip)
        displayPath->setToolTip(QCoreApplication::translate("PicStatus", "Location where the image file is stored", nullptr));
#endif // QT_CONFIG(tooltip)
        displayPath->setText(QString());
#if QT_CONFIG(tooltip)
        searchButton->setToolTip(QCoreApplication::translate("PicStatus", "Search for a missing image", nullptr));
#endif // QT_CONFIG(tooltip)
        searchButton->setText(QCoreApplication::translate("PicStatus", "Search...", nullptr));
#if QT_CONFIG(tooltip)
        fileManagerButton->setToolTip(QCoreApplication::translate("PicStatus", "Show Image in the File Manager", nullptr));
#endif // QT_CONFIG(tooltip)
        fileManagerButton->setText(QCoreApplication::translate("PicStatus", "Show directory...", nullptr));
        groupImage->setTitle(QCoreApplication::translate("PicStatus", "Image", nullptr));
        textLabel6->setText(QCoreApplication::translate("PicStatus", "Format:", nullptr));
#if QT_CONFIG(tooltip)
        displayFormat->setToolTip(QCoreApplication::translate("PicStatus", "Type of the image", nullptr));
#endif // QT_CONFIG(tooltip)
        displayFormat->setText(QString());
#if QT_CONFIG(tooltip)
        textLabel5_2->setToolTip(QCoreApplication::translate("PicStatus", "The colorspace of the image", nullptr));
#endif // QT_CONFIG(tooltip)
        textLabel5_2->setText(QCoreApplication::translate("PicStatus", "Colorspace:", nullptr));
#if QT_CONFIG(tooltip)
        displayColorspace->setToolTip(QCoreApplication::translate("PicStatus", "Colorspace used within the image, eg RGB or CMYK", nullptr));
#endif // QT_CONFIG(tooltip)
        displayColorspace->setText(QString());
        textLabel1_2->setText(QCoreApplication::translate("PicStatus", "DPI:", nullptr));
#if QT_CONFIG(tooltip)
        displayDPI->setToolTip(QCoreApplication::translate("PicStatus", "Native resolution of the image, in dots per inch", nullptr));
#endif // QT_CONFIG(tooltip)
        displayDPI->setText(QString());
        groupSize->setTitle(QCoreApplication::translate("PicStatus", "Size", nullptr));
        textLabel9->setText(QCoreApplication::translate("PicStatus", "Pixels:", nullptr));
#if QT_CONFIG(tooltip)
        displaySizePixel->setToolTip(QCoreApplication::translate("PicStatus", "Height and width of the image", nullptr));
#endif // QT_CONFIG(tooltip)
        displaySizePixel->setText(QString());
        textLabel10->setText(QCoreApplication::translate("PicStatus", "Scale:", nullptr));
#if QT_CONFIG(tooltip)
        displayScale->setToolTip(QCoreApplication::translate("PicStatus", "Horizontal and vertical scaling applied to the image", nullptr));
#endif // QT_CONFIG(tooltip)
        displayScale->setText(QString());
        textLabel1_4->setText(QCoreApplication::translate("PicStatus", "Printed:", nullptr));
#if QT_CONFIG(tooltip)
        displayPrintSize->setToolTip(QCoreApplication::translate("PicStatus", "Size of the image when printed", nullptr));
#endif // QT_CONFIG(tooltip)
        displayPrintSize->setText(QString());
        groupLayout->setTitle(QCoreApplication::translate("PicStatus", "Layout", nullptr));
        textLabel5->setText(QCoreApplication::translate("PicStatus", "On Page:", nullptr));
#if QT_CONFIG(tooltip)
        displayPage->setToolTip(QCoreApplication::translate("PicStatus", "Page that the image is displayed on", nullptr));
#endif // QT_CONFIG(tooltip)
        displayPage->setText(QString());
        textLabel1_3->setText(QCoreApplication::translate("PicStatus", "Page Item:", nullptr));
#if QT_CONFIG(tooltip)
        displayObjekt->setToolTip(QCoreApplication::translate("PicStatus", "Name of the page item that contains the image", nullptr));
#endif // QT_CONFIG(tooltip)
        displayObjekt->setText(QString());
        textLabel2->setText(QCoreApplication::translate("PicStatus", "Effective DPI:", nullptr));
#if QT_CONFIG(tooltip)
        displayEffDPI->setToolTip(QCoreApplication::translate("PicStatus", "Effective resolution of the image after scaling", nullptr));
#endif // QT_CONFIG(tooltip)
        displayEffDPI->setText(QString());
#if QT_CONFIG(tooltip)
        goPageButton->setToolTip(QCoreApplication::translate("PicStatus", "Move to the page that the image is on", nullptr));
#endif // QT_CONFIG(tooltip)
        goPageButton->setText(QCoreApplication::translate("PicStatus", "Go to", nullptr));
#if QT_CONFIG(tooltip)
        selectButton->setToolTip(QCoreApplication::translate("PicStatus", "Move to the page that the item is on and select it", nullptr));
#endif // QT_CONFIG(tooltip)
        selectButton->setText(QCoreApplication::translate("PicStatus", "Select", nullptr));
        workTab->setTabText(workTab->indexOf(tabInfo), QCoreApplication::translate("PicStatus", "Information", nullptr));
#if QT_CONFIG(tooltip)
        isPrinting->setToolTip(QCoreApplication::translate("PicStatus", "Enable or disable exporting of the item", nullptr));
#endif // QT_CONFIG(tooltip)
        isPrinting->setText(QCoreApplication::translate("PicStatus", "Export/Print Image", nullptr));
#if QT_CONFIG(tooltip)
        buttonLayers->setToolTip(QCoreApplication::translate("PicStatus", "Set format specific properties of certain image types, like clipping paths", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonLayers->setText(QCoreApplication::translate("PicStatus", "Extended Image Properties...", nullptr));
#if QT_CONFIG(shortcut)
        buttonLayers->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        buttonEdit->setToolTip(QCoreApplication::translate("PicStatus", "Edit the image in the default editor", nullptr));
#endif // QT_CONFIG(tooltip)
        buttonEdit->setText(QCoreApplication::translate("PicStatus", "Edit Image...", nullptr));
#if QT_CONFIG(tooltip)
        isVisibleCheck->setToolTip(QCoreApplication::translate("PicStatus", "Make the image visible or invisible", nullptr));
#endif // QT_CONFIG(tooltip)
        isVisibleCheck->setText(QCoreApplication::translate("PicStatus", "Image Visible", nullptr));
#if QT_CONFIG(tooltip)
        effectsButton->setToolTip(QCoreApplication::translate("PicStatus", "Apply non destructive effects to the image in its frame", nullptr));
#endif // QT_CONFIG(tooltip)
        effectsButton->setText(QCoreApplication::translate("PicStatus", "Image Effects...", nullptr));
        workTab->setTabText(workTab->indexOf(tabTasks), QCoreApplication::translate("PicStatus", "Image Tools", nullptr));
        closeButton->setText(QCoreApplication::translate("PicStatus", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PicStatus: public Ui_PicStatus {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PICSTATUS_H
