/********************************************************************************
** Form generated from reading UI file 'prefs_pathsbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_PATHSBASE_H
#define UI_PREFS_PATHSBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Prefs_Paths
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QFrame *line;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_2;
    QGridLayout *gridLayout;
    QLabel *docDirLabel;
    QLineEdit *docDirLineEdit;
    QPushButton *docDirChangeButton;
    QLabel *profileDirLabel;
    QLineEdit *profileDirLineEdit;
    QPushButton *profileDirChangeButton;
    QLabel *scriptDirLabel;
    QLineEdit *scriptDirLineEdit;
    QPushButton *scriptDirChangeButton;
    QLabel *templateDirLabel;
    QLineEdit *templateDirLineEdit;
    QPushButton *templateDirChangeButton;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *Prefs_Paths)
    {
        if (Prefs_Paths->objectName().isEmpty())
            Prefs_Paths->setObjectName(QString::fromUtf8("Prefs_Paths"));
        Prefs_Paths->resize(538, 275);
        verticalLayout = new QVBoxLayout(Prefs_Paths);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(Prefs_Paths);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        verticalLayout->addWidget(label);

        line = new QFrame(Prefs_Paths);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        scrollArea = new QScrollArea(Prefs_Paths);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 510, 205));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, 0);
        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        docDirLabel = new QLabel(scrollAreaWidgetContents);
        docDirLabel->setObjectName(QString::fromUtf8("docDirLabel"));
        docDirLabel->setWordWrap(false);

        gridLayout->addWidget(docDirLabel, 0, 0, 1, 1);

        docDirLineEdit = new QLineEdit(scrollAreaWidgetContents);
        docDirLineEdit->setObjectName(QString::fromUtf8("docDirLineEdit"));
        docDirLineEdit->setMinimumSize(QSize(200, 0));

        gridLayout->addWidget(docDirLineEdit, 0, 1, 1, 1);

        docDirChangeButton = new QPushButton(scrollAreaWidgetContents);
        docDirChangeButton->setObjectName(QString::fromUtf8("docDirChangeButton"));
        docDirChangeButton->setMinimumSize(QSize(88, 0));

        gridLayout->addWidget(docDirChangeButton, 0, 2, 1, 1);

        profileDirLabel = new QLabel(scrollAreaWidgetContents);
        profileDirLabel->setObjectName(QString::fromUtf8("profileDirLabel"));
        profileDirLabel->setWordWrap(false);

        gridLayout->addWidget(profileDirLabel, 1, 0, 1, 1);

        profileDirLineEdit = new QLineEdit(scrollAreaWidgetContents);
        profileDirLineEdit->setObjectName(QString::fromUtf8("profileDirLineEdit"));
        profileDirLineEdit->setMinimumSize(QSize(200, 0));

        gridLayout->addWidget(profileDirLineEdit, 1, 1, 1, 1);

        profileDirChangeButton = new QPushButton(scrollAreaWidgetContents);
        profileDirChangeButton->setObjectName(QString::fromUtf8("profileDirChangeButton"));

        gridLayout->addWidget(profileDirChangeButton, 1, 2, 1, 1);

        scriptDirLabel = new QLabel(scrollAreaWidgetContents);
        scriptDirLabel->setObjectName(QString::fromUtf8("scriptDirLabel"));
        scriptDirLabel->setWordWrap(false);

        gridLayout->addWidget(scriptDirLabel, 2, 0, 1, 1);

        scriptDirLineEdit = new QLineEdit(scrollAreaWidgetContents);
        scriptDirLineEdit->setObjectName(QString::fromUtf8("scriptDirLineEdit"));
        scriptDirLineEdit->setMinimumSize(QSize(200, 0));

        gridLayout->addWidget(scriptDirLineEdit, 2, 1, 1, 1);

        scriptDirChangeButton = new QPushButton(scrollAreaWidgetContents);
        scriptDirChangeButton->setObjectName(QString::fromUtf8("scriptDirChangeButton"));

        gridLayout->addWidget(scriptDirChangeButton, 2, 2, 1, 1);

        templateDirLabel = new QLabel(scrollAreaWidgetContents);
        templateDirLabel->setObjectName(QString::fromUtf8("templateDirLabel"));
        templateDirLabel->setWordWrap(false);

        gridLayout->addWidget(templateDirLabel, 3, 0, 1, 1);

        templateDirLineEdit = new QLineEdit(scrollAreaWidgetContents);
        templateDirLineEdit->setObjectName(QString::fromUtf8("templateDirLineEdit"));
        templateDirLineEdit->setMinimumSize(QSize(200, 0));

        gridLayout->addWidget(templateDirLineEdit, 3, 1, 1, 1);

        templateDirChangeButton = new QPushButton(scrollAreaWidgetContents);
        templateDirChangeButton->setObjectName(QString::fromUtf8("templateDirChangeButton"));

        gridLayout->addWidget(templateDirChangeButton, 3, 2, 1, 1);


        verticalLayout_2->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 27, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout->addWidget(scrollArea);

#if QT_CONFIG(shortcut)
        docDirLabel->setBuddy(docDirLineEdit);
        profileDirLabel->setBuddy(profileDirLineEdit);
        scriptDirLabel->setBuddy(scriptDirLineEdit);
        templateDirLabel->setBuddy(templateDirLineEdit);
#endif // QT_CONFIG(shortcut)

        retranslateUi(Prefs_Paths);

        QMetaObject::connectSlotsByName(Prefs_Paths);
    } // setupUi

    void retranslateUi(QWidget *Prefs_Paths)
    {
        label->setText(QCoreApplication::translate("Prefs_Paths", "Paths", nullptr));
        docDirLabel->setText(QCoreApplication::translate("Prefs_Paths", "Documents:", nullptr));
        docDirChangeButton->setText(QCoreApplication::translate("Prefs_Paths", "&Change...", nullptr));
#if QT_CONFIG(shortcut)
        docDirChangeButton->setShortcut(QCoreApplication::translate("Prefs_Paths", "Alt+C", nullptr));
#endif // QT_CONFIG(shortcut)
        profileDirLabel->setText(QCoreApplication::translate("Prefs_Paths", "Color Profiles:", nullptr));
        profileDirChangeButton->setText(QCoreApplication::translate("Prefs_Paths", "C&hange...", nullptr));
#if QT_CONFIG(shortcut)
        profileDirChangeButton->setShortcut(QCoreApplication::translate("Prefs_Paths", "Alt+H", nullptr));
#endif // QT_CONFIG(shortcut)
        scriptDirLabel->setText(QCoreApplication::translate("Prefs_Paths", "Scripts:", nullptr));
        scriptDirChangeButton->setText(QCoreApplication::translate("Prefs_Paths", "Ch&ange...", nullptr));
#if QT_CONFIG(shortcut)
        scriptDirChangeButton->setShortcut(QCoreApplication::translate("Prefs_Paths", "Alt+A", nullptr));
#endif // QT_CONFIG(shortcut)
        templateDirLabel->setText(QCoreApplication::translate("Prefs_Paths", "Document Templates:", nullptr));
        templateDirChangeButton->setText(QCoreApplication::translate("Prefs_Paths", "Cha&nge...", nullptr));
#if QT_CONFIG(shortcut)
        templateDirChangeButton->setShortcut(QCoreApplication::translate("Prefs_Paths", "Alt+N", nullptr));
#endif // QT_CONFIG(shortcut)
        (void)Prefs_Paths;
    } // retranslateUi

};

namespace Ui {
    class Prefs_Paths: public Ui_Prefs_Paths {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_PATHSBASE_H
