/********************************************************************************
** Form generated from reading UI file 'pageitemattributes.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PAGEITEMATTRIBUTES_H
#define UI_PAGEITEMATTRIBUTES_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "pagestructs.h"
#include "ui/sctablewidget.h"

QT_BEGIN_NAMESPACE

class Ui_PageItemAttributes
{
public:
    QVBoxLayout *vboxLayout;
    ScTableWidget *attributesTable;
    QHBoxLayout *hboxLayout;
    QPushButton *addButton;
    QPushButton *copyButton;
    QPushButton *deleteButton;
    QPushButton *clearButton;
    QSpacerItem *spacerItem;
    QPushButton *okButton;
    QPushButton *cancelButton;

    void setupUi(QDialog *PageItemAttributes)
    {
        if (PageItemAttributes->objectName().isEmpty())
            PageItemAttributes->setObjectName(QString::fromUtf8("PageItemAttributes"));
        PageItemAttributes->resize(823, 188);
        PageItemAttributes->setSizeGripEnabled(true);
        vboxLayout = new QVBoxLayout(PageItemAttributes);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        attributesTable = new ScTableWidget(PageItemAttributes);
        if (attributesTable->columnCount() < 6)
            attributesTable->setColumnCount(6);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        attributesTable->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        attributesTable->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        attributesTable->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        attributesTable->setHorizontalHeaderItem(3, __qtablewidgetitem3);
        QTableWidgetItem *__qtablewidgetitem4 = new QTableWidgetItem();
        attributesTable->setHorizontalHeaderItem(4, __qtablewidgetitem4);
        QTableWidgetItem *__qtablewidgetitem5 = new QTableWidgetItem();
        attributesTable->setHorizontalHeaderItem(5, __qtablewidgetitem5);
        attributesTable->setObjectName(QString::fromUtf8("attributesTable"));

        vboxLayout->addWidget(attributesTable);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        addButton = new QPushButton(PageItemAttributes);
        addButton->setObjectName(QString::fromUtf8("addButton"));

        hboxLayout->addWidget(addButton);

        copyButton = new QPushButton(PageItemAttributes);
        copyButton->setObjectName(QString::fromUtf8("copyButton"));

        hboxLayout->addWidget(copyButton);

        deleteButton = new QPushButton(PageItemAttributes);
        deleteButton->setObjectName(QString::fromUtf8("deleteButton"));

        hboxLayout->addWidget(deleteButton);

        clearButton = new QPushButton(PageItemAttributes);
        clearButton->setObjectName(QString::fromUtf8("clearButton"));

        hboxLayout->addWidget(clearButton);

        spacerItem = new QSpacerItem(90, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        okButton = new QPushButton(PageItemAttributes);
        okButton->setObjectName(QString::fromUtf8("okButton"));
        okButton->setAutoDefault(true);

        hboxLayout->addWidget(okButton);

        cancelButton = new QPushButton(PageItemAttributes);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));
        cancelButton->setAutoDefault(true);

        hboxLayout->addWidget(cancelButton);


        vboxLayout->addLayout(hboxLayout);


        retranslateUi(PageItemAttributes);

        okButton->setDefault(true);


        QMetaObject::connectSlotsByName(PageItemAttributes);
    } // setupUi

    void retranslateUi(QDialog *PageItemAttributes)
    {
        PageItemAttributes->setWindowTitle(QCoreApplication::translate("PageItemAttributes", "Attributes", nullptr));
        QTableWidgetItem *___qtablewidgetitem = attributesTable->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("PageItemAttributes", "Name", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = attributesTable->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("PageItemAttributes", "Type", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = attributesTable->horizontalHeaderItem(2);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("PageItemAttributes", "Value", nullptr));
        QTableWidgetItem *___qtablewidgetitem3 = attributesTable->horizontalHeaderItem(3);
        ___qtablewidgetitem3->setText(QCoreApplication::translate("PageItemAttributes", "Parameter", nullptr));
        QTableWidgetItem *___qtablewidgetitem4 = attributesTable->horizontalHeaderItem(4);
        ___qtablewidgetitem4->setText(QCoreApplication::translate("PageItemAttributes", "Relationship", nullptr));
        QTableWidgetItem *___qtablewidgetitem5 = attributesTable->horizontalHeaderItem(5);
        ___qtablewidgetitem5->setText(QCoreApplication::translate("PageItemAttributes", "Relationship To", nullptr));
        addButton->setText(QCoreApplication::translate("PageItemAttributes", "&Add", nullptr));
#if QT_CONFIG(shortcut)
        addButton->setShortcut(QCoreApplication::translate("PageItemAttributes", "Alt+A", nullptr));
#endif // QT_CONFIG(shortcut)
        copyButton->setText(QCoreApplication::translate("PageItemAttributes", "&Copy", nullptr));
#if QT_CONFIG(shortcut)
        copyButton->setShortcut(QCoreApplication::translate("PageItemAttributes", "Alt+C", nullptr));
#endif // QT_CONFIG(shortcut)
        deleteButton->setText(QCoreApplication::translate("PageItemAttributes", "&Delete", nullptr));
#if QT_CONFIG(shortcut)
        deleteButton->setShortcut(QCoreApplication::translate("PageItemAttributes", "Alt+D", nullptr));
#endif // QT_CONFIG(shortcut)
        clearButton->setText(QCoreApplication::translate("PageItemAttributes", "C&lear", nullptr));
#if QT_CONFIG(shortcut)
        clearButton->setShortcut(QCoreApplication::translate("PageItemAttributes", "Alt+L", nullptr));
#endif // QT_CONFIG(shortcut)
        okButton->setText(QCoreApplication::translate("PageItemAttributes", "&OK", nullptr));
#if QT_CONFIG(shortcut)
        okButton->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
        cancelButton->setText(QCoreApplication::translate("PageItemAttributes", "&Cancel", nullptr));
#if QT_CONFIG(shortcut)
        cancelButton->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class PageItemAttributes: public Ui_PageItemAttributes {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PAGEITEMATTRIBUTES_H
