/********************************************************************************
** Form generated from reading UI file 'hunspelldialogbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_HUNSPELLDIALOGBASE_H
#define UI_HUNSPELLDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_HunspellDialogBase
{
public:
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QComboBox *languagesComboBox;
    QSpacerItem *horizontalSpacer;
    QGridLayout *gridLayout;
    QLabel *label_3;
    QTextEdit *sentenceTextEdit;
    QLabel *label_2;
    QListWidget *suggestionsListWidget;
    QVBoxLayout *verticalLayout_2;
    QPushButton *changePushButton;
    QPushButton *changeAllPushButton;
    QSpacerItem *verticalSpacer;
    QVBoxLayout *verticalLayout;
    QPushButton *ignoreOncePushButton;
    QPushButton *ignoreAllPushButton;
    QSpacerItem *verticalSpacer_2;
    QHBoxLayout *horizontalLayout_4;
    QLabel *statusLabel;
    QSpacerItem *horizontalSpacer_2;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *HunspellDialogBase)
    {
        if (HunspellDialogBase->objectName().isEmpty())
            HunspellDialogBase->setObjectName(QString::fromUtf8("HunspellDialogBase"));
        HunspellDialogBase->resize(687, 406);
        verticalLayout_3 = new QVBoxLayout(HunspellDialogBase);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label = new QLabel(HunspellDialogBase);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout->addWidget(label);

        languagesComboBox = new QComboBox(HunspellDialogBase);
        languagesComboBox->setObjectName(QString::fromUtf8("languagesComboBox"));

        horizontalLayout->addWidget(languagesComboBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_3->addLayout(horizontalLayout);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label_3 = new QLabel(HunspellDialogBase);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 0, 0, 1, 1);

        sentenceTextEdit = new QTextEdit(HunspellDialogBase);
        sentenceTextEdit->setObjectName(QString::fromUtf8("sentenceTextEdit"));

        gridLayout->addWidget(sentenceTextEdit, 1, 0, 1, 1);

        label_2 = new QLabel(HunspellDialogBase);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 2, 0, 1, 1);

        suggestionsListWidget = new QListWidget(HunspellDialogBase);
        suggestionsListWidget->setObjectName(QString::fromUtf8("suggestionsListWidget"));

        gridLayout->addWidget(suggestionsListWidget, 3, 0, 1, 1);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        changePushButton = new QPushButton(HunspellDialogBase);
        changePushButton->setObjectName(QString::fromUtf8("changePushButton"));

        verticalLayout_2->addWidget(changePushButton);

        changeAllPushButton = new QPushButton(HunspellDialogBase);
        changeAllPushButton->setObjectName(QString::fromUtf8("changeAllPushButton"));

        verticalLayout_2->addWidget(changeAllPushButton);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);


        gridLayout->addLayout(verticalLayout_2, 3, 1, 1, 1);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        ignoreOncePushButton = new QPushButton(HunspellDialogBase);
        ignoreOncePushButton->setObjectName(QString::fromUtf8("ignoreOncePushButton"));

        verticalLayout->addWidget(ignoreOncePushButton);

        ignoreAllPushButton = new QPushButton(HunspellDialogBase);
        ignoreAllPushButton->setObjectName(QString::fromUtf8("ignoreAllPushButton"));

        verticalLayout->addWidget(ignoreAllPushButton);

        verticalSpacer_2 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer_2);


        gridLayout->addLayout(verticalLayout, 1, 1, 1, 1);


        verticalLayout_3->addLayout(gridLayout);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        statusLabel = new QLabel(HunspellDialogBase);
        statusLabel->setObjectName(QString::fromUtf8("statusLabel"));

        horizontalLayout_4->addWidget(statusLabel);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_2);

        buttonBox = new QDialogButtonBox(HunspellDialogBase);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        horizontalLayout_4->addWidget(buttonBox);


        verticalLayout_3->addLayout(horizontalLayout_4);


        retranslateUi(HunspellDialogBase);
        QObject::connect(buttonBox, SIGNAL(accepted()), HunspellDialogBase, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), HunspellDialogBase, SLOT(reject()));

        QMetaObject::connectSlotsByName(HunspellDialogBase);
    } // setupUi

    void retranslateUi(QDialog *HunspellDialogBase)
    {
        HunspellDialogBase->setWindowTitle(QCoreApplication::translate("HunspellDialogBase", "Check Spelling", nullptr));
        label->setText(QCoreApplication::translate("HunspellDialogBase", "Text Language:", nullptr));
        label_3->setText(QCoreApplication::translate("HunspellDialogBase", "Not in dictionary", nullptr));
        label_2->setText(QCoreApplication::translate("HunspellDialogBase", "Suggestions", nullptr));
        changePushButton->setText(QCoreApplication::translate("HunspellDialogBase", "Change", nullptr));
        changeAllPushButton->setText(QCoreApplication::translate("HunspellDialogBase", "Change All", nullptr));
        ignoreOncePushButton->setText(QCoreApplication::translate("HunspellDialogBase", "Ignore Once", nullptr));
        ignoreAllPushButton->setText(QCoreApplication::translate("HunspellDialogBase", "Ignore All", nullptr));
        statusLabel->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class HunspellDialogBase: public Ui_HunspellDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_HUNSPELLDIALOGBASE_H
