/********************************************************************************
** Form generated from reading UI file 'propertywidget_pareffectbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYWIDGET_PAREFFECTBASE_H
#define UI_PROPERTYWIDGET_PAREFFECTBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include "ui/numformatcombo.h"
#include "ui/scrspinbox.h"
#include "ui/stylecombos.h"

QT_BEGIN_NAMESPACE

class Ui_PropertyWidget_ParEffectBase
{
public:
    QVBoxLayout *verticalLayout;
    QComboBox *peCombo;
    QGroupBox *dropCapsGroup;
    QHBoxLayout *horizontalLayout;
    QLabel *dropCapLinesLabel;
    QSpinBox *dropCapLines;
    QSpacerItem *horizontalSpacer;
    QGroupBox *bullGroup;
    QGridLayout *gridLayout_3;
    QComboBox *bulletStrEdit;
    QLabel *bulletCharLabel;
    QPushButton *bulletCharTableButton;
    QGroupBox *numGroup;
    QGridLayout *gridLayout_2;
    QHBoxLayout *horizontalLayout_2;
    QLabel *numLevelLabel;
    QSpinBox *numLevelSpin;
    QSpacerItem *horizontalSpacer_2;
    QLabel *numStartLabel;
    QSpinBox *numStart;
    QHBoxLayout *horizontalLayout_3;
    QLabel *numComboLabel;
    QComboBox *numComboBox;
    QSpacerItem *horizontalSpacer_3;
    QLabel *numFormatLabel;
    NumFormatCombo *numFormatCombo;
    QHBoxLayout *horizontalLayout_4;
    QLabel *numPrefixLabel;
    QLineEdit *numPrefix;
    QSpacerItem *horizontalSpacer_4;
    QLabel *numSuffixLabel;
    QLineEdit *numSuffix;
    QGroupBox *peGroup;
    QGridLayout *gridLayout_4;
    QLabel *peCharStyleLabel;
    CharStyleComboBox *peCharStyleCombo;
    QHBoxLayout *horizontalLayout_5;
    QCheckBox *peIndent;
    QSpacerItem *horizontalSpacer_5;
    QLabel *peOffsetLabel;
    ScrSpinBox *peOffset;

    void setupUi(QFrame *PropertyWidget_ParEffectBase)
    {
        if (PropertyWidget_ParEffectBase->objectName().isEmpty())
            PropertyWidget_ParEffectBase->setObjectName(QString::fromUtf8("PropertyWidget_ParEffectBase"));
        PropertyWidget_ParEffectBase->resize(354, 390);
        verticalLayout = new QVBoxLayout(PropertyWidget_ParEffectBase);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        peCombo = new QComboBox(PropertyWidget_ParEffectBase);
        peCombo->setObjectName(QString::fromUtf8("peCombo"));

        verticalLayout->addWidget(peCombo);

        dropCapsGroup = new QGroupBox(PropertyWidget_ParEffectBase);
        dropCapsGroup->setObjectName(QString::fromUtf8("dropCapsGroup"));
        horizontalLayout = new QHBoxLayout(dropCapsGroup);
        horizontalLayout->setSpacing(3);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(6, 6, 6, 6);
        dropCapLinesLabel = new QLabel(dropCapsGroup);
        dropCapLinesLabel->setObjectName(QString::fromUtf8("dropCapLinesLabel"));

        horizontalLayout->addWidget(dropCapLinesLabel);

        dropCapLines = new QSpinBox(dropCapsGroup);
        dropCapLines->setObjectName(QString::fromUtf8("dropCapLines"));
        dropCapLines->setMinimum(2);

        horizontalLayout->addWidget(dropCapLines);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout->addWidget(dropCapsGroup);

        bullGroup = new QGroupBox(PropertyWidget_ParEffectBase);
        bullGroup->setObjectName(QString::fromUtf8("bullGroup"));
        gridLayout_3 = new QGridLayout(bullGroup);
        gridLayout_3->setSpacing(3);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        gridLayout_3->setContentsMargins(6, 6, 6, 6);
        bulletStrEdit = new QComboBox(bullGroup);
        bulletStrEdit->setObjectName(QString::fromUtf8("bulletStrEdit"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(bulletStrEdit->sizePolicy().hasHeightForWidth());
        bulletStrEdit->setSizePolicy(sizePolicy);
        bulletStrEdit->setMinimumSize(QSize(50, 0));
        bulletStrEdit->setEditable(true);

        gridLayout_3->addWidget(bulletStrEdit, 0, 1, 1, 1);

        bulletCharLabel = new QLabel(bullGroup);
        bulletCharLabel->setObjectName(QString::fromUtf8("bulletCharLabel"));

        gridLayout_3->addWidget(bulletCharLabel, 0, 0, 1, 1);

        bulletCharTableButton = new QPushButton(bullGroup);
        bulletCharTableButton->setObjectName(QString::fromUtf8("bulletCharTableButton"));
        sizePolicy.setHeightForWidth(bulletCharTableButton->sizePolicy().hasHeightForWidth());
        bulletCharTableButton->setSizePolicy(sizePolicy);
        bulletCharTableButton->setCheckable(true);

        gridLayout_3->addWidget(bulletCharTableButton, 0, 2, 1, 1);


        verticalLayout->addWidget(bullGroup);

        numGroup = new QGroupBox(PropertyWidget_ParEffectBase);
        numGroup->setObjectName(QString::fromUtf8("numGroup"));
        gridLayout_2 = new QGridLayout(numGroup);
        gridLayout_2->setSpacing(3);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(6, 6, 6, 6);
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(3);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        numLevelLabel = new QLabel(numGroup);
        numLevelLabel->setObjectName(QString::fromUtf8("numLevelLabel"));

        horizontalLayout_2->addWidget(numLevelLabel);

        numLevelSpin = new QSpinBox(numGroup);
        numLevelSpin->setObjectName(QString::fromUtf8("numLevelSpin"));
        numLevelSpin->setMinimum(1);

        horizontalLayout_2->addWidget(numLevelSpin);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_2);

        numStartLabel = new QLabel(numGroup);
        numStartLabel->setObjectName(QString::fromUtf8("numStartLabel"));

        horizontalLayout_2->addWidget(numStartLabel);

        numStart = new QSpinBox(numGroup);
        numStart->setObjectName(QString::fromUtf8("numStart"));
        numStart->setMinimum(1);
        numStart->setMaximum(999);

        horizontalLayout_2->addWidget(numStart);


        gridLayout_2->addLayout(horizontalLayout_2, 3, 0, 1, 2);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(3);
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        numComboLabel = new QLabel(numGroup);
        numComboLabel->setObjectName(QString::fromUtf8("numComboLabel"));

        horizontalLayout_3->addWidget(numComboLabel);

        numComboBox = new QComboBox(numGroup);
        numComboBox->setObjectName(QString::fromUtf8("numComboBox"));

        horizontalLayout_3->addWidget(numComboBox);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer_3);

        numFormatLabel = new QLabel(numGroup);
        numFormatLabel->setObjectName(QString::fromUtf8("numFormatLabel"));

        horizontalLayout_3->addWidget(numFormatLabel);

        numFormatCombo = new NumFormatCombo(numGroup);
        numFormatCombo->setObjectName(QString::fromUtf8("numFormatCombo"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(numFormatCombo->sizePolicy().hasHeightForWidth());
        numFormatCombo->setSizePolicy(sizePolicy1);

        horizontalLayout_3->addWidget(numFormatCombo);


        gridLayout_2->addLayout(horizontalLayout_3, 0, 0, 1, 2);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(3);
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        numPrefixLabel = new QLabel(numGroup);
        numPrefixLabel->setObjectName(QString::fromUtf8("numPrefixLabel"));

        horizontalLayout_4->addWidget(numPrefixLabel);

        numPrefix = new QLineEdit(numGroup);
        numPrefix->setObjectName(QString::fromUtf8("numPrefix"));

        horizontalLayout_4->addWidget(numPrefix);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_4);

        numSuffixLabel = new QLabel(numGroup);
        numSuffixLabel->setObjectName(QString::fromUtf8("numSuffixLabel"));

        horizontalLayout_4->addWidget(numSuffixLabel);

        numSuffix = new QLineEdit(numGroup);
        numSuffix->setObjectName(QString::fromUtf8("numSuffix"));

        horizontalLayout_4->addWidget(numSuffix);


        gridLayout_2->addLayout(horizontalLayout_4, 4, 0, 1, 2);


        verticalLayout->addWidget(numGroup);

        peGroup = new QGroupBox(PropertyWidget_ParEffectBase);
        peGroup->setObjectName(QString::fromUtf8("peGroup"));
        gridLayout_4 = new QGridLayout(peGroup);
        gridLayout_4->setSpacing(3);
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        gridLayout_4->setContentsMargins(6, 6, 6, 6);
        peCharStyleLabel = new QLabel(peGroup);
        peCharStyleLabel->setObjectName(QString::fromUtf8("peCharStyleLabel"));

        gridLayout_4->addWidget(peCharStyleLabel, 6, 0, 1, 1);

        peCharStyleCombo = new CharStyleComboBox(peGroup);
        peCharStyleCombo->setObjectName(QString::fromUtf8("peCharStyleCombo"));

        gridLayout_4->addWidget(peCharStyleCombo, 6, 1, 1, 1);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setSpacing(3);
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        peIndent = new QCheckBox(peGroup);
        peIndent->setObjectName(QString::fromUtf8("peIndent"));

        horizontalLayout_5->addWidget(peIndent);

        horizontalSpacer_5 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_5);

        peOffsetLabel = new QLabel(peGroup);
        peOffsetLabel->setObjectName(QString::fromUtf8("peOffsetLabel"));

        horizontalLayout_5->addWidget(peOffsetLabel);

        peOffset = new ScrSpinBox(peGroup);
        peOffset->setObjectName(QString::fromUtf8("peOffset"));
        peOffset->setMinimum(-3000.000000000000000);
        peOffset->setMaximum(3000.000000000000000);

        horizontalLayout_5->addWidget(peOffset);


        gridLayout_4->addLayout(horizontalLayout_5, 1, 0, 1, 2);


        verticalLayout->addWidget(peGroup);

#if QT_CONFIG(shortcut)
        bulletCharLabel->setBuddy(bulletStrEdit);
        numLevelLabel->setBuddy(numLevelSpin);
        numFormatLabel->setBuddy(numFormatCombo);
        numPrefixLabel->setBuddy(numPrefix);
        numSuffixLabel->setBuddy(numSuffix);
        peCharStyleLabel->setBuddy(peCharStyleCombo);
        peOffsetLabel->setBuddy(peOffset);
#endif // QT_CONFIG(shortcut)

        retranslateUi(PropertyWidget_ParEffectBase);

        numComboBox->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(PropertyWidget_ParEffectBase);
    } // setupUi

    void retranslateUi(QFrame *PropertyWidget_ParEffectBase)
    {
        dropCapsGroup->setTitle(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Drop Caps", nullptr));
#if QT_CONFIG(tooltip)
        dropCapLinesLabel->setToolTip(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Number of lines used by the drop cap", nullptr));
#endif // QT_CONFIG(tooltip)
        dropCapLinesLabel->setText(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Lines:", nullptr));
        bullGroup->setTitle(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Bulleted List", nullptr));
        bulletCharLabel->setText(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Bullet Char(s):", nullptr));
#if QT_CONFIG(tooltip)
        bulletCharTableButton->setToolTip(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Enhanced Character Table to choose bullet characters", nullptr));
#endif // QT_CONFIG(tooltip)
        bulletCharTableButton->setText(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Char Table", nullptr));
        numGroup->setTitle(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Numbered List", nullptr));
        numLevelLabel->setText(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Level:", nullptr));
        numStartLabel->setText(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Start:", nullptr));
        numComboLabel->setText(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Set:", nullptr));
        numFormatLabel->setText(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Format:", nullptr));
        numPrefixLabel->setText(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Prefix:", nullptr));
        numSuffixLabel->setText(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Suffix:", nullptr));
        peGroup->setTitle(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Effect Parameters", nullptr));
        peCharStyleLabel->setText(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Char Style:", nullptr));
#if QT_CONFIG(tooltip)
        peCharStyleCombo->setToolTip(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Choose Character Style or leave blank to use default Paragraph Style", nullptr));
#endif // QT_CONFIG(tooltip)
        peIndent->setText(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Auto-Indent", nullptr));
        peOffsetLabel->setText(QCoreApplication::translate("PropertyWidget_ParEffectBase", "Offset:", nullptr));
        (void)PropertyWidget_ParEffectBase;
    } // retranslateUi

};

namespace Ui {
    class PropertyWidget_ParEffectBase: public Ui_PropertyWidget_ParEffectBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTYWIDGET_PAREFFECTBASE_H
