/********************************************************************************
** Form generated from reading UI file 'newmarginwidgetbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NEWMARGINWIDGETBASE_H
#define UI_NEWMARGINWIDGETBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QWidget>
#include <ui/marginpresetlayout.h>
#include "ui/linkbutton.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_NewMarginWidget
{
public:
    QGridLayout *gridLayout_2;
    QGridLayout *gridLayout;
    QLabel *bottomMarginLabel;
    QLabel *leftMarginLabel;
    ScrSpinBox *bottomMarginSpinBox;
    QLabel *rightMarginLabel;
    QLabel *presetLayoutLabel;
    QLabel *topMarginLabel;
    ScrSpinBox *leftMarginSpinBox;
    ScrSpinBox *rightMarginSpinBox;
    ScrSpinBox *topMarginSpinBox;
    PresetLayout *presetLayoutComboBox;
    QPushButton *printerMarginsPushButton;
    LinkButton *marginLinkButton;

    void setupUi(QWidget *NewMarginWidget)
    {
        if (NewMarginWidget->objectName().isEmpty())
            NewMarginWidget->setObjectName(QString::fromUtf8("NewMarginWidget"));
        NewMarginWidget->resize(308, 230);
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(NewMarginWidget->sizePolicy().hasHeightForWidth());
        NewMarginWidget->setSizePolicy(sizePolicy);
        gridLayout_2 = new QGridLayout(NewMarginWidget);
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        gridLayout_2->setContentsMargins(0, 0, 0, 0);
        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        bottomMarginLabel = new QLabel(NewMarginWidget);
        bottomMarginLabel->setObjectName(QString::fromUtf8("bottomMarginLabel"));

        gridLayout->addWidget(bottomMarginLabel, 4, 0, 1, 1);

        leftMarginLabel = new QLabel(NewMarginWidget);
        leftMarginLabel->setObjectName(QString::fromUtf8("leftMarginLabel"));

        gridLayout->addWidget(leftMarginLabel, 1, 0, 1, 1);

        bottomMarginSpinBox = new ScrSpinBox(NewMarginWidget);
        bottomMarginSpinBox->setObjectName(QString::fromUtf8("bottomMarginSpinBox"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(bottomMarginSpinBox->sizePolicy().hasHeightForWidth());
        bottomMarginSpinBox->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(bottomMarginSpinBox, 4, 1, 1, 1);

        rightMarginLabel = new QLabel(NewMarginWidget);
        rightMarginLabel->setObjectName(QString::fromUtf8("rightMarginLabel"));

        gridLayout->addWidget(rightMarginLabel, 2, 0, 1, 1);

        presetLayoutLabel = new QLabel(NewMarginWidget);
        presetLayoutLabel->setObjectName(QString::fromUtf8("presetLayoutLabel"));

        gridLayout->addWidget(presetLayoutLabel, 0, 0, 1, 1);

        topMarginLabel = new QLabel(NewMarginWidget);
        topMarginLabel->setObjectName(QString::fromUtf8("topMarginLabel"));

        gridLayout->addWidget(topMarginLabel, 3, 0, 1, 1);

        leftMarginSpinBox = new ScrSpinBox(NewMarginWidget);
        leftMarginSpinBox->setObjectName(QString::fromUtf8("leftMarginSpinBox"));
        sizePolicy1.setHeightForWidth(leftMarginSpinBox->sizePolicy().hasHeightForWidth());
        leftMarginSpinBox->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(leftMarginSpinBox, 1, 1, 1, 1);

        rightMarginSpinBox = new ScrSpinBox(NewMarginWidget);
        rightMarginSpinBox->setObjectName(QString::fromUtf8("rightMarginSpinBox"));
        sizePolicy1.setHeightForWidth(rightMarginSpinBox->sizePolicy().hasHeightForWidth());
        rightMarginSpinBox->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(rightMarginSpinBox, 2, 1, 1, 1);

        topMarginSpinBox = new ScrSpinBox(NewMarginWidget);
        topMarginSpinBox->setObjectName(QString::fromUtf8("topMarginSpinBox"));
        sizePolicy1.setHeightForWidth(topMarginSpinBox->sizePolicy().hasHeightForWidth());
        topMarginSpinBox->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(topMarginSpinBox, 3, 1, 1, 1);

        presetLayoutComboBox = new PresetLayout(NewMarginWidget);
        presetLayoutComboBox->setObjectName(QString::fromUtf8("presetLayoutComboBox"));
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(presetLayoutComboBox->sizePolicy().hasHeightForWidth());
        presetLayoutComboBox->setSizePolicy(sizePolicy2);

        gridLayout->addWidget(presetLayoutComboBox, 0, 1, 1, 1);

        printerMarginsPushButton = new QPushButton(NewMarginWidget);
        printerMarginsPushButton->setObjectName(QString::fromUtf8("printerMarginsPushButton"));

        gridLayout->addWidget(printerMarginsPushButton, 5, 1, 1, 1);

        marginLinkButton = new LinkButton(NewMarginWidget);
        marginLinkButton->setObjectName(QString::fromUtf8("marginLinkButton"));
        QSizePolicy sizePolicy3(QSizePolicy::Fixed, QSizePolicy::Preferred);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(marginLinkButton->sizePolicy().hasHeightForWidth());
        marginLinkButton->setSizePolicy(sizePolicy3);
        marginLinkButton->setMinimumSize(QSize(15, 0));
        marginLinkButton->setCheckable(true);
        marginLinkButton->setToolButtonStyle(Qt::ToolButtonIconOnly);
        marginLinkButton->setAutoRaise(true);

        gridLayout->addWidget(marginLinkButton, 1, 2, 4, 1);


        gridLayout_2->addLayout(gridLayout, 0, 0, 1, 1);

#if QT_CONFIG(shortcut)
        bottomMarginLabel->setBuddy(bottomMarginSpinBox);
        leftMarginLabel->setBuddy(leftMarginSpinBox);
        rightMarginLabel->setBuddy(rightMarginSpinBox);
        topMarginLabel->setBuddy(topMarginSpinBox);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(presetLayoutComboBox, leftMarginSpinBox);
        QWidget::setTabOrder(leftMarginSpinBox, rightMarginSpinBox);
        QWidget::setTabOrder(rightMarginSpinBox, topMarginSpinBox);
        QWidget::setTabOrder(topMarginSpinBox, bottomMarginSpinBox);
        QWidget::setTabOrder(bottomMarginSpinBox, marginLinkButton);
        QWidget::setTabOrder(marginLinkButton, printerMarginsPushButton);

        retranslateUi(NewMarginWidget);

        QMetaObject::connectSlotsByName(NewMarginWidget);
    } // setupUi

    void retranslateUi(QWidget *NewMarginWidget)
    {
        bottomMarginLabel->setText(QCoreApplication::translate("NewMarginWidget", "Bottom:", nullptr));
        leftMarginLabel->setText(QCoreApplication::translate("NewMarginWidget", "Left:", nullptr));
        rightMarginLabel->setText(QCoreApplication::translate("NewMarginWidget", "Right:", nullptr));
        presetLayoutLabel->setText(QCoreApplication::translate("NewMarginWidget", "Preset Layouts:", nullptr));
        topMarginLabel->setText(QCoreApplication::translate("NewMarginWidget", "Top:", nullptr));
        printerMarginsPushButton->setText(QCoreApplication::translate("NewMarginWidget", "Printer Margins...", nullptr));
        marginLinkButton->setText(QString());
        (void)NewMarginWidget;
    } // retranslateUi

};

namespace Ui {
    class NewMarginWidget: public Ui_NewMarginWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_NEWMARGINWIDGETBASE_H
