/********************************************************************************
** Form generated from reading UI file 'prefs_itemtoolsbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_ITEMTOOLSBASE_H
#define UI_PREFS_ITEMTOOLSBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/arcwidget.h"
#include "ui/arrowchooser.h"
#include "ui/colorcombo.h"
#include "ui/fontcombo.h"
#include "ui/linecombo.h"
#include "ui/linkbutton.h"
#include "ui/polygonwidget.h"
#include "ui/scrspinbox.h"
#include "ui/spiralwidget.h"

QT_BEGIN_NAMESPACE

class Ui_Prefs_ItemTools
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QFrame *line;
    QTabWidget *tabWidget;
    QWidget *tabWidgetPage1;
    QVBoxLayout *verticalLayout_3;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QGridLayout *gridLayout_5;
    QSpacerItem *verticalSpacer_3;
    QSpacerItem *verticalSpacer;
    QLabel *label_19;
    QLabel *label_43;
    QLabel *label_42;
    QGridLayout *gridLayout_3;
    ScrSpinBox *rightTextDistanceSpinBox;
    ScrSpinBox *textColumnGapSpinBox;
    QSpinBox *textColumnsSpinBox;
    ScrSpinBox *topTextDistanceSpinBox;
    QLabel *TopLabel;
    ScrSpinBox *leftTextDistanceSpinBox;
    QLabel *BottomLabel;
    QLabel *LefLabel;
    QLabel *ColumnsLabel;
    QLabel *ColumnGapLabel;
    ScrSpinBox *bottomTextDistanceSpinBox;
    QLabel *RightLabel;
    QSpacerItem *horizontalSpacer_9;
    QLabel *textPreviewWidget;
    QLabel *ColumnsAndTextDistances;
    QGridLayout *gridLayout_4;
    QLabel *textColorStrokeLabel;
    ColorCombo *textStrokeColorComboBox;
    QSpinBox *textStrokeShadingSpinBox;
    QLabel *textColorShadingLabel;
    QLabel *StrokeLabel;
    QSpinBox *textColorShadingSpinBox;
    QLabel *FontLabel;
    ScrSpinBox *textSizeSpinBox;
    QLabel *textColorLabel;
    ColorCombo *textColorComboBox;
    QLabel *SizeLabel;
    QSpacerItem *horizontalSpacer_2;
    QSpacerItem *horizontalSpacer;
    FontCombo *textFontComboBox;
    QSpacerItem *verticalSpacer_2;
    QGridLayout *gridLayout_2;
    QLabel *textLineColorLabel;
    ScrSpinBox *textTabWidthSpinBox;
    ColorCombo *textFrameLineColorComboBox;
    QLabel *TabFillCharacterLabel;
    QSpinBox *textFrameLineShadingSpinBox;
    QLabel *textFillColorShadingLabel;
    ColorCombo *textFrameFillColorComboBox;
    QLabel *TabWidthLabel;
    QLabel *textLineColorShadingLabel;
    QSpinBox *textFrameFillShadingSpinBox;
    QComboBox *textTabFillCharComboBox;
    QLabel *textFillColorLabel;
    QSpacerItem *horizontalSpacer_13;
    QSpacerItem *horizontalSpacer_8;
    QSpacerItem *verticalSpacer_10;
    QLabel *label_11;
    QLabel *label_2;
    QFrame *line_5;
    QFrame *line_6;
    QFrame *line_7;
    QFrame *line_3;
    QSpacerItem *horizontalSpacer_15;
    QSpacerItem *verticalSpacer_13;
    QGridLayout *gridLayout_8;
    QLabel *label_3;
    QComboBox *firstLineOffsetComboBox;
    QSpacerItem *horizontalSpacer_19;
    QWidget *tab;
    QVBoxLayout *verticalLayout_4;
    QScrollArea *scrollArea_2;
    QWidget *scrollAreaWidgetContents_2;
    QFormLayout *formLayout_6;
    QCheckBox *imageUseEmbeddedClippingPathCheckBox;
    QSpacerItem *verticalSpacer_4;
    QLabel *ScalingLabel;
    QFrame *line_2;
    QRadioButton *imageFreeScalingRadioButton;
    QRadioButton *imageFrameScalingRadioButton;
    QCheckBox *imageKeepAspectRatioCheckBox;
    QSpacerItem *verticalSpacer_5;
    QLabel *OnScreenPreviewLabel;
    QFrame *line_4;
    QRadioButton *onscreenResolutionFullRadioButton;
    QRadioButton *onscreenResolutionNormalRadioButton;
    QRadioButton *onscreenResolutionLowRadioButton;
    QSpacerItem *verticalSpacer_6;
    QFormLayout *formLayout_5;
    QGridLayout *gridLayout_7;
    QLabel *imageVerticalScaleLabel;
    QSpinBox *imageHorizontalScalingSpinBox;
    QLabel *imageHorizontalScaleLabel;
    LinkButton *scalingLockToolButton;
    QSpinBox *imageVerticalScalingSpinBox;
    QSpacerItem *horizontalSpacer_4;
    QGridLayout *gridLayout_6;
    QSpinBox *imageFrameLineShadingSpinBox;
    QLabel *imageFillColorLabel;
    QLabel *imageLineColorShadeLabel;
    ColorCombo *imageFrameFillColorComboBox;
    QSpinBox *imageFrameFillShadingSpinBox;
    QLabel *imageLineColorLabel;
    ColorCombo *imageFrameLineColorComboBox;
    QSpacerItem *horizontalSpacer_16;
    QLabel *imageFillShadingLabel;
    QSpacerItem *horizontalSpacer_3;
    QWidget *tabWidgetPage2;
    QFormLayout *formLayout;
    QSpacerItem *verticalSpacer_8;
    QGridLayout *gridLayout_9;
    QLabel *shapeFillColorShadingLabel;
    QLabel *shapeLineColorLabel;
    QSpinBox *shapeFrameFillShadingSpinBox;
    QLabel *shapeLineStyleLabel;
    QLabel *shapeLineWidthLabel;
    QLabel *shapeLineColorShadingLabel;
    ColorCombo *shapeFrameLineColorComboBox;
    QSpinBox *shapeFrameLineShadingSpinBox;
    QSpacerItem *horizontalSpacer_5;
    LineCombo *shapeFrameLineStyleComboBox;
    ColorCombo *shapeFrameFillColorComboBox;
    QLabel *shapeFillColorLabel;
    ScrSpinBox *shapeLineWidthSpinBox;
    QSpacerItem *horizontalSpacer_6;
    QWidget *tab_2;
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout_7;
    PolygonWidget *polygonWidget;
    QSpacerItem *horizontalSpacer_12;
    QSpacerItem *verticalSpacer_9;
    QWidget *tab_3;
    QFormLayout *formLayout_2;
    QSpacerItem *verticalSpacer_7;
    QGridLayout *gridLayout_10;
    QLabel *lineColorShadingLabel;
    ColorCombo *lineFillColorComboBox;
    QSpacerItem *horizontalSpacer_7;
    QLabel *endArrowLabel;
    QLabel *lineWidthLabel;
    QLabel *lineStyleLabel;
    QLabel *startArrowLabel;
    QSpinBox *lineFillColorShadingSpinBox;
    LineCombo *lineStyleComboBox;
    ArrowChooser *lineStartArrowComboBox;
    ArrowChooser *lineEndArrowComboBox;
    QLabel *lineColorLabel;
    ScrSpinBox *lineWidthSpinBox;
    QSpacerItem *horizontalSpacer_10;
    QWidget *tab_4;
    QFormLayout *formLayout_3;
    QSpacerItem *verticalSpacer_11;
    QGridLayout *gridLayout;
    QSpinBox *calPenFillShadingSpinBox;
    ScrSpinBox *calPenAngleSpinBox;
    QLabel *calLineStyleLabel;
    ColorCombo *calPenFillColorComboBox;
    QLabel *calFillColorShadingLabel;
    QLabel *calLineColorLabel;
    ScrSpinBox *calPenLineWidthSpinBox;
    QSpinBox *calPenLineShadingSpinBox;
    QLabel *calPenAngleLabel;
    QLabel *calLineWidthLabel;
    QLabel *calFillColorLabel;
    ColorCombo *calPenColorComboBox;
    QLabel *calLineColorShadingLabel;
    QSpacerItem *horizontalSpacer_11;
    LineCombo *calPenStyleComboBox;
    ScrSpinBox *calPenWidthSpinBox;
    QLabel *calPenWidthLabel;
    QSpacerItem *horizontalSpacer_14;
    QWidget *tab_5;
    QVBoxLayout *verticalLayout_10;
    QHBoxLayout *horizontalLayout_11;
    ArcWidget *arcDisplay;
    QSpacerItem *horizontalSpacer_17;
    QSpacerItem *verticalSpacer_12;
    QWidget *tab_6;
    QVBoxLayout *verticalLayout_11;
    QHBoxLayout *horizontalLayout_12;
    SpiralWidget *spiralDisplay;
    QSpacerItem *horizontalSpacer_18;
    QButtonGroup *previewButtonGroup;
    QButtonGroup *scalingButtonGroup;

    void setupUi(QWidget *Prefs_ItemTools)
    {
        if (Prefs_ItemTools->objectName().isEmpty())
            Prefs_ItemTools->setObjectName(QString::fromUtf8("Prefs_ItemTools"));
        Prefs_ItemTools->resize(659, 762);
        verticalLayout = new QVBoxLayout(Prefs_ItemTools);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(Prefs_ItemTools);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        verticalLayout->addWidget(label);

        line = new QFrame(Prefs_ItemTools);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        tabWidget = new QTabWidget(Prefs_ItemTools);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tabWidget->setTabPosition(QTabWidget::North);
        tabWidget->setTabShape(QTabWidget::Rounded);
        tabWidget->setElideMode(Qt::ElideRight);
        tabWidget->setUsesScrollButtons(false);
        tabWidgetPage1 = new QWidget();
        tabWidgetPage1->setObjectName(QString::fromUtf8("tabWidgetPage1"));
        verticalLayout_3 = new QVBoxLayout(tabWidgetPage1);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        scrollArea = new QScrollArea(tabWidgetPage1);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setLineWidth(0);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 635, 680));
        gridLayout_5 = new QGridLayout(scrollAreaWidgetContents);
        gridLayout_5->setObjectName(QString::fromUtf8("gridLayout_5"));
        verticalSpacer_3 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        gridLayout_5->addItem(verticalSpacer_3, 11, 0, 1, 3);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        gridLayout_5->addItem(verticalSpacer, 19, 0, 1, 2);

        label_19 = new QLabel(scrollAreaWidgetContents);
        label_19->setObjectName(QString::fromUtf8("label_19"));
        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
        label_19->setFont(font1);

        gridLayout_5->addWidget(label_19, 8, 0, 1, 1);

        label_43 = new QLabel(scrollAreaWidgetContents);
        label_43->setObjectName(QString::fromUtf8("label_43"));

        gridLayout_5->addWidget(label_43, 2, 1, 1, 1);

        label_42 = new QLabel(scrollAreaWidgetContents);
        label_42->setObjectName(QString::fromUtf8("label_42"));

        gridLayout_5->addWidget(label_42, 1, 1, 1, 1);

        gridLayout_3 = new QGridLayout();
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        rightTextDistanceSpinBox = new ScrSpinBox(scrollAreaWidgetContents);
        rightTextDistanceSpinBox->setObjectName(QString::fromUtf8("rightTextDistanceSpinBox"));

        gridLayout_3->addWidget(rightTextDistanceSpinBox, 2, 4, 1, 1);

        textColumnGapSpinBox = new ScrSpinBox(scrollAreaWidgetContents);
        textColumnGapSpinBox->setObjectName(QString::fromUtf8("textColumnGapSpinBox"));

        gridLayout_3->addWidget(textColumnGapSpinBox, 0, 4, 1, 1);

        textColumnsSpinBox = new QSpinBox(scrollAreaWidgetContents);
        textColumnsSpinBox->setObjectName(QString::fromUtf8("textColumnsSpinBox"));
        textColumnsSpinBox->setMinimum(1);

        gridLayout_3->addWidget(textColumnsSpinBox, 0, 1, 1, 1);

        topTextDistanceSpinBox = new ScrSpinBox(scrollAreaWidgetContents);
        topTextDistanceSpinBox->setObjectName(QString::fromUtf8("topTextDistanceSpinBox"));

        gridLayout_3->addWidget(topTextDistanceSpinBox, 1, 1, 1, 1);

        TopLabel = new QLabel(scrollAreaWidgetContents);
        TopLabel->setObjectName(QString::fromUtf8("TopLabel"));

        gridLayout_3->addWidget(TopLabel, 1, 0, 1, 1);

        leftTextDistanceSpinBox = new ScrSpinBox(scrollAreaWidgetContents);
        leftTextDistanceSpinBox->setObjectName(QString::fromUtf8("leftTextDistanceSpinBox"));

        gridLayout_3->addWidget(leftTextDistanceSpinBox, 1, 4, 1, 1);

        BottomLabel = new QLabel(scrollAreaWidgetContents);
        BottomLabel->setObjectName(QString::fromUtf8("BottomLabel"));

        gridLayout_3->addWidget(BottomLabel, 2, 0, 1, 1);

        LefLabel = new QLabel(scrollAreaWidgetContents);
        LefLabel->setObjectName(QString::fromUtf8("LefLabel"));

        gridLayout_3->addWidget(LefLabel, 1, 3, 1, 1);

        ColumnsLabel = new QLabel(scrollAreaWidgetContents);
        ColumnsLabel->setObjectName(QString::fromUtf8("ColumnsLabel"));

        gridLayout_3->addWidget(ColumnsLabel, 0, 0, 1, 1);

        ColumnGapLabel = new QLabel(scrollAreaWidgetContents);
        ColumnGapLabel->setObjectName(QString::fromUtf8("ColumnGapLabel"));

        gridLayout_3->addWidget(ColumnGapLabel, 0, 3, 1, 1);

        bottomTextDistanceSpinBox = new ScrSpinBox(scrollAreaWidgetContents);
        bottomTextDistanceSpinBox->setObjectName(QString::fromUtf8("bottomTextDistanceSpinBox"));

        gridLayout_3->addWidget(bottomTextDistanceSpinBox, 2, 1, 1, 1);

        RightLabel = new QLabel(scrollAreaWidgetContents);
        RightLabel->setObjectName(QString::fromUtf8("RightLabel"));

        gridLayout_3->addWidget(RightLabel, 2, 3, 1, 1);

        horizontalSpacer_9 = new QSpacerItem(20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout_3->addItem(horizontalSpacer_9, 0, 2, 3, 1);


        gridLayout_5->addLayout(gridLayout_3, 18, 0, 1, 1);

        textPreviewWidget = new QLabel(scrollAreaWidgetContents);
        textPreviewWidget->setObjectName(QString::fromUtf8("textPreviewWidget"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(textPreviewWidget->sizePolicy().hasHeightForWidth());
        textPreviewWidget->setSizePolicy(sizePolicy);
        textPreviewWidget->setMinimumSize(QSize(400, 100));
        textPreviewWidget->setFrameShape(QFrame::NoFrame);

        gridLayout_5->addWidget(textPreviewWidget, 22, 0, 1, 3);

        ColumnsAndTextDistances = new QLabel(scrollAreaWidgetContents);
        ColumnsAndTextDistances->setObjectName(QString::fromUtf8("ColumnsAndTextDistances"));
        ColumnsAndTextDistances->setFont(font1);

        gridLayout_5->addWidget(ColumnsAndTextDistances, 16, 0, 1, 1);

        gridLayout_4 = new QGridLayout();
        gridLayout_4->setObjectName(QString::fromUtf8("gridLayout_4"));
        textColorStrokeLabel = new QLabel(scrollAreaWidgetContents);
        textColorStrokeLabel->setObjectName(QString::fromUtf8("textColorStrokeLabel"));

        gridLayout_4->addWidget(textColorStrokeLabel, 3, 3, 1, 1);

        textStrokeColorComboBox = new ColorCombo(scrollAreaWidgetContents);
        textStrokeColorComboBox->setObjectName(QString::fromUtf8("textStrokeColorComboBox"));

        gridLayout_4->addWidget(textStrokeColorComboBox, 3, 1, 1, 1);

        textStrokeShadingSpinBox = new QSpinBox(scrollAreaWidgetContents);
        textStrokeShadingSpinBox->setObjectName(QString::fromUtf8("textStrokeShadingSpinBox"));
        textStrokeShadingSpinBox->setMaximum(100);

        gridLayout_4->addWidget(textStrokeShadingSpinBox, 3, 4, 1, 1);

        textColorShadingLabel = new QLabel(scrollAreaWidgetContents);
        textColorShadingLabel->setObjectName(QString::fromUtf8("textColorShadingLabel"));

        gridLayout_4->addWidget(textColorShadingLabel, 2, 3, 1, 1);

        StrokeLabel = new QLabel(scrollAreaWidgetContents);
        StrokeLabel->setObjectName(QString::fromUtf8("StrokeLabel"));

        gridLayout_4->addWidget(StrokeLabel, 3, 0, 1, 1);

        textColorShadingSpinBox = new QSpinBox(scrollAreaWidgetContents);
        textColorShadingSpinBox->setObjectName(QString::fromUtf8("textColorShadingSpinBox"));
        textColorShadingSpinBox->setMaximum(100);

        gridLayout_4->addWidget(textColorShadingSpinBox, 2, 4, 1, 1);

        FontLabel = new QLabel(scrollAreaWidgetContents);
        FontLabel->setObjectName(QString::fromUtf8("FontLabel"));

        gridLayout_4->addWidget(FontLabel, 0, 0, 1, 1);

        textSizeSpinBox = new ScrSpinBox(scrollAreaWidgetContents);
        textSizeSpinBox->setObjectName(QString::fromUtf8("textSizeSpinBox"));

        gridLayout_4->addWidget(textSizeSpinBox, 1, 1, 1, 1);

        textColorLabel = new QLabel(scrollAreaWidgetContents);
        textColorLabel->setObjectName(QString::fromUtf8("textColorLabel"));

        gridLayout_4->addWidget(textColorLabel, 2, 0, 1, 1);

        textColorComboBox = new ColorCombo(scrollAreaWidgetContents);
        textColorComboBox->setObjectName(QString::fromUtf8("textColorComboBox"));

        gridLayout_4->addWidget(textColorComboBox, 2, 1, 1, 1);

        SizeLabel = new QLabel(scrollAreaWidgetContents);
        SizeLabel->setObjectName(QString::fromUtf8("SizeLabel"));

        gridLayout_4->addWidget(SizeLabel, 1, 0, 1, 1);

        horizontalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout_4->addItem(horizontalSpacer_2, 2, 2, 1, 1);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_4->addItem(horizontalSpacer, 1, 5, 3, 1);

        textFontComboBox = new FontCombo(scrollAreaWidgetContents);
        textFontComboBox->setObjectName(QString::fromUtf8("textFontComboBox"));

        gridLayout_4->addWidget(textFontComboBox, 0, 1, 1, 5);


        gridLayout_5->addLayout(gridLayout_4, 10, 0, 1, 3);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        gridLayout_5->addItem(verticalSpacer_2, 7, 0, 1, 2);

        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        textLineColorLabel = new QLabel(scrollAreaWidgetContents);
        textLineColorLabel->setObjectName(QString::fromUtf8("textLineColorLabel"));

        gridLayout_2->addWidget(textLineColorLabel, 1, 0, 1, 1);

        textTabWidthSpinBox = new ScrSpinBox(scrollAreaWidgetContents);
        textTabWidthSpinBox->setObjectName(QString::fromUtf8("textTabWidthSpinBox"));

        gridLayout_2->addWidget(textTabWidthSpinBox, 3, 1, 1, 1);

        textFrameLineColorComboBox = new ColorCombo(scrollAreaWidgetContents);
        textFrameLineColorComboBox->setObjectName(QString::fromUtf8("textFrameLineColorComboBox"));
        QSizePolicy sizePolicy1(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(textFrameLineColorComboBox->sizePolicy().hasHeightForWidth());
        textFrameLineColorComboBox->setSizePolicy(sizePolicy1);

        gridLayout_2->addWidget(textFrameLineColorComboBox, 1, 1, 1, 1);

        TabFillCharacterLabel = new QLabel(scrollAreaWidgetContents);
        TabFillCharacterLabel->setObjectName(QString::fromUtf8("TabFillCharacterLabel"));

        gridLayout_2->addWidget(TabFillCharacterLabel, 2, 0, 1, 1);

        textFrameLineShadingSpinBox = new QSpinBox(scrollAreaWidgetContents);
        textFrameLineShadingSpinBox->setObjectName(QString::fromUtf8("textFrameLineShadingSpinBox"));
        textFrameLineShadingSpinBox->setMaximum(100);

        gridLayout_2->addWidget(textFrameLineShadingSpinBox, 1, 4, 1, 1);

        textFillColorShadingLabel = new QLabel(scrollAreaWidgetContents);
        textFillColorShadingLabel->setObjectName(QString::fromUtf8("textFillColorShadingLabel"));
        QSizePolicy sizePolicy2(QSizePolicy::Preferred, QSizePolicy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(textFillColorShadingLabel->sizePolicy().hasHeightForWidth());
        textFillColorShadingLabel->setSizePolicy(sizePolicy2);

        gridLayout_2->addWidget(textFillColorShadingLabel, 0, 3, 1, 1);

        textFrameFillColorComboBox = new ColorCombo(scrollAreaWidgetContents);
        textFrameFillColorComboBox->setObjectName(QString::fromUtf8("textFrameFillColorComboBox"));
        sizePolicy1.setHeightForWidth(textFrameFillColorComboBox->sizePolicy().hasHeightForWidth());
        textFrameFillColorComboBox->setSizePolicy(sizePolicy1);

        gridLayout_2->addWidget(textFrameFillColorComboBox, 0, 1, 1, 1);

        TabWidthLabel = new QLabel(scrollAreaWidgetContents);
        TabWidthLabel->setObjectName(QString::fromUtf8("TabWidthLabel"));

        gridLayout_2->addWidget(TabWidthLabel, 3, 0, 1, 1);

        textLineColorShadingLabel = new QLabel(scrollAreaWidgetContents);
        textLineColorShadingLabel->setObjectName(QString::fromUtf8("textLineColorShadingLabel"));
        sizePolicy2.setHeightForWidth(textLineColorShadingLabel->sizePolicy().hasHeightForWidth());
        textLineColorShadingLabel->setSizePolicy(sizePolicy2);

        gridLayout_2->addWidget(textLineColorShadingLabel, 1, 3, 1, 1);

        textFrameFillShadingSpinBox = new QSpinBox(scrollAreaWidgetContents);
        textFrameFillShadingSpinBox->setObjectName(QString::fromUtf8("textFrameFillShadingSpinBox"));
        textFrameFillShadingSpinBox->setMaximum(100);

        gridLayout_2->addWidget(textFrameFillShadingSpinBox, 0, 4, 1, 1);

        textTabFillCharComboBox = new QComboBox(scrollAreaWidgetContents);
        textTabFillCharComboBox->setObjectName(QString::fromUtf8("textTabFillCharComboBox"));
        sizePolicy1.setHeightForWidth(textTabFillCharComboBox->sizePolicy().hasHeightForWidth());
        textTabFillCharComboBox->setSizePolicy(sizePolicy1);

        gridLayout_2->addWidget(textTabFillCharComboBox, 2, 1, 1, 1);

        textFillColorLabel = new QLabel(scrollAreaWidgetContents);
        textFillColorLabel->setObjectName(QString::fromUtf8("textFillColorLabel"));

        gridLayout_2->addWidget(textFillColorLabel, 0, 0, 1, 1);

        horizontalSpacer_13 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_13, 0, 5, 1, 1);

        horizontalSpacer_8 = new QSpacerItem(20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_8, 0, 2, 4, 1);


        gridLayout_5->addLayout(gridLayout_2, 0, 0, 3, 3);

        verticalSpacer_10 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        gridLayout_5->addItem(verticalSpacer_10, 23, 0, 1, 1);

        label_11 = new QLabel(scrollAreaWidgetContents);
        label_11->setObjectName(QString::fromUtf8("label_11"));
        label_11->setFont(font1);

        gridLayout_5->addWidget(label_11, 20, 0, 1, 1);

        label_2 = new QLabel(scrollAreaWidgetContents);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setFont(font1);

        gridLayout_5->addWidget(label_2, 12, 0, 1, 1);

        line_5 = new QFrame(scrollAreaWidgetContents);
        line_5->setObjectName(QString::fromUtf8("line_5"));
        line_5->setFrameShape(QFrame::HLine);
        line_5->setFrameShadow(QFrame::Sunken);

        gridLayout_5->addWidget(line_5, 9, 0, 1, 3);

        line_6 = new QFrame(scrollAreaWidgetContents);
        line_6->setObjectName(QString::fromUtf8("line_6"));
        line_6->setFrameShape(QFrame::HLine);
        line_6->setFrameShadow(QFrame::Sunken);

        gridLayout_5->addWidget(line_6, 17, 0, 1, 3);

        line_7 = new QFrame(scrollAreaWidgetContents);
        line_7->setObjectName(QString::fromUtf8("line_7"));
        line_7->setFrameShape(QFrame::HLine);
        line_7->setFrameShadow(QFrame::Sunken);

        gridLayout_5->addWidget(line_7, 13, 0, 1, 2);

        line_3 = new QFrame(scrollAreaWidgetContents);
        line_3->setObjectName(QString::fromUtf8("line_3"));
        line_3->setFrameShape(QFrame::HLine);
        line_3->setFrameShadow(QFrame::Sunken);

        gridLayout_5->addWidget(line_3, 21, 0, 1, 3);

        horizontalSpacer_15 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_5->addItem(horizontalSpacer_15, 18, 1, 1, 1);

        verticalSpacer_13 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        gridLayout_5->addItem(verticalSpacer_13, 15, 0, 1, 2);

        gridLayout_8 = new QGridLayout();
        gridLayout_8->setObjectName(QString::fromUtf8("gridLayout_8"));
        label_3 = new QLabel(scrollAreaWidgetContents);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout_8->addWidget(label_3, 0, 0, 1, 1);

        firstLineOffsetComboBox = new QComboBox(scrollAreaWidgetContents);
        firstLineOffsetComboBox->addItem(QString());
        firstLineOffsetComboBox->addItem(QString());
        firstLineOffsetComboBox->addItem(QString());
        firstLineOffsetComboBox->addItem(QString());
        firstLineOffsetComboBox->setObjectName(QString::fromUtf8("firstLineOffsetComboBox"));

        gridLayout_8->addWidget(firstLineOffsetComboBox, 0, 1, 1, 1);

        horizontalSpacer_19 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_8->addItem(horizontalSpacer_19, 0, 2, 1, 1);


        gridLayout_5->addLayout(gridLayout_8, 14, 0, 1, 2);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout_3->addWidget(scrollArea);

        tabWidget->addTab(tabWidgetPage1, QString());
        tab = new QWidget();
        tab->setObjectName(QString::fromUtf8("tab"));
        verticalLayout_4 = new QVBoxLayout(tab);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        verticalLayout_4->setContentsMargins(0, 0, 0, 0);
        scrollArea_2 = new QScrollArea(tab);
        scrollArea_2->setObjectName(QString::fromUtf8("scrollArea_2"));
        scrollArea_2->setFrameShape(QFrame::NoFrame);
        scrollArea_2->setFrameShadow(QFrame::Plain);
        scrollArea_2->setLineWidth(0);
        scrollArea_2->setWidgetResizable(true);
        scrollAreaWidgetContents_2 = new QWidget();
        scrollAreaWidgetContents_2->setObjectName(QString::fromUtf8("scrollAreaWidgetContents_2"));
        scrollAreaWidgetContents_2->setGeometry(QRect(0, 0, 635, 680));
        formLayout_6 = new QFormLayout(scrollAreaWidgetContents_2);
        formLayout_6->setObjectName(QString::fromUtf8("formLayout_6"));
        imageUseEmbeddedClippingPathCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        imageUseEmbeddedClippingPathCheckBox->setObjectName(QString::fromUtf8("imageUseEmbeddedClippingPathCheckBox"));

        formLayout_6->setWidget(1, QFormLayout::LabelRole, imageUseEmbeddedClippingPathCheckBox);

        verticalSpacer_4 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        formLayout_6->setItem(2, QFormLayout::LabelRole, verticalSpacer_4);

        ScalingLabel = new QLabel(scrollAreaWidgetContents_2);
        ScalingLabel->setObjectName(QString::fromUtf8("ScalingLabel"));
        ScalingLabel->setFont(font1);

        formLayout_6->setWidget(3, QFormLayout::LabelRole, ScalingLabel);

        line_2 = new QFrame(scrollAreaWidgetContents_2);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        formLayout_6->setWidget(4, QFormLayout::SpanningRole, line_2);

        imageFreeScalingRadioButton = new QRadioButton(scrollAreaWidgetContents_2);
        scalingButtonGroup = new QButtonGroup(Prefs_ItemTools);
        scalingButtonGroup->setObjectName(QString::fromUtf8("scalingButtonGroup"));
        scalingButtonGroup->addButton(imageFreeScalingRadioButton);
        imageFreeScalingRadioButton->setObjectName(QString::fromUtf8("imageFreeScalingRadioButton"));

        formLayout_6->setWidget(5, QFormLayout::LabelRole, imageFreeScalingRadioButton);

        imageFrameScalingRadioButton = new QRadioButton(scrollAreaWidgetContents_2);
        scalingButtonGroup->addButton(imageFrameScalingRadioButton);
        imageFrameScalingRadioButton->setObjectName(QString::fromUtf8("imageFrameScalingRadioButton"));

        formLayout_6->setWidget(7, QFormLayout::LabelRole, imageFrameScalingRadioButton);

        imageKeepAspectRatioCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        imageKeepAspectRatioCheckBox->setObjectName(QString::fromUtf8("imageKeepAspectRatioCheckBox"));

        formLayout_6->setWidget(8, QFormLayout::LabelRole, imageKeepAspectRatioCheckBox);

        verticalSpacer_5 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        formLayout_6->setItem(9, QFormLayout::LabelRole, verticalSpacer_5);

        OnScreenPreviewLabel = new QLabel(scrollAreaWidgetContents_2);
        OnScreenPreviewLabel->setObjectName(QString::fromUtf8("OnScreenPreviewLabel"));
        OnScreenPreviewLabel->setFont(font1);

        formLayout_6->setWidget(10, QFormLayout::LabelRole, OnScreenPreviewLabel);

        line_4 = new QFrame(scrollAreaWidgetContents_2);
        line_4->setObjectName(QString::fromUtf8("line_4"));
        line_4->setFrameShape(QFrame::HLine);
        line_4->setFrameShadow(QFrame::Sunken);

        formLayout_6->setWidget(11, QFormLayout::SpanningRole, line_4);

        onscreenResolutionFullRadioButton = new QRadioButton(scrollAreaWidgetContents_2);
        previewButtonGroup = new QButtonGroup(Prefs_ItemTools);
        previewButtonGroup->setObjectName(QString::fromUtf8("previewButtonGroup"));
        previewButtonGroup->addButton(onscreenResolutionFullRadioButton);
        onscreenResolutionFullRadioButton->setObjectName(QString::fromUtf8("onscreenResolutionFullRadioButton"));

        formLayout_6->setWidget(12, QFormLayout::LabelRole, onscreenResolutionFullRadioButton);

        onscreenResolutionNormalRadioButton = new QRadioButton(scrollAreaWidgetContents_2);
        previewButtonGroup->addButton(onscreenResolutionNormalRadioButton);
        onscreenResolutionNormalRadioButton->setObjectName(QString::fromUtf8("onscreenResolutionNormalRadioButton"));

        formLayout_6->setWidget(13, QFormLayout::LabelRole, onscreenResolutionNormalRadioButton);

        onscreenResolutionLowRadioButton = new QRadioButton(scrollAreaWidgetContents_2);
        previewButtonGroup->addButton(onscreenResolutionLowRadioButton);
        onscreenResolutionLowRadioButton->setObjectName(QString::fromUtf8("onscreenResolutionLowRadioButton"));

        formLayout_6->setWidget(14, QFormLayout::LabelRole, onscreenResolutionLowRadioButton);

        verticalSpacer_6 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        formLayout_6->setItem(15, QFormLayout::LabelRole, verticalSpacer_6);

        formLayout_5 = new QFormLayout();
        formLayout_5->setObjectName(QString::fromUtf8("formLayout_5"));
        gridLayout_7 = new QGridLayout();
        gridLayout_7->setObjectName(QString::fromUtf8("gridLayout_7"));
        imageVerticalScaleLabel = new QLabel(scrollAreaWidgetContents_2);
        imageVerticalScaleLabel->setObjectName(QString::fromUtf8("imageVerticalScaleLabel"));

        gridLayout_7->addWidget(imageVerticalScaleLabel, 1, 0, 1, 1);

        imageHorizontalScalingSpinBox = new QSpinBox(scrollAreaWidgetContents_2);
        imageHorizontalScalingSpinBox->setObjectName(QString::fromUtf8("imageHorizontalScalingSpinBox"));
        QSizePolicy sizePolicy3(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(imageHorizontalScalingSpinBox->sizePolicy().hasHeightForWidth());
        imageHorizontalScalingSpinBox->setSizePolicy(sizePolicy3);
        imageHorizontalScalingSpinBox->setMaximum(1000);

        gridLayout_7->addWidget(imageHorizontalScalingSpinBox, 0, 1, 1, 1);

        imageHorizontalScaleLabel = new QLabel(scrollAreaWidgetContents_2);
        imageHorizontalScaleLabel->setObjectName(QString::fromUtf8("imageHorizontalScaleLabel"));

        gridLayout_7->addWidget(imageHorizontalScaleLabel, 0, 0, 1, 1);

        scalingLockToolButton = new LinkButton(scrollAreaWidgetContents_2);
        scalingLockToolButton->setObjectName(QString::fromUtf8("scalingLockToolButton"));
        QSizePolicy sizePolicy4(QSizePolicy::Fixed, QSizePolicy::Preferred);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(scalingLockToolButton->sizePolicy().hasHeightForWidth());
        scalingLockToolButton->setSizePolicy(sizePolicy4);
        scalingLockToolButton->setMinimumSize(QSize(15, 0));
        scalingLockToolButton->setMaximumSize(QSize(15, 16777215));
        scalingLockToolButton->setCheckable(true);
        scalingLockToolButton->setAutoRaise(true);

        gridLayout_7->addWidget(scalingLockToolButton, 0, 2, 2, 1);

        imageVerticalScalingSpinBox = new QSpinBox(scrollAreaWidgetContents_2);
        imageVerticalScalingSpinBox->setObjectName(QString::fromUtf8("imageVerticalScalingSpinBox"));
        sizePolicy3.setHeightForWidth(imageVerticalScalingSpinBox->sizePolicy().hasHeightForWidth());
        imageVerticalScalingSpinBox->setSizePolicy(sizePolicy3);
        imageVerticalScalingSpinBox->setMaximum(1000);

        gridLayout_7->addWidget(imageVerticalScalingSpinBox, 1, 1, 1, 1);


        formLayout_5->setLayout(0, QFormLayout::LabelRole, gridLayout_7);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        formLayout_5->setItem(0, QFormLayout::FieldRole, horizontalSpacer_4);


        formLayout_6->setLayout(6, QFormLayout::LabelRole, formLayout_5);

        gridLayout_6 = new QGridLayout();
        gridLayout_6->setObjectName(QString::fromUtf8("gridLayout_6"));
        imageFrameLineShadingSpinBox = new QSpinBox(scrollAreaWidgetContents_2);
        imageFrameLineShadingSpinBox->setObjectName(QString::fromUtf8("imageFrameLineShadingSpinBox"));
        imageFrameLineShadingSpinBox->setMaximum(100);

        gridLayout_6->addWidget(imageFrameLineShadingSpinBox, 1, 4, 1, 1);

        imageFillColorLabel = new QLabel(scrollAreaWidgetContents_2);
        imageFillColorLabel->setObjectName(QString::fromUtf8("imageFillColorLabel"));

        gridLayout_6->addWidget(imageFillColorLabel, 0, 0, 1, 1);

        imageLineColorShadeLabel = new QLabel(scrollAreaWidgetContents_2);
        imageLineColorShadeLabel->setObjectName(QString::fromUtf8("imageLineColorShadeLabel"));

        gridLayout_6->addWidget(imageLineColorShadeLabel, 1, 3, 1, 1);

        imageFrameFillColorComboBox = new ColorCombo(scrollAreaWidgetContents_2);
        imageFrameFillColorComboBox->setObjectName(QString::fromUtf8("imageFrameFillColorComboBox"));
        sizePolicy1.setHeightForWidth(imageFrameFillColorComboBox->sizePolicy().hasHeightForWidth());
        imageFrameFillColorComboBox->setSizePolicy(sizePolicy1);

        gridLayout_6->addWidget(imageFrameFillColorComboBox, 0, 1, 1, 1);

        imageFrameFillShadingSpinBox = new QSpinBox(scrollAreaWidgetContents_2);
        imageFrameFillShadingSpinBox->setObjectName(QString::fromUtf8("imageFrameFillShadingSpinBox"));
        imageFrameFillShadingSpinBox->setMaximum(100);

        gridLayout_6->addWidget(imageFrameFillShadingSpinBox, 0, 4, 1, 1);

        imageLineColorLabel = new QLabel(scrollAreaWidgetContents_2);
        imageLineColorLabel->setObjectName(QString::fromUtf8("imageLineColorLabel"));

        gridLayout_6->addWidget(imageLineColorLabel, 1, 0, 1, 1);

        imageFrameLineColorComboBox = new ColorCombo(scrollAreaWidgetContents_2);
        imageFrameLineColorComboBox->setObjectName(QString::fromUtf8("imageFrameLineColorComboBox"));
        sizePolicy1.setHeightForWidth(imageFrameLineColorComboBox->sizePolicy().hasHeightForWidth());
        imageFrameLineColorComboBox->setSizePolicy(sizePolicy1);

        gridLayout_6->addWidget(imageFrameLineColorComboBox, 1, 1, 1, 1);

        horizontalSpacer_16 = new QSpacerItem(20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout_6->addItem(horizontalSpacer_16, 0, 2, 2, 1);

        imageFillShadingLabel = new QLabel(scrollAreaWidgetContents_2);
        imageFillShadingLabel->setObjectName(QString::fromUtf8("imageFillShadingLabel"));

        gridLayout_6->addWidget(imageFillShadingLabel, 0, 3, 1, 1);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_6->addItem(horizontalSpacer_3, 0, 5, 1, 1);


        formLayout_6->setLayout(0, QFormLayout::SpanningRole, gridLayout_6);

        scrollArea_2->setWidget(scrollAreaWidgetContents_2);

        verticalLayout_4->addWidget(scrollArea_2);

        tabWidget->addTab(tab, QString());
        tabWidgetPage2 = new QWidget();
        tabWidgetPage2->setObjectName(QString::fromUtf8("tabWidgetPage2"));
        formLayout = new QFormLayout(tabWidgetPage2);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        verticalSpacer_8 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        formLayout->setItem(1, QFormLayout::LabelRole, verticalSpacer_8);

        gridLayout_9 = new QGridLayout();
        gridLayout_9->setObjectName(QString::fromUtf8("gridLayout_9"));
        shapeFillColorShadingLabel = new QLabel(tabWidgetPage2);
        shapeFillColorShadingLabel->setObjectName(QString::fromUtf8("shapeFillColorShadingLabel"));

        gridLayout_9->addWidget(shapeFillColorShadingLabel, 0, 3, 1, 1);

        shapeLineColorLabel = new QLabel(tabWidgetPage2);
        shapeLineColorLabel->setObjectName(QString::fromUtf8("shapeLineColorLabel"));

        gridLayout_9->addWidget(shapeLineColorLabel, 1, 0, 1, 1);

        shapeFrameFillShadingSpinBox = new QSpinBox(tabWidgetPage2);
        shapeFrameFillShadingSpinBox->setObjectName(QString::fromUtf8("shapeFrameFillShadingSpinBox"));
        shapeFrameFillShadingSpinBox->setMaximum(100);

        gridLayout_9->addWidget(shapeFrameFillShadingSpinBox, 0, 4, 1, 1);

        shapeLineStyleLabel = new QLabel(tabWidgetPage2);
        shapeLineStyleLabel->setObjectName(QString::fromUtf8("shapeLineStyleLabel"));

        gridLayout_9->addWidget(shapeLineStyleLabel, 2, 0, 1, 1);

        shapeLineWidthLabel = new QLabel(tabWidgetPage2);
        shapeLineWidthLabel->setObjectName(QString::fromUtf8("shapeLineWidthLabel"));

        gridLayout_9->addWidget(shapeLineWidthLabel, 3, 0, 1, 1);

        shapeLineColorShadingLabel = new QLabel(tabWidgetPage2);
        shapeLineColorShadingLabel->setObjectName(QString::fromUtf8("shapeLineColorShadingLabel"));
        sizePolicy2.setHeightForWidth(shapeLineColorShadingLabel->sizePolicy().hasHeightForWidth());
        shapeLineColorShadingLabel->setSizePolicy(sizePolicy2);

        gridLayout_9->addWidget(shapeLineColorShadingLabel, 1, 3, 1, 1);

        shapeFrameLineColorComboBox = new ColorCombo(tabWidgetPage2);
        shapeFrameLineColorComboBox->setObjectName(QString::fromUtf8("shapeFrameLineColorComboBox"));
        sizePolicy1.setHeightForWidth(shapeFrameLineColorComboBox->sizePolicy().hasHeightForWidth());
        shapeFrameLineColorComboBox->setSizePolicy(sizePolicy1);

        gridLayout_9->addWidget(shapeFrameLineColorComboBox, 1, 1, 1, 1);

        shapeFrameLineShadingSpinBox = new QSpinBox(tabWidgetPage2);
        shapeFrameLineShadingSpinBox->setObjectName(QString::fromUtf8("shapeFrameLineShadingSpinBox"));
        shapeFrameLineShadingSpinBox->setMaximum(100);

        gridLayout_9->addWidget(shapeFrameLineShadingSpinBox, 1, 4, 1, 1);

        horizontalSpacer_5 = new QSpacerItem(20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout_9->addItem(horizontalSpacer_5, 0, 2, 4, 1);

        shapeFrameLineStyleComboBox = new LineCombo(tabWidgetPage2);
        shapeFrameLineStyleComboBox->setObjectName(QString::fromUtf8("shapeFrameLineStyleComboBox"));
        sizePolicy1.setHeightForWidth(shapeFrameLineStyleComboBox->sizePolicy().hasHeightForWidth());
        shapeFrameLineStyleComboBox->setSizePolicy(sizePolicy1);

        gridLayout_9->addWidget(shapeFrameLineStyleComboBox, 2, 1, 1, 1);

        shapeFrameFillColorComboBox = new ColorCombo(tabWidgetPage2);
        shapeFrameFillColorComboBox->setObjectName(QString::fromUtf8("shapeFrameFillColorComboBox"));
        sizePolicy1.setHeightForWidth(shapeFrameFillColorComboBox->sizePolicy().hasHeightForWidth());
        shapeFrameFillColorComboBox->setSizePolicy(sizePolicy1);

        gridLayout_9->addWidget(shapeFrameFillColorComboBox, 0, 1, 1, 1);

        shapeFillColorLabel = new QLabel(tabWidgetPage2);
        shapeFillColorLabel->setObjectName(QString::fromUtf8("shapeFillColorLabel"));

        gridLayout_9->addWidget(shapeFillColorLabel, 0, 0, 1, 1);

        shapeLineWidthSpinBox = new ScrSpinBox(tabWidgetPage2);
        shapeLineWidthSpinBox->setObjectName(QString::fromUtf8("shapeLineWidthSpinBox"));

        gridLayout_9->addWidget(shapeLineWidthSpinBox, 3, 1, 1, 1);

        horizontalSpacer_6 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_9->addItem(horizontalSpacer_6, 0, 5, 1, 1);


        formLayout->setLayout(0, QFormLayout::SpanningRole, gridLayout_9);

        tabWidget->addTab(tabWidgetPage2, QString());
        tab_2 = new QWidget();
        tab_2->setObjectName(QString::fromUtf8("tab_2"));
        verticalLayout_6 = new QVBoxLayout(tab_2);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        polygonWidget = new PolygonWidget(tab_2);
        polygonWidget->setObjectName(QString::fromUtf8("polygonWidget"));
        polygonWidget->setMinimumSize(QSize(100, 100));

        horizontalLayout_7->addWidget(polygonWidget);

        horizontalSpacer_12 = new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_12);


        verticalLayout_6->addLayout(horizontalLayout_7);

        verticalSpacer_9 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_6->addItem(verticalSpacer_9);

        tabWidget->addTab(tab_2, QString());
        tab_3 = new QWidget();
        tab_3->setObjectName(QString::fromUtf8("tab_3"));
        formLayout_2 = new QFormLayout(tab_3);
        formLayout_2->setObjectName(QString::fromUtf8("formLayout_2"));
        verticalSpacer_7 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        formLayout_2->setItem(1, QFormLayout::LabelRole, verticalSpacer_7);

        gridLayout_10 = new QGridLayout();
        gridLayout_10->setObjectName(QString::fromUtf8("gridLayout_10"));
        lineColorShadingLabel = new QLabel(tab_3);
        lineColorShadingLabel->setObjectName(QString::fromUtf8("lineColorShadingLabel"));

        gridLayout_10->addWidget(lineColorShadingLabel, 0, 3, 1, 1);

        lineFillColorComboBox = new ColorCombo(tab_3);
        lineFillColorComboBox->setObjectName(QString::fromUtf8("lineFillColorComboBox"));
        sizePolicy1.setHeightForWidth(lineFillColorComboBox->sizePolicy().hasHeightForWidth());
        lineFillColorComboBox->setSizePolicy(sizePolicy1);

        gridLayout_10->addWidget(lineFillColorComboBox, 0, 1, 1, 1);

        horizontalSpacer_7 = new QSpacerItem(20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout_10->addItem(horizontalSpacer_7, 0, 2, 3, 1);

        endArrowLabel = new QLabel(tab_3);
        endArrowLabel->setObjectName(QString::fromUtf8("endArrowLabel"));

        gridLayout_10->addWidget(endArrowLabel, 4, 0, 1, 1);

        lineWidthLabel = new QLabel(tab_3);
        lineWidthLabel->setObjectName(QString::fromUtf8("lineWidthLabel"));

        gridLayout_10->addWidget(lineWidthLabel, 2, 0, 1, 1);

        lineStyleLabel = new QLabel(tab_3);
        lineStyleLabel->setObjectName(QString::fromUtf8("lineStyleLabel"));

        gridLayout_10->addWidget(lineStyleLabel, 1, 0, 1, 1);

        startArrowLabel = new QLabel(tab_3);
        startArrowLabel->setObjectName(QString::fromUtf8("startArrowLabel"));

        gridLayout_10->addWidget(startArrowLabel, 3, 0, 1, 1);

        lineFillColorShadingSpinBox = new QSpinBox(tab_3);
        lineFillColorShadingSpinBox->setObjectName(QString::fromUtf8("lineFillColorShadingSpinBox"));
        lineFillColorShadingSpinBox->setMaximum(100);

        gridLayout_10->addWidget(lineFillColorShadingSpinBox, 0, 4, 1, 1);

        lineStyleComboBox = new LineCombo(tab_3);
        lineStyleComboBox->setObjectName(QString::fromUtf8("lineStyleComboBox"));

        gridLayout_10->addWidget(lineStyleComboBox, 1, 1, 1, 1);

        lineStartArrowComboBox = new ArrowChooser(tab_3);
        lineStartArrowComboBox->setObjectName(QString::fromUtf8("lineStartArrowComboBox"));

        gridLayout_10->addWidget(lineStartArrowComboBox, 3, 1, 1, 2);

        lineEndArrowComboBox = new ArrowChooser(tab_3);
        lineEndArrowComboBox->setObjectName(QString::fromUtf8("lineEndArrowComboBox"));

        gridLayout_10->addWidget(lineEndArrowComboBox, 4, 1, 1, 2);

        lineColorLabel = new QLabel(tab_3);
        lineColorLabel->setObjectName(QString::fromUtf8("lineColorLabel"));

        gridLayout_10->addWidget(lineColorLabel, 0, 0, 1, 1);

        lineWidthSpinBox = new ScrSpinBox(tab_3);
        lineWidthSpinBox->setObjectName(QString::fromUtf8("lineWidthSpinBox"));

        gridLayout_10->addWidget(lineWidthSpinBox, 2, 1, 1, 1);

        horizontalSpacer_10 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_10->addItem(horizontalSpacer_10, 0, 5, 1, 1);


        formLayout_2->setLayout(0, QFormLayout::SpanningRole, gridLayout_10);

        tabWidget->addTab(tab_3, QString());
        tab_4 = new QWidget();
        tab_4->setObjectName(QString::fromUtf8("tab_4"));
        formLayout_3 = new QFormLayout(tab_4);
        formLayout_3->setObjectName(QString::fromUtf8("formLayout_3"));
        verticalSpacer_11 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        formLayout_3->setItem(1, QFormLayout::LabelRole, verticalSpacer_11);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        calPenFillShadingSpinBox = new QSpinBox(tab_4);
        calPenFillShadingSpinBox->setObjectName(QString::fromUtf8("calPenFillShadingSpinBox"));
        calPenFillShadingSpinBox->setMaximum(100);

        gridLayout->addWidget(calPenFillShadingSpinBox, 0, 4, 1, 1);

        calPenAngleSpinBox = new ScrSpinBox(tab_4);
        calPenAngleSpinBox->setObjectName(QString::fromUtf8("calPenAngleSpinBox"));

        gridLayout->addWidget(calPenAngleSpinBox, 2, 4, 1, 1);

        calLineStyleLabel = new QLabel(tab_4);
        calLineStyleLabel->setObjectName(QString::fromUtf8("calLineStyleLabel"));

        gridLayout->addWidget(calLineStyleLabel, 2, 0, 1, 1);

        calPenFillColorComboBox = new ColorCombo(tab_4);
        calPenFillColorComboBox->setObjectName(QString::fromUtf8("calPenFillColorComboBox"));
        sizePolicy1.setHeightForWidth(calPenFillColorComboBox->sizePolicy().hasHeightForWidth());
        calPenFillColorComboBox->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(calPenFillColorComboBox, 0, 1, 1, 1);

        calFillColorShadingLabel = new QLabel(tab_4);
        calFillColorShadingLabel->setObjectName(QString::fromUtf8("calFillColorShadingLabel"));

        gridLayout->addWidget(calFillColorShadingLabel, 0, 3, 1, 1);

        calLineColorLabel = new QLabel(tab_4);
        calLineColorLabel->setObjectName(QString::fromUtf8("calLineColorLabel"));

        gridLayout->addWidget(calLineColorLabel, 1, 0, 1, 1);

        calPenLineWidthSpinBox = new ScrSpinBox(tab_4);
        calPenLineWidthSpinBox->setObjectName(QString::fromUtf8("calPenLineWidthSpinBox"));

        gridLayout->addWidget(calPenLineWidthSpinBox, 3, 1, 1, 1);

        calPenLineShadingSpinBox = new QSpinBox(tab_4);
        calPenLineShadingSpinBox->setObjectName(QString::fromUtf8("calPenLineShadingSpinBox"));
        calPenLineShadingSpinBox->setMaximum(100);

        gridLayout->addWidget(calPenLineShadingSpinBox, 1, 4, 1, 1);

        calPenAngleLabel = new QLabel(tab_4);
        calPenAngleLabel->setObjectName(QString::fromUtf8("calPenAngleLabel"));

        gridLayout->addWidget(calPenAngleLabel, 2, 3, 1, 1);

        calLineWidthLabel = new QLabel(tab_4);
        calLineWidthLabel->setObjectName(QString::fromUtf8("calLineWidthLabel"));

        gridLayout->addWidget(calLineWidthLabel, 3, 0, 1, 1);

        calFillColorLabel = new QLabel(tab_4);
        calFillColorLabel->setObjectName(QString::fromUtf8("calFillColorLabel"));

        gridLayout->addWidget(calFillColorLabel, 0, 0, 1, 1);

        calPenColorComboBox = new ColorCombo(tab_4);
        calPenColorComboBox->setObjectName(QString::fromUtf8("calPenColorComboBox"));
        sizePolicy1.setHeightForWidth(calPenColorComboBox->sizePolicy().hasHeightForWidth());
        calPenColorComboBox->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(calPenColorComboBox, 1, 1, 1, 1);

        calLineColorShadingLabel = new QLabel(tab_4);
        calLineColorShadingLabel->setObjectName(QString::fromUtf8("calLineColorShadingLabel"));

        gridLayout->addWidget(calLineColorShadingLabel, 1, 3, 1, 1);

        horizontalSpacer_11 = new QSpacerItem(20, 20, QSizePolicy::Fixed, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer_11, 0, 2, 4, 1);

        calPenStyleComboBox = new LineCombo(tab_4);
        calPenStyleComboBox->setObjectName(QString::fromUtf8("calPenStyleComboBox"));
        sizePolicy1.setHeightForWidth(calPenStyleComboBox->sizePolicy().hasHeightForWidth());
        calPenStyleComboBox->setSizePolicy(sizePolicy1);

        gridLayout->addWidget(calPenStyleComboBox, 2, 1, 1, 1);

        calPenWidthSpinBox = new ScrSpinBox(tab_4);
        calPenWidthSpinBox->setObjectName(QString::fromUtf8("calPenWidthSpinBox"));

        gridLayout->addWidget(calPenWidthSpinBox, 3, 4, 1, 1);

        calPenWidthLabel = new QLabel(tab_4);
        calPenWidthLabel->setObjectName(QString::fromUtf8("calPenWidthLabel"));

        gridLayout->addWidget(calPenWidthLabel, 3, 3, 1, 1);

        horizontalSpacer_14 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(horizontalSpacer_14, 0, 5, 1, 1);


        formLayout_3->setLayout(0, QFormLayout::SpanningRole, gridLayout);

        tabWidget->addTab(tab_4, QString());
        tab_5 = new QWidget();
        tab_5->setObjectName(QString::fromUtf8("tab_5"));
        verticalLayout_10 = new QVBoxLayout(tab_5);
        verticalLayout_10->setObjectName(QString::fromUtf8("verticalLayout_10"));
        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName(QString::fromUtf8("horizontalLayout_11"));
        arcDisplay = new ArcWidget(tab_5);
        arcDisplay->setObjectName(QString::fromUtf8("arcDisplay"));
        arcDisplay->setMinimumSize(QSize(100, 100));

        horizontalLayout_11->addWidget(arcDisplay);

        horizontalSpacer_17 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_17);


        verticalLayout_10->addLayout(horizontalLayout_11);

        verticalSpacer_12 = new QSpacerItem(20, 10, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_10->addItem(verticalSpacer_12);

        tabWidget->addTab(tab_5, QString());
        tab_6 = new QWidget();
        tab_6->setObjectName(QString::fromUtf8("tab_6"));
        verticalLayout_11 = new QVBoxLayout(tab_6);
        verticalLayout_11->setObjectName(QString::fromUtf8("verticalLayout_11"));
        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName(QString::fromUtf8("horizontalLayout_12"));
        spiralDisplay = new SpiralWidget(tab_6);
        spiralDisplay->setObjectName(QString::fromUtf8("spiralDisplay"));
        sizePolicy2.setHeightForWidth(spiralDisplay->sizePolicy().hasHeightForWidth());
        spiralDisplay->setSizePolicy(sizePolicy2);
        spiralDisplay->setMinimumSize(QSize(100, 100));

        horizontalLayout_12->addWidget(spiralDisplay);

        horizontalSpacer_18 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer_18);


        verticalLayout_11->addLayout(horizontalLayout_12);

        tabWidget->addTab(tab_6, QString());

        verticalLayout->addWidget(tabWidget);

        QWidget::setTabOrder(tabWidget, textFrameFillColorComboBox);
        QWidget::setTabOrder(textFrameFillColorComboBox, textFrameFillShadingSpinBox);
        QWidget::setTabOrder(textFrameFillShadingSpinBox, textFrameLineColorComboBox);
        QWidget::setTabOrder(textFrameLineColorComboBox, textFrameLineShadingSpinBox);
        QWidget::setTabOrder(textFrameLineShadingSpinBox, textTabFillCharComboBox);
        QWidget::setTabOrder(textTabFillCharComboBox, textTabWidthSpinBox);
        QWidget::setTabOrder(textTabWidthSpinBox, textFontComboBox);
        QWidget::setTabOrder(textFontComboBox, textSizeSpinBox);
        QWidget::setTabOrder(textSizeSpinBox, textColorComboBox);
        QWidget::setTabOrder(textColorComboBox, textColorShadingSpinBox);
        QWidget::setTabOrder(textColorShadingSpinBox, textStrokeColorComboBox);
        QWidget::setTabOrder(textStrokeColorComboBox, textStrokeShadingSpinBox);
        QWidget::setTabOrder(textStrokeShadingSpinBox, firstLineOffsetComboBox);
        QWidget::setTabOrder(firstLineOffsetComboBox, textColumnsSpinBox);
        QWidget::setTabOrder(textColumnsSpinBox, textColumnGapSpinBox);
        QWidget::setTabOrder(textColumnGapSpinBox, topTextDistanceSpinBox);
        QWidget::setTabOrder(topTextDistanceSpinBox, leftTextDistanceSpinBox);
        QWidget::setTabOrder(leftTextDistanceSpinBox, bottomTextDistanceSpinBox);
        QWidget::setTabOrder(bottomTextDistanceSpinBox, rightTextDistanceSpinBox);
        QWidget::setTabOrder(rightTextDistanceSpinBox, imageFrameFillColorComboBox);
        QWidget::setTabOrder(imageFrameFillColorComboBox, imageFrameFillShadingSpinBox);
        QWidget::setTabOrder(imageFrameFillShadingSpinBox, imageFrameLineColorComboBox);
        QWidget::setTabOrder(imageFrameLineColorComboBox, imageFrameLineShadingSpinBox);
        QWidget::setTabOrder(imageFrameLineShadingSpinBox, imageUseEmbeddedClippingPathCheckBox);
        QWidget::setTabOrder(imageUseEmbeddedClippingPathCheckBox, imageFreeScalingRadioButton);
        QWidget::setTabOrder(imageFreeScalingRadioButton, imageHorizontalScalingSpinBox);
        QWidget::setTabOrder(imageHorizontalScalingSpinBox, scalingLockToolButton);
        QWidget::setTabOrder(scalingLockToolButton, imageVerticalScalingSpinBox);
        QWidget::setTabOrder(imageVerticalScalingSpinBox, imageFrameScalingRadioButton);
        QWidget::setTabOrder(imageFrameScalingRadioButton, imageKeepAspectRatioCheckBox);
        QWidget::setTabOrder(imageKeepAspectRatioCheckBox, onscreenResolutionFullRadioButton);
        QWidget::setTabOrder(onscreenResolutionFullRadioButton, onscreenResolutionNormalRadioButton);
        QWidget::setTabOrder(onscreenResolutionNormalRadioButton, onscreenResolutionLowRadioButton);
        QWidget::setTabOrder(onscreenResolutionLowRadioButton, shapeFrameFillColorComboBox);
        QWidget::setTabOrder(shapeFrameFillColorComboBox, shapeFrameFillShadingSpinBox);
        QWidget::setTabOrder(shapeFrameFillShadingSpinBox, shapeFrameLineColorComboBox);
        QWidget::setTabOrder(shapeFrameLineColorComboBox, shapeFrameLineShadingSpinBox);
        QWidget::setTabOrder(shapeFrameLineShadingSpinBox, shapeFrameLineStyleComboBox);
        QWidget::setTabOrder(shapeFrameLineStyleComboBox, shapeLineWidthSpinBox);
        QWidget::setTabOrder(shapeLineWidthSpinBox, lineFillColorComboBox);
        QWidget::setTabOrder(lineFillColorComboBox, lineFillColorShadingSpinBox);
        QWidget::setTabOrder(lineFillColorShadingSpinBox, lineStyleComboBox);
        QWidget::setTabOrder(lineStyleComboBox, lineWidthSpinBox);
        QWidget::setTabOrder(lineWidthSpinBox, lineStartArrowComboBox);
        QWidget::setTabOrder(lineStartArrowComboBox, lineEndArrowComboBox);
        QWidget::setTabOrder(lineEndArrowComboBox, calPenFillColorComboBox);
        QWidget::setTabOrder(calPenFillColorComboBox, calPenFillShadingSpinBox);
        QWidget::setTabOrder(calPenFillShadingSpinBox, calPenColorComboBox);
        QWidget::setTabOrder(calPenColorComboBox, calPenLineShadingSpinBox);
        QWidget::setTabOrder(calPenLineShadingSpinBox, calPenStyleComboBox);
        QWidget::setTabOrder(calPenStyleComboBox, calPenAngleSpinBox);
        QWidget::setTabOrder(calPenAngleSpinBox, calPenLineWidthSpinBox);
        QWidget::setTabOrder(calPenLineWidthSpinBox, calPenWidthSpinBox);
        QWidget::setTabOrder(calPenWidthSpinBox, scrollArea);
        QWidget::setTabOrder(scrollArea, scrollArea_2);

        retranslateUi(Prefs_ItemTools);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Prefs_ItemTools);
    } // setupUi

    void retranslateUi(QWidget *Prefs_ItemTools)
    {
        label->setText(QCoreApplication::translate("Prefs_ItemTools", "Item Tools", nullptr));
        label_19->setText(QCoreApplication::translate("Prefs_ItemTools", "Text", nullptr));
        label_43->setText(QString());
        label_42->setText(QString());
        TopLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Top:", nullptr));
        BottomLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Bottom:", nullptr));
        LefLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Left:", nullptr));
        ColumnsLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Columns:", nullptr));
        ColumnGapLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Column Gap:", nullptr));
        RightLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Right:", nullptr));
        textPreviewWidget->setText(QString());
        ColumnsAndTextDistances->setText(QCoreApplication::translate("Prefs_ItemTools", "Columns & Text Distances", nullptr));
        textColorStrokeLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Shading:", nullptr));
        textStrokeShadingSpinBox->setSuffix(QCoreApplication::translate("Prefs_ItemTools", " %", nullptr));
        textColorShadingLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Shading:", nullptr));
        StrokeLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Stroke:", nullptr));
        textColorShadingSpinBox->setSuffix(QCoreApplication::translate("Prefs_ItemTools", " %", nullptr));
        FontLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Font:", nullptr));
        textColorLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Color:", nullptr));
        SizeLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Size:", nullptr));
        textLineColorLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Line Color:", nullptr));
        TabFillCharacterLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Tab Fill Character:", nullptr));
        textFrameLineShadingSpinBox->setSuffix(QCoreApplication::translate("Prefs_ItemTools", " %", nullptr));
        textFillColorShadingLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Shading:", nullptr));
        TabWidthLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Tab Width:", nullptr));
        textLineColorShadingLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Shading:", nullptr));
        textFrameFillShadingSpinBox->setSuffix(QCoreApplication::translate("Prefs_ItemTools", " %", nullptr));
        textFillColorLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Fill Color:", nullptr));
        label_11->setText(QCoreApplication::translate("Prefs_ItemTools", "Preview", nullptr));
        label_2->setText(QCoreApplication::translate("Prefs_ItemTools", "First Line Offset", nullptr));
        label_3->setText(QCoreApplication::translate("Prefs_ItemTools", "Offset:", nullptr));
        firstLineOffsetComboBox->setItemText(0, QCoreApplication::translate("Prefs_ItemTools", "Maximum Ascent", nullptr));
        firstLineOffsetComboBox->setItemText(1, QCoreApplication::translate("Prefs_ItemTools", "Font Ascent", nullptr));
        firstLineOffsetComboBox->setItemText(2, QCoreApplication::translate("Prefs_ItemTools", "Line Spacing", nullptr));
        firstLineOffsetComboBox->setItemText(3, QCoreApplication::translate("Prefs_ItemTools", "Baseline Grid", nullptr));

        tabWidget->setTabText(tabWidget->indexOf(tabWidgetPage1), QCoreApplication::translate("Prefs_ItemTools", "Text", nullptr));
        imageUseEmbeddedClippingPathCheckBox->setText(QCoreApplication::translate("Prefs_ItemTools", "Use Embedded Clipping Path", nullptr));
        ScalingLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Scaling", nullptr));
        imageFreeScalingRadioButton->setText(QCoreApplication::translate("Prefs_ItemTools", "Free Scaling", nullptr));
        imageFrameScalingRadioButton->setText(QCoreApplication::translate("Prefs_ItemTools", "Scale Image to Frame Size", nullptr));
        imageKeepAspectRatioCheckBox->setText(QCoreApplication::translate("Prefs_ItemTools", "Keep Aspect Ratio", nullptr));
        OnScreenPreviewLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "On Screen Preview", nullptr));
        onscreenResolutionFullRadioButton->setText(QCoreApplication::translate("Prefs_ItemTools", "Full Resolution", nullptr));
        onscreenResolutionNormalRadioButton->setText(QCoreApplication::translate("Prefs_ItemTools", "Normal Resolution", nullptr));
        onscreenResolutionLowRadioButton->setText(QCoreApplication::translate("Prefs_ItemTools", "Low Resolution", nullptr));
        imageVerticalScaleLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Vertical Scaling:", nullptr));
        imageHorizontalScalingSpinBox->setSuffix(QCoreApplication::translate("Prefs_ItemTools", " %", nullptr));
        imageHorizontalScaleLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Horizontal Scaling:", nullptr));
        imageVerticalScalingSpinBox->setSuffix(QCoreApplication::translate("Prefs_ItemTools", " %", nullptr));
        imageFrameLineShadingSpinBox->setSuffix(QCoreApplication::translate("Prefs_ItemTools", " %", nullptr));
        imageFillColorLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Fill Color:", nullptr));
        imageLineColorShadeLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Shading:", nullptr));
        imageFrameFillShadingSpinBox->setSuffix(QCoreApplication::translate("Prefs_ItemTools", " %", nullptr));
        imageLineColorLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Line Color:", nullptr));
        imageFillShadingLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Shading:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("Prefs_ItemTools", "Images", nullptr));
        shapeFillColorShadingLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Shading:", nullptr));
        shapeLineColorLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Line Color:", nullptr));
        shapeFrameFillShadingSpinBox->setSuffix(QCoreApplication::translate("Prefs_ItemTools", " %", nullptr));
        shapeLineStyleLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Line Style:", nullptr));
        shapeLineWidthLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Line Width:", nullptr));
        shapeLineColorShadingLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Shading:", nullptr));
        shapeFrameLineShadingSpinBox->setSuffix(QCoreApplication::translate("Prefs_ItemTools", " %", nullptr));
        shapeFillColorLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Fill Color:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabWidgetPage2), QCoreApplication::translate("Prefs_ItemTools", "Shapes", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_2), QCoreApplication::translate("Prefs_ItemTools", "Regular Polygons", nullptr));
        lineColorShadingLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Shading:", nullptr));
        endArrowLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "End Arrow:", nullptr));
        lineWidthLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Line Width:", nullptr));
        lineStyleLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Line Style:", nullptr));
        startArrowLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Start Arrow:", nullptr));
        lineFillColorShadingSpinBox->setSuffix(QCoreApplication::translate("Prefs_ItemTools", " %", nullptr));
        lineColorLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Line Color:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_3), QCoreApplication::translate("Prefs_ItemTools", "Lines", nullptr));
        calPenFillShadingSpinBox->setSuffix(QCoreApplication::translate("Prefs_ItemTools", " %", nullptr));
        calLineStyleLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Line Style:", nullptr));
        calFillColorShadingLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Shading:", nullptr));
        calLineColorLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Line Color:", nullptr));
        calPenLineShadingSpinBox->setSuffix(QCoreApplication::translate("Prefs_ItemTools", " %", nullptr));
        calPenAngleLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Pen Angle:", nullptr));
        calLineWidthLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Line Width:", nullptr));
        calFillColorLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Fill Color:", nullptr));
        calLineColorShadingLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Shading:", nullptr));
        calPenWidthLabel->setText(QCoreApplication::translate("Prefs_ItemTools", "Pen Width:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), QCoreApplication::translate("Prefs_ItemTools", "Calligraphic Pen", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_5), QCoreApplication::translate("Prefs_ItemTools", "Arc", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_6), QCoreApplication::translate("Prefs_ItemTools", "Spiral", nullptr));
        (void)Prefs_ItemTools;
    } // retranslateUi

};

namespace Ui {
    class Prefs_ItemTools: public Ui_Prefs_ItemTools {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_ITEMTOOLSBASE_H
