/********************************************************************************
** Form generated from reading UI file 'gtfiledialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GTFILEDIALOG_H
#define UI_GTFILEDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include "ui/scfilewidget.h"

QT_BEGIN_NAMESPACE

class Ui_gtFileDialog
{
public:
    QGridLayout *gridLayout;
    ScFileWidget *fileWidget;
    QLabel *label;
    QComboBox *importerCombo;
    QCheckBox *textOnlyCheckBox;
    QLabel *label_2;
    QComboBox *encodingCombo;
    QCheckBox *prefixStylesCheckBox;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *gtFileDialog)
    {
        if (gtFileDialog->objectName().isEmpty())
            gtFileDialog->setObjectName(QString::fromUtf8("gtFileDialog"));
        gtFileDialog->resize(410, 320);
        gridLayout = new QGridLayout(gtFileDialog);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        fileWidget = new ScFileWidget(gtFileDialog);
        fileWidget->setObjectName(QString::fromUtf8("fileWidget"));

        gridLayout->addWidget(fileWidget, 0, 0, 1, 3);

        label = new QLabel(gtFileDialog);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 1, 0, 1, 1);

        importerCombo = new QComboBox(gtFileDialog);
        importerCombo->setObjectName(QString::fromUtf8("importerCombo"));

        gridLayout->addWidget(importerCombo, 1, 1, 1, 1);

        textOnlyCheckBox = new QCheckBox(gtFileDialog);
        textOnlyCheckBox->setObjectName(QString::fromUtf8("textOnlyCheckBox"));

        gridLayout->addWidget(textOnlyCheckBox, 1, 2, 1, 1);

        label_2 = new QLabel(gtFileDialog);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 2, 0, 1, 1);

        encodingCombo = new QComboBox(gtFileDialog);
        encodingCombo->setObjectName(QString::fromUtf8("encodingCombo"));

        gridLayout->addWidget(encodingCombo, 2, 1, 1, 1);

        prefixStylesCheckBox = new QCheckBox(gtFileDialog);
        prefixStylesCheckBox->setObjectName(QString::fromUtf8("prefixStylesCheckBox"));
        prefixStylesCheckBox->setChecked(true);

        gridLayout->addWidget(prefixStylesCheckBox, 2, 2, 1, 1);

        buttonBox = new QDialogButtonBox(gtFileDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(buttonBox, 3, 0, 1, 3);

#if QT_CONFIG(shortcut)
        label->setBuddy(importerCombo);
        label_2->setBuddy(encodingCombo);
#endif // QT_CONFIG(shortcut)

        retranslateUi(gtFileDialog);

        QMetaObject::connectSlotsByName(gtFileDialog);
    } // setupUi

    void retranslateUi(QDialog *gtFileDialog)
    {
        gtFileDialog->setWindowTitle(QCoreApplication::translate("gtFileDialog", "Open", nullptr));
        label->setText(QCoreApplication::translate("gtFileDialog", "&Importer:", nullptr));
        textOnlyCheckBox->setText(QCoreApplication::translate("gtFileDialog", "Import &Text Only", nullptr));
        label_2->setText(QCoreApplication::translate("gtFileDialog", "&Encoding:", nullptr));
        prefixStylesCheckBox->setText(QCoreApplication::translate("gtFileDialog", "Prefix Styles with Item Name", nullptr));
    } // retranslateUi

};

namespace Ui {
    class gtFileDialog: public Ui_gtFileDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_GTFILEDIALOG_H
