/********************************************************************************
** Form generated from reading UI file 'picsearchoptions.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PICSEARCHOPTIONS_H
#define UI_PICSEARCHOPTIONS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>

QT_BEGIN_NAMESPACE

class Ui_PicSearchOptions
{
public:
    QGridLayout *gridLayout;
    QLabel *textLabel1_2;
    QLabel *textLabel1;
    QLineEdit *directoryEdit;
    QLineEdit *fileEdit;
    QPushButton *changeDirButton;
    QHBoxLayout *hboxLayout;
    QLabel *searchLabel;
    QProgressBar *progressBar1;
    QSpacerItem *spacerItem;
    QPushButton *startButton;
    QHBoxLayout *hboxLayout1;
    QCheckBox *caseInsensitiveCheck;
    QCheckBox *recursiveSearch;
    QSpacerItem *spacerItem1;

    void setupUi(QDialog *PicSearchOptions)
    {
        if (PicSearchOptions->objectName().isEmpty())
            PicSearchOptions->setObjectName(QString::fromUtf8("PicSearchOptions"));
        PicSearchOptions->resize(484, 138);
        gridLayout = new QGridLayout(PicSearchOptions);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        textLabel1_2 = new QLabel(PicSearchOptions);
        textLabel1_2->setObjectName(QString::fromUtf8("textLabel1_2"));
        textLabel1_2->setWordWrap(false);

        gridLayout->addWidget(textLabel1_2, 0, 0, 1, 1);

        textLabel1 = new QLabel(PicSearchOptions);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 1, 0, 1, 1);

        directoryEdit = new QLineEdit(PicSearchOptions);
        directoryEdit->setObjectName(QString::fromUtf8("directoryEdit"));
        directoryEdit->setMinimumSize(QSize(300, 0));

        gridLayout->addWidget(directoryEdit, 1, 1, 1, 1);

        fileEdit = new QLineEdit(PicSearchOptions);
        fileEdit->setObjectName(QString::fromUtf8("fileEdit"));

        gridLayout->addWidget(fileEdit, 0, 1, 1, 2);

        changeDirButton = new QPushButton(PicSearchOptions);
        changeDirButton->setObjectName(QString::fromUtf8("changeDirButton"));

        gridLayout->addWidget(changeDirButton, 1, 2, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        searchLabel = new QLabel(PicSearchOptions);
        searchLabel->setObjectName(QString::fromUtf8("searchLabel"));
        searchLabel->setWordWrap(false);

        hboxLayout->addWidget(searchLabel);

        progressBar1 = new QProgressBar(PicSearchOptions);
        progressBar1->setObjectName(QString::fromUtf8("progressBar1"));
        progressBar1->setMinimumSize(QSize(0, 20));
        progressBar1->setMaximumSize(QSize(250, 22));
        progressBar1->setOrientation(Qt::Horizontal);

        hboxLayout->addWidget(progressBar1);

        spacerItem = new QSpacerItem(30, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        startButton = new QPushButton(PicSearchOptions);
        startButton->setObjectName(QString::fromUtf8("startButton"));

        hboxLayout->addWidget(startButton);


        gridLayout->addLayout(hboxLayout, 3, 0, 1, 3);

        hboxLayout1 = new QHBoxLayout();
        hboxLayout1->setObjectName(QString::fromUtf8("hboxLayout1"));
        caseInsensitiveCheck = new QCheckBox(PicSearchOptions);
        caseInsensitiveCheck->setObjectName(QString::fromUtf8("caseInsensitiveCheck"));

        hboxLayout1->addWidget(caseInsensitiveCheck);

        recursiveSearch = new QCheckBox(PicSearchOptions);
        recursiveSearch->setObjectName(QString::fromUtf8("recursiveSearch"));
        recursiveSearch->setChecked(true);

        hboxLayout1->addWidget(recursiveSearch);

        spacerItem1 = new QSpacerItem(81, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout1->addItem(spacerItem1);


        gridLayout->addLayout(hboxLayout1, 2, 0, 1, 3);

#if QT_CONFIG(shortcut)
        textLabel1_2->setBuddy(fileEdit);
        textLabel1->setBuddy(directoryEdit);
#endif // QT_CONFIG(shortcut)

        retranslateUi(PicSearchOptions);

        QMetaObject::connectSlotsByName(PicSearchOptions);
    } // setupUi

    void retranslateUi(QDialog *PicSearchOptions)
    {
        PicSearchOptions->setWindowTitle(QCoreApplication::translate("PicSearchOptions", "Search Images", nullptr));
        textLabel1_2->setText(QCoreApplication::translate("PicSearchOptions", "Search for:", nullptr));
        textLabel1->setText(QCoreApplication::translate("PicSearchOptions", "Start at:", nullptr));
        changeDirButton->setText(QCoreApplication::translate("PicSearchOptions", "Change...", nullptr));
        searchLabel->setText(QCoreApplication::translate("PicSearchOptions", "Searching", nullptr));
        startButton->setText(QCoreApplication::translate("PicSearchOptions", "Start Search", nullptr));
        caseInsensitiveCheck->setText(QCoreApplication::translate("PicSearchOptions", "Case insensitive search", nullptr));
        recursiveSearch->setText(QCoreApplication::translate("PicSearchOptions", "Search recursively", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PicSearchOptions: public Ui_PicSearchOptions {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PICSEARCHOPTIONS_H
