/********************************************************************************
** Form generated from reading UI file 'recoverdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RECOVERDIALOG_H
#define UI_RECOVERDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_recoverDialogBase
{
public:
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    QHBoxLayout *horizontalLayout;
    QTableWidget *filesList;
    QVBoxLayout *verticalLayout;
    QPushButton *buttonRecover;
    QPushButton *buttonRemove;
    QSpacerItem *verticalSpacer;
    QPushButton *buttonExit;

    void setupUi(QDialog *recoverDialogBase)
    {
        if (recoverDialogBase->objectName().isEmpty())
            recoverDialogBase->setObjectName(QString::fromUtf8("recoverDialogBase"));
        recoverDialogBase->setWindowModality(Qt::ApplicationModal);
        recoverDialogBase->resize(517, 436);
        verticalLayout_2 = new QVBoxLayout(recoverDialogBase);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(9, 9, 9, 9);
        label = new QLabel(recoverDialogBase);
        label->setObjectName(QString::fromUtf8("label"));
        label->setWordWrap(true);

        verticalLayout_2->addWidget(label);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        filesList = new QTableWidget(recoverDialogBase);
        if (filesList->columnCount() < 4)
            filesList->setColumnCount(4);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        filesList->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        filesList->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        filesList->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        filesList->setHorizontalHeaderItem(3, __qtablewidgetitem3);
        filesList->setObjectName(QString::fromUtf8("filesList"));
        filesList->setProperty("showDropIndicator", QVariant(false));
        filesList->setDragDropOverwriteMode(false);
        filesList->setShowGrid(false);
        filesList->setWordWrap(false);
        filesList->setCornerButtonEnabled(false);
        filesList->horizontalHeader()->setHighlightSections(false);
        filesList->horizontalHeader()->setStretchLastSection(true);
        filesList->verticalHeader()->setVisible(false);

        horizontalLayout->addWidget(filesList);

        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        buttonRecover = new QPushButton(recoverDialogBase);
        buttonRecover->setObjectName(QString::fromUtf8("buttonRecover"));
        buttonRecover->setEnabled(false);

        verticalLayout->addWidget(buttonRecover);

        buttonRemove = new QPushButton(recoverDialogBase);
        buttonRemove->setObjectName(QString::fromUtf8("buttonRemove"));
        buttonRemove->setEnabled(false);

        verticalLayout->addWidget(buttonRemove);

        verticalSpacer = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout->addItem(verticalSpacer);

        buttonExit = new QPushButton(recoverDialogBase);
        buttonExit->setObjectName(QString::fromUtf8("buttonExit"));

        verticalLayout->addWidget(buttonExit);


        horizontalLayout->addLayout(verticalLayout);


        verticalLayout_2->addLayout(horizontalLayout);


        retranslateUi(recoverDialogBase);
        QObject::connect(buttonExit, SIGNAL(clicked()), recoverDialogBase, SLOT(reject()));

        QMetaObject::connectSlotsByName(recoverDialogBase);
    } // setupUi

    void retranslateUi(QDialog *recoverDialogBase)
    {
        recoverDialogBase->setWindowTitle(QCoreApplication::translate("recoverDialogBase", "Crash Recovery", nullptr));
        label->setText(QCoreApplication::translate("recoverDialogBase", "<html><head/><body><p>It appears that Scribus crashed with the following files open.<br>Would you like to try to recover them?</p></body></html>", nullptr));
        QTableWidgetItem *___qtablewidgetitem = filesList->horizontalHeaderItem(1);
        ___qtablewidgetitem->setText(QCoreApplication::translate("recoverDialogBase", "Name", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = filesList->horizontalHeaderItem(2);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("recoverDialogBase", "Date", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = filesList->horizontalHeaderItem(3);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("recoverDialogBase", "Type", nullptr));
        buttonRecover->setText(QCoreApplication::translate("recoverDialogBase", "Recover", nullptr));
        buttonRemove->setText(QCoreApplication::translate("recoverDialogBase", "Remove", nullptr));
        buttonExit->setText(QCoreApplication::translate("recoverDialogBase", "Close", nullptr));
    } // retranslateUi

};

namespace Ui {
    class recoverDialogBase: public Ui_recoverDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RECOVERDIALOG_H
