/****************************************************************************
** Meta object code from reading C++ file 'tfdia.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../../../../scribus-1.6.6/scribus/plugins/gettext/textfilter/tfdia.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'tfdia.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_tfDia_t {
    QByteArrayData data[16];
    char stringdata0[156];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_tfDia_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_tfDia_t qt_meta_stringdata_tfDia = {
    {
QT_MOC_LITERAL(0, 0, 5), // "tfDia"
QT_MOC_LITERAL(1, 6, 14), // "adjustVBoxSize"
QT_MOC_LITERAL(2, 21, 0), // ""
QT_MOC_LITERAL(3, 22, 15), // "saveTextChanged"
QT_MOC_LITERAL(4, 38, 4), // "text"
QT_MOC_LITERAL(5, 43, 15), // "createFilterRow"
QT_MOC_LITERAL(6, 59, 9), // "tfFilter*"
QT_MOC_LITERAL(7, 69, 5), // "after"
QT_MOC_LITERAL(8, 75, 9), // "removeRow"
QT_MOC_LITERAL(9, 85, 3), // "tff"
QT_MOC_LITERAL(10, 89, 12), // "clearClicked"
QT_MOC_LITERAL(11, 102, 9), // "okClicked"
QT_MOC_LITERAL(12, 112, 13), // "cancelClicked"
QT_MOC_LITERAL(13, 126, 13), // "deleteClicked"
QT_MOC_LITERAL(14, 140, 10), // "loadFilter"
QT_MOC_LITERAL(15, 151, 4) // "name"

    },
    "tfDia\0adjustVBoxSize\0\0saveTextChanged\0"
    "text\0createFilterRow\0tfFilter*\0after\0"
    "removeRow\0tff\0clearClicked\0okClicked\0"
    "cancelClicked\0deleteClicked\0loadFilter\0"
    "name"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_tfDia[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       9,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   59,    2, 0x08 /* Private */,
       3,    1,   60,    2, 0x08 /* Private */,
       5,    1,   63,    2, 0x08 /* Private */,
       8,    1,   66,    2, 0x08 /* Private */,
      10,    0,   69,    2, 0x08 /* Private */,
      11,    0,   70,    2, 0x08 /* Private */,
      12,    0,   71,    2, 0x08 /* Private */,
      13,    0,   72,    2, 0x08 /* Private */,
      14,    1,   73,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, 0x80000000 | 6,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   15,

       0        // eod
};

void tfDia::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<tfDia *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->adjustVBoxSize(); break;
        case 1: _t->saveTextChanged((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 2: _t->createFilterRow((*reinterpret_cast< tfFilter*(*)>(_a[1]))); break;
        case 3: _t->removeRow((*reinterpret_cast< tfFilter*(*)>(_a[1]))); break;
        case 4: _t->clearClicked(); break;
        case 5: _t->okClicked(); break;
        case 6: _t->cancelClicked(); break;
        case 7: _t->deleteClicked(); break;
        case 8: _t->loadFilter((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 2:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< tfFilter* >(); break;
            }
            break;
        case 3:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< tfFilter* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject tfDia::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_tfDia.data,
    qt_meta_data_tfDia,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *tfDia::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *tfDia::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_tfDia.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int tfDia::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 9)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 9;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
