/********************************************************************************
** Form generated from reading UI file 'propertywidget_distancebase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYWIDGET_DISTANCEBASE_H
#define UI_PROPERTYWIDGET_DISTANCEBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QToolButton>
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_PropertyWidget_DistanceBase
{
public:
    QGridLayout *gridLayout;
    QLabel *verticalAlignLabel;
    QComboBox *verticalAlign;
    QLabel *columnsLabel;
    ScrSpinBox *columns;
    QComboBox *columnGapLabel;
    ScrSpinBox *columnGap;
    QLabel *topLabel;
    ScrSpinBox *topDistance;
    QLabel *bottomLabel;
    ScrSpinBox *bottomDistance;
    QLabel *leftLabel;
    ScrSpinBox *leftDistance;
    QLabel *rightLabel;
    ScrSpinBox *rightDistance;
    QToolButton *tabsButton;

    void setupUi(QFrame *PropertyWidget_DistanceBase)
    {
        if (PropertyWidget_DistanceBase->objectName().isEmpty())
            PropertyWidget_DistanceBase->setObjectName(QString::fromUtf8("PropertyWidget_DistanceBase"));
        PropertyWidget_DistanceBase->resize(223, 232);
        gridLayout = new QGridLayout(PropertyWidget_DistanceBase);
        gridLayout->setSpacing(3);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(3, 3, 3, 3);
        verticalAlignLabel = new QLabel(PropertyWidget_DistanceBase);
        verticalAlignLabel->setObjectName(QString::fromUtf8("verticalAlignLabel"));

        gridLayout->addWidget(verticalAlignLabel, 0, 0, 1, 1);

        verticalAlign = new QComboBox(PropertyWidget_DistanceBase);
        verticalAlign->addItem(QString());
        verticalAlign->addItem(QString());
        verticalAlign->addItem(QString());
        verticalAlign->setObjectName(QString::fromUtf8("verticalAlign"));

        gridLayout->addWidget(verticalAlign, 0, 1, 1, 1);

        columnsLabel = new QLabel(PropertyWidget_DistanceBase);
        columnsLabel->setObjectName(QString::fromUtf8("columnsLabel"));

        gridLayout->addWidget(columnsLabel, 1, 0, 1, 1);

        columns = new ScrSpinBox(PropertyWidget_DistanceBase);
        columns->setObjectName(QString::fromUtf8("columns"));
        columns->setMinimum(1.000000000000000);
        columns->setMaximum(3000.000000000000000);

        gridLayout->addWidget(columns, 1, 1, 1, 1);

        columnGapLabel = new QComboBox(PropertyWidget_DistanceBase);
        columnGapLabel->addItem(QString());
        columnGapLabel->addItem(QString());
        columnGapLabel->setObjectName(QString::fromUtf8("columnGapLabel"));

        gridLayout->addWidget(columnGapLabel, 2, 0, 1, 1);

        columnGap = new ScrSpinBox(PropertyWidget_DistanceBase);
        columnGap->setObjectName(QString::fromUtf8("columnGap"));

        gridLayout->addWidget(columnGap, 2, 1, 1, 1);

        topLabel = new QLabel(PropertyWidget_DistanceBase);
        topLabel->setObjectName(QString::fromUtf8("topLabel"));

        gridLayout->addWidget(topLabel, 3, 0, 1, 1);

        topDistance = new ScrSpinBox(PropertyWidget_DistanceBase);
        topDistance->setObjectName(QString::fromUtf8("topDistance"));

        gridLayout->addWidget(topDistance, 3, 1, 1, 1);

        bottomLabel = new QLabel(PropertyWidget_DistanceBase);
        bottomLabel->setObjectName(QString::fromUtf8("bottomLabel"));

        gridLayout->addWidget(bottomLabel, 4, 0, 1, 1);

        bottomDistance = new ScrSpinBox(PropertyWidget_DistanceBase);
        bottomDistance->setObjectName(QString::fromUtf8("bottomDistance"));

        gridLayout->addWidget(bottomDistance, 4, 1, 1, 1);

        leftLabel = new QLabel(PropertyWidget_DistanceBase);
        leftLabel->setObjectName(QString::fromUtf8("leftLabel"));

        gridLayout->addWidget(leftLabel, 5, 0, 1, 1);

        leftDistance = new ScrSpinBox(PropertyWidget_DistanceBase);
        leftDistance->setObjectName(QString::fromUtf8("leftDistance"));

        gridLayout->addWidget(leftDistance, 5, 1, 1, 1);

        rightLabel = new QLabel(PropertyWidget_DistanceBase);
        rightLabel->setObjectName(QString::fromUtf8("rightLabel"));

        gridLayout->addWidget(rightLabel, 6, 0, 1, 1);

        rightDistance = new ScrSpinBox(PropertyWidget_DistanceBase);
        rightDistance->setObjectName(QString::fromUtf8("rightDistance"));

        gridLayout->addWidget(rightDistance, 6, 1, 1, 1);

        tabsButton = new QToolButton(PropertyWidget_DistanceBase);
        tabsButton->setObjectName(QString::fromUtf8("tabsButton"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tabsButton->sizePolicy().hasHeightForWidth());
        tabsButton->setSizePolicy(sizePolicy);

        gridLayout->addWidget(tabsButton, 7, 0, 1, 2);


        retranslateUi(PropertyWidget_DistanceBase);

        QMetaObject::connectSlotsByName(PropertyWidget_DistanceBase);
    } // setupUi

    void retranslateUi(QFrame *PropertyWidget_DistanceBase)
    {
        verticalAlignLabel->setText(QCoreApplication::translate("PropertyWidget_DistanceBase", "Vertical Alignment:", nullptr));
        verticalAlign->setItemText(0, QCoreApplication::translate("PropertyWidget_DistanceBase", "Top", nullptr));
        verticalAlign->setItemText(1, QCoreApplication::translate("PropertyWidget_DistanceBase", "Middle", nullptr));
        verticalAlign->setItemText(2, QCoreApplication::translate("PropertyWidget_DistanceBase", "Bottom", nullptr));

        columnsLabel->setText(QCoreApplication::translate("PropertyWidget_DistanceBase", "Colu&mns:", nullptr));
#if QT_CONFIG(tooltip)
        columns->setToolTip(QCoreApplication::translate("PropertyWidget_DistanceBase", "Number of columns in text frame", nullptr));
#endif // QT_CONFIG(tooltip)
        columnGapLabel->setItemText(0, QCoreApplication::translate("PropertyWidget_DistanceBase", "Gap:", nullptr));
        columnGapLabel->setItemText(1, QCoreApplication::translate("PropertyWidget_DistanceBase", "Width:", nullptr));

#if QT_CONFIG(tooltip)
        columnGapLabel->setToolTip(QCoreApplication::translate("PropertyWidget_DistanceBase", "Switches between Gap or Column width", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        columnGap->setToolTip(QCoreApplication::translate("PropertyWidget_DistanceBase", "Distance between columns", nullptr));
#endif // QT_CONFIG(tooltip)
        topLabel->setText(QCoreApplication::translate("PropertyWidget_DistanceBase", "To&p:", nullptr));
#if QT_CONFIG(tooltip)
        topDistance->setToolTip(QCoreApplication::translate("PropertyWidget_DistanceBase", "Distance of text from top of frame", nullptr));
#endif // QT_CONFIG(tooltip)
        bottomLabel->setText(QCoreApplication::translate("PropertyWidget_DistanceBase", "&Bottom:", nullptr));
#if QT_CONFIG(tooltip)
        bottomDistance->setToolTip(QCoreApplication::translate("PropertyWidget_DistanceBase", "Distance of text from bottom of frame", nullptr));
#endif // QT_CONFIG(tooltip)
        leftLabel->setText(QCoreApplication::translate("PropertyWidget_DistanceBase", "&Left:", nullptr));
#if QT_CONFIG(tooltip)
        leftDistance->setToolTip(QCoreApplication::translate("PropertyWidget_DistanceBase", "Distance of text from left of frame", nullptr));
#endif // QT_CONFIG(tooltip)
        rightLabel->setText(QCoreApplication::translate("PropertyWidget_DistanceBase", "&Right:", nullptr));
#if QT_CONFIG(tooltip)
        rightDistance->setToolTip(QCoreApplication::translate("PropertyWidget_DistanceBase", "Distance of text from right of frame", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        tabsButton->setToolTip(QCoreApplication::translate("PropertyWidget_DistanceBase", "Edit tab settings of text frame...", nullptr));
#endif // QT_CONFIG(tooltip)
        tabsButton->setText(QCoreApplication::translate("PropertyWidget_DistanceBase", "T&abulators...", nullptr));
        (void)PropertyWidget_DistanceBase;
    } // retranslateUi

};

namespace Ui {
    class PropertyWidget_DistanceBase: public Ui_PropertyWidget_DistanceBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTYWIDGET_DISTANCEBASE_H
