/********************************************************************************
** Form generated from reading UI file 'propertywidget_flopbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYWIDGET_FLOPBASE_H
#define UI_PROPERTYWIDGET_FLOPBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QButtonGroup>
#include <QtWidgets/QFrame>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_PropertyWidget_FlopBase
{
public:
    QVBoxLayout *verticalLayout;
    QRadioButton *flopRealHeight;
    QRadioButton *flopFontAscent;
    QRadioButton *flopLineSpacing;
    QRadioButton *flopBaselineGrid;
    QButtonGroup *flopGroup;

    void setupUi(QFrame *PropertyWidget_FlopBase)
    {
        if (PropertyWidget_FlopBase->objectName().isEmpty())
            PropertyWidget_FlopBase->setObjectName(QString::fromUtf8("PropertyWidget_FlopBase"));
        PropertyWidget_FlopBase->resize(218, 107);
        verticalLayout = new QVBoxLayout(PropertyWidget_FlopBase);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        flopRealHeight = new QRadioButton(PropertyWidget_FlopBase);
        flopGroup = new QButtonGroup(PropertyWidget_FlopBase);
        flopGroup->setObjectName(QString::fromUtf8("flopGroup"));
        flopGroup->addButton(flopRealHeight);
        flopRealHeight->setObjectName(QString::fromUtf8("flopRealHeight"));

        verticalLayout->addWidget(flopRealHeight);

        flopFontAscent = new QRadioButton(PropertyWidget_FlopBase);
        flopGroup->addButton(flopFontAscent);
        flopFontAscent->setObjectName(QString::fromUtf8("flopFontAscent"));

        verticalLayout->addWidget(flopFontAscent);

        flopLineSpacing = new QRadioButton(PropertyWidget_FlopBase);
        flopGroup->addButton(flopLineSpacing);
        flopLineSpacing->setObjectName(QString::fromUtf8("flopLineSpacing"));

        verticalLayout->addWidget(flopLineSpacing);

        flopBaselineGrid = new QRadioButton(PropertyWidget_FlopBase);
        flopGroup->addButton(flopBaselineGrid);
        flopBaselineGrid->setObjectName(QString::fromUtf8("flopBaselineGrid"));

        verticalLayout->addWidget(flopBaselineGrid);


        retranslateUi(PropertyWidget_FlopBase);

        QMetaObject::connectSlotsByName(PropertyWidget_FlopBase);
    } // setupUi

    void retranslateUi(QFrame *PropertyWidget_FlopBase)
    {
#if QT_CONFIG(tooltip)
        flopRealHeight->setToolTip(QCoreApplication::translate("PropertyWidget_FlopBase", "Set the height of the first line of the text frame to use the tallest height of the included characters", nullptr));
#endif // QT_CONFIG(tooltip)
        flopRealHeight->setText(QCoreApplication::translate("PropertyWidget_FlopBase", "Maximum Ascent", nullptr));
#if QT_CONFIG(tooltip)
        flopFontAscent->setToolTip(QCoreApplication::translate("PropertyWidget_FlopBase", "Set the height of the first line of text frame to use the full ascent of the font(s) in use", nullptr));
#endif // QT_CONFIG(tooltip)
        flopFontAscent->setText(QCoreApplication::translate("PropertyWidget_FlopBase", "Font Ascent", nullptr));
#if QT_CONFIG(tooltip)
        flopLineSpacing->setToolTip(QCoreApplication::translate("PropertyWidget_FlopBase", "Set the height of the first line of the text frame to the specified line height", nullptr));
#endif // QT_CONFIG(tooltip)
        flopLineSpacing->setText(QCoreApplication::translate("PropertyWidget_FlopBase", "Line Spacing", nullptr));
#if QT_CONFIG(tooltip)
        flopBaselineGrid->setToolTip(QCoreApplication::translate("PropertyWidget_FlopBase", "Set the base line of the first line of the text frame to the base line grid", nullptr));
#endif // QT_CONFIG(tooltip)
        flopBaselineGrid->setText(QCoreApplication::translate("PropertyWidget_FlopBase", "Baseline Grid", nullptr));
        (void)PropertyWidget_FlopBase;
    } // retranslateUi

};

namespace Ui {
    class PropertyWidget_FlopBase: public Ui_PropertyWidget_FlopBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTYWIDGET_FLOPBASE_H
