/*
General prefs panel
*/

/********************************************************************************
** Form generated from reading UI file 'prefs_printerbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_PRINTERBASE_H
#define UI_PREFS_PRINTERBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/newmarginwidget.h>
#include "ui/printlanguagecombo.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_Prefs_Printer
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *titleLabel;
    QFrame *line;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents_2;
    QVBoxLayout *verticalLayout_2;
    QLabel *label;
    QFrame *line_2;
    QComboBox *destinationComboBox;
    QCheckBox *useAltPrinterCmdCheckBox;
    QHBoxLayout *horizontalLayout;
    QLabel *label_2;
    QLineEdit *altPrinterCmdLineEdit;
    QSpacerItem *spacerItem;
    QLabel *label_4;
    QFrame *line_3;
    QHBoxLayout *horizontalLayout_2;
    QLabel *label_3;
    QComboBox *outputComboBox;
    QSpacerItem *horizontalSpacer_4;
    QCheckBox *includePDFAnnotationsCheckBox;
    QSpacerItem *verticalSpacer_3;
    QLabel *label_5;
    QFrame *line_4;
    QCheckBox *pageMirrorHorizontallyCheckBox;
    QCheckBox *pageMirrorVerticallyCheckBox;
    QCheckBox *setMediaSizeCheckBox;
    QCheckBox *clipToPrinterMarginsCheckBox;
    QSpacerItem *verticalSpacer;
    QLabel *label_6;
    QFrame *line_5;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_8;
    QComboBox *colorOutputModeComboBox;
    QSpacerItem *horizontalSpacer_2;
    QCheckBox *applyUnderColorRemovalCheckBox;
    QCheckBox *convertSpotsToProcessCheckBox;
    QSpacerItem *verticalSpacer_2;
    QLabel *label_7;
    QFrame *line_6;
    QHBoxLayout *horizontalLayout_3;
    PrintLanguageCombo *printLanguageComboBox;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer_4;
    QLabel *label_9;
    QFrame *line_7;
    QGridLayout *gridLayout;
    QCheckBox *cropMarksCheckBox;
    QCheckBox *bleedMarksCheckBox;
    QCheckBox *registrationMarksCheckBox;
    QCheckBox *colorBarsCheckBox;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_10;
    ScrSpinBox *markOffsetSpinBox;
    QHBoxLayout *horizontalLayout_6;
    QLabel *label_12;
    ScrSpinBox *markLengthSpinBox;
    QSpacerItem *verticalSpacer_7;
    QLabel *label_11;
    QFrame *line_8;
    QHBoxLayout *horizontalLayout_7;
    NewMarginWidget *bleedsWidget;
    QSpacerItem *horizontalSpacer_7;
    QSpacerItem *verticalSpacer_5;

    void setupUi(QWidget *Prefs_Printer)
    {
        if (Prefs_Printer->objectName().isEmpty())
            Prefs_Printer->setObjectName(QString::fromUtf8("Prefs_Printer"));
        Prefs_Printer->resize(759, 693);
        verticalLayout = new QVBoxLayout(Prefs_Printer);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleLabel = new QLabel(Prefs_Printer);
        titleLabel->setObjectName(QString::fromUtf8("titleLabel"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        titleLabel->setFont(font);

        verticalLayout->addWidget(titleLabel);

        line = new QFrame(Prefs_Printer);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        scrollArea = new QScrollArea(Prefs_Printer);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents_2 = new QWidget();
        scrollAreaWidgetContents_2->setObjectName(QString::fromUtf8("scrollAreaWidgetContents_2"));
        scrollAreaWidgetContents_2->setGeometry(QRect(0, 0, 722, 824));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents_2);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        label = new QLabel(scrollAreaWidgetContents_2);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
        label->setFont(font1);

        verticalLayout_2->addWidget(label);

        line_2 = new QFrame(scrollAreaWidgetContents_2);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line_2);

        destinationComboBox = new QComboBox(scrollAreaWidgetContents_2);
        destinationComboBox->setObjectName(QString::fromUtf8("destinationComboBox"));

        verticalLayout_2->addWidget(destinationComboBox);

        useAltPrinterCmdCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        useAltPrinterCmdCheckBox->setObjectName(QString::fromUtf8("useAltPrinterCmdCheckBox"));

        verticalLayout_2->addWidget(useAltPrinterCmdCheckBox);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label_2 = new QLabel(scrollAreaWidgetContents_2);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        horizontalLayout->addWidget(label_2);

        altPrinterCmdLineEdit = new QLineEdit(scrollAreaWidgetContents_2);
        altPrinterCmdLineEdit->setObjectName(QString::fromUtf8("altPrinterCmdLineEdit"));

        horizontalLayout->addWidget(altPrinterCmdLineEdit);


        verticalLayout_2->addLayout(horizontalLayout);

        spacerItem = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_2->addItem(spacerItem);

        label_4 = new QLabel(scrollAreaWidgetContents_2);
        label_4->setObjectName(QString::fromUtf8("label_4"));
        label_4->setFont(font1);

        verticalLayout_2->addWidget(label_4);

        line_3 = new QFrame(scrollAreaWidgetContents_2);
        line_3->setObjectName(QString::fromUtf8("line_3"));
        line_3->setFrameShape(QFrame::HLine);
        line_3->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line_3);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        label_3 = new QLabel(scrollAreaWidgetContents_2);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        horizontalLayout_2->addWidget(label_3);

        outputComboBox = new QComboBox(scrollAreaWidgetContents_2);
        outputComboBox->addItem(QString());
        outputComboBox->addItem(QString());
        outputComboBox->setObjectName(QString::fromUtf8("outputComboBox"));

        horizontalLayout_2->addWidget(outputComboBox);

        horizontalSpacer_4 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_4);


        verticalLayout_2->addLayout(horizontalLayout_2);

        includePDFAnnotationsCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        includePDFAnnotationsCheckBox->setObjectName(QString::fromUtf8("includePDFAnnotationsCheckBox"));

        verticalLayout_2->addWidget(includePDFAnnotationsCheckBox);

        verticalSpacer_3 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_2->addItem(verticalSpacer_3);

        label_5 = new QLabel(scrollAreaWidgetContents_2);
        label_5->setObjectName(QString::fromUtf8("label_5"));
        label_5->setFont(font1);

        verticalLayout_2->addWidget(label_5);

        line_4 = new QFrame(scrollAreaWidgetContents_2);
        line_4->setObjectName(QString::fromUtf8("line_4"));
        line_4->setFrameShape(QFrame::HLine);
        line_4->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line_4);

        pageMirrorHorizontallyCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        pageMirrorHorizontallyCheckBox->setObjectName(QString::fromUtf8("pageMirrorHorizontallyCheckBox"));

        verticalLayout_2->addWidget(pageMirrorHorizontallyCheckBox);

        pageMirrorVerticallyCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        pageMirrorVerticallyCheckBox->setObjectName(QString::fromUtf8("pageMirrorVerticallyCheckBox"));

        verticalLayout_2->addWidget(pageMirrorVerticallyCheckBox);

        setMediaSizeCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        setMediaSizeCheckBox->setObjectName(QString::fromUtf8("setMediaSizeCheckBox"));

        verticalLayout_2->addWidget(setMediaSizeCheckBox);

        clipToPrinterMarginsCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        clipToPrinterMarginsCheckBox->setObjectName(QString::fromUtf8("clipToPrinterMarginsCheckBox"));

        verticalLayout_2->addWidget(clipToPrinterMarginsCheckBox);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_2->addItem(verticalSpacer);

        label_6 = new QLabel(scrollAreaWidgetContents_2);
        label_6->setObjectName(QString::fromUtf8("label_6"));
        label_6->setFont(font1);

        verticalLayout_2->addWidget(label_6);

        line_5 = new QFrame(scrollAreaWidgetContents_2);
        line_5->setObjectName(QString::fromUtf8("line_5"));
        line_5->setFrameShape(QFrame::HLine);
        line_5->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line_5);

        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        label_8 = new QLabel(scrollAreaWidgetContents_2);
        label_8->setObjectName(QString::fromUtf8("label_8"));

        horizontalLayout_4->addWidget(label_8);

        colorOutputModeComboBox = new QComboBox(scrollAreaWidgetContents_2);
        colorOutputModeComboBox->addItem(QString());
        colorOutputModeComboBox->addItem(QString());
        colorOutputModeComboBox->setObjectName(QString::fromUtf8("colorOutputModeComboBox"));

        horizontalLayout_4->addWidget(colorOutputModeComboBox);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_2);


        verticalLayout_2->addLayout(horizontalLayout_4);

        applyUnderColorRemovalCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        applyUnderColorRemovalCheckBox->setObjectName(QString::fromUtf8("applyUnderColorRemovalCheckBox"));

        verticalLayout_2->addWidget(applyUnderColorRemovalCheckBox);

        convertSpotsToProcessCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        convertSpotsToProcessCheckBox->setObjectName(QString::fromUtf8("convertSpotsToProcessCheckBox"));

        verticalLayout_2->addWidget(convertSpotsToProcessCheckBox);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_2->addItem(verticalSpacer_2);

        label_7 = new QLabel(scrollAreaWidgetContents_2);
        label_7->setObjectName(QString::fromUtf8("label_7"));
        label_7->setFont(font1);

        verticalLayout_2->addWidget(label_7);

        line_6 = new QFrame(scrollAreaWidgetContents_2);
        line_6->setObjectName(QString::fromUtf8("line_6"));
        line_6->setFrameShape(QFrame::HLine);
        line_6->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line_6);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        printLanguageComboBox = new PrintLanguageCombo(scrollAreaWidgetContents_2);
        printLanguageComboBox->setObjectName(QString::fromUtf8("printLanguageComboBox"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(printLanguageComboBox->sizePolicy().hasHeightForWidth());
        printLanguageComboBox->setSizePolicy(sizePolicy);

        horizontalLayout_3->addWidget(printLanguageComboBox);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_3->addItem(horizontalSpacer);


        verticalLayout_2->addLayout(horizontalLayout_3);

        verticalSpacer_4 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_2->addItem(verticalSpacer_4);

        label_9 = new QLabel(scrollAreaWidgetContents_2);
        label_9->setObjectName(QString::fromUtf8("label_9"));
        label_9->setFont(font1);

        verticalLayout_2->addWidget(label_9);

        line_7 = new QFrame(scrollAreaWidgetContents_2);
        line_7->setObjectName(QString::fromUtf8("line_7"));
        line_7->setFrameShape(QFrame::HLine);
        line_7->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line_7);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        cropMarksCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        cropMarksCheckBox->setObjectName(QString::fromUtf8("cropMarksCheckBox"));

        gridLayout->addWidget(cropMarksCheckBox, 0, 0, 1, 1);

        bleedMarksCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        bleedMarksCheckBox->setObjectName(QString::fromUtf8("bleedMarksCheckBox"));

        gridLayout->addWidget(bleedMarksCheckBox, 1, 0, 1, 1);

        registrationMarksCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        registrationMarksCheckBox->setObjectName(QString::fromUtf8("registrationMarksCheckBox"));

        gridLayout->addWidget(registrationMarksCheckBox, 2, 0, 1, 1);

        colorBarsCheckBox = new QCheckBox(scrollAreaWidgetContents_2);
        colorBarsCheckBox->setObjectName(QString::fromUtf8("colorBarsCheckBox"));

        gridLayout->addWidget(colorBarsCheckBox, 3, 0, 1, 1);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        label_10 = new QLabel(scrollAreaWidgetContents_2);
        label_10->setObjectName(QString::fromUtf8("label_10"));

        horizontalLayout_5->addWidget(label_10);

        markOffsetSpinBox = new ScrSpinBox(scrollAreaWidgetContents_2);
        markOffsetSpinBox->setObjectName(QString::fromUtf8("markOffsetSpinBox"));

        horizontalLayout_5->addWidget(markOffsetSpinBox);


        gridLayout->addLayout(horizontalLayout_5, 1, 1, 1, 1);

        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        label_12 = new QLabel(scrollAreaWidgetContents_2);
        label_12->setObjectName(QString::fromUtf8("label_12"));

        horizontalLayout_6->addWidget(label_12);

        markLengthSpinBox = new ScrSpinBox(scrollAreaWidgetContents_2);
        markLengthSpinBox->setObjectName(QString::fromUtf8("markLengthSpinBox"));
        markLengthSpinBox->setMinimum(1.000000000000000);
        markLengthSpinBox->setMaximum(100.000000000000000);
        markLengthSpinBox->setValue(20.000000000000000);

        horizontalLayout_6->addWidget(markLengthSpinBox);


        gridLayout->addLayout(horizontalLayout_6, 0, 1, 1, 1);


        verticalLayout_2->addLayout(gridLayout);

        verticalSpacer_7 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_2->addItem(verticalSpacer_7);

        label_11 = new QLabel(scrollAreaWidgetContents_2);
        label_11->setObjectName(QString::fromUtf8("label_11"));
        label_11->setFont(font1);

        verticalLayout_2->addWidget(label_11);

        line_8 = new QFrame(scrollAreaWidgetContents_2);
        line_8->setObjectName(QString::fromUtf8("line_8"));
        line_8->setFrameShape(QFrame::HLine);
        line_8->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line_8);

        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        bleedsWidget = new NewMarginWidget(scrollAreaWidgetContents_2);
        bleedsWidget->setObjectName(QString::fromUtf8("bleedsWidget"));

        horizontalLayout_7->addWidget(bleedsWidget);

        horizontalSpacer_7 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_7->addItem(horizontalSpacer_7);


        verticalLayout_2->addLayout(horizontalLayout_7);

        verticalSpacer_5 = new QSpacerItem(20, 40, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_5);

        scrollArea->setWidget(scrollAreaWidgetContents_2);

        verticalLayout->addWidget(scrollArea);

        QWidget::setTabOrder(scrollArea, destinationComboBox);
        QWidget::setTabOrder(destinationComboBox, useAltPrinterCmdCheckBox);
        QWidget::setTabOrder(useAltPrinterCmdCheckBox, altPrinterCmdLineEdit);
        QWidget::setTabOrder(altPrinterCmdLineEdit, outputComboBox);
        QWidget::setTabOrder(outputComboBox, includePDFAnnotationsCheckBox);
        QWidget::setTabOrder(includePDFAnnotationsCheckBox, pageMirrorHorizontallyCheckBox);
        QWidget::setTabOrder(pageMirrorHorizontallyCheckBox, pageMirrorVerticallyCheckBox);
        QWidget::setTabOrder(pageMirrorVerticallyCheckBox, setMediaSizeCheckBox);
        QWidget::setTabOrder(setMediaSizeCheckBox, clipToPrinterMarginsCheckBox);
        QWidget::setTabOrder(clipToPrinterMarginsCheckBox, applyUnderColorRemovalCheckBox);
        QWidget::setTabOrder(applyUnderColorRemovalCheckBox, convertSpotsToProcessCheckBox);
        QWidget::setTabOrder(convertSpotsToProcessCheckBox, printLanguageComboBox);
        QWidget::setTabOrder(printLanguageComboBox, colorOutputModeComboBox);
        QWidget::setTabOrder(colorOutputModeComboBox, cropMarksCheckBox);
        QWidget::setTabOrder(cropMarksCheckBox, bleedMarksCheckBox);
        QWidget::setTabOrder(bleedMarksCheckBox, registrationMarksCheckBox);
        QWidget::setTabOrder(registrationMarksCheckBox, colorBarsCheckBox);
        QWidget::setTabOrder(colorBarsCheckBox, markLengthSpinBox);
        QWidget::setTabOrder(markLengthSpinBox, markOffsetSpinBox);

        retranslateUi(Prefs_Printer);
        QObject::connect(cropMarksCheckBox, SIGNAL(clicked(bool)), markLengthSpinBox, SLOT(setEnabled(bool)));

        QMetaObject::connectSlotsByName(Prefs_Printer);
    } // setupUi

    void retranslateUi(QWidget *Prefs_Printer)
    {
        titleLabel->setText(QCoreApplication::translate("Prefs_Printer", "Printing", nullptr));
        label->setText(QCoreApplication::translate("Prefs_Printer", "Destination", nullptr));
        useAltPrinterCmdCheckBox->setText(QCoreApplication::translate("Prefs_Printer", "Alternative Printer Command", nullptr));
        label_2->setText(QCoreApplication::translate("Prefs_Printer", "Command:", nullptr));
        label_4->setText(QCoreApplication::translate("Prefs_Printer", "General Options", nullptr));
        label_3->setText(QCoreApplication::translate("Prefs_Printer", "Output:", nullptr));
        outputComboBox->setItemText(0, QCoreApplication::translate("Prefs_Printer", "Normal", nullptr));
        outputComboBox->setItemText(1, QCoreApplication::translate("Prefs_Printer", "Separations", nullptr));

        includePDFAnnotationsCheckBox->setText(QCoreApplication::translate("Prefs_Printer", "Include PDF Annotations and Links", nullptr));
        label_5->setText(QCoreApplication::translate("Prefs_Printer", "Page", nullptr));
        pageMirrorHorizontallyCheckBox->setText(QCoreApplication::translate("Prefs_Printer", "Mirror Page(s) Horizontally", nullptr));
        pageMirrorVerticallyCheckBox->setText(QCoreApplication::translate("Prefs_Printer", "Mirror Page(s) Vertically", nullptr));
        setMediaSizeCheckBox->setText(QCoreApplication::translate("Prefs_Printer", "Set Media Size", nullptr));
        clipToPrinterMarginsCheckBox->setText(QCoreApplication::translate("Prefs_Printer", "Clip to Printer Margins", nullptr));
        label_6->setText(QCoreApplication::translate("Prefs_Printer", "Color", nullptr));
        label_8->setText(QCoreApplication::translate("Prefs_Printer", "Print In:", nullptr));
        colorOutputModeComboBox->setItemText(0, QCoreApplication::translate("Prefs_Printer", "Color (If Available)", nullptr));
        colorOutputModeComboBox->setItemText(1, QCoreApplication::translate("Prefs_Printer", "Grayscale", nullptr));

        applyUnderColorRemovalCheckBox->setText(QCoreApplication::translate("Prefs_Printer", "Apply Under Color Removal", nullptr));
        convertSpotsToProcessCheckBox->setText(QCoreApplication::translate("Prefs_Printer", "Convert Spot Colors to Process Colors", nullptr));
        label_7->setText(QCoreApplication::translate("Prefs_Printer", "Printing Language", nullptr));
        label_9->setText(QCoreApplication::translate("Prefs_Printer", "Marks and Bleeds", nullptr));
        cropMarksCheckBox->setText(QCoreApplication::translate("Prefs_Printer", "Crop Marks", nullptr));
        bleedMarksCheckBox->setText(QCoreApplication::translate("Prefs_Printer", "Bleed Marks", nullptr));
        registrationMarksCheckBox->setText(QCoreApplication::translate("Prefs_Printer", "Registration Marks", nullptr));
        colorBarsCheckBox->setText(QCoreApplication::translate("Prefs_Printer", "Color Bars", nullptr));
        label_10->setText(QCoreApplication::translate("Prefs_Printer", "Offset:", nullptr));
        label_12->setText(QCoreApplication::translate("Prefs_Printer", "Length:", nullptr));
        label_11->setText(QCoreApplication::translate("Prefs_Printer", "Bleed Settings", nullptr));
        (void)Prefs_Printer;
    } // retranslateUi

};

namespace Ui {
    class Prefs_Printer: public Ui_Prefs_Printer {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_PRINTERBASE_H
