/****************************************************************************
** Meta object code from reading C++ file 'pagepalette_masterpages.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/pagepalette_masterpages.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pagepalette_masterpages.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PagePalette_MasterPages_t {
    QByteArrayData data[16];
    char stringdata0[209];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PagePalette_MasterPages_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PagePalette_MasterPages_t qt_meta_stringdata_PagePalette_MasterPages = {
    {
QT_MOC_LITERAL(0, 0, 23), // "PagePalette_MasterPages"
QT_MOC_LITERAL(1, 24, 10), // "removePage"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 8), // "finished"
QT_MOC_LITERAL(4, 45, 13), // "iconSetChange"
QT_MOC_LITERAL(5, 59, 14), // "languageChange"
QT_MOC_LITERAL(6, 74, 11), // "styleChange"
QT_MOC_LITERAL(7, 86, 16), // "selectMasterPage"
QT_MOC_LITERAL(8, 103, 4), // "name"
QT_MOC_LITERAL(9, 108, 19), // "duplicateMasterPage"
QT_MOC_LITERAL(10, 128, 16), // "deleteMasterPage"
QT_MOC_LITERAL(11, 145, 13), // "newMasterPage"
QT_MOC_LITERAL(12, 159, 16), // "renameMasterPage"
QT_MOC_LITERAL(13, 176, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(14, 193, 4), // "item"
QT_MOC_LITERAL(15, 198, 10) // "importPage"

    },
    "PagePalette_MasterPages\0removePage\0\0"
    "finished\0iconSetChange\0languageChange\0"
    "styleChange\0selectMasterPage\0name\0"
    "duplicateMasterPage\0deleteMasterPage\0"
    "newMasterPage\0renameMasterPage\0"
    "QListWidgetItem*\0item\0importPage"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PagePalette_MasterPages[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   74,    2, 0x06 /* Public */,
       3,    0,   77,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   78,    2, 0x0a /* Public */,
       5,    0,   79,    2, 0x0a /* Public */,
       6,    0,   80,    2, 0x0a /* Public */,
       7,    1,   81,    2, 0x0a /* Public */,
       9,    0,   84,    2, 0x08 /* Private */,
      10,    0,   85,    2, 0x08 /* Private */,
      11,    0,   86,    2, 0x08 /* Private */,
      12,    1,   87,    2, 0x08 /* Private */,
      15,    0,   90,    2, 0x08 /* Private */,
       7,    1,   91,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    8,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13,   14,

       0        // eod
};

void PagePalette_MasterPages::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PagePalette_MasterPages *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->removePage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->finished(); break;
        case 2: _t->iconSetChange(); break;
        case 3: _t->languageChange(); break;
        case 4: _t->styleChange(); break;
        case 5: _t->selectMasterPage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 6: _t->duplicateMasterPage(); break;
        case 7: _t->deleteMasterPage(); break;
        case 8: _t->newMasterPage(); break;
        case 9: _t->renameMasterPage((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 10: _t->importPage(); break;
        case 11: _t->selectMasterPage((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PagePalette_MasterPages::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PagePalette_MasterPages::removePage)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PagePalette_MasterPages::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PagePalette_MasterPages::finished)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PagePalette_MasterPages::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_PagePalette_MasterPages.data,
    qt_meta_data_PagePalette_MasterPages,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PagePalette_MasterPages::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PagePalette_MasterPages::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PagePalette_MasterPages.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PagePalette_MasterPagesBase"))
        return static_cast< Ui::PagePalette_MasterPagesBase*>(this);
    return QWidget::qt_metacast(_clname);
}

int PagePalette_MasterPages::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}

// SIGNAL 0
void PagePalette_MasterPages::removePage(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PagePalette_MasterPages::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
