/********************************************************************************
** Form generated from reading UI file 'fontpreview.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_FONTPREVIEW_H
#define UI_FONTPREVIEW_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include "ui/fontlistview.h"

QT_BEGIN_NAMESPACE

class Ui_FontPreview
{
public:
    QGridLayout *gridLayout;
    QLabel *searchLabel;
    QLineEdit *searchEdit;
    QCheckBox *extendedCheckBox;
    FontListView *fontList;
    QLabel *fontPreview;
    QLabel *sizeLabel;
    QSpinBox *sizeSpin;
    QLabel *displayLabel;
    QLineEdit *displayEdit;
    QPushButton *displayButton;
    QPushButton *resetDisplayButton;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *cancelButton;

    void setupUi(QDialog *FontPreview)
    {
        if (FontPreview->objectName().isEmpty())
            FontPreview->setObjectName(QString::fromUtf8("FontPreview"));
        FontPreview->resize(604, 463);
        gridLayout = new QGridLayout(FontPreview);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        searchLabel = new QLabel(FontPreview);
        searchLabel->setObjectName(QString::fromUtf8("searchLabel"));
        searchLabel->setWordWrap(false);

        gridLayout->addWidget(searchLabel, 0, 0, 1, 2);

        searchEdit = new QLineEdit(FontPreview);
        searchEdit->setObjectName(QString::fromUtf8("searchEdit"));

        gridLayout->addWidget(searchEdit, 0, 2, 1, 5);

        extendedCheckBox = new QCheckBox(FontPreview);
        extendedCheckBox->setObjectName(QString::fromUtf8("extendedCheckBox"));

        gridLayout->addWidget(extendedCheckBox, 1, 0, 1, 7);

        fontList = new FontListView(FontPreview);
        fontList->setObjectName(QString::fromUtf8("fontList"));
        fontList->setAlternatingRowColors(true);
        fontList->setSelectionMode(QAbstractItemView::SingleSelection);
        fontList->setSelectionBehavior(QAbstractItemView::SelectRows);
        fontList->setSortingEnabled(true);

        gridLayout->addWidget(fontList, 2, 0, 1, 7);

        fontPreview = new QLabel(FontPreview);
        fontPreview->setObjectName(QString::fromUtf8("fontPreview"));
        fontPreview->setMinimumSize(QSize(500, 100));
        fontPreview->setMaximumSize(QSize(16777215, 100));
        fontPreview->setFrameShape(QFrame::Box);
        fontPreview->setWordWrap(false);

        gridLayout->addWidget(fontPreview, 3, 0, 1, 7);

        sizeLabel = new QLabel(FontPreview);
        sizeLabel->setObjectName(QString::fromUtf8("sizeLabel"));
        sizeLabel->setWordWrap(false);

        gridLayout->addWidget(sizeLabel, 4, 0, 1, 1);

        sizeSpin = new QSpinBox(FontPreview);
        sizeSpin->setObjectName(QString::fromUtf8("sizeSpin"));
        sizeSpin->setMinimum(10);
        sizeSpin->setMaximum(72);

        gridLayout->addWidget(sizeSpin, 4, 1, 1, 2);

        displayLabel = new QLabel(FontPreview);
        displayLabel->setObjectName(QString::fromUtf8("displayLabel"));
        displayLabel->setWordWrap(false);

        gridLayout->addWidget(displayLabel, 4, 3, 1, 1);

        displayEdit = new QLineEdit(FontPreview);
        displayEdit->setObjectName(QString::fromUtf8("displayEdit"));

        gridLayout->addWidget(displayEdit, 4, 4, 1, 1);

        displayButton = new QPushButton(FontPreview);
        displayButton->setObjectName(QString::fromUtf8("displayButton"));

        gridLayout->addWidget(displayButton, 4, 5, 1, 1);

        resetDisplayButton = new QPushButton(FontPreview);
        resetDisplayButton->setObjectName(QString::fromUtf8("resetDisplayButton"));

        gridLayout->addWidget(resetDisplayButton, 4, 6, 1, 1);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        spacerItem = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        cancelButton = new QPushButton(FontPreview);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        hboxLayout->addWidget(cancelButton);


        gridLayout->addLayout(hboxLayout, 5, 0, 1, 7);

#if QT_CONFIG(shortcut)
        searchLabel->setBuddy(searchEdit);
        sizeLabel->setBuddy(sizeSpin);
#endif // QT_CONFIG(shortcut)

        retranslateUi(FontPreview);

        QMetaObject::connectSlotsByName(FontPreview);
    } // setupUi

    void retranslateUi(QDialog *FontPreview)
    {
        FontPreview->setWindowTitle(QCoreApplication::translate("FontPreview", "Fonts Preview", nullptr));
        searchLabel->setText(QCoreApplication::translate("FontPreview", "&Quick Search:", nullptr));
        extendedCheckBox->setText(QCoreApplication::translate("FontPreview", "Show Extended Font Information", nullptr));
        sizeLabel->setText(QCoreApplication::translate("FontPreview", "&Font Size:", nullptr));
        displayLabel->setText(QCoreApplication::translate("FontPreview", "Text", nullptr));
#if QT_CONFIG(tooltip)
        displayEdit->setToolTip(QCoreApplication::translate("FontPreview", "Sample text to display", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        displayButton->setToolTip(QCoreApplication::translate("FontPreview", "Set the preview text", nullptr));
#endif // QT_CONFIG(tooltip)
        displayButton->setText(QCoreApplication::translate("FontPreview", "Se&t", nullptr));
#if QT_CONFIG(shortcut)
        displayButton->setShortcut(QCoreApplication::translate("FontPreview", "Alt+T", nullptr));
#endif // QT_CONFIG(shortcut)
#if QT_CONFIG(tooltip)
        resetDisplayButton->setToolTip(QCoreApplication::translate("FontPreview", "Reset the text", nullptr));
#endif // QT_CONFIG(tooltip)
        resetDisplayButton->setText(QCoreApplication::translate("FontPreview", "&Default", nullptr));
        cancelButton->setText(QCoreApplication::translate("FontPreview", "&Close", nullptr));
#if QT_CONFIG(shortcut)
        cancelButton->setShortcut(QCoreApplication::translate("FontPreview", "Alt+C", nullptr));
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class FontPreview: public Ui_FontPreview {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_FONTPREVIEW_H
