/********************************************************************************
** Form generated from reading UI file 'charselect.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CHARSELECT_H
#define UI_CHARSELECT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include "chartableview.h"
#include "ui/unicodesearch.h"

QT_BEGIN_NAMESPACE

class Ui_CharSelect
{
public:
    QGridLayout *gridLayout;
    QHBoxLayout *hboxLayout;
    QPushButton *enhancedDialogButton;
    UnicodeChooseButton *unicodeButton;
    QPushButton *uniLoadButton;
    QPushButton *uniSaveButton;
    QPushButton *uniClearButton;
    CharTableView *m_userTable;
    QLabel *m_statusLabel;

    void setupUi(QDialog *CharSelect)
    {
        if (CharSelect->objectName().isEmpty())
            CharSelect->setObjectName(QString::fromUtf8("CharSelect"));
        CharSelect->resize(394, 319);
        gridLayout = new QGridLayout(CharSelect);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        enhancedDialogButton = new QPushButton(CharSelect);
        enhancedDialogButton->setObjectName(QString::fromUtf8("enhancedDialogButton"));
        enhancedDialogButton->setCheckable(true);

        hboxLayout->addWidget(enhancedDialogButton);

        unicodeButton = new UnicodeChooseButton(CharSelect);
        unicodeButton->setObjectName(QString::fromUtf8("unicodeButton"));

        hboxLayout->addWidget(unicodeButton);

        uniLoadButton = new QPushButton(CharSelect);
        uniLoadButton->setObjectName(QString::fromUtf8("uniLoadButton"));

        hboxLayout->addWidget(uniLoadButton);

        uniSaveButton = new QPushButton(CharSelect);
        uniSaveButton->setObjectName(QString::fromUtf8("uniSaveButton"));

        hboxLayout->addWidget(uniSaveButton);

        uniClearButton = new QPushButton(CharSelect);
        uniClearButton->setObjectName(QString::fromUtf8("uniClearButton"));

        hboxLayout->addWidget(uniClearButton);


        gridLayout->addLayout(hboxLayout, 0, 0, 1, 1);

        m_userTable = new CharTableView(CharSelect);
        m_userTable->setObjectName(QString::fromUtf8("m_userTable"));
        m_userTable->setDragDropOverwriteMode(false);
        m_userTable->setDragDropMode(QAbstractItemView::DropOnly);
        m_userTable->setSelectionMode(QAbstractItemView::SingleSelection);

        gridLayout->addWidget(m_userTable, 1, 0, 1, 1);

        m_statusLabel = new QLabel(CharSelect);
        m_statusLabel->setObjectName(QString::fromUtf8("m_statusLabel"));

        gridLayout->addWidget(m_statusLabel, 2, 0, 1, 1);

        QWidget::setTabOrder(enhancedDialogButton, unicodeButton);
        QWidget::setTabOrder(unicodeButton, uniLoadButton);
        QWidget::setTabOrder(uniLoadButton, uniSaveButton);
        QWidget::setTabOrder(uniSaveButton, uniClearButton);
        QWidget::setTabOrder(uniClearButton, m_userTable);

        retranslateUi(CharSelect);

        QMetaObject::connectSlotsByName(CharSelect);
    } // setupUi

    void retranslateUi(QDialog *CharSelect)
    {
        CharSelect->setWindowTitle(QCoreApplication::translate("CharSelect", "Character Palette", nullptr));
#if QT_CONFIG(tooltip)
        enhancedDialogButton->setToolTip(QCoreApplication::translate("CharSelect", "Show/Hide Enhanced Palette", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        unicodeButton->setToolTip(QCoreApplication::translate("CharSelect", "Unicode Search", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        uniLoadButton->setToolTip(QCoreApplication::translate("CharSelect", "Load a Character Palette", nullptr));
#endif // QT_CONFIG(tooltip)
        uniLoadButton->setText(QString());
#if QT_CONFIG(tooltip)
        uniSaveButton->setToolTip(QCoreApplication::translate("CharSelect", "Save the Character Palette", nullptr));
#endif // QT_CONFIG(tooltip)
        uniSaveButton->setText(QString());
#if QT_CONFIG(tooltip)
        uniClearButton->setToolTip(QCoreApplication::translate("CharSelect", "Empty the Character Palette", nullptr));
#endif // QT_CONFIG(tooltip)
        uniClearButton->setText(QString());
        m_statusLabel->setText(QString());
    } // retranslateUi

};

namespace Ui {
    class CharSelect: public Ui_CharSelect {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CHARSELECT_H
