/****************************************************************************
** Meta object code from reading C++ file 'scraction.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/scraction.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scraction.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ScrAction_t {
    QByteArrayData data[12];
    char stringdata0[150];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ScrAction_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ScrAction_t qt_meta_stringdata_ScrAction = {
    {
QT_MOC_LITERAL(0, 0, 9), // "ScrAction"
QT_MOC_LITERAL(1, 10, 13), // "triggeredData"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 11), // "ScribusDoc*"
QT_MOC_LITERAL(4, 37, 8), // "QWidget*"
QT_MOC_LITERAL(5, 46, 11), // "toggledData"
QT_MOC_LITERAL(6, 58, 24), // "triggeredUnicodeShortcut"
QT_MOC_LITERAL(7, 83, 6), // "toggle"
QT_MOC_LITERAL(8, 90, 8), // "loadIcon"
QT_MOC_LITERAL(9, 99, 24), // "triggeredToTriggeredData"
QT_MOC_LITERAL(10, 124, 20), // "toggledToToggledData"
QT_MOC_LITERAL(11, 145, 4) // "ison"

    },
    "ScrAction\0triggeredData\0\0ScribusDoc*\0"
    "QWidget*\0toggledData\0triggeredUnicodeShortcut\0"
    "toggle\0loadIcon\0triggeredToTriggeredData\0"
    "toggledToToggledData\0ison"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ScrAction[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      10,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   84,    2, 0x06 /* Public */,
       1,    1,   87,    2, 0x06 /* Public */,
       1,    1,   90,    2, 0x06 /* Public */,
       1,    1,   93,    2, 0x06 /* Public */,
       1,    1,   96,    2, 0x06 /* Public */,
       1,    2,   99,    2, 0x06 /* Public */,
       5,    2,  104,    2, 0x06 /* Public */,
       5,    2,  109,    2, 0x06 /* Public */,
       5,    2,  114,    2, 0x06 /* Public */,
       6,    1,  119,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       7,    0,  122,    2, 0x0a /* Public */,
       8,    0,  123,    2, 0x08 /* Private */,
       9,    0,  124,    2, 0x08 /* Private */,
      10,    1,  125,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::QVariant,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, 0x80000000 | 4, 0x80000000 | 3,    2,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Int,    2,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::Double,    2,    2,
    QMetaType::Void, QMetaType::Bool, QMetaType::QString,    2,    2,
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   11,

       0        // eod
};

void ScrAction::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ScrAction *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->triggeredData((*reinterpret_cast< QVariant(*)>(_a[1]))); break;
        case 1: _t->triggeredData((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->triggeredData((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 3: _t->triggeredData((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 4: _t->triggeredData((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 5: _t->triggeredData((*reinterpret_cast< QWidget*(*)>(_a[1])),(*reinterpret_cast< ScribusDoc*(*)>(_a[2]))); break;
        case 6: _t->toggledData((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 7: _t->toggledData((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 8: _t->toggledData((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2]))); break;
        case 9: _t->triggeredUnicodeShortcut((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 10: _t->toggle(); break;
        case 11: _t->loadIcon(); break;
        case 12: _t->triggeredToTriggeredData(); break;
        case 13: _t->toggledToToggledData((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 5:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QWidget* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ScrAction::*)(QVariant );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScrAction::triggeredData)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ScrAction::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScrAction::triggeredData)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ScrAction::*)(double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScrAction::triggeredData)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ScrAction::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScrAction::triggeredData)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ScrAction::*)(ScribusDoc * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScrAction::triggeredData)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (ScrAction::*)(QWidget * , ScribusDoc * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScrAction::triggeredData)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (ScrAction::*)(bool , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScrAction::toggledData)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (ScrAction::*)(bool , double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScrAction::toggledData)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (ScrAction::*)(bool , QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScrAction::toggledData)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (ScrAction::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScrAction::triggeredUnicodeShortcut)) {
                *result = 9;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ScrAction::staticMetaObject = { {
    QMetaObject::SuperData::link<QAction::staticMetaObject>(),
    qt_meta_stringdata_ScrAction.data,
    qt_meta_data_ScrAction,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ScrAction::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ScrAction::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ScrAction.stringdata0))
        return static_cast<void*>(this);
    return QAction::qt_metacast(_clname);
}

int ScrAction::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAction::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    }
    return _id;
}

// SIGNAL 0
void ScrAction::triggeredData(QVariant _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ScrAction::triggeredData(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ScrAction::triggeredData(double _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void ScrAction::triggeredData(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ScrAction::triggeredData(ScribusDoc * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void ScrAction::triggeredData(QWidget * _t1, ScribusDoc * _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void ScrAction::toggledData(bool _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void ScrAction::toggledData(bool _t1, double _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void ScrAction::toggledData(bool _t1, QString _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void ScrAction::triggeredUnicodeShortcut(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
