/********************************************************************************
** Form generated from reading UI file 'satdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SATDIALOG_H
#define UI_SATDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QTextEdit>

QT_BEGIN_NAMESPACE

class Ui_SATDialogBase
{
public:
    QFormLayout *formLayout;
    QLabel *label;
    QLineEdit *nameEdit;
    QCheckBox *detailButton;
    QLabel *catsLabel;
    QComboBox *catsCombo;
    QLabel *psizeLabel;
    QLineEdit *psizeEdit;
    QLabel *colorsLabel;
    QLineEdit *colorsEdit;
    QLabel *descrLabel;
    QTextEdit *descrEdit;
    QLabel *usageLabel;
    QTextEdit *usageEdit;
    QLabel *authorLabel;
    QLineEdit *authorEdit;
    QLabel *emailLabel;
    QLineEdit *emailEdit;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *SATDialogBase)
    {
        if (SATDialogBase->objectName().isEmpty())
            SATDialogBase->setObjectName(QString::fromUtf8("SATDialogBase"));
        SATDialogBase->resize(404, 447);
        formLayout = new QFormLayout(SATDialogBase);
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setContentsMargins(9, 9, 9, 9);
        label = new QLabel(SATDialogBase);
        label->setObjectName(QString::fromUtf8("label"));

        formLayout->setWidget(0, QFormLayout::LabelRole, label);

        nameEdit = new QLineEdit(SATDialogBase);
        nameEdit->setObjectName(QString::fromUtf8("nameEdit"));

        formLayout->setWidget(0, QFormLayout::FieldRole, nameEdit);

        detailButton = new QCheckBox(SATDialogBase);
        detailButton->setObjectName(QString::fromUtf8("detailButton"));

        formLayout->setWidget(1, QFormLayout::SpanningRole, detailButton);

        catsLabel = new QLabel(SATDialogBase);
        catsLabel->setObjectName(QString::fromUtf8("catsLabel"));

        formLayout->setWidget(2, QFormLayout::LabelRole, catsLabel);

        catsCombo = new QComboBox(SATDialogBase);
        catsCombo->setObjectName(QString::fromUtf8("catsCombo"));

        formLayout->setWidget(2, QFormLayout::FieldRole, catsCombo);

        psizeLabel = new QLabel(SATDialogBase);
        psizeLabel->setObjectName(QString::fromUtf8("psizeLabel"));

        formLayout->setWidget(3, QFormLayout::LabelRole, psizeLabel);

        psizeEdit = new QLineEdit(SATDialogBase);
        psizeEdit->setObjectName(QString::fromUtf8("psizeEdit"));

        formLayout->setWidget(3, QFormLayout::FieldRole, psizeEdit);

        colorsLabel = new QLabel(SATDialogBase);
        colorsLabel->setObjectName(QString::fromUtf8("colorsLabel"));

        formLayout->setWidget(4, QFormLayout::LabelRole, colorsLabel);

        colorsEdit = new QLineEdit(SATDialogBase);
        colorsEdit->setObjectName(QString::fromUtf8("colorsEdit"));

        formLayout->setWidget(4, QFormLayout::FieldRole, colorsEdit);

        descrLabel = new QLabel(SATDialogBase);
        descrLabel->setObjectName(QString::fromUtf8("descrLabel"));

        formLayout->setWidget(5, QFormLayout::LabelRole, descrLabel);

        descrEdit = new QTextEdit(SATDialogBase);
        descrEdit->setObjectName(QString::fromUtf8("descrEdit"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(descrEdit->sizePolicy().hasHeightForWidth());
        descrEdit->setSizePolicy(sizePolicy);

        formLayout->setWidget(5, QFormLayout::FieldRole, descrEdit);

        usageLabel = new QLabel(SATDialogBase);
        usageLabel->setObjectName(QString::fromUtf8("usageLabel"));

        formLayout->setWidget(6, QFormLayout::LabelRole, usageLabel);

        usageEdit = new QTextEdit(SATDialogBase);
        usageEdit->setObjectName(QString::fromUtf8("usageEdit"));
        sizePolicy.setHeightForWidth(usageEdit->sizePolicy().hasHeightForWidth());
        usageEdit->setSizePolicy(sizePolicy);

        formLayout->setWidget(6, QFormLayout::FieldRole, usageEdit);

        authorLabel = new QLabel(SATDialogBase);
        authorLabel->setObjectName(QString::fromUtf8("authorLabel"));

        formLayout->setWidget(7, QFormLayout::LabelRole, authorLabel);

        authorEdit = new QLineEdit(SATDialogBase);
        authorEdit->setObjectName(QString::fromUtf8("authorEdit"));

        formLayout->setWidget(7, QFormLayout::FieldRole, authorEdit);

        emailLabel = new QLabel(SATDialogBase);
        emailLabel->setObjectName(QString::fromUtf8("emailLabel"));

        formLayout->setWidget(8, QFormLayout::LabelRole, emailLabel);

        emailEdit = new QLineEdit(SATDialogBase);
        emailEdit->setObjectName(QString::fromUtf8("emailEdit"));

        formLayout->setWidget(8, QFormLayout::FieldRole, emailEdit);

        buttonBox = new QDialogButtonBox(SATDialogBase);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        formLayout->setWidget(9, QFormLayout::SpanningRole, buttonBox);


        retranslateUi(SATDialogBase);
        QObject::connect(buttonBox, SIGNAL(accepted()), SATDialogBase, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), SATDialogBase, SLOT(reject()));

        QMetaObject::connectSlotsByName(SATDialogBase);
    } // setupUi

    void retranslateUi(QDialog *SATDialogBase)
    {
        SATDialogBase->setWindowTitle(QCoreApplication::translate("SATDialogBase", "Save as Template", nullptr));
        label->setText(QCoreApplication::translate("SATDialogBase", "Name:", nullptr));
#if QT_CONFIG(tooltip)
        nameEdit->setToolTip(QCoreApplication::translate("SATDialogBase", "Name of the document template", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        detailButton->setToolTip(QCoreApplication::translate("SATDialogBase", "Show more details to describe the document template", nullptr));
#endif // QT_CONFIG(tooltip)
        detailButton->setText(QCoreApplication::translate("SATDialogBase", "&More Details", nullptr));
        catsLabel->setText(QCoreApplication::translate("SATDialogBase", "Category:", nullptr));
#if QT_CONFIG(tooltip)
        catsCombo->setToolTip(QCoreApplication::translate("SATDialogBase", "Category of the document template", nullptr));
#endif // QT_CONFIG(tooltip)
        psizeLabel->setText(QCoreApplication::translate("SATDialogBase", "Page Size:", nullptr));
#if QT_CONFIG(tooltip)
        psizeEdit->setToolTip(QCoreApplication::translate("SATDialogBase", "Page size of the document template", nullptr));
#endif // QT_CONFIG(tooltip)
        colorsLabel->setText(QCoreApplication::translate("SATDialogBase", "Colors:", nullptr));
#if QT_CONFIG(tooltip)
        colorsEdit->setToolTip(QCoreApplication::translate("SATDialogBase", "Description of the color format of the document, or some hints regarding colors used", nullptr));
#endif // QT_CONFIG(tooltip)
        descrLabel->setText(QCoreApplication::translate("SATDialogBase", "Description:", nullptr));
#if QT_CONFIG(tooltip)
        descrEdit->setToolTip(QCoreApplication::translate("SATDialogBase", "Description of the document template", nullptr));
#endif // QT_CONFIG(tooltip)
        usageLabel->setText(QCoreApplication::translate("SATDialogBase", "Usage:", nullptr));
#if QT_CONFIG(tooltip)
        usageEdit->setToolTip(QCoreApplication::translate("SATDialogBase", "Planned usage of the document template", nullptr));
#endif // QT_CONFIG(tooltip)
        authorLabel->setText(QCoreApplication::translate("SATDialogBase", "Author:", nullptr));
#if QT_CONFIG(tooltip)
        authorEdit->setToolTip(QCoreApplication::translate("SATDialogBase", "Author of the document template", nullptr));
#endif // QT_CONFIG(tooltip)
        emailLabel->setText(QCoreApplication::translate("SATDialogBase", "Email:", nullptr));
#if QT_CONFIG(tooltip)
        emailEdit->setToolTip(QCoreApplication::translate("SATDialogBase", "Email address of the author of the document template", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class SATDialogBase: public Ui_SATDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SATDIALOG_H
