/********************************************************************************
** Form generated from reading UI file 'multiprogressdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_MULTIPROGRESSDIALOG_H
#define UI_MULTIPROGRESSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QProgressBar>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include "scribusapi.h"

QT_BEGIN_NAMESPACE

class Ui_MultiProgressDialog
{
public:
    QVBoxLayout *vboxLayout;
    QGridLayout *gridLayout;
    QLabel *overallProgressLabel;
    QProgressBar *overallProgressBar;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *buttonCancel;

    void setupUi(QDialog *MultiProgressDialog)
    {
        if (MultiProgressDialog->objectName().isEmpty())
            MultiProgressDialog->setObjectName(QString::fromUtf8("MultiProgressDialog"));
        MultiProgressDialog->resize(230, 84);
        MultiProgressDialog->setSizeGripEnabled(false);
        vboxLayout = new QVBoxLayout(MultiProgressDialog);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        overallProgressLabel = new QLabel(MultiProgressDialog);
        overallProgressLabel->setObjectName(QString::fromUtf8("overallProgressLabel"));
        overallProgressLabel->setWordWrap(false);

        gridLayout->addWidget(overallProgressLabel, 0, 0, 1, 1);

        overallProgressBar = new QProgressBar(MultiProgressDialog);
        overallProgressBar->setObjectName(QString::fromUtf8("overallProgressBar"));
        overallProgressBar->setValue(0);

        gridLayout->addWidget(overallProgressBar, 0, 1, 1, 1);


        vboxLayout->addLayout(gridLayout);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        spacerItem = new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        buttonCancel = new QPushButton(MultiProgressDialog);
        buttonCancel->setObjectName(QString::fromUtf8("buttonCancel"));
        buttonCancel->setAutoDefault(true);

        hboxLayout->addWidget(buttonCancel);


        vboxLayout->addLayout(hboxLayout);


        retranslateUi(MultiProgressDialog);
        QObject::connect(buttonCancel, SIGNAL(clicked()), MultiProgressDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(MultiProgressDialog);
    } // setupUi

    void retranslateUi(QDialog *MultiProgressDialog)
    {
        MultiProgressDialog->setWindowTitle(QCoreApplication::translate("MultiProgressDialog", "Progress", nullptr));
        overallProgressLabel->setText(QCoreApplication::translate("MultiProgressDialog", "Overall Progress:", nullptr));
        buttonCancel->setText(QCoreApplication::translate("MultiProgressDialog", "&Cancel", nullptr));
#if QT_CONFIG(shortcut)
        buttonCancel->setShortcut(QString());
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class MultiProgressDialog: public Ui_MultiProgressDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_MULTIPROGRESSDIALOG_H
