/****************************************************************************
** Meta object code from reading C++ file 'scribus.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/scribus.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scribus.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ScribusMainWindow_t {
    QByteArrayData data[305];
    char stringdata0[4121];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ScribusMainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ScribusMainWindow_t qt_meta_stringdata_ScribusMainWindow = {
    {
QT_MOC_LITERAL(0, 0, 17), // "ScribusMainWindow"
QT_MOC_LITERAL(1, 18, 9), // "TextStyle"
QT_MOC_LITERAL(2, 28, 0), // ""
QT_MOC_LITERAL(3, 29, 14), // "ParagraphStyle"
QT_MOC_LITERAL(4, 44, 11), // "TextEffects"
QT_MOC_LITERAL(5, 56, 13), // "UpdateRequest"
QT_MOC_LITERAL(6, 70, 11), // "updateFlags"
QT_MOC_LITERAL(7, 82, 12), // "changeLayers"
QT_MOC_LITERAL(8, 95, 13), // "iconSetChange"
QT_MOC_LITERAL(9, 109, 14), // "languageChange"
QT_MOC_LITERAL(10, 124, 12), // "localeChange"
QT_MOC_LITERAL(11, 137, 23), // "statusBarLanguageChange"
QT_MOC_LITERAL(12, 161, 21), // "specialActionKeyEvent"
QT_MOC_LITERAL(13, 183, 12), // "unicodevalue"
QT_MOC_LITERAL(14, 196, 7), // "newView"
QT_MOC_LITERAL(15, 204, 17), // "ToggleStickyTools"
QT_MOC_LITERAL(16, 222, 15), // "ToggleAllGuides"
QT_MOC_LITERAL(17, 238, 17), // "ToggleAllPalettes"
QT_MOC_LITERAL(18, 256, 15), // "slotStoryEditor"
QT_MOC_LITERAL(19, 272, 9), // "fromTable"
QT_MOC_LITERAL(20, 282, 14), // "slotCharSelect"
QT_MOC_LITERAL(21, 297, 12), // "ImageEffects"
QT_MOC_LITERAL(22, 310, 11), // "fileCollect"
QT_MOC_LITERAL(23, 322, 8), // "compress"
QT_MOC_LITERAL(24, 331, 9), // "withFonts"
QT_MOC_LITERAL(25, 341, 12), // "withProfiles"
QT_MOC_LITERAL(26, 354, 12), // "newDirectory"
QT_MOC_LITERAL(27, 367, 11), // "AddBookMark"
QT_MOC_LITERAL(28, 379, 9), // "PageItem*"
QT_MOC_LITERAL(29, 389, 3), // "ite"
QT_MOC_LITERAL(30, 393, 11), // "DelBookMark"
QT_MOC_LITERAL(31, 405, 11), // "BookMarkTxT"
QT_MOC_LITERAL(32, 417, 14), // "StoreBookmarks"
QT_MOC_LITERAL(33, 432, 25), // "setStatusBarMousePosition"
QT_MOC_LITERAL(34, 458, 2), // "xp"
QT_MOC_LITERAL(35, 461, 2), // "yp"
QT_MOC_LITERAL(36, 464, 24), // "setStatusBarTextPosition"
QT_MOC_LITERAL(37, 489, 4), // "base"
QT_MOC_LITERAL(38, 494, 32), // "setStatusBarTextSelectedItemInfo"
QT_MOC_LITERAL(39, 527, 20), // "setTempStatusBarText"
QT_MOC_LITERAL(40, 548, 4), // "text"
QT_MOC_LITERAL(41, 553, 20), // "setStatusBarInfoText"
QT_MOC_LITERAL(42, 574, 7), // "newText"
QT_MOC_LITERAL(43, 582, 11), // "DoFileClose"
QT_MOC_LITERAL(44, 594, 22), // "windowsMenuAboutToShow"
QT_MOC_LITERAL(45, 617, 21), // "extrasMenuAboutToShow"
QT_MOC_LITERAL(46, 639, 9), // "newActWin"
QT_MOC_LITERAL(47, 649, 14), // "QMdiSubWindow*"
QT_MOC_LITERAL(48, 664, 1), // "w"
QT_MOC_LITERAL(49, 666, 33), // "closeActiveWindowMasterPageEd..."
QT_MOC_LITERAL(50, 700, 25), // "updateActiveWindowCaption"
QT_MOC_LITERAL(51, 726, 10), // "newCaption"
QT_MOC_LITERAL(52, 737, 20), // "windowsMenuActivated"
QT_MOC_LITERAL(53, 758, 2), // "id"
QT_MOC_LITERAL(54, 761, 8), // "PutScrap"
QT_MOC_LITERAL(55, 770, 4), // "scID"
QT_MOC_LITERAL(56, 775, 11), // "PutToInline"
QT_MOC_LITERAL(57, 787, 6), // "buffer"
QT_MOC_LITERAL(58, 794, 13), // "PutToPatterns"
QT_MOC_LITERAL(59, 808, 15), // "ConvertToSymbol"
QT_MOC_LITERAL(60, 824, 11), // "changeLayer"
QT_MOC_LITERAL(61, 836, 16), // "setLayerMenuText"
QT_MOC_LITERAL(62, 853, 9), // "showLayer"
QT_MOC_LITERAL(63, 863, 18), // "slotSetCurrentPage"
QT_MOC_LITERAL(64, 882, 9), // "pageIndex"
QT_MOC_LITERAL(65, 892, 14), // "setCurrentPage"
QT_MOC_LITERAL(66, 907, 1), // "p"
QT_MOC_LITERAL(67, 909, 10), // "ManageJava"
QT_MOC_LITERAL(68, 920, 23), // "editSelectedSymbolStart"
QT_MOC_LITERAL(69, 944, 15), // "editSymbolStart"
QT_MOC_LITERAL(70, 960, 4), // "temp"
QT_MOC_LITERAL(71, 965, 13), // "editSymbolEnd"
QT_MOC_LITERAL(72, 979, 15), // "editInlineStart"
QT_MOC_LITERAL(73, 995, 13), // "editInlineEnd"
QT_MOC_LITERAL(74, 1009, 20), // "editMasterPagesStart"
QT_MOC_LITERAL(75, 1030, 18), // "editMasterPagesEnd"
QT_MOC_LITERAL(76, 1049, 11), // "slotFileNew"
QT_MOC_LITERAL(77, 1061, 19), // "newFileFromTemplate"
QT_MOC_LITERAL(78, 1081, 14), // "slotPageImport"
QT_MOC_LITERAL(79, 1096, 8), // "loadPage"
QT_MOC_LITERAL(80, 1105, 8), // "fileName"
QT_MOC_LITERAL(81, 1114, 2), // "Nr"
QT_MOC_LITERAL(82, 1117, 3), // "Mpa"
QT_MOC_LITERAL(83, 1121, 15), // "renamedPageName"
QT_MOC_LITERAL(84, 1137, 9), // "gotoLayer"
QT_MOC_LITERAL(85, 1147, 1), // "l"
QT_MOC_LITERAL(86, 1149, 14), // "slotGetContent"
QT_MOC_LITERAL(87, 1164, 15), // "slotGetContent2"
QT_MOC_LITERAL(88, 1180, 21), // "slotGetClipboardImage"
QT_MOC_LITERAL(89, 1202, 17), // "toogleInlineState"
QT_MOC_LITERAL(90, 1220, 14), // "slotFileAppend"
QT_MOC_LITERAL(91, 1235, 12), // "removeRecent"
QT_MOC_LITERAL(92, 1248, 2), // "fn"
QT_MOC_LITERAL(93, 1251, 15), // "fromFileWatcher"
QT_MOC_LITERAL(94, 1267, 23), // "removeRecentFromWatcher"
QT_MOC_LITERAL(95, 1291, 8), // "filename"
QT_MOC_LITERAL(96, 1300, 10), // "loadRecent"
QT_MOC_LITERAL(97, 1311, 21), // "rebuildRecentFileMenu"
QT_MOC_LITERAL(98, 1333, 22), // "rebuildRecentPasteMenu"
QT_MOC_LITERAL(99, 1356, 20), // "rebuildScrapbookMenu"
QT_MOC_LITERAL(100, 1377, 11), // "pasteRecent"
QT_MOC_LITERAL(101, 1389, 18), // "pasteFromScrapbook"
QT_MOC_LITERAL(102, 1408, 16), // "importVectorFile"
QT_MOC_LITERAL(103, 1425, 17), // "rebuildLayersList"
QT_MOC_LITERAL(104, 1443, 12), // "slotFileOpen"
QT_MOC_LITERAL(105, 1456, 7), // "loadDoc"
QT_MOC_LITERAL(106, 1464, 11), // "postLoadDoc"
QT_MOC_LITERAL(107, 1476, 12), // "slotFileSave"
QT_MOC_LITERAL(108, 1489, 14), // "slotFileSaveAs"
QT_MOC_LITERAL(109, 1504, 14), // "slotFileRevert"
QT_MOC_LITERAL(110, 1519, 8), // "SaveText"
QT_MOC_LITERAL(111, 1528, 13), // "slotFileClose"
QT_MOC_LITERAL(112, 1542, 13), // "slotFilePrint"
QT_MOC_LITERAL(113, 1556, 15), // "slotReallyPrint"
QT_MOC_LITERAL(114, 1572, 18), // "slotEndSpecialEdit"
QT_MOC_LITERAL(115, 1591, 7), // "doPrint"
QT_MOC_LITERAL(116, 1599, 13), // "PrintOptions&"
QT_MOC_LITERAL(117, 1613, 7), // "options"
QT_MOC_LITERAL(118, 1621, 8), // "QString&"
QT_MOC_LITERAL(119, 1630, 5), // "error"
QT_MOC_LITERAL(120, 1636, 12), // "slotFileQuit"
QT_MOC_LITERAL(121, 1649, 11), // "slotEditCut"
QT_MOC_LITERAL(122, 1661, 12), // "slotEditCopy"
QT_MOC_LITERAL(123, 1674, 13), // "slotEditPaste"
QT_MOC_LITERAL(124, 1688, 20), // "slotEditCopyContents"
QT_MOC_LITERAL(125, 1709, 21), // "slotEditPasteContents"
QT_MOC_LITERAL(126, 1731, 8), // "absolute"
QT_MOC_LITERAL(127, 1740, 9), // "SelectAll"
QT_MOC_LITERAL(128, 1750, 13), // "docWideSelect"
QT_MOC_LITERAL(129, 1764, 16), // "SelectAllOnLayer"
QT_MOC_LITERAL(130, 1781, 11), // "deselectAll"
QT_MOC_LITERAL(131, 1793, 10), // "ClipChange"
QT_MOC_LITERAL(132, 1804, 17), // "setCopyCutEnabled"
QT_MOC_LITERAL(133, 1822, 1), // "b"
QT_MOC_LITERAL(134, 1824, 13), // "slotHelpAbout"
QT_MOC_LITERAL(135, 1838, 20), // "slotHelpAboutPlugins"
QT_MOC_LITERAL(136, 1859, 15), // "slotHelpAboutQt"
QT_MOC_LITERAL(137, 1875, 20), // "slotHelpActionSearch"
QT_MOC_LITERAL(138, 1896, 20), // "slotHelpCheckUpdates"
QT_MOC_LITERAL(139, 1917, 19), // "slotRaiseOnlineHelp"
QT_MOC_LITERAL(140, 1937, 14), // "slotOnlineHelp"
QT_MOC_LITERAL(141, 1952, 13), // "jumpToSection"
QT_MOC_LITERAL(142, 1966, 10), // "jumpToFile"
QT_MOC_LITERAL(143, 1977, 20), // "slotOnlineHelpClosed"
QT_MOC_LITERAL(144, 1998, 19), // "slotResourceManager"
QT_MOC_LITERAL(145, 2018, 10), // "ToggleTips"
QT_MOC_LITERAL(146, 2029, 15), // "ToggleMouseTips"
QT_MOC_LITERAL(147, 2045, 12), // "slotNewPageP"
QT_MOC_LITERAL(148, 2058, 2), // "wo"
QT_MOC_LITERAL(149, 2061, 5), // "templ"
QT_MOC_LITERAL(150, 2067, 12), // "slotNewPageM"
QT_MOC_LITERAL(151, 2080, 17), // "slotNewMasterPage"
QT_MOC_LITERAL(152, 2098, 11), // "slotNewPage"
QT_MOC_LITERAL(153, 2110, 14), // "masterPageName"
QT_MOC_LITERAL(154, 2125, 3), // "mov"
QT_MOC_LITERAL(155, 2129, 21), // "duplicateToMasterPage"
QT_MOC_LITERAL(156, 2151, 10), // "deletePage"
QT_MOC_LITERAL(157, 2162, 4), // "from"
QT_MOC_LITERAL(158, 2167, 2), // "to"
QT_MOC_LITERAL(159, 2170, 11), // "deletePage2"
QT_MOC_LITERAL(160, 2182, 2), // "pg"
QT_MOC_LITERAL(161, 2185, 8), // "movePage"
QT_MOC_LITERAL(162, 2194, 8), // "copyPage"
QT_MOC_LITERAL(163, 2203, 20), // "changePageProperties"
QT_MOC_LITERAL(164, 2224, 8), // "slotZoom"
QT_MOC_LITERAL(165, 2233, 10), // "zoomFactor"
QT_MOC_LITERAL(166, 2244, 11), // "toggleMarks"
QT_MOC_LITERAL(167, 2256, 12), // "toggleBleeds"
QT_MOC_LITERAL(168, 2269, 12), // "toggleFrames"
QT_MOC_LITERAL(169, 2282, 18), // "toggleLayerMarkers"
QT_MOC_LITERAL(170, 2301, 15), // "toggleTextLinks"
QT_MOC_LITERAL(171, 2317, 18), // "toggleTextControls"
QT_MOC_LITERAL(172, 2336, 19), // "toggleColumnBorders"
QT_MOC_LITERAL(173, 2356, 12), // "toggleRulers"
QT_MOC_LITERAL(174, 2369, 15), // "toggleRulerMode"
QT_MOC_LITERAL(175, 2385, 17), // "togglePagePalette"
QT_MOC_LITERAL(176, 2403, 17), // "toggleUndoPalette"
QT_MOC_LITERAL(177, 2421, 14), // "setUndoPalette"
QT_MOC_LITERAL(178, 2436, 7), // "visible"
QT_MOC_LITERAL(179, 2444, 14), // "toggleCheckPal"
QT_MOC_LITERAL(180, 2459, 21), // "toggleImageVisibility"
QT_MOC_LITERAL(181, 2481, 10), // "toggleGrid"
QT_MOC_LITERAL(182, 2492, 14), // "toggleSnapGrid"
QT_MOC_LITERAL(183, 2507, 14), // "toggleNodeEdit"
QT_MOC_LITERAL(184, 2522, 10), // "slotSelect"
QT_MOC_LITERAL(185, 2533, 18), // "setAppModeByToggle"
QT_MOC_LITERAL(186, 2552, 4), // "isOn"
QT_MOC_LITERAL(187, 2557, 7), // "newMode"
QT_MOC_LITERAL(188, 2565, 10), // "HaveNewDoc"
QT_MOC_LITERAL(189, 2576, 10), // "HaveNewSel"
QT_MOC_LITERAL(190, 2587, 9), // "slotDocCh"
QT_MOC_LITERAL(191, 2597, 3), // "reb"
QT_MOC_LITERAL(192, 2601, 15), // "setItemFontSize"
QT_MOC_LITERAL(193, 2617, 8), // "fontSize"
QT_MOC_LITERAL(194, 2626, 15), // "setItemLanguage"
QT_MOC_LITERAL(195, 2642, 8), // "language"
QT_MOC_LITERAL(196, 2651, 17), // "slotReplaceColors"
QT_MOC_LITERAL(197, 2669, 9), // "MakeFrame"
QT_MOC_LITERAL(198, 2679, 1), // "f"
QT_MOC_LITERAL(199, 2681, 1), // "c"
QT_MOC_LITERAL(200, 2683, 7), // "double*"
QT_MOC_LITERAL(201, 2691, 4), // "vals"
QT_MOC_LITERAL(202, 2696, 13), // "duplicateItem"
QT_MOC_LITERAL(203, 2710, 18), // "duplicateItemMulti"
QT_MOC_LITERAL(204, 2729, 16), // "objectAttributes"
QT_MOC_LITERAL(205, 2746, 12), // "getImageInfo"
QT_MOC_LITERAL(206, 2759, 23), // "generateTableOfContents"
QT_MOC_LITERAL(207, 2783, 14), // "updateDocument"
QT_MOC_LITERAL(208, 2798, 15), // "setNewAlignment"
QT_MOC_LITERAL(209, 2814, 1), // "i"
QT_MOC_LITERAL(210, 2816, 15), // "setNewDirection"
QT_MOC_LITERAL(211, 2832, 14), // "setNewParStyle"
QT_MOC_LITERAL(212, 2847, 4), // "name"
QT_MOC_LITERAL(213, 2852, 15), // "setNewCharStyle"
QT_MOC_LITERAL(214, 2868, 17), // "setAlignmentValue"
QT_MOC_LITERAL(215, 2886, 17), // "setDirectionValue"
QT_MOC_LITERAL(216, 2904, 21), // "editItemsFromOutlines"
QT_MOC_LITERAL(217, 2926, 23), // "selectItemsFromOutlines"
QT_MOC_LITERAL(218, 2950, 6), // "single"
QT_MOC_LITERAL(219, 2957, 8), // "position"
QT_MOC_LITERAL(220, 2966, 22), // "selectItemFromOutlines"
QT_MOC_LITERAL(221, 2989, 4), // "cPos"
QT_MOC_LITERAL(222, 2994, 23), // "selectPagesFromOutlines"
QT_MOC_LITERAL(223, 3018, 6), // "ScPage"
QT_MOC_LITERAL(224, 3025, 14), // "doPrintPreview"
QT_MOC_LITERAL(225, 3040, 12), // "printPreview"
QT_MOC_LITERAL(226, 3053, 18), // "doOutputPreviewPDF"
QT_MOC_LITERAL(227, 3072, 16), // "outputPreviewPDF"
QT_MOC_LITERAL(228, 3089, 17), // "doOutputPreviewPS"
QT_MOC_LITERAL(229, 3107, 15), // "outputPreviewPS"
QT_MOC_LITERAL(230, 3123, 9), // "SaveAsEps"
QT_MOC_LITERAL(231, 3133, 15), // "reallySaveAsEps"
QT_MOC_LITERAL(232, 3149, 9), // "SaveAsPDF"
QT_MOC_LITERAL(233, 3159, 11), // "doSaveAsPDF"
QT_MOC_LITERAL(234, 3171, 19), // "setMainWindowActive"
QT_MOC_LITERAL(235, 3191, 14), // "setItemEffects"
QT_MOC_LITERAL(236, 3206, 1), // "h"
QT_MOC_LITERAL(237, 3208, 15), // "setStyleEffects"
QT_MOC_LITERAL(238, 3224, 1), // "s"
QT_MOC_LITERAL(239, 3226, 16), // "setItemTypeStyle"
QT_MOC_LITERAL(240, 3243, 12), // "slotElemRead"
QT_MOC_LITERAL(241, 3256, 4), // "Name"
QT_MOC_LITERAL(242, 3261, 1), // "x"
QT_MOC_LITERAL(243, 3263, 1), // "y"
QT_MOC_LITERAL(244, 3265, 3), // "art"
QT_MOC_LITERAL(245, 3269, 4), // "loca"
QT_MOC_LITERAL(246, 3274, 11), // "ScribusDoc*"
QT_MOC_LITERAL(247, 3286, 4), // "docc"
QT_MOC_LITERAL(248, 3291, 12), // "ScribusView*"
QT_MOC_LITERAL(249, 3304, 3), // "vie"
QT_MOC_LITERAL(250, 3308, 14), // "slotChangeUnit"
QT_MOC_LITERAL(251, 3323, 4), // "draw"
QT_MOC_LITERAL(252, 3328, 15), // "ApplyMasterPage"
QT_MOC_LITERAL(253, 3344, 16), // "Apply_MasterPage"
QT_MOC_LITERAL(254, 3361, 8), // "pageName"
QT_MOC_LITERAL(255, 3370, 10), // "pageNumber"
QT_MOC_LITERAL(256, 3381, 8), // "GroupObj"
QT_MOC_LITERAL(257, 3390, 11), // "showLockDia"
QT_MOC_LITERAL(258, 3402, 10), // "UnGroupObj"
QT_MOC_LITERAL(259, 3413, 14), // "AdjustGroupObj"
QT_MOC_LITERAL(260, 3428, 9), // "StatusPic"
QT_MOC_LITERAL(261, 3438, 11), // "ModifyAnnot"
QT_MOC_LITERAL(262, 3450, 12), // "toggleGuides"
QT_MOC_LITERAL(263, 3463, 10), // "toggleBase"
QT_MOC_LITERAL(264, 3474, 16), // "toggleSnapGuides"
QT_MOC_LITERAL(265, 3491, 18), // "toggleSnapElements"
QT_MOC_LITERAL(266, 3510, 15), // "SetSnapElements"
QT_MOC_LITERAL(267, 3526, 8), // "EditTabs"
QT_MOC_LITERAL(268, 3535, 10), // "SearchText"
QT_MOC_LITERAL(269, 3546, 15), // "callImageEditor"
QT_MOC_LITERAL(270, 3562, 14), // "docCheckToggle"
QT_MOC_LITERAL(271, 3577, 12), // "scanDocument"
QT_MOC_LITERAL(272, 3590, 11), // "setUndoMode"
QT_MOC_LITERAL(273, 3602, 16), // "isObjectSpecific"
QT_MOC_LITERAL(274, 3619, 16), // "insertSampleText"
QT_MOC_LITERAL(275, 3636, 19), // "updateItemLayerList"
QT_MOC_LITERAL(276, 3656, 16), // "updateColorLists"
QT_MOC_LITERAL(277, 3673, 12), // "slotPrefsOrg"
QT_MOC_LITERAL(278, 3686, 12), // "slotDocSetup"
QT_MOC_LITERAL(279, 3699, 15), // "slotInsertFrame"
QT_MOC_LITERAL(280, 3715, 17), // "slotItemTransform"
QT_MOC_LITERAL(281, 3733, 20), // "manageColorsAndFills"
QT_MOC_LITERAL(282, 3754, 8), // "styleMgr"
QT_MOC_LITERAL(283, 3763, 13), // "StyleManager*"
QT_MOC_LITERAL(284, 3777, 26), // "updateGUIAfterPagesChanged"
QT_MOC_LITERAL(285, 3804, 22), // "updateTableMenuActions"
QT_MOC_LITERAL(286, 3827, 17), // "emitUpdateRequest"
QT_MOC_LITERAL(287, 3845, 19), // "slotInsertMark2Mark"
QT_MOC_LITERAL(288, 3865, 20), // "slotInsertMarkAnchor"
QT_MOC_LITERAL(289, 3886, 26), // "slotInsertMarkVariableText"
QT_MOC_LITERAL(290, 3913, 18), // "slotInsertMarkItem"
QT_MOC_LITERAL(291, 3932, 18), // "slotInsertMarkNote"
QT_MOC_LITERAL(292, 3951, 19), // "slotInsertMarkIndex"
QT_MOC_LITERAL(293, 3971, 12), // "slotEditMark"
QT_MOC_LITERAL(294, 3984, 15), // "slotUpdateMarks"
QT_MOC_LITERAL(295, 4000, 11), // "editMarkDlg"
QT_MOC_LITERAL(296, 4012, 5), // "Mark*"
QT_MOC_LITERAL(297, 4018, 3), // "mrk"
QT_MOC_LITERAL(298, 4022, 19), // "PageItem_TextFrame*"
QT_MOC_LITERAL(299, 4042, 8), // "currItem"
QT_MOC_LITERAL(300, 4051, 20), // "changePreviewQuality"
QT_MOC_LITERAL(301, 4072, 5), // "index"
QT_MOC_LITERAL(302, 4078, 14), // "enablePalettes"
QT_MOC_LITERAL(303, 4093, 15), // "ToggleFrameEdit"
QT_MOC_LITERAL(304, 4109, 11) // "NoFrameEdit"

    },
    "ScribusMainWindow\0TextStyle\0\0"
    "ParagraphStyle\0TextEffects\0UpdateRequest\0"
    "updateFlags\0changeLayers\0iconSetChange\0"
    "languageChange\0localeChange\0"
    "statusBarLanguageChange\0specialActionKeyEvent\0"
    "unicodevalue\0newView\0ToggleStickyTools\0"
    "ToggleAllGuides\0ToggleAllPalettes\0"
    "slotStoryEditor\0fromTable\0slotCharSelect\0"
    "ImageEffects\0fileCollect\0compress\0"
    "withFonts\0withProfiles\0newDirectory\0"
    "AddBookMark\0PageItem*\0ite\0DelBookMark\0"
    "BookMarkTxT\0StoreBookmarks\0"
    "setStatusBarMousePosition\0xp\0yp\0"
    "setStatusBarTextPosition\0base\0"
    "setStatusBarTextSelectedItemInfo\0"
    "setTempStatusBarText\0text\0"
    "setStatusBarInfoText\0newText\0DoFileClose\0"
    "windowsMenuAboutToShow\0extrasMenuAboutToShow\0"
    "newActWin\0QMdiSubWindow*\0w\0"
    "closeActiveWindowMasterPageEditor\0"
    "updateActiveWindowCaption\0newCaption\0"
    "windowsMenuActivated\0id\0PutScrap\0scID\0"
    "PutToInline\0buffer\0PutToPatterns\0"
    "ConvertToSymbol\0changeLayer\0"
    "setLayerMenuText\0showLayer\0"
    "slotSetCurrentPage\0pageIndex\0"
    "setCurrentPage\0p\0ManageJava\0"
    "editSelectedSymbolStart\0editSymbolStart\0"
    "temp\0editSymbolEnd\0editInlineStart\0"
    "editInlineEnd\0editMasterPagesStart\0"
    "editMasterPagesEnd\0slotFileNew\0"
    "newFileFromTemplate\0slotPageImport\0"
    "loadPage\0fileName\0Nr\0Mpa\0renamedPageName\0"
    "gotoLayer\0l\0slotGetContent\0slotGetContent2\0"
    "slotGetClipboardImage\0toogleInlineState\0"
    "slotFileAppend\0removeRecent\0fn\0"
    "fromFileWatcher\0removeRecentFromWatcher\0"
    "filename\0loadRecent\0rebuildRecentFileMenu\0"
    "rebuildRecentPasteMenu\0rebuildScrapbookMenu\0"
    "pasteRecent\0pasteFromScrapbook\0"
    "importVectorFile\0rebuildLayersList\0"
    "slotFileOpen\0loadDoc\0postLoadDoc\0"
    "slotFileSave\0slotFileSaveAs\0slotFileRevert\0"
    "SaveText\0slotFileClose\0slotFilePrint\0"
    "slotReallyPrint\0slotEndSpecialEdit\0"
    "doPrint\0PrintOptions&\0options\0QString&\0"
    "error\0slotFileQuit\0slotEditCut\0"
    "slotEditCopy\0slotEditPaste\0"
    "slotEditCopyContents\0slotEditPasteContents\0"
    "absolute\0SelectAll\0docWideSelect\0"
    "SelectAllOnLayer\0deselectAll\0ClipChange\0"
    "setCopyCutEnabled\0b\0slotHelpAbout\0"
    "slotHelpAboutPlugins\0slotHelpAboutQt\0"
    "slotHelpActionSearch\0slotHelpCheckUpdates\0"
    "slotRaiseOnlineHelp\0slotOnlineHelp\0"
    "jumpToSection\0jumpToFile\0slotOnlineHelpClosed\0"
    "slotResourceManager\0ToggleTips\0"
    "ToggleMouseTips\0slotNewPageP\0wo\0templ\0"
    "slotNewPageM\0slotNewMasterPage\0"
    "slotNewPage\0masterPageName\0mov\0"
    "duplicateToMasterPage\0deletePage\0from\0"
    "to\0deletePage2\0pg\0movePage\0copyPage\0"
    "changePageProperties\0slotZoom\0zoomFactor\0"
    "toggleMarks\0toggleBleeds\0toggleFrames\0"
    "toggleLayerMarkers\0toggleTextLinks\0"
    "toggleTextControls\0toggleColumnBorders\0"
    "toggleRulers\0toggleRulerMode\0"
    "togglePagePalette\0toggleUndoPalette\0"
    "setUndoPalette\0visible\0toggleCheckPal\0"
    "toggleImageVisibility\0toggleGrid\0"
    "toggleSnapGrid\0toggleNodeEdit\0slotSelect\0"
    "setAppModeByToggle\0isOn\0newMode\0"
    "HaveNewDoc\0HaveNewSel\0slotDocCh\0reb\0"
    "setItemFontSize\0fontSize\0setItemLanguage\0"
    "language\0slotReplaceColors\0MakeFrame\0"
    "f\0c\0double*\0vals\0duplicateItem\0"
    "duplicateItemMulti\0objectAttributes\0"
    "getImageInfo\0generateTableOfContents\0"
    "updateDocument\0setNewAlignment\0i\0"
    "setNewDirection\0setNewParStyle\0name\0"
    "setNewCharStyle\0setAlignmentValue\0"
    "setDirectionValue\0editItemsFromOutlines\0"
    "selectItemsFromOutlines\0single\0position\0"
    "selectItemFromOutlines\0cPos\0"
    "selectPagesFromOutlines\0ScPage\0"
    "doPrintPreview\0printPreview\0"
    "doOutputPreviewPDF\0outputPreviewPDF\0"
    "doOutputPreviewPS\0outputPreviewPS\0"
    "SaveAsEps\0reallySaveAsEps\0SaveAsPDF\0"
    "doSaveAsPDF\0setMainWindowActive\0"
    "setItemEffects\0h\0setStyleEffects\0s\0"
    "setItemTypeStyle\0slotElemRead\0Name\0x\0"
    "y\0art\0loca\0ScribusDoc*\0docc\0ScribusView*\0"
    "vie\0slotChangeUnit\0draw\0ApplyMasterPage\0"
    "Apply_MasterPage\0pageName\0pageNumber\0"
    "GroupObj\0showLockDia\0UnGroupObj\0"
    "AdjustGroupObj\0StatusPic\0ModifyAnnot\0"
    "toggleGuides\0toggleBase\0toggleSnapGuides\0"
    "toggleSnapElements\0SetSnapElements\0"
    "EditTabs\0SearchText\0callImageEditor\0"
    "docCheckToggle\0scanDocument\0setUndoMode\0"
    "isObjectSpecific\0insertSampleText\0"
    "updateItemLayerList\0updateColorLists\0"
    "slotPrefsOrg\0slotDocSetup\0slotInsertFrame\0"
    "slotItemTransform\0manageColorsAndFills\0"
    "styleMgr\0StyleManager*\0"
    "updateGUIAfterPagesChanged\0"
    "updateTableMenuActions\0emitUpdateRequest\0"
    "slotInsertMark2Mark\0slotInsertMarkAnchor\0"
    "slotInsertMarkVariableText\0"
    "slotInsertMarkItem\0slotInsertMarkNote\0"
    "slotInsertMarkIndex\0slotEditMark\0"
    "slotUpdateMarks\0editMarkDlg\0Mark*\0mrk\0"
    "PageItem_TextFrame*\0currItem\0"
    "changePreviewQuality\0index\0enablePalettes\0"
    "ToggleFrameEdit\0NoFrameEdit"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ScribusMainWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     238,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       4,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1, 1204,    2, 0x06 /* Public */,
       4,    1, 1207,    2, 0x06 /* Public */,
       5,    1, 1210,    2, 0x06 /* Public */,
       7,    1, 1213,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       8,    0, 1216,    2, 0x0a /* Public */,
       9,    0, 1217,    2, 0x0a /* Public */,
      10,    0, 1218,    2, 0x0a /* Public */,
      11,    0, 1219,    2, 0x0a /* Public */,
      12,    1, 1220,    2, 0x0a /* Public */,
      14,    0, 1223,    2, 0x0a /* Public */,
      15,    0, 1224,    2, 0x0a /* Public */,
      16,    0, 1225,    2, 0x0a /* Public */,
      17,    0, 1226,    2, 0x0a /* Public */,
      18,    1, 1227,    2, 0x0a /* Public */,
      20,    0, 1230,    2, 0x0a /* Public */,
      21,    0, 1231,    2, 0x0a /* Public */,
      22,    4, 1232,    2, 0x0a /* Public */,
      22,    3, 1241,    2, 0x2a /* Public | MethodCloned */,
      22,    2, 1248,    2, 0x2a /* Public | MethodCloned */,
      22,    1, 1253,    2, 0x2a /* Public | MethodCloned */,
      22,    0, 1256,    2, 0x2a /* Public | MethodCloned */,
      27,    1, 1257,    2, 0x0a /* Public */,
      30,    1, 1260,    2, 0x0a /* Public */,
      31,    1, 1263,    2, 0x0a /* Public */,
      32,    0, 1266,    2, 0x0a /* Public */,
      33,    2, 1267,    2, 0x0a /* Public */,
      36,    2, 1272,    2, 0x0a /* Public */,
      38,    0, 1277,    2, 0x0a /* Public */,
      39,    1, 1278,    2, 0x0a /* Public */,
      41,    1, 1281,    2, 0x0a /* Public */,
      43,    0, 1284,    2, 0x0a /* Public */,
      44,    0, 1285,    2, 0x0a /* Public */,
      45,    0, 1286,    2, 0x0a /* Public */,
      46,    1, 1287,    2, 0x0a /* Public */,
      49,    0, 1290,    2, 0x0a /* Public */,
      50,    1, 1291,    2, 0x0a /* Public */,
      52,    1, 1294,    2, 0x0a /* Public */,
      54,    1, 1297,    2, 0x0a /* Public */,
      56,    1, 1300,    2, 0x0a /* Public */,
      56,    0, 1303,    2, 0x0a /* Public */,
      58,    0, 1304,    2, 0x0a /* Public */,
      59,    0, 1305,    2, 0x0a /* Public */,
      60,    1, 1306,    2, 0x0a /* Public */,
      61,    1, 1309,    2, 0x0a /* Public */,
      62,    0, 1312,    2, 0x0a /* Public */,
      63,    1, 1313,    2, 0x0a /* Public */,
      65,    1, 1316,    2, 0x0a /* Public */,
      67,    0, 1319,    2, 0x0a /* Public */,
      68,    0, 1320,    2, 0x0a /* Public */,
      69,    1, 1321,    2, 0x0a /* Public */,
      71,    0, 1324,    2, 0x0a /* Public */,
      72,    1, 1325,    2, 0x0a /* Public */,
      73,    0, 1328,    2, 0x0a /* Public */,
      74,    1, 1329,    2, 0x0a /* Public */,
      74,    0, 1332,    2, 0x2a /* Public | MethodCloned */,
      75,    0, 1333,    2, 0x0a /* Public */,
      76,    0, 1334,    2, 0x0a /* Public */,
      77,    0, 1335,    2, 0x0a /* Public */,
      78,    0, 1336,    2, 0x0a /* Public */,
      79,    4, 1337,    2, 0x0a /* Public */,
      79,    3, 1346,    2, 0x2a /* Public | MethodCloned */,
      84,    1, 1353,    2, 0x0a /* Public */,
      86,    0, 1356,    2, 0x0a /* Public */,
      87,    0, 1357,    2, 0x0a /* Public */,
      88,    0, 1358,    2, 0x0a /* Public */,
      89,    0, 1359,    2, 0x0a /* Public */,
      90,    0, 1360,    2, 0x0a /* Public */,
      91,    2, 1361,    2, 0x0a /* Public */,
      91,    1, 1366,    2, 0x2a /* Public | MethodCloned */,
      94,    1, 1369,    2, 0x0a /* Public */,
      96,    1, 1372,    2, 0x0a /* Public */,
      97,    0, 1375,    2, 0x0a /* Public */,
      98,    0, 1376,    2, 0x0a /* Public */,
      99,    0, 1377,    2, 0x0a /* Public */,
     100,    1, 1378,    2, 0x0a /* Public */,
     101,    1, 1381,    2, 0x0a /* Public */,
     102,    0, 1384,    2, 0x0a /* Public */,
     103,    0, 1385,    2, 0x0a /* Public */,
     104,    0, 1386,    2, 0x0a /* Public */,
     105,    1, 1387,    2, 0x0a /* Public */,
     106,    0, 1390,    2, 0x0a /* Public */,
     107,    0, 1391,    2, 0x0a /* Public */,
     108,    0, 1392,    2, 0x0a /* Public */,
     109,    0, 1393,    2, 0x0a /* Public */,
     110,    0, 1394,    2, 0x0a /* Public */,
     111,    0, 1395,    2, 0x0a /* Public */,
     112,    0, 1396,    2, 0x0a /* Public */,
     113,    0, 1397,    2, 0x0a /* Public */,
     114,    0, 1398,    2, 0x0a /* Public */,
     115,    2, 1399,    2, 0x0a /* Public */,
     120,    0, 1404,    2, 0x0a /* Public */,
     121,    0, 1405,    2, 0x0a /* Public */,
     122,    0, 1406,    2, 0x0a /* Public */,
     123,    0, 1407,    2, 0x0a /* Public */,
     124,    0, 1408,    2, 0x0a /* Public */,
     125,    1, 1409,    2, 0x0a /* Public */,
     125,    0, 1412,    2, 0x2a /* Public | MethodCloned */,
     127,    1, 1413,    2, 0x0a /* Public */,
     127,    0, 1416,    2, 0x2a /* Public | MethodCloned */,
     129,    0, 1417,    2, 0x0a /* Public */,
     130,    0, 1418,    2, 0x0a /* Public */,
     131,    0, 1419,    2, 0x0a /* Public */,
     132,    1, 1420,    2, 0x0a /* Public */,
     134,    0, 1423,    2, 0x0a /* Public */,
     135,    0, 1424,    2, 0x0a /* Public */,
     136,    0, 1425,    2, 0x0a /* Public */,
     137,    0, 1426,    2, 0x0a /* Public */,
     138,    0, 1427,    2, 0x0a /* Public */,
     139,    0, 1428,    2, 0x0a /* Public */,
     140,    2, 1429,    2, 0x0a /* Public */,
     140,    1, 1434,    2, 0x2a /* Public | MethodCloned */,
     140,    0, 1437,    2, 0x2a /* Public | MethodCloned */,
     143,    0, 1438,    2, 0x0a /* Public */,
     144,    0, 1439,    2, 0x0a /* Public */,
     145,    0, 1440,    2, 0x0a /* Public */,
     146,    0, 1441,    2, 0x0a /* Public */,
     147,    2, 1442,    2, 0x0a /* Public */,
     150,    0, 1447,    2, 0x0a /* Public */,
     151,    2, 1448,    2, 0x0a /* Public */,
     152,    3, 1453,    2, 0x0a /* Public */,
     152,    2, 1460,    2, 0x2a /* Public | MethodCloned */,
     152,    1, 1465,    2, 0x2a /* Public | MethodCloned */,
     155,    0, 1468,    2, 0x0a /* Public */,
     156,    0, 1469,    2, 0x0a /* Public */,
     156,    2, 1470,    2, 0x0a /* Public */,
     159,    1, 1475,    2, 0x0a /* Public */,
     161,    0, 1478,    2, 0x0a /* Public */,
     162,    0, 1479,    2, 0x0a /* Public */,
     163,    0, 1480,    2, 0x0a /* Public */,
     164,    1, 1481,    2, 0x0a /* Public */,
     166,    0, 1484,    2, 0x0a /* Public */,
     167,    0, 1485,    2, 0x0a /* Public */,
     168,    0, 1486,    2, 0x0a /* Public */,
     169,    0, 1487,    2, 0x0a /* Public */,
     170,    0, 1488,    2, 0x0a /* Public */,
     171,    0, 1489,    2, 0x0a /* Public */,
     172,    0, 1490,    2, 0x0a /* Public */,
     173,    0, 1491,    2, 0x0a /* Public */,
     174,    0, 1492,    2, 0x0a /* Public */,
     175,    0, 1493,    2, 0x0a /* Public */,
     176,    0, 1494,    2, 0x0a /* Public */,
     177,    1, 1495,    2, 0x0a /* Public */,
     179,    0, 1498,    2, 0x0a /* Public */,
     180,    0, 1499,    2, 0x0a /* Public */,
     181,    0, 1500,    2, 0x0a /* Public */,
     182,    0, 1501,    2, 0x0a /* Public */,
     183,    0, 1502,    2, 0x0a /* Public */,
     184,    0, 1503,    2, 0x0a /* Public */,
     185,    2, 1504,    2, 0x0a /* Public */,
     188,    0, 1509,    2, 0x0a /* Public */,
     189,    0, 1510,    2, 0x0a /* Public */,
     190,    1, 1511,    2, 0x0a /* Public */,
     190,    0, 1514,    2, 0x2a /* Public | MethodCloned */,
     192,    1, 1515,    2, 0x0a /* Public */,
     194,    1, 1518,    2, 0x0a /* Public */,
     196,    0, 1521,    2, 0x0a /* Public */,
     197,    3, 1522,    2, 0x0a /* Public */,
     202,    0, 1529,    2, 0x0a /* Public */,
     203,    0, 1530,    2, 0x0a /* Public */,
     204,    0, 1531,    2, 0x0a /* Public */,
     205,    0, 1532,    2, 0x0a /* Public */,
     206,    0, 1533,    2, 0x0a /* Public */,
     207,    0, 1534,    2, 0x0a /* Public */,
     208,    1, 1535,    2, 0x0a /* Public */,
     210,    1, 1538,    2, 0x0a /* Public */,
     211,    1, 1541,    2, 0x0a /* Public */,
     213,    1, 1544,    2, 0x0a /* Public */,
     214,    1, 1547,    2, 0x0a /* Public */,
     215,    1, 1550,    2, 0x0a /* Public */,
     216,    1, 1553,    2, 0x0a /* Public */,
     217,    3, 1556,    2, 0x0a /* Public */,
     217,    2, 1563,    2, 0x2a /* Public | MethodCloned */,
     217,    1, 1568,    2, 0x2a /* Public | MethodCloned */,
     220,    3, 1571,    2, 0x0a /* Public */,
     222,    1, 1578,    2, 0x0a /* Public */,
     224,    0, 1581,    2, 0x0a /* Public */,
     225,    0, 1582,    2, 0x0a /* Public */,
     226,    0, 1583,    2, 0x0a /* Public */,
     227,    0, 1584,    2, 0x0a /* Public */,
     228,    0, 1585,    2, 0x0a /* Public */,
     229,    0, 1586,    2, 0x0a /* Public */,
     230,    0, 1587,    2, 0x0a /* Public */,
     231,    0, 1588,    2, 0x0a /* Public */,
     232,    0, 1589,    2, 0x0a /* Public */,
     233,    0, 1590,    2, 0x0a /* Public */,
     234,    0, 1591,    2, 0x0a /* Public */,
     235,    1, 1592,    2, 0x0a /* Public */,
     237,    1, 1595,    2, 0x0a /* Public */,
     239,    1, 1598,    2, 0x0a /* Public */,
     240,    7, 1601,    2, 0x0a /* Public */,
     250,    2, 1616,    2, 0x0a /* Public */,
     250,    1, 1621,    2, 0x2a /* Public | MethodCloned */,
     252,    0, 1624,    2, 0x0a /* Public */,
     253,    3, 1625,    2, 0x0a /* Public */,
     253,    2, 1632,    2, 0x2a /* Public | MethodCloned */,
     256,    1, 1637,    2, 0x0a /* Public */,
     256,    0, 1640,    2, 0x2a /* Public | MethodCloned */,
     258,    0, 1641,    2, 0x0a /* Public */,
     259,    0, 1642,    2, 0x0a /* Public */,
     260,    0, 1643,    2, 0x0a /* Public */,
     261,    0, 1644,    2, 0x0a /* Public */,
     262,    0, 1645,    2, 0x0a /* Public */,
     263,    0, 1646,    2, 0x0a /* Public */,
     264,    0, 1647,    2, 0x0a /* Public */,
     265,    0, 1648,    2, 0x0a /* Public */,
     266,    1, 1649,    2, 0x0a /* Public */,
     267,    0, 1652,    2, 0x0a /* Public */,
     268,    0, 1653,    2, 0x0a /* Public */,
     269,    0, 1654,    2, 0x0a /* Public */,
     270,    1, 1655,    2, 0x0a /* Public */,
     271,    0, 1658,    2, 0x0a /* Public */,
     272,    1, 1659,    2, 0x0a /* Public */,
     274,    0, 1662,    2, 0x0a /* Public */,
     275,    0, 1663,    2, 0x0a /* Public */,
     276,    0, 1664,    2, 0x0a /* Public */,
     277,    0, 1665,    2, 0x0a /* Public */,
     278,    0, 1666,    2, 0x0a /* Public */,
     279,    0, 1667,    2, 0x0a /* Public */,
     280,    0, 1668,    2, 0x0a /* Public */,
     281,    0, 1669,    2, 0x0a /* Public */,
     282,    0, 1670,    2, 0x0a /* Public */,
     284,    0, 1671,    2, 0x0a /* Public */,
     285,    0, 1672,    2, 0x0a /* Public */,
     286,    1, 1673,    2, 0x0a /* Public */,
     287,    0, 1676,    2, 0x0a /* Public */,
     288,    0, 1677,    2, 0x0a /* Public */,
     289,    0, 1678,    2, 0x0a /* Public */,
     290,    0, 1679,    2, 0x0a /* Public */,
     291,    0, 1680,    2, 0x0a /* Public */,
     292,    0, 1681,    2, 0x0a /* Public */,
     293,    0, 1682,    2, 0x0a /* Public */,
     294,    0, 1683,    2, 0x0a /* Public */,
     295,    2, 1684,    2, 0x0a /* Public */,
     295,    1, 1689,    2, 0x2a /* Public | MethodCloned */,
     300,    1, 1692,    2, 0x0a /* Public */,
     302,    1, 1695,    2, 0x0a /* Public */,
     303,    0, 1698,    2, 0x0a /* Public */,
     304,    0, 1699,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   19,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::QString, QMetaType::Bool, QMetaType::Bool, QMetaType::Bool, QMetaType::QString,   23,   24,   25,   26,
    QMetaType::QString, QMetaType::Bool, QMetaType::Bool, QMetaType::Bool,   23,   24,   25,
    QMetaType::QString, QMetaType::Bool, QMetaType::Bool,   23,   24,
    QMetaType::QString, QMetaType::Bool,   23,
    QMetaType::QString,
    QMetaType::Void, 0x80000000 | 28,   29,
    QMetaType::Void, 0x80000000 | 28,   29,
    QMetaType::Void, 0x80000000 | 28,   29,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,   34,   35,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,   37,   34,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   40,
    QMetaType::Void, QMetaType::QString,   42,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 47,   48,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   51,
    QMetaType::Void, QMetaType::Int,   53,
    QMetaType::Void, QMetaType::Int,   55,
    QMetaType::Void, QMetaType::QString,   57,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   64,
    QMetaType::Void, QMetaType::Int,   66,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   70,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   53,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   70,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Bool, QMetaType::QString, QMetaType::Int, QMetaType::Bool, QMetaType::QString,   80,   81,   82,   83,
    QMetaType::Bool, QMetaType::QString, QMetaType::Int, QMetaType::Bool,   80,   81,   82,
    QMetaType::Void, QMetaType::Int,   85,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Bool,   92,   93,
    QMetaType::Void, QMetaType::QString,   92,
    QMetaType::Void, QMetaType::QString,   95,
    QMetaType::Void, QMetaType::QString,   95,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   92,
    QMetaType::Void, QMetaType::QString,   92,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Bool, QMetaType::QString,    2,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool, 0x80000000 | 116, 0x80000000 | 118,  117,  119,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,  126,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  128,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  133,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::QString,  141,  142,
    QMetaType::Void, QMetaType::QString,  141,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,  148,  149,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   48,    2,
    QMetaType::Void, QMetaType::Int, QMetaType::QString, QMetaType::Bool,   48,  153,  154,
    QMetaType::Void, QMetaType::Int, QMetaType::QString,   48,  153,
    QMetaType::Void, QMetaType::Int,   48,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,  157,  158,
    QMetaType::Void, QMetaType::Int,  160,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,  165,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  178,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool, QMetaType::Int,  186,  187,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  191,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,  193,
    QMetaType::Void, QMetaType::QString,  195,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, 0x80000000 | 200,  198,  199,  201,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,  209,
    QMetaType::Void, QMetaType::Int,  209,
    QMetaType::Void, QMetaType::QString,  212,
    QMetaType::Void, QMetaType::QString,  212,
    QMetaType::Void, QMetaType::Int,  209,
    QMetaType::Void, QMetaType::Int,  209,
    QMetaType::Void, 0x80000000 | 28,   29,
    QMetaType::Void, 0x80000000 | 28, QMetaType::Bool, QMetaType::Int,   29,  218,  219,
    QMetaType::Void, 0x80000000 | 28, QMetaType::Bool,   29,  218,
    QMetaType::Void, 0x80000000 | 28,   29,
    QMetaType::Void, 0x80000000 | 28, QMetaType::Bool, QMetaType::Int,   29,  218,  221,
    QMetaType::Void, QMetaType::Int,  223,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,  236,
    QMetaType::Void, QMetaType::Int,  238,
    QMetaType::Void, QMetaType::Int,   53,
    QMetaType::Void, QMetaType::QString, QMetaType::Double, QMetaType::Double, QMetaType::Bool, QMetaType::Bool, 0x80000000 | 246, 0x80000000 | 248,  241,  242,  243,  244,  245,  247,  249,
    QMetaType::Void, QMetaType::Int, QMetaType::Bool,  244,  251,
    QMetaType::Void, QMetaType::Int,  244,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Int, QMetaType::Bool,  254,  255,  191,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,  254,  255,
    QMetaType::Void, QMetaType::Bool,  257,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  133,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  178,
    QMetaType::Bool,
    QMetaType::Void, QMetaType::Bool,  273,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    0x80000000 | 283,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Bool, 0x80000000 | 296, 0x80000000 | 298,  297,  299,
    QMetaType::Bool, 0x80000000 | 296,  297,
    QMetaType::Void, QMetaType::Int,  301,
    QMetaType::Void, QMetaType::Bool,  133,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ScribusMainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ScribusMainWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->TextStyle((*reinterpret_cast< const ParagraphStyle(*)>(_a[1]))); break;
        case 1: _t->TextEffects((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->UpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 3: _t->changeLayers((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->iconSetChange(); break;
        case 5: _t->languageChange(); break;
        case 6: _t->localeChange(); break;
        case 7: _t->statusBarLanguageChange(); break;
        case 8: _t->specialActionKeyEvent((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 9: _t->newView(); break;
        case 10: _t->ToggleStickyTools(); break;
        case 11: _t->ToggleAllGuides(); break;
        case 12: _t->ToggleAllPalettes(); break;
        case 13: _t->slotStoryEditor((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 14: _t->slotCharSelect(); break;
        case 15: _t->ImageEffects(); break;
        case 16: { QString _r = _t->fileCollect((*reinterpret_cast< const bool(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2])),(*reinterpret_cast< const bool(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 17: { QString _r = _t->fileCollect((*reinterpret_cast< const bool(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2])),(*reinterpret_cast< const bool(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 18: { QString _r = _t->fileCollect((*reinterpret_cast< const bool(*)>(_a[1])),(*reinterpret_cast< const bool(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 19: { QString _r = _t->fileCollect((*reinterpret_cast< const bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 20: { QString _r = _t->fileCollect();
            if (_a[0]) *reinterpret_cast< QString*>(_a[0]) = std::move(_r); }  break;
        case 21: _t->AddBookMark((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 22: _t->DelBookMark((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 23: _t->BookMarkTxT((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 24: _t->StoreBookmarks(); break;
        case 25: _t->setStatusBarMousePosition((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 26: _t->setStatusBarTextPosition((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 27: _t->setStatusBarTextSelectedItemInfo(); break;
        case 28: _t->setTempStatusBarText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 29: _t->setStatusBarInfoText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 30: { bool _r = _t->DoFileClose();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 31: _t->windowsMenuAboutToShow(); break;
        case 32: _t->extrasMenuAboutToShow(); break;
        case 33: _t->newActWin((*reinterpret_cast< QMdiSubWindow*(*)>(_a[1]))); break;
        case 34: _t->closeActiveWindowMasterPageEditor(); break;
        case 35: _t->updateActiveWindowCaption((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 36: _t->windowsMenuActivated((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 37: _t->PutScrap((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 38: _t->PutToInline((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 39: _t->PutToInline(); break;
        case 40: _t->PutToPatterns(); break;
        case 41: _t->ConvertToSymbol(); break;
        case 42: _t->changeLayer((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 43: _t->setLayerMenuText((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 44: _t->showLayer(); break;
        case 45: _t->slotSetCurrentPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 46: _t->setCurrentPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 47: _t->ManageJava(); break;
        case 48: _t->editSelectedSymbolStart(); break;
        case 49: _t->editSymbolStart((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 50: _t->editSymbolEnd(); break;
        case 51: _t->editInlineStart((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 52: _t->editInlineEnd(); break;
        case 53: _t->editMasterPagesStart((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 54: _t->editMasterPagesStart(); break;
        case 55: _t->editMasterPagesEnd(); break;
        case 56: { bool _r = _t->slotFileNew();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 57: _t->newFileFromTemplate(); break;
        case 58: { bool _r = _t->slotPageImport();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 59: { bool _r = _t->loadPage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< const QString(*)>(_a[4])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 60: { bool _r = _t->loadPage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 61: _t->gotoLayer((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 62: _t->slotGetContent(); break;
        case 63: _t->slotGetContent2(); break;
        case 64: _t->slotGetClipboardImage(); break;
        case 65: _t->toogleInlineState(); break;
        case 66: _t->slotFileAppend(); break;
        case 67: _t->removeRecent((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 68: _t->removeRecent((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 69: _t->removeRecentFromWatcher((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 70: _t->loadRecent((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 71: _t->rebuildRecentFileMenu(); break;
        case 72: _t->rebuildRecentPasteMenu(); break;
        case 73: _t->rebuildScrapbookMenu(); break;
        case 74: _t->pasteRecent((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 75: _t->pasteFromScrapbook((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 76: _t->importVectorFile(); break;
        case 77: _t->rebuildLayersList(); break;
        case 78: { bool _r = _t->slotFileOpen();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 79: { bool _r = _t->loadDoc((*reinterpret_cast< const QString(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 80: { bool _r = _t->postLoadDoc();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 81: { bool _r = _t->slotFileSave();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 82: { bool _r = _t->slotFileSaveAs();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 83: _t->slotFileRevert(); break;
        case 84: _t->SaveText(); break;
        case 85: { bool _r = _t->slotFileClose();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 86: _t->slotFilePrint(); break;
        case 87: _t->slotReallyPrint(); break;
        case 88: _t->slotEndSpecialEdit(); break;
        case 89: { bool _r = _t->doPrint((*reinterpret_cast< PrintOptions(*)>(_a[1])),(*reinterpret_cast< QString(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 90: _t->slotFileQuit(); break;
        case 91: _t->slotEditCut(); break;
        case 92: _t->slotEditCopy(); break;
        case 93: _t->slotEditPaste(); break;
        case 94: _t->slotEditCopyContents(); break;
        case 95: _t->slotEditPasteContents((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 96: _t->slotEditPasteContents(); break;
        case 97: _t->SelectAll((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 98: _t->SelectAll(); break;
        case 99: _t->SelectAllOnLayer(); break;
        case 100: _t->deselectAll(); break;
        case 101: _t->ClipChange(); break;
        case 102: _t->setCopyCutEnabled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 103: _t->slotHelpAbout(); break;
        case 104: _t->slotHelpAboutPlugins(); break;
        case 105: _t->slotHelpAboutQt(); break;
        case 106: _t->slotHelpActionSearch(); break;
        case 107: _t->slotHelpCheckUpdates(); break;
        case 108: _t->slotRaiseOnlineHelp(); break;
        case 109: _t->slotOnlineHelp((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 110: _t->slotOnlineHelp((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 111: _t->slotOnlineHelp(); break;
        case 112: _t->slotOnlineHelpClosed(); break;
        case 113: _t->slotResourceManager(); break;
        case 114: _t->ToggleTips(); break;
        case 115: _t->ToggleMouseTips(); break;
        case 116: _t->slotNewPageP((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 117: _t->slotNewPageM(); break;
        case 118: _t->slotNewMasterPage((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 119: _t->slotNewPage((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 120: _t->slotNewPage((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2]))); break;
        case 121: _t->slotNewPage((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 122: _t->duplicateToMasterPage(); break;
        case 123: _t->deletePage(); break;
        case 124: _t->deletePage((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 125: _t->deletePage2((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 126: _t->movePage(); break;
        case 127: _t->copyPage(); break;
        case 128: _t->changePageProperties(); break;
        case 129: _t->slotZoom((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 130: _t->toggleMarks(); break;
        case 131: _t->toggleBleeds(); break;
        case 132: _t->toggleFrames(); break;
        case 133: _t->toggleLayerMarkers(); break;
        case 134: _t->toggleTextLinks(); break;
        case 135: _t->toggleTextControls(); break;
        case 136: _t->toggleColumnBorders(); break;
        case 137: _t->toggleRulers(); break;
        case 138: _t->toggleRulerMode(); break;
        case 139: _t->togglePagePalette(); break;
        case 140: _t->toggleUndoPalette(); break;
        case 141: _t->setUndoPalette((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 142: _t->toggleCheckPal(); break;
        case 143: _t->toggleImageVisibility(); break;
        case 144: _t->toggleGrid(); break;
        case 145: _t->toggleSnapGrid(); break;
        case 146: _t->toggleNodeEdit(); break;
        case 147: _t->slotSelect(); break;
        case 148: _t->setAppModeByToggle((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 149: _t->HaveNewDoc(); break;
        case 150: _t->HaveNewSel(); break;
        case 151: _t->slotDocCh((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 152: _t->slotDocCh(); break;
        case 153: _t->setItemFontSize((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 154: _t->setItemLanguage((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 155: _t->slotReplaceColors(); break;
        case 156: _t->MakeFrame((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< double*(*)>(_a[3]))); break;
        case 157: _t->duplicateItem(); break;
        case 158: _t->duplicateItemMulti(); break;
        case 159: _t->objectAttributes(); break;
        case 160: _t->getImageInfo(); break;
        case 161: _t->generateTableOfContents(); break;
        case 162: _t->updateDocument(); break;
        case 163: _t->setNewAlignment((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 164: _t->setNewDirection((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 165: _t->setNewParStyle((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 166: _t->setNewCharStyle((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 167: _t->setAlignmentValue((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 168: _t->setDirectionValue((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 169: _t->editItemsFromOutlines((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 170: _t->selectItemsFromOutlines((*reinterpret_cast< PageItem*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 171: _t->selectItemsFromOutlines((*reinterpret_cast< PageItem*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 172: _t->selectItemsFromOutlines((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 173: _t->selectItemFromOutlines((*reinterpret_cast< PageItem*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 174: _t->selectPagesFromOutlines((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 175: _t->doPrintPreview(); break;
        case 176: _t->printPreview(); break;
        case 177: _t->doOutputPreviewPDF(); break;
        case 178: _t->outputPreviewPDF(); break;
        case 179: _t->doOutputPreviewPS(); break;
        case 180: _t->outputPreviewPS(); break;
        case 181: _t->SaveAsEps(); break;
        case 182: _t->reallySaveAsEps(); break;
        case 183: _t->SaveAsPDF(); break;
        case 184: _t->doSaveAsPDF(); break;
        case 185: _t->setMainWindowActive(); break;
        case 186: _t->setItemEffects((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 187: _t->setStyleEffects((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 188: _t->setItemTypeStyle((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 189: _t->slotElemRead((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4])),(*reinterpret_cast< bool(*)>(_a[5])),(*reinterpret_cast< ScribusDoc*(*)>(_a[6])),(*reinterpret_cast< ScribusView*(*)>(_a[7]))); break;
        case 190: _t->slotChangeUnit((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 191: _t->slotChangeUnit((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 192: _t->ApplyMasterPage(); break;
        case 193: _t->Apply_MasterPage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 194: _t->Apply_MasterPage((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 195: _t->GroupObj((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 196: _t->GroupObj(); break;
        case 197: _t->UnGroupObj(); break;
        case 198: _t->AdjustGroupObj(); break;
        case 199: _t->StatusPic(); break;
        case 200: _t->ModifyAnnot(); break;
        case 201: _t->toggleGuides(); break;
        case 202: _t->toggleBase(); break;
        case 203: _t->toggleSnapGuides(); break;
        case 204: _t->toggleSnapElements(); break;
        case 205: _t->SetSnapElements((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 206: _t->EditTabs(); break;
        case 207: _t->SearchText(); break;
        case 208: _t->callImageEditor(); break;
        case 209: _t->docCheckToggle((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 210: { bool _r = _t->scanDocument();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 211: _t->setUndoMode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 212: _t->insertSampleText(); break;
        case 213: _t->updateItemLayerList(); break;
        case 214: _t->updateColorLists(); break;
        case 215: _t->slotPrefsOrg(); break;
        case 216: _t->slotDocSetup(); break;
        case 217: _t->slotInsertFrame(); break;
        case 218: _t->slotItemTransform(); break;
        case 219: _t->manageColorsAndFills(); break;
        case 220: { StyleManager* _r = _t->styleMgr();
            if (_a[0]) *reinterpret_cast< StyleManager**>(_a[0]) = std::move(_r); }  break;
        case 221: _t->updateGUIAfterPagesChanged(); break;
        case 222: _t->updateTableMenuActions(); break;
        case 223: _t->emitUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 224: _t->slotInsertMark2Mark(); break;
        case 225: _t->slotInsertMarkAnchor(); break;
        case 226: _t->slotInsertMarkVariableText(); break;
        case 227: _t->slotInsertMarkItem(); break;
        case 228: _t->slotInsertMarkNote(); break;
        case 229: _t->slotInsertMarkIndex(); break;
        case 230: _t->slotEditMark(); break;
        case 231: _t->slotUpdateMarks(); break;
        case 232: { bool _r = _t->editMarkDlg((*reinterpret_cast< Mark*(*)>(_a[1])),(*reinterpret_cast< PageItem_TextFrame*(*)>(_a[2])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 233: { bool _r = _t->editMarkDlg((*reinterpret_cast< Mark*(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 234: _t->changePreviewQuality((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 235: _t->enablePalettes((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 236: _t->ToggleFrameEdit(); break;
        case 237: _t->NoFrameEdit(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 21:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 169:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 170:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 171:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 172:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 173:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 189:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 5:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< ScribusDoc* >(); break;
            }
            break;
        case 232:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 1:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem_TextFrame* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ScribusMainWindow::*)(const ParagraphStyle & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusMainWindow::TextStyle)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ScribusMainWindow::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusMainWindow::TextEffects)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ScribusMainWindow::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusMainWindow::UpdateRequest)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ScribusMainWindow::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusMainWindow::changeLayers)) {
                *result = 3;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ScribusMainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_ScribusMainWindow.data,
    qt_meta_data_ScribusMainWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ScribusMainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ScribusMainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ScribusMainWindow.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "UndoObject"))
        return static_cast< UndoObject*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int ScribusMainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 238)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 238;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 238)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 238;
    }
    return _id;
}

// SIGNAL 0
void ScribusMainWindow::TextStyle(const ParagraphStyle & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ScribusMainWindow::TextEffects(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ScribusMainWindow::UpdateRequest(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void ScribusMainWindow::changeLayers(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
