/********************************************************************************
** Form generated from reading UI file 'preferencesdialogbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFERENCESDIALOGBASE_H
#define UI_PREFERENCESDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_PreferencesDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *frameHorizontalLayout;
    QListWidget *preferencesTypeList;
    QVBoxLayout *verticalLayout_5;
    QStackedWidget *prefsStackWidget;
    QWidget *page_2;
    QHBoxLayout *buttonHorizontalLayout;
    QPushButton *exportButton;
    QPushButton *defaultsButton;
    QSpacerItem *buttonHorizontalSpacer;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *PreferencesDialog)
    {
        if (PreferencesDialog->objectName().isEmpty())
            PreferencesDialog->setObjectName(QString::fromUtf8("PreferencesDialog"));
        PreferencesDialog->resize(919, 586);
        verticalLayout = new QVBoxLayout(PreferencesDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        frameHorizontalLayout = new QHBoxLayout();
        frameHorizontalLayout->setSpacing(9);
        frameHorizontalLayout->setObjectName(QString::fromUtf8("frameHorizontalLayout"));
        preferencesTypeList = new QListWidget(PreferencesDialog);
        preferencesTypeList->setObjectName(QString::fromUtf8("preferencesTypeList"));
        preferencesTypeList->setMinimumSize(QSize(0, 0));
        preferencesTypeList->setMaximumSize(QSize(200, 16777215));
        preferencesTypeList->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
        preferencesTypeList->setEditTriggers(QAbstractItemView::NoEditTriggers);
        preferencesTypeList->setProperty("showDropIndicator", QVariant(false));
        preferencesTypeList->setDragDropMode(QAbstractItemView::NoDragDrop);
        preferencesTypeList->setAlternatingRowColors(true);
        preferencesTypeList->setIconSize(QSize(16, 16));
        preferencesTypeList->setResizeMode(QListView::Adjust);
        preferencesTypeList->setSpacing(1);
        preferencesTypeList->setViewMode(QListView::ListMode);
        preferencesTypeList->setWordWrap(true);

        frameHorizontalLayout->addWidget(preferencesTypeList);

        verticalLayout_5 = new QVBoxLayout();
        verticalLayout_5->setSpacing(9);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        prefsStackWidget = new QStackedWidget(PreferencesDialog);
        prefsStackWidget->setObjectName(QString::fromUtf8("prefsStackWidget"));
        page_2 = new QWidget();
        page_2->setObjectName(QString::fromUtf8("page_2"));
        prefsStackWidget->addWidget(page_2);

        verticalLayout_5->addWidget(prefsStackWidget);

        buttonHorizontalLayout = new QHBoxLayout();
        buttonHorizontalLayout->setObjectName(QString::fromUtf8("buttonHorizontalLayout"));
        buttonHorizontalLayout->setContentsMargins(0, -1, 0, -1);
        exportButton = new QPushButton(PreferencesDialog);
        exportButton->setObjectName(QString::fromUtf8("exportButton"));
        exportButton->setAutoDefault(false);

        buttonHorizontalLayout->addWidget(exportButton);

        defaultsButton = new QPushButton(PreferencesDialog);
        defaultsButton->setObjectName(QString::fromUtf8("defaultsButton"));
        defaultsButton->setAutoDefault(false);

        buttonHorizontalLayout->addWidget(defaultsButton);

        buttonHorizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        buttonHorizontalLayout->addItem(buttonHorizontalSpacer);

        buttonBox = new QDialogButtonBox(PreferencesDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        buttonHorizontalLayout->addWidget(buttonBox);


        verticalLayout_5->addLayout(buttonHorizontalLayout);


        frameHorizontalLayout->addLayout(verticalLayout_5);


        verticalLayout->addLayout(frameHorizontalLayout);


        retranslateUi(PreferencesDialog);

        QMetaObject::connectSlotsByName(PreferencesDialog);
    } // setupUi

    void retranslateUi(QDialog *PreferencesDialog)
    {
        PreferencesDialog->setWindowTitle(QCoreApplication::translate("PreferencesDialog", "Preferences", nullptr));
        exportButton->setText(QCoreApplication::translate("PreferencesDialog", "Export...", nullptr));
        defaultsButton->setText(QCoreApplication::translate("PreferencesDialog", "Defaults", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PreferencesDialog: public Ui_PreferencesDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFERENCESDIALOGBASE_H
