/********************************************************************************
** Form generated from reading UI file 'prefs_shortwordsbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_SHORTWORDSBASE_H
#define UI_PREFS_SHORTWORDSBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Prefs_ShortWords
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *titleLabel;
    QFrame *line;
    QTextEdit *cfgEdit;
    QHBoxLayout *horizontalLayout;
    QLabel *messageLabel;
    QSpacerItem *horizontalSpacer;
    QPushButton *saveButton;
    QPushButton *resetButton;

    void setupUi(QWidget *Prefs_ShortWords)
    {
        if (Prefs_ShortWords->objectName().isEmpty())
            Prefs_ShortWords->setObjectName(QString::fromUtf8("Prefs_ShortWords"));
        Prefs_ShortWords->resize(691, 420);
        verticalLayout = new QVBoxLayout(Prefs_ShortWords);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        titleLabel = new QLabel(Prefs_ShortWords);
        titleLabel->setObjectName(QString::fromUtf8("titleLabel"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        titleLabel->setFont(font);

        verticalLayout->addWidget(titleLabel);

        line = new QFrame(Prefs_ShortWords);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        cfgEdit = new QTextEdit(Prefs_ShortWords);
        cfgEdit->setObjectName(QString::fromUtf8("cfgEdit"));

        verticalLayout->addWidget(cfgEdit);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        messageLabel = new QLabel(Prefs_ShortWords);
        messageLabel->setObjectName(QString::fromUtf8("messageLabel"));

        horizontalLayout->addWidget(messageLabel);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);

        saveButton = new QPushButton(Prefs_ShortWords);
        saveButton->setObjectName(QString::fromUtf8("saveButton"));

        horizontalLayout->addWidget(saveButton);

        resetButton = new QPushButton(Prefs_ShortWords);
        resetButton->setObjectName(QString::fromUtf8("resetButton"));

        horizontalLayout->addWidget(resetButton);


        verticalLayout->addLayout(horizontalLayout);


        retranslateUi(Prefs_ShortWords);

        QMetaObject::connectSlotsByName(Prefs_ShortWords);
    } // setupUi

    void retranslateUi(QWidget *Prefs_ShortWords)
    {
        titleLabel->setText(QCoreApplication::translate("Prefs_ShortWords", "Short Words", nullptr));
        messageLabel->setText(QString());
        saveButton->setText(QCoreApplication::translate("Prefs_ShortWords", "Save", nullptr));
        resetButton->setText(QCoreApplication::translate("Prefs_ShortWords", "Reset", nullptr));
        (void)Prefs_ShortWords;
    } // retranslateUi

};

namespace Ui {
    class Prefs_ShortWords: public Ui_Prefs_ShortWords {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_SHORTWORDSBASE_H
