/********************************************************************************
** Form generated from reading UI file 'tablecolumnwidthsdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TABLECOLUMNWIDTHSDIALOG_H
#define UI_TABLECOLUMNWIDTHSDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QVBoxLayout>
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_TableColumnWidthsDialog
{
public:
    QVBoxLayout *verticalLayout;
    QFormLayout *formLayout;
    QLabel *tableColumnWidthLabel;
    ScrSpinBox *tableColumnWidth;
    QDialogButtonBox *dialogButtonBox;

    void setupUi(QDialog *TableColumnWidthsDialog)
    {
        if (TableColumnWidthsDialog->objectName().isEmpty())
            TableColumnWidthsDialog->setObjectName(QString::fromUtf8("TableColumnWidthsDialog"));
        TableColumnWidthsDialog->resize(243, 61);
        TableColumnWidthsDialog->setModal(true);
        verticalLayout = new QVBoxLayout(TableColumnWidthsDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        tableColumnWidthLabel = new QLabel(TableColumnWidthsDialog);
        tableColumnWidthLabel->setObjectName(QString::fromUtf8("tableColumnWidthLabel"));

        formLayout->setWidget(0, QFormLayout::LabelRole, tableColumnWidthLabel);

        tableColumnWidth = new ScrSpinBox(TableColumnWidthsDialog);
        tableColumnWidth->setObjectName(QString::fromUtf8("tableColumnWidth"));
        QSizePolicy sizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(tableColumnWidth->sizePolicy().hasHeightForWidth());
        tableColumnWidth->setSizePolicy(sizePolicy);
        tableColumnWidth->setMinimumSize(QSize(140, 0));
        tableColumnWidth->setMaximum(1000000.000000000000000);

        formLayout->setWidget(0, QFormLayout::FieldRole, tableColumnWidth);


        verticalLayout->addLayout(formLayout);

        dialogButtonBox = new QDialogButtonBox(TableColumnWidthsDialog);
        dialogButtonBox->setObjectName(QString::fromUtf8("dialogButtonBox"));
        dialogButtonBox->setOrientation(Qt::Horizontal);
        dialogButtonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(dialogButtonBox);

#if QT_CONFIG(shortcut)
        tableColumnWidthLabel->setBuddy(tableColumnWidth);
#endif // QT_CONFIG(shortcut)

        retranslateUi(TableColumnWidthsDialog);
        QObject::connect(dialogButtonBox, SIGNAL(accepted()), TableColumnWidthsDialog, SLOT(accept()));
        QObject::connect(dialogButtonBox, SIGNAL(rejected()), TableColumnWidthsDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(TableColumnWidthsDialog);
    } // setupUi

    void retranslateUi(QDialog *TableColumnWidthsDialog)
    {
        TableColumnWidthsDialog->setWindowTitle(QCoreApplication::translate("TableColumnWidthsDialog", "Set Column Widths", nullptr));
        tableColumnWidthLabel->setText(QCoreApplication::translate("TableColumnWidthsDialog", "Column Width:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class TableColumnWidthsDialog: public Ui_TableColumnWidthsDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TABLECOLUMNWIDTHSDIALOG_H
