/********************************************************************************
** Form generated from reading UI file 'prefs_documentitemattributesbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_DOCUMENTITEMATTRIBUTESBASE_H
#define UI_PREFS_DOCUMENTITEMATTRIBUTESBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/sctablewidget.h"

QT_BEGIN_NAMESPACE

class Ui_Prefs_DocumentItemAttributes
{
public:
    QVBoxLayout *vboxLayout;
    QLabel *label;
    QFrame *line;
    ScTableWidget *attributesTable;
    QHBoxLayout *hboxLayout;
    QSpacerItem *spacerItem;
    QPushButton *addButton;
    QPushButton *copyButton;
    QPushButton *deleteButton;
    QPushButton *clearButton;

    void setupUi(QWidget *Prefs_DocumentItemAttributes)
    {
        if (Prefs_DocumentItemAttributes->objectName().isEmpty())
            Prefs_DocumentItemAttributes->setObjectName(QString::fromUtf8("Prefs_DocumentItemAttributes"));
        Prefs_DocumentItemAttributes->resize(758, 517);
        vboxLayout = new QVBoxLayout(Prefs_DocumentItemAttributes);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(Prefs_DocumentItemAttributes);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        vboxLayout->addWidget(label);

        line = new QFrame(Prefs_DocumentItemAttributes);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        vboxLayout->addWidget(line);

        attributesTable = new ScTableWidget(Prefs_DocumentItemAttributes);
        if (attributesTable->columnCount() < 7)
            attributesTable->setColumnCount(7);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        attributesTable->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        attributesTable->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        attributesTable->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        attributesTable->setHorizontalHeaderItem(3, __qtablewidgetitem3);
        QTableWidgetItem *__qtablewidgetitem4 = new QTableWidgetItem();
        attributesTable->setHorizontalHeaderItem(4, __qtablewidgetitem4);
        QTableWidgetItem *__qtablewidgetitem5 = new QTableWidgetItem();
        attributesTable->setHorizontalHeaderItem(5, __qtablewidgetitem5);
        QTableWidgetItem *__qtablewidgetitem6 = new QTableWidgetItem();
        attributesTable->setHorizontalHeaderItem(6, __qtablewidgetitem6);
        attributesTable->setObjectName(QString::fromUtf8("attributesTable"));

        vboxLayout->addWidget(attributesTable);

        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        spacerItem = new QSpacerItem(192, 21, QSizePolicy::Expanding, QSizePolicy::Minimum);

        hboxLayout->addItem(spacerItem);

        addButton = new QPushButton(Prefs_DocumentItemAttributes);
        addButton->setObjectName(QString::fromUtf8("addButton"));

        hboxLayout->addWidget(addButton);

        copyButton = new QPushButton(Prefs_DocumentItemAttributes);
        copyButton->setObjectName(QString::fromUtf8("copyButton"));

        hboxLayout->addWidget(copyButton);

        deleteButton = new QPushButton(Prefs_DocumentItemAttributes);
        deleteButton->setObjectName(QString::fromUtf8("deleteButton"));

        hboxLayout->addWidget(deleteButton);

        clearButton = new QPushButton(Prefs_DocumentItemAttributes);
        clearButton->setObjectName(QString::fromUtf8("clearButton"));

        hboxLayout->addWidget(clearButton);


        vboxLayout->addLayout(hboxLayout);


        retranslateUi(Prefs_DocumentItemAttributes);

        QMetaObject::connectSlotsByName(Prefs_DocumentItemAttributes);
    } // setupUi

    void retranslateUi(QWidget *Prefs_DocumentItemAttributes)
    {
        Prefs_DocumentItemAttributes->setWindowTitle(QCoreApplication::translate("Prefs_DocumentItemAttributes", "Document Item Attributes", nullptr));
        label->setText(QCoreApplication::translate("Prefs_DocumentItemAttributes", "Document Item Attributes", nullptr));
        QTableWidgetItem *___qtablewidgetitem = attributesTable->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QCoreApplication::translate("Prefs_DocumentItemAttributes", "Name", nullptr));
        QTableWidgetItem *___qtablewidgetitem1 = attributesTable->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QCoreApplication::translate("Prefs_DocumentItemAttributes", "Type", nullptr));
        QTableWidgetItem *___qtablewidgetitem2 = attributesTable->horizontalHeaderItem(2);
        ___qtablewidgetitem2->setText(QCoreApplication::translate("Prefs_DocumentItemAttributes", "Value", nullptr));
        QTableWidgetItem *___qtablewidgetitem3 = attributesTable->horizontalHeaderItem(3);
        ___qtablewidgetitem3->setText(QCoreApplication::translate("Prefs_DocumentItemAttributes", "Parameter", nullptr));
        QTableWidgetItem *___qtablewidgetitem4 = attributesTable->horizontalHeaderItem(4);
        ___qtablewidgetitem4->setText(QCoreApplication::translate("Prefs_DocumentItemAttributes", "Relationship", nullptr));
        QTableWidgetItem *___qtablewidgetitem5 = attributesTable->horizontalHeaderItem(5);
        ___qtablewidgetitem5->setText(QCoreApplication::translate("Prefs_DocumentItemAttributes", "Relationship To", nullptr));
        QTableWidgetItem *___qtablewidgetitem6 = attributesTable->horizontalHeaderItem(6);
        ___qtablewidgetitem6->setText(QCoreApplication::translate("Prefs_DocumentItemAttributes", "Auto Add To", nullptr));
        addButton->setText(QCoreApplication::translate("Prefs_DocumentItemAttributes", "&Add", nullptr));
#if QT_CONFIG(shortcut)
        addButton->setShortcut(QCoreApplication::translate("Prefs_DocumentItemAttributes", "Alt+A", nullptr));
#endif // QT_CONFIG(shortcut)
        copyButton->setText(QCoreApplication::translate("Prefs_DocumentItemAttributes", "&Copy", nullptr));
#if QT_CONFIG(shortcut)
        copyButton->setShortcut(QCoreApplication::translate("Prefs_DocumentItemAttributes", "Alt+C", nullptr));
#endif // QT_CONFIG(shortcut)
        deleteButton->setText(QCoreApplication::translate("Prefs_DocumentItemAttributes", "&Delete", nullptr));
#if QT_CONFIG(shortcut)
        deleteButton->setShortcut(QCoreApplication::translate("Prefs_DocumentItemAttributes", "Alt+D", nullptr));
#endif // QT_CONFIG(shortcut)
        clearButton->setText(QCoreApplication::translate("Prefs_DocumentItemAttributes", "C&lear", nullptr));
#if QT_CONFIG(shortcut)
        clearButton->setShortcut(QCoreApplication::translate("Prefs_DocumentItemAttributes", "Alt+L", nullptr));
#endif // QT_CONFIG(shortcut)
    } // retranslateUi

};

namespace Ui {
    class Prefs_DocumentItemAttributes: public Ui_Prefs_DocumentItemAttributes {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_DOCUMENTITEMATTRIBUTESBASE_H
