/********************************************************************************
** Form generated from reading UI file 'charselectenhanced.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_CHARSELECTENHANCED_H
#define UI_CHARSELECTENHANCED_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include "chartableview.h"
#include "ui/fontcombo.h"

QT_BEGIN_NAMESPACE

class Ui_CharSelectEnhanced
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    FontCombo *fontSelector;
    QSpacerItem *spacerItem;
    QLabel *label_2;
    QComboBox *rangeSelector;
    QSpacerItem *spacerItem1;
    CharTableView *m_charTable;
    QHBoxLayout *horizontalLayout;
    QLabel *label_3;
    QLineEdit *hexLineEdit;
    QSpacerItem *horizontalSpacer;
    QGroupBox *groupBox;
    QGridLayout *gridLayout_1;
    QLabel *sample;
    QHBoxLayout *_2;
    QSpacerItem *spacerItem2;
    QPushButton *insertButton;
    QPushButton *deleteButton;

    void setupUi(QDialog *CharSelectEnhanced)
    {
        if (CharSelectEnhanced->objectName().isEmpty())
            CharSelectEnhanced->setObjectName(QString::fromUtf8("CharSelectEnhanced"));
        CharSelectEnhanced->resize(506, 492);
        gridLayout = new QGridLayout(CharSelectEnhanced);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        label = new QLabel(CharSelectEnhanced);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 0, 1, 1);

        fontSelector = new FontCombo(CharSelectEnhanced);
        fontSelector->setObjectName(QString::fromUtf8("fontSelector"));

        gridLayout->addWidget(fontSelector, 0, 1, 1, 1);

        spacerItem = new QSpacerItem(38, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(spacerItem, 0, 2, 1, 1);

        label_2 = new QLabel(CharSelectEnhanced);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 0, 3, 1, 1);

        rangeSelector = new QComboBox(CharSelectEnhanced);
        rangeSelector->setObjectName(QString::fromUtf8("rangeSelector"));

        gridLayout->addWidget(rangeSelector, 0, 4, 1, 1);

        spacerItem1 = new QSpacerItem(66, 28, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout->addItem(spacerItem1, 0, 5, 1, 1);

        m_charTable = new CharTableView(CharSelectEnhanced);
        m_charTable->setObjectName(QString::fromUtf8("m_charTable"));
        m_charTable->setMinimumSize(QSize(340, 0));
        m_charTable->setDragDropOverwriteMode(false);
        m_charTable->setDragDropMode(QAbstractItemView::DragOnly);
        m_charTable->setSelectionMode(QAbstractItemView::SingleSelection);

        gridLayout->addWidget(m_charTable, 1, 0, 1, 6);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        label_3 = new QLabel(CharSelectEnhanced);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        horizontalLayout->addWidget(label_3);

        hexLineEdit = new QLineEdit(CharSelectEnhanced);
        hexLineEdit->setObjectName(QString::fromUtf8("hexLineEdit"));
        hexLineEdit->setMaxLength(32767);

        horizontalLayout->addWidget(hexLineEdit);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        gridLayout->addLayout(horizontalLayout, 2, 0, 1, 6);

        groupBox = new QGroupBox(CharSelectEnhanced);
        groupBox->setObjectName(QString::fromUtf8("groupBox"));
        gridLayout_1 = new QGridLayout(groupBox);
        gridLayout_1->setObjectName(QString::fromUtf8("gridLayout_1"));
        sample = new QLabel(groupBox);
        sample->setObjectName(QString::fromUtf8("sample"));
        sample->setMinimumSize(QSize(460, 48));
        sample->setFrameShape(QFrame::Box);
        sample->setLineWidth(1);

        gridLayout_1->addWidget(sample, 0, 0, 1, 1);


        gridLayout->addWidget(groupBox, 3, 0, 1, 6);

        _2 = new QHBoxLayout();
        _2->setObjectName(QString::fromUtf8("_2"));
        spacerItem2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        _2->addItem(spacerItem2);

        insertButton = new QPushButton(CharSelectEnhanced);
        insertButton->setObjectName(QString::fromUtf8("insertButton"));
        insertButton->setAutoDefault(false);

        _2->addWidget(insertButton);

        deleteButton = new QPushButton(CharSelectEnhanced);
        deleteButton->setObjectName(QString::fromUtf8("deleteButton"));
        deleteButton->setAutoDefault(false);

        _2->addWidget(deleteButton);


        gridLayout->addLayout(_2, 4, 0, 1, 6);

#if QT_CONFIG(shortcut)
        label->setBuddy(fontSelector);
        label_2->setBuddy(rangeSelector);
        label_3->setBuddy(hexLineEdit);
#endif // QT_CONFIG(shortcut)

        retranslateUi(CharSelectEnhanced);

        QMetaObject::connectSlotsByName(CharSelectEnhanced);
    } // setupUi

    void retranslateUi(QDialog *CharSelectEnhanced)
    {
        CharSelectEnhanced->setWindowTitle(QCoreApplication::translate("CharSelectEnhanced", "Enhanced Character Palette", nullptr));
        label->setText(QCoreApplication::translate("CharSelectEnhanced", "&Font:", nullptr));
        label_2->setText(QCoreApplication::translate("CharSelectEnhanced", "C&haracter Class:", nullptr));
#if QT_CONFIG(tooltip)
        m_charTable->setToolTip(QCoreApplication::translate("CharSelectEnhanced", "You can see a thumbnail if you press and hold down the right mouse button.\n"
"The Insert key inserts a Glyph into the Selection below and the Delete key removes the last inserted one", nullptr));
#endif // QT_CONFIG(tooltip)
        label_3->setText(QCoreApplication::translate("CharSelectEnhanced", "Insert &Code:", nullptr));
#if QT_CONFIG(tooltip)
        hexLineEdit->setToolTip(QCoreApplication::translate("CharSelectEnhanced", "Type in a four digit Unicode value directly here", nullptr));
#endif // QT_CONFIG(tooltip)
        groupBox->setTitle(QCoreApplication::translate("CharSelectEnhanced", "Glyphs to Insert", nullptr));
#if QT_CONFIG(tooltip)
        insertButton->setToolTip(QCoreApplication::translate("CharSelectEnhanced", "Insert the characters at the cursor in the text", nullptr));
#endif // QT_CONFIG(tooltip)
        insertButton->setText(QCoreApplication::translate("CharSelectEnhanced", "&Insert", nullptr));
#if QT_CONFIG(tooltip)
        deleteButton->setToolTip(QCoreApplication::translate("CharSelectEnhanced", "Delete the current selection(s).", nullptr));
#endif // QT_CONFIG(tooltip)
        deleteButton->setText(QCoreApplication::translate("CharSelectEnhanced", "C&lear", nullptr));
    } // retranslateUi

};

namespace Ui {
    class CharSelectEnhanced: public Ui_CharSelectEnhanced {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_CHARSELECTENHANCED_H
