/********************************************************************************
** Form generated from reading UI file 'unicodesearch.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_UNICODESEARCH_H
#define UI_UNICODESEARCH_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QTableView>

QT_BEGIN_NAMESPACE

class Ui_UnicodeSearch
{
public:
    QGridLayout *gridLayout;
    QLabel *textLabel1;
    QLineEdit *searchEdit;
    QTableView *tableView;

    void setupUi(QDialog *UnicodeSearch)
    {
        if (UnicodeSearch->objectName().isEmpty())
            UnicodeSearch->setObjectName(QString::fromUtf8("UnicodeSearch"));
        UnicodeSearch->resize(369, 208);
        gridLayout = new QGridLayout(UnicodeSearch);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        textLabel1 = new QLabel(UnicodeSearch);
        textLabel1->setObjectName(QString::fromUtf8("textLabel1"));
        textLabel1->setWordWrap(false);

        gridLayout->addWidget(textLabel1, 0, 0, 1, 1);

        searchEdit = new QLineEdit(UnicodeSearch);
        searchEdit->setObjectName(QString::fromUtf8("searchEdit"));

        gridLayout->addWidget(searchEdit, 0, 1, 1, 1);

        tableView = new QTableView(UnicodeSearch);
        tableView->setObjectName(QString::fromUtf8("tableView"));
        tableView->setAlternatingRowColors(true);
        tableView->setSelectionMode(QAbstractItemView::SingleSelection);
        tableView->setSelectionBehavior(QAbstractItemView::SelectRows);

        gridLayout->addWidget(tableView, 1, 0, 1, 2);

#if QT_CONFIG(shortcut)
        textLabel1->setBuddy(searchEdit);
#endif // QT_CONFIG(shortcut)

        retranslateUi(UnicodeSearch);

        QMetaObject::connectSlotsByName(UnicodeSearch);
    } // setupUi

    void retranslateUi(QDialog *UnicodeSearch)
    {
        UnicodeSearch->setWindowTitle(QCoreApplication::translate("UnicodeSearch", "Unicode Search", nullptr));
        textLabel1->setText(QCoreApplication::translate("UnicodeSearch", "&Search:", nullptr));
#if QT_CONFIG(tooltip)
        searchEdit->setToolTip(QCoreApplication::translate("UnicodeSearch", "Enter the search phrase. Then press Enter.", nullptr));
#endif // QT_CONFIG(tooltip)
    } // retranslateUi

};

namespace Ui {
    class UnicodeSearch: public Ui_UnicodeSearch {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_UNICODESEARCH_H
