/****************************************************************************
** Meta object code from reading C++ file 'smpstylewidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/smpstylewidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QVector>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'smpstylewidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SMPStyleWidget_t {
    QByteArrayData data[19];
    char stringdata0[317];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SMPStyleWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SMPStyleWidget_t qt_meta_stringdata_SMPStyleWidget = {
    {
QT_MOC_LITERAL(0, 0, 14), // "SMPStyleWidget"
QT_MOC_LITERAL(1, 15, 20), // "useParentParaEffects"
QT_MOC_LITERAL(2, 36, 0), // ""
QT_MOC_LITERAL(3, 37, 19), // "useParentOptMargins"
QT_MOC_LITERAL(4, 57, 13), // "iconSetChange"
QT_MOC_LITERAL(5, 71, 26), // "slotLineSpacingModeChanged"
QT_MOC_LITERAL(6, 98, 11), // "slotDropCap"
QT_MOC_LITERAL(7, 110, 4), // "isOn"
QT_MOC_LITERAL(8, 115, 11), // "slotBullets"
QT_MOC_LITERAL(9, 127, 18), // "insertSpecialChars"
QT_MOC_LITERAL(10, 146, 13), // "QVector<uint>"
QT_MOC_LITERAL(11, 160, 9), // "charCodes"
QT_MOC_LITERAL(12, 170, 13), // "slotNumbering"
QT_MOC_LITERAL(13, 184, 20), // "slotParentParEffects"
QT_MOC_LITERAL(14, 205, 25), // "slotDefaultOpticalMargins"
QT_MOC_LITERAL(15, 231, 24), // "slotParentOpticalMargins"
QT_MOC_LITERAL(16, 256, 32), // "on_bulletCharTableButton_toggled"
QT_MOC_LITERAL(17, 289, 7), // "checked"
QT_MOC_LITERAL(18, 297, 19) // "handleUpdateRequest"

    },
    "SMPStyleWidget\0useParentParaEffects\0"
    "\0useParentOptMargins\0iconSetChange\0"
    "slotLineSpacingModeChanged\0slotDropCap\0"
    "isOn\0slotBullets\0insertSpecialChars\0"
    "QVector<uint>\0charCodes\0slotNumbering\0"
    "slotParentParEffects\0slotDefaultOpticalMargins\0"
    "slotParentOpticalMargins\0"
    "on_bulletCharTableButton_toggled\0"
    "checked\0handleUpdateRequest"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SMPStyleWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      13,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       2,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   79,    2, 0x06 /* Public */,
       3,    0,   80,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    0,   81,    2, 0x08 /* Private */,
       5,    1,   82,    2, 0x08 /* Private */,
       6,    1,   85,    2, 0x08 /* Private */,
       8,    1,   88,    2, 0x08 /* Private */,
       9,    1,   91,    2, 0x08 /* Private */,
      12,    1,   94,    2, 0x08 /* Private */,
      13,    0,   97,    2, 0x08 /* Private */,
      14,    0,   98,    2, 0x08 /* Private */,
      15,    0,   99,    2, 0x08 /* Private */,
      16,    1,  100,    2, 0x08 /* Private */,
      18,    1,  103,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void, QMetaType::Bool,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   17,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void SMPStyleWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SMPStyleWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->useParentParaEffects(); break;
        case 1: _t->useParentOptMargins(); break;
        case 2: _t->iconSetChange(); break;
        case 3: _t->slotLineSpacingModeChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->slotDropCap((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 5: _t->slotBullets((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 6: _t->insertSpecialChars((*reinterpret_cast< const QVector<uint>(*)>(_a[1]))); break;
        case 7: _t->slotNumbering((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->slotParentParEffects(); break;
        case 9: _t->slotDefaultOpticalMargins(); break;
        case 10: _t->slotParentOpticalMargins(); break;
        case 11: _t->on_bulletCharTableButton_toggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 12: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< QVector<uint> >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SMPStyleWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SMPStyleWidget::useParentParaEffects)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SMPStyleWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SMPStyleWidget::useParentOptMargins)) {
                *result = 1;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SMPStyleWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_SMPStyleWidget.data,
    qt_meta_data_SMPStyleWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SMPStyleWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SMPStyleWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SMPStyleWidget.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::SMPStyleWidget"))
        return static_cast< Ui::SMPStyleWidget*>(this);
    return QWidget::qt_metacast(_clname);
}

int SMPStyleWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 13)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 13;
    }
    return _id;
}

// SIGNAL 0
void SMPStyleWidget::useParentParaEffects()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void SMPStyleWidget::useParentOptMargins()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
