/********************************************************************************
** Form generated from reading UI file 'notesstyleseditor.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_NOTESSTYLESEDITOR_H
#define UI_NOTESSTYLESEDITOR_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QRadioButton>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include "ui/numformatcombo.h"
#include "ui/stylecombos.h"

QT_BEGIN_NAMESPACE

class Ui_NotesStylesEditor
{
public:
    QGridLayout *gridLayout;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_2;
    QComboBox *NSlistBox;
    QPushButton *NewButton;
    QFormLayout *formLayout;
    QLabel *NewNameLabel;
    QLineEdit *NewNameEdit;
    QRadioButton *FootRadio;
    QRadioButton *EndRadio;
    QLabel *NumberingLabel;
    NumFormatCombo *NumberingBox;
    QLabel *RangeLabel;
    QComboBox *RangeBox;
    QLabel *StartLabel;
    QSpinBox *StartSpinBox;
    QLabel *PrefixLabel;
    QLineEdit *PrefixEdit;
    QLabel *SuffixLabel;
    QLineEdit *SuffixEdit;
    QLabel *SuperMasterLabel;
    QCheckBox *SuperMasterCheck;
    QLabel *SuperNoteLabel;
    QCheckBox *SuperNoteCheck;
    QLabel *charStyleComboLabel;
    CharStyleComboBox *charStyleCombo;
    QLabel *paraStyleComboLabel;
    ParaStyleComboBox *paraStyleCombo;
    QLabel *AutoHLabel;
    QCheckBox *AutoH;
    QLabel *AutoWLabel;
    QCheckBox *AutoW;
    QLabel *AutoWeldLabel;
    QCheckBox *AutoWeld;
    QLabel *AutoRemoveLabel;
    QCheckBox *AutoRemove;
    QHBoxLayout *horizontalLayout;
    QPushButton *ApplyButton;
    QPushButton *DeleteButton;
    QPushButton *OKButton;

    void setupUi(QDialog *NotesStylesEditor)
    {
        if (NotesStylesEditor->objectName().isEmpty())
            NotesStylesEditor->setObjectName(QString::fromUtf8("NotesStylesEditor"));
        NotesStylesEditor->resize(324, 576);
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(NotesStylesEditor->sizePolicy().hasHeightForWidth());
        NotesStylesEditor->setSizePolicy(sizePolicy);
        gridLayout = new QGridLayout(NotesStylesEditor);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        verticalLayout = new QVBoxLayout();
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        NSlistBox = new QComboBox(NotesStylesEditor);
        NSlistBox->setObjectName(QString::fromUtf8("NSlistBox"));
        QSizePolicy sizePolicy1(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(NSlistBox->sizePolicy().hasHeightForWidth());
        NSlistBox->setSizePolicy(sizePolicy1);

        horizontalLayout_2->addWidget(NSlistBox);

        NewButton = new QPushButton(NotesStylesEditor);
        NewButton->setObjectName(QString::fromUtf8("NewButton"));

        horizontalLayout_2->addWidget(NewButton);


        verticalLayout->addLayout(horizontalLayout_2);

        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        NewNameLabel = new QLabel(NotesStylesEditor);
        NewNameLabel->setObjectName(QString::fromUtf8("NewNameLabel"));

        formLayout->setWidget(0, QFormLayout::LabelRole, NewNameLabel);

        NewNameEdit = new QLineEdit(NotesStylesEditor);
        NewNameEdit->setObjectName(QString::fromUtf8("NewNameEdit"));

        formLayout->setWidget(0, QFormLayout::FieldRole, NewNameEdit);

        FootRadio = new QRadioButton(NotesStylesEditor);
        FootRadio->setObjectName(QString::fromUtf8("FootRadio"));
        sizePolicy1.setHeightForWidth(FootRadio->sizePolicy().hasHeightForWidth());
        FootRadio->setSizePolicy(sizePolicy1);

        formLayout->setWidget(1, QFormLayout::LabelRole, FootRadio);

        EndRadio = new QRadioButton(NotesStylesEditor);
        EndRadio->setObjectName(QString::fromUtf8("EndRadio"));
        sizePolicy1.setHeightForWidth(EndRadio->sizePolicy().hasHeightForWidth());
        EndRadio->setSizePolicy(sizePolicy1);

        formLayout->setWidget(1, QFormLayout::FieldRole, EndRadio);

        NumberingLabel = new QLabel(NotesStylesEditor);
        NumberingLabel->setObjectName(QString::fromUtf8("NumberingLabel"));

        formLayout->setWidget(2, QFormLayout::LabelRole, NumberingLabel);

        NumberingBox = new NumFormatCombo(NotesStylesEditor);
        NumberingBox->setObjectName(QString::fromUtf8("NumberingBox"));
        sizePolicy1.setHeightForWidth(NumberingBox->sizePolicy().hasHeightForWidth());
        NumberingBox->setSizePolicy(sizePolicy1);

        formLayout->setWidget(2, QFormLayout::FieldRole, NumberingBox);

        RangeLabel = new QLabel(NotesStylesEditor);
        RangeLabel->setObjectName(QString::fromUtf8("RangeLabel"));

        formLayout->setWidget(3, QFormLayout::LabelRole, RangeLabel);

        RangeBox = new QComboBox(NotesStylesEditor);
        RangeBox->setObjectName(QString::fromUtf8("RangeBox"));
        sizePolicy1.setHeightForWidth(RangeBox->sizePolicy().hasHeightForWidth());
        RangeBox->setSizePolicy(sizePolicy1);

        formLayout->setWidget(3, QFormLayout::FieldRole, RangeBox);

        StartLabel = new QLabel(NotesStylesEditor);
        StartLabel->setObjectName(QString::fromUtf8("StartLabel"));

        formLayout->setWidget(4, QFormLayout::LabelRole, StartLabel);

        StartSpinBox = new QSpinBox(NotesStylesEditor);
        StartSpinBox->setObjectName(QString::fromUtf8("StartSpinBox"));
        sizePolicy1.setHeightForWidth(StartSpinBox->sizePolicy().hasHeightForWidth());
        StartSpinBox->setSizePolicy(sizePolicy1);

        formLayout->setWidget(4, QFormLayout::FieldRole, StartSpinBox);

        PrefixLabel = new QLabel(NotesStylesEditor);
        PrefixLabel->setObjectName(QString::fromUtf8("PrefixLabel"));

        formLayout->setWidget(5, QFormLayout::LabelRole, PrefixLabel);

        PrefixEdit = new QLineEdit(NotesStylesEditor);
        PrefixEdit->setObjectName(QString::fromUtf8("PrefixEdit"));

        formLayout->setWidget(5, QFormLayout::FieldRole, PrefixEdit);

        SuffixLabel = new QLabel(NotesStylesEditor);
        SuffixLabel->setObjectName(QString::fromUtf8("SuffixLabel"));

        formLayout->setWidget(6, QFormLayout::LabelRole, SuffixLabel);

        SuffixEdit = new QLineEdit(NotesStylesEditor);
        SuffixEdit->setObjectName(QString::fromUtf8("SuffixEdit"));

        formLayout->setWidget(6, QFormLayout::FieldRole, SuffixEdit);

        SuperMasterLabel = new QLabel(NotesStylesEditor);
        SuperMasterLabel->setObjectName(QString::fromUtf8("SuperMasterLabel"));

        formLayout->setWidget(7, QFormLayout::LabelRole, SuperMasterLabel);

        SuperMasterCheck = new QCheckBox(NotesStylesEditor);
        SuperMasterCheck->setObjectName(QString::fromUtf8("SuperMasterCheck"));

        formLayout->setWidget(7, QFormLayout::FieldRole, SuperMasterCheck);

        SuperNoteLabel = new QLabel(NotesStylesEditor);
        SuperNoteLabel->setObjectName(QString::fromUtf8("SuperNoteLabel"));

        formLayout->setWidget(8, QFormLayout::LabelRole, SuperNoteLabel);

        SuperNoteCheck = new QCheckBox(NotesStylesEditor);
        SuperNoteCheck->setObjectName(QString::fromUtf8("SuperNoteCheck"));

        formLayout->setWidget(8, QFormLayout::FieldRole, SuperNoteCheck);

        charStyleComboLabel = new QLabel(NotesStylesEditor);
        charStyleComboLabel->setObjectName(QString::fromUtf8("charStyleComboLabel"));
        charStyleComboLabel->setLineWidth(2);
        charStyleComboLabel->setTextFormat(Qt::RichText);

        formLayout->setWidget(9, QFormLayout::LabelRole, charStyleComboLabel);

        charStyleCombo = new CharStyleComboBox(NotesStylesEditor);
        charStyleCombo->setObjectName(QString::fromUtf8("charStyleCombo"));
        sizePolicy1.setHeightForWidth(charStyleCombo->sizePolicy().hasHeightForWidth());
        charStyleCombo->setSizePolicy(sizePolicy1);

        formLayout->setWidget(9, QFormLayout::FieldRole, charStyleCombo);

        paraStyleComboLabel = new QLabel(NotesStylesEditor);
        paraStyleComboLabel->setObjectName(QString::fromUtf8("paraStyleComboLabel"));

        formLayout->setWidget(10, QFormLayout::LabelRole, paraStyleComboLabel);

        paraStyleCombo = new ParaStyleComboBox(NotesStylesEditor);
        paraStyleCombo->setObjectName(QString::fromUtf8("paraStyleCombo"));
        sizePolicy1.setHeightForWidth(paraStyleCombo->sizePolicy().hasHeightForWidth());
        paraStyleCombo->setSizePolicy(sizePolicy1);

        formLayout->setWidget(10, QFormLayout::FieldRole, paraStyleCombo);

        AutoHLabel = new QLabel(NotesStylesEditor);
        AutoHLabel->setObjectName(QString::fromUtf8("AutoHLabel"));

        formLayout->setWidget(11, QFormLayout::LabelRole, AutoHLabel);

        AutoH = new QCheckBox(NotesStylesEditor);
        AutoH->setObjectName(QString::fromUtf8("AutoH"));

        formLayout->setWidget(11, QFormLayout::FieldRole, AutoH);

        AutoWLabel = new QLabel(NotesStylesEditor);
        AutoWLabel->setObjectName(QString::fromUtf8("AutoWLabel"));

        formLayout->setWidget(12, QFormLayout::LabelRole, AutoWLabel);

        AutoW = new QCheckBox(NotesStylesEditor);
        AutoW->setObjectName(QString::fromUtf8("AutoW"));

        formLayout->setWidget(12, QFormLayout::FieldRole, AutoW);

        AutoWeldLabel = new QLabel(NotesStylesEditor);
        AutoWeldLabel->setObjectName(QString::fromUtf8("AutoWeldLabel"));

        formLayout->setWidget(13, QFormLayout::LabelRole, AutoWeldLabel);

        AutoWeld = new QCheckBox(NotesStylesEditor);
        AutoWeld->setObjectName(QString::fromUtf8("AutoWeld"));

        formLayout->setWidget(13, QFormLayout::FieldRole, AutoWeld);

        AutoRemoveLabel = new QLabel(NotesStylesEditor);
        AutoRemoveLabel->setObjectName(QString::fromUtf8("AutoRemoveLabel"));

        formLayout->setWidget(14, QFormLayout::LabelRole, AutoRemoveLabel);

        AutoRemove = new QCheckBox(NotesStylesEditor);
        AutoRemove->setObjectName(QString::fromUtf8("AutoRemove"));

        formLayout->setWidget(14, QFormLayout::FieldRole, AutoRemove);


        verticalLayout->addLayout(formLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        ApplyButton = new QPushButton(NotesStylesEditor);
        ApplyButton->setObjectName(QString::fromUtf8("ApplyButton"));
        sizePolicy1.setHeightForWidth(ApplyButton->sizePolicy().hasHeightForWidth());
        ApplyButton->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(ApplyButton);

        DeleteButton = new QPushButton(NotesStylesEditor);
        DeleteButton->setObjectName(QString::fromUtf8("DeleteButton"));
        sizePolicy1.setHeightForWidth(DeleteButton->sizePolicy().hasHeightForWidth());
        DeleteButton->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(DeleteButton);

        OKButton = new QPushButton(NotesStylesEditor);
        OKButton->setObjectName(QString::fromUtf8("OKButton"));
        sizePolicy1.setHeightForWidth(OKButton->sizePolicy().hasHeightForWidth());
        OKButton->setSizePolicy(sizePolicy1);

        horizontalLayout->addWidget(OKButton);


        verticalLayout->addLayout(horizontalLayout);


        gridLayout->addLayout(verticalLayout, 1, 0, 1, 1);


        retranslateUi(NotesStylesEditor);

        QMetaObject::connectSlotsByName(NotesStylesEditor);
    } // setupUi

    void retranslateUi(QDialog *NotesStylesEditor)
    {
        NotesStylesEditor->setWindowTitle(QCoreApplication::translate("NotesStylesEditor", "Notes Styles Editor", nullptr));
        NewButton->setText(QCoreApplication::translate("NotesStylesEditor", "Add New Style", nullptr));
        NewNameLabel->setText(QCoreApplication::translate("NotesStylesEditor", "New Style Name", nullptr));
        FootRadio->setText(QCoreApplication::translate("NotesStylesEditor", "Footnotes", nullptr));
        EndRadio->setText(QCoreApplication::translate("NotesStylesEditor", "Endnotes", nullptr));
        NumberingLabel->setText(QCoreApplication::translate("NotesStylesEditor", "Numbering", nullptr));
        RangeLabel->setText(QCoreApplication::translate("NotesStylesEditor", "Range", nullptr));
        StartLabel->setText(QCoreApplication::translate("NotesStylesEditor", "Start Number", nullptr));
        PrefixLabel->setText(QCoreApplication::translate("NotesStylesEditor", "Prefix", nullptr));
        SuffixLabel->setText(QCoreApplication::translate("NotesStylesEditor", "Suffix", nullptr));
        SuperMasterLabel->setText(QCoreApplication::translate("NotesStylesEditor", "Superscript in text", nullptr));
        SuperMasterCheck->setText(QString());
        SuperNoteLabel->setText(QCoreApplication::translate("NotesStylesEditor", "Superscript in notes", nullptr));
        SuperNoteCheck->setText(QString());
        charStyleComboLabel->setText(QCoreApplication::translate("NotesStylesEditor", "Mark Character Style", nullptr));
        paraStyleComboLabel->setText(QCoreApplication::translate("NotesStylesEditor", "Note Paragraph Style", nullptr));
        AutoHLabel->setText(QCoreApplication::translate("NotesStylesEditor", "Auto Height", nullptr));
        AutoH->setText(QString());
        AutoWLabel->setText(QCoreApplication::translate("NotesStylesEditor", "Auto Width", nullptr));
        AutoW->setText(QString());
        AutoWeldLabel->setText(QCoreApplication::translate("NotesStylesEditor", "Auto Welding", nullptr));
        AutoWeld->setText(QString());
        AutoRemoveLabel->setText(QCoreApplication::translate("NotesStylesEditor", "Remove if empty", nullptr));
        AutoRemove->setText(QString());
        ApplyButton->setText(QCoreApplication::translate("NotesStylesEditor", "Apply", nullptr));
        DeleteButton->setText(QCoreApplication::translate("NotesStylesEditor", "Delete", nullptr));
        OKButton->setText(QCoreApplication::translate("NotesStylesEditor", "OK", nullptr));
    } // retranslateUi

};

namespace Ui {
    class NotesStylesEditor: public Ui_NotesStylesEditor {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_NOTESSTYLESEDITOR_H
