/****************************************************************************
** Meta object code from reading C++ file 'pageitem.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/pageitem.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'pageitem.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PageItem_t {
    QByteArrayData data[62];
    char stringdata0[676];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PageItem_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PageItem_t qt_meta_stringdata_PageItem = {
    {
QT_MOC_LITERAL(0, 0, 8), // "PageItem"
QT_MOC_LITERAL(1, 9, 6), // "myself"
QT_MOC_LITERAL(2, 16, 0), // ""
QT_MOC_LITERAL(3, 17, 9), // "PageItem*"
QT_MOC_LITERAL(4, 27, 9), // "frameType"
QT_MOC_LITERAL(5, 37, 9), // "textStyle"
QT_MOC_LITERAL(6, 47, 8), // "itemName"
QT_MOC_LITERAL(7, 56, 9), // "fillColor"
QT_MOC_LITERAL(8, 66, 9), // "lineColor"
QT_MOC_LITERAL(9, 76, 9), // "fillShade"
QT_MOC_LITERAL(10, 86, 9), // "lineShade"
QT_MOC_LITERAL(11, 96, 16), // "fillTransparency"
QT_MOC_LITERAL(12, 113, 16), // "lineTransparency"
QT_MOC_LITERAL(13, 130, 6), // "locked"
QT_MOC_LITERAL(14, 137, 10), // "sizeLocked"
QT_MOC_LITERAL(15, 148, 11), // "sizeHLocked"
QT_MOC_LITERAL(16, 160, 11), // "sizeVLocked"
QT_MOC_LITERAL(17, 172, 13), // "imageFlippedV"
QT_MOC_LITERAL(18, 186, 13), // "imageFlippedH"
QT_MOC_LITERAL(19, 200, 9), // "lineWidth"
QT_MOC_LITERAL(20, 210, 15), // "customLineStyle"
QT_MOC_LITERAL(21, 226, 15), // "startArrowIndex"
QT_MOC_LITERAL(22, 242, 13), // "endArrowIndex"
QT_MOC_LITERAL(23, 256, 12), // "printEnabled"
QT_MOC_LITERAL(24, 269, 4), // "xPos"
QT_MOC_LITERAL(25, 274, 4), // "yPos"
QT_MOC_LITERAL(26, 279, 5), // "width"
QT_MOC_LITERAL(27, 285, 6), // "height"
QT_MOC_LITERAL(28, 292, 8), // "rotation"
QT_MOC_LITERAL(29, 301, 11), // "imageXScale"
QT_MOC_LITERAL(30, 313, 11), // "imageYScale"
QT_MOC_LITERAL(31, 325, 12), // "imageXOffset"
QT_MOC_LITERAL(32, 338, 12), // "imageYOffset"
QT_MOC_LITERAL(33, 351, 12), // "cornerRadius"
QT_MOC_LITERAL(34, 364, 19), // "textToFrameDistLeft"
QT_MOC_LITERAL(35, 384, 20), // "textToFrameDistRight"
QT_MOC_LITERAL(36, 405, 18), // "textToFrameDistTop"
QT_MOC_LITERAL(37, 424, 21), // "textToFrameDistBottom"
QT_MOC_LITERAL(38, 446, 9), // "columnGap"
QT_MOC_LITERAL(39, 456, 7), // "columns"
QT_MOC_LITERAL(40, 464, 15), // "firstLineOffset"
QT_MOC_LITERAL(41, 480, 21), // "FirstLineOffsetPolicy"
QT_MOC_LITERAL(42, 502, 8), // "itemType"
QT_MOC_LITERAL(43, 511, 8), // "ItemType"
QT_MOC_LITERAL(44, 520, 9), // "ItemType1"
QT_MOC_LITERAL(45, 530, 10), // "ImageFrame"
QT_MOC_LITERAL(46, 541, 9), // "ItemType3"
QT_MOC_LITERAL(47, 551, 9), // "TextFrame"
QT_MOC_LITERAL(48, 561, 4), // "Line"
QT_MOC_LITERAL(49, 566, 7), // "Polygon"
QT_MOC_LITERAL(50, 574, 8), // "PolyLine"
QT_MOC_LITERAL(51, 583, 8), // "PathText"
QT_MOC_LITERAL(52, 592, 10), // "LatexFrame"
QT_MOC_LITERAL(53, 603, 8), // "OSGFrame"
QT_MOC_LITERAL(54, 612, 6), // "Symbol"
QT_MOC_LITERAL(55, 619, 5), // "Group"
QT_MOC_LITERAL(56, 625, 14), // "RegularPolygon"
QT_MOC_LITERAL(57, 640, 3), // "Arc"
QT_MOC_LITERAL(58, 644, 6), // "Spiral"
QT_MOC_LITERAL(59, 651, 5), // "Table"
QT_MOC_LITERAL(60, 657, 9), // "NoteFrame"
QT_MOC_LITERAL(61, 667, 8) // "Multiple"

    },
    "PageItem\0myself\0\0PageItem*\0frameType\0"
    "textStyle\0itemName\0fillColor\0lineColor\0"
    "fillShade\0lineShade\0fillTransparency\0"
    "lineTransparency\0locked\0sizeLocked\0"
    "sizeHLocked\0sizeVLocked\0imageFlippedV\0"
    "imageFlippedH\0lineWidth\0customLineStyle\0"
    "startArrowIndex\0endArrowIndex\0"
    "printEnabled\0xPos\0yPos\0width\0height\0"
    "rotation\0imageXScale\0imageYScale\0"
    "imageXOffset\0imageYOffset\0cornerRadius\0"
    "textToFrameDistLeft\0textToFrameDistRight\0"
    "textToFrameDistTop\0textToFrameDistBottom\0"
    "columnGap\0columns\0firstLineOffset\0"
    "FirstLineOffsetPolicy\0itemType\0ItemType\0"
    "ItemType1\0ImageFrame\0ItemType3\0TextFrame\0"
    "Line\0Polygon\0PolyLine\0PathText\0"
    "LatexFrame\0OSGFrame\0Symbol\0Group\0"
    "RegularPolygon\0Arc\0Spiral\0Table\0"
    "NoteFrame\0Multiple"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PageItem[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
      36,   38, // properties
       1,  146, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,
       4,    1,   32,    2, 0x06 /* Public */,
       5,    1,   35,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, 0x80000000 | 3,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,

 // properties: name, type, flags
       6, QMetaType::QString, 0x00094103,
       7, QMetaType::QString, 0x00094103,
       8, QMetaType::QString, 0x00094103,
       9, QMetaType::Double, 0x00094103,
      10, QMetaType::Double, 0x00094103,
      11, QMetaType::Double, 0x00094103,
      12, QMetaType::Double, 0x00094103,
      13, QMetaType::Bool, 0x00094103,
      14, QMetaType::Bool, 0x00094103,
      15, QMetaType::Bool, 0x00094103,
      16, QMetaType::Bool, 0x00094103,
      17, QMetaType::Bool, 0x00094103,
      18, QMetaType::Bool, 0x00094103,
      19, QMetaType::Double, 0x00094103,
      20, QMetaType::QString, 0x00094103,
      21, QMetaType::Int, 0x00094103,
      22, QMetaType::Int, 0x00094103,
      23, QMetaType::Bool, 0x00094103,
      24, QMetaType::Double, 0x00094103,
      25, QMetaType::Double, 0x00094103,
      26, QMetaType::Double, 0x00094103,
      27, QMetaType::Double, 0x00094103,
      28, QMetaType::Double, 0x00094103,
      29, QMetaType::Double, 0x00094103,
      30, QMetaType::Double, 0x00094103,
      31, QMetaType::Double, 0x00094103,
      32, QMetaType::Double, 0x00094103,
      33, QMetaType::Double, 0x00094103,
      34, QMetaType::Double, 0x00094103,
      35, QMetaType::Double, 0x00094103,
      36, QMetaType::Double, 0x00094103,
      37, QMetaType::Double, 0x00094103,
      38, QMetaType::Double, 0x00094103,
      39, QMetaType::Int, 0x00094103,
      40, 0x80000000 | 41, 0x0009410b,
      42, 0x80000000 | 43, 0x0009400b,

 // enums: name, alias, flags, count, data
      43,   43, 0x0,   18,  151,

 // enum data: key, value
      44, uint(PageItem::ItemType1),
      45, uint(PageItem::ImageFrame),
      46, uint(PageItem::ItemType3),
      47, uint(PageItem::TextFrame),
      48, uint(PageItem::Line),
      49, uint(PageItem::Polygon),
      50, uint(PageItem::PolyLine),
      51, uint(PageItem::PathText),
      52, uint(PageItem::LatexFrame),
      53, uint(PageItem::OSGFrame),
      54, uint(PageItem::Symbol),
      55, uint(PageItem::Group),
      56, uint(PageItem::RegularPolygon),
      57, uint(PageItem::Arc),
      58, uint(PageItem::Spiral),
      59, uint(PageItem::Table),
      60, uint(PageItem::NoteFrame),
      61, uint(PageItem::Multiple),

       0        // eod
};

void PageItem::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PageItem *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->myself((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 1: _t->frameType((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->textStyle((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 0:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PageItem::*)(PageItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageItem::myself)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (PageItem::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageItem::frameType)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (PageItem::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PageItem::textStyle)) {
                *result = 2;
                return;
            }
        }
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty) {
        auto *_t = static_cast<PageItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: *reinterpret_cast< QString*>(_v) = _t->itemName(); break;
        case 1: *reinterpret_cast< QString*>(_v) = _t->fillColor(); break;
        case 2: *reinterpret_cast< QString*>(_v) = _t->lineColor(); break;
        case 3: *reinterpret_cast< double*>(_v) = _t->fillShade(); break;
        case 4: *reinterpret_cast< double*>(_v) = _t->lineShade(); break;
        case 5: *reinterpret_cast< double*>(_v) = _t->fillTransparency(); break;
        case 6: *reinterpret_cast< double*>(_v) = _t->lineTransparency(); break;
        case 7: *reinterpret_cast< bool*>(_v) = _t->locked(); break;
        case 8: *reinterpret_cast< bool*>(_v) = _t->sizeLocked(); break;
        case 9: *reinterpret_cast< bool*>(_v) = _t->sizeHLocked(); break;
        case 10: *reinterpret_cast< bool*>(_v) = _t->sizeVLocked(); break;
        case 11: *reinterpret_cast< bool*>(_v) = _t->imageFlippedV(); break;
        case 12: *reinterpret_cast< bool*>(_v) = _t->imageFlippedH(); break;
        case 13: *reinterpret_cast< double*>(_v) = _t->lineWidth(); break;
        case 14: *reinterpret_cast< QString*>(_v) = _t->customLineStyle(); break;
        case 15: *reinterpret_cast< int*>(_v) = _t->startArrowIndex(); break;
        case 16: *reinterpret_cast< int*>(_v) = _t->endArrowIndex(); break;
        case 17: *reinterpret_cast< bool*>(_v) = _t->printEnabled(); break;
        case 18: *reinterpret_cast< double*>(_v) = _t->xPos(); break;
        case 19: *reinterpret_cast< double*>(_v) = _t->yPos(); break;
        case 20: *reinterpret_cast< double*>(_v) = _t->width(); break;
        case 21: *reinterpret_cast< double*>(_v) = _t->height(); break;
        case 22: *reinterpret_cast< double*>(_v) = _t->rotation(); break;
        case 23: *reinterpret_cast< double*>(_v) = _t->imageXScale(); break;
        case 24: *reinterpret_cast< double*>(_v) = _t->imageYScale(); break;
        case 25: *reinterpret_cast< double*>(_v) = _t->imageXOffset(); break;
        case 26: *reinterpret_cast< double*>(_v) = _t->imageYOffset(); break;
        case 27: *reinterpret_cast< double*>(_v) = _t->cornerRadius(); break;
        case 28: *reinterpret_cast< double*>(_v) = _t->textToFrameDistLeft(); break;
        case 29: *reinterpret_cast< double*>(_v) = _t->textToFrameDistRight(); break;
        case 30: *reinterpret_cast< double*>(_v) = _t->textToFrameDistTop(); break;
        case 31: *reinterpret_cast< double*>(_v) = _t->textToFrameDistBottom(); break;
        case 32: *reinterpret_cast< double*>(_v) = _t->columnGap(); break;
        case 33: *reinterpret_cast< int*>(_v) = _t->columns(); break;
        case 34: *reinterpret_cast< FirstLineOffsetPolicy*>(_v) = _t->firstLineOffset(); break;
        case 35: *reinterpret_cast< ItemType*>(_v) = _t->itemType(); break;
        default: break;
        }
    } else if (_c == QMetaObject::WriteProperty) {
        auto *_t = static_cast<PageItem *>(_o);
        (void)_t;
        void *_v = _a[0];
        switch (_id) {
        case 0: _t->setItemName(*reinterpret_cast< QString*>(_v)); break;
        case 1: _t->setFillColor(*reinterpret_cast< QString*>(_v)); break;
        case 2: _t->setLineColor(*reinterpret_cast< QString*>(_v)); break;
        case 3: _t->setFillShade(*reinterpret_cast< double*>(_v)); break;
        case 4: _t->setLineShade(*reinterpret_cast< double*>(_v)); break;
        case 5: _t->setFillTransparency(*reinterpret_cast< double*>(_v)); break;
        case 6: _t->setLineTransparency(*reinterpret_cast< double*>(_v)); break;
        case 7: _t->setLocked(*reinterpret_cast< bool*>(_v)); break;
        case 8: _t->setSizeLocked(*reinterpret_cast< bool*>(_v)); break;
        case 9: _t->setSizeHLocked(*reinterpret_cast< bool*>(_v)); break;
        case 10: _t->setSizeVLocked(*reinterpret_cast< bool*>(_v)); break;
        case 11: _t->setImageFlippedV(*reinterpret_cast< bool*>(_v)); break;
        case 12: _t->setImageFlippedH(*reinterpret_cast< bool*>(_v)); break;
        case 13: _t->setLineWidth(*reinterpret_cast< double*>(_v)); break;
        case 14: _t->setCustomLineStyle(*reinterpret_cast< QString*>(_v)); break;
        case 15: _t->setStartArrowIndex(*reinterpret_cast< int*>(_v)); break;
        case 16: _t->setEndArrowIndex(*reinterpret_cast< int*>(_v)); break;
        case 17: _t->setPrintEnabled(*reinterpret_cast< bool*>(_v)); break;
        case 18: _t->setXPos(*reinterpret_cast< double*>(_v)); break;
        case 19: _t->setYPos(*reinterpret_cast< double*>(_v)); break;
        case 20: _t->setWidth(*reinterpret_cast< double*>(_v)); break;
        case 21: _t->setHeight(*reinterpret_cast< double*>(_v)); break;
        case 22: _t->setRotation(*reinterpret_cast< double*>(_v)); break;
        case 23: _t->setImageXScale(*reinterpret_cast< double*>(_v)); break;
        case 24: _t->setImageYScale(*reinterpret_cast< double*>(_v)); break;
        case 25: _t->setImageXOffset(*reinterpret_cast< double*>(_v)); break;
        case 26: _t->setImageYOffset(*reinterpret_cast< double*>(_v)); break;
        case 27: _t->setCornerRadius(*reinterpret_cast< double*>(_v)); break;
        case 28: _t->setTextToFrameDistLeft(*reinterpret_cast< double*>(_v)); break;
        case 29: _t->setTextToFrameDistRight(*reinterpret_cast< double*>(_v)); break;
        case 30: _t->setTextToFrameDistTop(*reinterpret_cast< double*>(_v)); break;
        case 31: _t->setTextToFrameDistBottom(*reinterpret_cast< double*>(_v)); break;
        case 32: _t->setColumnGap(*reinterpret_cast< double*>(_v)); break;
        case 33: _t->setColumns(*reinterpret_cast< int*>(_v)); break;
        case 34: _t->setFirstLineOffset(*reinterpret_cast< FirstLineOffsetPolicy*>(_v)); break;
        case 35: _t->convertTo(*reinterpret_cast< ItemType*>(_v)); break;
        default: break;
        }
    } else if (_c == QMetaObject::ResetProperty) {
    }
#endif // QT_NO_PROPERTIES
}

QT_INIT_METAOBJECT const QMetaObject PageItem::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_PageItem.data,
    qt_meta_data_PageItem,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PageItem::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PageItem::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PageItem.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "UndoObject"))
        return static_cast< UndoObject*>(this);
    if (!strcmp(_clname, "SaxIO"))
        return static_cast< SaxIO*>(this);
    if (!strcmp(_clname, "SingleObservable<PageItem>"))
        return static_cast< SingleObservable<PageItem>*>(this);
    if (!strcmp(_clname, "TextContext"))
        return static_cast< TextContext*>(this);
    return QObject::qt_metacast(_clname);
}

int PageItem::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    }
#ifndef QT_NO_PROPERTIES
    else if (_c == QMetaObject::ReadProperty || _c == QMetaObject::WriteProperty
            || _c == QMetaObject::ResetProperty || _c == QMetaObject::RegisterPropertyMetaType) {
        qt_static_metacall(this, _c, _id, _a);
        _id -= 36;
    } else if (_c == QMetaObject::QueryPropertyDesignable) {
        _id -= 36;
    } else if (_c == QMetaObject::QueryPropertyScriptable) {
        _id -= 36;
    } else if (_c == QMetaObject::QueryPropertyStored) {
        _id -= 36;
    } else if (_c == QMetaObject::QueryPropertyEditable) {
        _id -= 36;
    } else if (_c == QMetaObject::QueryPropertyUser) {
        _id -= 36;
    }
#endif // QT_NO_PROPERTIES
    return _id;
}

// SIGNAL 0
void PageItem::myself(PageItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void PageItem::frameType(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void PageItem::textStyle(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
