/********************************************************************************
** Form generated from reading UI file 'prefs_operatortoolsbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_OPERATORTOOLSBASE_H
#define UI_PREFS_OPERATORTOOLSBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/scrspinbox.h>

QT_BEGIN_NAMESPACE

class Ui_Prefs_OperatorTools
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QFrame *line;
    QTabWidget *tabWidget;
    QWidget *tab_4;
    QVBoxLayout *verticalLayout_2;
    QFormLayout *formLayout;
    QLabel *label_2;
    QLabel *label_3;
    QLabel *label_4;
    ScrSpinBox *zoomMinimumSpinBox;
    QSpinBox *zoomMaximumSpinBox;
    QSpinBox *zoomSteppingSpinBox;
    QSpacerItem *verticalSpacer;
    QWidget *tab_5;
    QVBoxLayout *verticalLayout_3;
    QLabel *label_5;
    QFrame *line_2;
    QFormLayout *formLayout_2;
    QLabel *label_6;
    QLabel *label_7;
    ScrSpinBox *itemDuplicateXDispSpinBox;
    ScrSpinBox *itemDuplicateYDispSpinBox;
    QSpacerItem *verticalSpacer_2;
    QLabel *label_8;
    QFrame *line_3;
    QFormLayout *formLayout_3;
    QLabel *label_9;
    ScrSpinBox *rotationConstraintSpinBox;
    QSpacerItem *verticalSpacer_3;

    void setupUi(QWidget *Prefs_OperatorTools)
    {
        if (Prefs_OperatorTools->objectName().isEmpty())
            Prefs_OperatorTools->setObjectName(QString::fromUtf8("Prefs_OperatorTools"));
        Prefs_OperatorTools->resize(579, 495);
        verticalLayout = new QVBoxLayout(Prefs_OperatorTools);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(Prefs_OperatorTools);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        verticalLayout->addWidget(label);

        line = new QFrame(Prefs_OperatorTools);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        tabWidget = new QTabWidget(Prefs_OperatorTools);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tabWidget->setTabPosition(QTabWidget::North);
        tabWidget->setTabShape(QTabWidget::Rounded);
        tabWidget->setElideMode(Qt::ElideRight);
        tabWidget->setUsesScrollButtons(false);
        tab_4 = new QWidget();
        tab_4->setObjectName(QString::fromUtf8("tab_4"));
        verticalLayout_2 = new QVBoxLayout(tab_4);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_2 = new QLabel(tab_4);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        formLayout->setWidget(0, QFormLayout::LabelRole, label_2);

        label_3 = new QLabel(tab_4);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        formLayout->setWidget(1, QFormLayout::LabelRole, label_3);

        label_4 = new QLabel(tab_4);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        formLayout->setWidget(2, QFormLayout::LabelRole, label_4);

        zoomMinimumSpinBox = new ScrSpinBox(tab_4);
        zoomMinimumSpinBox->setObjectName(QString::fromUtf8("zoomMinimumSpinBox"));
        zoomMinimumSpinBox->setMinimum(1);
        zoomMinimumSpinBox->setMaximum(3200);

        formLayout->setWidget(0, QFormLayout::FieldRole, zoomMinimumSpinBox);

        zoomMaximumSpinBox = new QSpinBox(tab_4);
        zoomMaximumSpinBox->setObjectName(QString::fromUtf8("zoomMaximumSpinBox"));
        zoomMaximumSpinBox->setMinimum(10);
        zoomMaximumSpinBox->setMaximum(32000);

        formLayout->setWidget(1, QFormLayout::FieldRole, zoomMaximumSpinBox);

        zoomSteppingSpinBox = new QSpinBox(tab_4);
        zoomSteppingSpinBox->setObjectName(QString::fromUtf8("zoomSteppingSpinBox"));
        zoomSteppingSpinBox->setMinimum(1);
        zoomSteppingSpinBox->setMaximum(500);

        formLayout->setWidget(2, QFormLayout::FieldRole, zoomSteppingSpinBox);


        verticalLayout_2->addLayout(formLayout);

        verticalSpacer = new QSpacerItem(20, 231, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        tabWidget->addTab(tab_4, QString());
        tab_5 = new QWidget();
        tab_5->setObjectName(QString::fromUtf8("tab_5"));
        verticalLayout_3 = new QVBoxLayout(tab_5);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        label_5 = new QLabel(tab_5);
        label_5->setObjectName(QString::fromUtf8("label_5"));
        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
        label_5->setFont(font1);

        verticalLayout_3->addWidget(label_5);

        line_2 = new QFrame(tab_5);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        verticalLayout_3->addWidget(line_2);

        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName(QString::fromUtf8("formLayout_2"));
        formLayout_2->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_6 = new QLabel(tab_5);
        label_6->setObjectName(QString::fromUtf8("label_6"));

        formLayout_2->setWidget(0, QFormLayout::LabelRole, label_6);

        label_7 = new QLabel(tab_5);
        label_7->setObjectName(QString::fromUtf8("label_7"));

        formLayout_2->setWidget(1, QFormLayout::LabelRole, label_7);

        itemDuplicateXDispSpinBox = new ScrSpinBox(tab_5);
        itemDuplicateXDispSpinBox->setObjectName(QString::fromUtf8("itemDuplicateXDispSpinBox"));

        formLayout_2->setWidget(0, QFormLayout::FieldRole, itemDuplicateXDispSpinBox);

        itemDuplicateYDispSpinBox = new ScrSpinBox(tab_5);
        itemDuplicateYDispSpinBox->setObjectName(QString::fromUtf8("itemDuplicateYDispSpinBox"));

        formLayout_2->setWidget(1, QFormLayout::FieldRole, itemDuplicateYDispSpinBox);


        verticalLayout_3->addLayout(formLayout_2);

        verticalSpacer_2 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_3->addItem(verticalSpacer_2);

        label_8 = new QLabel(tab_5);
        label_8->setObjectName(QString::fromUtf8("label_8"));
        label_8->setFont(font1);

        verticalLayout_3->addWidget(label_8);

        line_3 = new QFrame(tab_5);
        line_3->setObjectName(QString::fromUtf8("line_3"));
        line_3->setFrameShape(QFrame::HLine);
        line_3->setFrameShadow(QFrame::Sunken);

        verticalLayout_3->addWidget(line_3);

        formLayout_3 = new QFormLayout();
        formLayout_3->setObjectName(QString::fromUtf8("formLayout_3"));
        formLayout_3->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_9 = new QLabel(tab_5);
        label_9->setObjectName(QString::fromUtf8("label_9"));

        formLayout_3->setWidget(0, QFormLayout::LabelRole, label_9);

        rotationConstraintSpinBox = new ScrSpinBox(tab_5);
        rotationConstraintSpinBox->setObjectName(QString::fromUtf8("rotationConstraintSpinBox"));

        formLayout_3->setWidget(0, QFormLayout::FieldRole, rotationConstraintSpinBox);


        verticalLayout_3->addLayout(formLayout_3);

        verticalSpacer_3 = new QSpacerItem(20, 112, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_3);

        tabWidget->addTab(tab_5, QString());

        verticalLayout->addWidget(tabWidget);


        retranslateUi(Prefs_OperatorTools);

        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(Prefs_OperatorTools);
    } // setupUi

    void retranslateUi(QWidget *Prefs_OperatorTools)
    {
        label->setText(QCoreApplication::translate("Prefs_OperatorTools", "Operator Tools", nullptr));
        label_2->setText(QCoreApplication::translate("Prefs_OperatorTools", "Minimum:", nullptr));
        label_3->setText(QCoreApplication::translate("Prefs_OperatorTools", "Maximum:", nullptr));
        label_4->setText(QCoreApplication::translate("Prefs_OperatorTools", "Stepping:", nullptr));
        zoomMinimumSpinBox->setSuffix(QCoreApplication::translate("Prefs_OperatorTools", " %", nullptr));
        zoomMaximumSpinBox->setSuffix(QCoreApplication::translate("Prefs_OperatorTools", " %", nullptr));
        zoomSteppingSpinBox->setSuffix(QCoreApplication::translate("Prefs_OperatorTools", " %", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_4), QCoreApplication::translate("Prefs_OperatorTools", "Zoom", nullptr));
        label_5->setText(QCoreApplication::translate("Prefs_OperatorTools", "Item Duplicate", nullptr));
        label_6->setText(QCoreApplication::translate("Prefs_OperatorTools", "X Displacement:", nullptr));
        label_7->setText(QCoreApplication::translate("Prefs_OperatorTools", "Y Displacement:", nullptr));
        label_8->setText(QCoreApplication::translate("Prefs_OperatorTools", "Rotation Tool", nullptr));
        label_9->setText(QCoreApplication::translate("Prefs_OperatorTools", "Constrain To:", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tab_5), QCoreApplication::translate("Prefs_OperatorTools", "Miscellaneous", nullptr));
        (void)Prefs_OperatorTools;
    } // retranslateUi

};

namespace Ui {
    class Prefs_OperatorTools: public Ui_Prefs_OperatorTools {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_OPERATORTOOLSBASE_H
