/****************************************************************************
** Meta object code from reading C++ file 'guidesdelegate.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/guidesdelegate.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'guidesdelegate.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_GuidesDelegate_t {
    QByteArrayData data[1];
    char stringdata0[15];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_GuidesDelegate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_GuidesDelegate_t qt_meta_stringdata_GuidesDelegate = {
    {
QT_MOC_LITERAL(0, 0, 14) // "GuidesDelegate"

    },
    "GuidesDelegate"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_GuidesDelegate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void GuidesDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject GuidesDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<QItemDelegate::staticMetaObject>(),
    qt_meta_stringdata_GuidesDelegate.data,
    qt_meta_data_GuidesDelegate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *GuidesDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GuidesDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_GuidesDelegate.stringdata0))
        return static_cast<void*>(this);
    return QItemDelegate::qt_metacast(_clname);
}

int GuidesDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QItemDelegate::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_GuidesHDelegate_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_GuidesHDelegate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_GuidesHDelegate_t qt_meta_stringdata_GuidesHDelegate = {
    {
QT_MOC_LITERAL(0, 0, 15) // "GuidesHDelegate"

    },
    "GuidesHDelegate"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_GuidesHDelegate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void GuidesHDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject GuidesHDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<GuidesDelegate::staticMetaObject>(),
    qt_meta_stringdata_GuidesHDelegate.data,
    qt_meta_data_GuidesHDelegate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *GuidesHDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GuidesHDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_GuidesHDelegate.stringdata0))
        return static_cast<void*>(this);
    return GuidesDelegate::qt_metacast(_clname);
}

int GuidesHDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GuidesDelegate::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_GuidesVDelegate_t {
    QByteArrayData data[1];
    char stringdata0[16];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_GuidesVDelegate_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_GuidesVDelegate_t qt_meta_stringdata_GuidesVDelegate = {
    {
QT_MOC_LITERAL(0, 0, 15) // "GuidesVDelegate"

    },
    "GuidesVDelegate"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_GuidesVDelegate[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void GuidesVDelegate::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject GuidesVDelegate::staticMetaObject = { {
    QMetaObject::SuperData::link<GuidesDelegate::staticMetaObject>(),
    qt_meta_stringdata_GuidesVDelegate.data,
    qt_meta_data_GuidesVDelegate,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *GuidesVDelegate::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *GuidesVDelegate::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_GuidesVDelegate.stringdata0))
        return static_cast<void*>(this);
    return GuidesDelegate::qt_metacast(_clname);
}

int GuidesVDelegate::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = GuidesDelegate::qt_metacall(_c, _id, _a);
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
