/********************************************************************************
** Form generated from reading UI file 'outputpreview_pdf.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OUTPUTPREVIEW_PDF_H
#define UI_OUTPUTPREVIEW_PDF_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/pdfversioncombo.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_OutputPreview_PDF
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *previewSettings;
    QVBoxLayout *verticalLayout_2;
    PdfVersionCombo *pdfVersionCombo;
    QComboBox *outputModeCombo;
    QCheckBox *mirrorH;
    QCheckBox *mirrorV;
    QCheckBox *clipToMargins;
    QCheckBox *convertSpots;
    QGroupBox *displaySettings;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *antiAliasing;
    QCheckBox *showTransparency;
    QCheckBox *enableCMYK;
    QTableWidget *inkTable;
    QCheckBox *displayInkCoverage;
    QHBoxLayout *horizontalLayout;
    QLabel *coverThresholdLabel;
    ScrSpinBox *coverThresholdValue;

    void setupUi(QWidget *OutputPreview_PDF)
    {
        if (OutputPreview_PDF->objectName().isEmpty())
            OutputPreview_PDF->setObjectName(QString::fromUtf8("OutputPreview_PDF"));
        OutputPreview_PDF->resize(204, 442);
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(OutputPreview_PDF->sizePolicy().hasHeightForWidth());
        OutputPreview_PDF->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(OutputPreview_PDF);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        previewSettings = new QGroupBox(OutputPreview_PDF);
        previewSettings->setObjectName(QString::fromUtf8("previewSettings"));
        previewSettings->setCheckable(false);
        verticalLayout_2 = new QVBoxLayout(previewSettings);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        pdfVersionCombo = new PdfVersionCombo(previewSettings);
        pdfVersionCombo->setObjectName(QString::fromUtf8("pdfVersionCombo"));

        verticalLayout_2->addWidget(pdfVersionCombo);

        outputModeCombo = new QComboBox(previewSettings);
        outputModeCombo->addItem(QString());
        outputModeCombo->addItem(QString());
        outputModeCombo->addItem(QString());
        outputModeCombo->setObjectName(QString::fromUtf8("outputModeCombo"));

        verticalLayout_2->addWidget(outputModeCombo);

        mirrorH = new QCheckBox(previewSettings);
        mirrorH->setObjectName(QString::fromUtf8("mirrorH"));

        verticalLayout_2->addWidget(mirrorH);

        mirrorV = new QCheckBox(previewSettings);
        mirrorV->setObjectName(QString::fromUtf8("mirrorV"));

        verticalLayout_2->addWidget(mirrorV);

        clipToMargins = new QCheckBox(previewSettings);
        clipToMargins->setObjectName(QString::fromUtf8("clipToMargins"));

        verticalLayout_2->addWidget(clipToMargins);

        convertSpots = new QCheckBox(previewSettings);
        convertSpots->setObjectName(QString::fromUtf8("convertSpots"));

        verticalLayout_2->addWidget(convertSpots);


        verticalLayout->addWidget(previewSettings);

        displaySettings = new QGroupBox(OutputPreview_PDF);
        displaySettings->setObjectName(QString::fromUtf8("displaySettings"));
        verticalLayout_3 = new QVBoxLayout(displaySettings);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        antiAliasing = new QCheckBox(displaySettings);
        antiAliasing->setObjectName(QString::fromUtf8("antiAliasing"));

        verticalLayout_3->addWidget(antiAliasing);

        showTransparency = new QCheckBox(displaySettings);
        showTransparency->setObjectName(QString::fromUtf8("showTransparency"));

        verticalLayout_3->addWidget(showTransparency);

        enableCMYK = new QCheckBox(displaySettings);
        enableCMYK->setObjectName(QString::fromUtf8("enableCMYK"));

        verticalLayout_3->addWidget(enableCMYK);

        inkTable = new QTableWidget(displaySettings);
        inkTable->setObjectName(QString::fromUtf8("inkTable"));
        inkTable->setFocusPolicy(Qt::NoFocus);
        inkTable->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContentsOnFirstShow);
        inkTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
        inkTable->setSelectionMode(QAbstractItemView::NoSelection);
        inkTable->setColumnCount(0);
        inkTable->horizontalHeader()->setStretchLastSection(true);

        verticalLayout_3->addWidget(inkTable);

        displayInkCoverage = new QCheckBox(displaySettings);
        displayInkCoverage->setObjectName(QString::fromUtf8("displayInkCoverage"));

        verticalLayout_3->addWidget(displayInkCoverage);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setSizeConstraint(QLayout::SetMaximumSize);
        coverThresholdLabel = new QLabel(displaySettings);
        coverThresholdLabel->setObjectName(QString::fromUtf8("coverThresholdLabel"));

        horizontalLayout->addWidget(coverThresholdLabel);

        coverThresholdValue = new ScrSpinBox(displaySettings);
        coverThresholdValue->setObjectName(QString::fromUtf8("coverThresholdValue"));
        coverThresholdValue->setDecimals(0);
        coverThresholdValue->setMaximum(400.000000000000000);
        coverThresholdValue->setSingleStep(10.000000000000000);
        coverThresholdValue->setValue(300.000000000000000);

        horizontalLayout->addWidget(coverThresholdValue);


        verticalLayout_3->addLayout(horizontalLayout);


        verticalLayout->addWidget(displaySettings);


        retranslateUi(OutputPreview_PDF);

        QMetaObject::connectSlotsByName(OutputPreview_PDF);
    } // setupUi

    void retranslateUi(QWidget *OutputPreview_PDF)
    {
        previewSettings->setTitle(QCoreApplication::translate("OutputPreview_PDF", "Preview Settings", nullptr));
        outputModeCombo->setItemText(0, QCoreApplication::translate("OutputPreview_PDF", "Screen / Web", nullptr));
        outputModeCombo->setItemText(1, QCoreApplication::translate("OutputPreview_PDF", "Printer", nullptr));
        outputModeCombo->setItemText(2, QCoreApplication::translate("OutputPreview_PDF", "Grayscale", nullptr));

        mirrorH->setText(QCoreApplication::translate("OutputPreview_PDF", "Mirror Page(s) Horizontally", nullptr));
        mirrorV->setText(QCoreApplication::translate("OutputPreview_PDF", "Mirror Page(s) Vertically", nullptr));
        clipToMargins->setText(QCoreApplication::translate("OutputPreview_PDF", "Clip to Printer Margins", nullptr));
#if QT_CONFIG(tooltip)
        convertSpots->setToolTip(QCoreApplication::translate("OutputPreview_PDF", "Enables Spot Colors to be converted to composite colors. Unless you are planning to print spot colors at a commercial printer, this is probably best left enabled.", nullptr));
#endif // QT_CONFIG(tooltip)
        convertSpots->setText(QCoreApplication::translate("OutputPreview_PDF", "Convert Spot Colors", nullptr));
        displaySettings->setTitle(QCoreApplication::translate("OutputPreview_PDF", "Display Settings", nullptr));
#if QT_CONFIG(tooltip)
        antiAliasing->setToolTip(QCoreApplication::translate("OutputPreview_PDF", "Provides a more pleasant view of Type 1 fonts, TrueType Fonts, OpenType Fonts, EPS, PDF and vector graphics in the preview, at the expense of a slight slowdown in previewing", nullptr));
#endif // QT_CONFIG(tooltip)
        antiAliasing->setText(QCoreApplication::translate("OutputPreview_PDF", "Enable &Antialiasing", nullptr));
#if QT_CONFIG(tooltip)
        showTransparency->setToolTip(QCoreApplication::translate("OutputPreview_PDF", "Shows transparency and transparent items in your document. Requires Ghostscript 7.07 or later.", nullptr));
#endif // QT_CONFIG(tooltip)
        showTransparency->setText(QCoreApplication::translate("OutputPreview_PDF", "Display Trans&parency", nullptr));
#if QT_CONFIG(tooltip)
        enableCMYK->setToolTip(QCoreApplication::translate("OutputPreview_PDF", "Gives a print preview using simulations of generic CMYK inks, instead of RGB colors", nullptr));
#endif // QT_CONFIG(tooltip)
        enableCMYK->setText(QCoreApplication::translate("OutputPreview_PDF", "&Display CMYK", nullptr));
        displayInkCoverage->setText(QCoreApplication::translate("OutputPreview_PDF", "Display Ink Coverage", nullptr));
        coverThresholdLabel->setText(QCoreApplication::translate("OutputPreview_PDF", "Threshold:", nullptr));
        (void)OutputPreview_PDF;
    } // retranslateUi

};

namespace Ui {
    class OutputPreview_PDF: public Ui_OutputPreview_PDF {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OUTPUTPREVIEW_PDF_H
