/********************************************************************************
** Form generated from reading UI file 'aboutplugins.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_ABOUTPLUGINS_H
#define UI_ABOUTPLUGINS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QSplitter>
#include <QtWidgets/QTextBrowser>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_AboutPlugins
{
public:
    QVBoxLayout *vboxLayout;
    QSplitter *splitter;
    QListWidget *pluginList;
    QTextBrowser *infoBrowser;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *AboutPlugins)
    {
        if (AboutPlugins->objectName().isEmpty())
            AboutPlugins->setObjectName(QString::fromUtf8("AboutPlugins"));
        AboutPlugins->resize(562, 401);
        vboxLayout = new QVBoxLayout(AboutPlugins);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        vboxLayout->setContentsMargins(9, 9, 9, 9);
        splitter = new QSplitter(AboutPlugins);
        splitter->setObjectName(QString::fromUtf8("splitter"));
        splitter->setOrientation(Qt::Horizontal);
        splitter->setOpaqueResize(false);
        splitter->setChildrenCollapsible(false);
        pluginList = new QListWidget(splitter);
        pluginList->setObjectName(QString::fromUtf8("pluginList"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(pluginList->sizePolicy().hasHeightForWidth());
        pluginList->setSizePolicy(sizePolicy);
        splitter->addWidget(pluginList);
        infoBrowser = new QTextBrowser(splitter);
        infoBrowser->setObjectName(QString::fromUtf8("infoBrowser"));
        splitter->addWidget(infoBrowser);

        vboxLayout->addWidget(splitter);

        buttonBox = new QDialogButtonBox(AboutPlugins);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Close);

        vboxLayout->addWidget(buttonBox);


        retranslateUi(AboutPlugins);
        QObject::connect(buttonBox, SIGNAL(accepted()), AboutPlugins, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), AboutPlugins, SLOT(reject()));

        QMetaObject::connectSlotsByName(AboutPlugins);
    } // setupUi

    void retranslateUi(QDialog *AboutPlugins)
    {
        AboutPlugins->setWindowTitle(QCoreApplication::translate("AboutPlugins", "Scribus: About Plug-ins", nullptr));
    } // retranslateUi

};

namespace Ui {
    class AboutPlugins: public Ui_AboutPlugins {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_ABOUTPLUGINS_H
