/********************************************************************************
** Form generated from reading UI file 'pathconnectdialogbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PATHCONNECTDIALOGBASE_H
#define UI_PATHCONNECTDIALOGBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>

QT_BEGIN_NAMESPACE

class Ui_PathConnectDialogBase
{
public:
    QGridLayout *gridLayout;
    QLabel *label;
    QComboBox *firstLinePoint;
    QLabel *label_2;
    QComboBox *secondLinePoint;
    QLabel *label_3;
    QComboBox *modeCombo;
    QCheckBox *previewCheck;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *PathConnectDialogBase)
    {
        if (PathConnectDialogBase->objectName().isEmpty())
            PathConnectDialogBase->setObjectName(QString::fromUtf8("PathConnectDialogBase"));
        PathConnectDialogBase->resize(235, 160);
        gridLayout = new QGridLayout(PathConnectDialogBase);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(9, 9, 9, 9);
        label = new QLabel(PathConnectDialogBase);
        label->setObjectName(QString::fromUtf8("label"));

        gridLayout->addWidget(label, 0, 0, 1, 1);

        firstLinePoint = new QComboBox(PathConnectDialogBase);
        firstLinePoint->addItem(QString());
        firstLinePoint->addItem(QString());
        firstLinePoint->setObjectName(QString::fromUtf8("firstLinePoint"));

        gridLayout->addWidget(firstLinePoint, 0, 1, 1, 1);

        label_2 = new QLabel(PathConnectDialogBase);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 1, 0, 1, 1);

        secondLinePoint = new QComboBox(PathConnectDialogBase);
        secondLinePoint->addItem(QString());
        secondLinePoint->addItem(QString());
        secondLinePoint->setObjectName(QString::fromUtf8("secondLinePoint"));

        gridLayout->addWidget(secondLinePoint, 1, 1, 1, 1);

        label_3 = new QLabel(PathConnectDialogBase);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 2, 0, 1, 1);

        modeCombo = new QComboBox(PathConnectDialogBase);
        modeCombo->addItem(QString());
        modeCombo->addItem(QString());
        modeCombo->setObjectName(QString::fromUtf8("modeCombo"));

        gridLayout->addWidget(modeCombo, 2, 1, 1, 1);

        previewCheck = new QCheckBox(PathConnectDialogBase);
        previewCheck->setObjectName(QString::fromUtf8("previewCheck"));

        gridLayout->addWidget(previewCheck, 3, 0, 1, 2);

        buttonBox = new QDialogButtonBox(PathConnectDialogBase);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);
        buttonBox->setCenterButtons(false);

        gridLayout->addWidget(buttonBox, 4, 0, 1, 2);


        retranslateUi(PathConnectDialogBase);
        QObject::connect(buttonBox, SIGNAL(accepted()), PathConnectDialogBase, SLOT(accept()));
        QObject::connect(buttonBox, SIGNAL(rejected()), PathConnectDialogBase, SLOT(reject()));

        QMetaObject::connectSlotsByName(PathConnectDialogBase);
    } // setupUi

    void retranslateUi(QDialog *PathConnectDialogBase)
    {
        PathConnectDialogBase->setWindowTitle(QCoreApplication::translate("PathConnectDialogBase", "Connect Paths", nullptr));
        label->setText(QCoreApplication::translate("PathConnectDialogBase", "Connect First Line", nullptr));
        firstLinePoint->setItemText(0, QCoreApplication::translate("PathConnectDialogBase", "Starting Point", nullptr));
        firstLinePoint->setItemText(1, QCoreApplication::translate("PathConnectDialogBase", "End Point", nullptr));

        label_2->setText(QCoreApplication::translate("PathConnectDialogBase", "with Second Line", nullptr));
        secondLinePoint->setItemText(0, QCoreApplication::translate("PathConnectDialogBase", "Starting Point", nullptr));
        secondLinePoint->setItemText(1, QCoreApplication::translate("PathConnectDialogBase", "End Point", nullptr));

        label_3->setText(QCoreApplication::translate("PathConnectDialogBase", "by using:", nullptr));
        modeCombo->setItemText(0, QCoreApplication::translate("PathConnectDialogBase", "a straight Line", nullptr));
        modeCombo->setItemText(1, QCoreApplication::translate("PathConnectDialogBase", "Points moving", nullptr));

        previewCheck->setText(QCoreApplication::translate("PathConnectDialogBase", "Preview on Canvas", nullptr));
    } // retranslateUi

};

namespace Ui {
    class PathConnectDialogBase: public Ui_PathConnectDialogBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PATHCONNECTDIALOGBASE_H
