/****************************************************************************
** Meta object code from reading C++ file 'aligndistribute.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/aligndistribute.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'aligndistribute.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_AlignDistributePalette_t {
    QByteArrayData data[36];
    char stringdata0[562];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_AlignDistributePalette_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_AlignDistributePalette_t qt_meta_stringdata_AlignDistributePalette = {
    {
QT_MOC_LITERAL(0, 0, 22), // "AlignDistributePalette"
QT_MOC_LITERAL(1, 23, 15), // "documentChanged"
QT_MOC_LITERAL(2, 39, 0), // ""
QT_MOC_LITERAL(3, 40, 8), // "setGuide"
QT_MOC_LITERAL(4, 49, 12), // "localeChange"
QT_MOC_LITERAL(5, 62, 13), // "iconSetChange"
QT_MOC_LITERAL(6, 76, 14), // "languageChange"
QT_MOC_LITERAL(7, 91, 12), // "alignLeftOut"
QT_MOC_LITERAL(8, 104, 13), // "alignRightOut"
QT_MOC_LITERAL(9, 118, 13), // "alignBottomIn"
QT_MOC_LITERAL(10, 132, 12), // "alignRightIn"
QT_MOC_LITERAL(11, 145, 14), // "alignBottomOut"
QT_MOC_LITERAL(12, 160, 14), // "alignCenterHor"
QT_MOC_LITERAL(13, 175, 11), // "alignLeftIn"
QT_MOC_LITERAL(14, 187, 14), // "alignCenterVer"
QT_MOC_LITERAL(15, 202, 11), // "alignTopOut"
QT_MOC_LITERAL(16, 214, 10), // "alignTopIn"
QT_MOC_LITERAL(17, 225, 15), // "distributeDistH"
QT_MOC_LITERAL(18, 241, 13), // "usingDistance"
QT_MOC_LITERAL(19, 255, 18), // "distributeDistValH"
QT_MOC_LITERAL(20, 274, 24), // "distributeDistAcrossPage"
QT_MOC_LITERAL(21, 299, 27), // "distributeDistAcrossMargins"
QT_MOC_LITERAL(22, 327, 15), // "distributeRight"
QT_MOC_LITERAL(23, 343, 16), // "distributeBottom"
QT_MOC_LITERAL(24, 360, 17), // "distributeCenterH"
QT_MOC_LITERAL(25, 378, 15), // "distributeDistV"
QT_MOC_LITERAL(26, 394, 18), // "distributeDistValV"
QT_MOC_LITERAL(27, 413, 22), // "distributeDistDownPage"
QT_MOC_LITERAL(28, 436, 25), // "distributeDistDownMargins"
QT_MOC_LITERAL(29, 462, 14), // "distributeLeft"
QT_MOC_LITERAL(30, 477, 17), // "distributeCenterV"
QT_MOC_LITERAL(31, 495, 13), // "distributeTop"
QT_MOC_LITERAL(32, 509, 8), // "swapLeft"
QT_MOC_LITERAL(33, 518, 9), // "swapRight"
QT_MOC_LITERAL(34, 528, 14), // "alignToChanged"
QT_MOC_LITERAL(35, 543, 18) // "alignMethodChanged"

    },
    "AlignDistributePalette\0documentChanged\0"
    "\0setGuide\0localeChange\0iconSetChange\0"
    "languageChange\0alignLeftOut\0alignRightOut\0"
    "alignBottomIn\0alignRightIn\0alignBottomOut\0"
    "alignCenterHor\0alignLeftIn\0alignCenterVer\0"
    "alignTopOut\0alignTopIn\0distributeDistH\0"
    "usingDistance\0distributeDistValH\0"
    "distributeDistAcrossPage\0"
    "distributeDistAcrossMargins\0distributeRight\0"
    "distributeBottom\0distributeCenterH\0"
    "distributeDistV\0distributeDistValV\0"
    "distributeDistDownPage\0distributeDistDownMargins\0"
    "distributeLeft\0distributeCenterV\0"
    "distributeTop\0swapLeft\0swapRight\0"
    "alignToChanged\0alignMethodChanged"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_AlignDistributePalette[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      35,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  189,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    2,  190,    2, 0x0a /* Public */,
       4,    0,  195,    2, 0x0a /* Public */,
       5,    0,  196,    2, 0x09 /* Protected */,
       6,    0,  197,    2, 0x09 /* Protected */,
       7,    0,  198,    2, 0x09 /* Protected */,
       8,    0,  199,    2, 0x09 /* Protected */,
       9,    0,  200,    2, 0x09 /* Protected */,
      10,    0,  201,    2, 0x09 /* Protected */,
      11,    0,  202,    2, 0x09 /* Protected */,
      12,    0,  203,    2, 0x09 /* Protected */,
      13,    0,  204,    2, 0x09 /* Protected */,
      14,    0,  205,    2, 0x09 /* Protected */,
      15,    0,  206,    2, 0x09 /* Protected */,
      16,    0,  207,    2, 0x09 /* Protected */,
      17,    1,  208,    2, 0x09 /* Protected */,
      17,    0,  211,    2, 0x29 /* Protected | MethodCloned */,
      19,    0,  212,    2, 0x09 /* Protected */,
      20,    0,  213,    2, 0x09 /* Protected */,
      21,    0,  214,    2, 0x09 /* Protected */,
      22,    0,  215,    2, 0x09 /* Protected */,
      23,    0,  216,    2, 0x09 /* Protected */,
      24,    0,  217,    2, 0x09 /* Protected */,
      25,    1,  218,    2, 0x09 /* Protected */,
      25,    0,  221,    2, 0x29 /* Protected | MethodCloned */,
      26,    0,  222,    2, 0x09 /* Protected */,
      27,    0,  223,    2, 0x09 /* Protected */,
      28,    0,  224,    2, 0x09 /* Protected */,
      29,    0,  225,    2, 0x09 /* Protected */,
      30,    0,  226,    2, 0x09 /* Protected */,
      31,    0,  227,    2, 0x09 /* Protected */,
      32,    0,  228,    2, 0x09 /* Protected */,
      33,    0,  229,    2, 0x09 /* Protected */,
      34,    1,  230,    2, 0x09 /* Protected */,
      35,    1,  233,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::Int, QMetaType::QReal,    2,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   18,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,

       0        // eod
};

void AlignDistributePalette::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<AlignDistributePalette *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->documentChanged(); break;
        case 1: _t->setGuide((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< qreal(*)>(_a[2]))); break;
        case 2: _t->localeChange(); break;
        case 3: _t->iconSetChange(); break;
        case 4: _t->languageChange(); break;
        case 5: _t->alignLeftOut(); break;
        case 6: _t->alignRightOut(); break;
        case 7: _t->alignBottomIn(); break;
        case 8: _t->alignRightIn(); break;
        case 9: _t->alignBottomOut(); break;
        case 10: _t->alignCenterHor(); break;
        case 11: _t->alignLeftIn(); break;
        case 12: _t->alignCenterVer(); break;
        case 13: _t->alignTopOut(); break;
        case 14: _t->alignTopIn(); break;
        case 15: _t->distributeDistH((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 16: _t->distributeDistH(); break;
        case 17: _t->distributeDistValH(); break;
        case 18: _t->distributeDistAcrossPage(); break;
        case 19: _t->distributeDistAcrossMargins(); break;
        case 20: _t->distributeRight(); break;
        case 21: _t->distributeBottom(); break;
        case 22: _t->distributeCenterH(); break;
        case 23: _t->distributeDistV((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 24: _t->distributeDistV(); break;
        case 25: _t->distributeDistValV(); break;
        case 26: _t->distributeDistDownPage(); break;
        case 27: _t->distributeDistDownMargins(); break;
        case 28: _t->distributeLeft(); break;
        case 29: _t->distributeCenterV(); break;
        case 30: _t->distributeTop(); break;
        case 31: _t->swapLeft(); break;
        case 32: _t->swapRight(); break;
        case 33: _t->alignToChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 34: _t->alignMethodChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (AlignDistributePalette::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&AlignDistributePalette::documentChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject AlignDistributePalette::staticMetaObject = { {
    QMetaObject::SuperData::link<ScDockPalette::staticMetaObject>(),
    qt_meta_stringdata_AlignDistributePalette.data,
    qt_meta_data_AlignDistributePalette,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *AlignDistributePalette::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *AlignDistributePalette::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_AlignDistributePalette.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::AlignDistribute"))
        return static_cast< Ui::AlignDistribute*>(this);
    return ScDockPalette::qt_metacast(_clname);
}

int AlignDistributePalette::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScDockPalette::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 35)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 35;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 35)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 35;
    }
    return _id;
}

// SIGNAL 0
void AlignDistributePalette::documentChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
