/********************************************************************************
** Form generated from reading UI file 'resourcemanagerlicensebase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RESOURCEMANAGERLICENSEBASE_H
#define UI_RESOURCEMANAGERLICENSEBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QTextEdit>
#include <QtWidgets/QVBoxLayout>

QT_BEGIN_NAMESPACE

class Ui_ResourceManagerLicenseBase
{
public:
    QVBoxLayout *verticalLayout;
    QTextEdit *licenseTextEdit;

    void setupUi(QDialog *ResourceManagerLicenseBase)
    {
        if (ResourceManagerLicenseBase->objectName().isEmpty())
            ResourceManagerLicenseBase->setObjectName(QString::fromUtf8("ResourceManagerLicenseBase"));
        ResourceManagerLicenseBase->resize(480, 640);
        verticalLayout = new QVBoxLayout(ResourceManagerLicenseBase);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        licenseTextEdit = new QTextEdit(ResourceManagerLicenseBase);
        licenseTextEdit->setObjectName(QString::fromUtf8("licenseTextEdit"));
        licenseTextEdit->setDocumentTitle(QString::fromUtf8(""));
        licenseTextEdit->setReadOnly(true);
        licenseTextEdit->setHtml(QString::fromUtf8("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'.Helvetica Neue DeskInterface'; font-size:13pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"));

        verticalLayout->addWidget(licenseTextEdit);


        retranslateUi(ResourceManagerLicenseBase);

        QMetaObject::connectSlotsByName(ResourceManagerLicenseBase);
    } // setupUi

    void retranslateUi(QDialog *ResourceManagerLicenseBase)
    {
        ResourceManagerLicenseBase->setWindowTitle(QCoreApplication::translate("ResourceManagerLicenseBase", "License", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ResourceManagerLicenseBase: public Ui_ResourceManagerLicenseBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RESOURCEMANAGERLICENSEBASE_H
