/********************************************************************************
** Form generated from reading UI file 'colorsandfillsbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_COLORSANDFILLSBASE_H
#define UI_COLORSANDFILLSBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTreeWidget>
#include <QtWidgets/QVBoxLayout>
#include "ui/swatchcombo.h"

QT_BEGIN_NAMESPACE

class Ui_ColorsAndFillsBase
{
public:
    QHBoxLayout *horizontalLayout;
    QTreeWidget *dataTree;
    QVBoxLayout *verticalLayout_2;
    QPushButton *importButton;
    QPushButton *newButton;
    QPushButton *editButton;
    QPushButton *duplicateButton;
    QPushButton *deleteButton;
    QPushButton *deleteUnusedButton;
    QGroupBox *groupBox_3;
    QVBoxLayout *verticalLayout;
    QLabel *label;
    SwatchCombo *LoadColSet;
    QPushButton *SaveColSet;
    QSpacerItem *verticalSpacer;
    QVBoxLayout *verticalLayout_4;
    QPushButton *okButton;
    QPushButton *cancelButton;

    void setupUi(QDialog *ColorsAndFillsBase)
    {
        if (ColorsAndFillsBase->objectName().isEmpty())
            ColorsAndFillsBase->setObjectName(QString::fromUtf8("ColorsAndFillsBase"));
        ColorsAndFillsBase->resize(412, 412);
        ColorsAndFillsBase->setSizeGripEnabled(false);
        ColorsAndFillsBase->setModal(true);
        horizontalLayout = new QHBoxLayout(ColorsAndFillsBase);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setContentsMargins(9, 9, 9, 9);
        dataTree = new QTreeWidget(ColorsAndFillsBase);
        dataTree->setObjectName(QString::fromUtf8("dataTree"));
        dataTree->header()->setVisible(false);

        horizontalLayout->addWidget(dataTree);

        verticalLayout_2 = new QVBoxLayout();
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        importButton = new QPushButton(ColorsAndFillsBase);
        importButton->setObjectName(QString::fromUtf8("importButton"));

        verticalLayout_2->addWidget(importButton);

        newButton = new QPushButton(ColorsAndFillsBase);
        newButton->setObjectName(QString::fromUtf8("newButton"));

        verticalLayout_2->addWidget(newButton);

        editButton = new QPushButton(ColorsAndFillsBase);
        editButton->setObjectName(QString::fromUtf8("editButton"));

        verticalLayout_2->addWidget(editButton);

        duplicateButton = new QPushButton(ColorsAndFillsBase);
        duplicateButton->setObjectName(QString::fromUtf8("duplicateButton"));

        verticalLayout_2->addWidget(duplicateButton);

        deleteButton = new QPushButton(ColorsAndFillsBase);
        deleteButton->setObjectName(QString::fromUtf8("deleteButton"));

        verticalLayout_2->addWidget(deleteButton);

        deleteUnusedButton = new QPushButton(ColorsAndFillsBase);
        deleteUnusedButton->setObjectName(QString::fromUtf8("deleteUnusedButton"));

        verticalLayout_2->addWidget(deleteUnusedButton);

        groupBox_3 = new QGroupBox(ColorsAndFillsBase);
        groupBox_3->setObjectName(QString::fromUtf8("groupBox_3"));
        verticalLayout = new QVBoxLayout(groupBox_3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        label = new QLabel(groupBox_3);
        label->setObjectName(QString::fromUtf8("label"));

        verticalLayout->addWidget(label);

        LoadColSet = new SwatchCombo(groupBox_3);
        LoadColSet->setObjectName(QString::fromUtf8("LoadColSet"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(LoadColSet->sizePolicy().hasHeightForWidth());
        LoadColSet->setSizePolicy(sizePolicy);
        LoadColSet->setPopupMode(QToolButton::MenuButtonPopup);

        verticalLayout->addWidget(LoadColSet);

        SaveColSet = new QPushButton(groupBox_3);
        SaveColSet->setObjectName(QString::fromUtf8("SaveColSet"));

        verticalLayout->addWidget(SaveColSet);


        verticalLayout_2->addWidget(groupBox_3);

        verticalSpacer = new QSpacerItem(115, 13, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        verticalLayout_4 = new QVBoxLayout();
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        okButton = new QPushButton(ColorsAndFillsBase);
        okButton->setObjectName(QString::fromUtf8("okButton"));

        verticalLayout_4->addWidget(okButton);

        cancelButton = new QPushButton(ColorsAndFillsBase);
        cancelButton->setObjectName(QString::fromUtf8("cancelButton"));

        verticalLayout_4->addWidget(cancelButton);


        verticalLayout_2->addLayout(verticalLayout_4);


        horizontalLayout->addLayout(verticalLayout_2);

        horizontalLayout->setStretch(0, 1);

        retranslateUi(ColorsAndFillsBase);

        QMetaObject::connectSlotsByName(ColorsAndFillsBase);
    } // setupUi

    void retranslateUi(QDialog *ColorsAndFillsBase)
    {
        ColorsAndFillsBase->setWindowTitle(QCoreApplication::translate("ColorsAndFillsBase", "Colors and Fills", nullptr));
        QTreeWidgetItem *___qtreewidgetitem = dataTree->headerItem();
        ___qtreewidgetitem->setText(0, QCoreApplication::translate("ColorsAndFillsBase", "1", nullptr));
#if QT_CONFIG(tooltip)
        dataTree->setToolTip(QCoreApplication::translate("ColorsAndFillsBase", "If color management is enabled, an exclamation mark indicates that the color may be outside of the color gamut of the current printer profile selected. What this means is the color may not print exactly as indicated on screen. Spot colors are indicated by a red circle. More hints about gamut warnings are in the online help under Color Management. Registration colors will have a registration mark next to the color. Use Registration only for printers marks and crop marks.", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        importButton->setToolTip(QCoreApplication::translate("ColorsAndFillsBase", "Import colors to the current set from an existing document", nullptr));
#endif // QT_CONFIG(tooltip)
        importButton->setText(QCoreApplication::translate("ColorsAndFillsBase", "&Import", nullptr));
#if QT_CONFIG(tooltip)
        newButton->setToolTip(QCoreApplication::translate("ColorsAndFillsBase", "Create a new color within the current set", nullptr));
#endif // QT_CONFIG(tooltip)
        newButton->setText(QCoreApplication::translate("ColorsAndFillsBase", "Add", nullptr));
#if QT_CONFIG(tooltip)
        editButton->setToolTip(QCoreApplication::translate("ColorsAndFillsBase", "Edit the currently selected color", nullptr));
#endif // QT_CONFIG(tooltip)
        editButton->setText(QCoreApplication::translate("ColorsAndFillsBase", "&Edit", nullptr));
#if QT_CONFIG(tooltip)
        duplicateButton->setToolTip(QCoreApplication::translate("ColorsAndFillsBase", "Make a copy of the currently selected color", nullptr));
#endif // QT_CONFIG(tooltip)
        duplicateButton->setText(QCoreApplication::translate("ColorsAndFillsBase", "D&uplicate", nullptr));
#if QT_CONFIG(tooltip)
        deleteButton->setToolTip(QCoreApplication::translate("ColorsAndFillsBase", "Delete the currently selected color", nullptr));
#endif // QT_CONFIG(tooltip)
        deleteButton->setText(QCoreApplication::translate("ColorsAndFillsBase", "&Delete", nullptr));
#if QT_CONFIG(tooltip)
        deleteUnusedButton->setToolTip(QCoreApplication::translate("ColorsAndFillsBase", "Remove unused colors from current document's color set", nullptr));
#endif // QT_CONFIG(tooltip)
        deleteUnusedButton->setText(QCoreApplication::translate("ColorsAndFillsBase", "&Remove Unused", nullptr));
        groupBox_3->setTitle(QCoreApplication::translate("ColorsAndFillsBase", "Color Sets", nullptr));
        label->setText(QCoreApplication::translate("ColorsAndFillsBase", "Current Color Set:", nullptr));
#if QT_CONFIG(tooltip)
        SaveColSet->setToolTip(QCoreApplication::translate("ColorsAndFillsBase", "Save the current color set", nullptr));
#endif // QT_CONFIG(tooltip)
        SaveColSet->setText(QCoreApplication::translate("ColorsAndFillsBase", "&Save Color Set", nullptr));
#if QT_CONFIG(tooltip)
        okButton->setToolTip(QCoreApplication::translate("ColorsAndFillsBase", "Make the current colorset the default color set", nullptr));
#endif // QT_CONFIG(tooltip)
        okButton->setText(QCoreApplication::translate("ColorsAndFillsBase", "OK", nullptr));
        cancelButton->setText(QCoreApplication::translate("ColorsAndFillsBase", "Cancel", nullptr));
    } // retranslateUi

};

namespace Ui {
    class ColorsAndFillsBase: public Ui_ColorsAndFillsBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_COLORSANDFILLSBASE_H
