/****************************************************************************
** Meta object code from reading C++ file 'propertywidget_distance.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/propertywidget_distance.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertywidget_distance.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PropertyWidget_Distance_t {
    QByteArrayData data[30];
    char stringdata0[344];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PropertyWidget_Distance_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PropertyWidget_Distance_t qt_meta_stringdata_PropertyWidget_Distance = {
    {
QT_MOC_LITERAL(0, 0, 23), // "PropertyWidget_Distance"
QT_MOC_LITERAL(1, 24, 13), // "setMainWindow"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 18), // "ScribusMainWindow*"
QT_MOC_LITERAL(4, 58, 2), // "mw"
QT_MOC_LITERAL(5, 61, 6), // "setDoc"
QT_MOC_LITERAL(6, 68, 11), // "ScribusDoc*"
QT_MOC_LITERAL(7, 80, 1), // "d"
QT_MOC_LITERAL(8, 82, 20), // "handleAppModeChanged"
QT_MOC_LITERAL(9, 103, 7), // "oldMode"
QT_MOC_LITERAL(10, 111, 4), // "mode"
QT_MOC_LITERAL(11, 116, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(12, 139, 19), // "handleUpdateRequest"
QT_MOC_LITERAL(13, 159, 14), // "languageChange"
QT_MOC_LITERAL(14, 174, 10), // "unitChange"
QT_MOC_LITERAL(15, 185, 12), // "localeChange"
QT_MOC_LITERAL(16, 198, 11), // "showColumns"
QT_MOC_LITERAL(17, 210, 1), // "r"
QT_MOC_LITERAL(18, 212, 1), // "g"
QT_MOC_LITERAL(19, 214, 17), // "showTextDistances"
QT_MOC_LITERAL(20, 232, 4), // "left"
QT_MOC_LITERAL(21, 237, 3), // "top"
QT_MOC_LITERAL(22, 241, 6), // "bottom"
QT_MOC_LITERAL(23, 248, 5), // "right"
QT_MOC_LITERAL(24, 254, 13), // "handleColumns"
QT_MOC_LITERAL(25, 268, 15), // "handleColumnGap"
QT_MOC_LITERAL(26, 284, 15), // "handleGapSwitch"
QT_MOC_LITERAL(27, 300, 10), // "handleTabs"
QT_MOC_LITERAL(28, 311, 12), // "handleVAlign"
QT_MOC_LITERAL(29, 324, 19) // "handleTextDistances"

    },
    "PropertyWidget_Distance\0setMainWindow\0"
    "\0ScribusMainWindow*\0mw\0setDoc\0ScribusDoc*\0"
    "d\0handleAppModeChanged\0oldMode\0mode\0"
    "handleSelectionChanged\0handleUpdateRequest\0"
    "languageChange\0unitChange\0localeChange\0"
    "showColumns\0r\0g\0showTextDistances\0"
    "left\0top\0bottom\0right\0handleColumns\0"
    "handleColumnGap\0handleGapSwitch\0"
    "handleTabs\0handleVAlign\0handleTextDistances"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PropertyWidget_Distance[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      16,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   94,    2, 0x0a /* Public */,
       5,    1,   97,    2, 0x0a /* Public */,
       8,    2,  100,    2, 0x0a /* Public */,
      11,    0,  105,    2, 0x0a /* Public */,
      12,    1,  106,    2, 0x0a /* Public */,
      13,    0,  109,    2, 0x0a /* Public */,
      14,    0,  110,    2, 0x0a /* Public */,
      15,    0,  111,    2, 0x0a /* Public */,
      16,    2,  112,    2, 0x0a /* Public */,
      19,    4,  117,    2, 0x0a /* Public */,
      24,    0,  126,    2, 0x08 /* Private */,
      25,    0,  127,    2, 0x08 /* Private */,
      26,    0,  128,    2, 0x08 /* Private */,
      27,    0,  129,    2, 0x08 /* Private */,
      28,    0,  130,    2, 0x08 /* Private */,
      29,    0,  131,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,    9,   10,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Double,   17,   18,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double,   20,   21,   22,   23,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PropertyWidget_Distance::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertyWidget_Distance *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setMainWindow((*reinterpret_cast< ScribusMainWindow*(*)>(_a[1]))); break;
        case 1: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 2: _t->handleAppModeChanged((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 3: _t->handleSelectionChanged(); break;
        case 4: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->languageChange(); break;
        case 6: _t->unitChange(); break;
        case 7: _t->localeChange(); break;
        case 8: _t->showColumns((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 9: _t->showTextDistances((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4]))); break;
        case 10: _t->handleColumns(); break;
        case 11: _t->handleColumnGap(); break;
        case 12: _t->handleGapSwitch(); break;
        case 13: _t->handleTabs(); break;
        case 14: _t->handleVAlign(); break;
        case 15: _t->handleTextDistances(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PropertyWidget_Distance::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_PropertyWidget_Distance.data,
    qt_meta_data_PropertyWidget_Distance,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PropertyWidget_Distance::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PropertyWidget_Distance::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PropertyWidget_Distance.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PropertyWidget_DistanceBase"))
        return static_cast< Ui::PropertyWidget_DistanceBase*>(this);
    if (!strcmp(_clname, "PropertyWidgetBase"))
        return static_cast< PropertyWidgetBase*>(this);
    return QFrame::qt_metacast(_clname);
}

int PropertyWidget_Distance::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 16)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 16;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 16)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 16;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
