/********************************************************************************
** Form generated from reading UI file 'prefs_imagecachebase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_IMAGECACHEBASE_H
#define UI_PREFS_IMAGECACHEBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QLabel>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Prefs_ImageCache
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QFrame *line;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_2;
    QCheckBox *enableImageCacheCheckBox;
    QFormLayout *formLayout;
    QLabel *cacheSizeLimitLabel;
    QSpinBox *cacheSizeLimitSpinBox;
    QLabel *cacheEntryLimitLabel;
    QSpinBox *cacheEntryLimitSpinBox;
    QLabel *compressionLevelLabel;
    QSpinBox *compressionLevelSpinBox;
    QSpacerItem *verticalSpacer;

    void setupUi(QWidget *Prefs_ImageCache)
    {
        if (Prefs_ImageCache->objectName().isEmpty())
            Prefs_ImageCache->setObjectName(QString::fromUtf8("Prefs_ImageCache"));
        Prefs_ImageCache->resize(582, 277);
        verticalLayout = new QVBoxLayout(Prefs_ImageCache);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(Prefs_ImageCache);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        verticalLayout->addWidget(label);

        line = new QFrame(Prefs_ImageCache);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        scrollArea = new QScrollArea(Prefs_ImageCache);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setFrameShape(QFrame::NoFrame);
        scrollArea->setFrameShadow(QFrame::Plain);
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 556, 209));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(0, 0, 0, -1);
        enableImageCacheCheckBox = new QCheckBox(scrollAreaWidgetContents);
        enableImageCacheCheckBox->setObjectName(QString::fromUtf8("enableImageCacheCheckBox"));

        verticalLayout_2->addWidget(enableImageCacheCheckBox);

        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        cacheSizeLimitLabel = new QLabel(scrollAreaWidgetContents);
        cacheSizeLimitLabel->setObjectName(QString::fromUtf8("cacheSizeLimitLabel"));
        cacheSizeLimitLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        cacheSizeLimitLabel->setWordWrap(false);

        formLayout->setWidget(0, QFormLayout::LabelRole, cacheSizeLimitLabel);

        cacheSizeLimitSpinBox = new QSpinBox(scrollAreaWidgetContents);
        cacheSizeLimitSpinBox->setObjectName(QString::fromUtf8("cacheSizeLimitSpinBox"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(cacheSizeLimitSpinBox->sizePolicy().hasHeightForWidth());
        cacheSizeLimitSpinBox->setSizePolicy(sizePolicy);
        cacheSizeLimitSpinBox->setMinimumSize(QSize(100, 0));
        cacheSizeLimitSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        cacheSizeLimitSpinBox->setButtonSymbols(QAbstractSpinBox::UpDownArrows);
        cacheSizeLimitSpinBox->setMinimum(100);
        cacheSizeLimitSpinBox->setMaximum(1000000);
        cacheSizeLimitSpinBox->setSingleStep(100);
        cacheSizeLimitSpinBox->setValue(1000);

        formLayout->setWidget(0, QFormLayout::FieldRole, cacheSizeLimitSpinBox);

        cacheEntryLimitLabel = new QLabel(scrollAreaWidgetContents);
        cacheEntryLimitLabel->setObjectName(QString::fromUtf8("cacheEntryLimitLabel"));
        cacheEntryLimitLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        cacheEntryLimitLabel->setWordWrap(false);

        formLayout->setWidget(1, QFormLayout::LabelRole, cacheEntryLimitLabel);

        cacheEntryLimitSpinBox = new QSpinBox(scrollAreaWidgetContents);
        cacheEntryLimitSpinBox->setObjectName(QString::fromUtf8("cacheEntryLimitSpinBox"));
        sizePolicy.setHeightForWidth(cacheEntryLimitSpinBox->sizePolicy().hasHeightForWidth());
        cacheEntryLimitSpinBox->setSizePolicy(sizePolicy);
        cacheEntryLimitSpinBox->setMinimumSize(QSize(100, 0));
        cacheEntryLimitSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        cacheEntryLimitSpinBox->setMinimum(100);
        cacheEntryLimitSpinBox->setMaximum(100000);
        cacheEntryLimitSpinBox->setSingleStep(100);
        cacheEntryLimitSpinBox->setValue(1000);

        formLayout->setWidget(1, QFormLayout::FieldRole, cacheEntryLimitSpinBox);

        compressionLevelLabel = new QLabel(scrollAreaWidgetContents);
        compressionLevelLabel->setObjectName(QString::fromUtf8("compressionLevelLabel"));
        compressionLevelLabel->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);

        formLayout->setWidget(2, QFormLayout::LabelRole, compressionLevelLabel);

        compressionLevelSpinBox = new QSpinBox(scrollAreaWidgetContents);
        compressionLevelSpinBox->setObjectName(QString::fromUtf8("compressionLevelSpinBox"));
        sizePolicy.setHeightForWidth(compressionLevelSpinBox->sizePolicy().hasHeightForWidth());
        compressionLevelSpinBox->setSizePolicy(sizePolicy);
        compressionLevelSpinBox->setMinimumSize(QSize(100, 0));
        compressionLevelSpinBox->setAlignment(Qt::AlignRight|Qt::AlignTrailing|Qt::AlignVCenter);
        compressionLevelSpinBox->setMinimum(0);
        compressionLevelSpinBox->setMaximum(9);
        compressionLevelSpinBox->setValue(6);

        formLayout->setWidget(2, QFormLayout::FieldRole, compressionLevelSpinBox);


        verticalLayout_2->addLayout(formLayout);

        verticalSpacer = new QSpacerItem(20, 27, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout->addWidget(scrollArea);

        QWidget::setTabOrder(scrollArea, enableImageCacheCheckBox);
        QWidget::setTabOrder(enableImageCacheCheckBox, cacheSizeLimitSpinBox);
        QWidget::setTabOrder(cacheSizeLimitSpinBox, cacheEntryLimitSpinBox);

        retranslateUi(Prefs_ImageCache);

        QMetaObject::connectSlotsByName(Prefs_ImageCache);
    } // setupUi

    void retranslateUi(QWidget *Prefs_ImageCache)
    {
        label->setText(QCoreApplication::translate("Prefs_ImageCache", "Image Cache", nullptr));
        enableImageCacheCheckBox->setText(QCoreApplication::translate("Prefs_ImageCache", "Enable Image Cache", nullptr));
        cacheSizeLimitLabel->setText(QCoreApplication::translate("Prefs_ImageCache", "Cache Size Limit:", nullptr));
        cacheSizeLimitSpinBox->setSuffix(QCoreApplication::translate("Prefs_ImageCache", " Mb", nullptr));
        cacheEntryLimitLabel->setText(QCoreApplication::translate("Prefs_ImageCache", "Cache Entry Limit:", nullptr));
        compressionLevelLabel->setText(QCoreApplication::translate("Prefs_ImageCache", "Compression Level:", nullptr));
        (void)Prefs_ImageCache;
    } // retranslateUi

};

namespace Ui {
    class Prefs_ImageCache: public Ui_Prefs_ImageCache {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_IMAGECACHEBASE_H
