/****************************************************************************
** Meta object code from reading C++ file 'colorpalette.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/colorpalette.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'colorpalette.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ColorPalette_t {
    QByteArrayData data[77];
    char stringdata0[1116];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ColorPalette_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ColorPalette_t qt_meta_stringdata_ColorPalette = {
    {
QT_MOC_LITERAL(0, 0, 12), // "ColorPalette"
QT_MOC_LITERAL(1, 13, 6), // "NewPen"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 8), // "NewBrush"
QT_MOC_LITERAL(4, 30, 11), // "NewPenShade"
QT_MOC_LITERAL(5, 42, 13), // "NewBrushShade"
QT_MOC_LITERAL(6, 56, 11), // "NewGradient"
QT_MOC_LITERAL(7, 68, 12), // "NewGradientS"
QT_MOC_LITERAL(8, 81, 10), // "NewPattern"
QT_MOC_LITERAL(9, 92, 15), // "NewPatternProps"
QT_MOC_LITERAL(10, 108, 11), // "NewPatternS"
QT_MOC_LITERAL(11, 120, 15), // "NewPatternTypeS"
QT_MOC_LITERAL(12, 136, 16), // "NewPatternPropsS"
QT_MOC_LITERAL(13, 153, 10), // "NewSpecial"
QT_MOC_LITERAL(14, 164, 12), // "NewOverprint"
QT_MOC_LITERAL(15, 177, 12), // "editGradient"
QT_MOC_LITERAL(16, 190, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(17, 213, 19), // "handleUpdateRequest"
QT_MOC_LITERAL(18, 233, 18), // "fillStrokeSelector"
QT_MOC_LITERAL(19, 252, 15), // "handleFillShade"
QT_MOC_LITERAL(20, 268, 17), // "handleStrokeShade"
QT_MOC_LITERAL(21, 286, 15), // "handleOverprint"
QT_MOC_LITERAL(22, 302, 18), // "handleFillGradient"
QT_MOC_LITERAL(23, 321, 20), // "handleStrokeGradient"
QT_MOC_LITERAL(24, 342, 26), // "handleStrokeGradientExtend"
QT_MOC_LITERAL(25, 369, 3), // "val"
QT_MOC_LITERAL(26, 373, 20), // "handleGradientExtend"
QT_MOC_LITERAL(27, 394, 15), // "showColorValues"
QT_MOC_LITERAL(28, 410, 6), // "stroke"
QT_MOC_LITERAL(29, 417, 5), // "fille"
QT_MOC_LITERAL(30, 423, 6), // "sShade"
QT_MOC_LITERAL(31, 430, 6), // "fShade"
QT_MOC_LITERAL(32, 437, 12), // "showGradient"
QT_MOC_LITERAL(33, 450, 2), // "nr"
QT_MOC_LITERAL(34, 453, 18), // "showGradientStroke"
QT_MOC_LITERAL(35, 472, 13), // "showOverprint"
QT_MOC_LITERAL(36, 486, 18), // "toggleColorDisplay"
QT_MOC_LITERAL(37, 505, 16), // "setNamedGradient"
QT_MOC_LITERAL(38, 522, 4), // "name"
QT_MOC_LITERAL(39, 527, 22), // "setNamedGradientStroke"
QT_MOC_LITERAL(40, 550, 13), // "selectPattern"
QT_MOC_LITERAL(41, 564, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(42, 581, 1), // "c"
QT_MOC_LITERAL(43, 583, 14), // "selectPatternS"
QT_MOC_LITERAL(44, 598, 18), // "hideEditedPatterns"
QT_MOC_LITERAL(45, 617, 5), // "names"
QT_MOC_LITERAL(46, 623, 12), // "selectColorS"
QT_MOC_LITERAL(47, 636, 3), // "row"
QT_MOC_LITERAL(48, 640, 12), // "selectColorF"
QT_MOC_LITERAL(49, 653, 14), // "slotGradStroke"
QT_MOC_LITERAL(50, 668, 6), // "number"
QT_MOC_LITERAL(51, 675, 8), // "slotGrad"
QT_MOC_LITERAL(52, 684, 12), // "slotGradType"
QT_MOC_LITERAL(53, 697, 4), // "type"
QT_MOC_LITERAL(54, 702, 18), // "slotGradTypeStroke"
QT_MOC_LITERAL(55, 721, 17), // "setGradientColors"
QT_MOC_LITERAL(56, 739, 18), // "editMeshPointColor"
QT_MOC_LITERAL(57, 758, 21), // "createNewMeshGradient"
QT_MOC_LITERAL(58, 780, 17), // "resetMeshGradient"
QT_MOC_LITERAL(59, 798, 19), // "meshGradientToShape"
QT_MOC_LITERAL(60, 818, 20), // "resetOneControlPoint"
QT_MOC_LITERAL(61, 839, 21), // "resetAllControlPoints"
QT_MOC_LITERAL(62, 861, 18), // "editGradientVector"
QT_MOC_LITERAL(63, 880, 24), // "editGradientVectorStroke"
QT_MOC_LITERAL(64, 905, 16), // "setActiveGradDia"
QT_MOC_LITERAL(65, 922, 6), // "active"
QT_MOC_LITERAL(66, 929, 12), // "setMeshPoint"
QT_MOC_LITERAL(67, 942, 11), // "endPatchAdd"
QT_MOC_LITERAL(68, 954, 15), // "snapToPatchGrid"
QT_MOC_LITERAL(69, 970, 17), // "handleRemovePatch"
QT_MOC_LITERAL(70, 988, 12), // "setMeshPatch"
QT_MOC_LITERAL(71, 1001, 17), // "setMeshPatchPoint"
QT_MOC_LITERAL(72, 1019, 15), // "updateMeshPoint"
QT_MOC_LITERAL(73, 1035, 18), // "changePatternProps"
QT_MOC_LITERAL(74, 1054, 24), // "changePatternPropsStroke"
QT_MOC_LITERAL(75, 1079, 19), // "toggleStrokePattern"
QT_MOC_LITERAL(76, 1099, 16) // "changeHatchProps"

    },
    "ColorPalette\0NewPen\0\0NewBrush\0NewPenShade\0"
    "NewBrushShade\0NewGradient\0NewGradientS\0"
    "NewPattern\0NewPatternProps\0NewPatternS\0"
    "NewPatternTypeS\0NewPatternPropsS\0"
    "NewSpecial\0NewOverprint\0editGradient\0"
    "handleSelectionChanged\0handleUpdateRequest\0"
    "fillStrokeSelector\0handleFillShade\0"
    "handleStrokeShade\0handleOverprint\0"
    "handleFillGradient\0handleStrokeGradient\0"
    "handleStrokeGradientExtend\0val\0"
    "handleGradientExtend\0showColorValues\0"
    "stroke\0fille\0sShade\0fShade\0showGradient\0"
    "nr\0showGradientStroke\0showOverprint\0"
    "toggleColorDisplay\0setNamedGradient\0"
    "name\0setNamedGradientStroke\0selectPattern\0"
    "QListWidgetItem*\0c\0selectPatternS\0"
    "hideEditedPatterns\0names\0selectColorS\0"
    "row\0selectColorF\0slotGradStroke\0number\0"
    "slotGrad\0slotGradType\0type\0"
    "slotGradTypeStroke\0setGradientColors\0"
    "editMeshPointColor\0createNewMeshGradient\0"
    "resetMeshGradient\0meshGradientToShape\0"
    "resetOneControlPoint\0resetAllControlPoints\0"
    "editGradientVector\0editGradientVectorStroke\0"
    "setActiveGradDia\0active\0setMeshPoint\0"
    "endPatchAdd\0snapToPatchGrid\0"
    "handleRemovePatch\0setMeshPatch\0"
    "setMeshPatchPoint\0updateMeshPoint\0"
    "changePatternProps\0changePatternPropsStroke\0"
    "toggleStrokePattern\0changeHatchProps"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ColorPalette[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      61,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
      14,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  319,    2, 0x06 /* Public */,
       3,    1,  322,    2, 0x06 /* Public */,
       4,    1,  325,    2, 0x06 /* Public */,
       5,    1,  328,    2, 0x06 /* Public */,
       6,    1,  331,    2, 0x06 /* Public */,
       7,    1,  334,    2, 0x06 /* Public */,
       8,    1,  337,    2, 0x06 /* Public */,
       9,    9,  340,    2, 0x06 /* Public */,
      10,    1,  359,    2, 0x06 /* Public */,
      11,    1,  362,    2, 0x06 /* Public */,
      12,   10,  365,    2, 0x06 /* Public */,
      13,   10,  386,    2, 0x06 /* Public */,
      14,    1,  407,    2, 0x06 /* Public */,
      15,    1,  410,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      16,    0,  413,    2, 0x0a /* Public */,
      17,    1,  414,    2, 0x0a /* Public */,
      18,    1,  417,    2, 0x0a /* Public */,
      19,    1,  420,    2, 0x0a /* Public */,
      20,    1,  423,    2, 0x0a /* Public */,
      21,    1,  426,    2, 0x0a /* Public */,
      22,    0,  429,    2, 0x0a /* Public */,
      23,    0,  430,    2, 0x0a /* Public */,
      24,    1,  431,    2, 0x0a /* Public */,
      26,    1,  434,    2, 0x0a /* Public */,
      27,    4,  437,    2, 0x0a /* Public */,
      32,    1,  446,    2, 0x0a /* Public */,
      34,    1,  449,    2, 0x0a /* Public */,
      35,    1,  452,    2, 0x0a /* Public */,
      36,    0,  455,    2, 0x0a /* Public */,
      37,    1,  456,    2, 0x0a /* Public */,
      39,    1,  459,    2, 0x0a /* Public */,
      40,    1,  462,    2, 0x0a /* Public */,
      43,    1,  465,    2, 0x0a /* Public */,
      44,    1,  468,    2, 0x0a /* Public */,
      46,    1,  471,    2, 0x0a /* Public */,
      48,    1,  474,    2, 0x0a /* Public */,
      49,    1,  477,    2, 0x0a /* Public */,
      51,    1,  480,    2, 0x0a /* Public */,
      52,    1,  483,    2, 0x0a /* Public */,
      54,    1,  486,    2, 0x0a /* Public */,
      55,    0,  489,    2, 0x0a /* Public */,
      56,    0,  490,    2, 0x0a /* Public */,
      57,    0,  491,    2, 0x0a /* Public */,
      58,    0,  492,    2, 0x0a /* Public */,
      59,    0,  493,    2, 0x0a /* Public */,
      60,    0,  494,    2, 0x0a /* Public */,
      61,    0,  495,    2, 0x0a /* Public */,
      62,    0,  496,    2, 0x0a /* Public */,
      63,    0,  497,    2, 0x0a /* Public */,
      64,    1,  498,    2, 0x0a /* Public */,
      66,    0,  501,    2, 0x0a /* Public */,
      67,    0,  502,    2, 0x0a /* Public */,
      68,    1,  503,    2, 0x0a /* Public */,
      69,    0,  506,    2, 0x0a /* Public */,
      70,    0,  507,    2, 0x0a /* Public */,
      71,    0,  508,    2, 0x0a /* Public */,
      72,    0,  509,    2, 0x0a /* Public */,
      73,    0,  510,    2, 0x0a /* Public */,
      74,    0,  511,    2, 0x0a /* Public */,
      75,    0,  512,    2, 0x0a /* Public */,
      76,    0,  513,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Bool, QMetaType::Bool,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::Bool,    2,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Bool, QMetaType::Bool,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double,    2,    2,    2,    2,    2,    2,    2,    2,    2,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Double,    2,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void, QMetaType::QString, QMetaType::QString, QMetaType::Int, QMetaType::Int,   28,   29,   30,   31,
    QMetaType::Void, QMetaType::Int,   33,
    QMetaType::Void, QMetaType::Int,   33,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   38,
    QMetaType::Void, QMetaType::QString,   38,
    QMetaType::Void, 0x80000000 | 41,   42,
    QMetaType::Void, 0x80000000 | 41,   42,
    QMetaType::Void, QMetaType::QStringList,   45,
    QMetaType::Void, QMetaType::Int,   47,
    QMetaType::Void, QMetaType::Int,   47,
    QMetaType::Void, QMetaType::Int,   50,
    QMetaType::Void, QMetaType::Int,   33,
    QMetaType::Void, QMetaType::Int,   53,
    QMetaType::Void, QMetaType::Int,   53,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   65,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   25,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ColorPalette::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ColorPalette *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->NewPen((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->NewBrush((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 2: _t->NewPenShade((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 3: _t->NewBrushShade((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 4: _t->NewGradient((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 5: _t->NewGradientS((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 6: _t->NewPattern((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 7: _t->NewPatternProps((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6])),(*reinterpret_cast< double(*)>(_a[7])),(*reinterpret_cast< bool(*)>(_a[8])),(*reinterpret_cast< bool(*)>(_a[9]))); break;
        case 8: _t->NewPatternS((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 9: _t->NewPatternTypeS((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 10: _t->NewPatternPropsS((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6])),(*reinterpret_cast< double(*)>(_a[7])),(*reinterpret_cast< double(*)>(_a[8])),(*reinterpret_cast< bool(*)>(_a[9])),(*reinterpret_cast< bool(*)>(_a[10]))); break;
        case 11: _t->NewSpecial((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6])),(*reinterpret_cast< double(*)>(_a[7])),(*reinterpret_cast< double(*)>(_a[8])),(*reinterpret_cast< double(*)>(_a[9])),(*reinterpret_cast< double(*)>(_a[10]))); break;
        case 12: _t->NewOverprint((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 13: _t->editGradient((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 14: _t->handleSelectionChanged(); break;
        case 15: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 16: _t->fillStrokeSelector((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->handleFillShade((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 18: _t->handleStrokeShade((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 19: _t->handleOverprint((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->handleFillGradient(); break;
        case 21: _t->handleStrokeGradient(); break;
        case 22: _t->handleStrokeGradientExtend((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 23: _t->handleGradientExtend((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 24: _t->showColorValues((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< const QString(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< int(*)>(_a[4]))); break;
        case 25: _t->showGradient((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 26: _t->showGradientStroke((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 27: _t->showOverprint((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 28: _t->toggleColorDisplay(); break;
        case 29: _t->setNamedGradient((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 30: _t->setNamedGradientStroke((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 31: _t->selectPattern((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 32: _t->selectPatternS((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 33: _t->hideEditedPatterns((*reinterpret_cast< QStringList(*)>(_a[1]))); break;
        case 34: _t->selectColorS((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 35: _t->selectColorF((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 36: _t->slotGradStroke((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 37: _t->slotGrad((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 38: _t->slotGradType((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 39: _t->slotGradTypeStroke((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 40: _t->setGradientColors(); break;
        case 41: _t->editMeshPointColor(); break;
        case 42: _t->createNewMeshGradient(); break;
        case 43: _t->resetMeshGradient(); break;
        case 44: _t->meshGradientToShape(); break;
        case 45: _t->resetOneControlPoint(); break;
        case 46: _t->resetAllControlPoints(); break;
        case 47: _t->editGradientVector(); break;
        case 48: _t->editGradientVectorStroke(); break;
        case 49: _t->setActiveGradDia((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 50: _t->setMeshPoint(); break;
        case 51: _t->endPatchAdd(); break;
        case 52: _t->snapToPatchGrid((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 53: _t->handleRemovePatch(); break;
        case 54: _t->setMeshPatch(); break;
        case 55: _t->setMeshPatchPoint(); break;
        case 56: _t->updateMeshPoint(); break;
        case 57: _t->changePatternProps(); break;
        case 58: _t->changePatternPropsStroke(); break;
        case 59: _t->toggleStrokePattern(); break;
        case 60: _t->changeHatchProps(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ColorPalette::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorPalette::NewPen)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ColorPalette::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorPalette::NewBrush)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ColorPalette::*)(double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorPalette::NewPenShade)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ColorPalette::*)(double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorPalette::NewBrushShade)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ColorPalette::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorPalette::NewGradient)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (ColorPalette::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorPalette::NewGradientS)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (ColorPalette::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorPalette::NewPattern)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (ColorPalette::*)(double , double , double , double , double , double , double , bool , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorPalette::NewPatternProps)) {
                *result = 7;
                return;
            }
        }
        {
            using _t = void (ColorPalette::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorPalette::NewPatternS)) {
                *result = 8;
                return;
            }
        }
        {
            using _t = void (ColorPalette::*)(bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorPalette::NewPatternTypeS)) {
                *result = 9;
                return;
            }
        }
        {
            using _t = void (ColorPalette::*)(double , double , double , double , double , double , double , double , bool , bool );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorPalette::NewPatternPropsS)) {
                *result = 10;
                return;
            }
        }
        {
            using _t = void (ColorPalette::*)(double , double , double , double , double , double , double , double , double , double );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorPalette::NewSpecial)) {
                *result = 11;
                return;
            }
        }
        {
            using _t = void (ColorPalette::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorPalette::NewOverprint)) {
                *result = 12;
                return;
            }
        }
        {
            using _t = void (ColorPalette::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ColorPalette::editGradient)) {
                *result = 13;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ColorPalette::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ColorPalette.data,
    qt_meta_data_ColorPalette,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ColorPalette::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ColorPalette::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ColorPalette.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::colorPalette"))
        return static_cast< Ui::colorPalette*>(this);
    return QWidget::qt_metacast(_clname);
}

int ColorPalette::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 61)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 61;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 61)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 61;
    }
    return _id;
}

// SIGNAL 0
void ColorPalette::NewPen(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void ColorPalette::NewBrush(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void ColorPalette::NewPenShade(double _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void ColorPalette::NewBrushShade(double _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ColorPalette::NewGradient(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void ColorPalette::NewGradientS(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}

// SIGNAL 6
void ColorPalette::NewPattern(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void ColorPalette::NewPatternProps(double _t1, double _t2, double _t3, double _t4, double _t5, double _t6, double _t7, bool _t8, bool _t9)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t6))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t7))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t8))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t9))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}

// SIGNAL 8
void ColorPalette::NewPatternS(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 8, _a);
}

// SIGNAL 9
void ColorPalette::NewPatternTypeS(bool _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 9, _a);
}

// SIGNAL 10
void ColorPalette::NewPatternPropsS(double _t1, double _t2, double _t3, double _t4, double _t5, double _t6, double _t7, double _t8, bool _t9, bool _t10)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t6))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t7))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t8))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t9))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t10))) };
    QMetaObject::activate(this, &staticMetaObject, 10, _a);
}

// SIGNAL 11
void ColorPalette::NewSpecial(double _t1, double _t2, double _t3, double _t4, double _t5, double _t6, double _t7, double _t8, double _t9, double _t10)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t3))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t4))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t5))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t6))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t7))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t8))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t9))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t10))) };
    QMetaObject::activate(this, &staticMetaObject, 11, _a);
}

// SIGNAL 12
void ColorPalette::NewOverprint(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 12, _a);
}

// SIGNAL 13
void ColorPalette::editGradient(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 13, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
