/********************************************************************************
** Form generated from reading UI file 'gradientaddedit.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_GRADIENTADDEDIT_H
#define UI_GRADIENTADDEDIT_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QVBoxLayout>
#include "ui/gradienteditor.h"

QT_BEGIN_NAMESPACE

class Ui_gradientAddEditDialog
{
public:
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout;
    QLabel *label;
    QLineEdit *gradientName;
    GradientEditor *editor;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *gradientAddEditDialog)
    {
        if (gradientAddEditDialog->objectName().isEmpty())
            gradientAddEditDialog->setObjectName(QString::fromUtf8("gradientAddEditDialog"));
        gradientAddEditDialog->resize(302, 277);
        gradientAddEditDialog->setMinimumSize(QSize(210, 0));
        gradientAddEditDialog->setModal(true);
        verticalLayout = new QVBoxLayout(gradientAddEditDialog);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(9, 9, 9, 9);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setSizeConstraint(QLayout::SetMinimumSize);
        label = new QLabel(gradientAddEditDialog);
        label->setObjectName(QString::fromUtf8("label"));

        horizontalLayout->addWidget(label);

        gradientName = new QLineEdit(gradientAddEditDialog);
        gradientName->setObjectName(QString::fromUtf8("gradientName"));

        horizontalLayout->addWidget(gradientName);


        verticalLayout->addLayout(horizontalLayout);

        editor = new GradientEditor(gradientAddEditDialog);
        editor->setObjectName(QString::fromUtf8("editor"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::MinimumExpanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(editor->sizePolicy().hasHeightForWidth());
        editor->setSizePolicy(sizePolicy);
        editor->setFrameShape(QFrame::NoFrame);
        editor->setFrameShadow(QFrame::Plain);

        verticalLayout->addWidget(editor);

        buttonBox = new QDialogButtonBox(gradientAddEditDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        verticalLayout->addWidget(buttonBox);


        retranslateUi(gradientAddEditDialog);
        QObject::connect(buttonBox, SIGNAL(rejected()), gradientAddEditDialog, SLOT(reject()));

        QMetaObject::connectSlotsByName(gradientAddEditDialog);
    } // setupUi

    void retranslateUi(QDialog *gradientAddEditDialog)
    {
        gradientAddEditDialog->setWindowTitle(QCoreApplication::translate("gradientAddEditDialog", "Edit Gradient", nullptr));
        label->setText(QCoreApplication::translate("gradientAddEditDialog", "Name:", nullptr));
    } // retranslateUi

};

namespace Ui {
    class gradientAddEditDialog: public Ui_gradientAddEditDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_GRADIENTADDEDIT_H
