/*
For general Scribus (>=1.3.2) copyright and licensing information please refer
to the COPYING file provided with the program. Following this notice may exist
a copyright and/or license notice that predates the release of Scribus 1.3.2
for which a new license (GPL+exception) is in place.
*/
/***************************************************************************
 *   Copyright (C) 2004 by Riku Leino                                      *
 *   tsoots@gmail.com                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.             *
 ***************************************************************************/

#ifndef HTMLIM_H
#define HTMLIM_H

#include <memory>

#include <QString>
#include <QStringList>

#include "pluginapi.h"
#include "scconfig.h"

class gtParagraphStyle;
class gtWriter;

extern "C" PLUGIN_API void GetText(const QString& filename, const QString& encoding, bool textOnly, gtWriter *writer);

extern "C" PLUGIN_API QString FileFormatName();

extern "C" PLUGIN_API QStringList FileExtensions();

class HTMLIm 
{
public:
	HTMLIm(const QString& fname, const QString& encoding, gtWriter *w);
	HTMLIm(const HTMLIm&) = delete;
	HTMLIm& operator=(const HTMLIm&) = delete;

	void importText(bool textOnly);

private:
	QString encoding;
	QString filename;
	gtWriter *writer { nullptr };
	std::unique_ptr<gtParagraphStyle> pstyle;
};

#endif // HTMLIM_H
