/********************************************************************************
** Form generated from reading UI file 'propertywidget_orphansbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTYWIDGET_ORPHANSBASE_H
#define UI_PROPERTYWIDGET_ORPHANSBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QFrame>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QLabel>
#include <ui/scrspinbox.h>

QT_BEGIN_NAMESPACE

class Ui_PropertyWidget_OrphansBase
{
public:
    QGridLayout *gridLayout;
    QLabel *keepLabelStart;
    ScrSpinBox *keepLinesStart;
    QLabel *keepLabelEnd;
    ScrSpinBox *keepLinesEnd;
    QCheckBox *keepTogether;
    QCheckBox *keepWithNext;

    void setupUi(QFrame *PropertyWidget_OrphansBase)
    {
        if (PropertyWidget_OrphansBase->objectName().isEmpty())
            PropertyWidget_OrphansBase->setObjectName(QString::fromUtf8("PropertyWidget_OrphansBase"));
        PropertyWidget_OrphansBase->resize(258, 113);
        gridLayout = new QGridLayout(PropertyWidget_OrphansBase);
        gridLayout->setSpacing(3);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(3, 3, 3, 3);
        keepLabelStart = new QLabel(PropertyWidget_OrphansBase);
        keepLabelStart->setObjectName(QString::fromUtf8("keepLabelStart"));

        gridLayout->addWidget(keepLabelStart, 0, 0, 1, 1);

        keepLinesStart = new ScrSpinBox(PropertyWidget_OrphansBase);
        keepLinesStart->setObjectName(QString::fromUtf8("keepLinesStart"));
        keepLinesStart->setMaximum(10);

        gridLayout->addWidget(keepLinesStart, 0, 1, 1, 1);

        keepLabelEnd = new QLabel(PropertyWidget_OrphansBase);
        keepLabelEnd->setObjectName(QString::fromUtf8("keepLabelEnd"));

        gridLayout->addWidget(keepLabelEnd, 1, 0, 1, 1);

        keepLinesEnd = new ScrSpinBox(PropertyWidget_OrphansBase);
        keepLinesEnd->setObjectName(QString::fromUtf8("keepLinesEnd"));
        keepLinesEnd->setMaximum(10);

        gridLayout->addWidget(keepLinesEnd, 1, 1, 1, 1);

        keepTogether = new QCheckBox(PropertyWidget_OrphansBase);
        keepTogether->setObjectName(QString::fromUtf8("keepTogether"));

        gridLayout->addWidget(keepTogether, 2, 0, 1, 2);

        keepWithNext = new QCheckBox(PropertyWidget_OrphansBase);
        keepWithNext->setObjectName(QString::fromUtf8("keepWithNext"));

        gridLayout->addWidget(keepWithNext, 3, 0, 1, 2);

        gridLayout->setColumnStretch(1, 1);

        retranslateUi(PropertyWidget_OrphansBase);

        QMetaObject::connectSlotsByName(PropertyWidget_OrphansBase);
    } // setupUi

    void retranslateUi(QFrame *PropertyWidget_OrphansBase)
    {
#if QT_CONFIG(tooltip)
        keepLabelStart->setToolTip(QCoreApplication::translate("PropertyWidget_OrphansBase", "Ensure that first lines of a paragraph won't end up separated from the rest (known as widow/orphan control)", nullptr));
#endif // QT_CONFIG(tooltip)
        keepLabelStart->setText(QCoreApplication::translate("PropertyWidget_OrphansBase", "Don't separate first:", nullptr));
#if QT_CONFIG(tooltip)
        keepLinesStart->setToolTip(QCoreApplication::translate("PropertyWidget_OrphansBase", "Ensure that first lines of a paragraph won't end up separated from the rest (known as widow/orphan control)", nullptr));
#endif // QT_CONFIG(tooltip)
        keepLinesStart->setSuffix(QCoreApplication::translate("PropertyWidget_OrphansBase", " lines", nullptr));
        keepLabelEnd->setText(QCoreApplication::translate("PropertyWidget_OrphansBase", "Don't separate last:", nullptr));
#if QT_CONFIG(tooltip)
        keepLinesEnd->setToolTip(QCoreApplication::translate("PropertyWidget_OrphansBase", "Ensure that last lines of a paragraph won't end up separated from the rest (known as widow/orphan control)", nullptr));
#endif // QT_CONFIG(tooltip)
        keepLinesEnd->setSuffix(QCoreApplication::translate("PropertyWidget_OrphansBase", " lines", nullptr));
#if QT_CONFIG(tooltip)
        keepTogether->setToolTip(QCoreApplication::translate("PropertyWidget_OrphansBase", "If checked, ensures that the paragraph won't be split across multiple pages or columns", nullptr));
#endif // QT_CONFIG(tooltip)
        keepTogether->setText(QCoreApplication::translate("PropertyWidget_OrphansBase", "Do not split paragraph", nullptr));
#if QT_CONFIG(tooltip)
        keepWithNext->setToolTip(QCoreApplication::translate("PropertyWidget_OrphansBase", "If checked, automatically moves the paragraph to the next column or page if the next paragraph isn't on the same page or column", nullptr));
#endif // QT_CONFIG(tooltip)
        keepWithNext->setText(QCoreApplication::translate("PropertyWidget_OrphansBase", "Keep with next paragraph", nullptr));
        (void)PropertyWidget_OrphansBase;
    } // retranslateUi

};

namespace Ui {
    class PropertyWidget_OrphansBase: public Ui_PropertyWidget_OrphansBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTYWIDGET_ORPHANSBASE_H
