/****************************************************************************
** Meta object code from reading C++ file 'contentpalette.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/contentpalette.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'contentpalette.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ContentPalette_t {
    QByteArrayData data[21];
    char stringdata0[233];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ContentPalette_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ContentPalette_t qt_meta_stringdata_ContentPalette = {
    {
QT_MOC_LITERAL(0, 0, 14), // "ContentPalette"
QT_MOC_LITERAL(1, 15, 13), // "setMainWindow"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 18), // "ScribusMainWindow*"
QT_MOC_LITERAL(4, 49, 2), // "mw"
QT_MOC_LITERAL(5, 52, 6), // "setDoc"
QT_MOC_LITERAL(6, 59, 11), // "ScribusDoc*"
QT_MOC_LITERAL(7, 71, 3), // "doc"
QT_MOC_LITERAL(8, 75, 8), // "unsetDoc"
QT_MOC_LITERAL(9, 84, 9), // "unsetItem"
QT_MOC_LITERAL(10, 94, 14), // "AppModeChanged"
QT_MOC_LITERAL(11, 109, 14), // "setCurrentItem"
QT_MOC_LITERAL(12, 124, 9), // "PageItem*"
QT_MOC_LITERAL(13, 134, 1), // "i"
QT_MOC_LITERAL(14, 136, 24), // "currentItemFromSelection"
QT_MOC_LITERAL(15, 161, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(16, 184, 6), // "update"
QT_MOC_LITERAL(17, 191, 9), // "CharStyle"
QT_MOC_LITERAL(18, 201, 5), // "style"
QT_MOC_LITERAL(19, 207, 10), // "unitChange"
QT_MOC_LITERAL(20, 218, 14) // "languageChange"

    },
    "ContentPalette\0setMainWindow\0\0"
    "ScribusMainWindow*\0mw\0setDoc\0ScribusDoc*\0"
    "doc\0unsetDoc\0unsetItem\0AppModeChanged\0"
    "setCurrentItem\0PageItem*\0i\0"
    "currentItemFromSelection\0"
    "handleSelectionChanged\0update\0CharStyle\0"
    "style\0unitChange\0languageChange"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ContentPalette[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,   69,    2, 0x0a /* Public */,
       5,    1,   72,    2, 0x0a /* Public */,
       8,    0,   75,    2, 0x0a /* Public */,
       9,    0,   76,    2, 0x0a /* Public */,
      10,    0,   77,    2, 0x0a /* Public */,
      11,    1,   78,    2, 0x0a /* Public */,
      14,    0,   81,    2, 0x0a /* Public */,
      15,    0,   82,    2, 0x0a /* Public */,
      16,    1,   83,    2, 0x0a /* Public */,
      19,    0,   86,    2, 0x0a /* Public */,
      20,    0,   87,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 12,   13,
    0x80000000 | 12,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ContentPalette::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ContentPalette *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setMainWindow((*reinterpret_cast< ScribusMainWindow*(*)>(_a[1]))); break;
        case 1: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 2: _t->unsetDoc(); break;
        case 3: _t->unsetItem(); break;
        case 4: _t->AppModeChanged(); break;
        case 5: _t->setCurrentItem((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 6: { PageItem* _r = _t->currentItemFromSelection();
            if (_a[0]) *reinterpret_cast< PageItem**>(_a[0]) = std::move(_r); }  break;
        case 7: _t->handleSelectionChanged(); break;
        case 8: _t->update((*reinterpret_cast< const CharStyle(*)>(_a[1]))); break;
        case 9: _t->unitChange(); break;
        case 10: _t->languageChange(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ContentPalette::staticMetaObject = { {
    QMetaObject::SuperData::link<ScDockPalette::staticMetaObject>(),
    qt_meta_stringdata_ContentPalette.data,
    qt_meta_data_ContentPalette,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ContentPalette::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ContentPalette::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ContentPalette.stringdata0))
        return static_cast<void*>(this);
    return ScDockPalette::qt_metacast(_clname);
}

int ContentPalette::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScDockPalette::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
