/********************************************************************************
** Form generated from reading UI file 'smpstylewidget.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_SMPSTYLEWIDGET_H
#define UI_SMPSTYLEWIDGET_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QLineEdit>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/smdirectionselect.h>
#include "ui/smalignselect.h"
#include "ui/smcheckbox.h"
#include "ui/smcolorcombo.h"
#include "ui/smcstylewidget.h"
#include "ui/smnumformatcombo.h"
#include "ui/smradiobutton.h"
#include "ui/smsccombobox.h"
#include "ui/smscrspinbox.h"
#include "ui/smshadebutton.h"
#include "ui/smspinbox.h"
#include "ui/smtabruler.h"

QT_BEGIN_NAMESPACE

class Ui_SMPStyleWidget
{
public:
    QVBoxLayout *vboxLayout;
    QTabWidget *tabWidget;
    QWidget *tab;
    QVBoxLayout *verticalLayout_8;
    QHBoxLayout *hboxLayout;
    QLabel *parentLabel;
    QComboBox *parentCombo;
    QGridLayout *gridLayout;
    QGroupBox *advSettingsGroupBox;
    QVBoxLayout *verticalLayout_9;
    QHBoxLayout *horizontalLayout_13;
    QVBoxLayout *verticalLayout_7;
    QHBoxLayout *horizontalLayout;
    QLabel *minSpaceLabel;
    SMScrSpinBox *minSpaceSpin;
    QLabel *glyphExtensionLabel;
    QHBoxLayout *horizontalLayout_2;
    QLabel *minGlyphExtLabel;
    SMScrSpinBox *minGlyphExtSpin;
    QLabel *maxGlyphExtLabel;
    SMScrSpinBox *maxGlyphExtSpin;
    QHBoxLayout *horizontalLayout_3;
    QLabel *label_3;
    SMSpinBox *maxConsecutiveCountSpinBox;
    QSpacerItem *horizontalSpacer_7;
    QGroupBox *opticalMarginsGroupBox;
    QGridLayout *gridLayout_3;
    QHBoxLayout *horizontalLayout_4;
    QGridLayout *gridLayout_5;
    SMRadioButton *optMarginRadioLeft;
    SMRadioButton *optMarginRadioBoth;
    SMRadioButton *optMarginRadioNone;
    SMRadioButton *optMarginRadioRight;
    QPushButton *optMarginDefaultButton;
    QPushButton *optMarginParentButton;
    QSpacerItem *horizontalSpacer_15;
    QGroupBox *textFlowBox;
    QVBoxLayout *verticalLayout_3;
    QHBoxLayout *horizontalLayout_11;
    QLabel *keepLabelStart;
    SMSpinBox *keepLinesStart;
    QLabel *keepLabelEnd;
    SMSpinBox *keepLinesEnd;
    QSpacerItem *horizontalSpacer_6;
    SMCheckBox *keepTogether;
    SMCheckBox *keepWithNext;
    QSpacerItem *verticalSpacer_4;
    QGroupBox *distancesBox;
    QHBoxLayout *horizontalLayout_10;
    QVBoxLayout *verticalLayout_10;
    QHBoxLayout *horizontalLayout_6;
    SMAlignSelect *alignment;
    SMDirectionSelect *direction;
    QSpacerItem *horizontalSpacer_4;
    QHBoxLayout *horizontalLayout_9;
    QLabel *lineSpacingLabel;
    SMScComboBox *lineSpacingMode;
    SMScrSpinBox *lineSpacing;
    QSpacerItem *horizontalSpacer_8;
    QHBoxLayout *horizontalLayout_5;
    QLabel *spaceAboveLabel;
    SMScrSpinBox *spaceAbove;
    QLabel *spaceBelowLabel;
    SMScrSpinBox *spaceBelow;
    QSpacerItem *horizontalSpacer_11;
    QSpacerItem *verticalSpacer_3;
    QSpacerItem *horizontalSpacer_16;
    QGroupBox *tabsBox;
    QVBoxLayout *vboxLayout1;
    QHBoxLayout *tabsBoxLayout;
    SMTabruler *tabList;
    QSpacerItem *horizontalSpacer_13;
    QGroupBox *smColorGroup;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *smColorGroupLayout;
    QLabel *backIcon;
    SMColorCombo *backColor_;
    QLabel *backShadeLabel;
    SMShadeButton *backShade_;
    QSpacerItem *horizontalSpacer_12;
    QSpacerItem *verticalSpacer;
    QWidget *tabParagraphEffects;
    QVBoxLayout *verticalLayout_6;
    QGroupBox *bulletBox;
    QVBoxLayout *verticalLayout;
    QHBoxLayout *horizontalLayout_7;
    QLabel *bulletCharLabel;
    SMScComboBox *bulletStrEdit;
    QSpacerItem *spacer;
    QPushButton *bulletCharTableButton;
    QSpacerItem *spacer_2;
    QGroupBox *numBox;
    QVBoxLayout *verticalLayout_5;
    QGridLayout *gridLayout_2;
    SMScComboBox *numComboBox;
    QLabel *numFormatLabel;
    QSpacerItem *horizontalSpacer_10;
    SMScComboBox *numRestartCombo;
    QLabel *numRestartLabel;
    QLineEdit *numNewLineEdit;
    SMNumFormatCombo *numFormatCombo;
    SMSpinBox *numStartSpin;
    QLabel *numStartLabel;
    SMCheckBox *numRestartHigherBox;
    QLabel *numLevelLabel;
    QLineEdit *numPrefix;
    QLabel *numSuffixLabel;
    QLabel *numComboLabel;
    SMSpinBox *numLevelSpin;
    QLabel *numPrefixLabel;
    QLabel *numNewLabel;
    QLineEdit *numSuffix;
    SMCheckBox *numRestartOtherBox;
    QGroupBox *dropCapsBox;
    QVBoxLayout *_2;
    QHBoxLayout *dropCapsBoxLayout;
    QLabel *dropCapsLineLabel;
    SMSpinBox *dropCapLines;
    QSpacerItem *horizontalSpacer_2;
    QHBoxLayout *horizontalLayout_12;
    QLabel *parEffectCharStyleComboLabel;
    SMScComboBox *parEffectCharStyleCombo;
    QSpacerItem *spacer_6;
    QLabel *distFromTextLabel;
    SMScrSpinBox *parEffectOffset;
    QSpacerItem *horizontalSpacer_14;
    SMCheckBox *parEffectIndentBox;
    QHBoxLayout *horizontalLayout_15;
    QSpacerItem *spacer_10;
    QToolButton *parentParEffectsButton;
    QSpacerItem *verticalSpacer_2;
    QWidget *tabCharacterStyle;
    QVBoxLayout *vboxLayout2;
    SMCStyleWidget *cpage;

    void setupUi(QWidget *SMPStyleWidget)
    {
        if (SMPStyleWidget->objectName().isEmpty())
            SMPStyleWidget->setObjectName(QString::fromUtf8("SMPStyleWidget"));
        SMPStyleWidget->resize(678, 615);
        vboxLayout = new QVBoxLayout(SMPStyleWidget);
        vboxLayout->setObjectName(QString::fromUtf8("vboxLayout"));
        tabWidget = new QTabWidget(SMPStyleWidget);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tab = new QWidget();
        tab->setObjectName(QString::fromUtf8("tab"));
        verticalLayout_8 = new QVBoxLayout(tab);
        verticalLayout_8->setObjectName(QString::fromUtf8("verticalLayout_8"));
        hboxLayout = new QHBoxLayout();
        hboxLayout->setObjectName(QString::fromUtf8("hboxLayout"));
        parentLabel = new QLabel(tab);
        parentLabel->setObjectName(QString::fromUtf8("parentLabel"));
        parentLabel->setWordWrap(false);

        hboxLayout->addWidget(parentLabel);

        parentCombo = new QComboBox(tab);
        parentCombo->setObjectName(QString::fromUtf8("parentCombo"));
        QSizePolicy sizePolicy(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(5);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(parentCombo->sizePolicy().hasHeightForWidth());
        parentCombo->setSizePolicy(sizePolicy);

        hboxLayout->addWidget(parentCombo);


        verticalLayout_8->addLayout(hboxLayout);

        gridLayout = new QGridLayout();
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        advSettingsGroupBox = new QGroupBox(tab);
        advSettingsGroupBox->setObjectName(QString::fromUtf8("advSettingsGroupBox"));
        advSettingsGroupBox->setEnabled(true);
        QSizePolicy sizePolicy1(QSizePolicy::Preferred, QSizePolicy::Minimum);
        sizePolicy1.setHorizontalStretch(0);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(advSettingsGroupBox->sizePolicy().hasHeightForWidth());
        advSettingsGroupBox->setSizePolicy(sizePolicy1);
        advSettingsGroupBox->setMinimumSize(QSize(0, 0));
        verticalLayout_9 = new QVBoxLayout(advSettingsGroupBox);
        verticalLayout_9->setObjectName(QString::fromUtf8("verticalLayout_9"));
        horizontalLayout_13 = new QHBoxLayout();
        horizontalLayout_13->setObjectName(QString::fromUtf8("horizontalLayout_13"));
        verticalLayout_7 = new QVBoxLayout();
        verticalLayout_7->setObjectName(QString::fromUtf8("verticalLayout_7"));
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        minSpaceLabel = new QLabel(advSettingsGroupBox);
        minSpaceLabel->setObjectName(QString::fromUtf8("minSpaceLabel"));

        horizontalLayout->addWidget(minSpaceLabel);

        minSpaceSpin = new SMScrSpinBox(advSettingsGroupBox);
        minSpaceSpin->setObjectName(QString::fromUtf8("minSpaceSpin"));
        minSpaceSpin->setMinimumSize(QSize(0, 0));
        minSpaceSpin->setBaseSize(QSize(0, 0));

        horizontalLayout->addWidget(minSpaceSpin);


        verticalLayout_7->addLayout(horizontalLayout);

        glyphExtensionLabel = new QLabel(advSettingsGroupBox);
        glyphExtensionLabel->setObjectName(QString::fromUtf8("glyphExtensionLabel"));

        verticalLayout_7->addWidget(glyphExtensionLabel);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        minGlyphExtLabel = new QLabel(advSettingsGroupBox);
        minGlyphExtLabel->setObjectName(QString::fromUtf8("minGlyphExtLabel"));

        horizontalLayout_2->addWidget(minGlyphExtLabel);

        minGlyphExtSpin = new SMScrSpinBox(advSettingsGroupBox);
        minGlyphExtSpin->setObjectName(QString::fromUtf8("minGlyphExtSpin"));
        minGlyphExtSpin->setMinimumSize(QSize(0, 0));

        horizontalLayout_2->addWidget(minGlyphExtSpin);

        maxGlyphExtLabel = new QLabel(advSettingsGroupBox);
        maxGlyphExtLabel->setObjectName(QString::fromUtf8("maxGlyphExtLabel"));

        horizontalLayout_2->addWidget(maxGlyphExtLabel);

        maxGlyphExtSpin = new SMScrSpinBox(advSettingsGroupBox);
        maxGlyphExtSpin->setObjectName(QString::fromUtf8("maxGlyphExtSpin"));
        maxGlyphExtSpin->setMinimumSize(QSize(0, 0));

        horizontalLayout_2->addWidget(maxGlyphExtSpin);


        verticalLayout_7->addLayout(horizontalLayout_2);

        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        label_3 = new QLabel(advSettingsGroupBox);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        QSizePolicy sizePolicy2(QSizePolicy::Minimum, QSizePolicy::Minimum);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(label_3->sizePolicy().hasHeightForWidth());
        label_3->setSizePolicy(sizePolicy2);

        horizontalLayout_3->addWidget(label_3);

        maxConsecutiveCountSpinBox = new SMSpinBox(advSettingsGroupBox);
        maxConsecutiveCountSpinBox->setObjectName(QString::fromUtf8("maxConsecutiveCountSpinBox"));

        horizontalLayout_3->addWidget(maxConsecutiveCountSpinBox);


        verticalLayout_7->addLayout(horizontalLayout_3);


        horizontalLayout_13->addLayout(verticalLayout_7);

        horizontalSpacer_7 = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_13->addItem(horizontalSpacer_7);


        verticalLayout_9->addLayout(horizontalLayout_13);


        gridLayout->addWidget(advSettingsGroupBox, 1, 1, 1, 1);

        opticalMarginsGroupBox = new QGroupBox(tab);
        opticalMarginsGroupBox->setObjectName(QString::fromUtf8("opticalMarginsGroupBox"));
        gridLayout_3 = new QGridLayout(opticalMarginsGroupBox);
        gridLayout_3->setObjectName(QString::fromUtf8("gridLayout_3"));
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        gridLayout_5 = new QGridLayout();
        gridLayout_5->setObjectName(QString::fromUtf8("gridLayout_5"));
        optMarginRadioLeft = new SMRadioButton(opticalMarginsGroupBox);
        optMarginRadioLeft->setObjectName(QString::fromUtf8("optMarginRadioLeft"));

        gridLayout_5->addWidget(optMarginRadioLeft, 2, 0, 1, 1);

        optMarginRadioBoth = new SMRadioButton(opticalMarginsGroupBox);
        optMarginRadioBoth->setObjectName(QString::fromUtf8("optMarginRadioBoth"));
        QSizePolicy sizePolicy3(QSizePolicy::Minimum, QSizePolicy::Fixed);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(0);
        sizePolicy3.setHeightForWidth(optMarginRadioBoth->sizePolicy().hasHeightForWidth());
        optMarginRadioBoth->setSizePolicy(sizePolicy3);

        gridLayout_5->addWidget(optMarginRadioBoth, 1, 0, 1, 1);

        optMarginRadioNone = new SMRadioButton(opticalMarginsGroupBox);
        optMarginRadioNone->setObjectName(QString::fromUtf8("optMarginRadioNone"));
        sizePolicy3.setHeightForWidth(optMarginRadioNone->sizePolicy().hasHeightForWidth());
        optMarginRadioNone->setSizePolicy(sizePolicy3);
        optMarginRadioNone->setChecked(true);

        gridLayout_5->addWidget(optMarginRadioNone, 0, 0, 1, 1);

        optMarginRadioRight = new SMRadioButton(opticalMarginsGroupBox);
        optMarginRadioRight->setObjectName(QString::fromUtf8("optMarginRadioRight"));
        sizePolicy3.setHeightForWidth(optMarginRadioRight->sizePolicy().hasHeightForWidth());
        optMarginRadioRight->setSizePolicy(sizePolicy3);

        gridLayout_5->addWidget(optMarginRadioRight, 3, 0, 1, 1);

        optMarginDefaultButton = new QPushButton(opticalMarginsGroupBox);
        optMarginDefaultButton->setObjectName(QString::fromUtf8("optMarginDefaultButton"));
        sizePolicy3.setHeightForWidth(optMarginDefaultButton->sizePolicy().hasHeightForWidth());
        optMarginDefaultButton->setSizePolicy(sizePolicy3);

        gridLayout_5->addWidget(optMarginDefaultButton, 0, 1, 1, 1);

        optMarginParentButton = new QPushButton(opticalMarginsGroupBox);
        optMarginParentButton->setObjectName(QString::fromUtf8("optMarginParentButton"));
        sizePolicy3.setHeightForWidth(optMarginParentButton->sizePolicy().hasHeightForWidth());
        optMarginParentButton->setSizePolicy(sizePolicy3);

        gridLayout_5->addWidget(optMarginParentButton, 1, 1, 1, 1);


        horizontalLayout_4->addLayout(gridLayout_5);

        horizontalSpacer_15 = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_4->addItem(horizontalSpacer_15);


        gridLayout_3->addLayout(horizontalLayout_4, 0, 0, 1, 1);


        gridLayout->addWidget(opticalMarginsGroupBox, 1, 0, 1, 1);

        textFlowBox = new QGroupBox(tab);
        textFlowBox->setObjectName(QString::fromUtf8("textFlowBox"));
        sizePolicy1.setHeightForWidth(textFlowBox->sizePolicy().hasHeightForWidth());
        textFlowBox->setSizePolicy(sizePolicy1);
        verticalLayout_3 = new QVBoxLayout(textFlowBox);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        horizontalLayout_11 = new QHBoxLayout();
        horizontalLayout_11->setObjectName(QString::fromUtf8("horizontalLayout_11"));
        keepLabelStart = new QLabel(textFlowBox);
        keepLabelStart->setObjectName(QString::fromUtf8("keepLabelStart"));
        keepLabelStart->setMinimumSize(QSize(20, 0));
        keepLabelStart->setMaximumSize(QSize(22, 22));

        horizontalLayout_11->addWidget(keepLabelStart);

        keepLinesStart = new SMSpinBox(textFlowBox);
        keepLinesStart->setObjectName(QString::fromUtf8("keepLinesStart"));
        keepLinesStart->setMaximum(10);

        horizontalLayout_11->addWidget(keepLinesStart);

        keepLabelEnd = new QLabel(textFlowBox);
        keepLabelEnd->setObjectName(QString::fromUtf8("keepLabelEnd"));
        keepLabelEnd->setMinimumSize(QSize(20, 0));
        keepLabelEnd->setMaximumSize(QSize(22, 22));

        horizontalLayout_11->addWidget(keepLabelEnd);

        keepLinesEnd = new SMSpinBox(textFlowBox);
        keepLinesEnd->setObjectName(QString::fromUtf8("keepLinesEnd"));
        keepLinesEnd->setMaximum(10);

        horizontalLayout_11->addWidget(keepLinesEnd);

        horizontalSpacer_6 = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_11->addItem(horizontalSpacer_6);


        verticalLayout_3->addLayout(horizontalLayout_11);

        keepTogether = new SMCheckBox(textFlowBox);
        keepTogether->setObjectName(QString::fromUtf8("keepTogether"));

        verticalLayout_3->addWidget(keepTogether);

        keepWithNext = new SMCheckBox(textFlowBox);
        keepWithNext->setObjectName(QString::fromUtf8("keepWithNext"));

        verticalLayout_3->addWidget(keepWithNext);

        verticalSpacer_4 = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_3->addItem(verticalSpacer_4);


        gridLayout->addWidget(textFlowBox, 0, 1, 1, 1);

        distancesBox = new QGroupBox(tab);
        distancesBox->setObjectName(QString::fromUtf8("distancesBox"));
        sizePolicy1.setHeightForWidth(distancesBox->sizePolicy().hasHeightForWidth());
        distancesBox->setSizePolicy(sizePolicy1);
        horizontalLayout_10 = new QHBoxLayout(distancesBox);
        horizontalLayout_10->setObjectName(QString::fromUtf8("horizontalLayout_10"));
        verticalLayout_10 = new QVBoxLayout();
        verticalLayout_10->setObjectName(QString::fromUtf8("verticalLayout_10"));
        horizontalLayout_6 = new QHBoxLayout();
        horizontalLayout_6->setObjectName(QString::fromUtf8("horizontalLayout_6"));
        alignment = new SMAlignSelect(distancesBox);
        alignment->setObjectName(QString::fromUtf8("alignment"));
        sizePolicy2.setHeightForWidth(alignment->sizePolicy().hasHeightForWidth());
        alignment->setSizePolicy(sizePolicy2);

        horizontalLayout_6->addWidget(alignment);

        direction = new SMDirectionSelect(distancesBox);
        direction->setObjectName(QString::fromUtf8("direction"));

        horizontalLayout_6->addWidget(direction);

        horizontalSpacer_4 = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_6->addItem(horizontalSpacer_4);


        verticalLayout_10->addLayout(horizontalLayout_6);

        horizontalLayout_9 = new QHBoxLayout();
        horizontalLayout_9->setObjectName(QString::fromUtf8("horizontalLayout_9"));
        lineSpacingLabel = new QLabel(distancesBox);
        lineSpacingLabel->setObjectName(QString::fromUtf8("lineSpacingLabel"));
        lineSpacingLabel->setMinimumSize(QSize(20, 0));
        lineSpacingLabel->setMaximumSize(QSize(22, 22));

        horizontalLayout_9->addWidget(lineSpacingLabel);

        lineSpacingMode = new SMScComboBox(distancesBox);
        lineSpacingMode->setObjectName(QString::fromUtf8("lineSpacingMode"));
        QSizePolicy sizePolicy4(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(lineSpacingMode->sizePolicy().hasHeightForWidth());
        lineSpacingMode->setSizePolicy(sizePolicy4);

        horizontalLayout_9->addWidget(lineSpacingMode);

        lineSpacing = new SMScrSpinBox(distancesBox);
        lineSpacing->setObjectName(QString::fromUtf8("lineSpacing"));
        lineSpacing->setMinimum(1.000000000000000);
        lineSpacing->setMaximum(300.000000000000000);

        horizontalLayout_9->addWidget(lineSpacing);

        horizontalSpacer_8 = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_9->addItem(horizontalSpacer_8);


        verticalLayout_10->addLayout(horizontalLayout_9);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        spaceAboveLabel = new QLabel(distancesBox);
        spaceAboveLabel->setObjectName(QString::fromUtf8("spaceAboveLabel"));
        spaceAboveLabel->setMinimumSize(QSize(20, 0));
        spaceAboveLabel->setMaximumSize(QSize(22, 22));

        horizontalLayout_5->addWidget(spaceAboveLabel);

        spaceAbove = new SMScrSpinBox(distancesBox);
        spaceAbove->setObjectName(QString::fromUtf8("spaceAbove"));
        spaceAbove->setMaximum(300.000000000000000);

        horizontalLayout_5->addWidget(spaceAbove);

        spaceBelowLabel = new QLabel(distancesBox);
        spaceBelowLabel->setObjectName(QString::fromUtf8("spaceBelowLabel"));
        spaceBelowLabel->setMinimumSize(QSize(20, 0));
        spaceBelowLabel->setMaximumSize(QSize(22, 22));

        horizontalLayout_5->addWidget(spaceBelowLabel);

        spaceBelow = new SMScrSpinBox(distancesBox);
        spaceBelow->setObjectName(QString::fromUtf8("spaceBelow"));
        spaceBelow->setMaximum(300.000000000000000);

        horizontalLayout_5->addWidget(spaceBelow);

        horizontalSpacer_11 = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_5->addItem(horizontalSpacer_11);


        verticalLayout_10->addLayout(horizontalLayout_5);

        verticalSpacer_3 = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_10->addItem(verticalSpacer_3);


        horizontalLayout_10->addLayout(verticalLayout_10);

        horizontalSpacer_16 = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_10->addItem(horizontalSpacer_16);


        gridLayout->addWidget(distancesBox, 0, 0, 1, 1);


        verticalLayout_8->addLayout(gridLayout);

        tabsBox = new QGroupBox(tab);
        tabsBox->setObjectName(QString::fromUtf8("tabsBox"));
        QSizePolicy sizePolicy5(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(tabsBox->sizePolicy().hasHeightForWidth());
        tabsBox->setSizePolicy(sizePolicy5);
        vboxLayout1 = new QVBoxLayout(tabsBox);
        vboxLayout1->setObjectName(QString::fromUtf8("vboxLayout1"));
        tabsBoxLayout = new QHBoxLayout();
        tabsBoxLayout->setObjectName(QString::fromUtf8("tabsBoxLayout"));
        tabList = new SMTabruler(tabsBox);
        tabList->setObjectName(QString::fromUtf8("tabList"));
        QSizePolicy sizePolicy6(QSizePolicy::MinimumExpanding, QSizePolicy::MinimumExpanding);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(0);
        sizePolicy6.setHeightForWidth(tabList->sizePolicy().hasHeightForWidth());
        tabList->setSizePolicy(sizePolicy6);

        tabsBoxLayout->addWidget(tabList);

        horizontalSpacer_13 = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        tabsBoxLayout->addItem(horizontalSpacer_13);


        vboxLayout1->addLayout(tabsBoxLayout);


        verticalLayout_8->addWidget(tabsBox);

        smColorGroup = new QGroupBox(tab);
        smColorGroup->setObjectName(QString::fromUtf8("smColorGroup"));
        verticalLayout_4 = new QVBoxLayout(smColorGroup);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        smColorGroupLayout = new QHBoxLayout();
        smColorGroupLayout->setObjectName(QString::fromUtf8("smColorGroupLayout"));
        backIcon = new QLabel(smColorGroup);
        backIcon->setObjectName(QString::fromUtf8("backIcon"));
        backIcon->setMinimumSize(QSize(20, 0));
        backIcon->setMaximumSize(QSize(22, 22));

        smColorGroupLayout->addWidget(backIcon);

        backColor_ = new SMColorCombo(smColorGroup);
        backColor_->setObjectName(QString::fromUtf8("backColor_"));

        smColorGroupLayout->addWidget(backColor_);

        backShadeLabel = new QLabel(smColorGroup);
        backShadeLabel->setObjectName(QString::fromUtf8("backShadeLabel"));
        backShadeLabel->setMinimumSize(QSize(20, 0));
        backShadeLabel->setMaximumSize(QSize(22, 22));

        smColorGroupLayout->addWidget(backShadeLabel);

        backShade_ = new SMShadeButton(smColorGroup);
        backShade_->setObjectName(QString::fromUtf8("backShade_"));

        smColorGroupLayout->addWidget(backShade_);

        horizontalSpacer_12 = new QSpacerItem(0, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        smColorGroupLayout->addItem(horizontalSpacer_12);


        verticalLayout_4->addLayout(smColorGroupLayout);


        verticalLayout_8->addWidget(smColorGroup);

        verticalSpacer = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_8->addItem(verticalSpacer);

        tabWidget->addTab(tab, QString());
        tabParagraphEffects = new QWidget();
        tabParagraphEffects->setObjectName(QString::fromUtf8("tabParagraphEffects"));
        verticalLayout_6 = new QVBoxLayout(tabParagraphEffects);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        bulletBox = new QGroupBox(tabParagraphEffects);
        bulletBox->setObjectName(QString::fromUtf8("bulletBox"));
        sizePolicy1.setHeightForWidth(bulletBox->sizePolicy().hasHeightForWidth());
        bulletBox->setSizePolicy(sizePolicy1);
        bulletBox->setCheckable(true);
        bulletBox->setChecked(false);
        verticalLayout = new QVBoxLayout(bulletBox);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        horizontalLayout_7 = new QHBoxLayout();
        horizontalLayout_7->setObjectName(QString::fromUtf8("horizontalLayout_7"));
        bulletCharLabel = new QLabel(bulletBox);
        bulletCharLabel->setObjectName(QString::fromUtf8("bulletCharLabel"));

        horizontalLayout_7->addWidget(bulletCharLabel);

        bulletStrEdit = new SMScComboBox(bulletBox);
        bulletStrEdit->setObjectName(QString::fromUtf8("bulletStrEdit"));
        sizePolicy4.setHeightForWidth(bulletStrEdit->sizePolicy().hasHeightForWidth());
        bulletStrEdit->setSizePolicy(sizePolicy4);
        bulletStrEdit->setMinimumSize(QSize(50, 0));
        bulletStrEdit->setEditable(true);

        horizontalLayout_7->addWidget(bulletStrEdit);

        spacer = new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_7->addItem(spacer);

        bulletCharTableButton = new QPushButton(bulletBox);
        bulletCharTableButton->setObjectName(QString::fromUtf8("bulletCharTableButton"));
        sizePolicy4.setHeightForWidth(bulletCharTableButton->sizePolicy().hasHeightForWidth());
        bulletCharTableButton->setSizePolicy(sizePolicy4);
        bulletCharTableButton->setCheckable(true);

        horizontalLayout_7->addWidget(bulletCharTableButton);

        spacer_2 = new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_7->addItem(spacer_2);


        verticalLayout->addLayout(horizontalLayout_7);


        verticalLayout_6->addWidget(bulletBox);

        numBox = new QGroupBox(tabParagraphEffects);
        numBox->setObjectName(QString::fromUtf8("numBox"));
        sizePolicy1.setHeightForWidth(numBox->sizePolicy().hasHeightForWidth());
        numBox->setSizePolicy(sizePolicy1);
        numBox->setCheckable(true);
        numBox->setChecked(false);
        verticalLayout_5 = new QVBoxLayout(numBox);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        gridLayout_2 = new QGridLayout();
        gridLayout_2->setObjectName(QString::fromUtf8("gridLayout_2"));
        numComboBox = new SMScComboBox(numBox);
        numComboBox->setObjectName(QString::fromUtf8("numComboBox"));
        numComboBox->setMinimumContentsLength(10);

        gridLayout_2->addWidget(numComboBox, 0, 1, 1, 1);

        numFormatLabel = new QLabel(numBox);
        numFormatLabel->setObjectName(QString::fromUtf8("numFormatLabel"));

        gridLayout_2->addWidget(numFormatLabel, 1, 0, 1, 1);

        horizontalSpacer_10 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        gridLayout_2->addItem(horizontalSpacer_10, 0, 2, 1, 1);

        numRestartCombo = new SMScComboBox(numBox);
        numRestartCombo->setObjectName(QString::fromUtf8("numRestartCombo"));

        gridLayout_2->addWidget(numRestartCombo, 4, 1, 1, 1);

        numRestartLabel = new QLabel(numBox);
        numRestartLabel->setObjectName(QString::fromUtf8("numRestartLabel"));
        QSizePolicy sizePolicy7(QSizePolicy::Maximum, QSizePolicy::Preferred);
        sizePolicy7.setHorizontalStretch(0);
        sizePolicy7.setVerticalStretch(0);
        sizePolicy7.setHeightForWidth(numRestartLabel->sizePolicy().hasHeightForWidth());
        numRestartLabel->setSizePolicy(sizePolicy7);

        gridLayout_2->addWidget(numRestartLabel, 4, 0, 1, 1);

        numNewLineEdit = new QLineEdit(numBox);
        numNewLineEdit->setObjectName(QString::fromUtf8("numNewLineEdit"));

        gridLayout_2->addWidget(numNewLineEdit, 0, 4, 1, 1);

        numFormatCombo = new SMNumFormatCombo(numBox);
        numFormatCombo->setObjectName(QString::fromUtf8("numFormatCombo"));
        QSizePolicy sizePolicy8(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy8.setHorizontalStretch(0);
        sizePolicy8.setVerticalStretch(0);
        sizePolicy8.setHeightForWidth(numFormatCombo->sizePolicy().hasHeightForWidth());
        numFormatCombo->setSizePolicy(sizePolicy8);

        gridLayout_2->addWidget(numFormatCombo, 1, 1, 1, 1);

        numStartSpin = new SMSpinBox(numBox);
        numStartSpin->setObjectName(QString::fromUtf8("numStartSpin"));
        numStartSpin->setMaximum(99999);
        numStartSpin->setValue(1);

        gridLayout_2->addWidget(numStartSpin, 2, 3, 1, 1);

        numStartLabel = new QLabel(numBox);
        numStartLabel->setObjectName(QString::fromUtf8("numStartLabel"));

        gridLayout_2->addWidget(numStartLabel, 2, 2, 1, 1);

        numRestartHigherBox = new SMCheckBox(numBox);
        numRestartHigherBox->setObjectName(QString::fromUtf8("numRestartHigherBox"));

        gridLayout_2->addWidget(numRestartHigherBox, 5, 3, 1, 2);

        numLevelLabel = new QLabel(numBox);
        numLevelLabel->setObjectName(QString::fromUtf8("numLevelLabel"));

        gridLayout_2->addWidget(numLevelLabel, 2, 0, 1, 1);

        numPrefix = new QLineEdit(numBox);
        numPrefix->setObjectName(QString::fromUtf8("numPrefix"));

        gridLayout_2->addWidget(numPrefix, 3, 1, 1, 1);

        numSuffixLabel = new QLabel(numBox);
        numSuffixLabel->setObjectName(QString::fromUtf8("numSuffixLabel"));

        gridLayout_2->addWidget(numSuffixLabel, 3, 2, 1, 1);

        numComboLabel = new QLabel(numBox);
        numComboLabel->setObjectName(QString::fromUtf8("numComboLabel"));

        gridLayout_2->addWidget(numComboLabel, 0, 0, 1, 1);

        numLevelSpin = new SMSpinBox(numBox);
        numLevelSpin->setObjectName(QString::fromUtf8("numLevelSpin"));

        gridLayout_2->addWidget(numLevelSpin, 2, 1, 1, 1);

        numPrefixLabel = new QLabel(numBox);
        numPrefixLabel->setObjectName(QString::fromUtf8("numPrefixLabel"));

        gridLayout_2->addWidget(numPrefixLabel, 3, 0, 1, 1);

        numNewLabel = new QLabel(numBox);
        numNewLabel->setObjectName(QString::fromUtf8("numNewLabel"));

        gridLayout_2->addWidget(numNewLabel, 0, 3, 1, 1);

        numSuffix = new QLineEdit(numBox);
        numSuffix->setObjectName(QString::fromUtf8("numSuffix"));

        gridLayout_2->addWidget(numSuffix, 3, 3, 1, 1);

        numRestartOtherBox = new SMCheckBox(numBox);
        numRestartOtherBox->setObjectName(QString::fromUtf8("numRestartOtherBox"));

        gridLayout_2->addWidget(numRestartOtherBox, 4, 3, 1, 2);


        verticalLayout_5->addLayout(gridLayout_2);


        verticalLayout_6->addWidget(numBox);

        dropCapsBox = new QGroupBox(tabParagraphEffects);
        dropCapsBox->setObjectName(QString::fromUtf8("dropCapsBox"));
        sizePolicy1.setHeightForWidth(dropCapsBox->sizePolicy().hasHeightForWidth());
        dropCapsBox->setSizePolicy(sizePolicy1);
        dropCapsBox->setCheckable(true);
        dropCapsBox->setChecked(true);
        _2 = new QVBoxLayout(dropCapsBox);
        _2->setObjectName(QString::fromUtf8("_2"));
        dropCapsBoxLayout = new QHBoxLayout();
        dropCapsBoxLayout->setObjectName(QString::fromUtf8("dropCapsBoxLayout"));
        dropCapsLineLabel = new QLabel(dropCapsBox);
        dropCapsLineLabel->setObjectName(QString::fromUtf8("dropCapsLineLabel"));

        dropCapsBoxLayout->addWidget(dropCapsLineLabel);

        dropCapLines = new SMSpinBox(dropCapsBox);
        dropCapLines->setObjectName(QString::fromUtf8("dropCapLines"));
        dropCapLines->setMinimum(2);
        dropCapLines->setMaximum(20);

        dropCapsBoxLayout->addWidget(dropCapLines);

        horizontalSpacer_2 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        dropCapsBoxLayout->addItem(horizontalSpacer_2);


        _2->addLayout(dropCapsBoxLayout);


        verticalLayout_6->addWidget(dropCapsBox);

        horizontalLayout_12 = new QHBoxLayout();
        horizontalLayout_12->setObjectName(QString::fromUtf8("horizontalLayout_12"));
        parEffectCharStyleComboLabel = new QLabel(tabParagraphEffects);
        parEffectCharStyleComboLabel->setObjectName(QString::fromUtf8("parEffectCharStyleComboLabel"));
        sizePolicy3.setHeightForWidth(parEffectCharStyleComboLabel->sizePolicy().hasHeightForWidth());
        parEffectCharStyleComboLabel->setSizePolicy(sizePolicy3);

        horizontalLayout_12->addWidget(parEffectCharStyleComboLabel);

        parEffectCharStyleCombo = new SMScComboBox(tabParagraphEffects);
        parEffectCharStyleCombo->setObjectName(QString::fromUtf8("parEffectCharStyleCombo"));

        horizontalLayout_12->addWidget(parEffectCharStyleCombo);

        spacer_6 = new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_12->addItem(spacer_6);

        distFromTextLabel = new QLabel(tabParagraphEffects);
        distFromTextLabel->setObjectName(QString::fromUtf8("distFromTextLabel"));
        QSizePolicy sizePolicy9(QSizePolicy::Minimum, QSizePolicy::Maximum);
        sizePolicy9.setHorizontalStretch(0);
        sizePolicy9.setVerticalStretch(0);
        sizePolicy9.setHeightForWidth(distFromTextLabel->sizePolicy().hasHeightForWidth());
        distFromTextLabel->setSizePolicy(sizePolicy9);

        horizontalLayout_12->addWidget(distFromTextLabel);

        parEffectOffset = new SMScrSpinBox(tabParagraphEffects);
        parEffectOffset->setObjectName(QString::fromUtf8("parEffectOffset"));
        parEffectOffset->setMinimum(-3000.000000000000000);
        parEffectOffset->setMaximum(3000.000000000000000);

        horizontalLayout_12->addWidget(parEffectOffset);

        horizontalSpacer_14 = new QSpacerItem(20, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_12->addItem(horizontalSpacer_14);

        parEffectIndentBox = new SMCheckBox(tabParagraphEffects);
        parEffectIndentBox->setObjectName(QString::fromUtf8("parEffectIndentBox"));

        horizontalLayout_12->addWidget(parEffectIndentBox);


        verticalLayout_6->addLayout(horizontalLayout_12);

        horizontalLayout_15 = new QHBoxLayout();
        horizontalLayout_15->setObjectName(QString::fromUtf8("horizontalLayout_15"));
        spacer_10 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_15->addItem(spacer_10);

        parentParEffectsButton = new QToolButton(tabParagraphEffects);
        parentParEffectsButton->setObjectName(QString::fromUtf8("parentParEffectsButton"));
        parentParEffectsButton->setEnabled(true);

        horizontalLayout_15->addWidget(parentParEffectsButton);


        verticalLayout_6->addLayout(horizontalLayout_15);

        verticalSpacer_2 = new QSpacerItem(20, 0, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_6->addItem(verticalSpacer_2);

        tabWidget->addTab(tabParagraphEffects, QString());
        dropCapsBox->raise();
        numBox->raise();
        bulletBox->raise();
        tabCharacterStyle = new QWidget();
        tabCharacterStyle->setObjectName(QString::fromUtf8("tabCharacterStyle"));
        vboxLayout2 = new QVBoxLayout(tabCharacterStyle);
        vboxLayout2->setObjectName(QString::fromUtf8("vboxLayout2"));
        vboxLayout2->setContentsMargins(0, 0, 0, 0);
        cpage = new SMCStyleWidget(tabCharacterStyle);
        cpage->setObjectName(QString::fromUtf8("cpage"));

        vboxLayout2->addWidget(cpage);

        tabWidget->addTab(tabCharacterStyle, QString());

        vboxLayout->addWidget(tabWidget);

#if QT_CONFIG(shortcut)
        minSpaceLabel->setBuddy(minSpaceSpin);
        minGlyphExtLabel->setBuddy(minGlyphExtSpin);
        maxGlyphExtLabel->setBuddy(maxGlyphExtSpin);
        lineSpacingLabel->setBuddy(lineSpacingMode);
        spaceAboveLabel->setBuddy(spaceAbove);
        spaceBelowLabel->setBuddy(spaceBelow);
        numFormatLabel->setBuddy(numFormatCombo);
        dropCapsLineLabel->setBuddy(dropCapLines);
        distFromTextLabel->setBuddy(parEffectOffset);
#endif // QT_CONFIG(shortcut)
        QWidget::setTabOrder(tabWidget, parentCombo);
        QWidget::setTabOrder(parentCombo, lineSpacingMode);
        QWidget::setTabOrder(lineSpacingMode, lineSpacing);
        QWidget::setTabOrder(lineSpacing, spaceAbove);
        QWidget::setTabOrder(spaceAbove, spaceBelow);
        QWidget::setTabOrder(spaceBelow, keepTogether);
        QWidget::setTabOrder(keepTogether, keepWithNext);
        QWidget::setTabOrder(keepWithNext, optMarginRadioNone);
        QWidget::setTabOrder(optMarginRadioNone, optMarginRadioBoth);
        QWidget::setTabOrder(optMarginRadioBoth, optMarginRadioLeft);
        QWidget::setTabOrder(optMarginRadioLeft, optMarginRadioRight);
        QWidget::setTabOrder(optMarginRadioRight, optMarginDefaultButton);
        QWidget::setTabOrder(optMarginDefaultButton, optMarginParentButton);
        QWidget::setTabOrder(optMarginParentButton, minSpaceSpin);
        QWidget::setTabOrder(minSpaceSpin, maxGlyphExtSpin);
        QWidget::setTabOrder(maxGlyphExtSpin, bulletBox);
        QWidget::setTabOrder(bulletBox, bulletStrEdit);
        QWidget::setTabOrder(bulletStrEdit, bulletCharTableButton);
        QWidget::setTabOrder(bulletCharTableButton, numBox);
        QWidget::setTabOrder(numBox, dropCapsBox);
        QWidget::setTabOrder(dropCapsBox, dropCapLines);
        QWidget::setTabOrder(dropCapLines, parEffectCharStyleCombo);
        QWidget::setTabOrder(parEffectCharStyleCombo, parEffectOffset);
        QWidget::setTabOrder(parEffectOffset, parEffectIndentBox);
        QWidget::setTabOrder(parEffectIndentBox, parentParEffectsButton);

        retranslateUi(SMPStyleWidget);

        tabWidget->setCurrentIndex(0);
        numComboBox->setCurrentIndex(-1);


        QMetaObject::connectSlotsByName(SMPStyleWidget);
    } // setupUi

    void retranslateUi(QWidget *SMPStyleWidget)
    {
        parentLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Based On:", nullptr));
#if QT_CONFIG(tooltip)
        parentCombo->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Parent Style", nullptr));
#endif // QT_CONFIG(tooltip)
        advSettingsGroupBox->setTitle(QCoreApplication::translate("SMPStyleWidget", "Advanced Settings", nullptr));
        minSpaceLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Min. Space Width:", nullptr));
#if QT_CONFIG(tooltip)
        minSpaceSpin->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Maximum white space compression allowed. Expressed as a percentage of the current white space value.", nullptr));
#endif // QT_CONFIG(tooltip)
        glyphExtensionLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Glyph Extension ", nullptr));
        minGlyphExtLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Min:", nullptr));
#if QT_CONFIG(tooltip)
        minGlyphExtSpin->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Maximum compression of glyphs", nullptr));
#endif // QT_CONFIG(tooltip)
        maxGlyphExtLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Max:", nullptr));
#if QT_CONFIG(tooltip)
        maxGlyphExtSpin->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Maximum extension of glyphs", nullptr));
#endif // QT_CONFIG(tooltip)
        label_3->setText(QCoreApplication::translate("SMPStyleWidget", "Consecutive Hyphenations Allowed:", nullptr));
        opticalMarginsGroupBox->setTitle(QCoreApplication::translate("SMPStyleWidget", "Optical Margins", nullptr));
        optMarginRadioLeft->setText(QCoreApplication::translate("SMPStyleWidget", "Left Only", nullptr));
        optMarginRadioBoth->setText(QCoreApplication::translate("SMPStyleWidget", "Both Sides", nullptr));
        optMarginRadioNone->setText(QCoreApplication::translate("SMPStyleWidget", "None", nullptr));
        optMarginRadioRight->setText(QCoreApplication::translate("SMPStyleWidget", "Right Only", nullptr));
        optMarginDefaultButton->setText(QCoreApplication::translate("SMPStyleWidget", "Reset to Default", nullptr));
        optMarginParentButton->setText(QCoreApplication::translate("SMPStyleWidget", "Use Parent Value", nullptr));
        textFlowBox->setTitle(QCoreApplication::translate("SMPStyleWidget", "Orphans and Widows", nullptr));
#if QT_CONFIG(tooltip)
        keepLabelStart->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Ensure that first lines of a paragraph won't end up separated from the rest (known as widow/orphan control)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        keepLinesStart->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Ensure that first lines of a paragraph won't end up separated from the rest (known as widow/orphan control)", nullptr));
#endif // QT_CONFIG(tooltip)
        keepLinesStart->setSuffix(QCoreApplication::translate("SMPStyleWidget", " lines", nullptr));
#if QT_CONFIG(tooltip)
        keepLabelEnd->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Ensure that last lines of a paragraph won't end up separated from the rest (known as widow/orphan control)", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        keepLinesEnd->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Ensure that last lines of a paragraph won't end up separated from the rest (known as widow/orphan control)", nullptr));
#endif // QT_CONFIG(tooltip)
        keepLinesEnd->setSuffix(QCoreApplication::translate("SMPStyleWidget", " lines", nullptr));
#if QT_CONFIG(tooltip)
        keepTogether->setToolTip(QCoreApplication::translate("SMPStyleWidget", "If checked, ensures that the paragraph won't be split across multiple pages or columns", nullptr));
#endif // QT_CONFIG(tooltip)
        keepTogether->setText(QCoreApplication::translate("SMPStyleWidget", "Do not split paragraph", nullptr));
#if QT_CONFIG(tooltip)
        keepWithNext->setToolTip(QCoreApplication::translate("SMPStyleWidget", "If checked, automatically moves the paragraph to the next column or page if the next paragraph isn't on the same page or column", nullptr));
#endif // QT_CONFIG(tooltip)
        keepWithNext->setText(QCoreApplication::translate("SMPStyleWidget", "Keep with next paragraph", nullptr));
        distancesBox->setTitle(QCoreApplication::translate("SMPStyleWidget", "Alignment and Distances", nullptr));
        lineSpacingLabel->setText(QString());
#if QT_CONFIG(tooltip)
        lineSpacingMode->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Line Spacing Mode", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        lineSpacing->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Line Spacing", nullptr));
#endif // QT_CONFIG(tooltip)
        spaceAboveLabel->setText(QString());
#if QT_CONFIG(tooltip)
        spaceAbove->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Space Above", nullptr));
#endif // QT_CONFIG(tooltip)
        spaceBelowLabel->setText(QString());
#if QT_CONFIG(tooltip)
        spaceBelow->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Space Below", nullptr));
#endif // QT_CONFIG(tooltip)
        tabsBox->setTitle(QCoreApplication::translate("SMPStyleWidget", "Tabs and Indentation", nullptr));
        smColorGroup->setTitle(QCoreApplication::translate("SMPStyleWidget", "Colors", nullptr));
        backIcon->setText(QString());
#if QT_CONFIG(tooltip)
        backColor_->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Background Color", nullptr));
#endif // QT_CONFIG(tooltip)
        backShadeLabel->setText(QString());
#if QT_CONFIG(tooltip)
        backShade_->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Background Shade", nullptr));
#endif // QT_CONFIG(tooltip)
        backShade_->setText(QString());
        tabWidget->setTabText(tabWidget->indexOf(tab), QCoreApplication::translate("SMPStyleWidget", "Properties", nullptr));
        bulletBox->setTitle(QCoreApplication::translate("SMPStyleWidget", "Bulleted List", nullptr));
        bulletCharLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Bullet Char(s):", nullptr));
#if QT_CONFIG(tooltip)
        bulletCharTableButton->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Enhanced Character Table for inserting custom characters as bullets", nullptr));
#endif // QT_CONFIG(tooltip)
        bulletCharTableButton->setText(QCoreApplication::translate("SMPStyleWidget", "Char Table", nullptr));
        numBox->setTitle(QCoreApplication::translate("SMPStyleWidget", "Numbered List", nullptr));
        numFormatLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Numbering Style:", nullptr));
        numRestartLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Restart:", nullptr));
        numStartLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Start with:", nullptr));
        numRestartHigherBox->setText(QCoreApplication::translate("SMPStyleWidget", "Restart after higher level", nullptr));
        numLevelLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Level:", nullptr));
        numSuffixLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Suffix:", nullptr));
        numComboLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Set:", nullptr));
        numPrefixLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Prefix:", nullptr));
        numNewLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Create New Set:", nullptr));
        numRestartOtherBox->setText(QCoreApplication::translate("SMPStyleWidget", "Restart after other format", nullptr));
        dropCapsBox->setTitle(QCoreApplication::translate("SMPStyleWidget", "Drop Caps", nullptr));
        dropCapsLineLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Lines:", nullptr));
#if QT_CONFIG(tooltip)
        dropCapLines->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Drop Cap Lines", nullptr));
#endif // QT_CONFIG(tooltip)
        parEffectCharStyleComboLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Character Style for Effect:", nullptr));
#if QT_CONFIG(tooltip)
        parEffectCharStyleCombo->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Choose Character Style or leave blank to use default Paragraph Style", nullptr));
#endif // QT_CONFIG(tooltip)
        distFromTextLabel->setText(QCoreApplication::translate("SMPStyleWidget", "Distance from Text:", nullptr));
#if QT_CONFIG(tooltip)
        parEffectOffset->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Paragraph Effects characters offset", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        parEffectIndentBox->setToolTip(QCoreApplication::translate("SMPStyleWidget", "Hang Paragraph Effect before paragraph indent", nullptr));
#endif // QT_CONFIG(tooltip)
        parEffectIndentBox->setText(QCoreApplication::translate("SMPStyleWidget", "Auto-Indent", nullptr));
        parentParEffectsButton->setText(QCoreApplication::translate("SMPStyleWidget", "Use Parent Values", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabParagraphEffects), QCoreApplication::translate("SMPStyleWidget", "Paragraph Effects", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabCharacterStyle), QCoreApplication::translate("SMPStyleWidget", "Character Style", nullptr));
        (void)SMPStyleWidget;
    } // retranslateUi

};

namespace Ui {
    class SMPStyleWidget: public Ui_SMPStyleWidget {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_SMPSTYLEWIDGET_H
