/****************************************************************************
** Meta object code from reading C++ file 'propertywidget_advanced.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/propertywidget_advanced.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertywidget_advanced.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PropertyWidget_Advanced_t {
    QByteArrayData data[32];
    char stringdata0[459];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PropertyWidget_Advanced_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PropertyWidget_Advanced_t qt_meta_stringdata_PropertyWidget_Advanced = {
    {
QT_MOC_LITERAL(0, 0, 23), // "PropertyWidget_Advanced"
QT_MOC_LITERAL(1, 24, 13), // "setMainWindow"
QT_MOC_LITERAL(2, 38, 0), // ""
QT_MOC_LITERAL(3, 39, 18), // "ScribusMainWindow*"
QT_MOC_LITERAL(4, 58, 2), // "mw"
QT_MOC_LITERAL(5, 61, 6), // "setDoc"
QT_MOC_LITERAL(6, 68, 11), // "ScribusDoc*"
QT_MOC_LITERAL(7, 80, 1), // "d"
QT_MOC_LITERAL(8, 82, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(9, 105, 13), // "iconSetChange"
QT_MOC_LITERAL(10, 119, 14), // "languageChange"
QT_MOC_LITERAL(11, 134, 10), // "unitChange"
QT_MOC_LITERAL(12, 145, 12), // "localeChange"
QT_MOC_LITERAL(13, 158, 18), // "showBaseLineOffset"
QT_MOC_LITERAL(14, 177, 1), // "e"
QT_MOC_LITERAL(15, 179, 14), // "showTextScaleH"
QT_MOC_LITERAL(16, 194, 14), // "showTextScaleV"
QT_MOC_LITERAL(17, 209, 12), // "showTracking"
QT_MOC_LITERAL(18, 222, 15), // "updateCharStyle"
QT_MOC_LITERAL(19, 238, 9), // "CharStyle"
QT_MOC_LITERAL(20, 248, 9), // "charStyle"
QT_MOC_LITERAL(21, 258, 11), // "updateStyle"
QT_MOC_LITERAL(22, 270, 14), // "ParagraphStyle"
QT_MOC_LITERAL(23, 285, 10), // "newCurrent"
QT_MOC_LITERAL(24, 296, 20), // "handleBaselineOffset"
QT_MOC_LITERAL(25, 317, 21), // "handleMinWordTracking"
QT_MOC_LITERAL(26, 339, 22), // "handleNormWordTracking"
QT_MOC_LITERAL(27, 362, 23), // "handleMinGlyphExtension"
QT_MOC_LITERAL(28, 386, 23), // "handleMaxGlyphExtension"
QT_MOC_LITERAL(29, 410, 16), // "handleTextScaleH"
QT_MOC_LITERAL(30, 427, 16), // "handleTextScaleV"
QT_MOC_LITERAL(31, 444, 14) // "handleTracking"

    },
    "PropertyWidget_Advanced\0setMainWindow\0"
    "\0ScribusMainWindow*\0mw\0setDoc\0ScribusDoc*\0"
    "d\0handleSelectionChanged\0iconSetChange\0"
    "languageChange\0unitChange\0localeChange\0"
    "showBaseLineOffset\0e\0showTextScaleH\0"
    "showTextScaleV\0showTracking\0updateCharStyle\0"
    "CharStyle\0charStyle\0updateStyle\0"
    "ParagraphStyle\0newCurrent\0"
    "handleBaselineOffset\0handleMinWordTracking\0"
    "handleNormWordTracking\0handleMinGlyphExtension\0"
    "handleMaxGlyphExtension\0handleTextScaleH\0"
    "handleTextScaleV\0handleTracking"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PropertyWidget_Advanced[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      21,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    1,  119,    2, 0x0a /* Public */,
       5,    1,  122,    2, 0x0a /* Public */,
       8,    0,  125,    2, 0x0a /* Public */,
       9,    0,  126,    2, 0x0a /* Public */,
      10,    0,  127,    2, 0x0a /* Public */,
      11,    0,  128,    2, 0x0a /* Public */,
      12,    0,  129,    2, 0x0a /* Public */,
      13,    1,  130,    2, 0x0a /* Public */,
      15,    1,  133,    2, 0x0a /* Public */,
      16,    1,  136,    2, 0x0a /* Public */,
      17,    1,  139,    2, 0x0a /* Public */,
      18,    1,  142,    2, 0x0a /* Public */,
      21,    1,  145,    2, 0x0a /* Public */,
      24,    0,  148,    2, 0x08 /* Private */,
      25,    0,  149,    2, 0x08 /* Private */,
      26,    0,  150,    2, 0x08 /* Private */,
      27,    0,  151,    2, 0x08 /* Private */,
      28,    0,  152,    2, 0x08 /* Private */,
      29,    0,  153,    2, 0x08 /* Private */,
      30,    0,  154,    2, 0x08 /* Private */,
      31,    0,  155,    2, 0x08 /* Private */,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 3,    4,
    QMetaType::Void, 0x80000000 | 6,    7,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,   14,
    QMetaType::Void, QMetaType::Double,   14,
    QMetaType::Void, QMetaType::Double,   14,
    QMetaType::Void, QMetaType::Double,   14,
    QMetaType::Void, 0x80000000 | 19,   20,
    QMetaType::Void, 0x80000000 | 22,   23,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PropertyWidget_Advanced::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertyWidget_Advanced *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->setMainWindow((*reinterpret_cast< ScribusMainWindow*(*)>(_a[1]))); break;
        case 1: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 2: _t->handleSelectionChanged(); break;
        case 3: _t->iconSetChange(); break;
        case 4: _t->languageChange(); break;
        case 5: _t->unitChange(); break;
        case 6: _t->localeChange(); break;
        case 7: _t->showBaseLineOffset((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 8: _t->showTextScaleH((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 9: _t->showTextScaleV((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 10: _t->showTracking((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 11: _t->updateCharStyle((*reinterpret_cast< const CharStyle(*)>(_a[1]))); break;
        case 12: _t->updateStyle((*reinterpret_cast< const ParagraphStyle(*)>(_a[1]))); break;
        case 13: _t->handleBaselineOffset(); break;
        case 14: _t->handleMinWordTracking(); break;
        case 15: _t->handleNormWordTracking(); break;
        case 16: _t->handleMinGlyphExtension(); break;
        case 17: _t->handleMaxGlyphExtension(); break;
        case 18: _t->handleTextScaleH(); break;
        case 19: _t->handleTextScaleV(); break;
        case 20: _t->handleTracking(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PropertyWidget_Advanced::staticMetaObject = { {
    QMetaObject::SuperData::link<QFrame::staticMetaObject>(),
    qt_meta_stringdata_PropertyWidget_Advanced.data,
    qt_meta_data_PropertyWidget_Advanced,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PropertyWidget_Advanced::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PropertyWidget_Advanced::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PropertyWidget_Advanced.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PropertyWidget_AdvancedBase"))
        return static_cast< Ui::PropertyWidget_AdvancedBase*>(this);
    if (!strcmp(_clname, "PropertyWidgetBase"))
        return static_cast< PropertyWidgetBase*>(this);
    return QFrame::qt_metacast(_clname);
}

int PropertyWidget_Advanced::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QFrame::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 21)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 21;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 21)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 21;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
