/********************************************************************************
** Form generated from reading UI file 'transparencypalette.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_TRANSPARENCYPALETTE_H
#define UI_TRANSPARENCYPALETTE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QStackedWidget>
#include <QtWidgets/QTabWidget>
#include <QtWidgets/QToolButton>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include <ui/scrspinbox.h>
#include "ui/gradienteditor.h"

QT_BEGIN_NAMESPACE

class Ui_transparencyPalette
{
public:
    QVBoxLayout *verticalLayout_6;
    QHBoxLayout *horizontalLayout;
    QToolButton *editLineSelector;
    QToolButton *editFillSelector;
    QSpacerItem *horizontalSpacer;
    QStackedWidget *stackedWidget;
    QWidget *pageS;
    QVBoxLayout *verticalLayout_5;
    QGridLayout *gridLayout;
    QLabel *label_2;
    ScrSpinBox *strokeOpacity;
    QLabel *label_3;
    QComboBox *blendModeStroke;
    QSpacerItem *verticalSpacer;
    QWidget *page_2;
    QVBoxLayout *verticalLayout_3;
    QTabWidget *tabWidget;
    QWidget *tabSolid;
    QVBoxLayout *verticalLayout_2;
    QHBoxLayout *horizontalLayout_4;
    QLabel *label_4;
    ScrSpinBox *fillOpacity;
    QSpacerItem *verticalSpacer_2;
    QWidget *tabGradient;
    QVBoxLayout *verticalLayout_4;
    QHBoxLayout *horizontalLayout_3;
    QComboBox *gradientType;
    QToolButton *gradEditButton;
    GradientEditor *gradEdit;
    QComboBox *namedGradient;
    QCheckBox *transpCalcGradient;
    QWidget *tabPattern;
    QVBoxLayout *verticalLayout;
    QListWidget *patternBox;
    QHBoxLayout *horizontalLayout_2;
    QToolButton *editPatternProps;
    QSpacerItem *horizontalSpacer_3;
    QCheckBox *transpCalcPattern;
    QCheckBox *usePatternInverted;
    QHBoxLayout *horizontalLayout_5;
    QLabel *label_5;
    QComboBox *blendModeFill;

    void setupUi(QWidget *transparencyPalette)
    {
        if (transparencyPalette->objectName().isEmpty())
            transparencyPalette->setObjectName(QString::fromUtf8("transparencyPalette"));
        transparencyPalette->resize(236, 373);
        verticalLayout_6 = new QVBoxLayout(transparencyPalette);
        verticalLayout_6->setSpacing(3);
        verticalLayout_6->setObjectName(QString::fromUtf8("verticalLayout_6"));
        verticalLayout_6->setContentsMargins(3, 3, 3, 3);
        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(3);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        editLineSelector = new QToolButton(transparencyPalette);
        editLineSelector->setObjectName(QString::fromUtf8("editLineSelector"));
        editLineSelector->setCheckable(true);
        editLineSelector->setAutoExclusive(true);

        horizontalLayout->addWidget(editLineSelector);

        editFillSelector = new QToolButton(transparencyPalette);
        editFillSelector->setObjectName(QString::fromUtf8("editFillSelector"));
        editFillSelector->setCheckable(true);
        editFillSelector->setAutoExclusive(true);

        horizontalLayout->addWidget(editFillSelector);

        horizontalSpacer = new QSpacerItem(40, 2, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_6->addLayout(horizontalLayout);

        stackedWidget = new QStackedWidget(transparencyPalette);
        stackedWidget->setObjectName(QString::fromUtf8("stackedWidget"));
        pageS = new QWidget();
        pageS->setObjectName(QString::fromUtf8("pageS"));
        verticalLayout_5 = new QVBoxLayout(pageS);
        verticalLayout_5->setSpacing(3);
        verticalLayout_5->setObjectName(QString::fromUtf8("verticalLayout_5"));
        verticalLayout_5->setContentsMargins(0, 0, 0, 0);
        gridLayout = new QGridLayout();
        gridLayout->setSpacing(3);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        label_2 = new QLabel(pageS);
        label_2->setObjectName(QString::fromUtf8("label_2"));

        gridLayout->addWidget(label_2, 0, 0, 1, 1);

        strokeOpacity = new ScrSpinBox(pageS);
        strokeOpacity->setObjectName(QString::fromUtf8("strokeOpacity"));
        strokeOpacity->setMaximum(100);
        strokeOpacity->setSingleStep(10);
        strokeOpacity->setValue(100);

        gridLayout->addWidget(strokeOpacity, 0, 1, 1, 1);

        label_3 = new QLabel(pageS);
        label_3->setObjectName(QString::fromUtf8("label_3"));

        gridLayout->addWidget(label_3, 1, 0, 1, 1);

        blendModeStroke = new QComboBox(pageS);
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->addItem(QString());
        blendModeStroke->setObjectName(QString::fromUtf8("blendModeStroke"));

        gridLayout->addWidget(blendModeStroke, 1, 1, 1, 1);


        verticalLayout_5->addLayout(gridLayout);

        verticalSpacer = new QSpacerItem(20, 240, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_5->addItem(verticalSpacer);

        stackedWidget->addWidget(pageS);
        page_2 = new QWidget();
        page_2->setObjectName(QString::fromUtf8("page_2"));
        verticalLayout_3 = new QVBoxLayout(page_2);
        verticalLayout_3->setSpacing(3);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        verticalLayout_3->setContentsMargins(0, 0, 0, 0);
        tabWidget = new QTabWidget(page_2);
        tabWidget->setObjectName(QString::fromUtf8("tabWidget"));
        tabSolid = new QWidget();
        tabSolid->setObjectName(QString::fromUtf8("tabSolid"));
        verticalLayout_2 = new QVBoxLayout(tabSolid);
        verticalLayout_2->setSpacing(3);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        verticalLayout_2->setContentsMargins(3, 3, 3, 3);
        horizontalLayout_4 = new QHBoxLayout();
        horizontalLayout_4->setSpacing(3);
        horizontalLayout_4->setObjectName(QString::fromUtf8("horizontalLayout_4"));
        label_4 = new QLabel(tabSolid);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        horizontalLayout_4->addWidget(label_4);

        fillOpacity = new ScrSpinBox(tabSolid);
        fillOpacity->setObjectName(QString::fromUtf8("fillOpacity"));
        fillOpacity->setMaximum(100);
        fillOpacity->setSingleStep(10);
        fillOpacity->setValue(100);

        horizontalLayout_4->addWidget(fillOpacity);


        verticalLayout_2->addLayout(horizontalLayout_4);

        verticalSpacer_2 = new QSpacerItem(20, 195, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_2);

        tabWidget->addTab(tabSolid, QString());
        tabGradient = new QWidget();
        tabGradient->setObjectName(QString::fromUtf8("tabGradient"));
        verticalLayout_4 = new QVBoxLayout(tabGradient);
        verticalLayout_4->setSpacing(3);
        verticalLayout_4->setObjectName(QString::fromUtf8("verticalLayout_4"));
        verticalLayout_4->setContentsMargins(3, 3, 3, 3);
        horizontalLayout_3 = new QHBoxLayout();
        horizontalLayout_3->setSpacing(3);
        horizontalLayout_3->setObjectName(QString::fromUtf8("horizontalLayout_3"));
        gradientType = new QComboBox(tabGradient);
        gradientType->addItem(QString());
        gradientType->addItem(QString());
        gradientType->setObjectName(QString::fromUtf8("gradientType"));

        horizontalLayout_3->addWidget(gradientType);

        gradEditButton = new QToolButton(tabGradient);
        gradEditButton->setObjectName(QString::fromUtf8("gradEditButton"));
        gradEditButton->setCheckable(true);

        horizontalLayout_3->addWidget(gradEditButton);


        verticalLayout_4->addLayout(horizontalLayout_3);

        gradEdit = new GradientEditor(tabGradient);
        gradEdit->setObjectName(QString::fromUtf8("gradEdit"));
        QSizePolicy sizePolicy(QSizePolicy::Preferred, QSizePolicy::Expanding);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(gradEdit->sizePolicy().hasHeightForWidth());
        gradEdit->setSizePolicy(sizePolicy);
        gradEdit->setMinimumSize(QSize(200, 173));
        gradEdit->setFrameShape(QFrame::NoFrame);
        gradEdit->setFrameShadow(QFrame::Raised);

        verticalLayout_4->addWidget(gradEdit);

        namedGradient = new QComboBox(tabGradient);
        namedGradient->setObjectName(QString::fromUtf8("namedGradient"));

        verticalLayout_4->addWidget(namedGradient);

        transpCalcGradient = new QCheckBox(tabGradient);
        transpCalcGradient->setObjectName(QString::fromUtf8("transpCalcGradient"));

        verticalLayout_4->addWidget(transpCalcGradient);

        tabWidget->addTab(tabGradient, QString());
        tabPattern = new QWidget();
        tabPattern->setObjectName(QString::fromUtf8("tabPattern"));
        verticalLayout = new QVBoxLayout(tabPattern);
        verticalLayout->setSpacing(3);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(3, 3, 3, 3);
        patternBox = new QListWidget(tabPattern);
        patternBox->setObjectName(QString::fromUtf8("patternBox"));
        patternBox->setMovement(QListView::Snap);
        patternBox->setFlow(QListView::LeftToRight);
        patternBox->setProperty("isWrapping", QVariant(true));
        patternBox->setResizeMode(QListView::Adjust);
        patternBox->setViewMode(QListView::IconMode);
        patternBox->setWordWrap(true);

        verticalLayout->addWidget(patternBox);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(3);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        editPatternProps = new QToolButton(tabPattern);
        editPatternProps->setObjectName(QString::fromUtf8("editPatternProps"));

        horizontalLayout_2->addWidget(editPatternProps);

        horizontalSpacer_3 = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout_2->addItem(horizontalSpacer_3);


        verticalLayout->addLayout(horizontalLayout_2);

        transpCalcPattern = new QCheckBox(tabPattern);
        transpCalcPattern->setObjectName(QString::fromUtf8("transpCalcPattern"));

        verticalLayout->addWidget(transpCalcPattern);

        usePatternInverted = new QCheckBox(tabPattern);
        usePatternInverted->setObjectName(QString::fromUtf8("usePatternInverted"));

        verticalLayout->addWidget(usePatternInverted);

        tabWidget->addTab(tabPattern, QString());

        verticalLayout_3->addWidget(tabWidget);

        horizontalLayout_5 = new QHBoxLayout();
        horizontalLayout_5->setSpacing(3);
        horizontalLayout_5->setObjectName(QString::fromUtf8("horizontalLayout_5"));
        label_5 = new QLabel(page_2);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        horizontalLayout_5->addWidget(label_5);

        blendModeFill = new QComboBox(page_2);
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->addItem(QString());
        blendModeFill->setObjectName(QString::fromUtf8("blendModeFill"));

        horizontalLayout_5->addWidget(blendModeFill);


        verticalLayout_3->addLayout(horizontalLayout_5);

        stackedWidget->addWidget(page_2);

        verticalLayout_6->addWidget(stackedWidget);


        retranslateUi(transparencyPalette);

        stackedWidget->setCurrentIndex(0);
        tabWidget->setCurrentIndex(0);


        QMetaObject::connectSlotsByName(transparencyPalette);
    } // setupUi

    void retranslateUi(QWidget *transparencyPalette)
    {
#if QT_CONFIG(tooltip)
        editLineSelector->setToolTip(QCoreApplication::translate("transparencyPalette", "Edit Line Color Properties", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(tooltip)
        editFillSelector->setToolTip(QCoreApplication::translate("transparencyPalette", "Edit Fill Color Properties", nullptr));
#endif // QT_CONFIG(tooltip)
        label_2->setText(QCoreApplication::translate("transparencyPalette", "Opacity:", nullptr));
#if QT_CONFIG(tooltip)
        strokeOpacity->setToolTip(QCoreApplication::translate("transparencyPalette", "Set the transparency for the color selected", nullptr));
#endif // QT_CONFIG(tooltip)
        strokeOpacity->setSuffix(QCoreApplication::translate("transparencyPalette", " %", nullptr));
        label_3->setText(QCoreApplication::translate("transparencyPalette", "Blend Mode:", nullptr));
        blendModeStroke->setItemText(0, QCoreApplication::translate("transparencyPalette", "Normal", nullptr));
        blendModeStroke->setItemText(1, QCoreApplication::translate("transparencyPalette", "Darken", nullptr));
        blendModeStroke->setItemText(2, QCoreApplication::translate("transparencyPalette", "Lighten", nullptr));
        blendModeStroke->setItemText(3, QCoreApplication::translate("transparencyPalette", "Multiply", nullptr));
        blendModeStroke->setItemText(4, QCoreApplication::translate("transparencyPalette", "Screen", nullptr));
        blendModeStroke->setItemText(5, QCoreApplication::translate("transparencyPalette", "Overlay", nullptr));
        blendModeStroke->setItemText(6, QCoreApplication::translate("transparencyPalette", "Hard Light", nullptr));
        blendModeStroke->setItemText(7, QCoreApplication::translate("transparencyPalette", "Soft Light", nullptr));
        blendModeStroke->setItemText(8, QCoreApplication::translate("transparencyPalette", "Difference", nullptr));
        blendModeStroke->setItemText(9, QCoreApplication::translate("transparencyPalette", "Exclusion", nullptr));
        blendModeStroke->setItemText(10, QCoreApplication::translate("transparencyPalette", "Color Dodge", nullptr));
        blendModeStroke->setItemText(11, QCoreApplication::translate("transparencyPalette", "Color Burn", nullptr));
        blendModeStroke->setItemText(12, QCoreApplication::translate("transparencyPalette", "Hue", nullptr));
        blendModeStroke->setItemText(13, QCoreApplication::translate("transparencyPalette", "Saturation", nullptr));
        blendModeStroke->setItemText(14, QCoreApplication::translate("transparencyPalette", "Color", nullptr));
        blendModeStroke->setItemText(15, QCoreApplication::translate("transparencyPalette", "Luminosity", nullptr));

        label_4->setText(QCoreApplication::translate("transparencyPalette", "Opacity:", nullptr));
#if QT_CONFIG(tooltip)
        fillOpacity->setToolTip(QCoreApplication::translate("transparencyPalette", "Set the transparency for the color selected", nullptr));
#endif // QT_CONFIG(tooltip)
        fillOpacity->setSuffix(QCoreApplication::translate("transparencyPalette", " %", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabSolid), QCoreApplication::translate("transparencyPalette", "Solid", nullptr));
        gradientType->setItemText(0, QCoreApplication::translate("transparencyPalette", "Linear", nullptr));
        gradientType->setItemText(1, QCoreApplication::translate("transparencyPalette", "Radial", nullptr));

#if QT_CONFIG(tooltip)
        gradEditButton->setToolTip(QCoreApplication::translate("transparencyPalette", "Move the start of the gradient vector with the left mouse button pressed and move the end of the gradient vector with the right mouse button pressed", nullptr));
#endif // QT_CONFIG(tooltip)
        gradEditButton->setText(QCoreApplication::translate("transparencyPalette", "Move Vector", nullptr));
        transpCalcGradient->setText(QCoreApplication::translate("transparencyPalette", "Use Luminosity as Alpha", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabGradient), QCoreApplication::translate("transparencyPalette", "Gradient", nullptr));
        editPatternProps->setText(QCoreApplication::translate("transparencyPalette", "Properties...", nullptr));
        transpCalcPattern->setText(QCoreApplication::translate("transparencyPalette", "Use Luminosity as Alpha", nullptr));
        usePatternInverted->setText(QCoreApplication::translate("transparencyPalette", "Use inverted Pattern", nullptr));
        tabWidget->setTabText(tabWidget->indexOf(tabPattern), QCoreApplication::translate("transparencyPalette", "Pattern", nullptr));
        label_5->setText(QCoreApplication::translate("transparencyPalette", "Blend Mode:", nullptr));
        blendModeFill->setItemText(0, QCoreApplication::translate("transparencyPalette", "Normal", nullptr));
        blendModeFill->setItemText(1, QCoreApplication::translate("transparencyPalette", "Darken", nullptr));
        blendModeFill->setItemText(2, QCoreApplication::translate("transparencyPalette", "Lighten", nullptr));
        blendModeFill->setItemText(3, QCoreApplication::translate("transparencyPalette", "Multiply", nullptr));
        blendModeFill->setItemText(4, QCoreApplication::translate("transparencyPalette", "Screen", nullptr));
        blendModeFill->setItemText(5, QCoreApplication::translate("transparencyPalette", "Overlay", nullptr));
        blendModeFill->setItemText(6, QCoreApplication::translate("transparencyPalette", "Hard Light", nullptr));
        blendModeFill->setItemText(7, QCoreApplication::translate("transparencyPalette", "Soft Light", nullptr));
        blendModeFill->setItemText(8, QCoreApplication::translate("transparencyPalette", "Difference", nullptr));
        blendModeFill->setItemText(9, QCoreApplication::translate("transparencyPalette", "Exclusion", nullptr));
        blendModeFill->setItemText(10, QCoreApplication::translate("transparencyPalette", "Color Dodge", nullptr));
        blendModeFill->setItemText(11, QCoreApplication::translate("transparencyPalette", "Color Burn", nullptr));
        blendModeFill->setItemText(12, QCoreApplication::translate("transparencyPalette", "Hue", nullptr));
        blendModeFill->setItemText(13, QCoreApplication::translate("transparencyPalette", "Saturation", nullptr));
        blendModeFill->setItemText(14, QCoreApplication::translate("transparencyPalette", "Color", nullptr));
        blendModeFill->setItemText(15, QCoreApplication::translate("transparencyPalette", "Luminosity", nullptr));

        (void)transparencyPalette;
    } // retranslateUi

};

namespace Ui {
    class transparencyPalette: public Ui_transparencyPalette {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_TRANSPARENCYPALETTE_H
