/****************************************************************************
** Meta object code from reading C++ file 'scribusdoc.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/scribusdoc.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scribusdoc.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ScribusDoc_t {
    QByteArrayData data[205];
    char stringdata0[4281];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ScribusDoc_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ScribusDoc_t qt_meta_stringdata_ScribusDoc = {
    {
QT_MOC_LITERAL(0, 0, 10), // "ScribusDoc"
QT_MOC_LITERAL(1, 11, 10), // "docChanged"
QT_MOC_LITERAL(2, 22, 0), // ""
QT_MOC_LITERAL(3, 23, 14), // "updateContents"
QT_MOC_LITERAL(4, 38, 1), // "r"
QT_MOC_LITERAL(5, 40, 21), // "firstSelectedItemType"
QT_MOC_LITERAL(6, 62, 14), // "updateEditItem"
QT_MOC_LITERAL(7, 77, 19), // "updateAutoSaveClock"
QT_MOC_LITERAL(8, 97, 11), // "addBookmark"
QT_MOC_LITERAL(9, 109, 9), // "PageItem*"
QT_MOC_LITERAL(10, 119, 14), // "deleteBookmark"
QT_MOC_LITERAL(11, 134, 16), // "selectionChanged"
QT_MOC_LITERAL(12, 151, 24), // "itemSelection_ToggleLock"
QT_MOC_LITERAL(13, 176, 28), // "itemSelection_ToggleSizeLock"
QT_MOC_LITERAL(14, 205, 30), // "itemSelection_ToggleImageShown"
QT_MOC_LITERAL(15, 236, 32), // "itemSelection_TogglePrintEnabled"
QT_MOC_LITERAL(16, 269, 28), // "itemSelection_ToggleBookMark"
QT_MOC_LITERAL(17, 298, 10), // "Selection*"
QT_MOC_LITERAL(18, 309, 15), // "customSelection"
QT_MOC_LITERAL(19, 325, 30), // "itemSelection_ToggleAnnotation"
QT_MOC_LITERAL(20, 356, 23), // "itemSelection_Transform"
QT_MOC_LITERAL(21, 380, 10), // "nrOfCopies"
QT_MOC_LITERAL(22, 391, 6), // "matrix"
QT_MOC_LITERAL(23, 398, 9), // "basepoint"
QT_MOC_LITERAL(24, 408, 37), // "itemSelection_ChangePreviewRe..."
QT_MOC_LITERAL(25, 446, 2), // "id"
QT_MOC_LITERAL(26, 449, 32), // "allItems_ChangePreviewResolution"
QT_MOC_LITERAL(27, 482, 18), // "item_setFrameShape"
QT_MOC_LITERAL(28, 501, 4), // "item"
QT_MOC_LITERAL(29, 506, 9), // "frameType"
QT_MOC_LITERAL(30, 516, 5), // "count"
QT_MOC_LITERAL(31, 522, 13), // "const double*"
QT_MOC_LITERAL(32, 536, 6), // "points"
QT_MOC_LITERAL(33, 543, 23), // "itemSelection_ClearItem"
QT_MOC_LITERAL(34, 567, 10), // "useWarning"
QT_MOC_LITERAL(35, 578, 26), // "itemSelection_TruncateItem"
QT_MOC_LITERAL(36, 605, 24), // "itemSelection_DeleteItem"
QT_MOC_LITERAL(37, 630, 13), // "forceDeletion"
QT_MOC_LITERAL(38, 644, 37), // "itemSelection_SetItemFillTran..."
QT_MOC_LITERAL(39, 682, 1), // "t"
QT_MOC_LITERAL(40, 684, 30), // "itemSelection_SetHyphenWordMin"
QT_MOC_LITERAL(41, 715, 7), // "wordMin"
QT_MOC_LITERAL(42, 723, 39), // "itemSelection_SetHyphenConsec..."
QT_MOC_LITERAL(43, 763, 16), // "consecutiveLines"
QT_MOC_LITERAL(44, 780, 27), // "itemSelection_SetHyphenChar"
QT_MOC_LITERAL(45, 808, 10), // "hyphenChar"
QT_MOC_LITERAL(46, 819, 37), // "itemSelection_SetItemLineTran..."
QT_MOC_LITERAL(47, 857, 30), // "itemSelection_SetItemFillBlend"
QT_MOC_LITERAL(48, 888, 30), // "itemSelection_SetItemLineBlend"
QT_MOC_LITERAL(49, 919, 29), // "itemSelection_SetLineGradient"
QT_MOC_LITERAL(50, 949, 9), // "VGradient"
QT_MOC_LITERAL(51, 959, 11), // "newGradient"
QT_MOC_LITERAL(52, 971, 35), // "itemSelection_SetLineGradient..."
QT_MOC_LITERAL(53, 1007, 32), // "VGradient::VGradientRepeatMethod"
QT_MOC_LITERAL(54, 1040, 12), // "repeatMethod"
QT_MOC_LITERAL(55, 1053, 33), // "itemSelection_SetLineGradient..."
QT_MOC_LITERAL(56, 1087, 12), // "gradientName"
QT_MOC_LITERAL(57, 1100, 29), // "itemSelection_SetFillGradient"
QT_MOC_LITERAL(58, 1130, 35), // "itemSelection_SetFillGradient..."
QT_MOC_LITERAL(59, 1166, 33), // "itemSelection_SetFillGradient..."
QT_MOC_LITERAL(60, 1200, 25), // "itemSelection_SetFillRule"
QT_MOC_LITERAL(61, 1226, 7), // "evenOdd"
QT_MOC_LITERAL(62, 1234, 29), // "itemSelection_SetMaskGradient"
QT_MOC_LITERAL(63, 1264, 33), // "itemSelection_SetMaskGradient..."
QT_MOC_LITERAL(64, 1298, 26), // "itemSelection_SetOverprint"
QT_MOC_LITERAL(65, 1325, 9), // "overprint"
QT_MOC_LITERAL(66, 1335, 31), // "itemSelection_ApplyImageEffects"
QT_MOC_LITERAL(67, 1367, 17), // "ScImageEffectList"
QT_MOC_LITERAL(68, 1385, 13), // "newEffectList"
QT_MOC_LITERAL(69, 1399, 19), // "itemSelection_FlipH"
QT_MOC_LITERAL(70, 1419, 19), // "itemSelection_FlipV"
QT_MOC_LITERAL(71, 1439, 20), // "itemSelection_Rotate"
QT_MOC_LITERAL(72, 1460, 5), // "angle"
QT_MOC_LITERAL(73, 1466, 25), // "itemSelection_DoHyphenate"
QT_MOC_LITERAL(74, 1492, 27), // "itemSelection_DoDeHyphenate"
QT_MOC_LITERAL(75, 1520, 39), // "itemSelection_UnlinkTextFrame..."
QT_MOC_LITERAL(76, 1560, 25), // "itemSelection_SendToLayer"
QT_MOC_LITERAL(77, 1586, 7), // "layerID"
QT_MOC_LITERAL(78, 1594, 28), // "itemSelection_SetImageOffset"
QT_MOC_LITERAL(79, 1623, 1), // "x"
QT_MOC_LITERAL(80, 1625, 1), // "y"
QT_MOC_LITERAL(81, 1627, 27), // "itemSelection_SetImageScale"
QT_MOC_LITERAL(82, 1655, 36), // "itemSelection_SetImageScaleAn..."
QT_MOC_LITERAL(83, 1692, 2), // "ox"
QT_MOC_LITERAL(84, 1695, 2), // "oy"
QT_MOC_LITERAL(85, 1698, 2), // "sx"
QT_MOC_LITERAL(86, 1701, 2), // "sy"
QT_MOC_LITERAL(87, 1704, 30), // "itemSelection_SetImageRotation"
QT_MOC_LITERAL(88, 1735, 3), // "rot"
QT_MOC_LITERAL(89, 1739, 27), // "itemSelection_AlignItemLeft"
QT_MOC_LITERAL(90, 1767, 1), // "i"
QT_MOC_LITERAL(91, 1769, 4), // "newX"
QT_MOC_LITERAL(92, 1774, 23), // "ScribusDoc::AlignMethod"
QT_MOC_LITERAL(93, 1798, 3), // "how"
QT_MOC_LITERAL(94, 1802, 28), // "itemSelection_AlignItemRight"
QT_MOC_LITERAL(95, 1831, 26), // "itemSelection_AlignItemTop"
QT_MOC_LITERAL(96, 1858, 4), // "newY"
QT_MOC_LITERAL(97, 1863, 29), // "itemSelection_AlignItemBottom"
QT_MOC_LITERAL(98, 1893, 26), // "itemSelection_AlignLeftOut"
QT_MOC_LITERAL(99, 1920, 19), // "ScribusDoc::AlignTo"
QT_MOC_LITERAL(100, 1940, 11), // "currAlignTo"
QT_MOC_LITERAL(101, 1952, 15), // "currAlignMethod"
QT_MOC_LITERAL(102, 1968, 13), // "guidePosition"
QT_MOC_LITERAL(103, 1982, 27), // "itemSelection_AlignRightOut"
QT_MOC_LITERAL(104, 2010, 27), // "itemSelection_AlignBottomIn"
QT_MOC_LITERAL(105, 2038, 26), // "itemSelection_AlignRightIn"
QT_MOC_LITERAL(106, 2065, 28), // "itemSelection_AlignBottomOut"
QT_MOC_LITERAL(107, 2094, 28), // "itemSelection_AlignCenterHor"
QT_MOC_LITERAL(108, 2123, 25), // "itemSelection_AlignLeftIn"
QT_MOC_LITERAL(109, 2149, 28), // "itemSelection_AlignCenterVer"
QT_MOC_LITERAL(110, 2178, 25), // "itemSelection_AlignTopOut"
QT_MOC_LITERAL(111, 2204, 24), // "itemSelection_AlignTopIn"
QT_MOC_LITERAL(112, 2229, 23), // "sendItemSelectionToBack"
QT_MOC_LITERAL(113, 2253, 25), // "bringItemSelectionToFront"
QT_MOC_LITERAL(114, 2279, 23), // "itemSelection_LowerItem"
QT_MOC_LITERAL(115, 2303, 23), // "itemSelection_RaiseItem"
QT_MOC_LITERAL(116, 2327, 29), // "itemSelection_DistributeDistH"
QT_MOC_LITERAL(117, 2357, 13), // "usingDistance"
QT_MOC_LITERAL(118, 2371, 8), // "distance"
QT_MOC_LITERAL(119, 2380, 17), // "reverseDistribute"
QT_MOC_LITERAL(120, 2398, 34), // "itemSelection_DistributeAcros..."
QT_MOC_LITERAL(121, 2433, 10), // "useMargins"
QT_MOC_LITERAL(122, 2444, 29), // "itemSelection_DistributeRight"
QT_MOC_LITERAL(123, 2474, 30), // "itemSelection_DistributeBottom"
QT_MOC_LITERAL(124, 2505, 31), // "itemSelection_DistributeCenterH"
QT_MOC_LITERAL(125, 2537, 29), // "itemSelection_DistributeDistV"
QT_MOC_LITERAL(126, 2567, 32), // "itemSelection_DistributeDownPage"
QT_MOC_LITERAL(127, 2600, 28), // "itemSelection_DistributeLeft"
QT_MOC_LITERAL(128, 2629, 31), // "itemSelection_DistributeCenterV"
QT_MOC_LITERAL(129, 2661, 27), // "itemSelection_DistributeTop"
QT_MOC_LITERAL(130, 2689, 22), // "itemSelection_SwapLeft"
QT_MOC_LITERAL(131, 2712, 23), // "itemSelection_SwapRight"
QT_MOC_LITERAL(132, 2736, 23), // "itemSelection_Duplicate"
QT_MOC_LITERAL(133, 2760, 6), // "shiftX"
QT_MOC_LITERAL(134, 2767, 6), // "shiftY"
QT_MOC_LITERAL(135, 2774, 31), // "itemSelection_MultipleDuplicate"
QT_MOC_LITERAL(136, 2806, 25), // "ItemMultipleDuplicateData"
QT_MOC_LITERAL(137, 2832, 24), // "itemSelection_UniteItems"
QT_MOC_LITERAL(138, 2857, 24), // "itemSelection_SplitItems"
QT_MOC_LITERAL(139, 2882, 36), // "itemSelection_AdjustFrametoIm..."
QT_MOC_LITERAL(140, 2919, 36), // "itemSelection_AdjustImagetoFr..."
QT_MOC_LITERAL(141, 2956, 37), // "itemSelection_AdjustFrameHeig..."
QT_MOC_LITERAL(142, 2994, 28), // "itemSelection_ApplyArrowHead"
QT_MOC_LITERAL(143, 3023, 12), // "startArrowID"
QT_MOC_LITERAL(144, 3036, 10), // "endArrowID"
QT_MOC_LITERAL(145, 3047, 29), // "itemSelection_ApplyArrowScale"
QT_MOC_LITERAL(146, 3077, 12), // "startArrowSc"
QT_MOC_LITERAL(147, 3090, 10), // "endArrowSc"
QT_MOC_LITERAL(148, 3101, 24), // "itemSelection_SetItemPen"
QT_MOC_LITERAL(149, 3126, 5), // "color"
QT_MOC_LITERAL(150, 3132, 29), // "itemSelection_SetItemPenShade"
QT_MOC_LITERAL(151, 3162, 3), // "sha"
QT_MOC_LITERAL(152, 3166, 31), // "itemSelection_SetItemGradStroke"
QT_MOC_LITERAL(153, 3198, 3), // "typ"
QT_MOC_LITERAL(154, 3202, 26), // "itemSelection_SetItemBrush"
QT_MOC_LITERAL(155, 3229, 9), // "colorName"
QT_MOC_LITERAL(156, 3239, 31), // "itemSelection_SetItemBrushShade"
QT_MOC_LITERAL(157, 3271, 29), // "itemSelection_SetItemGradMask"
QT_MOC_LITERAL(158, 3301, 29), // "itemSelection_SetItemGradFill"
QT_MOC_LITERAL(159, 3331, 4), // "typp"
QT_MOC_LITERAL(160, 3336, 32), // "itemSelection_SetItemPatternFill"
QT_MOC_LITERAL(161, 3369, 7), // "pattern"
QT_MOC_LITERAL(162, 3377, 33), // "itemSelection_SetItemPatternP..."
QT_MOC_LITERAL(163, 3411, 6), // "scaleX"
QT_MOC_LITERAL(164, 3418, 6), // "scaleY"
QT_MOC_LITERAL(165, 3425, 7), // "offsetX"
QT_MOC_LITERAL(166, 3433, 7), // "offsetY"
QT_MOC_LITERAL(167, 3441, 8), // "rotation"
QT_MOC_LITERAL(168, 3450, 5), // "skewX"
QT_MOC_LITERAL(169, 3456, 5), // "skewY"
QT_MOC_LITERAL(170, 3462, 7), // "mirrorX"
QT_MOC_LITERAL(171, 3470, 7), // "mirrorY"
QT_MOC_LITERAL(172, 3478, 34), // "itemSelection_SetItemStrokePa..."
QT_MOC_LITERAL(173, 3513, 39), // "itemSelection_SetItemStrokePa..."
QT_MOC_LITERAL(174, 3553, 5), // "space"
QT_MOC_LITERAL(175, 3559, 38), // "itemSelection_SetItemStrokePa..."
QT_MOC_LITERAL(176, 3598, 4), // "type"
QT_MOC_LITERAL(177, 3603, 32), // "itemSelection_SetItemPatternMask"
QT_MOC_LITERAL(178, 3636, 37), // "itemSelection_SetItemPatternM..."
QT_MOC_LITERAL(179, 3674, 29), // "itemSelection_InsertTableRows"
QT_MOC_LITERAL(180, 3704, 32), // "itemSelection_InsertTableColumns"
QT_MOC_LITERAL(181, 3737, 29), // "itemSelection_DeleteTableRows"
QT_MOC_LITERAL(182, 3767, 32), // "itemSelection_DeleteTableColumns"
QT_MOC_LITERAL(183, 3800, 29), // "itemSelection_MergeTableCells"
QT_MOC_LITERAL(184, 3830, 29), // "itemSelection_SplitTableCells"
QT_MOC_LITERAL(185, 3860, 32), // "itemSelection_SetTableRowHeights"
QT_MOC_LITERAL(186, 3893, 34), // "itemSelection_SetTableColumnW..."
QT_MOC_LITERAL(187, 3928, 39), // "itemSelection_DistributeTable..."
QT_MOC_LITERAL(188, 3968, 42), // "itemSelection_DistributeTable..."
QT_MOC_LITERAL(189, 4011, 32), // "itemSelection_AdjustFrameToTable"
QT_MOC_LITERAL(190, 4044, 32), // "itemSelection_AdjustTableToFrame"
QT_MOC_LITERAL(191, 4077, 13), // "undoRedoBegin"
QT_MOC_LITERAL(192, 4091, 12), // "undoRedoDone"
QT_MOC_LITERAL(193, 4104, 9), // "updatePic"
QT_MOC_LITERAL(194, 4114, 10), // "updatePict"
QT_MOC_LITERAL(195, 4125, 4), // "name"
QT_MOC_LITERAL(196, 4130, 13), // "updatePictDir"
QT_MOC_LITERAL(197, 4144, 10), // "removePict"
QT_MOC_LITERAL(198, 4155, 11), // "updateMarks"
QT_MOC_LITERAL(199, 4167, 16), // "updateNotesMarks"
QT_MOC_LITERAL(200, 4184, 20), // "itemSelection_UnWeld"
QT_MOC_LITERAL(201, 4205, 18), // "itemSelection_Weld"
QT_MOC_LITERAL(202, 4224, 22), // "itemSelection_EditWeld"
QT_MOC_LITERAL(203, 4247, 20), // "restartAutoSaveTimer"
QT_MOC_LITERAL(204, 4268, 12) // "slotAutoSave"

    },
    "ScribusDoc\0docChanged\0\0updateContents\0"
    "r\0firstSelectedItemType\0updateEditItem\0"
    "updateAutoSaveClock\0addBookmark\0"
    "PageItem*\0deleteBookmark\0selectionChanged\0"
    "itemSelection_ToggleLock\0"
    "itemSelection_ToggleSizeLock\0"
    "itemSelection_ToggleImageShown\0"
    "itemSelection_TogglePrintEnabled\0"
    "itemSelection_ToggleBookMark\0Selection*\0"
    "customSelection\0itemSelection_ToggleAnnotation\0"
    "itemSelection_Transform\0nrOfCopies\0"
    "matrix\0basepoint\0itemSelection_ChangePreviewResolution\0"
    "id\0allItems_ChangePreviewResolution\0"
    "item_setFrameShape\0item\0frameType\0"
    "count\0const double*\0points\0"
    "itemSelection_ClearItem\0useWarning\0"
    "itemSelection_TruncateItem\0"
    "itemSelection_DeleteItem\0forceDeletion\0"
    "itemSelection_SetItemFillTransparency\0"
    "t\0itemSelection_SetHyphenWordMin\0"
    "wordMin\0itemSelection_SetHyphenConsecutiveLines\0"
    "consecutiveLines\0itemSelection_SetHyphenChar\0"
    "hyphenChar\0itemSelection_SetItemLineTransparency\0"
    "itemSelection_SetItemFillBlend\0"
    "itemSelection_SetItemLineBlend\0"
    "itemSelection_SetLineGradient\0VGradient\0"
    "newGradient\0itemSelection_SetLineGradientExtend\0"
    "VGradient::VGradientRepeatMethod\0"
    "repeatMethod\0itemSelection_SetLineGradientName\0"
    "gradientName\0itemSelection_SetFillGradient\0"
    "itemSelection_SetFillGradientExtend\0"
    "itemSelection_SetFillGradientName\0"
    "itemSelection_SetFillRule\0evenOdd\0"
    "itemSelection_SetMaskGradient\0"
    "itemSelection_SetMaskGradientName\0"
    "itemSelection_SetOverprint\0overprint\0"
    "itemSelection_ApplyImageEffects\0"
    "ScImageEffectList\0newEffectList\0"
    "itemSelection_FlipH\0itemSelection_FlipV\0"
    "itemSelection_Rotate\0angle\0"
    "itemSelection_DoHyphenate\0"
    "itemSelection_DoDeHyphenate\0"
    "itemSelection_UnlinkTextFrameAndCutText\0"
    "itemSelection_SendToLayer\0layerID\0"
    "itemSelection_SetImageOffset\0x\0y\0"
    "itemSelection_SetImageScale\0"
    "itemSelection_SetImageScaleAndOffset\0"
    "ox\0oy\0sx\0sy\0itemSelection_SetImageRotation\0"
    "rot\0itemSelection_AlignItemLeft\0i\0"
    "newX\0ScribusDoc::AlignMethod\0how\0"
    "itemSelection_AlignItemRight\0"
    "itemSelection_AlignItemTop\0newY\0"
    "itemSelection_AlignItemBottom\0"
    "itemSelection_AlignLeftOut\0"
    "ScribusDoc::AlignTo\0currAlignTo\0"
    "currAlignMethod\0guidePosition\0"
    "itemSelection_AlignRightOut\0"
    "itemSelection_AlignBottomIn\0"
    "itemSelection_AlignRightIn\0"
    "itemSelection_AlignBottomOut\0"
    "itemSelection_AlignCenterHor\0"
    "itemSelection_AlignLeftIn\0"
    "itemSelection_AlignCenterVer\0"
    "itemSelection_AlignTopOut\0"
    "itemSelection_AlignTopIn\0"
    "sendItemSelectionToBack\0"
    "bringItemSelectionToFront\0"
    "itemSelection_LowerItem\0itemSelection_RaiseItem\0"
    "itemSelection_DistributeDistH\0"
    "usingDistance\0distance\0reverseDistribute\0"
    "itemSelection_DistributeAcrossPage\0"
    "useMargins\0itemSelection_DistributeRight\0"
    "itemSelection_DistributeBottom\0"
    "itemSelection_DistributeCenterH\0"
    "itemSelection_DistributeDistV\0"
    "itemSelection_DistributeDownPage\0"
    "itemSelection_DistributeLeft\0"
    "itemSelection_DistributeCenterV\0"
    "itemSelection_DistributeTop\0"
    "itemSelection_SwapLeft\0itemSelection_SwapRight\0"
    "itemSelection_Duplicate\0shiftX\0shiftY\0"
    "itemSelection_MultipleDuplicate\0"
    "ItemMultipleDuplicateData\0"
    "itemSelection_UniteItems\0"
    "itemSelection_SplitItems\0"
    "itemSelection_AdjustFrametoImageSize\0"
    "itemSelection_AdjustImagetoFrameSize\0"
    "itemSelection_AdjustFrameHeightToText\0"
    "itemSelection_ApplyArrowHead\0startArrowID\0"
    "endArrowID\0itemSelection_ApplyArrowScale\0"
    "startArrowSc\0endArrowSc\0"
    "itemSelection_SetItemPen\0color\0"
    "itemSelection_SetItemPenShade\0sha\0"
    "itemSelection_SetItemGradStroke\0typ\0"
    "itemSelection_SetItemBrush\0colorName\0"
    "itemSelection_SetItemBrushShade\0"
    "itemSelection_SetItemGradMask\0"
    "itemSelection_SetItemGradFill\0typp\0"
    "itemSelection_SetItemPatternFill\0"
    "pattern\0itemSelection_SetItemPatternProps\0"
    "scaleX\0scaleY\0offsetX\0offsetY\0rotation\0"
    "skewX\0skewY\0mirrorX\0mirrorY\0"
    "itemSelection_SetItemStrokePattern\0"
    "itemSelection_SetItemStrokePatternProps\0"
    "space\0itemSelection_SetItemStrokePatternType\0"
    "type\0itemSelection_SetItemPatternMask\0"
    "itemSelection_SetItemPatternMaskProps\0"
    "itemSelection_InsertTableRows\0"
    "itemSelection_InsertTableColumns\0"
    "itemSelection_DeleteTableRows\0"
    "itemSelection_DeleteTableColumns\0"
    "itemSelection_MergeTableCells\0"
    "itemSelection_SplitTableCells\0"
    "itemSelection_SetTableRowHeights\0"
    "itemSelection_SetTableColumnWidths\0"
    "itemSelection_DistributeTableRowsEvenly\0"
    "itemSelection_DistributeTableColumnsEvenly\0"
    "itemSelection_AdjustFrameToTable\0"
    "itemSelection_AdjustTableToFrame\0"
    "undoRedoBegin\0undoRedoDone\0updatePic\0"
    "updatePict\0name\0updatePictDir\0removePict\0"
    "updateMarks\0updateNotesMarks\0"
    "itemSelection_UnWeld\0itemSelection_Weld\0"
    "itemSelection_EditWeld\0restartAutoSaveTimer\0"
    "slotAutoSave"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ScribusDoc[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
     191,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       8,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  969,    2, 0x06 /* Public */,
       3,    0,  970,    2, 0x06 /* Public */,
       3,    1,  971,    2, 0x06 /* Public */,
       5,    1,  974,    2, 0x06 /* Public */,
       6,    0,  977,    2, 0x06 /* Public */,
       7,    0,  978,    2, 0x06 /* Public */,
       8,    1,  979,    2, 0x06 /* Public */,
      10,    1,  982,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      11,    0,  985,    2, 0x0a /* Public */,
      12,    0,  986,    2, 0x0a /* Public */,
      13,    0,  987,    2, 0x0a /* Public */,
      14,    0,  988,    2, 0x0a /* Public */,
      15,    0,  989,    2, 0x0a /* Public */,
      16,    1,  990,    2, 0x0a /* Public */,
      16,    0,  993,    2, 0x2a /* Public | MethodCloned */,
      19,    1,  994,    2, 0x0a /* Public */,
      19,    0,  997,    2, 0x2a /* Public | MethodCloned */,
      20,    3,  998,    2, 0x0a /* Public */,
      24,    1, 1005,    2, 0x0a /* Public */,
      26,    1, 1008,    2, 0x0a /* Public */,
      27,    4, 1011,    2, 0x0a /* Public */,
      33,    2, 1020,    2, 0x0a /* Public */,
      33,    1, 1025,    2, 0x2a /* Public | MethodCloned */,
      33,    0, 1028,    2, 0x2a /* Public | MethodCloned */,
      35,    1, 1029,    2, 0x0a /* Public */,
      35,    0, 1032,    2, 0x2a /* Public | MethodCloned */,
      36,    2, 1033,    2, 0x0a /* Public */,
      36,    1, 1038,    2, 0x2a /* Public | MethodCloned */,
      36,    0, 1041,    2, 0x2a /* Public | MethodCloned */,
      38,    2, 1042,    2, 0x0a /* Public */,
      38,    1, 1047,    2, 0x2a /* Public | MethodCloned */,
      40,    2, 1050,    2, 0x0a /* Public */,
      40,    1, 1055,    2, 0x2a /* Public | MethodCloned */,
      42,    2, 1058,    2, 0x0a /* Public */,
      42,    1, 1063,    2, 0x2a /* Public | MethodCloned */,
      44,    2, 1066,    2, 0x0a /* Public */,
      44,    1, 1071,    2, 0x2a /* Public | MethodCloned */,
      46,    2, 1074,    2, 0x0a /* Public */,
      46,    1, 1079,    2, 0x2a /* Public | MethodCloned */,
      47,    1, 1082,    2, 0x0a /* Public */,
      48,    1, 1085,    2, 0x0a /* Public */,
      49,    2, 1088,    2, 0x0a /* Public */,
      49,    1, 1093,    2, 0x2a /* Public | MethodCloned */,
      52,    2, 1096,    2, 0x0a /* Public */,
      52,    1, 1101,    2, 0x2a /* Public | MethodCloned */,
      55,    2, 1104,    2, 0x0a /* Public */,
      55,    1, 1109,    2, 0x2a /* Public | MethodCloned */,
      57,    2, 1112,    2, 0x0a /* Public */,
      57,    1, 1117,    2, 0x2a /* Public | MethodCloned */,
      58,    2, 1120,    2, 0x0a /* Public */,
      58,    1, 1125,    2, 0x2a /* Public | MethodCloned */,
      59,    2, 1128,    2, 0x0a /* Public */,
      59,    1, 1133,    2, 0x2a /* Public | MethodCloned */,
      60,    2, 1136,    2, 0x0a /* Public */,
      60,    1, 1141,    2, 0x2a /* Public | MethodCloned */,
      62,    2, 1144,    2, 0x0a /* Public */,
      62,    1, 1149,    2, 0x2a /* Public | MethodCloned */,
      63,    2, 1152,    2, 0x0a /* Public */,
      63,    1, 1157,    2, 0x2a /* Public | MethodCloned */,
      64,    2, 1160,    2, 0x0a /* Public */,
      64,    1, 1165,    2, 0x2a /* Public | MethodCloned */,
      66,    2, 1168,    2, 0x0a /* Public */,
      66,    1, 1173,    2, 0x2a /* Public | MethodCloned */,
      69,    1, 1176,    2, 0x0a /* Public */,
      69,    0, 1179,    2, 0x2a /* Public | MethodCloned */,
      70,    1, 1180,    2, 0x0a /* Public */,
      70,    0, 1183,    2, 0x2a /* Public | MethodCloned */,
      71,    2, 1184,    2, 0x0a /* Public */,
      71,    1, 1189,    2, 0x2a /* Public | MethodCloned */,
      73,    0, 1192,    2, 0x0a /* Public */,
      74,    0, 1193,    2, 0x0a /* Public */,
      75,    1, 1194,    2, 0x0a /* Public */,
      75,    0, 1197,    2, 0x2a /* Public | MethodCloned */,
      76,    1, 1198,    2, 0x0a /* Public */,
      78,    3, 1201,    2, 0x0a /* Public */,
      78,    2, 1208,    2, 0x2a /* Public | MethodCloned */,
      81,    3, 1213,    2, 0x0a /* Public */,
      81,    2, 1220,    2, 0x2a /* Public | MethodCloned */,
      82,    5, 1225,    2, 0x0a /* Public */,
      82,    4, 1236,    2, 0x2a /* Public | MethodCloned */,
      87,    2, 1245,    2, 0x0a /* Public */,
      87,    1, 1250,    2, 0x2a /* Public | MethodCloned */,
      89,    3, 1253,    2, 0x0a /* Public */,
      94,    3, 1260,    2, 0x0a /* Public */,
      95,    3, 1267,    2, 0x0a /* Public */,
      97,    3, 1274,    2, 0x0a /* Public */,
      98,    3, 1281,    2, 0x0a /* Public */,
     103,    3, 1288,    2, 0x0a /* Public */,
     104,    3, 1295,    2, 0x0a /* Public */,
     105,    3, 1302,    2, 0x0a /* Public */,
     106,    3, 1309,    2, 0x0a /* Public */,
     107,    3, 1316,    2, 0x0a /* Public */,
     108,    3, 1323,    2, 0x0a /* Public */,
     109,    3, 1330,    2, 0x0a /* Public */,
     110,    3, 1337,    2, 0x0a /* Public */,
     111,    3, 1344,    2, 0x0a /* Public */,
     112,    0, 1351,    2, 0x0a /* Public */,
     113,    0, 1352,    2, 0x0a /* Public */,
     114,    0, 1353,    2, 0x0a /* Public */,
     115,    0, 1354,    2, 0x0a /* Public */,
     116,    3, 1355,    2, 0x0a /* Public */,
     116,    2, 1362,    2, 0x2a /* Public | MethodCloned */,
     116,    1, 1367,    2, 0x2a /* Public | MethodCloned */,
     116,    0, 1370,    2, 0x2a /* Public | MethodCloned */,
     120,    1, 1371,    2, 0x0a /* Public */,
     120,    0, 1374,    2, 0x2a /* Public | MethodCloned */,
     122,    0, 1375,    2, 0x0a /* Public */,
     123,    0, 1376,    2, 0x0a /* Public */,
     124,    0, 1377,    2, 0x0a /* Public */,
     125,    3, 1378,    2, 0x0a /* Public */,
     125,    2, 1385,    2, 0x2a /* Public | MethodCloned */,
     125,    1, 1390,    2, 0x2a /* Public | MethodCloned */,
     125,    0, 1393,    2, 0x2a /* Public | MethodCloned */,
     126,    1, 1394,    2, 0x0a /* Public */,
     126,    0, 1397,    2, 0x2a /* Public | MethodCloned */,
     127,    0, 1398,    2, 0x0a /* Public */,
     128,    0, 1399,    2, 0x0a /* Public */,
     129,    0, 1400,    2, 0x0a /* Public */,
     130,    0, 1401,    2, 0x0a /* Public */,
     131,    0, 1402,    2, 0x0a /* Public */,
     132,    3, 1403,    2, 0x0a /* Public */,
     132,    2, 1410,    2, 0x2a /* Public | MethodCloned */,
     135,    2, 1415,    2, 0x0a /* Public */,
     135,    1, 1420,    2, 0x2a /* Public | MethodCloned */,
     137,    1, 1423,    2, 0x0a /* Public */,
     137,    0, 1426,    2, 0x2a /* Public | MethodCloned */,
     138,    1, 1427,    2, 0x0a /* Public */,
     138,    0, 1430,    2, 0x2a /* Public | MethodCloned */,
     139,    1, 1431,    2, 0x0a /* Public */,
     139,    0, 1434,    2, 0x2a /* Public | MethodCloned */,
     140,    1, 1435,    2, 0x0a /* Public */,
     140,    0, 1438,    2, 0x2a /* Public | MethodCloned */,
     141,    1, 1439,    2, 0x0a /* Public */,
     141,    0, 1442,    2, 0x2a /* Public | MethodCloned */,
     142,    3, 1443,    2, 0x0a /* Public */,
     142,    2, 1450,    2, 0x2a /* Public | MethodCloned */,
     142,    1, 1455,    2, 0x2a /* Public | MethodCloned */,
     142,    0, 1458,    2, 0x2a /* Public | MethodCloned */,
     145,    3, 1459,    2, 0x0a /* Public */,
     148,    2, 1466,    2, 0x0a /* Public */,
     148,    1, 1471,    2, 0x2a /* Public | MethodCloned */,
     150,    2, 1474,    2, 0x0a /* Public */,
     150,    1, 1479,    2, 0x2a /* Public | MethodCloned */,
     152,    2, 1482,    2, 0x0a /* Public */,
     152,    1, 1487,    2, 0x2a /* Public | MethodCloned */,
     154,    2, 1490,    2, 0x0a /* Public */,
     154,    1, 1495,    2, 0x2a /* Public | MethodCloned */,
     156,    2, 1498,    2, 0x0a /* Public */,
     156,    1, 1503,    2, 0x2a /* Public | MethodCloned */,
     157,    2, 1506,    2, 0x0a /* Public */,
     157,    1, 1511,    2, 0x2a /* Public | MethodCloned */,
     158,    2, 1514,    2, 0x0a /* Public */,
     158,    1, 1519,    2, 0x2a /* Public | MethodCloned */,
     160,    2, 1522,    2, 0x0a /* Public */,
     160,    1, 1527,    2, 0x2a /* Public | MethodCloned */,
     162,    9, 1530,    2, 0x0a /* Public */,
     172,    2, 1549,    2, 0x0a /* Public */,
     172,    1, 1554,    2, 0x2a /* Public | MethodCloned */,
     173,   11, 1557,    2, 0x0a /* Public */,
     173,   10, 1580,    2, 0x2a /* Public | MethodCloned */,
     175,    2, 1601,    2, 0x0a /* Public */,
     175,    1, 1606,    2, 0x2a /* Public | MethodCloned */,
     177,    2, 1609,    2, 0x0a /* Public */,
     177,    1, 1614,    2, 0x2a /* Public | MethodCloned */,
     178,   10, 1617,    2, 0x0a /* Public */,
     178,    9, 1638,    2, 0x2a /* Public | MethodCloned */,
     179,    0, 1657,    2, 0x0a /* Public */,
     180,    0, 1658,    2, 0x0a /* Public */,
     181,    0, 1659,    2, 0x0a /* Public */,
     182,    0, 1660,    2, 0x0a /* Public */,
     183,    0, 1661,    2, 0x0a /* Public */,
     184,    0, 1662,    2, 0x0a /* Public */,
     185,    0, 1663,    2, 0x0a /* Public */,
     186,    0, 1664,    2, 0x0a /* Public */,
     187,    0, 1665,    2, 0x0a /* Public */,
     188,    0, 1666,    2, 0x0a /* Public */,
     189,    0, 1667,    2, 0x0a /* Public */,
     190,    0, 1668,    2, 0x0a /* Public */,
     191,    0, 1669,    2, 0x0a /* Public */,
     192,    0, 1670,    2, 0x0a /* Public */,
     193,    0, 1671,    2, 0x0a /* Public */,
     194,    1, 1672,    2, 0x0a /* Public */,
     196,    1, 1675,    2, 0x0a /* Public */,
     197,    1, 1678,    2, 0x0a /* Public */,
     198,    1, 1681,    2, 0x0a /* Public */,
     198,    0, 1684,    2, 0x2a /* Public | MethodCloned */,
     200,    0, 1685,    2, 0x0a /* Public */,
     201,    0, 1686,    2, 0x0a /* Public */,
     202,    0, 1687,    2, 0x0a /* Public */,
     203,    0, 1688,    2, 0x0a /* Public */,
     204,    0, 1689,    2, 0x09 /* Protected */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QRect,    4,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 9,    2,
    QMetaType::Void, 0x80000000 | 9,    2,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::QTransform, QMetaType::Int,   21,   22,   23,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void, 0x80000000 | 9, QMetaType::Int, QMetaType::Int, 0x80000000 | 31,   28,   29,   30,   32,
    QMetaType::Void, 0x80000000 | 17, QMetaType::Bool,   18,   34,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17, QMetaType::Bool,   18,   37,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double, 0x80000000 | 17,   39,   18,
    QMetaType::Void, QMetaType::Double,   39,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 17,   41,   18,
    QMetaType::Void, QMetaType::Int,   41,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 17,   43,   18,
    QMetaType::Void, QMetaType::Int,   43,
    QMetaType::Void, QMetaType::UInt, 0x80000000 | 17,   45,   18,
    QMetaType::Void, QMetaType::UInt,   45,
    QMetaType::Void, QMetaType::Double, 0x80000000 | 17,   39,   18,
    QMetaType::Void, QMetaType::Double,   39,
    QMetaType::Void, QMetaType::Int,   39,
    QMetaType::Void, QMetaType::Int,   39,
    QMetaType::Void, 0x80000000 | 50, 0x80000000 | 17,   51,   18,
    QMetaType::Void, 0x80000000 | 50,   51,
    QMetaType::Void, 0x80000000 | 53, 0x80000000 | 17,   54,   18,
    QMetaType::Void, 0x80000000 | 53,   54,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 17,   56,   18,
    QMetaType::Void, QMetaType::QString,   56,
    QMetaType::Void, 0x80000000 | 50, 0x80000000 | 17,   51,   18,
    QMetaType::Void, 0x80000000 | 50,   51,
    QMetaType::Void, 0x80000000 | 53, 0x80000000 | 17,   54,   18,
    QMetaType::Void, 0x80000000 | 53,   54,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 17,   56,   18,
    QMetaType::Void, QMetaType::QString,   56,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 17,   61,   18,
    QMetaType::Void, QMetaType::Bool,   61,
    QMetaType::Void, 0x80000000 | 50, 0x80000000 | 17,   51,   18,
    QMetaType::Void, 0x80000000 | 50,   51,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 17,   56,   18,
    QMetaType::Void, QMetaType::QString,   56,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 17,   65,   18,
    QMetaType::Void, QMetaType::Bool,   65,
    QMetaType::Void, 0x80000000 | 67, 0x80000000 | 17,   68,   18,
    QMetaType::Void, 0x80000000 | 67,   68,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double, 0x80000000 | 17,   72,   18,
    QMetaType::Void, QMetaType::Double,   72,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   77,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, 0x80000000 | 17,   79,   80,   18,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,   79,   80,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, 0x80000000 | 17,   79,   80,   18,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,   79,   80,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, 0x80000000 | 17,   83,   84,   85,   86,   18,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double,   83,   84,   85,   86,
    QMetaType::Void, QMetaType::Double, 0x80000000 | 17,   88,   18,
    QMetaType::Void, QMetaType::Double,   88,
    QMetaType::Void, QMetaType::Int, QMetaType::Double, 0x80000000 | 92,   90,   91,   93,
    QMetaType::Void, QMetaType::Int, QMetaType::Double, 0x80000000 | 92,   90,   91,   93,
    QMetaType::Void, QMetaType::Int, QMetaType::Double, 0x80000000 | 92,   90,   96,   93,
    QMetaType::Void, QMetaType::Int, QMetaType::Double, 0x80000000 | 92,   90,   96,   93,
    QMetaType::Void, 0x80000000 | 99, 0x80000000 | 92, QMetaType::Double,  100,  101,  102,
    QMetaType::Void, 0x80000000 | 99, 0x80000000 | 92, QMetaType::Double,  100,  101,  102,
    QMetaType::Void, 0x80000000 | 99, 0x80000000 | 92, QMetaType::Double,  100,  101,  102,
    QMetaType::Void, 0x80000000 | 99, 0x80000000 | 92, QMetaType::Double,  100,  101,  102,
    QMetaType::Void, 0x80000000 | 99, 0x80000000 | 92, QMetaType::Double,  100,  101,  102,
    QMetaType::Void, 0x80000000 | 99, 0x80000000 | 92, QMetaType::Double,  100,  101,  102,
    QMetaType::Void, 0x80000000 | 99, 0x80000000 | 92, QMetaType::Double,  100,  101,  102,
    QMetaType::Void, 0x80000000 | 99, 0x80000000 | 92, QMetaType::Double,  100,  101,  102,
    QMetaType::Void, 0x80000000 | 99, 0x80000000 | 92, QMetaType::Double,  100,  101,  102,
    QMetaType::Void, 0x80000000 | 99, 0x80000000 | 92, QMetaType::Double,  100,  101,  102,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool, QMetaType::Double, QMetaType::Bool,  117,  118,  119,
    QMetaType::Void, QMetaType::Bool, QMetaType::Double,  117,  118,
    QMetaType::Void, QMetaType::Bool,  117,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  121,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool, QMetaType::Double, QMetaType::Bool,  117,  118,  119,
    QMetaType::Void, QMetaType::Bool, QMetaType::Double,  117,  118,
    QMetaType::Void, QMetaType::Bool,  117,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,  121,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, 0x80000000 | 17,  133,  134,   18,
    QMetaType::Void, QMetaType::Double, QMetaType::Double,  133,  134,
    QMetaType::Void, 0x80000000 | 136, 0x80000000 | 17,    2,   18,
    QMetaType::Void, 0x80000000 | 136,    2,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 17,   18,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, 0x80000000 | 17,  143,  144,   18,
    QMetaType::Void, QMetaType::Int, QMetaType::Int,  143,  144,
    QMetaType::Void, QMetaType::Int,  143,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, 0x80000000 | 17,  146,  147,   18,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 17,  149,   18,
    QMetaType::Void, QMetaType::QString,  149,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 17,  151,   18,
    QMetaType::Void, QMetaType::Int,  151,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 17,  153,   18,
    QMetaType::Void, QMetaType::Int,  153,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 17,  155,   18,
    QMetaType::Void, QMetaType::QString,  155,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 17,  151,   18,
    QMetaType::Void, QMetaType::Int,  151,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 17,  153,   18,
    QMetaType::Void, QMetaType::Int,  153,
    QMetaType::Void, QMetaType::Int, 0x80000000 | 17,  159,   18,
    QMetaType::Void, QMetaType::Int,  159,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 17,  161,   18,
    QMetaType::Void, QMetaType::QString,  161,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Bool, QMetaType::Bool,  163,  164,  165,  166,  167,  168,  169,  170,  171,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 17,  161,   18,
    QMetaType::Void, QMetaType::QString,  161,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Bool, QMetaType::Bool, 0x80000000 | 17,  163,  164,  165,  166,  167,  168,  169,  174,  170,  171,   18,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Bool, QMetaType::Bool,  163,  164,  165,  166,  167,  168,  169,  174,  170,  171,
    QMetaType::Void, QMetaType::Bool, 0x80000000 | 17,  176,   18,
    QMetaType::Void, QMetaType::Bool,  176,
    QMetaType::Void, QMetaType::QString, 0x80000000 | 17,  161,   18,
    QMetaType::Void, QMetaType::QString,  161,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Bool, QMetaType::Bool, 0x80000000 | 17,  163,  164,  165,  166,  167,  168,  169,  170,  171,   18,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Bool, QMetaType::Bool,  163,  164,  165,  166,  167,  168,  169,  170,  171,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,  195,
    QMetaType::Void, QMetaType::QString,  195,
    QMetaType::Void, QMetaType::QString,  195,
    QMetaType::Bool, QMetaType::Bool,  199,
    QMetaType::Bool,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ScribusDoc::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ScribusDoc *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->docChanged(); break;
        case 1: _t->updateContents(); break;
        case 2: _t->updateContents((*reinterpret_cast< const QRect(*)>(_a[1]))); break;
        case 3: _t->firstSelectedItemType((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->updateEditItem(); break;
        case 5: _t->updateAutoSaveClock(); break;
        case 6: _t->addBookmark((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 7: _t->deleteBookmark((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 8: _t->selectionChanged(); break;
        case 9: _t->itemSelection_ToggleLock(); break;
        case 10: _t->itemSelection_ToggleSizeLock(); break;
        case 11: _t->itemSelection_ToggleImageShown(); break;
        case 12: _t->itemSelection_TogglePrintEnabled(); break;
        case 13: _t->itemSelection_ToggleBookMark((*reinterpret_cast< Selection*(*)>(_a[1]))); break;
        case 14: _t->itemSelection_ToggleBookMark(); break;
        case 15: _t->itemSelection_ToggleAnnotation((*reinterpret_cast< Selection*(*)>(_a[1]))); break;
        case 16: _t->itemSelection_ToggleAnnotation(); break;
        case 17: _t->itemSelection_Transform((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< const QTransform(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3]))); break;
        case 18: _t->itemSelection_ChangePreviewResolution((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 19: _t->allItems_ChangePreviewResolution((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 20: _t->item_setFrameShape((*reinterpret_cast< PageItem*(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< int(*)>(_a[3])),(*reinterpret_cast< const double*(*)>(_a[4]))); break;
        case 21: _t->itemSelection_ClearItem((*reinterpret_cast< Selection*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 22: _t->itemSelection_ClearItem((*reinterpret_cast< Selection*(*)>(_a[1]))); break;
        case 23: _t->itemSelection_ClearItem(); break;
        case 24: _t->itemSelection_TruncateItem((*reinterpret_cast< Selection*(*)>(_a[1]))); break;
        case 25: _t->itemSelection_TruncateItem(); break;
        case 26: _t->itemSelection_DeleteItem((*reinterpret_cast< Selection*(*)>(_a[1])),(*reinterpret_cast< bool(*)>(_a[2]))); break;
        case 27: _t->itemSelection_DeleteItem((*reinterpret_cast< Selection*(*)>(_a[1]))); break;
        case 28: _t->itemSelection_DeleteItem(); break;
        case 29: _t->itemSelection_SetItemFillTransparency((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 30: _t->itemSelection_SetItemFillTransparency((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 31: _t->itemSelection_SetHyphenWordMin((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 32: _t->itemSelection_SetHyphenWordMin((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 33: _t->itemSelection_SetHyphenConsecutiveLines((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 34: _t->itemSelection_SetHyphenConsecutiveLines((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 35: _t->itemSelection_SetHyphenChar((*reinterpret_cast< uint(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 36: _t->itemSelection_SetHyphenChar((*reinterpret_cast< uint(*)>(_a[1]))); break;
        case 37: _t->itemSelection_SetItemLineTransparency((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 38: _t->itemSelection_SetItemLineTransparency((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 39: _t->itemSelection_SetItemFillBlend((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 40: _t->itemSelection_SetItemLineBlend((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 41: _t->itemSelection_SetLineGradient((*reinterpret_cast< const VGradient(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 42: _t->itemSelection_SetLineGradient((*reinterpret_cast< const VGradient(*)>(_a[1]))); break;
        case 43: _t->itemSelection_SetLineGradientExtend((*reinterpret_cast< VGradient::VGradientRepeatMethod(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 44: _t->itemSelection_SetLineGradientExtend((*reinterpret_cast< VGradient::VGradientRepeatMethod(*)>(_a[1]))); break;
        case 45: _t->itemSelection_SetLineGradientName((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 46: _t->itemSelection_SetLineGradientName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 47: _t->itemSelection_SetFillGradient((*reinterpret_cast< const VGradient(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 48: _t->itemSelection_SetFillGradient((*reinterpret_cast< const VGradient(*)>(_a[1]))); break;
        case 49: _t->itemSelection_SetFillGradientExtend((*reinterpret_cast< VGradient::VGradientRepeatMethod(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 50: _t->itemSelection_SetFillGradientExtend((*reinterpret_cast< VGradient::VGradientRepeatMethod(*)>(_a[1]))); break;
        case 51: _t->itemSelection_SetFillGradientName((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 52: _t->itemSelection_SetFillGradientName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 53: _t->itemSelection_SetFillRule((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 54: _t->itemSelection_SetFillRule((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 55: _t->itemSelection_SetMaskGradient((*reinterpret_cast< const VGradient(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 56: _t->itemSelection_SetMaskGradient((*reinterpret_cast< const VGradient(*)>(_a[1]))); break;
        case 57: _t->itemSelection_SetMaskGradientName((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 58: _t->itemSelection_SetMaskGradientName((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 59: _t->itemSelection_SetOverprint((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 60: _t->itemSelection_SetOverprint((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 61: _t->itemSelection_ApplyImageEffects((*reinterpret_cast< const ScImageEffectList(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 62: _t->itemSelection_ApplyImageEffects((*reinterpret_cast< const ScImageEffectList(*)>(_a[1]))); break;
        case 63: _t->itemSelection_FlipH((*reinterpret_cast< Selection*(*)>(_a[1]))); break;
        case 64: _t->itemSelection_FlipH(); break;
        case 65: _t->itemSelection_FlipV((*reinterpret_cast< Selection*(*)>(_a[1]))); break;
        case 66: _t->itemSelection_FlipV(); break;
        case 67: _t->itemSelection_Rotate((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 68: _t->itemSelection_Rotate((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 69: _t->itemSelection_DoHyphenate(); break;
        case 70: _t->itemSelection_DoDeHyphenate(); break;
        case 71: _t->itemSelection_UnlinkTextFrameAndCutText((*reinterpret_cast< Selection*(*)>(_a[1]))); break;
        case 72: _t->itemSelection_UnlinkTextFrameAndCutText(); break;
        case 73: _t->itemSelection_SendToLayer((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 74: _t->itemSelection_SetImageOffset((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< Selection*(*)>(_a[3]))); break;
        case 75: _t->itemSelection_SetImageOffset((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 76: _t->itemSelection_SetImageScale((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< Selection*(*)>(_a[3]))); break;
        case 77: _t->itemSelection_SetImageScale((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 78: _t->itemSelection_SetImageScaleAndOffset((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< Selection*(*)>(_a[5]))); break;
        case 79: _t->itemSelection_SetImageScaleAndOffset((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4]))); break;
        case 80: _t->itemSelection_SetImageRotation((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 81: _t->itemSelection_SetImageRotation((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 82: _t->itemSelection_AlignItemLeft((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< ScribusDoc::AlignMethod(*)>(_a[3]))); break;
        case 83: _t->itemSelection_AlignItemRight((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< ScribusDoc::AlignMethod(*)>(_a[3]))); break;
        case 84: _t->itemSelection_AlignItemTop((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< ScribusDoc::AlignMethod(*)>(_a[3]))); break;
        case 85: _t->itemSelection_AlignItemBottom((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< ScribusDoc::AlignMethod(*)>(_a[3]))); break;
        case 86: _t->itemSelection_AlignLeftOut((*reinterpret_cast< ScribusDoc::AlignTo(*)>(_a[1])),(*reinterpret_cast< ScribusDoc::AlignMethod(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 87: _t->itemSelection_AlignRightOut((*reinterpret_cast< ScribusDoc::AlignTo(*)>(_a[1])),(*reinterpret_cast< ScribusDoc::AlignMethod(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 88: _t->itemSelection_AlignBottomIn((*reinterpret_cast< ScribusDoc::AlignTo(*)>(_a[1])),(*reinterpret_cast< ScribusDoc::AlignMethod(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 89: _t->itemSelection_AlignRightIn((*reinterpret_cast< ScribusDoc::AlignTo(*)>(_a[1])),(*reinterpret_cast< ScribusDoc::AlignMethod(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 90: _t->itemSelection_AlignBottomOut((*reinterpret_cast< ScribusDoc::AlignTo(*)>(_a[1])),(*reinterpret_cast< ScribusDoc::AlignMethod(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 91: _t->itemSelection_AlignCenterHor((*reinterpret_cast< ScribusDoc::AlignTo(*)>(_a[1])),(*reinterpret_cast< ScribusDoc::AlignMethod(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 92: _t->itemSelection_AlignLeftIn((*reinterpret_cast< ScribusDoc::AlignTo(*)>(_a[1])),(*reinterpret_cast< ScribusDoc::AlignMethod(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 93: _t->itemSelection_AlignCenterVer((*reinterpret_cast< ScribusDoc::AlignTo(*)>(_a[1])),(*reinterpret_cast< ScribusDoc::AlignMethod(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 94: _t->itemSelection_AlignTopOut((*reinterpret_cast< ScribusDoc::AlignTo(*)>(_a[1])),(*reinterpret_cast< ScribusDoc::AlignMethod(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 95: _t->itemSelection_AlignTopIn((*reinterpret_cast< ScribusDoc::AlignTo(*)>(_a[1])),(*reinterpret_cast< ScribusDoc::AlignMethod(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3]))); break;
        case 96: _t->sendItemSelectionToBack(); break;
        case 97: _t->bringItemSelectionToFront(); break;
        case 98: _t->itemSelection_LowerItem(); break;
        case 99: _t->itemSelection_RaiseItem(); break;
        case 100: _t->itemSelection_DistributeDistH((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 101: _t->itemSelection_DistributeDistH((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 102: _t->itemSelection_DistributeDistH((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 103: _t->itemSelection_DistributeDistH(); break;
        case 104: _t->itemSelection_DistributeAcrossPage((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 105: _t->itemSelection_DistributeAcrossPage(); break;
        case 106: _t->itemSelection_DistributeRight(); break;
        case 107: _t->itemSelection_DistributeBottom(); break;
        case 108: _t->itemSelection_DistributeCenterH(); break;
        case 109: _t->itemSelection_DistributeDistV((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3]))); break;
        case 110: _t->itemSelection_DistributeDistV((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 111: _t->itemSelection_DistributeDistV((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 112: _t->itemSelection_DistributeDistV(); break;
        case 113: _t->itemSelection_DistributeDownPage((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 114: _t->itemSelection_DistributeDownPage(); break;
        case 115: _t->itemSelection_DistributeLeft(); break;
        case 116: _t->itemSelection_DistributeCenterV(); break;
        case 117: _t->itemSelection_DistributeTop(); break;
        case 118: _t->itemSelection_SwapLeft(); break;
        case 119: _t->itemSelection_SwapRight(); break;
        case 120: _t->itemSelection_Duplicate((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< Selection*(*)>(_a[3]))); break;
        case 121: _t->itemSelection_Duplicate((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2]))); break;
        case 122: _t->itemSelection_MultipleDuplicate((*reinterpret_cast< const ItemMultipleDuplicateData(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 123: _t->itemSelection_MultipleDuplicate((*reinterpret_cast< const ItemMultipleDuplicateData(*)>(_a[1]))); break;
        case 124: _t->itemSelection_UniteItems((*reinterpret_cast< Selection*(*)>(_a[1]))); break;
        case 125: _t->itemSelection_UniteItems(); break;
        case 126: _t->itemSelection_SplitItems((*reinterpret_cast< Selection*(*)>(_a[1]))); break;
        case 127: _t->itemSelection_SplitItems(); break;
        case 128: _t->itemSelection_AdjustFrametoImageSize((*reinterpret_cast< Selection*(*)>(_a[1]))); break;
        case 129: _t->itemSelection_AdjustFrametoImageSize(); break;
        case 130: _t->itemSelection_AdjustImagetoFrameSize((*reinterpret_cast< Selection*(*)>(_a[1]))); break;
        case 131: _t->itemSelection_AdjustImagetoFrameSize(); break;
        case 132: _t->itemSelection_AdjustFrameHeightToText((*reinterpret_cast< Selection*(*)>(_a[1]))); break;
        case 133: _t->itemSelection_AdjustFrameHeightToText(); break;
        case 134: _t->itemSelection_ApplyArrowHead((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< Selection*(*)>(_a[3]))); break;
        case 135: _t->itemSelection_ApplyArrowHead((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 136: _t->itemSelection_ApplyArrowHead((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 137: _t->itemSelection_ApplyArrowHead(); break;
        case 138: _t->itemSelection_ApplyArrowScale((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< Selection*(*)>(_a[3]))); break;
        case 139: _t->itemSelection_SetItemPen((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 140: _t->itemSelection_SetItemPen((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 141: _t->itemSelection_SetItemPenShade((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 142: _t->itemSelection_SetItemPenShade((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 143: _t->itemSelection_SetItemGradStroke((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 144: _t->itemSelection_SetItemGradStroke((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 145: _t->itemSelection_SetItemBrush((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 146: _t->itemSelection_SetItemBrush((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 147: _t->itemSelection_SetItemBrushShade((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 148: _t->itemSelection_SetItemBrushShade((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 149: _t->itemSelection_SetItemGradMask((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 150: _t->itemSelection_SetItemGradMask((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 151: _t->itemSelection_SetItemGradFill((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 152: _t->itemSelection_SetItemGradFill((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 153: _t->itemSelection_SetItemPatternFill((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 154: _t->itemSelection_SetItemPatternFill((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 155: _t->itemSelection_SetItemPatternProps((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6])),(*reinterpret_cast< double(*)>(_a[7])),(*reinterpret_cast< bool(*)>(_a[8])),(*reinterpret_cast< bool(*)>(_a[9]))); break;
        case 156: _t->itemSelection_SetItemStrokePattern((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 157: _t->itemSelection_SetItemStrokePattern((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 158: _t->itemSelection_SetItemStrokePatternProps((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6])),(*reinterpret_cast< double(*)>(_a[7])),(*reinterpret_cast< double(*)>(_a[8])),(*reinterpret_cast< bool(*)>(_a[9])),(*reinterpret_cast< bool(*)>(_a[10])),(*reinterpret_cast< Selection*(*)>(_a[11]))); break;
        case 159: _t->itemSelection_SetItemStrokePatternProps((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6])),(*reinterpret_cast< double(*)>(_a[7])),(*reinterpret_cast< double(*)>(_a[8])),(*reinterpret_cast< bool(*)>(_a[9])),(*reinterpret_cast< bool(*)>(_a[10]))); break;
        case 160: _t->itemSelection_SetItemStrokePatternType((*reinterpret_cast< bool(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 161: _t->itemSelection_SetItemStrokePatternType((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 162: _t->itemSelection_SetItemPatternMask((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< Selection*(*)>(_a[2]))); break;
        case 163: _t->itemSelection_SetItemPatternMask((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 164: _t->itemSelection_SetItemPatternMaskProps((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6])),(*reinterpret_cast< double(*)>(_a[7])),(*reinterpret_cast< bool(*)>(_a[8])),(*reinterpret_cast< bool(*)>(_a[9])),(*reinterpret_cast< Selection*(*)>(_a[10]))); break;
        case 165: _t->itemSelection_SetItemPatternMaskProps((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4])),(*reinterpret_cast< double(*)>(_a[5])),(*reinterpret_cast< double(*)>(_a[6])),(*reinterpret_cast< double(*)>(_a[7])),(*reinterpret_cast< bool(*)>(_a[8])),(*reinterpret_cast< bool(*)>(_a[9]))); break;
        case 166: _t->itemSelection_InsertTableRows(); break;
        case 167: _t->itemSelection_InsertTableColumns(); break;
        case 168: _t->itemSelection_DeleteTableRows(); break;
        case 169: _t->itemSelection_DeleteTableColumns(); break;
        case 170: _t->itemSelection_MergeTableCells(); break;
        case 171: _t->itemSelection_SplitTableCells(); break;
        case 172: _t->itemSelection_SetTableRowHeights(); break;
        case 173: _t->itemSelection_SetTableColumnWidths(); break;
        case 174: _t->itemSelection_DistributeTableRowsEvenly(); break;
        case 175: _t->itemSelection_DistributeTableColumnsEvenly(); break;
        case 176: _t->itemSelection_AdjustFrameToTable(); break;
        case 177: _t->itemSelection_AdjustTableToFrame(); break;
        case 178: _t->undoRedoBegin(); break;
        case 179: _t->undoRedoDone(); break;
        case 180: _t->updatePic(); break;
        case 181: _t->updatePict((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 182: _t->updatePictDir((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 183: _t->removePict((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 184: { bool _r = _t->updateMarks((*reinterpret_cast< bool(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 185: { bool _r = _t->updateMarks();
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 186: _t->itemSelection_UnWeld(); break;
        case 187: _t->itemSelection_Weld(); break;
        case 188: _t->itemSelection_EditWeld(); break;
        case 189: _t->restartAutoSaveTimer(); break;
        case 190: _t->slotAutoSave(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 7:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        case 20:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< PageItem* >(); break;
            }
            break;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ScribusDoc::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusDoc::docChanged)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ScribusDoc::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusDoc::updateContents)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ScribusDoc::*)(const QRect & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusDoc::updateContents)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ScribusDoc::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusDoc::firstSelectedItemType)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ScribusDoc::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusDoc::updateEditItem)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (ScribusDoc::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusDoc::updateAutoSaveClock)) {
                *result = 5;
                return;
            }
        }
        {
            using _t = void (ScribusDoc::*)(PageItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusDoc::addBookmark)) {
                *result = 6;
                return;
            }
        }
        {
            using _t = void (ScribusDoc::*)(PageItem * );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScribusDoc::deleteBookmark)) {
                *result = 7;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ScribusDoc::staticMetaObject = { {
    QMetaObject::SuperData::link<QObject::staticMetaObject>(),
    qt_meta_stringdata_ScribusDoc.data,
    qt_meta_data_ScribusDoc,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ScribusDoc::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ScribusDoc::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ScribusDoc.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "UndoObject"))
        return static_cast< UndoObject*>(this);
    if (!strcmp(_clname, "Observable<ScribusDoc>"))
        return static_cast< Observable<ScribusDoc>*>(this);
    return QObject::qt_metacast(_clname);
}

int ScribusDoc::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QObject::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 191)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 191;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 191)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 191;
    }
    return _id;
}

// SIGNAL 0
void ScribusDoc::docChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ScribusDoc::updateContents()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ScribusDoc::updateContents(const QRect & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void ScribusDoc::firstSelectedItemType(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ScribusDoc::updateEditItem()
{
    QMetaObject::activate(this, &staticMetaObject, 4, nullptr);
}

// SIGNAL 5
void ScribusDoc::updateAutoSaveClock()
{
    QMetaObject::activate(this, &staticMetaObject, 5, nullptr);
}

// SIGNAL 6
void ScribusDoc::addBookmark(PageItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 6, _a);
}

// SIGNAL 7
void ScribusDoc::deleteBookmark(PageItem * _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 7, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
