/****************************************************************************
** Meta object code from reading C++ file 'symbolpalette.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/symbolpalette.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'symbolpalette.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_SymbolView_t {
    QByteArrayData data[6];
    char stringdata0[60];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SymbolView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SymbolView_t qt_meta_stringdata_SymbolView = {
    {
QT_MOC_LITERAL(0, 0, 10), // "SymbolView"
QT_MOC_LITERAL(1, 11, 13), // "objectDropped"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 17), // "handleContextMenu"
QT_MOC_LITERAL(4, 44, 1), // "p"
QT_MOC_LITERAL(5, 46, 13) // "changeDisplay"

    },
    "SymbolView\0objectDropped\0\0handleContextMenu\0"
    "p\0changeDisplay"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SymbolView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   29,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,   30,    2, 0x0a /* Public */,
       5,    0,   33,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QPoint,    4,
    QMetaType::Void,

       0        // eod
};

void SymbolView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SymbolView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->objectDropped(); break;
        case 1: _t->handleContextMenu((*reinterpret_cast< QPoint(*)>(_a[1]))); break;
        case 2: _t->changeDisplay(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SymbolView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SymbolView::objectDropped)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SymbolView::staticMetaObject = { {
    QMetaObject::SuperData::link<QListWidget::staticMetaObject>(),
    qt_meta_stringdata_SymbolView.data,
    qt_meta_data_SymbolView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SymbolView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SymbolView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SymbolView.stringdata0))
        return static_cast<void*>(this);
    return QListWidget::qt_metacast(_clname);
}

int SymbolView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void SymbolView::objectDropped()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
struct qt_meta_stringdata_SymbolPalette_t {
    QByteArrayData data[17];
    char stringdata0[201];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_SymbolPalette_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_SymbolPalette_t qt_meta_stringdata_SymbolPalette = {
    {
QT_MOC_LITERAL(0, 0, 13), // "SymbolPalette"
QT_MOC_LITERAL(1, 14, 9), // "startEdit"
QT_MOC_LITERAL(2, 24, 0), // ""
QT_MOC_LITERAL(3, 25, 7), // "endEdit"
QT_MOC_LITERAL(4, 33, 13), // "objectDropped"
QT_MOC_LITERAL(5, 47, 17), // "handleContextMenu"
QT_MOC_LITERAL(6, 65, 1), // "p"
QT_MOC_LITERAL(7, 67, 17), // "handleDoubleClick"
QT_MOC_LITERAL(8, 85, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(9, 102, 4), // "item"
QT_MOC_LITERAL(10, 107, 14), // "handleEditItem"
QT_MOC_LITERAL(11, 122, 16), // "handleDeleteItem"
QT_MOC_LITERAL(12, 139, 19), // "handleUpdateRequest"
QT_MOC_LITERAL(13, 159, 14), // "languageChange"
QT_MOC_LITERAL(14, 174, 13), // "keyPressEvent"
QT_MOC_LITERAL(15, 188, 10), // "QKeyEvent*"
QT_MOC_LITERAL(16, 199, 1) // "e"

    },
    "SymbolPalette\0startEdit\0\0endEdit\0"
    "objectDropped\0handleContextMenu\0p\0"
    "handleDoubleClick\0QListWidgetItem*\0"
    "item\0handleEditItem\0handleDeleteItem\0"
    "handleUpdateRequest\0languageChange\0"
    "keyPressEvent\0QKeyEvent*\0e"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_SymbolPalette[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   64,    2, 0x06 /* Public */,
       3,    0,   67,    2, 0x06 /* Public */,
       4,    0,   68,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    1,   69,    2, 0x0a /* Public */,
       7,    1,   72,    2, 0x0a /* Public */,
      10,    0,   75,    2, 0x0a /* Public */,
      11,    0,   76,    2, 0x0a /* Public */,
      12,    1,   77,    2, 0x0a /* Public */,
      13,    0,   80,    2, 0x0a /* Public */,
      14,    1,   81,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void, QMetaType::QPoint,    6,
    QMetaType::Void, 0x80000000 | 8,    9,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 15,   16,

       0        // eod
};

void SymbolPalette::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<SymbolPalette *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->startEdit((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->endEdit(); break;
        case 2: _t->objectDropped(); break;
        case 3: _t->handleContextMenu((*reinterpret_cast< QPoint(*)>(_a[1]))); break;
        case 4: _t->handleDoubleClick((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 5: _t->handleEditItem(); break;
        case 6: _t->handleDeleteItem(); break;
        case 7: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->languageChange(); break;
        case 9: _t->keyPressEvent((*reinterpret_cast< QKeyEvent*(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (SymbolPalette::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SymbolPalette::startEdit)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (SymbolPalette::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SymbolPalette::endEdit)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (SymbolPalette::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&SymbolPalette::objectDropped)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject SymbolPalette::staticMetaObject = { {
    QMetaObject::SuperData::link<ScDockPalette::staticMetaObject>(),
    qt_meta_stringdata_SymbolPalette.data,
    qt_meta_data_SymbolPalette,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *SymbolPalette::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *SymbolPalette::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_SymbolPalette.stringdata0))
        return static_cast<void*>(this);
    return ScDockPalette::qt_metacast(_clname);
}

int SymbolPalette::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScDockPalette::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void SymbolPalette::startEdit(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void SymbolPalette::endEdit()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void SymbolPalette::objectDropped()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
