/****************************************************************************
** Meta object code from reading C++ file 'propertiespalette_image.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/propertiespalette_image.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'propertiespalette_image.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PropertiesPalette_Image_t {
    QByteArrayData data[49];
    char stringdata0[667];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PropertiesPalette_Image_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PropertiesPalette_Image_t qt_meta_stringdata_PropertiesPalette_Image = {
    {
QT_MOC_LITERAL(0, 0, 23), // "PropertiesPalette_Image"
QT_MOC_LITERAL(1, 24, 7), // "UpdtGui"
QT_MOC_LITERAL(2, 32, 0), // ""
QT_MOC_LITERAL(3, 33, 13), // "setMainWindow"
QT_MOC_LITERAL(4, 47, 18), // "ScribusMainWindow*"
QT_MOC_LITERAL(5, 66, 2), // "mw"
QT_MOC_LITERAL(6, 69, 6), // "setDoc"
QT_MOC_LITERAL(7, 76, 11), // "ScribusDoc*"
QT_MOC_LITERAL(8, 88, 1), // "d"
QT_MOC_LITERAL(9, 90, 14), // "setCurrentItem"
QT_MOC_LITERAL(10, 105, 9), // "PageItem*"
QT_MOC_LITERAL(11, 115, 4), // "item"
QT_MOC_LITERAL(12, 120, 8), // "unsetDoc"
QT_MOC_LITERAL(13, 129, 9), // "unsetItem"
QT_MOC_LITERAL(14, 139, 22), // "handleSelectionChanged"
QT_MOC_LITERAL(15, 162, 19), // "handleUpdateRequest"
QT_MOC_LITERAL(16, 182, 11), // "updateFlags"
QT_MOC_LITERAL(17, 194, 17), // "showImageRotation"
QT_MOC_LITERAL(18, 212, 3), // "rot"
QT_MOC_LITERAL(19, 216, 18), // "showScaleAndOffset"
QT_MOC_LITERAL(20, 235, 3), // "scx"
QT_MOC_LITERAL(21, 239, 3), // "scy"
QT_MOC_LITERAL(22, 243, 1), // "x"
QT_MOC_LITERAL(23, 245, 1), // "y"
QT_MOC_LITERAL(24, 247, 14), // "languageChange"
QT_MOC_LITERAL(25, 262, 10), // "unitChange"
QT_MOC_LITERAL(26, 273, 12), // "localeChange"
QT_MOC_LITERAL(27, 286, 22), // "spinboxStartUserAction"
QT_MOC_LITERAL(28, 309, 23), // "spinboxFinishUserAction"
QT_MOC_LITERAL(29, 333, 22), // "updateSpinBoxConstants"
QT_MOC_LITERAL(30, 356, 13), // "handleLocalXY"
QT_MOC_LITERAL(31, 370, 16), // "handleLocalScale"
QT_MOC_LITERAL(32, 387, 14), // "handleLocalDpi"
QT_MOC_LITERAL(33, 402, 19), // "handleLocalRotation"
QT_MOC_LITERAL(34, 422, 13), // "handleScaling"
QT_MOC_LITERAL(35, 436, 12), // "handleXScale"
QT_MOC_LITERAL(36, 449, 12), // "handleYScale"
QT_MOC_LITERAL(37, 462, 10), // "handleDpiX"
QT_MOC_LITERAL(38, 473, 10), // "handleDpiY"
QT_MOC_LITERAL(39, 484, 19), // "handleImageDPIRatio"
QT_MOC_LITERAL(40, 504, 18), // "handleImageWHRatio"
QT_MOC_LITERAL(41, 523, 21), // "handleImagePageNumber"
QT_MOC_LITERAL(42, 545, 13), // "handleProfile"
QT_MOC_LITERAL(43, 559, 3), // "prn"
QT_MOC_LITERAL(44, 563, 12), // "handleIntent"
QT_MOC_LITERAL(45, 576, 23), // "handleCompressionMethod"
QT_MOC_LITERAL(46, 600, 24), // "handleCompressionQuality"
QT_MOC_LITERAL(47, 625, 18), // "handleImageEffects"
QT_MOC_LITERAL(48, 644, 22) // "handleExtImgProperties"

    },
    "PropertiesPalette_Image\0UpdtGui\0\0"
    "setMainWindow\0ScribusMainWindow*\0mw\0"
    "setDoc\0ScribusDoc*\0d\0setCurrentItem\0"
    "PageItem*\0item\0unsetDoc\0unsetItem\0"
    "handleSelectionChanged\0handleUpdateRequest\0"
    "updateFlags\0showImageRotation\0rot\0"
    "showScaleAndOffset\0scx\0scy\0x\0y\0"
    "languageChange\0unitChange\0localeChange\0"
    "spinboxStartUserAction\0spinboxFinishUserAction\0"
    "updateSpinBoxConstants\0handleLocalXY\0"
    "handleLocalScale\0handleLocalDpi\0"
    "handleLocalRotation\0handleScaling\0"
    "handleXScale\0handleYScale\0handleDpiX\0"
    "handleDpiY\0handleImageDPIRatio\0"
    "handleImageWHRatio\0handleImagePageNumber\0"
    "handleProfile\0prn\0handleIntent\0"
    "handleCompressionMethod\0"
    "handleCompressionQuality\0handleImageEffects\0"
    "handleExtImgProperties"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PropertiesPalette_Image[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      34,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  184,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    1,  187,    2, 0x0a /* Public */,
       6,    1,  190,    2, 0x0a /* Public */,
       9,    1,  193,    2, 0x0a /* Public */,
      12,    0,  196,    2, 0x0a /* Public */,
      13,    0,  197,    2, 0x0a /* Public */,
      14,    0,  198,    2, 0x0a /* Public */,
      15,    1,  199,    2, 0x0a /* Public */,
      17,    1,  202,    2, 0x0a /* Public */,
      19,    4,  205,    2, 0x0a /* Public */,
      24,    0,  214,    2, 0x0a /* Public */,
      25,    0,  215,    2, 0x0a /* Public */,
      26,    0,  216,    2, 0x0a /* Public */,
      27,    0,  217,    2, 0x09 /* Protected */,
      28,    0,  218,    2, 0x09 /* Protected */,
      29,    0,  219,    2, 0x09 /* Protected */,
      30,    0,  220,    2, 0x08 /* Private */,
      31,    0,  221,    2, 0x08 /* Private */,
      32,    0,  222,    2, 0x08 /* Private */,
      33,    0,  223,    2, 0x08 /* Private */,
      34,    0,  224,    2, 0x08 /* Private */,
      35,    0,  225,    2, 0x08 /* Private */,
      36,    0,  226,    2, 0x08 /* Private */,
      37,    0,  227,    2, 0x08 /* Private */,
      38,    0,  228,    2, 0x08 /* Private */,
      39,    0,  229,    2, 0x08 /* Private */,
      40,    0,  230,    2, 0x08 /* Private */,
      41,    0,  231,    2, 0x08 /* Private */,
      42,    1,  232,    2, 0x08 /* Private */,
      44,    0,  235,    2, 0x08 /* Private */,
      45,    0,  236,    2, 0x08 /* Private */,
      46,    0,  237,    2, 0x08 /* Private */,
      47,    0,  238,    2, 0x08 /* Private */,
      48,    0,  239,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void, QMetaType::Int,    2,

 // slots: parameters
    QMetaType::Void, 0x80000000 | 4,    5,
    QMetaType::Void, 0x80000000 | 7,    8,
    QMetaType::Void, 0x80000000 | 10,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   16,
    QMetaType::Void, QMetaType::Double,   18,
    QMetaType::Void, QMetaType::Double, QMetaType::Double, QMetaType::Double, QMetaType::Double,   20,   21,   22,   23,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,   43,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void PropertiesPalette_Image::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PropertiesPalette_Image *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->UpdtGui((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 1: _t->setMainWindow((*reinterpret_cast< ScribusMainWindow*(*)>(_a[1]))); break;
        case 2: _t->setDoc((*reinterpret_cast< ScribusDoc*(*)>(_a[1]))); break;
        case 3: _t->setCurrentItem((*reinterpret_cast< PageItem*(*)>(_a[1]))); break;
        case 4: _t->unsetDoc(); break;
        case 5: _t->unsetItem(); break;
        case 6: _t->handleSelectionChanged(); break;
        case 7: _t->handleUpdateRequest((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 8: _t->showImageRotation((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 9: _t->showScaleAndOffset((*reinterpret_cast< double(*)>(_a[1])),(*reinterpret_cast< double(*)>(_a[2])),(*reinterpret_cast< double(*)>(_a[3])),(*reinterpret_cast< double(*)>(_a[4]))); break;
        case 10: _t->languageChange(); break;
        case 11: _t->unitChange(); break;
        case 12: _t->localeChange(); break;
        case 13: _t->spinboxStartUserAction(); break;
        case 14: _t->spinboxFinishUserAction(); break;
        case 15: _t->updateSpinBoxConstants(); break;
        case 16: _t->handleLocalXY(); break;
        case 17: _t->handleLocalScale(); break;
        case 18: _t->handleLocalDpi(); break;
        case 19: _t->handleLocalRotation(); break;
        case 20: _t->handleScaling(); break;
        case 21: _t->handleXScale(); break;
        case 22: _t->handleYScale(); break;
        case 23: _t->handleDpiX(); break;
        case 24: _t->handleDpiY(); break;
        case 25: _t->handleImageDPIRatio(); break;
        case 26: _t->handleImageWHRatio(); break;
        case 27: _t->handleImagePageNumber(); break;
        case 28: _t->handleProfile((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 29: _t->handleIntent(); break;
        case 30: _t->handleCompressionMethod(); break;
        case 31: _t->handleCompressionQuality(); break;
        case 32: _t->handleImageEffects(); break;
        case 33: _t->handleExtImgProperties(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PropertiesPalette_Image::*)(int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PropertiesPalette_Image::UpdtGui)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PropertiesPalette_Image::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_PropertiesPalette_Image.data,
    qt_meta_data_PropertiesPalette_Image,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PropertiesPalette_Image::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PropertiesPalette_Image::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PropertiesPalette_Image.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::PropertiesPalette_ImageBase"))
        return static_cast< Ui::PropertiesPalette_ImageBase*>(this);
    return QWidget::qt_metacast(_clname);
}

int PropertiesPalette_Image::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 34)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 34;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 34)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 34;
    }
    return _id;
}

// SIGNAL 0
void PropertiesPalette_Image::UpdtGui(int _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
