/****************************************************************************
** Meta object code from reading C++ file 'scdlthread.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/downloadmanager/scdlthread.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scdlthread.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ScDLThread_t {
    QByteArrayData data[17];
    char stringdata0[209];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ScDLThread_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ScDLThread_t qt_meta_stringdata_ScDLThread = {
    {
QT_MOC_LITERAL(0, 0, 10), // "ScDLThread"
QT_MOC_LITERAL(1, 11, 8), // "finished"
QT_MOC_LITERAL(2, 20, 0), // ""
QT_MOC_LITERAL(3, 21, 9), // "runSignal"
QT_MOC_LITERAL(4, 31, 11), // "fileStarted"
QT_MOC_LITERAL(5, 43, 12), // "fileReceived"
QT_MOC_LITERAL(6, 56, 10), // "fileFailed"
QT_MOC_LITERAL(7, 67, 20), // "fileDownloadProgress"
QT_MOC_LITERAL(8, 88, 13), // "bytesReceived"
QT_MOC_LITERAL(9, 102, 10), // "bytesTotal"
QT_MOC_LITERAL(10, 113, 17), // "startNextDownload"
QT_MOC_LITERAL(11, 131, 16), // "downloadFinished"
QT_MOC_LITERAL(12, 148, 13), // "replyFinished"
QT_MOC_LITERAL(13, 162, 14), // "QNetworkReply*"
QT_MOC_LITERAL(14, 177, 5), // "reply"
QT_MOC_LITERAL(15, 183, 17), // "downloadReadyRead"
QT_MOC_LITERAL(16, 201, 7) // "runSlot"

    },
    "ScDLThread\0finished\0\0runSignal\0"
    "fileStarted\0fileReceived\0fileFailed\0"
    "fileDownloadProgress\0bytesReceived\0"
    "bytesTotal\0startNextDownload\0"
    "downloadFinished\0replyFinished\0"
    "QNetworkReply*\0reply\0downloadReadyRead\0"
    "runSlot"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ScDLThread[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      11,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       6,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   69,    2, 0x06 /* Public */,
       3,    0,   70,    2, 0x06 /* Public */,
       4,    1,   71,    2, 0x06 /* Public */,
       5,    1,   74,    2, 0x06 /* Public */,
       6,    1,   77,    2, 0x06 /* Public */,
       7,    2,   80,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
      10,    0,   85,    2, 0x08 /* Private */,
      11,    0,   86,    2, 0x08 /* Private */,
      12,    1,   87,    2, 0x08 /* Private */,
      15,    0,   90,    2, 0x08 /* Private */,
      16,    0,   91,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void, QMetaType::LongLong, QMetaType::LongLong,    8,    9,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 13,   14,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ScDLThread::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ScDLThread *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->finished(); break;
        case 1: _t->runSignal(); break;
        case 2: _t->fileStarted((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 3: _t->fileReceived((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 4: _t->fileFailed((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 5: _t->fileDownloadProgress((*reinterpret_cast< qint64(*)>(_a[1])),(*reinterpret_cast< qint64(*)>(_a[2]))); break;
        case 6: _t->startNextDownload(); break;
        case 7: _t->downloadFinished(); break;
        case 8: _t->replyFinished((*reinterpret_cast< QNetworkReply*(*)>(_a[1]))); break;
        case 9: _t->downloadReadyRead(); break;
        case 10: _t->runSlot(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (ScDLThread::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScDLThread::finished)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (ScDLThread::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScDLThread::runSignal)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (ScDLThread::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScDLThread::fileStarted)) {
                *result = 2;
                return;
            }
        }
        {
            using _t = void (ScDLThread::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScDLThread::fileReceived)) {
                *result = 3;
                return;
            }
        }
        {
            using _t = void (ScDLThread::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScDLThread::fileFailed)) {
                *result = 4;
                return;
            }
        }
        {
            using _t = void (ScDLThread::*)(qint64 , qint64 );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&ScDLThread::fileDownloadProgress)) {
                *result = 5;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ScDLThread::staticMetaObject = { {
    QMetaObject::SuperData::link<QThread::staticMetaObject>(),
    qt_meta_stringdata_ScDLThread.data,
    qt_meta_data_ScDLThread,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ScDLThread::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ScDLThread::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ScDLThread.stringdata0))
        return static_cast<void*>(this);
    return QThread::qt_metacast(_clname);
}

int ScDLThread::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QThread::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 11)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 11;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 11)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 11;
    }
    return _id;
}

// SIGNAL 0
void ScDLThread::finished()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void ScDLThread::runSignal()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void ScDLThread::fileStarted(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 2, _a);
}

// SIGNAL 3
void ScDLThread::fileReceived(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 3, _a);
}

// SIGNAL 4
void ScDLThread::fileFailed(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 4, _a);
}

// SIGNAL 5
void ScDLThread::fileDownloadProgress(qint64 _t1, qint64 _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 5, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
