/****************************************************************************
** Meta object code from reading C++ file 'osgeditor.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/osgeditor.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'osgeditor.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_OSGEditorDialog_t {
    QByteArrayData data[20];
    char stringdata0[224];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_OSGEditorDialog_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_OSGEditorDialog_t qt_meta_stringdata_OSGEditorDialog = {
    {
QT_MOC_LITERAL(0, 0, 15), // "OSGEditorDialog"
QT_MOC_LITERAL(1, 16, 12), // "clearDisplay"
QT_MOC_LITERAL(2, 29, 0), // ""
QT_MOC_LITERAL(3, 30, 15), // "setCameraValues"
QT_MOC_LITERAL(4, 46, 10), // "changeView"
QT_MOC_LITERAL(5, 57, 8), // "viewName"
QT_MOC_LITERAL(6, 66, 7), // "addView"
QT_MOC_LITERAL(7, 74, 10), // "removeView"
QT_MOC_LITERAL(8, 85, 10), // "renameView"
QT_MOC_LITERAL(9, 96, 7), // "newName"
QT_MOC_LITERAL(10, 104, 16), // "changeRenderMode"
QT_MOC_LITERAL(11, 121, 4), // "mode"
QT_MOC_LITERAL(12, 126, 13), // "changeACcolor"
QT_MOC_LITERAL(13, 140, 13), // "changeFCcolor"
QT_MOC_LITERAL(14, 154, 18), // "changeTransparency"
QT_MOC_LITERAL(15, 173, 5), // "value"
QT_MOC_LITERAL(16, 179, 15), // "changeLightMode"
QT_MOC_LITERAL(17, 195, 12), // "reportCamera"
QT_MOC_LITERAL(18, 208, 8), // "openFile"
QT_MOC_LITERAL(19, 217, 6) // "accept"

    },
    "OSGEditorDialog\0clearDisplay\0\0"
    "setCameraValues\0changeView\0viewName\0"
    "addView\0removeView\0renameView\0newName\0"
    "changeRenderMode\0mode\0changeACcolor\0"
    "changeFCcolor\0changeTransparency\0value\0"
    "changeLightMode\0reportCamera\0openFile\0"
    "accept"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_OSGEditorDialog[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      14,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   84,    2, 0x0a /* Public */,
       3,    0,   85,    2, 0x0a /* Public */,
       4,    1,   86,    2, 0x0a /* Public */,
       6,    0,   89,    2, 0x0a /* Public */,
       7,    0,   90,    2, 0x0a /* Public */,
       8,    1,   91,    2, 0x0a /* Public */,
      10,    1,   94,    2, 0x0a /* Public */,
      12,    0,   97,    2, 0x0a /* Public */,
      13,    0,   98,    2, 0x0a /* Public */,
      14,    1,   99,    2, 0x0a /* Public */,
      16,    1,  102,    2, 0x0a /* Public */,
      17,    0,  105,    2, 0x0a /* Public */,
      18,    0,  106,    2, 0x0a /* Public */,
      19,    0,  107,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    9,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Double,   15,
    QMetaType::Void, QMetaType::Int,   11,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void OSGEditorDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<OSGEditorDialog *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->clearDisplay(); break;
        case 1: _t->setCameraValues(); break;
        case 2: _t->changeView((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 3: _t->addView(); break;
        case 4: _t->removeView(); break;
        case 5: _t->renameView((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 6: _t->changeRenderMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 7: _t->changeACcolor(); break;
        case 8: _t->changeFCcolor(); break;
        case 9: _t->changeTransparency((*reinterpret_cast< double(*)>(_a[1]))); break;
        case 10: _t->changeLightMode((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 11: _t->reportCamera(); break;
        case 12: _t->openFile(); break;
        case 13: _t->accept(); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject OSGEditorDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_OSGEditorDialog.data,
    qt_meta_data_OSGEditorDialog,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *OSGEditorDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *OSGEditorDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_OSGEditorDialog.stringdata0))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "Ui::OSGEditor"))
        return static_cast< Ui::OSGEditor*>(this);
    return QDialog::qt_metacast(_clname);
}

int OSGEditorDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 14)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 14;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 14)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 14;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
