/********************************************************************************
** Form generated from reading UI file 'outputpreview_ps.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_OUTPUTPREVIEW_PS_H
#define UI_OUTPUTPREVIEW_PS_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGroupBox>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QHeaderView>
#include <QtWidgets/QLabel>
#include <QtWidgets/QTableWidget>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_OutputPreview_PS
{
public:
    QVBoxLayout *verticalLayout;
    QGroupBox *previewSettings;
    QVBoxLayout *verticalLayout_2;
    QComboBox *psLevelCombo;
    QComboBox *colorOutputMode;
    QCheckBox *mirrorH;
    QCheckBox *mirrorV;
    QCheckBox *clipToMargins;
    QCheckBox *convertSpots;
    QCheckBox *enableGCR;
    QGroupBox *displaySettings;
    QVBoxLayout *verticalLayout_3;
    QCheckBox *antiAliasing;
    QCheckBox *showTransparency;
    QCheckBox *enableCMYK;
    QTableWidget *inkTable;
    QCheckBox *displayInkCoverage;
    QHBoxLayout *horizontalLayout;
    QLabel *coverThresholdLabel;
    ScrSpinBox *coverThresholdValue;

    void setupUi(QWidget *OutputPreview_PS)
    {
        if (OutputPreview_PS->objectName().isEmpty())
            OutputPreview_PS->setObjectName(QString::fromUtf8("OutputPreview_PS"));
        OutputPreview_PS->resize(204, 459);
        QSizePolicy sizePolicy(QSizePolicy::Maximum, QSizePolicy::Preferred);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(OutputPreview_PS->sizePolicy().hasHeightForWidth());
        OutputPreview_PS->setSizePolicy(sizePolicy);
        verticalLayout = new QVBoxLayout(OutputPreview_PS);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        previewSettings = new QGroupBox(OutputPreview_PS);
        previewSettings->setObjectName(QString::fromUtf8("previewSettings"));
        previewSettings->setCheckable(false);
        verticalLayout_2 = new QVBoxLayout(previewSettings);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        psLevelCombo = new QComboBox(previewSettings);
        psLevelCombo->setObjectName(QString::fromUtf8("psLevelCombo"));

        verticalLayout_2->addWidget(psLevelCombo);

        colorOutputMode = new QComboBox(previewSettings);
        colorOutputMode->addItem(QString());
        colorOutputMode->addItem(QString());
        colorOutputMode->setObjectName(QString::fromUtf8("colorOutputMode"));

        verticalLayout_2->addWidget(colorOutputMode);

        mirrorH = new QCheckBox(previewSettings);
        mirrorH->setObjectName(QString::fromUtf8("mirrorH"));

        verticalLayout_2->addWidget(mirrorH);

        mirrorV = new QCheckBox(previewSettings);
        mirrorV->setObjectName(QString::fromUtf8("mirrorV"));

        verticalLayout_2->addWidget(mirrorV);

        clipToMargins = new QCheckBox(previewSettings);
        clipToMargins->setObjectName(QString::fromUtf8("clipToMargins"));

        verticalLayout_2->addWidget(clipToMargins);

        convertSpots = new QCheckBox(previewSettings);
        convertSpots->setObjectName(QString::fromUtf8("convertSpots"));

        verticalLayout_2->addWidget(convertSpots);

        enableGCR = new QCheckBox(previewSettings);
        enableGCR->setObjectName(QString::fromUtf8("enableGCR"));

        verticalLayout_2->addWidget(enableGCR);


        verticalLayout->addWidget(previewSettings);

        displaySettings = new QGroupBox(OutputPreview_PS);
        displaySettings->setObjectName(QString::fromUtf8("displaySettings"));
        verticalLayout_3 = new QVBoxLayout(displaySettings);
        verticalLayout_3->setObjectName(QString::fromUtf8("verticalLayout_3"));
        antiAliasing = new QCheckBox(displaySettings);
        antiAliasing->setObjectName(QString::fromUtf8("antiAliasing"));

        verticalLayout_3->addWidget(antiAliasing);

        showTransparency = new QCheckBox(displaySettings);
        showTransparency->setObjectName(QString::fromUtf8("showTransparency"));

        verticalLayout_3->addWidget(showTransparency);

        enableCMYK = new QCheckBox(displaySettings);
        enableCMYK->setObjectName(QString::fromUtf8("enableCMYK"));

        verticalLayout_3->addWidget(enableCMYK);

        inkTable = new QTableWidget(displaySettings);
        inkTable->setObjectName(QString::fromUtf8("inkTable"));
        inkTable->setFocusPolicy(Qt::NoFocus);
        inkTable->setSizeAdjustPolicy(QAbstractScrollArea::AdjustToContentsOnFirstShow);
        inkTable->setEditTriggers(QAbstractItemView::NoEditTriggers);
        inkTable->setSelectionMode(QAbstractItemView::NoSelection);
        inkTable->setColumnCount(0);
        inkTable->horizontalHeader()->setStretchLastSection(true);

        verticalLayout_3->addWidget(inkTable);

        displayInkCoverage = new QCheckBox(displaySettings);
        displayInkCoverage->setObjectName(QString::fromUtf8("displayInkCoverage"));

        verticalLayout_3->addWidget(displayInkCoverage);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        horizontalLayout->setSizeConstraint(QLayout::SetMaximumSize);
        coverThresholdLabel = new QLabel(displaySettings);
        coverThresholdLabel->setObjectName(QString::fromUtf8("coverThresholdLabel"));

        horizontalLayout->addWidget(coverThresholdLabel);

        coverThresholdValue = new ScrSpinBox(displaySettings);
        coverThresholdValue->setObjectName(QString::fromUtf8("coverThresholdValue"));
        coverThresholdValue->setDecimals(0);
        coverThresholdValue->setMaximum(400.000000000000000);
        coverThresholdValue->setSingleStep(10.000000000000000);
        coverThresholdValue->setValue(300.000000000000000);

        horizontalLayout->addWidget(coverThresholdValue);


        verticalLayout_3->addLayout(horizontalLayout);


        verticalLayout->addWidget(displaySettings);


        retranslateUi(OutputPreview_PS);

        QMetaObject::connectSlotsByName(OutputPreview_PS);
    } // setupUi

    void retranslateUi(QWidget *OutputPreview_PS)
    {
        previewSettings->setTitle(QCoreApplication::translate("OutputPreview_PS", "Preview Settings", nullptr));
        colorOutputMode->setItemText(0, QCoreApplication::translate("OutputPreview_PS", "Color", nullptr));
        colorOutputMode->setItemText(1, QCoreApplication::translate("OutputPreview_PS", "Grayscale", nullptr));

        mirrorH->setText(QCoreApplication::translate("OutputPreview_PS", "Mirror Page(s) Horizontally", nullptr));
        mirrorV->setText(QCoreApplication::translate("OutputPreview_PS", "Mirror Page(s) Vertically", nullptr));
        clipToMargins->setText(QCoreApplication::translate("OutputPreview_PS", "Clip to Printer Margins", nullptr));
#if QT_CONFIG(tooltip)
        convertSpots->setToolTip(QCoreApplication::translate("OutputPreview_PS", "Enables Spot Colors to be converted to composite colors. Unless you are planning to print spot colors at a commercial printer, this is probably best left enabled.", nullptr));
#endif // QT_CONFIG(tooltip)
        convertSpots->setText(QCoreApplication::translate("OutputPreview_PS", "Convert Spot Colors", nullptr));
#if QT_CONFIG(tooltip)
        enableGCR->setToolTip(QCoreApplication::translate("OutputPreview_PS", "A way of switching off some of the gray shades which are composed of cyan, yellow and magenta and using black instead. UCR most affects parts of images which are neutral and/or dark tones which are close to the gray. Use of this may improve printing some images and some experimentation and testing is need on a case by case basis. UCR reduces the possibility of over saturation with CMY inks.", nullptr));
#endif // QT_CONFIG(tooltip)
        enableGCR->setText(QCoreApplication::translate("OutputPreview_PS", "&Under Color Removal", nullptr));
        displaySettings->setTitle(QCoreApplication::translate("OutputPreview_PS", "Display Settings", nullptr));
#if QT_CONFIG(tooltip)
        antiAliasing->setToolTip(QCoreApplication::translate("OutputPreview_PS", "Provides a more pleasant view of Type 1 fonts, TrueType Fonts, OpenType Fonts, EPS, PDF and vector graphics in the preview, at the expense of a slight slowdown in previewing", nullptr));
#endif // QT_CONFIG(tooltip)
        antiAliasing->setText(QCoreApplication::translate("OutputPreview_PS", "Enable &Antialiasing", nullptr));
#if QT_CONFIG(tooltip)
        showTransparency->setToolTip(QCoreApplication::translate("OutputPreview_PS", "Shows transparency and transparent items in your document. Requires Ghostscript 7.07 or later.", nullptr));
#endif // QT_CONFIG(tooltip)
        showTransparency->setText(QCoreApplication::translate("OutputPreview_PS", "Display Trans&parency", nullptr));
#if QT_CONFIG(tooltip)
        enableCMYK->setToolTip(QCoreApplication::translate("OutputPreview_PS", "Gives a print preview using simulations of generic CMYK inks, instead of RGB colors", nullptr));
#endif // QT_CONFIG(tooltip)
        enableCMYK->setText(QCoreApplication::translate("OutputPreview_PS", "&Display CMYK", nullptr));
        displayInkCoverage->setText(QCoreApplication::translate("OutputPreview_PS", "Display Ink Coverage", nullptr));
        coverThresholdLabel->setText(QCoreApplication::translate("OutputPreview_PS", "Threshold:", nullptr));
        (void)OutputPreview_PS;
    } // retranslateUi

};

namespace Ui {
    class OutputPreview_PS: public Ui_OutputPreview_PS {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_OUTPUTPREVIEW_PS_H
