/********************************************************************************
** Form generated from reading UI file 'prefs_preflightverifierbase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PREFS_PREFLIGHTVERIFIERBASE_H
#define UI_PREFS_PREFLIGHTVERIFIERBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QFormLayout>
#include <QtWidgets/QFrame>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QPushButton>
#include <QtWidgets/QScrollArea>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QSpinBox>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>

QT_BEGIN_NAMESPACE

class Ui_Prefs_PreflightVerifier
{
public:
    QVBoxLayout *verticalLayout;
    QLabel *label;
    QFrame *line;
    QCheckBox *showPagesWithoutErrorsCheckBox;
    QCheckBox *showNonPrintingLayerErrorsCheckBox;
    QSpacerItem *verticalSpacer_3;
    QLabel *label_6;
    QScrollArea *scrollArea;
    QWidget *scrollAreaWidgetContents;
    QVBoxLayout *verticalLayout_2;
    QFormLayout *formLayout;
    QLabel *label_2;
    QComboBox *currentProfileComboBox;
    QHBoxLayout *horizontalLayout;
    QPushButton *addProfilePushButton;
    QPushButton *removeProfilePushButton;
    QSpacerItem *horizontalSpacer;
    QSpacerItem *verticalSpacer;
    QLabel *label_3;
    QFrame *line_2;
    QCheckBox *ignoreAllErrorsCheckBox;
    QCheckBox *autoCheckBeforePrintExportCheckBox;
    QCheckBox *checkMissingGlyphsCheckBox;
    QCheckBox *checkItemsNotOnAPageCheckBox;
    QCheckBox *checkEmptyTextFramesCheckBox;
    QCheckBox *checkTextOverflowCheckBox;
    QCheckBox *checkTransparenciesCheckBox;
    QCheckBox *checkMissingImagesCheckBox;
    QCheckBox *checkPartFilledImageFramesCheckBox;
    QCheckBox *checkImageResolutionCheckBox;
    QFormLayout *formLayout_2;
    QLabel *label_4;
    QSpinBox *minimumResolutionSpinBox;
    QSpinBox *maximumResolutionSpinBox;
    QLabel *label_5;
    QCheckBox *checkPlacedPDFCheckBox;
    QCheckBox *checkGIFsCheckBox;
    QCheckBox *checkPDFAnnotFieldsCheckBox;
    QCheckBox *checkVisiblePrintableLayersCheckBox;
    QCheckBox *checkNonPrintableLayersCheckBox;
    QCheckBox *checkAppliedMasterPageLocationCheckBox;
    QSpacerItem *verticalSpacer_2;

    void setupUi(QWidget *Prefs_PreflightVerifier)
    {
        if (Prefs_PreflightVerifier->objectName().isEmpty())
            Prefs_PreflightVerifier->setObjectName(QString::fromUtf8("Prefs_PreflightVerifier"));
        Prefs_PreflightVerifier->resize(623, 757);
        verticalLayout = new QVBoxLayout(Prefs_PreflightVerifier);
        verticalLayout->setObjectName(QString::fromUtf8("verticalLayout"));
        verticalLayout->setContentsMargins(0, 0, 0, 0);
        label = new QLabel(Prefs_PreflightVerifier);
        label->setObjectName(QString::fromUtf8("label"));
        QFont font;
        font.setPointSize(14);
        font.setBold(true);
        font.setWeight(75);
        label->setFont(font);

        verticalLayout->addWidget(label);

        line = new QFrame(Prefs_PreflightVerifier);
        line->setObjectName(QString::fromUtf8("line"));
        line->setFrameShape(QFrame::HLine);
        line->setFrameShadow(QFrame::Sunken);

        verticalLayout->addWidget(line);

        showPagesWithoutErrorsCheckBox = new QCheckBox(Prefs_PreflightVerifier);
        showPagesWithoutErrorsCheckBox->setObjectName(QString::fromUtf8("showPagesWithoutErrorsCheckBox"));

        verticalLayout->addWidget(showPagesWithoutErrorsCheckBox);

        showNonPrintingLayerErrorsCheckBox = new QCheckBox(Prefs_PreflightVerifier);
        showNonPrintingLayerErrorsCheckBox->setObjectName(QString::fromUtf8("showNonPrintingLayerErrorsCheckBox"));

        verticalLayout->addWidget(showNonPrintingLayerErrorsCheckBox);

        verticalSpacer_3 = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Minimum);

        verticalLayout->addItem(verticalSpacer_3);

        label_6 = new QLabel(Prefs_PreflightVerifier);
        label_6->setObjectName(QString::fromUtf8("label_6"));
        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
        label_6->setFont(font1);

        verticalLayout->addWidget(label_6);

        scrollArea = new QScrollArea(Prefs_PreflightVerifier);
        scrollArea->setObjectName(QString::fromUtf8("scrollArea"));
        scrollArea->setWidgetResizable(true);
        scrollAreaWidgetContents = new QWidget();
        scrollAreaWidgetContents->setObjectName(QString::fromUtf8("scrollAreaWidgetContents"));
        scrollAreaWidgetContents->setGeometry(QRect(0, 0, 597, 591));
        verticalLayout_2 = new QVBoxLayout(scrollAreaWidgetContents);
        verticalLayout_2->setObjectName(QString::fromUtf8("verticalLayout_2"));
        formLayout = new QFormLayout();
        formLayout->setObjectName(QString::fromUtf8("formLayout"));
        formLayout->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_2 = new QLabel(scrollAreaWidgetContents);
        label_2->setObjectName(QString::fromUtf8("label_2"));
        label_2->setFont(font1);

        formLayout->setWidget(0, QFormLayout::LabelRole, label_2);

        currentProfileComboBox = new QComboBox(scrollAreaWidgetContents);
        currentProfileComboBox->setObjectName(QString::fromUtf8("currentProfileComboBox"));
        currentProfileComboBox->setEditable(true);

        formLayout->setWidget(0, QFormLayout::FieldRole, currentProfileComboBox);


        verticalLayout_2->addLayout(formLayout);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        addProfilePushButton = new QPushButton(scrollAreaWidgetContents);
        addProfilePushButton->setObjectName(QString::fromUtf8("addProfilePushButton"));

        horizontalLayout->addWidget(addProfilePushButton);

        removeProfilePushButton = new QPushButton(scrollAreaWidgetContents);
        removeProfilePushButton->setObjectName(QString::fromUtf8("removeProfilePushButton"));

        horizontalLayout->addWidget(removeProfilePushButton);

        horizontalSpacer = new QSpacerItem(40, 20, QSizePolicy::Expanding, QSizePolicy::Minimum);

        horizontalLayout->addItem(horizontalSpacer);


        verticalLayout_2->addLayout(horizontalLayout);

        verticalSpacer = new QSpacerItem(20, 20, QSizePolicy::Minimum, QSizePolicy::Fixed);

        verticalLayout_2->addItem(verticalSpacer);

        label_3 = new QLabel(scrollAreaWidgetContents);
        label_3->setObjectName(QString::fromUtf8("label_3"));
        label_3->setFont(font1);

        verticalLayout_2->addWidget(label_3);

        line_2 = new QFrame(scrollAreaWidgetContents);
        line_2->setObjectName(QString::fromUtf8("line_2"));
        line_2->setFrameShape(QFrame::HLine);
        line_2->setFrameShadow(QFrame::Sunken);

        verticalLayout_2->addWidget(line_2);

        ignoreAllErrorsCheckBox = new QCheckBox(scrollAreaWidgetContents);
        ignoreAllErrorsCheckBox->setObjectName(QString::fromUtf8("ignoreAllErrorsCheckBox"));

        verticalLayout_2->addWidget(ignoreAllErrorsCheckBox);

        autoCheckBeforePrintExportCheckBox = new QCheckBox(scrollAreaWidgetContents);
        autoCheckBeforePrintExportCheckBox->setObjectName(QString::fromUtf8("autoCheckBeforePrintExportCheckBox"));

        verticalLayout_2->addWidget(autoCheckBeforePrintExportCheckBox);

        checkMissingGlyphsCheckBox = new QCheckBox(scrollAreaWidgetContents);
        checkMissingGlyphsCheckBox->setObjectName(QString::fromUtf8("checkMissingGlyphsCheckBox"));

        verticalLayout_2->addWidget(checkMissingGlyphsCheckBox);

        checkItemsNotOnAPageCheckBox = new QCheckBox(scrollAreaWidgetContents);
        checkItemsNotOnAPageCheckBox->setObjectName(QString::fromUtf8("checkItemsNotOnAPageCheckBox"));

        verticalLayout_2->addWidget(checkItemsNotOnAPageCheckBox);

        checkEmptyTextFramesCheckBox = new QCheckBox(scrollAreaWidgetContents);
        checkEmptyTextFramesCheckBox->setObjectName(QString::fromUtf8("checkEmptyTextFramesCheckBox"));

        verticalLayout_2->addWidget(checkEmptyTextFramesCheckBox);

        checkTextOverflowCheckBox = new QCheckBox(scrollAreaWidgetContents);
        checkTextOverflowCheckBox->setObjectName(QString::fromUtf8("checkTextOverflowCheckBox"));

        verticalLayout_2->addWidget(checkTextOverflowCheckBox);

        checkTransparenciesCheckBox = new QCheckBox(scrollAreaWidgetContents);
        checkTransparenciesCheckBox->setObjectName(QString::fromUtf8("checkTransparenciesCheckBox"));

        verticalLayout_2->addWidget(checkTransparenciesCheckBox);

        checkMissingImagesCheckBox = new QCheckBox(scrollAreaWidgetContents);
        checkMissingImagesCheckBox->setObjectName(QString::fromUtf8("checkMissingImagesCheckBox"));

        verticalLayout_2->addWidget(checkMissingImagesCheckBox);

        checkPartFilledImageFramesCheckBox = new QCheckBox(scrollAreaWidgetContents);
        checkPartFilledImageFramesCheckBox->setObjectName(QString::fromUtf8("checkPartFilledImageFramesCheckBox"));

        verticalLayout_2->addWidget(checkPartFilledImageFramesCheckBox);

        checkImageResolutionCheckBox = new QCheckBox(scrollAreaWidgetContents);
        checkImageResolutionCheckBox->setObjectName(QString::fromUtf8("checkImageResolutionCheckBox"));

        verticalLayout_2->addWidget(checkImageResolutionCheckBox);

        formLayout_2 = new QFormLayout();
        formLayout_2->setObjectName(QString::fromUtf8("formLayout_2"));
        formLayout_2->setFormAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
        label_4 = new QLabel(scrollAreaWidgetContents);
        label_4->setObjectName(QString::fromUtf8("label_4"));

        formLayout_2->setWidget(0, QFormLayout::LabelRole, label_4);

        minimumResolutionSpinBox = new QSpinBox(scrollAreaWidgetContents);
        minimumResolutionSpinBox->setObjectName(QString::fromUtf8("minimumResolutionSpinBox"));
        minimumResolutionSpinBox->setMinimum(10);
        minimumResolutionSpinBox->setMaximum(4800);

        formLayout_2->setWidget(0, QFormLayout::FieldRole, minimumResolutionSpinBox);

        maximumResolutionSpinBox = new QSpinBox(scrollAreaWidgetContents);
        maximumResolutionSpinBox->setObjectName(QString::fromUtf8("maximumResolutionSpinBox"));
        maximumResolutionSpinBox->setMinimum(10);
        maximumResolutionSpinBox->setMaximum(4800);

        formLayout_2->setWidget(1, QFormLayout::FieldRole, maximumResolutionSpinBox);

        label_5 = new QLabel(scrollAreaWidgetContents);
        label_5->setObjectName(QString::fromUtf8("label_5"));

        formLayout_2->setWidget(1, QFormLayout::LabelRole, label_5);


        verticalLayout_2->addLayout(formLayout_2);

        checkPlacedPDFCheckBox = new QCheckBox(scrollAreaWidgetContents);
        checkPlacedPDFCheckBox->setObjectName(QString::fromUtf8("checkPlacedPDFCheckBox"));

        verticalLayout_2->addWidget(checkPlacedPDFCheckBox);

        checkGIFsCheckBox = new QCheckBox(scrollAreaWidgetContents);
        checkGIFsCheckBox->setObjectName(QString::fromUtf8("checkGIFsCheckBox"));

        verticalLayout_2->addWidget(checkGIFsCheckBox);

        checkPDFAnnotFieldsCheckBox = new QCheckBox(scrollAreaWidgetContents);
        checkPDFAnnotFieldsCheckBox->setObjectName(QString::fromUtf8("checkPDFAnnotFieldsCheckBox"));

        verticalLayout_2->addWidget(checkPDFAnnotFieldsCheckBox);

        checkVisiblePrintableLayersCheckBox = new QCheckBox(scrollAreaWidgetContents);
        checkVisiblePrintableLayersCheckBox->setObjectName(QString::fromUtf8("checkVisiblePrintableLayersCheckBox"));

        verticalLayout_2->addWidget(checkVisiblePrintableLayersCheckBox);

        checkNonPrintableLayersCheckBox = new QCheckBox(scrollAreaWidgetContents);
        checkNonPrintableLayersCheckBox->setObjectName(QString::fromUtf8("checkNonPrintableLayersCheckBox"));

        verticalLayout_2->addWidget(checkNonPrintableLayersCheckBox);

        checkAppliedMasterPageLocationCheckBox = new QCheckBox(scrollAreaWidgetContents);
        checkAppliedMasterPageLocationCheckBox->setObjectName(QString::fromUtf8("checkAppliedMasterPageLocationCheckBox"));

        verticalLayout_2->addWidget(checkAppliedMasterPageLocationCheckBox);

        verticalSpacer_2 = new QSpacerItem(20, 124, QSizePolicy::Minimum, QSizePolicy::Expanding);

        verticalLayout_2->addItem(verticalSpacer_2);

        scrollArea->setWidget(scrollAreaWidgetContents);

        verticalLayout->addWidget(scrollArea);


        retranslateUi(Prefs_PreflightVerifier);

        QMetaObject::connectSlotsByName(Prefs_PreflightVerifier);
    } // setupUi

    void retranslateUi(QWidget *Prefs_PreflightVerifier)
    {
        label->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Preflight Verifier", nullptr));
        showPagesWithoutErrorsCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Show pages without errors", nullptr));
        showNonPrintingLayerErrorsCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Show errors for items on layers that will not print", nullptr));
        label_6->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Profile Setup", nullptr));
        label_2->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Output Profile:", nullptr));
        addProfilePushButton->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Add New Profile", nullptr));
        removeProfilePushButton->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Remove Profile", nullptr));
        label_3->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Profile Options", nullptr));
        ignoreAllErrorsCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Ignore all errors", nullptr));
        autoCheckBeforePrintExportCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Automatic check before printing or exporting", nullptr));
        checkMissingGlyphsCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Check for missing glyphs", nullptr));
        checkItemsNotOnAPageCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Check for items not on a page", nullptr));
        checkEmptyTextFramesCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Check for empty text frames", nullptr));
        checkTextOverflowCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Check for overflow in text frames", nullptr));
        checkTransparenciesCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Check for used transparencies", nullptr));
        checkMissingImagesCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Check for missing images", nullptr));
        checkPartFilledImageFramesCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Check for part filled image frames", nullptr));
        checkImageResolutionCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Check image resolution", nullptr));
        label_4->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Minimum Resolution:", nullptr));
        minimumResolutionSpinBox->setSuffix(QCoreApplication::translate("Prefs_PreflightVerifier", " dpi", nullptr));
        maximumResolutionSpinBox->setSuffix(QCoreApplication::translate("Prefs_PreflightVerifier", " dpi", nullptr));
        label_5->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Maximum Resolution:", nullptr));
        checkPlacedPDFCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Check for placed PDF files", nullptr));
        checkGIFsCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Check for GIF images", nullptr));
        checkPDFAnnotFieldsCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Check for PDF Annotations and Fields", nullptr));
        checkVisiblePrintableLayersCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Check for Visible/Printable mismatch in layers", nullptr));
        checkNonPrintableLayersCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Ignore non-printable layers", nullptr));
        checkAppliedMasterPageLocationCheckBox->setText(QCoreApplication::translate("Prefs_PreflightVerifier", "Check if applied master page location is the same as each page's location", nullptr));
        (void)Prefs_PreflightVerifier;
    } // retranslateUi

};

namespace Ui {
    class Prefs_PreflightVerifier: public Ui_Prefs_PreflightVerifier {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PREFS_PREFLIGHTVERIFIERBASE_H
