/****************************************************************************
** Meta object code from reading C++ file 'scrapbookpalette.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.18)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "../../../../scribus-1.6.6/scribus/ui/scrapbookpalette.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'scrapbookpalette.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.18. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_BibView_t {
    QByteArrayData data[8];
    char stringdata0[67];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_BibView_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_BibView_t qt_meta_stringdata_BibView = {
    {
QT_MOC_LITERAL(0, 0, 7), // "BibView"
QT_MOC_LITERAL(1, 8, 10), // "objDropped"
QT_MOC_LITERAL(2, 19, 0), // ""
QT_MOC_LITERAL(3, 20, 4), // "text"
QT_MOC_LITERAL(4, 25, 11), // "fileDropped"
QT_MOC_LITERAL(5, 37, 4), // "path"
QT_MOC_LITERAL(6, 42, 10), // "testResult"
QT_MOC_LITERAL(7, 53, 13) // "paletteClosed"

    },
    "BibView\0objDropped\0\0text\0fileDropped\0"
    "path\0testResult\0paletteClosed"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_BibView[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       3,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,   29,    2, 0x06 /* Public */,
       4,    2,   32,    2, 0x06 /* Public */,
       7,    0,   37,    2, 0x06 /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    5,    6,
    QMetaType::Void,

       0        // eod
};

void BibView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<BibView *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->objDropped((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 1: _t->fileDropped((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 2: _t->paletteClosed(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (BibView::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BibView::objDropped)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (BibView::*)(QString , int );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BibView::fileDropped)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (BibView::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&BibView::paletteClosed)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject BibView::staticMetaObject = { {
    QMetaObject::SuperData::link<QListWidget::staticMetaObject>(),
    qt_meta_stringdata_BibView.data,
    qt_meta_data_BibView,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *BibView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *BibView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_BibView.stringdata0))
        return static_cast<void*>(this);
    return QListWidget::qt_metacast(_clname);
}

int BibView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QListWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 3)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 3;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 3)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 3;
    }
    return _id;
}

// SIGNAL 0
void BibView::objDropped(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}

// SIGNAL 1
void BibView::fileDropped(QString _t1, int _t2)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))), const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t2))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void BibView::paletteClosed()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
struct qt_meta_stringdata_Biblio_t {
    QByteArrayData data[37];
    char stringdata0[386];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_Biblio_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_Biblio_t qt_meta_stringdata_Biblio = {
    {
QT_MOC_LITERAL(0, 0, 6), // "Biblio"
QT_MOC_LITERAL(1, 7, 17), // "updateRecentMenue"
QT_MOC_LITERAL(2, 25, 0), // ""
QT_MOC_LITERAL(3, 26, 17), // "pasteToActualPage"
QT_MOC_LITERAL(4, 44, 20), // "scrapbookListChanged"
QT_MOC_LITERAL(5, 65, 13), // "iconSetChange"
QT_MOC_LITERAL(6, 79, 14), // "languageChange"
QT_MOC_LITERAL(7, 94, 11), // "objFromFile"
QT_MOC_LITERAL(8, 106, 4), // "path"
QT_MOC_LITERAL(9, 111, 10), // "testResult"
QT_MOC_LITERAL(10, 122, 11), // "objFromMenu"
QT_MOC_LITERAL(11, 134, 4), // "text"
QT_MOC_LITERAL(12, 139, 15), // "objFromMainMenu"
QT_MOC_LITERAL(13, 155, 7), // "scrapID"
QT_MOC_LITERAL(14, 163, 10), // "closeOnDel"
QT_MOC_LITERAL(15, 174, 7), // "libName"
QT_MOC_LITERAL(16, 182, 9), // "reloadLib"
QT_MOC_LITERAL(17, 192, 8), // "fileName"
QT_MOC_LITERAL(18, 201, 17), // "handleDoubleClick"
QT_MOC_LITERAL(19, 219, 16), // "QListWidgetItem*"
QT_MOC_LITERAL(20, 236, 3), // "ite"
QT_MOC_LITERAL(21, 240, 17), // "handlePasteToPage"
QT_MOC_LITERAL(22, 258, 11), // "handleMouse"
QT_MOC_LITERAL(23, 270, 1), // "p"
QT_MOC_LITERAL(24, 272, 7), // "copyObj"
QT_MOC_LITERAL(25, 280, 2), // "id"
QT_MOC_LITERAL(26, 283, 7), // "moveObj"
QT_MOC_LITERAL(27, 291, 9), // "deleteObj"
QT_MOC_LITERAL(28, 301, 12), // "deleteAllObj"
QT_MOC_LITERAL(29, 314, 9), // "renameObj"
QT_MOC_LITERAL(30, 324, 6), // "newLib"
QT_MOC_LITERAL(31, 331, 10), // "updateView"
QT_MOC_LITERAL(32, 342, 10), // "goOneDirUp"
QT_MOC_LITERAL(33, 353, 8), // "closeLib"
QT_MOC_LITERAL(34, 362, 10), // "libChanged"
QT_MOC_LITERAL(35, 373, 5), // "index"
QT_MOC_LITERAL(36, 379, 6) // "Import"

    },
    "Biblio\0updateRecentMenue\0\0pasteToActualPage\0"
    "scrapbookListChanged\0iconSetChange\0"
    "languageChange\0objFromFile\0path\0"
    "testResult\0objFromMenu\0text\0objFromMainMenu\0"
    "scrapID\0closeOnDel\0libName\0reloadLib\0"
    "fileName\0handleDoubleClick\0QListWidgetItem*\0"
    "ite\0handlePasteToPage\0handleMouse\0p\0"
    "copyObj\0id\0moveObj\0deleteObj\0deleteAllObj\0"
    "renameObj\0newLib\0updateView\0goOneDirUp\0"
    "closeLib\0libChanged\0index\0Import"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_Biblio[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      24,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       3,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,  134,    2, 0x06 /* Public */,
       3,    1,  135,    2, 0x06 /* Public */,
       4,    0,  138,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       5,    0,  139,    2, 0x0a /* Public */,
       6,    0,  140,    2, 0x0a /* Public */,
       7,    2,  141,    2, 0x0a /* Public */,
      10,    1,  146,    2, 0x0a /* Public */,
      12,    2,  149,    2, 0x0a /* Public */,
      14,    1,  154,    2, 0x0a /* Public */,
      16,    1,  157,    2, 0x0a /* Public */,
      18,    1,  160,    2, 0x08 /* Private */,
      21,    0,  163,    2, 0x08 /* Private */,
      22,    1,  164,    2, 0x08 /* Private */,
      24,    1,  167,    2, 0x08 /* Private */,
      26,    1,  170,    2, 0x08 /* Private */,
      27,    0,  173,    2, 0x08 /* Private */,
      28,    0,  174,    2, 0x08 /* Private */,
      29,    0,  175,    2, 0x08 /* Private */,
      30,    0,  176,    2, 0x08 /* Private */,
      31,    0,  177,    2, 0x08 /* Private */,
      32,    0,  178,    2, 0x08 /* Private */,
      33,    0,  179,    2, 0x08 /* Private */,
      34,    1,  180,    2, 0x08 /* Private */,
      36,    0,  183,    2, 0x08 /* Private */,

 // signals: parameters
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString,    2,
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,    8,    9,
    QMetaType::Void, QMetaType::QString,   11,
    QMetaType::Void, QMetaType::QString, QMetaType::Int,   11,   13,
    QMetaType::Void, QMetaType::QString,   15,
    QMetaType::Void, QMetaType::QString,   17,
    QMetaType::Void, 0x80000000 | 19,   20,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,   23,
    QMetaType::Bool, QMetaType::Int,   25,
    QMetaType::Void, QMetaType::Int,   25,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   35,
    QMetaType::Void,

       0        // eod
};

void Biblio::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<Biblio *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->updateRecentMenue(); break;
        case 1: _t->pasteToActualPage((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 2: _t->scrapbookListChanged(); break;
        case 3: _t->iconSetChange(); break;
        case 4: _t->languageChange(); break;
        case 5: _t->objFromFile((*reinterpret_cast< const QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 6: _t->objFromMenu((*reinterpret_cast< QString(*)>(_a[1]))); break;
        case 7: _t->objFromMainMenu((*reinterpret_cast< QString(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2]))); break;
        case 8: _t->closeOnDel((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 9: _t->reloadLib((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 10: _t->handleDoubleClick((*reinterpret_cast< QListWidgetItem*(*)>(_a[1]))); break;
        case 11: _t->handlePasteToPage(); break;
        case 12: _t->handleMouse((*reinterpret_cast< QPoint(*)>(_a[1]))); break;
        case 13: { bool _r = _t->copyObj((*reinterpret_cast< int(*)>(_a[1])));
            if (_a[0]) *reinterpret_cast< bool*>(_a[0]) = std::move(_r); }  break;
        case 14: _t->moveObj((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 15: _t->deleteObj(); break;
        case 16: _t->deleteAllObj(); break;
        case 17: _t->renameObj(); break;
        case 18: _t->newLib(); break;
        case 19: _t->updateView(); break;
        case 20: _t->goOneDirUp(); break;
        case 21: _t->closeLib(); break;
        case 22: _t->libChanged((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 23: _t->Import(); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (Biblio::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Biblio::updateRecentMenue)) {
                *result = 0;
                return;
            }
        }
        {
            using _t = void (Biblio::*)(QString );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Biblio::pasteToActualPage)) {
                *result = 1;
                return;
            }
        }
        {
            using _t = void (Biblio::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&Biblio::scrapbookListChanged)) {
                *result = 2;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject Biblio::staticMetaObject = { {
    QMetaObject::SuperData::link<ScDockPalette::staticMetaObject>(),
    qt_meta_stringdata_Biblio.data,
    qt_meta_data_Biblio,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *Biblio::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *Biblio::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_Biblio.stringdata0))
        return static_cast<void*>(this);
    return ScDockPalette::qt_metacast(_clname);
}

int Biblio::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = ScDockPalette::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 24)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 24;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 24)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 24;
    }
    return _id;
}

// SIGNAL 0
void Biblio::updateRecentMenue()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void Biblio::pasteToActualPage(QString _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 1, _a);
}

// SIGNAL 2
void Biblio::scrapbookListChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
