/********************************************************************************
** Form generated from reading UI file 'propertiespalette_linebase.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_PROPERTIESPALETTE_LINEBASE_H
#define UI_PROPERTIESPALETTE_LINEBASE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QComboBox>
#include <QtWidgets/QGridLayout>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QLabel>
#include <QtWidgets/QListWidget>
#include <QtWidgets/QWidget>
#include "ui/arrowchooser.h"
#include "ui/dasheditor.h"
#include "ui/linecombo.h"
#include "ui/scrspinbox.h"

QT_BEGIN_NAMESPACE

class Ui_PropertiesPalette_LineBase
{
public:
    QGridLayout *gridLayout;
    QLabel *lineTypeLabel;
    QLabel *lineWidthLabel;
    QComboBox *lineEndStyle;
    QComboBox *lineJoinStyle;
    QListWidget *lineStyles;
    ArrowChooser *endArrow;
    DashEditor *dashEditor;
    ScrSpinBox *lineWidth;
    ArrowChooser *startArrow;
    LineCombo *lineType;
    QComboBox *lineMode;
    QLabel *lineModeLabel;
    QHBoxLayout *horizontalLayout;
    QLabel *startArrowScaleLabel;
    ScrSpinBox *startArrowScale;
    QHBoxLayout *horizontalLayout_2;
    QLabel *endArrowScaleLabel;
    ScrSpinBox *endArrowScale;
    QLabel *startArrowLabel;
    QLabel *endArrowLabel;
    QLabel *lineJoinLabel;
    QLabel *lineEndLabel;

    void setupUi(QWidget *PropertiesPalette_LineBase)
    {
        if (PropertiesPalette_LineBase->objectName().isEmpty())
            PropertiesPalette_LineBase->setObjectName(QString::fromUtf8("PropertiesPalette_LineBase"));
        PropertiesPalette_LineBase->resize(265, 421);
        gridLayout = new QGridLayout(PropertiesPalette_LineBase);
        gridLayout->setSpacing(3);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        gridLayout->setContentsMargins(3, 3, 3, 3);
        lineTypeLabel = new QLabel(PropertiesPalette_LineBase);
        lineTypeLabel->setObjectName(QString::fromUtf8("lineTypeLabel"));

        gridLayout->addWidget(lineTypeLabel, 1, 0, 1, 1);

        lineWidthLabel = new QLabel(PropertiesPalette_LineBase);
        lineWidthLabel->setObjectName(QString::fromUtf8("lineWidthLabel"));

        gridLayout->addWidget(lineWidthLabel, 7, 0, 1, 1);

        lineEndStyle = new QComboBox(PropertiesPalette_LineBase);
        lineEndStyle->setObjectName(QString::fromUtf8("lineEndStyle"));

        gridLayout->addWidget(lineEndStyle, 9, 1, 1, 1);

        lineJoinStyle = new QComboBox(PropertiesPalette_LineBase);
        lineJoinStyle->setObjectName(QString::fromUtf8("lineJoinStyle"));

        gridLayout->addWidget(lineJoinStyle, 8, 1, 1, 1);

        lineStyles = new QListWidget(PropertiesPalette_LineBase);
        lineStyles->setObjectName(QString::fromUtf8("lineStyles"));

        gridLayout->addWidget(lineStyles, 10, 0, 1, 2);

        endArrow = new ArrowChooser(PropertiesPalette_LineBase);
        endArrow->setObjectName(QString::fromUtf8("endArrow"));

        gridLayout->addWidget(endArrow, 4, 1, 1, 1);

        dashEditor = new DashEditor(PropertiesPalette_LineBase);
        dashEditor->setObjectName(QString::fromUtf8("dashEditor"));
        dashEditor->setFrameShape(QFrame::StyledPanel);
        dashEditor->setFrameShadow(QFrame::Raised);

        gridLayout->addWidget(dashEditor, 2, 0, 1, 2);

        lineWidth = new ScrSpinBox(PropertiesPalette_LineBase);
        lineWidth->setObjectName(QString::fromUtf8("lineWidth"));
        lineWidth->setMaximum(300.000000000000000);

        gridLayout->addWidget(lineWidth, 7, 1, 1, 1);

        startArrow = new ArrowChooser(PropertiesPalette_LineBase);
        startArrow->setObjectName(QString::fromUtf8("startArrow"));

        gridLayout->addWidget(startArrow, 4, 0, 1, 1);

        lineType = new LineCombo(PropertiesPalette_LineBase);
        lineType->setObjectName(QString::fromUtf8("lineType"));

        gridLayout->addWidget(lineType, 1, 1, 1, 1);

        lineMode = new QComboBox(PropertiesPalette_LineBase);
        lineMode->setObjectName(QString::fromUtf8("lineMode"));

        gridLayout->addWidget(lineMode, 0, 1, 1, 1);

        lineModeLabel = new QLabel(PropertiesPalette_LineBase);
        lineModeLabel->setObjectName(QString::fromUtf8("lineModeLabel"));

        gridLayout->addWidget(lineModeLabel, 0, 0, 1, 1);

        horizontalLayout = new QHBoxLayout();
        horizontalLayout->setSpacing(3);
        horizontalLayout->setObjectName(QString::fromUtf8("horizontalLayout"));
        startArrowScaleLabel = new QLabel(PropertiesPalette_LineBase);
        startArrowScaleLabel->setObjectName(QString::fromUtf8("startArrowScaleLabel"));

        horizontalLayout->addWidget(startArrowScaleLabel);

        startArrowScale = new ScrSpinBox(PropertiesPalette_LineBase);
        startArrowScale->setObjectName(QString::fromUtf8("startArrowScale"));
        QSizePolicy sizePolicy(QSizePolicy::Expanding, QSizePolicy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(startArrowScale->sizePolicy().hasHeightForWidth());
        startArrowScale->setSizePolicy(sizePolicy);
        startArrowScale->setMinimum(1.000000000000000);
        startArrowScale->setMaximum(300.000000000000000);

        horizontalLayout->addWidget(startArrowScale);


        gridLayout->addLayout(horizontalLayout, 5, 0, 1, 1);

        horizontalLayout_2 = new QHBoxLayout();
        horizontalLayout_2->setSpacing(3);
        horizontalLayout_2->setObjectName(QString::fromUtf8("horizontalLayout_2"));
        endArrowScaleLabel = new QLabel(PropertiesPalette_LineBase);
        endArrowScaleLabel->setObjectName(QString::fromUtf8("endArrowScaleLabel"));

        horizontalLayout_2->addWidget(endArrowScaleLabel);

        endArrowScale = new ScrSpinBox(PropertiesPalette_LineBase);
        endArrowScale->setObjectName(QString::fromUtf8("endArrowScale"));
        sizePolicy.setHeightForWidth(endArrowScale->sizePolicy().hasHeightForWidth());
        endArrowScale->setSizePolicy(sizePolicy);
        endArrowScale->setMinimum(1.000000000000000);
        endArrowScale->setMaximum(300.000000000000000);

        horizontalLayout_2->addWidget(endArrowScale);


        gridLayout->addLayout(horizontalLayout_2, 5, 1, 1, 1);

        startArrowLabel = new QLabel(PropertiesPalette_LineBase);
        startArrowLabel->setObjectName(QString::fromUtf8("startArrowLabel"));

        gridLayout->addWidget(startArrowLabel, 3, 0, 1, 1);

        endArrowLabel = new QLabel(PropertiesPalette_LineBase);
        endArrowLabel->setObjectName(QString::fromUtf8("endArrowLabel"));

        gridLayout->addWidget(endArrowLabel, 3, 1, 1, 1);

        lineJoinLabel = new QLabel(PropertiesPalette_LineBase);
        lineJoinLabel->setObjectName(QString::fromUtf8("lineJoinLabel"));

        gridLayout->addWidget(lineJoinLabel, 8, 0, 1, 1);

        lineEndLabel = new QLabel(PropertiesPalette_LineBase);
        lineEndLabel->setObjectName(QString::fromUtf8("lineEndLabel"));

        gridLayout->addWidget(lineEndLabel, 9, 0, 1, 1);


        retranslateUi(PropertiesPalette_LineBase);

        QMetaObject::connectSlotsByName(PropertiesPalette_LineBase);
    } // setupUi

    void retranslateUi(QWidget *PropertiesPalette_LineBase)
    {
        lineTypeLabel->setText(QCoreApplication::translate("PropertiesPalette_LineBase", "T&ype of Line:", nullptr));
        lineWidthLabel->setText(QCoreApplication::translate("PropertiesPalette_LineBase", "Line &Width:", nullptr));
        lineWidth->setSpecialValueText(QCoreApplication::translate("PropertiesPalette_LineBase", "Hairline", nullptr));
        lineModeLabel->setText(QCoreApplication::translate("PropertiesPalette_LineBase", "&Basepoint:", nullptr));
        startArrowScaleLabel->setText(QCoreApplication::translate("PropertiesPalette_LineBase", "Scaling:", nullptr));
        endArrowScaleLabel->setText(QCoreApplication::translate("PropertiesPalette_LineBase", "Scaling:", nullptr));
        startArrowLabel->setText(QCoreApplication::translate("PropertiesPalette_LineBase", "Start Arrow:", nullptr));
        endArrowLabel->setText(QCoreApplication::translate("PropertiesPalette_LineBase", "End Arrow:", nullptr));
        lineJoinLabel->setText(QCoreApplication::translate("PropertiesPalette_LineBase", "Ed&ges:", nullptr));
        lineEndLabel->setText(QCoreApplication::translate("PropertiesPalette_LineBase", "&Endings:", nullptr));
        (void)PropertiesPalette_LineBase;
    } // retranslateUi

};

namespace Ui {
    class PropertiesPalette_LineBase: public Ui_PropertiesPalette_LineBase {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_PROPERTIESPALETTE_LINEBASE_H
