/********************************************************************************
** Form generated from reading UI file 'runscriptdialog.ui'
**
** Created by: Qt User Interface Compiler version 5.15.18
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_RUNSCRIPTDIALOG_H
#define UI_RUNSCRIPTDIALOG_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QCheckBox>
#include <QtWidgets/QDialog>
#include <QtWidgets/QDialogButtonBox>
#include <QtWidgets/QGridLayout>
#include "ui/scfilewidget.h"

QT_BEGIN_NAMESPACE

class Ui_RunScriptDialog
{
public:
    QGridLayout *gridLayout;
    ScFileWidget *fileWidget;
    QCheckBox *extChk;
    QDialogButtonBox *buttonBox;

    void setupUi(QDialog *RunScriptDialog)
    {
        if (RunScriptDialog->objectName().isEmpty())
            RunScriptDialog->setObjectName(QString::fromUtf8("RunScriptDialog"));
        RunScriptDialog->resize(400, 300);
        gridLayout = new QGridLayout(RunScriptDialog);
        gridLayout->setObjectName(QString::fromUtf8("gridLayout"));
        fileWidget = new ScFileWidget(RunScriptDialog);
        fileWidget->setObjectName(QString::fromUtf8("fileWidget"));

        gridLayout->addWidget(fileWidget, 0, 0, 1, 1);

        extChk = new QCheckBox(RunScriptDialog);
        extChk->setObjectName(QString::fromUtf8("extChk"));

        gridLayout->addWidget(extChk, 1, 0, 1, 1);

        buttonBox = new QDialogButtonBox(RunScriptDialog);
        buttonBox->setObjectName(QString::fromUtf8("buttonBox"));
        buttonBox->setOrientation(Qt::Horizontal);
        buttonBox->setStandardButtons(QDialogButtonBox::Cancel|QDialogButtonBox::Ok);

        gridLayout->addWidget(buttonBox, 2, 0, 1, 1);


        retranslateUi(RunScriptDialog);

        QMetaObject::connectSlotsByName(RunScriptDialog);
    } // setupUi

    void retranslateUi(QDialog *RunScriptDialog)
    {
        RunScriptDialog->setWindowTitle(QCoreApplication::translate("RunScriptDialog", "Run Script", nullptr));
        extChk->setText(QCoreApplication::translate("RunScriptDialog", "Run as Extension Script", nullptr));
    } // retranslateUi

};

namespace Ui {
    class RunScriptDialog: public Ui_RunScriptDialog {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_RUNSCRIPTDIALOG_H
